/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <gee.h>
#include <time.h>
#include <libexif/exif-data.h>
#include <gtk/gtk.h>
#include <gobject/gvaluecollector.h>


#define TYPE_BATCH_IMPORT_JOB (batch_import_job_get_type ())
#define BATCH_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BATCH_IMPORT_JOB, BatchImportJob))
#define BATCH_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BATCH_IMPORT_JOB, BatchImportJobClass))
#define IS_BATCH_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BATCH_IMPORT_JOB))
#define IS_BATCH_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BATCH_IMPORT_JOB))
#define BATCH_IMPORT_JOB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BATCH_IMPORT_JOB, BatchImportJobClass))

typedef struct _BatchImportJob BatchImportJob;
typedef struct _BatchImportJobClass BatchImportJobClass;
typedef struct _BatchImportJobPrivate BatchImportJobPrivate;
typedef struct _ParamSpecBatchImportJob ParamSpecBatchImportJob;

#define TYPE_BATCH_IMPORT (batch_import_get_type ())
#define BATCH_IMPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BATCH_IMPORT, BatchImport))
#define BATCH_IMPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BATCH_IMPORT, BatchImportClass))
#define IS_BATCH_IMPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BATCH_IMPORT))
#define IS_BATCH_IMPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BATCH_IMPORT))
#define BATCH_IMPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BATCH_IMPORT, BatchImportClass))

typedef struct _BatchImport BatchImport;
typedef struct _BatchImportClass BatchImportClass;
typedef struct _BatchImportPrivate BatchImportPrivate;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_IMPORT_ID (import_id_get_type ())
typedef struct _ImportID ImportID;

#define TYPE_PHOTO_EXIF (photo_exif_get_type ())
#define PHOTO_EXIF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_EXIF, PhotoExif))
#define PHOTO_EXIF_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_EXIF, PhotoExifClass))
#define IS_PHOTO_EXIF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_EXIF))
#define IS_PHOTO_EXIF_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_EXIF))
#define PHOTO_EXIF_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_EXIF, PhotoExifClass))

typedef struct _PhotoExif PhotoExif;
typedef struct _PhotoExifClass PhotoExifClass;

#define TYPE_IMPORT_RESULT (import_result_get_type ())

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

#define BATCH_IMPORT_TYPE_DATE_COMPARATOR (batch_import_date_comparator_get_type ())
#define BATCH_IMPORT_DATE_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BATCH_IMPORT_TYPE_DATE_COMPARATOR, BatchImportDateComparator))
#define BATCH_IMPORT_DATE_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BATCH_IMPORT_TYPE_DATE_COMPARATOR, BatchImportDateComparatorClass))
#define BATCH_IMPORT_IS_DATE_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BATCH_IMPORT_TYPE_DATE_COMPARATOR))
#define BATCH_IMPORT_IS_DATE_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BATCH_IMPORT_TYPE_DATE_COMPARATOR))
#define BATCH_IMPORT_DATE_COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BATCH_IMPORT_TYPE_DATE_COMPARATOR, BatchImportDateComparatorClass))

typedef struct _BatchImportDateComparator BatchImportDateComparator;
typedef struct _BatchImportDateComparatorClass BatchImportDateComparatorClass;

#define TYPE_DATABASE_TABLE (database_table_get_type ())
#define DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_TABLE, DatabaseTable))
#define DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_TABLE, DatabaseTableClass))
#define IS_DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_TABLE))
#define IS_DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_TABLE))
#define DATABASE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_TABLE, DatabaseTableClass))

typedef struct _DatabaseTable DatabaseTable;
typedef struct _DatabaseTableClass DatabaseTableClass;

#define TYPE_PHOTO_TABLE (photo_table_get_type ())
#define PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_TABLE, PhotoTable))
#define PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_TABLE, PhotoTableClass))
#define IS_PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_TABLE))
#define IS_PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_TABLE))
#define PHOTO_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_TABLE, PhotoTableClass))

typedef struct _PhotoTable PhotoTable;
typedef struct _PhotoTableClass PhotoTableClass;

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;
typedef struct _ComparatorPrivate ComparatorPrivate;
typedef struct _BatchImportDateComparatorPrivate BatchImportDateComparatorPrivate;
typedef struct _ParamSpecBatchImport ParamSpecBatchImport;

struct _BatchImportJob {
	GTypeInstance parent_instance;
	volatile int ref_count;
	BatchImportJobPrivate * priv;
};

struct _BatchImportJobClass {
	GTypeClass parent_class;
	void (*finalize) (BatchImportJob *self);
	char* (*get_identifier) (BatchImportJob* self);
	gboolean (*prepare) (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library);
};

struct _ParamSpecBatchImportJob {
	GParamSpec parent_instance;
};

/* BatchImport performs the work of taking a file (supplied by BatchImportJob's) and properly importing
 it into the system, including database additions, thumbnail creation, and reporting it to AppWindow
 so it's properly added to various views and events.*/
struct _BatchImport {
	GTypeInstance parent_instance;
	volatile int ref_count;
	BatchImportPrivate * priv;
};

struct _BatchImportClass {
	GTypeClass parent_class;
	void (*finalize) (BatchImport *self);
};

struct _ImportID {
	gint64 id;
};

struct _BatchImportPrivate {
	GeeIterable* jobs;
	char* name;
	guint64 total_bytes;
	BatchImport* ref_holder;
	SortedList* success;
	GeeArrayList* failed;
	GeeArrayList* skipped;
	ImportID import_id;
	gboolean scheduled;
	gboolean user_aborted;
	gint import_file_count;
	gint fail_every;
	gint skip_every;
};

typedef enum  {
	IMPORT_RESULT_SUCCESS,
	IMPORT_RESULT_FILE_ERROR,
	IMPORT_RESULT_DECODE_ERROR,
	IMPORT_RESULT_DATABASE_ERROR,
	IMPORT_RESULT_USER_ABORT,
	IMPORT_RESULT_NOT_A_FILE,
	IMPORT_RESULT_PHOTO_EXISTS,
	IMPORT_RESULT_UNSUPPORTED_FORMAT
} ImportResult;

struct _Comparator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ComparatorPrivate * priv;
};

struct _ComparatorClass {
	GTypeClass parent_class;
	void (*finalize) (Comparator *self);
	gint64 (*compare) (Comparator* self, gconstpointer a, gconstpointer b);
};

struct _BatchImportDateComparator {
	Comparator parent_instance;
	BatchImportDateComparatorPrivate * priv;
};

struct _BatchImportDateComparatorClass {
	ComparatorClass parent_class;
};

struct _ParamSpecBatchImport {
	GParamSpec parent_instance;
};


static gpointer batch_import_job_parent_class = NULL;
static gpointer batch_import_date_comparator_parent_class = NULL;
static gpointer batch_import_parent_class = NULL;

gpointer batch_import_job_ref (gpointer instance);
void batch_import_job_unref (gpointer instance);
GParamSpec* param_spec_batch_import_job (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_batch_import_job (GValue* value, gpointer v_object);
gpointer value_get_batch_import_job (const GValue* value);
GType batch_import_job_get_type (void);
enum  {
	BATCH_IMPORT_JOB_DUMMY_PROPERTY
};
char* batch_import_job_get_identifier (BatchImportJob* self);
static char* batch_import_job_real_get_identifier (BatchImportJob* self);
gboolean batch_import_job_prepare (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library);
static gboolean batch_import_job_real_prepare (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library);
static void batch_import_job_finalize (BatchImportJob* obj);
gpointer batch_import_ref (gpointer instance);
void batch_import_unref (gpointer instance);
GParamSpec* param_spec_batch_import (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_batch_import (GValue* value, gpointer v_object);
gpointer value_get_batch_import (const GValue* value);
GType batch_import_get_type (void);
GType sorted_list_get_type (void);
GType photo_get_type (void);
GType import_id_get_type (void);
ImportID* import_id_dup (const ImportID* self);
void import_id_free (ImportID* self);
#define BATCH_IMPORT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_BATCH_IMPORT, BatchImportPrivate))
enum  {
	BATCH_IMPORT_DUMMY_PROPERTY
};
#define IMPORT_ID_INVALID ((gint64) (-1))
void import_id_init (ImportID *self, gint64 id);
#define BATCH_IMPORT_IMPORT_DIRECTORY_DEPTH 3
GFile* app_window_get_photos_dir (void);
gboolean exif_get_timestamp (ExifData* exif, time_t* timestamp);
void disassemble_filename (const char* basename, char** name, char** ext);
GFile* batch_import_create_library_path (const char* filename, ExifData* exif, const time_t* ts, gboolean* collision);
PhotoExif* photo_exif_new (GFile* file);
PhotoExif* photo_exif_construct (GType object_type, GFile* file);
gpointer photo_exif_ref (gpointer instance);
void photo_exif_unref (gpointer instance);
GParamSpec* param_spec_photo_exif (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_photo_exif (GValue* value, gpointer v_object);
gpointer value_get_photo_exif (const GValue* value);
GType photo_exif_get_type (void);
time_t query_file_modified (GFile* file, GError** error);
ExifData* photo_exif_get_exif (PhotoExif* self);
GType import_result_get_type (void);
static void batch_import_on_copy_progress (gint64 current, gint64 total);
static void _batch_import_on_copy_progress_gfile_progress_callback (gint64 current_num_bytes, gint64 total_num_bytes, gpointer self);
static ImportResult batch_import_copy_file (GFile* src, GFile** dest);
gboolean spin_event_loop (void);
static gint batch_import_get_test_variable (const char* name);
BatchImport* batch_import_new (GeeIterable* jobs, const char* name, guint64 total_bytes);
BatchImport* batch_import_construct (GType object_type, GeeIterable* jobs, const char* name, guint64 total_bytes);
char* batch_import_get_name (BatchImport* self);
guint64 batch_import_get_total_bytes (BatchImport* self);
void batch_import_user_halt (BatchImport* self);
static gboolean batch_import_perform_import (BatchImport* self);
static gboolean _batch_import_perform_import_gsource_func (gpointer self);
void batch_import_schedule (BatchImport* self);
static BatchImportDateComparator* batch_import_date_comparator_new (void);
static BatchImportDateComparator* batch_import_date_comparator_construct (GType object_type);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
static GType batch_import_date_comparator_get_type (void);
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
PhotoTable* photo_table_new (void);
PhotoTable* photo_table_construct (GType object_type);
gpointer database_table_ref (gpointer instance);
void database_table_unref (gpointer instance);
GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_database_table (GValue* value, gpointer v_object);
gpointer value_get_database_table (const GValue* value);
GType database_table_get_type (void);
GType photo_table_get_type (void);
ImportID photo_table_generate_import_id (PhotoTable* self);
gboolean app_window_has_user_quit (void);
static void batch_import_import (BatchImport* self, BatchImportJob* job, GFile* file, gboolean copy_to_library, const char* id);
gint sorted_list_get_size (SortedList* self);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
void app_window_batch_import_complete (AppWindow* self, SortedList* imported_photos);
static ImportResult batch_import_import_dir (BatchImport* self, BatchImportJob* job, GFile* dir, gboolean copy_to_library);
static ImportResult batch_import_import_file (BatchImport* self, GFile* file, gboolean copy_to_library);
ImportResult photo_import (GFile* file, const ImportID* import_id, Photo** photo);
gboolean sorted_list_add (SortedList* self, gconstpointer item);
void app_window_photo_imported (AppWindow* self, Photo* photo);
enum  {
	BATCH_IMPORT_DATE_COMPARATOR_DUMMY_PROPERTY
};
time_t photo_get_exposure_time (Photo* self);
static gint64 batch_import_date_comparator_real_compare (Comparator* base, Photo* photo_a, Photo* photo_b);
static void batch_import_finalize (BatchImport* obj);


static void g_cclosure_user_marshal_VOID__ENUM_POINTER_OBJECT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void g_cclosure_user_marshal_VOID__BOXED_OBJECT_OBJECT_OBJECT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

#line 8 "BatchImport.vala"
static char* batch_import_job_real_get_identifier (BatchImportJob* self) {
#line 328 "BatchImport.c"
	g_return_val_if_fail (IS_BATCH_IMPORT_JOB (self), NULL);
	g_critical ("Type `%s' does not implement abstract method `batch_import_job_get_identifier'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


#line 8 "BatchImport.vala"
char* batch_import_job_get_identifier (BatchImportJob* self) {
#line 8 "BatchImport.vala"
	return BATCH_IMPORT_JOB_GET_CLASS (self)->get_identifier (self);
#line 339 "BatchImport.c"
}


#line 10 "BatchImport.vala"
static gboolean batch_import_job_real_prepare (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library) {
#line 345 "BatchImport.c"
	g_return_val_if_fail (IS_BATCH_IMPORT_JOB (self), FALSE);
	g_critical ("Type `%s' does not implement abstract method `batch_import_job_prepare'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return FALSE;
}


#line 10 "BatchImport.vala"
gboolean batch_import_job_prepare (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library) {
#line 10 "BatchImport.vala"
	return BATCH_IMPORT_JOB_GET_CLASS (self)->prepare (self, file_to_import, copy_to_library);
#line 356 "BatchImport.c"
}


static void value_batch_import_job_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_batch_import_job_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		batch_import_job_unref (value->data[0].v_pointer);
	}
}


static void value_batch_import_job_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = batch_import_job_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_batch_import_job_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_batch_import_job_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		BatchImportJob* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = batch_import_job_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_batch_import_job_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	BatchImportJob** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = batch_import_job_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_batch_import_job (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecBatchImportJob* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_BATCH_IMPORT_JOB), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_batch_import_job (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_BATCH_IMPORT_JOB), NULL);
	return value->data[0].v_pointer;
}


void value_set_batch_import_job (GValue* value, gpointer v_object) {
	BatchImportJob* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_BATCH_IMPORT_JOB));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_BATCH_IMPORT_JOB));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		batch_import_job_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		batch_import_job_unref (old);
	}
}


static void batch_import_job_class_init (BatchImportJobClass * klass) {
	batch_import_job_parent_class = g_type_class_peek_parent (klass);
	BATCH_IMPORT_JOB_CLASS (klass)->finalize = batch_import_job_finalize;
	BATCH_IMPORT_JOB_CLASS (klass)->get_identifier = batch_import_job_real_get_identifier;
	BATCH_IMPORT_JOB_CLASS (klass)->prepare = batch_import_job_real_prepare;
}


static void batch_import_job_instance_init (BatchImportJob * self) {
	self->ref_count = 1;
}


static void batch_import_job_finalize (BatchImportJob* obj) {
	BatchImportJob * self;
	self = BATCH_IMPORT_JOB (obj);
}


GType batch_import_job_get_type (void) {
	static GType batch_import_job_type_id = 0;
	if (batch_import_job_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_batch_import_job_init, value_batch_import_job_free_value, value_batch_import_job_copy_value, value_batch_import_job_peek_pointer, "p", value_batch_import_job_collect_value, "p", value_batch_import_job_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (BatchImportJobClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) batch_import_job_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BatchImportJob), 0, (GInstanceInitFunc) batch_import_job_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		batch_import_job_type_id = g_type_register_fundamental (g_type_fundamental_next (), "BatchImportJob", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return batch_import_job_type_id;
}


gpointer batch_import_job_ref (gpointer instance) {
	BatchImportJob* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void batch_import_job_unref (gpointer instance) {
	BatchImportJob* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		BATCH_IMPORT_JOB_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 25 "BatchImport.vala"
GFile* batch_import_create_library_path (const char* filename, ExifData* exif, const time_t* ts, gboolean* collision) {
#line 504 "BatchImport.c"
	GFile* result;
	GError * _inner_error_;
	GFile* dir;
	time_t timestamp;
	gboolean _tmp0_;
	struct tm _tmp2_ = {0};
	struct tm tm;
	GFile* _tmp4_;
	char* _tmp3_;
	GFile* _tmp6_;
	char* _tmp5_;
	GFile* _tmp8_;
	char* _tmp7_;
	GFile* file;
	char* name;
	char* ext;
	char* _tmp14_;
	char* _tmp13_;
	char* _tmp12_;
	char* _tmp11_;
	char* _tmp10_;
#line 25 "BatchImport.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 528 "BatchImport.c"
	_inner_error_ = NULL;
	dir = app_window_get_photos_dir ();
	timestamp = (*ts);
	_tmp0_ = FALSE;
#line 32 "BatchImport.vala"
	if (exif != NULL) {
#line 32 "BatchImport.vala"
		_tmp0_ = !exif_get_timestamp (exif, &timestamp);
#line 537 "BatchImport.c"
	} else {
#line 32 "BatchImport.vala"
		_tmp0_ = FALSE;
#line 541 "BatchImport.c"
	}
	/* use EXIF exposure timestamp over the supplied one (which probably comes from the file's
	 modified time, or is simply now())*/
#line 32 "BatchImport.vala"
	if (_tmp0_) {
#line 547 "BatchImport.c"
		/* if no exposure time supplied, use now()*/
#line 34 "BatchImport.vala"
		if ((*ts) == 0) {
#line 551 "BatchImport.c"
			time_t _tmp1_ = {0};
#line 35 "BatchImport.vala"
			timestamp = (time (&_tmp1_), _tmp1_);
#line 555 "BatchImport.c"
		}
	}
	tm = (localtime_r (&timestamp, &_tmp2_), _tmp2_);
	/* build a directory tree inside the library, as deep as IMPORT_DIRECTORY_DEPTH:
	 yyyy/mm/dd*/
	_tmp4_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp3_ = NULL;
#line 42 "BatchImport.vala"
	dir = (_tmp4_ = g_file_get_child (dir, _tmp3_ = g_strdup_printf ("%04u", (guint) (tm.tm_year + 1900))), (dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL)), _tmp4_);
#line 566 "BatchImport.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
	_tmp6_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp5_ = NULL;
#line 43 "BatchImport.vala"
	dir = (_tmp6_ = g_file_get_child (dir, _tmp5_ = g_strdup_printf ("%02u", (guint) (tm.tm_mon + 1))), (dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL)), _tmp6_);
#line 573 "BatchImport.c"
	_tmp5_ = (g_free (_tmp5_), NULL);
	_tmp8_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp7_ = NULL;
#line 44 "BatchImport.vala"
	dir = (_tmp8_ = g_file_get_child (dir, _tmp7_ = g_strdup_printf ("%02u", (guint) tm.tm_mday)), (dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL)), _tmp8_);
#line 580 "BatchImport.c"
	_tmp7_ = (g_free (_tmp7_), NULL);
	{
#line 47 "BatchImport.vala"
		if (!g_file_query_exists (dir, NULL)) {
#line 48 "BatchImport.vala"
			g_file_make_directory_with_parents (dir, NULL, &_inner_error_);
#line 587 "BatchImport.c"
			if (_inner_error_ != NULL) {
				goto __catch40_g_error;
				goto __finally40;
			}
		}
	}
	goto __finally40;
	__catch40_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp9_;
#line 952 "gio-2.0.vapi"
			_tmp9_ = NULL;
#line 50 "BatchImport.vala"
			g_error ("BatchImport.vala:50: Unable to create photo library directory %s", _tmp9_ = g_file_get_path (dir));
#line 606 "BatchImport.c"
			_tmp9_ = (g_free (_tmp9_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally40:
	if (_inner_error_ != NULL) {
		(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	/* if file doesn't exist, use that and done*/
	file = g_file_get_child (dir, filename);
#line 55 "BatchImport.vala"
	if (!g_file_query_exists (file, NULL)) {
#line 56 "BatchImport.vala"
		(*collision) = FALSE;
#line 624 "BatchImport.c"
		result = file;
		(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
		return result;
	}
#line 61 "BatchImport.vala"
	(*collision) = TRUE;
#line 631 "BatchImport.c"
	name = NULL;
	ext = NULL;
	_tmp14_ = NULL;
	_tmp13_ = NULL;
	_tmp12_ = NULL;
	_tmp11_ = NULL;
#line 947 "gio-2.0.vapi"
	_tmp10_ = NULL;
#line 64 "BatchImport.vala"
	(disassemble_filename (_tmp10_ = g_file_get_basename (file), &_tmp11_, &_tmp13_), name = (_tmp12_ = _tmp11_, name = (g_free (name), NULL), _tmp12_));
#line 64 "BatchImport.vala"
	ext = (_tmp14_ = _tmp13_, ext = (g_free (ext), NULL), _tmp14_);
#line 644 "BatchImport.c"
	_tmp10_ = (g_free (_tmp10_), NULL);
	{
		gint ctr;
		/* generate a unique filename*/
		ctr = 1;
		{
			gboolean _tmp15_;
			_tmp15_ = TRUE;
#line 67 "BatchImport.vala"
			while (TRUE) {
#line 655 "BatchImport.c"
				char* _tmp16_;
				const char* _tmp19_;
				char* new_name;
				GFile* _tmp20_;
#line 67 "BatchImport.vala"
				if (!_tmp15_) {
#line 67 "BatchImport.vala"
					ctr++;
#line 664 "BatchImport.c"
				}
#line 67 "BatchImport.vala"
				_tmp15_ = FALSE;
#line 67 "BatchImport.vala"
				if (!(ctr < G_MAXINT)) {
#line 67 "BatchImport.vala"
					break;
#line 672 "BatchImport.c"
				}
				_tmp16_ = NULL;
#line 68 "BatchImport.vala"
				if (ext != NULL) {
#line 677 "BatchImport.c"
					char* _tmp17_;
					_tmp17_ = NULL;
#line 68 "BatchImport.vala"
					_tmp16_ = (_tmp17_ = g_strdup_printf ("%s_%d.%s", name, ctr, ext), _tmp16_ = (g_free (_tmp16_), NULL), _tmp17_);
#line 682 "BatchImport.c"
				} else {
					char* _tmp18_;
					_tmp18_ = NULL;
#line 68 "BatchImport.vala"
					_tmp16_ = (_tmp18_ = g_strdup_printf ("%s_%d", name, ctr), _tmp16_ = (g_free (_tmp16_), NULL), _tmp18_);
#line 688 "BatchImport.c"
				}
#line 68 "BatchImport.vala"
				_tmp19_ = NULL;
#line 692 "BatchImport.c"
				new_name = (_tmp19_ = _tmp16_, (_tmp19_ == NULL) ? NULL : g_strdup (_tmp19_));
				_tmp20_ = NULL;
#line 70 "BatchImport.vala"
				file = (_tmp20_ = g_file_get_child (dir, new_name), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp20_);
#line 72 "BatchImport.vala"
				if (!g_file_query_exists (file, NULL)) {
#line 699 "BatchImport.c"
					result = file;
					_tmp16_ = (g_free (_tmp16_), NULL);
					new_name = (g_free (new_name), NULL);
					(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
					name = (g_free (name), NULL);
					ext = (g_free (ext), NULL);
					return result;
				}
				_tmp16_ = (g_free (_tmp16_), NULL);
				new_name = (g_free (new_name), NULL);
			}
		}
	}
	result = NULL;
	(dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL));
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	name = (g_free (name), NULL);
	ext = (g_free (ext), NULL);
	return result;
}


#line 107 "BatchImport.vala"
static void _batch_import_on_copy_progress_gfile_progress_callback (gint64 current_num_bytes, gint64 total_num_bytes, gpointer self) {
#line 724 "BatchImport.c"
	batch_import_on_copy_progress (current_num_bytes, total_num_bytes);
}


#line 79 "BatchImport.vala"
static ImportResult batch_import_copy_file (GFile* src, GFile** dest) {
#line 731 "BatchImport.c"
	ImportResult result;
	GError * _inner_error_;
	PhotoExif* exif;
	time_t timestamp;
	gboolean collision;
	GFile* _tmp4_;
	ExifData* _tmp3_;
	char* _tmp2_;
	char* _tmp6_;
	char* _tmp5_;
#line 79 "BatchImport.vala"
	g_return_val_if_fail (G_IS_FILE (src), 0);
#line 744 "BatchImport.c"
	if (dest != NULL) {
		*dest = NULL;
	}
	_inner_error_ = NULL;
	exif = photo_exif_new (src);
	timestamp = 0;
	{
		time_t _tmp0_;
		_tmp0_ = query_file_modified (src, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch41_g_error;
			goto __finally41;
		}
#line 83 "BatchImport.vala"
		timestamp = _tmp0_;
#line 760 "BatchImport.c"
	}
	goto __finally41;
	__catch41_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp1_;
#line 952 "gio-2.0.vapi"
			_tmp1_ = NULL;
#line 85 "BatchImport.vala"
			g_critical ("BatchImport.vala:85: Unable to access file modification for %s: %s", _tmp1_ = g_file_get_path (src), err->message);
#line 774 "BatchImport.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally41:
	if (_inner_error_ != NULL) {
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	collision = FALSE;
	_tmp4_ = NULL;
#line 151 "Exif.vala"
	_tmp3_ = NULL;
#line 947 "gio-2.0.vapi"
	_tmp2_ = NULL;
#line 89 "BatchImport.vala"
	(*dest) = (_tmp4_ = batch_import_create_library_path (_tmp2_ = g_file_get_basename (src), _tmp3_ = photo_exif_get_exif (exif), &timestamp, &collision), ((*dest) == NULL) ? NULL : ((*dest) = (g_object_unref ((*dest)), NULL)), _tmp4_);
#line 794 "BatchImport.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (exif_data_unref (_tmp3_), NULL));
	_tmp2_ = (g_free (_tmp2_), NULL);
#line 90 "BatchImport.vala"
	if ((*dest) == NULL) {
#line 799 "BatchImport.c"
		result = IMPORT_RESULT_FILE_ERROR;
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		return result;
	}
#line 952 "gio-2.0.vapi"
	_tmp6_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp5_ = NULL;
#line 93 "BatchImport.vala"
	g_debug ("BatchImport.vala:93: Copying %s to %s", _tmp5_ = g_file_get_path (src), _tmp6_ = g_file_get_path ((*dest)));
#line 810 "BatchImport.c"
	_tmp6_ = (g_free (_tmp6_), NULL);
	_tmp5_ = (g_free (_tmp5_), NULL);
	{
#line 96 "BatchImport.vala"
		g_file_copy (src, (*dest), G_FILE_COPY_ALL_METADATA, NULL, _batch_import_on_copy_progress_gfile_progress_callback, NULL, &_inner_error_);
#line 816 "BatchImport.c"
		if (_inner_error_ != NULL) {
			goto __catch42_g_error;
			goto __finally42;
		}
	}
	goto __finally42;
	__catch42_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp8_;
			char* _tmp7_;
#line 952 "gio-2.0.vapi"
			_tmp8_ = NULL;
#line 952 "gio-2.0.vapi"
			_tmp7_ = NULL;
#line 98 "BatchImport.vala"
			g_critical ("BatchImport.vala:98: Unable to copy file %s to %s: %s", _tmp7_ = g_file_get_path (src), _tmp8_ = g_file_get_path ((*dest)), err->message);
#line 837 "BatchImport.c"
			_tmp8_ = (g_free (_tmp8_), NULL);
			_tmp7_ = (g_free (_tmp7_), NULL);
			result = IMPORT_RESULT_FILE_ERROR;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
			return result;
		}
	}
	__finally42:
	if (_inner_error_ != NULL) {
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	result = IMPORT_RESULT_SUCCESS;
	(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
	return result;
}


#line 107 "BatchImport.vala"
static void batch_import_on_copy_progress (gint64 current, gint64 total) {
#line 108 "BatchImport.vala"
	spin_event_loop ();
#line 863 "BatchImport.c"
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


#line 111 "BatchImport.vala"
static gint batch_import_get_test_variable (const char* name) {
#line 877 "BatchImport.c"
	gint result;
	const char* _tmp0_;
	char* value;
	gboolean _tmp1_;
#line 111 "BatchImport.vala"
	g_return_val_if_fail (name != NULL, 0);
#line 112 "BatchImport.vala"
	_tmp0_ = NULL;
#line 886 "BatchImport.c"
	value = (_tmp0_ = g_getenv (name), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	_tmp1_ = FALSE;
#line 113 "BatchImport.vala"
	if (value == NULL) {
#line 113 "BatchImport.vala"
		_tmp1_ = TRUE;
#line 893 "BatchImport.c"
	} else {
#line 113 "BatchImport.vala"
		_tmp1_ = string_get_length (value) == 0;
#line 897 "BatchImport.c"
	}
#line 113 "BatchImport.vala"
	if (_tmp1_) {
#line 901 "BatchImport.c"
		result = 0;
		value = (g_free (value), NULL);
		return result;
	}
	result = atoi (value);
	value = (g_free (value), NULL);
	return result;
}


#line 135 "BatchImport.vala"
BatchImport* batch_import_construct (GType object_type, GeeIterable* jobs, const char* name, guint64 total_bytes) {
#line 914 "BatchImport.c"
	BatchImport* self;
	GeeIterable* _tmp1_;
	GeeIterable* _tmp0_;
	char* _tmp3_;
	const char* _tmp2_;
#line 135 "BatchImport.vala"
	g_return_val_if_fail (GEE_IS_ITERABLE (jobs), NULL);
#line 135 "BatchImport.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 924 "BatchImport.c"
	self = (BatchImport*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 136 "BatchImport.vala"
	_tmp0_ = NULL;
#line 136 "BatchImport.vala"
	self->priv->jobs = (_tmp1_ = (_tmp0_ = jobs, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->jobs == NULL) ? NULL : (self->priv->jobs = (g_object_unref (self->priv->jobs), NULL)), _tmp1_);
#line 931 "BatchImport.c"
	_tmp3_ = NULL;
#line 137 "BatchImport.vala"
	_tmp2_ = NULL;
#line 137 "BatchImport.vala"
	self->priv->name = (_tmp3_ = (_tmp2_ = name, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->name = (g_free (self->priv->name), NULL), _tmp3_);
#line 138 "BatchImport.vala"
	self->priv->total_bytes = total_bytes;
#line 139 "BatchImport.vala"
	self->priv->fail_every = batch_import_get_test_variable ("SHOTWELL_FAIL_EVERY");
#line 140 "BatchImport.vala"
	self->priv->skip_every = batch_import_get_test_variable ("SHOTWELL_SKIP_EVERY");
#line 943 "BatchImport.c"
	return self;
}


#line 135 "BatchImport.vala"
BatchImport* batch_import_new (GeeIterable* jobs, const char* name, guint64 total_bytes) {
#line 135 "BatchImport.vala"
	return batch_import_construct (TYPE_BATCH_IMPORT, jobs, name, total_bytes);
#line 952 "BatchImport.c"
}


#line 156 "BatchImport.vala"
char* batch_import_get_name (BatchImport* self) {
#line 958 "BatchImport.c"
	char* result;
	const char* _tmp0_;
#line 156 "BatchImport.vala"
	g_return_val_if_fail (IS_BATCH_IMPORT (self), NULL);
#line 157 "BatchImport.vala"
	_tmp0_ = NULL;
#line 965 "BatchImport.c"
	result = (_tmp0_ = self->priv->name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 160 "BatchImport.vala"
guint64 batch_import_get_total_bytes (BatchImport* self) {
#line 973 "BatchImport.c"
	guint64 result;
#line 160 "BatchImport.vala"
	g_return_val_if_fail (IS_BATCH_IMPORT (self), 0ULL);
#line 977 "BatchImport.c"
	result = self->priv->total_bytes;
	return result;
}


#line 164 "BatchImport.vala"
void batch_import_user_halt (BatchImport* self) {
#line 164 "BatchImport.vala"
	g_return_if_fail (IS_BATCH_IMPORT (self));
#line 165 "BatchImport.vala"
	self->priv->user_aborted = TRUE;
#line 989 "BatchImport.c"
}


#line 179 "BatchImport.vala"
static gboolean _batch_import_perform_import_gsource_func (gpointer self) {
#line 995 "BatchImport.c"
	return batch_import_perform_import (self);
}


#line 168 "BatchImport.vala"
void batch_import_schedule (BatchImport* self) {
#line 1002 "BatchImport.c"
	BatchImport* _tmp1_;
	BatchImport* _tmp0_;
#line 168 "BatchImport.vala"
	g_return_if_fail (IS_BATCH_IMPORT (self));
#line 169 "BatchImport.vala"
	g_assert (!self->priv->scheduled);
#line 1009 "BatchImport.c"
	/* XXX: This is necessary because Idle.add doesn't ref SourceFunc:
	 http://bugzilla.gnome.org/show_bug.cgi?id=548427*/
	_tmp1_ = NULL;
#line 173 "BatchImport.vala"
	_tmp0_ = NULL;
#line 173 "BatchImport.vala"
	self->priv->ref_holder = (_tmp1_ = (_tmp0_ = self, (_tmp0_ == NULL) ? NULL : batch_import_ref (_tmp0_)), (self->priv->ref_holder == NULL) ? NULL : (self->priv->ref_holder = (batch_import_unref (self->priv->ref_holder), NULL)), _tmp1_);
#line 175 "BatchImport.vala"
	g_idle_add (_batch_import_perform_import_gsource_func, self);
#line 176 "BatchImport.vala"
	self->priv->scheduled = TRUE;
#line 1021 "BatchImport.c"
}


#line 179 "BatchImport.vala"
static gboolean batch_import_perform_import (BatchImport* self) {
#line 1027 "BatchImport.c"
	gboolean result;
	SortedList* _tmp1_;
	BatchImportDateComparator* _tmp0_;
	GeeArrayList* _tmp2_;
	GeeArrayList* _tmp3_;
	PhotoTable* _tmp4_;
	BatchImport* _tmp12_;
#line 179 "BatchImport.vala"
	g_return_val_if_fail (IS_BATCH_IMPORT (self), FALSE);
#line 180 "BatchImport.vala"
	g_signal_emit_by_name (self, "starting");
#line 1039 "BatchImport.c"
	_tmp1_ = NULL;
	_tmp0_ = NULL;
#line 182 "BatchImport.vala"
	self->priv->success = (_tmp1_ = sorted_list_new (TYPE_PHOTO, (GBoxedCopyFunc) g_object_ref, g_object_unref, COMPARATOR (_tmp0_ = batch_import_date_comparator_new ())), (self->priv->success == NULL) ? NULL : (self->priv->success = (g_object_unref (self->priv->success), NULL)), _tmp1_);
#line 1044 "BatchImport.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (comparator_unref (_tmp0_), NULL));
	_tmp2_ = NULL;
#line 183 "BatchImport.vala"
	self->priv->failed = (_tmp2_ = gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal), (self->priv->failed == NULL) ? NULL : (self->priv->failed = (g_object_unref (self->priv->failed), NULL)), _tmp2_);
#line 1049 "BatchImport.c"
	_tmp3_ = NULL;
#line 184 "BatchImport.vala"
	self->priv->skipped = (_tmp3_ = gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal), (self->priv->skipped == NULL) ? NULL : (self->priv->skipped = (g_object_unref (self->priv->skipped), NULL)), _tmp3_);
#line 1053 "BatchImport.c"
	_tmp4_ = NULL;
#line 185 "BatchImport.vala"
	self->priv->import_id = photo_table_generate_import_id (_tmp4_ = photo_table_new ());
#line 1057 "BatchImport.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (database_table_unref (_tmp4_), NULL));
	{
		GeeIterator* _job_it;
		_job_it = gee_iterable_iterator (self->priv->jobs);
#line 187 "BatchImport.vala"
		while (TRUE) {
#line 1064 "BatchImport.c"
			BatchImportJob* job;
			GFile* file;
			gboolean copy_to_library;
			GFile* _tmp8_;
			gboolean _tmp7_;
			GFile* _tmp6_;
#line 187 "BatchImport.vala"
			if (!gee_iterator_next (_job_it)) {
#line 187 "BatchImport.vala"
				break;
#line 1075 "BatchImport.c"
			}
			job = (BatchImportJob*) gee_iterator_get (_job_it);
#line 188 "BatchImport.vala"
			if (app_window_has_user_quit ()) {
#line 189 "BatchImport.vala"
				self->priv->user_aborted = TRUE;
#line 1082 "BatchImport.c"
			}
#line 191 "BatchImport.vala"
			if (self->priv->user_aborted) {
#line 1086 "BatchImport.c"
				char* _tmp5_;
#line 192 "BatchImport.vala"
				g_signal_emit_by_name (self, "import-job-failed", IMPORT_RESULT_USER_ABORT, job, NULL);
#line 8 "BatchImport.vala"
				_tmp5_ = NULL;
#line 193 "BatchImport.vala"
				gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->skipped), _tmp5_ = batch_import_job_get_identifier (job));
#line 1094 "BatchImport.c"
				_tmp5_ = (g_free (_tmp5_), NULL);
				(job == NULL) ? NULL : (job = (batch_import_job_unref (job), NULL));
#line 195 "BatchImport.vala"
				continue;
#line 1099 "BatchImport.c"
			}
			file = NULL;
			copy_to_library = FALSE;
			_tmp8_ = NULL;
			_tmp6_ = NULL;
#line 200 "BatchImport.vala"
			if ((_tmp7_ = batch_import_job_prepare (job, &_tmp6_, &copy_to_library), file = (_tmp8_ = _tmp6_, (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp8_), _tmp7_)) {
#line 1107 "BatchImport.c"
				char* _tmp9_;
#line 8 "BatchImport.vala"
				_tmp9_ = NULL;
#line 201 "BatchImport.vala"
				batch_import_import (self, job, file, copy_to_library, _tmp9_ = batch_import_job_get_identifier (job));
#line 1113 "BatchImport.c"
				_tmp9_ = (g_free (_tmp9_), NULL);
			} else {
				char* _tmp10_;
#line 203 "BatchImport.vala"
				g_signal_emit_by_name (self, "import-job-failed", IMPORT_RESULT_FILE_ERROR, job, NULL);
#line 8 "BatchImport.vala"
				_tmp10_ = NULL;
#line 204 "BatchImport.vala"
				gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->failed), _tmp10_ = batch_import_job_get_identifier (job));
#line 1123 "BatchImport.c"
				_tmp10_ = (g_free (_tmp10_), NULL);
			}
			(job == NULL) ? NULL : (job = (batch_import_job_unref (job), NULL));
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		}
		(_job_it == NULL) ? NULL : (_job_it = (g_object_unref (_job_it), NULL));
	}
	/* report to AppWindow to organize into events*/
#line 209 "BatchImport.vala"
	if (sorted_list_get_size (self->priv->success) > 0) {
#line 1134 "BatchImport.c"
		AppWindow* _tmp11_;
#line 446 "AppWindow.vala"
		_tmp11_ = NULL;
#line 210 "BatchImport.vala"
		app_window_batch_import_complete (_tmp11_ = app_window_get_instance (), self->priv->success);
#line 1140 "BatchImport.c"
		(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
	}
	/* report completed*/
#line 213 "BatchImport.vala"
	g_signal_emit_by_name (self, "import-complete", &self->priv->import_id, self->priv->success, self->priv->failed, self->priv->skipped);
#line 1146 "BatchImport.c"
	/* XXX: unref "this" ... vital that the self pointer is not touched from here on out*/
	_tmp12_ = NULL;
#line 216 "BatchImport.vala"
	self->priv->ref_holder = (_tmp12_ = NULL, (self->priv->ref_holder == NULL) ? NULL : (self->priv->ref_holder = (batch_import_unref (self->priv->ref_holder), NULL)), _tmp12_);
#line 1151 "BatchImport.c"
	result = FALSE;
	return result;
}


#line 221 "BatchImport.vala"
static void batch_import_import (BatchImport* self, BatchImportJob* job, GFile* file, gboolean copy_to_library, const char* id) {
#line 1159 "BatchImport.c"
	GFileType type;
	ImportResult _result_;
#line 221 "BatchImport.vala"
	g_return_if_fail (IS_BATCH_IMPORT (self));
#line 221 "BatchImport.vala"
	g_return_if_fail (IS_BATCH_IMPORT_JOB (job));
#line 221 "BatchImport.vala"
	g_return_if_fail (G_IS_FILE (file));
#line 221 "BatchImport.vala"
	g_return_if_fail (id != NULL);
#line 222 "BatchImport.vala"
	if (self->priv->user_aborted) {
#line 223 "BatchImport.vala"
		gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->skipped), id);
#line 1174 "BatchImport.c"
		return;
	}
	type = g_file_query_file_type (file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL);
	_result_ = 0;
#line 231 "BatchImport.vala"
	switch (type) {
#line 1181 "BatchImport.c"
		case G_FILE_TYPE_DIRECTORY:
		{
#line 233 "BatchImport.vala"
			_result_ = batch_import_import_dir (self, job, file, copy_to_library);
#line 234 "BatchImport.vala"
			break;
#line 1188 "BatchImport.c"
		}
		case G_FILE_TYPE_REGULAR:
		{
#line 237 "BatchImport.vala"
			_result_ = batch_import_import_file (self, file, copy_to_library);
#line 238 "BatchImport.vala"
			break;
#line 1196 "BatchImport.c"
		}
		default:
		{
			char* _tmp0_;
#line 952 "gio-2.0.vapi"
			_tmp0_ = NULL;
#line 241 "BatchImport.vala"
			g_debug ("BatchImport.vala:241: Skipping file %s (neither a directory nor a file)", _tmp0_ = g_file_get_path (file));
#line 1205 "BatchImport.c"
			_tmp0_ = (g_free (_tmp0_), NULL);
#line 242 "BatchImport.vala"
			_result_ = IMPORT_RESULT_NOT_A_FILE;
#line 243 "BatchImport.vala"
			break;
#line 1211 "BatchImport.c"
		}
	}
#line 246 "BatchImport.vala"
	switch (_result_) {
#line 1216 "BatchImport.c"
		case IMPORT_RESULT_SUCCESS:
		{
#line 249 "BatchImport.vala"
			break;
#line 1221 "BatchImport.c"
		}
		case IMPORT_RESULT_USER_ABORT:
		{
#line 253 "BatchImport.vala"
			self->priv->user_aborted = TRUE;
#line 254 "BatchImport.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->skipped), id);
#line 255 "BatchImport.vala"
			g_signal_emit_by_name (self, "import-job-failed", _result_, job, file);
#line 256 "BatchImport.vala"
			break;
#line 1233 "BatchImport.c"
		}
		case IMPORT_RESULT_NOT_A_FILE:
		case IMPORT_RESULT_PHOTO_EXISTS:
		case IMPORT_RESULT_UNSUPPORTED_FORMAT:
		{
#line 261 "BatchImport.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->skipped), id);
#line 262 "BatchImport.vala"
			g_signal_emit_by_name (self, "import-job-failed", _result_, job, file);
#line 263 "BatchImport.vala"
			break;
#line 1245 "BatchImport.c"
		}
		default:
		{
#line 266 "BatchImport.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->failed), id);
#line 267 "BatchImport.vala"
			g_signal_emit_by_name (self, "import-job-failed", _result_, job, file);
#line 268 "BatchImport.vala"
			break;
#line 1255 "BatchImport.c"
		}
	}
}


#line 272 "BatchImport.vala"
static ImportResult batch_import_import_dir (BatchImport* self, BatchImportJob* job, GFile* dir, gboolean copy_to_library) {
#line 1263 "BatchImport.c"
	ImportResult result;
	GError * _inner_error_;
#line 272 "BatchImport.vala"
	g_return_val_if_fail (IS_BATCH_IMPORT (self), 0);
#line 272 "BatchImport.vala"
	g_return_val_if_fail (IS_BATCH_IMPORT_JOB (job), 0);
#line 272 "BatchImport.vala"
	g_return_val_if_fail (G_IS_FILE (dir), 0);
#line 1272 "BatchImport.c"
	_inner_error_ = NULL;
	{
		GFileEnumerator* enumerator;
		GFileInfo* info;
		enumerator = g_file_enumerate_children (dir, "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch43_g_error;
			goto __finally43;
		}
#line 276 "BatchImport.vala"
		if (enumerator == NULL) {
#line 1284 "BatchImport.c"
			result = IMPORT_RESULT_FILE_ERROR;
			(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
			return result;
		}
#line 279 "BatchImport.vala"
		if (!spin_event_loop ()) {
#line 1291 "BatchImport.c"
			result = IMPORT_RESULT_USER_ABORT;
			(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
			return result;
		}
		info = NULL;
#line 283 "BatchImport.vala"
		while (TRUE) {
#line 1299 "BatchImport.c"
			GFileInfo* _tmp0_;
			GFileInfo* _tmp1_;
			GFile* child;
			char* _tmp2_;
			_tmp0_ = g_file_enumerator_next_file (enumerator, NULL, &_inner_error_);
			if (_inner_error_ != NULL) {
				(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
				(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
				goto __catch43_g_error;
				goto __finally43;
			}
			_tmp1_ = NULL;
#line 283 "BatchImport.vala"
			if (!((info = (_tmp1_ = _tmp0_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp1_)) != NULL)) {
#line 283 "BatchImport.vala"
				break;
#line 1316 "BatchImport.c"
			}
			child = g_file_get_child (dir, g_file_info_get_name (info));
#line 954 "gio-2.0.vapi"
			_tmp2_ = NULL;
#line 285 "BatchImport.vala"
			batch_import_import (self, job, child, copy_to_library, _tmp2_ = g_file_get_uri (child));
#line 1323 "BatchImport.c"
			_tmp2_ = (g_free (_tmp2_), NULL);
			(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
		}
		(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	}
	goto __finally43;
	__catch43_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp3_;
#line 952 "gio-2.0.vapi"
			_tmp3_ = NULL;
#line 288 "BatchImport.vala"
			g_debug ("BatchImport.vala:288: Unable to import from %s: %s", _tmp3_ = g_file_get_path (dir), err->message);
#line 1342 "BatchImport.c"
			_tmp3_ = (g_free (_tmp3_), NULL);
			result = IMPORT_RESULT_FILE_ERROR;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			return result;
		}
	}
	__finally43:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	result = IMPORT_RESULT_SUCCESS;
	return result;
}


#line 296 "BatchImport.vala"
static ImportResult batch_import_import_file (BatchImport* self, GFile* file, gboolean copy_to_library) {
#line 1362 "BatchImport.c"
	ImportResult result;
	GError * _inner_error_;
	GFile* _tmp0_;
	GFile* import;
	GFile* _tmp1_;
	gboolean _tmp2_;
	gboolean is_in_library_dir;
	gboolean _tmp3_;
	Photo* photo;
	Photo* _tmp13_;
	ImportResult _tmp12_;
	Photo* _tmp11_;
	ImportResult _result_;
	AppWindow* _tmp15_;
#line 296 "BatchImport.vala"
	g_return_val_if_fail (IS_BATCH_IMPORT (self), 0);
#line 296 "BatchImport.vala"
	g_return_val_if_fail (G_IS_FILE (file), 0);
#line 1381 "BatchImport.c"
	_inner_error_ = NULL;
#line 297 "BatchImport.vala"
	if (!spin_event_loop ()) {
#line 1385 "BatchImport.c"
		result = IMPORT_RESULT_USER_ABORT;
		return result;
	}
#line 300 "BatchImport.vala"
	self->priv->import_file_count++;
#line 301 "BatchImport.vala"
	if (self->priv->fail_every > 0) {
#line 302 "BatchImport.vala"
		if ((self->priv->import_file_count % self->priv->fail_every) == 0) {
#line 1395 "BatchImport.c"
			result = IMPORT_RESULT_FILE_ERROR;
			return result;
		}
	}
#line 306 "BatchImport.vala"
	if (self->priv->skip_every > 0) {
#line 307 "BatchImport.vala"
		if ((self->priv->import_file_count % self->priv->skip_every) == 0) {
#line 1404 "BatchImport.c"
			result = IMPORT_RESULT_NOT_A_FILE;
			return result;
		}
	}
#line 311 "BatchImport.vala"
	_tmp0_ = NULL;
#line 1411 "BatchImport.c"
	import = (_tmp0_ = file, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	/* never copy file if already in library directory*/
#line 466 "AppWindow.vala"
	_tmp1_ = NULL;
#line 1416 "BatchImport.c"
	is_in_library_dir = (_tmp2_ = g_file_has_prefix (file, _tmp1_ = app_window_get_photos_dir ()), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
	_tmp3_ = FALSE;
#line 316 "BatchImport.vala"
	if (copy_to_library) {
#line 316 "BatchImport.vala"
		_tmp3_ = !is_in_library_dir;
#line 1423 "BatchImport.c"
	} else {
#line 316 "BatchImport.vala"
		_tmp3_ = FALSE;
#line 1427 "BatchImport.c"
	}
#line 316 "BatchImport.vala"
	if (_tmp3_) {
#line 1431 "BatchImport.c"
		GFile* copied;
		GFile* _tmp6_;
		ImportResult _tmp5_;
		GFile* _tmp4_;
		ImportResult _result_;
		char* _tmp8_;
		char* _tmp7_;
		GFile* _tmp10_;
		GFile* _tmp9_;
		copied = NULL;
		_tmp6_ = NULL;
		_tmp4_ = NULL;
		_result_ = (_tmp5_ = batch_import_copy_file (file, &_tmp4_), copied = (_tmp6_ = _tmp4_, (copied == NULL) ? NULL : (copied = (g_object_unref (copied), NULL)), _tmp6_), _tmp5_);
#line 319 "BatchImport.vala"
		if (_result_ != IMPORT_RESULT_SUCCESS) {
#line 1447 "BatchImport.c"
			result = _result_;
			(copied == NULL) ? NULL : (copied = (g_object_unref (copied), NULL));
			(import == NULL) ? NULL : (import = (g_object_unref (import), NULL));
			return result;
		}
#line 952 "gio-2.0.vapi"
		_tmp8_ = NULL;
#line 952 "gio-2.0.vapi"
		_tmp7_ = NULL;
#line 322 "BatchImport.vala"
		g_debug ("BatchImport.vala:322: Copied %s into library at %s", _tmp7_ = g_file_get_path (file), _tmp8_ = g_file_get_path (copied));
#line 1459 "BatchImport.c"
		_tmp8_ = (g_free (_tmp8_), NULL);
		_tmp7_ = (g_free (_tmp7_), NULL);
		_tmp10_ = NULL;
#line 324 "BatchImport.vala"
		_tmp9_ = NULL;
#line 324 "BatchImport.vala"
		import = (_tmp10_ = (_tmp9_ = copied, (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (import == NULL) ? NULL : (import = (g_object_unref (import), NULL)), _tmp10_);
#line 1467 "BatchImport.c"
		(copied == NULL) ? NULL : (copied = (g_object_unref (copied), NULL));
	}
	photo = NULL;
	_tmp13_ = NULL;
	_tmp11_ = NULL;
	_result_ = (_tmp12_ = photo_import (import, &self->priv->import_id, &_tmp11_), photo = (_tmp13_ = _tmp11_, (photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL)), _tmp13_), _tmp12_);
#line 329 "BatchImport.vala"
	if (_result_ != IMPORT_RESULT_SUCCESS) {
#line 330 "BatchImport.vala"
		if (copy_to_library) {
#line 1478 "BatchImport.c"
			{
#line 332 "BatchImport.vala"
				g_file_delete (import, NULL, &_inner_error_);
#line 1482 "BatchImport.c"
				if (_inner_error_ != NULL) {
					goto __catch44_g_error;
					goto __finally44;
				}
			}
			goto __finally44;
			__catch44_g_error:
			{
				GError * err;
				err = _inner_error_;
				_inner_error_ = NULL;
				{
					char* _tmp14_;
#line 952 "gio-2.0.vapi"
					_tmp14_ = NULL;
#line 334 "BatchImport.vala"
					g_critical ("BatchImport.vala:334: Unable to delete copy of imported file %s: %s", _tmp14_ = g_file_get_path (import), err->message);
#line 1500 "BatchImport.c"
					_tmp14_ = (g_free (_tmp14_), NULL);
					(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
				}
			}
			__finally44:
			if (_inner_error_ != NULL) {
				(import == NULL) ? NULL : (import = (g_object_unref (import), NULL));
				(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return 0;
			}
		}
		result = _result_;
		(import == NULL) ? NULL : (import = (g_object_unref (import), NULL));
		(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		return result;
	}
#line 342 "BatchImport.vala"
	sorted_list_add (self->priv->success, photo);
#line 1521 "BatchImport.c"
	/* report to AppWindow for system-wide inclusion*/
#line 446 "AppWindow.vala"
	_tmp15_ = NULL;
#line 345 "BatchImport.vala"
	app_window_photo_imported (_tmp15_ = app_window_get_instance (), photo);
#line 1527 "BatchImport.c"
	(_tmp15_ == NULL) ? NULL : (_tmp15_ = (g_object_unref (_tmp15_), NULL));
	/* report to observers*/
#line 348 "BatchImport.vala"
	g_signal_emit_by_name (self, "imported", photo);
#line 1532 "BatchImport.c"
	result = IMPORT_RESULT_SUCCESS;
	(import == NULL) ? NULL : (import = (g_object_unref (import), NULL));
	(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
	return result;
}


#line 20 "BatchImport.vala"
static gint64 batch_import_date_comparator_real_compare (Comparator* base, Photo* photo_a, Photo* photo_b) {
#line 1542 "BatchImport.c"
	BatchImportDateComparator * self;
	gint64 result;
	self = BATCH_IMPORT_DATE_COMPARATOR (base);
#line 20 "BatchImport.vala"
	g_return_val_if_fail (IS_PHOTO (photo_a), 0LL);
#line 20 "BatchImport.vala"
	g_return_val_if_fail (IS_PHOTO (photo_b), 0LL);
#line 1550 "BatchImport.c"
	result = (gint64) (photo_get_exposure_time (photo_a) - photo_get_exposure_time (photo_b));
	return result;
}


#line 19 "BatchImport.vala"
static BatchImportDateComparator* batch_import_date_comparator_construct (GType object_type) {
#line 1558 "BatchImport.c"
	BatchImportDateComparator* self;
	self = (BatchImportDateComparator*) g_type_create_instance (object_type);
	return self;
}


#line 19 "BatchImport.vala"
static BatchImportDateComparator* batch_import_date_comparator_new (void) {
#line 19 "BatchImport.vala"
	return batch_import_date_comparator_construct (BATCH_IMPORT_TYPE_DATE_COMPARATOR);
#line 1569 "BatchImport.c"
}


static void batch_import_date_comparator_class_init (BatchImportDateComparatorClass * klass) {
	batch_import_date_comparator_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->compare = batch_import_date_comparator_real_compare;
}


static void batch_import_date_comparator_instance_init (BatchImportDateComparator * self) {
}


static GType batch_import_date_comparator_get_type (void) {
	static GType batch_import_date_comparator_type_id = 0;
	if (batch_import_date_comparator_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (BatchImportDateComparatorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) batch_import_date_comparator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BatchImportDateComparator), 0, (GInstanceInitFunc) batch_import_date_comparator_instance_init, NULL };
		batch_import_date_comparator_type_id = g_type_register_static (TYPE_COMPARATOR, "BatchImportDateComparator", &g_define_type_info, 0);
	}
	return batch_import_date_comparator_type_id;
}


static void value_batch_import_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_batch_import_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		batch_import_unref (value->data[0].v_pointer);
	}
}


static void value_batch_import_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = batch_import_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_batch_import_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_batch_import_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		BatchImport* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = batch_import_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_batch_import_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	BatchImport** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = batch_import_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_batch_import (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecBatchImport* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_BATCH_IMPORT), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_batch_import (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_BATCH_IMPORT), NULL);
	return value->data[0].v_pointer;
}


void value_set_batch_import (GValue* value, gpointer v_object) {
	BatchImport* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_BATCH_IMPORT));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_BATCH_IMPORT));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		batch_import_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		batch_import_unref (old);
	}
}


static void batch_import_class_init (BatchImportClass * klass) {
	batch_import_parent_class = g_type_class_peek_parent (klass);
	BATCH_IMPORT_CLASS (klass)->finalize = batch_import_finalize;
	g_type_class_add_private (klass, sizeof (BatchImportPrivate));
	g_signal_new ("starting", TYPE_BATCH_IMPORT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("imported", TYPE_BATCH_IMPORT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, TYPE_PHOTO);
	g_signal_new ("import_job_failed", TYPE_BATCH_IMPORT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__ENUM_POINTER_OBJECT, G_TYPE_NONE, 3, TYPE_IMPORT_RESULT, TYPE_BATCH_IMPORT_JOB, G_TYPE_FILE);
	g_signal_new ("import_complete", TYPE_BATCH_IMPORT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__BOXED_OBJECT_OBJECT_OBJECT, G_TYPE_NONE, 4, TYPE_IMPORT_ID, TYPE_SORTED_LIST, GEE_TYPE_ARRAY_LIST, GEE_TYPE_ARRAY_LIST);
}


static void batch_import_instance_init (BatchImport * self) {
	ImportID _tmp0_ = {0};
	self->priv = BATCH_IMPORT_GET_PRIVATE (self);
	self->priv->ref_holder = NULL;
	self->priv->success = NULL;
	self->priv->failed = NULL;
	self->priv->skipped = NULL;
	self->priv->import_id = (import_id_init (&_tmp0_, IMPORT_ID_INVALID), _tmp0_);
	self->priv->scheduled = FALSE;
	self->priv->user_aborted = FALSE;
	self->priv->import_file_count = 0;
	self->priv->fail_every = 0;
	self->priv->skip_every = 0;
	self->ref_count = 1;
}


static void batch_import_finalize (BatchImport* obj) {
	BatchImport * self;
	self = BATCH_IMPORT (obj);
	(self->priv->jobs == NULL) ? NULL : (self->priv->jobs = (g_object_unref (self->priv->jobs), NULL));
	self->priv->name = (g_free (self->priv->name), NULL);
	(self->priv->ref_holder == NULL) ? NULL : (self->priv->ref_holder = (batch_import_unref (self->priv->ref_holder), NULL));
	(self->priv->success == NULL) ? NULL : (self->priv->success = (g_object_unref (self->priv->success), NULL));
	(self->priv->failed == NULL) ? NULL : (self->priv->failed = (g_object_unref (self->priv->failed), NULL));
	(self->priv->skipped == NULL) ? NULL : (self->priv->skipped = (g_object_unref (self->priv->skipped), NULL));
}


GType batch_import_get_type (void) {
	static GType batch_import_type_id = 0;
	if (batch_import_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_batch_import_init, value_batch_import_free_value, value_batch_import_copy_value, value_batch_import_peek_pointer, "p", value_batch_import_collect_value, "p", value_batch_import_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (BatchImportClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) batch_import_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BatchImport), 0, (GInstanceInitFunc) batch_import_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		batch_import_type_id = g_type_register_fundamental (g_type_fundamental_next (), "BatchImport", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return batch_import_type_id;
}


gpointer batch_import_ref (gpointer instance) {
	BatchImport* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void batch_import_unref (gpointer instance) {
	BatchImport* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		BATCH_IMPORT_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}



static void g_cclosure_user_marshal_VOID__ENUM_POINTER_OBJECT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__ENUM_POINTER_OBJECT) (gpointer data1, gint arg_1, gpointer arg_2, gpointer arg_3, gpointer data2);
	register GMarshalFunc_VOID__ENUM_POINTER_OBJECT callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 4);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__ENUM_POINTER_OBJECT) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_enum (param_values + 1), value_get_batch_import_job (param_values + 2), g_value_get_object (param_values + 3), data2);
}


static void g_cclosure_user_marshal_VOID__BOXED_OBJECT_OBJECT_OBJECT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__BOXED_OBJECT_OBJECT_OBJECT) (gpointer data1, gpointer arg_1, gpointer arg_2, gpointer arg_3, gpointer arg_4, gpointer data2);
	register GMarshalFunc_VOID__BOXED_OBJECT_OBJECT_OBJECT callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 5);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__BOXED_OBJECT_OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_boxed (param_values + 1), g_value_get_object (param_values + 2), g_value_get_object (param_values + 3), g_value_get_object (param_values + 4), data2);
}



