/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>


#define TYPE_BOX_LOCATION (box_location_get_type ())

#define TYPE_BOX_COMPLEMENTS (box_complements_get_type ())

#define TYPE_BOX (box_get_type ())
typedef struct _Box Box;

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

typedef enum  {
	BOX_LOCATION_OUTSIDE,
	BOX_LOCATION_INSIDE,
	BOX_LOCATION_TOP_SIDE,
	BOX_LOCATION_LEFT_SIDE,
	BOX_LOCATION_RIGHT_SIDE,
	BOX_LOCATION_BOTTOM_SIDE,
	BOX_LOCATION_TOP_LEFT,
	BOX_LOCATION_BOTTOM_LEFT,
	BOX_LOCATION_TOP_RIGHT,
	BOX_LOCATION_BOTTOM_RIGHT
} BoxLocation;

typedef enum  {
	BOX_COMPLEMENTS_NONE,
	BOX_COMPLEMENTS_VERTICAL,
	BOX_COMPLEMENTS_HORIZONTAL,
	BOX_COMPLEMENTS_BOTH
} BoxComplements;

struct _Box {
	gint left;
	gint top;
	gint right;
	gint bottom;
};

struct _Dimensions {
	gint width;
	gint height;
};



GType box_location_get_type (void);
GType box_complements_get_type (void);
BoxComplements box_complements_derive (gboolean horizontal_complement, gboolean vertical_complement);
GType box_get_type (void);
Box* box_dup (const Box* self);
void box_free (Box* self);
#define BOX_HAND_GRENADES 12
void box_init (Box *self, gint left, gint top, gint right, gint bottom);
Box box_from_rectangle (const GdkRectangle* rect);
Box box_from_allocation (const GtkAllocation* alloc);
Box box_from_points (const GdkPoint* corner1, const GdkPoint* corner2);
gint box_get_width (Box *self);
gint box_get_height (Box *self);
gboolean box_is_valid (Box *self);
gboolean box_equals (Box *self, const Box* box);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
Box box_get_scaled (Box *self, const Dimensions* scaled);
Box box_get_scaled_proportional (Box *self, const Dimensions* orig, const Dimensions* scaled);
Box box_get_offset (Box *self, gint xofs, gint yofs);
void dimensions_init (Dimensions *self, gint width, gint height);
Dimensions box_get_dimensions (Box *self);
void box_get_points (Box *self, GdkPoint* top_left, GdkPoint* bottom_right);
GdkRectangle box_get_rectangle (Box *self);
Box box_rotate_clockwise (Box *self, const Dimensions* space);
Box box_rotate_counterclockwise (Box *self, const Dimensions* space);
Box box_flip_left_to_right (Box *self, const Dimensions* space);
Box box_flip_top_to_bottom (Box *self, const Dimensions* space);
gboolean box_intersects (Box *self, const Box* compare);
Box box_get_reduced (Box *self, gint amount);
Box box_get_expanded (Box *self, gint amount);
gboolean box_contains (Box *self, const GdkPoint* point);
BoxComplements box_resized_complements (Box *self, const Box* resized, Box* horizontal, gboolean* horizontal_enlarged, Box* vertical, gboolean* vertical_enlarged);
BoxComplements box_shifted_complements (Box *self, const Box* shifted, Box* horizontal_this, Box* vertical_this, Box* horizontal_shifted, Box* vertical_shifted);
Box box_rubber_band (Box *self, const GdkPoint* point);
char* box_to_string (Box *self);
static gboolean box_in_between (double pos, gint top, gint bottom);
static gboolean box_in_zone (double pos, gint zone);
static gboolean box_near_in_between (double pos, gint top, gint bottom);
BoxLocation box_approx_location (Box *self, gint x, gint y);




GType box_location_get_type (void) {
	static GType box_location_type_id = 0;
	if (G_UNLIKELY (box_location_type_id == 0)) {
		static const GEnumValue values[] = {{BOX_LOCATION_OUTSIDE, "BOX_LOCATION_OUTSIDE", "outside"}, {BOX_LOCATION_INSIDE, "BOX_LOCATION_INSIDE", "inside"}, {BOX_LOCATION_TOP_SIDE, "BOX_LOCATION_TOP_SIDE", "top-side"}, {BOX_LOCATION_LEFT_SIDE, "BOX_LOCATION_LEFT_SIDE", "left-side"}, {BOX_LOCATION_RIGHT_SIDE, "BOX_LOCATION_RIGHT_SIDE", "right-side"}, {BOX_LOCATION_BOTTOM_SIDE, "BOX_LOCATION_BOTTOM_SIDE", "bottom-side"}, {BOX_LOCATION_TOP_LEFT, "BOX_LOCATION_TOP_LEFT", "top-left"}, {BOX_LOCATION_BOTTOM_LEFT, "BOX_LOCATION_BOTTOM_LEFT", "bottom-left"}, {BOX_LOCATION_TOP_RIGHT, "BOX_LOCATION_TOP_RIGHT", "top-right"}, {BOX_LOCATION_BOTTOM_RIGHT, "BOX_LOCATION_BOTTOM_RIGHT", "bottom-right"}, {0, NULL, NULL}};
		box_location_type_id = g_enum_register_static ("BoxLocation", values);
	}
	return box_location_type_id;
}


#line 26 "Box.vala"
BoxComplements box_complements_derive (gboolean horizontal_complement, gboolean vertical_complement) {
#line 119 "Box.c"
	BoxComplements result;
	gboolean _tmp0_;
	_tmp0_ = FALSE;
#line 27 "Box.vala"
	if (horizontal_complement) {
#line 27 "Box.vala"
		_tmp0_ = vertical_complement;
#line 127 "Box.c"
	} else {
#line 27 "Box.vala"
		_tmp0_ = FALSE;
#line 131 "Box.c"
	}
#line 27 "Box.vala"
	if (_tmp0_) {
#line 135 "Box.c"
		result = BOX_COMPLEMENTS_BOTH;
		return result;
	} else {
#line 29 "Box.vala"
		if (horizontal_complement) {
#line 141 "Box.c"
			result = BOX_COMPLEMENTS_HORIZONTAL;
			return result;
		} else {
#line 31 "Box.vala"
			if (vertical_complement) {
#line 147 "Box.c"
				result = BOX_COMPLEMENTS_VERTICAL;
				return result;
			}
		}
	}
	result = BOX_COMPLEMENTS_NONE;
	return result;
}



GType box_complements_get_type (void) {
	static GType box_complements_type_id = 0;
	if (G_UNLIKELY (box_complements_type_id == 0)) {
		static const GEnumValue values[] = {{BOX_COMPLEMENTS_NONE, "BOX_COMPLEMENTS_NONE", "none"}, {BOX_COMPLEMENTS_VERTICAL, "BOX_COMPLEMENTS_VERTICAL", "vertical"}, {BOX_COMPLEMENTS_HORIZONTAL, "BOX_COMPLEMENTS_HORIZONTAL", "horizontal"}, {BOX_COMPLEMENTS_BOTH, "BOX_COMPLEMENTS_BOTH", "both"}, {0, NULL, NULL}};
		box_complements_type_id = g_enum_register_static ("BoxComplements", values);
	}
	return box_complements_type_id;
}


#line 46 "Box.vala"
void box_init (Box *self, gint left, gint top, gint right, gint bottom) {
#line 171 "Box.c"
	memset (self, 0, sizeof (Box));
#line 47 "Box.vala"
	g_assert (left >= 0);
#line 48 "Box.vala"
	g_assert (top >= 0);
#line 49 "Box.vala"
	g_assert (right >= left);
#line 50 "Box.vala"
	g_assert (bottom >= top);
#line 52 "Box.vala"
	(*self).left = left;
#line 53 "Box.vala"
	(*self).top = top;
#line 54 "Box.vala"
	(*self).right = right;
#line 55 "Box.vala"
	(*self).bottom = bottom;
#line 189 "Box.c"
}


#line 58 "Box.vala"
Box box_from_rectangle (const GdkRectangle* rect) {
#line 195 "Box.c"
	Box result;
	Box _tmp0_ = {0};
	result = (box_init (&_tmp0_, (*rect).x, (*rect).y, ((*rect).x + (*rect).width) - 1, ((*rect).y + (*rect).height) - 1), _tmp0_);
	return result;
}


#line 62 "Box.vala"
Box box_from_allocation (const GtkAllocation* alloc) {
#line 205 "Box.c"
	Box result;
	Box _tmp0_ = {0};
	result = (box_init (&_tmp0_, (*alloc).x, (*alloc).y, ((*alloc).x + (*alloc).width) - 1, ((*alloc).y + (*alloc).height) - 1), _tmp0_);
	return result;
}


/* This ensures a proper box is built from the points supplied, no matter the relationship
 between the two points*/
#line 68 "Box.vala"
Box box_from_points (const GdkPoint* corner1, const GdkPoint* corner2) {
#line 217 "Box.c"
	Box result;
	Box _tmp0_ = {0};
	result = (box_init (&_tmp0_, MIN ((*corner1).x, (*corner2).x), MIN ((*corner1).y, (*corner2).y), MAX ((*corner1).x, (*corner2).x), MAX ((*corner1).y, (*corner2).y)), _tmp0_);
	return result;
}


#line 73 "Box.vala"
gint box_get_width (Box *self) {
#line 227 "Box.c"
	gint result;
#line 74 "Box.vala"
	g_assert ((*self).right >= (*self).left);
#line 231 "Box.c"
	result = ((*self).right - (*self).left) + 1;
	return result;
}


#line 79 "Box.vala"
gint box_get_height (Box *self) {
#line 239 "Box.c"
	gint result;
#line 80 "Box.vala"
	g_assert ((*self).bottom >= (*self).top);
#line 243 "Box.c"
	result = ((*self).bottom - (*self).top) + 1;
	return result;
}


#line 85 "Box.vala"
gboolean box_is_valid (Box *self) {
#line 251 "Box.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
#line 86 "Box.vala"
	if ((*self).left >= 0) {
#line 86 "Box.vala"
		_tmp2_ = (*self).top >= 0;
#line 263 "Box.c"
	} else {
#line 86 "Box.vala"
		_tmp2_ = FALSE;
#line 267 "Box.c"
	}
#line 86 "Box.vala"
	if (_tmp2_) {
#line 86 "Box.vala"
		_tmp1_ = (*self).right >= (*self).left;
#line 273 "Box.c"
	} else {
#line 86 "Box.vala"
		_tmp1_ = FALSE;
#line 277 "Box.c"
	}
#line 86 "Box.vala"
	if (_tmp1_) {
#line 86 "Box.vala"
		_tmp0_ = (*self).bottom >= (*self).top;
#line 283 "Box.c"
	} else {
#line 86 "Box.vala"
		_tmp0_ = FALSE;
#line 287 "Box.c"
	}
	result = _tmp0_;
	return result;
}


#line 89 "Box.vala"
gboolean box_equals (Box *self, const Box* box) {
#line 296 "Box.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
#line 90 "Box.vala"
	if ((*self).left == (*box).left) {
#line 90 "Box.vala"
		_tmp2_ = (*self).top == (*box).top;
#line 308 "Box.c"
	} else {
#line 90 "Box.vala"
		_tmp2_ = FALSE;
#line 312 "Box.c"
	}
#line 90 "Box.vala"
	if (_tmp2_) {
#line 90 "Box.vala"
		_tmp1_ = (*self).right == (*box).right;
#line 318 "Box.c"
	} else {
#line 90 "Box.vala"
		_tmp1_ = FALSE;
#line 322 "Box.c"
	}
#line 90 "Box.vala"
	if (_tmp1_) {
#line 90 "Box.vala"
		_tmp0_ = (*self).bottom == (*box).bottom;
#line 328 "Box.c"
	} else {
#line 90 "Box.vala"
		_tmp0_ = FALSE;
#line 332 "Box.c"
	}
	result = _tmp0_;
	return result;
}


#line 93 "Box.vala"
Box box_get_scaled (Box *self, const Dimensions* scaled) {
#line 341 "Box.c"
	Box result;
	double x_scale;
	double y_scale;
	gint l;
	gint t;
	gint r;
	gint b;
	Box _tmp0_ = {0};
	Box box;
	gboolean _tmp1_;
	x_scale = ((double) (*scaled).width) / ((double) box_get_width (&(*self)));
	y_scale = ((double) (*scaled).height) / ((double) box_get_height (&(*self)));
	l = (gint) round ((*self).left * x_scale);
	t = (gint) round ((*self).top * y_scale);
	/* fix-up to match the scaled dimensions*/
	r = (l + (*scaled).width) - 1;
	b = (t + (*scaled).height) - 1;
	box = (box_init (&_tmp0_, l, t, r, b), _tmp0_);
	_tmp1_ = FALSE;
#line 105 "Box.vala"
	if (box_get_width (&box) == (*scaled).width) {
#line 105 "Box.vala"
		_tmp1_ = TRUE;
#line 365 "Box.c"
	} else {
#line 105 "Box.vala"
		_tmp1_ = box_get_height (&box) == (*scaled).height;
#line 369 "Box.c"
	}
#line 105 "Box.vala"
	g_assert (_tmp1_);
#line 373 "Box.c"
	result = box;
	return result;
}


#line 110 "Box.vala"
Box box_get_scaled_proportional (Box *self, const Dimensions* orig, const Dimensions* scaled) {
#line 381 "Box.c"
	Box result;
	double x_scale;
	double y_scale;
	gint l;
	gint t;
	gint r;
	gint b;
	Box _tmp0_ = {0};
	x_scale = ((double) (*scaled).width) / ((double) (*orig).width);
	y_scale = ((double) (*scaled).height) / ((double) (*orig).height);
	l = (gint) round ((*self).left * x_scale);
	t = (gint) round ((*self).top * y_scale);
	r = (gint) round ((*self).right * x_scale);
	b = (gint) round ((*self).bottom * y_scale);
	/* catch rounding errors*/
#line 120 "Box.vala"
	if (r >= (*scaled).width) {
#line 121 "Box.vala"
		r = (*scaled).width - 1;
#line 401 "Box.c"
	}
#line 123 "Box.vala"
	if (b >= (*scaled).height) {
#line 124 "Box.vala"
		b = (*scaled).height - 1;
#line 407 "Box.c"
	}
	result = (box_init (&_tmp0_, l, t, r, b), _tmp0_);
	return result;
}


#line 129 "Box.vala"
Box box_get_offset (Box *self, gint xofs, gint yofs) {
#line 416 "Box.c"
	Box result;
	Box _tmp0_ = {0};
	result = (box_init (&_tmp0_, (*self).left + xofs, (*self).top + yofs, (*self).right + xofs, (*self).bottom + yofs), _tmp0_);
	return result;
}


#line 133 "Box.vala"
Dimensions box_get_dimensions (Box *self) {
#line 426 "Box.c"
	Dimensions result;
	Dimensions _tmp0_ = {0};
	result = (dimensions_init (&_tmp0_, box_get_width (&(*self)), box_get_height (&(*self))), _tmp0_);
	return result;
}


#line 137 "Box.vala"
void box_get_points (Box *self, GdkPoint* top_left, GdkPoint* bottom_right) {
#line 138 "Box.vala"
	(*top_left).x = (*self).left;
#line 139 "Box.vala"
	(*top_left).y = (*self).top;
#line 141 "Box.vala"
	(*bottom_right).x = (*self).right;
#line 142 "Box.vala"
	(*bottom_right).y = (*self).bottom;
#line 444 "Box.c"
}


#line 145 "Box.vala"
GdkRectangle box_get_rectangle (Box *self) {
#line 450 "Box.c"
	GdkRectangle result;
	GdkRectangle _tmp0_ = {0};
	GdkRectangle rect;
	rect = (memset (&_tmp0_, 0, sizeof (GdkRectangle)), _tmp0_);
#line 147 "Box.vala"
	rect.x = (*self).left;
#line 148 "Box.vala"
	rect.y = (*self).top;
#line 149 "Box.vala"
	rect.width = box_get_width (&(*self));
#line 150 "Box.vala"
	rect.height = box_get_height (&(*self));
#line 463 "Box.c"
	result = rect;
	return result;
}


#line 155 "Box.vala"
Box box_rotate_clockwise (Box *self, const Dimensions* space) {
#line 471 "Box.c"
	Box result;
	gint l;
	gint t;
	gint r;
	gint b;
	Box _tmp0_ = {0};
	l = ((*space).width - (*self).bottom) - 1;
	t = (*self).left;
	r = ((*space).width - (*self).top) - 1;
	b = (*self).right;
	result = (box_init (&_tmp0_, l, t, r, b), _tmp0_);
	return result;
}


#line 164 "Box.vala"
Box box_rotate_counterclockwise (Box *self, const Dimensions* space) {
#line 489 "Box.c"
	Box result;
	gint l;
	gint t;
	gint r;
	gint b;
	Box _tmp0_ = {0};
	l = (*self).top;
	t = ((*space).height - (*self).right) - 1;
	r = (*self).bottom;
	b = ((*space).height - (*self).left) - 1;
	result = (box_init (&_tmp0_, l, t, r, b), _tmp0_);
	return result;
}


#line 173 "Box.vala"
Box box_flip_left_to_right (Box *self, const Dimensions* space) {
#line 507 "Box.c"
	Box result;
	gint l;
	gint r;
	Box _tmp0_ = {0};
	l = ((*space).width - (*self).right) - 1;
	r = ((*space).width - (*self).left) - 1;
	result = (box_init (&_tmp0_, l, (*self).top, r, (*self).bottom), _tmp0_);
	return result;
}


#line 180 "Box.vala"
Box box_flip_top_to_bottom (Box *self, const Dimensions* space) {
#line 521 "Box.c"
	Box result;
	gint t;
	gint b;
	Box _tmp0_ = {0};
	t = ((*space).height - (*self).bottom) - 1;
	b = ((*space).height - (*self).top) - 1;
	result = (box_init (&_tmp0_, (*self).left, t, (*self).right, b), _tmp0_);
	return result;
}


#line 187 "Box.vala"
gboolean box_intersects (Box *self, const Box* compare) {
#line 535 "Box.c"
	gboolean result;
	gint left_intersect;
	gint top_intersect;
	gint right_intersect;
	gint bottom_intersect;
	gboolean _tmp0_;
	left_intersect = MAX ((*self).left, (*compare).left);
	top_intersect = MAX ((*self).top, (*compare).top);
	right_intersect = MIN ((*self).right, (*compare).right);
	bottom_intersect = MIN ((*self).bottom, (*compare).bottom);
	_tmp0_ = FALSE;
#line 193 "Box.vala"
	if (right_intersect >= left_intersect) {
#line 193 "Box.vala"
		_tmp0_ = bottom_intersect >= top_intersect;
#line 551 "Box.c"
	} else {
#line 193 "Box.vala"
		_tmp0_ = FALSE;
#line 555 "Box.c"
	}
	result = _tmp0_;
	return result;
}


#line 196 "Box.vala"
Box box_get_reduced (Box *self, gint amount) {
#line 564 "Box.c"
	Box result;
	Box _tmp0_ = {0};
	result = (box_init (&_tmp0_, (*self).left + amount, (*self).top + amount, (*self).right - amount, (*self).bottom - amount), _tmp0_);
	return result;
}


#line 200 "Box.vala"
Box box_get_expanded (Box *self, gint amount) {
#line 574 "Box.c"
	Box result;
	Box _tmp0_ = {0};
	result = (box_init (&_tmp0_, (*self).left - amount, (*self).top - amount, (*self).right + amount, (*self).bottom + amount), _tmp0_);
	return result;
}


#line 204 "Box.vala"
gboolean box_contains (Box *self, const GdkPoint* point) {
#line 584 "Box.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
#line 205 "Box.vala"
	if ((*point).x >= (*self).left) {
#line 205 "Box.vala"
		_tmp2_ = (*point).x <= (*self).right;
#line 596 "Box.c"
	} else {
#line 205 "Box.vala"
		_tmp2_ = FALSE;
#line 600 "Box.c"
	}
#line 205 "Box.vala"
	if (_tmp2_) {
#line 205 "Box.vala"
		_tmp1_ = (*point).y >= (*self).top;
#line 606 "Box.c"
	} else {
#line 205 "Box.vala"
		_tmp1_ = FALSE;
#line 610 "Box.c"
	}
#line 205 "Box.vala"
	if (_tmp1_) {
#line 205 "Box.vala"
		_tmp0_ = (*point).y <= (*self).bottom;
#line 616 "Box.c"
	} else {
#line 205 "Box.vala"
		_tmp0_ = FALSE;
#line 620 "Box.c"
	}
	result = _tmp0_;
	return result;
}


/* This specialized method is only concerned with resized comparisons between two Boxes, 
 of which one is altered in up to two dimensions: (top or bottom) and/or (left or right).
 There may be overlap between the two returned Boxes.*/
#line 211 "Box.vala"
BoxComplements box_resized_complements (Box *self, const Box* resized, Box* horizontal, gboolean* horizontal_enlarged, Box* vertical, gboolean* vertical_enlarged) {
#line 632 "Box.c"
	BoxComplements result;
	gboolean _tmp0_;
	gint min_left;
	gint min_top;
	gint max_right;
	gint max_bottom;
	gboolean horizontal_complement;
	gboolean vertical_complement;
	_tmp0_ = FALSE;
#line 213 "Box.vala"
	if (box_get_width (&(*self)) != box_get_width (&(*resized))) {
#line 213 "Box.vala"
		_tmp0_ = TRUE;
#line 646 "Box.c"
	} else {
#line 213 "Box.vala"
		_tmp0_ = box_get_height (&(*self)) != box_get_height (&(*resized));
#line 650 "Box.c"
	}
#line 213 "Box.vala"
	g_assert (_tmp0_);
#line 654 "Box.c"
	min_left = MIN ((*self).left, (*resized).left);
	min_top = MIN ((*self).top, (*resized).top);
	max_right = MAX ((*self).right, (*resized).right);
	max_bottom = MAX ((*self).bottom, (*resized).bottom);
	horizontal_complement = TRUE;
#line 221 "Box.vala"
	if ((*resized).top < (*self).top) {
#line 662 "Box.c"
		Box _tmp1_ = {0};
		/* enlarged from top*/
#line 223 "Box.vala"
		(*horizontal) = (box_init (&_tmp1_, min_left, (*resized).top, max_right, (*self).top), _tmp1_);
#line 224 "Box.vala"
		(*horizontal_enlarged) = TRUE;
#line 669 "Box.c"
	} else {
#line 225 "Box.vala"
		if ((*resized).top > (*self).top) {
#line 673 "Box.c"
			Box _tmp2_ = {0};
			/* shrunk from top*/
#line 227 "Box.vala"
			(*horizontal) = (box_init (&_tmp2_, min_left, (*self).top, max_right, (*resized).top), _tmp2_);
#line 228 "Box.vala"
			(*horizontal_enlarged) = FALSE;
#line 680 "Box.c"
		} else {
#line 229 "Box.vala"
			if ((*resized).bottom < (*self).bottom) {
#line 684 "Box.c"
				Box _tmp3_ = {0};
				/* shrunk from bottom*/
#line 231 "Box.vala"
				(*horizontal) = (box_init (&_tmp3_, min_left, (*resized).bottom, max_right, (*self).bottom), _tmp3_);
#line 232 "Box.vala"
				(*horizontal_enlarged) = FALSE;
#line 691 "Box.c"
			} else {
#line 233 "Box.vala"
				if ((*resized).bottom > (*self).bottom) {
#line 695 "Box.c"
					Box _tmp4_ = {0};
					/* enlarged from bottom*/
#line 235 "Box.vala"
					(*horizontal) = (box_init (&_tmp4_, min_left, (*self).bottom, max_right, (*resized).bottom), _tmp4_);
#line 236 "Box.vala"
					(*horizontal_enlarged) = TRUE;
#line 702 "Box.c"
				} else {
#line 238 "Box.vala"
					horizontal_complement = FALSE;
#line 706 "Box.c"
				}
			}
		}
	}
	vertical_complement = TRUE;
#line 242 "Box.vala"
	if ((*resized).left < (*self).left) {
#line 714 "Box.c"
		Box _tmp5_ = {0};
		/* enlarged left*/
#line 244 "Box.vala"
		(*vertical) = (box_init (&_tmp5_, (*resized).left, min_top, (*self).left, max_bottom), _tmp5_);
#line 245 "Box.vala"
		(*vertical_enlarged) = TRUE;
#line 721 "Box.c"
	} else {
#line 246 "Box.vala"
		if ((*resized).left > (*self).left) {
#line 725 "Box.c"
			Box _tmp6_ = {0};
			/* shrunk left*/
#line 248 "Box.vala"
			(*vertical) = (box_init (&_tmp6_, (*self).left, min_top, (*resized).left, max_bottom), _tmp6_);
#line 249 "Box.vala"
			(*vertical_enlarged) = FALSE;
#line 732 "Box.c"
		} else {
#line 250 "Box.vala"
			if ((*resized).right < (*self).right) {
#line 736 "Box.c"
				Box _tmp7_ = {0};
				/* shrunk right*/
#line 252 "Box.vala"
				(*vertical) = (box_init (&_tmp7_, (*resized).right, min_top, (*self).right, max_bottom), _tmp7_);
#line 253 "Box.vala"
				(*vertical_enlarged) = FALSE;
#line 743 "Box.c"
			} else {
#line 254 "Box.vala"
				if ((*resized).right > (*self).right) {
#line 747 "Box.c"
					Box _tmp8_ = {0};
					/* enlarged right*/
#line 256 "Box.vala"
					(*vertical) = (box_init (&_tmp8_, (*self).right, min_top, (*resized).right, max_bottom), _tmp8_);
#line 257 "Box.vala"
					(*vertical_enlarged) = TRUE;
#line 754 "Box.c"
				} else {
#line 259 "Box.vala"
					vertical_complement = FALSE;
#line 758 "Box.c"
				}
			}
		}
	}
	result = box_complements_derive (horizontal_complement, vertical_complement);
	return result;
}


/* This specialized method is only concerned with the complements of identical Boxes in two
 different, spatial locations.  There may be overlap between the four returned Boxes.  However,
 no portion of any of the four boxes will be outside the scope of the two compared boxes.*/
#line 268 "Box.vala"
BoxComplements box_shifted_complements (Box *self, const Box* shifted, Box* horizontal_this, Box* vertical_this, Box* horizontal_shifted, Box* vertical_shifted) {
#line 773 "Box.c"
	BoxComplements result;
	gboolean horizontal_complement;
	gboolean _tmp0_;
	gboolean vertical_complement;
	gboolean _tmp6_;
#line 270 "Box.vala"
	g_assert (box_get_width (&(*self)) == box_get_width (&(*shifted)));
#line 271 "Box.vala"
	g_assert (box_get_height (&(*self)) == box_get_height (&(*shifted)));
#line 783 "Box.c"
	horizontal_complement = TRUE;
	_tmp0_ = FALSE;
#line 274 "Box.vala"
	if ((*shifted).top < (*self).top) {
#line 274 "Box.vala"
		_tmp0_ = (*shifted).bottom > (*self).top;
#line 790 "Box.c"
	} else {
#line 274 "Box.vala"
		_tmp0_ = FALSE;
#line 794 "Box.c"
	}
#line 274 "Box.vala"
	if (_tmp0_) {
#line 798 "Box.c"
		Box _tmp1_ = {0};
		Box _tmp2_ = {0};
		/* shifted up*/
#line 276 "Box.vala"
		(*horizontal_this) = (box_init (&_tmp1_, (*self).left, (*shifted).bottom, (*self).right, (*self).bottom), _tmp1_);
#line 277 "Box.vala"
		(*horizontal_shifted) = (box_init (&_tmp2_, (*shifted).left, (*shifted).top, (*shifted).right, (*self).top), _tmp2_);
#line 806 "Box.c"
	} else {
		gboolean _tmp3_;
		_tmp3_ = FALSE;
#line 278 "Box.vala"
		if ((*shifted).top > (*self).top) {
#line 278 "Box.vala"
			_tmp3_ = (*shifted).top < (*self).bottom;
#line 814 "Box.c"
		} else {
#line 278 "Box.vala"
			_tmp3_ = FALSE;
#line 818 "Box.c"
		}
#line 278 "Box.vala"
		if (_tmp3_) {
#line 822 "Box.c"
			Box _tmp4_ = {0};
			Box _tmp5_ = {0};
			/* shifted down*/
#line 280 "Box.vala"
			(*horizontal_this) = (box_init (&_tmp4_, (*self).left, (*self).top, (*self).right, (*shifted).top), _tmp4_);
#line 281 "Box.vala"
			(*horizontal_shifted) = (box_init (&_tmp5_, (*shifted).left, (*self).bottom, (*shifted).right, (*shifted).bottom), _tmp5_);
#line 830 "Box.c"
		} else {
			/* no vertical shift*/
#line 284 "Box.vala"
			horizontal_complement = FALSE;
#line 835 "Box.c"
		}
	}
	vertical_complement = TRUE;
	_tmp6_ = FALSE;
#line 288 "Box.vala"
	if ((*shifted).left < (*self).left) {
#line 288 "Box.vala"
		_tmp6_ = (*shifted).right > (*self).left;
#line 844 "Box.c"
	} else {
#line 288 "Box.vala"
		_tmp6_ = FALSE;
#line 848 "Box.c"
	}
#line 288 "Box.vala"
	if (_tmp6_) {
#line 852 "Box.c"
		Box _tmp7_ = {0};
		Box _tmp8_ = {0};
		/* shifted left*/
#line 290 "Box.vala"
		(*vertical_this) = (box_init (&_tmp7_, (*shifted).right, (*self).top, (*self).right, (*self).bottom), _tmp7_);
#line 291 "Box.vala"
		(*vertical_shifted) = (box_init (&_tmp8_, (*shifted).left, (*shifted).top, (*self).left, (*shifted).bottom), _tmp8_);
#line 860 "Box.c"
	} else {
		gboolean _tmp9_;
		_tmp9_ = FALSE;
#line 292 "Box.vala"
		if ((*shifted).left > (*self).left) {
#line 292 "Box.vala"
			_tmp9_ = (*shifted).left < (*self).right;
#line 868 "Box.c"
		} else {
#line 292 "Box.vala"
			_tmp9_ = FALSE;
#line 872 "Box.c"
		}
#line 292 "Box.vala"
		if (_tmp9_) {
#line 876 "Box.c"
			Box _tmp10_ = {0};
			Box _tmp11_ = {0};
			/* shifted right*/
#line 294 "Box.vala"
			(*vertical_this) = (box_init (&_tmp10_, (*self).left, (*self).top, (*shifted).left, (*self).bottom), _tmp10_);
#line 295 "Box.vala"
			(*vertical_shifted) = (box_init (&_tmp11_, (*self).right, (*shifted).top, (*shifted).right, (*shifted).bottom), _tmp11_);
#line 884 "Box.c"
		} else {
			/* no horizontal shift*/
#line 298 "Box.vala"
			vertical_complement = FALSE;
#line 889 "Box.c"
		}
	}
	result = box_complements_derive (horizontal_complement, vertical_complement);
	return result;
}


#line 304 "Box.vala"
Box box_rubber_band (Box *self, const GdkPoint* point) {
#line 899 "Box.c"
	Box result;
	gint t;
	gint b;
	gint l;
	gint r;
	Box _tmp0_ = {0};
#line 305 "Box.vala"
	g_assert ((*point).x >= 0);
#line 306 "Box.vala"
	g_assert ((*point).y >= 0);
#line 910 "Box.c"
	t = MIN ((*self).top, (*point).y);
	b = MAX ((*self).bottom, (*point).y);
	l = MIN ((*self).left, (*point).x);
	r = MAX ((*self).right, (*point).x);
	result = (box_init (&_tmp0_, l, t, r, b), _tmp0_);
	return result;
}


#line 316 "Box.vala"
char* box_to_string (Box *self) {
#line 922 "Box.c"
	char* result;
	result = g_strdup_printf ("%d,%d %d,%d", (*self).left, (*self).top, (*self).right, (*self).bottom);
	return result;
}


#line 320 "Box.vala"
static gboolean box_in_zone (double pos, gint zone) {
#line 931 "Box.c"
	gboolean result;
	gint top_zone;
	gint bottom_zone;
	top_zone = zone - BOX_HAND_GRENADES;
	bottom_zone = zone + BOX_HAND_GRENADES;
	result = box_in_between (pos, top_zone, bottom_zone);
	return result;
}


#line 327 "Box.vala"
static gboolean box_in_between (double pos, gint top, gint bottom) {
#line 944 "Box.c"
	gboolean result;
	gint ipos;
	gboolean _tmp0_;
	ipos = (gint) pos;
	_tmp0_ = FALSE;
#line 330 "Box.vala"
	if (ipos > top) {
#line 330 "Box.vala"
		_tmp0_ = ipos < bottom;
#line 954 "Box.c"
	} else {
#line 330 "Box.vala"
		_tmp0_ = FALSE;
#line 958 "Box.c"
	}
	result = _tmp0_;
	return result;
}


#line 333 "Box.vala"
static gboolean box_near_in_between (double pos, gint top, gint bottom) {
#line 967 "Box.c"
	gboolean result;
	gint ipos;
	gint top_zone;
	gint bottom_zone;
	gboolean _tmp0_;
	ipos = (gint) pos;
	top_zone = top - BOX_HAND_GRENADES;
	bottom_zone = bottom + BOX_HAND_GRENADES;
	_tmp0_ = FALSE;
#line 338 "Box.vala"
	if (ipos > top_zone) {
#line 338 "Box.vala"
		_tmp0_ = ipos < bottom_zone;
#line 981 "Box.c"
	} else {
#line 338 "Box.vala"
		_tmp0_ = FALSE;
#line 985 "Box.c"
	}
	result = _tmp0_;
	return result;
}


#line 341 "Box.vala"
BoxLocation box_approx_location (Box *self, gint x, gint y) {
#line 994 "Box.c"
	BoxLocation result;
	gboolean near_width;
	gboolean near_height;
	gboolean _tmp0_;
	near_width = box_near_in_between ((double) x, (*self).left, (*self).right);
	near_height = box_near_in_between ((double) y, (*self).top, (*self).bottom);
	_tmp0_ = FALSE;
#line 345 "Box.vala"
	if (box_in_zone ((double) x, (*self).left)) {
#line 345 "Box.vala"
		_tmp0_ = near_height;
#line 1006 "Box.c"
	} else {
#line 345 "Box.vala"
		_tmp0_ = FALSE;
#line 1010 "Box.c"
	}
#line 345 "Box.vala"
	if (_tmp0_) {
#line 346 "Box.vala"
		if (box_in_zone ((double) y, (*self).top)) {
#line 1016 "Box.c"
			result = BOX_LOCATION_TOP_LEFT;
			return result;
		} else {
#line 348 "Box.vala"
			if (box_in_zone ((double) y, (*self).bottom)) {
#line 1022 "Box.c"
				result = BOX_LOCATION_BOTTOM_LEFT;
				return result;
			} else {
				result = BOX_LOCATION_LEFT_SIDE;
				return result;
			}
		}
	} else {
		gboolean _tmp1_;
		_tmp1_ = FALSE;
#line 353 "Box.vala"
		if (box_in_zone ((double) x, (*self).right)) {
#line 353 "Box.vala"
			_tmp1_ = near_height;
#line 1037 "Box.c"
		} else {
#line 353 "Box.vala"
			_tmp1_ = FALSE;
#line 1041 "Box.c"
		}
#line 353 "Box.vala"
		if (_tmp1_) {
#line 354 "Box.vala"
			if (box_in_zone ((double) y, (*self).top)) {
#line 1047 "Box.c"
				result = BOX_LOCATION_TOP_RIGHT;
				return result;
			} else {
#line 356 "Box.vala"
				if (box_in_zone ((double) y, (*self).bottom)) {
#line 1053 "Box.c"
					result = BOX_LOCATION_BOTTOM_RIGHT;
					return result;
				} else {
					result = BOX_LOCATION_RIGHT_SIDE;
					return result;
				}
			}
		} else {
			gboolean _tmp2_;
			_tmp2_ = FALSE;
#line 361 "Box.vala"
			if (box_in_zone ((double) y, (*self).top)) {
#line 361 "Box.vala"
				_tmp2_ = near_width;
#line 1068 "Box.c"
			} else {
#line 361 "Box.vala"
				_tmp2_ = FALSE;
#line 1072 "Box.c"
			}
#line 361 "Box.vala"
			if (_tmp2_) {
#line 1076 "Box.c"
				/* if left or right was in zone, already caught top left & top right*/
				result = BOX_LOCATION_TOP_SIDE;
				return result;
			} else {
				gboolean _tmp3_;
				_tmp3_ = FALSE;
#line 364 "Box.vala"
				if (box_in_zone ((double) y, (*self).bottom)) {
#line 364 "Box.vala"
					_tmp3_ = near_width;
#line 1087 "Box.c"
				} else {
#line 364 "Box.vala"
					_tmp3_ = FALSE;
#line 1091 "Box.c"
				}
#line 364 "Box.vala"
				if (_tmp3_) {
#line 1095 "Box.c"
					/* if left or right was in zone, already caught bottom left & bottom right*/
					result = BOX_LOCATION_BOTTOM_SIDE;
					return result;
				} else {
					gboolean _tmp4_;
					_tmp4_ = FALSE;
#line 367 "Box.vala"
					if (box_in_between ((double) x, (*self).left, (*self).right)) {
#line 367 "Box.vala"
						_tmp4_ = box_in_between ((double) y, (*self).top, (*self).bottom);
#line 1106 "Box.c"
					} else {
#line 367 "Box.vala"
						_tmp4_ = FALSE;
#line 1110 "Box.c"
					}
#line 367 "Box.vala"
					if (_tmp4_) {
#line 1114 "Box.c"
						result = BOX_LOCATION_INSIDE;
						return result;
					} else {
						result = BOX_LOCATION_OUTSIDE;
						return result;
					}
				}
			}
		}
	}
}


Box* box_dup (const Box* self) {
	Box* dup;
	dup = g_new0 (Box, 1);
	memcpy (dup, self, sizeof (Box));
	return dup;
}


void box_free (Box* self) {
	g_free (self);
}


GType box_get_type (void) {
	static GType box_type_id = 0;
	if (box_type_id == 0) {
		box_type_id = g_boxed_type_register_static ("Box", (GBoxedCopyFunc) box_dup, (GBoxedFreeFunc) box_free);
	}
	return box_type_id;
}




