/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gee.h>


#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;
typedef struct _LayoutItemPrivate LayoutItemPrivate;

#define TYPE_COLLECTION_LAYOUT (collection_layout_get_type ())
#define COLLECTION_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_LAYOUT, CollectionLayout))
#define COLLECTION_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_LAYOUT, CollectionLayoutClass))
#define IS_COLLECTION_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_LAYOUT))
#define IS_COLLECTION_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_LAYOUT))
#define COLLECTION_LAYOUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_LAYOUT, CollectionLayoutClass))

typedef struct _CollectionLayout CollectionLayout;
typedef struct _CollectionLayoutClass CollectionLayoutClass;
typedef struct _CollectionLayoutPrivate CollectionLayoutPrivate;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

#define TYPE_COMPASS_POINT (compass_point_get_type ())

struct _LayoutItem {
	GtkAlignment parent_instance;
	LayoutItemPrivate * priv;
};

struct _LayoutItemClass {
	GtkAlignmentClass parent_class;
	GtkWidget* (*get_control_panel) (LayoutItem* self);
	void (*exposed) (LayoutItem* self);
	void (*unexposed) (LayoutItem* self);
	void (*select) (LayoutItem* self);
	void (*unselect) (LayoutItem* self);
};

struct _LayoutItemPrivate {
	GtkLabel* title;
	GtkFrame* frame;
	GtkImage* image;
	GdkPixbuf* pixbuf;
	gboolean selected;
	GtkVBox* vbox;
	gboolean title_displayed;
	gboolean brightened;
	gint col;
	gint row;
};

struct _CollectionLayout {
	GtkLayout parent_instance;
	CollectionLayoutPrivate * priv;
	SortedList* items;
};

struct _CollectionLayoutClass {
	GtkLayoutClass parent_class;
};

struct _CollectionLayoutPrivate {
	GtkLabel* message;
	gboolean in_view;
	gint last_width;
	gboolean refresh_on_resize;
	gint columns;
	gint rows;
};

typedef enum  {
	COMPASS_POINT_NORTH,
	COMPASS_POINT_SOUTH,
	COMPASS_POINT_EAST,
	COMPASS_POINT_WEST
} CompassPoint;


static gpointer layout_item_parent_class = NULL;
extern GdkColor app_window_BG_COLOR;
static gpointer collection_layout_parent_class = NULL;

GType layout_item_get_type (void);
#define LAYOUT_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_LAYOUT_ITEM, LayoutItemPrivate))
enum  {
	LAYOUT_ITEM_DUMMY_PROPERTY
};
#define LAYOUT_ITEM_LABEL_PADDING 4
#define LAYOUT_ITEM_FRAME_PADDING 4
#define LAYOUT_ITEM_TEXT_COLOR "#FFF"
#define LAYOUT_ITEM_SELECTED_COLOR "#0FF"
#define LAYOUT_ITEM_UNSELECTED_COLOR "#FFF"
#define LAYOUT_ITEM_BRIGHTEN_SHIFT 0x18
GdkColor parse_color (const char* spec);
GtkWidget* layout_item_get_control_panel (LayoutItem* self);
LayoutItem* layout_item_new (void);
LayoutItem* layout_item_construct (GType object_type);
static GtkWidget* layout_item_real_get_control_panel (LayoutItem* self);
void layout_item_set_title (LayoutItem* self, const char* text);
char* layout_item_get_title (LayoutItem* self);
void layout_item_exposed (LayoutItem* self);
static void layout_item_real_exposed (LayoutItem* self);
void layout_item_unexposed (LayoutItem* self);
static void layout_item_real_unexposed (LayoutItem* self);
void layout_item_display_title (LayoutItem* self, gboolean display);
void layout_item_set_image (LayoutItem* self, GdkPixbuf* pixbuf);
void layout_item_clear_image (LayoutItem* self);
void layout_item_set_image_size (LayoutItem* self, gint width, gint height);
void layout_item_select (LayoutItem* self);
static void layout_item_real_select (LayoutItem* self);
void layout_item_unselect (LayoutItem* self);
static void layout_item_real_unselect (LayoutItem* self);
gboolean layout_item_toggle_select (LayoutItem* self);
gboolean layout_item_is_selected (LayoutItem* self);
void layout_item_set_coordinates (LayoutItem* self, gint col, gint row);
gint layout_item_get_column (LayoutItem* self);
gint layout_item_get_row (LayoutItem* self);
void shift_colors (GdkPixbuf* pixbuf, gint red, gint green, gint blue, gint alpha);
void layout_item_brighten (LayoutItem* self);
void layout_item_unbrighten (LayoutItem* self);
static void layout_item_finalize (GObject* obj);
GType collection_layout_get_type (void);
GType sorted_list_get_type (void);
#define COLLECTION_LAYOUT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_COLLECTION_LAYOUT, CollectionLayoutPrivate))
enum  {
	COLLECTION_LAYOUT_DUMMY_PROPERTY
};
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
#define COLLECTION_LAYOUT_TOP_PADDING 16
#define COLLECTION_LAYOUT_BOTTOM_PADDING 16
#define COLLECTION_LAYOUT_ROW_GUTTER_PADDING 24
#define COLLECTION_LAYOUT_LEFT_PADDING 16
#define COLLECTION_LAYOUT_RIGHT_PADDING 16
#define COLLECTION_LAYOUT_COLUMN_GUTTER_PADDING 24
static void collection_layout_on_resize (CollectionLayout* self);
static void _collection_layout_on_resize_gtk_widget_size_allocate (CollectionLayout* _sender, const GdkRectangle* allocation, gpointer self);
CollectionLayout* collection_layout_new (void);
CollectionLayout* collection_layout_construct (GType object_type);
void collection_layout_clear (CollectionLayout* self);
static void collection_layout_display_message (CollectionLayout* self);
void collection_layout_set_message (CollectionLayout* self, const char* text);
void collection_layout_set_refresh_on_resize (CollectionLayout* self, gboolean refresh_on_resize);
void collection_layout_set_in_view (CollectionLayout* self, gboolean in_view);
void sorted_list_resort (SortedList* self, Comparator* new_cmp);
void collection_layout_set_comparator (CollectionLayout* self, Comparator* cmp);
gboolean sorted_list_add (SortedList* self, gconstpointer item);
void collection_layout_add_item (CollectionLayout* self, LayoutItem* item);
gboolean sorted_list_remove (SortedList* self, gconstpointer item);
void collection_layout_remove_item (CollectionLayout* self, LayoutItem* item);
LayoutItem* collection_layout_get_item_at_pixel (CollectionLayout* self, double xd, double yd);
GeeList* collection_layout_intersection (CollectionLayout* self, const GdkRectangle* rect);
gint sorted_list_get_size (SortedList* self);
GType compass_point_get_type (void);
LayoutItem* collection_layout_get_item_at_coordinate (CollectionLayout* self, gint col, gint row);
LayoutItem* collection_layout_get_item_relative_to (CollectionLayout* self, LayoutItem* item, CompassPoint point);
void sorted_list_clear (SortedList* self);
void collection_layout_refresh (CollectionLayout* self);
static gboolean collection_layout_real_expose_event (GtkWidget* base, const GdkEventExpose* event);
static void collection_layout_finalize (GObject* obj);



#line 32 "CollectionLayout.vala"
LayoutItem* layout_item_construct (GType object_type) {
#line 211 "CollectionLayout.c"
	LayoutItem * self;
	GdkColor _tmp0_ = {0};
	GtkWidget* panel;
	GdkColor _tmp1_ = {0};
	self = g_object_newv (object_type, 0, NULL);
#line 34 "CollectionLayout.vala"
	gtk_alignment_set (GTK_ALIGNMENT (self), (float) 0, (float) 1, (float) 0, (float) 0);
#line 36 "CollectionLayout.vala"
	gtk_label_set_use_underline (self->priv->title, FALSE);
#line 37 "CollectionLayout.vala"
	gtk_label_set_justify (self->priv->title, GTK_JUSTIFY_LEFT);
#line 38 "CollectionLayout.vala"
	gtk_misc_set_alignment (GTK_MISC (self->priv->title), (float) 0, (float) 0);
#line 39 "CollectionLayout.vala"
	gtk_widget_modify_fg (GTK_WIDGET (self->priv->title), GTK_STATE_NORMAL, (_tmp0_ = parse_color (LAYOUT_ITEM_TEXT_COLOR), &_tmp0_));
#line 227 "CollectionLayout.c"
	panel = layout_item_get_control_panel (self);
#line 45 "CollectionLayout.vala"
	gtk_container_set_border_width (GTK_CONTAINER (self->priv->vbox), (guint) LAYOUT_ITEM_FRAME_PADDING);
#line 46 "CollectionLayout.vala"
	gtk_box_pack_start (GTK_BOX (self->priv->vbox), GTK_WIDGET (self->priv->image), FALSE, FALSE, (guint) 0);
#line 47 "CollectionLayout.vala"
	gtk_box_pack_end (GTK_BOX (self->priv->vbox), GTK_WIDGET (self->priv->title), FALSE, FALSE, (guint) LAYOUT_ITEM_LABEL_PADDING);
#line 48 "CollectionLayout.vala"
	if (panel != NULL) {
#line 49 "CollectionLayout.vala"
		gtk_box_pack_end (GTK_BOX (self->priv->vbox), panel, FALSE, FALSE, (guint) 0);
#line 239 "CollectionLayout.c"
	}
#line 52 "CollectionLayout.vala"
	gtk_frame_set_shadow_type (self->priv->frame, GTK_SHADOW_NONE);
#line 53 "CollectionLayout.vala"
	gtk_widget_modify_bg (GTK_WIDGET (self->priv->frame), GTK_STATE_NORMAL, (_tmp1_ = parse_color (LAYOUT_ITEM_UNSELECTED_COLOR), &_tmp1_));
#line 54 "CollectionLayout.vala"
	gtk_container_add (GTK_CONTAINER (self->priv->frame), GTK_WIDGET (self->priv->vbox));
#line 56 "CollectionLayout.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->priv->frame));
#line 249 "CollectionLayout.c"
	(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	return self;
}


#line 32 "CollectionLayout.vala"
LayoutItem* layout_item_new (void) {
#line 32 "CollectionLayout.vala"
	return layout_item_construct (TYPE_LAYOUT_ITEM);
#line 259 "CollectionLayout.c"
}


#line 59 "CollectionLayout.vala"
static GtkWidget* layout_item_real_get_control_panel (LayoutItem* self) {
#line 265 "CollectionLayout.c"
	GtkWidget* result;
#line 59 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (self), NULL);
#line 269 "CollectionLayout.c"
	result = NULL;
	return result;
}


#line 59 "CollectionLayout.vala"
GtkWidget* layout_item_get_control_panel (LayoutItem* self) {
#line 59 "CollectionLayout.vala"
	return LAYOUT_ITEM_GET_CLASS (self)->get_control_panel (self);
#line 279 "CollectionLayout.c"
}


#line 63 "CollectionLayout.vala"
void layout_item_set_title (LayoutItem* self, const char* text) {
#line 63 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 63 "CollectionLayout.vala"
	g_return_if_fail (text != NULL);
#line 64 "CollectionLayout.vala"
	gtk_label_set_text (self->priv->title, text);
#line 291 "CollectionLayout.c"
}


#line 67 "CollectionLayout.vala"
char* layout_item_get_title (LayoutItem* self) {
#line 297 "CollectionLayout.c"
	char* result;
	const char* _tmp0_;
#line 67 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (self), NULL);
#line 68 "CollectionLayout.vala"
	_tmp0_ = NULL;
#line 304 "CollectionLayout.c"
	result = (_tmp0_ = gtk_label_get_text (self->priv->title), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 71 "CollectionLayout.vala"
static void layout_item_real_exposed (LayoutItem* self) {
#line 71 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 314 "CollectionLayout.c"
}


#line 71 "CollectionLayout.vala"
void layout_item_exposed (LayoutItem* self) {
#line 71 "CollectionLayout.vala"
	LAYOUT_ITEM_GET_CLASS (self)->exposed (self);
#line 322 "CollectionLayout.c"
}


#line 74 "CollectionLayout.vala"
static void layout_item_real_unexposed (LayoutItem* self) {
#line 74 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 330 "CollectionLayout.c"
}


#line 74 "CollectionLayout.vala"
void layout_item_unexposed (LayoutItem* self) {
#line 74 "CollectionLayout.vala"
	LAYOUT_ITEM_GET_CLASS (self)->unexposed (self);
#line 338 "CollectionLayout.c"
}


#line 77 "CollectionLayout.vala"
void layout_item_display_title (LayoutItem* self, gboolean display) {
#line 344 "CollectionLayout.c"
	gboolean _tmp0_;
#line 77 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 348 "CollectionLayout.c"
	_tmp0_ = FALSE;
#line 78 "CollectionLayout.vala"
	if (display) {
#line 78 "CollectionLayout.vala"
		_tmp0_ = !self->priv->title_displayed;
#line 354 "CollectionLayout.c"
	} else {
#line 78 "CollectionLayout.vala"
		_tmp0_ = FALSE;
#line 358 "CollectionLayout.c"
	}
#line 78 "CollectionLayout.vala"
	if (_tmp0_) {
#line 79 "CollectionLayout.vala"
		gtk_box_pack_end (GTK_BOX (self->priv->vbox), GTK_WIDGET (self->priv->title), FALSE, FALSE, (guint) LAYOUT_ITEM_LABEL_PADDING);
#line 80 "CollectionLayout.vala"
		self->priv->title_displayed = TRUE;
#line 366 "CollectionLayout.c"
	} else {
		gboolean _tmp1_;
		_tmp1_ = FALSE;
#line 81 "CollectionLayout.vala"
		if (!display) {
#line 81 "CollectionLayout.vala"
			_tmp1_ = self->priv->title_displayed;
#line 374 "CollectionLayout.c"
		} else {
#line 81 "CollectionLayout.vala"
			_tmp1_ = FALSE;
#line 378 "CollectionLayout.c"
		}
#line 81 "CollectionLayout.vala"
		if (_tmp1_) {
#line 82 "CollectionLayout.vala"
			gtk_container_remove (GTK_CONTAINER (self->priv->vbox), GTK_WIDGET (self->priv->title));
#line 83 "CollectionLayout.vala"
			self->priv->title_displayed = FALSE;
#line 386 "CollectionLayout.c"
		}
	}
}


#line 87 "CollectionLayout.vala"
void layout_item_set_image (LayoutItem* self, GdkPixbuf* pixbuf) {
#line 394 "CollectionLayout.c"
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
#line 87 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 87 "CollectionLayout.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 401 "CollectionLayout.c"
	_tmp1_ = NULL;
#line 88 "CollectionLayout.vala"
	_tmp0_ = NULL;
#line 88 "CollectionLayout.vala"
	self->priv->pixbuf = (_tmp1_ = (_tmp0_ = pixbuf, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->pixbuf == NULL) ? NULL : (self->priv->pixbuf = (g_object_unref (self->priv->pixbuf), NULL)), _tmp1_);
#line 90 "CollectionLayout.vala"
	gtk_image_set_from_pixbuf (self->priv->image, pixbuf);
#line 409 "CollectionLayout.c"
}


#line 93 "CollectionLayout.vala"
void layout_item_clear_image (LayoutItem* self) {
#line 415 "CollectionLayout.c"
	GdkPixbuf* _tmp0_;
#line 93 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 94 "CollectionLayout.vala"
	gtk_image_clear (self->priv->image);
#line 421 "CollectionLayout.c"
	_tmp0_ = NULL;
#line 95 "CollectionLayout.vala"
	self->priv->pixbuf = (_tmp0_ = NULL, (self->priv->pixbuf == NULL) ? NULL : (self->priv->pixbuf = (g_object_unref (self->priv->pixbuf), NULL)), _tmp0_);
#line 425 "CollectionLayout.c"
}


#line 98 "CollectionLayout.vala"
void layout_item_set_image_size (LayoutItem* self, gint width, gint height) {
#line 98 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 99 "CollectionLayout.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->priv->image), width, height);
#line 435 "CollectionLayout.c"
}


#line 102 "CollectionLayout.vala"
static void layout_item_real_select (LayoutItem* self) {
#line 441 "CollectionLayout.c"
	GdkColor _tmp0_ = {0};
	GdkColor _tmp1_ = {0};
	GtkWidget* panel;
#line 102 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 103 "CollectionLayout.vala"
	self->priv->selected = TRUE;
#line 105 "CollectionLayout.vala"
	gtk_frame_set_shadow_type (self->priv->frame, GTK_SHADOW_OUT);
#line 106 "CollectionLayout.vala"
	gtk_widget_modify_bg (GTK_WIDGET (self->priv->frame), GTK_STATE_NORMAL, (_tmp0_ = parse_color (LAYOUT_ITEM_SELECTED_COLOR), &_tmp0_));
#line 107 "CollectionLayout.vala"
	gtk_widget_modify_fg (GTK_WIDGET (self->priv->title), GTK_STATE_NORMAL, (_tmp1_ = parse_color (LAYOUT_ITEM_SELECTED_COLOR), &_tmp1_));
#line 455 "CollectionLayout.c"
	panel = layout_item_get_control_panel (self);
#line 110 "CollectionLayout.vala"
	if (panel != NULL) {
#line 459 "CollectionLayout.c"
		GdkColor _tmp2_ = {0};
#line 111 "CollectionLayout.vala"
		gtk_widget_modify_fg (panel, GTK_STATE_NORMAL, (_tmp2_ = parse_color (LAYOUT_ITEM_SELECTED_COLOR), &_tmp2_));
#line 463 "CollectionLayout.c"
	}
	(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
}


#line 102 "CollectionLayout.vala"
void layout_item_select (LayoutItem* self) {
#line 102 "CollectionLayout.vala"
	LAYOUT_ITEM_GET_CLASS (self)->select (self);
#line 473 "CollectionLayout.c"
}


#line 114 "CollectionLayout.vala"
static void layout_item_real_unselect (LayoutItem* self) {
#line 479 "CollectionLayout.c"
	GdkColor _tmp0_ = {0};
	GdkColor _tmp1_ = {0};
	GtkWidget* panel;
#line 114 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 115 "CollectionLayout.vala"
	self->priv->selected = FALSE;
#line 117 "CollectionLayout.vala"
	gtk_frame_set_shadow_type (self->priv->frame, GTK_SHADOW_NONE);
#line 118 "CollectionLayout.vala"
	gtk_widget_modify_bg (GTK_WIDGET (self->priv->frame), GTK_STATE_NORMAL, (_tmp0_ = parse_color (LAYOUT_ITEM_UNSELECTED_COLOR), &_tmp0_));
#line 119 "CollectionLayout.vala"
	gtk_widget_modify_fg (GTK_WIDGET (self->priv->title), GTK_STATE_NORMAL, (_tmp1_ = parse_color (LAYOUT_ITEM_UNSELECTED_COLOR), &_tmp1_));
#line 493 "CollectionLayout.c"
	panel = layout_item_get_control_panel (self);
#line 122 "CollectionLayout.vala"
	if (panel != NULL) {
#line 497 "CollectionLayout.c"
		GdkColor _tmp2_ = {0};
#line 123 "CollectionLayout.vala"
		gtk_widget_modify_fg (panel, GTK_STATE_NORMAL, (_tmp2_ = parse_color (LAYOUT_ITEM_UNSELECTED_COLOR), &_tmp2_));
#line 501 "CollectionLayout.c"
	}
	(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
}


#line 114 "CollectionLayout.vala"
void layout_item_unselect (LayoutItem* self) {
#line 114 "CollectionLayout.vala"
	LAYOUT_ITEM_GET_CLASS (self)->unselect (self);
#line 511 "CollectionLayout.c"
}


#line 126 "CollectionLayout.vala"
gboolean layout_item_toggle_select (LayoutItem* self) {
#line 517 "CollectionLayout.c"
	gboolean result;
#line 126 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (self), FALSE);
#line 127 "CollectionLayout.vala"
	if (self->priv->selected) {
#line 128 "CollectionLayout.vala"
		layout_item_unselect (self);
#line 525 "CollectionLayout.c"
	} else {
#line 130 "CollectionLayout.vala"
		layout_item_select (self);
#line 529 "CollectionLayout.c"
	}
	result = self->priv->selected;
	return result;
}


#line 135 "CollectionLayout.vala"
gboolean layout_item_is_selected (LayoutItem* self) {
#line 538 "CollectionLayout.c"
	gboolean result;
#line 135 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (self), FALSE);
#line 542 "CollectionLayout.c"
	result = self->priv->selected;
	return result;
}


#line 139 "CollectionLayout.vala"
void layout_item_set_coordinates (LayoutItem* self, gint col, gint row) {
#line 139 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 140 "CollectionLayout.vala"
	self->priv->col = col;
#line 141 "CollectionLayout.vala"
	self->priv->row = row;
#line 556 "CollectionLayout.c"
}


#line 144 "CollectionLayout.vala"
gint layout_item_get_column (LayoutItem* self) {
#line 562 "CollectionLayout.c"
	gint result;
#line 144 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (self), 0);
#line 566 "CollectionLayout.c"
	result = self->priv->col;
	return result;
}


#line 148 "CollectionLayout.vala"
gint layout_item_get_row (LayoutItem* self) {
#line 574 "CollectionLayout.c"
	gint result;
#line 148 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (self), 0);
#line 578 "CollectionLayout.c"
	result = self->priv->row;
	return result;
}


#line 152 "CollectionLayout.vala"
void layout_item_brighten (LayoutItem* self) {
#line 586 "CollectionLayout.c"
	gboolean _tmp0_;
	GdkPixbuf* sunshine;
#line 152 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 591 "CollectionLayout.c"
	_tmp0_ = FALSE;
#line 153 "CollectionLayout.vala"
	if (self->priv->brightened) {
#line 153 "CollectionLayout.vala"
		_tmp0_ = TRUE;
#line 597 "CollectionLayout.c"
	} else {
#line 153 "CollectionLayout.vala"
		_tmp0_ = self->priv->pixbuf == NULL;
#line 601 "CollectionLayout.c"
	}
#line 153 "CollectionLayout.vala"
	if (_tmp0_) {
#line 605 "CollectionLayout.c"
		return;
	}
	/* create a new lightened pixbuf to display*/
	sunshine = gdk_pixbuf_copy (self->priv->pixbuf);
#line 158 "CollectionLayout.vala"
	shift_colors (sunshine, LAYOUT_ITEM_BRIGHTEN_SHIFT, LAYOUT_ITEM_BRIGHTEN_SHIFT, LAYOUT_ITEM_BRIGHTEN_SHIFT, 0);
#line 160 "CollectionLayout.vala"
	gtk_image_set_from_pixbuf (self->priv->image, sunshine);
#line 161 "CollectionLayout.vala"
	self->priv->brightened = TRUE;
#line 616 "CollectionLayout.c"
	(sunshine == NULL) ? NULL : (sunshine = (g_object_unref (sunshine), NULL));
}


#line 164 "CollectionLayout.vala"
void layout_item_unbrighten (LayoutItem* self) {
#line 623 "CollectionLayout.c"
	gboolean _tmp0_;
#line 164 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (self));
#line 627 "CollectionLayout.c"
	_tmp0_ = FALSE;
#line 165 "CollectionLayout.vala"
	if (!self->priv->brightened) {
#line 165 "CollectionLayout.vala"
		_tmp0_ = TRUE;
#line 633 "CollectionLayout.c"
	} else {
#line 165 "CollectionLayout.vala"
		_tmp0_ = self->priv->pixbuf == NULL;
#line 637 "CollectionLayout.c"
	}
#line 165 "CollectionLayout.vala"
	if (_tmp0_) {
#line 641 "CollectionLayout.c"
		return;
	}
	/* return to the normal image*/
#line 169 "CollectionLayout.vala"
	gtk_image_set_from_pixbuf (self->priv->image, self->priv->pixbuf);
#line 170 "CollectionLayout.vala"
	self->priv->brightened = FALSE;
#line 649 "CollectionLayout.c"
}


static void layout_item_class_init (LayoutItemClass * klass) {
	layout_item_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (LayoutItemPrivate));
	LAYOUT_ITEM_CLASS (klass)->get_control_panel = layout_item_real_get_control_panel;
	LAYOUT_ITEM_CLASS (klass)->exposed = layout_item_real_exposed;
	LAYOUT_ITEM_CLASS (klass)->unexposed = layout_item_real_unexposed;
	LAYOUT_ITEM_CLASS (klass)->select = layout_item_real_select;
	LAYOUT_ITEM_CLASS (klass)->unselect = layout_item_real_unselect;
	G_OBJECT_CLASS (klass)->finalize = layout_item_finalize;
}


static void layout_item_instance_init (LayoutItem * self) {
	self->priv = LAYOUT_ITEM_GET_PRIVATE (self);
	self->priv->title = g_object_ref_sink ((GtkLabel*) gtk_label_new (""));
	self->priv->frame = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL));
	self->priv->image = g_object_ref_sink ((GtkImage*) gtk_image_new ());
	self->priv->pixbuf = NULL;
	self->priv->selected = FALSE;
	self->priv->vbox = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0));
	self->priv->title_displayed = TRUE;
	self->priv->brightened = FALSE;
	self->priv->col = -1;
	self->priv->row = -1;
}


static void layout_item_finalize (GObject* obj) {
	LayoutItem * self;
	self = LAYOUT_ITEM (obj);
	(self->priv->title == NULL) ? NULL : (self->priv->title = (g_object_unref (self->priv->title), NULL));
	(self->priv->frame == NULL) ? NULL : (self->priv->frame = (g_object_unref (self->priv->frame), NULL));
	(self->priv->image == NULL) ? NULL : (self->priv->image = (g_object_unref (self->priv->image), NULL));
	(self->priv->pixbuf == NULL) ? NULL : (self->priv->pixbuf = (g_object_unref (self->priv->pixbuf), NULL));
	(self->priv->vbox == NULL) ? NULL : (self->priv->vbox = (g_object_unref (self->priv->vbox), NULL));
	G_OBJECT_CLASS (layout_item_parent_class)->finalize (obj);
}


GType layout_item_get_type (void) {
	static GType layout_item_type_id = 0;
	if (layout_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (LayoutItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) layout_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (LayoutItem), 0, (GInstanceInitFunc) layout_item_instance_init, NULL };
		layout_item_type_id = g_type_register_static (GTK_TYPE_ALIGNMENT, "LayoutItem", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return layout_item_type_id;
}


#line 561 "CollectionLayout.vala"
static void _collection_layout_on_resize_gtk_widget_size_allocate (CollectionLayout* _sender, const GdkRectangle* allocation, gpointer self) {
#line 704 "CollectionLayout.c"
	collection_layout_on_resize (self);
}


#line 193 "CollectionLayout.vala"
CollectionLayout* collection_layout_construct (GType object_type) {
#line 711 "CollectionLayout.c"
	CollectionLayout * self;
	GdkColor color;
	self = g_object_newv (object_type, 0, NULL);
#line 194 "CollectionLayout.vala"
	gtk_widget_modify_bg (GTK_WIDGET (self), GTK_STATE_NORMAL, &app_window_BG_COLOR);
#line 717 "CollectionLayout.c"
	color = parse_color (LAYOUT_ITEM_UNSELECTED_COLOR);
#line 197 "CollectionLayout.vala"
	gtk_widget_modify_fg (GTK_WIDGET (self->priv->message), GTK_STATE_NORMAL, &color);
#line 198 "CollectionLayout.vala"
	gtk_label_set_single_line_mode (self->priv->message, FALSE);
#line 199 "CollectionLayout.vala"
	gtk_label_set_use_underline (self->priv->message, FALSE);
#line 201 "CollectionLayout.vala"
	g_signal_connect_object (GTK_WIDGET (self), "size-allocate", (GCallback) _collection_layout_on_resize_gtk_widget_size_allocate, self, 0);
#line 727 "CollectionLayout.c"
	return self;
}


#line 193 "CollectionLayout.vala"
CollectionLayout* collection_layout_new (void) {
#line 193 "CollectionLayout.vala"
	return collection_layout_construct (TYPE_COLLECTION_LAYOUT);
#line 736 "CollectionLayout.c"
}


#line 206 "CollectionLayout.vala"
void collection_layout_set_message (CollectionLayout* self, const char* text) {
#line 206 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 206 "CollectionLayout.vala"
	g_return_if_fail (text != NULL);
#line 207 "CollectionLayout.vala"
	collection_layout_clear (self);
#line 209 "CollectionLayout.vala"
	gtk_label_set_text (self->priv->message, text);
#line 211 "CollectionLayout.vala"
	collection_layout_display_message (self);
#line 752 "CollectionLayout.c"
}


#line 214 "CollectionLayout.vala"
void collection_layout_set_refresh_on_resize (CollectionLayout* self, gboolean refresh_on_resize) {
#line 214 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 215 "CollectionLayout.vala"
	self->priv->refresh_on_resize = refresh_on_resize;
#line 762 "CollectionLayout.c"
}


#line 218 "CollectionLayout.vala"
void collection_layout_set_in_view (CollectionLayout* self, gboolean in_view) {
#line 218 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 219 "CollectionLayout.vala"
	self->priv->in_view = in_view;
#line 220 "CollectionLayout.vala"
	if (in_view) {
#line 774 "CollectionLayout.c"
		return;
	}
	/* need to wait for expose event to start exposing items, but if no longer in view, might
	 as well unload now*/
	{
		GeeIterator* _item_it;
		/* need to wait for expose event to start exposing items, but if no longer in view, might
		 as well unload now*/
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
		/* need to wait for expose event to start exposing items, but if no longer in view, might
		 as well unload now*/
#line 225 "CollectionLayout.vala"
		while (TRUE) {
#line 788 "CollectionLayout.c"
			LayoutItem* item;
			/* need to wait for expose event to start exposing items, but if no longer in view, might
			 as well unload now*/
#line 225 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 794 "CollectionLayout.c"
				/* need to wait for expose event to start exposing items, but if no longer in view, might
				 as well unload now*/
#line 225 "CollectionLayout.vala"
				break;
#line 799 "CollectionLayout.c"
			}
			/* need to wait for expose event to start exposing items, but if no longer in view, might
			 as well unload now*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 226 "CollectionLayout.vala"
			layout_item_unexposed (item);
#line 806 "CollectionLayout.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
}


#line 229 "CollectionLayout.vala"
void collection_layout_set_comparator (CollectionLayout* self, Comparator* cmp) {
#line 229 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 229 "CollectionLayout.vala"
	g_return_if_fail (IS_COMPARATOR (cmp));
#line 230 "CollectionLayout.vala"
	sorted_list_resort (self->items, cmp);
#line 822 "CollectionLayout.c"
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


#line 233 "CollectionLayout.vala"
void collection_layout_add_item (CollectionLayout* self, LayoutItem* item) {
#line 233 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 233 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 234 "CollectionLayout.vala"
	sorted_list_add (self->items, item);
#line 842 "CollectionLayout.c"
	/* this demolishes any message that's been set*/
#line 237 "CollectionLayout.vala"
	if (string_get_length (gtk_label_get_text (self->priv->message)) > 0) {
#line 238 "CollectionLayout.vala"
		gtk_container_remove (GTK_CONTAINER (self), GTK_WIDGET (self->priv->message));
#line 239 "CollectionLayout.vala"
		gtk_label_set_text (self->priv->message, "");
#line 850 "CollectionLayout.c"
	}
	/* need to do this to have its size requisitioned in refresh()*/
#line 243 "CollectionLayout.vala"
	gtk_widget_show_all (GTK_WIDGET (item));
#line 855 "CollectionLayout.c"
}


#line 246 "CollectionLayout.vala"
void collection_layout_remove_item (CollectionLayout* self, LayoutItem* item) {
#line 246 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 246 "CollectionLayout.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 247 "CollectionLayout.vala"
	sorted_list_remove (self->items, item);
#line 867 "CollectionLayout.c"
	/* this situation can happen if the item was added but the page not yet rendered ... because
	 CollectionLayout doens't know where to place the item until refresh(), that's when it's
	 initially added*/
#line 252 "CollectionLayout.vala"
	if (gtk_widget_get_parent (GTK_WIDGET (item)) != NULL) {
#line 253 "CollectionLayout.vala"
		gtk_container_remove (GTK_CONTAINER (self), GTK_WIDGET (item));
#line 875 "CollectionLayout.c"
	}
}


#line 256 "CollectionLayout.vala"
LayoutItem* collection_layout_get_item_at_pixel (CollectionLayout* self, double xd, double yd) {
#line 882 "CollectionLayout.c"
	LayoutItem* result;
	gint x;
	gint y;
#line 256 "CollectionLayout.vala"
	g_return_val_if_fail (IS_COLLECTION_LAYOUT (self), NULL);
#line 888 "CollectionLayout.c"
	x = (gint) xd;
	y = (gint) yd;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 260 "CollectionLayout.vala"
		while (TRUE) {
#line 896 "CollectionLayout.c"
			LayoutItem* item;
			GtkAllocation alloc;
			gboolean _tmp0_;
			gboolean _tmp1_;
			gboolean _tmp2_;
#line 260 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 260 "CollectionLayout.vala"
				break;
#line 906 "CollectionLayout.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			alloc = GTK_WIDGET (item)->allocation;
			_tmp0_ = FALSE;
			_tmp1_ = FALSE;
			_tmp2_ = FALSE;
#line 262 "CollectionLayout.vala"
			if (x >= alloc.x) {
#line 262 "CollectionLayout.vala"
				_tmp2_ = y >= alloc.y;
#line 917 "CollectionLayout.c"
			} else {
#line 262 "CollectionLayout.vala"
				_tmp2_ = FALSE;
#line 921 "CollectionLayout.c"
			}
#line 262 "CollectionLayout.vala"
			if (_tmp2_) {
#line 262 "CollectionLayout.vala"
				_tmp1_ = x <= (alloc.x + alloc.width);
#line 927 "CollectionLayout.c"
			} else {
#line 262 "CollectionLayout.vala"
				_tmp1_ = FALSE;
#line 931 "CollectionLayout.c"
			}
#line 262 "CollectionLayout.vala"
			if (_tmp1_) {
#line 263 "CollectionLayout.vala"
				_tmp0_ = y <= (alloc.y + alloc.height);
#line 937 "CollectionLayout.c"
			} else {
#line 262 "CollectionLayout.vala"
				_tmp0_ = FALSE;
#line 941 "CollectionLayout.c"
			}
#line 262 "CollectionLayout.vala"
			if (_tmp0_) {
#line 945 "CollectionLayout.c"
				result = item;
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				return result;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = NULL;
	return result;
}


#line 271 "CollectionLayout.vala"
GeeList* collection_layout_intersection (CollectionLayout* self, const GdkRectangle* rect) {
#line 961 "CollectionLayout.c"
	GeeList* result;
	gint bottom;
	gint right;
	GeeArrayList* intersects;
	GdkRectangle _tmp0_ = {0};
	GdkRectangle bitbucket;
#line 271 "CollectionLayout.vala"
	g_return_val_if_fail (IS_COLLECTION_LAYOUT (self), NULL);
#line 970 "CollectionLayout.c"
	bottom = ((*rect).y + (*rect).height) + 1;
	right = ((*rect).x + (*rect).width) + 1;
	intersects = gee_array_list_new (TYPE_LAYOUT_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	bitbucket = (memset (&_tmp0_, 0, sizeof (GdkRectangle)), _tmp0_);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 279 "CollectionLayout.vala"
		while (TRUE) {
#line 980 "CollectionLayout.c"
			LayoutItem* item;
			GdkRectangle _tmp1_ = {0};
			gboolean _tmp2_;
#line 279 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 279 "CollectionLayout.vala"
				break;
#line 988 "CollectionLayout.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 280 "CollectionLayout.vala"
			if (gdk_rectangle_intersect (&(*rect), (_tmp1_ = (GdkRectangle) GTK_WIDGET (item)->allocation, &_tmp1_), &bitbucket)) {
#line 281 "CollectionLayout.vala"
				gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (intersects), item);
#line 995 "CollectionLayout.c"
			}
			_tmp2_ = FALSE;
#line 284 "CollectionLayout.vala"
			if (GTK_WIDGET (item)->allocation.y > bottom) {
#line 284 "CollectionLayout.vala"
				_tmp2_ = GTK_WIDGET (item)->allocation.x > right;
#line 1002 "CollectionLayout.c"
			} else {
#line 284 "CollectionLayout.vala"
				_tmp2_ = FALSE;
#line 1006 "CollectionLayout.c"
			}
			/* short-circuit: if past the dimensions of the box in the sorted list, bail out*/
#line 284 "CollectionLayout.vala"
			if (_tmp2_) {
#line 1011 "CollectionLayout.c"
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
#line 285 "CollectionLayout.vala"
				break;
#line 1015 "CollectionLayout.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = GEE_LIST (intersects);
	return result;
}


#line 291 "CollectionLayout.vala"
LayoutItem* collection_layout_get_item_relative_to (CollectionLayout* self, LayoutItem* item, CompassPoint point) {
#line 1028 "CollectionLayout.c"
	LayoutItem* result;
	gint col;
	gint row;
	gboolean _tmp0_;
	LayoutItem* new_item;
	LayoutItem* _tmp2_;
	LayoutItem* _tmp3_;
#line 291 "CollectionLayout.vala"
	g_return_val_if_fail (IS_COLLECTION_LAYOUT (self), NULL);
#line 291 "CollectionLayout.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (item), NULL);
#line 292 "CollectionLayout.vala"
	if (sorted_list_get_size (self->items) == 0) {
#line 1042 "CollectionLayout.c"
		result = NULL;
		return result;
	}
#line 295 "CollectionLayout.vala"
	g_assert (self->priv->columns > 0);
#line 296 "CollectionLayout.vala"
	g_assert (self->priv->rows > 0);
#line 1050 "CollectionLayout.c"
	col = layout_item_get_column (item);
	row = layout_item_get_row (item);
	_tmp0_ = FALSE;
#line 301 "CollectionLayout.vala"
	if (col < 0) {
#line 301 "CollectionLayout.vala"
		_tmp0_ = TRUE;
#line 1058 "CollectionLayout.c"
	} else {
#line 301 "CollectionLayout.vala"
		_tmp0_ = row < 0;
#line 1062 "CollectionLayout.c"
	}
#line 301 "CollectionLayout.vala"
	if (_tmp0_) {
#line 1066 "CollectionLayout.c"
		char* _tmp1_;
#line 67 "CollectionLayout.vala"
		_tmp1_ = NULL;
#line 302 "CollectionLayout.vala"
		g_critical ("CollectionLayout.vala:302: Attempting to locate item not placed in layout: %s", _tmp1_ = layout_item_get_title (item));
#line 1072 "CollectionLayout.c"
		_tmp1_ = (g_free (_tmp1_), NULL);
		result = NULL;
		return result;
	}
#line 307 "CollectionLayout.vala"
	switch (point) {
#line 1079 "CollectionLayout.c"
		case COMPASS_POINT_NORTH:
		{
#line 309 "CollectionLayout.vala"
			if ((row = row - 1) < 0) {
#line 310 "CollectionLayout.vala"
				row = 0;
#line 1086 "CollectionLayout.c"
			}
#line 311 "CollectionLayout.vala"
			break;
#line 1090 "CollectionLayout.c"
		}
		case COMPASS_POINT_SOUTH:
		{
#line 314 "CollectionLayout.vala"
			if ((row = row + 1) >= self->priv->rows) {
#line 315 "CollectionLayout.vala"
				row = self->priv->rows - 1;
#line 1098 "CollectionLayout.c"
			}
#line 316 "CollectionLayout.vala"
			break;
#line 1102 "CollectionLayout.c"
		}
		case COMPASS_POINT_EAST:
		{
#line 319 "CollectionLayout.vala"
			if ((col = col + 1) >= self->priv->columns) {
#line 320 "CollectionLayout.vala"
				col = self->priv->columns - 1;
#line 1110 "CollectionLayout.c"
			}
#line 321 "CollectionLayout.vala"
			break;
#line 1114 "CollectionLayout.c"
		}
		case COMPASS_POINT_WEST:
		{
#line 324 "CollectionLayout.vala"
			if ((col = col - 1) < 0) {
#line 325 "CollectionLayout.vala"
				col = 0;
#line 1122 "CollectionLayout.c"
			}
#line 326 "CollectionLayout.vala"
			break;
#line 1126 "CollectionLayout.c"
		}
		default:
		{
#line 329 "CollectionLayout.vala"
			g_error ("CollectionLayout.vala:329: Bad compass point %d", (gint) point);
#line 330 "CollectionLayout.vala"
			break;
#line 1134 "CollectionLayout.c"
		}
	}
	new_item = collection_layout_get_item_at_coordinate (self, col, row);
	_tmp2_ = NULL;
#line 335 "CollectionLayout.vala"
	if (new_item != NULL) {
#line 335 "CollectionLayout.vala"
		_tmp2_ = new_item;
#line 1143 "CollectionLayout.c"
	} else {
#line 335 "CollectionLayout.vala"
		_tmp2_ = item;
#line 1147 "CollectionLayout.c"
	}
#line 335 "CollectionLayout.vala"
	_tmp3_ = NULL;
#line 1151 "CollectionLayout.c"
	result = (_tmp3_ = _tmp2_, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_));
	(new_item == NULL) ? NULL : (new_item = (g_object_unref (new_item), NULL));
	return result;
	(new_item == NULL) ? NULL : (new_item = (g_object_unref (new_item), NULL));
}


#line 338 "CollectionLayout.vala"
LayoutItem* collection_layout_get_item_at_coordinate (CollectionLayout* self, gint col, gint row) {
#line 1161 "CollectionLayout.c"
	LayoutItem* result;
#line 338 "CollectionLayout.vala"
	g_return_val_if_fail (IS_COLLECTION_LAYOUT (self), NULL);
#line 1165 "CollectionLayout.c"
	/* TODO: If searching by coordinates becomes more vital, the items could be stored
	 in an array of arrays for quicker lookup.*/
	{
		GeeIterator* _item_it;
		/* TODO: If searching by coordinates becomes more vital, the items could be stored
		 in an array of arrays for quicker lookup.*/
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
		/* TODO: If searching by coordinates becomes more vital, the items could be stored
		 in an array of arrays for quicker lookup.*/
#line 341 "CollectionLayout.vala"
		while (TRUE) {
#line 1177 "CollectionLayout.c"
			LayoutItem* item;
			gboolean _tmp0_;
			/* TODO: If searching by coordinates becomes more vital, the items could be stored
			 in an array of arrays for quicker lookup.*/
#line 341 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 1184 "CollectionLayout.c"
				/* TODO: If searching by coordinates becomes more vital, the items could be stored
				 in an array of arrays for quicker lookup.*/
#line 341 "CollectionLayout.vala"
				break;
#line 1189 "CollectionLayout.c"
			}
			/* TODO: If searching by coordinates becomes more vital, the items could be stored
			 in an array of arrays for quicker lookup.*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
			_tmp0_ = FALSE;
#line 342 "CollectionLayout.vala"
			if (layout_item_get_column (item) == col) {
#line 342 "CollectionLayout.vala"
				_tmp0_ = layout_item_get_row (item) == row;
#line 1199 "CollectionLayout.c"
			} else {
#line 342 "CollectionLayout.vala"
				_tmp0_ = FALSE;
#line 1203 "CollectionLayout.c"
			}
#line 342 "CollectionLayout.vala"
			if (_tmp0_) {
#line 1207 "CollectionLayout.c"
				result = item;
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				return result;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = NULL;
	return result;
}


#line 349 "CollectionLayout.vala"
void collection_layout_clear (CollectionLayout* self) {
#line 349 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 1225 "CollectionLayout.c"
	/* remove page message*/
#line 351 "CollectionLayout.vala"
	if (string_get_length (gtk_label_get_text (self->priv->message)) > 0) {
#line 352 "CollectionLayout.vala"
		gtk_container_remove (GTK_CONTAINER (self), GTK_WIDGET (self->priv->message));
#line 353 "CollectionLayout.vala"
		gtk_label_set_text (self->priv->message, "");
#line 1233 "CollectionLayout.c"
	}
	/* remove all items from Gtk.Layout*/
	{
		GeeIterator* _item_it;
		/* remove all items from Gtk.Layout*/
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
		/* remove all items from Gtk.Layout*/
#line 357 "CollectionLayout.vala"
		while (TRUE) {
#line 1243 "CollectionLayout.c"
			LayoutItem* item;
			/* remove all items from Gtk.Layout*/
#line 357 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 1248 "CollectionLayout.c"
				/* remove all items from Gtk.Layout*/
#line 357 "CollectionLayout.vala"
				break;
#line 1252 "CollectionLayout.c"
			}
			/* remove all items from Gtk.Layout*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 358 "CollectionLayout.vala"
			gtk_container_remove (GTK_CONTAINER (self), GTK_WIDGET (item));
#line 1258 "CollectionLayout.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* clear internal list*/
#line 361 "CollectionLayout.vala"
	sorted_list_clear (self->items);
#line 362 "CollectionLayout.vala"
	self->priv->columns = 0;
#line 363 "CollectionLayout.vala"
	self->priv->rows = 0;
#line 1270 "CollectionLayout.c"
}


#line 366 "CollectionLayout.vala"
void collection_layout_refresh (CollectionLayout* self) {
#line 1276 "CollectionLayout.c"
	gint x;
	gint col;
	gint max_cols;
	gint row_width;
	gint widest_row;
	gint row;
	gint tallest;
	gint total_width;
	gint* _tmp0_;
	gint column_widths_size;
	gint column_widths_length1;
	gint* column_widths;
	gint* _tmp2_;
	gint row_heights_size;
	gint row_heights_length1;
	gint _tmp1_;
	gint* row_heights;
	gint gutter;
	gint y;
#line 366 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 367 "CollectionLayout.vala"
	if (string_get_length (gtk_label_get_text (self->priv->message)) > 0) {
#line 368 "CollectionLayout.vala"
		collection_layout_display_message (self);
#line 1302 "CollectionLayout.c"
		return;
	}
	/* don't bother until layout is of some appreciable size*/
#line 374 "CollectionLayout.vala"
	if (GTK_WIDGET (self)->allocation.width <= 1) {
#line 1308 "CollectionLayout.c"
		return;
	}
	/* need to set_size in case all items were removed and the viewport size has changed*/
#line 378 "CollectionLayout.vala"
	if (sorted_list_get_size (self->items) == 0) {
#line 379 "CollectionLayout.vala"
		gtk_layout_set_size (GTK_LAYOUT (self), (guint) GTK_WIDGET (self)->allocation.width, (guint) 0);
#line 1316 "CollectionLayout.c"
		return;
	}
	/* Step 1: Determine the widest row in the layout, and from it the number of columns*/
	x = COLLECTION_LAYOUT_LEFT_PADDING;
	col = 0;
	max_cols = 0;
	row_width = 0;
	widest_row = 0;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 391 "CollectionLayout.vala"
		while (TRUE) {
#line 1330 "CollectionLayout.c"
			LayoutItem* item;
			GtkRequisition req = {0};
#line 391 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 391 "CollectionLayout.vala"
				break;
#line 1337 "CollectionLayout.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			/* perform size requests first time through, but not thereafter*/
#line 394 "CollectionLayout.vala"
			gtk_widget_size_request (GTK_WIDGET (item), &req);
#line 1343 "CollectionLayout.c"
			/* the items must be requisitioned for this code to work*/
#line 397 "CollectionLayout.vala"
			g_assert (req.height > 0);
#line 398 "CollectionLayout.vala"
			g_assert (req.width > 0);
#line 1349 "CollectionLayout.c"
			/* carriage return (i.e. this item will overflow the view)*/
#line 401 "CollectionLayout.vala"
			if (((x + req.width) + COLLECTION_LAYOUT_RIGHT_PADDING) > GTK_WIDGET (self)->allocation.width) {
#line 402 "CollectionLayout.vala"
				if (row_width > widest_row) {
#line 403 "CollectionLayout.vala"
					widest_row = row_width;
#line 404 "CollectionLayout.vala"
					max_cols = col;
#line 1359 "CollectionLayout.c"
				}
#line 407 "CollectionLayout.vala"
				col = 0;
#line 408 "CollectionLayout.vala"
				x = COLLECTION_LAYOUT_LEFT_PADDING;
#line 409 "CollectionLayout.vala"
				row_width = 0;
#line 1367 "CollectionLayout.c"
			}
#line 412 "CollectionLayout.vala"
			x = x + (req.width + COLLECTION_LAYOUT_COLUMN_GUTTER_PADDING);
#line 413 "CollectionLayout.vala"
			row_width = row_width + req.width;
#line 415 "CollectionLayout.vala"
			col++;
#line 1375 "CollectionLayout.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* account for dangling last row*/
#line 419 "CollectionLayout.vala"
	if (row_width > widest_row) {
#line 420 "CollectionLayout.vala"
		widest_row = row_width;
#line 421 "CollectionLayout.vala"
		max_cols = col;
#line 1387 "CollectionLayout.c"
	}
#line 424 "CollectionLayout.vala"
	g_assert (max_cols > 0);
#line 1391 "CollectionLayout.c"
	/* Step 2: Now that the number of columns is known, find the maximum height for each row
	 and the maximum width for each column*/
	row = 0;
	tallest = 0;
	total_width = 0;
#line 431 "CollectionLayout.vala"
	col = 0;
#line 432 "CollectionLayout.vala"
	_tmp0_ = NULL;
#line 1401 "CollectionLayout.c"
	column_widths = (_tmp0_ = g_new0 (gint, max_cols), column_widths_length1 = max_cols, column_widths_size = column_widths_length1, _tmp0_);
#line 433 "CollectionLayout.vala"
	_tmp2_ = NULL;
#line 1405 "CollectionLayout.c"
	row_heights = (_tmp2_ = g_new0 (gint, _tmp1_ = (sorted_list_get_size (self->items) / max_cols) + 1), row_heights_length1 = _tmp1_, row_heights_size = row_heights_length1, _tmp2_);
	gutter = 0;
	{
		gboolean _tmp3_;
		_tmp3_ = TRUE;
#line 436 "CollectionLayout.vala"
		while (TRUE) {
#line 1413 "CollectionLayout.c"
			gboolean _tmp4_;
			gboolean _tmp5_;
#line 436 "CollectionLayout.vala"
			if (!_tmp3_) {
#line 1418 "CollectionLayout.c"
			}
#line 436 "CollectionLayout.vala"
			_tmp3_ = FALSE;
#line 1422 "CollectionLayout.c"
			{
				GeeIterator* _item_it;
				_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 437 "CollectionLayout.vala"
				while (TRUE) {
#line 1428 "CollectionLayout.c"
					LayoutItem* item;
					GtkRequisition req;
#line 437 "CollectionLayout.vala"
					if (!gee_iterator_next (_item_it)) {
#line 437 "CollectionLayout.vala"
						break;
#line 1435 "CollectionLayout.c"
					}
					item = (LayoutItem*) gee_iterator_get (_item_it);
					req = GTK_WIDGET (item)->requisition;
#line 440 "CollectionLayout.vala"
					if (req.height > tallest) {
#line 441 "CollectionLayout.vala"
						tallest = req.height;
#line 1443 "CollectionLayout.c"
					}
					/* store largest thumb size of each column as well as track the total width of the
					 layout (which is the sum of the width of each column)*/
#line 445 "CollectionLayout.vala"
					if (column_widths[col] < req.width) {
#line 446 "CollectionLayout.vala"
						total_width = total_width - column_widths[col];
#line 447 "CollectionLayout.vala"
						column_widths[col] = req.width;
#line 448 "CollectionLayout.vala"
						total_width = total_width + req.width;
#line 1455 "CollectionLayout.c"
					}
#line 451 "CollectionLayout.vala"
					if ((col = col + 1) >= max_cols) {
#line 452 "CollectionLayout.vala"
						col = 0;
#line 453 "CollectionLayout.vala"
						row_heights[row++] = tallest;
#line 454 "CollectionLayout.vala"
						tallest = 0;
#line 1465 "CollectionLayout.c"
					}
					(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				}
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
			}
			/* account for final dangling row*/
#line 459 "CollectionLayout.vala"
			if (col != 0) {
#line 460 "CollectionLayout.vala"
				row_heights[row] = tallest;
#line 1476 "CollectionLayout.c"
			}
			/* Step 3: Calculate the gutter between the items as being equidistant of the
			 remaining space (adding one gutter to account for the right-hand one)*/
#line 464 "CollectionLayout.vala"
			gutter = (GTK_WIDGET (self)->allocation.width - total_width) / (max_cols + 1);
#line 1482 "CollectionLayout.c"
			/* if only one column, gutter size could be less than minimums*/
#line 467 "CollectionLayout.vala"
			if (max_cols == 1) {
#line 468 "CollectionLayout.vala"
				break;
#line 1488 "CollectionLayout.c"
			}
			_tmp4_ = FALSE;
			_tmp5_ = FALSE;
#line 473 "CollectionLayout.vala"
			if (gutter < COLLECTION_LAYOUT_LEFT_PADDING) {
#line 473 "CollectionLayout.vala"
				_tmp5_ = TRUE;
#line 1496 "CollectionLayout.c"
			} else {
#line 473 "CollectionLayout.vala"
				_tmp5_ = gutter < COLLECTION_LAYOUT_RIGHT_PADDING;
#line 1500 "CollectionLayout.c"
			}
#line 473 "CollectionLayout.vala"
			if (_tmp5_) {
#line 473 "CollectionLayout.vala"
				_tmp4_ = TRUE;
#line 1506 "CollectionLayout.c"
			} else {
#line 473 "CollectionLayout.vala"
				_tmp4_ = gutter < COLLECTION_LAYOUT_COLUMN_GUTTER_PADDING;
#line 1510 "CollectionLayout.c"
			}
			/* have to reassemble if the gutter is too small ... this happens because Step One
			 takes a guess at the best column count, but when the max. widths of the columns are
			 added up, they could overflow*/
#line 473 "CollectionLayout.vala"
			if (_tmp4_) {
#line 1517 "CollectionLayout.c"
				gint* _tmp6_;
				gint* _tmp8_;
				gint _tmp7_;
#line 474 "CollectionLayout.vala"
				max_cols--;
#line 475 "CollectionLayout.vala"
				col = 0;
#line 476 "CollectionLayout.vala"
				row = 0;
#line 477 "CollectionLayout.vala"
				tallest = 0;
#line 478 "CollectionLayout.vala"
				total_width = 0;
#line 1531 "CollectionLayout.c"
				_tmp6_ = NULL;
#line 479 "CollectionLayout.vala"
				column_widths = (_tmp6_ = g_new0 (gint, max_cols), column_widths = (g_free (column_widths), NULL), column_widths_length1 = max_cols, column_widths_size = column_widths_length1, _tmp6_);
#line 1535 "CollectionLayout.c"
				_tmp8_ = NULL;
#line 480 "CollectionLayout.vala"
				row_heights = (_tmp8_ = g_new0 (gint, _tmp7_ = (sorted_list_get_size (self->items) / max_cols) + 1), row_heights = (g_free (row_heights), NULL), row_heights_length1 = _tmp7_, row_heights_size = row_heights_length1, _tmp8_);
#line 1539 "CollectionLayout.c"
			} else {
				/*
				                debug("refresh(): readjusting columns: max_cols=%d", max_cols);
				                */
#line 485 "CollectionLayout.vala"
				break;
#line 1546 "CollectionLayout.c"
			}
		}
	}
	/*
	        debug("refresh(): width:%d total_width:%d max_cols:%d gutter:%d", allocation.width, total_width, 
	            max_cols, gutter);
	        
	 Step 4: Lay out the items in the space using all the information gathered*/
#line 495 "CollectionLayout.vala"
	x = gutter;
#line 1557 "CollectionLayout.c"
	y = COLLECTION_LAYOUT_TOP_PADDING;
#line 497 "CollectionLayout.vala"
	col = 0;
#line 498 "CollectionLayout.vala"
	row = 0;
#line 1563 "CollectionLayout.c"
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 500 "CollectionLayout.vala"
		while (TRUE) {
#line 1569 "CollectionLayout.c"
			LayoutItem* item;
			GtkRequisition req;
			gint xpadding;
			gint ypadding;
#line 500 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 500 "CollectionLayout.vala"
				break;
#line 1578 "CollectionLayout.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			req = GTK_WIDGET (item)->requisition;
			/* this centers the item in the column*/
			xpadding = (column_widths[col] - req.width) / 2;
#line 505 "CollectionLayout.vala"
			g_assert (xpadding >= 0);
#line 1586 "CollectionLayout.c"
			/* this bottom-aligns the item along the row*/
			ypadding = row_heights[row] - req.height;
#line 509 "CollectionLayout.vala"
			g_assert (ypadding >= 0);
#line 1591 "CollectionLayout.c"
			/* if item was recently appended, it needs to be put() rather than move()'d*/
#line 512 "CollectionLayout.vala"
			if (GTK_WIDGET (gtk_widget_get_parent (GTK_WIDGET (item))) == GTK_WIDGET (self)) {
#line 513 "CollectionLayout.vala"
				gtk_layout_move (GTK_LAYOUT (self), GTK_WIDGET (item), x + xpadding, y + ypadding);
#line 1597 "CollectionLayout.c"
			} else {
#line 515 "CollectionLayout.vala"
				gtk_layout_put (GTK_LAYOUT (self), GTK_WIDGET (item), x + xpadding, y + ypadding);
#line 1601 "CollectionLayout.c"
			}
#line 517 "CollectionLayout.vala"
			layout_item_set_coordinates (item, col, row);
#line 519 "CollectionLayout.vala"
			x = x + (column_widths[col] + gutter);
#line 1607 "CollectionLayout.c"
			/* carriage return*/
#line 522 "CollectionLayout.vala"
			if ((col = col + 1) >= max_cols) {
#line 523 "CollectionLayout.vala"
				x = gutter;
#line 524 "CollectionLayout.vala"
				y = y + (row_heights[row] + COLLECTION_LAYOUT_ROW_GUTTER_PADDING);
#line 525 "CollectionLayout.vala"
				col = 0;
#line 526 "CollectionLayout.vala"
				row++;
#line 1619 "CollectionLayout.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 530 "CollectionLayout.vala"
	self->priv->columns = max_cols;
#line 531 "CollectionLayout.vala"
	self->priv->rows = row + 1;
#line 1629 "CollectionLayout.c"
	/* Step 5: Define the total size of the page as the size of the allocated width and
	 the height of all the items plus padding*/
#line 535 "CollectionLayout.vala"
	gtk_layout_set_size (GTK_LAYOUT (self), (guint) GTK_WIDGET (self)->allocation.width, (guint) ((y + row_heights[row]) + COLLECTION_LAYOUT_BOTTOM_PADDING));
#line 1634 "CollectionLayout.c"
	column_widths = (g_free (column_widths), NULL);
	row_heights = (g_free (row_heights), NULL);
}


#line 538 "CollectionLayout.vala"
static void collection_layout_display_message (CollectionLayout* self) {
#line 1642 "CollectionLayout.c"
	GtkRequisition req = {0};
	gint x;
	gint y;
#line 538 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 539 "CollectionLayout.vala"
	g_assert (string_get_length (gtk_label_get_text (self->priv->message)) > 0);
#line 542 "CollectionLayout.vala"
	gtk_widget_size_request (GTK_WIDGET (self->priv->message), &req);
#line 1652 "CollectionLayout.c"
	x = (GTK_WIDGET (self)->allocation.width - req.width) / 2;
#line 545 "CollectionLayout.vala"
	if (x < 0) {
#line 546 "CollectionLayout.vala"
		x = 0;
#line 1658 "CollectionLayout.c"
	}
	y = (GTK_WIDGET (self)->allocation.height - req.height) / 2;
#line 549 "CollectionLayout.vala"
	if (y < 0) {
#line 550 "CollectionLayout.vala"
		y = 0;
#line 1665 "CollectionLayout.c"
	}
#line 552 "CollectionLayout.vala"
	if (GTK_WIDGET (gtk_widget_get_parent (GTK_WIDGET (self->priv->message))) == GTK_WIDGET (self)) {
#line 553 "CollectionLayout.vala"
		gtk_layout_move (GTK_LAYOUT (self), GTK_WIDGET (self->priv->message), x, y);
#line 1671 "CollectionLayout.c"
	} else {
#line 555 "CollectionLayout.vala"
		gtk_layout_put (GTK_LAYOUT (self), GTK_WIDGET (self->priv->message), x, y);
#line 1675 "CollectionLayout.c"
	}
#line 558 "CollectionLayout.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->message));
#line 1679 "CollectionLayout.c"
}


#line 561 "CollectionLayout.vala"
static void collection_layout_on_resize (CollectionLayout* self) {
#line 1685 "CollectionLayout.c"
	gboolean _tmp0_;
#line 561 "CollectionLayout.vala"
	g_return_if_fail (IS_COLLECTION_LAYOUT (self));
#line 1689 "CollectionLayout.c"
	_tmp0_ = FALSE;
#line 563 "CollectionLayout.vala"
	if (self->priv->refresh_on_resize) {
#line 563 "CollectionLayout.vala"
		_tmp0_ = GTK_WIDGET (self)->allocation.width != self->priv->last_width;
#line 1695 "CollectionLayout.c"
	} else {
#line 563 "CollectionLayout.vala"
		_tmp0_ = FALSE;
#line 1699 "CollectionLayout.c"
	}
	/* only refresh() if the width has changed*/
#line 563 "CollectionLayout.vala"
	if (_tmp0_) {
#line 564 "CollectionLayout.vala"
		self->priv->last_width = GTK_WIDGET (self)->allocation.width;
#line 565 "CollectionLayout.vala"
		collection_layout_refresh (self);
#line 1708 "CollectionLayout.c"
	}
	/* this fixes a bug in GtkLayout where the step_increment field is not updated
	 in size_allocate
	 http://bugzilla.gnome.org/show_bug.cgi?id=587007*/
#line 571 "CollectionLayout.vala"
	gtk_adjustment_set_step_increment (gtk_layout_get_vadjustment (GTK_LAYOUT (self)), ((double) GTK_WIDGET (self)->allocation.height) * 0.10);
#line 1715 "CollectionLayout.c"
}


#line 574 "CollectionLayout.vala"
static gboolean collection_layout_real_expose_event (GtkWidget* base, const GdkEventExpose* event) {
#line 1721 "CollectionLayout.c"
	CollectionLayout * self;
	gboolean result;
	GdkRectangle _tmp1_ = {0};
	GdkRectangle visible_rect;
	GdkRectangle _tmp2_ = {0};
	GdkRectangle bitbucket;
	gboolean _tmp4_;
	gboolean _result_;
	self = COLLECTION_LAYOUT (base);
#line 575 "CollectionLayout.vala"
	if (!self->priv->in_view) {
#line 1733 "CollectionLayout.c"
		gboolean _tmp0_;
		gboolean _result_;
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 576 "CollectionLayout.vala"
			while (TRUE) {
#line 1741 "CollectionLayout.c"
				LayoutItem* item;
#line 576 "CollectionLayout.vala"
				if (!gee_iterator_next (_item_it)) {
#line 576 "CollectionLayout.vala"
					break;
#line 1747 "CollectionLayout.c"
				}
				item = (LayoutItem*) gee_iterator_get (_item_it);
#line 577 "CollectionLayout.vala"
				layout_item_unexposed (item);
#line 1752 "CollectionLayout.c"
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
		}
		_tmp0_ = FALSE;
#line 579 "CollectionLayout.vala"
		if (GTK_WIDGET_CLASS (collection_layout_parent_class)->expose_event != NULL) {
#line 579 "CollectionLayout.vala"
			_tmp0_ = GTK_WIDGET_CLASS (collection_layout_parent_class)->expose_event (GTK_WIDGET (GTK_LAYOUT (self)), &(*event));
#line 1762 "CollectionLayout.c"
		} else {
#line 579 "CollectionLayout.vala"
			_tmp0_ = TRUE;
#line 1766 "CollectionLayout.c"
		}
		_result_ = _tmp0_;
#line 581 "CollectionLayout.vala"
		g_signal_emit_by_name (self, "expose-after");
#line 1771 "CollectionLayout.c"
		result = _result_;
		return result;
	}
	visible_rect = (memset (&_tmp1_, 0, sizeof (GdkRectangle)), _tmp1_);
#line 587 "CollectionLayout.vala"
	visible_rect.x = (gint) gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (self)));
#line 588 "CollectionLayout.vala"
	visible_rect.y = (gint) gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (self)));
#line 589 "CollectionLayout.vala"
	visible_rect.width = GTK_WIDGET (self)->allocation.width;
#line 590 "CollectionLayout.vala"
	visible_rect.height = GTK_WIDGET (self)->allocation.height;
#line 1784 "CollectionLayout.c"
	bitbucket = (memset (&_tmp2_, 0, sizeof (GdkRectangle)), _tmp2_);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->items));
#line 594 "CollectionLayout.vala"
		while (TRUE) {
#line 1791 "CollectionLayout.c"
			LayoutItem* item;
			GdkRectangle _tmp3_ = {0};
#line 594 "CollectionLayout.vala"
			if (!gee_iterator_next (_item_it)) {
#line 594 "CollectionLayout.vala"
				break;
#line 1798 "CollectionLayout.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 595 "CollectionLayout.vala"
			if (gdk_rectangle_intersect (&visible_rect, (_tmp3_ = (GdkRectangle) GTK_WIDGET (item)->allocation, &_tmp3_), &bitbucket)) {
#line 596 "CollectionLayout.vala"
				layout_item_exposed (item);
#line 1805 "CollectionLayout.c"
			} else {
#line 598 "CollectionLayout.vala"
				layout_item_unexposed (item);
#line 1809 "CollectionLayout.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	_tmp4_ = FALSE;
#line 601 "CollectionLayout.vala"
	if (GTK_WIDGET_CLASS (collection_layout_parent_class)->expose_event != NULL) {
#line 601 "CollectionLayout.vala"
		_tmp4_ = GTK_WIDGET_CLASS (collection_layout_parent_class)->expose_event (GTK_WIDGET (GTK_LAYOUT (self)), &(*event));
#line 1820 "CollectionLayout.c"
	} else {
#line 601 "CollectionLayout.vala"
		_tmp4_ = TRUE;
#line 1824 "CollectionLayout.c"
	}
	_result_ = _tmp4_;
#line 603 "CollectionLayout.vala"
	g_signal_emit_by_name (self, "expose-after");
#line 1829 "CollectionLayout.c"
	result = _result_;
	return result;
}


static void collection_layout_class_init (CollectionLayoutClass * klass) {
	collection_layout_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (CollectionLayoutPrivate));
	GTK_WIDGET_CLASS (klass)->expose_event = collection_layout_real_expose_event;
	G_OBJECT_CLASS (klass)->finalize = collection_layout_finalize;
	g_signal_new ("expose_after", TYPE_COLLECTION_LAYOUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void collection_layout_instance_init (CollectionLayout * self) {
	self->priv = COLLECTION_LAYOUT_GET_PRIVATE (self);
	self->items = sorted_list_new (TYPE_LAYOUT_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, NULL);
	self->priv->message = g_object_ref_sink ((GtkLabel*) gtk_label_new (""));
	self->priv->in_view = FALSE;
	self->priv->last_width = 0;
	self->priv->refresh_on_resize = TRUE;
	self->priv->columns = 0;
	self->priv->rows = 0;
}


static void collection_layout_finalize (GObject* obj) {
	CollectionLayout * self;
	self = COLLECTION_LAYOUT (obj);
	(self->items == NULL) ? NULL : (self->items = (g_object_unref (self->items), NULL));
	(self->priv->message == NULL) ? NULL : (self->priv->message = (g_object_unref (self->priv->message), NULL));
	G_OBJECT_CLASS (collection_layout_parent_class)->finalize (obj);
}


GType collection_layout_get_type (void) {
	static GType collection_layout_type_id = 0;
	if (collection_layout_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CollectionLayoutClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) collection_layout_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CollectionLayout), 0, (GInstanceInitFunc) collection_layout_instance_init, NULL };
		collection_layout_type_id = g_type_register_static (GTK_TYPE_LAYOUT, "CollectionLayout", &g_define_type_info, 0);
	}
	return collection_layout_type_id;
}




