/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gio/gio.h>
#include <float.h>
#include <math.h>
#include <time.h>


#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_SINGLE_PHOTO_PAGE (single_photo_page_get_type ())
#define SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPage))
#define SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))
#define IS_SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SINGLE_PHOTO_PAGE))
#define IS_SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SINGLE_PHOTO_PAGE))
#define SINGLE_PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))

typedef struct _SinglePhotoPage SinglePhotoPage;
typedef struct _SinglePhotoPageClass SinglePhotoPageClass;
typedef struct _SinglePhotoPagePrivate SinglePhotoPagePrivate;

#define SINGLE_PHOTO_PAGE_TYPE_UPDATE_REASON (single_photo_page_update_reason_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_SLIDESHOW_PAGE (slideshow_page_get_type ())
#define SLIDESHOW_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SLIDESHOW_PAGE, SlideshowPage))
#define SLIDESHOW_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SLIDESHOW_PAGE, SlideshowPageClass))
#define IS_SLIDESHOW_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SLIDESHOW_PAGE))
#define IS_SLIDESHOW_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SLIDESHOW_PAGE))
#define SLIDESHOW_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SLIDESHOW_PAGE, SlideshowPageClass))

typedef struct _SlideshowPage SlideshowPage;
typedef struct _SlideshowPageClass SlideshowPageClass;
typedef struct _SlideshowPagePrivate SlideshowPagePrivate;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;

#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;

#define TYPE_THUMBNAIL (thumbnail_get_type ())
#define THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL, Thumbnail))
#define THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL, ThumbnailClass))
#define IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL))
#define IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL))
#define THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL, ThumbnailClass))

typedef struct _Thumbnail Thumbnail;
typedef struct _ThumbnailClass ThumbnailClass;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;
typedef struct _CheckerboardPagePrivate CheckerboardPagePrivate;

#define TYPE_COLLECTION_PAGE (collection_page_get_type ())
#define COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_PAGE, CollectionPage))
#define COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_PAGE, CollectionPageClass))
#define IS_COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_PAGE))
#define IS_COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_PAGE))
#define COLLECTION_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_PAGE, CollectionPageClass))

typedef struct _CollectionPage CollectionPage;
typedef struct _CollectionPageClass CollectionPageClass;
typedef struct _CollectionPagePrivate CollectionPagePrivate;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

#define PAGE_TYPE_TARGET_TYPE (page_target_type_get_type ())

#define TYPE_EXPORT_DIALOG (export_dialog_get_type ())
#define EXPORT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORT_DIALOG, ExportDialog))
#define EXPORT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORT_DIALOG, ExportDialogClass))
#define IS_EXPORT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORT_DIALOG))
#define IS_EXPORT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORT_DIALOG))
#define EXPORT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORT_DIALOG, ExportDialogClass))

typedef struct _ExportDialog ExportDialog;
typedef struct _ExportDialogClass ExportDialogClass;

#define TYPE_SCALE_CONSTRAINT (scale_constraint_get_type ())

#define JPEG_TYPE_QUALITY (jpeg_quality_get_type ())

#define TYPE_ROTATION (rotation_get_type ())

#define TYPE_FULLSCREEN_WINDOW (fullscreen_window_get_type ())
#define FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindow))
#define FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))
#define IS_FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FULLSCREEN_WINDOW))
#define IS_FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FULLSCREEN_WINDOW))
#define FULLSCREEN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))

typedef struct _FullscreenWindow FullscreenWindow;
typedef struct _FullscreenWindowClass FullscreenWindowClass;

#define COLLECTION_PAGE_TYPE_COMPARE_NAME (collection_page_compare_name_get_type ())
#define COLLECTION_PAGE_COMPARE_NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COLLECTION_PAGE_TYPE_COMPARE_NAME, CollectionPageCompareName))
#define COLLECTION_PAGE_COMPARE_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COLLECTION_PAGE_TYPE_COMPARE_NAME, CollectionPageCompareNameClass))
#define COLLECTION_PAGE_IS_COMPARE_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COLLECTION_PAGE_TYPE_COMPARE_NAME))
#define COLLECTION_PAGE_IS_COMPARE_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COLLECTION_PAGE_TYPE_COMPARE_NAME))
#define COLLECTION_PAGE_COMPARE_NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COLLECTION_PAGE_TYPE_COMPARE_NAME, CollectionPageCompareNameClass))

typedef struct _CollectionPageCompareName CollectionPageCompareName;
typedef struct _CollectionPageCompareNameClass CollectionPageCompareNameClass;

#define COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME (collection_page_reverse_compare_name_get_type ())
#define COLLECTION_PAGE_REVERSE_COMPARE_NAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME, CollectionPageReverseCompareName))
#define COLLECTION_PAGE_REVERSE_COMPARE_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME, CollectionPageReverseCompareNameClass))
#define COLLECTION_PAGE_IS_REVERSE_COMPARE_NAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME))
#define COLLECTION_PAGE_IS_REVERSE_COMPARE_NAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME))
#define COLLECTION_PAGE_REVERSE_COMPARE_NAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME, CollectionPageReverseCompareNameClass))

typedef struct _CollectionPageReverseCompareName CollectionPageReverseCompareName;
typedef struct _CollectionPageReverseCompareNameClass CollectionPageReverseCompareNameClass;

#define COLLECTION_PAGE_TYPE_COMPARE_DATE (collection_page_compare_date_get_type ())
#define COLLECTION_PAGE_COMPARE_DATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COLLECTION_PAGE_TYPE_COMPARE_DATE, CollectionPageCompareDate))
#define COLLECTION_PAGE_COMPARE_DATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COLLECTION_PAGE_TYPE_COMPARE_DATE, CollectionPageCompareDateClass))
#define COLLECTION_PAGE_IS_COMPARE_DATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COLLECTION_PAGE_TYPE_COMPARE_DATE))
#define COLLECTION_PAGE_IS_COMPARE_DATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COLLECTION_PAGE_TYPE_COMPARE_DATE))
#define COLLECTION_PAGE_COMPARE_DATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COLLECTION_PAGE_TYPE_COMPARE_DATE, CollectionPageCompareDateClass))

typedef struct _CollectionPageCompareDate CollectionPageCompareDate;
typedef struct _CollectionPageCompareDateClass CollectionPageCompareDateClass;

#define COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE (collection_page_reverse_compare_date_get_type ())
#define COLLECTION_PAGE_REVERSE_COMPARE_DATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE, CollectionPageReverseCompareDate))
#define COLLECTION_PAGE_REVERSE_COMPARE_DATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE, CollectionPageReverseCompareDateClass))
#define COLLECTION_PAGE_IS_REVERSE_COMPARE_DATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE))
#define COLLECTION_PAGE_IS_REVERSE_COMPARE_DATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE))
#define COLLECTION_PAGE_REVERSE_COMPARE_DATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE, CollectionPageReverseCompareDateClass))

typedef struct _CollectionPageReverseCompareDate CollectionPageReverseCompareDate;
typedef struct _CollectionPageReverseCompareDateClass CollectionPageReverseCompareDateClass;
typedef struct _ComparatorPrivate ComparatorPrivate;
typedef struct _CollectionPageCompareNamePrivate CollectionPageCompareNamePrivate;
typedef struct _CollectionPageReverseCompareNamePrivate CollectionPageReverseCompareNamePrivate;
typedef struct _CollectionPageCompareDatePrivate CollectionPageCompareDatePrivate;
typedef struct _CollectionPageReverseCompareDatePrivate CollectionPageReverseCompareDatePrivate;

struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkActionGroup* action_group;
	GtkActionGroup* common_action_group;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkToolbar* (*get_toolbar) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*switching_to_fullscreen) (Page* self);
	void (*returning_from_fullscreen) (Page* self);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, const GdkEventKey* event);
	void (*on_move) (Page* self, const GdkRectangle* rect);
	void (*on_resize) (Page* self, const GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
};

typedef enum  {
	SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS
} SinglePhotoPageUpdateReason;

struct _Dimensions {
	gint width;
	gint height;
};

struct _SinglePhotoPage {
	Page parent_instance;
	SinglePhotoPagePrivate * priv;
	GdkGC* canvas_gc;
	GtkDrawingArea* canvas;
	GtkViewport* viewport;
};

struct _SinglePhotoPageClass {
	PageClass parent_class;
	void (*new_drawable) (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable);
	void (*updated_pixbuf) (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
	void (*paint) (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
};

struct _SlideshowPage {
	SinglePhotoPage parent_instance;
	SlideshowPagePrivate * priv;
};

struct _SlideshowPageClass {
	SinglePhotoPageClass parent_class;
};

struct _SlideshowPagePrivate {
	CheckerboardPage* controller;
	Thumbnail* thumbnail;
	GtkToolbar* toolbar;
	GtkToolButton* play_pause_button;
	GTimer* timer;
	gboolean playing;
	gboolean exiting;
};

struct _CheckerboardPage {
	Page parent_instance;
	CheckerboardPagePrivate * priv;
};

struct _CheckerboardPageClass {
	PageClass parent_class;
	void (*on_selection_changed) (CheckerboardPage* self, gint count);
	GtkMenu* (*get_context_menu) (CheckerboardPage* self);
	void (*on_item_activated) (CheckerboardPage* self, LayoutItem* item);
	gboolean (*on_context_invoked) (CheckerboardPage* self, GtkMenu* context_menu);
	LayoutItem* (*get_fullscreen_photo) (CheckerboardPage* self);
	gboolean (*on_mouse_over) (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
};

struct _CollectionPage {
	CheckerboardPage parent_instance;
	CollectionPagePrivate * priv;
};

struct _CollectionPageClass {
	CheckerboardPageClass parent_class;
	void (*on_photos_menu) (CollectionPage* self);
};

struct _CollectionPagePrivate {
	GtkToolbar* toolbar;
	GtkHScale* slider;
	GtkToolButton* rotate_button;
	GtkToolButton* slideshow_button;
	gint scale;
	gboolean improval_scheduled;
	gboolean reschedule_improval;
	GeeArrayList* drag_items;
	gboolean thumbs_resized;
};

typedef enum  {
	PAGE_TARGET_TYPE_URI_LIST
} PageTargetType;

typedef enum  {
	SCALE_CONSTRAINT_ORIGINAL,
	SCALE_CONSTRAINT_DIMENSIONS,
	SCALE_CONSTRAINT_WIDTH,
	SCALE_CONSTRAINT_HEIGHT
} ScaleConstraint;

typedef enum  {
	JPEG_QUALITY_LOW = 50,
	JPEG_QUALITY_MEDIUM = 75,
	JPEG_QUALITY_HIGH = 90,
	JPEG_QUALITY_MAXIMUM = 100
} JpegQuality;

typedef enum  {
	ROTATION_CLOCKWISE,
	ROTATION_COUNTERCLOCKWISE,
	ROTATION_MIRROR,
	ROTATION_UPSIDE_DOWN
} Rotation;

struct _Comparator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ComparatorPrivate * priv;
};

struct _ComparatorClass {
	GTypeClass parent_class;
	void (*finalize) (Comparator *self);
	gint64 (*compare) (Comparator* self, gconstpointer a, gconstpointer b);
};

struct _CollectionPageCompareName {
	Comparator parent_instance;
	CollectionPageCompareNamePrivate * priv;
};

struct _CollectionPageCompareNameClass {
	ComparatorClass parent_class;
};

struct _CollectionPageReverseCompareName {
	Comparator parent_instance;
	CollectionPageReverseCompareNamePrivate * priv;
};

struct _CollectionPageReverseCompareNameClass {
	ComparatorClass parent_class;
};

struct _CollectionPageCompareDate {
	Comparator parent_instance;
	CollectionPageCompareDatePrivate * priv;
};

struct _CollectionPageCompareDateClass {
	ComparatorClass parent_class;
};

struct _CollectionPageReverseCompareDate {
	Comparator parent_instance;
	CollectionPageReverseCompareDatePrivate * priv;
};

struct _CollectionPageReverseCompareDateClass {
	ComparatorClass parent_class;
};


static gpointer slideshow_page_parent_class = NULL;
static GtkAdjustment* collection_page_slider_adjustment;
static GtkAdjustment* collection_page_slider_adjustment = NULL;
static gpointer collection_page_compare_name_parent_class = NULL;
static gpointer collection_page_reverse_compare_name_parent_class = NULL;
static gpointer collection_page_compare_date_parent_class = NULL;
static gpointer collection_page_reverse_compare_date_parent_class = NULL;
static gpointer collection_page_parent_class = NULL;

GType page_get_type (void);
GType single_photo_page_get_type (void);
GType single_photo_page_update_reason_get_type (void);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
GType slideshow_page_get_type (void);
GType checkerboard_page_get_type (void);
GType layout_item_get_type (void);
GType thumbnail_get_type (void);
#define SLIDESHOW_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SLIDESHOW_PAGE, SlideshowPagePrivate))
enum  {
	SLIDESHOW_PAGE_DUMMY_PROPERTY
};
#define SLIDESHOW_PAGE_DELAY_SEC 3
#define SLIDESHOW_PAGE_CHECK_ADVANCE_MSEC 250
SinglePhotoPage* single_photo_page_new (const char* page_name);
SinglePhotoPage* single_photo_page_construct (GType object_type, const char* page_name);
void single_photo_page_set_default_interp (SinglePhotoPage* self, GdkInterpType default_interp);
#define SINGLE_PHOTO_PAGE_QUALITY_INTERP GDK_INTERP_HYPER
static void slideshow_page_on_previous_manual (SlideshowPage* self);
static void _slideshow_page_on_previous_manual_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void slideshow_page_on_play_pause (SlideshowPage* self);
static void _slideshow_page_on_play_pause_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void slideshow_page_on_next_manual (SlideshowPage* self);
static void _slideshow_page_on_next_manual_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
SlideshowPage* slideshow_page_new (CheckerboardPage* controller, Thumbnail* start);
SlideshowPage* slideshow_page_construct (GType object_type, CheckerboardPage* controller, Thumbnail* start);
static GtkToolbar* slideshow_page_real_get_toolbar (Page* base);
GType photo_get_type (void);
Photo* thumbnail_get_photo (Thumbnail* self);
GdkPixbuf* photo_get_pixbuf (Photo* self, gint exceptions, gint scale, GdkInterpType interp, GError** error);
#define PHOTO_EXCEPTION_NONE 0
void single_photo_page_set_pixbuf (SinglePhotoPage* self, GdkPixbuf* unscaled);
static gboolean slideshow_page_auto_advance (SlideshowPage* self);
static gboolean _slideshow_page_auto_advance_gsource_func (gpointer self);
static void slideshow_page_real_switched_to (Page* base);
static void slideshow_page_real_switching_from (Page* base);
static void slideshow_page_manual_advance (SlideshowPage* self, Thumbnail* thumbnail);
LayoutItem* checkerboard_page_get_previous_item (CheckerboardPage* self, LayoutItem* current);
LayoutItem* checkerboard_page_get_next_item (CheckerboardPage* self, LayoutItem* current);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
static void slideshow_page_on_next_automatic (SlideshowPage* self);
GdkPixbuf* photo_get_thumbnail (Photo* self, gint scale);
#define THUMBNAIL_CACHE_BIG_SCALE 360
static gboolean slideshow_page_on_improvement (SlideshowPage* self);
static gboolean _slideshow_page_on_improvement_gsource_func (gpointer self);
static gboolean slideshow_page_real_key_press_event (GtkWidget* base, const GdkEventKey* event);
static void slideshow_page_finalize (GObject* obj);
GType collection_page_get_type (void);
#define COLLECTION_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_COLLECTION_PAGE, CollectionPagePrivate))
enum  {
	COLLECTION_PAGE_DUMMY_PROPERTY
};
#define THUMBNAIL_DEFAULT_SCALE 128
#define COLLECTION_PAGE_SORT_BY_MIN 0
#define COLLECTION_PAGE_SORT_BY_NAME 0
#define COLLECTION_PAGE_SORT_BY_EXPOSURE_DATE 1
#define COLLECTION_PAGE_SORT_BY_MAX 1
#define COLLECTION_PAGE_SORT_ORDER_MIN 0
#define COLLECTION_PAGE_SORT_ORDER_ASCENDING 0
#define COLLECTION_PAGE_SORT_ORDER_DESCENDING 1
#define COLLECTION_PAGE_SORT_ORDER_MAX 1
#define COLLECTION_PAGE_DEFAULT_SORT_BY COLLECTION_PAGE_SORT_BY_EXPOSURE_DATE
#define COLLECTION_PAGE_DEFAULT_SORT_ORDER COLLECTION_PAGE_SORT_ORDER_DESCENDING
#define COLLECTION_PAGE_MANUAL_STEPPING 16
#define COLLECTION_PAGE_SLIDER_STEPPING 2
#define COLLECTION_PAGE_IMPROVAL_PRIORITY G_PRIORITY_LOW
#define COLLECTION_PAGE_IMPROVAL_DELAY_MS 250
static void collection_page_on_file_menu (CollectionPage* self);
static void _collection_page_on_file_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_export (CollectionPage* self);
static void _collection_page_on_export_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_edit_menu (CollectionPage* self);
static void _collection_page_on_edit_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_select_all (CollectionPage* self);
static void _collection_page_on_select_all_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_remove (CollectionPage* self);
static void _collection_page_on_remove_gtk_action_callback (GtkAction* action, gpointer self);
void collection_page_on_photos_menu (CollectionPage* self);
static void _collection_page_on_photos_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_increase_size (CollectionPage* self);
static void _collection_page_on_increase_size_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_decrease_size (CollectionPage* self);
static void _collection_page_on_decrease_size_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_CLOCKWISE "shotwell-rotate-clockwise"
static void collection_page_on_rotate_clockwise (CollectionPage* self);
static void _collection_page_on_rotate_clockwise_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_COUNTERCLOCKWISE "shotwell-rotate-counterclockwise"
static void collection_page_on_rotate_counterclockwise (CollectionPage* self);
static void _collection_page_on_rotate_counterclockwise_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_MIRROR "shotwell-mirror"
static void collection_page_on_mirror (CollectionPage* self);
static void _collection_page_on_mirror_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_revert (CollectionPage* self);
static void _collection_page_on_revert_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_slideshow (CollectionPage* self);
static void _collection_page_on_slideshow_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_view_menu (CollectionPage* self);
static void _collection_page_on_view_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void collection_page_on_display_titles (CollectionPage* self, GtkAction* action);
static void _collection_page_on_display_titles_gtk_action_callback (GtkAction* action, gpointer self);
CheckerboardPage* checkerboard_page_new (const char* page_name);
CheckerboardPage* checkerboard_page_construct (GType object_type, const char* page_name);
void page_init_ui_start (Page* self, const char* ui_filename, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1);
static void collection_page_on_sort_changed (CollectionPage* self);
static void _collection_page_on_sort_changed_gtk_radio_action_callback (GtkAction* action, GtkAction* current, gpointer self);
void page_init_load_ui (Page* self, const char* ui_filename);
void page_init_ui_bind (Page* self, const char* menubar_path);
void checkerboard_page_init_context_menu (CheckerboardPage* self, const char* path);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
void checkerboard_page_set_layout_comparator (CheckerboardPage* self, Comparator* cmp);
static Comparator* collection_page_get_sort_comparator (CollectionPage* self);
static double collection_page_scale_to_slider (gint value);
#define THUMBNAIL_MAX_SCALE 360
#define RESOURCES_ROTATE_CLOCKWISE_LABEL "Rotate"
#define RESOURCES_ROTATE_CLOCKWISE_TOOLTIP "Rotate the photo(s) right"
static void _collection_page_on_rotate_clockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void _collection_page_on_slideshow_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void collection_page_on_slider_changed (CollectionPage* self);
static void _collection_page_on_slider_changed_gtk_range_value_changed (GtkHScale* _sender, gpointer self);
static void collection_page_schedule_thumbnail_improval (CollectionPage* self);
static void _collection_page_schedule_thumbnail_improval_gtk_adjustment_value_changed (GtkAdjustment* _sender, gpointer self);
void checkerboard_page_set_refresh_on_resize (CheckerboardPage* self, gboolean refresh_on_resize);
void checkerboard_page_refresh (CheckerboardPage* self);
void page_enable_drag_source (Page* self, GdkDragAction actions);
CollectionPage* collection_page_new (const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1);
CollectionPage* collection_page_construct (GType object_type, const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1);
static GtkToolbar* collection_page_real_get_toolbar (Page* base);
void page_switching_from (Page* self);
static void collection_page_real_switching_from (Page* base);
void page_switched_to (Page* self);
void collection_page_set_thumb_size (CollectionPage* self, gint new_scale);
static gint collection_page_slider_to_scale (double value);
static void collection_page_real_switched_to (Page* base);
void page_returning_from_fullscreen (Page* self);
static void collection_page_real_returning_from_fullscreen (Page* base);
static void collection_page_real_on_selection_changed (CheckerboardPage* base, gint count);
char* photo_to_string (Photo* self);
void app_window_switch_to_photo_page (AppWindow* self, CheckerboardPage* controller, Thumbnail* current);
static void collection_page_real_on_item_activated (CheckerboardPage* base, LayoutItem* item);
gint checkerboard_page_get_selected_count (CheckerboardPage* self);
GtkMenu* checkerboard_page_get_context_menu (CheckerboardPage* self);
static GtkMenu* collection_page_real_get_context_menu (CheckerboardPage* base);
static gboolean collection_page_can_revert_selected (CollectionPage* self);
void page_set_item_sensitive (Page* self, const char* path, gboolean sensitive);
static gboolean collection_page_real_on_context_invoked (CheckerboardPage* base, GtkMenu* context_menu);
GeeIterable* checkerboard_page_get_selected (CheckerboardPage* self);
GeeIterable* checkerboard_page_get_items (CheckerboardPage* self);
static LayoutItem* collection_page_real_get_fullscreen_photo (CheckerboardPage* base);
static void collection_page_real_on_resize (Page* base, const GdkRectangle* rect);
GFile* photo_generate_exportable (Photo* self, GError** error);
#define THUMBNAIL_CACHE_MEDIUM_SCALE 128
GtkWidget* page_get_event_source (Page* self);
static void collection_page_real_drag_begin (GtkWidget* base, GdkDragContext* context);
GType page_target_type_get_type (void);
static void collection_page_real_drag_data_get (GtkWidget* base, GdkDragContext* context, GtkSelectionData* selection_data, guint target_type, guint time);
static void collection_page_real_drag_end (GtkWidget* base, GdkDragContext* context);
static gboolean collection_page_real_source_drag_failed (Page* base, GdkDragContext* context, GtkDragResult drag_result);
static Thumbnail* collection_page_get_thumbnail_for_photo (CollectionPage* self, Photo* photo);
static void collection_page_on_photo_removed (CollectionPage* self, Photo* photo);
static void _collection_page_on_photo_removed_photo_removed (Photo* _sender, gpointer self);
static void collection_page_on_thumbnail_altered (CollectionPage* self, Photo* photo);
static void _collection_page_on_thumbnail_altered_photo_thumbnail_altered (Photo* _sender, gpointer self);
Thumbnail* thumbnail_new (Photo* photo, gint scale);
Thumbnail* thumbnail_construct (GType object_type, Photo* photo, gint scale);
void layout_item_display_title (LayoutItem* self, gboolean display);
static gboolean collection_page_display_titles (CollectionPage* self);
void checkerboard_page_add_item (CheckerboardPage* self, LayoutItem* item);
void collection_page_add_photo (CollectionPage* self, Photo* photo);
void checkerboard_page_remove_item (CheckerboardPage* self, LayoutItem* item);
gint checkerboard_page_get_count (CheckerboardPage* self);
gboolean page_is_in_view (Page* self);
gboolean photo_equals (Photo* self, Photo* photo);
gint collection_page_increase_thumb_size (CollectionPage* self);
#define THUMBNAIL_MIN_SCALE 64
gint collection_page_decrease_thumb_size (CollectionPage* self);
void thumbnail_resize (Thumbnail* self, gint new_scale);
static gboolean collection_page_improve_thumbnail_quality (CollectionPage* self);
static gboolean _collection_page_improve_thumbnail_quality_gsource_func (gpointer self);
gboolean thumbnail_is_exposed (Thumbnail* self);
void thumbnail_paint_high_quality (Thumbnail* self);
ExportDialog* export_dialog_new (gint count);
ExportDialog* export_dialog_construct (GType object_type, gint count);
GType export_dialog_get_type (void);
GType scale_constraint_get_type (void);
GType jpeg_quality_get_type (void);
gboolean export_dialog_execute (ExportDialog* self, gint* scale, ScaleConstraint* constraint, JpegQuality* quality);
GFile* export_ui_choose_file (GFile* current_file);
GFile* photo_get_file (Photo* self);
gboolean spin_event_loop (void);
void photo_export (Photo* self, GFile* dest_file, gint scale, ScaleConstraint constraint, JpegQuality quality, GError** error);
void app_window_error_message (const char* message);
GFile* export_ui_choose_dir (void);
void app_window_set_busy_cursor (AppWindow* self);
gboolean export_ui_query_overwrite (GFile* file);
void app_window_set_normal_cursor (AppWindow* self);
void checkerboard_page_select_all (CheckerboardPage* self);
gboolean photo_has_transformations (Photo* self);
static void collection_page_real_on_photos_menu (CollectionPage* self);
void photo_remove (Photo* self, gboolean remove_original);
gint checkerboard_page_remove_selected (CheckerboardPage* self);
GType rotation_get_type (void);
void photo_rotate (Photo* self, Rotation rotation);
static void collection_page_do_rotations (CollectionPage* self, GeeIterable* c, Rotation rotation);
void photo_remove_all_transformations (Photo* self);
GType fullscreen_window_get_type (void);
void app_window_go_fullscreen (AppWindow* self, FullscreenWindow* fullscreen_window);
LayoutItem* checkerboard_page_get_first_item (CheckerboardPage* self);
FullscreenWindow* fullscreen_window_new (Page* page);
FullscreenWindow* fullscreen_window_construct (GType object_type, Page* page);
#define RESOURCES_ROTATE_COUNTERCLOCKWISE_LABEL "Rotate"
#define RESOURCES_ROTATE_COUNTERCLOCKWISE_TOOLTIP "Rotate the photo(s) left"
static void _collection_page_on_rotate_counterclockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static gboolean collection_page_real_on_ctrl_pressed (Page* base, const GdkEventKey* event);
static gboolean collection_page_real_on_ctrl_released (Page* base, const GdkEventKey* event);
static gint collection_page_get_sort_criteria (CollectionPage* self);
static gint collection_page_get_sort_order (CollectionPage* self);
static gboolean collection_page_is_sort_ascending (CollectionPage* self);
static CollectionPageCompareName* collection_page_compare_name_new (void);
static CollectionPageCompareName* collection_page_compare_name_construct (GType object_type);
static GType collection_page_compare_name_get_type (void);
static CollectionPageReverseCompareName* collection_page_reverse_compare_name_new (void);
static CollectionPageReverseCompareName* collection_page_reverse_compare_name_construct (GType object_type);
static GType collection_page_reverse_compare_name_get_type (void);
static CollectionPageCompareDate* collection_page_compare_date_new (void);
static CollectionPageCompareDate* collection_page_compare_date_construct (GType object_type);
static GType collection_page_compare_date_get_type (void);
static CollectionPageReverseCompareDate* collection_page_reverse_compare_date_new (void);
static CollectionPageReverseCompareDate* collection_page_reverse_compare_date_construct (GType object_type);
static GType collection_page_reverse_compare_date_get_type (void);
enum  {
	COLLECTION_PAGE_COMPARE_NAME_DUMMY_PROPERTY
};
char* layout_item_get_title (LayoutItem* self);
static gint64 collection_page_compare_name_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b);
enum  {
	COLLECTION_PAGE_REVERSE_COMPARE_NAME_DUMMY_PROPERTY
};
static gint64 collection_page_reverse_compare_name_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b);
enum  {
	COLLECTION_PAGE_COMPARE_DATE_DUMMY_PROPERTY
};
time_t photo_get_exposure_time (Photo* self);
static gint64 collection_page_compare_date_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b);
enum  {
	COLLECTION_PAGE_REVERSE_COMPARE_DATE_DUMMY_PROPERTY
};
static gint64 collection_page_reverse_compare_date_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b);
static void collection_page_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const GtkActionEntry COLLECTION_PAGE_ACTIONS[] = {{"FileMenu", NULL, "_File", NULL, NULL, (GCallback) _collection_page_on_file_menu_gtk_action_callback}, {"Export", GTK_STOCK_SAVE_AS, "_Export Photos...", "<Ctrl>E", "Export selected photos to disk", (GCallback) _collection_page_on_export_gtk_action_callback}, {"EditMenu", NULL, "_Edit", NULL, NULL, (GCallback) _collection_page_on_edit_menu_gtk_action_callback}, {"SelectAll", GTK_STOCK_SELECT_ALL, "Select _All", "<Ctrl>A", "Select all the photos in the library", (GCallback) _collection_page_on_select_all_gtk_action_callback}, {"Remove", GTK_STOCK_DELETE, "_Remove", "Delete", "Remove the selected photos from the library", (GCallback) _collection_page_on_remove_gtk_action_callback}, {"PhotosMenu", NULL, "_Photos", NULL, NULL, (GCallback) _collection_page_on_photos_menu_gtk_action_callback}, {"IncreaseSize", GTK_STOCK_ZOOM_IN, "Zoom _In", "bracketright", "Increase the magnification of the thumbnails", (GCallback) _collection_page_on_increase_size_gtk_action_callback}, {"DecreaseSize", GTK_STOCK_ZOOM_OUT, "Zoom _Out", "bracketleft", "Decrease the magnification of the thumbnails", (GCallback) _collection_page_on_decrease_size_gtk_action_callback}, {"RotateClockwise", RESOURCES_CLOCKWISE, "Rotate _Right", "<Ctrl>R", "Rotate the selected photos clockwise", (GCallback) _collection_page_on_rotate_clockwise_gtk_action_callback}, {"RotateCounterclockwise", RESOURCES_COUNTERCLOCKWISE, "Rotate _Left", "<Ctrl><Shift>R", "Rotate the selected photos counterclockwise", (GCallback) _collection_page_on_rotate_counterclockwise_gtk_action_callback}, {"Mirror", RESOURCES_MIRROR, "_Mirror", "<Ctrl>M", "Make mirror images of the selected photos", (GCallback) _collection_page_on_mirror_gtk_action_callback}, {"Revert", GTK_STOCK_REVERT_TO_SAVED, "Re_vert to Original", NULL, "Revert to original photo", (GCallback) _collection_page_on_revert_gtk_action_callback}, {"Slideshow", GTK_STOCK_MEDIA_PLAY, "_Slideshow", "F5", "Play a slideshow", (GCallback) _collection_page_on_slideshow_gtk_action_callback}, {"ViewMenu", NULL, "_View", NULL, NULL, (GCallback) _collection_page_on_view_menu_gtk_action_callback}, {"SortPhotos", NULL, "Sort _Photos", NULL, NULL, (GCallback) NULL}, {"HelpMenu", NULL, "_Help", NULL, NULL, (GCallback) NULL}};
static const GtkToggleActionEntry COLLECTION_PAGE_TOGGLE_ACTIONS[] = {{"ViewTitle", NULL, "_Titles", "<Ctrl><Shift>T", "Display the title of each photo", (GCallback) _collection_page_on_display_titles_gtk_action_callback, TRUE}};
static const GtkRadioActionEntry COLLECTION_PAGE_SORT_CRIT_ACTIONS[] = {{"SortByName", NULL, "By _Name", NULL, "Sort photos by name", COLLECTION_PAGE_SORT_BY_NAME}, {"SortByExposureDate", NULL, "By Exposure _Date", NULL, "Sort photos by exposure date", COLLECTION_PAGE_SORT_BY_EXPOSURE_DATE}};
static const GtkRadioActionEntry COLLECTION_PAGE_SORT_ORDER_ACTIONS[] = {{"SortAscending", GTK_STOCK_SORT_ASCENDING, "_Ascending", NULL, "Sort photos in an ascending order", COLLECTION_PAGE_SORT_ORDER_ASCENDING}, {"SortDescending", GTK_STOCK_SORT_DESCENDING, "D_escending", NULL, "Sort photos in a descending order", COLLECTION_PAGE_SORT_ORDER_DESCENDING}};


#line 83 "CollectionPage.vala"
static void _slideshow_page_on_previous_manual_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 672 "CollectionPage.c"
	slideshow_page_on_previous_manual (self);
}


#line 66 "CollectionPage.vala"
static void _slideshow_page_on_play_pause_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 679 "CollectionPage.c"
	slideshow_page_on_play_pause (self);
}


#line 101 "CollectionPage.vala"
static void _slideshow_page_on_next_manual_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 686 "CollectionPage.c"
	slideshow_page_on_next_manual (self);
}


#line 20 "CollectionPage.vala"
SlideshowPage* slideshow_page_construct (GType object_type, CheckerboardPage* controller, Thumbnail* start) {
#line 693 "CollectionPage.c"
	SlideshowPage * self;
	CheckerboardPage* _tmp1_;
	CheckerboardPage* _tmp0_;
	Thumbnail* _tmp3_;
	Thumbnail* _tmp2_;
	GtkToolButton* previous_button;
	GtkToolButton* _tmp4_;
	GtkToolButton* next_button;
#line 20 "CollectionPage.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (controller), NULL);
#line 20 "CollectionPage.vala"
	g_return_val_if_fail (IS_THUMBNAIL (start), NULL);
#line 21 "CollectionPage.vala"
	self = (SlideshowPage*) single_photo_page_construct (object_type, "Slideshow");
#line 708 "CollectionPage.c"
	_tmp1_ = NULL;
#line 23 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 23 "CollectionPage.vala"
	self->priv->controller = (_tmp1_ = (_tmp0_ = controller, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->controller == NULL) ? NULL : (self->priv->controller = (g_object_unref (self->priv->controller), NULL)), _tmp1_);
#line 714 "CollectionPage.c"
	_tmp3_ = NULL;
#line 24 "CollectionPage.vala"
	_tmp2_ = NULL;
#line 24 "CollectionPage.vala"
	self->priv->thumbnail = (_tmp3_ = (_tmp2_ = start, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL)), _tmp3_);
#line 26 "CollectionPage.vala"
	single_photo_page_set_default_interp (SINGLE_PHOTO_PAGE (self), SINGLE_PHOTO_PAGE_QUALITY_INTERP);
#line 722 "CollectionPage.c"
	previous_button = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_GO_BACK));
#line 30 "CollectionPage.vala"
	gtk_tool_button_set_label (previous_button, "Back");
#line 31 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (previous_button), "Go to the previous photo");
#line 32 "CollectionPage.vala"
	g_signal_connect_object (previous_button, "clicked", (GCallback) _slideshow_page_on_previous_manual_gtk_tool_button_clicked, self, 0);
#line 34 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (previous_button), -1);
#line 732 "CollectionPage.c"
	_tmp4_ = NULL;
#line 36 "CollectionPage.vala"
	self->priv->play_pause_button = (_tmp4_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_PAUSE)), (self->priv->play_pause_button == NULL) ? NULL : (self->priv->play_pause_button = (g_object_unref (self->priv->play_pause_button), NULL)), _tmp4_);
#line 37 "CollectionPage.vala"
	gtk_tool_button_set_label (self->priv->play_pause_button, "Pause");
#line 38 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->play_pause_button), "Pause the slideshow");
#line 39 "CollectionPage.vala"
	g_signal_connect_object (self->priv->play_pause_button, "clicked", (GCallback) _slideshow_page_on_play_pause_gtk_tool_button_clicked, self, 0);
#line 41 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->play_pause_button), -1);
#line 744 "CollectionPage.c"
	next_button = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD));
#line 44 "CollectionPage.vala"
	gtk_tool_button_set_label (next_button, "Next");
#line 45 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (next_button), "Go to the next photo");
#line 46 "CollectionPage.vala"
	g_signal_connect_object (next_button, "clicked", (GCallback) _slideshow_page_on_next_manual_gtk_tool_button_clicked, self, 0);
#line 48 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (next_button), -1);
#line 754 "CollectionPage.c"
	(previous_button == NULL) ? NULL : (previous_button = (g_object_unref (previous_button), NULL));
	(next_button == NULL) ? NULL : (next_button = (g_object_unref (next_button), NULL));
	return self;
}


#line 20 "CollectionPage.vala"
SlideshowPage* slideshow_page_new (CheckerboardPage* controller, Thumbnail* start) {
#line 20 "CollectionPage.vala"
	return slideshow_page_construct (TYPE_SLIDESHOW_PAGE, controller, start);
#line 765 "CollectionPage.c"
}


#line 51 "CollectionPage.vala"
static GtkToolbar* slideshow_page_real_get_toolbar (Page* base) {
#line 771 "CollectionPage.c"
	SlideshowPage * self;
	GtkToolbar* result;
	GtkToolbar* _tmp0_;
	self = SLIDESHOW_PAGE (base);
#line 52 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 778 "CollectionPage.c"
	result = (_tmp0_ = self->priv->toolbar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 124 "CollectionPage.vala"
static gboolean _slideshow_page_auto_advance_gsource_func (gpointer self) {
#line 786 "CollectionPage.c"
	return slideshow_page_auto_advance (self);
}


#line 55 "CollectionPage.vala"
static void slideshow_page_real_switched_to (Page* base) {
#line 793 "CollectionPage.c"
	SlideshowPage * self;
	GError * _inner_error_;
	Photo* _tmp0_;
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp2_;
	GdkPixbuf* _tmp3_;
	self = SLIDESHOW_PAGE (base);
	_inner_error_ = NULL;
#line 39 "Thumbnail.vala"
	_tmp0_ = NULL;
#line 56 "CollectionPage.vala"
	_tmp1_ = NULL;
#line 806 "CollectionPage.c"
	_tmp2_ = (_tmp1_ = photo_get_pixbuf (_tmp0_ = thumbnail_get_photo (self->priv->thumbnail), PHOTO_EXCEPTION_NONE, 0, GDK_INTERP_HYPER, &_inner_error_), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
#line 615 "Photo.vala"
	_tmp3_ = NULL;
#line 56 "CollectionPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp3_ = _tmp2_);
#line 817 "CollectionPage.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 58 "CollectionPage.vala"
	g_timeout_add ((guint) SLIDESHOW_PAGE_CHECK_ADVANCE_MSEC, _slideshow_page_auto_advance_gsource_func, self);
#line 59 "CollectionPage.vala"
	g_timer_start (self->priv->timer);
#line 823 "CollectionPage.c"
}


#line 62 "CollectionPage.vala"
static void slideshow_page_real_switching_from (Page* base) {
#line 829 "CollectionPage.c"
	SlideshowPage * self;
	self = SLIDESHOW_PAGE (base);
#line 63 "CollectionPage.vala"
	self->priv->exiting = TRUE;
#line 834 "CollectionPage.c"
}


#line 66 "CollectionPage.vala"
static void slideshow_page_on_play_pause (SlideshowPage* self) {
#line 66 "CollectionPage.vala"
	g_return_if_fail (IS_SLIDESHOW_PAGE (self));
#line 67 "CollectionPage.vala"
	if (self->priv->playing) {
#line 68 "CollectionPage.vala"
		gtk_tool_button_set_stock_id (self->priv->play_pause_button, GTK_STOCK_MEDIA_PLAY);
#line 69 "CollectionPage.vala"
		gtk_tool_button_set_label (self->priv->play_pause_button, "Play");
#line 70 "CollectionPage.vala"
		gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->play_pause_button), "Continue the slideshow");
#line 850 "CollectionPage.c"
	} else {
#line 72 "CollectionPage.vala"
		gtk_tool_button_set_stock_id (self->priv->play_pause_button, GTK_STOCK_MEDIA_PAUSE);
#line 73 "CollectionPage.vala"
		gtk_tool_button_set_label (self->priv->play_pause_button, "Pause");
#line 74 "CollectionPage.vala"
		gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->play_pause_button), "Pause the slideshow");
#line 858 "CollectionPage.c"
	}
#line 77 "CollectionPage.vala"
	self->priv->playing = !self->priv->playing;
#line 862 "CollectionPage.c"
	/* reset the timer*/
#line 80 "CollectionPage.vala"
	g_timer_start (self->priv->timer);
#line 866 "CollectionPage.c"
}


#line 83 "CollectionPage.vala"
static void slideshow_page_on_previous_manual (SlideshowPage* self) {
#line 872 "CollectionPage.c"
	Thumbnail* _tmp0_;
#line 83 "CollectionPage.vala"
	g_return_if_fail (IS_SLIDESHOW_PAGE (self));
#line 84 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 84 "CollectionPage.vala"
	slideshow_page_manual_advance (self, _tmp0_ = THUMBNAIL (checkerboard_page_get_previous_item (self->priv->controller, LAYOUT_ITEM (self->priv->thumbnail))));
#line 880 "CollectionPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 87 "CollectionPage.vala"
static void slideshow_page_on_next_automatic (SlideshowPage* self) {
#line 887 "CollectionPage.c"
	GError * _inner_error_;
	Thumbnail* _tmp0_;
	GdkScreen* _tmp2_;
	AppWindow* _tmp1_;
	GdkScreen* _tmp3_;
	GdkScreen* screen;
	gint scale;
	Photo* _tmp4_;
	GdkPixbuf* _tmp5_;
	GdkPixbuf* _tmp6_;
	GdkPixbuf* _tmp7_;
#line 87 "CollectionPage.vala"
	g_return_if_fail (IS_SLIDESHOW_PAGE (self));
#line 901 "CollectionPage.c"
	_inner_error_ = NULL;
	_tmp0_ = NULL;
#line 88 "CollectionPage.vala"
	self->priv->thumbnail = (_tmp0_ = THUMBNAIL (checkerboard_page_get_next_item (self->priv->controller, LAYOUT_ITEM (self->priv->thumbnail))), (self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL)), _tmp0_);
#line 906 "CollectionPage.c"
	/* Photo.get_pixbuf can optimize its pipeline with a scaled pixbuf, so ask for one that
	 fits in the screen*/
#line 92 "CollectionPage.vala"
	_tmp2_ = NULL;
#line 446 "AppWindow.vala"
	_tmp1_ = NULL;
#line 92 "CollectionPage.vala"
	_tmp3_ = NULL;
#line 915 "CollectionPage.c"
	screen = (_tmp3_ = (_tmp2_ = gdk_drawable_get_screen (GDK_DRAWABLE (GTK_WIDGET (_tmp1_ = app_window_get_instance ())->window)), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp3_);
	scale = MAX (gdk_screen_get_width (screen), gdk_screen_get_height (screen));
#line 39 "Thumbnail.vala"
	_tmp4_ = NULL;
#line 95 "CollectionPage.vala"
	_tmp5_ = NULL;
#line 922 "CollectionPage.c"
	_tmp6_ = (_tmp5_ = photo_get_pixbuf (_tmp4_ = thumbnail_get_photo (self->priv->thumbnail), PHOTO_EXCEPTION_NONE, scale, GDK_INTERP_HYPER, &_inner_error_), (_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL)), _tmp5_);
	if (_inner_error_ != NULL) {
		(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
#line 615 "Photo.vala"
	_tmp7_ = NULL;
#line 95 "CollectionPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp7_ = _tmp6_);
#line 934 "CollectionPage.c"
	(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
	/* reset the timer*/
#line 98 "CollectionPage.vala"
	g_timer_start (self->priv->timer);
#line 939 "CollectionPage.c"
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


#line 101 "CollectionPage.vala"
static void slideshow_page_on_next_manual (SlideshowPage* self) {
#line 946 "CollectionPage.c"
	Thumbnail* _tmp0_;
#line 101 "CollectionPage.vala"
	g_return_if_fail (IS_SLIDESHOW_PAGE (self));
#line 102 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 102 "CollectionPage.vala"
	slideshow_page_manual_advance (self, _tmp0_ = THUMBNAIL (checkerboard_page_get_next_item (self->priv->controller, LAYOUT_ITEM (self->priv->thumbnail))));
#line 954 "CollectionPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 118 "CollectionPage.vala"
static gboolean _slideshow_page_on_improvement_gsource_func (gpointer self) {
#line 961 "CollectionPage.c"
	return slideshow_page_on_improvement (self);
}


#line 105 "CollectionPage.vala"
static void slideshow_page_manual_advance (SlideshowPage* self, Thumbnail* thumbnail) {
#line 968 "CollectionPage.c"
	Thumbnail* _tmp1_;
	Thumbnail* _tmp0_;
	GdkPixbuf* _tmp3_;
	Photo* _tmp2_;
#line 105 "CollectionPage.vala"
	g_return_if_fail (IS_SLIDESHOW_PAGE (self));
#line 105 "CollectionPage.vala"
	g_return_if_fail (IS_THUMBNAIL (thumbnail));
#line 977 "CollectionPage.c"
	_tmp1_ = NULL;
#line 106 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 106 "CollectionPage.vala"
	self->priv->thumbnail = (_tmp1_ = (_tmp0_ = thumbnail, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL)), _tmp1_);
#line 983 "CollectionPage.c"
	/* start with blown-up thumbnail*/
#line 841 "Photo.vala"
	_tmp3_ = NULL;
#line 39 "Thumbnail.vala"
	_tmp2_ = NULL;
#line 109 "CollectionPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp3_ = photo_get_thumbnail (_tmp2_ = thumbnail_get_photo (thumbnail), THUMBNAIL_CACHE_BIG_SCALE));
#line 991 "CollectionPage.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
	/* schedule improvement to real photo*/
#line 112 "CollectionPage.vala"
	g_idle_add (_slideshow_page_on_improvement_gsource_func, self);
#line 997 "CollectionPage.c"
	/* reset the advance timer*/
#line 115 "CollectionPage.vala"
	g_timer_start (self->priv->timer);
#line 1001 "CollectionPage.c"
}


#line 118 "CollectionPage.vala"
static gboolean slideshow_page_on_improvement (SlideshowPage* self) {
#line 1007 "CollectionPage.c"
	gboolean result;
	GError * _inner_error_;
	Photo* _tmp0_;
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp2_;
	GdkPixbuf* _tmp3_;
#line 118 "CollectionPage.vala"
	g_return_val_if_fail (IS_SLIDESHOW_PAGE (self), FALSE);
#line 1016 "CollectionPage.c"
	_inner_error_ = NULL;
#line 39 "Thumbnail.vala"
	_tmp0_ = NULL;
#line 119 "CollectionPage.vala"
	_tmp1_ = NULL;
#line 1022 "CollectionPage.c"
	_tmp2_ = (_tmp1_ = photo_get_pixbuf (_tmp0_ = thumbnail_get_photo (self->priv->thumbnail), PHOTO_EXCEPTION_NONE, 0, GDK_INTERP_HYPER, &_inner_error_), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
#line 615 "Photo.vala"
	_tmp3_ = NULL;
#line 119 "CollectionPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp3_ = _tmp2_);
#line 1033 "CollectionPage.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
	result = FALSE;
	return result;
}


#line 124 "CollectionPage.vala"
static gboolean slideshow_page_auto_advance (SlideshowPage* self) {
#line 1042 "CollectionPage.c"
	gboolean result;
#line 124 "CollectionPage.vala"
	g_return_val_if_fail (IS_SLIDESHOW_PAGE (self), FALSE);
#line 125 "CollectionPage.vala"
	if (self->priv->exiting) {
#line 1048 "CollectionPage.c"
		result = FALSE;
		return result;
	}
#line 128 "CollectionPage.vala"
	if (!self->priv->playing) {
#line 1054 "CollectionPage.c"
		result = TRUE;
		return result;
	}
#line 131 "CollectionPage.vala"
	if (((gint) g_timer_elapsed (self->priv->timer, NULL)) < SLIDESHOW_PAGE_DELAY_SEC) {
#line 1060 "CollectionPage.c"
		result = TRUE;
		return result;
	}
#line 134 "CollectionPage.vala"
	slideshow_page_on_next_automatic (self);
#line 1066 "CollectionPage.c"
	result = TRUE;
	return result;
}


#line 139 "CollectionPage.vala"
static gboolean slideshow_page_real_key_press_event (GtkWidget* base, const GdkEventKey* event) {
#line 1074 "CollectionPage.c"
	SlideshowPage * self;
	gboolean result;
	gboolean handled;
	GQuark _tmp1_;
	const char* _tmp0_;
	static GQuark _tmp1__label0 = 0;
	static GQuark _tmp1__label1 = 0;
	static GQuark _tmp1__label2 = 0;
	static GQuark _tmp1__label3 = 0;
	static GQuark _tmp1__label4 = 0;
	gboolean _tmp2_;
	self = SLIDESHOW_PAGE (base);
	handled = TRUE;
#line 141 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 1090 "CollectionPage.c"
	_tmp0_ = gdk_keyval_name ((*event).keyval);
	_tmp1_ = (NULL == _tmp0_) ? 0 : g_quark_from_string (_tmp0_);
	if (_tmp1_ == ((0 != _tmp1__label0) ? _tmp1__label0 : (_tmp1__label0 = g_quark_from_static_string ("space"))))
	do {
#line 143 "CollectionPage.vala"
		slideshow_page_on_play_pause (self);
#line 144 "CollectionPage.vala"
		break;
#line 1099 "CollectionPage.c"
	} while (0); else if ((_tmp1_ == ((0 != _tmp1__label1) ? _tmp1__label1 : (_tmp1__label1 = g_quark_from_static_string ("Left")))) || (_tmp1_ == ((0 != _tmp1__label2) ? _tmp1__label2 : (_tmp1__label2 = g_quark_from_static_string ("KP_Left")))))
	do {
#line 148 "CollectionPage.vala"
		slideshow_page_on_previous_manual (self);
#line 149 "CollectionPage.vala"
		break;
#line 1106 "CollectionPage.c"
	} while (0); else if ((_tmp1_ == ((0 != _tmp1__label3) ? _tmp1__label3 : (_tmp1__label3 = g_quark_from_static_string ("Right")))) || (_tmp1_ == ((0 != _tmp1__label4) ? _tmp1__label4 : (_tmp1__label4 = g_quark_from_static_string ("KP_Right")))))
	do {
#line 153 "CollectionPage.vala"
		slideshow_page_on_next_manual (self);
#line 154 "CollectionPage.vala"
		break;
#line 1113 "CollectionPage.c"
	} while (0); else
	do {
#line 157 "CollectionPage.vala"
		handled = FALSE;
#line 158 "CollectionPage.vala"
		break;
#line 1120 "CollectionPage.c"
	} while (0);
#line 161 "CollectionPage.vala"
	if (handled) {
#line 1124 "CollectionPage.c"
		result = TRUE;
		return result;
	}
	_tmp2_ = FALSE;
#line 164 "CollectionPage.vala"
	if (GTK_WIDGET_CLASS (slideshow_page_parent_class)->key_press_event != NULL) {
#line 164 "CollectionPage.vala"
		_tmp2_ = GTK_WIDGET_CLASS (slideshow_page_parent_class)->key_press_event (GTK_WIDGET (SINGLE_PHOTO_PAGE (self)), &(*event));
#line 1133 "CollectionPage.c"
	} else {
#line 164 "CollectionPage.vala"
		_tmp2_ = TRUE;
#line 1137 "CollectionPage.c"
	}
	result = _tmp2_;
	return result;
}


static void slideshow_page_class_init (SlideshowPageClass * klass) {
	slideshow_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SlideshowPagePrivate));
	PAGE_CLASS (klass)->get_toolbar = slideshow_page_real_get_toolbar;
	PAGE_CLASS (klass)->switched_to = slideshow_page_real_switched_to;
	PAGE_CLASS (klass)->switching_from = slideshow_page_real_switching_from;
	GTK_WIDGET_CLASS (klass)->key_press_event = slideshow_page_real_key_press_event;
	G_OBJECT_CLASS (klass)->finalize = slideshow_page_finalize;
}


static void slideshow_page_instance_init (SlideshowPage * self) {
	self->priv = SLIDESHOW_PAGE_GET_PRIVATE (self);
	self->priv->toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
	self->priv->timer = g_timer_new ();
	self->priv->playing = TRUE;
	self->priv->exiting = FALSE;
}


static void slideshow_page_finalize (GObject* obj) {
	SlideshowPage * self;
	self = SLIDESHOW_PAGE (obj);
	(self->priv->controller == NULL) ? NULL : (self->priv->controller = (g_object_unref (self->priv->controller), NULL));
	(self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL));
	(self->priv->toolbar == NULL) ? NULL : (self->priv->toolbar = (g_object_unref (self->priv->toolbar), NULL));
	(self->priv->play_pause_button == NULL) ? NULL : (self->priv->play_pause_button = (g_object_unref (self->priv->play_pause_button), NULL));
	(self->priv->timer == NULL) ? NULL : (self->priv->timer = (g_timer_destroy (self->priv->timer), NULL));
	G_OBJECT_CLASS (slideshow_page_parent_class)->finalize (obj);
}


GType slideshow_page_get_type (void) {
	static GType slideshow_page_type_id = 0;
	if (slideshow_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SlideshowPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) slideshow_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SlideshowPage), 0, (GInstanceInitFunc) slideshow_page_instance_init, NULL };
		slideshow_page_type_id = g_type_register_static (TYPE_SINGLE_PHOTO_PAGE, "SlideshowPage", &g_define_type_info, 0);
	}
	return slideshow_page_type_id;
}


#line 628 "CollectionPage.vala"
static void _collection_page_on_file_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1188 "CollectionPage.c"
	collection_page_on_file_menu (self);
}


#line 632 "CollectionPage.vala"
static void _collection_page_on_export_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1195 "CollectionPage.c"
	collection_page_on_export (self);
}


#line 695 "CollectionPage.vala"
static void _collection_page_on_edit_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1202 "CollectionPage.c"
	collection_page_on_edit_menu (self);
}


#line 700 "CollectionPage.vala"
static void _collection_page_on_select_all_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1209 "CollectionPage.c"
	collection_page_on_select_all (self);
}


#line 737 "CollectionPage.vala"
static void _collection_page_on_remove_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1216 "CollectionPage.c"
	collection_page_on_remove (self);
}


#line 714 "CollectionPage.vala"
static void _collection_page_on_photos_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1223 "CollectionPage.c"
	collection_page_on_photos_menu (self);
}


#line 727 "CollectionPage.vala"
static void _collection_page_on_increase_size_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1230 "CollectionPage.c"
	collection_page_on_increase_size (self);
}


#line 732 "CollectionPage.vala"
static void _collection_page_on_decrease_size_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1237 "CollectionPage.c"
	collection_page_on_decrease_size (self);
}


#line 787 "CollectionPage.vala"
static void _collection_page_on_rotate_clockwise_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1244 "CollectionPage.c"
	collection_page_on_rotate_clockwise (self);
}


#line 791 "CollectionPage.vala"
static void _collection_page_on_rotate_counterclockwise_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1251 "CollectionPage.c"
	collection_page_on_rotate_counterclockwise (self);
}


#line 795 "CollectionPage.vala"
static void _collection_page_on_mirror_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1258 "CollectionPage.c"
	collection_page_on_mirror (self);
}


#line 799 "CollectionPage.vala"
static void _collection_page_on_revert_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1265 "CollectionPage.c"
	collection_page_on_revert (self);
}


#line 813 "CollectionPage.vala"
static void _collection_page_on_slideshow_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1272 "CollectionPage.c"
	collection_page_on_slideshow (self);
}


#line 821 "CollectionPage.vala"
static void _collection_page_on_view_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1279 "CollectionPage.c"
	collection_page_on_view_menu (self);
}


#line 831 "CollectionPage.vala"
static void _collection_page_on_display_titles_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1286 "CollectionPage.c"
	collection_page_on_display_titles (self, action);
}


#line 914 "CollectionPage.vala"
static void _collection_page_on_sort_changed_gtk_radio_action_callback (GtkAction* action, GtkAction* current, gpointer self) {
#line 1293 "CollectionPage.c"
	collection_page_on_sort_changed (self);
}


#line 787 "CollectionPage.vala"
static void _collection_page_on_rotate_clockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 1300 "CollectionPage.c"
	collection_page_on_rotate_clockwise (self);
}


#line 813 "CollectionPage.vala"
static void _collection_page_on_slideshow_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 1307 "CollectionPage.c"
	collection_page_on_slideshow (self);
}


#line 856 "CollectionPage.vala"
static void _collection_page_on_slider_changed_gtk_range_value_changed (GtkHScale* _sender, gpointer self) {
#line 1314 "CollectionPage.c"
	collection_page_on_slider_changed (self);
}


#line 594 "CollectionPage.vala"
static void _collection_page_schedule_thumbnail_improval_gtk_adjustment_value_changed (GtkAdjustment* _sender, gpointer self) {
#line 1321 "CollectionPage.c"
	collection_page_schedule_thumbnail_improval (self);
}


#line 275 "CollectionPage.vala"
CollectionPage* collection_page_construct (GType object_type, const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1) {
#line 1328 "CollectionPage.c"
	CollectionPage * self;
	const char* _tmp0_;
	Comparator* _tmp1_;
	GtkToolButton* _tmp3_;
	GtkToolButton* _tmp4_;
	GtkSeparatorToolItem* separator;
	GtkHScale* _tmp5_;
	GtkToolItem* toolitem;
	_tmp0_ = NULL;
#line 277 "CollectionPage.vala"
	if (page_name != NULL) {
#line 277 "CollectionPage.vala"
		_tmp0_ = page_name;
#line 1342 "CollectionPage.c"
	} else {
#line 277 "CollectionPage.vala"
		_tmp0_ = "Photos";
#line 1346 "CollectionPage.c"
	}
#line 277 "CollectionPage.vala"
	self = (CollectionPage*) checkerboard_page_construct (object_type, _tmp0_);
#line 279 "CollectionPage.vala"
	page_init_ui_start (PAGE (self), "collection.ui", "CollectionActionGroup", COLLECTION_PAGE_ACTIONS, G_N_ELEMENTS (COLLECTION_PAGE_ACTIONS), COLLECTION_PAGE_TOGGLE_ACTIONS, G_N_ELEMENTS (COLLECTION_PAGE_TOGGLE_ACTIONS));
#line 280 "CollectionPage.vala"
	gtk_action_group_add_radio_actions (PAGE (self)->action_group, COLLECTION_PAGE_SORT_CRIT_ACTIONS, G_N_ELEMENTS (COLLECTION_PAGE_SORT_CRIT_ACTIONS), COLLECTION_PAGE_DEFAULT_SORT_BY, (GCallback) _collection_page_on_sort_changed_gtk_radio_action_callback, self);
#line 281 "CollectionPage.vala"
	gtk_action_group_add_radio_actions (PAGE (self)->action_group, COLLECTION_PAGE_SORT_ORDER_ACTIONS, G_N_ELEMENTS (COLLECTION_PAGE_SORT_ORDER_ACTIONS), COLLECTION_PAGE_DEFAULT_SORT_ORDER, (GCallback) _collection_page_on_sort_changed_gtk_radio_action_callback, self);
#line 283 "CollectionPage.vala"
	if (ui_filename != NULL) {
#line 284 "CollectionPage.vala"
		page_init_load_ui (PAGE (self), ui_filename);
#line 1360 "CollectionPage.c"
	}
#line 286 "CollectionPage.vala"
	if (child_actions != NULL) {
#line 287 "CollectionPage.vala"
		gtk_action_group_add_actions (PAGE (self)->action_group, child_actions, child_actions_length1, self);
#line 1366 "CollectionPage.c"
	}
#line 289 "CollectionPage.vala"
	page_init_ui_bind (PAGE (self), "/CollectionMenuBar");
#line 290 "CollectionPage.vala"
	checkerboard_page_init_context_menu (CHECKERBOARD_PAGE (self), "/CollectionContextMenu");
#line 919 "CollectionPage.vala"
	_tmp1_ = NULL;
#line 292 "CollectionPage.vala"
	checkerboard_page_set_layout_comparator (CHECKERBOARD_PAGE (self), _tmp1_ = collection_page_get_sort_comparator (self));
#line 1376 "CollectionPage.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (comparator_unref (_tmp1_), NULL));
#line 295 "CollectionPage.vala"
	if (collection_page_slider_adjustment == NULL) {
#line 1380 "CollectionPage.c"
		GtkAdjustment* _tmp2_;
		_tmp2_ = NULL;
#line 296 "CollectionPage.vala"
		collection_page_slider_adjustment = (_tmp2_ = g_object_ref_sink ((GtkAdjustment*) gtk_adjustment_new (collection_page_scale_to_slider (self->priv->scale), (double) 0, collection_page_scale_to_slider (THUMBNAIL_MAX_SCALE), (double) 1, (double) 10, (double) 0)), (collection_page_slider_adjustment == NULL) ? NULL : (collection_page_slider_adjustment = (g_object_unref (collection_page_slider_adjustment), NULL)), _tmp2_);
#line 1385 "CollectionPage.c"
	}
	_tmp3_ = NULL;
#line 302 "CollectionPage.vala"
	self->priv->rotate_button = (_tmp3_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (RESOURCES_CLOCKWISE)), (self->priv->rotate_button == NULL) ? NULL : (self->priv->rotate_button = (g_object_unref (self->priv->rotate_button), NULL)), _tmp3_);
#line 303 "CollectionPage.vala"
	gtk_tool_button_set_label (self->priv->rotate_button, RESOURCES_ROTATE_CLOCKWISE_LABEL);
#line 304 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->rotate_button), RESOURCES_ROTATE_CLOCKWISE_TOOLTIP);
#line 305 "CollectionPage.vala"
	g_object_set (GTK_WIDGET (self->priv->rotate_button), "sensitive", FALSE, NULL);
#line 306 "CollectionPage.vala"
	g_signal_connect_object (self->priv->rotate_button, "clicked", (GCallback) _collection_page_on_rotate_clockwise_gtk_tool_button_clicked, self, 0);
#line 308 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->rotate_button), -1);
#line 1400 "CollectionPage.c"
	_tmp4_ = NULL;
#line 311 "CollectionPage.vala"
	self->priv->slideshow_button = (_tmp4_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_PLAY)), (self->priv->slideshow_button == NULL) ? NULL : (self->priv->slideshow_button = (g_object_unref (self->priv->slideshow_button), NULL)), _tmp4_);
#line 312 "CollectionPage.vala"
	gtk_tool_button_set_label (self->priv->slideshow_button, "Slideshow");
#line 313 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->slideshow_button), "Start a slideshow of these photos");
#line 314 "CollectionPage.vala"
	g_object_set (GTK_WIDGET (self->priv->slideshow_button), "sensitive", FALSE, NULL);
#line 315 "CollectionPage.vala"
	g_signal_connect_object (self->priv->slideshow_button, "clicked", (GCallback) _collection_page_on_slideshow_gtk_tool_button_clicked, self, 0);
#line 317 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->slideshow_button), -1);
#line 1414 "CollectionPage.c"
	separator = g_object_ref_sink ((GtkSeparatorToolItem*) gtk_separator_tool_item_new ());
#line 321 "CollectionPage.vala"
	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separator), TRUE);
#line 322 "CollectionPage.vala"
	gtk_separator_tool_item_set_draw (separator, FALSE);
#line 324 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (separator), -1);
#line 1422 "CollectionPage.c"
	_tmp5_ = NULL;
#line 327 "CollectionPage.vala"
	self->priv->slider = (_tmp5_ = g_object_ref_sink ((GtkHScale*) gtk_hscale_new (collection_page_slider_adjustment)), (self->priv->slider == NULL) ? NULL : (self->priv->slider = (g_object_unref (self->priv->slider), NULL)), _tmp5_);
#line 328 "CollectionPage.vala"
	g_signal_connect_object (GTK_RANGE (self->priv->slider), "value-changed", (GCallback) _collection_page_on_slider_changed_gtk_range_value_changed, self, 0);
#line 329 "CollectionPage.vala"
	gtk_scale_set_draw_value (GTK_SCALE (self->priv->slider), FALSE);
#line 1430 "CollectionPage.c"
	toolitem = g_object_ref_sink (gtk_tool_item_new ());
#line 332 "CollectionPage.vala"
	gtk_container_add (GTK_CONTAINER (toolitem), GTK_WIDGET (self->priv->slider));
#line 333 "CollectionPage.vala"
	gtk_tool_item_set_expand (toolitem, FALSE);
#line 334 "CollectionPage.vala"
	gtk_widget_set_size_request (GTK_WIDGET (toolitem), 200, -1);
#line 335 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (toolitem, "Adjust the size of the thumbnails");
#line 337 "CollectionPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, toolitem, -1);
#line 340 "CollectionPage.vala"
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 344 "CollectionPage.vala"
	g_signal_connect_object (gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self)), "value-changed", (GCallback) _collection_page_schedule_thumbnail_improval_gtk_adjustment_value_changed, self, 0);
#line 345 "CollectionPage.vala"
	g_signal_connect_object (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)), "value-changed", (GCallback) _collection_page_schedule_thumbnail_improval_gtk_adjustment_value_changed, self, 0);
#line 348 "CollectionPage.vala"
	checkerboard_page_set_refresh_on_resize (CHECKERBOARD_PAGE (self), FALSE);
#line 350 "CollectionPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 352 "CollectionPage.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 354 "CollectionPage.vala"
	collection_page_schedule_thumbnail_improval (self);
#line 356 "CollectionPage.vala"
	page_enable_drag_source (PAGE (self), GDK_ACTION_COPY);
#line 1458 "CollectionPage.c"
	(separator == NULL) ? NULL : (separator = (g_object_unref (separator), NULL));
	(toolitem == NULL) ? NULL : (toolitem = (g_object_unref (toolitem), NULL));
	return self;
}


#line 275 "CollectionPage.vala"
CollectionPage* collection_page_new (const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1) {
#line 275 "CollectionPage.vala"
	return collection_page_construct (TYPE_COLLECTION_PAGE, page_name, ui_filename, child_actions, child_actions_length1);
#line 1469 "CollectionPage.c"
}


#line 359 "CollectionPage.vala"
static GtkToolbar* collection_page_real_get_toolbar (Page* base) {
#line 1475 "CollectionPage.c"
	CollectionPage * self;
	GtkToolbar* result;
	GtkToolbar* _tmp0_;
	self = COLLECTION_PAGE (base);
#line 360 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 1482 "CollectionPage.c"
	result = (_tmp0_ = self->priv->toolbar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 363 "CollectionPage.vala"
static void collection_page_real_switching_from (Page* base) {
#line 1490 "CollectionPage.c"
	CollectionPage * self;
	self = COLLECTION_PAGE (base);
#line 364 "CollectionPage.vala"
	PAGE_CLASS (collection_page_parent_class)->switching_from (PAGE (CHECKERBOARD_PAGE (self)));
#line 366 "CollectionPage.vala"
	checkerboard_page_set_refresh_on_resize (CHECKERBOARD_PAGE (self), FALSE);
#line 1497 "CollectionPage.c"
}


#line 369 "CollectionPage.vala"
static void collection_page_real_switched_to (Page* base) {
#line 1503 "CollectionPage.c"
	CollectionPage * self;
	self = COLLECTION_PAGE (base);
#line 370 "CollectionPage.vala"
	PAGE_CLASS (collection_page_parent_class)->switched_to (PAGE (CHECKERBOARD_PAGE (self)));
#line 372 "CollectionPage.vala"
	checkerboard_page_set_refresh_on_resize (CHECKERBOARD_PAGE (self), TRUE);
#line 1510 "CollectionPage.c"
	/* if the thumbnails were resized while viewing another page, resize the ones on this page
	 now ... set_thumb_size does the refresh and thumbnail improval, so don't schedule if
	 going this route*/
#line 377 "CollectionPage.vala"
	if (self->priv->thumbs_resized) {
#line 378 "CollectionPage.vala"
		collection_page_set_thumb_size (self, collection_page_slider_to_scale (gtk_range_get_value (GTK_RANGE (self->priv->slider))));
#line 379 "CollectionPage.vala"
		self->priv->thumbs_resized = FALSE;
#line 1520 "CollectionPage.c"
	} else {
		/* need to refresh the layout in case any of the thumbnail dimensions were altered while we
		 were gone*/
#line 383 "CollectionPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 1526 "CollectionPage.c"
		/* schedule improvement in case any new photos were added*/
#line 386 "CollectionPage.vala"
		collection_page_schedule_thumbnail_improval (self);
#line 1530 "CollectionPage.c"
	}
}


#line 390 "CollectionPage.vala"
static void collection_page_real_returning_from_fullscreen (Page* base) {
#line 1537 "CollectionPage.c"
	CollectionPage * self;
	self = COLLECTION_PAGE (base);
#line 391 "CollectionPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 393 "CollectionPage.vala"
	PAGE_CLASS (collection_page_parent_class)->returning_from_fullscreen (PAGE (CHECKERBOARD_PAGE (self)));
#line 1544 "CollectionPage.c"
}


#line 396 "CollectionPage.vala"
static void collection_page_real_on_selection_changed (CheckerboardPage* base, gint count) {
#line 1550 "CollectionPage.c"
	CollectionPage * self;
	self = COLLECTION_PAGE (base);
#line 397 "CollectionPage.vala"
	g_object_set (GTK_WIDGET (self->priv->rotate_button), "sensitive", count > 0, NULL);
#line 1555 "CollectionPage.c"
}


#line 400 "CollectionPage.vala"
static void collection_page_real_on_item_activated (CheckerboardPage* base, LayoutItem* item) {
#line 1561 "CollectionPage.c"
	CollectionPage * self;
	Thumbnail* _tmp0_;
	Thumbnail* thumbnail;
	char* _tmp2_;
	Photo* _tmp1_;
	AppWindow* _tmp3_;
	self = COLLECTION_PAGE (base);
#line 400 "CollectionPage.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 401 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 1573 "CollectionPage.c"
	thumbnail = (_tmp0_ = THUMBNAIL (item), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	/* switch to full-page view*/
#line 208 "Photo.vala"
	_tmp2_ = NULL;
#line 39 "Thumbnail.vala"
	_tmp1_ = NULL;
#line 404 "CollectionPage.vala"
	g_debug ("CollectionPage.vala:404: switching to %s", _tmp2_ = photo_to_string (_tmp1_ = thumbnail_get_photo (thumbnail)));
#line 1582 "CollectionPage.c"
	_tmp2_ = (g_free (_tmp2_), NULL);
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 446 "AppWindow.vala"
	_tmp3_ = NULL;
#line 406 "CollectionPage.vala"
	app_window_switch_to_photo_page (_tmp3_ = app_window_get_instance (), CHECKERBOARD_PAGE (self), thumbnail);
#line 1589 "CollectionPage.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
	(thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL));
}


#line 409 "CollectionPage.vala"
static GtkMenu* collection_page_real_get_context_menu (CheckerboardPage* base) {
#line 1597 "CollectionPage.c"
	CollectionPage * self;
	GtkMenu* result;
	GtkMenu* _tmp0_;
	self = COLLECTION_PAGE (base);
	_tmp0_ = NULL;
#line 411 "CollectionPage.vala"
	if (checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) != 0) {
#line 1605 "CollectionPage.c"
		GtkMenu* _tmp1_;
		_tmp1_ = NULL;
#line 411 "CollectionPage.vala"
		_tmp0_ = (_tmp1_ = CHECKERBOARD_PAGE_CLASS (collection_page_parent_class)->get_context_menu (CHECKERBOARD_PAGE (self)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 1610 "CollectionPage.c"
	} else {
		GtkMenu* _tmp2_;
		_tmp2_ = NULL;
#line 411 "CollectionPage.vala"
		_tmp0_ = (_tmp2_ = NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
#line 1616 "CollectionPage.c"
	}
	/* don't show a context menu if nothing is selected*/
	result = _tmp0_;
	return result;
}


#line 414 "CollectionPage.vala"
static gboolean collection_page_real_on_context_invoked (CheckerboardPage* base, GtkMenu* context_menu) {
#line 1626 "CollectionPage.c"
	CollectionPage * self;
	gboolean result;
	gboolean selected;
	gboolean revert_possible;
	gboolean _tmp0_;
	self = COLLECTION_PAGE (base);
#line 414 "CollectionPage.vala"
	g_return_val_if_fail (GTK_IS_MENU (context_menu), FALSE);
#line 1635 "CollectionPage.c"
	selected = checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0;
	revert_possible = collection_page_can_revert_selected (self);
#line 418 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionContextMenu/ContextRemove", selected);
#line 419 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionContextMenu/ContextRotateClockwise", selected);
#line 420 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionContextMenu/ContextRotateCounterclockwise", selected);
#line 421 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionContextMenu/ContextMirror", selected);
#line 1646 "CollectionPage.c"
	_tmp0_ = FALSE;
#line 422 "CollectionPage.vala"
	if (selected) {
#line 422 "CollectionPage.vala"
		_tmp0_ = revert_possible;
#line 1652 "CollectionPage.c"
	} else {
#line 422 "CollectionPage.vala"
		_tmp0_ = FALSE;
#line 1656 "CollectionPage.c"
	}
#line 422 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionContextMenu/ContextRevert", _tmp0_);
#line 1660 "CollectionPage.c"
	result = TRUE;
	return result;
}


#line 427 "CollectionPage.vala"
static LayoutItem* collection_page_real_get_fullscreen_photo (CheckerboardPage* base) {
#line 1668 "CollectionPage.c"
	CollectionPage * self;
	LayoutItem* result;
	GeeIterable* iter;
	self = COLLECTION_PAGE (base);
	iter = NULL;
	/* if no selection, use the first item*/
#line 431 "CollectionPage.vala"
	if (checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0) {
#line 1677 "CollectionPage.c"
		GeeIterable* _tmp0_;
		_tmp0_ = NULL;
#line 432 "CollectionPage.vala"
		iter = (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self)), (iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL)), _tmp0_);
#line 1682 "CollectionPage.c"
	} else {
		GeeIterable* _tmp1_;
		_tmp1_ = NULL;
#line 434 "CollectionPage.vala"
		iter = (_tmp1_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self)), (iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL)), _tmp1_);
#line 1688 "CollectionPage.c"
	}
	/* use the first item of the selected collection to start things off*/
	{
		GeeIterator* _item_it;
		/* use the first item of the selected collection to start things off*/
		_item_it = gee_iterable_iterator (iter);
		/* use the first item of the selected collection to start things off*/
#line 438 "CollectionPage.vala"
		while (TRUE) {
#line 1698 "CollectionPage.c"
			LayoutItem* item;
			/* use the first item of the selected collection to start things off*/
#line 438 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 1703 "CollectionPage.c"
				/* use the first item of the selected collection to start things off*/
#line 438 "CollectionPage.vala"
				break;
#line 1707 "CollectionPage.c"
			}
			/* use the first item of the selected collection to start things off*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
			result = item;
			(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
			(iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL));
			return result;
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = NULL;
	(iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL));
	return result;
}


#line 444 "CollectionPage.vala"
static void collection_page_real_on_resize (Page* base, const GdkRectangle* rect) {
#line 1726 "CollectionPage.c"
	CollectionPage * self;
	self = COLLECTION_PAGE (base);
	/* this schedules thumbnail improvement whenever the window size changes (and new thumbnails
	 may be exposed), therefore, uninterested in window position move*/
#line 447 "CollectionPage.vala"
	collection_page_schedule_thumbnail_improval (self);
#line 1733 "CollectionPage.c"
}


#line 450 "CollectionPage.vala"
static void collection_page_real_drag_begin (GtkWidget* base, GdkDragContext* context) {
#line 1739 "CollectionPage.c"
	CollectionPage * self;
	GError * _inner_error_;
	GdkPixbuf* icon;
	GtkWidget* _tmp6_;
	self = COLLECTION_PAGE (base);
#line 450 "CollectionPage.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 1747 "CollectionPage.c"
	_inner_error_ = NULL;
#line 451 "CollectionPage.vala"
	if (checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) == 0) {
#line 1751 "CollectionPage.c"
		return;
	}
#line 454 "CollectionPage.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->drag_items));
#line 1756 "CollectionPage.c"
	/* because drag_data_get may be called multiple times in a single drag, prepare all the exported
	 files first*/
	icon = NULL;
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 498 "Page.vala"
		_tmp0_ = NULL;
#line 1766 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 459 "CollectionPage.vala"
		while (TRUE) {
#line 1771 "CollectionPage.c"
			LayoutItem* item;
			Photo* photo;
			GFile* file;
			char* _tmp5_;
#line 459 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 459 "CollectionPage.vala"
				break;
#line 1780 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			photo = thumbnail_get_photo (THUMBNAIL (item));
			file = NULL;
			{
				GFile* _tmp2_;
				GFile* _tmp3_;
				_tmp2_ = photo_generate_exportable (photo, &_inner_error_);
				if (_inner_error_ != NULL) {
					goto __catch10_g_error;
					goto __finally10;
				}
				_tmp3_ = NULL;
#line 464 "CollectionPage.vala"
				file = (_tmp3_ = _tmp2_, (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp3_);
#line 1796 "CollectionPage.c"
			}
			goto __finally10;
			__catch10_g_error:
			{
				GError * err;
				err = _inner_error_;
				_inner_error_ = NULL;
				{
#line 466 "CollectionPage.vala"
					g_error ("CollectionPage.vala:466: %s", err->message);
#line 1807 "CollectionPage.c"
					(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
				}
			}
			__finally10:
			if (_inner_error_ != NULL) {
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
#line 469 "CollectionPage.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->drag_items), file);
#line 1824 "CollectionPage.c"
			/* set up icon using the "first" photo, although Sets are not ordered*/
#line 472 "CollectionPage.vala"
			if (icon == NULL) {
#line 1828 "CollectionPage.c"
				GdkPixbuf* _tmp4_;
				_tmp4_ = NULL;
#line 473 "CollectionPage.vala"
				icon = (_tmp4_ = photo_get_thumbnail (photo, THUMBNAIL_CACHE_MEDIUM_SCALE), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp4_);
#line 1833 "CollectionPage.c"
			}
#line 952 "gio-2.0.vapi"
			_tmp5_ = NULL;
#line 475 "CollectionPage.vala"
			g_debug ("CollectionPage.vala:475: Prepared %s for export", _tmp5_ = g_file_get_path (file));
#line 1839 "CollectionPage.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 478 "CollectionPage.vala"
	g_assert (icon != NULL);
#line 94 "Page.vala"
	_tmp6_ = NULL;
#line 479 "CollectionPage.vala"
	gtk_drag_source_set_icon_pixbuf (_tmp6_ = page_get_event_source (PAGE (self)), icon);
#line 1853 "CollectionPage.c"
	(_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL));
	(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
}


#line 482 "CollectionPage.vala"
static void collection_page_real_drag_data_get (GtkWidget* base, GdkDragContext* context, GtkSelectionData* selection_data, guint target_type, guint time) {
#line 1861 "CollectionPage.c"
	CollectionPage * self;
	char** _tmp1_;
	gint uris_size;
	gint uris_length1;
	gint _tmp0_;
	char** uris;
	gint ctr;
	self = COLLECTION_PAGE (base);
#line 482 "CollectionPage.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 482 "CollectionPage.vala"
	g_return_if_fail (selection_data != NULL);
#line 484 "CollectionPage.vala"
	g_assert (target_type == PAGE_TARGET_TYPE_URI_LIST);
#line 486 "CollectionPage.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->drag_items)) == 0) {
#line 1878 "CollectionPage.c"
		return;
	}
	/* prepare list of uris*/
#line 490 "CollectionPage.vala"
	_tmp1_ = NULL;
#line 1884 "CollectionPage.c"
	uris = (_tmp1_ = g_new0 (char*, (_tmp0_ = gee_collection_get_size (GEE_COLLECTION (self->priv->drag_items))) + 1), uris_length1 = _tmp0_, uris_size = uris_length1, _tmp1_);
	ctr = 0;
	{
		GeeIterator* _file_it;
		_file_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->drag_items));
#line 492 "CollectionPage.vala"
		while (TRUE) {
#line 1892 "CollectionPage.c"
			GFile* file;
			char* _tmp3_;
			char* *_tmp2_;
#line 492 "CollectionPage.vala"
			if (!gee_iterator_next (_file_it)) {
#line 492 "CollectionPage.vala"
				break;
#line 1900 "CollectionPage.c"
			}
			file = (GFile*) gee_iterator_get (_file_it);
			_tmp3_ = NULL;
#line 493 "CollectionPage.vala"
			_tmp2_ = &uris[ctr++];
#line 493 "CollectionPage.vala"
			(*_tmp2_) = (_tmp3_ = g_file_get_uri (file), (*_tmp2_) = (g_free ((*_tmp2_)), NULL), _tmp3_);
#line 1908 "CollectionPage.c"
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		}
		(_file_it == NULL) ? NULL : (_file_it = (g_object_unref (_file_it), NULL));
	}
#line 495 "CollectionPage.vala"
	gtk_selection_data_set_uris (selection_data, uris);
#line 1915 "CollectionPage.c"
	uris = (_vala_array_free (uris, uris_length1, (GDestroyNotify) g_free), NULL);
}


#line 498 "CollectionPage.vala"
static void collection_page_real_drag_end (GtkWidget* base, GdkDragContext* context) {
#line 1922 "CollectionPage.c"
	CollectionPage * self;
	self = COLLECTION_PAGE (base);
#line 498 "CollectionPage.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 499 "CollectionPage.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->drag_items));
#line 1929 "CollectionPage.c"
}


#line 502 "CollectionPage.vala"
static gboolean collection_page_real_source_drag_failed (Page* base, GdkDragContext* context, GtkDragResult drag_result) {
#line 1935 "CollectionPage.c"
	CollectionPage * self;
	gboolean result;
	self = COLLECTION_PAGE (base);
#line 502 "CollectionPage.vala"
	g_return_val_if_fail (GDK_IS_DRAG_CONTEXT (context), FALSE);
#line 503 "CollectionPage.vala"
	g_debug ("CollectionPage.vala:503: Drag failed: %d", (gint) drag_result);
#line 505 "CollectionPage.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->drag_items));
#line 1945 "CollectionPage.c"
	result = FALSE;
	return result;
}


#line 526 "CollectionPage.vala"
static void _collection_page_on_photo_removed_photo_removed (Photo* _sender, gpointer self) {
#line 1953 "CollectionPage.c"
	collection_page_on_photo_removed (self, _sender);
}


#line 534 "CollectionPage.vala"
static void _collection_page_on_thumbnail_altered_photo_thumbnail_altered (Photo* _sender, gpointer self) {
#line 1960 "CollectionPage.c"
	collection_page_on_thumbnail_altered (self, _sender);
}


#line 510 "CollectionPage.vala"
void collection_page_add_photo (CollectionPage* self, Photo* photo) {
#line 1967 "CollectionPage.c"
	Thumbnail* _tmp0_;
	gboolean _tmp1_;
	Thumbnail* thumbnail;
#line 510 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 510 "CollectionPage.vala"
	g_return_if_fail (IS_PHOTO (photo));
#line 1975 "CollectionPage.c"
	/* search for duplicates*/
#line 543 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 512 "CollectionPage.vala"
	if ((_tmp1_ = (_tmp0_ = collection_page_get_thumbnail_for_photo (self, photo)) != NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_)) {
#line 1981 "CollectionPage.c"
		return;
	}
#line 515 "CollectionPage.vala"
	g_signal_connect_object (photo, "removed", (GCallback) _collection_page_on_photo_removed_photo_removed, self, 0);
#line 516 "CollectionPage.vala"
	g_signal_connect_object (photo, "thumbnail-altered", (GCallback) _collection_page_on_thumbnail_altered_photo_thumbnail_altered, self, 0);
#line 1988 "CollectionPage.c"
	thumbnail = g_object_ref_sink (thumbnail_new (photo, self->priv->scale));
#line 519 "CollectionPage.vala"
	layout_item_display_title (LAYOUT_ITEM (thumbnail), collection_page_display_titles (self));
#line 521 "CollectionPage.vala"
	checkerboard_page_add_item (CHECKERBOARD_PAGE (self), LAYOUT_ITEM (thumbnail));
#line 523 "CollectionPage.vala"
	g_object_set (GTK_WIDGET (self->priv->slideshow_button), "sensitive", TRUE, NULL);
#line 1996 "CollectionPage.c"
	(thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL));
}


#line 526 "CollectionPage.vala"
static void collection_page_on_photo_removed (CollectionPage* self, Photo* photo) {
#line 2003 "CollectionPage.c"
	Thumbnail* found;
#line 526 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 526 "CollectionPage.vala"
	g_return_if_fail (IS_PHOTO (photo));
#line 2009 "CollectionPage.c"
	found = collection_page_get_thumbnail_for_photo (self, photo);
#line 528 "CollectionPage.vala"
	if (found != NULL) {
#line 529 "CollectionPage.vala"
		checkerboard_page_remove_item (CHECKERBOARD_PAGE (self), LAYOUT_ITEM (found));
#line 2015 "CollectionPage.c"
	}
#line 531 "CollectionPage.vala"
	g_object_set (GTK_WIDGET (self->priv->slideshow_button), "sensitive", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0, NULL);
#line 2019 "CollectionPage.c"
	(found == NULL) ? NULL : (found = (g_object_unref (found), NULL));
}


#line 534 "CollectionPage.vala"
static void collection_page_on_thumbnail_altered (CollectionPage* self, Photo* photo) {
#line 534 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 534 "CollectionPage.vala"
	g_return_if_fail (IS_PHOTO (photo));
#line 2030 "CollectionPage.c"
	/* the thumbnail is only going to reload a low-quality interp, so schedule improval*/
#line 536 "CollectionPage.vala"
	collection_page_schedule_thumbnail_improval (self);
#line 2034 "CollectionPage.c"
	/* since the geometry might have changed, refresh the layout*/
#line 539 "CollectionPage.vala"
	if (page_is_in_view (PAGE (self))) {
#line 540 "CollectionPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 2040 "CollectionPage.c"
	}
}


#line 543 "CollectionPage.vala"
static Thumbnail* collection_page_get_thumbnail_for_photo (CollectionPage* self, Photo* photo) {
#line 2047 "CollectionPage.c"
	Thumbnail* result;
#line 543 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), NULL);
#line 543 "CollectionPage.vala"
	g_return_val_if_fail (IS_PHOTO (photo), NULL);
#line 2053 "CollectionPage.c"
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 494 "Page.vala"
		_tmp0_ = NULL;
#line 2060 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 544 "CollectionPage.vala"
		while (TRUE) {
#line 2065 "CollectionPage.c"
			LayoutItem* item;
			Thumbnail* _tmp2_;
			Thumbnail* thumbnail;
			Photo* _tmp3_;
			gboolean _tmp4_;
#line 544 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 544 "CollectionPage.vala"
				break;
#line 2075 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 545 "CollectionPage.vala"
			_tmp2_ = NULL;
#line 2080 "CollectionPage.c"
			thumbnail = (_tmp2_ = THUMBNAIL (item), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
#line 39 "Thumbnail.vala"
			_tmp3_ = NULL;
#line 546 "CollectionPage.vala"
			if ((_tmp4_ = photo_equals (_tmp3_ = thumbnail_get_photo (thumbnail), photo), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL)), _tmp4_)) {
#line 2086 "CollectionPage.c"
				result = thumbnail;
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				return result;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = NULL;
	return result;
}


#line 553 "CollectionPage.vala"
gint collection_page_increase_thumb_size (CollectionPage* self) {
#line 2104 "CollectionPage.c"
	gint result;
#line 553 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), 0);
#line 554 "CollectionPage.vala"
	if (self->priv->scale == THUMBNAIL_MAX_SCALE) {
#line 2110 "CollectionPage.c"
		result = self->priv->scale;
		return result;
	}
#line 557 "CollectionPage.vala"
	self->priv->scale = self->priv->scale + COLLECTION_PAGE_MANUAL_STEPPING;
#line 558 "CollectionPage.vala"
	if (self->priv->scale > THUMBNAIL_MAX_SCALE) {
#line 559 "CollectionPage.vala"
		self->priv->scale = THUMBNAIL_MAX_SCALE;
#line 2120 "CollectionPage.c"
	}
#line 561 "CollectionPage.vala"
	collection_page_set_thumb_size (self, self->priv->scale);
#line 2124 "CollectionPage.c"
	result = self->priv->scale;
	return result;
}


#line 566 "CollectionPage.vala"
gint collection_page_decrease_thumb_size (CollectionPage* self) {
#line 2132 "CollectionPage.c"
	gint result;
#line 566 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), 0);
#line 567 "CollectionPage.vala"
	if (self->priv->scale == THUMBNAIL_MIN_SCALE) {
#line 2138 "CollectionPage.c"
		result = self->priv->scale;
		return result;
	}
#line 570 "CollectionPage.vala"
	self->priv->scale = self->priv->scale - COLLECTION_PAGE_MANUAL_STEPPING;
#line 571 "CollectionPage.vala"
	if (self->priv->scale < THUMBNAIL_MIN_SCALE) {
#line 572 "CollectionPage.vala"
		self->priv->scale = THUMBNAIL_MIN_SCALE;
#line 2148 "CollectionPage.c"
	}
#line 574 "CollectionPage.vala"
	collection_page_set_thumb_size (self, self->priv->scale);
#line 2152 "CollectionPage.c"
	result = self->priv->scale;
	return result;
}


#line 579 "CollectionPage.vala"
void collection_page_set_thumb_size (CollectionPage* self, gint new_scale) {
#line 579 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 580 "CollectionPage.vala"
	g_assert (new_scale >= THUMBNAIL_MIN_SCALE);
#line 581 "CollectionPage.vala"
	g_assert (new_scale <= THUMBNAIL_MAX_SCALE);
#line 583 "CollectionPage.vala"
	self->priv->scale = new_scale;
#line 2168 "CollectionPage.c"
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 494 "Page.vala"
		_tmp0_ = NULL;
#line 2175 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 585 "CollectionPage.vala"
		while (TRUE) {
#line 2180 "CollectionPage.c"
			LayoutItem* item;
#line 585 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 585 "CollectionPage.vala"
				break;
#line 2186 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 586 "CollectionPage.vala"
			thumbnail_resize (THUMBNAIL (item), self->priv->scale);
#line 2191 "CollectionPage.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 588 "CollectionPage.vala"
	if (page_is_in_view (PAGE (self))) {
#line 589 "CollectionPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 590 "CollectionPage.vala"
		collection_page_schedule_thumbnail_improval (self);
#line 2202 "CollectionPage.c"
	}
}


#line 607 "CollectionPage.vala"
static gboolean _collection_page_improve_thumbnail_quality_gsource_func (gpointer self) {
#line 2209 "CollectionPage.c"
	return collection_page_improve_thumbnail_quality (self);
}


#line 594 "CollectionPage.vala"
static void collection_page_schedule_thumbnail_improval (CollectionPage* self) {
#line 594 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 2218 "CollectionPage.c"
	/* don't bother if not in view*/
#line 596 "CollectionPage.vala"
	if (!page_is_in_view (PAGE (self))) {
#line 2222 "CollectionPage.c"
		return;
	}
#line 599 "CollectionPage.vala"
	if (self->priv->improval_scheduled == FALSE) {
#line 600 "CollectionPage.vala"
		self->priv->improval_scheduled = TRUE;
#line 601 "CollectionPage.vala"
		g_timeout_add_full (COLLECTION_PAGE_IMPROVAL_PRIORITY, (guint) COLLECTION_PAGE_IMPROVAL_DELAY_MS, _collection_page_improve_thumbnail_quality_gsource_func, g_object_ref (self), g_object_unref);
#line 2231 "CollectionPage.c"
	} else {
#line 603 "CollectionPage.vala"
		self->priv->reschedule_improval = TRUE;
#line 2235 "CollectionPage.c"
	}
}


#line 607 "CollectionPage.vala"
static gboolean collection_page_improve_thumbnail_quality (CollectionPage* self) {
#line 2242 "CollectionPage.c"
	gboolean result;
#line 607 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), FALSE);
#line 608 "CollectionPage.vala"
	if (self->priv->reschedule_improval) {
#line 609 "CollectionPage.vala"
		g_debug ("CollectionPage.vala:609: rescheduled improval");
#line 610 "CollectionPage.vala"
		self->priv->reschedule_improval = FALSE;
#line 2252 "CollectionPage.c"
		result = TRUE;
		return result;
	}
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 494 "Page.vala"
		_tmp0_ = NULL;
#line 2262 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 615 "CollectionPage.vala"
		while (TRUE) {
#line 2267 "CollectionPage.c"
			LayoutItem* item;
			Thumbnail* _tmp2_;
			Thumbnail* thumbnail;
#line 615 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 615 "CollectionPage.vala"
				break;
#line 2275 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 616 "CollectionPage.vala"
			_tmp2_ = NULL;
#line 2280 "CollectionPage.c"
			thumbnail = (_tmp2_ = THUMBNAIL (item), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
#line 617 "CollectionPage.vala"
			if (thumbnail_is_exposed (thumbnail)) {
#line 618 "CollectionPage.vala"
				thumbnail_paint_high_quality (thumbnail);
#line 2286 "CollectionPage.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 621 "CollectionPage.vala"
	self->priv->improval_scheduled = FALSE;
#line 623 "CollectionPage.vala"
	g_debug ("CollectionPage.vala:623: improve_thumbnail_quality");
#line 2297 "CollectionPage.c"
	result = FALSE;
	return result;
}


#line 628 "CollectionPage.vala"
static void collection_page_on_file_menu (CollectionPage* self) {
#line 628 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 629 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/FileMenu/Export", checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0);
#line 2309 "CollectionPage.c"
}


#line 632 "CollectionPage.vala"
static void collection_page_on_export (CollectionPage* self) {
#line 2315 "CollectionPage.c"
	GError * _inner_error_;
	GeeArrayList* export_list;
	ExportDialog* export_dialog;
	gint scale;
	ScaleConstraint constraint;
	JpegQuality quality;
	GFile* export_dir;
	AppWindow* _tmp8_;
	AppWindow* _tmp14_;
#line 632 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 2327 "CollectionPage.c"
	_inner_error_ = NULL;
	export_list = gee_array_list_new (TYPE_PHOTO, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 498 "Page.vala"
		_tmp0_ = NULL;
#line 2336 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 634 "CollectionPage.vala"
		while (TRUE) {
#line 2341 "CollectionPage.c"
			LayoutItem* item;
			Photo* _tmp2_;
#line 634 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 634 "CollectionPage.vala"
				break;
#line 2348 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 39 "Thumbnail.vala"
			_tmp2_ = NULL;
#line 635 "CollectionPage.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (export_list), _tmp2_ = thumbnail_get_photo (THUMBNAIL (item)));
#line 2355 "CollectionPage.c"
			(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 637 "CollectionPage.vala"
	if (gee_collection_get_size (GEE_COLLECTION (export_list)) == 0) {
#line 2363 "CollectionPage.c"
		(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
		return;
	}
	export_dialog = g_object_ref_sink (export_dialog_new (gee_collection_get_size (GEE_COLLECTION (export_list))));
	scale = 0;
	constraint = 0;
	quality = 0;
#line 645 "CollectionPage.vala"
	if (!export_dialog_execute (export_dialog, &scale, &constraint, &quality)) {
#line 2373 "CollectionPage.c"
		(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
		(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
		return;
	}
	/* handle the single-photo case*/
#line 649 "CollectionPage.vala"
	if (gee_collection_get_size (GEE_COLLECTION (export_list)) == 1) {
#line 2381 "CollectionPage.c"
		Photo* photo;
		GFile* _tmp3_;
		GFile* _tmp4_;
		GFile* save_as;
		photo = (Photo*) gee_abstract_list_get (GEE_ABSTRACT_LIST (export_list), 0);
#line 167 "Photo.vala"
		_tmp3_ = NULL;
#line 652 "CollectionPage.vala"
		_tmp4_ = NULL;
#line 2391 "CollectionPage.c"
		save_as = (_tmp4_ = export_ui_choose_file (_tmp3_ = photo_get_file (photo)), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL)), _tmp4_);
#line 653 "CollectionPage.vala"
		if (save_as == NULL) {
#line 2395 "CollectionPage.c"
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
			(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
			(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
			(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
			return;
		}
#line 656 "CollectionPage.vala"
		spin_event_loop ();
#line 2404 "CollectionPage.c"
		{
#line 659 "CollectionPage.vala"
			photo_export (photo, save_as, scale, constraint, quality, &_inner_error_);
#line 2408 "CollectionPage.c"
			if (_inner_error_ != NULL) {
				goto __catch11_g_error;
				goto __finally11;
			}
		}
		goto __finally11;
		__catch11_g_error:
		{
			GError * err;
			err = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp7_;
				char* _tmp6_;
				GFile* _tmp5_;
#line 820 "glib-2.0.vapi"
				_tmp7_ = NULL;
#line 952 "gio-2.0.vapi"
				_tmp6_ = NULL;
#line 167 "Photo.vala"
				_tmp5_ = NULL;
#line 661 "CollectionPage.vala"
				app_window_error_message (_tmp7_ = g_strdup_printf ("Unable to export photo %s: %s", _tmp6_ = g_file_get_path (_tmp5_ = photo_get_file (photo)), err->message));
#line 2432 "CollectionPage.c"
				_tmp7_ = (g_free (_tmp7_), NULL);
				_tmp6_ = (g_free (_tmp6_), NULL);
				(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally11:
		if (_inner_error_ != NULL) {
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
			(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
			(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
			(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
		(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
		(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
		(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
		return;
	}
	/* multiple photos*/
	export_dir = export_ui_choose_dir ();
#line 670 "CollectionPage.vala"
	if (export_dir == NULL) {
#line 2459 "CollectionPage.c"
		(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
		(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
		(export_dir == NULL) ? NULL : (export_dir = (g_object_unref (export_dir), NULL));
		return;
	}
#line 446 "AppWindow.vala"
	_tmp8_ = NULL;
#line 673 "CollectionPage.vala"
	app_window_set_busy_cursor (_tmp8_ = app_window_get_instance ());
#line 2469 "CollectionPage.c"
	(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
	{
		GeeIterator* _photo_it;
		_photo_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (export_list));
#line 675 "CollectionPage.vala"
		while (TRUE) {
#line 2476 "CollectionPage.c"
			Photo* photo;
			char* _tmp10_;
			GFile* _tmp9_;
			GFile* _tmp11_;
			GFile* save_as;
#line 675 "CollectionPage.vala"
			if (!gee_iterator_next (_photo_it)) {
#line 675 "CollectionPage.vala"
				break;
#line 2486 "CollectionPage.c"
			}
			photo = (Photo*) gee_iterator_get (_photo_it);
#line 947 "gio-2.0.vapi"
			_tmp10_ = NULL;
#line 167 "Photo.vala"
			_tmp9_ = NULL;
#line 676 "CollectionPage.vala"
			_tmp11_ = NULL;
#line 2495 "CollectionPage.c"
			save_as = (_tmp11_ = g_file_get_child (export_dir, _tmp10_ = g_file_get_basename (_tmp9_ = photo_get_file (photo))), _tmp10_ = (g_free (_tmp10_), NULL), (_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL)), _tmp11_);
#line 677 "CollectionPage.vala"
			if (g_file_query_exists (save_as, NULL)) {
#line 678 "CollectionPage.vala"
				if (!export_ui_query_overwrite (save_as)) {
#line 2501 "CollectionPage.c"
					(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
					(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
#line 679 "CollectionPage.vala"
					continue;
#line 2506 "CollectionPage.c"
				}
			}
#line 682 "CollectionPage.vala"
			spin_event_loop ();
#line 2511 "CollectionPage.c"
			{
#line 685 "CollectionPage.vala"
				photo_export (photo, save_as, scale, constraint, quality, &_inner_error_);
#line 2515 "CollectionPage.c"
				if (_inner_error_ != NULL) {
					goto __catch12_g_error;
					goto __finally12;
				}
			}
			goto __finally12;
			__catch12_g_error:
			{
				GError * err;
				err = _inner_error_;
				_inner_error_ = NULL;
				{
					char* _tmp13_;
					char* _tmp12_;
#line 820 "glib-2.0.vapi"
					_tmp13_ = NULL;
#line 952 "gio-2.0.vapi"
					_tmp12_ = NULL;
#line 687 "CollectionPage.vala"
					app_window_error_message (_tmp13_ = g_strdup_printf ("Unable to export photo %s: %s", _tmp12_ = g_file_get_path (save_as), err->message));
#line 2536 "CollectionPage.c"
					_tmp13_ = (g_free (_tmp13_), NULL);
					_tmp12_ = (g_free (_tmp12_), NULL);
					(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
				}
			}
			__finally12:
			if (_inner_error_ != NULL) {
				(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
				(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
				(_photo_it == NULL) ? NULL : (_photo_it = (g_object_unref (_photo_it), NULL));
				(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
				(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
				(export_dir == NULL) ? NULL : (export_dir = (g_object_unref (export_dir), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
			(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
		}
		(_photo_it == NULL) ? NULL : (_photo_it = (g_object_unref (_photo_it), NULL));
	}
#line 446 "AppWindow.vala"
	_tmp14_ = NULL;
#line 692 "CollectionPage.vala"
	app_window_set_normal_cursor (_tmp14_ = app_window_get_instance ());
#line 2563 "CollectionPage.c"
	(_tmp14_ == NULL) ? NULL : (_tmp14_ = (g_object_unref (_tmp14_), NULL));
	(export_list == NULL) ? NULL : (export_list = (g_object_unref (export_list), NULL));
	(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
	(export_dir == NULL) ? NULL : (export_dir = (g_object_unref (export_dir), NULL));
}


#line 695 "CollectionPage.vala"
static void collection_page_on_edit_menu (CollectionPage* self) {
#line 695 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 696 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/EditMenu/SelectAll", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0);
#line 697 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/EditMenu/Remove", checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0);
#line 2579 "CollectionPage.c"
}


#line 700 "CollectionPage.vala"
static void collection_page_on_select_all (CollectionPage* self) {
#line 700 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 701 "CollectionPage.vala"
	checkerboard_page_select_all (CHECKERBOARD_PAGE (self));
#line 2589 "CollectionPage.c"
}


#line 704 "CollectionPage.vala"
static gboolean collection_page_can_revert_selected (CollectionPage* self) {
#line 2595 "CollectionPage.c"
	gboolean result;
#line 704 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), FALSE);
#line 2599 "CollectionPage.c"
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 498 "Page.vala"
		_tmp0_ = NULL;
#line 2606 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 705 "CollectionPage.vala"
		while (TRUE) {
#line 2611 "CollectionPage.c"
			LayoutItem* item;
			Photo* photo;
#line 705 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 705 "CollectionPage.vala"
				break;
#line 2618 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			photo = thumbnail_get_photo (THUMBNAIL (item));
#line 707 "CollectionPage.vala"
			if (photo_has_transformations (photo)) {
#line 2624 "CollectionPage.c"
				result = TRUE;
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				return result;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = FALSE;
	return result;
}


#line 714 "CollectionPage.vala"
static void collection_page_real_on_photos_menu (CollectionPage* self) {
#line 2643 "CollectionPage.c"
	gboolean selected;
	gboolean revert_possible;
	gboolean _tmp0_;
#line 714 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 2649 "CollectionPage.c"
	selected = checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0;
	revert_possible = collection_page_can_revert_selected (self);
#line 718 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/IncreaseSize", self->priv->scale < THUMBNAIL_MAX_SCALE);
#line 719 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/DecreaseSize", self->priv->scale > THUMBNAIL_MIN_SCALE);
#line 720 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/RotateClockwise", selected);
#line 721 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/RotateCounterclockwise", selected);
#line 722 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/Mirror", selected);
#line 2662 "CollectionPage.c"
	_tmp0_ = FALSE;
#line 723 "CollectionPage.vala"
	if (selected) {
#line 723 "CollectionPage.vala"
		_tmp0_ = revert_possible;
#line 2668 "CollectionPage.c"
	} else {
#line 723 "CollectionPage.vala"
		_tmp0_ = FALSE;
#line 2672 "CollectionPage.c"
	}
#line 723 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/Revert", _tmp0_);
#line 724 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/Slideshow", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0);
#line 2678 "CollectionPage.c"
}


#line 714 "CollectionPage.vala"
void collection_page_on_photos_menu (CollectionPage* self) {
#line 714 "CollectionPage.vala"
	COLLECTION_PAGE_GET_CLASS (self)->on_photos_menu (self);
#line 2686 "CollectionPage.c"
}


#line 727 "CollectionPage.vala"
static void collection_page_on_increase_size (CollectionPage* self) {
#line 727 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 728 "CollectionPage.vala"
	collection_page_increase_thumb_size (self);
#line 729 "CollectionPage.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->slider), collection_page_scale_to_slider (self->priv->scale));
#line 2698 "CollectionPage.c"
}


#line 732 "CollectionPage.vala"
static void collection_page_on_decrease_size (CollectionPage* self) {
#line 732 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 733 "CollectionPage.vala"
	collection_page_decrease_thumb_size (self);
#line 734 "CollectionPage.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->slider), collection_page_scale_to_slider (self->priv->scale));
#line 2710 "CollectionPage.c"
}


#line 737 "CollectionPage.vala"
static void collection_page_on_remove (CollectionPage* self) {
#line 2716 "CollectionPage.c"
	AppWindow* _tmp0_;
	GtkMessageDialog* _tmp1_;
	GtkMessageDialog* dialog;
	GtkResponseType _result_;
	gboolean _tmp2_;
#line 737 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 738 "CollectionPage.vala"
	if (checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) == 0) {
#line 2726 "CollectionPage.c"
		return;
	}
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 741 "CollectionPage.vala"
	_tmp1_ = NULL;
#line 2733 "CollectionPage.c"
	dialog = (_tmp1_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (GTK_WINDOW (_tmp0_ = app_window_get_instance ()), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_CANCEL, "If you remove these photos from your library you will lose all edits you've made to " "them.  Shotwell can also delete the files from your drive.\n\nThis action cannot be undone.", NULL)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 745 "CollectionPage.vala"
	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_DELETE, (gint) GTK_RESPONSE_NO);
#line 746 "CollectionPage.vala"
	gtk_dialog_add_button (GTK_DIALOG (dialog), "Keep files", (gint) GTK_RESPONSE_YES);
#line 747 "CollectionPage.vala"
	gtk_window_set_title (GTK_WINDOW (dialog), "Remove photos?");
#line 2741 "CollectionPage.c"
	_result_ = (GtkResponseType) gtk_dialog_run (GTK_DIALOG (dialog));
#line 751 "CollectionPage.vala"
	gtk_object_destroy (GTK_OBJECT (dialog));
#line 2745 "CollectionPage.c"
	_tmp2_ = FALSE;
#line 753 "CollectionPage.vala"
	if (_result_ != GTK_RESPONSE_YES) {
#line 753 "CollectionPage.vala"
		_tmp2_ = _result_ != GTK_RESPONSE_NO;
#line 2751 "CollectionPage.c"
	} else {
#line 753 "CollectionPage.vala"
		_tmp2_ = FALSE;
#line 2755 "CollectionPage.c"
	}
#line 753 "CollectionPage.vala"
	if (_tmp2_) {
#line 2759 "CollectionPage.c"
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		return;
	}
	/* iterate over selected photos and remove them from entire system .. this will result
	 in on_photo_removed being called, which we don't want in this case is because it will
	 remove from the list while iterating, so disconnect the signals and do the work here*/
	{
		GeeIterable* _tmp3_;
		GeeIterator* _tmp4_;
		GeeIterator* _item_it;
		/* iterate over selected photos and remove them from entire system .. this will result
		 in on_photo_removed being called, which we don't want in this case is because it will
		 remove from the list while iterating, so disconnect the signals and do the work here*/
#line 498 "Page.vala"
		_tmp3_ = NULL;
#line 2775 "CollectionPage.c"
		_tmp4_ = NULL;
		_item_it = (_tmp4_ = gee_iterable_iterator (_tmp3_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self))), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL)), _tmp4_);
		/* iterate over selected photos and remove them from entire system .. this will result
		 in on_photo_removed being called, which we don't want in this case is because it will
		 remove from the list while iterating, so disconnect the signals and do the work here*/
#line 759 "CollectionPage.vala"
		while (TRUE) {
#line 2783 "CollectionPage.c"
			LayoutItem* item;
			Photo* photo;
			guint _tmp5_;
			guint _tmp6_;
			/* iterate over selected photos and remove them from entire system .. this will result
			 in on_photo_removed being called, which we don't want in this case is because it will
			 remove from the list while iterating, so disconnect the signals and do the work here*/
#line 759 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 2793 "CollectionPage.c"
				/* iterate over selected photos and remove them from entire system .. this will result
				 in on_photo_removed being called, which we don't want in this case is because it will
				 remove from the list while iterating, so disconnect the signals and do the work here*/
#line 759 "CollectionPage.vala"
				break;
#line 2799 "CollectionPage.c"
			}
			/* iterate over selected photos and remove them from entire system .. this will result
			 in on_photo_removed being called, which we don't want in this case is because it will
			 remove from the list while iterating, so disconnect the signals and do the work here*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
			photo = thumbnail_get_photo (THUMBNAIL (item));
#line 761 "CollectionPage.vala"
			g_signal_handlers_disconnect_matched (photo, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("removed", TYPE_PHOTO, &_tmp5_, NULL, FALSE), _tmp5_), 0, NULL, (GCallback) _collection_page_on_photo_removed_photo_removed, self);
#line 762 "CollectionPage.vala"
			g_signal_handlers_disconnect_matched (photo, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("thumbnail-altered", TYPE_PHOTO, &_tmp6_, NULL, FALSE), _tmp6_), 0, NULL, (GCallback) _collection_page_on_thumbnail_altered_photo_thumbnail_altered, self);
#line 764 "CollectionPage.vala"
			photo_remove (photo, _result_ == GTK_RESPONSE_NO);
#line 2812 "CollectionPage.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* now remove from page, outside of iterator*/
#line 768 "CollectionPage.vala"
	checkerboard_page_remove_selected (CHECKERBOARD_PAGE (self));
#line 770 "CollectionPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 2823 "CollectionPage.c"
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


#line 773 "CollectionPage.vala"
static void collection_page_do_rotations (CollectionPage* self, GeeIterable* c, Rotation rotation) {
#line 2830 "CollectionPage.c"
	gboolean rotation_performed;
#line 773 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 773 "CollectionPage.vala"
	g_return_if_fail (GEE_IS_ITERABLE (c));
#line 2836 "CollectionPage.c"
	rotation_performed = FALSE;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (c);
#line 775 "CollectionPage.vala"
		while (TRUE) {
#line 2843 "CollectionPage.c"
			LayoutItem* item;
			Photo* photo;
#line 775 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 775 "CollectionPage.vala"
				break;
#line 2850 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			photo = thumbnail_get_photo (THUMBNAIL (item));
#line 777 "CollectionPage.vala"
			photo_rotate (photo, rotation);
#line 779 "CollectionPage.vala"
			rotation_performed = TRUE;
#line 2858 "CollectionPage.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* geometry could've changed*/
#line 783 "CollectionPage.vala"
	if (rotation_performed) {
#line 784 "CollectionPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 2869 "CollectionPage.c"
	}
}


#line 787 "CollectionPage.vala"
static void collection_page_on_rotate_clockwise (CollectionPage* self) {
#line 2876 "CollectionPage.c"
	GeeIterable* _tmp0_;
#line 787 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 498 "Page.vala"
	_tmp0_ = NULL;
#line 788 "CollectionPage.vala"
	collection_page_do_rotations (self, _tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self)), ROTATION_CLOCKWISE);
#line 2884 "CollectionPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 791 "CollectionPage.vala"
static void collection_page_on_rotate_counterclockwise (CollectionPage* self) {
#line 2891 "CollectionPage.c"
	GeeIterable* _tmp0_;
#line 791 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 498 "Page.vala"
	_tmp0_ = NULL;
#line 792 "CollectionPage.vala"
	collection_page_do_rotations (self, _tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self)), ROTATION_COUNTERCLOCKWISE);
#line 2899 "CollectionPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 795 "CollectionPage.vala"
static void collection_page_on_mirror (CollectionPage* self) {
#line 2906 "CollectionPage.c"
	GeeIterable* _tmp0_;
#line 795 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 498 "Page.vala"
	_tmp0_ = NULL;
#line 796 "CollectionPage.vala"
	collection_page_do_rotations (self, _tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self)), ROTATION_MIRROR);
#line 2914 "CollectionPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 799 "CollectionPage.vala"
static void collection_page_on_revert (CollectionPage* self) {
#line 2921 "CollectionPage.c"
	gboolean revert_performed;
#line 799 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 2925 "CollectionPage.c"
	revert_performed = FALSE;
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 498 "Page.vala"
		_tmp0_ = NULL;
#line 2933 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 801 "CollectionPage.vala"
		while (TRUE) {
#line 2938 "CollectionPage.c"
			LayoutItem* item;
			Photo* photo;
#line 801 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 801 "CollectionPage.vala"
				break;
#line 2945 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
			photo = thumbnail_get_photo (THUMBNAIL (item));
#line 803 "CollectionPage.vala"
			photo_remove_all_transformations (photo);
#line 805 "CollectionPage.vala"
			revert_performed = TRUE;
#line 2953 "CollectionPage.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* geometry could change*/
#line 809 "CollectionPage.vala"
	if (revert_performed) {
#line 810 "CollectionPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 2964 "CollectionPage.c"
	}
}


#line 813 "CollectionPage.vala"
static void collection_page_on_slideshow (CollectionPage* self) {
#line 2971 "CollectionPage.c"
	FullscreenWindow* _tmp3_;
	SlideshowPage* _tmp2_;
	Thumbnail* _tmp1_;
	AppWindow* _tmp0_;
#line 813 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 814 "CollectionPage.vala"
	if (checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) == 0) {
#line 2980 "CollectionPage.c"
		return;
	}
	_tmp3_ = NULL;
	_tmp2_ = NULL;
#line 818 "CollectionPage.vala"
	_tmp1_ = NULL;
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 817 "CollectionPage.vala"
	app_window_go_fullscreen (_tmp0_ = app_window_get_instance (), _tmp3_ = g_object_ref_sink (fullscreen_window_new (PAGE (_tmp2_ = g_object_ref_sink (slideshow_page_new (CHECKERBOARD_PAGE (self), _tmp1_ = THUMBNAIL (checkerboard_page_get_first_item (CHECKERBOARD_PAGE (self)))))))));
#line 2991 "CollectionPage.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 821 "CollectionPage.vala"
static void collection_page_on_view_menu (CollectionPage* self) {
#line 821 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 822 "CollectionPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/ViewMenu/Fullscreen", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0);
#line 3005 "CollectionPage.c"
}


#line 825 "CollectionPage.vala"
static gboolean collection_page_display_titles (CollectionPage* self) {
#line 3011 "CollectionPage.c"
	gboolean result;
	GtkToggleAction* _tmp0_;
	GtkToggleAction* action;
#line 825 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), FALSE);
#line 826 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 3019 "CollectionPage.c"
	action = (_tmp0_ = GTK_TOGGLE_ACTION (gtk_ui_manager_get_action (PAGE (self)->ui, "/CollectionMenuBar/ViewMenu/ViewTitle")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	result = gtk_toggle_action_get_active (action);
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
	return result;
}


#line 831 "CollectionPage.vala"
static void collection_page_on_display_titles (CollectionPage* self, GtkAction* action) {
#line 3029 "CollectionPage.c"
	gboolean display;
#line 831 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 831 "CollectionPage.vala"
	g_return_if_fail (GTK_IS_ACTION (action));
#line 3035 "CollectionPage.c"
	display = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
#line 494 "Page.vala"
		_tmp0_ = NULL;
#line 3043 "CollectionPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 834 "CollectionPage.vala"
		while (TRUE) {
#line 3048 "CollectionPage.c"
			LayoutItem* item;
#line 834 "CollectionPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 834 "CollectionPage.vala"
				break;
#line 3054 "CollectionPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 835 "CollectionPage.vala"
			layout_item_display_title (item, display);
#line 3059 "CollectionPage.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 837 "CollectionPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 3066 "CollectionPage.c"
}


#line 840 "CollectionPage.vala"
static double collection_page_scale_to_slider (gint value) {
#line 3072 "CollectionPage.c"
	double result;
#line 841 "CollectionPage.vala"
	g_assert (value >= THUMBNAIL_MIN_SCALE);
#line 842 "CollectionPage.vala"
	g_assert (value <= THUMBNAIL_MAX_SCALE);
#line 3078 "CollectionPage.c"
	result = (double) ((value - THUMBNAIL_MIN_SCALE) / COLLECTION_PAGE_SLIDER_STEPPING);
	return result;
}


#line 847 "CollectionPage.vala"
static gint collection_page_slider_to_scale (double value) {
#line 3086 "CollectionPage.c"
	gint result;
	gint res;
	res = ((gint) (value * COLLECTION_PAGE_SLIDER_STEPPING)) + THUMBNAIL_MIN_SCALE;
#line 850 "CollectionPage.vala"
	g_assert (res >= THUMBNAIL_MIN_SCALE);
#line 851 "CollectionPage.vala"
	g_assert (res <= THUMBNAIL_MAX_SCALE);
#line 3094 "CollectionPage.c"
	result = res;
	return result;
}


#line 856 "CollectionPage.vala"
static void collection_page_on_slider_changed (CollectionPage* self) {
#line 856 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 857 "CollectionPage.vala"
	if (!page_is_in_view (PAGE (self))) {
#line 858 "CollectionPage.vala"
		self->priv->thumbs_resized = TRUE;
#line 3108 "CollectionPage.c"
		return;
	}
#line 863 "CollectionPage.vala"
	collection_page_set_thumb_size (self, collection_page_slider_to_scale (gtk_range_get_value (GTK_RANGE (self->priv->slider))));
#line 3113 "CollectionPage.c"
}


#line 791 "CollectionPage.vala"
static void _collection_page_on_rotate_counterclockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 3119 "CollectionPage.c"
	collection_page_on_rotate_counterclockwise (self);
}


#line 866 "CollectionPage.vala"
static gboolean collection_page_real_on_ctrl_pressed (Page* base, const GdkEventKey* event) {
#line 3126 "CollectionPage.c"
	CollectionPage * self;
	gboolean result;
	guint _tmp0_;
	self = COLLECTION_PAGE (base);
#line 867 "CollectionPage.vala"
	gtk_tool_button_set_stock_id (self->priv->rotate_button, RESOURCES_COUNTERCLOCKWISE);
#line 868 "CollectionPage.vala"
	gtk_tool_button_set_label (self->priv->rotate_button, RESOURCES_ROTATE_COUNTERCLOCKWISE_LABEL);
#line 869 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->rotate_button), RESOURCES_ROTATE_COUNTERCLOCKWISE_TOOLTIP);
#line 870 "CollectionPage.vala"
	g_signal_handlers_disconnect_matched (self->priv->rotate_button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_TOOL_BUTTON, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _collection_page_on_rotate_clockwise_gtk_tool_button_clicked, self);
#line 871 "CollectionPage.vala"
	g_signal_connect_object (self->priv->rotate_button, "clicked", (GCallback) _collection_page_on_rotate_counterclockwise_gtk_tool_button_clicked, self, 0);
#line 3141 "CollectionPage.c"
	result = FALSE;
	return result;
}


#line 876 "CollectionPage.vala"
static gboolean collection_page_real_on_ctrl_released (Page* base, const GdkEventKey* event) {
#line 3149 "CollectionPage.c"
	CollectionPage * self;
	gboolean result;
	guint _tmp0_;
	self = COLLECTION_PAGE (base);
#line 877 "CollectionPage.vala"
	gtk_tool_button_set_stock_id (self->priv->rotate_button, RESOURCES_CLOCKWISE);
#line 878 "CollectionPage.vala"
	gtk_tool_button_set_label (self->priv->rotate_button, RESOURCES_ROTATE_CLOCKWISE_LABEL);
#line 879 "CollectionPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->rotate_button), RESOURCES_ROTATE_CLOCKWISE_TOOLTIP);
#line 880 "CollectionPage.vala"
	g_signal_handlers_disconnect_matched (self->priv->rotate_button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_TOOL_BUTTON, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _collection_page_on_rotate_counterclockwise_gtk_tool_button_clicked, self);
#line 881 "CollectionPage.vala"
	g_signal_connect_object (self->priv->rotate_button, "clicked", (GCallback) _collection_page_on_rotate_clockwise_gtk_tool_button_clicked, self, 0);
#line 3164 "CollectionPage.c"
	result = FALSE;
	return result;
}


#line 886 "CollectionPage.vala"
static gint collection_page_get_sort_criteria (CollectionPage* self) {
#line 3172 "CollectionPage.c"
	gint result;
	GtkRadioAction* _tmp0_;
	GtkRadioAction* action;
	gint value;
#line 886 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), 0);
#line 3179 "CollectionPage.c"
	/* any member of the group knows the current value*/
#line 888 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 3183 "CollectionPage.c"
	action = (_tmp0_ = GTK_RADIO_ACTION (gtk_ui_manager_get_action (PAGE (self)->ui, "/CollectionMenuBar/ViewMenu/SortPhotos/SortByName")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 889 "CollectionPage.vala"
	g_assert (action != NULL);
#line 3187 "CollectionPage.c"
	value = gtk_radio_action_get_current_value (action);
#line 892 "CollectionPage.vala"
	g_assert (value >= COLLECTION_PAGE_SORT_BY_MIN);
#line 893 "CollectionPage.vala"
	g_assert (value <= COLLECTION_PAGE_SORT_BY_MAX);
#line 3193 "CollectionPage.c"
	result = value;
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
	return result;
}


#line 898 "CollectionPage.vala"
static gint collection_page_get_sort_order (CollectionPage* self) {
#line 3202 "CollectionPage.c"
	gint result;
	GtkRadioAction* _tmp0_;
	GtkRadioAction* action;
	gint value;
#line 898 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), 0);
#line 3209 "CollectionPage.c"
	/* any member of the group knows the current value*/
#line 900 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 3213 "CollectionPage.c"
	action = (_tmp0_ = GTK_RADIO_ACTION (gtk_ui_manager_get_action (PAGE (self)->ui, "/CollectionMenuBar/ViewMenu/SortPhotos/SortAscending")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 901 "CollectionPage.vala"
	g_assert (action != NULL);
#line 3217 "CollectionPage.c"
	value = gtk_radio_action_get_current_value (action);
#line 904 "CollectionPage.vala"
	g_assert (value >= COLLECTION_PAGE_SORT_ORDER_MIN);
#line 905 "CollectionPage.vala"
	g_assert (value <= COLLECTION_PAGE_SORT_ORDER_MAX);
#line 3223 "CollectionPage.c"
	result = value;
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
	return result;
}


#line 910 "CollectionPage.vala"
static gboolean collection_page_is_sort_ascending (CollectionPage* self) {
#line 3232 "CollectionPage.c"
	gboolean result;
#line 910 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), FALSE);
#line 3236 "CollectionPage.c"
	result = collection_page_get_sort_order (self) == COLLECTION_PAGE_SORT_ORDER_ASCENDING;
	return result;
}


#line 914 "CollectionPage.vala"
static void collection_page_on_sort_changed (CollectionPage* self) {
#line 3244 "CollectionPage.c"
	Comparator* _tmp0_;
#line 914 "CollectionPage.vala"
	g_return_if_fail (IS_COLLECTION_PAGE (self));
#line 919 "CollectionPage.vala"
	_tmp0_ = NULL;
#line 915 "CollectionPage.vala"
	checkerboard_page_set_layout_comparator (CHECKERBOARD_PAGE (self), _tmp0_ = collection_page_get_sort_comparator (self));
#line 3252 "CollectionPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (comparator_unref (_tmp0_), NULL));
#line 916 "CollectionPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 3256 "CollectionPage.c"
}


#line 919 "CollectionPage.vala"
static Comparator* collection_page_get_sort_comparator (CollectionPage* self) {
#line 3262 "CollectionPage.c"
	Comparator* result;
#line 919 "CollectionPage.vala"
	g_return_val_if_fail (IS_COLLECTION_PAGE (self), NULL);
#line 920 "CollectionPage.vala"
	switch (collection_page_get_sort_criteria (self)) {
#line 3268 "CollectionPage.c"
		case COLLECTION_PAGE_SORT_BY_NAME:
		{
#line 922 "CollectionPage.vala"
			if (collection_page_is_sort_ascending (self)) {
#line 3273 "CollectionPage.c"
				result = COMPARATOR (collection_page_compare_name_new ());
				return result;
			} else {
				result = COMPARATOR (collection_page_reverse_compare_name_new ());
				return result;
			}
		}
		case COLLECTION_PAGE_SORT_BY_EXPOSURE_DATE:
		{
#line 928 "CollectionPage.vala"
			if (collection_page_is_sort_ascending (self)) {
#line 3285 "CollectionPage.c"
				result = COMPARATOR (collection_page_compare_date_new ());
				return result;
			} else {
				result = COMPARATOR (collection_page_reverse_compare_date_new ());
				return result;
			}
		}
		default:
		{
#line 934 "CollectionPage.vala"
			g_error ("CollectionPage.vala:934: Unknown sort criteria: %d", collection_page_get_sort_criteria (self));
#line 3297 "CollectionPage.c"
			result = COMPARATOR (collection_page_compare_name_new ());
			return result;
		}
	}
}


#line 190 "CollectionPage.vala"
static gint64 collection_page_compare_name_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b) {
#line 3307 "CollectionPage.c"
	CollectionPageCompareName * self;
	gint64 result;
	char* namea;
	char* nameb;
	self = COLLECTION_PAGE_COMPARE_NAME (base);
#line 190 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (a), 0LL);
#line 190 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (b), 0LL);
#line 3317 "CollectionPage.c"
	namea = layout_item_get_title (LAYOUT_ITEM (THUMBNAIL (a)));
	nameb = layout_item_get_title (LAYOUT_ITEM (THUMBNAIL (b)));
	result = (gint64) strcmp (namea, nameb);
	namea = (g_free (namea), NULL);
	nameb = (g_free (nameb), NULL);
	return result;
}


#line 189 "CollectionPage.vala"
static CollectionPageCompareName* collection_page_compare_name_construct (GType object_type) {
#line 3329 "CollectionPage.c"
	CollectionPageCompareName* self;
	self = (CollectionPageCompareName*) g_type_create_instance (object_type);
	return self;
}


#line 189 "CollectionPage.vala"
static CollectionPageCompareName* collection_page_compare_name_new (void) {
#line 189 "CollectionPage.vala"
	return collection_page_compare_name_construct (COLLECTION_PAGE_TYPE_COMPARE_NAME);
#line 3340 "CollectionPage.c"
}


static void collection_page_compare_name_class_init (CollectionPageCompareNameClass * klass) {
	collection_page_compare_name_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->compare = collection_page_compare_name_real_compare;
}


static void collection_page_compare_name_instance_init (CollectionPageCompareName * self) {
}


static GType collection_page_compare_name_get_type (void) {
	static GType collection_page_compare_name_type_id = 0;
	if (collection_page_compare_name_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CollectionPageCompareNameClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) collection_page_compare_name_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CollectionPageCompareName), 0, (GInstanceInitFunc) collection_page_compare_name_instance_init, NULL };
		collection_page_compare_name_type_id = g_type_register_static (TYPE_COMPARATOR, "CollectionPageCompareName", &g_define_type_info, 0);
	}
	return collection_page_compare_name_type_id;
}


#line 199 "CollectionPage.vala"
static gint64 collection_page_reverse_compare_name_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b) {
#line 3366 "CollectionPage.c"
	CollectionPageReverseCompareName * self;
	gint64 result;
	char* namea;
	char* nameb;
	self = COLLECTION_PAGE_REVERSE_COMPARE_NAME (base);
#line 199 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (a), 0LL);
#line 199 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (b), 0LL);
#line 3376 "CollectionPage.c"
	namea = layout_item_get_title (LAYOUT_ITEM (THUMBNAIL (a)));
	nameb = layout_item_get_title (LAYOUT_ITEM (THUMBNAIL (b)));
	result = (gint64) strcmp (nameb, namea);
	namea = (g_free (namea), NULL);
	nameb = (g_free (nameb), NULL);
	return result;
}


#line 198 "CollectionPage.vala"
static CollectionPageReverseCompareName* collection_page_reverse_compare_name_construct (GType object_type) {
#line 3388 "CollectionPage.c"
	CollectionPageReverseCompareName* self;
	self = (CollectionPageReverseCompareName*) g_type_create_instance (object_type);
	return self;
}


#line 198 "CollectionPage.vala"
static CollectionPageReverseCompareName* collection_page_reverse_compare_name_new (void) {
#line 198 "CollectionPage.vala"
	return collection_page_reverse_compare_name_construct (COLLECTION_PAGE_TYPE_REVERSE_COMPARE_NAME);
#line 3399 "CollectionPage.c"
}


static void collection_page_reverse_compare_name_class_init (CollectionPageReverseCompareNameClass * klass) {
	collection_page_reverse_compare_name_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->compare = collection_page_reverse_compare_name_real_compare;
}


static void collection_page_reverse_compare_name_instance_init (CollectionPageReverseCompareName * self) {
}


static GType collection_page_reverse_compare_name_get_type (void) {
	static GType collection_page_reverse_compare_name_type_id = 0;
	if (collection_page_reverse_compare_name_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CollectionPageReverseCompareNameClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) collection_page_reverse_compare_name_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CollectionPageReverseCompareName), 0, (GInstanceInitFunc) collection_page_reverse_compare_name_instance_init, NULL };
		collection_page_reverse_compare_name_type_id = g_type_register_static (TYPE_COMPARATOR, "CollectionPageReverseCompareName", &g_define_type_info, 0);
	}
	return collection_page_reverse_compare_name_type_id;
}


#line 208 "CollectionPage.vala"
static gint64 collection_page_compare_date_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b) {
#line 3425 "CollectionPage.c"
	CollectionPageCompareDate * self;
	gint64 result;
	Photo* _tmp0_;
	time_t _tmp1_ = {0};
	time_t timea;
	Photo* _tmp2_;
	time_t _tmp3_ = {0};
	time_t timeb;
	self = COLLECTION_PAGE_COMPARE_DATE (base);
#line 208 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (a), 0LL);
#line 208 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (b), 0LL);
#line 39 "Thumbnail.vala"
	_tmp0_ = NULL;
#line 3441 "CollectionPage.c"
	timea = (_tmp1_ = photo_get_exposure_time (_tmp0_ = thumbnail_get_photo (THUMBNAIL (a))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 39 "Thumbnail.vala"
	_tmp2_ = NULL;
#line 3445 "CollectionPage.c"
	timeb = (_tmp3_ = photo_get_exposure_time (_tmp2_ = thumbnail_get_photo (THUMBNAIL (b))), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL)), _tmp3_);
	result = (gint64) (timea - timeb);
	return result;
}


#line 207 "CollectionPage.vala"
static CollectionPageCompareDate* collection_page_compare_date_construct (GType object_type) {
#line 3454 "CollectionPage.c"
	CollectionPageCompareDate* self;
	self = (CollectionPageCompareDate*) g_type_create_instance (object_type);
	return self;
}


#line 207 "CollectionPage.vala"
static CollectionPageCompareDate* collection_page_compare_date_new (void) {
#line 207 "CollectionPage.vala"
	return collection_page_compare_date_construct (COLLECTION_PAGE_TYPE_COMPARE_DATE);
#line 3465 "CollectionPage.c"
}


static void collection_page_compare_date_class_init (CollectionPageCompareDateClass * klass) {
	collection_page_compare_date_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->compare = collection_page_compare_date_real_compare;
}


static void collection_page_compare_date_instance_init (CollectionPageCompareDate * self) {
}


static GType collection_page_compare_date_get_type (void) {
	static GType collection_page_compare_date_type_id = 0;
	if (collection_page_compare_date_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CollectionPageCompareDateClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) collection_page_compare_date_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CollectionPageCompareDate), 0, (GInstanceInitFunc) collection_page_compare_date_instance_init, NULL };
		collection_page_compare_date_type_id = g_type_register_static (TYPE_COMPARATOR, "CollectionPageCompareDate", &g_define_type_info, 0);
	}
	return collection_page_compare_date_type_id;
}


#line 217 "CollectionPage.vala"
static gint64 collection_page_reverse_compare_date_real_compare (Comparator* base, LayoutItem* a, LayoutItem* b) {
#line 3491 "CollectionPage.c"
	CollectionPageReverseCompareDate * self;
	gint64 result;
	Photo* _tmp0_;
	time_t _tmp1_ = {0};
	time_t timea;
	Photo* _tmp2_;
	time_t _tmp3_ = {0};
	time_t timeb;
	self = COLLECTION_PAGE_REVERSE_COMPARE_DATE (base);
#line 217 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (a), 0LL);
#line 217 "CollectionPage.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (b), 0LL);
#line 39 "Thumbnail.vala"
	_tmp0_ = NULL;
#line 3507 "CollectionPage.c"
	timea = (_tmp1_ = photo_get_exposure_time (_tmp0_ = thumbnail_get_photo (THUMBNAIL (a))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 39 "Thumbnail.vala"
	_tmp2_ = NULL;
#line 3511 "CollectionPage.c"
	timeb = (_tmp3_ = photo_get_exposure_time (_tmp2_ = thumbnail_get_photo (THUMBNAIL (b))), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL)), _tmp3_);
	result = (gint64) (timeb - timea);
	return result;
}


#line 216 "CollectionPage.vala"
static CollectionPageReverseCompareDate* collection_page_reverse_compare_date_construct (GType object_type) {
#line 3520 "CollectionPage.c"
	CollectionPageReverseCompareDate* self;
	self = (CollectionPageReverseCompareDate*) g_type_create_instance (object_type);
	return self;
}


#line 216 "CollectionPage.vala"
static CollectionPageReverseCompareDate* collection_page_reverse_compare_date_new (void) {
#line 216 "CollectionPage.vala"
	return collection_page_reverse_compare_date_construct (COLLECTION_PAGE_TYPE_REVERSE_COMPARE_DATE);
#line 3531 "CollectionPage.c"
}


static void collection_page_reverse_compare_date_class_init (CollectionPageReverseCompareDateClass * klass) {
	collection_page_reverse_compare_date_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->compare = collection_page_reverse_compare_date_real_compare;
}


static void collection_page_reverse_compare_date_instance_init (CollectionPageReverseCompareDate * self) {
}


static GType collection_page_reverse_compare_date_get_type (void) {
	static GType collection_page_reverse_compare_date_type_id = 0;
	if (collection_page_reverse_compare_date_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CollectionPageReverseCompareDateClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) collection_page_reverse_compare_date_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CollectionPageReverseCompareDate), 0, (GInstanceInitFunc) collection_page_reverse_compare_date_instance_init, NULL };
		collection_page_reverse_compare_date_type_id = g_type_register_static (TYPE_COMPARATOR, "CollectionPageReverseCompareDate", &g_define_type_info, 0);
	}
	return collection_page_reverse_compare_date_type_id;
}


static void collection_page_class_init (CollectionPageClass * klass) {
	collection_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (CollectionPagePrivate));
	PAGE_CLASS (klass)->get_toolbar = collection_page_real_get_toolbar;
	PAGE_CLASS (klass)->switching_from = collection_page_real_switching_from;
	PAGE_CLASS (klass)->switched_to = collection_page_real_switched_to;
	PAGE_CLASS (klass)->returning_from_fullscreen = collection_page_real_returning_from_fullscreen;
	CHECKERBOARD_PAGE_CLASS (klass)->on_selection_changed = collection_page_real_on_selection_changed;
	CHECKERBOARD_PAGE_CLASS (klass)->on_item_activated = collection_page_real_on_item_activated;
	CHECKERBOARD_PAGE_CLASS (klass)->get_context_menu = collection_page_real_get_context_menu;
	CHECKERBOARD_PAGE_CLASS (klass)->on_context_invoked = collection_page_real_on_context_invoked;
	CHECKERBOARD_PAGE_CLASS (klass)->get_fullscreen_photo = collection_page_real_get_fullscreen_photo;
	PAGE_CLASS (klass)->on_resize = collection_page_real_on_resize;
	GTK_WIDGET_CLASS (klass)->drag_begin = collection_page_real_drag_begin;
	GTK_WIDGET_CLASS (klass)->drag_data_get = collection_page_real_drag_data_get;
	GTK_WIDGET_CLASS (klass)->drag_end = collection_page_real_drag_end;
	PAGE_CLASS (klass)->source_drag_failed = collection_page_real_source_drag_failed;
	COLLECTION_PAGE_CLASS (klass)->on_photos_menu = collection_page_real_on_photos_menu;
	PAGE_CLASS (klass)->on_ctrl_pressed = collection_page_real_on_ctrl_pressed;
	PAGE_CLASS (klass)->on_ctrl_released = collection_page_real_on_ctrl_released;
	G_OBJECT_CLASS (klass)->finalize = collection_page_finalize;
}


static void collection_page_instance_init (CollectionPage * self) {
	self->priv = COLLECTION_PAGE_GET_PRIVATE (self);
	self->priv->toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
	self->priv->slider = NULL;
	self->priv->rotate_button = NULL;
	self->priv->slideshow_button = NULL;
	self->priv->scale = THUMBNAIL_DEFAULT_SCALE;
	self->priv->improval_scheduled = FALSE;
	self->priv->reschedule_improval = FALSE;
	self->priv->drag_items = gee_array_list_new (G_TYPE_FILE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->thumbs_resized = FALSE;
}


static void collection_page_finalize (GObject* obj) {
	CollectionPage * self;
	self = COLLECTION_PAGE (obj);
	(self->priv->toolbar == NULL) ? NULL : (self->priv->toolbar = (g_object_unref (self->priv->toolbar), NULL));
	(self->priv->slider == NULL) ? NULL : (self->priv->slider = (g_object_unref (self->priv->slider), NULL));
	(self->priv->rotate_button == NULL) ? NULL : (self->priv->rotate_button = (g_object_unref (self->priv->rotate_button), NULL));
	(self->priv->slideshow_button == NULL) ? NULL : (self->priv->slideshow_button = (g_object_unref (self->priv->slideshow_button), NULL));
	(self->priv->drag_items == NULL) ? NULL : (self->priv->drag_items = (g_object_unref (self->priv->drag_items), NULL));
	G_OBJECT_CLASS (collection_page_parent_class)->finalize (obj);
}


GType collection_page_get_type (void) {
	static GType collection_page_type_id = 0;
	if (collection_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CollectionPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) collection_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CollectionPage), 0, (GInstanceInitFunc) collection_page_instance_init, NULL };
		collection_page_type_id = g_type_register_static (TYPE_CHECKERBOARD_PAGE, "CollectionPage", &g_define_type_info, 0);
	}
	return collection_page_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




