/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <float.h>
#include <math.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gobject/gvaluecollector.h>


#define TYPE_ANALYTIC_PIXEL (analytic_pixel_get_type ())
typedef struct _AnalyticPixel AnalyticPixel;

#define TYPE_COLOR_TRANSFORMATION_KIND (color_transformation_kind_get_type ())

#define TYPE_COLOR_TRANSFORMATION_INSTANCE (color_transformation_instance_get_type ())
typedef struct _ColorTransformationInstance ColorTransformationInstance;

#define TYPE_COLOR_TRANSFORMATION (color_transformation_get_type ())
#define COLOR_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLOR_TRANSFORMATION, ColorTransformation))
#define COLOR_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLOR_TRANSFORMATION, ColorTransformationClass))
#define IS_COLOR_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLOR_TRANSFORMATION))
#define IS_COLOR_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_TRANSFORMATION))
#define COLOR_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLOR_TRANSFORMATION, ColorTransformationClass))

typedef struct _ColorTransformation ColorTransformation;
typedef struct _ColorTransformationClass ColorTransformationClass;
typedef struct _ColorTransformationPrivate ColorTransformationPrivate;
typedef struct _ParamSpecColorTransformation ParamSpecColorTransformation;

#define TYPE_EXPOSURE_TRANSFORMATION (exposure_transformation_get_type ())
#define EXPOSURE_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPOSURE_TRANSFORMATION, ExposureTransformation))
#define EXPOSURE_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPOSURE_TRANSFORMATION, ExposureTransformationClass))
#define IS_EXPOSURE_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPOSURE_TRANSFORMATION))
#define IS_EXPOSURE_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPOSURE_TRANSFORMATION))
#define EXPOSURE_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPOSURE_TRANSFORMATION, ExposureTransformationClass))

typedef struct _ExposureTransformation ExposureTransformation;
typedef struct _ExposureTransformationClass ExposureTransformationClass;
typedef struct _ExposureTransformationPrivate ExposureTransformationPrivate;

#define TYPE_SATURATION_TRANSFORMATION (saturation_transformation_get_type ())
#define SATURATION_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SATURATION_TRANSFORMATION, SaturationTransformation))
#define SATURATION_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SATURATION_TRANSFORMATION, SaturationTransformationClass))
#define IS_SATURATION_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SATURATION_TRANSFORMATION))
#define IS_SATURATION_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SATURATION_TRANSFORMATION))
#define SATURATION_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SATURATION_TRANSFORMATION, SaturationTransformationClass))

typedef struct _SaturationTransformation SaturationTransformation;
typedef struct _SaturationTransformationClass SaturationTransformationClass;
typedef struct _SaturationTransformationPrivate SaturationTransformationPrivate;

#define SATURATION_TRANSFORMATION_TYPE_WEIGHT_KIND (saturation_transformation_weight_kind_get_type ())

#define TYPE_TEMPERATURE_TRANSFORMATION (temperature_transformation_get_type ())
#define TEMPERATURE_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TEMPERATURE_TRANSFORMATION, TemperatureTransformation))
#define TEMPERATURE_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TEMPERATURE_TRANSFORMATION, TemperatureTransformationClass))
#define IS_TEMPERATURE_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TEMPERATURE_TRANSFORMATION))
#define IS_TEMPERATURE_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TEMPERATURE_TRANSFORMATION))
#define TEMPERATURE_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TEMPERATURE_TRANSFORMATION, TemperatureTransformationClass))

typedef struct _TemperatureTransformation TemperatureTransformation;
typedef struct _TemperatureTransformationClass TemperatureTransformationClass;
typedef struct _TemperatureTransformationPrivate TemperatureTransformationPrivate;

#define TYPE_TINT_TRANSFORMATION (tint_transformation_get_type ())
#define TINT_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TINT_TRANSFORMATION, TintTransformation))
#define TINT_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TINT_TRANSFORMATION, TintTransformationClass))
#define IS_TINT_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TINT_TRANSFORMATION))
#define IS_TINT_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TINT_TRANSFORMATION))
#define TINT_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TINT_TRANSFORMATION, TintTransformationClass))

typedef struct _TintTransformation TintTransformation;
typedef struct _TintTransformationClass TintTransformationClass;
typedef struct _TintTransformationPrivate TintTransformationPrivate;

#define TYPE_COLOR_TRANSFORMATION_FACTORY (color_transformation_factory_get_type ())
#define COLOR_TRANSFORMATION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactory))
#define COLOR_TRANSFORMATION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactoryClass))
#define IS_COLOR_TRANSFORMATION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY))
#define IS_COLOR_TRANSFORMATION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_TRANSFORMATION_FACTORY))
#define COLOR_TRANSFORMATION_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactoryClass))

typedef struct _ColorTransformationFactory ColorTransformationFactory;
typedef struct _ColorTransformationFactoryClass ColorTransformationFactoryClass;
typedef struct _ColorTransformationFactoryPrivate ColorTransformationFactoryPrivate;
typedef struct _ParamSpecColorTransformationFactory ParamSpecColorTransformationFactory;

#define TYPE_IMAGE_HISTOGRAM (image_histogram_get_type ())
#define IMAGE_HISTOGRAM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMAGE_HISTOGRAM, ImageHistogram))
#define IMAGE_HISTOGRAM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMAGE_HISTOGRAM, ImageHistogramClass))
#define IS_IMAGE_HISTOGRAM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMAGE_HISTOGRAM))
#define IS_IMAGE_HISTOGRAM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMAGE_HISTOGRAM))
#define IMAGE_HISTOGRAM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMAGE_HISTOGRAM, ImageHistogramClass))

typedef struct _ImageHistogram ImageHistogram;
typedef struct _ImageHistogramClass ImageHistogramClass;
typedef struct _ImageHistogramPrivate ImageHistogramPrivate;
typedef struct _ParamSpecImageHistogram ParamSpecImageHistogram;

struct _AnalyticPixel {
	float red;
	float green;
	float blue;
};

typedef enum  {
	COLOR_TRANSFORMATION_KIND_EXPOSURE,
	COLOR_TRANSFORMATION_KIND_SATURATION,
	COLOR_TRANSFORMATION_KIND_TINT,
	COLOR_TRANSFORMATION_KIND_TEMPERATURE
} ColorTransformationKind;

struct _ColorTransformationInstance {
	ColorTransformationKind kind;
	float parameter;
};

struct _ColorTransformation {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ColorTransformationPrivate * priv;
	float* matrix_entries;
	gint matrix_entries_length1;
	gboolean identity;
};

struct _ColorTransformationClass {
	GTypeClass parent_class;
	void (*finalize) (ColorTransformation *self);
};

struct _ParamSpecColorTransformation {
	GParamSpec parent_instance;
};

struct _ExposureTransformation {
	ColorTransformation parent_instance;
	ExposureTransformationPrivate * priv;
};

struct _ExposureTransformationClass {
	ColorTransformationClass parent_class;
};

struct _SaturationTransformation {
	ColorTransformation parent_instance;
	SaturationTransformationPrivate * priv;
};

struct _SaturationTransformationClass {
	ColorTransformationClass parent_class;
};

typedef enum  {
	SATURATION_TRANSFORMATION_WEIGHT_KIND_NTSC,
	SATURATION_TRANSFORMATION_WEIGHT_KIND_LINEAR,
	SATURATION_TRANSFORMATION_WEIGHT_KIND_FLAT
} SaturationTransformationWeightKind;

struct _SaturationTransformationPrivate {
	SaturationTransformationWeightKind weight_kind;
};

struct _TemperatureTransformation {
	ColorTransformation parent_instance;
	TemperatureTransformationPrivate * priv;
};

struct _TemperatureTransformationClass {
	ColorTransformationClass parent_class;
};

struct _TintTransformation {
	ColorTransformation parent_instance;
	TintTransformationPrivate * priv;
};

struct _TintTransformationClass {
	ColorTransformationClass parent_class;
};

struct _ColorTransformationFactory {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ColorTransformationFactoryPrivate * priv;
};

struct _ColorTransformationFactoryClass {
	GTypeClass parent_class;
	void (*finalize) (ColorTransformationFactory *self);
};

struct _ParamSpecColorTransformationFactory {
	GParamSpec parent_instance;
};

struct _ImageHistogram {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ImageHistogramPrivate * priv;
};

struct _ImageHistogramClass {
	GTypeClass parent_class;
	void (*finalize) (ImageHistogram *self);
};

struct _ImageHistogramPrivate {
	gint* red_counts;
	gint red_counts_length1;
	gint red_counts_size;
	gint* green_counts;
	gint green_counts_length1;
	gint green_counts_size;
	gint* blue_counts;
	gint blue_counts_length1;
	gint blue_counts_size;
	gint* qualitative_red_counts;
	gint qualitative_red_counts_length1;
	gint qualitative_red_counts_size;
	gint* qualitative_green_counts;
	gint qualitative_green_counts_length1;
	gint qualitative_green_counts_size;
	gint* qualitative_blue_counts;
	gint qualitative_blue_counts_length1;
	gint qualitative_blue_counts_size;
	GdkPixbuf* graphic;
};

struct _ParamSpecImageHistogram {
	GParamSpec parent_instance;
};


static gpointer color_transformation_parent_class = NULL;
static gpointer exposure_transformation_parent_class = NULL;
static gpointer saturation_transformation_parent_class = NULL;
static gpointer temperature_transformation_parent_class = NULL;
static gpointer tint_transformation_parent_class = NULL;
static ColorTransformationFactory* color_transformation_factory_instance;
static ColorTransformationFactory* color_transformation_factory_instance = NULL;
static gpointer color_transformation_factory_parent_class = NULL;
static gpointer image_histogram_parent_class = NULL;

GType analytic_pixel_get_type (void);
AnalyticPixel* analytic_pixel_dup (const AnalyticPixel* self);
void analytic_pixel_free (AnalyticPixel* self);
void analytic_pixel_init (AnalyticPixel *self);
void analytic_pixel_init_from_components (AnalyticPixel *self, float red, float green, float blue);
void analytic_pixel_init_from_quantized_components (AnalyticPixel *self, guchar red_quantized, guchar green_quantized, guchar blue_quantized);
guchar analytic_pixel_quantized_red (AnalyticPixel *self);
guchar analytic_pixel_quantized_green (AnalyticPixel *self);
guchar analytic_pixel_quantized_blue (AnalyticPixel *self);
AnalyticPixel analytic_pixel_get_pixbuf_pixel (GdkPixbuf* pixbuf, gint x, gint y);
void analytic_pixel_set_pixbuf_pixel (GdkPixbuf* pixbuf, const AnalyticPixel* pixel, gint x, gint y);
GType color_transformation_kind_get_type (void);
GType color_transformation_instance_get_type (void);
ColorTransformationInstance* color_transformation_instance_dup (const ColorTransformationInstance* self);
void color_transformation_instance_free (ColorTransformationInstance* self);
gpointer color_transformation_ref (gpointer instance);
void color_transformation_unref (gpointer instance);
GParamSpec* param_spec_color_transformation (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_color_transformation (GValue* value, gpointer v_object);
gpointer value_get_color_transformation (const GValue* value);
GType color_transformation_get_type (void);
enum  {
	COLOR_TRANSFORMATION_DUMMY_PROPERTY
};
ColorTransformation* color_transformation_new (void);
ColorTransformation* color_transformation_construct (GType object_type);
gboolean color_transformation_is_identity (ColorTransformation* self);
AnalyticPixel color_transformation_transform_pixel (ColorTransformation* self, const AnalyticPixel* pixel);
ColorTransformation* color_transformation_compose_against (ColorTransformation* self, ColorTransformation* transform);
void color_transformation_transform_pixbuf (ColorTransformation* transform, GdkPixbuf* pixbuf);
void color_transformation_transform_existing_pixbuf (ColorTransformation* transform, GdkPixbuf* in_pixbuf, GdkPixbuf* out_pixbuf);
static void color_transformation_finalize (ColorTransformation* obj);
GType exposure_transformation_get_type (void);
enum  {
	EXPOSURE_TRANSFORMATION_DUMMY_PROPERTY
};
#define EXPOSURE_TRANSFORMATION_EPSILON 0.08f
#define EXPOSURE_TRANSFORMATION_PARAMETER_SCALE (1.0f / 32.0f)
#define EXPOSURE_TRANSFORMATION_MIN_PARAMETER (-16.0f)
#define EXPOSURE_TRANSFORMATION_MAX_PARAMETER 16.0f
ExposureTransformation* exposure_transformation_new (float parameter);
ExposureTransformation* exposure_transformation_construct (GType object_type, float parameter);
GType saturation_transformation_get_type (void);
static GType saturation_transformation_weight_kind_get_type (void);
#define SATURATION_TRANSFORMATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SATURATION_TRANSFORMATION, SaturationTransformationPrivate))
enum  {
	SATURATION_TRANSFORMATION_DUMMY_PROPERTY
};
#define SATURATION_TRANSFORMATION_NTSC_WEIGHT_RED 0.299f
#define SATURATION_TRANSFORMATION_NTSC_WEIGHT_GREEN 0.587f
#define SATURATION_TRANSFORMATION_NTSC_WEIGHT_BLUE 0.114f
#define SATURATION_TRANSFORMATION_LINEAR_WEIGHT_RED 0.3086f
#define SATURATION_TRANSFORMATION_LINEAR_WEIGHT_GREEN 0.6094f
#define SATURATION_TRANSFORMATION_LINEAR_WEIGHT_BLUE 0.0820f
#define SATURATION_TRANSFORMATION_FLAT_WEIGHT_RED 0.333f
#define SATURATION_TRANSFORMATION_FLAT_WEIGHT_GREEN 0.333f
#define SATURATION_TRANSFORMATION_FLAT_WEIGHT_BLUE 0.333f
#define SATURATION_TRANSFORMATION_MIN_PARAMETER (-16.0f)
#define SATURATION_TRANSFORMATION_MAX_PARAMETER 16.0f
SaturationTransformation* saturation_transformation_new (float parameter);
SaturationTransformation* saturation_transformation_construct (GType object_type, float parameter);
static void saturation_transformation_finalize (ColorTransformation* obj);
GType temperature_transformation_get_type (void);
enum  {
	TEMPERATURE_TRANSFORMATION_DUMMY_PROPERTY
};
#define TEMPERATURE_TRANSFORMATION_INTENSITY_FACTOR 0.33f
#define TEMPERATURE_TRANSFORMATION_MIN_PARAMETER (-16.0f)
#define TEMPERATURE_TRANSFORMATION_MAX_PARAMETER 16.0f
TemperatureTransformation* temperature_transformation_new (float parameter);
TemperatureTransformation* temperature_transformation_construct (GType object_type, float parameter);
GType tint_transformation_get_type (void);
enum  {
	TINT_TRANSFORMATION_DUMMY_PROPERTY
};
#define TINT_TRANSFORMATION_INTENSITY_FACTOR 0.25f
#define TINT_TRANSFORMATION_MIN_PARAMETER (-16.0f)
#define TINT_TRANSFORMATION_MAX_PARAMETER 16.0f
TintTransformation* tint_transformation_new (float parameter);
TintTransformation* tint_transformation_construct (GType object_type, float parameter);
gpointer color_transformation_factory_ref (gpointer instance);
void color_transformation_factory_unref (gpointer instance);
GParamSpec* param_spec_color_transformation_factory (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_color_transformation_factory (GValue* value, gpointer v_object);
gpointer value_get_color_transformation_factory (const GValue* value);
GType color_transformation_factory_get_type (void);
enum  {
	COLOR_TRANSFORMATION_FACTORY_DUMMY_PROPERTY
};
ColorTransformationFactory* color_transformation_factory_new (void);
ColorTransformationFactory* color_transformation_factory_construct (GType object_type);
ColorTransformationFactory* color_transformation_factory_get_instance (void);
ColorTransformation* color_transformation_factory_from_parameter (ColorTransformationFactory* self, ColorTransformationKind kind, float parameter);
static void color_transformation_factory_finalize (ColorTransformationFactory* obj);
gpointer image_histogram_ref (gpointer instance);
void image_histogram_unref (gpointer instance);
GParamSpec* param_spec_image_histogram (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_image_histogram (GValue* value, gpointer v_object);
gpointer value_get_image_histogram (const GValue* value);
GType image_histogram_get_type (void);
#define IMAGE_HISTOGRAM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_IMAGE_HISTOGRAM, ImageHistogramPrivate))
enum  {
	IMAGE_HISTOGRAM_DUMMY_PROPERTY
};
#define IMAGE_HISTOGRAM_MARKED_BACKGROUND ((guchar) 30)
#define IMAGE_HISTOGRAM_MARKED_FOREGROUND ((guchar) 210)
#define IMAGE_HISTOGRAM_UNMARKED_BACKGROUND ((guchar) 40)
#define IMAGE_HISTOGRAM_GRAPHIC_WIDTH 256
#define IMAGE_HISTOGRAM_GRAPHIC_HEIGHT 100
ImageHistogram* image_histogram_new (GdkPixbuf* pixbuf);
ImageHistogram* image_histogram_construct (GType object_type, GdkPixbuf* pixbuf);
static gint image_histogram_correct_snap_to_quantization (ImageHistogram* self, gint* buckets, int buckets_length1, gint i);
static gint image_histogram_correct_snap_from_quantization (ImageHistogram* self, gint* buckets, int buckets_length1, gint i);
static void image_histogram_smooth_extrema (ImageHistogram* self, gint* count_data, int count_data_length1);
static gint* _vala_array_dup3 (gint* self, int length);
static gint* _vala_array_dup4 (gint* self, int length);
static gint* _vala_array_dup5 (gint* self, int length);
static gint* _vala_array_dup6 (gint* self, int length);
static gint* _vala_array_dup7 (gint* self, int length);
static gint* _vala_array_dup8 (gint* self, int length);
static gint* _vala_array_dup9 (gint* self, int length);
static gint* _vala_array_dup10 (gint* self, int length);
static gint* _vala_array_dup11 (gint* self, int length);
static void image_histogram_prepare_qualitative_counts (ImageHistogram* self);
GdkPixbuf* image_histogram_get_graphic (ImageHistogram* self);
static void image_histogram_finalize (ImageHistogram* obj);



#line 12 "ColorTransformation.vala"
void analytic_pixel_init (AnalyticPixel *self) {
#line 383 "ColorTransformation.c"
	memset (self, 0, sizeof (AnalyticPixel));
#line 13 "ColorTransformation.vala"
	(*self).red = 0.0f;
#line 14 "ColorTransformation.vala"
	(*self).green = 0.0f;
#line 15 "ColorTransformation.vala"
	(*self).blue = 0.0f;
#line 391 "ColorTransformation.c"
}


#line 18 "ColorTransformation.vala"
void analytic_pixel_init_from_components (AnalyticPixel *self, float red, float green, float blue) {
#line 397 "ColorTransformation.c"
	memset (self, 0, sizeof (AnalyticPixel));
#line 20 "ColorTransformation.vala"
	(*self).red = CLAMP (red, 0.0f, 1.0f);
#line 21 "ColorTransformation.vala"
	(*self).green = CLAMP (green, 0.0f, 1.0f);
#line 22 "ColorTransformation.vala"
	(*self).blue = CLAMP (blue, 0.0f, 1.0f);
#line 405 "ColorTransformation.c"
}


#line 25 "ColorTransformation.vala"
void analytic_pixel_init_from_quantized_components (AnalyticPixel *self, guchar red_quantized, guchar green_quantized, guchar blue_quantized) {
#line 411 "ColorTransformation.c"
	memset (self, 0, sizeof (AnalyticPixel));
#line 27 "ColorTransformation.vala"
	(*self).red = ((float) red_quantized) / 255.0f;
#line 28 "ColorTransformation.vala"
	(*self).green = ((float) green_quantized) / 255.0f;
#line 29 "ColorTransformation.vala"
	(*self).blue = ((float) blue_quantized) / 255.0f;
#line 419 "ColorTransformation.c"
}


#line 32 "ColorTransformation.vala"
guchar analytic_pixel_quantized_red (AnalyticPixel *self) {
#line 425 "ColorTransformation.c"
	guchar result;
	result = (guchar) ((*self).red * 255.0f);
	return result;
}


#line 36 "ColorTransformation.vala"
guchar analytic_pixel_quantized_green (AnalyticPixel *self) {
#line 434 "ColorTransformation.c"
	guchar result;
	result = (guchar) ((*self).green * 255.0f);
	return result;
}


#line 40 "ColorTransformation.vala"
guchar analytic_pixel_quantized_blue (AnalyticPixel *self) {
#line 443 "ColorTransformation.c"
	guchar result;
	result = (guchar) ((*self).blue * 255.0f);
	return result;
}


#line 44 "ColorTransformation.vala"
AnalyticPixel analytic_pixel_get_pixbuf_pixel (GdkPixbuf* pixbuf, gint x, gint y) {
#line 452 "ColorTransformation.c"
	AnalyticPixel result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gint px_start_byte_offset;
	guchar* _tmp2_;
	gint pixel_data_size;
	gint pixel_data_length1;
	guchar* pixel_data;
	AnalyticPixel _tmp3_ = {0};
	_tmp0_ = FALSE;
#line 46 "ColorTransformation.vala"
	if (x >= 0) {
#line 46 "ColorTransformation.vala"
		_tmp0_ = x < gdk_pixbuf_get_width (pixbuf);
#line 467 "ColorTransformation.c"
	} else {
#line 46 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 471 "ColorTransformation.c"
	}
#line 46 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 475 "ColorTransformation.c"
	_tmp1_ = FALSE;
#line 47 "ColorTransformation.vala"
	if (y >= 0) {
#line 47 "ColorTransformation.vala"
		_tmp1_ = y < gdk_pixbuf_get_height (pixbuf);
#line 481 "ColorTransformation.c"
	} else {
#line 47 "ColorTransformation.vala"
		_tmp1_ = FALSE;
#line 485 "ColorTransformation.c"
	}
#line 47 "ColorTransformation.vala"
	g_assert (_tmp1_);
#line 489 "ColorTransformation.c"
	px_start_byte_offset = (y * gdk_pixbuf_get_rowstride (pixbuf)) + (x * gdk_pixbuf_get_n_channels (pixbuf));
#line 52 "ColorTransformation.vala"
	_tmp2_ = NULL;
#line 493 "ColorTransformation.c"
	pixel_data = (_tmp2_ = gdk_pixbuf_get_pixels (pixbuf), pixel_data_length1 = -1, pixel_data_size = pixel_data_length1, _tmp2_);
	result = (analytic_pixel_init_from_quantized_components (&_tmp3_, pixel_data[px_start_byte_offset], pixel_data[px_start_byte_offset + 1], pixel_data[px_start_byte_offset + 2]), _tmp3_);
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	return result;
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


#line 60 "ColorTransformation.vala"
void analytic_pixel_set_pixbuf_pixel (GdkPixbuf* pixbuf, const AnalyticPixel* pixel, gint x, gint y) {
#line 504 "ColorTransformation.c"
	gboolean _tmp0_;
	gboolean _tmp1_;
	gint px_start_byte_offset;
	guchar* _tmp2_;
	gint pixel_data_size;
	gint pixel_data_length1;
	guchar* pixel_data;
#line 60 "ColorTransformation.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 514 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 62 "ColorTransformation.vala"
	if (x >= 0) {
#line 62 "ColorTransformation.vala"
		_tmp0_ = x < gdk_pixbuf_get_width (pixbuf);
#line 520 "ColorTransformation.c"
	} else {
#line 62 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 524 "ColorTransformation.c"
	}
#line 62 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 528 "ColorTransformation.c"
	_tmp1_ = FALSE;
#line 63 "ColorTransformation.vala"
	if (y >= 0) {
#line 63 "ColorTransformation.vala"
		_tmp1_ = y < gdk_pixbuf_get_height (pixbuf);
#line 534 "ColorTransformation.c"
	} else {
#line 63 "ColorTransformation.vala"
		_tmp1_ = FALSE;
#line 538 "ColorTransformation.c"
	}
#line 63 "ColorTransformation.vala"
	g_assert (_tmp1_);
#line 542 "ColorTransformation.c"
	px_start_byte_offset = (y * gdk_pixbuf_get_rowstride (pixbuf)) + (x * gdk_pixbuf_get_n_channels (pixbuf));
#line 68 "ColorTransformation.vala"
	_tmp2_ = NULL;
#line 546 "ColorTransformation.c"
	pixel_data = (_tmp2_ = gdk_pixbuf_get_pixels (pixbuf), pixel_data_length1 = -1, pixel_data_size = pixel_data_length1, _tmp2_);
#line 70 "ColorTransformation.vala"
	pixel_data[px_start_byte_offset] = analytic_pixel_quantized_red (&(*pixel));
#line 71 "ColorTransformation.vala"
	pixel_data[px_start_byte_offset + 1] = analytic_pixel_quantized_green (&(*pixel));
#line 72 "ColorTransformation.vala"
	pixel_data[px_start_byte_offset + 2] = analytic_pixel_quantized_blue (&(*pixel));
#line 554 "ColorTransformation.c"
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


AnalyticPixel* analytic_pixel_dup (const AnalyticPixel* self) {
	AnalyticPixel* dup;
	dup = g_new0 (AnalyticPixel, 1);
	memcpy (dup, self, sizeof (AnalyticPixel));
	return dup;
}


void analytic_pixel_free (AnalyticPixel* self) {
	g_free (self);
}


GType analytic_pixel_get_type (void) {
	static GType analytic_pixel_type_id = 0;
	if (analytic_pixel_type_id == 0) {
		analytic_pixel_type_id = g_boxed_type_register_static ("AnalyticPixel", (GBoxedCopyFunc) analytic_pixel_dup, (GBoxedFreeFunc) analytic_pixel_free);
	}
	return analytic_pixel_type_id;
}



GType color_transformation_kind_get_type (void) {
	static GType color_transformation_kind_type_id = 0;
	if (G_UNLIKELY (color_transformation_kind_type_id == 0)) {
		static const GEnumValue values[] = {{COLOR_TRANSFORMATION_KIND_EXPOSURE, "COLOR_TRANSFORMATION_KIND_EXPOSURE", "exposure"}, {COLOR_TRANSFORMATION_KIND_SATURATION, "COLOR_TRANSFORMATION_KIND_SATURATION", "saturation"}, {COLOR_TRANSFORMATION_KIND_TINT, "COLOR_TRANSFORMATION_KIND_TINT", "tint"}, {COLOR_TRANSFORMATION_KIND_TEMPERATURE, "COLOR_TRANSFORMATION_KIND_TEMPERATURE", "temperature"}, {0, NULL, NULL}};
		color_transformation_kind_type_id = g_enum_register_static ("ColorTransformationKind", values);
	}
	return color_transformation_kind_type_id;
}


ColorTransformationInstance* color_transformation_instance_dup (const ColorTransformationInstance* self) {
	ColorTransformationInstance* dup;
	dup = g_new0 (ColorTransformationInstance, 1);
	memcpy (dup, self, sizeof (ColorTransformationInstance));
	return dup;
}


void color_transformation_instance_free (ColorTransformationInstance* self) {
	g_free (self);
}


GType color_transformation_instance_get_type (void) {
	static GType color_transformation_instance_type_id = 0;
	if (color_transformation_instance_type_id == 0) {
		color_transformation_instance_type_id = g_boxed_type_register_static ("ColorTransformationInstance", (GBoxedCopyFunc) color_transformation_instance_dup, (GBoxedFreeFunc) color_transformation_instance_free);
	}
	return color_transformation_instance_type_id;
}


#line 99 "ColorTransformation.vala"
ColorTransformation* color_transformation_construct (GType object_type) {
#line 616 "ColorTransformation.c"
	ColorTransformation* self;
	self = (ColorTransformation*) g_type_create_instance (object_type);
	return self;
}


#line 99 "ColorTransformation.vala"
ColorTransformation* color_transformation_new (void) {
#line 99 "ColorTransformation.vala"
	return color_transformation_construct (TYPE_COLOR_TRANSFORMATION);
#line 627 "ColorTransformation.c"
}


#line 102 "ColorTransformation.vala"
gboolean color_transformation_is_identity (ColorTransformation* self) {
#line 633 "ColorTransformation.c"
	gboolean result;
#line 102 "ColorTransformation.vala"
	g_return_val_if_fail (IS_COLOR_TRANSFORMATION (self), FALSE);
#line 637 "ColorTransformation.c"
	result = self->identity;
	return result;
}


#line 106 "ColorTransformation.vala"
AnalyticPixel color_transformation_transform_pixel (ColorTransformation* self, const AnalyticPixel* pixel) {
#line 645 "ColorTransformation.c"
	AnalyticPixel result;
	float red_out;
	float green_out;
	float blue_out;
	AnalyticPixel _tmp0_ = {0};
	red_out = ((((*pixel).red * self->matrix_entries[0]) + ((*pixel).green * self->matrix_entries[1])) + ((*pixel).blue * self->matrix_entries[2])) + self->matrix_entries[3];
#line 111 "ColorTransformation.vala"
	red_out = CLAMP (red_out, 0.0f, 1.0f);
#line 654 "ColorTransformation.c"
	green_out = ((((*pixel).red * self->matrix_entries[4]) + ((*pixel).green * self->matrix_entries[5])) + ((*pixel).blue * self->matrix_entries[6])) + self->matrix_entries[7];
#line 117 "ColorTransformation.vala"
	green_out = CLAMP (green_out, 0.0f, 1.0f);
#line 658 "ColorTransformation.c"
	blue_out = ((((*pixel).red * self->matrix_entries[8]) + ((*pixel).green * self->matrix_entries[9])) + ((*pixel).blue * self->matrix_entries[10])) + self->matrix_entries[11];
#line 123 "ColorTransformation.vala"
	blue_out = CLAMP (blue_out, 0.0f, 1.0f);
#line 662 "ColorTransformation.c"
	result = (analytic_pixel_init_from_components (&_tmp0_, red_out, green_out, blue_out), _tmp0_);
	return result;
}


#line 129 "ColorTransformation.vala"
ColorTransformation* color_transformation_compose_against (ColorTransformation* self, ColorTransformation* transform) {
#line 670 "ColorTransformation.c"
	ColorTransformation* result;
	ColorTransformation* _result_;
#line 129 "ColorTransformation.vala"
	g_return_val_if_fail (IS_COLOR_TRANSFORMATION (self), NULL);
#line 129 "ColorTransformation.vala"
	g_return_val_if_fail (IS_COLOR_TRANSFORMATION (transform), NULL);
#line 677 "ColorTransformation.c"
	_result_ = color_transformation_new ();
	/* row 0 */
#line 133 "ColorTransformation.vala"
	_result_->matrix_entries[0] = (((self->matrix_entries[0] * transform->matrix_entries[0]) + (self->matrix_entries[1] * transform->matrix_entries[4])) + (self->matrix_entries[2] * transform->matrix_entries[8])) + (self->matrix_entries[3] * transform->matrix_entries[12]);
#line 139 "ColorTransformation.vala"
	_result_->matrix_entries[1] = (((self->matrix_entries[0] * transform->matrix_entries[1]) + (self->matrix_entries[1] * transform->matrix_entries[5])) + (self->matrix_entries[2] * transform->matrix_entries[9])) + (self->matrix_entries[3] * transform->matrix_entries[13]);
#line 145 "ColorTransformation.vala"
	_result_->matrix_entries[2] = (((self->matrix_entries[0] * transform->matrix_entries[2]) + (self->matrix_entries[1] * transform->matrix_entries[6])) + (self->matrix_entries[2] * transform->matrix_entries[10])) + (self->matrix_entries[3] * transform->matrix_entries[14]);
#line 151 "ColorTransformation.vala"
	_result_->matrix_entries[3] = (((self->matrix_entries[0] * transform->matrix_entries[3]) + (self->matrix_entries[1] * transform->matrix_entries[7])) + (self->matrix_entries[2] * transform->matrix_entries[11])) + (self->matrix_entries[3] * transform->matrix_entries[15]);
#line 688 "ColorTransformation.c"
	/* row 1 */
#line 158 "ColorTransformation.vala"
	_result_->matrix_entries[4] = (((self->matrix_entries[4] * transform->matrix_entries[0]) + (self->matrix_entries[5] * transform->matrix_entries[4])) + (self->matrix_entries[6] * transform->matrix_entries[8])) + (self->matrix_entries[7] * transform->matrix_entries[12]);
#line 164 "ColorTransformation.vala"
	_result_->matrix_entries[5] = (((self->matrix_entries[4] * transform->matrix_entries[1]) + (self->matrix_entries[5] * transform->matrix_entries[5])) + (self->matrix_entries[6] * transform->matrix_entries[9])) + (self->matrix_entries[7] * transform->matrix_entries[13]);
#line 170 "ColorTransformation.vala"
	_result_->matrix_entries[6] = (((self->matrix_entries[4] * transform->matrix_entries[2]) + (self->matrix_entries[5] * transform->matrix_entries[6])) + (self->matrix_entries[6] * transform->matrix_entries[10])) + (self->matrix_entries[7] * transform->matrix_entries[14]);
#line 176 "ColorTransformation.vala"
	_result_->matrix_entries[7] = (((self->matrix_entries[4] * transform->matrix_entries[3]) + (self->matrix_entries[5] * transform->matrix_entries[7])) + (self->matrix_entries[6] * transform->matrix_entries[11])) + (self->matrix_entries[7] * transform->matrix_entries[15]);
#line 698 "ColorTransformation.c"
	/* row 2 */
#line 183 "ColorTransformation.vala"
	_result_->matrix_entries[8] = (((self->matrix_entries[8] * transform->matrix_entries[0]) + (self->matrix_entries[9] * transform->matrix_entries[4])) + (self->matrix_entries[10] * transform->matrix_entries[8])) + (self->matrix_entries[11] * transform->matrix_entries[12]);
#line 189 "ColorTransformation.vala"
	_result_->matrix_entries[9] = (((self->matrix_entries[8] * transform->matrix_entries[1]) + (self->matrix_entries[9] * transform->matrix_entries[5])) + (self->matrix_entries[10] * transform->matrix_entries[9])) + (self->matrix_entries[11] * transform->matrix_entries[13]);
#line 195 "ColorTransformation.vala"
	_result_->matrix_entries[10] = (((self->matrix_entries[8] * transform->matrix_entries[2]) + (self->matrix_entries[9] * transform->matrix_entries[6])) + (self->matrix_entries[10] * transform->matrix_entries[10])) + (self->matrix_entries[11] * transform->matrix_entries[14]);
#line 201 "ColorTransformation.vala"
	_result_->matrix_entries[11] = (((self->matrix_entries[8] * transform->matrix_entries[3]) + (self->matrix_entries[9] * transform->matrix_entries[7])) + (self->matrix_entries[10] * transform->matrix_entries[11])) + (self->matrix_entries[11] * transform->matrix_entries[15]);
#line 708 "ColorTransformation.c"
	/* row 3 */
#line 208 "ColorTransformation.vala"
	_result_->matrix_entries[12] = (((self->matrix_entries[12] * transform->matrix_entries[0]) + (self->matrix_entries[13] * transform->matrix_entries[4])) + (self->matrix_entries[14] * transform->matrix_entries[8])) + (self->matrix_entries[15] * transform->matrix_entries[12]);
#line 214 "ColorTransformation.vala"
	_result_->matrix_entries[13] = (((self->matrix_entries[12] * transform->matrix_entries[1]) + (self->matrix_entries[13] * transform->matrix_entries[5])) + (self->matrix_entries[14] * transform->matrix_entries[9])) + (self->matrix_entries[15] * transform->matrix_entries[13]);
#line 220 "ColorTransformation.vala"
	_result_->matrix_entries[14] = (((self->matrix_entries[12] * transform->matrix_entries[2]) + (self->matrix_entries[13] * transform->matrix_entries[6])) + (self->matrix_entries[14] * transform->matrix_entries[10])) + (self->matrix_entries[15] * transform->matrix_entries[14]);
#line 226 "ColorTransformation.vala"
	_result_->matrix_entries[15] = (((self->matrix_entries[12] * transform->matrix_entries[3]) + (self->matrix_entries[13] * transform->matrix_entries[7])) + (self->matrix_entries[14] * transform->matrix_entries[11])) + (self->matrix_entries[15] * transform->matrix_entries[15]);
#line 232 "ColorTransformation.vala"
	if (!self->identity) {
#line 233 "ColorTransformation.vala"
		_result_->identity = FALSE;
#line 722 "ColorTransformation.c"
	}
#line 235 "ColorTransformation.vala"
	if (!transform->identity) {
#line 236 "ColorTransformation.vala"
		_result_->identity = FALSE;
#line 728 "ColorTransformation.c"
	}
	result = _result_;
	return result;
}


#line 242 "ColorTransformation.vala"
void color_transformation_transform_pixbuf (ColorTransformation* transform, GdkPixbuf* pixbuf) {
#line 242 "ColorTransformation.vala"
	g_return_if_fail (IS_COLOR_TRANSFORMATION (transform));
#line 242 "ColorTransformation.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 741 "ColorTransformation.c"
	{
		gint j;
		j = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 244 "ColorTransformation.vala"
			while (TRUE) {
#line 244 "ColorTransformation.vala"
				if (!_tmp0_) {
#line 244 "ColorTransformation.vala"
					j++;
#line 754 "ColorTransformation.c"
				}
#line 244 "ColorTransformation.vala"
				_tmp0_ = FALSE;
#line 244 "ColorTransformation.vala"
				if (!(j < gdk_pixbuf_get_height (pixbuf))) {
#line 244 "ColorTransformation.vala"
					break;
#line 762 "ColorTransformation.c"
				}
				{
					gint i;
					i = 0;
					{
						gboolean _tmp1_;
						_tmp1_ = TRUE;
#line 245 "ColorTransformation.vala"
						while (TRUE) {
#line 772 "ColorTransformation.c"
							GdkPixbuf* _tmp2_;
							AnalyticPixel pixel_ij;
							AnalyticPixel pixel_ij_transformed;
							GdkPixbuf* _tmp3_;
#line 245 "ColorTransformation.vala"
							if (!_tmp1_) {
#line 245 "ColorTransformation.vala"
								i++;
#line 781 "ColorTransformation.c"
							}
#line 245 "ColorTransformation.vala"
							_tmp1_ = FALSE;
#line 245 "ColorTransformation.vala"
							if (!(i < gdk_pixbuf_get_width (pixbuf))) {
#line 245 "ColorTransformation.vala"
								break;
#line 789 "ColorTransformation.c"
							}
#line 248 "ColorTransformation.vala"
							_tmp2_ = NULL;
#line 793 "ColorTransformation.c"
							pixel_ij = analytic_pixel_get_pixbuf_pixel ((_tmp2_ = pixbuf, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), i, j);
							pixel_ij_transformed = color_transformation_transform_pixel (transform, &pixel_ij);
#line 253 "ColorTransformation.vala"
							_tmp3_ = NULL;
#line 253 "ColorTransformation.vala"
							analytic_pixel_set_pixbuf_pixel ((_tmp3_ = pixbuf, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), &pixel_ij_transformed, i, j);
#line 800 "ColorTransformation.c"
						}
					}
				}
			}
		}
	}
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


#line 259 "ColorTransformation.vala"
void color_transformation_transform_existing_pixbuf (ColorTransformation* transform, GdkPixbuf* in_pixbuf, GdkPixbuf* out_pixbuf) {
#line 813 "ColorTransformation.c"
	gboolean _tmp0_;
#line 259 "ColorTransformation.vala"
	g_return_if_fail (IS_COLOR_TRANSFORMATION (transform));
#line 259 "ColorTransformation.vala"
	g_return_if_fail (GDK_IS_PIXBUF (in_pixbuf));
#line 259 "ColorTransformation.vala"
	g_return_if_fail (GDK_IS_PIXBUF (out_pixbuf));
#line 821 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 261 "ColorTransformation.vala"
	if (gdk_pixbuf_get_width (in_pixbuf) == gdk_pixbuf_get_width (out_pixbuf)) {
#line 261 "ColorTransformation.vala"
		_tmp0_ = gdk_pixbuf_get_height (in_pixbuf) == gdk_pixbuf_get_height (out_pixbuf);
#line 827 "ColorTransformation.c"
	} else {
#line 261 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 831 "ColorTransformation.c"
	}
#line 261 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 835 "ColorTransformation.c"
	{
		gint j;
		j = 0;
		{
			gboolean _tmp1_;
			_tmp1_ = TRUE;
#line 263 "ColorTransformation.vala"
			while (TRUE) {
#line 263 "ColorTransformation.vala"
				if (!_tmp1_) {
#line 263 "ColorTransformation.vala"
					j++;
#line 848 "ColorTransformation.c"
				}
#line 263 "ColorTransformation.vala"
				_tmp1_ = FALSE;
#line 263 "ColorTransformation.vala"
				if (!(j < gdk_pixbuf_get_height (in_pixbuf))) {
#line 263 "ColorTransformation.vala"
					break;
#line 856 "ColorTransformation.c"
				}
				{
					gint i;
					i = 0;
					{
						gboolean _tmp2_;
						_tmp2_ = TRUE;
#line 264 "ColorTransformation.vala"
						while (TRUE) {
#line 866 "ColorTransformation.c"
							GdkPixbuf* _tmp3_;
							AnalyticPixel pixel_ij;
							AnalyticPixel pixel_ij_transformed;
							GdkPixbuf* _tmp4_;
#line 264 "ColorTransformation.vala"
							if (!_tmp2_) {
#line 264 "ColorTransformation.vala"
								i++;
#line 875 "ColorTransformation.c"
							}
#line 264 "ColorTransformation.vala"
							_tmp2_ = FALSE;
#line 264 "ColorTransformation.vala"
							if (!(i < gdk_pixbuf_get_width (in_pixbuf))) {
#line 264 "ColorTransformation.vala"
								break;
#line 883 "ColorTransformation.c"
							}
#line 267 "ColorTransformation.vala"
							_tmp3_ = NULL;
#line 887 "ColorTransformation.c"
							pixel_ij = analytic_pixel_get_pixbuf_pixel ((_tmp3_ = in_pixbuf, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), i, j);
							pixel_ij_transformed = color_transformation_transform_pixel (transform, &pixel_ij);
#line 272 "ColorTransformation.vala"
							_tmp4_ = NULL;
#line 272 "ColorTransformation.vala"
							analytic_pixel_set_pixbuf_pixel ((_tmp4_ = out_pixbuf, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), &pixel_ij_transformed, i, j);
#line 894 "ColorTransformation.c"
						}
					}
				}
			}
		}
	}
	(in_pixbuf == NULL) ? NULL : (in_pixbuf = (g_object_unref (in_pixbuf), NULL));
	(out_pixbuf == NULL) ? NULL : (out_pixbuf = (g_object_unref (out_pixbuf), NULL));
}


static void value_color_transformation_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_color_transformation_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		color_transformation_unref (value->data[0].v_pointer);
	}
}


static void value_color_transformation_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = color_transformation_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_color_transformation_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_color_transformation_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ColorTransformation* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = color_transformation_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_color_transformation_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ColorTransformation** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = color_transformation_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_color_transformation (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecColorTransformation* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_COLOR_TRANSFORMATION), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_color_transformation (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_COLOR_TRANSFORMATION), NULL);
	return value->data[0].v_pointer;
}


void value_set_color_transformation (GValue* value, gpointer v_object) {
	ColorTransformation* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_COLOR_TRANSFORMATION));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_COLOR_TRANSFORMATION));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		color_transformation_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		color_transformation_unref (old);
	}
}


static void color_transformation_class_init (ColorTransformationClass * klass) {
	color_transformation_parent_class = g_type_class_peek_parent (klass);
	COLOR_TRANSFORMATION_CLASS (klass)->finalize = color_transformation_finalize;
}


static void color_transformation_instance_init (ColorTransformation * self) {
	float* _tmp0_;
	self->matrix_entries = (_tmp0_ = g_new0 (float, 16), _tmp0_[0] = 1.0f, _tmp0_[1] = 0.0f, _tmp0_[2] = 0.0f, _tmp0_[3] = 0.0f, _tmp0_[4] = 0.0f, _tmp0_[5] = 1.0f, _tmp0_[6] = 0.0f, _tmp0_[7] = 0.0f, _tmp0_[8] = 0.0f, _tmp0_[9] = 0.0f, _tmp0_[10] = 1.0f, _tmp0_[11] = 0.0f, _tmp0_[12] = 0.0f, _tmp0_[13] = 0.0f, _tmp0_[14] = 0.0f, _tmp0_[15] = 1.0f, _tmp0_);
	self->matrix_entries_length1 = 16;
#line 91 "ColorTransformation.vala"
	_tmp0_ = NULL;
#line 1011 "ColorTransformation.c"
	self->identity = TRUE;
	self->ref_count = 1;
}


static void color_transformation_finalize (ColorTransformation* obj) {
	ColorTransformation * self;
	self = COLOR_TRANSFORMATION (obj);
	self->matrix_entries = (g_free (self->matrix_entries), NULL);
}


GType color_transformation_get_type (void) {
	static GType color_transformation_type_id = 0;
	if (color_transformation_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_color_transformation_init, value_color_transformation_free_value, value_color_transformation_copy_value, value_color_transformation_peek_pointer, "p", value_color_transformation_collect_value, "p", value_color_transformation_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ColorTransformationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) color_transformation_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ColorTransformation), 0, (GInstanceInitFunc) color_transformation_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		color_transformation_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ColorTransformation", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return color_transformation_type_id;
}


gpointer color_transformation_ref (gpointer instance) {
	ColorTransformation* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void color_transformation_unref (gpointer instance) {
	ColorTransformation* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		COLOR_TRANSFORMATION_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 287 "ColorTransformation.vala"
ExposureTransformation* exposure_transformation_construct (GType object_type, float parameter) {
#line 1056 "ColorTransformation.c"
	ExposureTransformation* self;
	gboolean _tmp0_;
#line 288 "ColorTransformation.vala"
	self = (ExposureTransformation*) color_transformation_construct (object_type);
#line 1061 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 290 "ColorTransformation.vala"
	if (parameter >= EXPOSURE_TRANSFORMATION_MIN_PARAMETER) {
#line 290 "ColorTransformation.vala"
		_tmp0_ = parameter <= EXPOSURE_TRANSFORMATION_MAX_PARAMETER;
#line 1067 "ColorTransformation.c"
	} else {
#line 290 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 1071 "ColorTransformation.c"
	}
#line 290 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 292 "ColorTransformation.vala"
	if (parameter != 0.0f) {
#line 294 "ColorTransformation.vala"
		parameter = parameter * EXPOSURE_TRANSFORMATION_PARAMETER_SCALE;
#line 295 "ColorTransformation.vala"
		if (parameter < 0.0f) {
#line 296 "ColorTransformation.vala"
			parameter = 1.0f / ((-parameter) + 1.0f);
#line 1083 "ColorTransformation.c"
		} else {
#line 298 "ColorTransformation.vala"
			parameter = parameter + 1.0f;
#line 1087 "ColorTransformation.c"
		}
#line 300 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[0] = parameter;
#line 301 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[5] = parameter;
#line 302 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[10] = parameter;
#line 303 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[3] = parameter * EXPOSURE_TRANSFORMATION_EPSILON;
#line 304 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[7] = parameter * EXPOSURE_TRANSFORMATION_EPSILON;
#line 305 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[11] = parameter * EXPOSURE_TRANSFORMATION_EPSILON;
#line 307 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->identity = FALSE;
#line 1103 "ColorTransformation.c"
	}
	return self;
}


#line 287 "ColorTransformation.vala"
ExposureTransformation* exposure_transformation_new (float parameter) {
#line 287 "ColorTransformation.vala"
	return exposure_transformation_construct (TYPE_EXPOSURE_TRANSFORMATION, parameter);
#line 1113 "ColorTransformation.c"
}


static void exposure_transformation_class_init (ExposureTransformationClass * klass) {
	exposure_transformation_parent_class = g_type_class_peek_parent (klass);
}


static void exposure_transformation_instance_init (ExposureTransformation * self) {
}


GType exposure_transformation_get_type (void) {
	static GType exposure_transformation_type_id = 0;
	if (exposure_transformation_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ExposureTransformationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) exposure_transformation_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExposureTransformation), 0, (GInstanceInitFunc) exposure_transformation_instance_init, NULL };
		exposure_transformation_type_id = g_type_register_static (TYPE_COLOR_TRANSFORMATION, "ExposureTransformation", &g_define_type_info, 0);
	}
	return exposure_transformation_type_id;
}



static GType saturation_transformation_weight_kind_get_type (void) {
	static GType saturation_transformation_weight_kind_type_id = 0;
	if (G_UNLIKELY (saturation_transformation_weight_kind_type_id == 0)) {
		static const GEnumValue values[] = {{SATURATION_TRANSFORMATION_WEIGHT_KIND_NTSC, "SATURATION_TRANSFORMATION_WEIGHT_KIND_NTSC", "ntsc"}, {SATURATION_TRANSFORMATION_WEIGHT_KIND_LINEAR, "SATURATION_TRANSFORMATION_WEIGHT_KIND_LINEAR", "linear"}, {SATURATION_TRANSFORMATION_WEIGHT_KIND_FLAT, "SATURATION_TRANSFORMATION_WEIGHT_KIND_FLAT", "flat"}, {0, NULL, NULL}};
		saturation_transformation_weight_kind_type_id = g_enum_register_static ("SaturationTransformationWeightKind", values);
	}
	return saturation_transformation_weight_kind_type_id;
}


#line 331 "ColorTransformation.vala"
SaturationTransformation* saturation_transformation_construct (GType object_type, float parameter) {
#line 1149 "ColorTransformation.c"
	SaturationTransformation* self;
	gboolean _tmp0_;
#line 332 "ColorTransformation.vala"
	self = (SaturationTransformation*) color_transformation_construct (object_type);
#line 1154 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 334 "ColorTransformation.vala"
	if (parameter >= SATURATION_TRANSFORMATION_MIN_PARAMETER) {
#line 334 "ColorTransformation.vala"
		_tmp0_ = parameter <= SATURATION_TRANSFORMATION_MAX_PARAMETER;
#line 1160 "ColorTransformation.c"
	} else {
#line 334 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 1164 "ColorTransformation.c"
	}
#line 334 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 336 "ColorTransformation.vala"
	if (parameter != 0.0f) {
#line 1170 "ColorTransformation.c"
		float adjusted_param;
		float red_weight;
		float green_weight;
		float blue_weight;
		adjusted_param = parameter / SATURATION_TRANSFORMATION_MAX_PARAMETER;
#line 338 "ColorTransformation.vala"
		adjusted_param = adjusted_param + 1.0f;
#line 1178 "ColorTransformation.c"
		red_weight = 0.0f;
		green_weight = 0.0f;
		blue_weight = 0.0f;
#line 343 "ColorTransformation.vala"
		if (self->priv->weight_kind == SATURATION_TRANSFORMATION_WEIGHT_KIND_NTSC) {
#line 344 "ColorTransformation.vala"
			red_weight = SATURATION_TRANSFORMATION_NTSC_WEIGHT_RED;
#line 345 "ColorTransformation.vala"
			green_weight = SATURATION_TRANSFORMATION_NTSC_WEIGHT_GREEN;
#line 346 "ColorTransformation.vala"
			blue_weight = SATURATION_TRANSFORMATION_NTSC_WEIGHT_BLUE;
#line 1190 "ColorTransformation.c"
		} else {
#line 347 "ColorTransformation.vala"
			if (self->priv->weight_kind == SATURATION_TRANSFORMATION_WEIGHT_KIND_LINEAR) {
#line 348 "ColorTransformation.vala"
				red_weight = SATURATION_TRANSFORMATION_LINEAR_WEIGHT_RED;
#line 349 "ColorTransformation.vala"
				green_weight = SATURATION_TRANSFORMATION_LINEAR_WEIGHT_GREEN;
#line 350 "ColorTransformation.vala"
				blue_weight = SATURATION_TRANSFORMATION_LINEAR_WEIGHT_BLUE;
#line 1200 "ColorTransformation.c"
			} else {
#line 351 "ColorTransformation.vala"
				if (self->priv->weight_kind == SATURATION_TRANSFORMATION_WEIGHT_KIND_FLAT) {
#line 352 "ColorTransformation.vala"
					red_weight = SATURATION_TRANSFORMATION_FLAT_WEIGHT_RED;
#line 353 "ColorTransformation.vala"
					green_weight = SATURATION_TRANSFORMATION_FLAT_WEIGHT_GREEN;
#line 354 "ColorTransformation.vala"
					blue_weight = SATURATION_TRANSFORMATION_FLAT_WEIGHT_BLUE;
#line 1210 "ColorTransformation.c"
				} else {
#line 356 "ColorTransformation.vala"
					g_error ("ColorTransformation.vala:356: unrecognized weight kind.\n");
#line 1214 "ColorTransformation.c"
				}
			}
		}
#line 359 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[0] = ((1.0f - adjusted_param) * red_weight) + adjusted_param;
#line 361 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[1] = (1.0f - adjusted_param) * red_weight;
#line 362 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[2] = (1.0f - adjusted_param) * red_weight;
#line 364 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[4] = (1.0f - adjusted_param) * green_weight;
#line 365 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[5] = ((1.0f - adjusted_param) * green_weight) + adjusted_param;
#line 367 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[6] = (1.0f - adjusted_param) * green_weight;
#line 369 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[8] = (1.0f - adjusted_param) * blue_weight;
#line 370 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[9] = (1.0f - adjusted_param) * blue_weight;
#line 371 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[10] = ((1.0f - adjusted_param) * blue_weight) + adjusted_param;
#line 374 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->identity = FALSE;
#line 1238 "ColorTransformation.c"
	}
	return self;
}


#line 331 "ColorTransformation.vala"
SaturationTransformation* saturation_transformation_new (float parameter) {
#line 331 "ColorTransformation.vala"
	return saturation_transformation_construct (TYPE_SATURATION_TRANSFORMATION, parameter);
#line 1248 "ColorTransformation.c"
}


static void saturation_transformation_class_init (SaturationTransformationClass * klass) {
	saturation_transformation_parent_class = g_type_class_peek_parent (klass);
	COLOR_TRANSFORMATION_CLASS (klass)->finalize = saturation_transformation_finalize;
	g_type_class_add_private (klass, sizeof (SaturationTransformationPrivate));
}


static void saturation_transformation_instance_init (SaturationTransformation * self) {
	self->priv = SATURATION_TRANSFORMATION_GET_PRIVATE (self);
	self->priv->weight_kind = SATURATION_TRANSFORMATION_WEIGHT_KIND_FLAT;
}


static void saturation_transformation_finalize (ColorTransformation* obj) {
	SaturationTransformation * self;
	self = SATURATION_TRANSFORMATION (obj);
	COLOR_TRANSFORMATION_CLASS (saturation_transformation_parent_class)->finalize (obj);
}


GType saturation_transformation_get_type (void) {
	static GType saturation_transformation_type_id = 0;
	if (saturation_transformation_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SaturationTransformationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) saturation_transformation_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SaturationTransformation), 0, (GInstanceInitFunc) saturation_transformation_instance_init, NULL };
		saturation_transformation_type_id = g_type_register_static (TYPE_COLOR_TRANSFORMATION, "SaturationTransformation", &g_define_type_info, 0);
	}
	return saturation_transformation_type_id;
}


#line 384 "ColorTransformation.vala"
TemperatureTransformation* temperature_transformation_construct (GType object_type, float parameter) {
#line 1284 "ColorTransformation.c"
	TemperatureTransformation* self;
	gboolean _tmp0_;
#line 385 "ColorTransformation.vala"
	self = (TemperatureTransformation*) color_transformation_construct (object_type);
#line 1289 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 387 "ColorTransformation.vala"
	if (parameter >= TEMPERATURE_TRANSFORMATION_MIN_PARAMETER) {
#line 387 "ColorTransformation.vala"
		_tmp0_ = parameter <= TEMPERATURE_TRANSFORMATION_MAX_PARAMETER;
#line 1295 "ColorTransformation.c"
	} else {
#line 387 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 1299 "ColorTransformation.c"
	}
#line 387 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 389 "ColorTransformation.vala"
	if (parameter != 0.0f) {
#line 1305 "ColorTransformation.c"
		float adjusted_param;
		adjusted_param = parameter / TEMPERATURE_TRANSFORMATION_MAX_PARAMETER;
#line 391 "ColorTransformation.vala"
		adjusted_param = adjusted_param * TEMPERATURE_TRANSFORMATION_INTENSITY_FACTOR;
#line 393 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[11] -= adjusted_param;
#line 394 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[7] += adjusted_param / 2;
#line 395 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[3] += adjusted_param / 2;
#line 397 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->identity = FALSE;
#line 1318 "ColorTransformation.c"
	}
	return self;
}


#line 384 "ColorTransformation.vala"
TemperatureTransformation* temperature_transformation_new (float parameter) {
#line 384 "ColorTransformation.vala"
	return temperature_transformation_construct (TYPE_TEMPERATURE_TRANSFORMATION, parameter);
#line 1328 "ColorTransformation.c"
}


static void temperature_transformation_class_init (TemperatureTransformationClass * klass) {
	temperature_transformation_parent_class = g_type_class_peek_parent (klass);
}


static void temperature_transformation_instance_init (TemperatureTransformation * self) {
}


GType temperature_transformation_get_type (void) {
	static GType temperature_transformation_type_id = 0;
	if (temperature_transformation_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (TemperatureTransformationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) temperature_transformation_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TemperatureTransformation), 0, (GInstanceInitFunc) temperature_transformation_instance_init, NULL };
		temperature_transformation_type_id = g_type_register_static (TYPE_COLOR_TRANSFORMATION, "TemperatureTransformation", &g_define_type_info, 0);
	}
	return temperature_transformation_type_id;
}


#line 407 "ColorTransformation.vala"
TintTransformation* tint_transformation_construct (GType object_type, float parameter) {
#line 1353 "ColorTransformation.c"
	TintTransformation* self;
	gboolean _tmp0_;
#line 408 "ColorTransformation.vala"
	self = (TintTransformation*) color_transformation_construct (object_type);
#line 1358 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 410 "ColorTransformation.vala"
	if (parameter >= TINT_TRANSFORMATION_MIN_PARAMETER) {
#line 410 "ColorTransformation.vala"
		_tmp0_ = parameter <= TINT_TRANSFORMATION_MAX_PARAMETER;
#line 1364 "ColorTransformation.c"
	} else {
#line 410 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 1368 "ColorTransformation.c"
	}
#line 410 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 412 "ColorTransformation.vala"
	if (parameter != 0.0f) {
#line 1374 "ColorTransformation.c"
		float adjusted_param;
		adjusted_param = parameter / TINT_TRANSFORMATION_MAX_PARAMETER;
#line 414 "ColorTransformation.vala"
		adjusted_param = adjusted_param * TINT_TRANSFORMATION_INTENSITY_FACTOR;
#line 416 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[11] -= adjusted_param / 2;
#line 417 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[7] += adjusted_param;
#line 418 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->matrix_entries[3] -= adjusted_param / 2;
#line 420 "ColorTransformation.vala"
		COLOR_TRANSFORMATION (self)->identity = FALSE;
#line 1387 "ColorTransformation.c"
	}
	return self;
}


#line 407 "ColorTransformation.vala"
TintTransformation* tint_transformation_new (float parameter) {
#line 407 "ColorTransformation.vala"
	return tint_transformation_construct (TYPE_TINT_TRANSFORMATION, parameter);
#line 1397 "ColorTransformation.c"
}


static void tint_transformation_class_init (TintTransformationClass * klass) {
	tint_transformation_parent_class = g_type_class_peek_parent (klass);
}


static void tint_transformation_instance_init (TintTransformation * self) {
}


GType tint_transformation_get_type (void) {
	static GType tint_transformation_type_id = 0;
	if (tint_transformation_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (TintTransformationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tint_transformation_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TintTransformation), 0, (GInstanceInitFunc) tint_transformation_instance_init, NULL };
		tint_transformation_type_id = g_type_register_static (TYPE_COLOR_TRANSFORMATION, "TintTransformation", &g_define_type_info, 0);
	}
	return tint_transformation_type_id;
}


#line 428 "ColorTransformation.vala"
ColorTransformationFactory* color_transformation_factory_construct (GType object_type) {
#line 1422 "ColorTransformation.c"
	ColorTransformationFactory* self;
	self = (ColorTransformationFactory*) g_type_create_instance (object_type);
	return self;
}


#line 428 "ColorTransformation.vala"
ColorTransformationFactory* color_transformation_factory_new (void) {
#line 428 "ColorTransformation.vala"
	return color_transformation_factory_construct (TYPE_COLOR_TRANSFORMATION_FACTORY);
#line 1433 "ColorTransformation.c"
}


#line 431 "ColorTransformation.vala"
ColorTransformationFactory* color_transformation_factory_get_instance (void) {
#line 1439 "ColorTransformation.c"
	ColorTransformationFactory* result;
	ColorTransformationFactory* _tmp1_;
#line 432 "ColorTransformation.vala"
	if (color_transformation_factory_instance == NULL) {
#line 1444 "ColorTransformation.c"
		ColorTransformationFactory* _tmp0_;
		_tmp0_ = NULL;
#line 433 "ColorTransformation.vala"
		color_transformation_factory_instance = (_tmp0_ = color_transformation_factory_new (), (color_transformation_factory_instance == NULL) ? NULL : (color_transformation_factory_instance = (color_transformation_factory_unref (color_transformation_factory_instance), NULL)), _tmp0_);
#line 1449 "ColorTransformation.c"
	}
#line 435 "ColorTransformation.vala"
	_tmp1_ = NULL;
#line 1453 "ColorTransformation.c"
	result = (_tmp1_ = color_transformation_factory_instance, (_tmp1_ == NULL) ? NULL : color_transformation_factory_ref (_tmp1_));
	return result;
}


#line 438 "ColorTransformation.vala"
ColorTransformation* color_transformation_factory_from_parameter (ColorTransformationFactory* self, ColorTransformationKind kind, float parameter) {
#line 1461 "ColorTransformation.c"
	ColorTransformation* result;
#line 438 "ColorTransformation.vala"
	g_return_val_if_fail (IS_COLOR_TRANSFORMATION_FACTORY (self), NULL);
#line 440 "ColorTransformation.vala"
	switch (kind) {
#line 1467 "ColorTransformation.c"
		case COLOR_TRANSFORMATION_KIND_EXPOSURE:
		{
			result = COLOR_TRANSFORMATION (exposure_transformation_new ((float) parameter));
			return result;
		}
		case COLOR_TRANSFORMATION_KIND_SATURATION:
		{
			result = COLOR_TRANSFORMATION (saturation_transformation_new ((float) parameter));
			return result;
		}
		case COLOR_TRANSFORMATION_KIND_TINT:
		{
			result = COLOR_TRANSFORMATION (tint_transformation_new ((float) parameter));
			return result;
		}
		case COLOR_TRANSFORMATION_KIND_TEMPERATURE:
		{
			result = COLOR_TRANSFORMATION (temperature_transformation_new ((float) parameter));
			return result;
		}
		default:
		{
#line 450 "ColorTransformation.vala"
			g_error ("ColorTransformation.vala:450: unrecognized ColorTransformationKind enumeration value");
#line 451 "ColorTransformation.vala"
			break;
#line 1494 "ColorTransformation.c"
		}
	}
	result = color_transformation_new ();
	return result;
}


static void value_color_transformation_factory_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_color_transformation_factory_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		color_transformation_factory_unref (value->data[0].v_pointer);
	}
}


static void value_color_transformation_factory_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = color_transformation_factory_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_color_transformation_factory_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_color_transformation_factory_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ColorTransformationFactory* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = color_transformation_factory_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_color_transformation_factory_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ColorTransformationFactory** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = color_transformation_factory_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_color_transformation_factory (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecColorTransformationFactory* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_COLOR_TRANSFORMATION_FACTORY), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_color_transformation_factory (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_COLOR_TRANSFORMATION_FACTORY), NULL);
	return value->data[0].v_pointer;
}


void value_set_color_transformation_factory (GValue* value, gpointer v_object) {
	ColorTransformationFactory* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_COLOR_TRANSFORMATION_FACTORY));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_COLOR_TRANSFORMATION_FACTORY));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		color_transformation_factory_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		color_transformation_factory_unref (old);
	}
}


static void color_transformation_factory_class_init (ColorTransformationFactoryClass * klass) {
	color_transformation_factory_parent_class = g_type_class_peek_parent (klass);
	COLOR_TRANSFORMATION_FACTORY_CLASS (klass)->finalize = color_transformation_factory_finalize;
}


static void color_transformation_factory_instance_init (ColorTransformationFactory * self) {
	self->ref_count = 1;
}


static void color_transformation_factory_finalize (ColorTransformationFactory* obj) {
	ColorTransformationFactory * self;
	self = COLOR_TRANSFORMATION_FACTORY (obj);
}


GType color_transformation_factory_get_type (void) {
	static GType color_transformation_factory_type_id = 0;
	if (color_transformation_factory_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_color_transformation_factory_init, value_color_transformation_factory_free_value, value_color_transformation_factory_copy_value, value_color_transformation_factory_peek_pointer, "p", value_color_transformation_factory_collect_value, "p", value_color_transformation_factory_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ColorTransformationFactoryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) color_transformation_factory_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ColorTransformationFactory), 0, (GInstanceInitFunc) color_transformation_factory_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		color_transformation_factory_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ColorTransformationFactory", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return color_transformation_factory_type_id;
}


gpointer color_transformation_factory_ref (gpointer instance) {
	ColorTransformationFactory* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void color_transformation_factory_unref (gpointer instance) {
	ColorTransformationFactory* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		COLOR_TRANSFORMATION_FACTORY_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 474 "ColorTransformation.vala"
ImageHistogram* image_histogram_construct (GType object_type, GdkPixbuf* pixbuf) {
#line 1644 "ColorTransformation.c"
	ImageHistogram* self;
	gint sample_bytes;
	gint pixel_bytes;
	gint row_length_bytes;
	guchar* _tmp1_;
	gint pixel_data_size;
	gint pixel_data_length1;
	guchar* pixel_data;
#line 474 "ColorTransformation.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 1655 "ColorTransformation.c"
	self = (ImageHistogram*) g_type_create_instance (object_type);
	{
		gint i;
		i = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 475 "ColorTransformation.vala"
			while (TRUE) {
#line 475 "ColorTransformation.vala"
				if (!_tmp0_) {
#line 475 "ColorTransformation.vala"
					i++;
#line 1669 "ColorTransformation.c"
				}
#line 475 "ColorTransformation.vala"
				_tmp0_ = FALSE;
#line 475 "ColorTransformation.vala"
				if (!(i < 256)) {
#line 475 "ColorTransformation.vala"
					break;
#line 1677 "ColorTransformation.c"
				}
#line 476 "ColorTransformation.vala"
				self->priv->red_counts[i] = self->priv->green_counts[i] = self->priv->blue_counts[i] = 0;
#line 1681 "ColorTransformation.c"
			}
		}
	}
	sample_bytes = gdk_pixbuf_get_bits_per_sample (pixbuf) / 8;
	pixel_bytes = sample_bytes * gdk_pixbuf_get_n_channels (pixbuf);
	row_length_bytes = pixel_bytes * gdk_pixbuf_get_width (pixbuf);
#line 482 "ColorTransformation.vala"
	_tmp1_ = NULL;
#line 1690 "ColorTransformation.c"
	pixel_data = (_tmp1_ = gdk_pixbuf_get_pixels (pixbuf), pixel_data_length1 = -1, pixel_data_size = pixel_data_length1, _tmp1_);
	{
		gint y;
		y = 0;
		{
			gboolean _tmp2_;
			_tmp2_ = TRUE;
#line 484 "ColorTransformation.vala"
			while (TRUE) {
#line 1700 "ColorTransformation.c"
				gint row_start_offset;
				gint r_offset;
				gint g_offset;
				gint b_offset;
#line 484 "ColorTransformation.vala"
				if (!_tmp2_) {
#line 484 "ColorTransformation.vala"
					y++;
#line 1709 "ColorTransformation.c"
				}
#line 484 "ColorTransformation.vala"
				_tmp2_ = FALSE;
#line 484 "ColorTransformation.vala"
				if (!(y < gdk_pixbuf_get_height (pixbuf))) {
#line 484 "ColorTransformation.vala"
					break;
#line 1717 "ColorTransformation.c"
				}
				row_start_offset = y * gdk_pixbuf_get_rowstride (pixbuf);
				r_offset = row_start_offset;
				g_offset = row_start_offset + sample_bytes;
				b_offset = (row_start_offset + sample_bytes) + sample_bytes;
#line 491 "ColorTransformation.vala"
				while (TRUE) {
#line 491 "ColorTransformation.vala"
					if (!(b_offset < (row_start_offset + row_length_bytes))) {
#line 491 "ColorTransformation.vala"
						break;
#line 1729 "ColorTransformation.c"
					}
#line 492 "ColorTransformation.vala"
					self->priv->red_counts[pixel_data[r_offset]]++;
#line 493 "ColorTransformation.vala"
					self->priv->green_counts[pixel_data[g_offset]]++;
#line 494 "ColorTransformation.vala"
					self->priv->blue_counts[pixel_data[b_offset]]++;
#line 496 "ColorTransformation.vala"
					r_offset = r_offset + pixel_bytes;
#line 497 "ColorTransformation.vala"
					g_offset = g_offset + pixel_bytes;
#line 498 "ColorTransformation.vala"
					b_offset = b_offset + pixel_bytes;
#line 1743 "ColorTransformation.c"
				}
			}
		}
	}
	return self;
}


#line 474 "ColorTransformation.vala"
ImageHistogram* image_histogram_new (GdkPixbuf* pixbuf) {
#line 474 "ColorTransformation.vala"
	return image_histogram_construct (TYPE_IMAGE_HISTOGRAM, pixbuf);
#line 1756 "ColorTransformation.c"
}


#line 503 "ColorTransformation.vala"
static gint image_histogram_correct_snap_to_quantization (ImageHistogram* self, gint* buckets, int buckets_length1, gint i) {
#line 1762 "ColorTransformation.c"
	gint result;
	gboolean _tmp0_;
#line 503 "ColorTransformation.vala"
	g_return_val_if_fail (IS_IMAGE_HISTOGRAM (self), 0);
#line 504 "ColorTransformation.vala"
	g_assert (buckets_length1 == 256);
#line 1769 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 505 "ColorTransformation.vala"
	if (i >= 0) {
#line 505 "ColorTransformation.vala"
		_tmp0_ = i <= 255;
#line 1775 "ColorTransformation.c"
	} else {
#line 505 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 1779 "ColorTransformation.c"
	}
#line 505 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 507 "ColorTransformation.vala"
	if (i == 0) {
#line 508 "ColorTransformation.vala"
		if (buckets[i] > 0) {
#line 509 "ColorTransformation.vala"
			if (buckets[i + 1] > 0) {
#line 510 "ColorTransformation.vala"
				if (buckets[i] > (2 * buckets[i + 1])) {
#line 1791 "ColorTransformation.c"
					result = buckets[i + 1];
					buckets = (g_free (buckets), NULL);
					return result;
				}
			}
		}
	} else {
#line 512 "ColorTransformation.vala"
		if (i == 255) {
#line 513 "ColorTransformation.vala"
			if (buckets[i] > 0) {
#line 514 "ColorTransformation.vala"
				if (buckets[i - 1] > 0) {
#line 515 "ColorTransformation.vala"
					if (buckets[i] > (2 * buckets[i - 1])) {
#line 1807 "ColorTransformation.c"
						result = buckets[i - 1];
						buckets = (g_free (buckets), NULL);
						return result;
					}
				}
			}
		} else {
#line 518 "ColorTransformation.vala"
			if (buckets[i] > 0) {
#line 519 "ColorTransformation.vala"
				if (buckets[i] > ((buckets[i - 1] + buckets[i + 1]) / 2)) {
#line 1819 "ColorTransformation.c"
					result = (buckets[i - 1] + buckets[i + 1]) / 2;
					buckets = (g_free (buckets), NULL);
					return result;
				}
			}
		}
	}
	result = buckets[i];
	buckets = (g_free (buckets), NULL);
	return result;
}


#line 526 "ColorTransformation.vala"
static gint image_histogram_correct_snap_from_quantization (ImageHistogram* self, gint* buckets, int buckets_length1, gint i) {
#line 1835 "ColorTransformation.c"
	gint result;
	gboolean _tmp0_;
#line 526 "ColorTransformation.vala"
	g_return_val_if_fail (IS_IMAGE_HISTOGRAM (self), 0);
#line 527 "ColorTransformation.vala"
	g_assert (buckets_length1 == 256);
#line 1842 "ColorTransformation.c"
	_tmp0_ = FALSE;
#line 528 "ColorTransformation.vala"
	if (i >= 0) {
#line 528 "ColorTransformation.vala"
		_tmp0_ = i <= 255;
#line 1848 "ColorTransformation.c"
	} else {
#line 528 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 1852 "ColorTransformation.c"
	}
#line 528 "ColorTransformation.vala"
	g_assert (_tmp0_);
#line 530 "ColorTransformation.vala"
	if (i == 0) {
#line 1858 "ColorTransformation.c"
		result = buckets[i];
		buckets = (g_free (buckets), NULL);
		return result;
	} else {
#line 532 "ColorTransformation.vala"
		if (i == 255) {
#line 1865 "ColorTransformation.c"
			result = buckets[i];
			buckets = (g_free (buckets), NULL);
			return result;
		} else {
#line 535 "ColorTransformation.vala"
			if (buckets[i] == 0) {
#line 536 "ColorTransformation.vala"
				if (buckets[i - 1] > 0) {
#line 537 "ColorTransformation.vala"
					if (buckets[i + 1] > 0) {
#line 1876 "ColorTransformation.c"
						result = (buckets[i - 1] + buckets[i + 1]) / 2;
						buckets = (g_free (buckets), NULL);
						return result;
					}
				}
			}
		}
	}
	result = buckets[i];
	buckets = (g_free (buckets), NULL);
	return result;
}


#line 544 "ColorTransformation.vala"
static void image_histogram_smooth_extrema (ImageHistogram* self, gint* count_data, int count_data_length1) {
#line 544 "ColorTransformation.vala"
	g_return_if_fail (IS_IMAGE_HISTOGRAM (self));
#line 545 "ColorTransformation.vala"
	g_assert (count_data_length1 == 256);
#line 1897 "ColorTransformation.c"
	/* the blocks of code below are unrolled loops that replace values at the extrema
	           (buckets 0-4 and 251-255, inclusive) of the histogram with a weighted
	           average of their neighbors. This mitigates quantization and pooling artifacts */
#line 551 "ColorTransformation.vala"
	count_data[0] = (((5 * count_data[0]) + (3 * count_data[1])) + (2 * count_data[2])) / 10;
#line 553 "ColorTransformation.vala"
	count_data[1] = ((((3 * count_data[0]) + (5 * count_data[1])) + (3 * count_data[2])) + (2 * count_data[3])) / 13;
#line 555 "ColorTransformation.vala"
	count_data[2] = (((((2 * count_data[0]) + (3 * count_data[1])) + (5 * count_data[2])) + (3 * count_data[3])) + (2 * count_data[4])) / 15;
#line 557 "ColorTransformation.vala"
	count_data[3] = (((((2 * count_data[1]) + (3 * count_data[2])) + (5 * count_data[3])) + (3 * count_data[4])) + (2 * count_data[5])) / 15;
#line 559 "ColorTransformation.vala"
	count_data[4] = (((((2 * count_data[2]) + (3 * count_data[3])) + (5 * count_data[4])) + (3 * count_data[5])) + (2 * count_data[6])) / 15;
#line 562 "ColorTransformation.vala"
	count_data[255] = (((5 * count_data[255]) + (3 * count_data[254])) + (2 * count_data[253])) / 10;
#line 564 "ColorTransformation.vala"
	count_data[254] = ((((3 * count_data[255]) + (5 * count_data[254])) + (3 * count_data[253])) + (2 * count_data[252])) / 13;
#line 566 "ColorTransformation.vala"
	count_data[253] = (((((2 * count_data[255]) + (3 * count_data[254])) + (5 * count_data[253])) + (3 * count_data[252])) + (2 * count_data[251])) / 15;
#line 568 "ColorTransformation.vala"
	count_data[252] = (((((2 * count_data[254]) + (3 * count_data[253])) + (5 * count_data[252])) + (3 * count_data[251])) + (2 * count_data[250])) / 15;
#line 570 "ColorTransformation.vala"
	count_data[251] = (((((2 * count_data[253]) + (3 * count_data[252])) + (5 * count_data[251])) + (3 * count_data[250])) + (2 * count_data[249])) / 15;
#line 1921 "ColorTransformation.c"
	count_data = (g_free (count_data), NULL);
}


static gint* _vala_array_dup3 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup4 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup5 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup6 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup7 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup8 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup9 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup10 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


static gint* _vala_array_dup11 (gint* self, int length) {
	return g_memdup (self, length * sizeof (gint));
}


#line 574 "ColorTransformation.vala"
static void image_histogram_prepare_qualitative_counts (ImageHistogram* self) {
#line 1973 "ColorTransformation.c"
	gboolean _tmp0_;
	gboolean _tmp1_;
	gint* _tmp2_;
	gint* _tmp3_;
	gint* _tmp4_;
	gint* _tmp5_;
	gint temp_red_counts_size;
	gint temp_red_counts_length1;
	gint* temp_red_counts;
	gint* _tmp6_;
	gint temp_green_counts_size;
	gint temp_green_counts_length1;
	gint* temp_green_counts;
	gint* _tmp7_;
	gint temp_blue_counts_size;
	gint temp_blue_counts_length1;
	gint* temp_blue_counts;
	gint mean_qual_count;
	gint constrained_max_qual_count;
	gint* _tmp19_;
	gint* _tmp20_;
	gint* _tmp21_;
#line 574 "ColorTransformation.vala"
	g_return_if_fail (IS_IMAGE_HISTOGRAM (self));
#line 1998 "ColorTransformation.c"
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
#line 575 "ColorTransformation.vala"
	if (self->priv->qualitative_red_counts != NULL) {
#line 575 "ColorTransformation.vala"
		_tmp1_ = self->priv->qualitative_green_counts != NULL;
#line 2005 "ColorTransformation.c"
	} else {
#line 575 "ColorTransformation.vala"
		_tmp1_ = FALSE;
#line 2009 "ColorTransformation.c"
	}
#line 575 "ColorTransformation.vala"
	if (_tmp1_) {
#line 576 "ColorTransformation.vala"
		_tmp0_ = self->priv->qualitative_blue_counts != NULL;
#line 2015 "ColorTransformation.c"
	} else {
#line 575 "ColorTransformation.vala"
		_tmp0_ = FALSE;
#line 2019 "ColorTransformation.c"
	}
#line 575 "ColorTransformation.vala"
	if (_tmp0_) {
#line 2023 "ColorTransformation.c"
		return;
	}
	_tmp2_ = NULL;
#line 579 "ColorTransformation.vala"
	self->priv->qualitative_red_counts = (_tmp2_ = g_new0 (gint, 256), self->priv->qualitative_red_counts = (g_free (self->priv->qualitative_red_counts), NULL), self->priv->qualitative_red_counts_length1 = 256, self->priv->qualitative_red_counts_size = self->priv->qualitative_red_counts_length1, _tmp2_);
#line 2029 "ColorTransformation.c"
	_tmp3_ = NULL;
#line 580 "ColorTransformation.vala"
	self->priv->qualitative_green_counts = (_tmp3_ = g_new0 (gint, 256), self->priv->qualitative_green_counts = (g_free (self->priv->qualitative_green_counts), NULL), self->priv->qualitative_green_counts_length1 = 256, self->priv->qualitative_green_counts_size = self->priv->qualitative_green_counts_length1, _tmp3_);
#line 2033 "ColorTransformation.c"
	_tmp4_ = NULL;
#line 581 "ColorTransformation.vala"
	self->priv->qualitative_blue_counts = (_tmp4_ = g_new0 (gint, 256), self->priv->qualitative_blue_counts = (g_free (self->priv->qualitative_blue_counts), NULL), self->priv->qualitative_blue_counts_length1 = 256, self->priv->qualitative_blue_counts_size = self->priv->qualitative_blue_counts_length1, _tmp4_);
#line 583 "ColorTransformation.vala"
	_tmp5_ = NULL;
#line 2039 "ColorTransformation.c"
	temp_red_counts = (_tmp5_ = g_new0 (gint, 256), temp_red_counts_length1 = 256, temp_red_counts_size = temp_red_counts_length1, _tmp5_);
#line 584 "ColorTransformation.vala"
	_tmp6_ = NULL;
#line 2043 "ColorTransformation.c"
	temp_green_counts = (_tmp6_ = g_new0 (gint, 256), temp_green_counts_length1 = 256, temp_green_counts_size = temp_green_counts_length1, _tmp6_);
#line 585 "ColorTransformation.vala"
	_tmp7_ = NULL;
#line 2047 "ColorTransformation.c"
	temp_blue_counts = (_tmp7_ = g_new0 (gint, 256), temp_blue_counts_length1 = 256, temp_blue_counts_size = temp_blue_counts_length1, _tmp7_);
	{
		gint i;
		/* Remove snap-away-from-value quantization artifacts from the qualitative
		           histogram. While these are indeed present in the underlying data as a
		           consequence of sampling, transformation, and reconstruction, they lead
		           to an unpleasant looking histogram, so we detect and eliminate them here */
		i = 0;
		{
			gboolean _tmp8_;
			_tmp8_ = TRUE;
#line 591 "ColorTransformation.vala"
			while (TRUE) {
#line 2061 "ColorTransformation.c"
				gint* _tmp9_;
				gint* _tmp10_;
				gint* _tmp11_;
#line 591 "ColorTransformation.vala"
				if (!_tmp8_) {
#line 591 "ColorTransformation.vala"
					i++;
#line 2069 "ColorTransformation.c"
				}
#line 591 "ColorTransformation.vala"
				_tmp8_ = FALSE;
#line 591 "ColorTransformation.vala"
				if (!(i < 256)) {
#line 591 "ColorTransformation.vala"
					break;
#line 2077 "ColorTransformation.c"
				}
#line 593 "ColorTransformation.vala"
				_tmp9_ = NULL;
#line 592 "ColorTransformation.vala"
				self->priv->qualitative_red_counts[i] = image_histogram_correct_snap_from_quantization (self, (_tmp9_ = self->priv->red_counts, (_tmp9_ == NULL) ? ((gpointer) _tmp9_) : _vala_array_dup3 (_tmp9_, self->priv->red_counts_length1)), self->priv->red_counts_length1, i);
#line 595 "ColorTransformation.vala"
				_tmp10_ = NULL;
#line 594 "ColorTransformation.vala"
				self->priv->qualitative_green_counts[i] = image_histogram_correct_snap_from_quantization (self, (_tmp10_ = self->priv->green_counts, (_tmp10_ == NULL) ? ((gpointer) _tmp10_) : _vala_array_dup4 (_tmp10_, self->priv->green_counts_length1)), self->priv->green_counts_length1, i);
#line 597 "ColorTransformation.vala"
				_tmp11_ = NULL;
#line 596 "ColorTransformation.vala"
				self->priv->qualitative_blue_counts[i] = image_histogram_correct_snap_from_quantization (self, (_tmp11_ = self->priv->blue_counts, (_tmp11_ == NULL) ? ((gpointer) _tmp11_) : _vala_array_dup5 (_tmp11_, self->priv->blue_counts_length1)), self->priv->blue_counts_length1, i);
#line 2091 "ColorTransformation.c"
			}
		}
	}
	{
		gint i;
		i = 0;
		{
			gboolean _tmp12_;
			_tmp12_ = TRUE;
#line 600 "ColorTransformation.vala"
			while (TRUE) {
#line 600 "ColorTransformation.vala"
				if (!_tmp12_) {
#line 600 "ColorTransformation.vala"
					i++;
#line 2107 "ColorTransformation.c"
				}
#line 600 "ColorTransformation.vala"
				_tmp12_ = FALSE;
#line 600 "ColorTransformation.vala"
				if (!(i < 256)) {
#line 600 "ColorTransformation.vala"
					break;
#line 2115 "ColorTransformation.c"
				}
#line 601 "ColorTransformation.vala"
				temp_red_counts[i] = self->priv->qualitative_red_counts[i];
#line 602 "ColorTransformation.vala"
				temp_green_counts[i] = self->priv->qualitative_green_counts[i];
#line 603 "ColorTransformation.vala"
				temp_blue_counts[i] = self->priv->qualitative_blue_counts[i];
#line 2123 "ColorTransformation.c"
			}
		}
	}
	{
		gint i;
		/* Remove snap-to-value quantization artifacts from the qualitative
		           histogram */
		i = 0;
		{
			gboolean _tmp13_;
			_tmp13_ = TRUE;
#line 608 "ColorTransformation.vala"
			while (TRUE) {
#line 2137 "ColorTransformation.c"
				gint* _tmp14_;
				gint* _tmp15_;
				gint* _tmp16_;
#line 608 "ColorTransformation.vala"
				if (!_tmp13_) {
#line 608 "ColorTransformation.vala"
					i++;
#line 2145 "ColorTransformation.c"
				}
#line 608 "ColorTransformation.vala"
				_tmp13_ = FALSE;
#line 608 "ColorTransformation.vala"
				if (!(i < 256)) {
#line 608 "ColorTransformation.vala"
					break;
#line 2153 "ColorTransformation.c"
				}
#line 610 "ColorTransformation.vala"
				_tmp14_ = NULL;
#line 609 "ColorTransformation.vala"
				self->priv->qualitative_red_counts[i] = image_histogram_correct_snap_to_quantization (self, (_tmp14_ = temp_red_counts, (_tmp14_ == NULL) ? ((gpointer) _tmp14_) : _vala_array_dup6 (_tmp14_, temp_red_counts_length1)), temp_red_counts_length1, i);
#line 612 "ColorTransformation.vala"
				_tmp15_ = NULL;
#line 611 "ColorTransformation.vala"
				self->priv->qualitative_green_counts[i] = image_histogram_correct_snap_to_quantization (self, (_tmp15_ = temp_green_counts, (_tmp15_ == NULL) ? ((gpointer) _tmp15_) : _vala_array_dup7 (_tmp15_, temp_green_counts_length1)), temp_green_counts_length1, i);
#line 614 "ColorTransformation.vala"
				_tmp16_ = NULL;
#line 613 "ColorTransformation.vala"
				self->priv->qualitative_blue_counts[i] = image_histogram_correct_snap_to_quantization (self, (_tmp16_ = temp_blue_counts, (_tmp16_ == NULL) ? ((gpointer) _tmp16_) : _vala_array_dup8 (_tmp16_, temp_blue_counts_length1)), temp_blue_counts_length1, i);
#line 2167 "ColorTransformation.c"
			}
		}
	}
	/* constrain the peaks in the qualitative histogram so that no peak can be more
	           than 8 times higher than the mean height of the entire image */
	mean_qual_count = 0;
	{
		gint i;
		i = 0;
		{
			gboolean _tmp17_;
			_tmp17_ = TRUE;
#line 620 "ColorTransformation.vala"
			while (TRUE) {
#line 620 "ColorTransformation.vala"
				if (!_tmp17_) {
#line 620 "ColorTransformation.vala"
					i++;
#line 2186 "ColorTransformation.c"
				}
#line 620 "ColorTransformation.vala"
				_tmp17_ = FALSE;
#line 620 "ColorTransformation.vala"
				if (!(i < 256)) {
#line 620 "ColorTransformation.vala"
					break;
#line 2194 "ColorTransformation.c"
				}
#line 621 "ColorTransformation.vala"
				mean_qual_count = mean_qual_count + ((self->priv->qualitative_red_counts[i] + self->priv->qualitative_green_counts[i]) + self->priv->qualitative_blue_counts[i]);
#line 2198 "ColorTransformation.c"
			}
		}
	}
#line 624 "ColorTransformation.vala"
	mean_qual_count = mean_qual_count / (256 * 3);
#line 2204 "ColorTransformation.c"
	constrained_max_qual_count = 8 * mean_qual_count;
	{
		gint i;
		i = 0;
		{
			gboolean _tmp18_;
			_tmp18_ = TRUE;
#line 626 "ColorTransformation.vala"
			while (TRUE) {
#line 626 "ColorTransformation.vala"
				if (!_tmp18_) {
#line 626 "ColorTransformation.vala"
					i++;
#line 2218 "ColorTransformation.c"
				}
#line 626 "ColorTransformation.vala"
				_tmp18_ = FALSE;
#line 626 "ColorTransformation.vala"
				if (!(i < 256)) {
#line 626 "ColorTransformation.vala"
					break;
#line 2226 "ColorTransformation.c"
				}
#line 627 "ColorTransformation.vala"
				if (self->priv->qualitative_red_counts[i] > constrained_max_qual_count) {
#line 628 "ColorTransformation.vala"
					self->priv->qualitative_red_counts[i] = constrained_max_qual_count;
#line 2232 "ColorTransformation.c"
				}
#line 630 "ColorTransformation.vala"
				if (self->priv->qualitative_green_counts[i] > constrained_max_qual_count) {
#line 631 "ColorTransformation.vala"
					self->priv->qualitative_green_counts[i] = constrained_max_qual_count;
#line 2238 "ColorTransformation.c"
				}
#line 633 "ColorTransformation.vala"
				if (self->priv->qualitative_blue_counts[i] > constrained_max_qual_count) {
#line 634 "ColorTransformation.vala"
					self->priv->qualitative_blue_counts[i] = constrained_max_qual_count;
#line 2244 "ColorTransformation.c"
				}
			}
		}
	}
#line 637 "ColorTransformation.vala"
	_tmp19_ = NULL;
#line 637 "ColorTransformation.vala"
	image_histogram_smooth_extrema (self, (_tmp19_ = self->priv->qualitative_red_counts, (_tmp19_ == NULL) ? ((gpointer) _tmp19_) : _vala_array_dup9 (_tmp19_, self->priv->qualitative_red_counts_length1)), self->priv->qualitative_red_counts_length1);
#line 638 "ColorTransformation.vala"
	_tmp20_ = NULL;
#line 638 "ColorTransformation.vala"
	image_histogram_smooth_extrema (self, (_tmp20_ = self->priv->qualitative_green_counts, (_tmp20_ == NULL) ? ((gpointer) _tmp20_) : _vala_array_dup10 (_tmp20_, self->priv->qualitative_green_counts_length1)), self->priv->qualitative_green_counts_length1);
#line 639 "ColorTransformation.vala"
	_tmp21_ = NULL;
#line 639 "ColorTransformation.vala"
	image_histogram_smooth_extrema (self, (_tmp21_ = self->priv->qualitative_blue_counts, (_tmp21_ == NULL) ? ((gpointer) _tmp21_) : _vala_array_dup11 (_tmp21_, self->priv->qualitative_blue_counts_length1)), self->priv->qualitative_blue_counts_length1);
#line 2261 "ColorTransformation.c"
	temp_red_counts = (g_free (temp_red_counts), NULL);
	temp_green_counts = (g_free (temp_green_counts), NULL);
	temp_blue_counts = (g_free (temp_blue_counts), NULL);
}


#line 642 "ColorTransformation.vala"
GdkPixbuf* image_histogram_get_graphic (ImageHistogram* self) {
#line 2270 "ColorTransformation.c"
	GdkPixbuf* result;
	GdkPixbuf* _tmp6_;
#line 642 "ColorTransformation.vala"
	g_return_val_if_fail (IS_IMAGE_HISTOGRAM (self), NULL);
#line 643 "ColorTransformation.vala"
	if (self->priv->graphic == NULL) {
#line 2277 "ColorTransformation.c"
		gint max_count;
		GdkPixbuf* _tmp1_;
		gint rowstride;
		gint sample_bytes;
		gint pixel_bytes;
		double scale_bar;
		guchar* _tmp2_;
		gint pixel_data_size;
		gint pixel_data_length1;
		guchar* pixel_data;
#line 644 "ColorTransformation.vala"
		image_histogram_prepare_qualitative_counts (self);
#line 2290 "ColorTransformation.c"
		max_count = 0;
		{
			gint i;
			i = 0;
			{
				gboolean _tmp0_;
				_tmp0_ = TRUE;
#line 646 "ColorTransformation.vala"
				while (TRUE) {
#line 646 "ColorTransformation.vala"
					if (!_tmp0_) {
#line 646 "ColorTransformation.vala"
						i++;
#line 2304 "ColorTransformation.c"
					}
#line 646 "ColorTransformation.vala"
					_tmp0_ = FALSE;
#line 646 "ColorTransformation.vala"
					if (!(i < 256)) {
#line 646 "ColorTransformation.vala"
						break;
#line 2312 "ColorTransformation.c"
					}
#line 647 "ColorTransformation.vala"
					if (self->priv->qualitative_red_counts[i] > max_count) {
#line 648 "ColorTransformation.vala"
						max_count = self->priv->qualitative_red_counts[i];
#line 2318 "ColorTransformation.c"
					}
#line 649 "ColorTransformation.vala"
					if (self->priv->qualitative_green_counts[i] > max_count) {
#line 650 "ColorTransformation.vala"
						max_count = self->priv->qualitative_green_counts[i];
#line 2324 "ColorTransformation.c"
					}
#line 651 "ColorTransformation.vala"
					if (self->priv->qualitative_blue_counts[i] > max_count) {
#line 652 "ColorTransformation.vala"
						max_count = self->priv->qualitative_blue_counts[i];
#line 2330 "ColorTransformation.c"
					}
				}
			}
		}
		_tmp1_ = NULL;
#line 655 "ColorTransformation.vala"
		self->priv->graphic = (_tmp1_ = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, IMAGE_HISTOGRAM_GRAPHIC_WIDTH, IMAGE_HISTOGRAM_GRAPHIC_HEIGHT), (self->priv->graphic == NULL) ? NULL : (self->priv->graphic = (g_object_unref (self->priv->graphic), NULL)), _tmp1_);
#line 2338 "ColorTransformation.c"
		rowstride = gdk_pixbuf_get_rowstride (self->priv->graphic);
		sample_bytes = gdk_pixbuf_get_bits_per_sample (self->priv->graphic) / 8;
		pixel_bytes = sample_bytes * gdk_pixbuf_get_n_channels (self->priv->graphic);
		scale_bar = (0.98 * ((double) IMAGE_HISTOGRAM_GRAPHIC_HEIGHT)) / ((double) max_count);
#line 665 "ColorTransformation.vala"
		_tmp2_ = NULL;
#line 2345 "ColorTransformation.c"
		pixel_data = (_tmp2_ = gdk_pixbuf_get_pixels (self->priv->graphic), pixel_data_length1 = -1, pixel_data_size = pixel_data_length1, _tmp2_);
		{
			gint x;
			x = 0;
			{
				gboolean _tmp3_;
				_tmp3_ = TRUE;
#line 667 "ColorTransformation.vala"
				while (TRUE) {
#line 2355 "ColorTransformation.c"
					gint red_bar_height;
					gint green_bar_height;
					gint blue_bar_height;
					gint max_bar_height;
					gint y;
					gint pixel_index;
#line 667 "ColorTransformation.vala"
					if (!_tmp3_) {
#line 667 "ColorTransformation.vala"
						x++;
#line 2366 "ColorTransformation.c"
					}
#line 667 "ColorTransformation.vala"
					_tmp3_ = FALSE;
#line 667 "ColorTransformation.vala"
					if (!(x < 256)) {
#line 667 "ColorTransformation.vala"
						break;
#line 2374 "ColorTransformation.c"
					}
					red_bar_height = (gint) (((double) self->priv->qualitative_red_counts[x]) * scale_bar);
					green_bar_height = (gint) (((double) self->priv->qualitative_green_counts[x]) * scale_bar);
					blue_bar_height = (gint) (((double) self->priv->qualitative_blue_counts[x]) * scale_bar);
					max_bar_height = MAX (MAX (red_bar_height, green_bar_height), blue_bar_height);
					y = IMAGE_HISTOGRAM_GRAPHIC_HEIGHT - 1;
					pixel_index = (x * pixel_bytes) + (y * rowstride);
					{
						gboolean _tmp4_;
						_tmp4_ = TRUE;
#line 680 "ColorTransformation.vala"
						while (TRUE) {
#line 680 "ColorTransformation.vala"
							if (!_tmp4_) {
#line 680 "ColorTransformation.vala"
								y--;
#line 2391 "ColorTransformation.c"
							}
#line 680 "ColorTransformation.vala"
							_tmp4_ = FALSE;
#line 680 "ColorTransformation.vala"
							if (!(y >= (IMAGE_HISTOGRAM_GRAPHIC_HEIGHT - max_bar_height))) {
#line 680 "ColorTransformation.vala"
								break;
#line 2399 "ColorTransformation.c"
							}
#line 681 "ColorTransformation.vala"
							pixel_data[pixel_index] = IMAGE_HISTOGRAM_MARKED_BACKGROUND;
#line 682 "ColorTransformation.vala"
							pixel_data[pixel_index + 1] = IMAGE_HISTOGRAM_MARKED_BACKGROUND;
#line 683 "ColorTransformation.vala"
							pixel_data[pixel_index + 2] = IMAGE_HISTOGRAM_MARKED_BACKGROUND;
#line 685 "ColorTransformation.vala"
							if (y >= ((IMAGE_HISTOGRAM_GRAPHIC_HEIGHT - red_bar_height) - 1)) {
#line 686 "ColorTransformation.vala"
								pixel_data[pixel_index] = IMAGE_HISTOGRAM_MARKED_FOREGROUND;
#line 2411 "ColorTransformation.c"
							}
#line 687 "ColorTransformation.vala"
							if (y >= ((IMAGE_HISTOGRAM_GRAPHIC_HEIGHT - green_bar_height) - 1)) {
#line 688 "ColorTransformation.vala"
								pixel_data[pixel_index + 1] = IMAGE_HISTOGRAM_MARKED_FOREGROUND;
#line 2417 "ColorTransformation.c"
							}
#line 689 "ColorTransformation.vala"
							if (y >= ((IMAGE_HISTOGRAM_GRAPHIC_HEIGHT - blue_bar_height) - 1)) {
#line 690 "ColorTransformation.vala"
								pixel_data[pixel_index + 2] = IMAGE_HISTOGRAM_MARKED_FOREGROUND;
#line 2423 "ColorTransformation.c"
							}
#line 692 "ColorTransformation.vala"
							pixel_index = pixel_index - rowstride;
#line 2427 "ColorTransformation.c"
						}
					}
					{
						gboolean _tmp5_;
						_tmp5_ = TRUE;
#line 695 "ColorTransformation.vala"
						while (TRUE) {
#line 695 "ColorTransformation.vala"
							if (!_tmp5_) {
#line 695 "ColorTransformation.vala"
								y--;
#line 2439 "ColorTransformation.c"
							}
#line 695 "ColorTransformation.vala"
							_tmp5_ = FALSE;
#line 695 "ColorTransformation.vala"
							if (!(y >= 0)) {
#line 695 "ColorTransformation.vala"
								break;
#line 2447 "ColorTransformation.c"
							}
#line 696 "ColorTransformation.vala"
							pixel_data[pixel_index] = IMAGE_HISTOGRAM_UNMARKED_BACKGROUND;
#line 697 "ColorTransformation.vala"
							pixel_data[pixel_index + 1] = IMAGE_HISTOGRAM_UNMARKED_BACKGROUND;
#line 698 "ColorTransformation.vala"
							pixel_data[pixel_index + 2] = IMAGE_HISTOGRAM_UNMARKED_BACKGROUND;
#line 700 "ColorTransformation.vala"
							pixel_index = pixel_index - rowstride;
#line 2457 "ColorTransformation.c"
						}
					}
				}
			}
		}
	}
#line 705 "ColorTransformation.vala"
	_tmp6_ = NULL;
#line 2466 "ColorTransformation.c"
	result = (_tmp6_ = self->priv->graphic, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_));
	return result;
}


static void value_image_histogram_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_image_histogram_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		image_histogram_unref (value->data[0].v_pointer);
	}
}


static void value_image_histogram_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = image_histogram_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_image_histogram_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_image_histogram_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ImageHistogram* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = image_histogram_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_image_histogram_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ImageHistogram** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = image_histogram_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_image_histogram (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecImageHistogram* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_IMAGE_HISTOGRAM), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_image_histogram (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_IMAGE_HISTOGRAM), NULL);
	return value->data[0].v_pointer;
}


void value_set_image_histogram (GValue* value, gpointer v_object) {
	ImageHistogram* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_IMAGE_HISTOGRAM));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_IMAGE_HISTOGRAM));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		image_histogram_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		image_histogram_unref (old);
	}
}


static void image_histogram_class_init (ImageHistogramClass * klass) {
	image_histogram_parent_class = g_type_class_peek_parent (klass);
	IMAGE_HISTOGRAM_CLASS (klass)->finalize = image_histogram_finalize;
	g_type_class_add_private (klass, sizeof (ImageHistogramPrivate));
}


static void image_histogram_instance_init (ImageHistogram * self) {
	self->priv = IMAGE_HISTOGRAM_GET_PRIVATE (self);
	self->priv->red_counts = g_new0 (gint, 256);
	self->priv->red_counts_length1 = 256;
	self->priv->green_counts = g_new0 (gint, 256);
	self->priv->green_counts_length1 = 256;
	self->priv->blue_counts = g_new0 (gint, 256);
	self->priv->blue_counts_length1 = 256;
	self->priv->qualitative_red_counts = NULL;
	self->priv->qualitative_green_counts = NULL;
	self->priv->qualitative_blue_counts = NULL;
	self->priv->graphic = NULL;
	self->ref_count = 1;
}


static void image_histogram_finalize (ImageHistogram* obj) {
	ImageHistogram * self;
	self = IMAGE_HISTOGRAM (obj);
	self->priv->red_counts = (g_free (self->priv->red_counts), NULL);
	self->priv->green_counts = (g_free (self->priv->green_counts), NULL);
	self->priv->blue_counts = (g_free (self->priv->blue_counts), NULL);
	self->priv->qualitative_red_counts = (g_free (self->priv->qualitative_red_counts), NULL);
	self->priv->qualitative_green_counts = (g_free (self->priv->qualitative_green_counts), NULL);
	self->priv->qualitative_blue_counts = (g_free (self->priv->qualitative_blue_counts), NULL);
	(self->priv->graphic == NULL) ? NULL : (self->priv->graphic = (g_object_unref (self->priv->graphic), NULL));
}


GType image_histogram_get_type (void) {
	static GType image_histogram_type_id = 0;
	if (image_histogram_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_image_histogram_init, value_image_histogram_free_value, value_image_histogram_copy_value, value_image_histogram_peek_pointer, "p", value_image_histogram_collect_value, "p", value_image_histogram_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ImageHistogramClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) image_histogram_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImageHistogram), 0, (GInstanceInitFunc) image_histogram_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		image_histogram_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ImageHistogram", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return image_histogram_type_id;
}


gpointer image_histogram_ref (gpointer instance) {
	ImageHistogram* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void image_histogram_unref (gpointer instance) {
	ImageHistogram* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		IMAGE_HISTOGRAM_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




