/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <sqlite3.h>
#include <gio/gio.h>
#include <gee.h>
#include <time.h>
#include <gobject/gvaluecollector.h>


#define TYPE_DATABASE_TABLE (database_table_get_type ())
#define DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_TABLE, DatabaseTable))
#define DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_TABLE, DatabaseTableClass))
#define IS_DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_TABLE))
#define IS_DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_TABLE))
#define DATABASE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_TABLE, DatabaseTableClass))

typedef struct _DatabaseTable DatabaseTable;
typedef struct _DatabaseTableClass DatabaseTableClass;
typedef struct _DatabaseTablePrivate DatabaseTablePrivate;
typedef struct _ParamSpecDatabaseTable ParamSpecDatabaseTable;

#define TYPE_VERSION_TABLE (version_table_get_type ())
#define VERSION_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_VERSION_TABLE, VersionTable))
#define VERSION_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_VERSION_TABLE, VersionTableClass))
#define IS_VERSION_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_VERSION_TABLE))
#define IS_VERSION_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_VERSION_TABLE))
#define VERSION_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_VERSION_TABLE, VersionTableClass))

typedef struct _VersionTable VersionTable;
typedef struct _VersionTableClass VersionTableClass;

#define TYPE_PHOTO_TABLE (photo_table_get_type ())
#define PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_TABLE, PhotoTable))
#define PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_TABLE, PhotoTableClass))
#define IS_PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_TABLE))
#define IS_PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_TABLE))
#define PHOTO_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_TABLE, PhotoTableClass))

typedef struct _PhotoTable PhotoTable;
typedef struct _PhotoTableClass PhotoTableClass;

#define TYPE_PHOTO_ID (photo_id_get_type ())
typedef struct _PhotoID PhotoID;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define PHOTO_TYPE_CURRENCY (photo_currency_get_type ())

#define TYPE_EVENT_TABLE (event_table_get_type ())
#define EVENT_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENT_TABLE, EventTable))
#define EVENT_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENT_TABLE, EventTableClass))
#define IS_EVENT_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENT_TABLE))
#define IS_EVENT_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENT_TABLE))
#define EVENT_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENT_TABLE, EventTableClass))

typedef struct _EventTable EventTable;
typedef struct _EventTableClass EventTableClass;

#define TYPE_EVENT_ID (event_id_get_type ())
typedef struct _EventID EventID;
typedef struct _VersionTablePrivate VersionTablePrivate;

#define TYPE_IMPORT_ID (import_id_get_type ())
typedef struct _ImportID ImportID;
typedef struct _PhotoTablePrivate PhotoTablePrivate;

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_ORIENTATION (orientation_get_type ())

#define TYPE_KEY_VALUE_MAP (key_value_map_get_type ())
#define KEY_VALUE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_KEY_VALUE_MAP, KeyValueMap))
#define KEY_VALUE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_KEY_VALUE_MAP, KeyValueMapClass))
#define IS_KEY_VALUE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_KEY_VALUE_MAP))
#define IS_KEY_VALUE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_KEY_VALUE_MAP))
#define KEY_VALUE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_KEY_VALUE_MAP, KeyValueMapClass))

typedef struct _KeyValueMap KeyValueMap;
typedef struct _KeyValueMapClass KeyValueMapClass;

#define TYPE_THUMBNAIL_CACHE_TABLE (thumbnail_cache_table_get_type ())
#define THUMBNAIL_CACHE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL_CACHE_TABLE, ThumbnailCacheTable))
#define THUMBNAIL_CACHE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL_CACHE_TABLE, ThumbnailCacheTableClass))
#define IS_THUMBNAIL_CACHE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL_CACHE_TABLE))
#define IS_THUMBNAIL_CACHE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL_CACHE_TABLE))
#define THUMBNAIL_CACHE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL_CACHE_TABLE, ThumbnailCacheTableClass))

typedef struct _ThumbnailCacheTable ThumbnailCacheTable;
typedef struct _ThumbnailCacheTableClass ThumbnailCacheTableClass;
typedef struct _ThumbnailCacheTablePrivate ThumbnailCacheTablePrivate;
typedef struct _EventTablePrivate EventTablePrivate;

struct _DatabaseTable {
	GTypeInstance parent_instance;
	volatile int ref_count;
	DatabaseTablePrivate * priv;
	char* table_name;
};

struct _DatabaseTableClass {
	GTypeClass parent_class;
	void (*finalize) (DatabaseTable *self);
};

struct _ParamSpecDatabaseTable {
	GParamSpec parent_instance;
};

struct _PhotoID {
	gint64 id;
};

typedef enum  {
	PHOTO_CURRENCY_CURRENT,
	PHOTO_CURRENCY_DIRTY,
	PHOTO_CURRENCY_GONE
} PhotoCurrency;

struct _EventID {
	gint64 id;
};

struct _VersionTable {
	DatabaseTable parent_instance;
	VersionTablePrivate * priv;
};

struct _VersionTableClass {
	DatabaseTableClass parent_class;
};

struct _ImportID {
	gint64 id;
};

struct _PhotoTable {
	DatabaseTable parent_instance;
	PhotoTablePrivate * priv;
};

struct _PhotoTableClass {
	DatabaseTableClass parent_class;
};

struct _Dimensions {
	gint width;
	gint height;
};

typedef enum  {
	ORIENTATION_MIN = 1,
	ORIENTATION_TOP_LEFT = 1,
	ORIENTATION_TOP_RIGHT = 2,
	ORIENTATION_BOTTOM_RIGHT = 3,
	ORIENTATION_BOTTOM_LEFT = 4,
	ORIENTATION_LEFT_TOP = 5,
	ORIENTATION_RIGHT_TOP = 6,
	ORIENTATION_RIGHT_BOTTOM = 7,
	ORIENTATION_LEFT_BOTTOM = 8,
	ORIENTATION_MAX = 8
} Orientation;

struct _ThumbnailCacheTable {
	DatabaseTable parent_instance;
	ThumbnailCacheTablePrivate * priv;
};

struct _ThumbnailCacheTableClass {
	DatabaseTableClass parent_class;
};

struct _EventTable {
	DatabaseTable parent_instance;
	EventTablePrivate * priv;
};

struct _EventTableClass {
	DatabaseTableClass parent_class;
};


extern sqlite3* database_table_db;
sqlite3* database_table_db = NULL;
static gpointer database_table_parent_class = NULL;
static gpointer version_table_parent_class = NULL;
static gpointer photo_table_parent_class = NULL;
static gpointer thumbnail_cache_table_parent_class = NULL;
static gpointer event_table_parent_class = NULL;

gpointer database_table_ref (gpointer instance);
void database_table_unref (gpointer instance);
GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_database_table (GValue* value, gpointer v_object);
gpointer value_get_database_table (const GValue* value);
GType database_table_get_type (void);
enum  {
	DATABASE_TABLE_DUMMY_PROPERTY
};
#define DATABASE_TABLE_SCHEMA_VERSION 1
GFile* app_window_get_data_subdir (const char* name, const char* subname);
void database_table_init (void);
void database_table_terminate (void);
void database_table_fatal (const char* op, gint res);
void database_table_warning (const char* op, gint res);
void database_table_set_table_name (DatabaseTable* self, const char* table_name);
gboolean database_table_exists_by_id (DatabaseTable* self, gint64 id);
gboolean database_table_select_by_id (DatabaseTable* self, gint64 id, const char* columns, sqlite3_stmt** stmt);
DatabaseTable* database_table_new (void);
DatabaseTable* database_table_construct (GType object_type);
static void database_table_finalize (DatabaseTable* obj);
VersionTable* version_table_new (void);
VersionTable* version_table_construct (GType object_type);
GType version_table_get_type (void);
gint version_table_get_version (VersionTable* self, char** app_version);
void version_table_set_version (VersionTable* self, gint version, const char* app_version, const char* user_data);
#define RESOURCES_APP_VERSION _VERSION
PhotoTable* photo_table_new (void);
PhotoTable* photo_table_construct (GType object_type);
GType photo_table_get_type (void);
GType photo_id_get_type (void);
PhotoID* photo_id_dup (const PhotoID* self);
void photo_id_free (PhotoID* self);
GeeArrayList* photo_table_get_photos (PhotoTable* self);
GType photo_get_type (void);
Photo* photo_fetch (const PhotoID* photo_id);
GType photo_currency_get_type (void);
PhotoCurrency photo_check_currency (Photo* self);
char* photo_to_string (Photo* self);
void photo_update (Photo* self);
void photo_remove (Photo* self, gboolean remove_original);
EventTable* event_table_new (void);
EventTable* event_table_construct (GType object_type);
GType event_table_get_type (void);
GType event_id_get_type (void);
EventID* event_id_dup (const EventID* self);
void event_id_free (EventID* self);
GeeArrayList* event_table_get_events (EventTable* self);
gboolean photo_table_event_has_photos (PhotoTable* self, const EventID* event_id);
gboolean event_table_remove (EventTable* self, const EventID* event_id);
gboolean verify_databases (char** app_version);
enum  {
	VERSION_TABLE_DUMMY_PROPERTY
};
#define PHOTO_ID_INVALID ((gint64) (-1))
void photo_id_init (PhotoID *self, gint64 id);
gboolean photo_id_is_invalid (PhotoID *self);
gboolean photo_id_is_valid (PhotoID *self);
GType import_id_get_type (void);
ImportID* import_id_dup (const ImportID* self);
void import_id_free (ImportID* self);
#define IMPORT_ID_INVALID ((gint64) (-1))
void import_id_init (ImportID *self, gint64 id);
gboolean import_id_is_invalid (ImportID *self);
gboolean import_id_is_valid (ImportID *self);
enum  {
	PHOTO_TABLE_DUMMY_PROPERTY
};
ImportID photo_table_generate_import_id (PhotoTable* self);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
GType orientation_get_type (void);
PhotoID photo_table_add (PhotoTable* self, GFile* file, const Dimensions* dim, gint64 filesize, glong timestamp, const time_t* exposure_time, Orientation orientation, const ImportID* import_id);
gboolean photo_table_update (PhotoTable* self, const PhotoID* photoID, const Dimensions* dim, gint64 filesize, glong timestamp, const time_t* exposure_time, Orientation orientation);
gboolean photo_table_exists (PhotoTable* self, const PhotoID* photo_id);
GFile* photo_table_get_file (PhotoTable* self, const PhotoID* photo_id);
char* photo_table_get_name (PhotoTable* self, const PhotoID* photo_id);
time_t photo_table_get_exposure_time (PhotoTable* self, const PhotoID* photo_id);
time_t photo_table_get_timestamp (PhotoTable* self, const PhotoID* photo_id);
gint64 photo_table_get_filesize (PhotoTable* self, const PhotoID* photo_id);
gboolean photo_table_remove_by_file (PhotoTable* self, GFile* file);
gboolean photo_table_remove (PhotoTable* self, const PhotoID* photoID);
PhotoID photo_table_get_id (PhotoTable* self, GFile* file);
gboolean photo_table_is_photo_stored (PhotoTable* self, GFile* file);
void dimensions_init (Dimensions *self, gint width, gint height);
Dimensions photo_table_get_dimensions (PhotoTable* self, const PhotoID* photo_id);
Orientation photo_table_get_original_orientation (PhotoTable* self, const PhotoID* photo_id);
Orientation photo_table_get_orientation (PhotoTable* self, const PhotoID* photo_id);
gboolean photo_table_set_orientation (PhotoTable* self, const PhotoID* photo_id, Orientation orientation);
#define EVENT_ID_INVALID ((gint64) (-1))
void event_id_init (EventID *self, gint64 id);
EventID photo_table_get_event (PhotoTable* self, const PhotoID* photo_id);
GeeArrayList* photo_table_get_event_photos (PhotoTable* self, const EventID* event_id);
gboolean photo_table_set_event (PhotoTable* self, const PhotoID* photo_id, const EventID* event_id);
static char* photo_table_get_raw_transformations (PhotoTable* self, const PhotoID* photo_id);
static gboolean photo_table_set_raw_transformations (PhotoTable* self, const PhotoID* photo_id, const char* trans);
gboolean photo_table_has_transformations (PhotoTable* self, const PhotoID* photo_id);
KeyValueMap* key_value_map_new (const char* group);
KeyValueMap* key_value_map_construct (GType object_type, const char* group);
gpointer key_value_map_ref (gpointer instance);
void key_value_map_unref (gpointer instance);
GParamSpec* param_spec_key_value_map (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_key_value_map (GValue* value, gpointer v_object);
gpointer value_get_key_value_map (const GValue* value);
GType key_value_map_get_type (void);
void key_value_map_set_string (KeyValueMap* self, const char* key, const char* value);
KeyValueMap* photo_table_get_transformation (PhotoTable* self, const PhotoID* photo_id, const char* object);
GeeSet* key_value_map_get_keys (KeyValueMap* self);
char* key_value_map_get_string (KeyValueMap* self, const char* key, const char* def);
char* key_value_map_get_group (KeyValueMap* self);
gboolean photo_table_set_transformation (PhotoTable* self, const PhotoID* photo_id, KeyValueMap* map);
gboolean photo_table_remove_transformation (PhotoTable* self, const PhotoID* photo_id, const char* object);
gboolean photo_table_remove_all_transformations (PhotoTable* self, const PhotoID* photo_id);
GType thumbnail_cache_table_get_type (void);
enum  {
	THUMBNAIL_CACHE_TABLE_DUMMY_PROPERTY
};
ThumbnailCacheTable* thumbnail_cache_table_new (gint scale);
ThumbnailCacheTable* thumbnail_cache_table_construct (GType object_type, gint scale);
gboolean thumbnail_cache_table_remove (ThumbnailCacheTable* self, const PhotoID* photo_id);
gboolean thumbnail_cache_table_exists (ThumbnailCacheTable* self, const PhotoID* photo_id);
void thumbnail_cache_table_add (ThumbnailCacheTable* self, const PhotoID* photo_id, gint filesize, const Dimensions* dim);
void thumbnail_cache_table_replace (ThumbnailCacheTable* self, const PhotoID* photo_id, gint filesize, const Dimensions* dim);
Dimensions thumbnail_cache_table_get_dimensions (ThumbnailCacheTable* self, const PhotoID* photo_id);
gint thumbnail_cache_table_get_filesize (ThumbnailCacheTable* self, const PhotoID* photo_id);
gboolean event_id_is_invalid (EventID *self);
gboolean event_id_is_valid (EventID *self);
enum  {
	EVENT_TABLE_DUMMY_PROPERTY
};
EventID event_table_create (EventTable* self, const PhotoID* primary_photo_id, const time_t* start_time);
time_t event_table_get_start_time (EventTable* self, const EventID* event_id);
gboolean event_table_set_end_time (EventTable* self, const EventID* event_id, const time_t* end_time);
gboolean event_table_rename (EventTable* self, const EventID* event_id, const char* name);
char* event_table_get_name (EventTable* self, const EventID* event_id);
PhotoID event_table_get_primary_photo (EventTable* self, const EventID* event_id);
gboolean event_table_set_primary_photo (EventTable* self, const EventID* event_id, const PhotoID* photo_id);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



/* Doing this because static construct {} not working*/
#line 19 "DatabaseTables.vala"
void database_table_init (void) {
#line 354 "DatabaseTables.c"
	GFile* _tmp0_;
	GFile* _tmp1_;
	GFile* dbFile;
	sqlite3* _tmp5_;
	gint _tmp4_;
	sqlite3* _tmp3_;
	char* _tmp2_;
	gint _tmp6_;
	gint res;
#line 479 "AppWindow.vala"
	_tmp0_ = NULL;
#line 20 "DatabaseTables.vala"
	_tmp1_ = NULL;
#line 368 "DatabaseTables.c"
	dbFile = (_tmp1_ = g_file_get_child (_tmp0_ = app_window_get_data_subdir ("data", NULL), "photo.db"), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	_tmp5_ = NULL;
	_tmp3_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp2_ = NULL;
#line 374 "DatabaseTables.c"
	res = (_tmp6_ = (_tmp4_ = sqlite3_open_v2 (_tmp2_ = g_file_get_path (dbFile), &_tmp3_, SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE, NULL), database_table_db = (_tmp5_ = _tmp3_, (database_table_db == NULL) ? NULL : (database_table_db = (sqlite3_close (database_table_db), NULL)), _tmp5_), _tmp4_), _tmp2_ = (g_free (_tmp2_), NULL), _tmp6_);
#line 23 "DatabaseTables.vala"
	if (res != SQLITE_OK) {
#line 378 "DatabaseTables.c"
		char* _tmp7_;
#line 952 "gio-2.0.vapi"
		_tmp7_ = NULL;
#line 24 "DatabaseTables.vala"
		g_error ("DatabaseTables.vala:24: Unable to open/create photo database %s: %d", _tmp7_ = g_file_get_path (dbFile), res);
#line 384 "DatabaseTables.c"
		_tmp7_ = (g_free (_tmp7_), NULL);
	}
	(dbFile == NULL) ? NULL : (dbFile = (g_object_unref (dbFile), NULL));
}


#line 27 "DatabaseTables.vala"
void database_table_terminate (void) {
#line 393 "DatabaseTables.c"
}


/* XXX: errmsg() is global, and so this will not be accurate in a threaded situation*/
#line 31 "DatabaseTables.vala"
void database_table_fatal (const char* op, gint res) {
#line 31 "DatabaseTables.vala"
	g_return_if_fail (op != NULL);
#line 32 "DatabaseTables.vala"
	g_error ("DatabaseTables.vala:32: %s: [%d] %s", op, res, sqlite3_errmsg (database_table_db));
#line 404 "DatabaseTables.c"
}


/* XXX: errmsg() is global, and so this will not be accurate in a threaded situation*/
#line 36 "DatabaseTables.vala"
void database_table_warning (const char* op, gint res) {
#line 36 "DatabaseTables.vala"
	g_return_if_fail (op != NULL);
#line 37 "DatabaseTables.vala"
	g_warning ("DatabaseTables.vala:37: %s: [%d] %s", op, res, sqlite3_errmsg (database_table_db));
#line 415 "DatabaseTables.c"
}


#line 40 "DatabaseTables.vala"
void database_table_set_table_name (DatabaseTable* self, const char* table_name) {
#line 421 "DatabaseTables.c"
	char* _tmp1_;
	const char* _tmp0_;
#line 40 "DatabaseTables.vala"
	g_return_if_fail (IS_DATABASE_TABLE (self));
#line 40 "DatabaseTables.vala"
	g_return_if_fail (table_name != NULL);
#line 428 "DatabaseTables.c"
	_tmp1_ = NULL;
#line 41 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 41 "DatabaseTables.vala"
	self->table_name = (_tmp1_ = (_tmp0_ = table_name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->table_name = (g_free (self->table_name), NULL), _tmp1_);
#line 434 "DatabaseTables.c"
}


#line 44 "DatabaseTables.vala"
gboolean database_table_exists_by_id (DatabaseTable* self, gint64 id) {
#line 440 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
	gboolean _tmp5_;
#line 44 "DatabaseTables.vala"
	g_return_val_if_fail (IS_DATABASE_TABLE (self), FALSE);
#line 452 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 458 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("SELECT id FROM %s WHERE id=?", self->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 47 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 49 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, id);
#line 50 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 52 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 468 "DatabaseTables.c"
	_tmp5_ = FALSE;
#line 53 "DatabaseTables.vala"
	if (res != SQLITE_ROW) {
#line 53 "DatabaseTables.vala"
		_tmp5_ = res != SQLITE_DONE;
#line 474 "DatabaseTables.c"
	} else {
#line 53 "DatabaseTables.vala"
		_tmp5_ = FALSE;
#line 478 "DatabaseTables.c"
	}
#line 53 "DatabaseTables.vala"
	if (_tmp5_) {
#line 482 "DatabaseTables.c"
		char* _tmp6_;
#line 820 "glib-2.0.vapi"
		_tmp6_ = NULL;
#line 54 "DatabaseTables.vala"
		database_table_fatal (_tmp6_ = g_strdup_printf ("exists_by_id [%lld] %s", id, self->table_name), res);
#line 488 "DatabaseTables.c"
		_tmp6_ = (g_free (_tmp6_), NULL);
	}
	result = res == SQLITE_ROW;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 59 "DatabaseTables.vala"
gboolean database_table_select_by_id (DatabaseTable* self, gint64 id, const char* columns, sqlite3_stmt** stmt) {
#line 499 "DatabaseTables.c"
	gboolean result;
	char* sql;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	gboolean _tmp3_;
#line 59 "DatabaseTables.vala"
	g_return_val_if_fail (IS_DATABASE_TABLE (self), FALSE);
#line 59 "DatabaseTables.vala"
	g_return_val_if_fail (columns != NULL, FALSE);
#line 511 "DatabaseTables.c"
	if (stmt != NULL) {
		*stmt = NULL;
	}
	sql = g_strdup_printf ("SELECT %s FROM %s WHERE id=?", columns, self->table_name);
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, sql, -1, &_tmp0_, NULL), (*stmt) = (_tmp2_ = _tmp0_, ((*stmt) == NULL) ? NULL : ((*stmt) = (sqlite3_finalize ((*stmt)), NULL)), _tmp2_), _tmp1_);
#line 63 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 65 "DatabaseTables.vala"
	res = sqlite3_bind_int64 ((*stmt), 1, id);
#line 66 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 68 "DatabaseTables.vala"
	res = sqlite3_step ((*stmt));
#line 527 "DatabaseTables.c"
	_tmp3_ = FALSE;
#line 69 "DatabaseTables.vala"
	if (res != SQLITE_ROW) {
#line 69 "DatabaseTables.vala"
		_tmp3_ = res != SQLITE_DONE;
#line 533 "DatabaseTables.c"
	} else {
#line 69 "DatabaseTables.vala"
		_tmp3_ = FALSE;
#line 537 "DatabaseTables.c"
	}
#line 69 "DatabaseTables.vala"
	if (_tmp3_) {
#line 541 "DatabaseTables.c"
		char* _tmp4_;
#line 820 "glib-2.0.vapi"
		_tmp4_ = NULL;
#line 70 "DatabaseTables.vala"
		database_table_fatal (_tmp4_ = g_strdup_printf ("select_by_id [%lld] %s %s", id, self->table_name, columns), res);
#line 547 "DatabaseTables.c"
		_tmp4_ = (g_free (_tmp4_), NULL);
	}
	result = res == SQLITE_ROW;
	sql = (g_free (sql), NULL);
	return result;
}


#line 7 "DatabaseTables.vala"
DatabaseTable* database_table_construct (GType object_type) {
#line 558 "DatabaseTables.c"
	DatabaseTable* self;
	self = (DatabaseTable*) g_type_create_instance (object_type);
	return self;
}


#line 7 "DatabaseTables.vala"
DatabaseTable* database_table_new (void) {
#line 7 "DatabaseTables.vala"
	return database_table_construct (TYPE_DATABASE_TABLE);
#line 569 "DatabaseTables.c"
}


static void value_database_table_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_database_table_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		database_table_unref (value->data[0].v_pointer);
	}
}


static void value_database_table_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = database_table_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_database_table_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_database_table_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		DatabaseTable* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = database_table_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_database_table_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	DatabaseTable** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = database_table_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecDatabaseTable* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_DATABASE_TABLE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_database_table (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_DATABASE_TABLE), NULL);
	return value->data[0].v_pointer;
}


void value_set_database_table (GValue* value, gpointer v_object) {
	DatabaseTable* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_DATABASE_TABLE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_DATABASE_TABLE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		database_table_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		database_table_unref (old);
	}
}


static void database_table_class_init (DatabaseTableClass * klass) {
	database_table_parent_class = g_type_class_peek_parent (klass);
	DATABASE_TABLE_CLASS (klass)->finalize = database_table_finalize;
}


static void database_table_instance_init (DatabaseTable * self) {
	self->table_name = NULL;
	self->ref_count = 1;
}


static void database_table_finalize (DatabaseTable* obj) {
	DatabaseTable * self;
	self = DATABASE_TABLE (obj);
	self->table_name = (g_free (self->table_name), NULL);
}


GType database_table_get_type (void) {
	static GType database_table_type_id = 0;
	if (database_table_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_database_table_init, value_database_table_free_value, value_database_table_copy_value, value_database_table_peek_pointer, "p", value_database_table_collect_value, "p", value_database_table_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (DatabaseTableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) database_table_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DatabaseTable), 0, (GInstanceInitFunc) database_table_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		database_table_type_id = g_type_register_fundamental (g_type_fundamental_next (), "DatabaseTable", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return database_table_type_id;
}


gpointer database_table_ref (gpointer instance) {
	DatabaseTable* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void database_table_unref (gpointer instance) {
	DatabaseTable* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		DATABASE_TABLE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 76 "DatabaseTables.vala"
gboolean verify_databases (char** app_version) {
#line 717 "DatabaseTables.c"
	gboolean result;
	VersionTable* version_table;
	char* _tmp2_;
	gint _tmp1_;
	char* _tmp0_;
	gint version;
	PhotoTable* photo_table;
	GeeArrayList* photo_ids;
	EventTable* event_table;
	GeeArrayList* event_ids;
	if (app_version != NULL) {
		*app_version = NULL;
	}
	/* Since this is the first database version, the only thing checked for is the right one,
	 no upgrades attempted (since the only real alternative is downgrading)*/
	version_table = version_table_new ();
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	version = (_tmp1_ = version_table_get_version (version_table, &_tmp0_), (*app_version) = (_tmp2_ = _tmp0_, (*app_version) = (g_free ((*app_version)), NULL), _tmp2_), _tmp1_);
#line 81 "DatabaseTables.vala"
	g_debug ("DatabaseTables.vala:81: Database version %d create by app version %s", version, (*app_version));
#line 83 "DatabaseTables.vala"
	if (version == (-1)) {
#line 741 "DatabaseTables.c"
		/* no version set, do it now*/
#line 85 "DatabaseTables.vala"
		version_table_set_version (version_table, DATABASE_TABLE_SCHEMA_VERSION, RESOURCES_APP_VERSION, NULL);
#line 745 "DatabaseTables.c"
	} else {
#line 86 "DatabaseTables.vala"
		if (version != DATABASE_TABLE_SCHEMA_VERSION) {
#line 749 "DatabaseTables.c"
			result = FALSE;
			(version_table == NULL) ? NULL : (version_table = (database_table_unref (version_table), NULL));
			return result;
		}
	}
	photo_table = photo_table_new ();
	photo_ids = photo_table_get_photos (photo_table);
	/* verify photo table*/
	{
		GeeIterator* _photo_id_it;
		/* verify photo table*/
		_photo_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (photo_ids));
		/* verify photo table*/
#line 94 "DatabaseTables.vala"
		while (TRUE) {
#line 765 "DatabaseTables.c"
			PhotoID* _tmp3_;
			PhotoID _tmp4_ = {0};
			PhotoID photo_id;
			Photo* photo;
			/* verify photo table*/
#line 94 "DatabaseTables.vala"
			if (!gee_iterator_next (_photo_id_it)) {
#line 773 "DatabaseTables.c"
				/* verify photo table*/
#line 94 "DatabaseTables.vala"
				break;
#line 777 "DatabaseTables.c"
			}
			/* verify photo table*/
#line 91 "DatabaseTables.vala"
			_tmp3_ = NULL;
#line 782 "DatabaseTables.c"
			photo_id = (_tmp4_ = *(_tmp3_ = (PhotoID*) gee_iterator_get (_photo_id_it)), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (photo_id_free (_tmp3_), NULL)), _tmp4_);
			photo = photo_fetch (&photo_id);
#line 96 "DatabaseTables.vala"
			switch (photo_check_currency (photo)) {
#line 787 "DatabaseTables.c"
				case PHOTO_CURRENCY_CURRENT:
				{
#line 99 "DatabaseTables.vala"
					break;
#line 792 "DatabaseTables.c"
				}
				case PHOTO_CURRENCY_DIRTY:
				{
					char* _tmp5_;
#line 208 "Photo.vala"
					_tmp5_ = NULL;
#line 102 "DatabaseTables.vala"
					g_message ("DatabaseTables.vala:102: Time or filesize changed on %s, reimporting ...", _tmp5_ = photo_to_string (photo));
#line 801 "DatabaseTables.c"
					_tmp5_ = (g_free (_tmp5_), NULL);
#line 103 "DatabaseTables.vala"
					photo_update (photo);
#line 104 "DatabaseTables.vala"
					break;
#line 807 "DatabaseTables.c"
				}
				case PHOTO_CURRENCY_GONE:
				{
					char* _tmp6_;
#line 208 "Photo.vala"
					_tmp6_ = NULL;
#line 107 "DatabaseTables.vala"
					g_message ("DatabaseTables.vala:107: Unable to locate %s: Removing from photo library", _tmp6_ = photo_to_string (photo));
#line 816 "DatabaseTables.c"
					_tmp6_ = (g_free (_tmp6_), NULL);
#line 108 "DatabaseTables.vala"
					photo_remove (photo, TRUE);
#line 109 "DatabaseTables.vala"
					break;
#line 822 "DatabaseTables.c"
				}
				default:
				{
#line 112 "DatabaseTables.vala"
					g_warn_if_reached ();
#line 113 "DatabaseTables.vala"
					break;
#line 830 "DatabaseTables.c"
				}
			}
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_photo_id_it == NULL) ? NULL : (_photo_id_it = (g_object_unref (_photo_id_it), NULL));
	}
	event_table = event_table_new ();
	event_ids = event_table_get_events (event_table);
	/* verify primary photo for all events*/
	{
		GeeIterator* _event_id_it;
		/* verify primary photo for all events*/
		_event_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (event_ids));
		/* verify primary photo for all events*/
#line 121 "DatabaseTables.vala"
		while (TRUE) {
#line 847 "DatabaseTables.c"
			EventID* _tmp7_;
			EventID _tmp8_ = {0};
			EventID event_id;
			/* verify primary photo for all events*/
#line 121 "DatabaseTables.vala"
			if (!gee_iterator_next (_event_id_it)) {
#line 854 "DatabaseTables.c"
				/* verify primary photo for all events*/
#line 121 "DatabaseTables.vala"
				break;
#line 858 "DatabaseTables.c"
			}
			/* verify primary photo for all events*/
#line 118 "DatabaseTables.vala"
			_tmp7_ = NULL;
#line 863 "DatabaseTables.c"
			event_id = (_tmp8_ = *(_tmp7_ = (EventID*) gee_iterator_get (_event_id_it)), (_tmp7_ == NULL) ? NULL : (_tmp7_ = (event_id_free (_tmp7_), NULL)), _tmp8_);
#line 122 "DatabaseTables.vala"
			if (!photo_table_event_has_photos (photo_table, &event_id)) {
#line 123 "DatabaseTables.vala"
				g_message ("DatabaseTables.vala:123: Removing event %lld: No photos associated with event", event_id.id);
#line 124 "DatabaseTables.vala"
				event_table_remove (event_table, &event_id);
#line 871 "DatabaseTables.c"
			}
		}
		(_event_id_it == NULL) ? NULL : (_event_id_it = (g_object_unref (_event_id_it), NULL));
	}
	result = TRUE;
	(version_table == NULL) ? NULL : (version_table = (database_table_unref (version_table), NULL));
	(photo_table == NULL) ? NULL : (photo_table = (database_table_unref (photo_table), NULL));
	(photo_ids == NULL) ? NULL : (photo_ids = (g_object_unref (photo_ids), NULL));
	(event_table == NULL) ? NULL : (event_table = (database_table_unref (event_table), NULL));
	(event_ids == NULL) ? NULL : (event_ids = (g_object_unref (event_ids), NULL));
	return result;
}


#line 132 "DatabaseTables.vala"
VersionTable* version_table_construct (GType object_type) {
#line 888 "DatabaseTables.c"
	VersionTable* self;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	self = (VersionTable*) g_type_create_instance (object_type);
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "CREATE TABLE IF NOT EXISTS VersionTable (" "id INTEGER PRIMARY KEY, " "schema_version INTEGER, " "app_version TEXT, " "user_data TEXT NULL" ")", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 140 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 142 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 143 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 144 "DatabaseTables.vala"
		database_table_fatal ("create version table", res);
#line 908 "DatabaseTables.c"
	}
#line 146 "DatabaseTables.vala"
	database_table_set_table_name (DATABASE_TABLE (self), "VersionTable");
#line 912 "DatabaseTables.c"
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return self;
}


#line 132 "DatabaseTables.vala"
VersionTable* version_table_new (void) {
#line 132 "DatabaseTables.vala"
	return version_table_construct (TYPE_VERSION_TABLE);
#line 922 "DatabaseTables.c"
}


#line 149 "DatabaseTables.vala"
gint version_table_get_version (VersionTable* self, char** app_version) {
#line 928 "DatabaseTables.c"
	gint result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	char* _tmp4_;
	const char* _tmp3_;
#line 149 "DatabaseTables.vala"
	g_return_val_if_fail (IS_VERSION_TABLE (self), 0);
#line 939 "DatabaseTables.c"
	if (app_version != NULL) {
		*app_version = NULL;
	}
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "SELECT schema_version, app_version FROM VersionTable ORDER BY schema_version DESC LIMIT 1", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 153 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 155 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 156 "DatabaseTables.vala"
	if (res != SQLITE_ROW) {
#line 157 "DatabaseTables.vala"
		if (res != SQLITE_DONE) {
#line 158 "DatabaseTables.vala"
			database_table_fatal ("get_version", res);
#line 957 "DatabaseTables.c"
		}
		result = -1;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	_tmp4_ = NULL;
#line 163 "DatabaseTables.vala"
	_tmp3_ = NULL;
#line 163 "DatabaseTables.vala"
	(*app_version) = (_tmp4_ = (_tmp3_ = sqlite3_column_text (stmt, 1), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), (*app_version) = (g_free ((*app_version)), NULL), _tmp4_);
#line 968 "DatabaseTables.c"
	result = sqlite3_column_int (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 168 "DatabaseTables.vala"
void version_table_set_version (VersionTable* self, gint version, const char* app_version, const char* user_data) {
#line 977 "DatabaseTables.c"
	sqlite3_stmt* stmt;
	char* bitbucket;
	char* _tmp2_;
	gint _tmp1_;
	char* _tmp0_;
	gint res;
	const char* _tmp9_;
	const char* _tmp10_;
#line 168 "DatabaseTables.vala"
	g_return_if_fail (IS_VERSION_TABLE (self));
#line 168 "DatabaseTables.vala"
	g_return_if_fail (app_version != NULL);
#line 990 "DatabaseTables.c"
	stmt = NULL;
	bitbucket = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 172 "DatabaseTables.vala"
	if ((_tmp1_ = version_table_get_version (self, &_tmp0_), bitbucket = (_tmp2_ = _tmp0_, bitbucket = (g_free (bitbucket), NULL), _tmp2_), _tmp1_) != (-1)) {
#line 997 "DatabaseTables.c"
		sqlite3_stmt* _tmp5_;
		gint _tmp4_;
		sqlite3_stmt* _tmp3_;
		gint res;
		/* overwrite existing row*/
		_tmp5_ = NULL;
		_tmp3_ = NULL;
		res = (_tmp4_ = sqlite3_prepare_v2 (database_table_db, "UPDATE VersionTable SET schema_version=?, app_version=?, user_data=?", -1, &_tmp3_, NULL), stmt = (_tmp5_ = _tmp3_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp5_), _tmp4_);
#line 176 "DatabaseTables.vala"
		g_assert (res == SQLITE_OK);
#line 1008 "DatabaseTables.c"
	} else {
		sqlite3_stmt* _tmp8_;
		gint _tmp7_;
		sqlite3_stmt* _tmp6_;
		gint res;
		/* insert new row*/
		_tmp8_ = NULL;
		_tmp6_ = NULL;
		res = (_tmp7_ = sqlite3_prepare_v2 (database_table_db, "INSERT INTO VersionTable (schema_version, app_version, user_data) VALUES (?,?, ?)", -1, &_tmp6_, NULL), stmt = (_tmp8_ = _tmp6_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp8_), _tmp7_);
#line 181 "DatabaseTables.vala"
		g_assert (res == SQLITE_OK);
#line 1020 "DatabaseTables.c"
	}
	res = sqlite3_bind_int (stmt, 1, version);
#line 185 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 186 "DatabaseTables.vala"
	_tmp9_ = NULL;
#line 186 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 2, (_tmp9_ = app_version, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), -1, g_free);
#line 187 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 188 "DatabaseTables.vala"
	_tmp10_ = NULL;
#line 188 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 3, (_tmp10_ = user_data, (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), -1, g_free);
#line 189 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 191 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 192 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 1041 "DatabaseTables.c"
		char* _tmp11_;
#line 820 "glib-2.0.vapi"
		_tmp11_ = NULL;
#line 193 "DatabaseTables.vala"
		database_table_fatal (_tmp11_ = g_strdup_printf ("set_version %d %s %s", version, app_version, user_data), res);
#line 1047 "DatabaseTables.c"
		_tmp11_ = (g_free (_tmp11_), NULL);
	}
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	bitbucket = (g_free (bitbucket), NULL);
}


static void version_table_class_init (VersionTableClass * klass) {
	version_table_parent_class = g_type_class_peek_parent (klass);
}


static void version_table_instance_init (VersionTable * self) {
}


GType version_table_get_type (void) {
	static GType version_table_type_id = 0;
	if (version_table_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VersionTableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) version_table_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VersionTable), 0, (GInstanceInitFunc) version_table_instance_init, NULL };
		version_table_type_id = g_type_register_static (TYPE_DATABASE_TABLE, "VersionTable", &g_define_type_info, 0);
	}
	return version_table_type_id;
}


#line 202 "DatabaseTables.vala"
void photo_id_init (PhotoID *self, gint64 id) {
#line 1076 "DatabaseTables.c"
	memset (self, 0, sizeof (PhotoID));
#line 203 "DatabaseTables.vala"
	(*self).id = id;
#line 1080 "DatabaseTables.c"
}


#line 206 "DatabaseTables.vala"
gboolean photo_id_is_invalid (PhotoID *self) {
#line 1086 "DatabaseTables.c"
	gboolean result;
	result = (*self).id == PHOTO_ID_INVALID;
	return result;
}


#line 210 "DatabaseTables.vala"
gboolean photo_id_is_valid (PhotoID *self) {
#line 1095 "DatabaseTables.c"
	gboolean result;
	result = (*self).id != PHOTO_ID_INVALID;
	return result;
}


PhotoID* photo_id_dup (const PhotoID* self) {
	PhotoID* dup;
	dup = g_new0 (PhotoID, 1);
	memcpy (dup, self, sizeof (PhotoID));
	return dup;
}


void photo_id_free (PhotoID* self) {
	g_free (self);
}


GType photo_id_get_type (void) {
	static GType photo_id_type_id = 0;
	if (photo_id_type_id == 0) {
		photo_id_type_id = g_boxed_type_register_static ("PhotoID", (GBoxedCopyFunc) photo_id_dup, (GBoxedFreeFunc) photo_id_free);
	}
	return photo_id_type_id;
}


#line 220 "DatabaseTables.vala"
void import_id_init (ImportID *self, gint64 id) {
#line 1126 "DatabaseTables.c"
	memset (self, 0, sizeof (ImportID));
#line 221 "DatabaseTables.vala"
	(*self).id = id;
#line 1130 "DatabaseTables.c"
}


#line 224 "DatabaseTables.vala"
gboolean import_id_is_invalid (ImportID *self) {
#line 1136 "DatabaseTables.c"
	gboolean result;
	result = (*self).id == IMPORT_ID_INVALID;
	return result;
}


#line 228 "DatabaseTables.vala"
gboolean import_id_is_valid (ImportID *self) {
#line 1145 "DatabaseTables.c"
	gboolean result;
	result = (*self).id != IMPORT_ID_INVALID;
	return result;
}


ImportID* import_id_dup (const ImportID* self) {
	ImportID* dup;
	dup = g_new0 (ImportID, 1);
	memcpy (dup, self, sizeof (ImportID));
	return dup;
}


void import_id_free (ImportID* self) {
	g_free (self);
}


GType import_id_get_type (void) {
	static GType import_id_type_id = 0;
	if (import_id_type_id == 0) {
		import_id_type_id = g_boxed_type_register_static ("ImportID", (GBoxedCopyFunc) import_id_dup, (GBoxedFreeFunc) import_id_free);
	}
	return import_id_type_id;
}


#line 234 "DatabaseTables.vala"
PhotoTable* photo_table_construct (GType object_type) {
#line 1176 "DatabaseTables.c"
	PhotoTable* self;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	self = (PhotoTable*) g_type_create_instance (object_type);
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "CREATE TABLE IF NOT EXISTS PhotoTable (" "id INTEGER PRIMARY KEY, " "filename TEXT UNIQUE NOT NULL, " "width INTEGER, " "height INTEGER, " "filesize INTEGER, " "timestamp INTEGER, " "exposure_time INTEGER, " "orientation INTEGER, " "original_orientation INTEGER, " "import_id INTEGER, " "event_id INTEGER, " "transformations TEXT" ")", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 250 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 252 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 253 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 254 "DatabaseTables.vala"
		database_table_fatal ("create photo table", res);
#line 1196 "DatabaseTables.c"
	}
#line 256 "DatabaseTables.vala"
	database_table_set_table_name (DATABASE_TABLE (self), "PhotoTable");
#line 1200 "DatabaseTables.c"
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return self;
}


#line 234 "DatabaseTables.vala"
PhotoTable* photo_table_new (void) {
#line 234 "DatabaseTables.vala"
	return photo_table_construct (TYPE_PHOTO_TABLE);
#line 1210 "DatabaseTables.c"
}


#line 259 "DatabaseTables.vala"
ImportID photo_table_generate_import_id (PhotoTable* self) {
#line 1216 "DatabaseTables.c"
	ImportID result;
	GTimeVal _tmp0_ = {0};
	GTimeVal timestamp;
	gint64 id;
	ImportID _tmp1_ = {0};
	/* TODO: Use a guid here?  Key here is that last imported photos can be easily identified
	 by finding the largest value in the PhotoTable*/
	timestamp = (g_get_current_time (&_tmp0_), _tmp0_);
#line 263 "DatabaseTables.vala"
	g_get_current_time (&timestamp);
#line 1227 "DatabaseTables.c"
	id = (gint64) timestamp.tv_sec;
	result = (import_id_init (&_tmp1_, id), _tmp1_);
	return result;
}


#line 269 "DatabaseTables.vala"
PhotoID photo_table_add (PhotoTable* self, GFile* file, const Dimensions* dim, gint64 filesize, glong timestamp, const time_t* exposure_time, Orientation orientation, const ImportID* import_id) {
#line 1236 "DatabaseTables.c"
	PhotoID result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	PhotoID _tmp4_ = {0};
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "INSERT INTO PhotoTable (filename, width, height, filesize, timestamp, exposure_time, orientation, original_orientation, import_id, event_id) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 275 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 277 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 1, g_file_get_path (file), -1, g_free);
#line 278 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 279 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 2, (*dim).width);
#line 280 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 281 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 3, (*dim).height);
#line 282 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 283 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 4, filesize);
#line 284 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 285 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 5, (gint64) timestamp);
#line 286 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 287 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 6, (gint64) (*exposure_time));
#line 288 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 289 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 7, (gint) orientation);
#line 290 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 291 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 8, (gint) orientation);
#line 292 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 293 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 9, (*import_id).id);
#line 294 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 295 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 10, PHOTO_ID_INVALID);
#line 296 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 298 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 299 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 1294 "DatabaseTables.c"
		PhotoID _tmp3_ = {0};
#line 300 "DatabaseTables.vala"
		if (res != SQLITE_CONSTRAINT) {
#line 301 "DatabaseTables.vala"
			database_table_fatal ("add_photo", res);
#line 1300 "DatabaseTables.c"
		}
		result = (photo_id_init (&_tmp3_, PHOTO_ID_INVALID), _tmp3_);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (photo_id_init (&_tmp4_, sqlite3_last_insert_rowid (database_table_db)), _tmp4_);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 309 "DatabaseTables.vala"
gboolean photo_table_update (PhotoTable* self, const PhotoID* photoID, const Dimensions* dim, gint64 filesize, glong timestamp, const time_t* exposure_time, Orientation orientation) {
#line 1315 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 309 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 1324 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE PhotoTable SET width = ?, height = ?, filesize = ?, timestamp = ?, " "exposure_time = ?, orientation = ?, original_orientation = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 316 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 318 "DatabaseTables.vala"
	g_debug ("DatabaseTables.vala:318: Update [%lld] %dx%d size=%lld mod=%ld exp=%ld or=%d", (*photoID).id, (*dim).width, (*dim).height, filesize, timestamp, (*exposure_time), (gint) orientation);
#line 321 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 1, (*dim).width);
#line 322 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 323 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 2, (*dim).height);
#line 324 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 325 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 3, filesize);
#line 326 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 327 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 4, (gint64) timestamp);
#line 328 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 329 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 5, (gint64) (*exposure_time));
#line 330 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 331 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 6, (gint) orientation);
#line 332 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 333 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 7, (gint) orientation);
#line 334 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 335 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 7, (*photoID).id);
#line 336 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 338 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 339 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 340 "DatabaseTables.vala"
		if (res != SQLITE_CONSTRAINT) {
#line 341 "DatabaseTables.vala"
			database_table_fatal ("update_photo", res);
#line 1373 "DatabaseTables.c"
		}
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 349 "DatabaseTables.vala"
gboolean photo_table_exists (PhotoTable* self, const PhotoID* photo_id) {
#line 1387 "DatabaseTables.c"
	gboolean result;
#line 349 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 1391 "DatabaseTables.c"
	result = database_table_exists_by_id (DATABASE_TABLE (self), (*photo_id).id);
	return result;
}


#line 353 "DatabaseTables.vala"
GFile* photo_table_get_file (PhotoTable* self, const PhotoID* photo_id) {
#line 1399 "DatabaseTables.c"
	GFile* result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
#line 353 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), NULL);
#line 1407 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 355 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "filename", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1413 "DatabaseTables.c"
		result = NULL;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = g_file_new_for_path (sqlite3_column_text (stmt, 0));
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 361 "DatabaseTables.vala"
char* photo_table_get_name (PhotoTable* self, const PhotoID* photo_id) {
#line 1426 "DatabaseTables.c"
	char* result;
	GFile* file;
	char* _tmp0_;
#line 361 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), NULL);
#line 1432 "DatabaseTables.c"
	file = photo_table_get_file (self, &(*photo_id));
	_tmp0_ = NULL;
#line 364 "DatabaseTables.vala"
	if (file != NULL) {
#line 1437 "DatabaseTables.c"
		char* _tmp1_;
		_tmp1_ = NULL;
#line 364 "DatabaseTables.vala"
		_tmp0_ = (_tmp1_ = g_file_get_basename (file), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
#line 1442 "DatabaseTables.c"
	} else {
		char* _tmp2_;
		_tmp2_ = NULL;
#line 364 "DatabaseTables.vala"
		_tmp0_ = (_tmp2_ = NULL, _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
#line 1448 "DatabaseTables.c"
	}
	result = _tmp0_;
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	return result;
}


#line 367 "DatabaseTables.vala"
time_t photo_table_get_exposure_time (PhotoTable* self, const PhotoID* photo_id) {
#line 1458 "DatabaseTables.c"
	time_t result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 369 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "exposure_time", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1469 "DatabaseTables.c"
		result = 0;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (time_t) sqlite3_column_int64 (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 375 "DatabaseTables.vala"
time_t photo_table_get_timestamp (PhotoTable* self, const PhotoID* photo_id) {
#line 1482 "DatabaseTables.c"
	time_t result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 377 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "timestamp", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1493 "DatabaseTables.c"
		result = 0;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (time_t) sqlite3_column_int64 (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 383 "DatabaseTables.vala"
gint64 photo_table_get_filesize (PhotoTable* self, const PhotoID* photo_id) {
#line 1506 "DatabaseTables.c"
	gint64 result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
#line 383 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), 0LL);
#line 1514 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 385 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "filesize", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1520 "DatabaseTables.c"
		result = (gint64) (-1);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = sqlite3_column_int64 (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 391 "DatabaseTables.vala"
gboolean photo_table_remove_by_file (PhotoTable* self, GFile* file) {
#line 1533 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 391 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 391 "DatabaseTables.vala"
	g_return_val_if_fail (G_IS_FILE (file), FALSE);
#line 1544 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "DELETE FROM PhotoTable WHERE filename=?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 394 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 396 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 1, g_file_get_path (file), -1, g_free);
#line 397 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 399 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 400 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 401 "DatabaseTables.vala"
		database_table_warning ("remove", res);
#line 1561 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 409 "DatabaseTables.vala"
gboolean photo_table_remove (PhotoTable* self, const PhotoID* photoID) {
#line 1574 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 409 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 1583 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "DELETE FROM PhotoTable WHERE id=?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 412 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 414 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photoID).id);
#line 415 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 417 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 418 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 419 "DatabaseTables.vala"
		database_table_warning ("remove", res);
#line 1600 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 427 "DatabaseTables.vala"
gboolean photo_table_is_photo_stored (PhotoTable* self, GFile* file) {
#line 1613 "DatabaseTables.c"
	gboolean result;
	PhotoID _tmp0_ = {0};
#line 427 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 427 "DatabaseTables.vala"
	g_return_val_if_fail (G_IS_FILE (file), FALSE);
#line 1620 "DatabaseTables.c"
	result = photo_id_is_invalid ((_tmp0_ = photo_table_get_id (self, file), &_tmp0_)) == FALSE;
	return result;
}


#line 431 "DatabaseTables.vala"
PhotoID photo_table_get_id (PhotoTable* self, GFile* file) {
#line 1628 "DatabaseTables.c"
	PhotoID result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	PhotoID _tmp3_ = {0};
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "SELECT ID FROM PhotoTable WHERE filename=?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 434 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 436 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 1, g_file_get_path (file), -1, g_free);
#line 437 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 439 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 441 "DatabaseTables.vala"
	if (res == SQLITE_ROW) {
#line 1650 "DatabaseTables.c"
		PhotoID _tmp4_ = {0};
#line 441 "DatabaseTables.vala"
		_tmp3_ = (photo_id_init (&_tmp4_, sqlite3_column_int64 (stmt, 0)), _tmp4_);
#line 1654 "DatabaseTables.c"
	} else {
		PhotoID _tmp5_ = {0};
#line 441 "DatabaseTables.vala"
		_tmp3_ = (photo_id_init (&_tmp5_, PHOTO_ID_INVALID), _tmp5_);
#line 1659 "DatabaseTables.c"
	}
	result = _tmp3_;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 444 "DatabaseTables.vala"
GeeArrayList* photo_table_get_photos (PhotoTable* self) {
#line 1669 "DatabaseTables.c"
	GeeArrayList* result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	GeeArrayList* photo_ids;
#line 444 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), NULL);
#line 1679 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "SELECT id FROM PhotoTable", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 447 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1686 "DatabaseTables.c"
	photo_ids = gee_array_list_new (TYPE_PHOTO_ID, (GBoxedCopyFunc) photo_id_dup, photo_id_free, g_direct_equal);
	{
		gboolean _tmp3_;
		_tmp3_ = TRUE;
#line 450 "DatabaseTables.vala"
		while (TRUE) {
#line 1693 "DatabaseTables.c"
			PhotoID _tmp5_ = {0};
			PhotoID _tmp4_ = {0};
#line 450 "DatabaseTables.vala"
			if (!_tmp3_) {
#line 1698 "DatabaseTables.c"
			}
#line 450 "DatabaseTables.vala"
			_tmp3_ = FALSE;
#line 451 "DatabaseTables.vala"
			res = sqlite3_step (stmt);
#line 452 "DatabaseTables.vala"
			if (res == SQLITE_DONE) {
#line 453 "DatabaseTables.vala"
				break;
#line 1708 "DatabaseTables.c"
			} else {
#line 454 "DatabaseTables.vala"
				if (res != SQLITE_ROW) {
#line 455 "DatabaseTables.vala"
					database_table_fatal ("get_photos", res);
#line 457 "DatabaseTables.vala"
					break;
#line 1716 "DatabaseTables.c"
				}
			}
#line 460 "DatabaseTables.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (photo_ids), (_tmp5_ = (photo_id_init (&_tmp4_, sqlite3_column_int64 (stmt, 0)), _tmp4_), &_tmp5_));
#line 1721 "DatabaseTables.c"
		}
	}
	result = photo_ids;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 466 "DatabaseTables.vala"
Dimensions photo_table_get_dimensions (PhotoTable* self, const PhotoID* photo_id) {
#line 1732 "DatabaseTables.c"
	Dimensions result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	Dimensions _tmp4_ = {0};
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 468 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "width, height", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1744 "DatabaseTables.c"
		Dimensions _tmp3_ = {0};
		result = (dimensions_init (&_tmp3_, 0, 0), _tmp3_);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (dimensions_init (&_tmp4_, sqlite3_column_int (stmt, 0), sqlite3_column_int (stmt, 1)), _tmp4_);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 474 "DatabaseTables.vala"
Orientation photo_table_get_original_orientation (PhotoTable* self, const PhotoID* photo_id) {
#line 1759 "DatabaseTables.c"
	Orientation result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
#line 474 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), 0);
#line 1767 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 476 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "original_orientation", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1773 "DatabaseTables.c"
		result = ORIENTATION_TOP_LEFT;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (Orientation) sqlite3_column_int (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 482 "DatabaseTables.vala"
Orientation photo_table_get_orientation (PhotoTable* self, const PhotoID* photo_id) {
#line 1786 "DatabaseTables.c"
	Orientation result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
#line 482 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), 0);
#line 1794 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 484 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "orientation", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1800 "DatabaseTables.c"
		result = ORIENTATION_TOP_LEFT;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (Orientation) sqlite3_column_int (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 490 "DatabaseTables.vala"
gboolean photo_table_set_orientation (PhotoTable* self, const PhotoID* photo_id, Orientation orientation) {
#line 1813 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 490 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 1822 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE PhotoTable SET orientation = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 493 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 495 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 1, (gint) orientation);
#line 496 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 497 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (*photo_id).id);
#line 498 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 500 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 501 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 502 "DatabaseTables.vala"
		database_table_fatal ("photo set_orientation", res);
#line 1843 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 510 "DatabaseTables.vala"
EventID photo_table_get_event (PhotoTable* self, const PhotoID* photo_id) {
#line 1856 "DatabaseTables.c"
	EventID result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	EventID _tmp4_ = {0};
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 512 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "event_id", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 1868 "DatabaseTables.c"
		EventID _tmp3_ = {0};
		result = (event_id_init (&_tmp3_, EVENT_ID_INVALID), _tmp3_);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (event_id_init (&_tmp4_, sqlite3_column_int64 (stmt, 0)), _tmp4_);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 518 "DatabaseTables.vala"
GeeArrayList* photo_table_get_event_photos (PhotoTable* self, const EventID* event_id) {
#line 1883 "DatabaseTables.c"
	GeeArrayList* result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	GeeArrayList* photo_ids;
#line 518 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), NULL);
#line 1893 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "SELECT id FROM PhotoTable WHERE event_id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 521 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 523 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*event_id).id);
#line 524 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1904 "DatabaseTables.c"
	photo_ids = gee_array_list_new (TYPE_PHOTO_ID, (GBoxedCopyFunc) photo_id_dup, photo_id_free, g_direct_equal);
	{
		gboolean _tmp3_;
		_tmp3_ = TRUE;
#line 527 "DatabaseTables.vala"
		while (TRUE) {
#line 1911 "DatabaseTables.c"
			PhotoID _tmp5_ = {0};
			PhotoID _tmp4_ = {0};
#line 527 "DatabaseTables.vala"
			if (!_tmp3_) {
#line 1916 "DatabaseTables.c"
			}
#line 527 "DatabaseTables.vala"
			_tmp3_ = FALSE;
#line 528 "DatabaseTables.vala"
			res = sqlite3_step (stmt);
#line 529 "DatabaseTables.vala"
			if (res == SQLITE_DONE) {
#line 530 "DatabaseTables.vala"
				break;
#line 1926 "DatabaseTables.c"
			} else {
#line 531 "DatabaseTables.vala"
				if (res != SQLITE_ROW) {
#line 532 "DatabaseTables.vala"
					database_table_fatal ("get_event_photos", res);
#line 534 "DatabaseTables.vala"
					break;
#line 1934 "DatabaseTables.c"
				}
			}
#line 537 "DatabaseTables.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (photo_ids), (_tmp5_ = (photo_id_init (&_tmp4_, sqlite3_column_int64 (stmt, 0)), _tmp4_), &_tmp5_));
#line 1939 "DatabaseTables.c"
		}
	}
	result = photo_ids;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 543 "DatabaseTables.vala"
gboolean photo_table_event_has_photos (PhotoTable* self, const EventID* event_id) {
#line 1950 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 543 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 1959 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "SELECT id FROM PhotoTable WHERE event_id = ? LIMIT 1", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 546 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 548 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*event_id).id);
#line 549 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 551 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 552 "DatabaseTables.vala"
	if (res == SQLITE_DONE) {
#line 1974 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	} else {
#line 554 "DatabaseTables.vala"
		if (res != SQLITE_ROW) {
#line 555 "DatabaseTables.vala"
			database_table_fatal ("event_has_photos", res);
#line 1983 "DatabaseTables.c"
			result = FALSE;
			(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
			return result;
		}
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 563 "DatabaseTables.vala"
gboolean photo_table_set_event (PhotoTable* self, const PhotoID* photo_id, const EventID* event_id) {
#line 1997 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 563 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 2006 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE PhotoTable SET event_id = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 566 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 568 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*event_id).id);
#line 569 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 570 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (*photo_id).id);
#line 571 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 573 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 574 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 575 "DatabaseTables.vala"
		database_table_fatal ("set_event", res);
#line 2027 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


#line 583 "DatabaseTables.vala"
static char* photo_table_get_raw_transformations (PhotoTable* self, const PhotoID* photo_id) {
#line 2048 "DatabaseTables.c"
	char* result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	const char* _tmp3_;
	char* trans;
	gboolean _tmp4_;
#line 583 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), NULL);
#line 2059 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 585 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*photo_id).id, "transformations", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 2065 "DatabaseTables.c"
		result = NULL;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
#line 588 "DatabaseTables.vala"
	_tmp3_ = NULL;
#line 2072 "DatabaseTables.c"
	trans = (_tmp3_ = sqlite3_column_text (stmt, 0), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
	_tmp4_ = FALSE;
#line 589 "DatabaseTables.vala"
	if (trans != NULL) {
#line 589 "DatabaseTables.vala"
		_tmp4_ = string_get_length (trans) == 0;
#line 2079 "DatabaseTables.c"
	} else {
#line 589 "DatabaseTables.vala"
		_tmp4_ = FALSE;
#line 2083 "DatabaseTables.c"
	}
#line 589 "DatabaseTables.vala"
	if (_tmp4_) {
#line 2087 "DatabaseTables.c"
		result = NULL;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		trans = (g_free (trans), NULL);
		return result;
	}
	result = trans;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 595 "DatabaseTables.vala"
static gboolean photo_table_set_raw_transformations (PhotoTable* self, const PhotoID* photo_id, const char* trans) {
#line 2101 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	const char* _tmp3_;
#line 595 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 595 "DatabaseTables.vala"
	g_return_val_if_fail (trans != NULL, FALSE);
#line 2113 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE PhotoTable SET transformations = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 598 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 600 "DatabaseTables.vala"
	_tmp3_ = NULL;
#line 600 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 1, (_tmp3_ = trans, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), -1, g_free);
#line 601 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 602 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (*photo_id).id);
#line 603 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 605 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 606 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 607 "DatabaseTables.vala"
		database_table_fatal ("set_raw_transformations", res);
#line 2136 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 615 "DatabaseTables.vala"
gboolean photo_table_has_transformations (PhotoTable* self, const PhotoID* photo_id) {
#line 2149 "DatabaseTables.c"
	gboolean result;
	char* _tmp0_;
	gboolean _tmp1_;
#line 615 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 583 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 2157 "DatabaseTables.c"
	result = (_tmp1_ = (_tmp0_ = photo_table_get_raw_transformations (self, &(*photo_id))) != NULL, _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


#line 619 "DatabaseTables.vala"
KeyValueMap* photo_table_get_transformation (PhotoTable* self, const PhotoID* photo_id, const char* object) {
#line 2165 "DatabaseTables.c"
	KeyValueMap* result;
	GError * _inner_error_;
	char* trans;
#line 619 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), NULL);
#line 619 "DatabaseTables.vala"
	g_return_val_if_fail (object != NULL, NULL);
#line 2173 "DatabaseTables.c"
	_inner_error_ = NULL;
	trans = photo_table_get_raw_transformations (self, &(*photo_id));
#line 621 "DatabaseTables.vala"
	if (trans == NULL) {
#line 2178 "DatabaseTables.c"
		result = NULL;
		trans = (g_free (trans), NULL);
		return result;
	}
	{
		GKeyFile* keyfile;
		gboolean _tmp0_;
		gsize count;
		char** _tmp1_;
		gint keys_size;
		gint keys_length1;
		char** keys;
		gboolean _tmp2_;
		KeyValueMap* map;
		keyfile = g_key_file_new ();
		_tmp0_ = g_key_file_load_from_data (keyfile, trans, (gulong) string_get_length (trans), G_KEY_FILE_NONE, &_inner_error_);
		if (_inner_error_ != NULL) {
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			goto __catch13_g_error;
			goto __finally13;
		}
#line 626 "DatabaseTables.vala"
		if (!_tmp0_) {
#line 2202 "DatabaseTables.c"
			result = NULL;
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			trans = (g_free (trans), NULL);
			return result;
		}
#line 629 "DatabaseTables.vala"
		if (!g_key_file_has_group (keyfile, object)) {
#line 2210 "DatabaseTables.c"
			result = NULL;
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			trans = (g_free (trans), NULL);
			return result;
		}
		count = 0UL;
#line 633 "DatabaseTables.vala"
		_tmp1_ = NULL;
#line 2219 "DatabaseTables.c"
		keys = (_tmp1_ = g_key_file_get_keys (keyfile, object, &count, &_inner_error_), keys_length1 = -1, keys_size = keys_length1, _tmp1_);
		if (_inner_error_ != NULL) {
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			goto __catch13_g_error;
			goto __finally13;
		}
		_tmp2_ = FALSE;
#line 634 "DatabaseTables.vala"
		if (keys == NULL) {
#line 634 "DatabaseTables.vala"
			_tmp2_ = TRUE;
#line 2231 "DatabaseTables.c"
		} else {
#line 634 "DatabaseTables.vala"
			_tmp2_ = count == 0;
#line 2235 "DatabaseTables.c"
		}
#line 634 "DatabaseTables.vala"
		if (_tmp2_) {
#line 2239 "DatabaseTables.c"
			result = NULL;
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
			trans = (g_free (trans), NULL);
			return result;
		}
		map = key_value_map_new (object);
		{
			gint ctr;
			ctr = 0;
			{
				gboolean _tmp3_;
				_tmp3_ = TRUE;
#line 638 "DatabaseTables.vala"
				while (TRUE) {
#line 2255 "DatabaseTables.c"
					char* _tmp4_;
					char* _tmp5_;
#line 638 "DatabaseTables.vala"
					if (!_tmp3_) {
#line 638 "DatabaseTables.vala"
						ctr++;
#line 2262 "DatabaseTables.c"
					}
#line 638 "DatabaseTables.vala"
					_tmp3_ = FALSE;
#line 638 "DatabaseTables.vala"
					if (!(ctr < count)) {
#line 638 "DatabaseTables.vala"
						break;
#line 2270 "DatabaseTables.c"
					}
					_tmp4_ = g_key_file_get_string (keyfile, object, keys[ctr], &_inner_error_);
					if (_inner_error_ != NULL) {
						(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
						keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
						(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
						goto __catch13_g_error;
						goto __finally13;
					}
#line 22 "FixedKeyFile.vapi"
					_tmp5_ = NULL;
#line 639 "DatabaseTables.vala"
					key_value_map_set_string (map, keys[ctr], _tmp5_ = _tmp4_);
#line 2284 "DatabaseTables.c"
					_tmp5_ = (g_free (_tmp5_), NULL);
				}
			}
		}
		result = map;
		(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
		keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
		trans = (g_free (trans), NULL);
		return result;
	}
	goto __finally13;
	__catch13_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 643 "DatabaseTables.vala"
			g_error ("DatabaseTables.vala:643: %s", err->message);
#line 2304 "DatabaseTables.c"
			result = NULL;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			trans = (g_free (trans), NULL);
			return result;
		}
	}
	__finally13:
	if (_inner_error_ != NULL) {
		trans = (g_free (trans), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	trans = (g_free (trans), NULL);
}


#line 649 "DatabaseTables.vala"
gboolean photo_table_set_transformation (PhotoTable* self, const PhotoID* photo_id, KeyValueMap* map) {
#line 2324 "DatabaseTables.c"
	gboolean result;
	GError * _inner_error_;
	char* trans;
#line 649 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 649 "DatabaseTables.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (map), FALSE);
#line 2332 "DatabaseTables.c"
	_inner_error_ = NULL;
	trans = photo_table_get_raw_transformations (self, &(*photo_id));
	{
		GKeyFile* keyfile;
		GeeSet* keys;
		gsize length;
		char* _tmp2_;
		keyfile = g_key_file_new ();
#line 654 "DatabaseTables.vala"
		if (trans != NULL) {
#line 2343 "DatabaseTables.c"
			gboolean _tmp0_;
			_tmp0_ = g_key_file_load_from_data (keyfile, trans, (gulong) string_get_length (trans), G_KEY_FILE_NONE, &_inner_error_);
			if (_inner_error_ != NULL) {
				(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
				goto __catch14_g_error;
				goto __finally14;
			}
#line 655 "DatabaseTables.vala"
			if (!_tmp0_) {
#line 2353 "DatabaseTables.c"
				result = FALSE;
				(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
				trans = (g_free (trans), NULL);
				return result;
			}
		}
		keys = key_value_map_get_keys (map);
		{
			GeeIterator* _key_it;
			_key_it = gee_iterable_iterator (GEE_ITERABLE (keys));
#line 660 "DatabaseTables.vala"
			while (TRUE) {
#line 2366 "DatabaseTables.c"
				char* key;
				char* value;
				char* _tmp1_;
#line 660 "DatabaseTables.vala"
				if (!gee_iterator_next (_key_it)) {
#line 660 "DatabaseTables.vala"
					break;
#line 2374 "DatabaseTables.c"
				}
				key = (char*) gee_iterator_get (_key_it);
				value = key_value_map_get_string (map, key, NULL);
#line 662 "DatabaseTables.vala"
				g_assert (value != NULL);
#line 47 "util.vala"
				_tmp1_ = NULL;
#line 664 "DatabaseTables.vala"
				g_key_file_set_string (keyfile, _tmp1_ = key_value_map_get_group (map), key, value);
#line 2384 "DatabaseTables.c"
				_tmp1_ = (g_free (_tmp1_), NULL);
				key = (g_free (key), NULL);
				value = (g_free (value), NULL);
			}
			(_key_it == NULL) ? NULL : (_key_it = (g_object_unref (_key_it), NULL));
		}
		length = 0UL;
		_tmp2_ = NULL;
#line 668 "DatabaseTables.vala"
		trans = (_tmp2_ = g_key_file_to_data (keyfile, &length, NULL), trans = (g_free (trans), NULL), _tmp2_);
#line 669 "DatabaseTables.vala"
		g_assert (trans != NULL);
#line 670 "DatabaseTables.vala"
		g_assert (string_get_length (trans) > 0);
#line 2399 "DatabaseTables.c"
		(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
		(keys == NULL) ? NULL : (keys = (g_object_unref (keys), NULL));
	}
	goto __finally14;
	__catch14_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 672 "DatabaseTables.vala"
			g_error ("DatabaseTables.vala:672: %s", err->message);
#line 2412 "DatabaseTables.c"
			result = FALSE;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			trans = (g_free (trans), NULL);
			return result;
		}
	}
	__finally14:
	if (_inner_error_ != NULL) {
		trans = (g_free (trans), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	result = photo_table_set_raw_transformations (self, &(*photo_id), trans);
	trans = (g_free (trans), NULL);
	return result;
}


#line 680 "DatabaseTables.vala"
gboolean photo_table_remove_transformation (PhotoTable* self, const PhotoID* photo_id, const char* object) {
#line 2434 "DatabaseTables.c"
	gboolean result;
	GError * _inner_error_;
	char* trans;
#line 680 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 680 "DatabaseTables.vala"
	g_return_val_if_fail (object != NULL, FALSE);
#line 2442 "DatabaseTables.c"
	_inner_error_ = NULL;
	trans = photo_table_get_raw_transformations (self, &(*photo_id));
#line 682 "DatabaseTables.vala"
	if (trans == NULL) {
#line 2447 "DatabaseTables.c"
		result = TRUE;
		trans = (g_free (trans), NULL);
		return result;
	}
	{
		GKeyFile* keyfile;
		gboolean _tmp0_;
		gsize length;
		char* _tmp1_;
		keyfile = g_key_file_new ();
		_tmp0_ = g_key_file_load_from_data (keyfile, trans, (gulong) string_get_length (trans), G_KEY_FILE_NONE, &_inner_error_);
		if (_inner_error_ != NULL) {
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			goto __catch15_g_error;
			goto __finally15;
		}
#line 687 "DatabaseTables.vala"
		if (!_tmp0_) {
#line 2466 "DatabaseTables.c"
			result = FALSE;
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			trans = (g_free (trans), NULL);
			return result;
		}
#line 690 "DatabaseTables.vala"
		g_key_file_remove_group (keyfile, object, &_inner_error_);
#line 2474 "DatabaseTables.c"
		if (_inner_error_ != NULL) {
			(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
			goto __catch15_g_error;
			goto __finally15;
		}
		length = 0UL;
		_tmp1_ = NULL;
#line 693 "DatabaseTables.vala"
		trans = (_tmp1_ = g_key_file_to_data (keyfile, &length, NULL), trans = (g_free (trans), NULL), _tmp1_);
#line 694 "DatabaseTables.vala"
		g_assert (trans != NULL);
#line 2486 "DatabaseTables.c"
		(keyfile == NULL) ? NULL : (keyfile = (g_key_file_free (keyfile), NULL));
	}
	goto __finally15;
	__catch15_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 696 "DatabaseTables.vala"
			g_error ("DatabaseTables.vala:696: %s", err->message);
#line 2498 "DatabaseTables.c"
			result = FALSE;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			trans = (g_free (trans), NULL);
			return result;
		}
	}
	__finally15:
	if (_inner_error_ != NULL) {
		trans = (g_free (trans), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	result = photo_table_set_raw_transformations (self, &(*photo_id), trans);
	trans = (g_free (trans), NULL);
	return result;
}


#line 704 "DatabaseTables.vala"
gboolean photo_table_remove_all_transformations (PhotoTable* self, const PhotoID* photo_id) {
#line 2520 "DatabaseTables.c"
	gboolean result;
	char* _tmp0_;
	gboolean _tmp1_;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp4_;
	gint _tmp3_;
	sqlite3_stmt* _tmp2_;
	gint res;
#line 704 "DatabaseTables.vala"
	g_return_val_if_fail (IS_PHOTO_TABLE (self), FALSE);
#line 583 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 705 "DatabaseTables.vala"
	if ((_tmp1_ = (_tmp0_ = photo_table_get_raw_transformations (self, &(*photo_id))) == NULL, _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_)) {
#line 2535 "DatabaseTables.c"
		result = FALSE;
		return result;
	}
	stmt = NULL;
	_tmp4_ = NULL;
	_tmp2_ = NULL;
	res = (_tmp3_ = sqlite3_prepare_v2 (database_table_db, "UPDATE PhotoTable SET transformations='' WHERE id=?", -1, &_tmp2_, NULL), stmt = (_tmp4_ = _tmp2_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp4_), _tmp3_);
#line 710 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 712 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 713 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 715 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 716 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 717 "DatabaseTables.vala"
		database_table_fatal ("remove_all_transformations", res);
#line 2555 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


static void photo_table_class_init (PhotoTableClass * klass) {
	photo_table_parent_class = g_type_class_peek_parent (klass);
}


static void photo_table_instance_init (PhotoTable * self) {
}


GType photo_table_get_type (void) {
	static GType photo_table_type_id = 0;
	if (photo_table_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PhotoTableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_table_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PhotoTable), 0, (GInstanceInitFunc) photo_table_instance_init, NULL };
		photo_table_type_id = g_type_register_static (TYPE_DATABASE_TABLE, "PhotoTable", &g_define_type_info, 0);
	}
	return photo_table_type_id;
}


#line 727 "DatabaseTables.vala"
ThumbnailCacheTable* thumbnail_cache_table_construct (GType object_type, gint scale) {
#line 2587 "DatabaseTables.c"
	ThumbnailCacheTable* self;
	char* _tmp0_;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp11_;
	gint _tmp10_;
	sqlite3_stmt* _tmp9_;
	char* _tmp8_;
	char* _tmp7_;
	char* _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	gint _tmp12_;
	gint res;
	self = (ThumbnailCacheTable*) g_type_create_instance (object_type);
#line 728 "DatabaseTables.vala"
	g_assert (scale > 0);
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 730 "DatabaseTables.vala"
	database_table_set_table_name (DATABASE_TABLE (self), _tmp0_ = g_strdup_printf ("Thumb%dTable", scale));
#line 2611 "DatabaseTables.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	stmt = NULL;
	_tmp11_ = NULL;
	_tmp9_ = NULL;
	_tmp8_ = NULL;
	_tmp7_ = NULL;
	_tmp6_ = NULL;
	_tmp5_ = NULL;
	_tmp4_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	res = (_tmp12_ = (_tmp10_ = sqlite3_prepare_v2 (database_table_db, _tmp8_ = g_strconcat (_tmp7_ = g_strconcat (_tmp6_ = g_strconcat (_tmp5_ = g_strconcat (_tmp4_ = g_strconcat (_tmp3_ = g_strconcat (_tmp2_ = g_strconcat (_tmp1_ = g_strconcat ("CREATE TABLE IF NOT EXISTS ", DATABASE_TABLE (self)->table_name, NULL), "(", NULL), "id INTEGER PRIMARY KEY, ", NULL), "photo_id INTEGER UNIQUE, ", NULL), "width INTEGER, ", NULL), "height INTEGER, ", NULL), "filesize INTEGER", NULL), ")", NULL), -1, &_tmp9_, NULL), stmt = (_tmp11_ = _tmp9_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp11_), _tmp10_), _tmp8_ = (g_free (_tmp8_), NULL), _tmp7_ = (g_free (_tmp7_), NULL), _tmp6_ = (g_free (_tmp6_), NULL), _tmp5_ = (g_free (_tmp5_), NULL), _tmp4_ = (g_free (_tmp4_), NULL), _tmp3_ = (g_free (_tmp3_), NULL), _tmp2_ = (g_free (_tmp2_), NULL), _tmp1_ = (g_free (_tmp1_), NULL), _tmp12_);
#line 742 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 744 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 745 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 2631 "DatabaseTables.c"
		char* _tmp13_;
#line 820 "glib-2.0.vapi"
		_tmp13_ = NULL;
#line 746 "DatabaseTables.vala"
		database_table_fatal (_tmp13_ = g_strdup_printf ("create %s", DATABASE_TABLE (self)->table_name), res);
#line 2637 "DatabaseTables.c"
		_tmp13_ = (g_free (_tmp13_), NULL);
	}
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return self;
}


#line 727 "DatabaseTables.vala"
ThumbnailCacheTable* thumbnail_cache_table_new (gint scale) {
#line 727 "DatabaseTables.vala"
	return thumbnail_cache_table_construct (TYPE_THUMBNAIL_CACHE_TABLE, scale);
#line 2649 "DatabaseTables.c"
}


#line 749 "DatabaseTables.vala"
gboolean thumbnail_cache_table_remove (ThumbnailCacheTable* self, const PhotoID* photo_id) {
#line 2655 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
#line 749 "DatabaseTables.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE_TABLE (self), FALSE);
#line 2666 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 2672 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("DELETE FROM %s WHERE photo_id=?", DATABASE_TABLE (self)->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 752 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 754 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 755 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 757 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 758 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 2684 "DatabaseTables.c"
		char* _tmp5_;
#line 820 "glib-2.0.vapi"
		_tmp5_ = NULL;
#line 759 "DatabaseTables.vala"
		database_table_warning (_tmp5_ = g_strdup_printf ("%s remove", DATABASE_TABLE (self)->table_name), res);
#line 2690 "DatabaseTables.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 767 "DatabaseTables.vala"
gboolean thumbnail_cache_table_exists (ThumbnailCacheTable* self, const PhotoID* photo_id) {
#line 2704 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
#line 767 "DatabaseTables.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE_TABLE (self), FALSE);
#line 2715 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 2721 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("SELECT id FROM %s WHERE photo_id=?", DATABASE_TABLE (self)->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 770 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 772 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 773 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 775 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 776 "DatabaseTables.vala"
	if (res != SQLITE_ROW) {
#line 777 "DatabaseTables.vala"
		if (res != SQLITE_DONE) {
#line 2735 "DatabaseTables.c"
			char* _tmp5_;
#line 820 "glib-2.0.vapi"
			_tmp5_ = NULL;
#line 778 "DatabaseTables.vala"
			database_table_fatal (_tmp5_ = g_strdup_printf ("%s exists", DATABASE_TABLE (self)->table_name), res);
#line 2741 "DatabaseTables.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
		}
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 786 "DatabaseTables.vala"
void thumbnail_cache_table_add (ThumbnailCacheTable* self, const PhotoID* photo_id, gint filesize, const Dimensions* dim) {
#line 2756 "DatabaseTables.c"
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
#line 786 "DatabaseTables.vala"
	g_return_if_fail (IS_THUMBNAIL_CACHE_TABLE (self));
#line 2766 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 2772 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("INSERT INTO %s (photo_id, filesize, width, height) VALUES (?, ?, ?, ?)", DATABASE_TABLE (self)->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 791 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 793 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 794 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 795 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 2, filesize);
#line 796 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 797 "DatabaseTables.vala"
	sqlite3_bind_int (stmt, 3, (*dim).width);
#line 798 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 799 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 4, (*dim).height);
#line 800 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 802 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 803 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 2796 "DatabaseTables.c"
		char* _tmp5_;
#line 820 "glib-2.0.vapi"
		_tmp5_ = NULL;
#line 804 "DatabaseTables.vala"
		database_table_fatal (_tmp5_ = g_strdup_printf ("%s add", DATABASE_TABLE (self)->table_name), res);
#line 2802 "DatabaseTables.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
	}
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 807 "DatabaseTables.vala"
void thumbnail_cache_table_replace (ThumbnailCacheTable* self, const PhotoID* photo_id, gint filesize, const Dimensions* dim) {
#line 2811 "DatabaseTables.c"
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
#line 807 "DatabaseTables.vala"
	g_return_if_fail (IS_THUMBNAIL_CACHE_TABLE (self));
#line 2821 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 2827 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("UPDATE %s SET filesize=?, width=?, height=? WHERE photo_id=?", DATABASE_TABLE (self)->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 812 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 814 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 1, filesize);
#line 815 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 816 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 2, (*dim).width);
#line 817 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 818 "DatabaseTables.vala"
	res = sqlite3_bind_int (stmt, 3, (*dim).height);
#line 819 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 820 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 4, (*photo_id).id);
#line 821 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 823 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 824 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 2851 "DatabaseTables.c"
		char* _tmp5_;
#line 820 "glib-2.0.vapi"
		_tmp5_ = NULL;
#line 825 "DatabaseTables.vala"
		database_table_fatal (_tmp5_ = g_strdup_printf ("%s replace", DATABASE_TABLE (self)->table_name), res);
#line 2857 "DatabaseTables.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
	}
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 828 "DatabaseTables.vala"
Dimensions thumbnail_cache_table_get_dimensions (ThumbnailCacheTable* self, const PhotoID* photo_id) {
#line 2866 "DatabaseTables.c"
	Dimensions result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
	Dimensions _tmp7_ = {0};
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 2881 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("SELECT width, height FROM %s WHERE photo_id=?", DATABASE_TABLE (self)->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 832 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 834 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 835 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 837 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 838 "DatabaseTables.vala"
	if (res != SQLITE_ROW) {
#line 2893 "DatabaseTables.c"
		Dimensions _tmp6_ = {0};
#line 839 "DatabaseTables.vala"
		if (res != SQLITE_DONE) {
#line 2897 "DatabaseTables.c"
			char* _tmp5_;
#line 820 "glib-2.0.vapi"
			_tmp5_ = NULL;
#line 840 "DatabaseTables.vala"
			database_table_fatal (_tmp5_ = g_strdup_printf ("%s get_dimensions", DATABASE_TABLE (self)->table_name), res);
#line 2903 "DatabaseTables.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
		}
		result = (dimensions_init (&_tmp6_, 0, 0), _tmp6_);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (dimensions_init (&_tmp7_, sqlite3_column_int (stmt, 0), sqlite3_column_int (stmt, 1)), _tmp7_);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 848 "DatabaseTables.vala"
gint thumbnail_cache_table_get_filesize (ThumbnailCacheTable* self, const PhotoID* photo_id) {
#line 2919 "DatabaseTables.c"
	gint result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	char* _tmp0_;
	gint _tmp4_;
	gint res;
#line 848 "DatabaseTables.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE_TABLE (self), 0);
#line 2930 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 2936 "DatabaseTables.c"
	res = (_tmp4_ = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, _tmp0_ = g_strdup_printf ("SELECT filesize FROM %s WHERE photo_id=?", DATABASE_TABLE (self)->table_name), -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp4_);
#line 852 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 854 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 855 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 857 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 858 "DatabaseTables.vala"
	if (res != SQLITE_ROW) {
#line 859 "DatabaseTables.vala"
		if (res != SQLITE_DONE) {
#line 2950 "DatabaseTables.c"
			char* _tmp5_;
#line 820 "glib-2.0.vapi"
			_tmp5_ = NULL;
#line 860 "DatabaseTables.vala"
			database_table_fatal (_tmp5_ = g_strdup_printf ("%s get_filesize", DATABASE_TABLE (self)->table_name), res);
#line 2956 "DatabaseTables.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
		}
		result = -1;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = sqlite3_column_int (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


static void thumbnail_cache_table_class_init (ThumbnailCacheTableClass * klass) {
	thumbnail_cache_table_parent_class = g_type_class_peek_parent (klass);
}


static void thumbnail_cache_table_instance_init (ThumbnailCacheTable * self) {
}


GType thumbnail_cache_table_get_type (void) {
	static GType thumbnail_cache_table_type_id = 0;
	if (thumbnail_cache_table_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ThumbnailCacheTableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) thumbnail_cache_table_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ThumbnailCacheTable), 0, (GInstanceInitFunc) thumbnail_cache_table_instance_init, NULL };
		thumbnail_cache_table_type_id = g_type_register_static (TYPE_DATABASE_TABLE, "ThumbnailCacheTable", &g_define_type_info, 0);
	}
	return thumbnail_cache_table_type_id;
}


#line 874 "DatabaseTables.vala"
void event_id_init (EventID *self, gint64 id) {
#line 2990 "DatabaseTables.c"
	memset (self, 0, sizeof (EventID));
#line 875 "DatabaseTables.vala"
	(*self).id = id;
#line 2994 "DatabaseTables.c"
}


#line 878 "DatabaseTables.vala"
gboolean event_id_is_invalid (EventID *self) {
#line 3000 "DatabaseTables.c"
	gboolean result;
	result = (*self).id == EVENT_ID_INVALID;
	return result;
}


#line 882 "DatabaseTables.vala"
gboolean event_id_is_valid (EventID *self) {
#line 3009 "DatabaseTables.c"
	gboolean result;
	result = (*self).id != EVENT_ID_INVALID;
	return result;
}


EventID* event_id_dup (const EventID* self) {
	EventID* dup;
	dup = g_new0 (EventID, 1);
	memcpy (dup, self, sizeof (EventID));
	return dup;
}


void event_id_free (EventID* self) {
	g_free (self);
}


GType event_id_get_type (void) {
	static GType event_id_type_id = 0;
	if (event_id_type_id == 0) {
		event_id_type_id = g_boxed_type_register_static ("EventID", (GBoxedCopyFunc) event_id_dup, (GBoxedFreeFunc) event_id_free);
	}
	return event_id_type_id;
}


#line 888 "DatabaseTables.vala"
EventTable* event_table_construct (GType object_type) {
#line 3040 "DatabaseTables.c"
	EventTable* self;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	self = (EventTable*) g_type_create_instance (object_type);
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "CREATE TABLE IF NOT EXISTS EventTable (" "id INTEGER PRIMARY KEY, " "name TEXT, " "primary_photo_id INTEGER, " "start_time INTEGER, " "end_time INTEGER, " "time_created INTEGER" ")", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 898 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 900 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 901 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 902 "DatabaseTables.vala"
		database_table_fatal ("create photo table", res);
#line 3060 "DatabaseTables.c"
	}
#line 904 "DatabaseTables.vala"
	database_table_set_table_name (DATABASE_TABLE (self), "EventTable");
#line 3064 "DatabaseTables.c"
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return self;
}


#line 888 "DatabaseTables.vala"
EventTable* event_table_new (void) {
#line 888 "DatabaseTables.vala"
	return event_table_construct (TYPE_EVENT_TABLE);
#line 3074 "DatabaseTables.c"
}


#line 907 "DatabaseTables.vala"
EventID event_table_create (EventTable* self, const PhotoID* primary_photo_id, const time_t* start_time) {
#line 3080 "DatabaseTables.c"
	EventID result;
	GTimeVal _tmp0_ = {0};
	GTimeVal time_created;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp3_;
	gint _tmp2_;
	sqlite3_stmt* _tmp1_;
	gint res;
	EventID _tmp5_ = {0};
#line 908 "DatabaseTables.vala"
	g_assert (photo_id_is_valid (&(*primary_photo_id)));
#line 909 "DatabaseTables.vala"
	g_assert ((*start_time) != 0);
#line 3094 "DatabaseTables.c"
	time_created = (g_get_current_time (&_tmp0_), _tmp0_);
#line 912 "DatabaseTables.vala"
	g_get_current_time (&time_created);
#line 3098 "DatabaseTables.c"
	stmt = NULL;
	_tmp3_ = NULL;
	_tmp1_ = NULL;
	res = (_tmp2_ = sqlite3_prepare_v2 (database_table_db, "INSERT INTO EventTable (primary_photo_id, time_created, start_time) VALUES (?, ?, ?)", -1, &_tmp1_, NULL), stmt = (_tmp3_ = _tmp1_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp3_), _tmp2_);
#line 918 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 920 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*primary_photo_id).id);
#line 921 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 922 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (gint64) time_created.tv_sec);
#line 923 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 924 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 3, (gint64) (*start_time));
#line 925 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 927 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 928 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 3121 "DatabaseTables.c"
		EventID _tmp4_ = {0};
#line 929 "DatabaseTables.vala"
		database_table_fatal ("create_event", res);
#line 3125 "DatabaseTables.c"
		result = (event_id_init (&_tmp4_, EVENT_ID_INVALID), _tmp4_);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (event_id_init (&_tmp5_, sqlite3_last_insert_rowid (database_table_db)), _tmp5_);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 937 "DatabaseTables.vala"
gboolean event_table_remove (EventTable* self, const EventID* event_id) {
#line 3139 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 937 "DatabaseTables.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (self), FALSE);
#line 3148 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "DELETE FROM EventTable WHERE id=?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 940 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 942 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*event_id).id);
#line 943 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 945 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 946 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 947 "DatabaseTables.vala"
		database_table_warning ("event remove", res);
#line 3165 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 955 "DatabaseTables.vala"
time_t event_table_get_start_time (EventTable* self, const EventID* event_id) {
#line 3178 "DatabaseTables.c"
	time_t result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 957 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*event_id).id, "start_time", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 3189 "DatabaseTables.c"
		result = 0;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (time_t) sqlite3_column_int64 (stmt, 0);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 963 "DatabaseTables.vala"
gboolean event_table_set_end_time (EventTable* self, const EventID* event_id, const time_t* end_time) {
#line 3202 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 963 "DatabaseTables.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (self), FALSE);
#line 3211 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE EventTable SET end_time = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 966 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 968 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (gint64) (*end_time));
#line 969 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 970 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (*event_id).id);
#line 971 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 973 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 974 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 975 "DatabaseTables.vala"
		database_table_fatal ("set_end_time", res);
#line 3232 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 983 "DatabaseTables.vala"
GeeArrayList* event_table_get_events (EventTable* self) {
#line 3245 "DatabaseTables.c"
	GeeArrayList* result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	GeeArrayList* event_ids;
#line 983 "DatabaseTables.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (self), NULL);
#line 3255 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "SELECT id FROM EventTable", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 986 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 3262 "DatabaseTables.c"
	event_ids = gee_array_list_new (TYPE_EVENT_ID, (GBoxedCopyFunc) event_id_dup, event_id_free, g_direct_equal);
	{
		gboolean _tmp3_;
		_tmp3_ = TRUE;
#line 989 "DatabaseTables.vala"
		while (TRUE) {
#line 3269 "DatabaseTables.c"
			EventID _tmp5_ = {0};
			EventID _tmp4_ = {0};
#line 989 "DatabaseTables.vala"
			if (!_tmp3_) {
#line 3274 "DatabaseTables.c"
			}
#line 989 "DatabaseTables.vala"
			_tmp3_ = FALSE;
#line 990 "DatabaseTables.vala"
			res = sqlite3_step (stmt);
#line 991 "DatabaseTables.vala"
			if (res == SQLITE_DONE) {
#line 992 "DatabaseTables.vala"
				break;
#line 3284 "DatabaseTables.c"
			} else {
#line 993 "DatabaseTables.vala"
				if (res != SQLITE_ROW) {
#line 994 "DatabaseTables.vala"
					database_table_fatal ("get_events", res);
#line 996 "DatabaseTables.vala"
					break;
#line 3292 "DatabaseTables.c"
				}
			}
#line 999 "DatabaseTables.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (event_ids), (_tmp5_ = (event_id_init (&_tmp4_, sqlite3_column_int64 (stmt, 0)), _tmp4_), &_tmp5_));
#line 3297 "DatabaseTables.c"
		}
	}
	result = event_ids;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 1005 "DatabaseTables.vala"
gboolean event_table_rename (EventTable* self, const EventID* event_id, const char* name) {
#line 3308 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
	const char* _tmp3_;
#line 1005 "DatabaseTables.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (self), FALSE);
#line 1005 "DatabaseTables.vala"
	g_return_val_if_fail (name != NULL, FALSE);
#line 3320 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE EventTable SET name = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 1008 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1010 "DatabaseTables.vala"
	_tmp3_ = NULL;
#line 1010 "DatabaseTables.vala"
	res = sqlite3_bind_text (stmt, 1, (_tmp3_ = name, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), -1, g_free);
#line 1011 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1012 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (*event_id).id);
#line 1013 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1015 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 1016 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 1017 "DatabaseTables.vala"
		database_table_fatal ("event rename", res);
#line 3343 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 1918 "glib-2.0.vapi"
static char* g_time_format (struct tm *self, const char* format) {
#line 3356 "DatabaseTables.c"
	char* result;
	gchar* _tmp0_;
	gint buffer_size;
	gint buffer_length1;
	gchar* buffer;
	const char* _tmp1_;
#line 1918 "glib-2.0.vapi"
	g_return_val_if_fail (format != NULL, NULL);
#line 1919 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 3367 "DatabaseTables.c"
	buffer = (_tmp0_ = g_new0 (gchar, 64), buffer_length1 = 64, buffer_size = buffer_length1, _tmp0_);
#line 1920 "glib-2.0.vapi"
	strftime (buffer, buffer_length1, format, &(*self));
#line 1921 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 3373 "DatabaseTables.c"
	result = (_tmp1_ = (const char*) buffer, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	buffer = (g_free (buffer), NULL);
	return result;
	buffer = (g_free (buffer), NULL);
}


#line 1025 "DatabaseTables.vala"
char* event_table_get_name (EventTable* self, const EventID* event_id) {
#line 3383 "DatabaseTables.c"
	char* result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	const char* _tmp3_;
	char* name;
	gboolean _tmp4_;
#line 1025 "DatabaseTables.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (self), NULL);
#line 3394 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 1027 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*event_id).id, "name, start_time", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 3400 "DatabaseTables.c"
		result = NULL;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	/* if no name, pretty up the start time*/
#line 1031 "DatabaseTables.vala"
	_tmp3_ = NULL;
#line 3408 "DatabaseTables.c"
	name = (_tmp3_ = sqlite3_column_text (stmt, 0), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
	_tmp4_ = FALSE;
#line 1032 "DatabaseTables.vala"
	if (name == NULL) {
#line 1032 "DatabaseTables.vala"
		_tmp4_ = TRUE;
#line 3415 "DatabaseTables.c"
	} else {
#line 1032 "DatabaseTables.vala"
		_tmp4_ = string_get_length (name) == 0;
#line 3419 "DatabaseTables.c"
	}
#line 1032 "DatabaseTables.vala"
	if (_tmp4_) {
#line 3423 "DatabaseTables.c"
		gint64 timet;
		time_t _tmp6_ = {0};
		struct tm _tmp5_ = {0};
		struct tm start_time;
		char* _tmp9_;
		char* _tmp8_;
		char* _tmp7_;
		timet = sqlite3_column_int64 (stmt, 1);
#line 1034 "DatabaseTables.vala"
		g_assert (timet != 0);
#line 3434 "DatabaseTables.c"
		start_time = (localtime_r ((_tmp6_ = (time_t) timet, &_tmp6_), &_tmp5_), _tmp5_);
		_tmp9_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp8_ = NULL;
#line 1918 "glib-2.0.vapi"
		_tmp7_ = NULL;
#line 1037 "DatabaseTables.vala"
		name = (_tmp9_ = g_strconcat (_tmp7_ = g_time_format (&start_time, "%a %b"), _tmp8_ = g_strdup_printf (" %d, %d", start_time.tm_mday, 1900 + start_time.tm_year), NULL), name = (g_free (name), NULL), _tmp9_);
#line 3443 "DatabaseTables.c"
		_tmp8_ = (g_free (_tmp8_), NULL);
		_tmp7_ = (g_free (_tmp7_), NULL);
	}
	result = name;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


#line 1044 "DatabaseTables.vala"
PhotoID event_table_get_primary_photo (EventTable* self, const EventID* event_id) {
#line 3455 "DatabaseTables.c"
	PhotoID result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gboolean _tmp1_;
	sqlite3_stmt* _tmp0_;
	PhotoID _tmp4_ = {0};
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 1046 "DatabaseTables.vala"
	if (!(_tmp1_ = database_table_select_by_id (DATABASE_TABLE (self), (*event_id).id, "primary_photo_id", &_tmp0_), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_)) {
#line 3467 "DatabaseTables.c"
		PhotoID _tmp3_ = {0};
		result = (photo_id_init (&_tmp3_, PHOTO_ID_INVALID), _tmp3_);
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = (photo_id_init (&_tmp4_, sqlite3_column_int64 (stmt, 0)), _tmp4_);
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
}


#line 1052 "DatabaseTables.vala"
gboolean event_table_set_primary_photo (EventTable* self, const EventID* event_id, const PhotoID* photo_id) {
#line 3482 "DatabaseTables.c"
	gboolean result;
	sqlite3_stmt* stmt;
	sqlite3_stmt* _tmp2_;
	gint _tmp1_;
	sqlite3_stmt* _tmp0_;
	gint res;
#line 1052 "DatabaseTables.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (self), FALSE);
#line 3491 "DatabaseTables.c"
	stmt = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = sqlite3_prepare_v2 (database_table_db, "UPDATE EventTable SET primary_photo_id = ? WHERE id = ?", -1, &_tmp0_, NULL), stmt = (_tmp2_ = _tmp0_, (stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL)), _tmp2_), _tmp1_);
#line 1055 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1057 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 1, (*photo_id).id);
#line 1058 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1059 "DatabaseTables.vala"
	res = sqlite3_bind_int64 (stmt, 2, (*event_id).id);
#line 1060 "DatabaseTables.vala"
	g_assert (res == SQLITE_OK);
#line 1062 "DatabaseTables.vala"
	res = sqlite3_step (stmt);
#line 1063 "DatabaseTables.vala"
	if (res != SQLITE_DONE) {
#line 1064 "DatabaseTables.vala"
		database_table_fatal ("set_primary_photo", res);
#line 3512 "DatabaseTables.c"
		result = FALSE;
		(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
		return result;
	}
	result = TRUE;
	(stmt == NULL) ? NULL : (stmt = (sqlite3_finalize (stmt), NULL));
	return result;
}


static void event_table_class_init (EventTableClass * klass) {
	event_table_parent_class = g_type_class_peek_parent (klass);
}


static void event_table_instance_init (EventTable * self) {
}


GType event_table_get_type (void) {
	static GType event_table_type_id = 0;
	if (event_table_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (EventTableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) event_table_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EventTable), 0, (GInstanceInitFunc) event_table_instance_init, NULL };
		event_table_type_id = g_type_register_static (TYPE_DATABASE_TABLE, "EventTable", &g_define_type_info, 0);
	}
	return event_table_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




