/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <float.h>
#include <math.h>


#define TYPE_SCALE_CONSTRAINT (scale_constraint_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

typedef enum  {
	SCALE_CONSTRAINT_ORIGINAL,
	SCALE_CONSTRAINT_DIMENSIONS,
	SCALE_CONSTRAINT_WIDTH,
	SCALE_CONSTRAINT_HEIGHT
} ScaleConstraint;

struct _Dimensions {
	gint width;
	gint height;
};



GType scale_constraint_get_type (void);
char* scale_constraint_to_string (ScaleConstraint self);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
void dimensions_init (Dimensions *self, gint width, gint height);
Dimensions dimensions_for_pixbuf (GdkPixbuf* pixbuf);
Dimensions dimensions_for_allocation (const GtkAllocation* allocation);
Dimensions dimensions_for_rectangle (const GdkRectangle* rect);
gboolean dimensions_has_area (Dimensions *self);
char* dimensions_to_string (Dimensions *self);
Dimensions dimensions_get_scaled_by_height (Dimensions *self, gint scale);
Dimensions dimensions_get_scaled_by_width (Dimensions *self, gint scale);
Dimensions dimensions_get_scaled (Dimensions *self, gint scale);
Dimensions dimensions_get_scaled_proportional (Dimensions *self, const Dimensions* viewport);
GdkRectangle dimensions_get_scaled_rectangle (Dimensions *self, const Dimensions* scale, const GdkRectangle* rect);
Dimensions dimensions_get_scaled_by_constraint (Dimensions *self, gint scale, ScaleConstraint constraint);



#line 13 "Dimensions.vala"
char* scale_constraint_to_string (ScaleConstraint self) {
#line 60 "Dimensions.c"
	char* result;
#line 14 "Dimensions.vala"
	switch (self) {
#line 64 "Dimensions.c"
		case SCALE_CONSTRAINT_ORIGINAL:
		{
			result = g_strdup ("Original size");
			return result;
		}
		case SCALE_CONSTRAINT_DIMENSIONS:
		{
			result = g_strdup ("Width or height");
			return result;
		}
		case SCALE_CONSTRAINT_WIDTH:
		{
			result = g_strdup ("Width");
			return result;
		}
		case SCALE_CONSTRAINT_HEIGHT:
		{
			result = g_strdup ("Height");
			return result;
		}
	}
#line 28 "Dimensions.vala"
	g_warn_if_reached ();
#line 88 "Dimensions.c"
	result = NULL;
	return result;
}



GType scale_constraint_get_type (void) {
	static GType scale_constraint_type_id = 0;
	if (G_UNLIKELY (scale_constraint_type_id == 0)) {
		static const GEnumValue values[] = {{SCALE_CONSTRAINT_ORIGINAL, "SCALE_CONSTRAINT_ORIGINAL", "original"}, {SCALE_CONSTRAINT_DIMENSIONS, "SCALE_CONSTRAINT_DIMENSIONS", "dimensions"}, {SCALE_CONSTRAINT_WIDTH, "SCALE_CONSTRAINT_WIDTH", "width"}, {SCALE_CONSTRAINT_HEIGHT, "SCALE_CONSTRAINT_HEIGHT", "height"}, {0, NULL, NULL}};
		scale_constraint_type_id = g_enum_register_static ("ScaleConstraint", values);
	}
	return scale_constraint_type_id;
}


#line 38 "Dimensions.vala"
void dimensions_init (Dimensions *self, gint width, gint height) {
#line 107 "Dimensions.c"
	gboolean _tmp0_;
	memset (self, 0, sizeof (Dimensions));
	_tmp0_ = FALSE;
#line 39 "Dimensions.vala"
	if (width >= 0) {
#line 39 "Dimensions.vala"
		_tmp0_ = height >= 0;
#line 115 "Dimensions.c"
	} else {
#line 39 "Dimensions.vala"
		_tmp0_ = FALSE;
#line 119 "Dimensions.c"
	}
#line 39 "Dimensions.vala"
	g_assert (_tmp0_);
#line 41 "Dimensions.vala"
	(*self).width = width;
#line 42 "Dimensions.vala"
	(*self).height = height;
#line 127 "Dimensions.c"
}


#line 45 "Dimensions.vala"
Dimensions dimensions_for_pixbuf (GdkPixbuf* pixbuf) {
#line 133 "Dimensions.c"
	Dimensions result;
	Dimensions _tmp0_ = {0};
	result = (dimensions_init (&_tmp0_, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf)), _tmp0_);
	return result;
}


#line 49 "Dimensions.vala"
Dimensions dimensions_for_allocation (const GtkAllocation* allocation) {
#line 143 "Dimensions.c"
	Dimensions result;
	Dimensions _tmp0_ = {0};
	result = (dimensions_init (&_tmp0_, (*allocation).width, (*allocation).height), _tmp0_);
	return result;
}


#line 53 "Dimensions.vala"
Dimensions dimensions_for_rectangle (const GdkRectangle* rect) {
#line 153 "Dimensions.c"
	Dimensions result;
	Dimensions _tmp0_ = {0};
	result = (dimensions_init (&_tmp0_, (*rect).width, (*rect).height), _tmp0_);
	return result;
}


#line 57 "Dimensions.vala"
gboolean dimensions_has_area (Dimensions *self) {
#line 163 "Dimensions.c"
	gboolean result;
	gboolean _tmp0_;
	_tmp0_ = FALSE;
#line 58 "Dimensions.vala"
	if ((*self).width > 0) {
#line 58 "Dimensions.vala"
		_tmp0_ = (*self).height > 0;
#line 171 "Dimensions.c"
	} else {
#line 58 "Dimensions.vala"
		_tmp0_ = FALSE;
#line 175 "Dimensions.c"
	}
	result = _tmp0_;
	return result;
}


#line 61 "Dimensions.vala"
char* dimensions_to_string (Dimensions *self) {
#line 184 "Dimensions.c"
	char* result;
	result = g_strdup_printf ("%dx%d", (*self).width, (*self).height);
	return result;
}


#line 65 "Dimensions.vala"
Dimensions dimensions_get_scaled (Dimensions *self, gint scale) {
#line 193 "Dimensions.c"
	Dimensions result;
	gint diff_width;
	gint diff_height;
#line 66 "Dimensions.vala"
	g_assert (scale > 0);
#line 199 "Dimensions.c"
	diff_width = (*self).width - scale;
	diff_height = (*self).height - scale;
#line 71 "Dimensions.vala"
	if (diff_width == diff_height) {
#line 204 "Dimensions.c"
		Dimensions _tmp0_ = {0};
		/* square image -- unlikely -- but this is the easy case*/
		result = (dimensions_init (&_tmp0_, scale, scale), _tmp0_);
		return result;
	} else {
#line 74 "Dimensions.vala"
		if (diff_width <= 0) {
#line 75 "Dimensions.vala"
			if (diff_height <= 0) {
#line 214 "Dimensions.c"
				Dimensions _tmp1_ = {0};
				/* if both dimensions are less than the scaled size, return as-is*/
				result = (dimensions_init (&_tmp1_, (*self).width, (*self).height), _tmp1_);
				return result;
			}
			/* height needs to be scaled down, so it determines the ratio*/
			result = dimensions_get_scaled_by_height (&(*self), scale);
			return result;
		} else {
#line 82 "Dimensions.vala"
			if (diff_width > diff_height) {
#line 226 "Dimensions.c"
				/* width is greater, so it's the determining factor
				 (this case is true even when diff_height is negative)*/
				result = dimensions_get_scaled_by_width (&(*self), scale);
				return result;
			} else {
				/* height is the determining factor*/
				result = dimensions_get_scaled_by_height (&(*self), scale);
				return result;
			}
		}
	}
}


#line 92 "Dimensions.vala"
Dimensions dimensions_get_scaled_proportional (Dimensions *self, const Dimensions* viewport) {
#line 243 "Dimensions.c"
	Dimensions result;
	Dimensions _tmp0_ = {0};
	Dimensions scaled;
	double ratio;
	scaled = (dimensions_init (&_tmp0_, 0, 0), _tmp0_);
	/* TODO: Surely this can be done by examining dimensions to avoid double calculations.*/
#line 96 "Dimensions.vala"
	scaled.width = (*viewport).width;
#line 252 "Dimensions.c"
	ratio = ((double) (*viewport).width) / ((double) (*self).width);
#line 98 "Dimensions.vala"
	scaled.height = (gint) (((double) (*self).height) * ratio);
#line 99 "Dimensions.vala"
	if (scaled.height > (*viewport).height) {
#line 100 "Dimensions.vala"
		scaled.height = (*viewport).height;
#line 101 "Dimensions.vala"
		ratio = ((double) (*viewport).height) / ((double) (*self).height);
#line 102 "Dimensions.vala"
		scaled.width = (gint) (((double) (*self).width) * ratio);
#line 264 "Dimensions.c"
	}
#line 105 "Dimensions.vala"
	g_assert (scaled.height <= (*viewport).height);
#line 106 "Dimensions.vala"
	g_assert (scaled.width <= (*viewport).width);
#line 270 "Dimensions.c"
	result = scaled;
	return result;
}


#line 111 "Dimensions.vala"
GdkRectangle dimensions_get_scaled_rectangle (Dimensions *self, const Dimensions* scale, const GdkRectangle* rect) {
#line 278 "Dimensions.c"
	GdkRectangle result;
	double x_scale;
	double y_scale;
	GdkRectangle _tmp0_ = {0};
	GdkRectangle scaled_rect;
	x_scale = ((double) (*scale).width) / ((double) (*self).width);
	y_scale = ((double) (*scale).height) / ((double) (*self).height);
	scaled_rect = (memset (&_tmp0_, 0, sizeof (GdkRectangle)), _tmp0_);
#line 116 "Dimensions.vala"
	scaled_rect.x = (gint) ((*rect).x * x_scale);
#line 117 "Dimensions.vala"
	scaled_rect.y = (gint) ((*rect).y * y_scale);
#line 118 "Dimensions.vala"
	scaled_rect.width = (gint) ((*rect).width * x_scale);
#line 119 "Dimensions.vala"
	scaled_rect.height = (gint) ((*rect).height * y_scale);
#line 295 "Dimensions.c"
	result = scaled_rect;
	return result;
}


#line 124 "Dimensions.vala"
Dimensions dimensions_get_scaled_by_width (Dimensions *self, gint scale) {
#line 303 "Dimensions.c"
	Dimensions result;
	double ratio;
	Dimensions _tmp0_ = {0};
	ratio = ((double) scale) / ((double) (*self).width);
	result = (dimensions_init (&_tmp0_, scale, (gint) round (((double) (*self).height) * ratio)), _tmp0_);
	return result;
}


#line 130 "Dimensions.vala"
Dimensions dimensions_get_scaled_by_height (Dimensions *self, gint scale) {
#line 315 "Dimensions.c"
	Dimensions result;
	double ratio;
	Dimensions _tmp0_ = {0};
	ratio = ((double) scale) / ((double) (*self).height);
	result = (dimensions_init (&_tmp0_, (gint) round (((double) (*self).width) * ratio), scale), _tmp0_);
	return result;
}


#line 136 "Dimensions.vala"
Dimensions dimensions_get_scaled_by_constraint (Dimensions *self, gint scale, ScaleConstraint constraint) {
#line 327 "Dimensions.c"
	Dimensions result;
	Dimensions _tmp2_ = {0};
#line 137 "Dimensions.vala"
	switch (constraint) {
#line 332 "Dimensions.c"
		case SCALE_CONSTRAINT_ORIGINAL:
		{
			Dimensions _tmp0_ = {0};
			result = (dimensions_init (&_tmp0_, (*self).width, (*self).height), _tmp0_);
			return result;
		}
		case SCALE_CONSTRAINT_DIMENSIONS:
		{
			Dimensions _tmp1_ = {0};
#line 142 "Dimensions.vala"
			if ((*self).width >= (*self).height) {
#line 142 "Dimensions.vala"
				_tmp1_ = dimensions_get_scaled_by_width (&(*self), scale);
#line 346 "Dimensions.c"
			} else {
#line 142 "Dimensions.vala"
				_tmp1_ = dimensions_get_scaled_by_height (&(*self), scale);
#line 350 "Dimensions.c"
			}
			result = _tmp1_;
			return result;
		}
		case SCALE_CONSTRAINT_WIDTH:
		{
			result = dimensions_get_scaled_by_width (&(*self), scale);
			return result;
		}
		case SCALE_CONSTRAINT_HEIGHT:
		{
			result = dimensions_get_scaled_by_height (&(*self), scale);
			return result;
		}
	}
#line 151 "Dimensions.vala"
	g_error ("Dimensions.vala:151: Bad constraint: %d", (gint) constraint);
#line 368 "Dimensions.c"
	result = (dimensions_init (&_tmp2_, 0, 0), _tmp2_);
	return result;
}


Dimensions* dimensions_dup (const Dimensions* self) {
	Dimensions* dup;
	dup = g_new0 (Dimensions, 1);
	memcpy (dup, self, sizeof (Dimensions));
	return dup;
}


void dimensions_free (Dimensions* self) {
	g_free (self);
}


GType dimensions_get_type (void) {
	static GType dimensions_type_id = 0;
	if (dimensions_type_id == 0) {
		dimensions_type_id = g_boxed_type_register_static ("Dimensions", (GBoxedCopyFunc) dimensions_dup, (GBoxedFreeFunc) dimensions_free);
	}
	return dimensions_type_id;
}




