/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gee.h>
#include <time.h>


#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;
typedef struct _LayoutItemPrivate LayoutItemPrivate;

#define TYPE_DIRECTORY_ITEM (directory_item_get_type ())
#define DIRECTORY_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DIRECTORY_ITEM, DirectoryItem))
#define DIRECTORY_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DIRECTORY_ITEM, DirectoryItemClass))
#define IS_DIRECTORY_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DIRECTORY_ITEM))
#define IS_DIRECTORY_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DIRECTORY_ITEM))
#define DIRECTORY_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DIRECTORY_ITEM, DirectoryItemClass))

typedef struct _DirectoryItem DirectoryItem;
typedef struct _DirectoryItemClass DirectoryItemClass;
typedef struct _DirectoryItemPrivate DirectoryItemPrivate;

#define TYPE_EVENT_ID (event_id_get_type ())
typedef struct _EventID EventID;

#define TYPE_DATABASE_TABLE (database_table_get_type ())
#define DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_TABLE, DatabaseTable))
#define DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_TABLE, DatabaseTableClass))
#define IS_DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_TABLE))
#define IS_DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_TABLE))
#define DATABASE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_TABLE, DatabaseTableClass))

typedef struct _DatabaseTable DatabaseTable;
typedef struct _DatabaseTableClass DatabaseTableClass;

#define TYPE_EVENT_TABLE (event_table_get_type ())
#define EVENT_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENT_TABLE, EventTable))
#define EVENT_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENT_TABLE, EventTableClass))
#define IS_EVENT_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENT_TABLE))
#define IS_EVENT_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENT_TABLE))
#define EVENT_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENT_TABLE, EventTableClass))

typedef struct _EventTable EventTable;
typedef struct _EventTableClass EventTableClass;

#define TYPE_PHOTO_ID (photo_id_get_type ())
typedef struct _PhotoID PhotoID;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;
typedef struct _CheckerboardPagePrivate CheckerboardPagePrivate;

#define TYPE_EVENTS_DIRECTORY_PAGE (events_directory_page_get_type ())
#define EVENTS_DIRECTORY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPage))
#define EVENTS_DIRECTORY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPageClass))
#define IS_EVENTS_DIRECTORY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENTS_DIRECTORY_PAGE))
#define IS_EVENTS_DIRECTORY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENTS_DIRECTORY_PAGE))
#define EVENTS_DIRECTORY_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPageClass))

typedef struct _EventsDirectoryPage EventsDirectoryPage;
typedef struct _EventsDirectoryPageClass EventsDirectoryPageClass;
typedef struct _EventsDirectoryPagePrivate EventsDirectoryPagePrivate;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM (events_directory_page_compare_event_item_get_type ())
#define EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM, EventsDirectoryPageCompareEventItem))
#define EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM, EventsDirectoryPageCompareEventItemClass))
#define EVENTS_DIRECTORY_PAGE_IS_COMPARE_EVENT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM))
#define EVENTS_DIRECTORY_PAGE_IS_COMPARE_EVENT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM))
#define EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM, EventsDirectoryPageCompareEventItemClass))

typedef struct _EventsDirectoryPageCompareEventItem EventsDirectoryPageCompareEventItem;
typedef struct _EventsDirectoryPageCompareEventItemClass EventsDirectoryPageCompareEventItemClass;

#define TYPE_COLLECTION_PAGE (collection_page_get_type ())
#define COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_PAGE, CollectionPage))
#define COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_PAGE, CollectionPageClass))
#define IS_COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_PAGE))
#define IS_COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_PAGE))
#define COLLECTION_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_PAGE, CollectionPageClass))

typedef struct _CollectionPage CollectionPage;
typedef struct _CollectionPageClass CollectionPageClass;

#define TYPE_EVENT_PAGE (event_page_get_type ())
#define EVENT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENT_PAGE, EventPage))
#define EVENT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENT_PAGE, EventPageClass))
#define IS_EVENT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENT_PAGE))
#define IS_EVENT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENT_PAGE))
#define EVENT_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENT_PAGE, EventPageClass))

typedef struct _EventPage EventPage;
typedef struct _EventPageClass EventPageClass;
typedef struct _ComparatorPrivate ComparatorPrivate;
typedef struct _EventsDirectoryPageCompareEventItemPrivate EventsDirectoryPageCompareEventItemPrivate;
typedef struct _CollectionPagePrivate CollectionPagePrivate;
typedef struct _EventPagePrivate EventPagePrivate;

#define TYPE_THUMBNAIL (thumbnail_get_type ())
#define THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL, Thumbnail))
#define THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL, ThumbnailClass))
#define IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL))
#define IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL))
#define THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL, ThumbnailClass))

typedef struct _Thumbnail Thumbnail;
typedef struct _ThumbnailClass ThumbnailClass;

struct _LayoutItem {
	GtkAlignment parent_instance;
	LayoutItemPrivate * priv;
};

struct _LayoutItemClass {
	GtkAlignmentClass parent_class;
	GtkWidget* (*get_control_panel) (LayoutItem* self);
	void (*exposed) (LayoutItem* self);
	void (*unexposed) (LayoutItem* self);
	void (*select) (LayoutItem* self);
	void (*unselect) (LayoutItem* self);
};

struct _EventID {
	gint64 id;
};

struct _DirectoryItem {
	LayoutItem parent_instance;
	DirectoryItemPrivate * priv;
	EventID event_id;
};

struct _DirectoryItemClass {
	LayoutItemClass parent_class;
};

struct _PhotoID {
	gint64 id;
};

struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkActionGroup* action_group;
	GtkActionGroup* common_action_group;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkToolbar* (*get_toolbar) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*switching_to_fullscreen) (Page* self);
	void (*returning_from_fullscreen) (Page* self);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, const GdkEventKey* event);
	void (*on_move) (Page* self, const GdkRectangle* rect);
	void (*on_resize) (Page* self, const GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
};

struct _CheckerboardPage {
	Page parent_instance;
	CheckerboardPagePrivate * priv;
};

struct _CheckerboardPageClass {
	PageClass parent_class;
	void (*on_selection_changed) (CheckerboardPage* self, gint count);
	GtkMenu* (*get_context_menu) (CheckerboardPage* self);
	void (*on_item_activated) (CheckerboardPage* self, LayoutItem* item);
	gboolean (*on_context_invoked) (CheckerboardPage* self, GtkMenu* context_menu);
	LayoutItem* (*get_fullscreen_photo) (CheckerboardPage* self);
	gboolean (*on_mouse_over) (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
};

struct _EventsDirectoryPage {
	CheckerboardPage parent_instance;
	EventsDirectoryPagePrivate * priv;
};

struct _EventsDirectoryPageClass {
	CheckerboardPageClass parent_class;
};

struct _EventsDirectoryPagePrivate {
	EventTable* event_table;
	GtkToolbar* toolbar;
};

struct _Comparator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ComparatorPrivate * priv;
};

struct _ComparatorClass {
	GTypeClass parent_class;
	void (*finalize) (Comparator *self);
	gint64 (*compare) (Comparator* self, gconstpointer a, gconstpointer b);
};

struct _EventsDirectoryPageCompareEventItem {
	Comparator parent_instance;
	EventsDirectoryPageCompareEventItemPrivate * priv;
};

struct _EventsDirectoryPageCompareEventItemClass {
	ComparatorClass parent_class;
};

struct _EventsDirectoryPageCompareEventItemPrivate {
	EventTable* event_table;
	gint sort;
};

struct _CollectionPage {
	CheckerboardPage parent_instance;
	CollectionPagePrivate * priv;
};

struct _CollectionPageClass {
	CheckerboardPageClass parent_class;
	void (*on_photos_menu) (CollectionPage* self);
};

struct _EventPage {
	CollectionPage parent_instance;
	EventPagePrivate * priv;
	EventID event_id;
};

struct _EventPageClass {
	CollectionPageClass parent_class;
};

struct _EventPagePrivate {
	EventTable* event_table;
};


static gpointer directory_item_parent_class = NULL;
static gpointer events_directory_page_compare_event_item_parent_class = NULL;
static gpointer events_directory_page_parent_class = NULL;
static gpointer event_page_parent_class = NULL;

GType layout_item_get_type (void);
GType directory_item_get_type (void);
GType event_id_get_type (void);
EventID* event_id_dup (const EventID* self);
void event_id_free (EventID* self);
enum  {
	DIRECTORY_ITEM_DUMMY_PROPERTY
};
#define DIRECTORY_ITEM_INTERP GDK_INTERP_BILINEAR
#define THUMBNAIL_CACHE_MEDIUM_SCALE 128
#define THUMBNAIL_CACHE_BIG_SCALE 360
#define DIRECTORY_ITEM_SCALE (THUMBNAIL_CACHE_MEDIUM_SCALE + ((THUMBNAIL_CACHE_BIG_SCALE - THUMBNAIL_CACHE_MEDIUM_SCALE) / 2))
void layout_item_set_title (LayoutItem* self, const char* text);
gpointer database_table_ref (gpointer instance);
void database_table_unref (gpointer instance);
GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_database_table (GValue* value, gpointer v_object);
gpointer value_get_database_table (const GValue* value);
GType database_table_get_type (void);
GType event_table_get_type (void);
char* event_table_get_name (EventTable* self, const EventID* event_id);
GType photo_id_get_type (void);
PhotoID* photo_id_dup (const PhotoID* self);
void photo_id_free (PhotoID* self);
PhotoID event_table_get_primary_photo (EventTable* self, const EventID* event_id);
gboolean photo_id_is_valid (PhotoID *self);
GType photo_get_type (void);
Photo* photo_fetch (const PhotoID* photo_id);
GdkPixbuf* photo_get_thumbnail (Photo* self, gint scale);
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp);
void layout_item_set_image (LayoutItem* self, GdkPixbuf* pixbuf);
LayoutItem* layout_item_new (void);
LayoutItem* layout_item_construct (GType object_type);
DirectoryItem* directory_item_new (const EventID* event_id, EventTable* event_table);
DirectoryItem* directory_item_construct (GType object_type, const EventID* event_id, EventTable* event_table);
static void directory_item_finalize (GObject* obj);
GType page_get_type (void);
GType checkerboard_page_get_type (void);
GType events_directory_page_get_type (void);
#define EVENTS_DIRECTORY_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPagePrivate))
enum  {
	EVENTS_DIRECTORY_PAGE_DUMMY_PROPERTY
};
EventTable* event_table_new (void);
EventTable* event_table_construct (GType object_type);
static void events_directory_page_on_view_menu (EventsDirectoryPage* self);
static void _events_directory_page_on_view_menu_gtk_action_callback (GtkAction* action, gpointer self);
CheckerboardPage* checkerboard_page_new (const char* page_name);
CheckerboardPage* checkerboard_page_construct (GType object_type, const char* page_name);
void page_init_ui_start (Page* self, const char* ui_filename, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1);
void page_init_ui_bind (Page* self, const char* menubar_path);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
void checkerboard_page_set_layout_comparator (CheckerboardPage* self, Comparator* cmp);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
gint app_window_get_events_sort (AppWindow* self);
static EventsDirectoryPageCompareEventItem* events_directory_page_compare_event_item_new (EventTable* event_table, gint sort);
static EventsDirectoryPageCompareEventItem* events_directory_page_compare_event_item_construct (GType object_type, EventTable* event_table, gint sort);
static GType events_directory_page_compare_event_item_get_type (void);
EventsDirectoryPage* events_directory_page_new (void);
EventsDirectoryPage* events_directory_page_construct (GType object_type);
static GtkToolbar* events_directory_page_real_get_toolbar (Page* base);
void app_window_switch_to_event (AppWindow* self, const EventID* event_id);
static void events_directory_page_real_on_item_activated (CheckerboardPage* base, LayoutItem* item);
gint checkerboard_page_get_selected_count (CheckerboardPage* self);
GeeIterable* checkerboard_page_get_selected (CheckerboardPage* self);
GeeIterable* checkerboard_page_get_items (CheckerboardPage* self);
GType collection_page_get_type (void);
GType event_page_get_type (void);
EventPage* app_window_find_event_page (AppWindow* self, const EventID* event_id);
LayoutItem* checkerboard_page_get_fullscreen_photo (CheckerboardPage* self);
static LayoutItem* events_directory_page_real_get_fullscreen_photo (CheckerboardPage* base);
void checkerboard_page_add_item (CheckerboardPage* self, LayoutItem* item);
void events_directory_page_add_event (EventsDirectoryPage* self, const EventID* event_id);
void page_switched_to (Page* self);
gint checkerboard_page_remove_all (CheckerboardPage* self);
GeeArrayList* event_table_get_events (EventTable* self);
void checkerboard_page_refresh (CheckerboardPage* self);
static void events_directory_page_real_switched_to (Page* base);
void events_directory_page_notify_sort_changed (EventsDirectoryPage* self, gint sort);
void page_set_item_sensitive (Page* self, const char* path, gboolean sensitive);
gint checkerboard_page_get_count (CheckerboardPage* self);
#define EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM, EventsDirectoryPageCompareEventItemPrivate))
enum  {
	EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM_DUMMY_PROPERTY
};
#define APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING 0
#define APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING 1
time_t event_table_get_start_time (EventTable* self, const EventID* event_id);
static gint64 events_directory_page_compare_event_item_real_compare (Comparator* base, DirectoryItem* a, DirectoryItem* b);
static void events_directory_page_compare_event_item_finalize (Comparator* obj);
static void events_directory_page_finalize (GObject* obj);
#define EVENT_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_EVENT_PAGE, EventPagePrivate))
enum  {
	EVENT_PAGE_DUMMY_PROPERTY
};
#define RESOURCES_MAKE_PRIMARY "shotwell-make-primary"
static void event_page_on_make_primary (EventPage* self);
static void _event_page_on_make_primary_gtk_action_callback (GtkAction* action, gpointer self);
CollectionPage* collection_page_new (const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1);
CollectionPage* collection_page_construct (GType object_type, const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1);
void page_set_page_name (Page* self, const char* page_name);
EventPage* event_page_new (const EventID* event_id);
EventPage* event_page_construct (GType object_type, const EventID* event_id);
void collection_page_on_photos_menu (CollectionPage* self);
static void event_page_real_on_photos_menu (CollectionPage* base);
GType thumbnail_get_type (void);
gboolean event_table_set_primary_photo (EventTable* self, const EventID* event_id, const PhotoID* photo_id);
Photo* thumbnail_get_photo (Thumbnail* self);
PhotoID photo_get_photo_id (Photo* self);
static void event_page_finalize (GObject* obj);

static const GtkActionEntry EVENTS_DIRECTORY_PAGE_ACTIONS[] = {{"FileMenu", NULL, "_File", NULL, NULL, (GCallback) NULL}, {"ViewMenu", NULL, "_View", NULL, NULL, (GCallback) _events_directory_page_on_view_menu_gtk_action_callback}, {"HelpMenu", NULL, "_Help", NULL, NULL, (GCallback) NULL}};
static const GtkActionEntry EVENT_PAGE_ACTIONS[] = {{"MakePrimary", RESOURCES_MAKE_PRIMARY, "Make _Key Photo for Event", NULL, NULL, (GCallback) _event_page_on_make_primary_gtk_action_callback}};


#line 14 "EventsDirectoryPage.vala"
DirectoryItem* directory_item_construct (GType object_type, const EventID* event_id, EventTable* event_table) {
#line 449 "EventsDirectoryPage.c"
	DirectoryItem * self;
	char* _tmp0_;
	PhotoID photo_id;
	Photo* photo;
	GdkPixbuf* pixbuf;
	GdkPixbuf* _tmp1_;
#line 14 "EventsDirectoryPage.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (event_table), NULL);
#line 458 "EventsDirectoryPage.c"
	self = (DirectoryItem*) layout_item_construct (object_type);
#line 15 "EventsDirectoryPage.vala"
	self->event_id = (*event_id);
#line 1025 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 17 "EventsDirectoryPage.vala"
	layout_item_set_title (LAYOUT_ITEM (self), _tmp0_ = event_table_get_name (event_table, &(*event_id)));
#line 466 "EventsDirectoryPage.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	photo_id = event_table_get_primary_photo (event_table, &(*event_id));
#line 20 "EventsDirectoryPage.vala"
	g_assert (photo_id_is_valid (&photo_id));
#line 471 "EventsDirectoryPage.c"
	photo = photo_fetch (&photo_id);
	pixbuf = photo_get_thumbnail (photo, DIRECTORY_ITEM_SCALE);
	_tmp1_ = NULL;
#line 24 "EventsDirectoryPage.vala"
	pixbuf = (_tmp1_ = scale_pixbuf (pixbuf, DIRECTORY_ITEM_SCALE, DIRECTORY_ITEM_INTERP), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 26 "EventsDirectoryPage.vala"
	layout_item_set_image (LAYOUT_ITEM (self), pixbuf);
#line 479 "EventsDirectoryPage.c"
	(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	return self;
}


#line 14 "EventsDirectoryPage.vala"
DirectoryItem* directory_item_new (const EventID* event_id, EventTable* event_table) {
#line 14 "EventsDirectoryPage.vala"
	return directory_item_construct (TYPE_DIRECTORY_ITEM, event_id, event_table);
#line 490 "EventsDirectoryPage.c"
}


static void directory_item_class_init (DirectoryItemClass * klass) {
	directory_item_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = directory_item_finalize;
}


static void directory_item_instance_init (DirectoryItem * self) {
}


static void directory_item_finalize (GObject* obj) {
	DirectoryItem * self;
	self = DIRECTORY_ITEM (obj);
	G_OBJECT_CLASS (directory_item_parent_class)->finalize (obj);
}


GType directory_item_get_type (void) {
	static GType directory_item_type_id = 0;
	if (directory_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (DirectoryItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) directory_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DirectoryItem), 0, (GInstanceInitFunc) directory_item_instance_init, NULL };
		directory_item_type_id = g_type_register_static (TYPE_LAYOUT_ITEM, "DirectoryItem", &g_define_type_info, 0);
	}
	return directory_item_type_id;
}


#line 134 "EventsDirectoryPage.vala"
static void _events_directory_page_on_view_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 523 "EventsDirectoryPage.c"
	events_directory_page_on_view_menu (self);
}


#line 70 "EventsDirectoryPage.vala"
EventsDirectoryPage* events_directory_page_construct (GType object_type) {
#line 530 "EventsDirectoryPage.c"
	EventsDirectoryPage * self;
	EventsDirectoryPageCompareEventItem* _tmp1_;
	AppWindow* _tmp0_;
#line 71 "EventsDirectoryPage.vala"
	self = (EventsDirectoryPage*) checkerboard_page_construct (object_type, "Events");
#line 73 "EventsDirectoryPage.vala"
	page_init_ui_start (PAGE (self), "events_directory.ui", "EventsDirectoryActionGroup", EVENTS_DIRECTORY_PAGE_ACTIONS, G_N_ELEMENTS (EVENTS_DIRECTORY_PAGE_ACTIONS), NULL, 0);
#line 74 "EventsDirectoryPage.vala"
	page_init_ui_bind (PAGE (self), "/EventsDirectoryMenuBar");
#line 77 "EventsDirectoryPage.vala"
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 542 "EventsDirectoryPage.c"
	_tmp1_ = NULL;
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 79 "EventsDirectoryPage.vala"
	checkerboard_page_set_layout_comparator (CHECKERBOARD_PAGE (self), COMPARATOR (_tmp1_ = events_directory_page_compare_event_item_new (self->priv->event_table, app_window_get_events_sort (_tmp0_ = app_window_get_instance ()))));
#line 548 "EventsDirectoryPage.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (comparator_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	return self;
}


#line 70 "EventsDirectoryPage.vala"
EventsDirectoryPage* events_directory_page_new (void) {
#line 70 "EventsDirectoryPage.vala"
	return events_directory_page_construct (TYPE_EVENTS_DIRECTORY_PAGE);
#line 559 "EventsDirectoryPage.c"
}


#line 82 "EventsDirectoryPage.vala"
static GtkToolbar* events_directory_page_real_get_toolbar (Page* base) {
#line 565 "EventsDirectoryPage.c"
	EventsDirectoryPage * self;
	GtkToolbar* result;
	GtkToolbar* _tmp0_;
	self = EVENTS_DIRECTORY_PAGE (base);
#line 83 "EventsDirectoryPage.vala"
	_tmp0_ = NULL;
#line 572 "EventsDirectoryPage.c"
	result = (_tmp0_ = self->priv->toolbar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 86 "EventsDirectoryPage.vala"
static void events_directory_page_real_on_item_activated (CheckerboardPage* base, LayoutItem* item) {
#line 580 "EventsDirectoryPage.c"
	EventsDirectoryPage * self;
	DirectoryItem* _tmp0_;
	DirectoryItem* event;
	AppWindow* _tmp1_;
	self = EVENTS_DIRECTORY_PAGE (base);
#line 86 "EventsDirectoryPage.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 87 "EventsDirectoryPage.vala"
	_tmp0_ = NULL;
#line 590 "EventsDirectoryPage.c"
	event = (_tmp0_ = DIRECTORY_ITEM (item), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 446 "AppWindow.vala"
	_tmp1_ = NULL;
#line 88 "EventsDirectoryPage.vala"
	app_window_switch_to_event (_tmp1_ = app_window_get_instance (), &event->event_id);
#line 596 "EventsDirectoryPage.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(event == NULL) ? NULL : (event = (g_object_unref (event), NULL));
}


#line 91 "EventsDirectoryPage.vala"
static LayoutItem* events_directory_page_real_get_fullscreen_photo (CheckerboardPage* base) {
#line 604 "EventsDirectoryPage.c"
	EventsDirectoryPage * self;
	LayoutItem* result;
	GeeIterable* iter;
	self = EVENTS_DIRECTORY_PAGE (base);
	iter = NULL;
	/* use first selected item, otherwise use first item*/
#line 95 "EventsDirectoryPage.vala"
	if (checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0) {
#line 613 "EventsDirectoryPage.c"
		GeeIterable* _tmp0_;
		_tmp0_ = NULL;
#line 96 "EventsDirectoryPage.vala"
		iter = (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self)), (iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL)), _tmp0_);
#line 618 "EventsDirectoryPage.c"
	} else {
		GeeIterable* _tmp1_;
		_tmp1_ = NULL;
#line 98 "EventsDirectoryPage.vala"
		iter = (_tmp1_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self)), (iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL)), _tmp1_);
#line 624 "EventsDirectoryPage.c"
	}
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (iter);
#line 101 "EventsDirectoryPage.vala"
		while (TRUE) {
#line 631 "EventsDirectoryPage.c"
			LayoutItem* item;
			AppWindow* _tmp2_;
			EventPage* _tmp3_;
			EventPage* page;
#line 101 "EventsDirectoryPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 101 "EventsDirectoryPage.vala"
				break;
#line 640 "EventsDirectoryPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 446 "AppWindow.vala"
			_tmp2_ = NULL;
#line 102 "EventsDirectoryPage.vala"
			_tmp3_ = NULL;
#line 647 "EventsDirectoryPage.c"
			page = (_tmp3_ = app_window_find_event_page (_tmp2_ = app_window_get_instance (), &DIRECTORY_ITEM (item)->event_id), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL)), _tmp3_);
#line 103 "EventsDirectoryPage.vala"
			if (page != NULL) {
#line 651 "EventsDirectoryPage.c"
				result = checkerboard_page_get_fullscreen_photo (CHECKERBOARD_PAGE (page));
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				(iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL));
				return result;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = NULL;
	(iter == NULL) ? NULL : (iter = (g_object_unref (iter), NULL));
	return result;
}


#line 110 "EventsDirectoryPage.vala"
void events_directory_page_add_event (EventsDirectoryPage* self, const EventID* event_id) {
#line 672 "EventsDirectoryPage.c"
	DirectoryItem* item;
#line 110 "EventsDirectoryPage.vala"
	g_return_if_fail (IS_EVENTS_DIRECTORY_PAGE (self));
#line 676 "EventsDirectoryPage.c"
	item = g_object_ref_sink (directory_item_new (&(*event_id), self->priv->event_table));
#line 112 "EventsDirectoryPage.vala"
	checkerboard_page_add_item (CHECKERBOARD_PAGE (self), LAYOUT_ITEM (item));
#line 680 "EventsDirectoryPage.c"
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


#line 115 "EventsDirectoryPage.vala"
static void events_directory_page_real_switched_to (Page* base) {
#line 687 "EventsDirectoryPage.c"
	EventsDirectoryPage * self;
	GeeArrayList* event_ids;
	self = EVENTS_DIRECTORY_PAGE (base);
#line 116 "EventsDirectoryPage.vala"
	PAGE_CLASS (events_directory_page_parent_class)->switched_to (PAGE (CHECKERBOARD_PAGE (self)));
#line 118 "EventsDirectoryPage.vala"
	checkerboard_page_remove_all (CHECKERBOARD_PAGE (self));
#line 695 "EventsDirectoryPage.c"
	event_ids = event_table_get_events (self->priv->event_table);
	{
		GeeIterator* _event_id_it;
		_event_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (event_ids));
#line 121 "EventsDirectoryPage.vala"
		while (TRUE) {
#line 702 "EventsDirectoryPage.c"
			EventID* _tmp0_;
			EventID _tmp1_ = {0};
			EventID event_id;
#line 121 "EventsDirectoryPage.vala"
			if (!gee_iterator_next (_event_id_it)) {
#line 121 "EventsDirectoryPage.vala"
				break;
#line 710 "EventsDirectoryPage.c"
			}
#line 120 "EventsDirectoryPage.vala"
			_tmp0_ = NULL;
#line 714 "EventsDirectoryPage.c"
			event_id = (_tmp1_ = *(_tmp0_ = (EventID*) gee_iterator_get (_event_id_it)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (event_id_free (_tmp0_), NULL)), _tmp1_);
#line 122 "EventsDirectoryPage.vala"
			events_directory_page_add_event (self, &event_id);
#line 718 "EventsDirectoryPage.c"
		}
		(_event_id_it == NULL) ? NULL : (_event_id_it = (g_object_unref (_event_id_it), NULL));
	}
#line 124 "EventsDirectoryPage.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 126 "EventsDirectoryPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 726 "EventsDirectoryPage.c"
	(event_ids == NULL) ? NULL : (event_ids = (g_object_unref (event_ids), NULL));
}


#line 129 "EventsDirectoryPage.vala"
void events_directory_page_notify_sort_changed (EventsDirectoryPage* self, gint sort) {
#line 733 "EventsDirectoryPage.c"
	EventsDirectoryPageCompareEventItem* _tmp0_;
#line 129 "EventsDirectoryPage.vala"
	g_return_if_fail (IS_EVENTS_DIRECTORY_PAGE (self));
#line 737 "EventsDirectoryPage.c"
	_tmp0_ = NULL;
#line 130 "EventsDirectoryPage.vala"
	checkerboard_page_set_layout_comparator (CHECKERBOARD_PAGE (self), COMPARATOR (_tmp0_ = events_directory_page_compare_event_item_new (self->priv->event_table, sort)));
#line 741 "EventsDirectoryPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (comparator_unref (_tmp0_), NULL));
#line 131 "EventsDirectoryPage.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 745 "EventsDirectoryPage.c"
}


#line 134 "EventsDirectoryPage.vala"
static void events_directory_page_on_view_menu (EventsDirectoryPage* self) {
#line 134 "EventsDirectoryPage.vala"
	g_return_if_fail (IS_EVENTS_DIRECTORY_PAGE (self));
#line 135 "EventsDirectoryPage.vala"
	page_set_item_sensitive (PAGE (self), "/EventsDirectoryMenuBar/ViewMenu/Fullscreen", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0);
#line 755 "EventsDirectoryPage.c"
}


#line 35 "EventsDirectoryPage.vala"
static EventsDirectoryPageCompareEventItem* events_directory_page_compare_event_item_construct (GType object_type, EventTable* event_table, gint sort) {
#line 761 "EventsDirectoryPage.c"
	EventsDirectoryPageCompareEventItem* self;
	gboolean _tmp0_;
	EventTable* _tmp2_;
	EventTable* _tmp1_;
#line 35 "EventsDirectoryPage.vala"
	g_return_val_if_fail (IS_EVENT_TABLE (event_table), NULL);
#line 768 "EventsDirectoryPage.c"
	self = (EventsDirectoryPageCompareEventItem*) g_type_create_instance (object_type);
	_tmp0_ = FALSE;
#line 36 "EventsDirectoryPage.vala"
	if (sort == APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING) {
#line 36 "EventsDirectoryPage.vala"
		_tmp0_ = TRUE;
#line 775 "EventsDirectoryPage.c"
	} else {
#line 37 "EventsDirectoryPage.vala"
		_tmp0_ = sort == APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING;
#line 779 "EventsDirectoryPage.c"
	}
#line 36 "EventsDirectoryPage.vala"
	g_assert (_tmp0_);
#line 783 "EventsDirectoryPage.c"
	_tmp2_ = NULL;
#line 39 "EventsDirectoryPage.vala"
	_tmp1_ = NULL;
#line 39 "EventsDirectoryPage.vala"
	self->priv->event_table = (_tmp2_ = (_tmp1_ = event_table, (_tmp1_ == NULL) ? NULL : database_table_ref (_tmp1_)), (self->priv->event_table == NULL) ? NULL : (self->priv->event_table = (database_table_unref (self->priv->event_table), NULL)), _tmp2_);
#line 40 "EventsDirectoryPage.vala"
	self->priv->sort = sort;
#line 791 "EventsDirectoryPage.c"
	return self;
}


#line 35 "EventsDirectoryPage.vala"
static EventsDirectoryPageCompareEventItem* events_directory_page_compare_event_item_new (EventTable* event_table, gint sort) {
#line 35 "EventsDirectoryPage.vala"
	return events_directory_page_compare_event_item_construct (EVENTS_DIRECTORY_PAGE_TYPE_COMPARE_EVENT_ITEM, event_table, sort);
#line 800 "EventsDirectoryPage.c"
}


#line 43 "EventsDirectoryPage.vala"
static gint64 events_directory_page_compare_event_item_real_compare (Comparator* base, DirectoryItem* a, DirectoryItem* b) {
#line 806 "EventsDirectoryPage.c"
	EventsDirectoryPageCompareEventItem * self;
	gint64 result;
	gint64 start_a;
	gint64 start_b;
	self = EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM (base);
#line 43 "EventsDirectoryPage.vala"
	g_return_val_if_fail (IS_DIRECTORY_ITEM (a), 0LL);
#line 43 "EventsDirectoryPage.vala"
	g_return_val_if_fail (IS_DIRECTORY_ITEM (b), 0LL);
#line 816 "EventsDirectoryPage.c"
	start_a = (gint64) event_table_get_start_time (self->priv->event_table, &a->event_id);
	start_b = (gint64) event_table_get_start_time (self->priv->event_table, &b->event_id);
#line 47 "EventsDirectoryPage.vala"
	switch (self->priv->sort) {
#line 821 "EventsDirectoryPage.c"
		case APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING:
		{
			result = start_a - start_b;
			return result;
		}
		default:
		{
			result = start_b - start_a;
			return result;
		}
	}
}


static void events_directory_page_compare_event_item_class_init (EventsDirectoryPageCompareEventItemClass * klass) {
	events_directory_page_compare_event_item_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->finalize = events_directory_page_compare_event_item_finalize;
	g_type_class_add_private (klass, sizeof (EventsDirectoryPageCompareEventItemPrivate));
	COMPARATOR_CLASS (klass)->compare = events_directory_page_compare_event_item_real_compare;
}


static void events_directory_page_compare_event_item_instance_init (EventsDirectoryPageCompareEventItem * self) {
	self->priv = EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM_GET_PRIVATE (self);
}


static void events_directory_page_compare_event_item_finalize (Comparator* obj) {
	EventsDirectoryPageCompareEventItem * self;
	self = EVENTS_DIRECTORY_PAGE_COMPARE_EVENT_ITEM (obj);
	(self->priv->event_table == NULL) ? NULL : (self->priv->event_table = (database_table_unref (self->priv->event_table), NULL));
	COMPARATOR_CLASS (events_directory_page_compare_event_item_parent_class)->finalize (obj);
}


static GType events_directory_page_compare_event_item_get_type (void) {
	static GType events_directory_page_compare_event_item_type_id = 0;
	if (events_directory_page_compare_event_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (EventsDirectoryPageCompareEventItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) events_directory_page_compare_event_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EventsDirectoryPageCompareEventItem), 0, (GInstanceInitFunc) events_directory_page_compare_event_item_instance_init, NULL };
		events_directory_page_compare_event_item_type_id = g_type_register_static (TYPE_COMPARATOR, "EventsDirectoryPageCompareEventItem", &g_define_type_info, 0);
	}
	return events_directory_page_compare_event_item_type_id;
}


static void events_directory_page_class_init (EventsDirectoryPageClass * klass) {
	events_directory_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (EventsDirectoryPagePrivate));
	PAGE_CLASS (klass)->get_toolbar = events_directory_page_real_get_toolbar;
	CHECKERBOARD_PAGE_CLASS (klass)->on_item_activated = events_directory_page_real_on_item_activated;
	CHECKERBOARD_PAGE_CLASS (klass)->get_fullscreen_photo = events_directory_page_real_get_fullscreen_photo;
	PAGE_CLASS (klass)->switched_to = events_directory_page_real_switched_to;
	G_OBJECT_CLASS (klass)->finalize = events_directory_page_finalize;
}


static void events_directory_page_instance_init (EventsDirectoryPage * self) {
	self->priv = EVENTS_DIRECTORY_PAGE_GET_PRIVATE (self);
	self->priv->event_table = event_table_new ();
	self->priv->toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
}


static void events_directory_page_finalize (GObject* obj) {
	EventsDirectoryPage * self;
	self = EVENTS_DIRECTORY_PAGE (obj);
	(self->priv->event_table == NULL) ? NULL : (self->priv->event_table = (database_table_unref (self->priv->event_table), NULL));
	(self->priv->toolbar == NULL) ? NULL : (self->priv->toolbar = (g_object_unref (self->priv->toolbar), NULL));
	G_OBJECT_CLASS (events_directory_page_parent_class)->finalize (obj);
}


GType events_directory_page_get_type (void) {
	static GType events_directory_page_type_id = 0;
	if (events_directory_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (EventsDirectoryPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) events_directory_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EventsDirectoryPage), 0, (GInstanceInitFunc) events_directory_page_instance_init, NULL };
		events_directory_page_type_id = g_type_register_static (TYPE_CHECKERBOARD_PAGE, "EventsDirectoryPage", &g_define_type_info, 0);
	}
	return events_directory_page_type_id;
}


#line 162 "EventsDirectoryPage.vala"
static void _event_page_on_make_primary_gtk_action_callback (GtkAction* action, gpointer self) {
#line 906 "EventsDirectoryPage.c"
	event_page_on_make_primary (self);
}


#line 148 "EventsDirectoryPage.vala"
EventPage* event_page_construct (GType object_type, const EventID* event_id) {
#line 913 "EventsDirectoryPage.c"
	EventPage * self;
	char* _tmp0_;
#line 149 "EventsDirectoryPage.vala"
	self = (EventPage*) collection_page_construct (object_type, "Event", "event.ui", EVENT_PAGE_ACTIONS, G_N_ELEMENTS (EVENT_PAGE_ACTIONS));
#line 151 "EventsDirectoryPage.vala"
	self->event_id = (*event_id);
#line 1025 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 153 "EventsDirectoryPage.vala"
	page_set_page_name (PAGE (self), _tmp0_ = event_table_get_name (self->priv->event_table, &(*event_id)));
#line 924 "EventsDirectoryPage.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	return self;
}


#line 148 "EventsDirectoryPage.vala"
EventPage* event_page_new (const EventID* event_id) {
#line 148 "EventsDirectoryPage.vala"
	return event_page_construct (TYPE_EVENT_PAGE, event_id);
#line 934 "EventsDirectoryPage.c"
}


#line 156 "EventsDirectoryPage.vala"
static void event_page_real_on_photos_menu (CollectionPage* base) {
#line 940 "EventsDirectoryPage.c"
	EventPage * self;
	self = EVENT_PAGE (base);
#line 157 "EventsDirectoryPage.vala"
	page_set_item_sensitive (PAGE (self), "/CollectionMenuBar/PhotosMenu/MakePrimary", checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) == 1);
#line 159 "EventsDirectoryPage.vala"
	COLLECTION_PAGE_CLASS (event_page_parent_class)->on_photos_menu (COLLECTION_PAGE (self));
#line 947 "EventsDirectoryPage.c"
}


#line 162 "EventsDirectoryPage.vala"
static void event_page_on_make_primary (EventPage* self) {
#line 162 "EventsDirectoryPage.vala"
	g_return_if_fail (IS_EVENT_PAGE (self));
#line 163 "EventsDirectoryPage.vala"
	g_assert (checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) == 1);
#line 957 "EventsDirectoryPage.c"
	/* iterate to first one, use that, bail out*/
	{
		GeeIterable* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
		/* iterate to first one, use that, bail out*/
#line 498 "Page.vala"
		_tmp0_ = NULL;
#line 966 "EventsDirectoryPage.c"
		_tmp1_ = NULL;
		_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
		/* iterate to first one, use that, bail out*/
#line 166 "EventsDirectoryPage.vala"
		while (TRUE) {
#line 972 "EventsDirectoryPage.c"
			LayoutItem* item;
			Thumbnail* _tmp2_;
			Thumbnail* thumbnail;
			PhotoID _tmp4_ = {0};
			Photo* _tmp3_;
			/* iterate to first one, use that, bail out*/
#line 166 "EventsDirectoryPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 981 "EventsDirectoryPage.c"
				/* iterate to first one, use that, bail out*/
#line 166 "EventsDirectoryPage.vala"
				break;
#line 985 "EventsDirectoryPage.c"
			}
			/* iterate to first one, use that, bail out*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 167 "EventsDirectoryPage.vala"
			_tmp2_ = NULL;
#line 991 "EventsDirectoryPage.c"
			thumbnail = (_tmp2_ = THUMBNAIL (item), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
#line 39 "Thumbnail.vala"
			_tmp3_ = NULL;
#line 168 "EventsDirectoryPage.vala"
			event_table_set_primary_photo (self->priv->event_table, &self->event_id, (_tmp4_ = photo_get_photo_id (_tmp3_ = thumbnail_get_photo (thumbnail)), &_tmp4_));
#line 997 "EventsDirectoryPage.c"
			(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL));
#line 170 "EventsDirectoryPage.vala"
			break;
#line 1003 "EventsDirectoryPage.c"
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
}


static void event_page_class_init (EventPageClass * klass) {
	event_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (EventPagePrivate));
	COLLECTION_PAGE_CLASS (klass)->on_photos_menu = event_page_real_on_photos_menu;
	G_OBJECT_CLASS (klass)->finalize = event_page_finalize;
}


static void event_page_instance_init (EventPage * self) {
	self->priv = EVENT_PAGE_GET_PRIVATE (self);
	self->priv->event_table = event_table_new ();
}


static void event_page_finalize (GObject* obj) {
	EventPage * self;
	self = EVENT_PAGE (obj);
	(self->priv->event_table == NULL) ? NULL : (self->priv->event_table = (database_table_unref (self->priv->event_table), NULL));
	G_OBJECT_CLASS (event_page_parent_class)->finalize (obj);
}


GType event_page_get_type (void) {
	static GType event_page_type_id = 0;
	if (event_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (EventPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) event_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EventPage), 0, (GInstanceInitFunc) event_page_instance_init, NULL };
		event_page_type_id = g_type_register_static (TYPE_COLLECTION_PAGE, "EventPage", &g_define_type_info, 0);
	}
	return event_page_type_id;
}




