/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <libexif/exif-data.h>
#include <libexif/exif-content.h>
#include <libexif/exif-entry.h>
#include <libexif/exif-format.h>
#include <libexif/exif-tag.h>
#include <string.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <gio/gio.h>
#include <libexif/exif-ifd.h>
#include <libexif/exif-utils.h>
#include <errno.h>
#include <gobject/gvaluecollector.h>


#define TYPE_PHOTO_EXIF (photo_exif_get_type ())
#define PHOTO_EXIF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_EXIF, PhotoExif))
#define PHOTO_EXIF_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_EXIF, PhotoExifClass))
#define IS_PHOTO_EXIF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_EXIF))
#define IS_PHOTO_EXIF_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_EXIF))
#define PHOTO_EXIF_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_EXIF, PhotoExifClass))

typedef struct _PhotoExif PhotoExif;
typedef struct _PhotoExifClass PhotoExifClass;
typedef struct _PhotoExifPrivate PhotoExifPrivate;

#define JPEG_TYPE_MARKER (jpeg_marker_get_type ())

#define TYPE_ORIENTATION (orientation_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;
typedef struct _ParamSpecPhotoExif ParamSpecPhotoExif;

typedef enum  {
	EXIF_ERROR_FILE_FORMAT
} ExifError;
#define EXIF_ERROR exif_error_quark ()
struct _PhotoExif {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PhotoExifPrivate * priv;
};

struct _PhotoExifClass {
	GTypeClass parent_class;
	void (*finalize) (PhotoExif *self);
};

struct _PhotoExifPrivate {
	GFile* file;
	ExifData* exif;
	gboolean no_exif;
};

typedef enum  {
	JPEG_MARKER_SOI = 0xD8,
	JPEG_MARKER_EOI = 0xD9,
	JPEG_MARKER_APP0 = 0xE0,
	JPEG_MARKER_APP1 = 0xE1
} JpegMarker;

typedef enum  {
	ORIENTATION_MIN = 1,
	ORIENTATION_TOP_LEFT = 1,
	ORIENTATION_TOP_RIGHT = 2,
	ORIENTATION_BOTTOM_RIGHT = 3,
	ORIENTATION_BOTTOM_LEFT = 4,
	ORIENTATION_LEFT_TOP = 5,
	ORIENTATION_RIGHT_TOP = 6,
	ORIENTATION_RIGHT_BOTTOM = 7,
	ORIENTATION_LEFT_BOTTOM = 8,
	ORIENTATION_MAX = 8
} Orientation;

struct _Dimensions {
	gint width;
	gint height;
};

struct _ParamSpecPhotoExif {
	GParamSpec parent_instance;
};


static gpointer photo_exif_parent_class = NULL;

ExifEntry* exif_find_first_entry (ExifData* data, ExifTag tag, ExifFormat format);
gint exif_remove_all_tags (ExifData* data, ExifTag tag);
gboolean exif_convert_datetime (const char* datetime, time_t* timestamp);
gboolean exif_get_timestamp (ExifData* exif, time_t* timestamp);
GQuark exif_error_quark (void);
void free (void* ptr);
gpointer photo_exif_ref (gpointer instance);
void photo_exif_unref (gpointer instance);
GParamSpec* param_spec_photo_exif (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_photo_exif (GValue* value, gpointer v_object);
gpointer value_get_photo_exif (const GValue* value);
GType photo_exif_get_type (void);
#define PHOTO_EXIF_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PHOTO_EXIF, PhotoExifPrivate))
enum  {
	PHOTO_EXIF_DUMMY_PROPERTY
};
PhotoExif* photo_exif_new (GFile* file);
PhotoExif* photo_exif_construct (GType object_type, GFile* file);
static void photo_exif_update (PhotoExif* self);
GType jpeg_marker_get_type (void);
gint jpeg_read_marker (GFileInputStream* fins, JpegMarker* marker, GError** error);
gboolean photo_exif_has_exif (PhotoExif* self);
ExifData* photo_exif_get_exif (PhotoExif* self);
void photo_exif_set_exif (PhotoExif* self, ExifData* exif);
static ExifEntry* photo_exif_find_entry (PhotoExif* self, ExifIfd ifd, ExifTag tag, ExifFormat format);
GType orientation_get_type (void);
Orientation photo_exif_get_orientation (PhotoExif* self);
static ExifEntry* photo_exif_find_first_entry (PhotoExif* self, ExifTag tag, ExifFormat format);
void photo_exif_set_orientation (PhotoExif* self, Orientation orientation);
static ExifEntry* photo_exif_find_entry_multiformat (PhotoExif* self, ExifIfd ifd, ExifTag tag, ExifFormat format1, ExifFormat format2);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
gboolean photo_exif_get_dimensions (PhotoExif* self, Dimensions* dim);
void photo_exif_set_dimensions (PhotoExif* self, const Dimensions* dim);
char* photo_exif_get_datetime (PhotoExif* self);
gboolean photo_exif_get_timestamp (PhotoExif* self, time_t* timestamp);
gint photo_exif_remove_all_tags (PhotoExif* self, ExifTag tag);
gboolean photo_exif_remove_thumbnail (PhotoExif* self);
static GFileOutputStream* photo_exif_create_temp (GFile* original, GFile** temp, GError** error);
void jpeg_write_marker (GFileOutputStream* fouts, JpegMarker marker, gint length, GError** error);
void photo_exif_commit (PhotoExif* self, GError** error);
static void photo_exif_finalize (PhotoExif* obj);

static const guint8 EXIF_SIGNATURE[] = {(guint8) 0x45, (guint8) 0x78, (guint8) 0x69, (guint8) 0x66};


#line 11 "Exif.vala"
ExifEntry* exif_find_first_entry (ExifData* data, ExifTag tag, ExifFormat format) {
#line 147 "Exif.c"
	ExifEntry* result;
#line 11 "Exif.vala"
	g_return_val_if_fail (data != NULL, NULL);
#line 151 "Exif.c"
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 12 "Exif.vala"
			while (TRUE) {
#line 160 "Exif.c"
				ExifContent* _tmp1_;
				ExifContent* content;
				ExifEntry* _tmp2_;
				ExifEntry* entry;
				gboolean _tmp3_;
#line 12 "Exif.vala"
				if (!_tmp0_) {
#line 12 "Exif.vala"
					ctr++;
#line 170 "Exif.c"
				}
#line 12 "Exif.vala"
				_tmp0_ = FALSE;
#line 12 "Exif.vala"
				if (!(ctr < ((gint) EXIF_IFD_COUNT))) {
#line 12 "Exif.vala"
					break;
#line 178 "Exif.c"
				}
#line 13 "Exif.vala"
				_tmp1_ = NULL;
#line 182 "Exif.c"
				content = (_tmp1_ = data->ifd[ctr], (_tmp1_ == NULL) ? NULL : exif_content_ref (_tmp1_), _tmp1_);
#line 14 "Exif.vala"
				g_assert (content != NULL);
#line 16 "Exif.vala"
				_tmp2_ = NULL;
#line 188 "Exif.c"
				entry = (_tmp2_ = exif_content_get_entry (content, tag), (_tmp2_ == NULL) ? NULL : exif_entry_ref (_tmp2_), _tmp2_);
#line 17 "Exif.vala"
				if (entry == NULL) {
#line 192 "Exif.c"
					(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
					(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
#line 18 "Exif.vala"
					continue;
#line 197 "Exif.c"
				}
#line 20 "Exif.vala"
				g_assert (entry->format == format);
#line 201 "Exif.c"
				_tmp3_ = FALSE;
#line 21 "Exif.vala"
				if (format != EXIF_FORMAT_ASCII) {
#line 21 "Exif.vala"
					_tmp3_ = format != EXIF_FORMAT_UNDEFINED;
#line 207 "Exif.c"
				} else {
#line 21 "Exif.vala"
					_tmp3_ = FALSE;
#line 211 "Exif.c"
				}
#line 21 "Exif.vala"
				if (_tmp3_) {
#line 22 "Exif.vala"
					g_assert (entry->size == exif_format_get_size (format));
#line 217 "Exif.c"
				}
				result = entry;
				(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
				return result;
			}
		}
	}
	result = NULL;
	return result;
}


#line 30 "Exif.vala"
gint exif_remove_all_tags (ExifData* data, ExifTag tag) {
#line 232 "Exif.c"
	gint result;
	gint count;
#line 30 "Exif.vala"
	g_return_val_if_fail (data != NULL, 0);
#line 237 "Exif.c"
	count = 0;
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 32 "Exif.vala"
			while (TRUE) {
#line 247 "Exif.c"
				ExifContent* _tmp1_;
				ExifContent* content;
				ExifEntry* _tmp2_;
				ExifEntry* entry;
#line 32 "Exif.vala"
				if (!_tmp0_) {
#line 32 "Exif.vala"
					ctr++;
#line 256 "Exif.c"
				}
#line 32 "Exif.vala"
				_tmp0_ = FALSE;
#line 32 "Exif.vala"
				if (!(ctr < ((gint) EXIF_IFD_COUNT))) {
#line 32 "Exif.vala"
					break;
#line 264 "Exif.c"
				}
#line 33 "Exif.vala"
				_tmp1_ = NULL;
#line 268 "Exif.c"
				content = (_tmp1_ = data->ifd[ctr], (_tmp1_ == NULL) ? NULL : exif_content_ref (_tmp1_), _tmp1_);
#line 34 "Exif.vala"
				g_assert (content != NULL);
#line 36 "Exif.vala"
				_tmp2_ = NULL;
#line 274 "Exif.c"
				entry = (_tmp2_ = exif_content_get_entry (content, tag), (_tmp2_ == NULL) ? NULL : exif_entry_ref (_tmp2_), _tmp2_);
#line 37 "Exif.vala"
				if (entry == NULL) {
#line 278 "Exif.c"
					(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
					(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
#line 38 "Exif.vala"
					continue;
#line 283 "Exif.c"
				}
#line 40 "Exif.vala"
				exif_content_remove_entry (content, entry);
#line 41 "Exif.vala"
				count++;
#line 289 "Exif.c"
				(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
				(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
			}
		}
	}
	result = count;
	return result;
}


#line 47 "Exif.vala"
gboolean exif_convert_datetime (const char* datetime, time_t* timestamp) {
#line 302 "Exif.c"
	gboolean result;
	struct tm _tmp0_ = {0};
	struct tm tm;
	gint count;
#line 47 "Exif.vala"
	g_return_val_if_fail (datetime != NULL, FALSE);
#line 309 "Exif.c"
	tm = (memset (&_tmp0_, 0, sizeof (struct tm)), _tmp0_);
	count = sscanf (datetime, "%d:%d:%d %d:%d:%d", &tm.tm_year, &tm.tm_mon, &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
#line 51 "Exif.vala"
	if (count != 6) {
#line 314 "Exif.c"
		result = FALSE;
		return result;
	}
#line 54 "Exif.vala"
	tm.tm_year = tm.tm_year - 1900;
#line 55 "Exif.vala"
	tm.tm_mon--;
#line 56 "Exif.vala"
	tm.tm_isdst = -1;
#line 58 "Exif.vala"
	(*timestamp) = mktime (&tm);
#line 326 "Exif.c"
	result = TRUE;
	return result;
}


#line 63 "Exif.vala"
gboolean exif_get_timestamp (ExifData* exif, time_t* timestamp) {
#line 334 "Exif.c"
	gboolean result;
	ExifEntry* entry;
#line 63 "Exif.vala"
	g_return_val_if_fail (exif != NULL, FALSE);
#line 339 "Exif.c"
	entry = exif_find_first_entry (exif, EXIF_TAG_DATE_TIME_ORIGINAL, EXIF_FORMAT_ASCII);
#line 65 "Exif.vala"
	if (entry != NULL) {
#line 343 "Exif.c"
		const char* _tmp1_;
		gint _tmp0__length1;
		gchar* _tmp0_;
		char* _tmp2_;
		char* datetime;
#line 66 "Exif.vala"
		_tmp1_ = NULL;
#line 127 "libexif.vapi"
		_tmp0_ = NULL;
#line 66 "Exif.vala"
		_tmp2_ = NULL;
#line 355 "Exif.c"
		datetime = (_tmp2_ = (_tmp1_ = exif_entry_get_value (entry, (_tmp0_ = g_new0 (gchar, 256), _tmp0__length1 = 256, _tmp0_), 256), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
#line 67 "Exif.vala"
		if (datetime != NULL) {
#line 68 "Exif.vala"
			if (exif_convert_datetime (datetime, &(*timestamp))) {
#line 361 "Exif.c"
				result = TRUE;
				datetime = (g_free (datetime), NULL);
				(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
				return result;
			}
		}
		datetime = (g_free (datetime), NULL);
	}
	result = FALSE;
	(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
	return result;
}


GQuark exif_error_quark (void) {
	return g_quark_from_static_string ("exif_error-quark");
}


#line 88 "Exif.vala"
PhotoExif* photo_exif_construct (GType object_type, GFile* file) {
#line 383 "Exif.c"
	PhotoExif* self;
	GFile* _tmp1_;
	GFile* _tmp0_;
#line 88 "Exif.vala"
	g_return_val_if_fail (G_IS_FILE (file), NULL);
#line 389 "Exif.c"
	self = (PhotoExif*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 89 "Exif.vala"
	_tmp0_ = NULL;
#line 89 "Exif.vala"
	self->priv->file = (_tmp1_ = (_tmp0_ = file, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->file == NULL) ? NULL : (self->priv->file = (g_object_unref (self->priv->file), NULL)), _tmp1_);
#line 396 "Exif.c"
	return self;
}


#line 88 "Exif.vala"
PhotoExif* photo_exif_new (GFile* file) {
#line 88 "Exif.vala"
	return photo_exif_construct (TYPE_PHOTO_EXIF, file);
#line 405 "Exif.c"
}


#line 92 "Exif.vala"
gboolean photo_exif_has_exif (PhotoExif* self) {
#line 411 "Exif.c"
	gboolean result;
	GError * _inner_error_;
#line 92 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), FALSE);
#line 416 "Exif.c"
	_inner_error_ = NULL;
#line 93 "Exif.vala"
	photo_exif_update (self);
#line 95 "Exif.vala"
	if (self->priv->no_exif) {
#line 422 "Exif.c"
		result = FALSE;
		return result;
	}
	{
		GFileInputStream* fins;
		JpegMarker marker;
		gint segment_length;
		gint _tmp0_;
		gboolean _tmp1_;
		gint _tmp2_;
		gboolean _tmp3_;
		guint8* _tmp5_;
		gint sig_size;
		gint sig_length1;
		gint _tmp4_;
		guint8* sig;
		gsize bytes_read;
		fins = g_file_read (self->priv->file, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch22_g_error;
			goto __finally22;
		}
		marker = 0;
		segment_length = 0;
		_tmp0_ = jpeg_read_marker (fins, &marker, &_inner_error_);
		if (_inner_error_ != NULL) {
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			goto __catch22_g_error;
			goto __finally22;
		}
		/* first marker should be SOI*/
#line 107 "Exif.vala"
		segment_length = _tmp0_;
#line 456 "Exif.c"
		_tmp1_ = FALSE;
#line 108 "Exif.vala"
		if (marker != JPEG_MARKER_SOI) {
#line 108 "Exif.vala"
			_tmp1_ = TRUE;
#line 462 "Exif.c"
		} else {
#line 108 "Exif.vala"
			_tmp1_ = segment_length != 0;
#line 466 "Exif.c"
		}
#line 108 "Exif.vala"
		if (_tmp1_) {
#line 109 "Exif.vala"
			self->priv->no_exif = TRUE;
#line 472 "Exif.c"
			result = FALSE;
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			return result;
		}
		_tmp2_ = jpeg_read_marker (fins, &marker, &_inner_error_);
		if (_inner_error_ != NULL) {
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			goto __catch22_g_error;
			goto __finally22;
		}
		/* for EXIF, next marker is always APP1*/
#line 115 "Exif.vala"
		segment_length = _tmp2_;
#line 486 "Exif.c"
		_tmp3_ = FALSE;
#line 116 "Exif.vala"
		if (marker != JPEG_MARKER_APP1) {
#line 116 "Exif.vala"
			_tmp3_ = TRUE;
#line 492 "Exif.c"
		} else {
#line 116 "Exif.vala"
			_tmp3_ = segment_length < 0;
#line 496 "Exif.c"
		}
#line 116 "Exif.vala"
		if (_tmp3_) {
#line 117 "Exif.vala"
			self->priv->no_exif = TRUE;
#line 502 "Exif.c"
			result = FALSE;
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			return result;
		}
#line 122 "Exif.vala"
		_tmp5_ = NULL;
#line 509 "Exif.c"
		sig = (_tmp5_ = g_new0 (guint8, _tmp4_ = G_N_ELEMENTS (EXIF_SIGNATURE)), sig_length1 = _tmp4_, sig_size = sig_length1, _tmp5_);
		bytes_read = 0UL;
#line 124 "Exif.vala"
		g_input_stream_read_all (G_INPUT_STREAM (fins), sig, (gsize) G_N_ELEMENTS (EXIF_SIGNATURE), &bytes_read, NULL, &_inner_error_);
#line 514 "Exif.c"
		if (_inner_error_ != NULL) {
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			sig = (g_free (sig), NULL);
			goto __catch22_g_error;
			goto __finally22;
		}
#line 125 "Exif.vala"
		if (bytes_read != G_N_ELEMENTS (EXIF_SIGNATURE)) {
#line 126 "Exif.vala"
			self->priv->no_exif = TRUE;
#line 525 "Exif.c"
			result = FALSE;
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			sig = (g_free (sig), NULL);
			return result;
		}
		{
			gint ctr;
			ctr = 0;
			{
				gboolean _tmp6_;
				_tmp6_ = TRUE;
#line 131 "Exif.vala"
				while (TRUE) {
#line 131 "Exif.vala"
					if (!_tmp6_) {
#line 131 "Exif.vala"
						ctr++;
#line 543 "Exif.c"
					}
#line 131 "Exif.vala"
					_tmp6_ = FALSE;
#line 131 "Exif.vala"
					if (!(ctr < G_N_ELEMENTS (EXIF_SIGNATURE))) {
#line 131 "Exif.vala"
						break;
#line 551 "Exif.c"
					}
#line 132 "Exif.vala"
					if (sig[ctr] != EXIF_SIGNATURE[ctr]) {
#line 133 "Exif.vala"
						self->priv->no_exif = TRUE;
#line 557 "Exif.c"
						result = FALSE;
						(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
						sig = (g_free (sig), NULL);
						return result;
					}
				}
			}
		}
#line 139 "Exif.vala"
		self->priv->no_exif = FALSE;
#line 568 "Exif.c"
		result = TRUE;
		(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
		sig = (g_free (sig), NULL);
		return result;
	}
	goto __finally22;
	__catch22_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 143 "Exif.vala"
			g_debug ("Exif.vala:143: Error checking for EXIF presence: %s", err->message);
#line 583 "Exif.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally22:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
#line 146 "Exif.vala"
	self->priv->no_exif = TRUE;
#line 595 "Exif.c"
	result = FALSE;
	return result;
}


#line 151 "Exif.vala"
ExifData* photo_exif_get_exif (PhotoExif* self) {
#line 603 "Exif.c"
	ExifData* result;
#line 151 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), NULL);
#line 152 "Exif.vala"
	if (self->priv->exif != NULL) {
#line 609 "Exif.c"
		ExifData* _tmp0_;
#line 153 "Exif.vala"
		_tmp0_ = NULL;
#line 613 "Exif.c"
		result = (_tmp0_ = self->priv->exif, (_tmp0_ == NULL) ? NULL : exif_data_ref (_tmp0_), _tmp0_);
		return result;
	}
#line 155 "Exif.vala"
	if (!photo_exif_has_exif (self)) {
#line 619 "Exif.c"
		result = NULL;
		return result;
	}
#line 158 "Exif.vala"
	photo_exif_update (self);
#line 625 "Exif.c"
	result = NULL;
	return result;
}


#line 163 "Exif.vala"
void photo_exif_set_exif (PhotoExif* self, ExifData* exif) {
#line 633 "Exif.c"
	ExifData* _tmp1_;
	ExifData* _tmp0_;
#line 163 "Exif.vala"
	g_return_if_fail (IS_PHOTO_EXIF (self));
#line 163 "Exif.vala"
	g_return_if_fail (exif != NULL);
#line 640 "Exif.c"
	_tmp1_ = NULL;
#line 164 "Exif.vala"
	_tmp0_ = NULL;
#line 164 "Exif.vala"
	self->priv->exif = (_tmp1_ = (_tmp0_ = exif, (_tmp0_ == NULL) ? NULL : exif_data_ref (_tmp0_), _tmp0_), (self->priv->exif == NULL) ? NULL : (self->priv->exif = (exif_data_unref (self->priv->exif), NULL)), _tmp1_);
#line 165 "Exif.vala"
	self->priv->no_exif = FALSE;
#line 648 "Exif.c"
}


#line 168 "Exif.vala"
Orientation photo_exif_get_orientation (PhotoExif* self) {
#line 654 "Exif.c"
	Orientation result;
	ExifEntry* entry;
	gint o;
	gboolean _tmp0_;
#line 168 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), 0);
#line 169 "Exif.vala"
	photo_exif_update (self);
#line 663 "Exif.c"
	entry = photo_exif_find_entry (self, EXIF_IFD_0, EXIF_TAG_ORIENTATION, EXIF_FORMAT_SHORT);
#line 172 "Exif.vala"
	if (entry == NULL) {
#line 667 "Exif.c"
		result = ORIENTATION_TOP_LEFT;
		(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
		return result;
	}
	o = (gint) exif_get_short (entry->data, exif_data_get_byte_order (self->priv->exif));
	_tmp0_ = FALSE;
#line 176 "Exif.vala"
	if (o < ((gint) ORIENTATION_MIN)) {
#line 176 "Exif.vala"
		_tmp0_ = TRUE;
#line 678 "Exif.c"
	} else {
#line 176 "Exif.vala"
		_tmp0_ = o > ((gint) ORIENTATION_MAX);
#line 682 "Exif.c"
	}
#line 176 "Exif.vala"
	if (_tmp0_) {
#line 686 "Exif.c"
		result = ORIENTATION_TOP_LEFT;
		(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
		return result;
	}
	result = (Orientation) o;
	(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
	return result;
}


#line 182 "Exif.vala"
void photo_exif_set_orientation (PhotoExif* self, Orientation orientation) {
#line 699 "Exif.c"
	ExifEntry* entry;
#line 182 "Exif.vala"
	g_return_if_fail (IS_PHOTO_EXIF (self));
#line 183 "Exif.vala"
	photo_exif_update (self);
#line 705 "Exif.c"
	entry = photo_exif_find_first_entry (self, EXIF_TAG_ORIENTATION, EXIF_FORMAT_SHORT);
#line 186 "Exif.vala"
	if (entry == NULL) {
#line 709 "Exif.c"
		/* TODO: Need a fall-back here*/
#line 188 "Exif.vala"
		g_error ("Exif.vala:188: Unable to set orientation: no entry found");
#line 713 "Exif.c"
	}
#line 191 "Exif.vala"
	exif_set_short (entry->data, exif_data_get_byte_order (self->priv->exif), (guint16) orientation);
#line 717 "Exif.c"
	(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
}


#line 194 "Exif.vala"
gboolean photo_exif_get_dimensions (PhotoExif* self, Dimensions* dim) {
#line 724 "Exif.c"
	gboolean result;
	ExifEntry* width;
	ExifEntry* height;
	gboolean _tmp0_;
#line 194 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), FALSE);
#line 195 "Exif.vala"
	photo_exif_update (self);
#line 733 "Exif.c"
	width = photo_exif_find_entry_multiformat (self, EXIF_IFD_EXIF, EXIF_TAG_PIXEL_X_DIMENSION, EXIF_FORMAT_SHORT, EXIF_FORMAT_LONG);
	height = photo_exif_find_entry_multiformat (self, EXIF_IFD_EXIF, EXIF_TAG_PIXEL_Y_DIMENSION, EXIF_FORMAT_SHORT, EXIF_FORMAT_LONG);
	_tmp0_ = FALSE;
#line 201 "Exif.vala"
	if (width == NULL) {
#line 201 "Exif.vala"
		_tmp0_ = TRUE;
#line 741 "Exif.c"
	} else {
#line 201 "Exif.vala"
		_tmp0_ = height == NULL;
#line 745 "Exif.c"
	}
#line 201 "Exif.vala"
	if (_tmp0_) {
#line 749 "Exif.c"
		result = FALSE;
		(width == NULL) ? NULL : (width = (exif_entry_unref (width), NULL));
		(height == NULL) ? NULL : (height = (exif_entry_unref (height), NULL));
		return result;
	}
#line 204 "Exif.vala"
	if (width->format == EXIF_FORMAT_SHORT) {
#line 205 "Exif.vala"
		(*dim).width = (gint) exif_get_short (width->data, exif_data_get_byte_order (self->priv->exif));
#line 759 "Exif.c"
	} else {
#line 207 "Exif.vala"
		g_assert (width->format == EXIF_FORMAT_LONG);
#line 209 "Exif.vala"
		(*dim).width = (gint) exif_get_long (width->data, exif_data_get_byte_order (self->priv->exif));
#line 765 "Exif.c"
	}
#line 212 "Exif.vala"
	if (height->format == EXIF_FORMAT_SHORT) {
#line 213 "Exif.vala"
		(*dim).height = (gint) exif_get_short (height->data, exif_data_get_byte_order (self->priv->exif));
#line 771 "Exif.c"
	} else {
#line 215 "Exif.vala"
		g_assert (height->format == EXIF_FORMAT_LONG);
#line 217 "Exif.vala"
		(*dim).height = (gint) exif_get_long (height->data, exif_data_get_byte_order (self->priv->exif));
#line 777 "Exif.c"
	}
	result = TRUE;
	(width == NULL) ? NULL : (width = (exif_entry_unref (width), NULL));
	(height == NULL) ? NULL : (height = (exif_entry_unref (height), NULL));
	return result;
}


#line 223 "Exif.vala"
void photo_exif_set_dimensions (PhotoExif* self, const Dimensions* dim) {
#line 788 "Exif.c"
	ExifEntry* width;
	ExifEntry* height;
	gboolean _tmp0_;
#line 223 "Exif.vala"
	g_return_if_fail (IS_PHOTO_EXIF (self));
#line 224 "Exif.vala"
	photo_exif_update (self);
#line 796 "Exif.c"
	width = photo_exif_find_entry_multiformat (self, EXIF_IFD_EXIF, EXIF_TAG_PIXEL_X_DIMENSION, EXIF_FORMAT_SHORT, EXIF_FORMAT_LONG);
	height = photo_exif_find_entry_multiformat (self, EXIF_IFD_EXIF, EXIF_TAG_PIXEL_Y_DIMENSION, EXIF_FORMAT_SHORT, EXIF_FORMAT_LONG);
	_tmp0_ = FALSE;
#line 230 "Exif.vala"
	if (width == NULL) {
#line 230 "Exif.vala"
		_tmp0_ = TRUE;
#line 804 "Exif.c"
	} else {
#line 230 "Exif.vala"
		_tmp0_ = height == NULL;
#line 808 "Exif.c"
	}
#line 230 "Exif.vala"
	if (_tmp0_) {
#line 812 "Exif.c"
		(width == NULL) ? NULL : (width = (exif_entry_unref (width), NULL));
		(height == NULL) ? NULL : (height = (exif_entry_unref (height), NULL));
		return;
	}
#line 233 "Exif.vala"
	if (width->format == EXIF_FORMAT_SHORT) {
#line 234 "Exif.vala"
		exif_set_short (width->data, exif_data_get_byte_order (self->priv->exif), (guint16) (*dim).width);
#line 821 "Exif.c"
	} else {
#line 236 "Exif.vala"
		g_assert (width->format == EXIF_FORMAT_LONG);
#line 238 "Exif.vala"
		exif_set_long (width->data, exif_data_get_byte_order (self->priv->exif), (guint32) (*dim).width);
#line 827 "Exif.c"
	}
#line 241 "Exif.vala"
	if (height->format == EXIF_FORMAT_SHORT) {
#line 242 "Exif.vala"
		exif_set_short (height->data, exif_data_get_byte_order (self->priv->exif), (guint16) (*dim).height);
#line 833 "Exif.c"
	} else {
#line 244 "Exif.vala"
		g_assert (height->format == EXIF_FORMAT_LONG);
#line 246 "Exif.vala"
		exif_set_long (height->data, exif_data_get_byte_order (self->priv->exif), (guint32) (*dim).height);
#line 839 "Exif.c"
	}
	(width == NULL) ? NULL : (width = (exif_entry_unref (width), NULL));
	(height == NULL) ? NULL : (height = (exif_entry_unref (height), NULL));
}


#line 250 "Exif.vala"
char* photo_exif_get_datetime (PhotoExif* self) {
#line 848 "Exif.c"
	char* result;
	ExifEntry* datetime;
	const char* _tmp1_;
	gint _tmp0__length1;
	gchar* _tmp0_;
	char* _tmp2_;
#line 250 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), NULL);
#line 251 "Exif.vala"
	photo_exif_update (self);
#line 859 "Exif.c"
	datetime = photo_exif_find_entry (self, EXIF_IFD_EXIF, EXIF_TAG_DATE_TIME_ORIGINAL, EXIF_FORMAT_ASCII);
#line 254 "Exif.vala"
	if (datetime == NULL) {
#line 863 "Exif.c"
		result = NULL;
		(datetime == NULL) ? NULL : (datetime = (exif_entry_unref (datetime), NULL));
		return result;
	}
#line 257 "Exif.vala"
	_tmp1_ = NULL;
#line 127 "libexif.vapi"
	_tmp0_ = NULL;
#line 257 "Exif.vala"
	_tmp2_ = NULL;
#line 874 "Exif.c"
	result = (_tmp2_ = (_tmp1_ = exif_entry_get_value (datetime, (_tmp0_ = g_new0 (gchar, 256), _tmp0__length1 = 256, _tmp0_), 256), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
	(datetime == NULL) ? NULL : (datetime = (exif_entry_unref (datetime), NULL));
	return result;
	(datetime == NULL) ? NULL : (datetime = (exif_entry_unref (datetime), NULL));
}


#line 260 "Exif.vala"
gboolean photo_exif_get_timestamp (PhotoExif* self, time_t* timestamp) {
#line 884 "Exif.c"
	gboolean result;
	char* datetime;
#line 260 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), FALSE);
#line 889 "Exif.c"
	datetime = photo_exif_get_datetime (self);
#line 262 "Exif.vala"
	if (datetime == NULL) {
#line 893 "Exif.c"
		result = FALSE;
		datetime = (g_free (datetime), NULL);
		return result;
	}
	result = exif_convert_datetime (datetime, &(*timestamp));
	datetime = (g_free (datetime), NULL);
	return result;
}


#line 268 "Exif.vala"
gint photo_exif_remove_all_tags (PhotoExif* self, ExifTag tag) {
#line 906 "Exif.c"
	gint result;
#line 268 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), 0);
#line 269 "Exif.vala"
	photo_exif_update (self);
#line 271 "Exif.vala"
	if (self->priv->exif == NULL) {
#line 914 "Exif.c"
		result = 0;
		return result;
	}
	result = exif_remove_all_tags (self->priv->exif, tag);
	return result;
}


#line 277 "Exif.vala"
gboolean photo_exif_remove_thumbnail (PhotoExif* self) {
#line 925 "Exif.c"
	gboolean result;
#line 277 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), FALSE);
#line 278 "Exif.vala"
	photo_exif_update (self);
#line 280 "Exif.vala"
	if (self->priv->exif == NULL) {
#line 933 "Exif.c"
		result = FALSE;
		return result;
	}
#line 283 "Exif.vala"
	if (self->priv->exif->data == NULL) {
#line 939 "Exif.c"
		result = FALSE;
		return result;
	}
#line 286 "Exif.vala"
	free (self->priv->exif->data);
#line 287 "Exif.vala"
	self->priv->exif->data = NULL;
#line 288 "Exif.vala"
	self->priv->exif->size = (guint) 0;
#line 949 "Exif.c"
	result = TRUE;
	return result;
}


#line 293 "Exif.vala"
static void photo_exif_update (PhotoExif* self) {
#line 957 "Exif.c"
	ExifData* _tmp1_;
	char* _tmp0_;
#line 293 "Exif.vala"
	g_return_if_fail (IS_PHOTO_EXIF (self));
#line 294 "Exif.vala"
	if (self->priv->no_exif) {
#line 964 "Exif.c"
		return;
	}
	/* TODO: Update internal data structures if file changes*/
#line 298 "Exif.vala"
	if (self->priv->exif != NULL) {
#line 970 "Exif.c"
		return;
	}
	_tmp1_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 301 "Exif.vala"
	self->priv->exif = (_tmp1_ = exif_data_new_from_file (_tmp0_ = g_file_get_path (self->priv->file)), (self->priv->exif == NULL) ? NULL : (self->priv->exif = (exif_data_unref (self->priv->exif), NULL)), _tmp1_);
#line 978 "Exif.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 302 "Exif.vala"
	if (self->priv->exif == NULL) {
#line 303 "Exif.vala"
		self->priv->no_exif = TRUE;
#line 984 "Exif.c"
		return;
	}
	/* fix now, all at once*/
#line 309 "Exif.vala"
	exif_data_fix (self->priv->exif);
#line 990 "Exif.c"
}


#line 312 "Exif.vala"
static ExifEntry* photo_exif_find_entry (PhotoExif* self, ExifIfd ifd, ExifTag tag, ExifFormat format) {
#line 996 "Exif.c"
	ExifEntry* result;
	ExifContent* _tmp0_;
	ExifContent* content;
	ExifEntry* _tmp1_;
	ExifEntry* entry;
	gboolean _tmp2_;
#line 312 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), NULL);
#line 313 "Exif.vala"
	g_assert (self->priv->exif != NULL);
#line 315 "Exif.vala"
	_tmp0_ = NULL;
#line 1009 "Exif.c"
	content = (_tmp0_ = self->priv->exif->ifd[(gint) ifd], (_tmp0_ == NULL) ? NULL : exif_content_ref (_tmp0_), _tmp0_);
#line 316 "Exif.vala"
	g_assert (content != NULL);
#line 318 "Exif.vala"
	_tmp1_ = NULL;
#line 1015 "Exif.c"
	entry = (_tmp1_ = exif_content_get_entry (content, tag), (_tmp1_ == NULL) ? NULL : exif_entry_ref (_tmp1_), _tmp1_);
#line 319 "Exif.vala"
	if (entry == NULL) {
#line 1019 "Exif.c"
		result = NULL;
		(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
		(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
		return result;
	}
#line 322 "Exif.vala"
	g_assert (entry->format == format);
#line 1027 "Exif.c"
	_tmp2_ = FALSE;
#line 323 "Exif.vala"
	if (format != EXIF_FORMAT_ASCII) {
#line 323 "Exif.vala"
		_tmp2_ = format != EXIF_FORMAT_UNDEFINED;
#line 1033 "Exif.c"
	} else {
#line 323 "Exif.vala"
		_tmp2_ = FALSE;
#line 1037 "Exif.c"
	}
#line 323 "Exif.vala"
	if (_tmp2_) {
#line 324 "Exif.vala"
		g_assert (entry->size == exif_format_get_size (format));
#line 1043 "Exif.c"
	}
	result = entry;
	(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
	return result;
}


#line 329 "Exif.vala"
static ExifEntry* photo_exif_find_first_entry (PhotoExif* self, ExifTag tag, ExifFormat format) {
#line 1053 "Exif.c"
	ExifEntry* result;
#line 329 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), NULL);
#line 330 "Exif.vala"
	g_assert (self->priv->exif != NULL);
#line 1059 "Exif.c"
	result = exif_find_first_entry (self->priv->exif, tag, format);
	return result;
}


#line 335 "Exif.vala"
static ExifEntry* photo_exif_find_entry_multiformat (PhotoExif* self, ExifIfd ifd, ExifTag tag, ExifFormat format1, ExifFormat format2) {
#line 1067 "Exif.c"
	ExifEntry* result;
	ExifContent* _tmp0_;
	ExifContent* content;
	ExifEntry* _tmp1_;
	ExifEntry* entry;
	gboolean _tmp2_;
	gboolean _tmp3_;
#line 335 "Exif.vala"
	g_return_val_if_fail (IS_PHOTO_EXIF (self), NULL);
#line 337 "Exif.vala"
	g_assert (self->priv->exif != NULL);
#line 339 "Exif.vala"
	_tmp0_ = NULL;
#line 1081 "Exif.c"
	content = (_tmp0_ = self->priv->exif->ifd[(gint) ifd], (_tmp0_ == NULL) ? NULL : exif_content_ref (_tmp0_), _tmp0_);
#line 340 "Exif.vala"
	g_assert (content != NULL);
#line 342 "Exif.vala"
	_tmp1_ = NULL;
#line 1087 "Exif.c"
	entry = (_tmp1_ = exif_content_get_entry (content, tag), (_tmp1_ == NULL) ? NULL : exif_entry_ref (_tmp1_), _tmp1_);
#line 343 "Exif.vala"
	if (entry == NULL) {
#line 1091 "Exif.c"
		result = NULL;
		(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
		(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
		return result;
	}
	_tmp2_ = FALSE;
#line 346 "Exif.vala"
	if (entry->format == format1) {
#line 346 "Exif.vala"
		_tmp2_ = TRUE;
#line 1102 "Exif.c"
	} else {
#line 346 "Exif.vala"
		_tmp2_ = entry->format == format2;
#line 1106 "Exif.c"
	}
#line 346 "Exif.vala"
	g_assert (_tmp2_);
#line 1110 "Exif.c"
	_tmp3_ = FALSE;
#line 347 "Exif.vala"
	if (entry->format != EXIF_FORMAT_ASCII) {
#line 347 "Exif.vala"
		_tmp3_ = entry->format != EXIF_FORMAT_UNDEFINED;
#line 1116 "Exif.c"
	} else {
#line 347 "Exif.vala"
		_tmp3_ = FALSE;
#line 1120 "Exif.c"
	}
#line 347 "Exif.vala"
	if (_tmp3_) {
#line 1124 "Exif.c"
		gboolean _tmp4_;
		_tmp4_ = FALSE;
#line 348 "Exif.vala"
		if (entry->size == exif_format_get_size (format1)) {
#line 348 "Exif.vala"
			_tmp4_ = TRUE;
#line 1131 "Exif.c"
		} else {
#line 348 "Exif.vala"
			_tmp4_ = entry->size == exif_format_get_size (format2);
#line 1135 "Exif.c"
		}
#line 348 "Exif.vala"
		g_assert (_tmp4_);
#line 1139 "Exif.c"
	}
	result = entry;
	(content == NULL) ? NULL : (content = (exif_content_unref (content), NULL));
	return result;
}


#line 353 "Exif.vala"
void photo_exif_commit (PhotoExif* self, GError** error) {
#line 1149 "Exif.c"
	GError * _inner_error_;
	GFileInputStream* fins;
	JpegMarker marker;
	gint segment_length;
	gboolean original_has_exif;
	gint _tmp0_;
	gboolean _tmp1_;
	gint _tmp5_;
	guchar* flattened;
	gint flattened_size;
#line 353 "Exif.vala"
	g_return_if_fail (IS_PHOTO_EXIF (self));
#line 1162 "Exif.c"
	_inner_error_ = NULL;
#line 354 "Exif.vala"
	if (self->priv->exif == NULL) {
#line 1166 "Exif.c"
		return;
	}
	fins = g_file_read (self->priv->file, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
	marker = 0;
	segment_length = 0;
	original_has_exif = TRUE;
	_tmp0_ = jpeg_read_marker (fins, &marker, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
		return;
	}
	/* first marker is always SOI*/
#line 364 "Exif.vala"
	segment_length = _tmp0_;
#line 1186 "Exif.c"
	_tmp1_ = FALSE;
#line 365 "Exif.vala"
	if (marker != JPEG_MARKER_SOI) {
#line 365 "Exif.vala"
		_tmp1_ = TRUE;
#line 1192 "Exif.c"
	} else {
#line 365 "Exif.vala"
		_tmp1_ = segment_length != 0;
#line 1196 "Exif.c"
	}
#line 365 "Exif.vala"
	if (_tmp1_) {
#line 1200 "Exif.c"
		char* _tmp3_;
		char* _tmp2_;
		GError* _tmp4_;
#line 820 "glib-2.0.vapi"
		_tmp3_ = NULL;
#line 952 "gio-2.0.vapi"
		_tmp2_ = NULL;
#line 366 "Exif.vala"
		_tmp4_ = NULL;
#line 1210 "Exif.c"
		_inner_error_ = (_tmp4_ = g_error_new_literal (EXIF_ERROR, EXIF_ERROR_FILE_FORMAT, _tmp3_ = g_strdup_printf ("SOI not found in %s", _tmp2_ = g_file_get_path (self->priv->file))), _tmp3_ = (g_free (_tmp3_), NULL), _tmp2_ = (g_free (_tmp2_), NULL), _tmp4_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			return;
		}
	}
	_tmp5_ = jpeg_read_marker (fins, &marker, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
		return;
	}
	/* for EXIF, next marker is always APP1*/
#line 369 "Exif.vala"
	segment_length = _tmp5_;
#line 370 "Exif.vala"
	if (segment_length <= 0) {
#line 1229 "Exif.c"
		char* _tmp6_;
		GError* _tmp7_;
#line 820 "glib-2.0.vapi"
		_tmp6_ = NULL;
#line 371 "Exif.vala"
		_tmp7_ = NULL;
#line 1236 "Exif.c"
		_inner_error_ = (_tmp7_ = g_error_new_literal (EXIF_ERROR, EXIF_ERROR_FILE_FORMAT, _tmp6_ = g_strdup_printf ("EXIF APP1 length of %d", segment_length)), _tmp6_ = (g_free (_tmp6_), NULL), _tmp7_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
			return;
		}
	}
#line 372 "Exif.vala"
	if (marker != JPEG_MARKER_APP1) {
#line 373 "Exif.vala"
		original_has_exif = FALSE;
#line 1248 "Exif.c"
	}
	/* flatten exif to buffer*/
	flattened = NULL;
	flattened_size = 0;
#line 378 "Exif.vala"
	exif_data_save_data (self->priv->exif, &flattened, &flattened_size);
#line 379 "Exif.vala"
	g_assert (flattened != NULL);
#line 380 "Exif.vala"
	g_assert (flattened_size > 0);
#line 1259 "Exif.c"
	{
		gboolean _tmp8_;
		_tmp8_ = FALSE;
#line 383 "Exif.vala"
		if (flattened_size == segment_length) {
#line 383 "Exif.vala"
			_tmp8_ = original_has_exif;
#line 1267 "Exif.c"
		} else {
#line 383 "Exif.vala"
			_tmp8_ = FALSE;
#line 1271 "Exif.c"
		}
#line 383 "Exif.vala"
		if (_tmp8_) {
#line 1275 "Exif.c"
			char* _tmp9_;
			GFileInputStream* _tmp10_;
			char* _tmp11_;
			FILE* _tmp12_;
			FILE* fs;
			gsize count_written;
			/* the new EXIF data is exactly the same size as the data in the file, so simply
			 overwrite*/
#line 952 "gio-2.0.vapi"
			_tmp9_ = NULL;
#line 387 "Exif.vala"
			g_debug ("Exif.vala:387: Writing EXIF in-place of %d bytes to %s", flattened_size, _tmp9_ = g_file_get_path (self->priv->file));
#line 1288 "Exif.c"
			_tmp9_ = (g_free (_tmp9_), NULL);
			/* close for reading*/
#line 390 "Exif.vala"
			g_input_stream_close (G_INPUT_STREAM (fins), NULL, &_inner_error_);
#line 1293 "Exif.c"
			if (_inner_error_ != NULL) {
				goto __finally23;
			}
			_tmp10_ = NULL;
#line 391 "Exif.vala"
			fins = (_tmp10_ = NULL, (fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL)), _tmp10_);
#line 1300 "Exif.c"
			/* open for writing ... don't use FileOutputStream, as it will overwrite everything
			 it seeks over*/
#line 952 "gio-2.0.vapi"
			_tmp11_ = NULL;
#line 395 "Exif.vala"
			_tmp12_ = NULL;
#line 1307 "Exif.c"
			fs = (_tmp12_ = fopen (_tmp11_ = g_file_get_path (self->priv->file), "r+"), _tmp11_ = (g_free (_tmp11_), NULL), _tmp12_);
#line 396 "Exif.vala"
			if (fs == NULL) {
#line 1311 "Exif.c"
				char* _tmp14_;
				char* _tmp13_;
				GError* _tmp15_;
#line 820 "glib-2.0.vapi"
				_tmp14_ = NULL;
#line 952 "gio-2.0.vapi"
				_tmp13_ = NULL;
#line 397 "Exif.vala"
				_tmp15_ = NULL;
#line 1321 "Exif.c"
				_inner_error_ = (_tmp15_ = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _tmp14_ = g_strdup_printf ("%s: fopen() error", _tmp13_ = g_file_get_path (self->priv->file))), _tmp14_ = (g_free (_tmp14_), NULL), _tmp13_ = (g_free (_tmp13_), NULL), _tmp15_);
				if (_inner_error_ != NULL) {
					(fs == NULL) ? NULL : (fs = (fclose (fs), NULL));
					goto __finally23;
				}
			}
			/* seek over Marker:SOI, Marker:APP1, and length (none change)*/
#line 400 "Exif.vala"
			if (fseek (fs, (glong) ((2 + 2) + 2), SEEK_SET) < 0) {
#line 1331 "Exif.c"
				char* _tmp16_;
				GError* _tmp17_;
#line 952 "gio-2.0.vapi"
				_tmp16_ = NULL;
#line 401 "Exif.vala"
				_tmp17_ = NULL;
#line 1338 "Exif.c"
				_inner_error_ = (_tmp17_ = g_error_new (G_IO_ERROR, G_IO_ERROR_FAILED, "%s: fseek() error %d", _tmp16_ = g_file_get_path (self->priv->file), ferror (fs)), _tmp16_ = (g_free (_tmp16_), NULL), _tmp17_);
				if (_inner_error_ != NULL) {
					(fs == NULL) ? NULL : (fs = (fclose (fs), NULL));
					goto __finally23;
				}
			}
			/* write data in over current EXIF*/
			count_written = fwrite (flattened, (gsize) flattened_size, (gsize) 1, fs);
#line 405 "Exif.vala"
			if (count_written != 1) {
#line 1349 "Exif.c"
				char* _tmp18_;
				GError* _tmp19_;
#line 952 "gio-2.0.vapi"
				_tmp18_ = NULL;
#line 406 "Exif.vala"
				_tmp19_ = NULL;
#line 1356 "Exif.c"
				_inner_error_ = (_tmp19_ = g_error_new (G_IO_ERROR, G_IO_ERROR_FAILED, "%s: fwrite() error %d", _tmp18_ = g_file_get_path (self->priv->file), errno), _tmp18_ = (g_free (_tmp18_), NULL), _tmp19_);
				if (_inner_error_ != NULL) {
					(fs == NULL) ? NULL : (fs = (fclose (fs), NULL));
					goto __finally23;
				}
			}
			(fs == NULL) ? NULL : (fs = (fclose (fs), NULL));
		} else {
			GFile* temp;
			GFile* _tmp22_;
			GFileOutputStream* _tmp21_;
			GFile* _tmp20_;
			GFileOutputStream* fouts;
			gsize bytes_written;
			char* _tmp24_;
			char* _tmp23_;
			guint8* _tmp25_;
			gint copy_buffer_size;
			gint copy_buffer_length1;
			guint8* copy_buffer;
			/* create a new photo file with the updated EXIF and move it on top of the old one
			 skip past APP1*/
#line 411 "Exif.vala"
			if (original_has_exif) {
#line 412 "Exif.vala"
				g_input_stream_skip (G_INPUT_STREAM (fins), (gsize) segment_length, NULL, &_inner_error_);
#line 1383 "Exif.c"
				if (_inner_error_ != NULL) {
					goto __finally23;
				}
			}
			temp = NULL;
			_tmp22_ = NULL;
			_tmp21_ = NULL;
			_tmp20_ = NULL;
			fouts = (_tmp21_ = photo_exif_create_temp (self->priv->file, &_tmp20_, &_inner_error_), temp = (_tmp22_ = _tmp20_, (temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL)), _tmp22_), _tmp21_);
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				goto __finally23;
			}
			bytes_written = (gsize) 0;
#line 952 "gio-2.0.vapi"
			_tmp24_ = NULL;
#line 952 "gio-2.0.vapi"
			_tmp23_ = NULL;
#line 418 "Exif.vala"
			g_debug ("Exif.vala:418: Building new file at %s with %d bytes EXIF, overwriting %s", _tmp23_ = g_file_get_path (temp), flattened_size, _tmp24_ = g_file_get_path (self->priv->file));
#line 1404 "Exif.c"
			_tmp24_ = (g_free (_tmp24_), NULL);
			_tmp23_ = (g_free (_tmp23_), NULL);
			/* write SOI*/
#line 422 "Exif.vala"
			jpeg_write_marker (fouts, JPEG_MARKER_SOI, 0, &_inner_error_);
#line 1410 "Exif.c"
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
				goto __finally23;
			}
			/* write APP1 with EXIF data*/
#line 425 "Exif.vala"
			jpeg_write_marker (fouts, JPEG_MARKER_APP1, flattened_size, &_inner_error_);
#line 1419 "Exif.c"
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
				goto __finally23;
			}
#line 426 "Exif.vala"
			g_output_stream_write_all (G_OUTPUT_STREAM (fouts), flattened, (gsize) flattened_size, &bytes_written, NULL, &_inner_error_);
#line 1427 "Exif.c"
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
				goto __finally23;
			}
#line 427 "Exif.vala"
			g_assert (bytes_written == flattened_size);
#line 1435 "Exif.c"
			/* if original has no EXIF, need to write the marker read in earlier*/
#line 430 "Exif.vala"
			if (!original_has_exif) {
#line 1439 "Exif.c"
				/* if APP0, then it's JFIF, and don't want to write this segment*/
#line 432 "Exif.vala"
				if (marker != JPEG_MARKER_APP0) {
#line 433 "Exif.vala"
					jpeg_write_marker (fouts, marker, segment_length, &_inner_error_);
#line 1445 "Exif.c"
					if (_inner_error_ != NULL) {
						(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
						(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
						goto __finally23;
					}
				} else {
#line 435 "Exif.vala"
					g_input_stream_skip (G_INPUT_STREAM (fins), (gsize) segment_length, NULL, &_inner_error_);
#line 1454 "Exif.c"
					if (_inner_error_ != NULL) {
						(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
						(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
						goto __finally23;
					}
				}
			}
			/* copy remainder of file into new file*/
#line 440 "Exif.vala"
			_tmp25_ = NULL;
#line 1465 "Exif.c"
			copy_buffer = (_tmp25_ = g_new0 (guint8, 64 * 1024), copy_buffer_length1 = 64 * 1024, copy_buffer_size = copy_buffer_length1, _tmp25_);
			{
				gboolean _tmp26_;
				_tmp26_ = TRUE;
#line 441 "Exif.vala"
				while (TRUE) {
#line 1472 "Exif.c"
					gssize bytes_read;
#line 441 "Exif.vala"
					if (!_tmp26_) {
#line 1476 "Exif.c"
					}
#line 441 "Exif.vala"
					_tmp26_ = FALSE;
#line 1480 "Exif.c"
					bytes_read = g_input_stream_read (G_INPUT_STREAM (fins), copy_buffer, (gsize) copy_buffer_length1, NULL, &_inner_error_);
					if (_inner_error_ != NULL) {
						(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
						(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
						copy_buffer = (g_free (copy_buffer), NULL);
						goto __finally23;
					}
#line 443 "Exif.vala"
					if (bytes_read == 0) {
#line 444 "Exif.vala"
						break;
#line 1492 "Exif.c"
					}
#line 446 "Exif.vala"
					g_assert (bytes_read > 0);
#line 448 "Exif.vala"
					g_output_stream_write_all (G_OUTPUT_STREAM (fouts), copy_buffer, (gsize) bytes_read, &bytes_written, NULL, &_inner_error_);
#line 1498 "Exif.c"
					if (_inner_error_ != NULL) {
						(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
						(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
						copy_buffer = (g_free (copy_buffer), NULL);
						goto __finally23;
					}
#line 449 "Exif.vala"
					g_assert (bytes_written == bytes_read);
#line 1507 "Exif.c"
				}
			}
			/* close both for move*/
#line 453 "Exif.vala"
			g_output_stream_close (G_OUTPUT_STREAM (fouts), NULL, &_inner_error_);
#line 1513 "Exif.c"
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
				copy_buffer = (g_free (copy_buffer), NULL);
				goto __finally23;
			}
#line 454 "Exif.vala"
			g_input_stream_close (G_INPUT_STREAM (fins), NULL, &_inner_error_);
#line 1522 "Exif.c"
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
				copy_buffer = (g_free (copy_buffer), NULL);
				goto __finally23;
			}
#line 456 "Exif.vala"
			g_file_move (temp, self->priv->file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &_inner_error_);
#line 1531 "Exif.c"
			if (_inner_error_ != NULL) {
				(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
				copy_buffer = (g_free (copy_buffer), NULL);
				goto __finally23;
			}
			(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
			(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
			copy_buffer = (g_free (copy_buffer), NULL);
		}
	}
	__finally23:
	{
#line 459 "Exif.vala"
		free (flattened);
#line 1547 "Exif.c"
	}
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
		return;
	}
	(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
}


#line 463 "Exif.vala"
static GFileOutputStream* photo_exif_create_temp (GFile* original, GFile** temp, GError** error) {
#line 1560 "Exif.c"
	GFileOutputStream* result;
	GError * _inner_error_;
	GFile* parent;
#line 463 "Exif.vala"
	g_return_val_if_fail (G_IS_FILE (original), NULL);
#line 1566 "Exif.c"
	if (temp != NULL) {
		*temp = NULL;
	}
	_inner_error_ = NULL;
	parent = g_file_get_parent (original);
#line 465 "Exif.vala"
	g_assert (parent != NULL);
#line 1574 "Exif.c"
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 467 "Exif.vala"
			while (TRUE) {
#line 1583 "Exif.c"
				char* _tmp1_;
				GFile* _tmp2_;
				GFile* t;
				GFileOutputStream* fouts;
#line 467 "Exif.vala"
				if (!_tmp0_) {
#line 467 "Exif.vala"
					ctr++;
#line 1592 "Exif.c"
				}
#line 467 "Exif.vala"
				_tmp0_ = FALSE;
#line 467 "Exif.vala"
				if (!(ctr < G_MAXINT)) {
#line 467 "Exif.vala"
					break;
#line 1600 "Exif.c"
				}
#line 820 "glib-2.0.vapi"
				_tmp1_ = NULL;
#line 468 "Exif.vala"
				_tmp2_ = NULL;
#line 1606 "Exif.c"
				t = (_tmp2_ = g_file_get_child (parent, _tmp1_ = g_strdup_printf ("shotwell.%08X.tmp", (guint) ctr)), _tmp1_ = (g_free (_tmp1_), NULL), _tmp2_);
				fouts = g_file_create (t, G_FILE_CREATE_PRIVATE, NULL, &_inner_error_);
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(t == NULL) ? NULL : (t = (g_object_unref (t), NULL));
					(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
					return NULL;
				}
#line 470 "Exif.vala"
				if (fouts != NULL) {
#line 1617 "Exif.c"
					GFile* _tmp4_;
					GFile* _tmp3_;
					_tmp4_ = NULL;
#line 471 "Exif.vala"
					_tmp3_ = NULL;
#line 471 "Exif.vala"
					(*temp) = (_tmp4_ = (_tmp3_ = t, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), ((*temp) == NULL) ? NULL : ((*temp) = (g_object_unref ((*temp)), NULL)), _tmp4_);
#line 1625 "Exif.c"
					result = fouts;
					(t == NULL) ? NULL : (t = (g_object_unref (t), NULL));
					(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
					return result;
				}
				(t == NULL) ? NULL : (t = (g_object_unref (t), NULL));
				(fouts == NULL) ? NULL : (fouts = (g_object_unref (fouts), NULL));
			}
		}
	}
	result = NULL;
	(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
	return result;
}


static void value_photo_exif_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_photo_exif_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		photo_exif_unref (value->data[0].v_pointer);
	}
}


static void value_photo_exif_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = photo_exif_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_photo_exif_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_photo_exif_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		PhotoExif* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = photo_exif_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_photo_exif_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	PhotoExif** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = photo_exif_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_photo_exif (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecPhotoExif* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_PHOTO_EXIF), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_photo_exif (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_PHOTO_EXIF), NULL);
	return value->data[0].v_pointer;
}


void value_set_photo_exif (GValue* value, gpointer v_object) {
	PhotoExif* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_PHOTO_EXIF));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_PHOTO_EXIF));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		photo_exif_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		photo_exif_unref (old);
	}
}


static void photo_exif_class_init (PhotoExifClass * klass) {
	photo_exif_parent_class = g_type_class_peek_parent (klass);
	PHOTO_EXIF_CLASS (klass)->finalize = photo_exif_finalize;
	g_type_class_add_private (klass, sizeof (PhotoExifPrivate));
}


static void photo_exif_instance_init (PhotoExif * self) {
	self->priv = PHOTO_EXIF_GET_PRIVATE (self);
	self->priv->exif = NULL;
	self->priv->no_exif = FALSE;
	self->ref_count = 1;
}


static void photo_exif_finalize (PhotoExif* obj) {
	PhotoExif * self;
	self = PHOTO_EXIF (obj);
	(self->priv->file == NULL) ? NULL : (self->priv->file = (g_object_unref (self->priv->file), NULL));
	(self->priv->exif == NULL) ? NULL : (self->priv->exif = (exif_data_unref (self->priv->exif), NULL));
}


GType photo_exif_get_type (void) {
	static GType photo_exif_type_id = 0;
	if (photo_exif_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_photo_exif_init, value_photo_exif_free_value, value_photo_exif_copy_value, value_photo_exif_peek_pointer, "p", value_photo_exif_collect_value, "p", value_photo_exif_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (PhotoExifClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_exif_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PhotoExif), 0, (GInstanceInitFunc) photo_exif_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		photo_exif_type_id = g_type_register_fundamental (g_type_fundamental_next (), "PhotoExif", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return photo_exif_type_id;
}


gpointer photo_exif_ref (gpointer instance) {
	PhotoExif* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void photo_exif_unref (gpointer instance) {
	PhotoExif* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		PHOTO_EXIF_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




