/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>


#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_EXPORT_DIALOG (export_dialog_get_type ())
#define EXPORT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORT_DIALOG, ExportDialog))
#define EXPORT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORT_DIALOG, ExportDialogClass))
#define IS_EXPORT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORT_DIALOG))
#define IS_EXPORT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORT_DIALOG))
#define EXPORT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORT_DIALOG, ExportDialogClass))

typedef struct _ExportDialog ExportDialog;
typedef struct _ExportDialogClass ExportDialogClass;
typedef struct _ExportDialogPrivate ExportDialogPrivate;

#define TYPE_SCALE_CONSTRAINT (scale_constraint_get_type ())

#define JPEG_TYPE_QUALITY (jpeg_quality_get_type ())

struct _ExportDialog {
	GtkDialog parent_instance;
	ExportDialogPrivate * priv;
};

struct _ExportDialogClass {
	GtkDialogClass parent_class;
};

struct _ExportDialogPrivate {
	GtkTable* table;
	GtkComboBox* quality_combo;
	GtkComboBox* constraint_combo;
	GtkEntry* pixels_entry;
	GtkWidget* ok_button;
	gboolean in_insert;
};

typedef enum  {
	SCALE_CONSTRAINT_ORIGINAL,
	SCALE_CONSTRAINT_DIMENSIONS,
	SCALE_CONSTRAINT_WIDTH,
	SCALE_CONSTRAINT_HEIGHT
} ScaleConstraint;

typedef enum  {
	JPEG_QUALITY_LOW = 50,
	JPEG_QUALITY_MEDIUM = 75,
	JPEG_QUALITY_HIGH = 90,
	JPEG_QUALITY_MAXIMUM = 100
} JpegQuality;


extern GFile* export_ui_current_export_dir;
GFile* export_ui_current_export_dir = NULL;
static ScaleConstraint export_dialog_current_constraint;
static ScaleConstraint export_dialog_current_constraint = SCALE_CONSTRAINT_DIMENSIONS;
static JpegQuality export_dialog_current_quality;
static JpegQuality export_dialog_current_quality = JPEG_QUALITY_HIGH;
static gint export_dialog_current_scale;
static gint export_dialog_current_scale = 1200;
static gpointer export_dialog_parent_class = NULL;

GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
GFile* export_ui_choose_file (GFile* current_file);
GFile* export_ui_choose_dir (void);
gboolean export_ui_query_overwrite (GFile* file);
GType export_dialog_get_type (void);
#define EXPORT_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_EXPORT_DIALOG, ExportDialogPrivate))
enum  {
	EXPORT_DIALOG_DUMMY_PROPERTY
};
GType scale_constraint_get_type (void);
GType jpeg_quality_get_type (void);
char* jpeg_quality_to_string (JpegQuality self);
char* scale_constraint_to_string (ScaleConstraint self);
static void export_dialog_on_constraint_changed (ExportDialog* self);
static void _export_dialog_on_constraint_changed_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self);
static void export_dialog_on_pixels_changed (ExportDialog* self);
static void _export_dialog_on_pixels_changed_gtk_editable_changed (GtkEntry* _sender, gpointer self);
static void export_dialog_on_pixels_insert_text (ExportDialog* self, const char* text, gint length, void* position);
static void _export_dialog_on_pixels_insert_text_gtk_editable_insert_text (GtkEntry* _sender, const char* text, gint length, void* position, gpointer self);
static void export_dialog_on_activate (ExportDialog* self);
static void _export_dialog_on_activate_gtk_entry_activate (GtkEntry* _sender, gpointer self);
static void export_dialog_add_label (ExportDialog* self, const char* text, gint x, gint y);
static void export_dialog_add_control (ExportDialog* self, GtkWidget* widget, gint x, gint y);
ExportDialog* export_dialog_new (gint count);
ExportDialog* export_dialog_construct (GType object_type, gint count);
gboolean export_dialog_execute (ExportDialog* self, gint* scale, ScaleConstraint* constraint, JpegQuality* quality);
static void export_dialog_finalize (GObject* obj);

static const ScaleConstraint EXPORT_DIALOG_CONSTRAINT_ARRAY[] = {SCALE_CONSTRAINT_ORIGINAL, SCALE_CONSTRAINT_DIMENSIONS, SCALE_CONSTRAINT_WIDTH, SCALE_CONSTRAINT_HEIGHT};
static const JpegQuality EXPORT_DIALOG_QUALITY_ARRAY[] = {JPEG_QUALITY_LOW, JPEG_QUALITY_MEDIUM, JPEG_QUALITY_HIGH, JPEG_QUALITY_MAXIMUM};


#line 10 "ExportDialog.vala"
GFile* export_ui_choose_file (GFile* current_file) {
#line 121 "ExportDialog.c"
	GFile* result;
	AppWindow* _tmp1_;
	GtkFileChooserDialog* _tmp2_;
	GtkFileChooserDialog* chooser;
	char* _tmp3_;
	char* _tmp4_;
	GFile* file;
#line 10 "ExportDialog.vala"
	g_return_val_if_fail (G_IS_FILE (current_file), NULL);
#line 11 "ExportDialog.vala"
	if (export_ui_current_export_dir == NULL) {
#line 133 "ExportDialog.c"
		GFile* _tmp0_;
		_tmp0_ = NULL;
#line 12 "ExportDialog.vala"
		export_ui_current_export_dir = (_tmp0_ = g_file_new_for_path (g_get_home_dir ()), (export_ui_current_export_dir == NULL) ? NULL : (export_ui_current_export_dir = (g_object_unref (export_ui_current_export_dir), NULL)), _tmp0_);
#line 138 "ExportDialog.c"
	}
#line 446 "AppWindow.vala"
	_tmp1_ = NULL;
#line 14 "ExportDialog.vala"
	_tmp2_ = NULL;
#line 144 "ExportDialog.c"
	chooser = (_tmp2_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new ("Export Photo", GTK_WINDOW (_tmp1_ = app_window_get_instance ()), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL, NULL)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
#line 17 "ExportDialog.vala"
	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (chooser), TRUE);
#line 952 "gio-2.0.vapi"
	_tmp3_ = NULL;
#line 18 "ExportDialog.vala"
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), _tmp3_ = g_file_get_path (export_ui_current_export_dir));
#line 152 "ExportDialog.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
#line 947 "gio-2.0.vapi"
	_tmp4_ = NULL;
#line 19 "ExportDialog.vala"
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser), _tmp4_ = g_file_get_basename (current_file));
#line 158 "ExportDialog.c"
	_tmp4_ = (g_free (_tmp4_), NULL);
	file = NULL;
#line 22 "ExportDialog.vala"
	if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT) {
#line 163 "ExportDialog.c"
		GFile* _tmp6_;
		char* _tmp5_;
		GFile* _tmp7_;
		_tmp6_ = NULL;
#line 5545 "gtk+-2.0.vapi"
		_tmp5_ = NULL;
#line 23 "ExportDialog.vala"
		file = (_tmp6_ = g_file_new_for_path (_tmp5_ = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser))), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp6_);
#line 172 "ExportDialog.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
		_tmp7_ = NULL;
#line 24 "ExportDialog.vala"
		export_ui_current_export_dir = (_tmp7_ = g_file_get_parent (file), (export_ui_current_export_dir == NULL) ? NULL : (export_ui_current_export_dir = (g_object_unref (export_ui_current_export_dir), NULL)), _tmp7_);
#line 177 "ExportDialog.c"
	}
#line 27 "ExportDialog.vala"
	gtk_object_destroy (GTK_OBJECT (chooser));
#line 181 "ExportDialog.c"
	result = file;
	(chooser == NULL) ? NULL : (chooser = (g_object_unref (chooser), NULL));
	return result;
}


#line 32 "ExportDialog.vala"
GFile* export_ui_choose_dir (void) {
#line 190 "ExportDialog.c"
	GFile* result;
	AppWindow* _tmp1_;
	GtkFileChooserDialog* _tmp2_;
	GtkFileChooserDialog* chooser;
	char* _tmp3_;
	GFile* dir;
#line 33 "ExportDialog.vala"
	if (export_ui_current_export_dir == NULL) {
#line 199 "ExportDialog.c"
		GFile* _tmp0_;
		_tmp0_ = NULL;
#line 34 "ExportDialog.vala"
		export_ui_current_export_dir = (_tmp0_ = g_file_new_for_path (g_get_home_dir ()), (export_ui_current_export_dir == NULL) ? NULL : (export_ui_current_export_dir = (g_object_unref (export_ui_current_export_dir), NULL)), _tmp0_);
#line 204 "ExportDialog.c"
	}
#line 446 "AppWindow.vala"
	_tmp1_ = NULL;
#line 36 "ExportDialog.vala"
	_tmp2_ = NULL;
#line 210 "ExportDialog.c"
	chooser = (_tmp2_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new ("Export Photos", GTK_WINDOW (_tmp1_ = app_window_get_instance ()), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL, NULL)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
#line 952 "gio-2.0.vapi"
	_tmp3_ = NULL;
#line 39 "ExportDialog.vala"
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), _tmp3_ = g_file_get_path (export_ui_current_export_dir));
#line 216 "ExportDialog.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
	dir = NULL;
#line 42 "ExportDialog.vala"
	if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT) {
#line 221 "ExportDialog.c"
		GFile* _tmp5_;
		char* _tmp4_;
		GFile* _tmp7_;
		GFile* _tmp6_;
		_tmp5_ = NULL;
#line 5545 "gtk+-2.0.vapi"
		_tmp4_ = NULL;
#line 43 "ExportDialog.vala"
		dir = (_tmp5_ = g_file_new_for_path (_tmp4_ = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser))), (dir == NULL) ? NULL : (dir = (g_object_unref (dir), NULL)), _tmp5_);
#line 231 "ExportDialog.c"
		_tmp4_ = (g_free (_tmp4_), NULL);
		_tmp7_ = NULL;
#line 44 "ExportDialog.vala"
		_tmp6_ = NULL;
#line 44 "ExportDialog.vala"
		export_ui_current_export_dir = (_tmp7_ = (_tmp6_ = dir, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (export_ui_current_export_dir == NULL) ? NULL : (export_ui_current_export_dir = (g_object_unref (export_ui_current_export_dir), NULL)), _tmp7_);
#line 238 "ExportDialog.c"
	}
#line 47 "ExportDialog.vala"
	gtk_object_destroy (GTK_OBJECT (chooser));
#line 242 "ExportDialog.c"
	result = dir;
	(chooser == NULL) ? NULL : (chooser = (g_object_unref (chooser), NULL));
	return result;
}


#line 52 "ExportDialog.vala"
gboolean export_ui_query_overwrite (GFile* file) {
#line 251 "ExportDialog.c"
	gboolean result;
	char* _tmp1_;
	AppWindow* _tmp0_;
	GtkMessageDialog* _tmp2_;
	GtkMessageDialog* dialog;
	gboolean yes;
#line 52 "ExportDialog.vala"
	g_return_val_if_fail (G_IS_FILE (file), FALSE);
#line 952 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 53 "ExportDialog.vala"
	_tmp2_ = NULL;
#line 266 "ExportDialog.c"
	dialog = (_tmp2_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (GTK_WINDOW (_tmp0_ = app_window_get_instance ()), GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "%s already exists.  Overwrite?", _tmp1_ = g_file_get_path (file), NULL)), _tmp1_ = (g_free (_tmp1_), NULL), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
#line 56 "ExportDialog.vala"
	gtk_window_set_title (GTK_WINDOW (dialog), "Export Photos");
#line 270 "ExportDialog.c"
	yes = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
#line 60 "ExportDialog.vala"
	gtk_object_destroy (GTK_OBJECT (dialog));
#line 274 "ExportDialog.c"
	result = yes;
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	return result;
}


#line 190 "ExportDialog.vala"
static void _export_dialog_on_constraint_changed_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self) {
#line 283 "ExportDialog.c"
	export_dialog_on_constraint_changed (self);
}


#line 204 "ExportDialog.vala"
static void _export_dialog_on_pixels_changed_gtk_editable_changed (GtkEntry* _sender, gpointer self) {
#line 290 "ExportDialog.c"
	export_dialog_on_pixels_changed (self);
}


#line 208 "ExportDialog.vala"
static void _export_dialog_on_pixels_insert_text_gtk_editable_insert_text (GtkEntry* _sender, const char* text, gint length, void* position, gpointer self) {
#line 297 "ExportDialog.c"
	export_dialog_on_pixels_insert_text (self, text, length, position);
}


#line 200 "ExportDialog.vala"
static void _export_dialog_on_activate_gtk_entry_activate (GtkEntry* _sender, gpointer self) {
#line 304 "ExportDialog.c"
	export_dialog_on_activate (self);
}


#line 84 "ExportDialog.vala"
ExportDialog* export_dialog_construct (GType object_type, gint count) {
#line 311 "ExportDialog.c"
	ExportDialog * self;
	const char* _tmp0_;
	char* _tmp1_;
	GtkComboBox* _tmp2_;
	gint ctr;
	GtkComboBox* _tmp4_;
	GtkEntry* _tmp6_;
	char* _tmp7_;
	GtkHBox* pixels_box;
	GtkLabel* _tmp8_;
	GtkWidget* _tmp10_;
	GtkWidget* _tmp9_;
	self = g_object_newv (object_type, 0, NULL);
	_tmp0_ = NULL;
#line 86 "ExportDialog.vala"
	if (count > 1) {
#line 86 "ExportDialog.vala"
		_tmp0_ = "s";
#line 330 "ExportDialog.c"
	} else {
#line 86 "ExportDialog.vala"
		_tmp0_ = "";
#line 334 "ExportDialog.c"
	}
#line 820 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 86 "ExportDialog.vala"
	gtk_window_set_title (GTK_WINDOW (self), _tmp1_ = g_strdup_printf ("Export Photo%s", _tmp0_));
#line 340 "ExportDialog.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
#line 87 "ExportDialog.vala"
	gtk_dialog_set_has_separator (GTK_DIALOG (self), FALSE);
#line 88 "ExportDialog.vala"
	g_object_set (GTK_WINDOW (self), "allow-grow", FALSE, NULL);
#line 346 "ExportDialog.c"
	_tmp2_ = NULL;
#line 91 "ExportDialog.vala"
	self->priv->quality_combo = (_tmp2_ = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), (self->priv->quality_combo == NULL) ? NULL : (self->priv->quality_combo = (g_object_unref (self->priv->quality_combo), NULL)), _tmp2_);
#line 350 "ExportDialog.c"
	ctr = 0;
	{
		JpegQuality* quality_collection;
		int quality_collection_length1;
		int quality_it;
#line 93 "ExportDialog.vala"
		quality_collection = EXPORT_DIALOG_QUALITY_ARRAY;
#line 358 "ExportDialog.c"
		quality_collection_length1 = G_N_ELEMENTS (EXPORT_DIALOG_QUALITY_ARRAY);
		for (quality_it = 0; quality_it < G_N_ELEMENTS (EXPORT_DIALOG_QUALITY_ARRAY); quality_it = quality_it + 1) {
			JpegQuality quality;
			quality = quality_collection[quality_it];
			{
				char* _tmp3_;
#line 154 "image_util.vala"
				_tmp3_ = NULL;
#line 94 "ExportDialog.vala"
				gtk_combo_box_append_text (self->priv->quality_combo, _tmp3_ = jpeg_quality_to_string (quality));
#line 369 "ExportDialog.c"
				_tmp3_ = (g_free (_tmp3_), NULL);
#line 95 "ExportDialog.vala"
				if (quality == export_dialog_current_quality) {
#line 96 "ExportDialog.vala"
					gtk_combo_box_set_active (self->priv->quality_combo, ctr);
#line 375 "ExportDialog.c"
				}
#line 97 "ExportDialog.vala"
				ctr++;
#line 379 "ExportDialog.c"
			}
		}
	}
	_tmp4_ = NULL;
#line 100 "ExportDialog.vala"
	self->priv->constraint_combo = (_tmp4_ = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), (self->priv->constraint_combo == NULL) ? NULL : (self->priv->constraint_combo = (g_object_unref (self->priv->constraint_combo), NULL)), _tmp4_);
#line 101 "ExportDialog.vala"
	ctr = 0;
#line 388 "ExportDialog.c"
	{
		ScaleConstraint* constraint_collection;
		int constraint_collection_length1;
		int constraint_it;
#line 102 "ExportDialog.vala"
		constraint_collection = EXPORT_DIALOG_CONSTRAINT_ARRAY;
#line 395 "ExportDialog.c"
		constraint_collection_length1 = G_N_ELEMENTS (EXPORT_DIALOG_CONSTRAINT_ARRAY);
		for (constraint_it = 0; constraint_it < G_N_ELEMENTS (EXPORT_DIALOG_CONSTRAINT_ARRAY); constraint_it = constraint_it + 1) {
			ScaleConstraint constraint;
			constraint = constraint_collection[constraint_it];
			{
				char* _tmp5_;
#line 13 "Dimensions.vala"
				_tmp5_ = NULL;
#line 103 "ExportDialog.vala"
				gtk_combo_box_append_text (self->priv->constraint_combo, _tmp5_ = scale_constraint_to_string (constraint));
#line 406 "ExportDialog.c"
				_tmp5_ = (g_free (_tmp5_), NULL);
#line 104 "ExportDialog.vala"
				if (constraint == export_dialog_current_constraint) {
#line 105 "ExportDialog.vala"
					gtk_combo_box_set_active (self->priv->constraint_combo, ctr);
#line 412 "ExportDialog.c"
				}
#line 106 "ExportDialog.vala"
				ctr++;
#line 416 "ExportDialog.c"
			}
		}
	}
	_tmp6_ = NULL;
#line 109 "ExportDialog.vala"
	self->priv->pixels_entry = (_tmp6_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->pixels_entry == NULL) ? NULL : (self->priv->pixels_entry = (g_object_unref (self->priv->pixels_entry), NULL)), _tmp6_);
#line 110 "ExportDialog.vala"
	gtk_entry_set_max_length (self->priv->pixels_entry, 6);
#line 820 "glib-2.0.vapi"
	_tmp7_ = NULL;
#line 111 "ExportDialog.vala"
	gtk_entry_set_text (self->priv->pixels_entry, _tmp7_ = g_strdup_printf ("%d", export_dialog_current_scale));
#line 429 "ExportDialog.c"
	_tmp7_ = (g_free (_tmp7_), NULL);
#line 114 "ExportDialog.vala"
	g_signal_connect_object (self->priv->constraint_combo, "changed", (GCallback) _export_dialog_on_constraint_changed_gtk_combo_box_changed, self, 0);
#line 115 "ExportDialog.vala"
	g_signal_connect_object (GTK_EDITABLE (self->priv->pixels_entry), "changed", (GCallback) _export_dialog_on_pixels_changed_gtk_editable_changed, self, 0);
#line 116 "ExportDialog.vala"
	g_signal_connect_object (GTK_EDITABLE (self->priv->pixels_entry), "insert-text", (GCallback) _export_dialog_on_pixels_insert_text_gtk_editable_insert_text, self, 0);
#line 117 "ExportDialog.vala"
	g_signal_connect_object (self->priv->pixels_entry, "activate", (GCallback) _export_dialog_on_activate_gtk_entry_activate, self, 0);
#line 120 "ExportDialog.vala"
	export_dialog_add_label (self, "Quality", 0, 0);
#line 121 "ExportDialog.vala"
	export_dialog_add_control (self, GTK_WIDGET (self->priv->quality_combo), 1, 0);
#line 123 "ExportDialog.vala"
	export_dialog_add_label (self, "Scaling constraint", 0, 1);
#line 124 "ExportDialog.vala"
	export_dialog_add_control (self, GTK_WIDGET (self->priv->constraint_combo), 1, 1);
#line 447 "ExportDialog.c"
	pixels_box = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
#line 127 "ExportDialog.vala"
	gtk_box_pack_start (GTK_BOX (pixels_box), GTK_WIDGET (self->priv->pixels_entry), TRUE, TRUE, (guint) 0);
#line 451 "ExportDialog.c"
	_tmp8_ = NULL;
#line 128 "ExportDialog.vala"
	gtk_box_pack_end (GTK_BOX (pixels_box), GTK_WIDGET (_tmp8_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (" pixels"))), FALSE, FALSE, (guint) 0);
#line 455 "ExportDialog.c"
	(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 129 "ExportDialog.vala"
	export_dialog_add_control (self, GTK_WIDGET (pixels_box), 1, 2);
#line 131 "ExportDialog.vala"
	gtk_container_add (GTK_CONTAINER (GTK_VBOX (gtk_dialog_get_content_area (GTK_DIALOG (self)))), GTK_WIDGET (self->priv->table));
#line 134 "ExportDialog.vala"
	gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
#line 463 "ExportDialog.c"
	_tmp10_ = NULL;
#line 135 "ExportDialog.vala"
	_tmp9_ = NULL;
#line 135 "ExportDialog.vala"
	self->priv->ok_button = (_tmp10_ = (_tmp9_ = gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_OK, (gint) GTK_RESPONSE_OK), (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (self->priv->ok_button == NULL) ? NULL : (self->priv->ok_button = (g_object_unref (self->priv->ok_button), NULL)), _tmp10_);
#line 137 "ExportDialog.vala"
	GTK_WIDGET_SET_FLAGS (self->priv->ok_button, (GTK_WIDGET_FLAGS (self->priv->ok_button) | GTK_CAN_DEFAULT) | GTK_HAS_DEFAULT);
#line 138 "ExportDialog.vala"
	gtk_window_set_default (GTK_WINDOW (self), self->priv->ok_button);
#line 140 "ExportDialog.vala"
	if (export_dialog_current_constraint == SCALE_CONSTRAINT_ORIGINAL) {
#line 141 "ExportDialog.vala"
		g_object_set (GTK_WIDGET (self->priv->pixels_entry), "sensitive", FALSE, NULL);
#line 142 "ExportDialog.vala"
		g_object_set (GTK_WIDGET (self->priv->quality_combo), "sensitive", FALSE, NULL);
#line 479 "ExportDialog.c"
	}
#line 145 "ExportDialog.vala"
	gtk_widget_grab_focus (self->priv->ok_button);
#line 483 "ExportDialog.c"
	(pixels_box == NULL) ? NULL : (pixels_box = (g_object_unref (pixels_box), NULL));
	return self;
}


#line 84 "ExportDialog.vala"
ExportDialog* export_dialog_new (gint count) {
#line 84 "ExportDialog.vala"
	return export_dialog_construct (TYPE_EXPORT_DIALOG, count);
#line 493 "ExportDialog.c"
}


#line 148 "ExportDialog.vala"
gboolean export_dialog_execute (ExportDialog* self, gint* scale, ScaleConstraint* constraint, JpegQuality* quality) {
#line 499 "ExportDialog.c"
	gboolean result;
	gboolean ok;
#line 148 "ExportDialog.vala"
	g_return_val_if_fail (IS_EXPORT_DIALOG (self), FALSE);
#line 149 "ExportDialog.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 506 "ExportDialog.c"
	ok = gtk_dialog_run (GTK_DIALOG (self)) == GTK_RESPONSE_OK;
#line 152 "ExportDialog.vala"
	if (ok) {
#line 510 "ExportDialog.c"
		gint index;
		index = gtk_combo_box_get_active (self->priv->constraint_combo);
#line 154 "ExportDialog.vala"
		g_assert (index >= 0);
#line 155 "ExportDialog.vala"
		(*constraint) = EXPORT_DIALOG_CONSTRAINT_ARRAY[index];
#line 156 "ExportDialog.vala"
		export_dialog_current_constraint = (*constraint);
#line 158 "ExportDialog.vala"
		(*scale) = atoi (gtk_entry_get_text (self->priv->pixels_entry));
#line 159 "ExportDialog.vala"
		if ((*constraint) != SCALE_CONSTRAINT_ORIGINAL) {
#line 160 "ExportDialog.vala"
			g_assert ((*scale) > 0);
#line 525 "ExportDialog.c"
		}
#line 161 "ExportDialog.vala"
		export_dialog_current_scale = (*scale);
#line 163 "ExportDialog.vala"
		index = gtk_combo_box_get_active (self->priv->quality_combo);
#line 164 "ExportDialog.vala"
		g_assert (index >= 0);
#line 165 "ExportDialog.vala"
		(*quality) = EXPORT_DIALOG_QUALITY_ARRAY[index];
#line 166 "ExportDialog.vala"
		export_dialog_current_quality = (*quality);
#line 537 "ExportDialog.c"
	}
#line 169 "ExportDialog.vala"
	gtk_object_destroy (GTK_OBJECT (self));
#line 541 "ExportDialog.c"
	result = ok;
	return result;
}


#line 174 "ExportDialog.vala"
static void export_dialog_add_label (ExportDialog* self, const char* text, gint x, gint y) {
#line 549 "ExportDialog.c"
	GtkAlignment* left_aligned;
	GtkLabel* _tmp0_;
#line 174 "ExportDialog.vala"
	g_return_if_fail (IS_EXPORT_DIALOG (self));
#line 174 "ExportDialog.vala"
	g_return_if_fail (text != NULL);
#line 556 "ExportDialog.c"
	left_aligned = g_object_ref_sink ((GtkAlignment*) gtk_alignment_new (0.0f, 0.5f, (float) 0, (float) 0));
	_tmp0_ = NULL;
#line 176 "ExportDialog.vala"
	gtk_container_add (GTK_CONTAINER (left_aligned), GTK_WIDGET (_tmp0_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (text))));
#line 561 "ExportDialog.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 178 "ExportDialog.vala"
	gtk_table_attach (self->priv->table, GTK_WIDGET (left_aligned), (guint) x, (guint) (x + 1), (guint) y, (guint) (y + 1), GTK_FILL, GTK_FILL, (guint) 10, (guint) 5);
#line 565 "ExportDialog.c"
	(left_aligned == NULL) ? NULL : (left_aligned = (g_object_unref (left_aligned), NULL));
}


#line 182 "ExportDialog.vala"
static void export_dialog_add_control (ExportDialog* self, GtkWidget* widget, gint x, gint y) {
#line 572 "ExportDialog.c"
	GtkAlignment* left_aligned;
#line 182 "ExportDialog.vala"
	g_return_if_fail (IS_EXPORT_DIALOG (self));
#line 182 "ExportDialog.vala"
	g_return_if_fail (GTK_IS_WIDGET (widget));
#line 578 "ExportDialog.c"
	left_aligned = g_object_ref_sink ((GtkAlignment*) gtk_alignment_new ((float) 0, 0.5f, (float) 0, (float) 0));
#line 184 "ExportDialog.vala"
	gtk_container_add (GTK_CONTAINER (left_aligned), widget);
#line 186 "ExportDialog.vala"
	gtk_table_attach (self->priv->table, GTK_WIDGET (left_aligned), (guint) x, (guint) (x + 1), (guint) y, (guint) (y + 1), GTK_FILL, GTK_FILL, (guint) 10, (guint) 5);
#line 584 "ExportDialog.c"
	(left_aligned == NULL) ? NULL : (left_aligned = (g_object_unref (left_aligned), NULL));
}


#line 190 "ExportDialog.vala"
static void export_dialog_on_constraint_changed (ExportDialog* self) {
#line 591 "ExportDialog.c"
	gboolean original;
#line 190 "ExportDialog.vala"
	g_return_if_fail (IS_EXPORT_DIALOG (self));
#line 595 "ExportDialog.c"
	original = EXPORT_DIALOG_CONSTRAINT_ARRAY[gtk_combo_box_get_active (self->priv->constraint_combo)] == SCALE_CONSTRAINT_ORIGINAL;
#line 192 "ExportDialog.vala"
	g_object_set (GTK_WIDGET (self->priv->pixels_entry), "sensitive", !original, NULL);
#line 193 "ExportDialog.vala"
	g_object_set (GTK_WIDGET (self->priv->quality_combo), "sensitive", !original, NULL);
#line 194 "ExportDialog.vala"
	if (original) {
#line 195 "ExportDialog.vala"
		g_object_set (self->priv->ok_button, "sensitive", TRUE, NULL);
#line 605 "ExportDialog.c"
	} else {
#line 197 "ExportDialog.vala"
		export_dialog_on_pixels_changed (self);
#line 609 "ExportDialog.c"
	}
}


#line 200 "ExportDialog.vala"
static void export_dialog_on_activate (ExportDialog* self) {
#line 200 "ExportDialog.vala"
	g_return_if_fail (IS_EXPORT_DIALOG (self));
#line 201 "ExportDialog.vala"
	gtk_dialog_response (GTK_DIALOG (self), (gint) GTK_RESPONSE_OK);
#line 620 "ExportDialog.c"
}


#line 204 "ExportDialog.vala"
static void export_dialog_on_pixels_changed (ExportDialog* self) {
#line 626 "ExportDialog.c"
	gboolean _tmp0_;
#line 204 "ExportDialog.vala"
	g_return_if_fail (IS_EXPORT_DIALOG (self));
#line 630 "ExportDialog.c"
	_tmp0_ = FALSE;
#line 205 "ExportDialog.vala"
	if (gtk_entry_get_text_length (self->priv->pixels_entry) > 0) {
#line 205 "ExportDialog.vala"
		_tmp0_ = atoi (gtk_entry_get_text (self->priv->pixels_entry)) > 0;
#line 636 "ExportDialog.c"
	} else {
#line 205 "ExportDialog.vala"
		_tmp0_ = FALSE;
#line 640 "ExportDialog.c"
	}
#line 205 "ExportDialog.vala"
	g_object_set (self->priv->ok_button, "sensitive", _tmp0_, NULL);
#line 644 "ExportDialog.c"
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


#line 208 "ExportDialog.vala"
static void export_dialog_on_pixels_insert_text (ExportDialog* self, const char* text, gint length, void* position) {
#line 658 "ExportDialog.c"
	char* buffer;
	char* new_text;
#line 208 "ExportDialog.vala"
	g_return_if_fail (IS_EXPORT_DIALOG (self));
#line 208 "ExportDialog.vala"
	g_return_if_fail (text != NULL);
#line 665 "ExportDialog.c"
	/* This is necessary because SignalHandler.block_by_func() is not properly bound*/
#line 210 "ExportDialog.vala"
	if (self->priv->in_insert) {
#line 669 "ExportDialog.c"
		return;
	}
#line 213 "ExportDialog.vala"
	self->priv->in_insert = TRUE;
#line 215 "ExportDialog.vala"
	if (length == (-1)) {
#line 216 "ExportDialog.vala"
		length = (gint) string_get_length (text);
#line 678 "ExportDialog.c"
	}
	/* only permit numeric text*/
	buffer = g_new0 (char, 1);
	new_text = g_new0 (char, 1);
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 221 "ExportDialog.vala"
			while (TRUE) {
#line 221 "ExportDialog.vala"
				if (!_tmp0_) {
#line 221 "ExportDialog.vala"
					ctr++;
#line 695 "ExportDialog.c"
				}
#line 221 "ExportDialog.vala"
				_tmp0_ = FALSE;
#line 221 "ExportDialog.vala"
				if (!(ctr < length)) {
#line 221 "ExportDialog.vala"
					break;
#line 703 "ExportDialog.c"
				}
#line 222 "ExportDialog.vala"
				if (g_unichar_isdigit (g_utf8_get_char (g_utf8_offset_to_pointer (text, ctr)))) {
#line 707 "ExportDialog.c"
					char* _tmp1_;
#line 223 "ExportDialog.vala"
					g_unichar_to_utf8 (g_utf8_get_char (g_utf8_offset_to_pointer (text, ctr)), buffer);
#line 711 "ExportDialog.c"
					_tmp1_ = NULL;
#line 224 "ExportDialog.vala"
					new_text = (_tmp1_ = g_strconcat (new_text, buffer, NULL), new_text = (g_free (new_text), NULL), _tmp1_);
#line 715 "ExportDialog.c"
				}
			}
		}
	}
#line 228 "ExportDialog.vala"
	if (string_get_length (new_text) > 0) {
#line 229 "ExportDialog.vala"
		gtk_editable_insert_text (GTK_EDITABLE (self->priv->pixels_entry), new_text, (gint) string_get_length (new_text), position);
#line 724 "ExportDialog.c"
	}
#line 231 "ExportDialog.vala"
	g_signal_stop_emission_by_name (self->priv->pixels_entry, "insert-text");
#line 233 "ExportDialog.vala"
	self->priv->in_insert = FALSE;
#line 730 "ExportDialog.c"
	buffer = (g_free (buffer), NULL);
	new_text = (g_free (new_text), NULL);
}


static void export_dialog_class_init (ExportDialogClass * klass) {
	export_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ExportDialogPrivate));
	G_OBJECT_CLASS (klass)->finalize = export_dialog_finalize;
}


static void export_dialog_instance_init (ExportDialog * self) {
	self->priv = EXPORT_DIALOG_GET_PRIVATE (self);
	self->priv->table = g_object_ref_sink ((GtkTable*) gtk_table_new ((guint) 0, (guint) 0, FALSE));
	self->priv->in_insert = FALSE;
}


static void export_dialog_finalize (GObject* obj) {
	ExportDialog * self;
	self = EXPORT_DIALOG (obj);
	(self->priv->table == NULL) ? NULL : (self->priv->table = (g_object_unref (self->priv->table), NULL));
	(self->priv->quality_combo == NULL) ? NULL : (self->priv->quality_combo = (g_object_unref (self->priv->quality_combo), NULL));
	(self->priv->constraint_combo == NULL) ? NULL : (self->priv->constraint_combo = (g_object_unref (self->priv->constraint_combo), NULL));
	(self->priv->pixels_entry == NULL) ? NULL : (self->priv->pixels_entry = (g_object_unref (self->priv->pixels_entry), NULL));
	(self->priv->ok_button == NULL) ? NULL : (self->priv->ok_button = (g_object_unref (self->priv->ok_button), NULL));
	G_OBJECT_CLASS (export_dialog_parent_class)->finalize (obj);
}


GType export_dialog_get_type (void) {
	static GType export_dialog_type_id = 0;
	if (export_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ExportDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) export_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExportDialog), 0, (GInstanceInitFunc) export_dialog_instance_init, NULL };
		export_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "ExportDialog", &g_define_type_info, 0);
	}
	return export_dialog_type_id;
}




