/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <gphoto2/gphoto2-context.h>
#include <gphoto.h>
#include <gphoto2/gphoto2-camera.h>
#include <gphoto2/gphoto2-result.h>
#include <gphoto2/gphoto2-port-result.h>
#include <gphoto2/gphoto2-filesys.h>
#include <gphoto2/gphoto2-file.h>
#include <gio/gio.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <libexif/exif-data.h>
#include <gobject/gvaluecollector.h>


#define GP_TYPE_CONTEXT_WRAPPER (gp_context_wrapper_get_type ())
#define GP_CONTEXT_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GP_TYPE_CONTEXT_WRAPPER, GPContextWrapper))
#define GP_CONTEXT_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GP_TYPE_CONTEXT_WRAPPER, GPContextWrapperClass))
#define GP_IS_CONTEXT_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GP_TYPE_CONTEXT_WRAPPER))
#define GP_IS_CONTEXT_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GP_TYPE_CONTEXT_WRAPPER))
#define GP_CONTEXT_WRAPPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GP_TYPE_CONTEXT_WRAPPER, GPContextWrapperClass))

typedef struct _GPContextWrapper GPContextWrapper;
typedef struct _GPContextWrapperClass GPContextWrapperClass;
typedef struct _GPContextWrapperPrivate GPContextWrapperPrivate;
typedef struct _GPParamSpecContextWrapper GPParamSpecContextWrapper;

typedef enum  {
	GPHOTO_ERROR_LIBRARY
} GPhotoError;
#define GPHOTO_ERROR gphoto_error_quark ()
/* ContextWrapper assigns signals to the various GPhoto.Context callbacks, as well as spins
 the event loop at opportune times.*/
struct _GPContextWrapper {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GPContextWrapperPrivate * priv;
	GPContext* context;
};

struct _GPContextWrapperClass {
	GTypeClass parent_class;
	void (*finalize) (GPContextWrapper *self);
	void (*idle) (GPContextWrapper* self);
	void (*error) (GPContextWrapper* self, const char* format, void* va_list);
	void (*status) (GPContextWrapper* self, const char* format, void* va_list);
	void (*message) (GPContextWrapper* self, const char* format, void* va_list);
	void (*progress_start) (GPContextWrapper* self, float target, const char* format, void* va_list);
	void (*progress_update) (GPContextWrapper* self, float current);
	void (*progress_stop) (GPContextWrapper* self);
};

struct _GPParamSpecContextWrapper {
	GParamSpec parent_instance;
};


static gpointer gp_context_wrapper_parent_class = NULL;

GQuark gphoto_error_quark (void);
gpointer gp_context_wrapper_ref (gpointer instance);
void gp_context_wrapper_unref (gpointer instance);
GParamSpec* gp_param_spec_context_wrapper (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gp_value_set_context_wrapper (GValue* value, gpointer v_object);
gpointer gp_value_get_context_wrapper (const GValue* value);
GType gp_context_wrapper_get_type (void);
enum  {
	GP_CONTEXT_WRAPPER_DUMMY_PROPERTY
};
static void gp_context_wrapper_on_idle (GPContextWrapper* self, GPContext* context);
static void _gp_context_wrapper_on_idle_gp_context_idle_func (GPContext* context, gpointer self);
static void gp_context_wrapper_on_error (GPContextWrapper* self, GPContext* context, const char* format, void* va_list);
static void _gp_context_wrapper_on_error_gp_context_error_func (GPContext* context, const char* format, void* va_list, gpointer self);
static void gp_context_wrapper_on_status (GPContextWrapper* self, GPContext* context, const char* format, void* va_list);
static void _gp_context_wrapper_on_status_gp_context_status_func (GPContext* context, const char* format, void* va_list, gpointer self);
static void gp_context_wrapper_on_message (GPContextWrapper* self, GPContext* context, const char* format, void* va_list);
static void _gp_context_wrapper_on_message_gp_context_message_func (GPContext* context, const char* format, void* va_list, gpointer self);
static guint gp_context_wrapper_on_progress_start (GPContextWrapper* self, GPContext* context, float target, const char* format, void* va_list);
static guint _gp_context_wrapper_on_progress_start_gp_context_progress_start_func (GPContext* context, float target, const char* format, void* va_list, gpointer self);
static void gp_context_wrapper_on_progress_update (GPContextWrapper* self, GPContext* context, guint id, float current);
static void _gp_context_wrapper_on_progress_update_gp_context_progress_update_func (GPContext* context, guint id, float current, gpointer self);
static void gp_context_wrapper_on_progress_stop (GPContextWrapper* self, GPContext* context, guint id);
static void _gp_context_wrapper_on_progress_stop_gp_context_progress_stop_func (GPContext* context, guint id, gpointer self);
GPContextWrapper* gp_context_wrapper_new (void);
GPContextWrapper* gp_context_wrapper_construct (GType object_type);
void gp_context_wrapper_idle (GPContextWrapper* self);
static void gp_context_wrapper_real_idle (GPContextWrapper* self);
void gp_context_wrapper_error (GPContextWrapper* self, const char* format, void* va_list);
static void gp_context_wrapper_real_error (GPContextWrapper* self, const char* format, void* va_list);
void gp_context_wrapper_status (GPContextWrapper* self, const char* format, void* va_list);
static void gp_context_wrapper_real_status (GPContextWrapper* self, const char* format, void* va_list);
void gp_context_wrapper_message (GPContextWrapper* self, const char* format, void* va_list);
static void gp_context_wrapper_real_message (GPContextWrapper* self, const char* format, void* va_list);
void gp_context_wrapper_progress_start (GPContextWrapper* self, float target, const char* format, void* va_list);
static void gp_context_wrapper_real_progress_start (GPContextWrapper* self, float target, const char* format, void* va_list);
void gp_context_wrapper_progress_update (GPContextWrapper* self, float current);
static void gp_context_wrapper_real_progress_update (GPContextWrapper* self, float current);
void gp_context_wrapper_progress_stop (GPContextWrapper* self);
static void gp_context_wrapper_real_progress_stop (GPContextWrapper* self);
gboolean spin_event_loop (void);
static void gp_context_wrapper_finalize (GPContextWrapper* obj);
void gp_get_info (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileInfo* info, GError** error);
GInputStream* gp_load_file_into_stream (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileType filetype, GError** error);
GdkPixbuf* gp_load_preview (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error);
GdkPixbuf* gp_load_image (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error);
void gp_save_image (GPContext* context, Camera* camera, const char* folder, const char* filename, GFile* dest_file, GError** error);
guint8* gp_load_file_into_buffer (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileType filetype, int* result_length1, GError** error);
ExifData* gp_load_exif (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error);
void gp_on_mins_destroyed (void* data);
static void _gp_on_mins_destroyed_gdestroy_notify (void* data);
GFile* app_window_get_temp_dir (void);
void free (void* ptr);



GQuark gphoto_error_quark (void) {
	return g_quark_from_static_string ("gphoto_error-quark");
}


#line 46 "GPhoto.vala"
static void _gp_context_wrapper_on_idle_gp_context_idle_func (GPContext* context, gpointer self) {
#line 134 "GPhoto.c"
	gp_context_wrapper_on_idle (self, context);
}


#line 51 "GPhoto.vala"
static void _gp_context_wrapper_on_error_gp_context_error_func (GPContext* context, const char* format, void* va_list, gpointer self) {
#line 141 "GPhoto.c"
	gp_context_wrapper_on_error (self, context, format, va_list);
}


#line 55 "GPhoto.vala"
static void _gp_context_wrapper_on_status_gp_context_status_func (GPContext* context, const char* format, void* va_list, gpointer self) {
#line 148 "GPhoto.c"
	gp_context_wrapper_on_status (self, context, format, va_list);
}


#line 59 "GPhoto.vala"
static void _gp_context_wrapper_on_message_gp_context_message_func (GPContext* context, const char* format, void* va_list, gpointer self) {
#line 155 "GPhoto.c"
	gp_context_wrapper_on_message (self, context, format, va_list);
}


#line 63 "GPhoto.vala"
static guint _gp_context_wrapper_on_progress_start_gp_context_progress_start_func (GPContext* context, float target, const char* format, void* va_list, gpointer self) {
#line 162 "GPhoto.c"
	return gp_context_wrapper_on_progress_start (self, context, target, format, va_list);
}


#line 69 "GPhoto.vala"
static void _gp_context_wrapper_on_progress_update_gp_context_progress_update_func (GPContext* context, guint id, float current, gpointer self) {
#line 169 "GPhoto.c"
	gp_context_wrapper_on_progress_update (self, context, id, current);
}


#line 74 "GPhoto.vala"
static void _gp_context_wrapper_on_progress_stop_gp_context_progress_stop_func (GPContext* context, guint id, gpointer self) {
#line 176 "GPhoto.c"
	gp_context_wrapper_on_progress_stop (self, context, id);
}


#line 17 "GPhoto.vala"
GPContextWrapper* gp_context_wrapper_construct (GType object_type) {
#line 183 "GPhoto.c"
	GPContextWrapper* self;
	self = (GPContextWrapper*) g_type_create_instance (object_type);
#line 18 "GPhoto.vala"
	gp_context_set_idle_func (self->context, _gp_context_wrapper_on_idle_gp_context_idle_func, self);
#line 19 "GPhoto.vala"
	gp_context_set_error_func (self->context, _gp_context_wrapper_on_error_gp_context_error_func, self);
#line 20 "GPhoto.vala"
	gp_context_set_status_func (self->context, _gp_context_wrapper_on_status_gp_context_status_func, self);
#line 21 "GPhoto.vala"
	gp_context_set_message_func (self->context, _gp_context_wrapper_on_message_gp_context_message_func, self);
#line 22 "GPhoto.vala"
	gp_context_set_progress_funcs (self->context, _gp_context_wrapper_on_progress_start_gp_context_progress_start_func, _gp_context_wrapper_on_progress_update_gp_context_progress_update_func, _gp_context_wrapper_on_progress_stop_gp_context_progress_stop_func, self);
#line 196 "GPhoto.c"
	return self;
}


#line 17 "GPhoto.vala"
GPContextWrapper* gp_context_wrapper_new (void) {
#line 17 "GPhoto.vala"
	return gp_context_wrapper_construct (GP_TYPE_CONTEXT_WRAPPER);
#line 205 "GPhoto.c"
}


#line 25 "GPhoto.vala"
static void gp_context_wrapper_real_idle (GPContextWrapper* self) {
#line 25 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 213 "GPhoto.c"
}


#line 25 "GPhoto.vala"
void gp_context_wrapper_idle (GPContextWrapper* self) {
#line 25 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->idle (self);
#line 221 "GPhoto.c"
}


#line 28 "GPhoto.vala"
static void gp_context_wrapper_real_error (GPContextWrapper* self, const char* format, void* va_list) {
#line 28 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 28 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 231 "GPhoto.c"
}


#line 28 "GPhoto.vala"
void gp_context_wrapper_error (GPContextWrapper* self, const char* format, void* va_list) {
#line 28 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->error (self, format, va_list);
#line 239 "GPhoto.c"
}


#line 31 "GPhoto.vala"
static void gp_context_wrapper_real_status (GPContextWrapper* self, const char* format, void* va_list) {
#line 31 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 31 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 249 "GPhoto.c"
}


#line 31 "GPhoto.vala"
void gp_context_wrapper_status (GPContextWrapper* self, const char* format, void* va_list) {
#line 31 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->status (self, format, va_list);
#line 257 "GPhoto.c"
}


#line 34 "GPhoto.vala"
static void gp_context_wrapper_real_message (GPContextWrapper* self, const char* format, void* va_list) {
#line 34 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 34 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 267 "GPhoto.c"
}


#line 34 "GPhoto.vala"
void gp_context_wrapper_message (GPContextWrapper* self, const char* format, void* va_list) {
#line 34 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->message (self, format, va_list);
#line 275 "GPhoto.c"
}


#line 37 "GPhoto.vala"
static void gp_context_wrapper_real_progress_start (GPContextWrapper* self, float target, const char* format, void* va_list) {
#line 37 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 37 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 285 "GPhoto.c"
}


#line 37 "GPhoto.vala"
void gp_context_wrapper_progress_start (GPContextWrapper* self, float target, const char* format, void* va_list) {
#line 37 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->progress_start (self, target, format, va_list);
#line 293 "GPhoto.c"
}


#line 40 "GPhoto.vala"
static void gp_context_wrapper_real_progress_update (GPContextWrapper* self, float current) {
#line 40 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 301 "GPhoto.c"
}


#line 40 "GPhoto.vala"
void gp_context_wrapper_progress_update (GPContextWrapper* self, float current) {
#line 40 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->progress_update (self, current);
#line 309 "GPhoto.c"
}


#line 43 "GPhoto.vala"
static void gp_context_wrapper_real_progress_stop (GPContextWrapper* self) {
#line 43 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 317 "GPhoto.c"
}


#line 43 "GPhoto.vala"
void gp_context_wrapper_progress_stop (GPContextWrapper* self) {
#line 43 "GPhoto.vala"
	GP_CONTEXT_WRAPPER_GET_CLASS (self)->progress_stop (self);
#line 325 "GPhoto.c"
}


#line 46 "GPhoto.vala"
static void gp_context_wrapper_on_idle (GPContextWrapper* self, GPContext* context) {
#line 46 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 46 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 47 "GPhoto.vala"
	gp_context_wrapper_idle (self);
#line 48 "GPhoto.vala"
	spin_event_loop ();
#line 339 "GPhoto.c"
}


#line 51 "GPhoto.vala"
static void gp_context_wrapper_on_error (GPContextWrapper* self, GPContext* context, const char* format, void* va_list) {
#line 51 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 51 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 51 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 52 "GPhoto.vala"
	gp_context_wrapper_error (self, format, va_list);
#line 353 "GPhoto.c"
}


#line 55 "GPhoto.vala"
static void gp_context_wrapper_on_status (GPContextWrapper* self, GPContext* context, const char* format, void* va_list) {
#line 55 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 55 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 55 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 56 "GPhoto.vala"
	gp_context_wrapper_status (self, format, va_list);
#line 367 "GPhoto.c"
}


#line 59 "GPhoto.vala"
static void gp_context_wrapper_on_message (GPContextWrapper* self, GPContext* context, const char* format, void* va_list) {
#line 59 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 59 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 59 "GPhoto.vala"
	g_return_if_fail (format != NULL);
#line 60 "GPhoto.vala"
	gp_context_wrapper_message (self, format, va_list);
#line 381 "GPhoto.c"
}


#line 63 "GPhoto.vala"
static guint gp_context_wrapper_on_progress_start (GPContextWrapper* self, GPContext* context, float target, const char* format, void* va_list) {
#line 387 "GPhoto.c"
	guint result;
#line 63 "GPhoto.vala"
	g_return_val_if_fail (GP_IS_CONTEXT_WRAPPER (self), 0U);
#line 63 "GPhoto.vala"
	g_return_val_if_fail (context != NULL, 0U);
#line 63 "GPhoto.vala"
	g_return_val_if_fail (format != NULL, 0U);
#line 64 "GPhoto.vala"
	gp_context_wrapper_progress_start (self, target, format, va_list);
#line 397 "GPhoto.c"
	result = (guint) 0;
	return result;
}


#line 69 "GPhoto.vala"
static void gp_context_wrapper_on_progress_update (GPContextWrapper* self, GPContext* context, guint id, float current) {
#line 69 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 69 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 70 "GPhoto.vala"
	gp_context_wrapper_progress_update (self, current);
#line 71 "GPhoto.vala"
	spin_event_loop ();
#line 413 "GPhoto.c"
}


#line 74 "GPhoto.vala"
static void gp_context_wrapper_on_progress_stop (GPContextWrapper* self, GPContext* context, guint id) {
#line 74 "GPhoto.vala"
	g_return_if_fail (GP_IS_CONTEXT_WRAPPER (self));
#line 74 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 75 "GPhoto.vala"
	gp_context_wrapper_progress_stop (self);
#line 425 "GPhoto.c"
}


static void gp_value_context_wrapper_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void gp_value_context_wrapper_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		gp_context_wrapper_unref (value->data[0].v_pointer);
	}
}


static void gp_value_context_wrapper_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = gp_context_wrapper_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer gp_value_context_wrapper_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* gp_value_context_wrapper_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GPContextWrapper* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = gp_context_wrapper_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* gp_value_context_wrapper_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GPContextWrapper** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = gp_context_wrapper_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* gp_param_spec_context_wrapper (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GPParamSpecContextWrapper* spec;
	g_return_val_if_fail (g_type_is_a (object_type, GP_TYPE_CONTEXT_WRAPPER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer gp_value_get_context_wrapper (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GP_TYPE_CONTEXT_WRAPPER), NULL);
	return value->data[0].v_pointer;
}


void gp_value_set_context_wrapper (GValue* value, gpointer v_object) {
	GPContextWrapper* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GP_TYPE_CONTEXT_WRAPPER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, GP_TYPE_CONTEXT_WRAPPER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		gp_context_wrapper_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		gp_context_wrapper_unref (old);
	}
}


static void gp_context_wrapper_class_init (GPContextWrapperClass * klass) {
	gp_context_wrapper_parent_class = g_type_class_peek_parent (klass);
	GP_CONTEXT_WRAPPER_CLASS (klass)->finalize = gp_context_wrapper_finalize;
	GP_CONTEXT_WRAPPER_CLASS (klass)->idle = gp_context_wrapper_real_idle;
	GP_CONTEXT_WRAPPER_CLASS (klass)->error = gp_context_wrapper_real_error;
	GP_CONTEXT_WRAPPER_CLASS (klass)->status = gp_context_wrapper_real_status;
	GP_CONTEXT_WRAPPER_CLASS (klass)->message = gp_context_wrapper_real_message;
	GP_CONTEXT_WRAPPER_CLASS (klass)->progress_start = gp_context_wrapper_real_progress_start;
	GP_CONTEXT_WRAPPER_CLASS (klass)->progress_update = gp_context_wrapper_real_progress_update;
	GP_CONTEXT_WRAPPER_CLASS (klass)->progress_stop = gp_context_wrapper_real_progress_stop;
}


static void gp_context_wrapper_instance_init (GPContextWrapper * self) {
	self->context = gp_context_new ();
	self->ref_count = 1;
}


static void gp_context_wrapper_finalize (GPContextWrapper* obj) {
	GPContextWrapper * self;
	self = GP_CONTEXT_WRAPPER (obj);
	(self->context == NULL) ? NULL : (self->context = (gp_context_unref (self->context), NULL));
}


GType gp_context_wrapper_get_type (void) {
	static GType gp_context_wrapper_type_id = 0;
	if (gp_context_wrapper_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { gp_value_context_wrapper_init, gp_value_context_wrapper_free_value, gp_value_context_wrapper_copy_value, gp_value_context_wrapper_peek_pointer, "p", gp_value_context_wrapper_collect_value, "p", gp_value_context_wrapper_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GPContextWrapperClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gp_context_wrapper_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GPContextWrapper), 0, (GInstanceInitFunc) gp_context_wrapper_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		gp_context_wrapper_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GPContextWrapper", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return gp_context_wrapper_type_id;
}


gpointer gp_context_wrapper_ref (gpointer instance) {
	GPContextWrapper* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void gp_context_wrapper_unref (gpointer instance) {
	GPContextWrapper* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		GP_CONTEXT_WRAPPER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 79 "GPhoto.vala"
void gp_get_info (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileInfo* info, GError** error) {
#line 580 "GPhoto.c"
	GError * _inner_error_;
	CameraFileInfo _tmp2_;
	int _tmp1_;
	CameraFileInfo _tmp0_;
	int res;
#line 79 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 79 "GPhoto.vala"
	g_return_if_fail (camera != NULL);
#line 79 "GPhoto.vala"
	g_return_if_fail (folder != NULL);
#line 79 "GPhoto.vala"
	g_return_if_fail (filename != NULL);
#line 594 "GPhoto.c"
	_inner_error_ = NULL;
	res = (_tmp1_ = gp_camera_file_get_info (camera, folder, filename, &_tmp0_, context), (*info) = (_tmp2_ = _tmp0_,  (&(*info)), _tmp2_), _tmp1_);
#line 82 "GPhoto.vala"
	if (res != GP_OK) {
#line 599 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error retrieving file information for %s/%s: %s", (gint) res, folder, filename, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			return;
		}
	}
}


#line 87 "GPhoto.vala"
GdkPixbuf* gp_load_preview (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error) {
#line 611 "GPhoto.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	GInputStream* ins;
	GdkPixbuf* _tmp0_;
#line 87 "GPhoto.vala"
	g_return_val_if_fail (context != NULL, NULL);
#line 87 "GPhoto.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 87 "GPhoto.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 87 "GPhoto.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 624 "GPhoto.c"
	_inner_error_ = NULL;
	ins = gp_load_file_into_stream (context, camera, folder, filename, GP_FILE_TYPE_PREVIEW, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 90 "GPhoto.vala"
	if (ins == NULL) {
#line 633 "GPhoto.c"
		result = NULL;
		(ins == NULL) ? NULL : (ins = (g_object_unref (ins), NULL));
		return result;
	}
	_tmp0_ = gdk_pixbuf_new_from_stream (ins, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(ins == NULL) ? NULL : (ins = (g_object_unref (ins), NULL));
		return NULL;
	}
	result = _tmp0_;
	(ins == NULL) ? NULL : (ins = (g_object_unref (ins), NULL));
	return result;
}


#line 96 "GPhoto.vala"
GdkPixbuf* gp_load_image (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error) {
#line 652 "GPhoto.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	GInputStream* ins;
	GdkPixbuf* _tmp0_;
#line 96 "GPhoto.vala"
	g_return_val_if_fail (context != NULL, NULL);
#line 96 "GPhoto.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 96 "GPhoto.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 96 "GPhoto.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 665 "GPhoto.c"
	_inner_error_ = NULL;
	ins = gp_load_file_into_stream (context, camera, folder, filename, GP_FILE_TYPE_NORMAL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 99 "GPhoto.vala"
	if (ins == NULL) {
#line 674 "GPhoto.c"
		result = NULL;
		(ins == NULL) ? NULL : (ins = (g_object_unref (ins), NULL));
		return result;
	}
	_tmp0_ = gdk_pixbuf_new_from_stream (ins, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(ins == NULL) ? NULL : (ins = (g_object_unref (ins), NULL));
		return NULL;
	}
	result = _tmp0_;
	(ins == NULL) ? NULL : (ins = (g_object_unref (ins), NULL));
	return result;
}


#line 105 "GPhoto.vala"
void gp_save_image (GPContext* context, Camera* camera, const char* folder, const char* filename, GFile* dest_file, GError** error) {
#line 693 "GPhoto.c"
	GError * _inner_error_;
	CameraFile* camera_file;
	CameraFile* _tmp2_;
	int _tmp1_;
	CameraFile* _tmp0_;
	int res;
	char* _tmp3_;
#line 105 "GPhoto.vala"
	g_return_if_fail (context != NULL);
#line 105 "GPhoto.vala"
	g_return_if_fail (camera != NULL);
#line 105 "GPhoto.vala"
	g_return_if_fail (folder != NULL);
#line 105 "GPhoto.vala"
	g_return_if_fail (filename != NULL);
#line 105 "GPhoto.vala"
	g_return_if_fail (G_IS_FILE (dest_file));
#line 711 "GPhoto.c"
	_inner_error_ = NULL;
	camera_file = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = gp_file_new (&_tmp0_), camera_file = (_tmp2_ = _tmp0_, (camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL)), _tmp2_), _tmp1_);
#line 109 "GPhoto.vala"
	if (res != GP_OK) {
#line 719 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error allocating camera file: %s", (gint) res, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return;
		}
	}
#line 112 "GPhoto.vala"
	res = gp_camera_file_get (camera, folder, filename, GP_FILE_TYPE_NORMAL, camera_file, context);
#line 113 "GPhoto.vala"
	if (res != GP_OK) {
#line 731 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error retrieving file object for %s/%s: %s", (gint) res, folder, filename, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return;
		}
	}
#line 952 "gio-2.0.vapi"
	_tmp3_ = NULL;
#line 117 "GPhoto.vala"
	res = gp_file_save (camera_file, _tmp3_ = g_file_get_path (dest_file));
#line 743 "GPhoto.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
#line 118 "GPhoto.vala"
	if (res != GP_OK) {
#line 747 "GPhoto.c"
		char* _tmp4_;
		GError* _tmp5_;
#line 952 "gio-2.0.vapi"
		_tmp4_ = NULL;
#line 119 "GPhoto.vala"
		_tmp5_ = NULL;
#line 754 "GPhoto.c"
		_inner_error_ = (_tmp5_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error copying file %s/%s to %s: %s", (gint) res, folder, filename, _tmp4_ = g_file_get_path (dest_file), gp_port_result_as_string (res)), _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return;
		}
	}
	(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
}


#line 123 "GPhoto.vala"
ExifData* gp_load_exif (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error) {
#line 768 "GPhoto.c"
	ExifData* result;
	GError * _inner_error_;
	guint8* _tmp1_;
	gint buffer_size;
	gint buffer_length1;
	gint _tmp0_;
	guint8* buffer;
	ExifData* data;
#line 123 "GPhoto.vala"
	g_return_val_if_fail (context != NULL, NULL);
#line 123 "GPhoto.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 123 "GPhoto.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 123 "GPhoto.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 785 "GPhoto.c"
	_inner_error_ = NULL;
#line 125 "GPhoto.vala"
	_tmp1_ = NULL;
#line 789 "GPhoto.c"
	buffer = (_tmp1_ = gp_load_file_into_buffer (context, camera, folder, filename, GP_FILE_TYPE_EXIF, &_tmp0_, &_inner_error_), buffer_length1 = _tmp0_, buffer_size = buffer_length1, _tmp1_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 126 "GPhoto.vala"
	if (buffer == NULL) {
#line 797 "GPhoto.c"
		result = NULL;
		buffer = (g_free (buffer), NULL);
		return result;
	}
	data = exif_data_new_from_data (buffer, (gsize) buffer_length1);
#line 130 "GPhoto.vala"
	exif_data_fix (data);
#line 805 "GPhoto.c"
	result = data;
	buffer = (g_free (buffer), NULL);
	return result;
}


#line 171 "GPhoto.vala"
static void _gp_on_mins_destroyed_gdestroy_notify (void* data) {
#line 814 "GPhoto.c"
	gp_on_mins_destroyed (data);
}


/* Returns an InputStream for the requested camera file.  The stream should be used
 immediately rather than stored, as the backing is temporary in nature.*/
#line 137 "GPhoto.vala"
GInputStream* gp_load_file_into_stream (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileType filetype, GError** error) {
#line 823 "GPhoto.c"
	GInputStream* result;
	GError * _inner_error_;
	CameraFile* camera_file;
	CameraFile* _tmp2_;
	int _tmp1_;
	CameraFile* _tmp0_;
	int res;
	gint data_size;
	gint data_length1;
	guint8* data;
	GFile* _tmp3_;
	GFile* _tmp4_;
	GFile* temp;
	char* _tmp5_;
	GFileInputStream* _tmp8_;
#line 137 "GPhoto.vala"
	g_return_val_if_fail (context != NULL, NULL);
#line 137 "GPhoto.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 137 "GPhoto.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 137 "GPhoto.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 847 "GPhoto.c"
	_inner_error_ = NULL;
	camera_file = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = gp_file_new (&_tmp0_), camera_file = (_tmp2_ = _tmp0_, (camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL)), _tmp2_), _tmp1_);
#line 141 "GPhoto.vala"
	if (res != GP_OK) {
#line 855 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error allocating camera file: %s", (gint) res, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return NULL;
		}
	}
#line 144 "GPhoto.vala"
	res = gp_camera_file_get (camera, folder, filename, filetype, camera_file, context);
#line 145 "GPhoto.vala"
	if (res != GP_OK) {
#line 867 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error retrieving file object for %s/%s: %s", (gint) res, folder, filename, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return NULL;
		}
	}
	/* if entire file fits in memory, return a stream from that ... can't merely wrap
	 MemoryInputStream around the camera_file buffer, as that will be destroyed when the
	 function returns*/
	data = (data_length1 = 0, NULL);
#line 153 "GPhoto.vala"
	res = gp_file_get_data_and_size (camera_file, &data, &data_length1);
#line 154 "GPhoto.vala"
	if (res == GP_OK) {
#line 883 "GPhoto.c"
		guint8* buffer;
		buffer = g_malloc ((gulong) data_length1);
#line 156 "GPhoto.vala"
		memcpy (buffer, data, (gsize) data_length1);
#line 888 "GPhoto.c"
		result = G_INPUT_STREAM ((GMemoryInputStream*) g_memory_input_stream_new_from_data (buffer, (gssize) data_length1, _gp_on_mins_destroyed_gdestroy_notify));
		(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
		return result;
	}
	/* if not stored in memory, try copying it to a temp file and then reading out of that*/
#line 474 "AppWindow.vala"
	_tmp3_ = NULL;
#line 162 "GPhoto.vala"
	_tmp4_ = NULL;
#line 898 "GPhoto.c"
	temp = (_tmp4_ = g_file_get_child (_tmp3_ = app_window_get_temp_dir (), "import.tmp"), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL)), _tmp4_);
#line 952 "gio-2.0.vapi"
	_tmp5_ = NULL;
#line 163 "GPhoto.vala"
	res = gp_file_save (camera_file, _tmp5_ = g_file_get_path (temp));
#line 904 "GPhoto.c"
	_tmp5_ = (g_free (_tmp5_), NULL);
#line 164 "GPhoto.vala"
	if (res != GP_OK) {
#line 908 "GPhoto.c"
		char* _tmp6_;
		GError* _tmp7_;
#line 952 "gio-2.0.vapi"
		_tmp6_ = NULL;
#line 165 "GPhoto.vala"
		_tmp7_ = NULL;
#line 915 "GPhoto.c"
		_inner_error_ = (_tmp7_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error copying file %s/%s to %s: %s", (gint) res, folder, filename, _tmp6_ = g_file_get_path (temp), gp_port_result_as_string (res)), _tmp6_ = (g_free (_tmp6_), NULL), _tmp7_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
			return NULL;
		}
	}
	_tmp8_ = g_file_read (temp, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
		(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
		return NULL;
	}
	result = G_INPUT_STREAM (_tmp8_);
	(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
	(temp == NULL) ? NULL : (temp = (g_object_unref (temp), NULL));
	return result;
}


#line 171 "GPhoto.vala"
void gp_on_mins_destroyed (void* data) {
#line 172 "GPhoto.vala"
	free (data);
#line 942 "GPhoto.c"
}


/* Returns a buffer with the requested file, if within reason.  Use load_file for larger files.*/
#line 176 "GPhoto.vala"
guint8* gp_load_file_into_buffer (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileType filetype, int* result_length1, GError** error) {
#line 949 "GPhoto.c"
	guint8* result;
	GError * _inner_error_;
	CameraFile* camera_file;
	CameraFile* _tmp2_;
	int _tmp1_;
	CameraFile* _tmp0_;
	int res;
	gint data_size;
	gint data_length1;
	guint8* data;
	guint8* _tmp4_;
	gint buffer_size;
	gint buffer_length1;
	guint8* buffer;
	guint8* _tmp5_;
#line 176 "GPhoto.vala"
	g_return_val_if_fail (context != NULL, NULL);
#line 176 "GPhoto.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 176 "GPhoto.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 176 "GPhoto.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 973 "GPhoto.c"
	_inner_error_ = NULL;
	camera_file = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	res = (_tmp1_ = gp_file_new (&_tmp0_), camera_file = (_tmp2_ = _tmp0_, (camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL)), _tmp2_), _tmp1_);
#line 180 "GPhoto.vala"
	if (res != GP_OK) {
#line 981 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error allocating camera file: %s", (gint) res, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return NULL;
		}
	}
#line 183 "GPhoto.vala"
	res = gp_camera_file_get (camera, folder, filename, filetype, camera_file, context);
#line 184 "GPhoto.vala"
	if (res != GP_OK) {
#line 993 "GPhoto.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Error retrieving file object for %s/%s: %s", (gint) res, folder, filename, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
			return NULL;
		}
	}
	/* if buffer can be loaded into memory, return a copy of that (can't return buffer itself
	 as it will be destroyed when the camera_file is unref'd)*/
	data = (data_length1 = 0, NULL);
#line 191 "GPhoto.vala"
	res = gp_file_get_data_and_size (camera_file, &data, &data_length1);
#line 192 "GPhoto.vala"
	if (res != GP_OK) {
#line 1008 "GPhoto.c"
		gpointer _tmp3_;
		result = (_tmp3_ = NULL, *result_length1 = 0, _tmp3_);
		(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
		return result;
	}
#line 195 "GPhoto.vala"
	_tmp4_ = NULL;
#line 1016 "GPhoto.c"
	buffer = (_tmp4_ = g_new0 (guint8, data_length1), buffer_length1 = data_length1, buffer_size = buffer_length1, _tmp4_);
#line 196 "GPhoto.vala"
	memcpy (buffer, data, (gsize) buffer_length1);
#line 198 "GPhoto.vala"
	_tmp5_ = NULL;
#line 1022 "GPhoto.c"
	result = (_tmp5_ = buffer, *result_length1 = buffer_length1, _tmp5_);
	(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
	return result;
	(camera_file == NULL) ? NULL : (camera_file = (gp_file_unref (camera_file), NULL));
	buffer = (g_free (buffer), NULL);
}




