/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <libexif/exif-data.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <libexif/exif-tag.h>
#include <libexif/exif-format.h>
#include <libexif/exif-entry.h>
#include <libexif/exif-utils.h>
#include <gdk/gdk.h>
#include <gphoto2/gphoto2-camera.h>
#include <gphoto.h>
#include <gphoto2/gphoto2-result.h>
#include <gphoto2/gphoto2-port-result.h>
#include <gphoto2/gphoto2-abilities-list.h>
#include <gio/gio.h>
#include <float.h>
#include <math.h>
#include <gphoto2/gphoto2-context.h>
#include <gee.h>
#include <gphoto2/gphoto2-filesys.h>
#include <gphoto2/gphoto2-list.h>
#include <gphoto2/gphoto2-file.h>
#include <time.h>
#include <gobject/gvaluecollector.h>


#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;
typedef struct _LayoutItemPrivate LayoutItemPrivate;

#define TYPE_IMPORT_PREVIEW (import_preview_get_type ())
#define IMPORT_PREVIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMPORT_PREVIEW, ImportPreview))
#define IMPORT_PREVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMPORT_PREVIEW, ImportPreviewClass))
#define IS_IMPORT_PREVIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMPORT_PREVIEW))
#define IS_IMPORT_PREVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMPORT_PREVIEW))
#define IMPORT_PREVIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMPORT_PREVIEW, ImportPreviewClass))

typedef struct _ImportPreview ImportPreview;
typedef struct _ImportPreviewClass ImportPreviewClass;
typedef struct _ImportPreviewPrivate ImportPreviewPrivate;

#define TYPE_ORIENTATION (orientation_get_type ())

#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;
typedef struct _CheckerboardPagePrivate CheckerboardPagePrivate;

#define TYPE_IMPORT_PAGE (import_page_get_type ())
#define IMPORT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMPORT_PAGE, ImportPage))
#define IMPORT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMPORT_PAGE, ImportPageClass))
#define IS_IMPORT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMPORT_PAGE))
#define IS_IMPORT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMPORT_PAGE))
#define IMPORT_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMPORT_PAGE, ImportPageClass))

typedef struct _ImportPage ImportPage;
typedef struct _ImportPageClass ImportPageClass;
typedef struct _ImportPagePrivate ImportPagePrivate;

#define GP_TYPE_CONTEXT_WRAPPER (gp_context_wrapper_get_type ())
#define GP_CONTEXT_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GP_TYPE_CONTEXT_WRAPPER, GPContextWrapper))
#define GP_CONTEXT_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GP_TYPE_CONTEXT_WRAPPER, GPContextWrapperClass))
#define GP_IS_CONTEXT_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GP_TYPE_CONTEXT_WRAPPER))
#define GP_IS_CONTEXT_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GP_TYPE_CONTEXT_WRAPPER))
#define GP_CONTEXT_WRAPPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GP_TYPE_CONTEXT_WRAPPER, GPContextWrapperClass))

typedef struct _GPContextWrapper GPContextWrapper;
typedef struct _GPContextWrapperClass GPContextWrapperClass;

#define IMPORT_PAGE_TYPE_REFRESH_RESULT (import_page_refresh_result_get_type ())

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;
typedef struct _GPContextWrapperPrivate GPContextWrapperPrivate;

#define IMPORT_PAGE_TYPE_IMPORT_FILE (import_page_import_file_get_type ())
#define IMPORT_PAGE_IMPORT_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), IMPORT_PAGE_TYPE_IMPORT_FILE, ImportPageImportFile))
#define IMPORT_PAGE_IMPORT_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), IMPORT_PAGE_TYPE_IMPORT_FILE, ImportPageImportFileClass))
#define IMPORT_PAGE_IS_IMPORT_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IMPORT_PAGE_TYPE_IMPORT_FILE))
#define IMPORT_PAGE_IS_IMPORT_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), IMPORT_PAGE_TYPE_IMPORT_FILE))
#define IMPORT_PAGE_IMPORT_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), IMPORT_PAGE_TYPE_IMPORT_FILE, ImportPageImportFileClass))

typedef struct _ImportPageImportFile ImportPageImportFile;
typedef struct _ImportPageImportFileClass ImportPageImportFileClass;
typedef struct _ImportPageImportFilePrivate ImportPageImportFilePrivate;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

#define IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR (import_page_camera_import_comparator_get_type ())
#define IMPORT_PAGE_CAMERA_IMPORT_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR, ImportPageCameraImportComparator))
#define IMPORT_PAGE_CAMERA_IMPORT_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR, ImportPageCameraImportComparatorClass))
#define IMPORT_PAGE_IS_CAMERA_IMPORT_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR))
#define IMPORT_PAGE_IS_CAMERA_IMPORT_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR))
#define IMPORT_PAGE_CAMERA_IMPORT_COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR, ImportPageCameraImportComparatorClass))

typedef struct _ImportPageCameraImportComparator ImportPageCameraImportComparator;
typedef struct _ImportPageCameraImportComparatorClass ImportPageCameraImportComparatorClass;

#define TYPE_BATCH_IMPORT_JOB (batch_import_job_get_type ())
#define BATCH_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BATCH_IMPORT_JOB, BatchImportJob))
#define BATCH_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BATCH_IMPORT_JOB, BatchImportJobClass))
#define IS_BATCH_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BATCH_IMPORT_JOB))
#define IS_BATCH_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BATCH_IMPORT_JOB))
#define BATCH_IMPORT_JOB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BATCH_IMPORT_JOB, BatchImportJobClass))

typedef struct _BatchImportJob BatchImportJob;
typedef struct _BatchImportJobClass BatchImportJobClass;

#define IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB (import_page_camera_import_job_get_type ())
#define IMPORT_PAGE_CAMERA_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB, ImportPageCameraImportJob))
#define IMPORT_PAGE_CAMERA_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB, ImportPageCameraImportJobClass))
#define IMPORT_PAGE_IS_CAMERA_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB))
#define IMPORT_PAGE_IS_CAMERA_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB))
#define IMPORT_PAGE_CAMERA_IMPORT_JOB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB, ImportPageCameraImportJobClass))

typedef struct _ImportPageCameraImportJob ImportPageCameraImportJob;
typedef struct _ImportPageCameraImportJobClass ImportPageCameraImportJobClass;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;

#define TYPE_BATCH_IMPORT (batch_import_get_type ())
#define BATCH_IMPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BATCH_IMPORT, BatchImport))
#define BATCH_IMPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BATCH_IMPORT, BatchImportClass))
#define IS_BATCH_IMPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BATCH_IMPORT))
#define IS_BATCH_IMPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BATCH_IMPORT))
#define BATCH_IMPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BATCH_IMPORT, BatchImportClass))

typedef struct _BatchImport BatchImport;
typedef struct _BatchImportClass BatchImportClass;

#define TYPE_IMPORT_RESULT (import_result_get_type ())

#define TYPE_IMPORT_ID (import_id_get_type ())
typedef struct _ImportID ImportID;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;
typedef struct _ImportPageParamSpecImportFile ImportPageParamSpecImportFile;
typedef struct _BatchImportJobPrivate BatchImportJobPrivate;
typedef struct _ImportPageCameraImportJobPrivate ImportPageCameraImportJobPrivate;
typedef struct _ComparatorPrivate ComparatorPrivate;
typedef struct _ImportPageCameraImportComparatorPrivate ImportPageCameraImportComparatorPrivate;

#define TYPE_SINGLE_PHOTO_PAGE (single_photo_page_get_type ())
#define SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPage))
#define SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))
#define IS_SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SINGLE_PHOTO_PAGE))
#define IS_SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SINGLE_PHOTO_PAGE))
#define SINGLE_PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))

typedef struct _SinglePhotoPage SinglePhotoPage;
typedef struct _SinglePhotoPageClass SinglePhotoPageClass;
typedef struct _SinglePhotoPagePrivate SinglePhotoPagePrivate;

#define SINGLE_PHOTO_PAGE_TYPE_UPDATE_REASON (single_photo_page_update_reason_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_IMPORT_QUEUE_PAGE (import_queue_page_get_type ())
#define IMPORT_QUEUE_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePage))
#define IMPORT_QUEUE_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePageClass))
#define IS_IMPORT_QUEUE_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMPORT_QUEUE_PAGE))
#define IS_IMPORT_QUEUE_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMPORT_QUEUE_PAGE))
#define IMPORT_QUEUE_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePageClass))

typedef struct _ImportQueuePage ImportQueuePage;
typedef struct _ImportQueuePageClass ImportQueuePageClass;
typedef struct _ImportQueuePagePrivate ImportQueuePagePrivate;

struct _LayoutItem {
	GtkAlignment parent_instance;
	LayoutItemPrivate * priv;
};

struct _LayoutItemClass {
	GtkAlignmentClass parent_class;
	GtkWidget* (*get_control_panel) (LayoutItem* self);
	void (*exposed) (LayoutItem* self);
	void (*unexposed) (LayoutItem* self);
	void (*select) (LayoutItem* self);
	void (*unselect) (LayoutItem* self);
};

struct _ImportPreview {
	LayoutItem parent_instance;
	ImportPreviewPrivate * priv;
	gint fsid;
	char* folder;
	char* filename;
	gulong file_size;
	ExifData* exif;
};

struct _ImportPreviewClass {
	LayoutItemClass parent_class;
};

typedef enum  {
	ORIENTATION_MIN = 1,
	ORIENTATION_TOP_LEFT = 1,
	ORIENTATION_TOP_RIGHT = 2,
	ORIENTATION_BOTTOM_RIGHT = 3,
	ORIENTATION_BOTTOM_LEFT = 4,
	ORIENTATION_LEFT_TOP = 5,
	ORIENTATION_RIGHT_TOP = 6,
	ORIENTATION_RIGHT_BOTTOM = 7,
	ORIENTATION_LEFT_BOTTOM = 8,
	ORIENTATION_MAX = 8
} Orientation;

struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkActionGroup* action_group;
	GtkActionGroup* common_action_group;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkToolbar* (*get_toolbar) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*switching_to_fullscreen) (Page* self);
	void (*returning_from_fullscreen) (Page* self);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, const GdkEventKey* event);
	void (*on_move) (Page* self, const GdkRectangle* rect);
	void (*on_resize) (Page* self, const GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
};

struct _CheckerboardPage {
	Page parent_instance;
	CheckerboardPagePrivate * priv;
};

struct _CheckerboardPageClass {
	PageClass parent_class;
	void (*on_selection_changed) (CheckerboardPage* self, gint count);
	GtkMenu* (*get_context_menu) (CheckerboardPage* self);
	void (*on_item_activated) (CheckerboardPage* self, LayoutItem* item);
	gboolean (*on_context_invoked) (CheckerboardPage* self, GtkMenu* context_menu);
	LayoutItem* (*get_fullscreen_photo) (CheckerboardPage* self);
	gboolean (*on_mouse_over) (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
};

struct _ImportPage {
	CheckerboardPage parent_instance;
	ImportPagePrivate * priv;
};

struct _ImportPageClass {
	CheckerboardPageClass parent_class;
};

struct _ImportPagePrivate {
	GtkToolbar* toolbar;
	GtkLabel* camera_label;
	GtkToolButton* import_selected_button;
	GtkToolButton* import_all_button;
	GtkProgressBar* progress_bar;
	Camera* camera;
	GPContextWrapper* null_context;
	char* uri;
	gboolean busy;
	gboolean refreshed;
	int refresh_result;
	char* refresh_error;
	char* camera_name;
};

typedef enum  {
	IMPORT_PAGE_REFRESH_RESULT_OK,
	IMPORT_PAGE_REFRESH_RESULT_BUSY,
	IMPORT_PAGE_REFRESH_RESULT_LOCKED,
	IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR
} ImportPageRefreshResult;

/* ContextWrapper assigns signals to the various GPhoto.Context callbacks, as well as spins
 the event loop at opportune times.*/
struct _GPContextWrapper {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GPContextWrapperPrivate * priv;
	GPContext* context;
};

struct _GPContextWrapperClass {
	GTypeClass parent_class;
	void (*finalize) (GPContextWrapper *self);
	void (*idle) (GPContextWrapper* self);
	void (*error) (GPContextWrapper* self, const char* format, void* va_list);
	void (*status) (GPContextWrapper* self, const char* format, void* va_list);
	void (*message) (GPContextWrapper* self, const char* format, void* va_list);
	void (*progress_start) (GPContextWrapper* self, float target, const char* format, void* va_list);
	void (*progress_update) (GPContextWrapper* self, float current);
	void (*progress_stop) (GPContextWrapper* self);
};

struct _ImportPageImportFile {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ImportPageImportFilePrivate * priv;
	gint fsid;
	char* dir;
	char* filename;
	gulong file_size;
	gulong preview_size;
};

struct _ImportPageImportFileClass {
	GTypeClass parent_class;
	void (*finalize) (ImportPageImportFile *self);
};

typedef enum  {
	IMPORT_RESULT_SUCCESS,
	IMPORT_RESULT_FILE_ERROR,
	IMPORT_RESULT_DECODE_ERROR,
	IMPORT_RESULT_DATABASE_ERROR,
	IMPORT_RESULT_USER_ABORT,
	IMPORT_RESULT_NOT_A_FILE,
	IMPORT_RESULT_PHOTO_EXISTS,
	IMPORT_RESULT_UNSUPPORTED_FORMAT
} ImportResult;

struct _ImportID {
	gint64 id;
};

struct _ImportPageParamSpecImportFile {
	GParamSpec parent_instance;
};

struct _BatchImportJob {
	GTypeInstance parent_instance;
	volatile int ref_count;
	BatchImportJobPrivate * priv;
};

struct _BatchImportJobClass {
	GTypeClass parent_class;
	void (*finalize) (BatchImportJob *self);
	char* (*get_identifier) (BatchImportJob* self);
	gboolean (*prepare) (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library);
};

struct _ImportPageCameraImportJob {
	BatchImportJob parent_instance;
	ImportPageCameraImportJobPrivate * priv;
};

struct _ImportPageCameraImportJobClass {
	BatchImportJobClass parent_class;
};

struct _ImportPageCameraImportJobPrivate {
	GPContextWrapper* context;
	Camera* camera;
	char* dir;
	char* filename;
	gulong file_size;
	time_t exposure_time;
	GFile* dest_file;
};

struct _Comparator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ComparatorPrivate * priv;
};

struct _ComparatorClass {
	GTypeClass parent_class;
	void (*finalize) (Comparator *self);
	gint64 (*compare) (Comparator* self, gconstpointer a, gconstpointer b);
};

struct _ImportPageCameraImportComparator {
	Comparator parent_instance;
	ImportPageCameraImportComparatorPrivate * priv;
};

struct _ImportPageCameraImportComparatorClass {
	ComparatorClass parent_class;
};

typedef enum  {
	SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS
} SinglePhotoPageUpdateReason;

struct _Dimensions {
	gint width;
	gint height;
};

struct _SinglePhotoPage {
	Page parent_instance;
	SinglePhotoPagePrivate * priv;
	GdkGC* canvas_gc;
	GtkDrawingArea* canvas;
	GtkViewport* viewport;
};

struct _SinglePhotoPageClass {
	PageClass parent_class;
	void (*new_drawable) (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable);
	void (*updated_pixbuf) (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
	void (*paint) (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
};

struct _ImportQueuePage {
	SinglePhotoPage parent_instance;
	ImportQueuePagePrivate * priv;
};

struct _ImportQueuePageClass {
	SinglePhotoPageClass parent_class;
};

struct _ImportQueuePagePrivate {
	GtkToolbar* toolbar;
	GtkToolButton* stop_button;
	GeeArrayList* queue;
	BatchImport* current_batch;
	GtkProgressBar* progress_bar;
	guint64 progress_bytes;
	guint64 total_bytes;
};


static gpointer import_preview_parent_class = NULL;
static gpointer import_page_import_file_parent_class = NULL;
static gpointer import_page_camera_import_job_parent_class = NULL;
static gpointer import_page_camera_import_comparator_parent_class = NULL;
static gpointer import_page_parent_class = NULL;
static gpointer import_queue_page_parent_class = NULL;

GType layout_item_get_type (void);
GType import_preview_get_type (void);
enum  {
	IMPORT_PREVIEW_DUMMY_PROPERTY
};
#define IMPORT_PREVIEW_MAX_SCALE 128
#define IMPORT_PREVIEW_INTERP GDK_INTERP_BILINEAR
GType orientation_get_type (void);
ExifEntry* exif_find_first_entry (ExifData* data, ExifTag tag, ExifFormat format);
void layout_item_set_title (LayoutItem* self, const char* text);
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp);
void layout_item_set_image (LayoutItem* self, GdkPixbuf* pixbuf);
GdkPixbuf* orientation_rotate_pixbuf (Orientation self, GdkPixbuf* pixbuf);
LayoutItem* layout_item_new (void);
LayoutItem* layout_item_construct (GType object_type);
ImportPreview* import_preview_new (GdkPixbuf* pixbuf, ExifData* exif, gint fsid, const char* folder, const char* filename, gulong file_size);
ImportPreview* import_preview_construct (GType object_type, GdkPixbuf* pixbuf, ExifData* exif, gint fsid, const char* folder, const char* filename, gulong file_size);
static void import_preview_finalize (GObject* obj);
GType page_get_type (void);
GType checkerboard_page_get_type (void);
GType import_page_get_type (void);
gpointer gp_context_wrapper_ref (gpointer instance);
void gp_context_wrapper_unref (gpointer instance);
GParamSpec* gp_param_spec_context_wrapper (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gp_value_set_context_wrapper (GValue* value, gpointer v_object);
gpointer gp_value_get_context_wrapper (const GValue* value);
GType gp_context_wrapper_get_type (void);
#define IMPORT_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_IMPORT_PAGE, ImportPagePrivate))
enum  {
	IMPORT_PAGE_DUMMY_PROPERTY
};
GType import_page_refresh_result_get_type (void);
GPContextWrapper* gp_context_wrapper_new (void);
GPContextWrapper* gp_context_wrapper_construct (GType object_type);
static void import_page_on_file_menu (ImportPage* self);
static void _import_page_on_file_menu_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_IMPORT "shotwell-import"
static void import_page_on_import_selected (ImportPage* self);
static void _import_page_on_import_selected_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_IMPORT_ALL "shotwell-import-all"
static void import_page_on_import_all (ImportPage* self);
static void _import_page_on_import_all_gtk_action_callback (GtkAction* action, gpointer self);
static void import_page_on_edit_menu (ImportPage* self);
static void _import_page_on_edit_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void import_page_on_select_all (ImportPage* self);
static void _import_page_on_select_all_gtk_action_callback (GtkAction* action, gpointer self);
CheckerboardPage* checkerboard_page_new (const char* page_name);
CheckerboardPage* checkerboard_page_construct (GType object_type, const char* page_name);
void page_init_ui (Page* self, const char* ui_filename, const char* menubar_path, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1);
static void _import_page_on_import_selected_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void _import_page_on_import_all_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
void page_set_page_name (Page* self, const char* page_name);
ImportPage* import_page_new (Camera* camera, const char* uri);
ImportPage* import_page_construct (GType object_type, Camera* camera, const char* uri);
Camera* import_page_get_camera (ImportPage* self);
char* import_page_get_uri (ImportPage* self);
gboolean import_page_is_busy (ImportPage* self);
gboolean import_page_is_refreshed (ImportPage* self);
char* import_page_get_refresh_message (ImportPage* self);
static GtkToolbar* import_page_real_get_toolbar (Page* base);
static void import_page_real_on_selection_changed (CheckerboardPage* base, gint count);
static LayoutItem* import_page_real_get_fullscreen_photo (CheckerboardPage* base);
static void import_page_on_unmounted (ImportPage* self, GObject* source, GAsyncResult* aresult);
static void _import_page_on_unmounted_gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
gboolean import_page_unmount_camera (ImportPage* self, GMount* mount);
void import_page_switch_to_and_refresh (ImportPage* self);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
void app_window_switch_to_page (AppWindow* self, Page* page);
ImportPageRefreshResult import_page_refresh_camera (ImportPage* self);
void app_window_error_message (const char* message);
static gpointer import_page_import_file_ref (gpointer instance);
static void import_page_import_file_unref (gpointer instance);
static GParamSpec* import_page_param_spec_import_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
static void import_page_value_set_import_file (GValue* value, gpointer v_object);
static gpointer import_page_value_get_import_file (const GValue* value);
static GType import_page_import_file_get_type (void);
gint checkerboard_page_remove_all (CheckerboardPage* self);
void checkerboard_page_refresh (CheckerboardPage* self);
static gboolean import_page_enumerate_files (ImportPage* self, gint fsid, const char* dir, GeeArrayList* file_list, gulong* total_bytes, gulong* total_preview_bytes);
static void import_page_load_previews (ImportPage* self, GeeArrayList* file_list, gulong total_preview_bytes);
gint checkerboard_page_get_selected_count (CheckerboardPage* self);
gint checkerboard_page_get_count (CheckerboardPage* self);
static char* import_page_get_fs_basedir (ImportPage* self, gint fsid);
static char* import_page_get_fulldir (ImportPage* self, gint fsid, const char* dir);
void gp_get_info (GPContext* context, Camera* camera, const char* folder, const char* filename, CameraFileInfo* info, GError** error);
static ImportPageImportFile* import_page_import_file_new (gint fsid, const char* dir, const char* filename, gulong file_size, gulong preview_size);
static ImportPageImportFile* import_page_import_file_construct (GType object_type, gint fsid, const char* dir, const char* filename, gulong file_size, gulong preview_size);
gboolean spin_event_loop (void);
GdkPixbuf* gp_load_preview (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error);
GdkPixbuf* gp_load_image (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error);
ExifData* gp_load_exif (GPContext* context, Camera* camera, const char* folder, const char* filename, GError** error);
void checkerboard_page_add_item (CheckerboardPage* self, LayoutItem* item);
void page_set_item_sensitive (Page* self, const char* path, gboolean sensitive);
static void import_page_import (ImportPage* self, GeeIterable* items);
GeeIterable* checkerboard_page_get_selected (CheckerboardPage* self);
GeeIterable* checkerboard_page_get_items (CheckerboardPage* self);
void checkerboard_page_select_all (CheckerboardPage* self);
static ImportPageCameraImportComparator* import_page_camera_import_comparator_new (void);
static ImportPageCameraImportComparator* import_page_camera_import_comparator_construct (GType object_type);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
static GType import_page_camera_import_comparator_get_type (void);
gpointer batch_import_job_ref (gpointer instance);
void batch_import_job_unref (gpointer instance);
GParamSpec* param_spec_batch_import_job (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_batch_import_job (GValue* value, gpointer v_object);
gpointer value_get_batch_import_job (const GValue* value);
GType batch_import_job_get_type (void);
static GType import_page_camera_import_job_get_type (void);
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
GType sorted_list_get_type (void);
GFile* batch_import_create_library_path (const char* filename, ExifData* exif, const time_t* ts, gboolean* collision);
gboolean exif_get_timestamp (ExifData* exif, time_t* timestamp);
gboolean sorted_list_add (SortedList* self, gconstpointer item);
static ImportPageCameraImportJob* import_page_camera_import_job_new (GPContextWrapper* context, Camera* camera, const char* dir, const char* filename, gulong file_size, const time_t* exposure_time, GFile* dest_file);
static ImportPageCameraImportJob* import_page_camera_import_job_construct (GType object_type, GPContextWrapper* context, Camera* camera, const char* dir, const char* filename, gulong file_size, const time_t* exposure_time, GFile* dest_file);
gint sorted_list_get_size (SortedList* self);
BatchImport* batch_import_new (GeeIterable* jobs, const char* name, guint64 total_bytes);
BatchImport* batch_import_construct (GType object_type, GeeIterable* jobs, const char* name, guint64 total_bytes);
gpointer batch_import_ref (gpointer instance);
void batch_import_unref (gpointer instance);
GParamSpec* param_spec_batch_import (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_batch_import (GValue* value, gpointer v_object);
gpointer value_get_batch_import (const GValue* value);
GType batch_import_get_type (void);
GType import_result_get_type (void);
static void import_page_on_import_job_failed (ImportPage* self, ImportResult _result_, BatchImportJob* job, GFile* file);
static void _import_page_on_import_job_failed_batch_import_import_job_failed (BatchImport* _sender, ImportResult _result_, BatchImportJob* job, GFile* file, gpointer self);
static void import_page_close_import (ImportPage* self);
GType import_id_get_type (void);
ImportID* import_id_dup (const ImportID* self);
void import_id_free (ImportID* self);
GType photo_get_type (void);
static void _import_page_close_import_batch_import_import_complete (BatchImport* _sender, const ImportID* import_id, SortedList* photos_by_date, GeeArrayList* failed, GeeArrayList* skipped, gpointer self);
void app_window_enqueue_batch_import (AppWindow* self, BatchImport* batch_import);
void app_window_switch_to_import_queue_page (AppWindow* self);
enum  {
	IMPORT_PAGE_IMPORT_FILE_DUMMY_PROPERTY
};
static void import_page_import_file_finalize (ImportPageImportFile* obj);
#define IMPORT_PAGE_CAMERA_IMPORT_JOB_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB, ImportPageCameraImportJobPrivate))
enum  {
	IMPORT_PAGE_CAMERA_IMPORT_JOB_DUMMY_PROPERTY
};
static time_t import_page_camera_import_job_get_exposure_time (ImportPageCameraImportJob* self);
static char* import_page_camera_import_job_real_get_identifier (BatchImportJob* base);
void gp_save_image (GPContext* context, Camera* camera, const char* folder, const char* filename, GFile* dest_file, GError** error);
static gboolean import_page_camera_import_job_real_prepare (BatchImportJob* base, GFile** file_to_import, gboolean* copy_to_library);
static void import_page_camera_import_job_finalize (BatchImportJob* obj);
enum  {
	IMPORT_PAGE_CAMERA_IMPORT_COMPARATOR_DUMMY_PROPERTY
};
static gint64 import_page_camera_import_comparator_real_compare (Comparator* base, ImportPageCameraImportJob* a, ImportPageCameraImportJob* b);
static void import_page_finalize (GObject* obj);
GType single_photo_page_get_type (void);
GType single_photo_page_update_reason_get_type (void);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
GType import_queue_page_get_type (void);
#define IMPORT_QUEUE_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePagePrivate))
enum  {
	IMPORT_QUEUE_PAGE_DUMMY_PROPERTY
};
static void import_queue_page_on_file_menu (ImportQueuePage* self);
static void _import_queue_page_on_file_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void import_queue_page_on_stop (ImportQueuePage* self);
static void _import_queue_page_on_stop_gtk_action_callback (GtkAction* action, gpointer self);
SinglePhotoPage* single_photo_page_new (const char* page_name);
SinglePhotoPage* single_photo_page_construct (GType object_type, const char* page_name);
static void _import_queue_page_on_stop_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
ImportQueuePage* import_queue_page_new (void);
ImportQueuePage* import_queue_page_construct (GType object_type);
static GtkToolbar* import_queue_page_real_get_toolbar (Page* base);
guint64 batch_import_get_total_bytes (BatchImport* self);
static void import_queue_page_on_starting (ImportQueuePage* self, BatchImport* batch_import);
static void _import_queue_page_on_starting_batch_import_starting (BatchImport* _sender, gpointer self);
static void import_queue_page_on_imported (ImportQueuePage* self, Photo* photo);
static void _import_queue_page_on_imported_batch_import_imported (BatchImport* _sender, Photo* photo, gpointer self);
static void import_queue_page_on_import_complete (ImportQueuePage* self, BatchImport* batch_import, const ImportID* import_id, SortedList* imported, GeeArrayList* failed, GeeArrayList* skipped);
static void _import_queue_page_on_import_complete_batch_import_import_complete (BatchImport* _sender, const ImportID* import_id, SortedList* photos_by_date, GeeArrayList* failed, GeeArrayList* skipped, gpointer self);
void batch_import_schedule (BatchImport* self);
void import_queue_page_enqueue_and_schedule (ImportQueuePage* self, BatchImport* batch_import);
gint import_queue_page_get_batch_count (ImportQueuePage* self);
void batch_import_user_halt (BatchImport* self);
GdkPixbuf* photo_get_pixbuf (Photo* self, gint exceptions, gint scale, GdkInterpType interp, GError** error);
#define PHOTO_EXCEPTION_NONE 0
void single_photo_page_set_pixbuf (SinglePhotoPage* self, GdkPixbuf* unscaled);
guint64 photo_query_filesize (Photo* self);
char* photo_get_name (Photo* self);
void app_window_report_import_failures (const char* name, GeeList* failed, GeeList* skipped);
char* batch_import_get_name (BatchImport* self);
static void import_queue_page_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GtkActionEntry IMPORT_PAGE_ACTIONS[] = {{"FileMenu", NULL, "_File", NULL, NULL, (GCallback) _import_page_on_file_menu_gtk_action_callback}, {"ImportSelected", RESOURCES_IMPORT, "Import _Selected", NULL, NULL, (GCallback) _import_page_on_import_selected_gtk_action_callback}, {"ImportAll", RESOURCES_IMPORT_ALL, "Import _All", NULL, NULL, (GCallback) _import_page_on_import_all_gtk_action_callback}, {"EditMenu", NULL, "_Edit", NULL, NULL, (GCallback) _import_page_on_edit_menu_gtk_action_callback}, {"SelectAll", GTK_STOCK_SELECT_ALL, "Select _All", "<Ctrl>A", "Select all the photos for importing", (GCallback) _import_page_on_select_all_gtk_action_callback}, {"ViewMenu", NULL, "_View", NULL, NULL, (GCallback) NULL}, {"HelpMenu", NULL, "_Help", NULL, NULL, (GCallback) NULL}};
static const GtkActionEntry IMPORT_QUEUE_PAGE_ACTIONS[] = {{"FileMenu", NULL, "_File", NULL, NULL, (GCallback) _import_queue_page_on_file_menu_gtk_action_callback}, {"Stop", GTK_STOCK_STOP, "_Stop Import", NULL, "Stop importing photos", (GCallback) _import_queue_page_on_stop_gtk_action_callback}, {"ViewMenu", NULL, "_View", NULL, NULL, (GCallback) NULL}, {"HelpMenu", NULL, "_Help", NULL, NULL, (GCallback) NULL}};


#line 17 "ImportPage.vala"
ImportPreview* import_preview_construct (GType object_type, GdkPixbuf* pixbuf, ExifData* exif, gint fsid, const char* folder, const char* filename, gulong file_size) {
#line 727 "ImportPage.c"
	ImportPreview * self;
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp3_;
	const char* _tmp2_;
	ExifData* _tmp5_;
	ExifData* _tmp4_;
	Orientation orientation;
	ExifEntry* entry;
	GdkPixbuf* _tmp7_;
	GdkPixbuf* scaled;
	gboolean _tmp8_;
	GdkPixbuf* _tmp11_;
	GdkPixbuf* _tmp10_;
#line 17 "ImportPage.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 17 "ImportPage.vala"
	g_return_val_if_fail (exif != NULL, NULL);
#line 17 "ImportPage.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 17 "ImportPage.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 750 "ImportPage.c"
	self = (ImportPreview*) layout_item_construct (object_type);
#line 19 "ImportPage.vala"
	self->fsid = fsid;
#line 754 "ImportPage.c"
	_tmp1_ = NULL;
#line 20 "ImportPage.vala"
	_tmp0_ = NULL;
#line 20 "ImportPage.vala"
	self->folder = (_tmp1_ = (_tmp0_ = folder, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->folder = (g_free (self->folder), NULL), _tmp1_);
#line 760 "ImportPage.c"
	_tmp3_ = NULL;
#line 21 "ImportPage.vala"
	_tmp2_ = NULL;
#line 21 "ImportPage.vala"
	self->filename = (_tmp3_ = (_tmp2_ = filename, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->filename = (g_free (self->filename), NULL), _tmp3_);
#line 22 "ImportPage.vala"
	self->file_size = file_size;
#line 768 "ImportPage.c"
	_tmp5_ = NULL;
#line 23 "ImportPage.vala"
	_tmp4_ = NULL;
#line 23 "ImportPage.vala"
	self->exif = (_tmp5_ = (_tmp4_ = exif, (_tmp4_ == NULL) ? NULL : exif_data_ref (_tmp4_), _tmp4_), (self->exif == NULL) ? NULL : (self->exif = (exif_data_unref (self->exif), NULL)), _tmp5_);
#line 774 "ImportPage.c"
	orientation = ORIENTATION_TOP_LEFT;
	entry = exif_find_first_entry (exif, EXIF_TAG_ORIENTATION, EXIF_FORMAT_SHORT);
#line 28 "ImportPage.vala"
	if (entry != NULL) {
#line 779 "ImportPage.c"
		gint o;
		gboolean _tmp6_;
		o = (gint) exif_get_short (entry->data, exif_data_get_byte_order (exif));
		_tmp6_ = FALSE;
#line 30 "ImportPage.vala"
		if (o >= ((gint) ORIENTATION_MIN)) {
#line 30 "ImportPage.vala"
			_tmp6_ = o <= ((gint) ORIENTATION_MAX);
#line 788 "ImportPage.c"
		} else {
#line 30 "ImportPage.vala"
			_tmp6_ = FALSE;
#line 792 "ImportPage.c"
		}
#line 30 "ImportPage.vala"
		if (_tmp6_) {
#line 31 "ImportPage.vala"
			orientation = (Orientation) o;
#line 798 "ImportPage.c"
		}
	}
#line 34 "ImportPage.vala"
	layout_item_set_title (LAYOUT_ITEM (self), filename);
#line 37 "ImportPage.vala"
	_tmp7_ = NULL;
#line 805 "ImportPage.c"
	scaled = (_tmp7_ = pixbuf, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_));
	_tmp8_ = FALSE;
#line 38 "ImportPage.vala"
	if (gdk_pixbuf_get_width (pixbuf) > IMPORT_PREVIEW_MAX_SCALE) {
#line 38 "ImportPage.vala"
		_tmp8_ = TRUE;
#line 812 "ImportPage.c"
	} else {
#line 38 "ImportPage.vala"
		_tmp8_ = gdk_pixbuf_get_height (pixbuf) > IMPORT_PREVIEW_MAX_SCALE;
#line 816 "ImportPage.c"
	}
#line 38 "ImportPage.vala"
	if (_tmp8_) {
#line 820 "ImportPage.c"
		GdkPixbuf* _tmp9_;
		_tmp9_ = NULL;
#line 39 "ImportPage.vala"
		scaled = (_tmp9_ = scale_pixbuf (pixbuf, IMPORT_PREVIEW_MAX_SCALE, IMPORT_PREVIEW_INTERP), (scaled == NULL) ? NULL : (scaled = (g_object_unref (scaled), NULL)), _tmp9_);
#line 825 "ImportPage.c"
	}
#line 230 "Orientation.vala"
	_tmp11_ = NULL;
#line 42 "ImportPage.vala"
	_tmp10_ = NULL;
#line 42 "ImportPage.vala"
	layout_item_set_image (LAYOUT_ITEM (self), _tmp11_ = orientation_rotate_pixbuf (orientation, (_tmp10_ = scaled, (_tmp10_ == NULL) ? NULL : g_object_ref (_tmp10_))));
#line 833 "ImportPage.c"
	(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
	(entry == NULL) ? NULL : (entry = (exif_entry_unref (entry), NULL));
	(scaled == NULL) ? NULL : (scaled = (g_object_unref (scaled), NULL));
	return self;
}


#line 17 "ImportPage.vala"
ImportPreview* import_preview_new (GdkPixbuf* pixbuf, ExifData* exif, gint fsid, const char* folder, const char* filename, gulong file_size) {
#line 17 "ImportPage.vala"
	return import_preview_construct (TYPE_IMPORT_PREVIEW, pixbuf, exif, fsid, folder, filename, file_size);
#line 845 "ImportPage.c"
}


static void import_preview_class_init (ImportPreviewClass * klass) {
	import_preview_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = import_preview_finalize;
}


static void import_preview_instance_init (ImportPreview * self) {
}


static void import_preview_finalize (GObject* obj) {
	ImportPreview * self;
	self = IMPORT_PREVIEW (obj);
	self->folder = (g_free (self->folder), NULL);
	self->filename = (g_free (self->filename), NULL);
	(self->exif == NULL) ? NULL : (self->exif = (exif_data_unref (self->exif), NULL));
	G_OBJECT_CLASS (import_preview_parent_class)->finalize (obj);
}


GType import_preview_get_type (void) {
	static GType import_preview_type_id = 0;
	if (import_preview_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ImportPreviewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) import_preview_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImportPreview), 0, (GInstanceInitFunc) import_preview_instance_init, NULL };
		import_preview_type_id = g_type_register_static (TYPE_LAYOUT_ITEM, "ImportPreview", &g_define_type_info, 0);
	}
	return import_preview_type_id;
}



GType import_page_refresh_result_get_type (void) {
	static GType import_page_refresh_result_type_id = 0;
	if (G_UNLIKELY (import_page_refresh_result_type_id == 0)) {
		static const GEnumValue values[] = {{IMPORT_PAGE_REFRESH_RESULT_OK, "IMPORT_PAGE_REFRESH_RESULT_OK", "ok"}, {IMPORT_PAGE_REFRESH_RESULT_BUSY, "IMPORT_PAGE_REFRESH_RESULT_BUSY", "busy"}, {IMPORT_PAGE_REFRESH_RESULT_LOCKED, "IMPORT_PAGE_REFRESH_RESULT_LOCKED", "locked"}, {IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR, "IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR", "library-error"}, {0, NULL, NULL}};
		import_page_refresh_result_type_id = g_enum_register_static ("ImportPageRefreshResult", values);
	}
	return import_page_refresh_result_type_id;
}


#line 559 "ImportPage.vala"
static void _import_page_on_file_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 892 "ImportPage.c"
	import_page_on_file_menu (self);
}


#line 564 "ImportPage.vala"
static void _import_page_on_import_selected_gtk_action_callback (GtkAction* action, gpointer self) {
#line 899 "ImportPage.c"
	import_page_on_import_selected (self);
}


#line 568 "ImportPage.vala"
static void _import_page_on_import_all_gtk_action_callback (GtkAction* action, gpointer self) {
#line 906 "ImportPage.c"
	import_page_on_import_all (self);
}


#line 572 "ImportPage.vala"
static void _import_page_on_edit_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 913 "ImportPage.c"
	import_page_on_edit_menu (self);
}


#line 576 "ImportPage.vala"
static void _import_page_on_select_all_gtk_action_callback (GtkAction* action, gpointer self) {
#line 920 "ImportPage.c"
	import_page_on_select_all (self);
}


#line 564 "ImportPage.vala"
static void _import_page_on_import_selected_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 927 "ImportPage.c"
	import_page_on_import_selected (self);
}


#line 568 "ImportPage.vala"
static void _import_page_on_import_all_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 934 "ImportPage.c"
	import_page_on_import_all (self);
}


#line 147 "ImportPage.vala"
ImportPage* import_page_construct (GType object_type, Camera* camera, const char* uri) {
#line 941 "ImportPage.c"
	ImportPage * self;
	char* _tmp0_;
	Camera* _tmp2_;
	Camera* _tmp1_;
	char* _tmp4_;
	const char* _tmp3_;
	GtkToolItem* camera_label_item;
	GtkSeparatorToolItem* separator;
	GtkToolItem* progress_item;
	GtkToolButton* _tmp5_;
	GtkToolButton* _tmp6_;
	CameraAbilities abilities;
	CameraAbilities _tmp9_;
	int _tmp8_;
	CameraAbilities _tmp7_;
	int res;
#line 147 "ImportPage.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 147 "ImportPage.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 148 "ImportPage.vala"
	self = (ImportPage*) checkerboard_page_construct (object_type, "Camera");
#line 964 "ImportPage.c"
	_tmp0_ = NULL;
#line 149 "ImportPage.vala"
	self->priv->camera_name = (_tmp0_ = g_strdup ("Camera"), self->priv->camera_name = (g_free (self->priv->camera_name), NULL), _tmp0_);
#line 968 "ImportPage.c"
	_tmp2_ = NULL;
#line 151 "ImportPage.vala"
	_tmp1_ = NULL;
#line 151 "ImportPage.vala"
	self->priv->camera = (_tmp2_ = (_tmp1_ = camera, (_tmp1_ == NULL) ? NULL : GPHOTO_REF_CAMERA (_tmp1_)), (self->priv->camera == NULL) ? NULL : (self->priv->camera = (gp_camera_unref (self->priv->camera), NULL)), _tmp2_);
#line 974 "ImportPage.c"
	_tmp4_ = NULL;
#line 152 "ImportPage.vala"
	_tmp3_ = NULL;
#line 152 "ImportPage.vala"
	self->priv->uri = (_tmp4_ = (_tmp3_ = uri, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), self->priv->uri = (g_free (self->priv->uri), NULL), _tmp4_);
#line 154 "ImportPage.vala"
	page_init_ui (PAGE (self), "import.ui", "/ImportMenuBar", "ImportActionGroup", IMPORT_PAGE_ACTIONS, G_N_ELEMENTS (IMPORT_PAGE_ACTIONS), NULL, 0);
#line 982 "ImportPage.c"
	camera_label_item = g_object_ref_sink (gtk_tool_item_new ());
#line 159 "ImportPage.vala"
	gtk_container_add (GTK_CONTAINER (camera_label_item), GTK_WIDGET (self->priv->camera_label));
#line 161 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, camera_label_item, -1);
#line 988 "ImportPage.c"
	separator = g_object_ref_sink ((GtkSeparatorToolItem*) gtk_separator_tool_item_new ());
#line 165 "ImportPage.vala"
	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separator), TRUE);
#line 166 "ImportPage.vala"
	gtk_separator_tool_item_set_draw (separator, FALSE);
#line 168 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (separator), -1);
#line 170 "ImportPage.vala"
	gtk_progress_bar_set_orientation (self->priv->progress_bar, GTK_PROGRESS_LEFT_TO_RIGHT);
#line 171 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->progress_bar), "visible", FALSE, NULL);
#line 1000 "ImportPage.c"
	progress_item = g_object_ref_sink (gtk_tool_item_new ());
#line 173 "ImportPage.vala"
	gtk_container_add (GTK_CONTAINER (progress_item), GTK_WIDGET (self->priv->progress_bar));
#line 175 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, progress_item, -1);
#line 1006 "ImportPage.c"
	_tmp5_ = NULL;
#line 177 "ImportPage.vala"
	self->priv->import_selected_button = (_tmp5_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (RESOURCES_IMPORT)), (self->priv->import_selected_button == NULL) ? NULL : (self->priv->import_selected_button = (g_object_unref (self->priv->import_selected_button), NULL)), _tmp5_);
#line 178 "ImportPage.vala"
	gtk_tool_button_set_label (self->priv->import_selected_button, "Import Selected");
#line 179 "ImportPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->import_selected_button), "Import the selected photos into your library");
#line 180 "ImportPage.vala"
	g_signal_connect_object (self->priv->import_selected_button, "clicked", (GCallback) _import_page_on_import_selected_gtk_tool_button_clicked, self, 0);
#line 181 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_selected_button), "sensitive", FALSE, NULL);
#line 183 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->import_selected_button), -1);
#line 1020 "ImportPage.c"
	_tmp6_ = NULL;
#line 185 "ImportPage.vala"
	self->priv->import_all_button = (_tmp6_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (RESOURCES_IMPORT_ALL)), (self->priv->import_all_button == NULL) ? NULL : (self->priv->import_all_button = (g_object_unref (self->priv->import_all_button), NULL)), _tmp6_);
#line 186 "ImportPage.vala"
	gtk_tool_button_set_label (self->priv->import_all_button, "Import All");
#line 187 "ImportPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->import_all_button), "Import all the photos on this camera into your library");
#line 188 "ImportPage.vala"
	g_signal_connect_object (self->priv->import_all_button, "clicked", (GCallback) _import_page_on_import_all_gtk_tool_button_clicked, self, 0);
#line 189 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_all_button), "sensitive", FALSE, NULL);
#line 191 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->import_all_button), -1);
#line 194 "ImportPage.vala"
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 1036 "ImportPage.c"
	res = (_tmp8_ = gp_camera_get_abilities (camera, &_tmp7_), abilities = (_tmp9_ = _tmp7_,  (&abilities), _tmp9_), _tmp8_);
#line 198 "ImportPage.vala"
	if (res != GP_OK) {
#line 199 "ImportPage.vala"
		g_debug ("ImportPage.vala:199: [%d] Unable to get camera abilities: %s", (gint) res, gp_port_result_as_string (res));
#line 1042 "ImportPage.c"
	} else {
		char* _tmp11_;
		const char* _tmp10_;
		_tmp11_ = NULL;
#line 201 "ImportPage.vala"
		_tmp10_ = NULL;
#line 201 "ImportPage.vala"
		self->priv->camera_name = (_tmp11_ = (_tmp10_ = abilities.model, (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), self->priv->camera_name = (g_free (self->priv->camera_name), NULL), _tmp11_);
#line 202 "ImportPage.vala"
		gtk_label_set_text (self->priv->camera_label, abilities.model);
#line 204 "ImportPage.vala"
		page_set_page_name (PAGE (self), self->priv->camera_name);
#line 1055 "ImportPage.c"
	}
#line 207 "ImportPage.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 1059 "ImportPage.c"
	(camera_label_item == NULL) ? NULL : (camera_label_item = (g_object_unref (camera_label_item), NULL));
	(separator == NULL) ? NULL : (separator = (g_object_unref (separator), NULL));
	(progress_item == NULL) ? NULL : (progress_item = (g_object_unref (progress_item), NULL));
	 (&abilities);
	return self;
}


#line 147 "ImportPage.vala"
ImportPage* import_page_new (Camera* camera, const char* uri) {
#line 147 "ImportPage.vala"
	return import_page_construct (TYPE_IMPORT_PAGE, camera, uri);
#line 1072 "ImportPage.c"
}


#line 210 "ImportPage.vala"
Camera* import_page_get_camera (ImportPage* self) {
#line 1078 "ImportPage.c"
	Camera* result;
	Camera* _tmp0_;
#line 210 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), NULL);
#line 211 "ImportPage.vala"
	_tmp0_ = NULL;
#line 1085 "ImportPage.c"
	result = (_tmp0_ = self->priv->camera, (_tmp0_ == NULL) ? NULL : GPHOTO_REF_CAMERA (_tmp0_));
	return result;
}


#line 214 "ImportPage.vala"
char* import_page_get_uri (ImportPage* self) {
#line 1093 "ImportPage.c"
	char* result;
	const char* _tmp0_;
#line 214 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), NULL);
#line 215 "ImportPage.vala"
	_tmp0_ = NULL;
#line 1100 "ImportPage.c"
	result = (_tmp0_ = self->priv->uri, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 218 "ImportPage.vala"
gboolean import_page_is_busy (ImportPage* self) {
#line 1108 "ImportPage.c"
	gboolean result;
#line 218 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), FALSE);
#line 1112 "ImportPage.c"
	result = self->priv->busy;
	return result;
}


#line 222 "ImportPage.vala"
gboolean import_page_is_refreshed (ImportPage* self) {
#line 1120 "ImportPage.c"
	gboolean result;
	gboolean _tmp0_;
#line 222 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), FALSE);
#line 1125 "ImportPage.c"
	_tmp0_ = FALSE;
#line 223 "ImportPage.vala"
	if (self->priv->refreshed) {
#line 223 "ImportPage.vala"
		_tmp0_ = !self->priv->busy;
#line 1131 "ImportPage.c"
	} else {
#line 223 "ImportPage.vala"
		_tmp0_ = FALSE;
#line 1135 "ImportPage.c"
	}
	result = _tmp0_;
	return result;
}


#line 226 "ImportPage.vala"
char* import_page_get_refresh_message (ImportPage* self) {
#line 1144 "ImportPage.c"
	char* result;
	char* msg;
#line 226 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), NULL);
#line 1149 "ImportPage.c"
	msg = NULL;
#line 228 "ImportPage.vala"
	if (self->priv->refresh_error != NULL) {
#line 1153 "ImportPage.c"
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
#line 229 "ImportPage.vala"
		_tmp0_ = NULL;
#line 229 "ImportPage.vala"
		msg = (_tmp1_ = (_tmp0_ = self->priv->refresh_error, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), msg = (g_free (msg), NULL), _tmp1_);
#line 1161 "ImportPage.c"
	} else {
#line 230 "ImportPage.vala"
		if (self->priv->refresh_result == GP_OK) {
#line 1165 "ImportPage.c"
		} else {
			char* _tmp2_;
			/* all went well*/
			_tmp2_ = NULL;
#line 233 "ImportPage.vala"
			msg = (_tmp2_ = g_strdup_printf ("%s (%d)", gp_port_result_as_string (self->priv->refresh_result), (gint) self->priv->refresh_result), msg = (g_free (msg), NULL), _tmp2_);
#line 1172 "ImportPage.c"
		}
	}
	result = msg;
	return result;
}


#line 239 "ImportPage.vala"
static GtkToolbar* import_page_real_get_toolbar (Page* base) {
#line 1182 "ImportPage.c"
	ImportPage * self;
	GtkToolbar* result;
	GtkToolbar* _tmp0_;
	self = IMPORT_PAGE (base);
#line 240 "ImportPage.vala"
	_tmp0_ = NULL;
#line 1189 "ImportPage.c"
	result = (_tmp0_ = self->priv->toolbar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 243 "ImportPage.vala"
static void import_page_real_on_selection_changed (CheckerboardPage* base, gint count) {
#line 1197 "ImportPage.c"
	ImportPage * self;
	gboolean _tmp0_;
	gboolean _tmp1_;
	self = IMPORT_PAGE (base);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
#line 244 "ImportPage.vala"
	if (!self->priv->busy) {
#line 244 "ImportPage.vala"
		_tmp1_ = self->priv->refreshed;
#line 1208 "ImportPage.c"
	} else {
#line 244 "ImportPage.vala"
		_tmp1_ = FALSE;
#line 1212 "ImportPage.c"
	}
#line 244 "ImportPage.vala"
	if (_tmp1_) {
#line 244 "ImportPage.vala"
		_tmp0_ = count > 0;
#line 1218 "ImportPage.c"
	} else {
#line 244 "ImportPage.vala"
		_tmp0_ = FALSE;
#line 1222 "ImportPage.c"
	}
#line 244 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_selected_button), "sensitive", _tmp0_, NULL);
#line 1226 "ImportPage.c"
}


#line 247 "ImportPage.vala"
static LayoutItem* import_page_real_get_fullscreen_photo (CheckerboardPage* base) {
#line 1232 "ImportPage.c"
	ImportPage * self;
	LayoutItem* result;
	self = IMPORT_PAGE (base);
#line 248 "ImportPage.vala"
	g_error ("ImportPage.vala:248: No fullscreen support for import pages");
#line 1238 "ImportPage.c"
	result = NULL;
	return result;
}


#line 267 "ImportPage.vala"
static void _import_page_on_unmounted_gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 1246 "ImportPage.c"
	import_page_on_unmounted (self, source_object, res);
}


#line 253 "ImportPage.vala"
gboolean import_page_unmount_camera (ImportPage* self, GMount* mount) {
#line 1253 "ImportPage.c"
	gboolean result;
#line 253 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), FALSE);
#line 253 "ImportPage.vala"
	g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
#line 254 "ImportPage.vala"
	if (self->priv->busy) {
#line 1261 "ImportPage.c"
		result = FALSE;
		return result;
	}
#line 257 "ImportPage.vala"
	self->priv->busy = TRUE;
#line 258 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->progress_bar), "visible", TRUE, NULL);
#line 259 "ImportPage.vala"
	gtk_progress_bar_set_fraction (self->priv->progress_bar, 0.0);
#line 260 "ImportPage.vala"
	gtk_progress_bar_set_text (self->priv->progress_bar, "Unmounting ...");
#line 262 "ImportPage.vala"
	g_mount_unmount (mount, G_MOUNT_UNMOUNT_NONE, NULL, _import_page_on_unmounted_gasync_ready_callback, self);
#line 1275 "ImportPage.c"
	result = TRUE;
	return result;
}


#line 267 "ImportPage.vala"
static void import_page_on_unmounted (ImportPage* self, GObject* source, GAsyncResult* aresult) {
#line 1283 "ImportPage.c"
	GError * _inner_error_;
	GMount* _tmp0_;
	GMount* mount;
#line 267 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 267 "ImportPage.vala"
	g_return_if_fail (G_IS_OBJECT (source));
#line 267 "ImportPage.vala"
	g_return_if_fail (G_IS_ASYNC_RESULT (aresult));
#line 1293 "ImportPage.c"
	_inner_error_ = NULL;
#line 268 "ImportPage.vala"
	_tmp0_ = NULL;
#line 1297 "ImportPage.c"
	mount = (_tmp0_ = G_MOUNT (source), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	{
#line 270 "ImportPage.vala"
		g_mount_unmount_finish (mount, aresult, &_inner_error_);
#line 1302 "ImportPage.c"
		if (_inner_error_ != NULL) {
			goto __catch25_g_error;
			goto __finally25;
		}
	}
	goto __finally25;
	__catch25_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			/* TODO: Better error reporting*/
#line 273 "ImportPage.vala"
			g_debug ("ImportPage.vala:273: %s", err->message);
#line 1318 "ImportPage.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
			return;
		}
	}
	__finally25:
	if (_inner_error_ != NULL) {
		(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* XXX: iPhone/iPod returns a USB error if a camera_init() is done too quickly after an
	 unmount.  A 50ms sleep gives it time to reorient itself.*/
#line 280 "ImportPage.vala"
	g_usleep ((gulong) 50000);
#line 282 "ImportPage.vala"
	self->priv->busy = FALSE;
#line 283 "ImportPage.vala"
	gtk_progress_bar_set_text (self->priv->progress_bar, "");
#line 284 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->progress_bar), "visible", FALSE, NULL);
#line 286 "ImportPage.vala"
	import_page_switch_to_and_refresh (self);
#line 1343 "ImportPage.c"
	(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
}


#line 289 "ImportPage.vala"
void import_page_switch_to_and_refresh (ImportPage* self) {
#line 1350 "ImportPage.c"
	AppWindow* _tmp0_;
	ImportPageRefreshResult res;
#line 289 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 1355 "ImportPage.c"
	/* jump to page*/
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 291 "ImportPage.vala"
	app_window_switch_to_page (_tmp0_ = app_window_get_instance (), PAGE (self));
#line 1361 "ImportPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	/* now with camera unmounted, refresh the view*/
	res = import_page_refresh_camera (self);
#line 295 "ImportPage.vala"
	if (res != IMPORT_PAGE_REFRESH_RESULT_OK) {
#line 1367 "ImportPage.c"
		char* reason;
		char* _tmp4_;
		reason = NULL;
#line 297 "ImportPage.vala"
		switch (res) {
#line 1373 "ImportPage.c"
			case IMPORT_PAGE_REFRESH_RESULT_LOCKED:
			{
				char* _tmp1_;
				_tmp1_ = NULL;
#line 299 "ImportPage.vala"
				reason = (_tmp1_ = g_strdup ("The camera is locked."), reason = (g_free (reason), NULL), _tmp1_);
#line 300 "ImportPage.vala"
				break;
#line 1382 "ImportPage.c"
			}
			case IMPORT_PAGE_REFRESH_RESULT_BUSY:
			{
				char* _tmp2_;
				_tmp2_ = NULL;
#line 303 "ImportPage.vala"
				reason = (_tmp2_ = g_strdup ("The camera is busy."), reason = (g_free (reason), NULL), _tmp2_);
#line 304 "ImportPage.vala"
				break;
#line 1392 "ImportPage.c"
			}
			default:
			{
				char* _tmp3_;
				_tmp3_ = NULL;
#line 308 "ImportPage.vala"
				reason = (_tmp3_ = g_strdup ("The camera is unavailable at this time."), reason = (g_free (reason), NULL), _tmp3_);
#line 309 "ImportPage.vala"
				break;
#line 1402 "ImportPage.c"
			}
		}
#line 820 "glib-2.0.vapi"
		_tmp4_ = NULL;
#line 312 "ImportPage.vala"
		app_window_error_message (_tmp4_ = g_strdup_printf ("Unable to unmount camera.  %s  Please try again.", reason));
#line 1409 "ImportPage.c"
		_tmp4_ = (g_free (_tmp4_), NULL);
		reason = (g_free (reason), NULL);
	}
}


#line 316 "ImportPage.vala"
ImportPageRefreshResult import_page_refresh_camera (ImportPage* self) {
#line 1418 "ImportPage.c"
	ImportPageRefreshResult result;
	char* _tmp0_;
	GeeArrayList* file_list;
	gulong total_bytes;
	gulong total_preview_bytes;
	CameraStorageInformation* sifs;
	gint count;
	int res;
#line 316 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), 0);
#line 317 "ImportPage.vala"
	if (self->priv->busy) {
#line 1431 "ImportPage.c"
		result = IMPORT_PAGE_REFRESH_RESULT_BUSY;
		return result;
	}
#line 320 "ImportPage.vala"
	self->priv->refreshed = FALSE;
#line 1437 "ImportPage.c"
	_tmp0_ = NULL;
#line 322 "ImportPage.vala"
	self->priv->refresh_error = (_tmp0_ = NULL, self->priv->refresh_error = (g_free (self->priv->refresh_error), NULL), _tmp0_);
#line 323 "ImportPage.vala"
	self->priv->refresh_result = gp_camera_init (self->priv->camera, self->priv->null_context->context);
#line 324 "ImportPage.vala"
	if (self->priv->refresh_result != GP_OK) {
#line 1445 "ImportPage.c"
		ImportPageRefreshResult _tmp1_;
		_tmp1_ = 0;
#line 325 "ImportPage.vala"
		if (self->priv->refresh_result == GP_ERROR_IO_LOCK) {
#line 325 "ImportPage.vala"
			_tmp1_ = IMPORT_PAGE_REFRESH_RESULT_LOCKED;
#line 1452 "ImportPage.c"
		} else {
#line 325 "ImportPage.vala"
			_tmp1_ = IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR;
#line 1456 "ImportPage.c"
		}
		result = _tmp1_;
		return result;
	}
#line 327 "ImportPage.vala"
	self->priv->busy = TRUE;
#line 329 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_selected_button), "sensitive", FALSE, NULL);
#line 330 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_all_button), "sensitive", FALSE, NULL);
#line 1467 "ImportPage.c"
	file_list = gee_array_list_new (IMPORT_PAGE_TYPE_IMPORT_FILE, (GBoxedCopyFunc) import_page_import_file_ref, import_page_import_file_unref, g_direct_equal);
	total_bytes = (gulong) 0;
	total_preview_bytes = (gulong) 0;
#line 336 "ImportPage.vala"
	gtk_progress_bar_set_text (self->priv->progress_bar, "Fetching photo information");
#line 337 "ImportPage.vala"
	gtk_progress_bar_set_fraction (self->priv->progress_bar, 0.0);
#line 338 "ImportPage.vala"
	gtk_progress_bar_set_pulse_step (self->priv->progress_bar, 0.01);
#line 339 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->progress_bar), "visible", TRUE, NULL);
#line 1479 "ImportPage.c"
	sifs = NULL;
	count = 0;
#line 343 "ImportPage.vala"
	self->priv->refresh_result = gp_camera_get_storageinfo (self->priv->camera, &sifs, &count, self->priv->null_context->context);
#line 344 "ImportPage.vala"
	if (self->priv->refresh_result == GP_OK) {
#line 345 "ImportPage.vala"
		checkerboard_page_remove_all (CHECKERBOARD_PAGE (self));
#line 346 "ImportPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 1490 "ImportPage.c"
		{
			gint fsid;
			fsid = 0;
			{
				gboolean _tmp2_;
				_tmp2_ = TRUE;
#line 348 "ImportPage.vala"
				while (TRUE) {
#line 348 "ImportPage.vala"
					if (!_tmp2_) {
#line 348 "ImportPage.vala"
						fsid++;
#line 1503 "ImportPage.c"
					}
#line 348 "ImportPage.vala"
					_tmp2_ = FALSE;
#line 348 "ImportPage.vala"
					if (!(fsid < count)) {
#line 348 "ImportPage.vala"
						break;
#line 1511 "ImportPage.c"
					}
#line 349 "ImportPage.vala"
					if (!import_page_enumerate_files (self, fsid, "/", file_list, &total_bytes, &total_preview_bytes)) {
#line 350 "ImportPage.vala"
						break;
#line 1517 "ImportPage.c"
					}
				}
			}
		}
	}
#line 354 "ImportPage.vala"
	import_page_load_previews (self, file_list, total_preview_bytes);
#line 356 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->progress_bar), "visible", FALSE, NULL);
#line 357 "ImportPage.vala"
	gtk_progress_bar_set_text (self->priv->progress_bar, "");
#line 358 "ImportPage.vala"
	gtk_progress_bar_set_fraction (self->priv->progress_bar, 0.0);
#line 1531 "ImportPage.c"
	res = gp_camera_exit (self->priv->camera, self->priv->null_context->context);
#line 361 "ImportPage.vala"
	if (res != GP_OK) {
#line 1535 "ImportPage.c"
		/* log but don't fail*/
#line 363 "ImportPage.vala"
		g_message ("ImportPage.vala:363: Unable to unlock camera: %s (%d)", gp_port_result_as_string (res), (gint) res);
#line 1539 "ImportPage.c"
	}
#line 366 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_selected_button), "sensitive", checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0, NULL);
#line 367 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_all_button), "sensitive", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0, NULL);
#line 369 "ImportPage.vala"
	self->priv->busy = FALSE;
#line 371 "ImportPage.vala"
	if (self->priv->refresh_result != GP_OK) {
#line 1549 "ImportPage.c"
		ImportPageRefreshResult _tmp3_;
#line 372 "ImportPage.vala"
		self->priv->refreshed = FALSE;
#line 1553 "ImportPage.c"
		/* show 'em all or show none*/
#line 375 "ImportPage.vala"
		checkerboard_page_remove_all (CHECKERBOARD_PAGE (self));
#line 376 "ImportPage.vala"
		checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 1559 "ImportPage.c"
		_tmp3_ = 0;
#line 378 "ImportPage.vala"
		if (self->priv->refresh_result == GP_ERROR_IO_LOCK) {
#line 378 "ImportPage.vala"
			_tmp3_ = IMPORT_PAGE_REFRESH_RESULT_LOCKED;
#line 1565 "ImportPage.c"
		} else {
#line 378 "ImportPage.vala"
			_tmp3_ = IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR;
#line 1569 "ImportPage.c"
		}
		result = _tmp3_;
		(file_list == NULL) ? NULL : (file_list = (g_object_unref (file_list), NULL));
		return result;
	}
#line 381 "ImportPage.vala"
	self->priv->refreshed = TRUE;
#line 1577 "ImportPage.c"
	result = IMPORT_PAGE_REFRESH_RESULT_OK;
	(file_list == NULL) ? NULL : (file_list = (g_object_unref (file_list), NULL));
	return result;
}


/* Need to do this because some phones (iPhone, in particular) changes the name of their filesystem
 between each mount*/
#line 388 "ImportPage.vala"
static char* import_page_get_fs_basedir (ImportPage* self, gint fsid) {
#line 1588 "ImportPage.c"
	char* result;
	CameraStorageInformation* sifs;
	gint count;
	int res;
	CameraStorageInformation* ifs;
	const char* _tmp0_;
	const char* _tmp1_;
#line 388 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), NULL);
#line 1598 "ImportPage.c"
	sifs = NULL;
	count = 0;
	res = gp_camera_get_storageinfo (self->priv->camera, &sifs, &count, self->priv->null_context->context);
#line 392 "ImportPage.vala"
	if (res != GP_OK) {
#line 1604 "ImportPage.c"
		result = NULL;
		return result;
	}
#line 395 "ImportPage.vala"
	if (fsid >= count) {
#line 1610 "ImportPage.c"
		result = NULL;
		return result;
	}
	ifs = sifs + fsid;
	_tmp0_ = NULL;
#line 400 "ImportPage.vala"
	if (((*ifs).fields & GP_STORAGEINFO_BASE) != 0) {
#line 400 "ImportPage.vala"
		_tmp0_ = (*ifs).basedir;
#line 1620 "ImportPage.c"
	} else {
#line 400 "ImportPage.vala"
		_tmp0_ = "/";
#line 1624 "ImportPage.c"
	}
#line 400 "ImportPage.vala"
	_tmp1_ = NULL;
#line 1628 "ImportPage.c"
	result = (_tmp1_ = _tmp0_, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	return result;
}


#line 403 "ImportPage.vala"
static char* import_page_get_fulldir (ImportPage* self, gint fsid, const char* dir) {
#line 1636 "ImportPage.c"
	char* result;
	char* basedir;
	gboolean _tmp0_;
#line 403 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), NULL);
#line 403 "ImportPage.vala"
	g_return_val_if_fail (dir != NULL, NULL);
#line 1644 "ImportPage.c"
	basedir = import_page_get_fs_basedir (self, fsid);
#line 405 "ImportPage.vala"
	if (basedir == NULL) {
#line 406 "ImportPage.vala"
		g_debug ("ImportPage.vala:406: Unable to find base directory for fsid %d", fsid);
#line 1650 "ImportPage.c"
		result = NULL;
		basedir = (g_free (basedir), NULL);
		return result;
	}
	_tmp0_ = FALSE;
#line 411 "ImportPage.vala"
	if (!g_str_has_suffix (basedir, "/")) {
#line 411 "ImportPage.vala"
		_tmp0_ = !g_str_has_prefix (dir, "/");
#line 1660 "ImportPage.c"
	} else {
#line 411 "ImportPage.vala"
		_tmp0_ = FALSE;
#line 1664 "ImportPage.c"
	}
#line 411 "ImportPage.vala"
	if (_tmp0_) {
#line 1668 "ImportPage.c"
		char* _tmp1_;
		_tmp1_ = NULL;
#line 412 "ImportPage.vala"
		basedir = (_tmp1_ = g_strconcat (basedir, "/", NULL), basedir = (g_free (basedir), NULL), _tmp1_);
#line 1673 "ImportPage.c"
	}
	result = g_strconcat (basedir, dir, NULL);
	basedir = (g_free (basedir), NULL);
	return result;
}


#line 417 "ImportPage.vala"
static gboolean import_page_enumerate_files (ImportPage* self, gint fsid, const char* dir, GeeArrayList* file_list, gulong* total_bytes, gulong* total_preview_bytes) {
#line 1683 "ImportPage.c"
	gboolean result;
	GError * _inner_error_;
	char* fulldir;
	CameraList* files;
	CameraList* _tmp2_;
	int _tmp1_;
	CameraList* _tmp0_;
	CameraList* folders;
	CameraList* _tmp16_;
	int _tmp15_;
	CameraList* _tmp14_;
#line 417 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_PAGE (self), FALSE);
#line 417 "ImportPage.vala"
	g_return_val_if_fail (dir != NULL, FALSE);
#line 417 "ImportPage.vala"
	g_return_val_if_fail (GEE_IS_ARRAY_LIST (file_list), FALSE);
#line 1701 "ImportPage.c"
	_inner_error_ = NULL;
	fulldir = import_page_get_fulldir (self, fsid, dir);
#line 420 "ImportPage.vala"
	if (fulldir == NULL) {
#line 1706 "ImportPage.c"
		result = FALSE;
		fulldir = (g_free (fulldir), NULL);
		return result;
	}
	files = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 424 "ImportPage.vala"
	self->priv->refresh_result = (_tmp1_ = gp_list_new (&_tmp0_), files = (_tmp2_ = _tmp0_, (files == NULL) ? NULL : (files = (gp_list_unref (files), NULL)), _tmp2_), _tmp1_);
#line 425 "ImportPage.vala"
	if (self->priv->refresh_result != GP_OK) {
#line 1718 "ImportPage.c"
		result = FALSE;
		fulldir = (g_free (fulldir), NULL);
		(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
		return result;
	}
#line 428 "ImportPage.vala"
	self->priv->refresh_result = gp_camera_folder_list_files (self->priv->camera, fulldir, files, self->priv->null_context->context);
#line 429 "ImportPage.vala"
	if (self->priv->refresh_result != GP_OK) {
#line 1728 "ImportPage.c"
		result = FALSE;
		fulldir = (g_free (fulldir), NULL);
		(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
		return result;
	}
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp3_;
			_tmp3_ = TRUE;
#line 432 "ImportPage.vala"
			while (TRUE) {
#line 1742 "ImportPage.c"
				char* filename;
				const char* _tmp7_;
				char* _tmp6_;
				int _tmp5_;
				const char* _tmp4_;
#line 432 "ImportPage.vala"
				if (!_tmp3_) {
#line 432 "ImportPage.vala"
					ctr++;
#line 1752 "ImportPage.c"
				}
#line 432 "ImportPage.vala"
				_tmp3_ = FALSE;
#line 432 "ImportPage.vala"
				if (!(ctr < gp_list_count (files))) {
#line 432 "ImportPage.vala"
					break;
#line 1760 "ImportPage.c"
				}
				filename = NULL;
#line 434 "ImportPage.vala"
				_tmp7_ = NULL;
#line 1765 "ImportPage.c"
				_tmp6_ = NULL;
				_tmp4_ = NULL;
#line 434 "ImportPage.vala"
				self->priv->refresh_result = (_tmp5_ = gp_list_get_name (files, ctr, &_tmp4_), filename = (_tmp6_ = (_tmp7_ = _tmp4_, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), filename = (g_free (filename), NULL), _tmp6_), _tmp5_);
#line 435 "ImportPage.vala"
				if (self->priv->refresh_result != GP_OK) {
#line 1772 "ImportPage.c"
					result = FALSE;
					filename = (g_free (filename), NULL);
					fulldir = (g_free (fulldir), NULL);
					(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
					return result;
				}
				{
					CameraFileInfo info;
					CameraFileInfo _tmp9_;
					CameraFileInfo _tmp8_;
					gulong preview_size;
					ImportPageImportFile* import_file;
					gulong _tmp11_;
#line 440 "ImportPage.vala"
					gp_get_info (self->priv->null_context->context, self->priv->camera, fulldir, filename, &_tmp8_, &_inner_error_);
#line 440 "ImportPage.vala"
					info = (_tmp9_ = _tmp8_,  (&info), _tmp9_);
#line 1790 "ImportPage.c"
					if (_inner_error_ != NULL) {
						 (&info);
						goto __catch26_g_error;
						goto __finally26;
					}
					/* at this point, only interested in JPEG files
					 TODO: Increase file format support, for TIFF and RAW at least*/
#line 444 "ImportPage.vala"
					if ((info.file.fields & GP_FILE_INFO_TYPE) == 0) {
#line 445 "ImportPage.vala"
						g_message ("ImportPage.vala:445: Skipping %s/%s: No file (file=%02Xh)", fulldir, filename, (guint) info.file.fields);
#line 1802 "ImportPage.c"
						 (&info);
						filename = (g_free (filename), NULL);
#line 448 "ImportPage.vala"
						continue;
#line 1807 "ImportPage.c"
					}
#line 451 "ImportPage.vala"
					if (_vala_strcmp0 (info.file.type, GP_MIME_JPEG) != 0) {
#line 452 "ImportPage.vala"
						g_message ("ImportPage.vala:452: Skipping %s/%s: Not a JPEG (%s)", fulldir, filename, info.file.type);
#line 1813 "ImportPage.c"
						 (&info);
						filename = (g_free (filename), NULL);
#line 454 "ImportPage.vala"
						continue;
#line 1818 "ImportPage.c"
					}
					preview_size = info.preview.size;
					/* skip preview if it isn't JPEG
					 TODO: Accept previews if of any type recognized by Gdk.Pixbuf*/
#line 461 "ImportPage.vala"
					if (preview_size != 0) {
#line 1825 "ImportPage.c"
						gboolean _tmp10_;
						_tmp10_ = FALSE;
#line 462 "ImportPage.vala"
						if ((info.preview.fields & GP_FILE_INFO_TYPE) != 0) {
#line 463 "ImportPage.vala"
							_tmp10_ = _vala_strcmp0 (info.preview.type, GP_MIME_JPEG) != 0;
#line 1832 "ImportPage.c"
						} else {
#line 462 "ImportPage.vala"
							_tmp10_ = FALSE;
#line 1836 "ImportPage.c"
						}
#line 462 "ImportPage.vala"
						if (_tmp10_) {
#line 464 "ImportPage.vala"
							g_message ("ImportPage.vala:464: Not previewing %s/%s: Not a JPEG preview (%s)", fulldir, filename, info.preview.type);
#line 467 "ImportPage.vala"
							preview_size = (gulong) 0;
#line 1844 "ImportPage.c"
						}
					}
					import_file = import_page_import_file_new (fsid, dir, filename, info.file.size, preview_size);
#line 474 "ImportPage.vala"
					gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (file_list), import_file);
#line 475 "ImportPage.vala"
					(*total_bytes) = (*total_bytes) + info.file.size;
#line 1852 "ImportPage.c"
					_tmp11_ = 0UL;
#line 476 "ImportPage.vala"
					if (preview_size != 0) {
#line 476 "ImportPage.vala"
						_tmp11_ = preview_size;
#line 1858 "ImportPage.c"
					} else {
#line 476 "ImportPage.vala"
						_tmp11_ = info.file.size;
#line 1862 "ImportPage.c"
					}
#line 476 "ImportPage.vala"
					(*total_preview_bytes) = (*total_preview_bytes) + _tmp11_;
#line 478 "ImportPage.vala"
					gtk_progress_bar_pulse (self->priv->progress_bar);
#line 1868 "ImportPage.c"
					/* spin the event loop so the UI doesn't freeze*/
#line 481 "ImportPage.vala"
					if (!spin_event_loop ()) {
#line 1872 "ImportPage.c"
						result = FALSE;
						 (&info);
						(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
						filename = (g_free (filename), NULL);
						fulldir = (g_free (fulldir), NULL);
						(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
						return result;
					}
					 (&info);
					(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
				}
				goto __finally26;
				__catch26_g_error:
				{
					GError * err;
					err = _inner_error_;
					_inner_error_ = NULL;
					{
						char* _tmp13_;
						const char* _tmp12_;
						_tmp13_ = NULL;
#line 484 "ImportPage.vala"
						_tmp12_ = NULL;
#line 484 "ImportPage.vala"
						self->priv->refresh_error = (_tmp13_ = (_tmp12_ = err->message, (_tmp12_ == NULL) ? NULL : g_strdup (_tmp12_)), self->priv->refresh_error = (g_free (self->priv->refresh_error), NULL), _tmp13_);
#line 1898 "ImportPage.c"
						result = FALSE;
						(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
						filename = (g_free (filename), NULL);
						fulldir = (g_free (fulldir), NULL);
						(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
						return result;
					}
				}
				__finally26:
				if (_inner_error_ != NULL) {
					filename = (g_free (filename), NULL);
					fulldir = (g_free (fulldir), NULL);
					(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return FALSE;
				}
				filename = (g_free (filename), NULL);
			}
		}
	}
	folders = NULL;
	_tmp16_ = NULL;
	_tmp14_ = NULL;
#line 491 "ImportPage.vala"
	self->priv->refresh_result = (_tmp15_ = gp_list_new (&_tmp14_), folders = (_tmp16_ = _tmp14_, (folders == NULL) ? NULL : (folders = (gp_list_unref (folders), NULL)), _tmp16_), _tmp15_);
#line 492 "ImportPage.vala"
	if (self->priv->refresh_result != GP_OK) {
#line 1927 "ImportPage.c"
		result = FALSE;
		fulldir = (g_free (fulldir), NULL);
		(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
		(folders == NULL) ? NULL : (folders = (gp_list_unref (folders), NULL));
		return result;
	}
#line 495 "ImportPage.vala"
	self->priv->refresh_result = gp_camera_folder_list_folders (self->priv->camera, fulldir, folders, self->priv->null_context->context);
#line 496 "ImportPage.vala"
	if (self->priv->refresh_result != GP_OK) {
#line 1938 "ImportPage.c"
		result = FALSE;
		fulldir = (g_free (fulldir), NULL);
		(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
		(folders == NULL) ? NULL : (folders = (gp_list_unref (folders), NULL));
		return result;
	}
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp17_;
			_tmp17_ = TRUE;
#line 499 "ImportPage.vala"
			while (TRUE) {
#line 1953 "ImportPage.c"
				char* subdir;
				const char* _tmp21_;
				char* _tmp20_;
				int _tmp19_;
				const char* _tmp18_;
				char* recurse_dir;
				gboolean _tmp22_;
#line 499 "ImportPage.vala"
				if (!_tmp17_) {
#line 499 "ImportPage.vala"
					ctr++;
#line 1965 "ImportPage.c"
				}
#line 499 "ImportPage.vala"
				_tmp17_ = FALSE;
#line 499 "ImportPage.vala"
				if (!(ctr < gp_list_count (folders))) {
#line 499 "ImportPage.vala"
					break;
#line 1973 "ImportPage.c"
				}
				subdir = NULL;
#line 501 "ImportPage.vala"
				_tmp21_ = NULL;
#line 1978 "ImportPage.c"
				_tmp20_ = NULL;
				_tmp18_ = NULL;
#line 501 "ImportPage.vala"
				self->priv->refresh_result = (_tmp19_ = gp_list_get_name (folders, ctr, &_tmp18_), subdir = (_tmp20_ = (_tmp21_ = _tmp18_, (_tmp21_ == NULL) ? NULL : g_strdup (_tmp21_)), subdir = (g_free (subdir), NULL), _tmp20_), _tmp19_);
#line 502 "ImportPage.vala"
				if (self->priv->refresh_result != GP_OK) {
#line 1985 "ImportPage.c"
					result = FALSE;
					subdir = (g_free (subdir), NULL);
					fulldir = (g_free (fulldir), NULL);
					(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
					(folders == NULL) ? NULL : (folders = (gp_list_unref (folders), NULL));
					return result;
				}
				recurse_dir = NULL;
				_tmp22_ = FALSE;
#line 506 "ImportPage.vala"
				if (!g_str_has_suffix (dir, "/")) {
#line 506 "ImportPage.vala"
					_tmp22_ = !g_str_has_prefix (subdir, "/");
#line 1999 "ImportPage.c"
				} else {
#line 506 "ImportPage.vala"
					_tmp22_ = FALSE;
#line 2003 "ImportPage.c"
				}
#line 506 "ImportPage.vala"
				if (_tmp22_) {
#line 2007 "ImportPage.c"
					char* _tmp24_;
					char* _tmp23_;
					_tmp24_ = NULL;
					_tmp23_ = NULL;
#line 507 "ImportPage.vala"
					recurse_dir = (_tmp24_ = g_strconcat (_tmp23_ = g_strconcat (dir, "/", NULL), subdir, NULL), recurse_dir = (g_free (recurse_dir), NULL), _tmp24_);
#line 2014 "ImportPage.c"
					_tmp23_ = (g_free (_tmp23_), NULL);
				} else {
					char* _tmp25_;
					_tmp25_ = NULL;
#line 509 "ImportPage.vala"
					recurse_dir = (_tmp25_ = g_strconcat (dir, subdir, NULL), recurse_dir = (g_free (recurse_dir), NULL), _tmp25_);
#line 2021 "ImportPage.c"
				}
#line 511 "ImportPage.vala"
				if (!import_page_enumerate_files (self, fsid, recurse_dir, file_list, &(*total_bytes), &(*total_preview_bytes))) {
#line 2025 "ImportPage.c"
					result = FALSE;
					subdir = (g_free (subdir), NULL);
					recurse_dir = (g_free (recurse_dir), NULL);
					fulldir = (g_free (fulldir), NULL);
					(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
					(folders == NULL) ? NULL : (folders = (gp_list_unref (folders), NULL));
					return result;
				}
				subdir = (g_free (subdir), NULL);
				recurse_dir = (g_free (recurse_dir), NULL);
			}
		}
	}
	result = TRUE;
	fulldir = (g_free (fulldir), NULL);
	(files == NULL) ? NULL : (files = (gp_list_unref (files), NULL));
	(folders == NULL) ? NULL : (folders = (gp_list_unref (folders), NULL));
	return result;
}


#line 518 "ImportPage.vala"
static void import_page_load_previews (ImportPage* self, GeeArrayList* file_list, gulong total_preview_bytes) {
#line 2049 "ImportPage.c"
	GError * _inner_error_;
	gulong bytes;
#line 518 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 518 "ImportPage.vala"
	g_return_if_fail (GEE_IS_ARRAY_LIST (file_list));
#line 2056 "ImportPage.c"
	_inner_error_ = NULL;
	bytes = (gulong) 0;
	{
		{
			GeeIterator* _import_file_it;
			_import_file_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (file_list));
#line 521 "ImportPage.vala"
			while (TRUE) {
#line 2065 "ImportPage.c"
				ImportPageImportFile* import_file;
				char* fulldir;
				char* _tmp0_;
				GdkPixbuf* pixbuf;
				ExifData* exif;
				gulong _tmp5_;
				ImportPreview* preview;
#line 521 "ImportPage.vala"
				if (!gee_iterator_next (_import_file_it)) {
#line 521 "ImportPage.vala"
					break;
#line 2077 "ImportPage.c"
				}
				import_file = (ImportPageImportFile*) gee_iterator_get (_import_file_it);
				fulldir = import_page_get_fulldir (self, import_file->fsid, import_file->dir);
#line 523 "ImportPage.vala"
				if (fulldir == NULL) {
#line 2083 "ImportPage.c"
					(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
					fulldir = (g_free (fulldir), NULL);
#line 524 "ImportPage.vala"
					continue;
#line 2088 "ImportPage.c"
				}
#line 820 "glib-2.0.vapi"
				_tmp0_ = NULL;
#line 526 "ImportPage.vala"
				gtk_progress_bar_set_text (self->priv->progress_bar, _tmp0_ = g_strdup_printf ("Fetching preview for %s", import_file->filename));
#line 2094 "ImportPage.c"
				_tmp0_ = (g_free (_tmp0_), NULL);
				/* if no preview, load full image for preview*/
				pixbuf = NULL;
#line 530 "ImportPage.vala"
				if (import_file->preview_size > 0) {
#line 2100 "ImportPage.c"
					GdkPixbuf* _tmp1_;
					GdkPixbuf* _tmp2_;
					_tmp1_ = gp_load_preview (self->priv->null_context->context, self->priv->camera, fulldir, import_file->filename, &_inner_error_);
					if (_inner_error_ != NULL) {
						(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
						fulldir = (g_free (fulldir), NULL);
						(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
						(_import_file_it == NULL) ? NULL : (_import_file_it = (g_object_unref (_import_file_it), NULL));
						goto __catch27_g_error;
						goto __finally27;
					}
					_tmp2_ = NULL;
#line 531 "ImportPage.vala"
					pixbuf = (_tmp2_ = _tmp1_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp2_);
#line 2115 "ImportPage.c"
				} else {
					GdkPixbuf* _tmp3_;
					GdkPixbuf* _tmp4_;
					_tmp3_ = gp_load_image (self->priv->null_context->context, self->priv->camera, fulldir, import_file->filename, &_inner_error_);
					if (_inner_error_ != NULL) {
						(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
						fulldir = (g_free (fulldir), NULL);
						(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
						(_import_file_it == NULL) ? NULL : (_import_file_it = (g_object_unref (_import_file_it), NULL));
						goto __catch27_g_error;
						goto __finally27;
					}
					_tmp4_ = NULL;
#line 534 "ImportPage.vala"
					pixbuf = (_tmp4_ = _tmp3_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp4_);
#line 2131 "ImportPage.c"
				}
				exif = gp_load_exif (self->priv->null_context->context, self->priv->camera, fulldir, import_file->filename, &_inner_error_);
				if (_inner_error_ != NULL) {
					(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
					fulldir = (g_free (fulldir), NULL);
					(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
					(_import_file_it == NULL) ? NULL : (_import_file_it = (g_object_unref (_import_file_it), NULL));
					goto __catch27_g_error;
					goto __finally27;
				}
				_tmp5_ = 0UL;
#line 540 "ImportPage.vala"
				if (import_file->preview_size != 0) {
#line 540 "ImportPage.vala"
					_tmp5_ = import_file->preview_size;
#line 2147 "ImportPage.c"
				} else {
#line 540 "ImportPage.vala"
					_tmp5_ = import_file->file_size;
#line 2151 "ImportPage.c"
				}
#line 540 "ImportPage.vala"
				bytes = bytes + _tmp5_;
#line 541 "ImportPage.vala"
				gtk_progress_bar_set_fraction (self->priv->progress_bar, ((double) bytes) / ((double) total_preview_bytes));
#line 2157 "ImportPage.c"
				preview = g_object_ref_sink (import_preview_new (pixbuf, exif, import_file->fsid, import_file->dir, import_file->filename, import_file->file_size));
#line 545 "ImportPage.vala"
				checkerboard_page_add_item (CHECKERBOARD_PAGE (self), LAYOUT_ITEM (preview));
#line 547 "ImportPage.vala"
				checkerboard_page_refresh (CHECKERBOARD_PAGE (self));
#line 2163 "ImportPage.c"
				/* spin the event loop so the UI doesn't freeze*/
#line 550 "ImportPage.vala"
				if (!spin_event_loop ()) {
#line 2167 "ImportPage.c"
					(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
					fulldir = (g_free (fulldir), NULL);
					(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
					(exif == NULL) ? NULL : (exif = (exif_data_unref (exif), NULL));
					(preview == NULL) ? NULL : (preview = (g_object_unref (preview), NULL));
#line 551 "ImportPage.vala"
					break;
#line 2175 "ImportPage.c"
				}
				(import_file == NULL) ? NULL : (import_file = (import_page_import_file_unref (import_file), NULL));
				fulldir = (g_free (fulldir), NULL);
				(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
				(exif == NULL) ? NULL : (exif = (exif_data_unref (exif), NULL));
				(preview == NULL) ? NULL : (preview = (g_object_unref (preview), NULL));
			}
			(_import_file_it == NULL) ? NULL : (_import_file_it = (g_object_unref (_import_file_it), NULL));
		}
	}
	goto __finally27;
	__catch27_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp6_;
#line 820 "glib-2.0.vapi"
			_tmp6_ = NULL;
#line 554 "ImportPage.vala"
			app_window_error_message (_tmp6_ = g_strdup_printf ("Error while fetching previews from %s: %s", self->priv->camera_name, err->message));
#line 2198 "ImportPage.c"
			_tmp6_ = (g_free (_tmp6_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally27:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 559 "ImportPage.vala"
static void import_page_on_file_menu (ImportPage* self) {
#line 2214 "ImportPage.c"
	gboolean _tmp0_;
	gboolean _tmp1_;
#line 559 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 2219 "ImportPage.c"
	_tmp0_ = FALSE;
#line 560 "ImportPage.vala"
	if (!self->priv->busy) {
#line 560 "ImportPage.vala"
		_tmp0_ = checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0;
#line 2225 "ImportPage.c"
	} else {
#line 560 "ImportPage.vala"
		_tmp0_ = FALSE;
#line 2229 "ImportPage.c"
	}
#line 560 "ImportPage.vala"
	page_set_item_sensitive (PAGE (self), "/ImportMenuBar/FileMenu/ImportSelected", _tmp0_);
#line 2233 "ImportPage.c"
	_tmp1_ = FALSE;
#line 561 "ImportPage.vala"
	if (!self->priv->busy) {
#line 561 "ImportPage.vala"
		_tmp1_ = checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0;
#line 2239 "ImportPage.c"
	} else {
#line 561 "ImportPage.vala"
		_tmp1_ = FALSE;
#line 2243 "ImportPage.c"
	}
#line 561 "ImportPage.vala"
	page_set_item_sensitive (PAGE (self), "/ImportMenuBar/FileMenu/ImportAll", _tmp1_);
#line 2247 "ImportPage.c"
}


#line 564 "ImportPage.vala"
static void import_page_on_import_selected (ImportPage* self) {
#line 2253 "ImportPage.c"
	GeeIterable* _tmp0_;
#line 564 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 498 "Page.vala"
	_tmp0_ = NULL;
#line 565 "ImportPage.vala"
	import_page_import (self, _tmp0_ = checkerboard_page_get_selected (CHECKERBOARD_PAGE (self)));
#line 2261 "ImportPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 568 "ImportPage.vala"
static void import_page_on_import_all (ImportPage* self) {
#line 2268 "ImportPage.c"
	GeeIterable* _tmp0_;
#line 568 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 494 "Page.vala"
	_tmp0_ = NULL;
#line 569 "ImportPage.vala"
	import_page_import (self, _tmp0_ = checkerboard_page_get_items (CHECKERBOARD_PAGE (self)));
#line 2276 "ImportPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 572 "ImportPage.vala"
static void import_page_on_edit_menu (ImportPage* self) {
#line 2283 "ImportPage.c"
	gboolean _tmp0_;
#line 572 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 2287 "ImportPage.c"
	_tmp0_ = FALSE;
#line 573 "ImportPage.vala"
	if (!self->priv->busy) {
#line 573 "ImportPage.vala"
		_tmp0_ = checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0;
#line 2293 "ImportPage.c"
	} else {
#line 573 "ImportPage.vala"
		_tmp0_ = FALSE;
#line 2297 "ImportPage.c"
	}
#line 573 "ImportPage.vala"
	page_set_item_sensitive (PAGE (self), "/ImportMenuBar/EditMenu/SelectAll", _tmp0_);
#line 2301 "ImportPage.c"
}


#line 576 "ImportPage.vala"
static void import_page_on_select_all (ImportPage* self) {
#line 576 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 577 "ImportPage.vala"
	checkerboard_page_select_all (CHECKERBOARD_PAGE (self));
#line 2311 "ImportPage.c"
}


#line 652 "ImportPage.vala"
static void _import_page_on_import_job_failed_batch_import_import_job_failed (BatchImport* _sender, ImportResult _result_, BatchImportJob* job, GFile* file, gpointer self) {
#line 2317 "ImportPage.c"
	import_page_on_import_job_failed (self, _result_, job, file);
}


#line 664 "ImportPage.vala"
static void _import_page_close_import_batch_import_import_complete (BatchImport* _sender, const ImportID* import_id, SortedList* photos_by_date, GeeArrayList* failed, GeeArrayList* skipped, gpointer self) {
#line 2324 "ImportPage.c"
	import_page_close_import (self);
}


#line 580 "ImportPage.vala"
static void import_page_import (ImportPage* self, GeeIterable* items) {
#line 2331 "ImportPage.c"
	int res;
	gint failed;
	gulong total_bytes;
	ImportPageCameraImportComparator* _tmp1_;
	SortedList* _tmp2_;
	SortedList* jobs;
#line 580 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 580 "ImportPage.vala"
	g_return_if_fail (GEE_IS_ITERABLE (items));
#line 2342 "ImportPage.c"
	res = gp_camera_init (self->priv->camera, self->priv->null_context->context);
#line 582 "ImportPage.vala"
	if (res != GP_OK) {
#line 2346 "ImportPage.c"
		char* _tmp0_;
#line 820 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 583 "ImportPage.vala"
		app_window_error_message (_tmp0_ = g_strdup_printf ("Unable to lock camera: %s", gp_port_result_as_string (res)));
#line 2352 "ImportPage.c"
		_tmp0_ = (g_free (_tmp0_), NULL);
		return;
	}
#line 588 "ImportPage.vala"
	self->priv->busy = TRUE;
#line 589 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_selected_button), "sensitive", FALSE, NULL);
#line 590 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_all_button), "sensitive", FALSE, NULL);
#line 591 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->progress_bar), "visible", FALSE, NULL);
#line 2364 "ImportPage.c"
	failed = 0;
	total_bytes = (gulong) 0;
	_tmp1_ = NULL;
#line 595 "ImportPage.vala"
	_tmp2_ = NULL;
#line 2370 "ImportPage.c"
	jobs = (_tmp2_ = sorted_list_new (IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB, (GBoxedCopyFunc) batch_import_job_ref, batch_import_job_unref, COMPARATOR (_tmp1_ = import_page_camera_import_comparator_new ())), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (comparator_unref (_tmp1_), NULL)), _tmp2_);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (items);
#line 597 "ImportPage.vala"
		while (TRUE) {
#line 2377 "ImportPage.c"
			LayoutItem* item;
			ImportPreview* _tmp3_;
			ImportPreview* preview;
			char* basedir;
			const char* _tmp4_;
			char* dir;
			char* _tmp6_;
			gboolean collision;
			time_t _tmp8_ = {0};
			time_t _tmp7_ = {0};
			GFile* dest_file;
			time_t exposure_time = {0};
			ImportPageCameraImportJob* _tmp9_;
#line 597 "ImportPage.vala"
			if (!gee_iterator_next (_item_it)) {
#line 597 "ImportPage.vala"
				break;
#line 2395 "ImportPage.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 598 "ImportPage.vala"
			_tmp3_ = NULL;
#line 2400 "ImportPage.c"
			preview = (_tmp3_ = IMPORT_PREVIEW (item), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_));
			basedir = import_page_get_fs_basedir (self, preview->fsid);
#line 601 "ImportPage.vala"
			if (basedir == NULL) {
#line 602 "ImportPage.vala"
				g_debug ("ImportPage.vala:602: Unable to find basedir for fsid %d", preview->fsid);
#line 603 "ImportPage.vala"
				failed++;
#line 2409 "ImportPage.c"
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(preview == NULL) ? NULL : (preview = (g_object_unref (preview), NULL));
				basedir = (g_free (basedir), NULL);
#line 605 "ImportPage.vala"
				continue;
#line 2415 "ImportPage.c"
			}
#line 608 "ImportPage.vala"
			_tmp4_ = NULL;
#line 2419 "ImportPage.c"
			dir = (_tmp4_ = preview->folder, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_));
#line 609 "ImportPage.vala"
			if (!g_str_has_prefix (dir, "/")) {
#line 2423 "ImportPage.c"
				char* _tmp5_;
				_tmp5_ = NULL;
#line 610 "ImportPage.vala"
				dir = (_tmp5_ = g_strconcat ("/", dir, NULL), dir = (g_free (dir), NULL), _tmp5_);
#line 2428 "ImportPage.c"
			}
			_tmp6_ = NULL;
#line 612 "ImportPage.vala"
			dir = (_tmp6_ = g_strconcat (basedir, dir, NULL), dir = (g_free (dir), NULL), _tmp6_);
#line 2433 "ImportPage.c"
			collision = FALSE;
			dest_file = batch_import_create_library_path (preview->filename, preview->exif, (_tmp8_ = (time (&_tmp7_), _tmp7_), &_tmp8_), &collision);
#line 617 "ImportPage.vala"
			if (dest_file == NULL) {
#line 618 "ImportPage.vala"
				g_message ("ImportPage.vala:618: Unable to generate local file for %s/%s", dir, preview->filename);
#line 619 "ImportPage.vala"
				failed++;
#line 2442 "ImportPage.c"
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(preview == NULL) ? NULL : (preview = (g_object_unref (preview), NULL));
				basedir = (g_free (basedir), NULL);
				dir = (g_free (dir), NULL);
				(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
#line 621 "ImportPage.vala"
				continue;
#line 2450 "ImportPage.c"
			}
#line 625 "ImportPage.vala"
			if (!exif_get_timestamp (preview->exif, &exposure_time)) {
#line 626 "ImportPage.vala"
				exposure_time = 0;
#line 2456 "ImportPage.c"
			}
			_tmp9_ = NULL;
#line 628 "ImportPage.vala"
			sorted_list_add (jobs, _tmp9_ = import_page_camera_import_job_new (self->priv->null_context, self->priv->camera, dir, preview->filename, preview->file_size, &exposure_time, dest_file));
#line 2461 "ImportPage.c"
			(_tmp9_ == NULL) ? NULL : (_tmp9_ = (batch_import_job_unref (_tmp9_), NULL));
#line 630 "ImportPage.vala"
			total_bytes = total_bytes + preview->file_size;
#line 2465 "ImportPage.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(preview == NULL) ? NULL : (preview = (g_object_unref (preview), NULL));
			basedir = (g_free (basedir), NULL);
			dir = (g_free (dir), NULL);
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 633 "ImportPage.vala"
	if (failed > 0) {
#line 2476 "ImportPage.c"
		const char* _tmp10_;
		const char* _tmp11_;
		char* plural;
		char* _tmp12_;
		_tmp10_ = NULL;
#line 635 "ImportPage.vala"
		if (failed > 1) {
#line 635 "ImportPage.vala"
			_tmp10_ = "s";
#line 2486 "ImportPage.c"
		} else {
#line 635 "ImportPage.vala"
			_tmp10_ = "";
#line 2490 "ImportPage.c"
		}
		/* TODO: I18N*/
#line 635 "ImportPage.vala"
		_tmp11_ = NULL;
#line 2495 "ImportPage.c"
		plural = (_tmp11_ = _tmp10_, (_tmp11_ == NULL) ? NULL : g_strdup (_tmp11_));
#line 820 "glib-2.0.vapi"
		_tmp12_ = NULL;
#line 636 "ImportPage.vala"
		app_window_error_message (_tmp12_ = g_strdup_printf ("Unable to import %d photo%s from the camera due to fatal error%s.", failed, plural, plural));
#line 2501 "ImportPage.c"
		_tmp12_ = (g_free (_tmp12_), NULL);
		plural = (g_free (plural), NULL);
	}
#line 640 "ImportPage.vala"
	if (sorted_list_get_size (jobs) > 0) {
#line 2507 "ImportPage.c"
		BatchImport* batch_import;
		AppWindow* _tmp13_;
		AppWindow* _tmp14_;
		batch_import = batch_import_new (GEE_ITERABLE (jobs), self->priv->camera_name, (guint64) total_bytes);
#line 642 "ImportPage.vala"
		g_signal_connect_object (batch_import, "import-job-failed", (GCallback) _import_page_on_import_job_failed_batch_import_import_job_failed, self, 0);
#line 643 "ImportPage.vala"
		g_signal_connect_object (batch_import, "import-complete", (GCallback) _import_page_close_import_batch_import_import_complete, self, 0);
#line 446 "AppWindow.vala"
		_tmp13_ = NULL;
#line 644 "ImportPage.vala"
		app_window_enqueue_batch_import (_tmp13_ = app_window_get_instance (), batch_import);
#line 2520 "ImportPage.c"
		(_tmp13_ == NULL) ? NULL : (_tmp13_ = (g_object_unref (_tmp13_), NULL));
#line 446 "AppWindow.vala"
		_tmp14_ = NULL;
#line 645 "ImportPage.vala"
		app_window_switch_to_import_queue_page (_tmp14_ = app_window_get_instance ());
#line 2526 "ImportPage.c"
		(_tmp14_ == NULL) ? NULL : (_tmp14_ = (g_object_unref (_tmp14_), NULL));
		(batch_import == NULL) ? NULL : (batch_import = (batch_import_unref (batch_import), NULL));
	} else {
		/* camera.exit() and busy flag will be handled when the batch import completes*/
#line 648 "ImportPage.vala"
		import_page_close_import (self);
#line 2533 "ImportPage.c"
	}
	(jobs == NULL) ? NULL : (jobs = (g_object_unref (jobs), NULL));
}


#line 652 "ImportPage.vala"
static void import_page_on_import_job_failed (ImportPage* self, ImportResult _result_, BatchImportJob* job, GFile* file) {
#line 2541 "ImportPage.c"
	GError * _inner_error_;
	gboolean _tmp0_;
#line 652 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 652 "ImportPage.vala"
	g_return_if_fail (IS_BATCH_IMPORT_JOB (job));
#line 652 "ImportPage.vala"
	g_return_if_fail ((file == NULL) || G_IS_FILE (file));
#line 2550 "ImportPage.c"
	_inner_error_ = NULL;
	_tmp0_ = FALSE;
#line 653 "ImportPage.vala"
	if (file == NULL) {
#line 653 "ImportPage.vala"
		_tmp0_ = TRUE;
#line 2557 "ImportPage.c"
	} else {
#line 653 "ImportPage.vala"
		_tmp0_ = _result_ == IMPORT_RESULT_SUCCESS;
#line 2561 "ImportPage.c"
	}
#line 653 "ImportPage.vala"
	if (_tmp0_) {
#line 2565 "ImportPage.c"
		return;
	}
	{
#line 658 "ImportPage.vala"
		g_file_delete (file, NULL, &_inner_error_);
#line 2571 "ImportPage.c"
		if (_inner_error_ != NULL) {
			goto __catch28_g_error;
			goto __finally28;
		}
	}
	goto __finally28;
	__catch28_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp1_;
#line 952 "gio-2.0.vapi"
			_tmp1_ = NULL;
#line 660 "ImportPage.vala"
			g_message ("ImportPage.vala:660: Unable to delete downloaded file %s: %s", _tmp1_ = g_file_get_path (file), err->message);
#line 2589 "ImportPage.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally28:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 664 "ImportPage.vala"
static void import_page_close_import (ImportPage* self) {
#line 2605 "ImportPage.c"
	int res;
#line 664 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_PAGE (self));
#line 665 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_selected_button), "sensitive", checkerboard_page_get_selected_count (CHECKERBOARD_PAGE (self)) > 0, NULL);
#line 666 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->import_all_button), "sensitive", checkerboard_page_get_count (CHECKERBOARD_PAGE (self)) > 0, NULL);
#line 2613 "ImportPage.c"
	res = gp_camera_exit (self->priv->camera, self->priv->null_context->context);
#line 669 "ImportPage.vala"
	if (res != GP_OK) {
#line 2617 "ImportPage.c"
		/* log but don't fail*/
#line 671 "ImportPage.vala"
		g_message ("ImportPage.vala:671: Unable to unlock camera: %s (%d)", gp_port_result_as_string (res), (gint) res);
#line 2621 "ImportPage.c"
	}
#line 674 "ImportPage.vala"
	self->priv->busy = FALSE;
#line 2625 "ImportPage.c"
}


#line 54 "ImportPage.vala"
static ImportPageImportFile* import_page_import_file_construct (GType object_type, gint fsid, const char* dir, const char* filename, gulong file_size, gulong preview_size) {
#line 2631 "ImportPage.c"
	ImportPageImportFile* self;
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp3_;
	const char* _tmp2_;
#line 54 "ImportPage.vala"
	g_return_val_if_fail (dir != NULL, NULL);
#line 54 "ImportPage.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 2641 "ImportPage.c"
	self = (ImportPageImportFile*) g_type_create_instance (object_type);
#line 55 "ImportPage.vala"
	self->fsid = fsid;
#line 2645 "ImportPage.c"
	_tmp1_ = NULL;
#line 56 "ImportPage.vala"
	_tmp0_ = NULL;
#line 56 "ImportPage.vala"
	self->dir = (_tmp1_ = (_tmp0_ = dir, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->dir = (g_free (self->dir), NULL), _tmp1_);
#line 2651 "ImportPage.c"
	_tmp3_ = NULL;
#line 57 "ImportPage.vala"
	_tmp2_ = NULL;
#line 57 "ImportPage.vala"
	self->filename = (_tmp3_ = (_tmp2_ = filename, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->filename = (g_free (self->filename), NULL), _tmp3_);
#line 58 "ImportPage.vala"
	self->file_size = file_size;
#line 59 "ImportPage.vala"
	self->preview_size = preview_size;
#line 2661 "ImportPage.c"
	return self;
}


#line 54 "ImportPage.vala"
static ImportPageImportFile* import_page_import_file_new (gint fsid, const char* dir, const char* filename, gulong file_size, gulong preview_size) {
#line 54 "ImportPage.vala"
	return import_page_import_file_construct (IMPORT_PAGE_TYPE_IMPORT_FILE, fsid, dir, filename, file_size, preview_size);
#line 2670 "ImportPage.c"
}


static void import_page_value_import_file_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void import_page_value_import_file_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		import_page_import_file_unref (value->data[0].v_pointer);
	}
}


static void import_page_value_import_file_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = import_page_import_file_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer import_page_value_import_file_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* import_page_value_import_file_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ImportPageImportFile* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = import_page_import_file_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* import_page_value_import_file_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ImportPageImportFile** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = import_page_import_file_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* import_page_param_spec_import_file (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ImportPageParamSpecImportFile* spec;
	g_return_val_if_fail (g_type_is_a (object_type, IMPORT_PAGE_TYPE_IMPORT_FILE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer import_page_value_get_import_file (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, IMPORT_PAGE_TYPE_IMPORT_FILE), NULL);
	return value->data[0].v_pointer;
}


static void import_page_value_set_import_file (GValue* value, gpointer v_object) {
	ImportPageImportFile* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, IMPORT_PAGE_TYPE_IMPORT_FILE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, IMPORT_PAGE_TYPE_IMPORT_FILE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		import_page_import_file_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		import_page_import_file_unref (old);
	}
}


static void import_page_import_file_class_init (ImportPageImportFileClass * klass) {
	import_page_import_file_parent_class = g_type_class_peek_parent (klass);
	IMPORT_PAGE_IMPORT_FILE_CLASS (klass)->finalize = import_page_import_file_finalize;
}


static void import_page_import_file_instance_init (ImportPageImportFile * self) {
	self->ref_count = 1;
}


static void import_page_import_file_finalize (ImportPageImportFile* obj) {
	ImportPageImportFile * self;
	self = IMPORT_PAGE_IMPORT_FILE (obj);
	self->dir = (g_free (self->dir), NULL);
	self->filename = (g_free (self->filename), NULL);
}


static GType import_page_import_file_get_type (void) {
	static GType import_page_import_file_type_id = 0;
	if (import_page_import_file_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { import_page_value_import_file_init, import_page_value_import_file_free_value, import_page_value_import_file_copy_value, import_page_value_import_file_peek_pointer, "p", import_page_value_import_file_collect_value, "p", import_page_value_import_file_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ImportPageImportFileClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) import_page_import_file_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImportPageImportFile), 0, (GInstanceInitFunc) import_page_import_file_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		import_page_import_file_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ImportPageImportFile", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return import_page_import_file_type_id;
}


static gpointer import_page_import_file_ref (gpointer instance) {
	ImportPageImportFile* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void import_page_import_file_unref (gpointer instance) {
	ImportPageImportFile* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		IMPORT_PAGE_IMPORT_FILE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 72 "ImportPage.vala"
static ImportPageCameraImportJob* import_page_camera_import_job_construct (GType object_type, GPContextWrapper* context, Camera* camera, const char* dir, const char* filename, gulong file_size, const time_t* exposure_time, GFile* dest_file) {
#line 2818 "ImportPage.c"
	ImportPageCameraImportJob* self;
	GPContextWrapper* _tmp1_;
	GPContextWrapper* _tmp0_;
	Camera* _tmp3_;
	Camera* _tmp2_;
	char* _tmp5_;
	const char* _tmp4_;
	char* _tmp7_;
	const char* _tmp6_;
	GFile* _tmp9_;
	GFile* _tmp8_;
#line 72 "ImportPage.vala"
	g_return_val_if_fail (GP_IS_CONTEXT_WRAPPER (context), NULL);
#line 72 "ImportPage.vala"
	g_return_val_if_fail (camera != NULL, NULL);
#line 72 "ImportPage.vala"
	g_return_val_if_fail (dir != NULL, NULL);
#line 72 "ImportPage.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 72 "ImportPage.vala"
	g_return_val_if_fail (G_IS_FILE (dest_file), NULL);
#line 2840 "ImportPage.c"
	self = (ImportPageCameraImportJob*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 74 "ImportPage.vala"
	_tmp0_ = NULL;
#line 74 "ImportPage.vala"
	self->priv->context = (_tmp1_ = (_tmp0_ = context, (_tmp0_ == NULL) ? NULL : gp_context_wrapper_ref (_tmp0_)), (self->priv->context == NULL) ? NULL : (self->priv->context = (gp_context_wrapper_unref (self->priv->context), NULL)), _tmp1_);
#line 2847 "ImportPage.c"
	_tmp3_ = NULL;
#line 75 "ImportPage.vala"
	_tmp2_ = NULL;
#line 75 "ImportPage.vala"
	self->priv->camera = (_tmp3_ = (_tmp2_ = camera, (_tmp2_ == NULL) ? NULL : GPHOTO_REF_CAMERA (_tmp2_)), (self->priv->camera == NULL) ? NULL : (self->priv->camera = (gp_camera_unref (self->priv->camera), NULL)), _tmp3_);
#line 2853 "ImportPage.c"
	_tmp5_ = NULL;
#line 76 "ImportPage.vala"
	_tmp4_ = NULL;
#line 76 "ImportPage.vala"
	self->priv->dir = (_tmp5_ = (_tmp4_ = dir, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), self->priv->dir = (g_free (self->priv->dir), NULL), _tmp5_);
#line 2859 "ImportPage.c"
	_tmp7_ = NULL;
#line 77 "ImportPage.vala"
	_tmp6_ = NULL;
#line 77 "ImportPage.vala"
	self->priv->filename = (_tmp7_ = (_tmp6_ = filename, (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), self->priv->filename = (g_free (self->priv->filename), NULL), _tmp7_);
#line 78 "ImportPage.vala"
	self->priv->file_size = file_size;
#line 79 "ImportPage.vala"
	self->priv->exposure_time = (*exposure_time);
#line 2869 "ImportPage.c"
	_tmp9_ = NULL;
#line 80 "ImportPage.vala"
	_tmp8_ = NULL;
#line 80 "ImportPage.vala"
	self->priv->dest_file = (_tmp9_ = (_tmp8_ = dest_file, (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_)), (self->priv->dest_file == NULL) ? NULL : (self->priv->dest_file = (g_object_unref (self->priv->dest_file), NULL)), _tmp9_);
#line 2875 "ImportPage.c"
	return self;
}


#line 72 "ImportPage.vala"
static ImportPageCameraImportJob* import_page_camera_import_job_new (GPContextWrapper* context, Camera* camera, const char* dir, const char* filename, gulong file_size, const time_t* exposure_time, GFile* dest_file) {
#line 72 "ImportPage.vala"
	return import_page_camera_import_job_construct (IMPORT_PAGE_TYPE_CAMERA_IMPORT_JOB, context, camera, dir, filename, file_size, exposure_time, dest_file);
#line 2884 "ImportPage.c"
}


#line 83 "ImportPage.vala"
static time_t import_page_camera_import_job_get_exposure_time (ImportPageCameraImportJob* self) {
#line 2890 "ImportPage.c"
	time_t result;
	result = self->priv->exposure_time;
	return result;
}


#line 87 "ImportPage.vala"
static char* import_page_camera_import_job_real_get_identifier (BatchImportJob* base) {
#line 2899 "ImportPage.c"
	ImportPageCameraImportJob * self;
	char* result;
	const char* _tmp0_;
	self = IMPORT_PAGE_CAMERA_IMPORT_JOB (base);
#line 88 "ImportPage.vala"
	_tmp0_ = NULL;
#line 2906 "ImportPage.c"
	result = (_tmp0_ = self->priv->filename, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 91 "ImportPage.vala"
static gboolean import_page_camera_import_job_real_prepare (BatchImportJob* base, GFile** file_to_import, gboolean* copy_to_library) {
#line 2914 "ImportPage.c"
	ImportPageCameraImportJob * self;
	gboolean result;
	GError * _inner_error_;
	GFile* _tmp1_;
	GFile* _tmp0_;
	self = IMPORT_PAGE_CAMERA_IMPORT_JOB (base);
	if (file_to_import != NULL) {
		*file_to_import = NULL;
	}
	_inner_error_ = NULL;
	{
#line 93 "ImportPage.vala"
		gp_save_image (self->priv->context->context, self->priv->camera, self->priv->dir, self->priv->filename, self->priv->dest_file, &_inner_error_);
#line 2928 "ImportPage.c"
		if (_inner_error_ != NULL) {
			goto __catch29_g_error;
			goto __finally29;
		}
	}
	goto __finally29;
	__catch29_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			result = FALSE;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			return result;
		}
	}
	__finally29:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	_tmp1_ = NULL;
#line 98 "ImportPage.vala"
	_tmp0_ = NULL;
#line 98 "ImportPage.vala"
	(*file_to_import) = (_tmp1_ = (_tmp0_ = self->priv->dest_file, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), ((*file_to_import) == NULL) ? NULL : ((*file_to_import) = (g_object_unref ((*file_to_import)), NULL)), _tmp1_);
#line 99 "ImportPage.vala"
	(*copy_to_library) = FALSE;
#line 2959 "ImportPage.c"
	result = TRUE;
	return result;
}


static void import_page_camera_import_job_class_init (ImportPageCameraImportJobClass * klass) {
	import_page_camera_import_job_parent_class = g_type_class_peek_parent (klass);
	BATCH_IMPORT_JOB_CLASS (klass)->finalize = import_page_camera_import_job_finalize;
	g_type_class_add_private (klass, sizeof (ImportPageCameraImportJobPrivate));
	BATCH_IMPORT_JOB_CLASS (klass)->get_identifier = import_page_camera_import_job_real_get_identifier;
	BATCH_IMPORT_JOB_CLASS (klass)->prepare = import_page_camera_import_job_real_prepare;
}


static void import_page_camera_import_job_instance_init (ImportPageCameraImportJob * self) {
	self->priv = IMPORT_PAGE_CAMERA_IMPORT_JOB_GET_PRIVATE (self);
}


static void import_page_camera_import_job_finalize (BatchImportJob* obj) {
	ImportPageCameraImportJob * self;
	self = IMPORT_PAGE_CAMERA_IMPORT_JOB (obj);
	(self->priv->context == NULL) ? NULL : (self->priv->context = (gp_context_wrapper_unref (self->priv->context), NULL));
	(self->priv->camera == NULL) ? NULL : (self->priv->camera = (gp_camera_unref (self->priv->camera), NULL));
	self->priv->dir = (g_free (self->priv->dir), NULL);
	self->priv->filename = (g_free (self->priv->filename), NULL);
	(self->priv->dest_file == NULL) ? NULL : (self->priv->dest_file = (g_object_unref (self->priv->dest_file), NULL));
	BATCH_IMPORT_JOB_CLASS (import_page_camera_import_job_parent_class)->finalize (obj);
}


static GType import_page_camera_import_job_get_type (void) {
	static GType import_page_camera_import_job_type_id = 0;
	if (import_page_camera_import_job_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ImportPageCameraImportJobClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) import_page_camera_import_job_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImportPageCameraImportJob), 0, (GInstanceInitFunc) import_page_camera_import_job_instance_init, NULL };
		import_page_camera_import_job_type_id = g_type_register_static (TYPE_BATCH_IMPORT_JOB, "ImportPageCameraImportJob", &g_define_type_info, 0);
	}
	return import_page_camera_import_job_type_id;
}


#line 106 "ImportPage.vala"
static gint64 import_page_camera_import_comparator_real_compare (Comparator* base, ImportPageCameraImportJob* a, ImportPageCameraImportJob* b) {
#line 3003 "ImportPage.c"
	ImportPageCameraImportComparator * self;
	gint64 result;
	self = IMPORT_PAGE_CAMERA_IMPORT_COMPARATOR (base);
#line 106 "ImportPage.vala"
	g_return_val_if_fail (IMPORT_PAGE_IS_CAMERA_IMPORT_JOB (a), 0LL);
#line 106 "ImportPage.vala"
	g_return_val_if_fail (IMPORT_PAGE_IS_CAMERA_IMPORT_JOB (b), 0LL);
#line 3011 "ImportPage.c"
	result = ((gint64) import_page_camera_import_job_get_exposure_time (a)) - ((gint64) import_page_camera_import_job_get_exposure_time (b));
	return result;
}


#line 105 "ImportPage.vala"
static ImportPageCameraImportComparator* import_page_camera_import_comparator_construct (GType object_type) {
#line 3019 "ImportPage.c"
	ImportPageCameraImportComparator* self;
	self = (ImportPageCameraImportComparator*) g_type_create_instance (object_type);
	return self;
}


#line 105 "ImportPage.vala"
static ImportPageCameraImportComparator* import_page_camera_import_comparator_new (void) {
#line 105 "ImportPage.vala"
	return import_page_camera_import_comparator_construct (IMPORT_PAGE_TYPE_CAMERA_IMPORT_COMPARATOR);
#line 3030 "ImportPage.c"
}


static void import_page_camera_import_comparator_class_init (ImportPageCameraImportComparatorClass * klass) {
	import_page_camera_import_comparator_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->compare = import_page_camera_import_comparator_real_compare;
}


static void import_page_camera_import_comparator_instance_init (ImportPageCameraImportComparator * self) {
}


static GType import_page_camera_import_comparator_get_type (void) {
	static GType import_page_camera_import_comparator_type_id = 0;
	if (import_page_camera_import_comparator_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ImportPageCameraImportComparatorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) import_page_camera_import_comparator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImportPageCameraImportComparator), 0, (GInstanceInitFunc) import_page_camera_import_comparator_instance_init, NULL };
		import_page_camera_import_comparator_type_id = g_type_register_static (TYPE_COMPARATOR, "ImportPageCameraImportComparator", &g_define_type_info, 0);
	}
	return import_page_camera_import_comparator_type_id;
}


static void import_page_class_init (ImportPageClass * klass) {
	import_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ImportPagePrivate));
	PAGE_CLASS (klass)->get_toolbar = import_page_real_get_toolbar;
	CHECKERBOARD_PAGE_CLASS (klass)->on_selection_changed = import_page_real_on_selection_changed;
	CHECKERBOARD_PAGE_CLASS (klass)->get_fullscreen_photo = import_page_real_get_fullscreen_photo;
	G_OBJECT_CLASS (klass)->finalize = import_page_finalize;
}


static void import_page_instance_init (ImportPage * self) {
	self->priv = IMPORT_PAGE_GET_PRIVATE (self);
	self->priv->toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
	self->priv->camera_label = g_object_ref_sink ((GtkLabel*) gtk_label_new (NULL));
	self->priv->progress_bar = g_object_ref_sink ((GtkProgressBar*) gtk_progress_bar_new ());
	self->priv->null_context = gp_context_wrapper_new ();
	self->priv->busy = FALSE;
	self->priv->refreshed = FALSE;
	self->priv->refresh_result = GP_OK;
	self->priv->refresh_error = NULL;
}


static void import_page_finalize (GObject* obj) {
	ImportPage * self;
	self = IMPORT_PAGE (obj);
	(self->priv->toolbar == NULL) ? NULL : (self->priv->toolbar = (g_object_unref (self->priv->toolbar), NULL));
	(self->priv->camera_label == NULL) ? NULL : (self->priv->camera_label = (g_object_unref (self->priv->camera_label), NULL));
	(self->priv->import_selected_button == NULL) ? NULL : (self->priv->import_selected_button = (g_object_unref (self->priv->import_selected_button), NULL));
	(self->priv->import_all_button == NULL) ? NULL : (self->priv->import_all_button = (g_object_unref (self->priv->import_all_button), NULL));
	(self->priv->progress_bar == NULL) ? NULL : (self->priv->progress_bar = (g_object_unref (self->priv->progress_bar), NULL));
	(self->priv->camera == NULL) ? NULL : (self->priv->camera = (gp_camera_unref (self->priv->camera), NULL));
	(self->priv->null_context == NULL) ? NULL : (self->priv->null_context = (gp_context_wrapper_unref (self->priv->null_context), NULL));
	self->priv->uri = (g_free (self->priv->uri), NULL);
	self->priv->refresh_error = (g_free (self->priv->refresh_error), NULL);
	self->priv->camera_name = (g_free (self->priv->camera_name), NULL);
	G_OBJECT_CLASS (import_page_parent_class)->finalize (obj);
}


GType import_page_get_type (void) {
	static GType import_page_type_id = 0;
	if (import_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ImportPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) import_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImportPage), 0, (GInstanceInitFunc) import_page_instance_init, NULL };
		import_page_type_id = g_type_register_static (TYPE_CHECKERBOARD_PAGE, "ImportPage", &g_define_type_info, 0);
	}
	return import_page_type_id;
}


#line 750 "ImportPage.vala"
static void _import_queue_page_on_file_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 3106 "ImportPage.c"
	import_queue_page_on_file_menu (self);
}


#line 754 "ImportPage.vala"
static void _import_queue_page_on_stop_gtk_action_callback (GtkAction* action, gpointer self) {
#line 3113 "ImportPage.c"
	import_queue_page_on_stop (self);
}


#line 754 "ImportPage.vala"
static void _import_queue_page_on_stop_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 3120 "ImportPage.c"
	import_queue_page_on_stop (self);
}


#line 697 "ImportPage.vala"
ImportQueuePage* import_queue_page_construct (GType object_type) {
#line 3127 "ImportPage.c"
	ImportQueuePage * self;
	GtkToolButton* _tmp0_;
	GtkSeparatorToolItem* separator;
	GtkToolItem* progress_item;
#line 698 "ImportPage.vala"
	self = (ImportQueuePage*) single_photo_page_construct (object_type, "Importing ...");
#line 700 "ImportPage.vala"
	page_init_ui (PAGE (self), "import_queue.ui", "/ImportQueueMenuBar", "ImportQueueActionGroup", IMPORT_QUEUE_PAGE_ACTIONS, G_N_ELEMENTS (IMPORT_QUEUE_PAGE_ACTIONS), NULL, 0);
#line 3136 "ImportPage.c"
	_tmp0_ = NULL;
#line 702 "ImportPage.vala"
	self->priv->stop_button = (_tmp0_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_STOP)), (self->priv->stop_button == NULL) ? NULL : (self->priv->stop_button = (g_object_unref (self->priv->stop_button), NULL)), _tmp0_);
#line 703 "ImportPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->stop_button), "Stop importing photos");
#line 704 "ImportPage.vala"
	g_signal_connect_object (self->priv->stop_button, "clicked", (GCallback) _import_queue_page_on_stop_gtk_tool_button_clicked, self, 0);
#line 705 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->stop_button), "sensitive", FALSE, NULL);
#line 707 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->stop_button), -1);
#line 3148 "ImportPage.c"
	separator = g_object_ref_sink ((GtkSeparatorToolItem*) gtk_separator_tool_item_new ());
#line 711 "ImportPage.vala"
	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separator), TRUE);
#line 712 "ImportPage.vala"
	gtk_separator_tool_item_set_draw (separator, FALSE);
#line 714 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (separator), -1);
#line 3156 "ImportPage.c"
	progress_item = g_object_ref_sink (gtk_tool_item_new ());
#line 717 "ImportPage.vala"
	gtk_container_add (GTK_CONTAINER (progress_item), GTK_WIDGET (self->priv->progress_bar));
#line 719 "ImportPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, progress_item, -1);
#line 3162 "ImportPage.c"
	(separator == NULL) ? NULL : (separator = (g_object_unref (separator), NULL));
	(progress_item == NULL) ? NULL : (progress_item = (g_object_unref (progress_item), NULL));
	return self;
}


#line 697 "ImportPage.vala"
ImportQueuePage* import_queue_page_new (void) {
#line 697 "ImportPage.vala"
	return import_queue_page_construct (TYPE_IMPORT_QUEUE_PAGE);
#line 3173 "ImportPage.c"
}


#line 726 "ImportPage.vala"
static GtkToolbar* import_queue_page_real_get_toolbar (Page* base) {
#line 3179 "ImportPage.c"
	ImportQueuePage * self;
	GtkToolbar* result;
	GtkToolbar* _tmp0_;
	self = IMPORT_QUEUE_PAGE (base);
#line 727 "ImportPage.vala"
	_tmp0_ = NULL;
#line 3186 "ImportPage.c"
	result = (_tmp0_ = self->priv->toolbar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 760 "ImportPage.vala"
static void _import_queue_page_on_starting_batch_import_starting (BatchImport* _sender, gpointer self) {
#line 3194 "ImportPage.c"
	import_queue_page_on_starting (self, _sender);
}


#line 764 "ImportPage.vala"
static void _import_queue_page_on_imported_batch_import_imported (BatchImport* _sender, Photo* photo, gpointer self) {
#line 3201 "ImportPage.c"
	import_queue_page_on_imported (self, photo);
}


#line 775 "ImportPage.vala"
static void _import_queue_page_on_import_complete_batch_import_import_complete (BatchImport* _sender, const ImportID* import_id, SortedList* photos_by_date, GeeArrayList* failed, GeeArrayList* skipped, gpointer self) {
#line 3208 "ImportPage.c"
	import_queue_page_on_import_complete (self, _sender, import_id, photos_by_date, failed, skipped);
}


#line 730 "ImportPage.vala"
void import_queue_page_enqueue_and_schedule (ImportQueuePage* self, BatchImport* batch_import) {
#line 730 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_QUEUE_PAGE (self));
#line 730 "ImportPage.vala"
	g_return_if_fail (IS_BATCH_IMPORT (batch_import));
#line 731 "ImportPage.vala"
	self->priv->total_bytes = self->priv->total_bytes + batch_import_get_total_bytes (batch_import);
#line 733 "ImportPage.vala"
	g_signal_connect_object (batch_import, "starting", (GCallback) _import_queue_page_on_starting_batch_import_starting, self, 0);
#line 734 "ImportPage.vala"
	g_signal_connect_object (batch_import, "imported", (GCallback) _import_queue_page_on_imported_batch_import_imported, self, 0);
#line 735 "ImportPage.vala"
	g_signal_connect_object (batch_import, "import-complete", (GCallback) _import_queue_page_on_import_complete_batch_import_import_complete, self, 0);
#line 737 "ImportPage.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->queue), batch_import);
#line 738 "ImportPage.vala"
	g_signal_emit_by_name (self, "batch-added", batch_import);
#line 740 "ImportPage.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->queue)) == 1) {
#line 741 "ImportPage.vala"
		batch_import_schedule (batch_import);
#line 3235 "ImportPage.c"
	}
#line 743 "ImportPage.vala"
	g_object_set (GTK_WIDGET (self->priv->stop_button), "sensitive", TRUE, NULL);
#line 3239 "ImportPage.c"
}


#line 746 "ImportPage.vala"
gint import_queue_page_get_batch_count (ImportQueuePage* self) {
#line 3245 "ImportPage.c"
	gint result;
#line 746 "ImportPage.vala"
	g_return_val_if_fail (IS_IMPORT_QUEUE_PAGE (self), 0);
#line 3249 "ImportPage.c"
	result = gee_collection_get_size (GEE_COLLECTION (self->priv->queue));
	return result;
}


#line 750 "ImportPage.vala"
static void import_queue_page_on_file_menu (ImportQueuePage* self) {
#line 750 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_QUEUE_PAGE (self));
#line 751 "ImportPage.vala"
	page_set_item_sensitive (PAGE (self), "/ImportQueueMenuBar/FileMenu/Stop", gee_collection_get_size (GEE_COLLECTION (self->priv->queue)) > 0);
#line 3261 "ImportPage.c"
}


#line 754 "ImportPage.vala"
static void import_queue_page_on_stop (ImportQueuePage* self) {
#line 754 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_QUEUE_PAGE (self));
#line 3269 "ImportPage.c"
	/* mark all as halted and let each signal failure*/
	{
		GeeIterator* _batch_import_it;
		/* mark all as halted and let each signal failure*/
		_batch_import_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->queue));
		/* mark all as halted and let each signal failure*/
#line 756 "ImportPage.vala"
		while (TRUE) {
#line 3278 "ImportPage.c"
			BatchImport* batch_import;
			/* mark all as halted and let each signal failure*/
#line 756 "ImportPage.vala"
			if (!gee_iterator_next (_batch_import_it)) {
#line 3283 "ImportPage.c"
				/* mark all as halted and let each signal failure*/
#line 756 "ImportPage.vala"
				break;
#line 3287 "ImportPage.c"
			}
			/* mark all as halted and let each signal failure*/
			batch_import = (BatchImport*) gee_iterator_get (_batch_import_it);
#line 757 "ImportPage.vala"
			batch_import_user_halt (batch_import);
#line 3293 "ImportPage.c"
			(batch_import == NULL) ? NULL : (batch_import = (batch_import_unref (batch_import), NULL));
		}
		(_batch_import_it == NULL) ? NULL : (_batch_import_it = (g_object_unref (_batch_import_it), NULL));
	}
}


#line 760 "ImportPage.vala"
static void import_queue_page_on_starting (ImportQueuePage* self, BatchImport* batch_import) {
#line 3303 "ImportPage.c"
	BatchImport* _tmp1_;
	BatchImport* _tmp0_;
#line 760 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_QUEUE_PAGE (self));
#line 760 "ImportPage.vala"
	g_return_if_fail (IS_BATCH_IMPORT (batch_import));
#line 3310 "ImportPage.c"
	_tmp1_ = NULL;
#line 761 "ImportPage.vala"
	_tmp0_ = NULL;
#line 761 "ImportPage.vala"
	self->priv->current_batch = (_tmp1_ = (_tmp0_ = batch_import, (_tmp0_ == NULL) ? NULL : batch_import_ref (_tmp0_)), (self->priv->current_batch == NULL) ? NULL : (self->priv->current_batch = (batch_import_unref (self->priv->current_batch), NULL)), _tmp1_);
#line 3316 "ImportPage.c"
}


#line 764 "ImportPage.vala"
static void import_queue_page_on_imported (ImportQueuePage* self, Photo* photo) {
#line 3322 "ImportPage.c"
	GError * _inner_error_;
	GdkPixbuf* _tmp0_;
	GdkPixbuf* _tmp1_;
	double _tmp2_;
	double pct;
	char* _tmp4_;
	char* _tmp3_;
#line 764 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_QUEUE_PAGE (self));
#line 764 "ImportPage.vala"
	g_return_if_fail (IS_PHOTO (photo));
#line 3334 "ImportPage.c"
	_inner_error_ = NULL;
	_tmp0_ = photo_get_pixbuf (photo, PHOTO_EXCEPTION_NONE, 0, GDK_INTERP_HYPER, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
#line 615 "Photo.vala"
	_tmp1_ = NULL;
#line 765 "ImportPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp1_ = _tmp0_);
#line 3346 "ImportPage.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 767 "ImportPage.vala"
	self->priv->progress_bytes = self->priv->progress_bytes + photo_query_filesize (photo);
#line 3350 "ImportPage.c"
	_tmp2_ = 0.0;
#line 768 "ImportPage.vala"
	if (self->priv->progress_bytes <= self->priv->total_bytes) {
#line 768 "ImportPage.vala"
		_tmp2_ = ((double) self->priv->progress_bytes) / ((double) self->priv->total_bytes);
#line 3356 "ImportPage.c"
	} else {
#line 769 "ImportPage.vala"
		_tmp2_ = 0.0;
#line 3360 "ImportPage.c"
	}
	pct = _tmp2_;
#line 820 "glib-2.0.vapi"
	_tmp4_ = NULL;
#line 171 "Photo.vala"
	_tmp3_ = NULL;
#line 771 "ImportPage.vala"
	gtk_progress_bar_set_text (self->priv->progress_bar, _tmp4_ = g_strdup_printf ("Imported %s", _tmp3_ = photo_get_name (photo)));
#line 3369 "ImportPage.c"
	_tmp4_ = (g_free (_tmp4_), NULL);
	_tmp3_ = (g_free (_tmp3_), NULL);
#line 772 "ImportPage.vala"
	gtk_progress_bar_set_fraction (self->priv->progress_bar, pct);
#line 3374 "ImportPage.c"
}


#line 775 "ImportPage.vala"
static void import_queue_page_on_import_complete (ImportQueuePage* self, BatchImport* batch_import, const ImportID* import_id, SortedList* imported, GeeArrayList* failed, GeeArrayList* skipped) {
#line 3380 "ImportPage.c"
	BatchImport* _tmp0_;
	gboolean removed;
	gboolean _tmp1_;
#line 775 "ImportPage.vala"
	g_return_if_fail (IS_IMPORT_QUEUE_PAGE (self));
#line 775 "ImportPage.vala"
	g_return_if_fail (IS_BATCH_IMPORT (batch_import));
#line 775 "ImportPage.vala"
	g_return_if_fail (IS_SORTED_LIST (imported));
#line 775 "ImportPage.vala"
	g_return_if_fail (GEE_IS_ARRAY_LIST (failed));
#line 775 "ImportPage.vala"
	g_return_if_fail (GEE_IS_ARRAY_LIST (skipped));
#line 777 "ImportPage.vala"
	g_assert (batch_import == self->priv->current_batch);
#line 3396 "ImportPage.c"
	_tmp0_ = NULL;
#line 778 "ImportPage.vala"
	self->priv->current_batch = (_tmp0_ = NULL, (self->priv->current_batch == NULL) ? NULL : (self->priv->current_batch = (batch_import_unref (self->priv->current_batch), NULL)), _tmp0_);
#line 3400 "ImportPage.c"
	removed = gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->queue), batch_import);
#line 781 "ImportPage.vala"
	g_assert (removed);
#line 3404 "ImportPage.c"
	_tmp1_ = FALSE;
#line 783 "ImportPage.vala"
	if (gee_collection_get_size (GEE_COLLECTION (failed)) > 0) {
#line 783 "ImportPage.vala"
		_tmp1_ = TRUE;
#line 3410 "ImportPage.c"
	} else {
#line 783 "ImportPage.vala"
		_tmp1_ = gee_collection_get_size (GEE_COLLECTION (skipped)) > 0;
#line 3414 "ImportPage.c"
	}
#line 783 "ImportPage.vala"
	if (_tmp1_) {
#line 3418 "ImportPage.c"
		char* _tmp2_;
#line 156 "BatchImport.vala"
		_tmp2_ = NULL;
#line 784 "ImportPage.vala"
		app_window_report_import_failures (_tmp2_ = batch_import_get_name (batch_import), GEE_LIST (failed), GEE_LIST (skipped));
#line 3424 "ImportPage.c"
		_tmp2_ = (g_free (_tmp2_), NULL);
	}
#line 786 "ImportPage.vala"
	g_signal_emit_by_name (self, "batch-removed", batch_import);
#line 3429 "ImportPage.c"
	/* schedule next if available*/
#line 789 "ImportPage.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->queue)) > 0) {
#line 3433 "ImportPage.c"
		BatchImport* _tmp3_;
#line 790 "ImportPage.vala"
		g_object_set (GTK_WIDGET (self->priv->stop_button), "sensitive", TRUE, NULL);
#line 681 "ImportPage.vala"
		_tmp3_ = NULL;
#line 791 "ImportPage.vala"
		batch_import_schedule (_tmp3_ = (BatchImport*) gee_abstract_list_get (GEE_ABSTRACT_LIST (self->priv->queue), 0));
#line 3441 "ImportPage.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (batch_import_unref (_tmp3_), NULL));
	} else {
#line 793 "ImportPage.vala"
		g_object_set (GTK_WIDGET (self->priv->stop_button), "sensitive", FALSE, NULL);
#line 3446 "ImportPage.c"
	}
}


static void import_queue_page_class_init (ImportQueuePageClass * klass) {
	import_queue_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ImportQueuePagePrivate));
	PAGE_CLASS (klass)->get_toolbar = import_queue_page_real_get_toolbar;
	G_OBJECT_CLASS (klass)->finalize = import_queue_page_finalize;
	g_signal_new ("batch_added", TYPE_IMPORT_QUEUE_PAGE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1, TYPE_BATCH_IMPORT);
	g_signal_new ("batch_removed", TYPE_IMPORT_QUEUE_PAGE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1, TYPE_BATCH_IMPORT);
}


static void import_queue_page_instance_init (ImportQueuePage * self) {
	self->priv = IMPORT_QUEUE_PAGE_GET_PRIVATE (self);
	self->priv->toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
	self->priv->stop_button = NULL;
	self->priv->queue = gee_array_list_new (TYPE_BATCH_IMPORT, (GBoxedCopyFunc) batch_import_ref, batch_import_unref, g_direct_equal);
	self->priv->current_batch = NULL;
	self->priv->progress_bar = g_object_ref_sink ((GtkProgressBar*) gtk_progress_bar_new ());
	self->priv->progress_bytes = (guint64) 0;
	self->priv->total_bytes = (guint64) 0;
}


static void import_queue_page_finalize (GObject* obj) {
	ImportQueuePage * self;
	self = IMPORT_QUEUE_PAGE (obj);
	(self->priv->toolbar == NULL) ? NULL : (self->priv->toolbar = (g_object_unref (self->priv->toolbar), NULL));
	(self->priv->stop_button == NULL) ? NULL : (self->priv->stop_button = (g_object_unref (self->priv->stop_button), NULL));
	(self->priv->queue == NULL) ? NULL : (self->priv->queue = (g_object_unref (self->priv->queue), NULL));
	(self->priv->current_batch == NULL) ? NULL : (self->priv->current_batch = (batch_import_unref (self->priv->current_batch), NULL));
	(self->priv->progress_bar == NULL) ? NULL : (self->priv->progress_bar = (g_object_unref (self->priv->progress_bar), NULL));
	G_OBJECT_CLASS (import_queue_page_parent_class)->finalize (obj);
}


GType import_queue_page_get_type (void) {
	static GType import_queue_page_type_id = 0;
	if (import_queue_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ImportQueuePageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) import_queue_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ImportQueuePage), 0, (GInstanceInitFunc) import_queue_page_instance_init, NULL };
		import_queue_page_type_id = g_type_register_static (TYPE_SINGLE_PHOTO_PAGE, "ImportQueuePage", &g_define_type_info, 0);
	}
	return import_queue_page_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




