/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gdk/gdk.h>


#define TYPE_ORIENTATION (orientation_get_type ())

#define TYPE_ROTATION (rotation_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_BOX (box_get_type ())
typedef struct _Box Box;

typedef enum  {
	ORIENTATION_MIN = 1,
	ORIENTATION_TOP_LEFT = 1,
	ORIENTATION_TOP_RIGHT = 2,
	ORIENTATION_BOTTOM_RIGHT = 3,
	ORIENTATION_BOTTOM_LEFT = 4,
	ORIENTATION_LEFT_TOP = 5,
	ORIENTATION_RIGHT_TOP = 6,
	ORIENTATION_RIGHT_BOTTOM = 7,
	ORIENTATION_LEFT_BOTTOM = 8,
	ORIENTATION_MAX = 8
} Orientation;

typedef enum  {
	ROTATION_CLOCKWISE,
	ROTATION_COUNTERCLOCKWISE,
	ROTATION_MIRROR,
	ROTATION_UPSIDE_DOWN
} Rotation;

struct _Dimensions {
	gint width;
	gint height;
};

struct _Box {
	gint left;
	gint top;
	gint right;
	gint bottom;
};



GType orientation_get_type (void);
char* orientation_to_string (Orientation self);
Orientation orientation_rotate_clockwise (Orientation self);
Orientation orientation_rotate_counterclockwise (Orientation self);
Orientation orientation_flip_top_to_bottom (Orientation self);
Orientation orientation_flip_left_to_right (Orientation self);
GType rotation_get_type (void);
Orientation orientation_perform (Orientation self, Rotation rotation);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
void dimensions_init (Dimensions *self, gint width, gint height);
Dimensions orientation_rotate_dimensions (Orientation self, const Dimensions* dim);
GdkPixbuf* orientation_rotate_pixbuf (Orientation self, GdkPixbuf* pixbuf);
gboolean dimensions_has_area (Dimensions *self);
GdkPoint orientation_rotate_point (Orientation self, const Dimensions* space, const GdkPoint* point);
GdkPoint orientation_derotate_point (Orientation self, const Dimensions* space, const GdkPoint* point);
GType box_get_type (void);
Box* box_dup (const Box* self);
void box_free (Box* self);
void box_get_points (Box *self, GdkPoint* top_left, GdkPoint* bottom_right);
Box box_from_points (const GdkPoint* corner1, const GdkPoint* corner2);
Box orientation_rotate_box (Orientation self, const Dimensions* space, const Box* box);
Box orientation_derotate_box (Orientation self, const Dimensions* space, const Box* box);



#line 19 "Orientation.vala"
char* orientation_to_string (Orientation self) {
#line 89 "Orientation.c"
	char* result;
#line 20 "Orientation.vala"
	switch (self) {
#line 93 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
			result = g_strdup ("top-left");
			return result;
		}
		case ORIENTATION_TOP_RIGHT:
		{
			result = g_strdup ("top-right");
			return result;
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
			result = g_strdup ("bottom-right");
			return result;
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
			result = g_strdup ("bottom-left");
			return result;
		}
		case ORIENTATION_LEFT_TOP:
		{
			result = g_strdup ("left-top");
			return result;
		}
		case ORIENTATION_RIGHT_TOP:
		{
			result = g_strdup ("right-top");
			return result;
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
			result = g_strdup ("right-bottom");
			return result;
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
			result = g_strdup ("left-bottom");
			return result;
		}
		default:
		{
			result = g_strdup_printf ("unknown orientation %d", (gint) self);
			return result;
		}
	}
}


#line 50 "Orientation.vala"
Orientation orientation_rotate_clockwise (Orientation self) {
#line 145 "Orientation.c"
	Orientation result;
#line 51 "Orientation.vala"
	switch (self) {
#line 149 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
			result = ORIENTATION_RIGHT_TOP;
			return result;
		}
		case ORIENTATION_TOP_RIGHT:
		{
			result = ORIENTATION_RIGHT_BOTTOM;
			return result;
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
			result = ORIENTATION_LEFT_BOTTOM;
			return result;
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
			result = ORIENTATION_LEFT_TOP;
			return result;
		}
		case ORIENTATION_LEFT_TOP:
		{
			result = ORIENTATION_TOP_RIGHT;
			return result;
		}
		case ORIENTATION_RIGHT_TOP:
		{
			result = ORIENTATION_BOTTOM_RIGHT;
			return result;
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
			result = ORIENTATION_BOTTOM_LEFT;
			return result;
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
			result = ORIENTATION_TOP_LEFT;
			return result;
		}
		default:
		{
			{
#line 77 "Orientation.vala"
				g_error ("Orientation.vala:77: rotate_clockwise: %d", (gint) self);
#line 195 "Orientation.c"
				result = self;
				return result;
			}
		}
	}
}


#line 84 "Orientation.vala"
Orientation orientation_rotate_counterclockwise (Orientation self) {
#line 206 "Orientation.c"
	Orientation result;
#line 85 "Orientation.vala"
	switch (self) {
#line 210 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
			result = ORIENTATION_LEFT_BOTTOM;
			return result;
		}
		case ORIENTATION_TOP_RIGHT:
		{
			result = ORIENTATION_LEFT_TOP;
			return result;
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
			result = ORIENTATION_RIGHT_TOP;
			return result;
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
			result = ORIENTATION_RIGHT_BOTTOM;
			return result;
		}
		case ORIENTATION_LEFT_TOP:
		{
			result = ORIENTATION_BOTTOM_LEFT;
			return result;
		}
		case ORIENTATION_RIGHT_TOP:
		{
			result = ORIENTATION_TOP_LEFT;
			return result;
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
			result = ORIENTATION_TOP_RIGHT;
			return result;
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
			result = ORIENTATION_BOTTOM_RIGHT;
			return result;
		}
		default:
		{
			{
#line 111 "Orientation.vala"
				g_error ("Orientation.vala:111: rotate_counterclockwise: %d", (gint) self);
#line 256 "Orientation.c"
				result = self;
				return result;
			}
		}
	}
}


#line 118 "Orientation.vala"
Orientation orientation_flip_top_to_bottom (Orientation self) {
#line 267 "Orientation.c"
	Orientation result;
#line 119 "Orientation.vala"
	switch (self) {
#line 271 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
			result = ORIENTATION_BOTTOM_LEFT;
			return result;
		}
		case ORIENTATION_TOP_RIGHT:
		{
			result = ORIENTATION_BOTTOM_RIGHT;
			return result;
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
			result = ORIENTATION_TOP_RIGHT;
			return result;
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
			result = ORIENTATION_TOP_LEFT;
			return result;
		}
		case ORIENTATION_LEFT_TOP:
		{
			result = ORIENTATION_RIGHT_TOP;
			return result;
		}
		case ORIENTATION_RIGHT_TOP:
		{
			result = ORIENTATION_LEFT_TOP;
			return result;
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
			result = ORIENTATION_LEFT_BOTTOM;
			return result;
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
			result = ORIENTATION_RIGHT_BOTTOM;
			return result;
		}
		default:
		{
			{
#line 145 "Orientation.vala"
				g_error ("Orientation.vala:145: flip_top_to_bottom: %d", (gint) self);
#line 317 "Orientation.c"
				result = self;
				return result;
			}
		}
	}
}


#line 152 "Orientation.vala"
Orientation orientation_flip_left_to_right (Orientation self) {
#line 328 "Orientation.c"
	Orientation result;
#line 153 "Orientation.vala"
	switch (self) {
#line 332 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
			result = ORIENTATION_TOP_RIGHT;
			return result;
		}
		case ORIENTATION_TOP_RIGHT:
		{
			result = ORIENTATION_TOP_LEFT;
			return result;
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
			result = ORIENTATION_BOTTOM_LEFT;
			return result;
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
			result = ORIENTATION_BOTTOM_RIGHT;
			return result;
		}
		case ORIENTATION_LEFT_TOP:
		{
			result = ORIENTATION_RIGHT_TOP;
			return result;
		}
		case ORIENTATION_RIGHT_TOP:
		{
			result = ORIENTATION_LEFT_TOP;
			return result;
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
			result = ORIENTATION_LEFT_BOTTOM;
			return result;
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
			result = ORIENTATION_RIGHT_BOTTOM;
			return result;
		}
		default:
		{
			{
#line 179 "Orientation.vala"
				g_error ("Orientation.vala:179: flip_left_to_right: %d", (gint) self);
#line 378 "Orientation.c"
				result = self;
				return result;
			}
		}
	}
}


#line 186 "Orientation.vala"
Orientation orientation_perform (Orientation self, Rotation rotation) {
#line 389 "Orientation.c"
	Orientation result;
#line 187 "Orientation.vala"
	switch (rotation) {
#line 393 "Orientation.c"
		case ROTATION_CLOCKWISE:
		{
			result = orientation_rotate_clockwise (self);
			return result;
		}
		case ROTATION_COUNTERCLOCKWISE:
		{
			result = orientation_rotate_counterclockwise (self);
			return result;
		}
		case ROTATION_MIRROR:
		{
			result = orientation_flip_left_to_right (self);
			return result;
		}
		case ROTATION_UPSIDE_DOWN:
		{
			result = orientation_flip_top_to_bottom (self);
			return result;
		}
		default:
		{
#line 201 "Orientation.vala"
			g_error ("Orientation.vala:201: perform: %d", (gint) rotation);
#line 418 "Orientation.c"
			result = self;
			return result;
		}
	}
}


#line 207 "Orientation.vala"
Dimensions orientation_rotate_dimensions (Orientation self, const Dimensions* dim) {
#line 428 "Orientation.c"
	Dimensions result;
#line 208 "Orientation.vala"
	switch (self) {
#line 432 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		case ORIENTATION_TOP_RIGHT:
		case ORIENTATION_BOTTOM_RIGHT:
		case ORIENTATION_BOTTOM_LEFT:
		{
			result = (*dim);
			return result;
		}
		case ORIENTATION_LEFT_TOP:
		case ORIENTATION_RIGHT_TOP:
		case ORIENTATION_RIGHT_BOTTOM:
		case ORIENTATION_LEFT_BOTTOM:
		{
			Dimensions _tmp0_ = {0};
			result = (dimensions_init (&_tmp0_, (*dim).height, (*dim).width), _tmp0_);
			return result;
		}
		default:
		{
#line 224 "Orientation.vala"
			g_error ("Orientation.vala:224: rotate_dimensions: %d", (gint) self);
#line 454 "Orientation.c"
			result = (*dim);
			return result;
		}
	}
}


#line 230 "Orientation.vala"
GdkPixbuf* orientation_rotate_pixbuf (Orientation self, GdkPixbuf* pixbuf) {
#line 464 "Orientation.c"
	GdkPixbuf* result;
	GdkPixbuf* _tmp9_;
#line 230 "Orientation.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 231 "Orientation.vala"
	switch (self) {
#line 471 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
#line 234 "Orientation.vala"
			break;
#line 476 "Orientation.c"
		}
		case ORIENTATION_TOP_RIGHT:
		{
			GdkPixbuf* _tmp0_;
			_tmp0_ = NULL;
#line 238 "Orientation.vala"
			pixbuf = (_tmp0_ = gdk_pixbuf_flip (pixbuf, TRUE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp0_);
#line 239 "Orientation.vala"
			break;
#line 486 "Orientation.c"
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
			GdkPixbuf* _tmp1_;
			_tmp1_ = NULL;
#line 242 "Orientation.vala"
			pixbuf = (_tmp1_ = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_UPSIDEDOWN), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 243 "Orientation.vala"
			break;
#line 496 "Orientation.c"
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
			GdkPixbuf* _tmp2_;
			_tmp2_ = NULL;
#line 247 "Orientation.vala"
			pixbuf = (_tmp2_ = gdk_pixbuf_flip (pixbuf, FALSE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp2_);
#line 248 "Orientation.vala"
			break;
#line 506 "Orientation.c"
		}
		case ORIENTATION_LEFT_TOP:
		{
			GdkPixbuf* _tmp3_;
			GdkPixbuf* _tmp4_;
			_tmp3_ = NULL;
#line 251 "Orientation.vala"
			pixbuf = (_tmp3_ = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp3_);
#line 515 "Orientation.c"
			_tmp4_ = NULL;
#line 252 "Orientation.vala"
			pixbuf = (_tmp4_ = gdk_pixbuf_flip (pixbuf, FALSE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp4_);
#line 253 "Orientation.vala"
			break;
#line 521 "Orientation.c"
		}
		case ORIENTATION_RIGHT_TOP:
		{
			GdkPixbuf* _tmp5_;
			_tmp5_ = NULL;
#line 256 "Orientation.vala"
			pixbuf = (_tmp5_ = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_CLOCKWISE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp5_);
#line 257 "Orientation.vala"
			break;
#line 531 "Orientation.c"
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
			GdkPixbuf* _tmp6_;
			GdkPixbuf* _tmp7_;
			_tmp6_ = NULL;
#line 260 "Orientation.vala"
			pixbuf = (_tmp6_ = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_CLOCKWISE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp6_);
#line 540 "Orientation.c"
			_tmp7_ = NULL;
#line 261 "Orientation.vala"
			pixbuf = (_tmp7_ = gdk_pixbuf_flip (pixbuf, FALSE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp7_);
#line 262 "Orientation.vala"
			break;
#line 546 "Orientation.c"
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
			GdkPixbuf* _tmp8_;
			_tmp8_ = NULL;
#line 265 "Orientation.vala"
			pixbuf = (_tmp8_ = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp8_);
#line 266 "Orientation.vala"
			break;
#line 556 "Orientation.c"
		}
		default:
		{
#line 269 "Orientation.vala"
			g_error ("Orientation.vala:269: rotate_pixbuf: %d", (gint) self);
#line 270 "Orientation.vala"
			break;
#line 564 "Orientation.c"
		}
	}
#line 273 "Orientation.vala"
	_tmp9_ = NULL;
#line 569 "Orientation.c"
	result = (_tmp9_ = pixbuf, (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_));
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	return result;
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


/* space is the unrotated dimensions the point is rotating with*/
#line 277 "Orientation.vala"
GdkPoint orientation_rotate_point (Orientation self, const Dimensions* space, const GdkPoint* point) {
#line 580 "Orientation.c"
	GdkPoint result;
	GdkPoint _tmp0_ = {0};
	GdkPoint rotated;
#line 278 "Orientation.vala"
	g_assert (dimensions_has_area (&(*space)));
#line 279 "Orientation.vala"
	g_assert ((*point).x >= 0);
#line 280 "Orientation.vala"
	g_assert ((*point).x < (*space).width);
#line 281 "Orientation.vala"
	g_assert ((*point).y >= 0);
#line 282 "Orientation.vala"
	g_assert ((*point).y < (*space).height);
#line 594 "Orientation.c"
	rotated = (memset (&_tmp0_, 0, sizeof (GdkPoint)), _tmp0_);
#line 286 "Orientation.vala"
	switch (self) {
#line 598 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
#line 289 "Orientation.vala"
			rotated = (*point);
#line 290 "Orientation.vala"
			break;
#line 605 "Orientation.c"
		}
		case ORIENTATION_TOP_RIGHT:
		{
#line 294 "Orientation.vala"
			rotated.x = ((*space).width - (*point).x) - 1;
#line 295 "Orientation.vala"
			rotated.y = (*point).y;
#line 296 "Orientation.vala"
			break;
#line 615 "Orientation.c"
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
#line 300 "Orientation.vala"
			rotated.x = ((*space).width - (*point).x) - 1;
#line 301 "Orientation.vala"
			rotated.y = ((*space).height - (*point).y) - 1;
#line 302 "Orientation.vala"
			break;
#line 625 "Orientation.c"
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
#line 306 "Orientation.vala"
			rotated.x = (*point).x;
#line 307 "Orientation.vala"
			rotated.y = ((*space).height - (*point).y) - 1;
#line 308 "Orientation.vala"
			break;
#line 635 "Orientation.c"
		}
		case ORIENTATION_LEFT_TOP:
		{
#line 312 "Orientation.vala"
			rotated.x = (*point).y;
#line 313 "Orientation.vala"
			rotated.y = (*point).x;
#line 314 "Orientation.vala"
			break;
#line 645 "Orientation.c"
		}
		case ORIENTATION_RIGHT_TOP:
		{
#line 318 "Orientation.vala"
			rotated.x = ((*space).height - (*point).y) - 1;
#line 319 "Orientation.vala"
			rotated.y = (*point).x;
#line 320 "Orientation.vala"
			break;
#line 655 "Orientation.c"
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
#line 324 "Orientation.vala"
			rotated.x = ((*space).height - (*point).y) - 1;
#line 325 "Orientation.vala"
			rotated.y = ((*space).width - (*point).x) - 1;
#line 326 "Orientation.vala"
			break;
#line 665 "Orientation.c"
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
#line 330 "Orientation.vala"
			rotated.x = (*point).y;
#line 331 "Orientation.vala"
			rotated.y = ((*space).width - (*point).x) - 1;
#line 332 "Orientation.vala"
			break;
#line 675 "Orientation.c"
		}
		default:
		{
#line 335 "Orientation.vala"
			g_error ("Orientation.vala:335: rotate_point: %d", (gint) self);
#line 336 "Orientation.vala"
			break;
#line 683 "Orientation.c"
		}
	}
	result = rotated;
	return result;
}


/* space is the unrotated dimensions the point is return to*/
#line 343 "Orientation.vala"
GdkPoint orientation_derotate_point (Orientation self, const Dimensions* space, const GdkPoint* point) {
#line 694 "Orientation.c"
	GdkPoint result;
	GdkPoint _tmp0_ = {0};
	GdkPoint derotated;
#line 344 "Orientation.vala"
	g_assert (dimensions_has_area (&(*space)));
#line 700 "Orientation.c"
	derotated = (memset (&_tmp0_, 0, sizeof (GdkPoint)), _tmp0_);
#line 348 "Orientation.vala"
	switch (self) {
#line 704 "Orientation.c"
		case ORIENTATION_TOP_LEFT:
		{
#line 351 "Orientation.vala"
			derotated = (*point);
#line 352 "Orientation.vala"
			break;
#line 711 "Orientation.c"
		}
		case ORIENTATION_TOP_RIGHT:
		{
#line 356 "Orientation.vala"
			derotated.x = ((*space).width - (*point).x) - 1;
#line 357 "Orientation.vala"
			derotated.y = (*point).y;
#line 358 "Orientation.vala"
			break;
#line 721 "Orientation.c"
		}
		case ORIENTATION_BOTTOM_RIGHT:
		{
#line 362 "Orientation.vala"
			derotated.x = ((*space).width - (*point).x) - 1;
#line 363 "Orientation.vala"
			derotated.y = ((*space).height - (*point).y) - 1;
#line 364 "Orientation.vala"
			break;
#line 731 "Orientation.c"
		}
		case ORIENTATION_BOTTOM_LEFT:
		{
#line 368 "Orientation.vala"
			derotated.x = (*point).x;
#line 369 "Orientation.vala"
			derotated.y = ((*space).height - (*point).y) - 1;
#line 370 "Orientation.vala"
			break;
#line 741 "Orientation.c"
		}
		case ORIENTATION_LEFT_TOP:
		{
#line 374 "Orientation.vala"
			derotated.x = (*point).y;
#line 375 "Orientation.vala"
			derotated.y = (*point).x;
#line 376 "Orientation.vala"
			break;
#line 751 "Orientation.c"
		}
		case ORIENTATION_RIGHT_TOP:
		{
#line 380 "Orientation.vala"
			derotated.x = (*point).y;
#line 381 "Orientation.vala"
			derotated.y = ((*space).height - (*point).x) - 1;
#line 382 "Orientation.vala"
			break;
#line 761 "Orientation.c"
		}
		case ORIENTATION_RIGHT_BOTTOM:
		{
#line 386 "Orientation.vala"
			derotated.x = ((*space).width - (*point).y) - 1;
#line 387 "Orientation.vala"
			derotated.y = ((*space).height - (*point).x) - 1;
#line 388 "Orientation.vala"
			break;
#line 771 "Orientation.c"
		}
		case ORIENTATION_LEFT_BOTTOM:
		{
#line 392 "Orientation.vala"
			derotated.x = ((*space).width - (*point).y) - 1;
#line 393 "Orientation.vala"
			derotated.y = (*point).x;
#line 394 "Orientation.vala"
			break;
#line 781 "Orientation.c"
		}
		default:
		{
#line 397 "Orientation.vala"
			g_error ("Orientation.vala:397: rotate_point: %d", (gint) self);
#line 398 "Orientation.vala"
			break;
#line 789 "Orientation.c"
		}
	}
	result = derotated;
	return result;
}


#line 404 "Orientation.vala"
Box orientation_rotate_box (Orientation self, const Dimensions* space, const Box* box) {
#line 799 "Orientation.c"
	Box result;
	GdkPoint top_left = {0};
	GdkPoint bottom_right = {0};
#line 406 "Orientation.vala"
	box_get_points (&(*box), &top_left, &bottom_right);
#line 408 "Orientation.vala"
	top_left = orientation_rotate_point (self, &(*space), &top_left);
#line 409 "Orientation.vala"
	bottom_right = orientation_rotate_point (self, &(*space), &bottom_right);
#line 809 "Orientation.c"
	result = box_from_points (&top_left, &bottom_right);
	return result;
}


#line 414 "Orientation.vala"
Box orientation_derotate_box (Orientation self, const Dimensions* space, const Box* box) {
#line 817 "Orientation.c"
	Box result;
	GdkPoint top_left = {0};
	GdkPoint bottom_right = {0};
#line 416 "Orientation.vala"
	box_get_points (&(*box), &top_left, &bottom_right);
#line 418 "Orientation.vala"
	top_left = orientation_derotate_point (self, &(*space), &top_left);
#line 419 "Orientation.vala"
	bottom_right = orientation_derotate_point (self, &(*space), &bottom_right);
#line 827 "Orientation.c"
	result = box_from_points (&top_left, &bottom_right);
	return result;
}



GType orientation_get_type (void) {
	static GType orientation_type_id = 0;
	if (G_UNLIKELY (orientation_type_id == 0)) {
		static const GEnumValue values[] = {{ORIENTATION_MIN, "ORIENTATION_MIN", "min"}, {ORIENTATION_TOP_LEFT, "ORIENTATION_TOP_LEFT", "top-left"}, {ORIENTATION_TOP_RIGHT, "ORIENTATION_TOP_RIGHT", "top-right"}, {ORIENTATION_BOTTOM_RIGHT, "ORIENTATION_BOTTOM_RIGHT", "bottom-right"}, {ORIENTATION_BOTTOM_LEFT, "ORIENTATION_BOTTOM_LEFT", "bottom-left"}, {ORIENTATION_LEFT_TOP, "ORIENTATION_LEFT_TOP", "left-top"}, {ORIENTATION_RIGHT_TOP, "ORIENTATION_RIGHT_TOP", "right-top"}, {ORIENTATION_RIGHT_BOTTOM, "ORIENTATION_RIGHT_BOTTOM", "right-bottom"}, {ORIENTATION_LEFT_BOTTOM, "ORIENTATION_LEFT_BOTTOM", "left-bottom"}, {ORIENTATION_MAX, "ORIENTATION_MAX", "max"}, {0, NULL, NULL}};
		orientation_type_id = g_enum_register_static ("Orientation", values);
	}
	return orientation_type_id;
}




