/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <time.h>
#include <gee.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <string.h>
#include <gio/gio.h>
#include <gphoto2/gphoto2-file.h>
#include <stdlib.h>
#include <gdk/gdk.h>
#include <float.h>
#include <math.h>
#include <libexif/exif-data.h>
#include <libexif/exif-tag.h>


#define TYPE_IMPORT_RESULT (import_result_get_type ())

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;
typedef struct _PhotoPrivate PhotoPrivate;

#define TYPE_PHOTO_ID (photo_id_get_type ())
typedef struct _PhotoID PhotoID;

#define PHOTO_TYPE_CURRENCY (photo_currency_get_type ())

#define TYPE_DATABASE_TABLE (database_table_get_type ())
#define DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_TABLE, DatabaseTable))
#define DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_TABLE, DatabaseTableClass))
#define IS_DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_TABLE))
#define IS_DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_TABLE))
#define DATABASE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_TABLE, DatabaseTableClass))

typedef struct _DatabaseTable DatabaseTable;
typedef struct _DatabaseTableClass DatabaseTableClass;

#define TYPE_PHOTO_TABLE (photo_table_get_type ())
#define PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_TABLE, PhotoTable))
#define PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_TABLE, PhotoTableClass))
#define IS_PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_TABLE))
#define IS_PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_TABLE))
#define PHOTO_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_TABLE, PhotoTableClass))

typedef struct _PhotoTable PhotoTable;
typedef struct _PhotoTableClass PhotoTableClass;

#define JPEG_TYPE_QUALITY (jpeg_quality_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_ORIENTATION (orientation_get_type ())

#define TYPE_PHOTO_EXIF (photo_exif_get_type ())
#define PHOTO_EXIF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_EXIF, PhotoExif))
#define PHOTO_EXIF_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_EXIF, PhotoExifClass))
#define IS_PHOTO_EXIF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_EXIF))
#define IS_PHOTO_EXIF_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_EXIF))
#define PHOTO_EXIF_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_EXIF, PhotoExifClass))

typedef struct _PhotoExif PhotoExif;
typedef struct _PhotoExifClass PhotoExifClass;

#define TYPE_IMPORT_ID (import_id_get_type ())
typedef struct _ImportID ImportID;

#define TYPE_EVENT_ID (event_id_get_type ())
typedef struct _EventID EventID;

#define TYPE_ROTATION (rotation_get_type ())

#define TYPE_BOX (box_get_type ())
typedef struct _Box Box;

#define TYPE_KEY_VALUE_MAP (key_value_map_get_type ())
#define KEY_VALUE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_KEY_VALUE_MAP, KeyValueMap))
#define KEY_VALUE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_KEY_VALUE_MAP, KeyValueMapClass))
#define IS_KEY_VALUE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_KEY_VALUE_MAP))
#define IS_KEY_VALUE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_KEY_VALUE_MAP))
#define KEY_VALUE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_KEY_VALUE_MAP, KeyValueMapClass))

typedef struct _KeyValueMap KeyValueMap;
typedef struct _KeyValueMapClass KeyValueMapClass;

#define TYPE_REDEYE_INSTANCE (redeye_instance_get_type ())
typedef struct _RedeyeInstance RedeyeInstance;

#define TYPE_COLOR_TRANSFORMATION_INSTANCE (color_transformation_instance_get_type ())

#define TYPE_COLOR_TRANSFORMATION_KIND (color_transformation_kind_get_type ())
typedef struct _ColorTransformationInstance ColorTransformationInstance;

#define TYPE_COLOR_TRANSFORMATION_FACTORY (color_transformation_factory_get_type ())
#define COLOR_TRANSFORMATION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactory))
#define COLOR_TRANSFORMATION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactoryClass))
#define IS_COLOR_TRANSFORMATION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY))
#define IS_COLOR_TRANSFORMATION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_TRANSFORMATION_FACTORY))
#define COLOR_TRANSFORMATION_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactoryClass))

typedef struct _ColorTransformationFactory ColorTransformationFactory;
typedef struct _ColorTransformationFactoryClass ColorTransformationFactoryClass;

#define TYPE_COLOR_TRANSFORMATION (color_transformation_get_type ())
#define COLOR_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLOR_TRANSFORMATION, ColorTransformation))
#define COLOR_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLOR_TRANSFORMATION, ColorTransformationClass))
#define IS_COLOR_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLOR_TRANSFORMATION))
#define IS_COLOR_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_TRANSFORMATION))
#define COLOR_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLOR_TRANSFORMATION, ColorTransformationClass))

typedef struct _ColorTransformation ColorTransformation;
typedef struct _ColorTransformationClass ColorTransformationClass;

#define TYPE_SCALE_CONSTRAINT (scale_constraint_get_type ())

typedef enum  {
	IMPORT_RESULT_SUCCESS,
	IMPORT_RESULT_FILE_ERROR,
	IMPORT_RESULT_DECODE_ERROR,
	IMPORT_RESULT_DATABASE_ERROR,
	IMPORT_RESULT_USER_ABORT,
	IMPORT_RESULT_NOT_A_FILE,
	IMPORT_RESULT_PHOTO_EXISTS,
	IMPORT_RESULT_UNSUPPORTED_FORMAT
} ImportResult;

struct _Photo {
	GObject parent_instance;
	PhotoPrivate * priv;
};

struct _PhotoClass {
	GObjectClass parent_class;
};

struct _PhotoID {
	gint64 id;
};

struct _PhotoPrivate {
	PhotoID photo_id;
	time_t exposure_time;
};

typedef enum  {
	PHOTO_CURRENCY_CURRENT,
	PHOTO_CURRENCY_DIRTY,
	PHOTO_CURRENCY_GONE
} PhotoCurrency;

typedef enum  {
	JPEG_QUALITY_LOW = 50,
	JPEG_QUALITY_MEDIUM = 75,
	JPEG_QUALITY_HIGH = 90,
	JPEG_QUALITY_MAXIMUM = 100
} JpegQuality;

struct _Dimensions {
	gint width;
	gint height;
};

typedef enum  {
	ORIENTATION_MIN = 1,
	ORIENTATION_TOP_LEFT = 1,
	ORIENTATION_TOP_RIGHT = 2,
	ORIENTATION_BOTTOM_RIGHT = 3,
	ORIENTATION_BOTTOM_LEFT = 4,
	ORIENTATION_LEFT_TOP = 5,
	ORIENTATION_RIGHT_TOP = 6,
	ORIENTATION_RIGHT_BOTTOM = 7,
	ORIENTATION_LEFT_BOTTOM = 8,
	ORIENTATION_MAX = 8
} Orientation;

struct _ImportID {
	gint64 id;
};

struct _EventID {
	gint64 id;
};

typedef enum  {
	ROTATION_CLOCKWISE,
	ROTATION_COUNTERCLOCKWISE,
	ROTATION_MIRROR,
	ROTATION_UPSIDE_DOWN
} Rotation;

struct _Box {
	gint left;
	gint top;
	gint right;
	gint bottom;
};

struct _RedeyeInstance {
	GdkPoint center;
	gint radius;
};

typedef enum  {
	COLOR_TRANSFORMATION_KIND_EXPOSURE,
	COLOR_TRANSFORMATION_KIND_SATURATION,
	COLOR_TRANSFORMATION_KIND_TINT,
	COLOR_TRANSFORMATION_KIND_TEMPERATURE
} ColorTransformationKind;

struct _ColorTransformationInstance {
	ColorTransformationKind kind;
	float parameter;
};

typedef enum  {
	SCALE_CONSTRAINT_ORIGINAL,
	SCALE_CONSTRAINT_DIMENSIONS,
	SCALE_CONSTRAINT_WIDTH,
	SCALE_CONSTRAINT_HEIGHT
} ScaleConstraint;


static GeeHashMap* photo_photo_map;
static GeeHashMap* photo_photo_map = NULL;
static PhotoTable* photo_photo_table;
static PhotoTable* photo_photo_table = NULL;
static PhotoID photo_cached_photo_id;
static PhotoID photo_cached_photo_id = {0};
static GdkPixbuf* photo_cached_raw;
static GdkPixbuf* photo_cached_raw = NULL;
static gpointer photo_parent_class = NULL;

GType import_result_get_type (void);
GType photo_get_type (void);
GType photo_id_get_type (void);
PhotoID* photo_id_dup (const PhotoID* self);
void photo_id_free (PhotoID* self);
#define PHOTO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PHOTO, PhotoPrivate))
enum  {
	PHOTO_DUMMY_PROPERTY
};
GType photo_currency_get_type (void);
gpointer database_table_ref (gpointer instance);
void database_table_unref (gpointer instance);
GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_database_table (GValue* value, gpointer v_object);
gpointer value_get_database_table (const GValue* value);
GType database_table_get_type (void);
GType photo_table_get_type (void);
#define PHOTO_ID_INVALID ((gint64) (-1))
void photo_id_init (PhotoID *self, gint64 id);
#define PHOTO_EXCEPTION_NONE 0
#define PHOTO_EXCEPTION_ORIENTATION (1 << 0)
#define PHOTO_EXCEPTION_CROP (1 << 1)
#define PHOTO_EXCEPTION_REDEYE (1 << 2)
#define PHOTO_EXCEPTION_ADJUST (1 << 3)
GType jpeg_quality_get_type (void);
#define PHOTO_EXPORT_JPEG_QUALITY JPEG_QUALITY_HIGH
#define PHOTO_EXPORT_INTERP GDK_INTERP_BILINEAR
guint int64_hash (void* p);
static guint _int64_hash_ghash_func (void* key);
gboolean int64_equal (void* a, void* b);
static gboolean _int64_equal_gequal_func (void* a, void* b);
static gint64* _int64_dup (gint64* self);
PhotoTable* photo_table_new (void);
PhotoTable* photo_table_construct (GType object_type);
void photo_init (void);
void photo_terminate (void);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
void dimensions_init (Dimensions *self, gint width, gint height);
GType orientation_get_type (void);
PhotoExif* photo_exif_new (GFile* file);
PhotoExif* photo_exif_construct (GType object_type, GFile* file);
gpointer photo_exif_ref (gpointer instance);
void photo_exif_unref (gpointer instance);
GParamSpec* param_spec_photo_exif (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_photo_exif (GValue* value, gpointer v_object);
gpointer value_get_photo_exif (const GValue* value);
GType photo_exif_get_type (void);
gboolean photo_exif_has_exif (PhotoExif* self);
gboolean photo_exif_get_dimensions (PhotoExif* self, Dimensions* dim);
gboolean photo_exif_get_timestamp (PhotoExif* self, time_t* timestamp);
Orientation photo_exif_get_orientation (PhotoExif* self);
gboolean dimensions_has_area (Dimensions *self);
gboolean photo_table_is_photo_stored (PhotoTable* self, GFile* file);
GType import_id_get_type (void);
ImportID* import_id_dup (const ImportID* self);
void import_id_free (ImportID* self);
PhotoID photo_table_add (PhotoTable* self, GFile* file, const Dimensions* dim, gint64 filesize, glong timestamp, const time_t* exposure_time, Orientation orientation, const ImportID* import_id);
gboolean photo_id_is_invalid (PhotoID *self);
GdkPixbuf* orientation_rotate_pixbuf (Orientation self, GdkPixbuf* pixbuf);
void thumbnail_cache_import (const PhotoID* photo_id, GdkPixbuf* original, gboolean force);
Photo* photo_fetch (const PhotoID* photo_id);
ImportResult photo_import (GFile* file, const ImportID* import_id, Photo** photo);
static Photo* photo_new (const PhotoID* photo_id);
static Photo* photo_construct (GType object_type, const PhotoID* photo_id);
gboolean photo_id_is_valid (PhotoID *self);
static void photo_remove_exportable_file (Photo* self);
static void _photo_remove_exportable_file_photo_altered (Photo* _sender, gpointer self);
PhotoID photo_get_photo_id (Photo* self);
GFile* photo_table_get_file (PhotoTable* self, const PhotoID* photo_id);
GFile* photo_get_file (Photo* self);
char* photo_table_get_name (PhotoTable* self, const PhotoID* photo_id);
char* photo_get_name (Photo* self);
guint64 photo_query_filesize (Photo* self);
time_t photo_table_get_exposure_time (PhotoTable* self, const PhotoID* photo_id);
time_t photo_get_exposure_time (Photo* self);
time_t photo_table_get_timestamp (PhotoTable* self, const PhotoID* photo_id);
time_t photo_get_timestamp (Photo* self);
GType event_id_get_type (void);
EventID* event_id_dup (const EventID* self);
void event_id_free (EventID* self);
EventID photo_table_get_event (PhotoTable* self, const PhotoID* photo_id);
EventID photo_get_event_id (Photo* self);
gboolean photo_table_set_event (PhotoTable* self, const PhotoID* photo_id, const EventID* event_id);
void photo_set_event_id (Photo* self, const EventID* event_id);
char* photo_to_string (Photo* self);
gboolean photo_equals (Photo* self, Photo* photo);
gboolean photo_table_has_transformations (PhotoTable* self, const PhotoID* photo_id);
Orientation photo_table_get_orientation (PhotoTable* self, const PhotoID* photo_id);
Orientation photo_table_get_original_orientation (PhotoTable* self, const PhotoID* photo_id);
gboolean photo_has_transformations (Photo* self);
gboolean photo_table_remove_all_transformations (PhotoTable* self, const PhotoID* photo_id);
gboolean photo_table_set_orientation (PhotoTable* self, const PhotoID* photo_id, Orientation orientation);
static void photo_photo_altered (Photo* self);
void photo_remove_all_transformations (Photo* self);
GType rotation_get_type (void);
Orientation orientation_perform (Orientation self, Rotation rotation);
#define THUMBNAIL_CACHE_BIG_SCALE 360
#define THUMBNAIL_CACHE_MEDIUM_SCALE 128
#define THUMBNAIL_CACHE_SMALL_SCALE 64
GdkPixbuf* thumbnail_cache_fetch (const PhotoID* photo_id, gint scale);
GdkPixbuf* rotation_perform (Rotation self, GdkPixbuf* pixbuf);
void thumbnail_cache_replace (const PhotoID* photo_id, gint scale, GdkPixbuf* replacement);
void photo_rotate (Photo* self, Rotation rotation);
Dimensions photo_table_get_dimensions (PhotoTable* self, const PhotoID* photo_id);
Dimensions orientation_rotate_dimensions (Orientation self, const Dimensions* dim);
Dimensions photo_get_uncropped_dimensions (Photo* self);
GType box_get_type (void);
Box* box_dup (const Box* self);
void box_free (Box* self);
gboolean photo_get_crop (Photo* self, Box* crop);
Dimensions box_get_dimensions (Box *self);
Dimensions photo_get_dimensions (Photo* self);
gpointer key_value_map_ref (gpointer instance);
void key_value_map_unref (gpointer instance);
GParamSpec* param_spec_key_value_map (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_key_value_map (GValue* value, gpointer v_object);
gpointer value_get_key_value_map (const GValue* value);
GType key_value_map_get_type (void);
KeyValueMap* photo_table_get_transformation (PhotoTable* self, const PhotoID* photo_id, const char* object);
gboolean photo_has_crop (Photo* self);
gint key_value_map_get_int (KeyValueMap* self, const char* key, gint def);
void box_init (Box *self, gint left, gint top, gint right, gint bottom);
static gboolean photo_get_raw_crop (Photo* self, Box* crop);
Box orientation_rotate_box (Orientation self, const Dimensions* space, const Box* box);
static Orientation photo_get_orientation (Photo* self);
KeyValueMap* key_value_map_new (const char* group);
KeyValueMap* key_value_map_construct (GType object_type, const char* group);
void key_value_map_set_int (KeyValueMap* self, const char* key, gint value);
gboolean photo_table_set_transformation (PhotoTable* self, const PhotoID* photo_id, KeyValueMap* map);
static gboolean photo_set_raw_crop (Photo* self, const Box* crop);
Box orientation_derotate_box (Orientation self, const Dimensions* space, const Box* box);
gint box_get_width (Box *self);
gint box_get_height (Box *self);
gboolean photo_set_crop (Photo* self, const Box* crop);
gboolean photo_table_remove_transformation (PhotoTable* self, const PhotoID* photo_id, const char* object);
gboolean photo_remove_crop (Photo* self);
GType redeye_instance_get_type (void);
RedeyeInstance* redeye_instance_dup (const RedeyeInstance* self);
void redeye_instance_free (RedeyeInstance* self);
GdkRectangle redeye_instance_to_bounds_rect (const RedeyeInstance* inst);
static GdkRectangle photo_unscaled_to_raw_rect (Photo* self, const GdkRectangle* unscaled_rect);
RedeyeInstance redeye_instance_from_bounds_rect (const GdkRectangle* rect);
void key_value_map_set_point (KeyValueMap* self, const char* key, const GdkPoint* point);
gboolean photo_add_redeye_instance (Photo* self, const RedeyeInstance* inst_unscaled);
GType color_transformation_instance_get_type (void);
GType color_transformation_kind_get_type (void);
ColorTransformationInstance* color_transformation_instance_dup (const ColorTransformationInstance* self);
void color_transformation_instance_free (ColorTransformationInstance* self);
void key_value_map_set_double (KeyValueMap* self, const char* key, double value);
gboolean photo_set_adjustments (Photo* self, GeeArrayList* adjustments);
double key_value_map_get_double (KeyValueMap* self, const char* key, double def);
float photo_get_adjustment_parameter (Photo* self, ColorTransformationKind adjust_kind);
gpointer color_transformation_factory_ref (gpointer instance);
void color_transformation_factory_unref (gpointer instance);
GParamSpec* param_spec_color_transformation_factory (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_color_transformation_factory (GValue* value, gpointer v_object);
gpointer value_get_color_transformation_factory (const GValue* value);
GType color_transformation_factory_get_type (void);
ColorTransformationFactory* color_transformation_factory_get_instance (void);
gpointer color_transformation_ref (gpointer instance);
void color_transformation_unref (gpointer instance);
GParamSpec* param_spec_color_transformation (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_color_transformation (GValue* value, gpointer v_object);
gpointer value_get_color_transformation (const GValue* value);
GType color_transformation_get_type (void);
ColorTransformation* color_transformation_factory_from_parameter (ColorTransformationFactory* self, ColorTransformationKind kind, float parameter);
ColorTransformation* color_transformation_compose_against (ColorTransformation* self, ColorTransformation* transform);
ColorTransformation* photo_get_composite_transformation (Photo* self);
GdkPoint key_value_map_get_point (KeyValueMap* self, const char* key, const GdkPoint* def);
static RedeyeInstance* photo_get_all_redeye (Photo* self, int* result_length1);
static GdkPixbuf* photo_red_reduce_pixel (Photo* self, GdkPixbuf* pixbuf, gint x, gint y);
static GdkPixbuf* photo_do_redeye (Photo* self, GdkPixbuf* pixbuf, RedeyeInstance* inst);
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp);
gboolean color_transformation_is_identity (ColorTransformation* self);
void color_transformation_transform_pixbuf (ColorTransformation* transform, GdkPixbuf* pixbuf);
GdkPixbuf* photo_get_pixbuf (Photo* self, gint exceptions, gint scale, GdkInterpType interp, GError** error);
GFile* app_window_get_data_subdir (const char* name, const char* subname);
static GFile* photo_generate_exportable_file (Photo* self, GError** error);
void photo_exif_set_exif (PhotoExif* self, ExifData* exif);
ExifData* photo_exif_get_exif (PhotoExif* self);
void photo_exif_set_dimensions (PhotoExif* self, const Dimensions* dim);
void photo_exif_set_orientation (PhotoExif* self, Orientation orientation);
gint photo_exif_remove_all_tags (PhotoExif* self, ExifTag tag);
gboolean photo_exif_remove_thumbnail (PhotoExif* self);
void photo_exif_commit (PhotoExif* self, GError** error);
static void photo_copy_exported_exif (Photo* self, PhotoExif* source, PhotoExif* dest, Orientation orientation, const Dimensions* dim, GError** error);
char* jpeg_quality_get_pct_text (JpegQuality self);
Dimensions dimensions_for_pixbuf (GdkPixbuf* pixbuf);
GFile* photo_generate_exportable (Photo* self, GError** error);
GType scale_constraint_get_type (void);
Dimensions dimensions_get_scaled_by_constraint (Dimensions *self, gint scale, ScaleConstraint constraint);
void photo_export (Photo* self, GFile* dest_file, gint scale, ScaleConstraint constraint, JpegQuality quality, GError** error);
GdkPixbuf* photo_get_thumbnail (Photo* self, gint scale);
void thumbnail_cache_remove (const PhotoID* photo_id);
static void photo_remove_original_file (Photo* self);
gboolean photo_table_remove (PhotoTable* self, const PhotoID* photoID);
void photo_remove (Photo* self, gboolean remove_original);
GFile* app_window_get_photos_dir (void);
#define BATCH_IMPORT_IMPORT_DIRECTORY_DEPTH 3
gboolean query_is_directory_empty (GFile* dir, GError** error);
gint64 photo_table_get_filesize (PhotoTable* self, const PhotoID* photo_id);
gboolean thumbnail_cache_exists (const PhotoID* photo_id);
PhotoCurrency photo_check_currency (Photo* self);
gboolean photo_table_update (PhotoTable* self, const PhotoID* photoID, const Dimensions* dim, gint64 filesize, glong timestamp, const time_t* exposure_time, Orientation orientation);
void photo_update (Photo* self);
GdkPoint orientation_derotate_point (Orientation self, const Dimensions* space, const GdkPoint* point);
static GdkPoint photo_unscaled_to_raw_point (Photo* self, const GdkPoint* unscaled_point);
static void photo_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const gint THUMBNAIL_CACHE_SCALES[] = {THUMBNAIL_CACHE_BIG_SCALE, THUMBNAIL_CACHE_MEDIUM_SCALE, THUMBNAIL_CACHE_SMALL_SCALE};



GType import_result_get_type (void) {
	static GType import_result_type_id = 0;
	if (G_UNLIKELY (import_result_type_id == 0)) {
		static const GEnumValue values[] = {{IMPORT_RESULT_SUCCESS, "IMPORT_RESULT_SUCCESS", "success"}, {IMPORT_RESULT_FILE_ERROR, "IMPORT_RESULT_FILE_ERROR", "file-error"}, {IMPORT_RESULT_DECODE_ERROR, "IMPORT_RESULT_DECODE_ERROR", "decode-error"}, {IMPORT_RESULT_DATABASE_ERROR, "IMPORT_RESULT_DATABASE_ERROR", "database-error"}, {IMPORT_RESULT_USER_ABORT, "IMPORT_RESULT_USER_ABORT", "user-abort"}, {IMPORT_RESULT_NOT_A_FILE, "IMPORT_RESULT_NOT_A_FILE", "not-a-file"}, {IMPORT_RESULT_PHOTO_EXISTS, "IMPORT_RESULT_PHOTO_EXISTS", "photo-exists"}, {IMPORT_RESULT_UNSUPPORTED_FORMAT, "IMPORT_RESULT_UNSUPPORTED_FORMAT", "unsupported-format"}, {0, NULL, NULL}};
		import_result_type_id = g_enum_register_static ("ImportResult", values);
	}
	return import_result_type_id;
}



GType photo_currency_get_type (void) {
	static GType photo_currency_type_id = 0;
	if (G_UNLIKELY (photo_currency_type_id == 0)) {
		static const GEnumValue values[] = {{PHOTO_CURRENCY_CURRENT, "PHOTO_CURRENCY_CURRENT", "current"}, {PHOTO_CURRENCY_DIRTY, "PHOTO_CURRENCY_DIRTY", "dirty"}, {PHOTO_CURRENCY_GONE, "PHOTO_CURRENCY_GONE", "gone"}, {0, NULL, NULL}};
		photo_currency_type_id = g_enum_register_static ("PhotoCurrency", values);
	}
	return photo_currency_type_id;
}


#line 7 "util.vala"
static guint _int64_hash_ghash_func (void* key) {
#line 486 "Photo.c"
	return int64_hash (key);
}


#line 18 "util.vala"
static gboolean _int64_equal_gequal_func (void* a, void* b) {
#line 493 "Photo.c"
	return int64_equal (a, b);
}


static gint64* _int64_dup (gint64* self) {
	gint64* dup;
	dup = g_new0 (gint64, 1);
	memcpy (dup, self, sizeof (gint64));
	return dup;
}


#line 45 "Photo.vala"
void photo_init (void) {
#line 508 "Photo.c"
	GeeHashMap* _tmp0_;
	PhotoTable* _tmp1_;
	_tmp0_ = NULL;
#line 46 "Photo.vala"
	photo_photo_map = (_tmp0_ = gee_hash_map_new (G_TYPE_INT64, (GBoxedCopyFunc) _int64_dup, g_free, TYPE_PHOTO, (GBoxedCopyFunc) g_object_ref, g_object_unref, _int64_hash_ghash_func, _int64_equal_gequal_func, g_direct_equal), (photo_photo_map == NULL) ? NULL : (photo_photo_map = (g_object_unref (photo_photo_map), NULL)), _tmp0_);
#line 514 "Photo.c"
	_tmp1_ = NULL;
#line 47 "Photo.vala"
	photo_photo_table = (_tmp1_ = photo_table_new (), (photo_photo_table == NULL) ? NULL : (photo_photo_table = (database_table_unref (photo_photo_table), NULL)), _tmp1_);
#line 518 "Photo.c"
}


#line 50 "Photo.vala"
void photo_terminate (void) {
#line 524 "Photo.c"
}


#line 53 "Photo.vala"
ImportResult photo_import (GFile* file, const ImportID* import_id, Photo** photo) {
#line 530 "Photo.c"
	ImportResult result;
	GError * _inner_error_;
	char* _tmp0_;
	GFileInfo* info;
	GTimeVal _tmp4_ = {0};
	GTimeVal timestamp;
	Dimensions _tmp5_ = {0};
	Dimensions dim;
	Orientation orientation;
	time_t exposure_time;
	PhotoExif* exif;
	GdkPixbuf* pixbuf;
	gboolean _tmp12_;
	gboolean _tmp13_;
	PhotoID photo_id;
	GdkPixbuf* _tmp17_;
	GdkPixbuf* _tmp16_;
	Photo* _tmp18_;
#line 53 "Photo.vala"
	g_return_val_if_fail (G_IS_FILE (file), 0);
#line 551 "Photo.c"
	if (photo != NULL) {
		*photo = NULL;
	}
	_inner_error_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 54 "Photo.vala"
	g_debug ("Photo.vala:54: Importing file %s", _tmp0_ = g_file_get_path (file));
#line 560 "Photo.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	info = NULL;
	{
		GFileInfo* _tmp1_;
		GFileInfo* _tmp2_;
		_tmp1_ = g_file_query_info (file, "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch30_g_error;
			goto __finally30;
		}
		_tmp2_ = NULL;
#line 58 "Photo.vala"
		info = (_tmp2_ = _tmp1_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp2_);
#line 574 "Photo.c"
	}
	goto __finally30;
	__catch30_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			result = IMPORT_RESULT_FILE_ERROR;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			return result;
		}
	}
	__finally30:
	if (_inner_error_ != NULL) {
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
#line 63 "Photo.vala"
	if (g_file_info_get_file_type (info) != G_FILE_TYPE_REGULAR) {
#line 598 "Photo.c"
		result = IMPORT_RESULT_NOT_A_FILE;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		return result;
	}
#line 66 "Photo.vala"
	if (_vala_strcmp0 (g_file_info_get_content_type (info), GP_MIME_JPEG) != 0) {
#line 605 "Photo.c"
		char* _tmp3_;
#line 952 "gio-2.0.vapi"
		_tmp3_ = NULL;
#line 67 "Photo.vala"
		g_message ("Photo.vala:67: Not importing %s: Unsupported content type %s", _tmp3_ = g_file_get_path (file), g_file_info_get_content_type (info));
#line 611 "Photo.c"
		_tmp3_ = (g_free (_tmp3_), NULL);
		result = IMPORT_RESULT_UNSUPPORTED_FORMAT;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		return result;
	}
	timestamp = (g_get_current_time (&_tmp4_), _tmp4_);
#line 74 "Photo.vala"
	g_file_info_get_modification_time (info, &timestamp);
#line 620 "Photo.c"
	dim = (dimensions_init (&_tmp5_, 0, 0), _tmp5_);
	orientation = ORIENTATION_TOP_LEFT;
	exposure_time = 0;
	/* TODO: Try to read JFIF metadata too*/
	exif = photo_exif_new (file);
#line 82 "Photo.vala"
	if (photo_exif_has_exif (exif)) {
#line 83 "Photo.vala"
		if (!photo_exif_get_dimensions (exif, &dim)) {
#line 630 "Photo.c"
			char* _tmp6_;
#line 952 "gio-2.0.vapi"
			_tmp6_ = NULL;
#line 84 "Photo.vala"
			g_message ("Photo.vala:84: Unable to read EXIF dimensions for %s", _tmp6_ = g_file_get_path (file));
#line 636 "Photo.c"
			_tmp6_ = (g_free (_tmp6_), NULL);
		}
#line 86 "Photo.vala"
		if (!photo_exif_get_timestamp (exif, &exposure_time)) {
#line 641 "Photo.c"
			char* _tmp7_;
#line 952 "gio-2.0.vapi"
			_tmp7_ = NULL;
#line 87 "Photo.vala"
			g_message ("Photo.vala:87: Unable to read EXIF orientation for %s", _tmp7_ = g_file_get_path (file));
#line 647 "Photo.c"
			_tmp7_ = (g_free (_tmp7_), NULL);
		}
#line 89 "Photo.vala"
		orientation = photo_exif_get_orientation (exif);
#line 652 "Photo.c"
	}
	pixbuf = NULL;
	{
		char* _tmp8_;
		GdkPixbuf* _tmp9_;
		GdkPixbuf* _tmp10_;
		GdkPixbuf* _tmp11_;
#line 952 "gio-2.0.vapi"
		_tmp8_ = NULL;
#line 94 "Photo.vala"
		_tmp9_ = NULL;
#line 664 "Photo.c"
		_tmp10_ = (_tmp9_ = gdk_pixbuf_new_from_file (_tmp8_ = g_file_get_path (file), &_inner_error_), _tmp8_ = (g_free (_tmp8_), NULL), _tmp9_);
		if (_inner_error_ != NULL) {
			goto __catch31_g_error;
			goto __finally31;
		}
		_tmp11_ = NULL;
#line 94 "Photo.vala"
		pixbuf = (_tmp11_ = _tmp10_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp11_);
#line 673 "Photo.c"
	}
	goto __finally31;
	__catch31_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			/* assume a decode error, although technically it could be I/O ... need better Gdk
			 bindings to determine which*/
			result = IMPORT_RESULT_DECODE_ERROR;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			return result;
		}
	}
	__finally31:
	if (_inner_error_ != NULL) {
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	_tmp12_ = FALSE;
	_tmp13_ = FALSE;
#line 102 "Photo.vala"
	if (gdk_pixbuf_get_colorspace (pixbuf) != GDK_COLORSPACE_RGB) {
#line 102 "Photo.vala"
		_tmp13_ = TRUE;
#line 707 "Photo.c"
	} else {
#line 102 "Photo.vala"
		_tmp13_ = gdk_pixbuf_get_n_channels (pixbuf) < 3;
#line 711 "Photo.c"
	}
#line 102 "Photo.vala"
	if (_tmp13_) {
#line 102 "Photo.vala"
		_tmp12_ = TRUE;
#line 717 "Photo.c"
	} else {
#line 103 "Photo.vala"
		_tmp12_ = gdk_pixbuf_get_bits_per_sample (pixbuf) != 8;
#line 721 "Photo.c"
	}
	/* verify basic mechanics of photo: RGB 8-bit encoding*/
#line 102 "Photo.vala"
	if (_tmp12_) {
#line 726 "Photo.c"
		char* _tmp14_;
#line 952 "gio-2.0.vapi"
		_tmp14_ = NULL;
#line 104 "Photo.vala"
		g_message ("Photo.vala:104: Not importing %s: Unsupported color format", _tmp14_ = g_file_get_path (file));
#line 732 "Photo.c"
		_tmp14_ = (g_free (_tmp14_), NULL);
		result = IMPORT_RESULT_UNSUPPORTED_FORMAT;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		return result;
	}
	/* XXX: Trust EXIF or Pixbuf for dimensions?*/
#line 110 "Photo.vala"
	if (!dimensions_has_area (&dim)) {
#line 743 "Photo.c"
		Dimensions _tmp15_ = {0};
#line 111 "Photo.vala"
		dim = (dimensions_init (&_tmp15_, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf)), _tmp15_);
#line 747 "Photo.c"
	}
#line 113 "Photo.vala"
	if (photo_table_is_photo_stored (photo_photo_table, file)) {
#line 751 "Photo.c"
		result = IMPORT_RESULT_PHOTO_EXISTS;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		return result;
	}
	/* photo information is stored in database in raw, non-modified format ... this is especially
	 important dealing with dimensions and orientation*/
	photo_id = photo_table_add (photo_photo_table, file, &dim, g_file_info_get_size (info), timestamp.tv_sec, &exposure_time, orientation, &(*import_id));
#line 120 "Photo.vala"
	if (photo_id_is_invalid (&photo_id)) {
#line 763 "Photo.c"
		result = IMPORT_RESULT_DATABASE_ERROR;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		return result;
	}
	/* sanity ... this would be very bad*/
#line 124 "Photo.vala"
	g_assert (!gee_abstract_map_contains (GEE_ABSTRACT_MAP (photo_photo_map), &photo_id.id));
#line 773 "Photo.c"
	/* modify pixbuf for thumbnails, which are stored with modifications*/
	_tmp17_ = NULL;
#line 127 "Photo.vala"
	_tmp16_ = NULL;
#line 127 "Photo.vala"
	pixbuf = (_tmp17_ = orientation_rotate_pixbuf (orientation, (_tmp16_ = pixbuf, (_tmp16_ == NULL) ? NULL : g_object_ref (_tmp16_))), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp17_);
#line 780 "Photo.c"
	/* import it into the thumbnail cache with modifications*/
#line 130 "Photo.vala"
	thumbnail_cache_import (&photo_id, pixbuf, FALSE);
#line 784 "Photo.c"
	_tmp18_ = NULL;
#line 132 "Photo.vala"
	(*photo) = (_tmp18_ = photo_fetch (&photo_id), ((*photo) == NULL) ? NULL : ((*photo) = (g_object_unref ((*photo)), NULL)), _tmp18_);
#line 788 "Photo.c"
	result = IMPORT_RESULT_SUCCESS;
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	return result;
}


#line 137 "Photo.vala"
Photo* photo_fetch (const PhotoID* photo_id) {
#line 799 "Photo.c"
	Photo* result;
	Photo* photo;
	photo = (Photo*) gee_abstract_map_get (GEE_ABSTRACT_MAP (photo_photo_map), &(*photo_id).id);
#line 140 "Photo.vala"
	if (photo == NULL) {
#line 805 "Photo.c"
		Photo* _tmp0_;
		_tmp0_ = NULL;
#line 141 "Photo.vala"
		photo = (_tmp0_ = photo_new (&(*photo_id)), (photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL)), _tmp0_);
#line 142 "Photo.vala"
		gee_abstract_map_set (GEE_ABSTRACT_MAP (photo_photo_map), &(*photo_id).id, photo);
#line 812 "Photo.c"
	}
	result = photo;
	return result;
}


#line 884 "Photo.vala"
static void _photo_remove_exportable_file_photo_altered (Photo* _sender, gpointer self) {
#line 821 "Photo.c"
	photo_remove_exportable_file (self);
}


#line 148 "Photo.vala"
static Photo* photo_construct (GType object_type, const PhotoID* photo_id) {
#line 828 "Photo.c"
	Photo * self;
	self = g_object_newv (object_type, 0, NULL);
#line 149 "Photo.vala"
	g_assert (photo_id_is_valid (&(*photo_id)));
#line 151 "Photo.vala"
	self->priv->photo_id = (*photo_id);
#line 154 "Photo.vala"
	g_signal_connect_object (self, "altered", (GCallback) _photo_remove_exportable_file_photo_altered, self, 0);
#line 837 "Photo.c"
	return self;
}


#line 148 "Photo.vala"
static Photo* photo_new (const PhotoID* photo_id) {
#line 148 "Photo.vala"
	return photo_construct (TYPE_PHOTO, photo_id);
#line 846 "Photo.c"
}


#line 163 "Photo.vala"
PhotoID photo_get_photo_id (Photo* self) {
#line 852 "Photo.c"
	PhotoID result;
	result = self->priv->photo_id;
	return result;
}


#line 167 "Photo.vala"
GFile* photo_get_file (Photo* self) {
#line 861 "Photo.c"
	GFile* result;
#line 167 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 865 "Photo.c"
	result = photo_table_get_file (photo_photo_table, &self->priv->photo_id);
	return result;
}


#line 171 "Photo.vala"
char* photo_get_name (Photo* self) {
#line 873 "Photo.c"
	char* result;
#line 171 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 877 "Photo.c"
	result = photo_table_get_name (photo_photo_table, &self->priv->photo_id);
	return result;
}


#line 175 "Photo.vala"
guint64 photo_query_filesize (Photo* self) {
#line 885 "Photo.c"
	guint64 result;
	GError * _inner_error_;
	GFileInfo* info;
#line 175 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), 0ULL);
#line 891 "Photo.c"
	_inner_error_ = NULL;
	info = NULL;
	{
		GFile* _tmp0_;
		GFileInfo* _tmp1_;
		GFileInfo* _tmp2_;
		GFileInfo* _tmp3_;
#line 167 "Photo.vala"
		_tmp0_ = NULL;
#line 178 "Photo.vala"
		_tmp1_ = NULL;
#line 903 "Photo.c"
		_tmp2_ = (_tmp1_ = g_file_query_info (_tmp0_ = photo_get_file (self), G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
		if (_inner_error_ != NULL) {
			goto __catch32_g_error;
			goto __finally32;
		}
		_tmp3_ = NULL;
#line 178 "Photo.vala"
		info = (_tmp3_ = _tmp2_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp3_);
#line 912 "Photo.c"
	}
	goto __finally32;
	__catch32_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp5_;
			GFile* _tmp4_;
#line 952 "gio-2.0.vapi"
			_tmp5_ = NULL;
#line 167 "Photo.vala"
			_tmp4_ = NULL;
#line 181 "Photo.vala"
			g_debug ("Photo.vala:181: Unable to query filesize for %s: %s", _tmp5_ = g_file_get_path (_tmp4_ = photo_get_file (self)), err->message);
#line 929 "Photo.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
			(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
			result = (guint64) 0;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			return result;
		}
	}
	__finally32:
	if (_inner_error_ != NULL) {
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0ULL;
	}
	result = (guint64) g_file_info_get_size (info);
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	return result;
}


#line 189 "Photo.vala"
time_t photo_get_exposure_time (Photo* self) {
#line 953 "Photo.c"
	time_t result;
#line 190 "Photo.vala"
	if (self->priv->exposure_time == (-1)) {
#line 191 "Photo.vala"
		self->priv->exposure_time = photo_table_get_exposure_time (photo_photo_table, &self->priv->photo_id);
#line 959 "Photo.c"
	}
	result = self->priv->exposure_time;
	return result;
}


#line 196 "Photo.vala"
time_t photo_get_timestamp (Photo* self) {
#line 968 "Photo.c"
	time_t result;
	result = photo_table_get_timestamp (photo_photo_table, &self->priv->photo_id);
	return result;
}


#line 200 "Photo.vala"
EventID photo_get_event_id (Photo* self) {
#line 977 "Photo.c"
	EventID result;
	result = photo_table_get_event (photo_photo_table, &self->priv->photo_id);
	return result;
}


#line 204 "Photo.vala"
void photo_set_event_id (Photo* self, const EventID* event_id) {
#line 204 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 205 "Photo.vala"
	photo_table_set_event (photo_photo_table, &self->priv->photo_id, &(*event_id));
#line 990 "Photo.c"
}


#line 208 "Photo.vala"
char* photo_to_string (Photo* self) {
#line 996 "Photo.c"
	char* result;
	char* _tmp1_;
	GFile* _tmp0_;
	char* _tmp2_;
#line 208 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 952 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 167 "Photo.vala"
	_tmp0_ = NULL;
#line 209 "Photo.vala"
	_tmp2_ = NULL;
#line 1009 "Photo.c"
	result = (_tmp2_ = g_strdup_printf ("[%lld] %s", self->priv->photo_id.id, _tmp1_ = g_file_get_path (_tmp0_ = photo_get_file (self))), _tmp1_ = (g_free (_tmp1_), NULL), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
	return result;
}


#line 212 "Photo.vala"
gboolean photo_equals (Photo* self, Photo* photo) {
#line 1017 "Photo.c"
	gboolean result;
#line 212 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 212 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (photo), FALSE);
#line 1023 "Photo.c"
	/* identity works because of the photo_map, but the photo_table primary key is where the
	 rubber hits the road*/
#line 215 "Photo.vala"
	if (self == photo) {
#line 216 "Photo.vala"
		g_assert (self->priv->photo_id.id == photo->priv->photo_id.id);
#line 1030 "Photo.c"
		result = TRUE;
		return result;
	}
#line 221 "Photo.vala"
	g_assert (self->priv->photo_id.id != photo->priv->photo_id.id);
#line 1036 "Photo.c"
	result = FALSE;
	return result;
}


#line 226 "Photo.vala"
gboolean photo_has_transformations (Photo* self) {
#line 1044 "Photo.c"
	gboolean result;
	gboolean _tmp0_;
#line 226 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 1049 "Photo.c"
	_tmp0_ = FALSE;
#line 227 "Photo.vala"
	if (photo_table_has_transformations (photo_photo_table, &self->priv->photo_id)) {
#line 227 "Photo.vala"
		_tmp0_ = TRUE;
#line 1055 "Photo.c"
	} else {
#line 228 "Photo.vala"
		_tmp0_ = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id) != photo_table_get_original_orientation (photo_photo_table, &self->priv->photo_id);
#line 1059 "Photo.c"
	}
	result = _tmp0_;
	return result;
}


#line 231 "Photo.vala"
void photo_remove_all_transformations (Photo* self) {
#line 1068 "Photo.c"
	gboolean altered;
	Orientation orientation;
	Orientation original_orientation;
#line 231 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 1074 "Photo.c"
	altered = photo_table_remove_all_transformations (photo_photo_table, &self->priv->photo_id);
	orientation = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
	original_orientation = photo_table_get_original_orientation (photo_photo_table, &self->priv->photo_id);
#line 236 "Photo.vala"
	if (orientation != original_orientation) {
#line 237 "Photo.vala"
		photo_table_set_orientation (photo_photo_table, &self->priv->photo_id, original_orientation);
#line 238 "Photo.vala"
		altered = TRUE;
#line 1084 "Photo.c"
	}
#line 241 "Photo.vala"
	if (altered) {
#line 1088 "Photo.c"
		GdkPixbuf* _tmp0_;
		/* REDEYE: if photo was altered, clear the pixbuf cache. This is
		 necessary because the redeye transformation, unlike rotate/crop,
		 actually modifies the pixel data in the pixbuf, so we need to
		 re-load the original pixel data from its source file when redeye
		 is cleared*/
		_tmp0_ = NULL;
#line 248 "Photo.vala"
		photo_cached_raw = (_tmp0_ = NULL, (photo_cached_raw == NULL) ? NULL : (photo_cached_raw = (g_object_unref (photo_cached_raw), NULL)), _tmp0_);
#line 250 "Photo.vala"
		photo_photo_altered (self);
#line 1100 "Photo.c"
	}
}


#line 254 "Photo.vala"
void photo_rotate (Photo* self, Rotation rotation) {
#line 1107 "Photo.c"
	Orientation orientation;
#line 254 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 1111 "Photo.c"
	orientation = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
#line 257 "Photo.vala"
	orientation = orientation_perform (orientation, rotation);
#line 259 "Photo.vala"
	photo_table_set_orientation (photo_photo_table, &self->priv->photo_id, orientation);
#line 261 "Photo.vala"
	g_signal_emit_by_name (self, "altered");
#line 1119 "Photo.c"
	/* because rotations are (a) common and available everywhere in the app, (b) the user expects
	 a level of responsiveness not necessarily required by other modifications, and (c) can't 
	 cache a lot of full-sized pixbufs for rotate-and-scale ops, perform the rotation directly 
	 on the already-modified thumbnails.*/
	{
		gint* scale_collection;
		int scale_collection_length1;
		int scale_it;
#line 267 "Photo.vala"
		scale_collection = THUMBNAIL_CACHE_SCALES;
#line 1130 "Photo.c"
		scale_collection_length1 = G_N_ELEMENTS (THUMBNAIL_CACHE_SCALES);
		for (scale_it = 0; scale_it < G_N_ELEMENTS (THUMBNAIL_CACHE_SCALES); scale_it = scale_it + 1) {
			gint scale;
			scale = scale_collection[scale_it];
			{
				GdkPixbuf* thumbnail;
				GdkPixbuf* _tmp0_;
				thumbnail = thumbnail_cache_fetch (&self->priv->photo_id, scale);
				_tmp0_ = NULL;
#line 269 "Photo.vala"
				thumbnail = (_tmp0_ = rotation_perform (rotation, thumbnail), (thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL)), _tmp0_);
#line 270 "Photo.vala"
				thumbnail_cache_replace (&self->priv->photo_id, scale, thumbnail);
#line 1144 "Photo.c"
				(thumbnail == NULL) ? NULL : (thumbnail = (g_object_unref (thumbnail), NULL));
			}
		}
	}
#line 273 "Photo.vala"
	g_signal_emit_by_name (self, "thumbnail-altered");
#line 1151 "Photo.c"
}


/* Returns uncropped (but rotated) dimensions*/
#line 277 "Photo.vala"
Dimensions photo_get_uncropped_dimensions (Photo* self) {
#line 1158 "Photo.c"
	Dimensions result;
	Dimensions dim;
	Orientation orientation;
	dim = photo_table_get_dimensions (photo_photo_table, &self->priv->photo_id);
	orientation = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
	result = orientation_rotate_dimensions (orientation, &dim);
	return result;
}


/* Returns dimensions for fully-modified photo*/
#line 285 "Photo.vala"
Dimensions photo_get_dimensions (Photo* self) {
#line 1172 "Photo.c"
	Dimensions result;
	Box crop = {0};
#line 287 "Photo.vala"
	if (photo_get_crop (self, &crop)) {
#line 1177 "Photo.c"
		result = box_get_dimensions (&crop);
		return result;
	}
	result = photo_get_uncropped_dimensions (self);
	return result;
}


#line 293 "Photo.vala"
gboolean photo_has_crop (Photo* self) {
#line 1188 "Photo.c"
	gboolean result;
	KeyValueMap* _tmp0_;
	gboolean _tmp1_;
#line 293 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 619 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 1196 "Photo.c"
	result = (_tmp1_ = (_tmp0_ = photo_table_get_transformation (photo_photo_table, &self->priv->photo_id, "crop")) != NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (key_value_map_unref (_tmp0_), NULL)), _tmp1_);
	return result;
}


/* Returns the crop in the raw photo's coordinate system*/
#line 298 "Photo.vala"
static gboolean photo_get_raw_crop (Photo* self, Box* crop) {
#line 1205 "Photo.c"
	gboolean result;
	KeyValueMap* map;
	gint left;
	gint top;
	gint right;
	gint bottom;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	Box _tmp3_ = {0};
#line 298 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 1218 "Photo.c"
	map = photo_table_get_transformation (photo_photo_table, &self->priv->photo_id, "crop");
#line 300 "Photo.vala"
	if (map == NULL) {
#line 1222 "Photo.c"
		result = FALSE;
		(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
		return result;
	}
	left = key_value_map_get_int (map, "left", -1);
	top = key_value_map_get_int (map, "top", -1);
	right = key_value_map_get_int (map, "right", -1);
	bottom = key_value_map_get_int (map, "bottom", -1);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
#line 308 "Photo.vala"
	if (left == (-1)) {
#line 308 "Photo.vala"
		_tmp2_ = TRUE;
#line 1238 "Photo.c"
	} else {
#line 308 "Photo.vala"
		_tmp2_ = top == (-1);
#line 1242 "Photo.c"
	}
#line 308 "Photo.vala"
	if (_tmp2_) {
#line 308 "Photo.vala"
		_tmp1_ = TRUE;
#line 1248 "Photo.c"
	} else {
#line 308 "Photo.vala"
		_tmp1_ = right == (-1);
#line 1252 "Photo.c"
	}
#line 308 "Photo.vala"
	if (_tmp1_) {
#line 308 "Photo.vala"
		_tmp0_ = TRUE;
#line 1258 "Photo.c"
	} else {
#line 308 "Photo.vala"
		_tmp0_ = bottom == (-1);
#line 1262 "Photo.c"
	}
#line 308 "Photo.vala"
	if (_tmp0_) {
#line 1266 "Photo.c"
		result = FALSE;
		(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
		return result;
	}
#line 311 "Photo.vala"
	(*crop) = (box_init (&_tmp3_, left, top, right, bottom), _tmp3_);
#line 1273 "Photo.c"
	result = TRUE;
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
	return result;
}


/* Returns the rotated crop for the photo*/
#line 317 "Photo.vala"
gboolean photo_get_crop (Photo* self, Box* crop) {
#line 1283 "Photo.c"
	gboolean result;
	Box raw = {0};
	Dimensions dim;
	Orientation orientation;
#line 317 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 319 "Photo.vala"
	if (!photo_get_raw_crop (self, &raw)) {
#line 1292 "Photo.c"
		result = FALSE;
		return result;
	}
	dim = photo_table_get_dimensions (photo_photo_table, &self->priv->photo_id);
	orientation = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
#line 324 "Photo.vala"
	(*crop) = orientation_rotate_box (orientation, &dim, &raw);
#line 1300 "Photo.c"
	result = TRUE;
	return result;
}


#line 329 "Photo.vala"
static Orientation photo_get_orientation (Photo* self) {
#line 1308 "Photo.c"
	Orientation result;
#line 329 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), 0);
#line 1312 "Photo.c"
	result = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
	return result;
}


/* Sets the crop using the raw photo's unrotated coordinate system*/
#line 334 "Photo.vala"
static gboolean photo_set_raw_crop (Photo* self, const Box* crop) {
#line 1321 "Photo.c"
	gboolean result;
	KeyValueMap* map;
	gboolean res;
#line 334 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 1327 "Photo.c"
	map = key_value_map_new ("crop");
#line 336 "Photo.vala"
	key_value_map_set_int (map, "left", (*crop).left);
#line 337 "Photo.vala"
	key_value_map_set_int (map, "top", (*crop).top);
#line 338 "Photo.vala"
	key_value_map_set_int (map, "right", (*crop).right);
#line 339 "Photo.vala"
	key_value_map_set_int (map, "bottom", (*crop).bottom);
#line 1337 "Photo.c"
	res = photo_table_set_transformation (photo_photo_table, &self->priv->photo_id, map);
#line 342 "Photo.vala"
	if (res) {
#line 343 "Photo.vala"
		photo_photo_altered (self);
#line 1343 "Photo.c"
	}
	result = res;
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
	return result;
}


/* Sets the crop, where the crop is in the rotated coordinate system*/
#line 349 "Photo.vala"
gboolean photo_set_crop (Photo* self, const Box* crop) {
#line 1354 "Photo.c"
	gboolean result;
	Dimensions dim;
	Orientation orientation;
	Box derotated;
#line 349 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 1361 "Photo.c"
	/* return crop to photo's coordinate system*/
	dim = photo_table_get_dimensions (photo_photo_table, &self->priv->photo_id);
	orientation = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
	derotated = orientation_derotate_box (orientation, &dim, &(*crop));
#line 355 "Photo.vala"
	g_assert (box_get_width (&derotated) <= dim.width);
#line 356 "Photo.vala"
	g_assert (box_get_height (&derotated) <= dim.height);
#line 1370 "Photo.c"
	result = photo_set_raw_crop (self, &derotated);
	return result;
}


#line 361 "Photo.vala"
gboolean photo_remove_crop (Photo* self) {
#line 1378 "Photo.c"
	gboolean result;
	gboolean res;
#line 361 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 1383 "Photo.c"
	res = photo_table_remove_transformation (photo_photo_table, &self->priv->photo_id, "crop");
#line 363 "Photo.vala"
	if (res) {
#line 364 "Photo.vala"
		photo_photo_altered (self);
#line 1389 "Photo.c"
	}
	result = res;
	return result;
}


#line 369 "Photo.vala"
gboolean photo_add_redeye_instance (Photo* self, const RedeyeInstance* inst_unscaled) {
#line 1398 "Photo.c"
	gboolean result;
	GdkRectangle bounds_rect_unscaled;
	GdkRectangle bounds_rect_raw;
	RedeyeInstance inst;
	KeyValueMap* map;
	gint num_points;
	char* radius_key;
	char* center_key;
	gboolean res;
#line 369 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 1410 "Photo.c"
	bounds_rect_unscaled = redeye_instance_to_bounds_rect (&(*inst_unscaled));
	bounds_rect_raw = photo_unscaled_to_raw_rect (self, &bounds_rect_unscaled);
	inst = redeye_instance_from_bounds_rect (&bounds_rect_raw);
	map = photo_table_get_transformation (photo_photo_table, &self->priv->photo_id, "redeye");
#line 377 "Photo.vala"
	if (map == NULL) {
#line 1417 "Photo.c"
		KeyValueMap* _tmp0_;
		_tmp0_ = NULL;
#line 378 "Photo.vala"
		map = (_tmp0_ = key_value_map_new ("redeye"), (map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL)), _tmp0_);
#line 379 "Photo.vala"
		key_value_map_set_int (map, "num_points", 0);
#line 1424 "Photo.c"
	}
	num_points = key_value_map_get_int (map, "num_points", -1);
#line 383 "Photo.vala"
	g_assert (num_points >= 0);
#line 385 "Photo.vala"
	num_points++;
#line 1431 "Photo.c"
	radius_key = g_strdup_printf ("radius%d", num_points - 1);
	center_key = g_strdup_printf ("center%d", num_points - 1);
#line 390 "Photo.vala"
	key_value_map_set_int (map, radius_key, inst.radius);
#line 391 "Photo.vala"
	key_value_map_set_point (map, center_key, &inst.center);
#line 393 "Photo.vala"
	key_value_map_set_int (map, "num_points", num_points);
#line 1440 "Photo.c"
	res = photo_table_set_transformation (photo_photo_table, &self->priv->photo_id, map);
#line 397 "Photo.vala"
	if (res) {
#line 398 "Photo.vala"
		photo_photo_altered (self);
#line 1446 "Photo.c"
	}
	result = res;
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
	radius_key = (g_free (radius_key), NULL);
	center_key = (g_free (center_key), NULL);
	return result;
}


#line 403 "Photo.vala"
gboolean photo_set_adjustments (Photo* self, GeeArrayList* adjustments) {
#line 1458 "Photo.c"
	gboolean result;
	KeyValueMap* map;
	gboolean res;
#line 403 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), FALSE);
#line 403 "Photo.vala"
	g_return_val_if_fail (GEE_IS_ARRAY_LIST (adjustments), FALSE);
#line 1466 "Photo.c"
	map = photo_table_get_transformation (photo_photo_table, &self->priv->photo_id, "adjustments");
#line 407 "Photo.vala"
	if (map == NULL) {
#line 1470 "Photo.c"
		KeyValueMap* _tmp0_;
		_tmp0_ = NULL;
#line 408 "Photo.vala"
		map = (_tmp0_ = key_value_map_new ("adjustments"), (map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL)), _tmp0_);
#line 1475 "Photo.c"
	}
	{
		GeeIterator* _adjustment_it;
		_adjustment_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (adjustments));
#line 411 "Photo.vala"
		while (TRUE) {
#line 1482 "Photo.c"
			ColorTransformationInstance* _tmp1_;
			ColorTransformationInstance _tmp2_ = {0};
			ColorTransformationInstance adjustment;
#line 411 "Photo.vala"
			if (!gee_iterator_next (_adjustment_it)) {
#line 411 "Photo.vala"
				break;
#line 1490 "Photo.c"
			}
#line 403 "Photo.vala"
			_tmp1_ = NULL;
#line 1494 "Photo.c"
			adjustment = (_tmp2_ = *(_tmp1_ = (ColorTransformationInstance*) gee_iterator_get (_adjustment_it)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (color_transformation_instance_free (_tmp1_), NULL)), _tmp2_);
#line 412 "Photo.vala"
			switch (adjustment.kind) {
#line 1498 "Photo.c"
				case COLOR_TRANSFORMATION_KIND_EXPOSURE:
				{
#line 414 "Photo.vala"
					key_value_map_set_double (map, "exposure", (double) adjustment.parameter);
#line 415 "Photo.vala"
					break;
#line 1505 "Photo.c"
				}
				case COLOR_TRANSFORMATION_KIND_SATURATION:
				{
#line 417 "Photo.vala"
					key_value_map_set_double (map, "saturation", (double) adjustment.parameter);
#line 418 "Photo.vala"
					break;
#line 1513 "Photo.c"
				}
				case COLOR_TRANSFORMATION_KIND_TEMPERATURE:
				{
#line 420 "Photo.vala"
					key_value_map_set_double (map, "temperature", (double) adjustment.parameter);
#line 421 "Photo.vala"
					break;
#line 1521 "Photo.c"
				}
				case COLOR_TRANSFORMATION_KIND_TINT:
				{
#line 423 "Photo.vala"
					key_value_map_set_double (map, "tint", (double) adjustment.parameter);
#line 424 "Photo.vala"
					break;
#line 1529 "Photo.c"
				}
				default:
				{
#line 426 "Photo.vala"
					g_error ("Photo.vala:426: unrecognized ColorTransformationKind enumeration value");
#line 427 "Photo.vala"
					break;
#line 1537 "Photo.c"
				}
			}
		}
		(_adjustment_it == NULL) ? NULL : (_adjustment_it = (g_object_unref (_adjustment_it), NULL));
	}
	res = photo_table_set_transformation (photo_photo_table, &self->priv->photo_id, map);
#line 433 "Photo.vala"
	if (res) {
#line 434 "Photo.vala"
		photo_photo_altered (self);
#line 1548 "Photo.c"
	}
	result = res;
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
	return result;
}


#line 439 "Photo.vala"
float photo_get_adjustment_parameter (Photo* self, ColorTransformationKind adjust_kind) {
#line 1558 "Photo.c"
	float result;
	KeyValueMap* map;
#line 439 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), 0.0F);
#line 1563 "Photo.c"
	map = photo_table_get_transformation (photo_photo_table, &self->priv->photo_id, "adjustments");
#line 443 "Photo.vala"
	if (map == NULL) {
#line 1567 "Photo.c"
		result = 0.0f;
		(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
		return result;
	}
#line 447 "Photo.vala"
	switch (adjust_kind) {
#line 1574 "Photo.c"
		case COLOR_TRANSFORMATION_KIND_EXPOSURE:
		{
			result = (float) key_value_map_get_double (map, "exposure", (double) 0.0f);
			(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
			return result;
		}
		case COLOR_TRANSFORMATION_KIND_SATURATION:
		{
			result = (float) key_value_map_get_double (map, "saturation", (double) 0.0f);
			(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
			return result;
		}
		case COLOR_TRANSFORMATION_KIND_TEMPERATURE:
		{
			result = (float) key_value_map_get_double (map, "temperature", (double) 0.0f);
			(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
			return result;
		}
		case COLOR_TRANSFORMATION_KIND_TINT:
		{
			result = (float) key_value_map_get_double (map, "tint", (double) 0.0f);
			(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
			return result;
		}
		default:
		{
#line 457 "Photo.vala"
			g_error ("Photo.vala:457: unrecognized ColorTransformationKind enumeration value");
#line 458 "Photo.vala"
			break;
#line 1605 "Photo.c"
		}
	}
	result = 0.0f;
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
	return result;
}


#line 463 "Photo.vala"
ColorTransformation* photo_get_composite_transformation (Photo* self) {
#line 1616 "Photo.c"
	ColorTransformation* result;
	float exposure_param;
	float saturation_param;
	float tint_param;
	float temperature_param;
	ColorTransformationFactory* _tmp0_;
	ColorTransformation* _tmp1_;
	ColorTransformation* exposure_transform;
	ColorTransformationFactory* _tmp2_;
	ColorTransformation* _tmp3_;
	ColorTransformation* saturation_transform;
	ColorTransformationFactory* _tmp4_;
	ColorTransformation* _tmp5_;
	ColorTransformation* tint_transform;
	ColorTransformationFactory* _tmp6_;
	ColorTransformation* _tmp7_;
	ColorTransformation* temperature_transform;
	ColorTransformation* _tmp9_;
	ColorTransformation* _tmp8_;
	ColorTransformation* _tmp10_;
	ColorTransformation* composite_transform;
#line 463 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 1640 "Photo.c"
	exposure_param = photo_get_adjustment_parameter (self, COLOR_TRANSFORMATION_KIND_EXPOSURE);
	saturation_param = photo_get_adjustment_parameter (self, COLOR_TRANSFORMATION_KIND_SATURATION);
	tint_param = photo_get_adjustment_parameter (self, COLOR_TRANSFORMATION_KIND_TINT);
	temperature_param = photo_get_adjustment_parameter (self, COLOR_TRANSFORMATION_KIND_TEMPERATURE);
#line 431 "ColorTransformation.vala"
	_tmp0_ = NULL;
#line 474 "Photo.vala"
	_tmp1_ = NULL;
#line 1649 "Photo.c"
	exposure_transform = (_tmp1_ = color_transformation_factory_from_parameter (_tmp0_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_EXPOSURE, exposure_param), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (color_transformation_factory_unref (_tmp0_), NULL)), _tmp1_);
#line 431 "ColorTransformation.vala"
	_tmp2_ = NULL;
#line 477 "Photo.vala"
	_tmp3_ = NULL;
#line 1655 "Photo.c"
	saturation_transform = (_tmp3_ = color_transformation_factory_from_parameter (_tmp2_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_SATURATION, saturation_param), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (color_transformation_factory_unref (_tmp2_), NULL)), _tmp3_);
#line 431 "ColorTransformation.vala"
	_tmp4_ = NULL;
#line 480 "Photo.vala"
	_tmp5_ = NULL;
#line 1661 "Photo.c"
	tint_transform = (_tmp5_ = color_transformation_factory_from_parameter (_tmp4_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_TINT, tint_param), (_tmp4_ == NULL) ? NULL : (_tmp4_ = (color_transformation_factory_unref (_tmp4_), NULL)), _tmp5_);
#line 431 "ColorTransformation.vala"
	_tmp6_ = NULL;
#line 483 "Photo.vala"
	_tmp7_ = NULL;
#line 1667 "Photo.c"
	temperature_transform = (_tmp7_ = color_transformation_factory_from_parameter (_tmp6_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_TEMPERATURE, temperature_param), (_tmp6_ == NULL) ? NULL : (_tmp6_ = (color_transformation_factory_unref (_tmp6_), NULL)), _tmp7_);
#line 129 "ColorTransformation.vala"
	_tmp9_ = NULL;
#line 129 "ColorTransformation.vala"
	_tmp8_ = NULL;
#line 486 "Photo.vala"
	_tmp10_ = NULL;
#line 1675 "Photo.c"
	composite_transform = (_tmp10_ = color_transformation_compose_against (_tmp9_ = color_transformation_compose_against (_tmp8_ = color_transformation_compose_against (exposure_transform, saturation_transform), temperature_transform), tint_transform), (_tmp9_ == NULL) ? NULL : (_tmp9_ = (color_transformation_unref (_tmp9_), NULL)), (_tmp8_ == NULL) ? NULL : (_tmp8_ = (color_transformation_unref (_tmp8_), NULL)), _tmp10_);
	result = composite_transform;
	(exposure_transform == NULL) ? NULL : (exposure_transform = (color_transformation_unref (exposure_transform), NULL));
	(saturation_transform == NULL) ? NULL : (saturation_transform = (color_transformation_unref (saturation_transform), NULL));
	(tint_transform == NULL) ? NULL : (tint_transform = (color_transformation_unref (tint_transform), NULL));
	(temperature_transform == NULL) ? NULL : (temperature_transform = (color_transformation_unref (temperature_transform), NULL));
	return result;
}


#line 495 "Photo.vala"
static RedeyeInstance* photo_get_all_redeye (Photo* self, int* result_length1) {
#line 1688 "Photo.c"
	RedeyeInstance* result;
	KeyValueMap* map;
	RedeyeInstance* _tmp3_;
#line 495 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 1694 "Photo.c"
	map = photo_table_get_transformation (photo_photo_table, &self->priv->photo_id, "redeye");
#line 497 "Photo.vala"
	if (map != NULL) {
#line 1698 "Photo.c"
		gint num_points;
		RedeyeInstance* _tmp0_;
		gint res_size;
		gint res_length1;
		RedeyeInstance* res;
		GdkPoint default_point = {0};
		RedeyeInstance* _tmp2_;
		num_points = key_value_map_get_int (map, "num_points", -1);
#line 499 "Photo.vala"
		g_assert (num_points > 0);
#line 501 "Photo.vala"
		_tmp0_ = NULL;
#line 1711 "Photo.c"
		res = (_tmp0_ = g_new0 (RedeyeInstance, num_points), res_length1 = num_points, res_size = res_length1, _tmp0_);
#line 504 "Photo.vala"
		default_point.x = -1;
#line 505 "Photo.vala"
		default_point.y = -1;
#line 1717 "Photo.c"
		{
			gint i;
			i = 0;
			{
				gboolean _tmp1_;
				_tmp1_ = TRUE;
#line 507 "Photo.vala"
				while (TRUE) {
#line 1726 "Photo.c"
					char* center_key;
					char* radius_key;
#line 507 "Photo.vala"
					if (!_tmp1_) {
#line 507 "Photo.vala"
						i++;
#line 1733 "Photo.c"
					}
#line 507 "Photo.vala"
					_tmp1_ = FALSE;
#line 507 "Photo.vala"
					if (!(i < num_points)) {
#line 507 "Photo.vala"
						break;
#line 1741 "Photo.c"
					}
					center_key = g_strdup_printf ("center%d", i);
					radius_key = g_strdup_printf ("radius%d", i);
#line 511 "Photo.vala"
					res[i].center = key_value_map_get_point (map, center_key, &default_point);
#line 512 "Photo.vala"
					g_assert (res[i].center.x != default_point.x);
#line 513 "Photo.vala"
					g_assert (res[i].center.y != default_point.y);
#line 514 "Photo.vala"
					res[i].radius = key_value_map_get_int (map, radius_key, -1);
#line 515 "Photo.vala"
					g_assert (res[i].radius != (-1));
#line 1755 "Photo.c"
					center_key = (g_free (center_key), NULL);
					radius_key = (g_free (radius_key), NULL);
				}
			}
		}
#line 518 "Photo.vala"
		_tmp2_ = NULL;
#line 1763 "Photo.c"
		result = (_tmp2_ = res, *result_length1 = res_length1, _tmp2_);
		(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
		return result;
		res = (g_free (res), NULL);
	}
#line 521 "Photo.vala"
	_tmp3_ = NULL;
#line 1771 "Photo.c"
	result = (_tmp3_ = g_new0 (RedeyeInstance, 0), *result_length1 = 0, _tmp3_);
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
	return result;
	(map == NULL) ? NULL : (map = (key_value_map_unref (map), NULL));
}


#line 524 "Photo.vala"
static GdkPixbuf* photo_do_redeye (Photo* self, GdkPixbuf* pixbuf, RedeyeInstance* inst) {
#line 1781 "Photo.c"
	GdkPixbuf* result;
	double r;
	gint* _tmp0_;
	gint x_insets_first_quadrant_size;
	gint x_insets_first_quadrant_length1;
	gint* x_insets_first_quadrant;
	gint i;
	gint x_bounds_min;
	gint x_bounds_max;
	gint ymin;
	gint _tmp2_;
	gint ymax;
	gint _tmp3_;
	gint inset_index;
	GdkPixbuf* _tmp16_;
#line 524 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 524 "Photo.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 1801 "Photo.c"
	/* we remove redeye within a circular region called the "effect
	           extent." the effect extent is inscribed within its "bounding
	           rectangle." 
	 for each scanline in the top half-circle of the effect extent,
	           compute the number of pixels by which the effect extent is inset
	           from the edges of its bounding rectangle. note that we only have
	           to do this for the first quadrant because the second quadrant's
	           insets can be derived by symmetry */
	r = (double) (*inst).radius;
#line 537 "Photo.vala"
	_tmp0_ = NULL;
#line 1813 "Photo.c"
	x_insets_first_quadrant = (_tmp0_ = g_new0 (gint, (*inst).radius + 1), x_insets_first_quadrant_length1 = (*inst).radius + 1, x_insets_first_quadrant_size = x_insets_first_quadrant_length1, _tmp0_);
	i = 0;
	{
		double y;
		y = r;
		{
			gboolean _tmp1_;
			_tmp1_ = TRUE;
#line 540 "Photo.vala"
			while (TRUE) {
#line 1824 "Photo.c"
				double theta;
				gint x;
#line 540 "Photo.vala"
				if (!_tmp1_) {
#line 540 "Photo.vala"
					y = y - 1.0;
#line 1831 "Photo.c"
				}
#line 540 "Photo.vala"
				_tmp1_ = FALSE;
#line 540 "Photo.vala"
				if (!(y >= 0.0)) {
#line 540 "Photo.vala"
					break;
#line 1839 "Photo.c"
				}
				theta = asin (y / r);
				x = (gint) ((r * cos (theta)) + 0.5);
#line 543 "Photo.vala"
				x_insets_first_quadrant[i] = (*inst).radius - x;
#line 545 "Photo.vala"
				i++;
#line 1847 "Photo.c"
			}
		}
	}
	x_bounds_min = (*inst).center.x - (*inst).radius;
	x_bounds_max = (*inst).center.x + (*inst).radius;
	ymin = (*inst).center.y - (*inst).radius;
	_tmp2_ = 0;
#line 551 "Photo.vala"
	if (ymin < 0) {
#line 551 "Photo.vala"
		_tmp2_ = 0;
#line 1859 "Photo.c"
	} else {
#line 551 "Photo.vala"
		_tmp2_ = ymin;
#line 1863 "Photo.c"
	}
#line 551 "Photo.vala"
	ymin = _tmp2_;
#line 1867 "Photo.c"
	ymax = (*inst).center.y;
	_tmp3_ = 0;
#line 553 "Photo.vala"
	if (ymax > (gdk_pixbuf_get_height (pixbuf) - 1)) {
#line 553 "Photo.vala"
		_tmp3_ = gdk_pixbuf_get_height (pixbuf) - 1;
#line 1874 "Photo.c"
	} else {
#line 553 "Photo.vala"
		_tmp3_ = ymax;
#line 1878 "Photo.c"
	}
#line 553 "Photo.vala"
	ymax = _tmp3_;
#line 1882 "Photo.c"
	/* iterate over all the pixels in the top half-circle of the effect
	           extent from top to bottom */
	inset_index = 0;
	{
		gint y_it;
		y_it = ymin;
		{
			gboolean _tmp4_;
			_tmp4_ = TRUE;
#line 558 "Photo.vala"
			while (TRUE) {
#line 1894 "Photo.c"
				gint xmin;
				gint _tmp5_;
				gint xmax;
				gint _tmp6_;
#line 558 "Photo.vala"
				if (!_tmp4_) {
#line 558 "Photo.vala"
					y_it++;
#line 1903 "Photo.c"
				}
#line 558 "Photo.vala"
				_tmp4_ = FALSE;
#line 558 "Photo.vala"
				if (!(y_it <= ymax)) {
#line 558 "Photo.vala"
					break;
#line 1911 "Photo.c"
				}
				xmin = x_bounds_min + x_insets_first_quadrant[inset_index];
				_tmp5_ = 0;
#line 560 "Photo.vala"
				if (xmin < 0) {
#line 560 "Photo.vala"
					_tmp5_ = 0;
#line 1919 "Photo.c"
				} else {
#line 560 "Photo.vala"
					_tmp5_ = xmin;
#line 1923 "Photo.c"
				}
#line 560 "Photo.vala"
				xmin = _tmp5_;
#line 1927 "Photo.c"
				xmax = x_bounds_max - x_insets_first_quadrant[inset_index];
				_tmp6_ = 0;
#line 562 "Photo.vala"
				if (xmax > (gdk_pixbuf_get_width (pixbuf) - 1)) {
#line 562 "Photo.vala"
					_tmp6_ = gdk_pixbuf_get_width (pixbuf) - 1;
#line 1934 "Photo.c"
				} else {
#line 562 "Photo.vala"
					_tmp6_ = xmax;
#line 1938 "Photo.c"
				}
#line 562 "Photo.vala"
				xmax = _tmp6_;
#line 1942 "Photo.c"
				{
					gint x_it;
					x_it = xmin;
					{
						gboolean _tmp7_;
						_tmp7_ = TRUE;
#line 564 "Photo.vala"
						while (TRUE) {
#line 1951 "Photo.c"
							GdkPixbuf* _tmp9_;
							GdkPixbuf* _tmp8_;
#line 564 "Photo.vala"
							if (!_tmp7_) {
#line 564 "Photo.vala"
								x_it++;
#line 1958 "Photo.c"
							}
#line 564 "Photo.vala"
							_tmp7_ = FALSE;
#line 564 "Photo.vala"
							if (!(x_it <= xmax)) {
#line 564 "Photo.vala"
								break;
#line 1966 "Photo.c"
							}
#line 590 "Photo.vala"
							_tmp9_ = NULL;
#line 565 "Photo.vala"
							_tmp8_ = NULL;
#line 565 "Photo.vala"
							_tmp9_ = photo_red_reduce_pixel (self, (_tmp8_ = pixbuf, (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_)), x_it, y_it);
#line 1974 "Photo.c"
							(_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL));
						}
					}
				}
#line 567 "Photo.vala"
				inset_index++;
#line 1981 "Photo.c"
			}
		}
	}
	/* iterate over all the pixels in the top half-circle of the effect
	           extent from top to bottom */
#line 572 "Photo.vala"
	ymin = (*inst).center.y;
#line 573 "Photo.vala"
	ymax = (*inst).center.y + (*inst).radius;
#line 574 "Photo.vala"
	inset_index = x_insets_first_quadrant_length1 - 1;
#line 1993 "Photo.c"
	{
		gint y_it;
		y_it = ymin;
		{
			gboolean _tmp10_;
			_tmp10_ = TRUE;
#line 575 "Photo.vala"
			while (TRUE) {
#line 2002 "Photo.c"
				gint xmin;
				gint _tmp11_;
				gint xmax;
				gint _tmp12_;
#line 575 "Photo.vala"
				if (!_tmp10_) {
#line 575 "Photo.vala"
					y_it++;
#line 2011 "Photo.c"
				}
#line 575 "Photo.vala"
				_tmp10_ = FALSE;
#line 575 "Photo.vala"
				if (!(y_it <= ymax)) {
#line 575 "Photo.vala"
					break;
#line 2019 "Photo.c"
				}
				xmin = x_bounds_min + x_insets_first_quadrant[inset_index];
				_tmp11_ = 0;
#line 577 "Photo.vala"
				if (xmin < 0) {
#line 577 "Photo.vala"
					_tmp11_ = 0;
#line 2027 "Photo.c"
				} else {
#line 577 "Photo.vala"
					_tmp11_ = xmin;
#line 2031 "Photo.c"
				}
#line 577 "Photo.vala"
				xmin = _tmp11_;
#line 2035 "Photo.c"
				xmax = x_bounds_max - x_insets_first_quadrant[inset_index];
				_tmp12_ = 0;
#line 579 "Photo.vala"
				if (xmax > (gdk_pixbuf_get_width (pixbuf) - 1)) {
#line 579 "Photo.vala"
					_tmp12_ = gdk_pixbuf_get_width (pixbuf) - 1;
#line 2042 "Photo.c"
				} else {
#line 579 "Photo.vala"
					_tmp12_ = xmax;
#line 2046 "Photo.c"
				}
#line 579 "Photo.vala"
				xmax = _tmp12_;
#line 2050 "Photo.c"
				{
					gint x_it;
					x_it = xmin;
					{
						gboolean _tmp13_;
						_tmp13_ = TRUE;
#line 581 "Photo.vala"
						while (TRUE) {
#line 2059 "Photo.c"
							GdkPixbuf* _tmp15_;
							GdkPixbuf* _tmp14_;
#line 581 "Photo.vala"
							if (!_tmp13_) {
#line 581 "Photo.vala"
								x_it++;
#line 2066 "Photo.c"
							}
#line 581 "Photo.vala"
							_tmp13_ = FALSE;
#line 581 "Photo.vala"
							if (!(x_it <= xmax)) {
#line 581 "Photo.vala"
								break;
#line 2074 "Photo.c"
							}
#line 590 "Photo.vala"
							_tmp15_ = NULL;
#line 582 "Photo.vala"
							_tmp14_ = NULL;
#line 582 "Photo.vala"
							_tmp15_ = photo_red_reduce_pixel (self, (_tmp14_ = pixbuf, (_tmp14_ == NULL) ? NULL : g_object_ref (_tmp14_)), x_it, y_it);
#line 2082 "Photo.c"
							(_tmp15_ == NULL) ? NULL : (_tmp15_ = (g_object_unref (_tmp15_), NULL));
						}
					}
				}
#line 584 "Photo.vala"
				inset_index--;
#line 2089 "Photo.c"
			}
		}
	}
#line 587 "Photo.vala"
	_tmp16_ = NULL;
#line 2095 "Photo.c"
	result = (_tmp16_ = pixbuf, (_tmp16_ == NULL) ? NULL : g_object_ref (_tmp16_));
	x_insets_first_quadrant = (g_free (x_insets_first_quadrant), NULL);
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	return result;
	x_insets_first_quadrant = (g_free (x_insets_first_quadrant), NULL);
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


#line 590 "Photo.vala"
static GdkPixbuf* photo_red_reduce_pixel (Photo* self, GdkPixbuf* pixbuf, gint x, gint y) {
#line 2107 "Photo.c"
	GdkPixbuf* result;
	gint px_start_byte_offset;
	guchar* _tmp0_;
	gint pixel_data_size;
	gint pixel_data_length1;
	guchar* pixel_data;
	guchar g;
	guchar b;
	guchar r;
	GdkPixbuf* _tmp1_;
#line 590 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 590 "Photo.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 2122 "Photo.c"
	px_start_byte_offset = (y * gdk_pixbuf_get_rowstride (pixbuf)) + (x * gdk_pixbuf_get_n_channels (pixbuf));
#line 594 "Photo.vala"
	_tmp0_ = NULL;
#line 2126 "Photo.c"
	pixel_data = (_tmp0_ = gdk_pixbuf_get_pixels (pixbuf), pixel_data_length1 = -1, pixel_data_size = pixel_data_length1, _tmp0_);
	/* The pupil of the human eye has no pigment, so we expect all
	           color channels to be of about equal intensity. This means that at
	           any point within the effects region, the value of the red channel
	           should be about the same as the values of the green and blue
	           channels. So set the value of the red channel to be the mean of the
	           values of the red and blue channels. This preserves achromatic
	           intensity across all channels while eliminating any extraneous flare
	           affecting the red channel only (i.e. the red-eye effect). */
	g = pixel_data[px_start_byte_offset + 1];
	b = pixel_data[px_start_byte_offset + 2];
	r = (guchar) ((g + b) / 2);
#line 609 "Photo.vala"
	pixel_data[px_start_byte_offset] = r;
#line 611 "Photo.vala"
	_tmp1_ = NULL;
#line 2143 "Photo.c"
	result = (_tmp1_ = pixbuf, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	return result;
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


/* Retrieves a full-sized pixbuf for the Photo with all modifications, except those specified*/
#line 615 "Photo.vala"
GdkPixbuf* photo_get_pixbuf (Photo* self, gint exceptions, gint scale, GdkInterpType interp, GError** error) {
#line 2154 "Photo.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	GdkPixbuf* pixbuf;
	gboolean _tmp0_;
#line 615 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 2161 "Photo.c"
	_inner_error_ = NULL;
	pixbuf = NULL;
	_tmp0_ = FALSE;
#line 627 "Photo.vala"
	if (photo_cached_raw != NULL) {
#line 627 "Photo.vala"
		_tmp0_ = photo_cached_photo_id.id == self->priv->photo_id.id;
#line 2169 "Photo.c"
	} else {
#line 627 "Photo.vala"
		_tmp0_ = FALSE;
#line 2173 "Photo.c"
	}
#line 627 "Photo.vala"
	if (_tmp0_) {
#line 2177 "Photo.c"
		GdkPixbuf* _tmp1_;
		/* used the cached raw pixbuf, which is merely the last loaded pixbuf*/
		_tmp1_ = NULL;
#line 632 "Photo.vala"
		pixbuf = (_tmp1_ = gdk_pixbuf_copy (photo_cached_raw), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 2183 "Photo.c"
	} else {
		GFile* file;
		char* _tmp2_;
		char* _tmp3_;
		GdkPixbuf* _tmp4_;
		GdkPixbuf* _tmp5_;
		GdkPixbuf* _tmp6_;
		GdkPixbuf* _tmp7_;
		file = photo_get_file (self);
#line 952 "gio-2.0.vapi"
		_tmp2_ = NULL;
#line 639 "Photo.vala"
		g_debug ("Photo.vala:639: Loading raw photo %s", _tmp2_ = g_file_get_path (file));
#line 2197 "Photo.c"
		_tmp2_ = (g_free (_tmp2_), NULL);
#line 952 "gio-2.0.vapi"
		_tmp3_ = NULL;
#line 643 "Photo.vala"
		_tmp4_ = NULL;
#line 2203 "Photo.c"
		_tmp5_ = (_tmp4_ = gdk_pixbuf_new_from_file (_tmp3_ = g_file_get_path (file), &_inner_error_), _tmp3_ = (g_free (_tmp3_), NULL), _tmp4_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			return NULL;
		}
		_tmp6_ = NULL;
#line 643 "Photo.vala"
		pixbuf = (_tmp6_ = _tmp5_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp6_);
#line 2214 "Photo.c"
		/* stash for next time*/
		_tmp7_ = NULL;
#line 652 "Photo.vala"
		photo_cached_raw = (_tmp7_ = gdk_pixbuf_copy (pixbuf), (photo_cached_raw == NULL) ? NULL : (photo_cached_raw = (g_object_unref (photo_cached_raw), NULL)), _tmp7_);
#line 656 "Photo.vala"
		photo_cached_photo_id = self->priv->photo_id;
#line 2221 "Photo.c"
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	}
	/*
	 Image modification pipeline
	
	 redeye reduction*/
#line 664 "Photo.vala"
	if ((exceptions & PHOTO_EXCEPTION_REDEYE) == 0) {
#line 2230 "Photo.c"
		RedeyeInstance* _tmp9_;
		gint redeye_instances_size;
		gint redeye_instances_length1;
		gint _tmp8_;
		RedeyeInstance* redeye_instances;
#line 668 "Photo.vala"
		_tmp9_ = NULL;
#line 2238 "Photo.c"
		redeye_instances = (_tmp9_ = photo_get_all_redeye (self, &_tmp8_), redeye_instances_length1 = _tmp8_, redeye_instances_size = redeye_instances_length1, _tmp9_);
		{
			gint i;
			i = 0;
			{
				gboolean _tmp10_;
				_tmp10_ = TRUE;
#line 669 "Photo.vala"
				while (TRUE) {
#line 2248 "Photo.c"
					GdkPixbuf* _tmp13_;
					RedeyeInstance _tmp12_ = {0};
					GdkPixbuf* _tmp11_;
#line 669 "Photo.vala"
					if (!_tmp10_) {
#line 669 "Photo.vala"
						i++;
#line 2256 "Photo.c"
					}
#line 669 "Photo.vala"
					_tmp10_ = FALSE;
#line 669 "Photo.vala"
					if (!(i < redeye_instances_length1)) {
#line 669 "Photo.vala"
						break;
#line 2264 "Photo.c"
					}
					_tmp13_ = NULL;
#line 670 "Photo.vala"
					_tmp11_ = NULL;
#line 670 "Photo.vala"
					pixbuf = (_tmp13_ = photo_do_redeye (self, (_tmp11_ = pixbuf, (_tmp11_ == NULL) ? NULL : g_object_ref (_tmp11_)), (_tmp12_ = redeye_instances[i], &_tmp12_)), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp13_);
#line 2271 "Photo.c"
				}
			}
		}
		redeye_instances = (g_free (redeye_instances), NULL);
	}
	/* crop*/
#line 678 "Photo.vala"
	if ((exceptions & PHOTO_EXCEPTION_CROP) == 0) {
#line 2280 "Photo.c"
		Box crop = {0};
#line 683 "Photo.vala"
		if (photo_get_raw_crop (self, &crop)) {
#line 2284 "Photo.c"
			GdkPixbuf* _tmp14_;
			_tmp14_ = NULL;
#line 684 "Photo.vala"
			pixbuf = (_tmp14_ = gdk_pixbuf_new_subpixbuf (pixbuf, crop.left, crop.top, box_get_width (&crop), box_get_height (&crop)), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp14_);
#line 2289 "Photo.c"
		}
	}
	/* scale*/
#line 693 "Photo.vala"
	if (scale > 0) {
#line 2295 "Photo.c"
		GdkPixbuf* _tmp15_;
		_tmp15_ = NULL;
#line 697 "Photo.vala"
		pixbuf = (_tmp15_ = scale_pixbuf (pixbuf, scale, interp), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp15_);
#line 2300 "Photo.c"
	}
	/* color adjustment*/
#line 704 "Photo.vala"
	if ((exceptions & PHOTO_EXCEPTION_ADJUST) == 0) {
#line 2305 "Photo.c"
		ColorTransformation* composite_transform;
		composite_transform = photo_get_composite_transformation (self);
#line 710 "Photo.vala"
		if (!color_transformation_is_identity (composite_transform)) {
#line 2310 "Photo.c"
			GdkPixbuf* _tmp16_;
#line 711 "Photo.vala"
			_tmp16_ = NULL;
#line 711 "Photo.vala"
			color_transformation_transform_pixbuf (composite_transform, (_tmp16_ = pixbuf, (_tmp16_ == NULL) ? NULL : g_object_ref (_tmp16_)));
#line 2316 "Photo.c"
		}
		(composite_transform == NULL) ? NULL : (composite_transform = (color_transformation_unref (composite_transform), NULL));
	}
	/* orientation (all modifications are stored in unrotated coordinate system)*/
#line 719 "Photo.vala"
	if ((exceptions & PHOTO_EXCEPTION_ORIENTATION) == 0) {
#line 2323 "Photo.c"
		Orientation orientation;
		GdkPixbuf* _tmp18_;
		GdkPixbuf* _tmp17_;
		orientation = photo_table_get_orientation (photo_photo_table, &self->priv->photo_id);
		_tmp18_ = NULL;
#line 724 "Photo.vala"
		_tmp17_ = NULL;
#line 724 "Photo.vala"
		pixbuf = (_tmp18_ = orientation_rotate_pixbuf (orientation, (_tmp17_ = pixbuf, (_tmp17_ == NULL) ? NULL : g_object_ref (_tmp17_))), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp18_);
#line 2333 "Photo.c"
	}
	result = pixbuf;
	return result;
}


#line 741 "Photo.vala"
static GFile* photo_generate_exportable_file (Photo* self, GError** error) {
#line 2342 "Photo.c"
	GFile* result;
	GFile* original_file;
	GFile* exportable_dir;
	time_t timestamp;
	char* _tmp7_;
	GFile* _tmp8_;
#line 741 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 2351 "Photo.c"
	original_file = photo_get_file (self);
	exportable_dir = app_window_get_data_subdir ("export", NULL);
	/* use exposure time, then file modified time, for directory (to prevent name collision)*/
	timestamp = photo_get_exposure_time (self);
#line 748 "Photo.vala"
	if (timestamp == 0) {
#line 749 "Photo.vala"
		timestamp = photo_get_timestamp (self);
#line 2360 "Photo.c"
	}
#line 751 "Photo.vala"
	if (timestamp != 0) {
#line 2364 "Photo.c"
		struct tm _tmp0_ = {0};
		struct tm tm;
		GFile* _tmp2_;
		char* _tmp1_;
		GFile* _tmp4_;
		char* _tmp3_;
		GFile* _tmp6_;
		char* _tmp5_;
		tm = (localtime_r (&timestamp, &_tmp0_), _tmp0_);
		_tmp2_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 753 "Photo.vala"
		exportable_dir = (_tmp2_ = g_file_get_child (exportable_dir, _tmp1_ = g_strdup_printf ("%04u", (guint) (tm.tm_year + 1900))), (exportable_dir == NULL) ? NULL : (exportable_dir = (g_object_unref (exportable_dir), NULL)), _tmp2_);
#line 2379 "Photo.c"
		_tmp1_ = (g_free (_tmp1_), NULL);
		_tmp4_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp3_ = NULL;
#line 754 "Photo.vala"
		exportable_dir = (_tmp4_ = g_file_get_child (exportable_dir, _tmp3_ = g_strdup_printf ("%02u", (guint) (tm.tm_mon + 1))), (exportable_dir == NULL) ? NULL : (exportable_dir = (g_object_unref (exportable_dir), NULL)), _tmp4_);
#line 2386 "Photo.c"
		_tmp3_ = (g_free (_tmp3_), NULL);
		_tmp6_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp5_ = NULL;
#line 755 "Photo.vala"
		exportable_dir = (_tmp6_ = g_file_get_child (exportable_dir, _tmp5_ = g_strdup_printf ("%02u", (guint) tm.tm_mday)), (exportable_dir == NULL) ? NULL : (exportable_dir = (g_object_unref (exportable_dir), NULL)), _tmp6_);
#line 2393 "Photo.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
	}
#line 947 "gio-2.0.vapi"
	_tmp7_ = NULL;
#line 758 "Photo.vala"
	_tmp8_ = NULL;
#line 2400 "Photo.c"
	result = (_tmp8_ = g_file_get_child (exportable_dir, _tmp7_ = g_file_get_basename (original_file)), _tmp7_ = (g_free (_tmp7_), NULL), _tmp8_);
	(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
	(exportable_dir == NULL) ? NULL : (exportable_dir = (g_object_unref (exportable_dir), NULL));
	return result;
	(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
	(exportable_dir == NULL) ? NULL : (exportable_dir = (g_object_unref (exportable_dir), NULL));
}


#line 761 "Photo.vala"
static void photo_copy_exported_exif (Photo* self, PhotoExif* source, PhotoExif* dest, Orientation orientation, const Dimensions* dim, GError** error) {
#line 2412 "Photo.c"
	GError * _inner_error_;
	ExifData* _tmp0_;
#line 761 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 761 "Photo.vala"
	g_return_if_fail (IS_PHOTO_EXIF (source));
#line 761 "Photo.vala"
	g_return_if_fail (IS_PHOTO_EXIF (dest));
#line 2421 "Photo.c"
	_inner_error_ = NULL;
#line 763 "Photo.vala"
	if (!photo_exif_has_exif (source)) {
#line 2425 "Photo.c"
		return;
	}
#line 151 "Exif.vala"
	_tmp0_ = NULL;
#line 766 "Photo.vala"
	photo_exif_set_exif (dest, _tmp0_ = photo_exif_get_exif (source));
#line 2432 "Photo.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (exif_data_unref (_tmp0_), NULL));
#line 767 "Photo.vala"
	photo_exif_set_dimensions (dest, &(*dim));
#line 768 "Photo.vala"
	photo_exif_set_orientation (dest, orientation);
#line 769 "Photo.vala"
	photo_exif_remove_all_tags (dest, EXIF_TAG_RELATED_IMAGE_WIDTH);
#line 770 "Photo.vala"
	photo_exif_remove_all_tags (dest, EXIF_TAG_RELATED_IMAGE_LENGTH);
#line 771 "Photo.vala"
	photo_exif_remove_thumbnail (dest);
#line 772 "Photo.vala"
	photo_exif_commit (dest, &_inner_error_);
#line 2446 "Photo.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
}


/* Returns a file appropriate for export.  The file should NOT be deleted once it's been used.

 TODO: Lossless transformations, especially for mere rotations of JFIF files.*/
#line 778 "Photo.vala"
GFile* photo_generate_exportable (Photo* self, GError** error) {
#line 2459 "Photo.c"
	GFile* result;
	GError * _inner_error_;
	GFile* dest_file;
	GFile* dest_dir;
	GFile* original_file;
	GFile* _tmp0_;
	PhotoExif* _tmp1_;
	PhotoExif* original_exif;
	gboolean _tmp2_;
#line 778 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 2471 "Photo.c"
	_inner_error_ = NULL;
#line 779 "Photo.vala"
	if (!photo_has_transformations (self)) {
#line 2475 "Photo.c"
		result = photo_get_file (self);
		return result;
	}
	dest_file = photo_generate_exportable_file (self, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 783 "Photo.vala"
	if (g_file_query_exists (dest_file, NULL)) {
#line 2486 "Photo.c"
		result = dest_file;
		return result;
	}
	/* generate_exportable_file only generates a filename; create directory if necessary*/
	dest_dir = g_file_get_parent (dest_file);
#line 788 "Photo.vala"
	if (!g_file_query_exists (dest_dir, NULL)) {
#line 789 "Photo.vala"
		g_file_make_directory_with_parents (dest_dir, NULL, &_inner_error_);
#line 2496 "Photo.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
			(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
			return NULL;
		}
	}
	original_file = photo_get_file (self);
#line 167 "Photo.vala"
	_tmp0_ = NULL;
#line 792 "Photo.vala"
	_tmp1_ = NULL;
#line 2509 "Photo.c"
	original_exif = (_tmp1_ = photo_exif_new (_tmp0_ = photo_get_file (self)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	_tmp2_ = FALSE;
#line 795 "Photo.vala"
	if (!photo_table_has_transformations (photo_photo_table, &self->priv->photo_id)) {
#line 795 "Photo.vala"
		_tmp2_ = photo_exif_has_exif (original_exif);
#line 2516 "Photo.c"
	} else {
#line 795 "Photo.vala"
		_tmp2_ = FALSE;
#line 2520 "Photo.c"
	}
	/* if only rotated, only need to copy and modify the EXIF*/
#line 795 "Photo.vala"
	if (_tmp2_) {
#line 2525 "Photo.c"
		PhotoExif* dest_exif;
#line 796 "Photo.vala"
		g_file_copy (original_file, dest_file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &_inner_error_);
#line 2529 "Photo.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
			(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
			(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
			(original_exif == NULL) ? NULL : (original_exif = (photo_exif_unref (original_exif), NULL));
			return NULL;
		}
		dest_exif = photo_exif_new (dest_file);
#line 799 "Photo.vala"
		photo_exif_set_orientation (dest_exif, photo_table_get_orientation (photo_photo_table, &self->priv->photo_id));
#line 800 "Photo.vala"
		photo_exif_commit (dest_exif, &_inner_error_);
#line 2543 "Photo.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(dest_exif == NULL) ? NULL : (dest_exif = (photo_exif_unref (dest_exif), NULL));
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
			(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
			(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
			(original_exif == NULL) ? NULL : (original_exif = (photo_exif_unref (original_exif), NULL));
			return NULL;
		}
		(dest_exif == NULL) ? NULL : (dest_exif = (photo_exif_unref (dest_exif), NULL));
	} else {
		GdkPixbuf* pixbuf;
		char* _tmp4_;
		char* _tmp3_;
		Dimensions _tmp6_ = {0};
		PhotoExif* _tmp5_;
		pixbuf = photo_get_pixbuf (self, PHOTO_EXCEPTION_NONE, 0, GDK_INTERP_HYPER, &_inner_error_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
			(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
			(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
			(original_exif == NULL) ? NULL : (original_exif = (photo_exif_unref (original_exif), NULL));
			return NULL;
		}
#line 150 "image_util.vala"
		_tmp4_ = NULL;
#line 952 "gio-2.0.vapi"
		_tmp3_ = NULL;
#line 803 "Photo.vala"
		gdk_pixbuf_save (pixbuf, _tmp3_ = g_file_get_path (dest_file), "jpeg", &_inner_error_, "quality", _tmp4_ = jpeg_quality_get_pct_text (PHOTO_EXPORT_JPEG_QUALITY), NULL);
#line 2575 "Photo.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
			(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
			(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
			(original_exif == NULL) ? NULL : (original_exif = (photo_exif_unref (original_exif), NULL));
			return NULL;
		}
		_tmp4_ = (g_free (_tmp4_), NULL);
		_tmp3_ = (g_free (_tmp3_), NULL);
		_tmp5_ = NULL;
#line 804 "Photo.vala"
		photo_copy_exported_exif (self, original_exif, _tmp5_ = photo_exif_new (dest_file), ORIENTATION_TOP_LEFT, (_tmp6_ = dimensions_for_pixbuf (pixbuf), &_tmp6_), &_inner_error_);
#line 2590 "Photo.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			(dest_file == NULL) ? NULL : (dest_file = (g_object_unref (dest_file), NULL));
			(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
			(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
			(original_exif == NULL) ? NULL : (original_exif = (photo_exif_unref (original_exif), NULL));
			return NULL;
		}
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (photo_exif_unref (_tmp5_), NULL));
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	}
	result = dest_file;
	(dest_dir == NULL) ? NULL : (dest_dir = (g_object_unref (dest_dir), NULL));
	(original_file == NULL) ? NULL : (original_file = (g_object_unref (original_file), NULL));
	(original_exif == NULL) ? NULL : (original_exif = (photo_exif_unref (original_exif), NULL));
	return result;
}


/* Writes a file appropriate for export meeting the specified parameters.

 TODO: Lossless transformations, especially for mere rotations of JFIF files.*/
#line 814 "Photo.vala"
void photo_export (Photo* self, GFile* dest_file, gint scale, ScaleConstraint constraint, JpegQuality quality, GError** error) {
#line 2616 "Photo.c"
	GError * _inner_error_;
	GdkPixbuf* pixbuf;
	Dimensions dim;
	Dimensions scaled;
	gboolean _tmp0_;
	char* _tmp3_;
	char* _tmp2_;
	PhotoExif* _tmp6_;
	PhotoExif* _tmp5_;
	GFile* _tmp4_;
#line 814 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 814 "Photo.vala"
	g_return_if_fail (G_IS_FILE (dest_file));
#line 2631 "Photo.c"
	_inner_error_ = NULL;
#line 816 "Photo.vala"
	if (constraint == SCALE_CONSTRAINT_ORIGINAL) {
#line 2635 "Photo.c"
		GFile* exportable;
		/* generate a raw exportable file and copy that*/
		exportable = photo_generate_exportable (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			return;
		}
#line 820 "Photo.vala"
		g_file_copy (exportable, dest_file, G_FILE_COPY_OVERWRITE | G_FILE_COPY_ALL_METADATA, NULL, NULL, NULL, &_inner_error_);
#line 2645 "Photo.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(exportable == NULL) ? NULL : (exportable = (g_object_unref (exportable), NULL));
			return;
		}
		(exportable == NULL) ? NULL : (exportable = (g_object_unref (exportable), NULL));
		return;
	}
	pixbuf = photo_get_pixbuf (self, PHOTO_EXCEPTION_NONE, 0, GDK_INTERP_HYPER, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
	dim = dimensions_for_pixbuf (pixbuf);
	scaled = dimensions_get_scaled_by_constraint (&dim, scale, constraint);
	_tmp0_ = FALSE;
#line 832 "Photo.vala"
	if (dim.width != scaled.width) {
#line 832 "Photo.vala"
		_tmp0_ = TRUE;
#line 2666 "Photo.c"
	} else {
#line 832 "Photo.vala"
		_tmp0_ = dim.height != scaled.height;
#line 2670 "Photo.c"
	}
	/* only scale if necessary ... although scale_simple probably catches this, it's an easy
	 check to avoid image loss*/
#line 832 "Photo.vala"
	if (_tmp0_) {
#line 2676 "Photo.c"
		GdkPixbuf* _tmp1_;
		_tmp1_ = NULL;
#line 833 "Photo.vala"
		pixbuf = (_tmp1_ = gdk_pixbuf_scale_simple (pixbuf, scaled.width, scaled.height, PHOTO_EXPORT_INTERP), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 2681 "Photo.c"
	}
#line 150 "image_util.vala"
	_tmp3_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp2_ = NULL;
#line 835 "Photo.vala"
	gdk_pixbuf_save (pixbuf, _tmp2_ = g_file_get_path (dest_file), "jpeg", &_inner_error_, "quality", _tmp3_ = jpeg_quality_get_pct_text (quality), NULL);
#line 2689 "Photo.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		return;
	}
	_tmp3_ = (g_free (_tmp3_), NULL);
	_tmp2_ = (g_free (_tmp2_), NULL);
	_tmp6_ = NULL;
	_tmp5_ = NULL;
#line 167 "Photo.vala"
	_tmp4_ = NULL;
#line 836 "Photo.vala"
	photo_copy_exported_exif (self, _tmp5_ = photo_exif_new (_tmp4_ = photo_get_file (self)), _tmp6_ = photo_exif_new (dest_file), ORIENTATION_TOP_LEFT, &scaled, &_inner_error_);
#line 2703 "Photo.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		return;
	}
	(_tmp6_ == NULL) ? NULL : (_tmp6_ = (photo_exif_unref (_tmp6_), NULL));
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (photo_exif_unref (_tmp5_), NULL));
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


/* Returns unscaled thumbnail with all modifications applied applicable to the scale*/
#line 841 "Photo.vala"
GdkPixbuf* photo_get_thumbnail (Photo* self, gint scale) {
#line 2719 "Photo.c"
	GdkPixbuf* result;
#line 841 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), NULL);
#line 2723 "Photo.c"
	result = thumbnail_cache_fetch (&self->priv->photo_id, scale);
	return result;
}


#line 845 "Photo.vala"
void photo_remove (Photo* self, gboolean remove_original) {
#line 845 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 2733 "Photo.c"
	/* signal all interested parties prior to removal from map*/
#line 847 "Photo.vala"
	g_signal_emit_by_name (self, "removed");
#line 2737 "Photo.c"
	/* remove all cached thumbnails*/
#line 850 "Photo.vala"
	thumbnail_cache_remove (&self->priv->photo_id);
#line 2741 "Photo.c"
	/* remove exportable file*/
#line 853 "Photo.vala"
	photo_remove_exportable_file (self);
#line 2745 "Photo.c"
	/* remove original*/
#line 856 "Photo.vala"
	if (remove_original) {
#line 857 "Photo.vala"
		photo_remove_original_file (self);
#line 2751 "Photo.c"
	}
	/* remove from photo table -- should be wiped from storage now (other classes may have added
	 photo_id to other parts of the database ... it's their responsibility to remove them
	 when removed() is called)*/
#line 862 "Photo.vala"
	photo_table_remove (photo_photo_table, &self->priv->photo_id);
#line 2758 "Photo.c"
	/* remove from global map*/
#line 865 "Photo.vala"
	gee_abstract_map_remove (GEE_ABSTRACT_MAP (photo_photo_map), &self->priv->photo_id.id, NULL);
#line 2762 "Photo.c"
}


#line 868 "Photo.vala"
static void photo_photo_altered (Photo* self) {
#line 2768 "Photo.c"
	GError * _inner_error_;
	GdkPixbuf* pixbuf;
#line 868 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 2773 "Photo.c"
	_inner_error_ = NULL;
#line 869 "Photo.vala"
	g_signal_emit_by_name (self, "altered");
#line 2777 "Photo.c"
	/* load transformed image for thumbnail generation*/
	pixbuf = NULL;
	{
		GdkPixbuf* _tmp0_;
		GdkPixbuf* _tmp1_;
		_tmp0_ = photo_get_pixbuf (self, PHOTO_EXCEPTION_NONE, 0, GDK_INTERP_HYPER, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch33_g_error;
			goto __finally33;
		}
		_tmp1_ = NULL;
#line 874 "Photo.vala"
		pixbuf = (_tmp1_ = _tmp0_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 2791 "Photo.c"
	}
	goto __finally33;
	__catch33_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 876 "Photo.vala"
			g_error ("Photo.vala:876: %s", err->message);
#line 2802 "Photo.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally33:
	if (_inner_error_ != NULL) {
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
#line 879 "Photo.vala"
	thumbnail_cache_import (&self->priv->photo_id, pixbuf, TRUE);
#line 881 "Photo.vala"
	g_signal_emit_by_name (self, "thumbnail-altered");
#line 2817 "Photo.c"
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


#line 884 "Photo.vala"
static void photo_remove_exportable_file (Photo* self) {
#line 2824 "Photo.c"
	GError * _inner_error_;
	GFile* file;
#line 884 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 2829 "Photo.c"
	_inner_error_ = NULL;
	file = NULL;
	{
		GFile* _tmp0_;
		GFile* _tmp1_;
		_tmp0_ = photo_generate_exportable_file (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch34_g_error;
			goto __finally34;
		}
		_tmp1_ = NULL;
#line 887 "Photo.vala"
		file = (_tmp1_ = _tmp0_, (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp1_);
#line 888 "Photo.vala"
		if (g_file_query_exists (file, NULL)) {
#line 889 "Photo.vala"
			g_file_delete (file, NULL, &_inner_error_);
#line 2847 "Photo.c"
			if (_inner_error_ != NULL) {
				goto __catch34_g_error;
				goto __finally34;
			}
		}
	}
	goto __finally34;
	__catch34_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 891 "Photo.vala"
			if (file != NULL) {
#line 2863 "Photo.c"
				char* _tmp2_;
#line 952 "gio-2.0.vapi"
				_tmp2_ = NULL;
#line 892 "Photo.vala"
				g_message ("Photo.vala:892: Unable to delete exportable photo file %s: %s", _tmp2_ = g_file_get_path (file), err->message);
#line 2869 "Photo.c"
				_tmp2_ = (g_free (_tmp2_), NULL);
			} else {
				char* _tmp3_;
#line 208 "Photo.vala"
				_tmp3_ = NULL;
#line 894 "Photo.vala"
				g_message ("Photo.vala:894: Unable to generate exportable filename for %s", _tmp3_ = photo_to_string (self));
#line 2877 "Photo.c"
				_tmp3_ = (g_free (_tmp3_), NULL);
			}
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally34:
	if (_inner_error_ != NULL) {
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
}


#line 898 "Photo.vala"
static void photo_remove_original_file (Photo* self) {
#line 2896 "Photo.c"
	GError * _inner_error_;
	GFile* file;
	char* _tmp0_;
	GFile* _tmp2_;
	gboolean _tmp3_;
#line 898 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 2904 "Photo.c"
	_inner_error_ = NULL;
	file = photo_get_file (self);
#line 952 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 901 "Photo.vala"
	g_debug ("Photo.vala:901: Deleting original photo file %s", _tmp0_ = g_file_get_path (file));
#line 2911 "Photo.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	{
#line 904 "Photo.vala"
		g_file_delete (file, NULL, &_inner_error_);
#line 2916 "Photo.c"
		if (_inner_error_ != NULL) {
			goto __catch35_g_error;
			goto __finally35;
		}
	}
	goto __finally35;
	__catch35_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp1_;
			/* log error but don't abend, as this is not fatal to operation (also, could be
			 the photo is removed because it could not be found during a verify)*/
#line 952 "gio-2.0.vapi"
			_tmp1_ = NULL;
#line 908 "Photo.vala"
			g_message ("Photo.vala:908: Unable to delete original photo %s: %s", _tmp1_ = g_file_get_path (file), err->message);
#line 2936 "Photo.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally35:
	if (_inner_error_ != NULL) {
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* remove empty directories corresponding to imported path, but only if file is located
	 inside the user's Pictures directory*/
#line 466 "AppWindow.vala"
	_tmp2_ = NULL;
#line 913 "Photo.vala"
	if ((_tmp3_ = g_file_has_prefix (file, _tmp2_ = app_window_get_photos_dir ()), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL)), _tmp3_)) {
#line 2954 "Photo.c"
		GFile* _tmp4_;
		GFile* parent;
#line 914 "Photo.vala"
		_tmp4_ = NULL;
#line 2959 "Photo.c"
		parent = (_tmp4_ = file, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
		{
			gint depth;
			depth = 0;
			{
				gboolean _tmp5_;
				_tmp5_ = TRUE;
#line 915 "Photo.vala"
				while (TRUE) {
#line 2969 "Photo.c"
					GFile* _tmp6_;
					gboolean _tmp7_;
#line 915 "Photo.vala"
					if (!_tmp5_) {
#line 915 "Photo.vala"
						depth++;
#line 2976 "Photo.c"
					}
#line 915 "Photo.vala"
					_tmp5_ = FALSE;
#line 915 "Photo.vala"
					if (!(depth < BATCH_IMPORT_IMPORT_DIRECTORY_DEPTH)) {
#line 915 "Photo.vala"
						break;
#line 2984 "Photo.c"
					}
					_tmp6_ = NULL;
#line 916 "Photo.vala"
					parent = (_tmp6_ = g_file_get_parent (parent), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp6_);
#line 917 "Photo.vala"
					if (parent == NULL) {
#line 918 "Photo.vala"
						break;
#line 2993 "Photo.c"
					}
					_tmp7_ = query_is_directory_empty (parent, &_inner_error_);
					if (_inner_error_ != NULL) {
						(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
						(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
#line 920 "Photo.vala"
					if (!_tmp7_) {
#line 921 "Photo.vala"
						break;
#line 3007 "Photo.c"
					}
					{
						char* _tmp8_;
#line 924 "Photo.vala"
						g_file_delete (parent, NULL, &_inner_error_);
#line 3013 "Photo.c"
						if (_inner_error_ != NULL) {
							goto __catch36_g_error;
							goto __finally36;
						}
#line 952 "gio-2.0.vapi"
						_tmp8_ = NULL;
#line 925 "Photo.vala"
						g_debug ("Photo.vala:925: Deleted empty directory %s", _tmp8_ = g_file_get_path (parent));
#line 3022 "Photo.c"
						_tmp8_ = (g_free (_tmp8_), NULL);
					}
					goto __finally36;
					__catch36_g_error:
					{
						GError * err;
						err = _inner_error_;
						_inner_error_ = NULL;
						{
							char* _tmp9_;
							/* again, log error but don't abend*/
#line 952 "gio-2.0.vapi"
							_tmp9_ = NULL;
#line 928 "Photo.vala"
							g_message ("Photo.vala:928: Unable to delete empty directory %s: %s", _tmp9_ = g_file_get_path (parent), err->message);
#line 3038 "Photo.c"
							_tmp9_ = (g_free (_tmp9_), NULL);
							(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
						}
					}
					__finally36:
					if (_inner_error_ != NULL) {
						(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
						(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
			}
		}
		(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
	}
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
}


#line 935 "Photo.vala"
PhotoCurrency photo_check_currency (Photo* self) {
#line 3062 "Photo.c"
	PhotoCurrency result;
	GError * _inner_error_;
	GFileInfo* info;
	GTimeVal _tmp4_ = {0};
	GTimeVal timestamp;
	gboolean _tmp5_;
#line 935 "Photo.vala"
	g_return_val_if_fail (IS_PHOTO (self), 0);
#line 3071 "Photo.c"
	_inner_error_ = NULL;
	info = NULL;
	{
		GFile* _tmp0_;
		GFileInfo* _tmp1_;
		GFileInfo* _tmp2_;
		GFileInfo* _tmp3_;
#line 167 "Photo.vala"
		_tmp0_ = NULL;
#line 938 "Photo.vala"
		_tmp1_ = NULL;
#line 3083 "Photo.c"
		_tmp2_ = (_tmp1_ = g_file_query_info (_tmp0_ = photo_get_file (self), "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
		if (_inner_error_ != NULL) {
			goto __catch37_g_error;
			goto __finally37;
		}
		_tmp3_ = NULL;
#line 938 "Photo.vala"
		info = (_tmp3_ = _tmp2_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp3_);
#line 3092 "Photo.c"
	}
	goto __finally37;
	__catch37_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			/* treat this as the file has been deleted from the filesystem*/
			result = PHOTO_CURRENCY_GONE;
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			return result;
		}
	}
	__finally37:
	if (_inner_error_ != NULL) {
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	timestamp = (g_get_current_time (&_tmp4_), _tmp4_);
#line 945 "Photo.vala"
	g_file_info_get_modification_time (info, &timestamp);
#line 3118 "Photo.c"
	_tmp5_ = FALSE;
#line 948 "Photo.vala"
	if (timestamp.tv_sec != photo_get_timestamp (self)) {
#line 948 "Photo.vala"
		_tmp5_ = TRUE;
#line 3124 "Photo.c"
	} else {
#line 948 "Photo.vala"
		_tmp5_ = g_file_info_get_size (info) != photo_table_get_filesize (photo_photo_table, &self->priv->photo_id);
#line 3128 "Photo.c"
	}
	/* trust modification time and file size*/
#line 948 "Photo.vala"
	if (_tmp5_) {
#line 3133 "Photo.c"
		result = PHOTO_CURRENCY_DIRTY;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		return result;
	}
	/* verify thumbnail cache is all set*/
#line 952 "Photo.vala"
	if (!thumbnail_cache_exists (&self->priv->photo_id)) {
#line 3141 "Photo.c"
		result = PHOTO_CURRENCY_DIRTY;
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		return result;
	}
	result = PHOTO_CURRENCY_CURRENT;
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	return result;
}


#line 958 "Photo.vala"
void photo_update (Photo* self) {
#line 3154 "Photo.c"
	GError * _inner_error_;
	GFile* file;
	Dimensions _tmp0_ = {0};
	Dimensions dim;
	Orientation orientation;
	time_t exposure_time;
	PhotoExif* exif;
	GFileInfo* info;
	GTimeVal _tmp6_ = {0};
	GTimeVal timestamp;
#line 958 "Photo.vala"
	g_return_if_fail (IS_PHOTO (self));
#line 3167 "Photo.c"
	_inner_error_ = NULL;
	file = photo_get_file (self);
	dim = (dimensions_init (&_tmp0_, 0, 0), _tmp0_);
	orientation = ORIENTATION_TOP_LEFT;
	exposure_time = 0;
	/* TODO: Try to read JFIF metadata too*/
	exif = photo_exif_new (file);
#line 967 "Photo.vala"
	if (photo_exif_has_exif (exif)) {
#line 968 "Photo.vala"
		if (!photo_exif_get_dimensions (exif, &dim)) {
#line 3179 "Photo.c"
			char* _tmp1_;
#line 208 "Photo.vala"
			_tmp1_ = NULL;
#line 969 "Photo.vala"
			g_error ("Photo.vala:969: Unable to read EXIF dimensions for %s", _tmp1_ = photo_to_string (self));
#line 3185 "Photo.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
		}
#line 971 "Photo.vala"
		if (!photo_exif_get_timestamp (exif, &exposure_time)) {
#line 3190 "Photo.c"
			char* _tmp2_;
#line 208 "Photo.vala"
			_tmp2_ = NULL;
#line 972 "Photo.vala"
			g_error ("Photo.vala:972: Unable to read EXIF orientation for %s", _tmp2_ = photo_to_string (self));
#line 3196 "Photo.c"
			_tmp2_ = (g_free (_tmp2_), NULL);
		}
#line 974 "Photo.vala"
		orientation = photo_exif_get_orientation (exif);
#line 3201 "Photo.c"
	}
	info = NULL;
	{
		GFileInfo* _tmp3_;
		GFileInfo* _tmp4_;
		_tmp3_ = g_file_query_info (file, "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch38_g_error;
			goto __finally38;
		}
		_tmp4_ = NULL;
#line 979 "Photo.vala"
		info = (_tmp4_ = _tmp3_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp4_);
#line 3215 "Photo.c"
	}
	goto __finally38;
	__catch38_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp5_;
#line 208 "Photo.vala"
			_tmp5_ = NULL;
#line 981 "Photo.vala"
			g_error ("Photo.vala:981: Unable to read file information for %s: %s", _tmp5_ = photo_to_string (self), err->message);
#line 3229 "Photo.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally38:
	if (_inner_error_ != NULL) {
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	timestamp = (g_get_current_time (&_tmp6_), _tmp6_);
#line 985 "Photo.vala"
	g_file_info_get_modification_time (info, &timestamp);
#line 987 "Photo.vala"
	if (photo_table_update (photo_photo_table, &self->priv->photo_id, &dim, g_file_info_get_size (info), timestamp.tv_sec, &exposure_time, orientation)) {
#line 3248 "Photo.c"
		/* cache coherency*/
#line 990 "Photo.vala"
		self->priv->exposure_time = exposure_time;
#line 992 "Photo.vala"
		photo_photo_altered (self);
#line 3254 "Photo.c"
	}
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(exif == NULL) ? NULL : (exif = (photo_exif_unref (exif), NULL));
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
}


#line 996 "Photo.vala"
static GdkPoint photo_unscaled_to_raw_point (Photo* self, const GdkPoint* unscaled_point) {
#line 3264 "Photo.c"
	GdkPoint result;
	Orientation unscaled_orientation;
	Dimensions _tmp0_ = {0};
	Dimensions unscaled_dims;
	gint unscaled_x_offset_raw;
	gint unscaled_y_offset_raw;
	Box crop_box = {0};
	gboolean is_cropped;
	GdkPoint derotated_point;
	unscaled_orientation = photo_get_orientation (self);
	unscaled_dims = orientation_rotate_dimensions (unscaled_orientation, (_tmp0_ = photo_get_dimensions (self), &_tmp0_));
	unscaled_x_offset_raw = 0;
	unscaled_y_offset_raw = 0;
	is_cropped = photo_get_raw_crop (self, &crop_box);
#line 1007 "Photo.vala"
	if (is_cropped) {
#line 1008 "Photo.vala"
		unscaled_x_offset_raw = crop_box.left;
#line 1009 "Photo.vala"
		unscaled_y_offset_raw = crop_box.top;
#line 3285 "Photo.c"
	}
	derotated_point = orientation_derotate_point (unscaled_orientation, &unscaled_dims, &(*unscaled_point));
#line 1016 "Photo.vala"
	derotated_point.x = derotated_point.x + unscaled_x_offset_raw;
#line 1017 "Photo.vala"
	derotated_point.y = derotated_point.y + unscaled_y_offset_raw;
#line 3292 "Photo.c"
	result = derotated_point;
	return result;
}


#line 1022 "Photo.vala"
static GdkRectangle photo_unscaled_to_raw_rect (Photo* self, const GdkRectangle* unscaled_rect) {
#line 3300 "Photo.c"
	GdkRectangle result;
	GdkPoint upper_left = {0};
	GdkPoint lower_right = {0};
	GdkRectangle raw_rect = {0};
#line 1025 "Photo.vala"
	upper_left.x = (*unscaled_rect).x;
#line 1026 "Photo.vala"
	upper_left.y = (*unscaled_rect).y;
#line 1027 "Photo.vala"
	lower_right.x = upper_left.x + (*unscaled_rect).width;
#line 1028 "Photo.vala"
	lower_right.y = upper_left.y + (*unscaled_rect).height;
#line 1030 "Photo.vala"
	upper_left = photo_unscaled_to_raw_point (self, &upper_left);
#line 1031 "Photo.vala"
	lower_right = photo_unscaled_to_raw_point (self, &lower_right);
#line 1033 "Photo.vala"
	if (upper_left.x > lower_right.x) {
#line 3319 "Photo.c"
		gint temp;
		temp = upper_left.x;
#line 1035 "Photo.vala"
		upper_left.x = lower_right.x;
#line 1036 "Photo.vala"
		lower_right.x = temp;
#line 3326 "Photo.c"
	}
#line 1038 "Photo.vala"
	if (upper_left.y > lower_right.y) {
#line 3330 "Photo.c"
		gint temp;
		temp = upper_left.y;
#line 1040 "Photo.vala"
		upper_left.y = lower_right.y;
#line 1041 "Photo.vala"
		lower_right.y = temp;
#line 3337 "Photo.c"
	}
#line 1045 "Photo.vala"
	raw_rect.x = upper_left.x;
#line 1046 "Photo.vala"
	raw_rect.y = upper_left.y;
#line 1047 "Photo.vala"
	raw_rect.width = lower_right.x - upper_left.x;
#line 1048 "Photo.vala"
	raw_rect.height = lower_right.y - upper_left.y;
#line 3347 "Photo.c"
	result = raw_rect;
	return result;
}


static void photo_class_init (PhotoClass * klass) {
	PhotoID _tmp0_ = {0};
	photo_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (PhotoPrivate));
	G_OBJECT_CLASS (klass)->finalize = photo_finalize;
	g_signal_new ("altered", TYPE_PHOTO, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("thumbnail_altered", TYPE_PHOTO, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("removed", TYPE_PHOTO, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	photo_cached_photo_id = (photo_id_init (&_tmp0_, PHOTO_ID_INVALID), _tmp0_);
}


static void photo_instance_init (Photo * self) {
	self->priv = PHOTO_GET_PRIVATE (self);
	self->priv->exposure_time = -1;
}


static void photo_finalize (GObject* obj) {
	Photo * self;
	self = PHOTO (obj);
	G_OBJECT_CLASS (photo_parent_class)->finalize (obj);
}


GType photo_get_type (void) {
	static GType photo_type_id = 0;
	if (photo_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PhotoClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Photo), 0, (GInstanceInitFunc) photo_instance_init, NULL };
		photo_type_id = g_type_register_static (G_TYPE_OBJECT, "Photo", &g_define_type_info, 0);
	}
	return photo_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




