/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_SINGLE_PHOTO_PAGE (single_photo_page_get_type ())
#define SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPage))
#define SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))
#define IS_SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SINGLE_PHOTO_PAGE))
#define IS_SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SINGLE_PHOTO_PAGE))
#define SINGLE_PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))

typedef struct _SinglePhotoPage SinglePhotoPage;
typedef struct _SinglePhotoPageClass SinglePhotoPageClass;
typedef struct _SinglePhotoPagePrivate SinglePhotoPagePrivate;

#define SINGLE_PHOTO_PAGE_TYPE_UPDATE_REASON (single_photo_page_update_reason_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_PHOTO_PAGE (photo_page_get_type ())
#define PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_PAGE, PhotoPage))
#define PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_PAGE, PhotoPageClass))
#define IS_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_PAGE))
#define IS_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_PAGE))
#define PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_PAGE, PhotoPageClass))

typedef struct _PhotoPage PhotoPage;
typedef struct _PhotoPageClass PhotoPageClass;
typedef struct _PhotoPagePrivate PhotoPagePrivate;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;

#define TYPE_THUMBNAIL (thumbnail_get_type ())
#define THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL, Thumbnail))
#define THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL, ThumbnailClass))
#define IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL))
#define IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL))
#define THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL, ThumbnailClass))

typedef struct _Thumbnail Thumbnail;
typedef struct _ThumbnailClass ThumbnailClass;

#define TYPE_EDITING_TOOL (editing_tool_get_type ())
#define EDITING_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EDITING_TOOL, EditingTool))
#define EDITING_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EDITING_TOOL, EditingToolClass))
#define IS_EDITING_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EDITING_TOOL))
#define IS_EDITING_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EDITING_TOOL))
#define EDITING_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EDITING_TOOL, EditingToolClass))

typedef struct _EditingTool EditingTool;
typedef struct _EditingToolClass EditingToolClass;

#define TYPE_FULLSCREEN_WINDOW (fullscreen_window_get_type ())
#define FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindow))
#define FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))
#define IS_FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FULLSCREEN_WINDOW))
#define IS_FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FULLSCREEN_WINDOW))
#define FULLSCREEN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))

typedef struct _FullscreenWindow FullscreenWindow;
typedef struct _FullscreenWindowClass FullscreenWindowClass;

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_PHOTO_CANVAS (photo_canvas_get_type ())
#define PHOTO_CANVAS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_CANVAS, PhotoCanvas))
#define PHOTO_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_CANVAS, PhotoCanvasClass))
#define IS_PHOTO_CANVAS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_CANVAS))
#define IS_PHOTO_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_CANVAS))
#define PHOTO_CANVAS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_CANVAS, PhotoCanvasClass))

typedef struct _PhotoCanvas PhotoCanvas;
typedef struct _PhotoCanvasClass PhotoCanvasClass;

#define PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS (photo_page_photo_page_canvas_get_type ())
#define PHOTO_PAGE_PHOTO_PAGE_CANVAS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS, PhotoPagePhotoPageCanvas))
#define PHOTO_PAGE_PHOTO_PAGE_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS, PhotoPagePhotoPageCanvasClass))
#define PHOTO_PAGE_IS_PHOTO_PAGE_CANVAS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS))
#define PHOTO_PAGE_IS_PHOTO_PAGE_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS))
#define PHOTO_PAGE_PHOTO_PAGE_CANVAS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS, PhotoPagePhotoPageCanvasClass))

typedef struct _PhotoPagePhotoPageCanvas PhotoPagePhotoPageCanvas;
typedef struct _PhotoPagePhotoPageCanvasClass PhotoPagePhotoPageCanvasClass;

#define PAGE_TYPE_TARGET_TYPE (page_target_type_get_type ())

#define TYPE_EXPORT_DIALOG (export_dialog_get_type ())
#define EXPORT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORT_DIALOG, ExportDialog))
#define EXPORT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORT_DIALOG, ExportDialogClass))
#define IS_EXPORT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORT_DIALOG))
#define IS_EXPORT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORT_DIALOG))
#define EXPORT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORT_DIALOG, ExportDialogClass))

typedef struct _ExportDialog ExportDialog;
typedef struct _ExportDialogClass ExportDialogClass;

#define TYPE_SCALE_CONSTRAINT (scale_constraint_get_type ())

#define JPEG_TYPE_QUALITY (jpeg_quality_get_type ())

#define TYPE_EDITING_TOOL_WINDOW (editing_tool_window_get_type ())
#define EDITING_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EDITING_TOOL_WINDOW, EditingToolWindow))
#define EDITING_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EDITING_TOOL_WINDOW, EditingToolWindowClass))
#define IS_EDITING_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EDITING_TOOL_WINDOW))
#define IS_EDITING_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EDITING_TOOL_WINDOW))
#define EDITING_TOOL_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EDITING_TOOL_WINDOW, EditingToolWindowClass))

typedef struct _EditingToolWindow EditingToolWindow;
typedef struct _EditingToolWindowClass EditingToolWindowClass;
typedef struct _EditingToolPrivate EditingToolPrivate;

#define TYPE_ROTATION (rotation_get_type ())

#define TYPE_CROP_TOOL (crop_tool_get_type ())
#define CROP_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CROP_TOOL, CropTool))
#define CROP_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CROP_TOOL, CropToolClass))
#define IS_CROP_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CROP_TOOL))
#define IS_CROP_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CROP_TOOL))
#define CROP_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CROP_TOOL, CropToolClass))

typedef struct _CropTool CropTool;
typedef struct _CropToolClass CropToolClass;

#define TYPE_REDEYE_TOOL (redeye_tool_get_type ())
#define REDEYE_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_REDEYE_TOOL, RedeyeTool))
#define REDEYE_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_REDEYE_TOOL, RedeyeToolClass))
#define IS_REDEYE_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_REDEYE_TOOL))
#define IS_REDEYE_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_REDEYE_TOOL))
#define REDEYE_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_REDEYE_TOOL, RedeyeToolClass))

typedef struct _RedeyeTool RedeyeTool;
typedef struct _RedeyeToolClass RedeyeToolClass;

#define TYPE_ADJUST_TOOL (adjust_tool_get_type ())
#define ADJUST_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ADJUST_TOOL, AdjustTool))
#define ADJUST_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ADJUST_TOOL, AdjustToolClass))
#define IS_ADJUST_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ADJUST_TOOL))
#define IS_ADJUST_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ADJUST_TOOL))
#define ADJUST_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ADJUST_TOOL, AdjustToolClass))

typedef struct _AdjustTool AdjustTool;
typedef struct _AdjustToolClass AdjustToolClass;
typedef struct _PhotoCanvasPrivate PhotoCanvasPrivate;
typedef struct _PhotoPagePhotoPageCanvasPrivate PhotoPagePhotoPageCanvasPrivate;

struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkActionGroup* action_group;
	GtkActionGroup* common_action_group;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkToolbar* (*get_toolbar) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*switching_to_fullscreen) (Page* self);
	void (*returning_from_fullscreen) (Page* self);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, const GdkEventKey* event);
	void (*on_move) (Page* self, const GdkRectangle* rect);
	void (*on_resize) (Page* self, const GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
};

typedef enum  {
	SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS
} SinglePhotoPageUpdateReason;

struct _Dimensions {
	gint width;
	gint height;
};

struct _SinglePhotoPage {
	Page parent_instance;
	SinglePhotoPagePrivate * priv;
	GdkGC* canvas_gc;
	GtkDrawingArea* canvas;
	GtkViewport* viewport;
};

struct _SinglePhotoPageClass {
	PageClass parent_class;
	void (*new_drawable) (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable);
	void (*updated_pixbuf) (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
	void (*paint) (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
};

struct _PhotoPage {
	SinglePhotoPage parent_instance;
	PhotoPagePrivate * priv;
};

struct _PhotoPageClass {
	SinglePhotoPageClass parent_class;
};

struct _PhotoPagePrivate {
	GtkWindow* container;
	GtkMenu* context_menu;
	CheckerboardPage* controller;
	Photo* photo;
	Thumbnail* thumbnail;
	GtkToolbar* toolbar;
	GtkToolButton* rotate_button;
	GtkToggleToolButton* crop_button;
	GtkToggleToolButton* redeye_button;
	GtkToggleToolButton* adjust_button;
	GtkToolButton* prev_button;
	GtkToolButton* next_button;
	EditingTool* current_tool;
	GFile* drag_file;
};

typedef enum  {
	PAGE_TARGET_TYPE_URI_LIST
} PageTargetType;

typedef enum  {
	SCALE_CONSTRAINT_ORIGINAL,
	SCALE_CONSTRAINT_DIMENSIONS,
	SCALE_CONSTRAINT_WIDTH,
	SCALE_CONSTRAINT_HEIGHT
} ScaleConstraint;

typedef enum  {
	JPEG_QUALITY_LOW = 50,
	JPEG_QUALITY_MEDIUM = 75,
	JPEG_QUALITY_HIGH = 90,
	JPEG_QUALITY_MAXIMUM = 100
} JpegQuality;

struct _EditingTool {
	GTypeInstance parent_instance;
	volatile int ref_count;
	EditingToolPrivate * priv;
	PhotoCanvas* canvas;
};

struct _EditingToolClass {
	GTypeClass parent_class;
	void (*finalize) (EditingTool *self);
	void (*activate) (EditingTool* self, PhotoCanvas* canvas);
	void (*deactivate) (EditingTool* self);
	EditingToolWindow* (*get_tool_window) (EditingTool* self);
	GdkPixbuf* (*get_unscaled_pixbuf) (EditingTool* self, Photo* photo);
	void (*on_left_click) (EditingTool* self, gint x, gint y);
	void (*on_left_released) (EditingTool* self, gint x, gint y);
	void (*on_motion) (EditingTool* self, gint x, gint y, GdkModifierType mask);
	gboolean (*on_keypress) (EditingTool* self, const GdkEventKey* event);
	void (*paint) (EditingTool* self, GdkGC* gc, GdkDrawable* drawable);
};

typedef enum  {
	ROTATION_CLOCKWISE,
	ROTATION_COUNTERCLOCKWISE,
	ROTATION_MIRROR,
	ROTATION_UPSIDE_DOWN
} Rotation;

/* The PhotoCanvas is an interface object between an EditingTool and its host.  It provides objects
 and primitives for an EditingTool to obtain information about the image, to draw on the host's
 canvas, and to be signalled when the canvas and its pixbuf changes (is resized).*/
struct _PhotoCanvas {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PhotoCanvasPrivate * priv;
};

struct _PhotoCanvasClass {
	GTypeClass parent_class;
	void (*finalize) (PhotoCanvas *self);
	void (*repaint) (PhotoCanvas* self);
};

struct _PhotoPagePhotoPageCanvas {
	PhotoCanvas parent_instance;
	PhotoPagePhotoPageCanvasPrivate * priv;
};

struct _PhotoPagePhotoPageCanvasClass {
	PhotoCanvasClass parent_class;
};

struct _PhotoPagePhotoPageCanvasPrivate {
	PhotoPage* photo_page;
};


static gpointer photo_page_photo_page_canvas_parent_class = NULL;
static gpointer photo_page_parent_class = NULL;

GType page_get_type (void);
GType single_photo_page_get_type (void);
GType single_photo_page_update_reason_get_type (void);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
GType photo_page_get_type (void);
GType checkerboard_page_get_type (void);
GType photo_get_type (void);
GType layout_item_get_type (void);
GType thumbnail_get_type (void);
gpointer editing_tool_ref (gpointer instance);
void editing_tool_unref (gpointer instance);
GParamSpec* param_spec_editing_tool (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_editing_tool (GValue* value, gpointer v_object);
gpointer value_get_editing_tool (const GValue* value);
GType editing_tool_get_type (void);
#define PHOTO_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PHOTO_PAGE, PhotoPagePrivate))
enum  {
	PHOTO_PAGE_DUMMY_PROPERTY
};
#define PHOTO_PAGE_TOOL_WINDOW_SEPARATOR 8
static void photo_page_on_export (PhotoPage* self);
static void _photo_page_on_export_gtk_action_callback (GtkAction* action, gpointer self);
static void photo_page_on_view_menu (PhotoPage* self);
static void _photo_page_on_view_menu_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_RETURN_TO_PAGE "shotwell-return-to-page"
static void photo_page_on_return_to_collection (PhotoPage* self);
static void _photo_page_on_return_to_collection_gtk_action_callback (GtkAction* action, gpointer self);
static void photo_page_on_photo_menu (PhotoPage* self);
static void _photo_page_on_photo_menu_gtk_action_callback (GtkAction* action, gpointer self);
static void photo_page_on_previous_photo (PhotoPage* self);
static void _photo_page_on_previous_photo_gtk_action_callback (GtkAction* action, gpointer self);
static void photo_page_on_next_photo (PhotoPage* self);
static void _photo_page_on_next_photo_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_CLOCKWISE "shotwell-rotate-clockwise"
static void photo_page_on_rotate_clockwise (PhotoPage* self);
static void _photo_page_on_rotate_clockwise_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_COUNTERCLOCKWISE "shotwell-rotate-counterclockwise"
static void photo_page_on_rotate_counterclockwise (PhotoPage* self);
static void _photo_page_on_rotate_counterclockwise_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_MIRROR "shotwell-mirror"
static void photo_page_on_mirror (PhotoPage* self);
static void _photo_page_on_mirror_gtk_action_callback (GtkAction* action, gpointer self);
static void photo_page_on_revert (PhotoPage* self);
static void _photo_page_on_revert_gtk_action_callback (GtkAction* action, gpointer self);
SinglePhotoPage* single_photo_page_new (const char* page_name);
SinglePhotoPage* single_photo_page_construct (GType object_type, const char* page_name);
void page_init_ui (Page* self, const char* ui_filename, const char* menubar_path, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1);
#define RESOURCES_ROTATE_CLOCKWISE_LABEL "Rotate"
#define RESOURCES_ROTATE_CLOCKWISE_TOOLTIP "Rotate the photo(s) right"
static void _photo_page_on_rotate_clockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
#define RESOURCES_CROP "shotwell-crop"
static void photo_page_on_crop_toggled (PhotoPage* self);
static void _photo_page_on_crop_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self);
#define RESOURCES_REDEYE "shotwell-redeye"
static void photo_page_on_redeye_toggled (PhotoPage* self);
static void _photo_page_on_redeye_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self);
#define RESOURCES_ADJUST "shotwell-adjust"
static void photo_page_on_adjust_toggled (PhotoPage* self);
static void _photo_page_on_adjust_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self);
static void _photo_page_on_previous_photo_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void _photo_page_on_next_photo_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
PhotoPage* photo_page_new (void);
PhotoPage* photo_page_construct (GType object_type);
GType fullscreen_window_get_type (void);
void page_enable_drag_source (Page* self, GdkDragAction actions);
void photo_page_set_container (PhotoPage* self, GtkWindow* container);
static GtkToolbar* photo_page_real_get_toolbar (Page* base);
CheckerboardPage* photo_page_get_controller (PhotoPage* self);
Thumbnail* photo_page_get_thumbnail (PhotoPage* self);
void page_switching_from (Page* self);
static void photo_page_deactivate_tool (PhotoPage* self);
static void photo_page_real_switching_from (Page* base);
void page_switching_to_fullscreen (Page* self);
static void photo_page_real_switching_to_fullscreen (Page* base);
void page_set_page_name (Page* self, const char* page_name);
char* layout_item_get_title (LayoutItem* self);
static void photo_page_update_display (PhotoPage* self);
static void photo_page_update_sensitivity (PhotoPage* self);
void photo_page_display (PhotoPage* self, CheckerboardPage* controller, Thumbnail* thumbnail);
static void photo_page_on_photo_altered (PhotoPage* self, Photo* p);
static void _photo_page_on_photo_altered_photo_altered (Photo* _sender, gpointer self);
Photo* thumbnail_get_photo (Thumbnail* self);
void single_photo_page_set_pixbuf (SinglePhotoPage* self, GdkPixbuf* unscaled);
GdkPixbuf* photo_get_thumbnail (Photo* self, gint scale);
#define THUMBNAIL_CACHE_BIG_SCALE 360
static gboolean photo_page_update_pixbuf (PhotoPage* self);
static gboolean _photo_page_update_pixbuf_gsource_func (gpointer self);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
GdkPixbuf* photo_get_pixbuf (Photo* self, gint exceptions, gint scale, GdkInterpType interp, GError** error);
#define PHOTO_EXCEPTION_NONE 0
gint checkerboard_page_get_count (CheckerboardPage* self);
void single_photo_page_set_default_interp (SinglePhotoPage* self, GdkInterpType default_interp);
#define SINGLE_PHOTO_PAGE_QUALITY_INTERP GDK_INTERP_HYPER
GdkPixbuf* editing_tool_get_unscaled_pixbuf (EditingTool* self, Photo* photo);
static PhotoPagePhotoPageCanvas* photo_page_photo_page_canvas_new (PhotoPage* photo_page);
static PhotoPagePhotoPageCanvas* photo_page_photo_page_canvas_construct (GType object_type, PhotoPage* photo_page);
gpointer photo_canvas_ref (gpointer instance);
void photo_canvas_unref (gpointer instance);
GParamSpec* param_spec_photo_canvas (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_photo_canvas (GValue* value, gpointer v_object);
gpointer value_get_photo_canvas (const GValue* value);
GType photo_canvas_get_type (void);
static GType photo_page_photo_page_canvas_get_type (void);
void editing_tool_activate (EditingTool* self, PhotoCanvas* canvas);
static void photo_page_place_tool_window (PhotoPage* self);
static void photo_page_show_tool_window (PhotoPage* self);
void single_photo_page_default_repaint (SinglePhotoPage* self);
static void photo_page_activate_tool (PhotoPage* self, EditingTool* tool);
void editing_tool_deactivate (EditingTool* self);
#define SINGLE_PHOTO_PAGE_FAST_INTERP GDK_INTERP_NEAREST
GFile* photo_generate_exportable (Photo* self, GError** error);
#define THUMBNAIL_CACHE_MEDIUM_SCALE 128
static void photo_page_real_drag_begin (GtkWidget* base, GdkDragContext* context);
GType page_target_type_get_type (void);
static void photo_page_real_drag_data_get (GtkWidget* base, GdkDragContext* context, GtkSelectionData* selection_data, guint target_type, guint time);
static void photo_page_real_drag_end (GtkWidget* base, GdkDragContext* context);
static gboolean photo_page_real_source_drag_failed (Page* base, GdkDragContext* context, GtkDragResult drag_result);
gboolean single_photo_page_is_inside_pixbuf (SinglePhotoPage* self, gint x, gint y);
void editing_tool_on_left_click (EditingTool* self, gint x, gint y);
static gboolean photo_page_real_on_left_click (Page* base, const GdkEventButton* event);
void editing_tool_on_left_released (EditingTool* self, gint x, gint y);
static gboolean photo_page_real_on_left_released (Page* base, const GdkEventButton* event);
static gboolean photo_page_on_context_menu (PhotoPage* self, const GdkEventButton* event);
static gboolean photo_page_real_on_right_click (Page* base, const GdkEventButton* event);
char* page_get_page_name (Page* self);
void app_window_switch_to_page (AppWindow* self, Page* page);
ExportDialog* export_dialog_new (gint count);
ExportDialog* export_dialog_construct (GType object_type, gint count);
GType export_dialog_get_type (void);
GType scale_constraint_get_type (void);
GType jpeg_quality_get_type (void);
gboolean export_dialog_execute (ExportDialog* self, gint* scale, ScaleConstraint* constraint, JpegQuality* quality);
GFile* export_ui_choose_file (GFile* current_file);
GFile* photo_get_file (Photo* self);
void photo_export (Photo* self, GFile* dest_file, gint scale, ScaleConstraint constraint, JpegQuality quality, GError** error);
void app_window_error_message (const char* message);
gboolean photo_equals (Photo* self, Photo* photo);
void editing_tool_on_motion (EditingTool* self, gint x, gint y, GdkModifierType mask);
static gboolean photo_page_real_on_motion (Page* base, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
void page_set_item_sensitive (Page* self, const char* path, gboolean sensitive);
gboolean photo_has_transformations (Photo* self);
GType editing_tool_window_get_type (void);
EditingToolWindow* editing_tool_get_tool_window (EditingTool* self);
gboolean editing_tool_window_has_user_moved (EditingToolWindow* self);
gboolean page_on_configure (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
static gboolean photo_page_real_on_configure (Page* base, const GdkEventConfigure* event, const GdkRectangle* rect);
gboolean editing_tool_on_keypress (EditingTool* self, const GdkEventKey* event);
static gboolean photo_page_real_key_press_event (GtkWidget* base, const GdkEventKey* event);
void photo_canvas_set_drawable (PhotoCanvas* self, GdkGC* default_gc, GdkDrawable* drawable);
static void photo_page_real_new_drawable (SinglePhotoPage* base, GdkGC* default_gc, GdkDrawable* drawable);
void photo_canvas_resized_pixbuf (PhotoCanvas* self, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
GdkRectangle single_photo_page_get_scaled_pixbuf_position (SinglePhotoPage* self);
static void photo_page_real_updated_pixbuf (SinglePhotoPage* base, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
void editing_tool_paint (EditingTool* self, GdkGC* gc, GdkDrawable* drawable);
void single_photo_page_paint (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
static void photo_page_real_paint (SinglePhotoPage* base, GdkGC* gc, GdkDrawable* drawable);
GType rotation_get_type (void);
void photo_rotate (Photo* self, Rotation rotation);
static void photo_page_rotate (PhotoPage* self, Rotation rotation);
void photo_remove_all_transformations (Photo* self);
#define RESOURCES_ROTATE_COUNTERCLOCKWISE_LABEL "Rotate"
#define RESOURCES_ROTATE_COUNTERCLOCKWISE_TOOLTIP "Rotate the photo(s) left"
static void _photo_page_on_rotate_counterclockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static gboolean photo_page_real_on_ctrl_pressed (Page* base, const GdkEventKey* event);
static gboolean photo_page_real_on_ctrl_released (Page* base, const GdkEventKey* event);
CropTool* crop_tool_new (void);
CropTool* crop_tool_construct (GType object_type);
GType crop_tool_get_type (void);
static void photo_page_on_crop_activated (PhotoPage* self);
static void _photo_page_on_crop_activated_editing_tool_activated (CropTool* _sender, gpointer self);
static void photo_page_on_crop_deactivated (PhotoPage* self);
static void _photo_page_on_crop_deactivated_editing_tool_deactivated (CropTool* _sender, gpointer self);
static void photo_page_on_crop_done (PhotoPage* self);
static void _photo_page_on_crop_done_editing_tool_applied (CropTool* _sender, gpointer self);
static void _photo_page_on_crop_done_editing_tool_cancelled (CropTool* _sender, gpointer self);
RedeyeTool* redeye_tool_new (void);
RedeyeTool* redeye_tool_construct (GType object_type);
GType redeye_tool_get_type (void);
static void photo_page_on_redeye_activated (PhotoPage* self);
static void _photo_page_on_redeye_activated_editing_tool_activated (RedeyeTool* _sender, gpointer self);
static void photo_page_on_redeye_deactivated (PhotoPage* self);
static void _photo_page_on_redeye_deactivated_editing_tool_deactivated (RedeyeTool* _sender, gpointer self);
static void photo_page_on_redeye_applied (PhotoPage* self);
static void _photo_page_on_redeye_applied_editing_tool_applied (RedeyeTool* _sender, gpointer self);
static void photo_page_on_redeye_closed (PhotoPage* self);
static void _photo_page_on_redeye_closed_editing_tool_cancelled (RedeyeTool* _sender, gpointer self);
AdjustTool* adjust_tool_new (void);
AdjustTool* adjust_tool_construct (GType object_type);
GType adjust_tool_get_type (void);
static void photo_page_on_adjust_activated (PhotoPage* self);
static void _photo_page_on_adjust_activated_editing_tool_activated (AdjustTool* _sender, gpointer self);
static void photo_page_on_adjust_deactivated (PhotoPage* self);
static void _photo_page_on_adjust_deactivated_editing_tool_deactivated (AdjustTool* _sender, gpointer self);
static void photo_page_on_adjust_closed (PhotoPage* self);
static void _photo_page_on_adjust_closed_editing_tool_cancelled (AdjustTool* _sender, gpointer self);
static void photo_page_on_adjust_applied (PhotoPage* self);
static void _photo_page_on_adjust_applied_editing_tool_applied (AdjustTool* _sender, gpointer self);
LayoutItem* checkerboard_page_get_next_item (CheckerboardPage* self, LayoutItem* current);
LayoutItem* checkerboard_page_get_previous_item (CheckerboardPage* self, LayoutItem* current);
#define PHOTO_PAGE_PHOTO_PAGE_CANVAS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS, PhotoPagePhotoPageCanvasPrivate))
enum  {
	PHOTO_PAGE_PHOTO_PAGE_CANVAS_DUMMY_PROPERTY
};
GdkDrawable* single_photo_page_get_drawable (SinglePhotoPage* self);
GdkPixbuf* single_photo_page_get_scaled_pixbuf (SinglePhotoPage* self);
PhotoCanvas* photo_canvas_new (GtkWindow* container, GdkWindow* drawing_window, Photo* photo, GdkGC* default_gc, GdkDrawable* drawable, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
PhotoCanvas* photo_canvas_construct (GType object_type, GtkWindow* container, GdkWindow* drawing_window, Photo* photo, GdkGC* default_gc, GdkDrawable* drawable, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
void single_photo_page_repaint (SinglePhotoPage* self, GdkInterpType repaint_interp);
static void photo_page_photo_page_canvas_real_repaint (PhotoCanvas* base);
static void photo_page_photo_page_canvas_finalize (PhotoCanvas* obj);
static void photo_page_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const GtkActionEntry PHOTO_PAGE_ACTIONS[] = {{"FileMenu", NULL, "_File", NULL, NULL, (GCallback) NULL}, {"Export", GTK_STOCK_SAVE_AS, "_Export Photos...", "<Ctrl>E", "Export photo to disk", (GCallback) _photo_page_on_export_gtk_action_callback}, {"ViewMenu", NULL, "_View", NULL, NULL, (GCallback) _photo_page_on_view_menu_gtk_action_callback}, {"ReturnToPage", RESOURCES_RETURN_TO_PAGE, "_Return to Photos", "Escape", NULL, (GCallback) _photo_page_on_return_to_collection_gtk_action_callback}, {"PhotoMenu", NULL, "_Photo", NULL, NULL, (GCallback) _photo_page_on_photo_menu_gtk_action_callback}, {"PrevPhoto", GTK_STOCK_GO_BACK, "_Previous Photo", NULL, "Previous Photo", (GCallback) _photo_page_on_previous_photo_gtk_action_callback}, {"NextPhoto", GTK_STOCK_GO_FORWARD, "_Next Photo", NULL, "Next Photo", (GCallback) _photo_page_on_next_photo_gtk_action_callback}, {"RotateClockwise", RESOURCES_CLOCKWISE, "Rotate _Right", "<Ctrl>R", "Rotate the selected photos clockwise", (GCallback) _photo_page_on_rotate_clockwise_gtk_action_callback}, {"RotateCounterclockwise", RESOURCES_COUNTERCLOCKWISE, "Rotate _Left", "<Ctrl><Shift>R", "Rotate the selected photos counterclockwise", (GCallback) _photo_page_on_rotate_counterclockwise_gtk_action_callback}, {"Mirror", RESOURCES_MIRROR, "_Mirror", "<Ctrl>M", "Make mirror images of the selected photos", (GCallback) _photo_page_on_mirror_gtk_action_callback}, {"Revert", GTK_STOCK_REVERT_TO_SAVED, "Re_vert to Original", NULL, "Revert to the original photo", (GCallback) _photo_page_on_revert_gtk_action_callback}, {"HelpMenu", NULL, "_Help", NULL, NULL, (GCallback) NULL}};


#line 344 "PhotoPage.vala"
static void _photo_page_on_export_gtk_action_callback (GtkAction* action, gpointer self) {
#line 594 "PhotoPage.c"
	photo_page_on_export (self);
}


#line 331 "PhotoPage.vala"
static void _photo_page_on_view_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 601 "PhotoPage.c"
	photo_page_on_view_menu (self);
}


#line 340 "PhotoPage.vala"
static void _photo_page_on_return_to_collection_gtk_action_callback (GtkAction* action, gpointer self) {
#line 608 "PhotoPage.c"
	photo_page_on_return_to_collection (self);
}


#line 631 "PhotoPage.vala"
static void _photo_page_on_photo_menu_gtk_action_callback (GtkAction* action, gpointer self) {
#line 615 "PhotoPage.c"
	photo_page_on_photo_menu (self);
}


#line 652 "PhotoPage.vala"
static void _photo_page_on_previous_photo_gtk_action_callback (GtkAction* action, gpointer self) {
#line 622 "PhotoPage.c"
	photo_page_on_previous_photo (self);
}


#line 645 "PhotoPage.vala"
static void _photo_page_on_next_photo_gtk_action_callback (GtkAction* action, gpointer self) {
#line 629 "PhotoPage.c"
	photo_page_on_next_photo (self);
}


#line 456 "PhotoPage.vala"
static void _photo_page_on_rotate_clockwise_gtk_action_callback (GtkAction* action, gpointer self) {
#line 636 "PhotoPage.c"
	photo_page_on_rotate_clockwise (self);
}


#line 460 "PhotoPage.vala"
static void _photo_page_on_rotate_counterclockwise_gtk_action_callback (GtkAction* action, gpointer self) {
#line 643 "PhotoPage.c"
	photo_page_on_rotate_counterclockwise (self);
}


#line 464 "PhotoPage.vala"
static void _photo_page_on_mirror_gtk_action_callback (GtkAction* action, gpointer self) {
#line 650 "PhotoPage.c"
	photo_page_on_mirror (self);
}


#line 468 "PhotoPage.vala"
static void _photo_page_on_revert_gtk_action_callback (GtkAction* action, gpointer self) {
#line 657 "PhotoPage.c"
	photo_page_on_revert (self);
}


#line 456 "PhotoPage.vala"
static void _photo_page_on_rotate_clockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 664 "PhotoPage.c"
	photo_page_on_rotate_clockwise (self);
}


#line 494 "PhotoPage.vala"
static void _photo_page_on_crop_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self) {
#line 671 "PhotoPage.c"
	photo_page_on_crop_toggled (self);
}


#line 509 "PhotoPage.vala"
static void _photo_page_on_redeye_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self) {
#line 678 "PhotoPage.c"
	photo_page_on_redeye_toggled (self);
}


#line 523 "PhotoPage.vala"
static void _photo_page_on_adjust_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self) {
#line 685 "PhotoPage.c"
	photo_page_on_adjust_toggled (self);
}


#line 652 "PhotoPage.vala"
static void _photo_page_on_previous_photo_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 692 "PhotoPage.c"
	photo_page_on_previous_photo (self);
}


#line 645 "PhotoPage.vala"
static void _photo_page_on_next_photo_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 699 "PhotoPage.c"
	photo_page_on_next_photo (self);
}


#line 61 "PhotoPage.vala"
PhotoPage* photo_page_construct (GType object_type) {
#line 706 "PhotoPage.c"
	PhotoPage * self;
	GtkMenu* _tmp1_;
	GtkMenu* _tmp0_;
	GtkToolButton* _tmp2_;
	GtkToggleToolButton* _tmp3_;
	GtkToggleToolButton* _tmp4_;
	GtkToggleToolButton* _tmp5_;
	GtkSeparatorToolItem* separator;
#line 62 "PhotoPage.vala"
	self = (PhotoPage*) single_photo_page_construct (object_type, "Photo");
#line 64 "PhotoPage.vala"
	page_init_ui (PAGE (self), "photo.ui", "/PhotoMenuBar", "PhotoActionGroup", PHOTO_PAGE_ACTIONS, G_N_ELEMENTS (PHOTO_PAGE_ACTIONS), NULL, 0);
#line 719 "PhotoPage.c"
	_tmp1_ = NULL;
#line 66 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 66 "PhotoPage.vala"
	self->priv->context_menu = (_tmp1_ = (_tmp0_ = GTK_MENU (gtk_ui_manager_get_widget (PAGE (self)->ui, "/PhotoContextMenu")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->context_menu == NULL) ? NULL : (self->priv->context_menu = (g_object_unref (self->priv->context_menu), NULL)), _tmp1_);
#line 725 "PhotoPage.c"
	_tmp2_ = NULL;
#line 71 "PhotoPage.vala"
	self->priv->rotate_button = (_tmp2_ = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (RESOURCES_CLOCKWISE)), (self->priv->rotate_button == NULL) ? NULL : (self->priv->rotate_button = (g_object_unref (self->priv->rotate_button), NULL)), _tmp2_);
#line 72 "PhotoPage.vala"
	gtk_tool_button_set_label (self->priv->rotate_button, RESOURCES_ROTATE_CLOCKWISE_LABEL);
#line 73 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->rotate_button), RESOURCES_ROTATE_CLOCKWISE_TOOLTIP);
#line 74 "PhotoPage.vala"
	g_signal_connect_object (self->priv->rotate_button, "clicked", (GCallback) _photo_page_on_rotate_clockwise_gtk_tool_button_clicked, self, 0);
#line 75 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->rotate_button), -1);
#line 737 "PhotoPage.c"
	_tmp3_ = NULL;
#line 78 "PhotoPage.vala"
	self->priv->crop_button = (_tmp3_ = g_object_ref_sink ((GtkToggleToolButton*) gtk_toggle_tool_button_new_from_stock (RESOURCES_CROP)), (self->priv->crop_button == NULL) ? NULL : (self->priv->crop_button = (g_object_unref (self->priv->crop_button), NULL)), _tmp3_);
#line 79 "PhotoPage.vala"
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (self->priv->crop_button), "Crop");
#line 80 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->crop_button), "Crop the photo's size");
#line 81 "PhotoPage.vala"
	g_signal_connect_object (self->priv->crop_button, "toggled", (GCallback) _photo_page_on_crop_toggled_gtk_toggle_tool_button_toggled, self, 0);
#line 82 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->crop_button), -1);
#line 749 "PhotoPage.c"
	_tmp4_ = NULL;
#line 85 "PhotoPage.vala"
	self->priv->redeye_button = (_tmp4_ = g_object_ref_sink ((GtkToggleToolButton*) gtk_toggle_tool_button_new_from_stock (RESOURCES_REDEYE)), (self->priv->redeye_button == NULL) ? NULL : (self->priv->redeye_button = (g_object_unref (self->priv->redeye_button), NULL)), _tmp4_);
#line 86 "PhotoPage.vala"
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (self->priv->redeye_button), "Red-eye");
#line 87 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->redeye_button), "Reduce or eliminate any red-eye effects in the photo");
#line 88 "PhotoPage.vala"
	g_signal_connect_object (self->priv->redeye_button, "toggled", (GCallback) _photo_page_on_redeye_toggled_gtk_toggle_tool_button_toggled, self, 0);
#line 89 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->redeye_button), -1);
#line 761 "PhotoPage.c"
	_tmp5_ = NULL;
#line 92 "PhotoPage.vala"
	self->priv->adjust_button = (_tmp5_ = g_object_ref_sink ((GtkToggleToolButton*) gtk_toggle_tool_button_new_from_stock (RESOURCES_ADJUST)), (self->priv->adjust_button == NULL) ? NULL : (self->priv->adjust_button = (g_object_unref (self->priv->adjust_button), NULL)), _tmp5_);
#line 93 "PhotoPage.vala"
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (self->priv->adjust_button), "Adjust");
#line 94 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->adjust_button), "Adjust the photo's color and tone");
#line 95 "PhotoPage.vala"
	g_signal_connect_object (self->priv->adjust_button, "toggled", (GCallback) _photo_page_on_adjust_toggled_gtk_toggle_tool_button_toggled, self, 0);
#line 96 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->adjust_button), -1);
#line 773 "PhotoPage.c"
	separator = g_object_ref_sink ((GtkSeparatorToolItem*) gtk_separator_tool_item_new ());
#line 100 "PhotoPage.vala"
	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separator), TRUE);
#line 101 "PhotoPage.vala"
	gtk_separator_tool_item_set_draw (separator, FALSE);
#line 103 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (separator), -1);
#line 106 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->prev_button), "Previous photo");
#line 107 "PhotoPage.vala"
	g_signal_connect_object (self->priv->prev_button, "clicked", (GCallback) _photo_page_on_previous_photo_gtk_tool_button_clicked, self, 0);
#line 108 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->prev_button), -1);
#line 111 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->next_button), "Next photo");
#line 112 "PhotoPage.vala"
	g_signal_connect_object (self->priv->next_button, "clicked", (GCallback) _photo_page_on_next_photo_gtk_tool_button_clicked, self, 0);
#line 113 "PhotoPage.vala"
	gtk_toolbar_insert (self->priv->toolbar, GTK_TOOL_ITEM (self->priv->next_button), -1);
#line 793 "PhotoPage.c"
	(separator == NULL) ? NULL : (separator = (g_object_unref (separator), NULL));
	return self;
}


#line 61 "PhotoPage.vala"
PhotoPage* photo_page_new (void) {
#line 61 "PhotoPage.vala"
	return photo_page_construct (TYPE_PHOTO_PAGE);
#line 803 "PhotoPage.c"
}


#line 117 "PhotoPage.vala"
void photo_page_set_container (PhotoPage* self, GtkWindow* container) {
#line 809 "PhotoPage.c"
	GtkWindow* _tmp1_;
	GtkWindow* _tmp0_;
#line 117 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 117 "PhotoPage.vala"
	g_return_if_fail (GTK_IS_WINDOW (container));
#line 816 "PhotoPage.c"
	/* this should only be called once*/
#line 119 "PhotoPage.vala"
	g_assert (self->priv->container == NULL);
#line 820 "PhotoPage.c"
	_tmp1_ = NULL;
#line 121 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 121 "PhotoPage.vala"
	self->priv->container = (_tmp1_ = (_tmp0_ = container, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->container == NULL) ? NULL : (self->priv->container = (g_object_unref (self->priv->container), NULL)), _tmp1_);
#line 826 "PhotoPage.c"
	/* DnD only available in full-window view*/
#line 124 "PhotoPage.vala"
	if (!IS_FULLSCREEN_WINDOW (container)) {
#line 125 "PhotoPage.vala"
		page_enable_drag_source (PAGE (self), GDK_ACTION_COPY);
#line 832 "PhotoPage.c"
	}
}


#line 128 "PhotoPage.vala"
static GtkToolbar* photo_page_real_get_toolbar (Page* base) {
#line 839 "PhotoPage.c"
	PhotoPage * self;
	GtkToolbar* result;
	GtkToolbar* _tmp0_;
	self = PHOTO_PAGE (base);
#line 129 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 846 "PhotoPage.c"
	result = (_tmp0_ = self->priv->toolbar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 132 "PhotoPage.vala"
CheckerboardPage* photo_page_get_controller (PhotoPage* self) {
#line 854 "PhotoPage.c"
	CheckerboardPage* result;
	CheckerboardPage* _tmp0_;
#line 132 "PhotoPage.vala"
	g_return_val_if_fail (IS_PHOTO_PAGE (self), NULL);
#line 133 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 861 "PhotoPage.c"
	result = (_tmp0_ = self->priv->controller, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 136 "PhotoPage.vala"
Thumbnail* photo_page_get_thumbnail (PhotoPage* self) {
#line 869 "PhotoPage.c"
	Thumbnail* result;
	Thumbnail* _tmp0_;
#line 136 "PhotoPage.vala"
	g_return_val_if_fail (IS_PHOTO_PAGE (self), NULL);
#line 137 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 876 "PhotoPage.c"
	result = (_tmp0_ = self->priv->thumbnail, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 140 "PhotoPage.vala"
static void photo_page_real_switching_from (Page* base) {
#line 884 "PhotoPage.c"
	PhotoPage * self;
	self = PHOTO_PAGE (base);
#line 141 "PhotoPage.vala"
	PAGE_CLASS (photo_page_parent_class)->switching_from (PAGE (SINGLE_PHOTO_PAGE (self)));
#line 143 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 891 "PhotoPage.c"
}


#line 146 "PhotoPage.vala"
static void photo_page_real_switching_to_fullscreen (Page* base) {
#line 897 "PhotoPage.c"
	PhotoPage * self;
	self = PHOTO_PAGE (base);
#line 147 "PhotoPage.vala"
	PAGE_CLASS (photo_page_parent_class)->switching_to_fullscreen (PAGE (SINGLE_PHOTO_PAGE (self)));
#line 149 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 904 "PhotoPage.c"
}


#line 152 "PhotoPage.vala"
void photo_page_display (PhotoPage* self, CheckerboardPage* controller, Thumbnail* thumbnail) {
#line 910 "PhotoPage.c"
	CheckerboardPage* _tmp1_;
	CheckerboardPage* _tmp0_;
	Thumbnail* _tmp3_;
	Thumbnail* _tmp2_;
	char* _tmp4_;
#line 152 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 152 "PhotoPage.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (controller));
#line 152 "PhotoPage.vala"
	g_return_if_fail (IS_THUMBNAIL (thumbnail));
#line 153 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 924 "PhotoPage.c"
	_tmp1_ = NULL;
#line 155 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 155 "PhotoPage.vala"
	self->priv->controller = (_tmp1_ = (_tmp0_ = controller, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->controller == NULL) ? NULL : (self->priv->controller = (g_object_unref (self->priv->controller), NULL)), _tmp1_);
#line 930 "PhotoPage.c"
	_tmp3_ = NULL;
#line 156 "PhotoPage.vala"
	_tmp2_ = NULL;
#line 156 "PhotoPage.vala"
	self->priv->thumbnail = (_tmp3_ = (_tmp2_ = thumbnail, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL)), _tmp3_);
#line 67 "CollectionLayout.vala"
	_tmp4_ = NULL;
#line 158 "PhotoPage.vala"
	page_set_page_name (PAGE (self), _tmp4_ = layout_item_get_title (LAYOUT_ITEM (thumbnail)));
#line 940 "PhotoPage.c"
	_tmp4_ = (g_free (_tmp4_), NULL);
#line 160 "PhotoPage.vala"
	photo_page_update_display (self);
#line 161 "PhotoPage.vala"
	photo_page_update_sensitivity (self);
#line 946 "PhotoPage.c"
}


#line 364 "PhotoPage.vala"
static void _photo_page_on_photo_altered_photo_altered (Photo* _sender, gpointer self) {
#line 952 "PhotoPage.c"
	photo_page_on_photo_altered (self, _sender);
}


#line 177 "PhotoPage.vala"
static gboolean _photo_page_update_pixbuf_gsource_func (gpointer self) {
#line 959 "PhotoPage.c"
	return photo_page_update_pixbuf (self);
}


#line 164 "PhotoPage.vala"
static void photo_page_update_display (PhotoPage* self) {
#line 966 "PhotoPage.c"
	Photo* _tmp1_;
	GdkPixbuf* _tmp2_;
#line 164 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 165 "PhotoPage.vala"
	if (self->priv->photo != NULL) {
#line 973 "PhotoPage.c"
		guint _tmp0_;
#line 166 "PhotoPage.vala"
		g_signal_handlers_disconnect_matched (self->priv->photo, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("altered", TYPE_PHOTO, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _photo_page_on_photo_altered_photo_altered, self);
#line 977 "PhotoPage.c"
	}
	_tmp1_ = NULL;
#line 168 "PhotoPage.vala"
	self->priv->photo = (_tmp1_ = thumbnail_get_photo (self->priv->thumbnail), (self->priv->photo == NULL) ? NULL : (self->priv->photo = (g_object_unref (self->priv->photo), NULL)), _tmp1_);
#line 169 "PhotoPage.vala"
	g_signal_connect_object (self->priv->photo, "altered", (GCallback) _photo_page_on_photo_altered_photo_altered, self, 0);
#line 984 "PhotoPage.c"
	/* throw a resized large thumbnail up to get an image on the screen quickly,
	 and when ready decode and display the full image*/
#line 841 "Photo.vala"
	_tmp2_ = NULL;
#line 173 "PhotoPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp2_ = photo_get_thumbnail (self->priv->photo, THUMBNAIL_CACHE_BIG_SCALE));
#line 991 "PhotoPage.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 174 "PhotoPage.vala"
	g_idle_add (_photo_page_update_pixbuf_gsource_func, self);
#line 995 "PhotoPage.c"
}


#line 177 "PhotoPage.vala"
static gboolean photo_page_update_pixbuf (PhotoPage* self) {
#line 1001 "PhotoPage.c"
	gboolean result;
	GError * _inner_error_;
	GdkScreen* _tmp1_;
	AppWindow* _tmp0_;
	GdkScreen* _tmp2_;
	GdkScreen* screen;
	gint scale;
	GdkPixbuf* _tmp3_;
	GdkPixbuf* _tmp4_;
#line 177 "PhotoPage.vala"
	g_return_val_if_fail (IS_PHOTO_PAGE (self), FALSE);
#line 1013 "PhotoPage.c"
	_inner_error_ = NULL;
	/* Photo.get_pixbuf() can optimize its pipeline if given a scale to work with ... since
	 SinglePhotoPage may need to resize its unscaled image thousands of times if the user
	 resizes the window, get a scaled image large enough for the screen*/
#line 181 "PhotoPage.vala"
	_tmp1_ = NULL;
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 181 "PhotoPage.vala"
	_tmp2_ = NULL;
#line 1024 "PhotoPage.c"
	screen = (_tmp2_ = (_tmp1_ = gdk_drawable_get_screen (GDK_DRAWABLE (GTK_WIDGET (_tmp0_ = app_window_get_instance ())->window)), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
	scale = MAX (gdk_screen_get_width (screen), gdk_screen_get_height (screen));
	_tmp3_ = photo_get_pixbuf (self->priv->photo, PHOTO_EXCEPTION_NONE, scale, GDK_INTERP_HYPER, &_inner_error_);
	if (_inner_error_ != NULL) {
		(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
#line 615 "Photo.vala"
	_tmp4_ = NULL;
#line 184 "PhotoPage.vala"
	single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), _tmp4_ = _tmp3_);
#line 1038 "PhotoPage.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
	result = FALSE;
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
	return result;
}


#line 189 "PhotoPage.vala"
static void photo_page_update_sensitivity (PhotoPage* self) {
#line 1048 "PhotoPage.c"
	gboolean multiple;
#line 189 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 1052 "PhotoPage.c"
	multiple = checkerboard_page_get_count (self->priv->controller) > 1;
#line 192 "PhotoPage.vala"
	g_object_set (GTK_WIDGET (self->priv->prev_button), "sensitive", multiple, NULL);
#line 193 "PhotoPage.vala"
	g_object_set (GTK_WIDGET (self->priv->next_button), "sensitive", multiple, NULL);
#line 1058 "PhotoPage.c"
}


#line 196 "PhotoPage.vala"
static void photo_page_activate_tool (PhotoPage* self, EditingTool* tool) {
#line 1064 "PhotoPage.c"
	GdkPixbuf* unscaled;
	PhotoCanvas* photo_canvas;
	EditingTool* _tmp1_;
	EditingTool* _tmp0_;
#line 196 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 196 "PhotoPage.vala"
	g_return_if_fail (IS_EDITING_TOOL (tool));
#line 1073 "PhotoPage.c"
	/* during editing, always use the quality interpolation, so the editing tool is only
	 dealing with one pixbuf (unless page is resized)*/
#line 199 "PhotoPage.vala"
	single_photo_page_set_default_interp (SINGLE_PHOTO_PAGE (self), SINGLE_PHOTO_PAGE_QUALITY_INTERP);
#line 1078 "PhotoPage.c"
	/* deactivate current tool ... current implementation is one tool at a time.  In the future,
	 tools may be allowed to be executing at the same time.*/
#line 203 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 1083 "PhotoPage.c"
	/* see if the tool wants a different pixbuf displayed*/
	unscaled = editing_tool_get_unscaled_pixbuf (tool, self->priv->photo);
#line 207 "PhotoPage.vala"
	if (unscaled != NULL) {
#line 208 "PhotoPage.vala"
		single_photo_page_set_pixbuf (SINGLE_PHOTO_PAGE (self), unscaled);
#line 1090 "PhotoPage.c"
	}
	/* create the PhotoCanvas object for a two-way interface to the tool*/
	photo_canvas = PHOTO_CANVAS (photo_page_photo_page_canvas_new (self));
	/* hook tool into event system and activate it*/
	_tmp1_ = NULL;
#line 214 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 214 "PhotoPage.vala"
	self->priv->current_tool = (_tmp1_ = (_tmp0_ = tool, (_tmp0_ == NULL) ? NULL : editing_tool_ref (_tmp0_)), (self->priv->current_tool == NULL) ? NULL : (self->priv->current_tool = (editing_tool_unref (self->priv->current_tool), NULL)), _tmp1_);
#line 215 "PhotoPage.vala"
	editing_tool_activate (self->priv->current_tool, photo_canvas);
#line 1102 "PhotoPage.c"
	/* if the tool has an auxilliary window, move it properly on the screen*/
#line 218 "PhotoPage.vala"
	photo_page_place_tool_window (self);
#line 1106 "PhotoPage.c"
	/* now that the tool window has been placed, show it*/
#line 221 "PhotoPage.vala"
	photo_page_show_tool_window (self);
#line 1110 "PhotoPage.c"
	/* repaint entire view, with the tool now hooked in*/
#line 224 "PhotoPage.vala"
	single_photo_page_default_repaint (SINGLE_PHOTO_PAGE (self));
#line 1114 "PhotoPage.c"
	(unscaled == NULL) ? NULL : (unscaled = (g_object_unref (unscaled), NULL));
	(photo_canvas == NULL) ? NULL : (photo_canvas = (photo_canvas_unref (photo_canvas), NULL));
}


#line 227 "PhotoPage.vala"
static void photo_page_deactivate_tool (PhotoPage* self) {
#line 1122 "PhotoPage.c"
	EditingTool* _tmp0_;
	EditingTool* tool;
	EditingTool* _tmp1_;
#line 227 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 228 "PhotoPage.vala"
	if (self->priv->current_tool == NULL) {
#line 1130 "PhotoPage.c"
		return;
	}
#line 231 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 1135 "PhotoPage.c"
	tool = (_tmp0_ = self->priv->current_tool, (_tmp0_ == NULL) ? NULL : editing_tool_ref (_tmp0_));
	_tmp1_ = NULL;
#line 232 "PhotoPage.vala"
	self->priv->current_tool = (_tmp1_ = NULL, (self->priv->current_tool == NULL) ? NULL : (self->priv->current_tool = (editing_tool_unref (self->priv->current_tool), NULL)), _tmp1_);
#line 1140 "PhotoPage.c"
	/* deactivate with the tool taken out of the hooks*/
#line 235 "PhotoPage.vala"
	editing_tool_deactivate (tool);
#line 1144 "PhotoPage.c"
	/* return to fast interpolation for viewing*/
#line 238 "PhotoPage.vala"
	single_photo_page_set_default_interp (SINGLE_PHOTO_PAGE (self), SINGLE_PHOTO_PAGE_FAST_INTERP);
#line 1148 "PhotoPage.c"
	/* display the (possibly) new photo*/
#line 241 "PhotoPage.vala"
	photo_page_update_pixbuf (self);
#line 1152 "PhotoPage.c"
	(tool == NULL) ? NULL : (tool = (editing_tool_unref (tool), NULL));
}


#line 244 "PhotoPage.vala"
static void photo_page_real_drag_begin (GtkWidget* base, GdkDragContext* context) {
#line 1159 "PhotoPage.c"
	PhotoPage * self;
	GError * _inner_error_;
	GFile* file;
	GdkPixbuf* icon;
	char* _tmp2_;
	GFile* _tmp4_;
	GFile* _tmp3_;
	self = PHOTO_PAGE (base);
#line 244 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 1170 "PhotoPage.c"
	_inner_error_ = NULL;
	/* drag_data_get may be called multiple times within a drag as different applications
	 query for target type and information ... to prevent a lot of file generation, do all
	 the work up front*/
	file = NULL;
	{
		GFile* _tmp0_;
		GFile* _tmp1_;
		_tmp0_ = photo_generate_exportable (self->priv->photo, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch20_g_error;
			goto __finally20;
		}
		_tmp1_ = NULL;
#line 250 "PhotoPage.vala"
		file = (_tmp1_ = _tmp0_, (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp1_);
#line 1187 "PhotoPage.c"
	}
	goto __finally20;
	__catch20_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 252 "PhotoPage.vala"
			g_error ("PhotoPage.vala:252: %s", err->message);
#line 1198 "PhotoPage.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally20:
	if (_inner_error_ != NULL) {
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* set up icon for drag-and-drop*/
	icon = photo_get_thumbnail (self->priv->photo, THUMBNAIL_CACHE_MEDIUM_SCALE);
#line 257 "PhotoPage.vala"
	gtk_drag_source_set_icon_pixbuf (GTK_WIDGET (SINGLE_PHOTO_PAGE (self)->canvas), icon);
#line 952 "gio-2.0.vapi"
	_tmp2_ = NULL;
#line 259 "PhotoPage.vala"
	g_debug ("PhotoPage.vala:259: Prepared for export %s", _tmp2_ = g_file_get_path (file));
#line 1217 "PhotoPage.c"
	_tmp2_ = (g_free (_tmp2_), NULL);
	_tmp4_ = NULL;
#line 261 "PhotoPage.vala"
	_tmp3_ = NULL;
#line 261 "PhotoPage.vala"
	self->priv->drag_file = (_tmp4_ = (_tmp3_ = file, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (self->priv->drag_file == NULL) ? NULL : (self->priv->drag_file = (g_object_unref (self->priv->drag_file), NULL)), _tmp4_);
#line 1224 "PhotoPage.c"
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
}


#line 264 "PhotoPage.vala"
static void photo_page_real_drag_data_get (GtkWidget* base, GdkDragContext* context, GtkSelectionData* selection_data, guint target_type, guint time) {
#line 1232 "PhotoPage.c"
	PhotoPage * self;
	char** _tmp0_;
	gint uris_size;
	gint uris_length1;
	char** uris;
	char* _tmp1_;
	self = PHOTO_PAGE (base);
#line 264 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 264 "PhotoPage.vala"
	g_return_if_fail (selection_data != NULL);
#line 266 "PhotoPage.vala"
	g_assert (target_type == PAGE_TARGET_TYPE_URI_LIST);
#line 268 "PhotoPage.vala"
	if (self->priv->drag_file == NULL) {
#line 1248 "PhotoPage.c"
		return;
	}
#line 271 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 1253 "PhotoPage.c"
	uris = (_tmp0_ = g_new0 (char*, 1 + 1), uris_length1 = 1, uris_size = uris_length1, _tmp0_);
	_tmp1_ = NULL;
#line 272 "PhotoPage.vala"
	uris[0] = (_tmp1_ = g_file_get_uri (self->priv->drag_file), uris[0] = (g_free (uris[0]), NULL), _tmp1_);
#line 274 "PhotoPage.vala"
	gtk_selection_data_set_uris (selection_data, uris);
#line 1260 "PhotoPage.c"
	uris = (_vala_array_free (uris, uris_length1, (GDestroyNotify) g_free), NULL);
}


#line 277 "PhotoPage.vala"
static void photo_page_real_drag_end (GtkWidget* base, GdkDragContext* context) {
#line 1267 "PhotoPage.c"
	PhotoPage * self;
	GFile* _tmp0_;
	self = PHOTO_PAGE (base);
#line 277 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 1273 "PhotoPage.c"
	_tmp0_ = NULL;
#line 278 "PhotoPage.vala"
	self->priv->drag_file = (_tmp0_ = NULL, (self->priv->drag_file == NULL) ? NULL : (self->priv->drag_file = (g_object_unref (self->priv->drag_file), NULL)), _tmp0_);
#line 1277 "PhotoPage.c"
}


#line 281 "PhotoPage.vala"
static gboolean photo_page_real_source_drag_failed (Page* base, GdkDragContext* context, GtkDragResult drag_result) {
#line 1283 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	GFile* _tmp0_;
	self = PHOTO_PAGE (base);
#line 281 "PhotoPage.vala"
	g_return_val_if_fail (GDK_IS_DRAG_CONTEXT (context), FALSE);
#line 282 "PhotoPage.vala"
	g_debug ("PhotoPage.vala:282: Drag failed: %d", (gint) drag_result);
#line 1292 "PhotoPage.c"
	_tmp0_ = NULL;
#line 284 "PhotoPage.vala"
	self->priv->drag_file = (_tmp0_ = NULL, (self->priv->drag_file == NULL) ? NULL : (self->priv->drag_file = (g_object_unref (self->priv->drag_file), NULL)), _tmp0_);
#line 1296 "PhotoPage.c"
	result = FALSE;
	return result;
}


/* Return true to block the DnD handler from activating a drag*/
#line 290 "PhotoPage.vala"
static gboolean photo_page_real_on_left_click (Page* base, const GdkEventButton* event) {
#line 1305 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gint x;
	gint y;
	self = PHOTO_PAGE (base);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
#line 293 "PhotoPage.vala"
	if ((*event).type == GDK_2BUTTON_PRESS) {
#line 293 "PhotoPage.vala"
		_tmp1_ = self->priv->current_tool == NULL;
#line 1319 "PhotoPage.c"
	} else {
#line 293 "PhotoPage.vala"
		_tmp1_ = FALSE;
#line 1323 "PhotoPage.c"
	}
#line 293 "PhotoPage.vala"
	if (_tmp1_) {
#line 294 "PhotoPage.vala"
		_tmp0_ = !IS_FULLSCREEN_WINDOW (self->priv->container);
#line 1329 "PhotoPage.c"
	} else {
#line 293 "PhotoPage.vala"
		_tmp0_ = FALSE;
#line 1333 "PhotoPage.c"
	}
	/* on double-click, if not editing and not hosted by a fullscreen window, return to the
	 controller collection*/
#line 293 "PhotoPage.vala"
	if (_tmp0_) {
#line 295 "PhotoPage.vala"
		photo_page_on_return_to_collection (self);
#line 1341 "PhotoPage.c"
		result = TRUE;
		return result;
	}
	x = (gint) (*event).x;
	y = (gint) (*event).y;
	/* only concerned about mouse-downs on the pixbuf ... return true prevents DnD when the
	 user drags outside the displayed photo*/
#line 305 "PhotoPage.vala"
	if (!single_photo_page_is_inside_pixbuf (SINGLE_PHOTO_PAGE (self), x, y)) {
#line 1351 "PhotoPage.c"
		result = TRUE;
		return result;
	}
	/* if no editing tool, then done*/
#line 309 "PhotoPage.vala"
	if (self->priv->current_tool == NULL) {
#line 1358 "PhotoPage.c"
		result = FALSE;
		return result;
	}
#line 312 "PhotoPage.vala"
	editing_tool_on_left_click (self->priv->current_tool, x, y);
#line 1364 "PhotoPage.c"
	/* block DnD handlers if tool is enabled*/
	result = TRUE;
	return result;
}


#line 318 "PhotoPage.vala"
static gboolean photo_page_real_on_left_released (Page* base, const GdkEventButton* event) {
#line 1373 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	self = PHOTO_PAGE (base);
	/* report all releases, as it's possible the user click and dragged from inside the
	 pixbuf to the gutters*/
#line 321 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 322 "PhotoPage.vala"
		editing_tool_on_left_released (self->priv->current_tool, (gint) (*event).x, (gint) (*event).y);
#line 1383 "PhotoPage.c"
	}
	result = FALSE;
	return result;
}


#line 327 "PhotoPage.vala"
static gboolean photo_page_real_on_right_click (Page* base, const GdkEventButton* event) {
#line 1392 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	self = PHOTO_PAGE (base);
	result = photo_page_on_context_menu (self, &(*event));
	return result;
}


#line 331 "PhotoPage.vala"
static void photo_page_on_view_menu (PhotoPage* self) {
#line 1403 "PhotoPage.c"
	GtkMenuItem* _tmp0_;
	GtkMenuItem* return_item;
	gboolean _tmp1_;
#line 331 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 332 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 1411 "PhotoPage.c"
	return_item = (_tmp0_ = GTK_MENU_ITEM (gtk_ui_manager_get_widget (PAGE (self)->ui, "/PhotoMenuBar/ViewMenu/ReturnToPage")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp1_ = FALSE;
#line 333 "PhotoPage.vala"
	if (return_item != NULL) {
#line 333 "PhotoPage.vala"
		_tmp1_ = self->priv->controller != NULL;
#line 1418 "PhotoPage.c"
	} else {
#line 333 "PhotoPage.vala"
		_tmp1_ = FALSE;
#line 1422 "PhotoPage.c"
	}
#line 333 "PhotoPage.vala"
	if (_tmp1_) {
#line 1426 "PhotoPage.c"
		GtkLabel* _tmp2_;
		GtkLabel* label;
#line 334 "PhotoPage.vala"
		_tmp2_ = NULL;
#line 1431 "PhotoPage.c"
		label = (_tmp2_ = GTK_LABEL (gtk_bin_get_child (GTK_BIN (return_item))), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
#line 335 "PhotoPage.vala"
		if (label != NULL) {
#line 1435 "PhotoPage.c"
			char* _tmp4_;
			char* _tmp3_;
#line 820 "glib-2.0.vapi"
			_tmp4_ = NULL;
#line 58 "Page.vala"
			_tmp3_ = NULL;
#line 336 "PhotoPage.vala"
			gtk_label_set_text (label, _tmp4_ = g_strdup_printf ("Return to %s", _tmp3_ = page_get_page_name (PAGE (self->priv->controller))));
#line 1444 "PhotoPage.c"
			_tmp4_ = (g_free (_tmp4_), NULL);
			_tmp3_ = (g_free (_tmp3_), NULL);
		}
		(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	}
	(return_item == NULL) ? NULL : (return_item = (g_object_unref (return_item), NULL));
}


#line 340 "PhotoPage.vala"
static void photo_page_on_return_to_collection (PhotoPage* self) {
#line 1456 "PhotoPage.c"
	AppWindow* _tmp0_;
#line 340 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 341 "PhotoPage.vala"
	app_window_switch_to_page (_tmp0_ = app_window_get_instance (), PAGE (self->priv->controller));
#line 1464 "PhotoPage.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 344 "PhotoPage.vala"
static void photo_page_on_export (PhotoPage* self) {
#line 1471 "PhotoPage.c"
	GError * _inner_error_;
	ExportDialog* export_dialog;
	gint scale;
	ScaleConstraint constraint;
	JpegQuality quality;
	GFile* _tmp0_;
	GFile* _tmp1_;
	GFile* save_as;
#line 344 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 1482 "PhotoPage.c"
	_inner_error_ = NULL;
	export_dialog = g_object_ref_sink (export_dialog_new (1));
	scale = 0;
	constraint = 0;
	quality = 0;
#line 350 "PhotoPage.vala"
	if (!export_dialog_execute (export_dialog, &scale, &constraint, &quality)) {
#line 1490 "PhotoPage.c"
		(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
		return;
	}
#line 167 "Photo.vala"
	_tmp0_ = NULL;
#line 353 "PhotoPage.vala"
	_tmp1_ = NULL;
#line 1498 "PhotoPage.c"
	save_as = (_tmp1_ = export_ui_choose_file (_tmp0_ = photo_get_file (self->priv->photo)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 354 "PhotoPage.vala"
	if (save_as == NULL) {
#line 1502 "PhotoPage.c"
		(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
		(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
		return;
	}
	{
#line 358 "PhotoPage.vala"
		photo_export (self->priv->photo, save_as, scale, constraint, quality, &_inner_error_);
#line 1510 "PhotoPage.c"
		if (_inner_error_ != NULL) {
			goto __catch21_g_error;
			goto __finally21;
		}
	}
	goto __finally21;
	__catch21_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp3_;
			char* _tmp2_;
#line 820 "glib-2.0.vapi"
			_tmp3_ = NULL;
#line 952 "gio-2.0.vapi"
			_tmp2_ = NULL;
#line 360 "PhotoPage.vala"
			app_window_error_message (_tmp3_ = g_strdup_printf ("Unable to export %s: %s", _tmp2_ = g_file_get_path (save_as), err->message));
#line 1531 "PhotoPage.c"
			_tmp3_ = (g_free (_tmp3_), NULL);
			_tmp2_ = (g_free (_tmp2_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally21:
	if (_inner_error_ != NULL) {
		(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
		(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(export_dialog == NULL) ? NULL : (export_dialog = (g_object_unref (export_dialog), NULL));
	(save_as == NULL) ? NULL : (save_as = (g_object_unref (save_as), NULL));
}


#line 364 "PhotoPage.vala"
static void photo_page_on_photo_altered (PhotoPage* self, Photo* p) {
#line 364 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 364 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO (p));
#line 365 "PhotoPage.vala"
	g_assert (photo_equals (self->priv->photo, p));
#line 367 "PhotoPage.vala"
	photo_page_update_pixbuf (self);
#line 1560 "PhotoPage.c"
}


#line 370 "PhotoPage.vala"
static gboolean photo_page_real_on_motion (Page* base, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask) {
#line 1566 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	self = PHOTO_PAGE (base);
#line 371 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 372 "PhotoPage.vala"
		editing_tool_on_motion (self->priv->current_tool, x, y, mask);
#line 1574 "PhotoPage.c"
	}
	result = FALSE;
	return result;
}


#line 377 "PhotoPage.vala"
static gboolean photo_page_on_context_menu (PhotoPage* self, const GdkEventButton* event) {
#line 1583 "PhotoPage.c"
	gboolean result;
#line 377 "PhotoPage.vala"
	g_return_val_if_fail (IS_PHOTO_PAGE (self), FALSE);
#line 378 "PhotoPage.vala"
	if (self->priv->photo == NULL) {
#line 1589 "PhotoPage.c"
		result = FALSE;
		return result;
	}
#line 381 "PhotoPage.vala"
	page_set_item_sensitive (PAGE (self), "/PhotoContextMenu/ContextRevert", photo_has_transformations (self->priv->photo));
#line 383 "PhotoPage.vala"
	gtk_menu_popup (self->priv->context_menu, NULL, NULL, NULL, NULL, (*event).button, (*event).time);
#line 1597 "PhotoPage.c"
	result = TRUE;
	return result;
}


#line 388 "PhotoPage.vala"
static gboolean photo_page_real_on_configure (Page* base, const GdkEventConfigure* event, const GdkRectangle* rect) {
#line 1605 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	gboolean _tmp1_;
	self = PHOTO_PAGE (base);
	/* if editing tool window is present and the user hasn't touched it, it moves with the window*/
#line 390 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 1613 "PhotoPage.c"
		EditingToolWindow* tool_window;
		gboolean _tmp0_;
		tool_window = editing_tool_get_tool_window (self->priv->current_tool);
		_tmp0_ = FALSE;
#line 392 "PhotoPage.vala"
		if (tool_window != NULL) {
#line 392 "PhotoPage.vala"
			_tmp0_ = !editing_tool_window_has_user_moved (tool_window);
#line 1622 "PhotoPage.c"
		} else {
#line 392 "PhotoPage.vala"
			_tmp0_ = FALSE;
#line 1626 "PhotoPage.c"
		}
#line 392 "PhotoPage.vala"
		if (_tmp0_) {
#line 393 "PhotoPage.vala"
			photo_page_place_tool_window (self);
#line 1632 "PhotoPage.c"
		}
		(tool_window == NULL) ? NULL : (tool_window = (g_object_unref (tool_window), NULL));
	}
	_tmp1_ = FALSE;
#line 396 "PhotoPage.vala"
	if (PAGE_CLASS (photo_page_parent_class)->on_configure != NULL) {
#line 396 "PhotoPage.vala"
		_tmp1_ = PAGE_CLASS (photo_page_parent_class)->on_configure (PAGE (SINGLE_PHOTO_PAGE (self)), &(*event), &(*rect));
#line 1641 "PhotoPage.c"
	} else {
#line 396 "PhotoPage.vala"
		_tmp1_ = FALSE;
#line 1645 "PhotoPage.c"
	}
	result = _tmp1_;
	return result;
}


#line 399 "PhotoPage.vala"
static gboolean photo_page_real_key_press_event (GtkWidget* base, const GdkEventKey* event) {
#line 1654 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	gboolean handled;
	GQuark _tmp1_;
	const char* _tmp0_;
	static GQuark _tmp1__label0 = 0;
	static GQuark _tmp1__label1 = 0;
	static GQuark _tmp1__label2 = 0;
	static GQuark _tmp1__label3 = 0;
	gboolean _tmp2_;
	self = PHOTO_PAGE (base);
	/* editing tool gets first crack at the keypress*/
#line 401 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 402 "PhotoPage.vala"
		if (editing_tool_on_keypress (self->priv->current_tool, &(*event))) {
#line 1671 "PhotoPage.c"
			result = TRUE;
			return result;
		}
	}
	handled = TRUE;
#line 407 "PhotoPage.vala"
	_tmp0_ = NULL;
#line 1679 "PhotoPage.c"
	_tmp0_ = gdk_keyval_name ((*event).keyval);
	_tmp1_ = (NULL == _tmp0_) ? 0 : g_quark_from_string (_tmp0_);
	if ((_tmp1_ == ((0 != _tmp1__label0) ? _tmp1__label0 : (_tmp1__label0 = g_quark_from_static_string ("Left")))) || (_tmp1_ == ((0 != _tmp1__label1) ? _tmp1__label1 : (_tmp1__label1 = g_quark_from_static_string ("KP_Left")))))
	do {
#line 410 "PhotoPage.vala"
		photo_page_on_previous_photo (self);
#line 411 "PhotoPage.vala"
		break;
#line 1688 "PhotoPage.c"
	} while (0); else if ((_tmp1_ == ((0 != _tmp1__label2) ? _tmp1__label2 : (_tmp1__label2 = g_quark_from_static_string ("Right")))) || (_tmp1_ == ((0 != _tmp1__label3) ? _tmp1__label3 : (_tmp1__label3 = g_quark_from_static_string ("KP_Right")))))
	do {
#line 415 "PhotoPage.vala"
		photo_page_on_next_photo (self);
#line 416 "PhotoPage.vala"
		break;
#line 1695 "PhotoPage.c"
	} while (0); else
	do {
#line 419 "PhotoPage.vala"
		handled = FALSE;
#line 420 "PhotoPage.vala"
		break;
#line 1702 "PhotoPage.c"
	} while (0);
#line 423 "PhotoPage.vala"
	if (handled) {
#line 1706 "PhotoPage.c"
		result = TRUE;
		return result;
	}
	_tmp2_ = FALSE;
#line 426 "PhotoPage.vala"
	if (GTK_WIDGET_CLASS (photo_page_parent_class)->key_press_event != NULL) {
#line 426 "PhotoPage.vala"
		_tmp2_ = GTK_WIDGET_CLASS (photo_page_parent_class)->key_press_event (GTK_WIDGET (SINGLE_PHOTO_PAGE (self)), &(*event));
#line 1715 "PhotoPage.c"
	} else {
#line 426 "PhotoPage.vala"
		_tmp2_ = TRUE;
#line 1719 "PhotoPage.c"
	}
	result = _tmp2_;
	return result;
}


#line 429 "PhotoPage.vala"
static void photo_page_real_new_drawable (SinglePhotoPage* base, GdkGC* default_gc, GdkDrawable* drawable) {
#line 1728 "PhotoPage.c"
	PhotoPage * self;
	self = PHOTO_PAGE (base);
#line 429 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_GC (default_gc));
#line 429 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 1735 "PhotoPage.c"
	/* if tool is open, update its canvas object*/
#line 431 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 432 "PhotoPage.vala"
		photo_canvas_set_drawable (self->priv->current_tool->canvas, default_gc, drawable);
#line 1741 "PhotoPage.c"
	}
}


#line 435 "PhotoPage.vala"
static void photo_page_real_updated_pixbuf (SinglePhotoPage* base, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim) {
#line 1748 "PhotoPage.c"
	PhotoPage * self;
	self = PHOTO_PAGE (base);
#line 435 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 1753 "PhotoPage.c"
	/* only purpose here is to inform editing tool of change*/
#line 438 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 1757 "PhotoPage.c"
		GdkRectangle _tmp0_ = {0};
#line 439 "PhotoPage.vala"
		photo_canvas_resized_pixbuf (self->priv->current_tool->canvas, &(*old_dim), pixbuf, (_tmp0_ = single_photo_page_get_scaled_pixbuf_position (SINGLE_PHOTO_PAGE (self)), &_tmp0_));
#line 1761 "PhotoPage.c"
	}
}


#line 442 "PhotoPage.vala"
static void photo_page_real_paint (SinglePhotoPage* base, GdkGC* gc, GdkDrawable* drawable) {
#line 1768 "PhotoPage.c"
	PhotoPage * self;
	self = PHOTO_PAGE (base);
#line 442 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 442 "PhotoPage.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 443 "PhotoPage.vala"
	if (self->priv->current_tool != NULL) {
#line 444 "PhotoPage.vala"
		editing_tool_paint (self->priv->current_tool, gc, drawable);
#line 1779 "PhotoPage.c"
	} else {
#line 446 "PhotoPage.vala"
		SINGLE_PHOTO_PAGE_CLASS (photo_page_parent_class)->paint (SINGLE_PHOTO_PAGE (self), gc, drawable);
#line 1783 "PhotoPage.c"
	}
}


#line 449 "PhotoPage.vala"
static void photo_page_rotate (PhotoPage* self, Rotation rotation) {
#line 449 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 450 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 1794 "PhotoPage.c"
	/* let the signal generate a repaint*/
#line 453 "PhotoPage.vala"
	photo_rotate (self->priv->photo, rotation);
#line 1798 "PhotoPage.c"
}


#line 456 "PhotoPage.vala"
static void photo_page_on_rotate_clockwise (PhotoPage* self) {
#line 456 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 457 "PhotoPage.vala"
	photo_page_rotate (self, ROTATION_CLOCKWISE);
#line 1808 "PhotoPage.c"
}


#line 460 "PhotoPage.vala"
static void photo_page_on_rotate_counterclockwise (PhotoPage* self) {
#line 460 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 461 "PhotoPage.vala"
	photo_page_rotate (self, ROTATION_COUNTERCLOCKWISE);
#line 1818 "PhotoPage.c"
}


#line 464 "PhotoPage.vala"
static void photo_page_on_mirror (PhotoPage* self) {
#line 464 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 465 "PhotoPage.vala"
	photo_page_rotate (self, ROTATION_MIRROR);
#line 1828 "PhotoPage.c"
}


#line 468 "PhotoPage.vala"
static void photo_page_on_revert (PhotoPage* self) {
#line 468 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 469 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 471 "PhotoPage.vala"
	photo_remove_all_transformations (self->priv->photo);
#line 1840 "PhotoPage.c"
}


#line 460 "PhotoPage.vala"
static void _photo_page_on_rotate_counterclockwise_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 1846 "PhotoPage.c"
	photo_page_on_rotate_counterclockwise (self);
}


#line 474 "PhotoPage.vala"
static gboolean photo_page_real_on_ctrl_pressed (Page* base, const GdkEventKey* event) {
#line 1853 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	guint _tmp0_;
	self = PHOTO_PAGE (base);
#line 475 "PhotoPage.vala"
	gtk_tool_button_set_stock_id (self->priv->rotate_button, RESOURCES_COUNTERCLOCKWISE);
#line 476 "PhotoPage.vala"
	gtk_tool_button_set_label (self->priv->rotate_button, RESOURCES_ROTATE_COUNTERCLOCKWISE_LABEL);
#line 477 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->rotate_button), RESOURCES_ROTATE_COUNTERCLOCKWISE_TOOLTIP);
#line 478 "PhotoPage.vala"
	g_signal_handlers_disconnect_matched (self->priv->rotate_button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_TOOL_BUTTON, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _photo_page_on_rotate_clockwise_gtk_tool_button_clicked, self);
#line 479 "PhotoPage.vala"
	g_signal_connect_object (self->priv->rotate_button, "clicked", (GCallback) _photo_page_on_rotate_counterclockwise_gtk_tool_button_clicked, self, 0);
#line 1868 "PhotoPage.c"
	result = FALSE;
	return result;
}


#line 484 "PhotoPage.vala"
static gboolean photo_page_real_on_ctrl_released (Page* base, const GdkEventKey* event) {
#line 1876 "PhotoPage.c"
	PhotoPage * self;
	gboolean result;
	guint _tmp0_;
	self = PHOTO_PAGE (base);
#line 485 "PhotoPage.vala"
	gtk_tool_button_set_stock_id (self->priv->rotate_button, RESOURCES_CLOCKWISE);
#line 486 "PhotoPage.vala"
	gtk_tool_button_set_label (self->priv->rotate_button, RESOURCES_ROTATE_CLOCKWISE_LABEL);
#line 487 "PhotoPage.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->rotate_button), RESOURCES_ROTATE_CLOCKWISE_TOOLTIP);
#line 488 "PhotoPage.vala"
	g_signal_handlers_disconnect_matched (self->priv->rotate_button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_TOOL_BUTTON, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _photo_page_on_rotate_counterclockwise_gtk_tool_button_clicked, self);
#line 489 "PhotoPage.vala"
	g_signal_connect_object (self->priv->rotate_button, "clicked", (GCallback) _photo_page_on_rotate_clockwise_gtk_tool_button_clicked, self, 0);
#line 1891 "PhotoPage.c"
	result = FALSE;
	return result;
}


#line 556 "PhotoPage.vala"
static void _photo_page_on_crop_activated_editing_tool_activated (CropTool* _sender, gpointer self) {
#line 1899 "PhotoPage.c"
	photo_page_on_crop_activated (self);
}


#line 564 "PhotoPage.vala"
static void _photo_page_on_crop_deactivated_editing_tool_deactivated (CropTool* _sender, gpointer self) {
#line 1906 "PhotoPage.c"
	photo_page_on_crop_deactivated (self);
}


#line 537 "PhotoPage.vala"
static void _photo_page_on_crop_done_editing_tool_applied (CropTool* _sender, gpointer self) {
#line 1913 "PhotoPage.c"
	photo_page_on_crop_done (self);
}


#line 537 "PhotoPage.vala"
static void _photo_page_on_crop_done_editing_tool_cancelled (CropTool* _sender, gpointer self) {
#line 1920 "PhotoPage.c"
	photo_page_on_crop_done (self);
}


#line 494 "PhotoPage.vala"
static void photo_page_on_crop_toggled (PhotoPage* self) {
#line 494 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 495 "PhotoPage.vala"
	if (gtk_toggle_tool_button_get_active (self->priv->crop_button)) {
#line 1931 "PhotoPage.c"
		CropTool* crop_tool;
		/* create the tool, hook its signals, and activate it*/
		crop_tool = crop_tool_new ();
#line 498 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (crop_tool), "activated", (GCallback) _photo_page_on_crop_activated_editing_tool_activated, self, 0);
#line 499 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (crop_tool), "deactivated", (GCallback) _photo_page_on_crop_deactivated_editing_tool_deactivated, self, 0);
#line 500 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (crop_tool), "applied", (GCallback) _photo_page_on_crop_done_editing_tool_applied, self, 0);
#line 501 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (crop_tool), "cancelled", (GCallback) _photo_page_on_crop_done_editing_tool_cancelled, self, 0);
#line 503 "PhotoPage.vala"
		photo_page_activate_tool (self, EDITING_TOOL (crop_tool));
#line 1945 "PhotoPage.c"
		(crop_tool == NULL) ? NULL : (crop_tool = (editing_tool_unref (crop_tool), NULL));
	} else {
#line 505 "PhotoPage.vala"
		photo_page_deactivate_tool (self);
#line 1950 "PhotoPage.c"
	}
}


#line 560 "PhotoPage.vala"
static void _photo_page_on_redeye_activated_editing_tool_activated (RedeyeTool* _sender, gpointer self) {
#line 1957 "PhotoPage.c"
	photo_page_on_redeye_activated (self);
}


#line 568 "PhotoPage.vala"
static void _photo_page_on_redeye_deactivated_editing_tool_deactivated (RedeyeTool* _sender, gpointer self) {
#line 1964 "PhotoPage.c"
	photo_page_on_redeye_deactivated (self);
}


#line 541 "PhotoPage.vala"
static void _photo_page_on_redeye_applied_editing_tool_applied (RedeyeTool* _sender, gpointer self) {
#line 1971 "PhotoPage.c"
	photo_page_on_redeye_applied (self);
}


#line 544 "PhotoPage.vala"
static void _photo_page_on_redeye_closed_editing_tool_cancelled (RedeyeTool* _sender, gpointer self) {
#line 1978 "PhotoPage.c"
	photo_page_on_redeye_closed (self);
}


#line 509 "PhotoPage.vala"
static void photo_page_on_redeye_toggled (PhotoPage* self) {
#line 509 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 510 "PhotoPage.vala"
	if (gtk_toggle_tool_button_get_active (self->priv->redeye_button)) {
#line 1989 "PhotoPage.c"
		RedeyeTool* redeye_tool;
		redeye_tool = redeye_tool_new ();
#line 512 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (redeye_tool), "activated", (GCallback) _photo_page_on_redeye_activated_editing_tool_activated, self, 0);
#line 513 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (redeye_tool), "deactivated", (GCallback) _photo_page_on_redeye_deactivated_editing_tool_deactivated, self, 0);
#line 514 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (redeye_tool), "applied", (GCallback) _photo_page_on_redeye_applied_editing_tool_applied, self, 0);
#line 515 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (redeye_tool), "cancelled", (GCallback) _photo_page_on_redeye_closed_editing_tool_cancelled, self, 0);
#line 517 "PhotoPage.vala"
		photo_page_activate_tool (self, EDITING_TOOL (redeye_tool));
#line 2002 "PhotoPage.c"
		(redeye_tool == NULL) ? NULL : (redeye_tool = (editing_tool_unref (redeye_tool), NULL));
	} else {
#line 519 "PhotoPage.vala"
		photo_page_deactivate_tool (self);
#line 2007 "PhotoPage.c"
	}
}


#line 572 "PhotoPage.vala"
static void _photo_page_on_adjust_activated_editing_tool_activated (AdjustTool* _sender, gpointer self) {
#line 2014 "PhotoPage.c"
	photo_page_on_adjust_activated (self);
}


#line 576 "PhotoPage.vala"
static void _photo_page_on_adjust_deactivated_editing_tool_deactivated (AdjustTool* _sender, gpointer self) {
#line 2021 "PhotoPage.c"
	photo_page_on_adjust_deactivated (self);
}


#line 548 "PhotoPage.vala"
static void _photo_page_on_adjust_closed_editing_tool_cancelled (AdjustTool* _sender, gpointer self) {
#line 2028 "PhotoPage.c"
	photo_page_on_adjust_closed (self);
}


#line 552 "PhotoPage.vala"
static void _photo_page_on_adjust_applied_editing_tool_applied (AdjustTool* _sender, gpointer self) {
#line 2035 "PhotoPage.c"
	photo_page_on_adjust_applied (self);
}


#line 523 "PhotoPage.vala"
static void photo_page_on_adjust_toggled (PhotoPage* self) {
#line 523 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 524 "PhotoPage.vala"
	if (gtk_toggle_tool_button_get_active (self->priv->adjust_button)) {
#line 2046 "PhotoPage.c"
		AdjustTool* adjust_tool;
		adjust_tool = adjust_tool_new ();
#line 526 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (adjust_tool), "activated", (GCallback) _photo_page_on_adjust_activated_editing_tool_activated, self, 0);
#line 527 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (adjust_tool), "deactivated", (GCallback) _photo_page_on_adjust_deactivated_editing_tool_deactivated, self, 0);
#line 528 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (adjust_tool), "cancelled", (GCallback) _photo_page_on_adjust_closed_editing_tool_cancelled, self, 0);
#line 529 "PhotoPage.vala"
		g_signal_connect_object (EDITING_TOOL (adjust_tool), "applied", (GCallback) _photo_page_on_adjust_applied_editing_tool_applied, self, 0);
#line 531 "PhotoPage.vala"
		photo_page_activate_tool (self, EDITING_TOOL (adjust_tool));
#line 2059 "PhotoPage.c"
		(adjust_tool == NULL) ? NULL : (adjust_tool = (editing_tool_unref (adjust_tool), NULL));
	} else {
#line 533 "PhotoPage.vala"
		photo_page_deactivate_tool (self);
#line 2064 "PhotoPage.c"
	}
}


#line 537 "PhotoPage.vala"
static void photo_page_on_crop_done (PhotoPage* self) {
#line 537 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 538 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 2075 "PhotoPage.c"
}


#line 541 "PhotoPage.vala"
static void photo_page_on_redeye_applied (PhotoPage* self) {
#line 541 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 2083 "PhotoPage.c"
}


#line 544 "PhotoPage.vala"
static void photo_page_on_redeye_closed (PhotoPage* self) {
#line 544 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 545 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 2093 "PhotoPage.c"
}


#line 548 "PhotoPage.vala"
static void photo_page_on_adjust_closed (PhotoPage* self) {
#line 548 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 549 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 2103 "PhotoPage.c"
}


#line 552 "PhotoPage.vala"
static void photo_page_on_adjust_applied (PhotoPage* self) {
#line 552 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 553 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 2113 "PhotoPage.c"
}


#line 556 "PhotoPage.vala"
static void photo_page_on_crop_activated (PhotoPage* self) {
#line 556 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 557 "PhotoPage.vala"
	gtk_toggle_tool_button_set_active (self->priv->crop_button, TRUE);
#line 2123 "PhotoPage.c"
}


#line 560 "PhotoPage.vala"
static void photo_page_on_redeye_activated (PhotoPage* self) {
#line 560 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 561 "PhotoPage.vala"
	gtk_toggle_tool_button_set_active (self->priv->redeye_button, TRUE);
#line 2133 "PhotoPage.c"
}


#line 564 "PhotoPage.vala"
static void photo_page_on_crop_deactivated (PhotoPage* self) {
#line 564 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 565 "PhotoPage.vala"
	gtk_toggle_tool_button_set_active (self->priv->crop_button, FALSE);
#line 2143 "PhotoPage.c"
}


#line 568 "PhotoPage.vala"
static void photo_page_on_redeye_deactivated (PhotoPage* self) {
#line 568 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 569 "PhotoPage.vala"
	gtk_toggle_tool_button_set_active (self->priv->redeye_button, FALSE);
#line 2153 "PhotoPage.c"
}


#line 572 "PhotoPage.vala"
static void photo_page_on_adjust_activated (PhotoPage* self) {
#line 572 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 573 "PhotoPage.vala"
	gtk_toggle_tool_button_set_active (self->priv->adjust_button, TRUE);
#line 2163 "PhotoPage.c"
}


#line 576 "PhotoPage.vala"
static void photo_page_on_adjust_deactivated (PhotoPage* self) {
#line 576 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 577 "PhotoPage.vala"
	gtk_toggle_tool_button_set_active (self->priv->adjust_button, FALSE);
#line 2173 "PhotoPage.c"
}


#line 580 "PhotoPage.vala"
static void photo_page_place_tool_window (PhotoPage* self) {
#line 2179 "PhotoPage.c"
	EditingToolWindow* tool_window;
	GtkRequisition req = {0};
	AppWindow* _tmp0_;
	gboolean _tmp1_;
#line 580 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 581 "PhotoPage.vala"
	if (self->priv->current_tool == NULL) {
#line 2188 "PhotoPage.c"
		return;
	}
	tool_window = editing_tool_get_tool_window (self->priv->current_tool);
#line 585 "PhotoPage.vala"
	if (tool_window == NULL) {
#line 2194 "PhotoPage.c"
		(tool_window == NULL) ? NULL : (tool_window = (g_object_unref (tool_window), NULL));
		return;
	}
#line 589 "PhotoPage.vala"
	gtk_widget_size_request (GTK_WIDGET (tool_window), &req);
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 591 "PhotoPage.vala"
	if ((_tmp1_ = self->priv->container == GTK_WINDOW (_tmp0_ = app_window_get_instance ()), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_)) {
#line 2204 "PhotoPage.c"
		gint rx;
		gint ry;
		gint cx;
		gint cy;
		gint cwidth;
		gint cheight;
		/* Normal: position crop tool window centered on viewport/canvas at the bottom, straddling
		 the canvas and the toolbar*/
		rx = 0;
		ry = 0;
#line 595 "PhotoPage.vala"
		gdk_window_get_root_origin (GTK_WIDGET (self->priv->container)->window, &rx, &ry);
#line 2217 "PhotoPage.c"
		cx = 0;
		cy = 0;
		cwidth = 0;
		cheight = 0;
#line 598 "PhotoPage.vala"
		cx = GTK_WIDGET (SINGLE_PHOTO_PAGE (self)->viewport)->allocation.x;
#line 599 "PhotoPage.vala"
		cy = GTK_WIDGET (SINGLE_PHOTO_PAGE (self)->viewport)->allocation.y;
#line 600 "PhotoPage.vala"
		cwidth = GTK_WIDGET (SINGLE_PHOTO_PAGE (self)->viewport)->allocation.width;
#line 601 "PhotoPage.vala"
		cheight = GTK_WIDGET (SINGLE_PHOTO_PAGE (self)->viewport)->allocation.height;
#line 603 "PhotoPage.vala"
		gtk_window_move (GTK_WINDOW (tool_window), ((rx + cx) + (cwidth / 2)) - (req.width / 2), (ry + cy) + cheight);
#line 2232 "PhotoPage.c"
	} else {
		GtkRequisition toolbar_req = {0};
		GdkScreen* _tmp2_;
		GdkScreen* screen;
		gint x;
		gint y;
#line 605 "PhotoPage.vala"
		g_assert (IS_FULLSCREEN_WINDOW (self->priv->container));
#line 2241 "PhotoPage.c"
		/* Fullscreen: position crop tool window centered on screen at the bottom, just above the
		 toolbar*/
#line 610 "PhotoPage.vala"
		gtk_widget_size_request (GTK_WIDGET (self->priv->toolbar), &toolbar_req);
#line 612 "PhotoPage.vala"
		_tmp2_ = NULL;
#line 2248 "PhotoPage.c"
		screen = (_tmp2_ = gtk_window_get_screen (self->priv->container), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
		x = (gdk_screen_get_width (screen) - req.width) / 2;
		y = ((gdk_screen_get_height (screen) - toolbar_req.height) - req.height) - PHOTO_PAGE_TOOL_WINDOW_SEPARATOR;
#line 616 "PhotoPage.vala"
		gtk_window_move (GTK_WINDOW (tool_window), x, y);
#line 2254 "PhotoPage.c"
		(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
	}
	(tool_window == NULL) ? NULL : (tool_window = (g_object_unref (tool_window), NULL));
}


#line 620 "PhotoPage.vala"
static void photo_page_show_tool_window (PhotoPage* self) {
#line 2263 "PhotoPage.c"
	EditingToolWindow* tool_window;
#line 620 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 621 "PhotoPage.vala"
	if (self->priv->current_tool == NULL) {
#line 2269 "PhotoPage.c"
		return;
	}
	tool_window = editing_tool_get_tool_window (self->priv->current_tool);
#line 625 "PhotoPage.vala"
	if (tool_window == NULL) {
#line 2275 "PhotoPage.c"
		(tool_window == NULL) ? NULL : (tool_window = (g_object_unref (tool_window), NULL));
		return;
	}
#line 628 "PhotoPage.vala"
	gtk_widget_show_all (GTK_WIDGET (tool_window));
#line 2281 "PhotoPage.c"
	(tool_window == NULL) ? NULL : (tool_window = (g_object_unref (tool_window), NULL));
}


#line 631 "PhotoPage.vala"
static void photo_page_on_photo_menu (PhotoPage* self) {
#line 2288 "PhotoPage.c"
	gboolean multiple;
	gboolean revert_possible;
#line 631 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 2293 "PhotoPage.c"
	multiple = FALSE;
#line 633 "PhotoPage.vala"
	if (self->priv->controller != NULL) {
#line 634 "PhotoPage.vala"
		multiple = checkerboard_page_get_count (self->priv->controller) > 1;
#line 2299 "PhotoPage.c"
	}
	revert_possible = FALSE;
#line 637 "PhotoPage.vala"
	if (self->priv->photo != NULL) {
#line 638 "PhotoPage.vala"
		revert_possible = photo_has_transformations (self->priv->photo);
#line 2306 "PhotoPage.c"
	}
#line 640 "PhotoPage.vala"
	page_set_item_sensitive (PAGE (self), "/PhotoMenuBar/PhotoMenu/PrevPhoto", multiple);
#line 641 "PhotoPage.vala"
	page_set_item_sensitive (PAGE (self), "/PhotoMenuBar/PhotoMenu/NextPhoto", multiple);
#line 642 "PhotoPage.vala"
	page_set_item_sensitive (PAGE (self), "/PhotoMenuBar/PhotoMenu/Revert", revert_possible);
#line 2314 "PhotoPage.c"
}


#line 645 "PhotoPage.vala"
static void photo_page_on_next_photo (PhotoPage* self) {
#line 2320 "PhotoPage.c"
	Thumbnail* _tmp0_;
#line 645 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 646 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 2326 "PhotoPage.c"
	_tmp0_ = NULL;
#line 648 "PhotoPage.vala"
	self->priv->thumbnail = (_tmp0_ = THUMBNAIL (checkerboard_page_get_next_item (self->priv->controller, LAYOUT_ITEM (self->priv->thumbnail))), (self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL)), _tmp0_);
#line 649 "PhotoPage.vala"
	photo_page_update_display (self);
#line 2332 "PhotoPage.c"
}


#line 652 "PhotoPage.vala"
static void photo_page_on_previous_photo (PhotoPage* self) {
#line 2338 "PhotoPage.c"
	Thumbnail* _tmp0_;
#line 652 "PhotoPage.vala"
	g_return_if_fail (IS_PHOTO_PAGE (self));
#line 653 "PhotoPage.vala"
	photo_page_deactivate_tool (self);
#line 2344 "PhotoPage.c"
	_tmp0_ = NULL;
#line 655 "PhotoPage.vala"
	self->priv->thumbnail = (_tmp0_ = THUMBNAIL (checkerboard_page_get_previous_item (self->priv->controller, LAYOUT_ITEM (self->priv->thumbnail))), (self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL)), _tmp0_);
#line 656 "PhotoPage.vala"
	photo_page_update_display (self);
#line 2350 "PhotoPage.c"
}


#line 13 "PhotoPage.vala"
static PhotoPagePhotoPageCanvas* photo_page_photo_page_canvas_construct (GType object_type, PhotoPage* photo_page) {
#line 2356 "PhotoPage.c"
	PhotoPagePhotoPageCanvas* self;
	GdkRectangle _tmp2_ = {0};
	GdkPixbuf* _tmp1_;
	GdkDrawable* _tmp0_;
	PhotoPage* _tmp4_;
	PhotoPage* _tmp3_;
#line 13 "PhotoPage.vala"
	g_return_val_if_fail (IS_PHOTO_PAGE (photo_page), NULL);
#line 1238 "Page.vala"
	_tmp1_ = NULL;
#line 1226 "Page.vala"
	_tmp0_ = NULL;
#line 14 "PhotoPage.vala"
	self = (PhotoPagePhotoPageCanvas*) photo_canvas_construct (object_type, photo_page->priv->container, GTK_WIDGET (SINGLE_PHOTO_PAGE (photo_page)->canvas)->window, photo_page->priv->photo, SINGLE_PHOTO_PAGE (photo_page)->canvas_gc, _tmp0_ = single_photo_page_get_drawable (SINGLE_PHOTO_PAGE (photo_page)), _tmp1_ = single_photo_page_get_scaled_pixbuf (SINGLE_PHOTO_PAGE (photo_page)), (_tmp2_ = single_photo_page_get_scaled_pixbuf_position (SINGLE_PHOTO_PAGE (photo_page)), &_tmp2_));
#line 2371 "PhotoPage.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	_tmp4_ = NULL;
#line 17 "PhotoPage.vala"
	_tmp3_ = NULL;
#line 17 "PhotoPage.vala"
	self->priv->photo_page = (_tmp4_ = (_tmp3_ = photo_page, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (self->priv->photo_page == NULL) ? NULL : (self->priv->photo_page = (g_object_unref (self->priv->photo_page), NULL)), _tmp4_);
#line 2379 "PhotoPage.c"
	return self;
}


#line 13 "PhotoPage.vala"
static PhotoPagePhotoPageCanvas* photo_page_photo_page_canvas_new (PhotoPage* photo_page) {
#line 13 "PhotoPage.vala"
	return photo_page_photo_page_canvas_construct (PHOTO_PAGE_TYPE_PHOTO_PAGE_CANVAS, photo_page);
#line 2388 "PhotoPage.c"
}


#line 20 "PhotoPage.vala"
static void photo_page_photo_page_canvas_real_repaint (PhotoCanvas* base) {
#line 2394 "PhotoPage.c"
	PhotoPagePhotoPageCanvas * self;
	self = PHOTO_PAGE_PHOTO_PAGE_CANVAS (base);
#line 21 "PhotoPage.vala"
	single_photo_page_repaint (SINGLE_PHOTO_PAGE (self->priv->photo_page), SINGLE_PHOTO_PAGE_QUALITY_INTERP);
#line 2399 "PhotoPage.c"
}


static void photo_page_photo_page_canvas_class_init (PhotoPagePhotoPageCanvasClass * klass) {
	photo_page_photo_page_canvas_parent_class = g_type_class_peek_parent (klass);
	PHOTO_CANVAS_CLASS (klass)->finalize = photo_page_photo_page_canvas_finalize;
	g_type_class_add_private (klass, sizeof (PhotoPagePhotoPageCanvasPrivate));
	PHOTO_CANVAS_CLASS (klass)->repaint = photo_page_photo_page_canvas_real_repaint;
}


static void photo_page_photo_page_canvas_instance_init (PhotoPagePhotoPageCanvas * self) {
	self->priv = PHOTO_PAGE_PHOTO_PAGE_CANVAS_GET_PRIVATE (self);
}


static void photo_page_photo_page_canvas_finalize (PhotoCanvas* obj) {
	PhotoPagePhotoPageCanvas * self;
	self = PHOTO_PAGE_PHOTO_PAGE_CANVAS (obj);
	(self->priv->photo_page == NULL) ? NULL : (self->priv->photo_page = (g_object_unref (self->priv->photo_page), NULL));
	PHOTO_CANVAS_CLASS (photo_page_photo_page_canvas_parent_class)->finalize (obj);
}


static GType photo_page_photo_page_canvas_get_type (void) {
	static GType photo_page_photo_page_canvas_type_id = 0;
	if (photo_page_photo_page_canvas_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PhotoPagePhotoPageCanvasClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_page_photo_page_canvas_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PhotoPagePhotoPageCanvas), 0, (GInstanceInitFunc) photo_page_photo_page_canvas_instance_init, NULL };
		photo_page_photo_page_canvas_type_id = g_type_register_static (TYPE_PHOTO_CANVAS, "PhotoPagePhotoPageCanvas", &g_define_type_info, 0);
	}
	return photo_page_photo_page_canvas_type_id;
}


static void photo_page_class_init (PhotoPageClass * klass) {
	photo_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (PhotoPagePrivate));
	PAGE_CLASS (klass)->get_toolbar = photo_page_real_get_toolbar;
	PAGE_CLASS (klass)->switching_from = photo_page_real_switching_from;
	PAGE_CLASS (klass)->switching_to_fullscreen = photo_page_real_switching_to_fullscreen;
	GTK_WIDGET_CLASS (klass)->drag_begin = photo_page_real_drag_begin;
	GTK_WIDGET_CLASS (klass)->drag_data_get = photo_page_real_drag_data_get;
	GTK_WIDGET_CLASS (klass)->drag_end = photo_page_real_drag_end;
	PAGE_CLASS (klass)->source_drag_failed = photo_page_real_source_drag_failed;
	PAGE_CLASS (klass)->on_left_click = photo_page_real_on_left_click;
	PAGE_CLASS (klass)->on_left_released = photo_page_real_on_left_released;
	PAGE_CLASS (klass)->on_right_click = photo_page_real_on_right_click;
	PAGE_CLASS (klass)->on_motion = photo_page_real_on_motion;
	PAGE_CLASS (klass)->on_configure = photo_page_real_on_configure;
	GTK_WIDGET_CLASS (klass)->key_press_event = photo_page_real_key_press_event;
	SINGLE_PHOTO_PAGE_CLASS (klass)->new_drawable = photo_page_real_new_drawable;
	SINGLE_PHOTO_PAGE_CLASS (klass)->updated_pixbuf = photo_page_real_updated_pixbuf;
	SINGLE_PHOTO_PAGE_CLASS (klass)->paint = photo_page_real_paint;
	PAGE_CLASS (klass)->on_ctrl_pressed = photo_page_real_on_ctrl_pressed;
	PAGE_CLASS (klass)->on_ctrl_released = photo_page_real_on_ctrl_released;
	G_OBJECT_CLASS (klass)->finalize = photo_page_finalize;
}


static void photo_page_instance_init (PhotoPage * self) {
	self->priv = PHOTO_PAGE_GET_PRIVATE (self);
	self->priv->container = NULL;
	self->priv->controller = NULL;
	self->priv->photo = NULL;
	self->priv->thumbnail = NULL;
	self->priv->toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
	self->priv->rotate_button = NULL;
	self->priv->crop_button = NULL;
	self->priv->redeye_button = NULL;
	self->priv->adjust_button = NULL;
	self->priv->prev_button = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_GO_BACK));
	self->priv->next_button = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD));
	self->priv->current_tool = NULL;
	self->priv->drag_file = NULL;
}


static void photo_page_finalize (GObject* obj) {
	PhotoPage * self;
	self = PHOTO_PAGE (obj);
	(self->priv->container == NULL) ? NULL : (self->priv->container = (g_object_unref (self->priv->container), NULL));
	(self->priv->context_menu == NULL) ? NULL : (self->priv->context_menu = (g_object_unref (self->priv->context_menu), NULL));
	(self->priv->controller == NULL) ? NULL : (self->priv->controller = (g_object_unref (self->priv->controller), NULL));
	(self->priv->photo == NULL) ? NULL : (self->priv->photo = (g_object_unref (self->priv->photo), NULL));
	(self->priv->thumbnail == NULL) ? NULL : (self->priv->thumbnail = (g_object_unref (self->priv->thumbnail), NULL));
	(self->priv->toolbar == NULL) ? NULL : (self->priv->toolbar = (g_object_unref (self->priv->toolbar), NULL));
	(self->priv->rotate_button == NULL) ? NULL : (self->priv->rotate_button = (g_object_unref (self->priv->rotate_button), NULL));
	(self->priv->crop_button == NULL) ? NULL : (self->priv->crop_button = (g_object_unref (self->priv->crop_button), NULL));
	(self->priv->redeye_button == NULL) ? NULL : (self->priv->redeye_button = (g_object_unref (self->priv->redeye_button), NULL));
	(self->priv->adjust_button == NULL) ? NULL : (self->priv->adjust_button = (g_object_unref (self->priv->adjust_button), NULL));
	(self->priv->prev_button == NULL) ? NULL : (self->priv->prev_button = (g_object_unref (self->priv->prev_button), NULL));
	(self->priv->next_button == NULL) ? NULL : (self->priv->next_button = (g_object_unref (self->priv->next_button), NULL));
	(self->priv->current_tool == NULL) ? NULL : (self->priv->current_tool = (editing_tool_unref (self->priv->current_tool), NULL));
	(self->priv->drag_file == NULL) ? NULL : (self->priv->drag_file = (g_object_unref (self->priv->drag_file), NULL));
	G_OBJECT_CLASS (photo_page_parent_class)->finalize (obj);
}


GType photo_page_get_type (void) {
	static GType photo_page_type_id = 0;
	if (photo_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PhotoPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PhotoPage), 0, (GInstanceInitFunc) photo_page_instance_init, NULL };
		photo_page_type_id = g_type_register_static (TYPE_SINGLE_PHOTO_PAGE, "PhotoPage", &g_define_type_info, 0);
	}
	return photo_page_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




