/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <gobject/gvaluecollector.h>


#define TYPE_SIDEBAR_MARKER (sidebar_marker_get_type ())
#define SIDEBAR_MARKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIDEBAR_MARKER, SidebarMarker))
#define SIDEBAR_MARKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIDEBAR_MARKER, SidebarMarkerClass))
#define IS_SIDEBAR_MARKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIDEBAR_MARKER))
#define IS_SIDEBAR_MARKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIDEBAR_MARKER))
#define SIDEBAR_MARKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIDEBAR_MARKER, SidebarMarkerClass))

typedef struct _SidebarMarker SidebarMarker;
typedef struct _SidebarMarkerClass SidebarMarkerClass;
typedef struct _SidebarMarkerPrivate SidebarMarkerPrivate;
typedef struct _ParamSpecSidebarMarker ParamSpecSidebarMarker;

#define TYPE_SIDEBAR (sidebar_get_type ())
#define SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIDEBAR, Sidebar))
#define SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIDEBAR, SidebarClass))
#define IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIDEBAR))
#define IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIDEBAR))
#define SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIDEBAR, SidebarClass))

typedef struct _Sidebar Sidebar;
typedef struct _SidebarClass SidebarClass;
typedef struct _SidebarPrivate SidebarPrivate;

#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

struct _SidebarMarker {
	GTypeInstance parent_instance;
	volatile int ref_count;
	SidebarMarkerPrivate * priv;
};

struct _SidebarMarkerClass {
	GTypeClass parent_class;
	void (*finalize) (SidebarMarker *self);
};

struct _SidebarMarkerPrivate {
	GtkTreeRowReference* row;
};

struct _ParamSpecSidebarMarker {
	GParamSpec parent_instance;
};

struct _Sidebar {
	GtkTreeView parent_instance;
	SidebarPrivate * priv;
};

struct _SidebarClass {
	GtkTreeViewClass parent_class;
};

struct _SidebarPrivate {
	GtkTreeStore* store;
	GeeHashSet* pages;
};


static gpointer sidebar_marker_parent_class = NULL;
static gpointer sidebar_parent_class = NULL;

gpointer sidebar_marker_ref (gpointer instance);
void sidebar_marker_unref (gpointer instance);
GParamSpec* param_spec_sidebar_marker (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_sidebar_marker (GValue* value, gpointer v_object);
gpointer value_get_sidebar_marker (const GValue* value);
GType sidebar_marker_get_type (void);
#define SIDEBAR_MARKER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SIDEBAR_MARKER, SidebarMarkerPrivate))
enum  {
	SIDEBAR_MARKER_DUMMY_PROPERTY
};
SidebarMarker* sidebar_marker_new (GtkTreeStore* store, const GtkTreePath* path);
SidebarMarker* sidebar_marker_construct (GType object_type, GtkTreeStore* store, const GtkTreePath* path);
GtkTreeRowReference* sidebar_marker_get_row (SidebarMarker* self);
GtkTreePath* sidebar_marker_get_path (SidebarMarker* self);
static void sidebar_marker_finalize (SidebarMarker* obj);
GType sidebar_get_type (void);
GType page_get_type (void);
#define SIDEBAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SIDEBAR, SidebarPrivate))
enum  {
	SIDEBAR_DUMMY_PROPERTY
};
static gboolean sidebar_on_selection (Sidebar* self, GtkTreeSelection* selection, GtkTreeModel* model, const GtkTreePath* path, gboolean path_currently_selected);
static gboolean _sidebar_on_selection_gtk_tree_selection_func (GtkTreeSelection* selection, GtkTreeModel* model, const GtkTreePath* path, gboolean path_currently_selected, gpointer self);
Sidebar* sidebar_new (void);
Sidebar* sidebar_construct (GType object_type);
SidebarMarker* page_get_marker (Page* self);
void sidebar_place_cursor (Sidebar* self, Page* page);
void sidebar_expand_branch (Sidebar* self, SidebarMarker* marker);
char* page_get_page_name (Page* self);
void page_set_marker (Page* self, SidebarMarker* marker);
static SidebarMarker* sidebar_attach_page (Sidebar* self, Page* page, GtkTreeIter* iter);
void page_clear_marker (Page* self);
static void sidebar_detach_page (Sidebar* self, Page* page);
static Page* sidebar_locate_page (Sidebar* self, const GtkTreePath* path);
SidebarMarker* sidebar_add_parent (Sidebar* self, Page* parent);
SidebarMarker* sidebar_add_child (Sidebar* self, SidebarMarker* parent, Page* child);
SidebarMarker* sidebar_add_grouping_row (Sidebar* self, const char* name);
SidebarMarker* sidebar_insert_grouping_after (Sidebar* self, SidebarMarker* after, const char* name);
SidebarMarker* sidebar_insert_sibling_before (Sidebar* self, SidebarMarker* before, Page* page);
SidebarMarker* sidebar_insert_sibling_after (Sidebar* self, SidebarMarker* after, Page* page);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
gint64 comparator_compare (Comparator* self, gconstpointer a, gconstpointer b);
SidebarMarker* sidebar_insert_child_sorted (Sidebar* self, SidebarMarker* parent, Page* child, Comparator* comparator);
void sidebar_prune_branch (Sidebar* self, SidebarMarker* parent);
void sidebar_prune_branch_children (Sidebar* self, SidebarMarker* parent);
void sidebar_remove_page (Sidebar* self, Page* page);
static void sidebar_finalize (GObject* obj);



#line 10 "Sidebar.vala"
SidebarMarker* sidebar_marker_construct (GType object_type, GtkTreeStore* store, const GtkTreePath* path) {
#line 154 "Sidebar.c"
	SidebarMarker* self;
	GtkTreeRowReference* _tmp0_;
#line 10 "Sidebar.vala"
	g_return_val_if_fail (GTK_IS_TREE_STORE (store), NULL);
#line 10 "Sidebar.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 161 "Sidebar.c"
	self = (SidebarMarker*) g_type_create_instance (object_type);
	_tmp0_ = NULL;
#line 11 "Sidebar.vala"
	self->priv->row = (_tmp0_ = gtk_tree_row_reference_new (GTK_TREE_MODEL (store), path), (self->priv->row == NULL) ? NULL : (self->priv->row = (gtk_tree_row_reference_free (self->priv->row), NULL)), _tmp0_);
#line 166 "Sidebar.c"
	return self;
}


#line 10 "Sidebar.vala"
SidebarMarker* sidebar_marker_new (GtkTreeStore* store, const GtkTreePath* path) {
#line 10 "Sidebar.vala"
	return sidebar_marker_construct (TYPE_SIDEBAR_MARKER, store, path);
#line 175 "Sidebar.c"
}


#line 14 "Sidebar.vala"
GtkTreeRowReference* sidebar_marker_get_row (SidebarMarker* self) {
#line 181 "Sidebar.c"
	GtkTreeRowReference* result;
	const GtkTreeRowReference* _tmp0_;
#line 14 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (self), NULL);
#line 15 "Sidebar.vala"
	_tmp0_ = NULL;
#line 188 "Sidebar.c"
	result = (_tmp0_ = self->priv->row, (_tmp0_ == NULL) ? NULL : gtk_tree_row_reference_copy (_tmp0_));
	return result;
}


/* The path should not be stored, as it may change as the store is altered.  See Gtk docs for
 more on the relationship of Gtk.TreePath and Gtk.TreeRowReference*/
#line 20 "Sidebar.vala"
GtkTreePath* sidebar_marker_get_path (SidebarMarker* self) {
#line 198 "Sidebar.c"
	GtkTreePath* result;
#line 20 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (self), NULL);
#line 202 "Sidebar.c"
	result = gtk_tree_row_reference_get_path (self->priv->row);
	return result;
}


static void value_sidebar_marker_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_sidebar_marker_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		sidebar_marker_unref (value->data[0].v_pointer);
	}
}


static void value_sidebar_marker_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = sidebar_marker_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_sidebar_marker_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_sidebar_marker_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		SidebarMarker* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = sidebar_marker_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_sidebar_marker_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	SidebarMarker** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = sidebar_marker_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_sidebar_marker (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecSidebarMarker* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_SIDEBAR_MARKER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_sidebar_marker (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_SIDEBAR_MARKER), NULL);
	return value->data[0].v_pointer;
}


void value_set_sidebar_marker (GValue* value, gpointer v_object) {
	SidebarMarker* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_SIDEBAR_MARKER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_SIDEBAR_MARKER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		sidebar_marker_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		sidebar_marker_unref (old);
	}
}


static void sidebar_marker_class_init (SidebarMarkerClass * klass) {
	sidebar_marker_parent_class = g_type_class_peek_parent (klass);
	SIDEBAR_MARKER_CLASS (klass)->finalize = sidebar_marker_finalize;
	g_type_class_add_private (klass, sizeof (SidebarMarkerPrivate));
}


static void sidebar_marker_instance_init (SidebarMarker * self) {
	self->priv = SIDEBAR_MARKER_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void sidebar_marker_finalize (SidebarMarker* obj) {
	SidebarMarker * self;
	self = SIDEBAR_MARKER (obj);
	(self->priv->row == NULL) ? NULL : (self->priv->row = (gtk_tree_row_reference_free (self->priv->row), NULL));
}


GType sidebar_marker_get_type (void) {
	static GType sidebar_marker_type_id = 0;
	if (sidebar_marker_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_sidebar_marker_init, value_sidebar_marker_free_value, value_sidebar_marker_copy_value, value_sidebar_marker_peek_pointer, "p", value_sidebar_marker_collect_value, "p", value_sidebar_marker_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (SidebarMarkerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) sidebar_marker_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SidebarMarker), 0, (GInstanceInitFunc) sidebar_marker_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		sidebar_marker_type_id = g_type_register_fundamental (g_type_fundamental_next (), "SidebarMarker", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return sidebar_marker_type_id;
}


gpointer sidebar_marker_ref (gpointer instance) {
	SidebarMarker* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void sidebar_marker_unref (gpointer instance) {
	SidebarMarker* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		SIDEBAR_MARKER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 234 "Sidebar.vala"
static gboolean _sidebar_on_selection_gtk_tree_selection_func (GtkTreeSelection* selection, GtkTreeModel* model, const GtkTreePath* path, gboolean path_currently_selected, gpointer self) {
#line 353 "Sidebar.c"
	return sidebar_on_selection (self, selection, model, path, path_currently_selected);
}


#line 29 "Sidebar.vala"
Sidebar* sidebar_construct (GType object_type) {
#line 360 "Sidebar.c"
	Sidebar * self;
	GtkCellRendererText* text;
	GtkTreeViewColumn* text_column;
	GtkTreeSelection* _tmp0_;
	GtkTreeSelection* selection;
	self = g_object_newv (object_type, 0, NULL);
#line 30 "Sidebar.vala"
	gtk_tree_view_set_model (GTK_TREE_VIEW (self), GTK_TREE_MODEL (self->priv->store));
#line 369 "Sidebar.c"
	text = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ());
	text_column = g_object_ref_sink (gtk_tree_view_column_new ());
#line 34 "Sidebar.vala"
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (text_column), GTK_CELL_RENDERER (text), TRUE);
#line 35 "Sidebar.vala"
	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (text_column), GTK_CELL_RENDERER (text), "text", 0);
#line 36 "Sidebar.vala"
	gtk_tree_view_append_column (GTK_TREE_VIEW (self), text_column);
#line 38 "Sidebar.vala"
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (self), FALSE);
#line 39 "Sidebar.vala"
	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (self), FALSE);
#line 40 "Sidebar.vala"
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (self), FALSE);
#line 41 "Sidebar.vala"
	gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (self), TRUE);
#line 42 "Sidebar.vala"
	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (self), FALSE);
#line 43 "Sidebar.vala"
	gtk_tree_view_set_enable_tree_lines (GTK_TREE_VIEW (self), FALSE);
#line 44 "Sidebar.vala"
	gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (self), GTK_TREE_VIEW_GRID_LINES_NONE);
#line 45 "Sidebar.vala"
	gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (self), 0);
#line 47 "Sidebar.vala"
	_tmp0_ = NULL;
#line 396 "Sidebar.c"
	selection = (_tmp0_ = gtk_tree_view_get_selection (GTK_TREE_VIEW (self)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 48 "Sidebar.vala"
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
#line 49 "Sidebar.vala"
	gtk_tree_selection_set_select_function (selection, _sidebar_on_selection_gtk_tree_selection_func, self, NULL);
#line 402 "Sidebar.c"
	(text == NULL) ? NULL : (text = (g_object_unref (text), NULL));
	(text_column == NULL) ? NULL : (text_column = (g_object_unref (text_column), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	return self;
}


#line 29 "Sidebar.vala"
Sidebar* sidebar_new (void) {
#line 29 "Sidebar.vala"
	return sidebar_construct (TYPE_SIDEBAR);
#line 414 "Sidebar.c"
}


#line 52 "Sidebar.vala"
void sidebar_place_cursor (Sidebar* self, Page* page) {
#line 420 "Sidebar.c"
	SidebarMarker* _tmp0_;
	gboolean _tmp1_;
#line 52 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR (self));
#line 52 "Sidebar.vala"
	g_return_if_fail (IS_PAGE (page));
#line 102 "Page.vala"
	_tmp0_ = NULL;
#line 53 "Sidebar.vala"
	if ((_tmp1_ = (_tmp0_ = page_get_marker (page)) != NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL)), _tmp1_)) {
#line 431 "Sidebar.c"
		GtkTreePath* _tmp3_;
		SidebarMarker* _tmp2_;
#line 20 "Sidebar.vala"
		_tmp3_ = NULL;
#line 102 "Page.vala"
		_tmp2_ = NULL;
#line 54 "Sidebar.vala"
		gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (self)), _tmp3_ = sidebar_marker_get_path (_tmp2_ = page_get_marker (page)));
#line 440 "Sidebar.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (gtk_tree_path_free (_tmp3_), NULL));
		(_tmp2_ == NULL) ? NULL : (_tmp2_ = (sidebar_marker_unref (_tmp2_), NULL));
	}
}


#line 57 "Sidebar.vala"
void sidebar_expand_branch (Sidebar* self, SidebarMarker* marker) {
#line 449 "Sidebar.c"
	GtkTreePath* _tmp0_;
#line 57 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR (self));
#line 57 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (marker));
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 58 "Sidebar.vala"
	gtk_tree_view_expand_row (GTK_TREE_VIEW (self), _tmp0_ = sidebar_marker_get_path (marker), TRUE);
#line 459 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
}


#line 61 "Sidebar.vala"
static SidebarMarker* sidebar_attach_page (Sidebar* self, Page* page, GtkTreeIter* iter) {
#line 466 "Sidebar.c"
	SidebarMarker* result;
	char* _tmp0_;
	GtkTreePath* _tmp1_;
	SidebarMarker* _tmp2_;
	SidebarMarker* marker;
#line 61 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 61 "Sidebar.vala"
	g_return_val_if_fail (IS_PAGE (page), NULL);
#line 476 "Sidebar.c"
	/* set up the columns*/
#line 58 "Page.vala"
	_tmp0_ = NULL;
#line 63 "Sidebar.vala"
	gtk_tree_store_set (self->priv->store, &(*iter), 0, _tmp0_ = page_get_page_name (page), -1);
#line 482 "Sidebar.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	/* create a marker for this page*/
#line 5703 "gtk+-2.0.vapi"
	_tmp1_ = NULL;
#line 66 "Sidebar.vala"
	_tmp2_ = NULL;
#line 489 "Sidebar.c"
	marker = (_tmp2_ = sidebar_marker_new (self->priv->store, _tmp1_ = gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->store), &(*iter))), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (gtk_tree_path_free (_tmp1_), NULL)), _tmp2_);
	/* stash the marker in the page itself*/
#line 69 "Sidebar.vala"
	page_set_marker (page, marker);
#line 494 "Sidebar.c"
	/* add to the master table*/
#line 72 "Sidebar.vala"
	g_assert (!gee_abstract_collection_contains (GEE_ABSTRACT_COLLECTION (self->priv->pages), page));
#line 73 "Sidebar.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->pages), page);
#line 500 "Sidebar.c"
	result = marker;
	return result;
}


#line 78 "Sidebar.vala"
static void sidebar_detach_page (Sidebar* self, Page* page) {
#line 78 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR (self));
#line 78 "Sidebar.vala"
	g_return_if_fail (IS_PAGE (page));
#line 512 "Sidebar.c"
	/* destroy the marker linking the page to the sidebar*/
#line 80 "Sidebar.vala"
	page_clear_marker (page);
#line 516 "Sidebar.c"
	/* remove from master table*/
#line 83 "Sidebar.vala"
	gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->pages), page);
#line 520 "Sidebar.c"
}


#line 86 "Sidebar.vala"
static Page* sidebar_locate_page (Sidebar* self, const GtkTreePath* path) {
#line 526 "Sidebar.c"
	Page* result;
#line 86 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 86 "Sidebar.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 532 "Sidebar.c"
	{
		GeeIterator* _page_it;
		_page_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->pages));
#line 87 "Sidebar.vala"
		while (TRUE) {
#line 538 "Sidebar.c"
			Page* page;
			GtkTreePath* _tmp1_;
			SidebarMarker* _tmp0_;
			gboolean _tmp2_;
#line 87 "Sidebar.vala"
			if (!gee_iterator_next (_page_it)) {
#line 87 "Sidebar.vala"
				break;
#line 547 "Sidebar.c"
			}
			page = (Page*) gee_iterator_get (_page_it);
#line 20 "Sidebar.vala"
			_tmp1_ = NULL;
#line 102 "Page.vala"
			_tmp0_ = NULL;
#line 88 "Sidebar.vala"
			if ((_tmp2_ = gtk_tree_path_compare (_tmp1_ = sidebar_marker_get_path (_tmp0_ = page_get_marker (page)), path) == 0, (_tmp1_ == NULL) ? NULL : (_tmp1_ = (gtk_tree_path_free (_tmp1_), NULL)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL)), _tmp2_)) {
#line 556 "Sidebar.c"
				result = page;
				(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
				return result;
			}
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		}
		(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
	}
	result = NULL;
	return result;
}


/* adds a top-level parent to the sidebar*/
#line 96 "Sidebar.vala"
SidebarMarker* sidebar_add_parent (Sidebar* self, Page* parent) {
#line 573 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter parent_iter = {0};
#line 96 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 96 "Sidebar.vala"
	g_return_val_if_fail (IS_PAGE (parent), NULL);
#line 580 "Sidebar.c"
	/* add a row, get its iter*/
#line 99 "Sidebar.vala"
	gtk_tree_store_append (self->priv->store, &parent_iter, NULL);
#line 584 "Sidebar.c"
	result = sidebar_attach_page (self, parent, &parent_iter);
	return result;
}


#line 104 "Sidebar.vala"
SidebarMarker* sidebar_add_child (Sidebar* self, SidebarMarker* parent, Page* child) {
#line 592 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter parent_iter = {0};
	GtkTreePath* _tmp0_;
	GtkTreeIter child_iter = {0};
#line 104 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 104 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (parent), NULL);
#line 104 "Sidebar.vala"
	g_return_val_if_fail (IS_PAGE (child), NULL);
#line 603 "Sidebar.c"
	/* find the parent*/
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 107 "Sidebar.vala"
	gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &parent_iter, _tmp0_ = sidebar_marker_get_path (parent));
#line 609 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
	/* append the child to its branch, get child's iter*/
#line 111 "Sidebar.vala"
	gtk_tree_store_append (self->priv->store, &child_iter, &parent_iter);
#line 614 "Sidebar.c"
	result = sidebar_attach_page (self, child, &child_iter);
	return result;
}


#line 116 "Sidebar.vala"
SidebarMarker* sidebar_add_grouping_row (Sidebar* self, const char* name) {
#line 622 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter grouping_iter = {0};
	GtkTreePath* _tmp0_;
	SidebarMarker* _tmp1_;
#line 116 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 116 "Sidebar.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 631 "Sidebar.c"
	/* add the row, get its iter*/
#line 119 "Sidebar.vala"
	gtk_tree_store_append (self->priv->store, &grouping_iter, NULL);
#line 635 "Sidebar.c"
	/* set the columns*/
#line 122 "Sidebar.vala"
	gtk_tree_store_set (self->priv->store, &grouping_iter, 0, name, -1);
#line 639 "Sidebar.c"
	/* return the row reference, which is the only way to refer to the grouping now*/
#line 5703 "gtk+-2.0.vapi"
	_tmp0_ = NULL;
#line 125 "Sidebar.vala"
	_tmp1_ = NULL;
#line 645 "Sidebar.c"
	result = (_tmp1_ = sidebar_marker_new (self->priv->store, _tmp0_ = gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->store), &grouping_iter)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL)), _tmp1_);
	return result;
}


#line 128 "Sidebar.vala"
SidebarMarker* sidebar_insert_grouping_after (Sidebar* self, SidebarMarker* after, const char* name) {
#line 653 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter after_iter = {0};
	GtkTreePath* _tmp0_;
	GtkTreeIter grouping_iter = {0};
	GtkTreePath* _tmp1_;
	SidebarMarker* _tmp2_;
#line 128 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 128 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (after), NULL);
#line 128 "Sidebar.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 666 "Sidebar.c"
	/* find sibling*/
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 131 "Sidebar.vala"
	gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &after_iter, _tmp0_ = sidebar_marker_get_path (after));
#line 672 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
	/* insert before sibling*/
#line 135 "Sidebar.vala"
	gtk_tree_store_insert_after (self->priv->store, &grouping_iter, NULL, &after_iter);
#line 677 "Sidebar.c"
	/* set the columns*/
#line 138 "Sidebar.vala"
	gtk_tree_store_set (self->priv->store, &grouping_iter, 0, name, -1);
#line 681 "Sidebar.c"
	/* return row reference, which is only way to refer to grouping*/
#line 5703 "gtk+-2.0.vapi"
	_tmp1_ = NULL;
#line 141 "Sidebar.vala"
	_tmp2_ = NULL;
#line 687 "Sidebar.c"
	result = (_tmp2_ = sidebar_marker_new (self->priv->store, _tmp1_ = gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->store), &grouping_iter)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (gtk_tree_path_free (_tmp1_), NULL)), _tmp2_);
	return result;
}


#line 144 "Sidebar.vala"
SidebarMarker* sidebar_insert_sibling_before (Sidebar* self, SidebarMarker* before, Page* page) {
#line 695 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter before_iter = {0};
	GtkTreePath* _tmp0_;
	GtkTreeIter page_iter = {0};
#line 144 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 144 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (before), NULL);
#line 144 "Sidebar.vala"
	g_return_val_if_fail (IS_PAGE (page), NULL);
#line 706 "Sidebar.c"
	/* find sibling in tree*/
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 147 "Sidebar.vala"
	gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &before_iter, _tmp0_ = sidebar_marker_get_path (before));
#line 712 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
	/* insert page before sibling, get its new iter*/
#line 151 "Sidebar.vala"
	gtk_tree_store_insert_before (self->priv->store, &page_iter, NULL, &before_iter);
#line 717 "Sidebar.c"
	result = sidebar_attach_page (self, page, &page_iter);
	return result;
}


#line 156 "Sidebar.vala"
SidebarMarker* sidebar_insert_sibling_after (Sidebar* self, SidebarMarker* after, Page* page) {
#line 725 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter after_iter = {0};
	GtkTreePath* _tmp0_;
	GtkTreeIter page_iter = {0};
#line 156 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 156 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (after), NULL);
#line 156 "Sidebar.vala"
	g_return_val_if_fail (IS_PAGE (page), NULL);
#line 736 "Sidebar.c"
	/* find sibling in tree*/
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 159 "Sidebar.vala"
	gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &after_iter, _tmp0_ = sidebar_marker_get_path (after));
#line 742 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
	/* insert page after sibling, get its new iter*/
#line 163 "Sidebar.vala"
	gtk_tree_store_insert_after (self->priv->store, &page_iter, NULL, &after_iter);
#line 747 "Sidebar.c"
	result = sidebar_attach_page (self, page, &page_iter);
	return result;
}


#line 168 "Sidebar.vala"
SidebarMarker* sidebar_insert_child_sorted (Sidebar* self, SidebarMarker* parent, Page* child, Comparator* comparator) {
#line 755 "Sidebar.c"
	SidebarMarker* result;
	GtkTreeIter parent_iter = {0};
	GtkTreePath* _tmp0_;
	gboolean _tmp1_;
	gboolean found;
	GtkTreeIter child_iter = {0};
#line 168 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), NULL);
#line 168 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR_MARKER (parent), NULL);
#line 168 "Sidebar.vala"
	g_return_val_if_fail (IS_PAGE (child), NULL);
#line 168 "Sidebar.vala"
	g_return_val_if_fail (IS_COMPARATOR (comparator), NULL);
#line 770 "Sidebar.c"
	/* find parent in sidebar using its row reference*/
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 774 "Sidebar.c"
	found = (_tmp1_ = gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &parent_iter, _tmp0_ = sidebar_marker_get_path (parent)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL)), _tmp1_);
#line 172 "Sidebar.vala"
	g_assert (found);
#line 778 "Sidebar.c"
	/* iterate the children of the parent page, inserting child in sorted order*/
#line 176 "Sidebar.vala"
	found = gtk_tree_model_iter_children (GTK_TREE_MODEL (self->priv->store), &child_iter, &parent_iter);
#line 177 "Sidebar.vala"
	while (TRUE) {
#line 784 "Sidebar.c"
		GtkTreePath* _tmp2_;
		Page* _tmp3_;
		Page* page;
#line 177 "Sidebar.vala"
		if (!found) {
#line 177 "Sidebar.vala"
			break;
#line 792 "Sidebar.c"
		}
#line 5703 "gtk+-2.0.vapi"
		_tmp2_ = NULL;
#line 178 "Sidebar.vala"
		_tmp3_ = NULL;
#line 798 "Sidebar.c"
		page = (_tmp3_ = sidebar_locate_page (self, _tmp2_ = gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->store), &child_iter)), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (gtk_tree_path_free (_tmp2_), NULL)), _tmp3_);
#line 179 "Sidebar.vala"
		if (page != NULL) {
#line 802 "Sidebar.c"
			/* look to insert before the current page*/
#line 181 "Sidebar.vala"
			if (comparator_compare (comparator, child, page) < 0) {
#line 806 "Sidebar.c"
				SidebarMarker* _tmp4_;
				SidebarMarker* _tmp5_;
#line 102 "Page.vala"
				_tmp4_ = NULL;
#line 182 "Sidebar.vala"
				_tmp5_ = NULL;
#line 813 "Sidebar.c"
				result = (_tmp5_ = sidebar_insert_sibling_before (self, _tmp4_ = page_get_marker (page), child), (_tmp4_ == NULL) ? NULL : (_tmp4_ = (sidebar_marker_unref (_tmp4_), NULL)), _tmp5_);
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				return result;
			}
		}
#line 185 "Sidebar.vala"
		found = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->store), &child_iter);
#line 821 "Sidebar.c"
		(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
	}
	/* not found or branch is empty, add to end*/
	result = sidebar_add_child (self, parent, child);
	return result;
}


#line 192 "Sidebar.vala"
void sidebar_prune_branch (Sidebar* self, SidebarMarker* parent) {
#line 832 "Sidebar.c"
	GtkTreeIter parent_iter = {0};
	GtkTreePath* _tmp0_;
#line 192 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR (self));
#line 192 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (parent));
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 194 "Sidebar.vala"
	gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &parent_iter, _tmp0_ = sidebar_marker_get_path (parent));
#line 843 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
#line 196 "Sidebar.vala"
	gtk_tree_store_remove (self->priv->store, &parent_iter);
#line 847 "Sidebar.c"
}


#line 199 "Sidebar.vala"
void sidebar_prune_branch_children (Sidebar* self, SidebarMarker* parent) {
#line 853 "Sidebar.c"
	GtkTreeIter parent_iter = {0};
	GtkTreePath* _tmp0_;
	GtkTreeIter child_iter = {0};
	gboolean valid;
#line 199 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR (self));
#line 199 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (parent));
#line 862 "Sidebar.c"
	/* get parent's iter*/
#line 20 "Sidebar.vala"
	_tmp0_ = NULL;
#line 202 "Sidebar.vala"
	gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &parent_iter, _tmp0_ = sidebar_marker_get_path (parent));
#line 868 "Sidebar.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_path_free (_tmp0_), NULL));
	/* iterate its children, removing them all and clearing their markers*/
	valid = gtk_tree_model_iter_children (GTK_TREE_MODEL (self->priv->store), &child_iter, &parent_iter);
#line 207 "Sidebar.vala"
	while (TRUE) {
#line 874 "Sidebar.c"
		GtkTreePath* _tmp1_;
		Page* _tmp2_;
		Page* page;
#line 207 "Sidebar.vala"
		if (!valid) {
#line 207 "Sidebar.vala"
			break;
#line 882 "Sidebar.c"
		}
#line 5703 "gtk+-2.0.vapi"
		_tmp1_ = NULL;
#line 208 "Sidebar.vala"
		_tmp2_ = NULL;
#line 888 "Sidebar.c"
		page = (_tmp2_ = sidebar_locate_page (self, _tmp1_ = gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->store), &child_iter)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (gtk_tree_path_free (_tmp1_), NULL)), _tmp2_);
#line 209 "Sidebar.vala"
		if (page != NULL) {
#line 210 "Sidebar.vala"
			sidebar_detach_page (self, page);
#line 894 "Sidebar.c"
		}
#line 212 "Sidebar.vala"
		valid = gtk_tree_store_remove (self->priv->store, &child_iter);
#line 898 "Sidebar.c"
		(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
	}
}


#line 216 "Sidebar.vala"
void sidebar_remove_page (Sidebar* self, Page* page) {
#line 906 "Sidebar.c"
	SidebarMarker* _tmp0_;
	gboolean _tmp1_;
#line 216 "Sidebar.vala"
	g_return_if_fail (IS_SIDEBAR (self));
#line 216 "Sidebar.vala"
	g_return_if_fail (IS_PAGE (page));
#line 913 "Sidebar.c"
	/* do nothing if page is not in sidebar*/
#line 102 "Page.vala"
	_tmp0_ = NULL;
#line 218 "Sidebar.vala"
	if ((_tmp1_ = (_tmp0_ = page_get_marker (page)) != NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL)), _tmp1_)) {
#line 919 "Sidebar.c"
		SidebarMarker* _tmp2_;
		GtkTreePath* _tmp3_;
		GtkTreePath* path;
		/* Path can be null if the row is blown away ... need to detach, but it's obviously
		 not located in the model any longer*/
#line 102 "Page.vala"
		_tmp2_ = NULL;
#line 221 "Sidebar.vala"
		_tmp3_ = NULL;
#line 929 "Sidebar.c"
		path = (_tmp3_ = sidebar_marker_get_path (_tmp2_ = page_get_marker (page)), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (sidebar_marker_unref (_tmp2_), NULL)), _tmp3_);
#line 222 "Sidebar.vala"
		if (path != NULL) {
#line 933 "Sidebar.c"
			GtkTreeIter iter = {0};
			/* locate in sidebar; again, if not there, don't complain*/
#line 225 "Sidebar.vala"
			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->store), &iter, path)) {
#line 226 "Sidebar.vala"
				gtk_tree_store_remove (self->priv->store, &iter);
#line 940 "Sidebar.c"
			}
		}
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	}
	/* in every case, attempt to detach from this object*/
#line 231 "Sidebar.vala"
	sidebar_detach_page (self, page);
#line 948 "Sidebar.c"
}


#line 234 "Sidebar.vala"
static gboolean sidebar_on_selection (Sidebar* self, GtkTreeSelection* selection, GtkTreeModel* model, const GtkTreePath* path, gboolean path_currently_selected) {
#line 954 "Sidebar.c"
	gboolean result;
	Page* _tmp0_;
	gboolean _tmp1_;
#line 234 "Sidebar.vala"
	g_return_val_if_fail (IS_SIDEBAR (self), FALSE);
#line 234 "Sidebar.vala"
	g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), FALSE);
#line 234 "Sidebar.vala"
	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), FALSE);
#line 234 "Sidebar.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 966 "Sidebar.c"
	/* only allow selection if a page is associated with the path; if not, it's a grouping row,
	 which is unselectable*/
#line 86 "Sidebar.vala"
	_tmp0_ = NULL;
#line 971 "Sidebar.c"
	result = (_tmp1_ = (_tmp0_ = sidebar_locate_page (self, path)) != NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	return result;
}


static void sidebar_class_init (SidebarClass * klass) {
	sidebar_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SidebarPrivate));
	G_OBJECT_CLASS (klass)->finalize = sidebar_finalize;
}


static void sidebar_instance_init (Sidebar * self) {
	self->priv = SIDEBAR_GET_PRIVATE (self);
	self->priv->store = gtk_tree_store_new (1, G_TYPE_STRING, NULL);
	self->priv->pages = gee_hash_set_new (TYPE_PAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_hash, g_direct_equal);
}


static void sidebar_finalize (GObject* obj) {
	Sidebar * self;
	self = SIDEBAR (obj);
	(self->priv->store == NULL) ? NULL : (self->priv->store = (g_object_unref (self->priv->store), NULL));
	(self->priv->pages == NULL) ? NULL : (self->priv->pages = (g_object_unref (self->priv->pages), NULL));
	G_OBJECT_CLASS (sidebar_parent_class)->finalize (obj);
}


GType sidebar_get_type (void) {
	static GType sidebar_type_id = 0;
	if (sidebar_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SidebarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) sidebar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Sidebar), 0, (GInstanceInitFunc) sidebar_instance_init, NULL };
		sidebar_type_id = g_type_register_static (GTK_TYPE_TREE_VIEW, "Sidebar", &g_define_type_info, 0);
	}
	return sidebar_type_id;
}




