/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>


#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;
typedef struct _ComparatorPrivate ComparatorPrivate;
typedef struct _ParamSpecComparator ParamSpecComparator;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;
typedef struct _SortedListPrivate SortedListPrivate;

struct _Comparator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ComparatorPrivate * priv;
};

struct _ComparatorClass {
	GTypeClass parent_class;
	void (*finalize) (Comparator *self);
	gint64 (*compare) (Comparator* self, gconstpointer a, gconstpointer b);
};

struct _ComparatorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _ParamSpecComparator {
	GParamSpec parent_instance;
};

struct _SortedList {
	GObject parent_instance;
	SortedListPrivate * priv;
};

struct _SortedListClass {
	GObjectClass parent_class;
};

struct _SortedListPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeList* list;
	Comparator* cmp;
};


static gpointer comparator_parent_class = NULL;
static gpointer sorted_list_parent_class = NULL;
static GeeIterableIface* sorted_list_gee_iterable_parent_iface = NULL;

gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
#define COMPARATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_COMPARATOR, ComparatorPrivate))
enum  {
	COMPARATOR_DUMMY_PROPERTY
};
gint64 comparator_compare (Comparator* self, gconstpointer a, gconstpointer b);
static gint64 comparator_real_compare (Comparator* self, gconstpointer a, gconstpointer b);
static void comparator_finalize (Comparator* obj);
GType sorted_list_get_type (void);
#define SORTED_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SORTED_LIST, SortedListPrivate))
enum  {
	SORTED_LIST_DUMMY_PROPERTY,
	SORTED_LIST_ELEMENT_TYPE,
	SORTED_LIST_SIZE,
	SORTED_LIST_G_TYPE,
	SORTED_LIST_G_DUP_FUNC,
	SORTED_LIST_G_DESTROY_FUNC
};
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp);
GType sorted_list_get_element_type (SortedList* self);
static GeeIterator* sorted_list_real_iterator (GeeIterable* base);
static gint sorted_list_get_sorted_insert_pos (SortedList* self, gconstpointer item);
gboolean sorted_list_add (SortedList* self, gconstpointer item);
void sorted_list_clear (SortedList* self);
gboolean sorted_list_contains (SortedList* self, gconstpointer item);
gboolean sorted_list_remove (SortedList* self, gconstpointer item);
gpointer sorted_list_get (SortedList* self, gint index);
void sorted_list_set (SortedList* self, gint index, gconstpointer item);
gint sorted_list_index_of (SortedList* self, gconstpointer item);
void sorted_list_insert (SortedList* self, gint index, gconstpointer item);
void sorted_list_remove_at (SortedList* self, gint index);
void sorted_list_resort (SortedList* self, Comparator* new_cmp);
gint sorted_list_get_size (SortedList* self);
static void sorted_list_finalize (GObject* obj);
static void sorted_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void sorted_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 8 "SortedList.vala"
static gint64 comparator_real_compare (Comparator* self, gconstpointer a, gconstpointer b) {
#line 128 "SortedList.c"
	g_return_val_if_fail (IS_COMPARATOR (self), 0LL);
	g_critical ("Type `%s' does not implement abstract method `comparator_compare'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return 0LL;
}


#line 8 "SortedList.vala"
gint64 comparator_compare (Comparator* self, gconstpointer a, gconstpointer b) {
#line 8 "SortedList.vala"
	return COMPARATOR_GET_CLASS (self)->compare (self, a, b);
#line 139 "SortedList.c"
}


static void value_comparator_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_comparator_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		comparator_unref (value->data[0].v_pointer);
	}
}


static void value_comparator_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = comparator_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_comparator_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_comparator_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		Comparator* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = comparator_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_comparator_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	Comparator** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = comparator_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecComparator* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_COMPARATOR), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_comparator (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_COMPARATOR), NULL);
	return value->data[0].v_pointer;
}


void value_set_comparator (GValue* value, gpointer v_object) {
	Comparator* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_COMPARATOR));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_COMPARATOR));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		comparator_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		comparator_unref (old);
	}
}


static void comparator_class_init (ComparatorClass * klass) {
	comparator_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->finalize = comparator_finalize;
	g_type_class_add_private (klass, sizeof (ComparatorPrivate));
	COMPARATOR_CLASS (klass)->compare = comparator_real_compare;
}


static void comparator_instance_init (Comparator * self) {
	self->priv = COMPARATOR_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void comparator_finalize (Comparator* obj) {
	Comparator * self;
	self = COMPARATOR (obj);
}


GType comparator_get_type (void) {
	static GType comparator_type_id = 0;
	if (comparator_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_comparator_init, value_comparator_free_value, value_comparator_copy_value, value_comparator_peek_pointer, "p", value_comparator_collect_value, "p", value_comparator_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ComparatorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) comparator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Comparator), 0, (GInstanceInitFunc) comparator_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		comparator_type_id = g_type_register_fundamental (g_type_fundamental_next (), "Comparator", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return comparator_type_id;
}


gpointer comparator_ref (gpointer instance) {
	Comparator* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void comparator_unref (gpointer instance) {
	Comparator* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		COMPARATOR_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 15 "SortedList.vala"
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp) {
#line 288 "SortedList.c"
	GParameter * __params;
	GParameter * __params_it;
	SortedList * self;
	GeeList* _tmp0_;
	Comparator* _tmp2_;
	Comparator* _tmp1_;
#line 15 "SortedList.vala"
	g_return_val_if_fail ((cmp == NULL) || IS_COMPARATOR (cmp), NULL);
#line 297 "SortedList.c"
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
	__params_it->name = "g-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, g_type);
	__params_it++;
	__params_it->name = "g-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g_dup_func);
	__params_it++;
	__params_it->name = "g-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	_tmp0_ = NULL;
#line 16 "SortedList.vala"
	self->priv->list = (_tmp0_ = GEE_LIST (gee_array_list_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, g_direct_equal)), (self->priv->list == NULL) ? NULL : (self->priv->list = (g_object_unref (self->priv->list), NULL)), _tmp0_);
#line 316 "SortedList.c"
	_tmp2_ = NULL;
#line 17 "SortedList.vala"
	_tmp1_ = NULL;
#line 17 "SortedList.vala"
	self->priv->cmp = (_tmp2_ = (_tmp1_ = cmp, (_tmp1_ == NULL) ? NULL : comparator_ref (_tmp1_)), (self->priv->cmp == NULL) ? NULL : (self->priv->cmp = (comparator_unref (self->priv->cmp), NULL)), _tmp2_);
#line 322 "SortedList.c"
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 15 "SortedList.vala"
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator* cmp) {
#line 15 "SortedList.vala"
	return sorted_list_construct (TYPE_SORTED_LIST, g_type, g_dup_func, g_destroy_func, cmp);
#line 336 "SortedList.c"
}


/* for libgee <= 0.1.6*/
#line 21 "SortedList.vala"
GType sorted_list_get_element_type (SortedList* self) {
#line 343 "SortedList.c"
	GType result;
#line 21 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), 0UL);
#line 347 "SortedList.c"
	result = self->priv->g_type;
	return result;
}


#line 30 "SortedList.vala"
static GeeIterator* sorted_list_real_iterator (GeeIterable* base) {
#line 355 "SortedList.c"
	SortedList * self;
	GeeIterator* result;
	self = SORTED_LIST (base);
	result = gee_iterable_iterator (GEE_ITERABLE (self->priv->list));
	return result;
}


#line 34 "SortedList.vala"
gboolean sorted_list_add (SortedList* self, gconstpointer item) {
#line 366 "SortedList.c"
	gboolean result;
#line 34 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), FALSE);
#line 35 "SortedList.vala"
	if (self->priv->cmp == NULL) {
#line 36 "SortedList.vala"
		gee_collection_add (GEE_COLLECTION (self->priv->list), item);
#line 374 "SortedList.c"
	} else {
#line 38 "SortedList.vala"
		gee_list_insert (self->priv->list, sorted_list_get_sorted_insert_pos (self, item), item);
#line 378 "SortedList.c"
	}
	result = TRUE;
	return result;
}


#line 43 "SortedList.vala"
void sorted_list_clear (SortedList* self) {
#line 43 "SortedList.vala"
	g_return_if_fail (IS_SORTED_LIST (self));
#line 44 "SortedList.vala"
	gee_collection_clear (GEE_COLLECTION (self->priv->list));
#line 391 "SortedList.c"
}


#line 47 "SortedList.vala"
gboolean sorted_list_contains (SortedList* self, gconstpointer item) {
#line 397 "SortedList.c"
	gboolean result;
#line 47 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), FALSE);
#line 401 "SortedList.c"
	result = gee_collection_contains (GEE_COLLECTION (self->priv->list), item);
	return result;
}


#line 51 "SortedList.vala"
gboolean sorted_list_remove (SortedList* self, gconstpointer item) {
#line 409 "SortedList.c"
	gboolean result;
#line 51 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), FALSE);
#line 413 "SortedList.c"
	result = gee_collection_remove (GEE_COLLECTION (self->priv->list), item);
	return result;
}


#line 59 "SortedList.vala"
gpointer sorted_list_get (SortedList* self, gint index) {
#line 421 "SortedList.c"
	gpointer result;
#line 59 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), NULL);
#line 425 "SortedList.c"
	result = gee_list_get (self->priv->list, index);
	return result;
}


#line 63 "SortedList.vala"
void sorted_list_set (SortedList* self, gint index, gconstpointer item) {
#line 63 "SortedList.vala"
	g_return_if_fail (IS_SORTED_LIST (self));
#line 64 "SortedList.vala"
	gee_list_set (self->priv->list, index, item);
#line 437 "SortedList.c"
}


#line 67 "SortedList.vala"
gint sorted_list_index_of (SortedList* self, gconstpointer item) {
#line 443 "SortedList.c"
	gint result;
#line 67 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), 0);
#line 447 "SortedList.c"
	result = gee_list_index_of (self->priv->list, item);
	return result;
}


#line 71 "SortedList.vala"
void sorted_list_insert (SortedList* self, gint index, gconstpointer item) {
#line 71 "SortedList.vala"
	g_return_if_fail (IS_SORTED_LIST (self));
#line 72 "SortedList.vala"
	gee_list_insert (self->priv->list, index, item);
#line 459 "SortedList.c"
}


#line 75 "SortedList.vala"
void sorted_list_remove_at (SortedList* self, gint index) {
#line 75 "SortedList.vala"
	g_return_if_fail (IS_SORTED_LIST (self));
#line 76 "SortedList.vala"
	gee_list_remove_at (self->priv->list, index);
#line 469 "SortedList.c"
}


#line 79 "SortedList.vala"
void sorted_list_resort (SortedList* self, Comparator* new_cmp) {
#line 475 "SortedList.c"
	Comparator* _tmp1_;
	Comparator* _tmp0_;
	GeeList* _tmp2_;
	GeeList* old_list;
	GeeList* _tmp3_;
#line 79 "SortedList.vala"
	g_return_if_fail (IS_SORTED_LIST (self));
#line 79 "SortedList.vala"
	g_return_if_fail (IS_COMPARATOR (new_cmp));
#line 485 "SortedList.c"
	_tmp1_ = NULL;
#line 80 "SortedList.vala"
	_tmp0_ = NULL;
#line 80 "SortedList.vala"
	self->priv->cmp = (_tmp1_ = (_tmp0_ = new_cmp, (_tmp0_ == NULL) ? NULL : comparator_ref (_tmp0_)), (self->priv->cmp == NULL) ? NULL : (self->priv->cmp = (comparator_unref (self->priv->cmp), NULL)), _tmp1_);
#line 82 "SortedList.vala"
	_tmp2_ = NULL;
#line 493 "SortedList.c"
	old_list = (_tmp2_ = self->priv->list, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
	_tmp3_ = NULL;
#line 83 "SortedList.vala"
	self->priv->list = (_tmp3_ = GEE_LIST (gee_array_list_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, g_direct_equal)), (self->priv->list == NULL) ? NULL : (self->priv->list = (g_object_unref (self->priv->list), NULL)), _tmp3_);
#line 498 "SortedList.c"
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (old_list));
#line 85 "SortedList.vala"
		while (TRUE) {
#line 504 "SortedList.c"
			gpointer item;
#line 85 "SortedList.vala"
			if (!gee_iterator_next (_item_it)) {
#line 85 "SortedList.vala"
				break;
#line 510 "SortedList.c"
			}
			item = gee_iterator_get (_item_it);
#line 86 "SortedList.vala"
			gee_list_insert (self->priv->list, sorted_list_get_sorted_insert_pos (self, item), item);
#line 515 "SortedList.c"
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	(old_list == NULL) ? NULL : (old_list = (g_object_unref (old_list), NULL));
}


#line 89 "SortedList.vala"
static gint sorted_list_get_sorted_insert_pos (SortedList* self, gconstpointer item) {
#line 526 "SortedList.c"
	gint result;
	gint low;
	gint high;
#line 89 "SortedList.vala"
	g_return_val_if_fail (IS_SORTED_LIST (self), 0);
#line 532 "SortedList.c"
	low = 0;
	high = gee_collection_get_size (GEE_COLLECTION (self->priv->list));
	{
		gboolean _tmp0_;
		_tmp0_ = TRUE;
#line 92 "SortedList.vala"
		while (TRUE) {
#line 540 "SortedList.c"
			gint mid;
			gpointer _tmp1_;
			gint64 _tmp2_;
			gint64 _result_;
#line 92 "SortedList.vala"
			if (!_tmp0_) {
#line 547 "SortedList.c"
			}
#line 92 "SortedList.vala"
			_tmp0_ = FALSE;
#line 93 "SortedList.vala"
			if (low == high) {
#line 553 "SortedList.c"
				result = low;
				return result;
			}
			mid = low + ((high - low) / 2);
			_tmp1_ = NULL;
			_result_ = (_tmp2_ = comparator_compare (self->priv->cmp, item, _tmp1_ = gee_list_get (self->priv->list, mid)), ((_tmp1_ == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp1_ = (self->priv->g_destroy_func (_tmp1_), NULL)), _tmp2_);
#line 99 "SortedList.vala"
			if (_result_ < 0) {
#line 100 "SortedList.vala"
				high = mid;
#line 564 "SortedList.c"
			} else {
#line 101 "SortedList.vala"
				if (_result_ > 0) {
#line 102 "SortedList.vala"
					low = mid + 1;
#line 570 "SortedList.c"
				} else {
					result = mid;
					return result;
				}
			}
		}
	}
}


static GType sorted_list_real_get_element_type (GeeIterable* base) {
	GType result;
	SortedList* self;
	self = SORTED_LIST (base);
	result = self->priv->g_type;
	return result;
}


gint sorted_list_get_size (SortedList* self) {
	gint result;
	g_return_val_if_fail (IS_SORTED_LIST (self), 0);
	result = gee_collection_get_size (GEE_COLLECTION (self->priv->list));
	return result;
}


static void sorted_list_class_init (SortedListClass * klass) {
	sorted_list_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SortedListPrivate));
	G_OBJECT_CLASS (klass)->get_property = sorted_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = sorted_list_set_property;
	G_OBJECT_CLASS (klass)->finalize = sorted_list_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), SORTED_LIST_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SORTED_LIST_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SORTED_LIST_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_override_property (G_OBJECT_CLASS (klass), SORTED_LIST_ELEMENT_TYPE, "element-type");
	g_object_class_install_property (G_OBJECT_CLASS (klass), SORTED_LIST_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void sorted_list_gee_iterable_interface_init (GeeIterableIface * iface) {
	sorted_list_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->iterator = sorted_list_real_iterator;
	iface->get_element_type = sorted_list_real_get_element_type;
}


static void sorted_list_instance_init (SortedList * self) {
	self->priv = SORTED_LIST_GET_PRIVATE (self);
}


static void sorted_list_finalize (GObject* obj) {
	SortedList * self;
	self = SORTED_LIST (obj);
	(self->priv->list == NULL) ? NULL : (self->priv->list = (g_object_unref (self->priv->list), NULL));
	(self->priv->cmp == NULL) ? NULL : (self->priv->cmp = (comparator_unref (self->priv->cmp), NULL));
	G_OBJECT_CLASS (sorted_list_parent_class)->finalize (obj);
}


GType sorted_list_get_type (void) {
	static GType sorted_list_type_id = 0;
	if (sorted_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SortedListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) sorted_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SortedList), 0, (GInstanceInitFunc) sorted_list_instance_init, NULL };
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) sorted_list_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		sorted_list_type_id = g_type_register_static (G_TYPE_OBJECT, "SortedList", &g_define_type_info, 0);
		g_type_add_interface_static (sorted_list_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
	}
	return sorted_list_type_id;
}


static void sorted_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SortedList * self;
	gpointer boxed;
	self = SORTED_LIST (object);
	switch (property_id) {
		case SORTED_LIST_ELEMENT_TYPE:
		g_value_set_gtype (value, gee_iterable_get_element_type (GEE_ITERABLE (self)));
		break;
		case SORTED_LIST_SIZE:
		g_value_set_int (value, sorted_list_get_size (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void sorted_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SortedList * self;
	self = SORTED_LIST (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
		case SORTED_LIST_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case SORTED_LIST_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case SORTED_LIST_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
	}
}




