/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>


#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;
typedef struct _LayoutItemPrivate LayoutItemPrivate;

#define TYPE_THUMBNAIL (thumbnail_get_type ())
#define THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL, Thumbnail))
#define THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL, ThumbnailClass))
#define IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL))
#define IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL))
#define THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL, ThumbnailClass))

typedef struct _Thumbnail Thumbnail;
typedef struct _ThumbnailClass ThumbnailClass;
typedef struct _ThumbnailPrivate ThumbnailPrivate;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

struct _LayoutItem {
	GtkAlignment parent_instance;
	LayoutItemPrivate * priv;
};

struct _LayoutItemClass {
	GtkAlignmentClass parent_class;
	GtkWidget* (*get_control_panel) (LayoutItem* self);
	void (*exposed) (LayoutItem* self);
	void (*unexposed) (LayoutItem* self);
	void (*select) (LayoutItem* self);
	void (*unselect) (LayoutItem* self);
};

struct _Thumbnail {
	LayoutItem parent_instance;
	ThumbnailPrivate * priv;
};

struct _ThumbnailClass {
	LayoutItemClass parent_class;
};

struct _Dimensions {
	gint width;
	gint height;
};

struct _ThumbnailPrivate {
	Photo* photo;
	gint scale;
	Dimensions dim;
	gboolean thumb_exposed;
	GdkInterpType interp;
};


static gpointer thumbnail_parent_class = NULL;

GType layout_item_get_type (void);
GType thumbnail_get_type (void);
GType photo_get_type (void);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
#define THUMBNAIL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_THUMBNAIL, ThumbnailPrivate))
enum  {
	THUMBNAIL_DUMMY_PROPERTY
};
#define THUMBNAIL_LOW_QUALITY_INTERP GDK_INTERP_NEAREST
#define THUMBNAIL_MIN_SCALE 64
#define THUMBNAIL_MAX_SCALE 360
#define THUMBNAIL_DEFAULT_SCALE 128
#define THUMBNAIL_HIGH_QUALITY_INTERP GDK_INTERP_BILINEAR
void layout_item_set_title (LayoutItem* self, const char* text);
GFile* photo_get_file (Photo* self);
Dimensions photo_get_dimensions (Photo* self);
Dimensions dimensions_get_scaled (Dimensions *self, gint scale);
void layout_item_set_image_size (LayoutItem* self, gint width, gint height);
static void thumbnail_on_thumbnail_altered (Thumbnail* self, Photo* p);
static void _thumbnail_on_thumbnail_altered_photo_thumbnail_altered (Photo* _sender, gpointer self);
LayoutItem* layout_item_new (void);
LayoutItem* layout_item_construct (GType object_type);
Thumbnail* thumbnail_new (Photo* photo, gint scale);
Thumbnail* thumbnail_construct (GType object_type, Photo* photo, gint scale);
Photo* thumbnail_get_photo (Thumbnail* self);
gboolean photo_equals (Photo* self, Photo* photo);
GdkPixbuf* photo_get_thumbnail (Photo* self, gint scale);
void layout_item_set_image (LayoutItem* self, GdkPixbuf* pixbuf);
void thumbnail_resize (Thumbnail* self, gint new_scale);
void thumbnail_paint_high_quality (Thumbnail* self);
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp);
static void thumbnail_real_exposed (LayoutItem* base);
void layout_item_clear_image (LayoutItem* self);
static void thumbnail_real_unexposed (LayoutItem* base);
gboolean thumbnail_is_exposed (Thumbnail* self);
static void thumbnail_finalize (GObject* obj);



#line 43 "Thumbnail.vala"
static void _thumbnail_on_thumbnail_altered_photo_thumbnail_altered (Photo* _sender, gpointer self) {
#line 134 "Thumbnail.c"
	thumbnail_on_thumbnail_altered (self, _sender);
}


#line 22 "Thumbnail.vala"
Thumbnail* thumbnail_construct (GType object_type, Photo* photo, gint scale) {
#line 141 "Thumbnail.c"
	Thumbnail * self;
	Photo* _tmp1_;
	Photo* _tmp0_;
	char* _tmp3_;
	GFile* _tmp2_;
	Dimensions _tmp4_ = {0};
#line 22 "Thumbnail.vala"
	g_return_val_if_fail (IS_PHOTO (photo), NULL);
#line 150 "Thumbnail.c"
	self = (Thumbnail*) layout_item_construct (object_type);
	_tmp1_ = NULL;
#line 23 "Thumbnail.vala"
	_tmp0_ = NULL;
#line 23 "Thumbnail.vala"
	self->priv->photo = (_tmp1_ = (_tmp0_ = photo, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->photo == NULL) ? NULL : (self->priv->photo = (g_object_unref (self->priv->photo), NULL)), _tmp1_);
#line 24 "Thumbnail.vala"
	self->priv->scale = scale;
#line 947 "gio-2.0.vapi"
	_tmp3_ = NULL;
#line 167 "Photo.vala"
	_tmp2_ = NULL;
#line 26 "Thumbnail.vala"
	layout_item_set_title (LAYOUT_ITEM (self), _tmp3_ = g_file_get_basename (_tmp2_ = photo_get_file (photo)));
#line 165 "Thumbnail.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 28 "Thumbnail.vala"
	self->priv->dim = dimensions_get_scaled ((_tmp4_ = photo_get_dimensions (photo), &_tmp4_), scale);
#line 34 "Thumbnail.vala"
	layout_item_set_image_size (LAYOUT_ITEM (self), self->priv->dim.width, self->priv->dim.height);
#line 36 "Thumbnail.vala"
	g_signal_connect_object (photo, "thumbnail-altered", (GCallback) _thumbnail_on_thumbnail_altered_photo_thumbnail_altered, self, 0);
#line 174 "Thumbnail.c"
	return self;
}


#line 22 "Thumbnail.vala"
Thumbnail* thumbnail_new (Photo* photo, gint scale) {
#line 22 "Thumbnail.vala"
	return thumbnail_construct (TYPE_THUMBNAIL, photo, scale);
#line 183 "Thumbnail.c"
}


#line 39 "Thumbnail.vala"
Photo* thumbnail_get_photo (Thumbnail* self) {
#line 189 "Thumbnail.c"
	Photo* result;
	Photo* _tmp0_;
#line 39 "Thumbnail.vala"
	g_return_val_if_fail (IS_THUMBNAIL (self), NULL);
#line 40 "Thumbnail.vala"
	_tmp0_ = NULL;
#line 196 "Thumbnail.c"
	result = (_tmp0_ = self->priv->photo, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 43 "Thumbnail.vala"
static void thumbnail_on_thumbnail_altered (Thumbnail* self, Photo* p) {
#line 204 "Thumbnail.c"
	Dimensions _tmp0_ = {0};
#line 43 "Thumbnail.vala"
	g_return_if_fail (IS_THUMBNAIL (self));
#line 43 "Thumbnail.vala"
	g_return_if_fail (IS_PHOTO (p));
#line 44 "Thumbnail.vala"
	g_assert (photo_equals (self->priv->photo, p));
#line 46 "Thumbnail.vala"
	self->priv->dim = dimensions_get_scaled ((_tmp0_ = photo_get_dimensions (self->priv->photo), &_tmp0_), self->priv->scale);
#line 214 "Thumbnail.c"
	/* only fetch and scale if exposed*/
#line 49 "Thumbnail.vala"
	if (self->priv->thumb_exposed) {
#line 218 "Thumbnail.c"
		GdkPixbuf* pixbuf;
		GdkPixbuf* _tmp1_;
		pixbuf = photo_get_thumbnail (self->priv->photo, self->priv->scale);
		_tmp1_ = NULL;
#line 51 "Thumbnail.vala"
		pixbuf = (_tmp1_ = gdk_pixbuf_scale_simple (pixbuf, self->priv->dim.width, self->priv->dim.height, THUMBNAIL_LOW_QUALITY_INTERP), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 52 "Thumbnail.vala"
		self->priv->interp = THUMBNAIL_LOW_QUALITY_INTERP;
#line 54 "Thumbnail.vala"
		layout_item_set_image (LAYOUT_ITEM (self), pixbuf);
#line 229 "Thumbnail.c"
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	}
#line 57 "Thumbnail.vala"
	layout_item_set_image_size (LAYOUT_ITEM (self), self->priv->dim.width, self->priv->dim.height);
#line 234 "Thumbnail.c"
}


#line 60 "Thumbnail.vala"
void thumbnail_resize (Thumbnail* self, gint new_scale) {
#line 60 "Thumbnail.vala"
	g_return_if_fail (IS_THUMBNAIL (self));
#line 61 "Thumbnail.vala"
	g_assert (new_scale >= THUMBNAIL_MIN_SCALE);
#line 62 "Thumbnail.vala"
	g_assert (new_scale <= THUMBNAIL_MAX_SCALE);
#line 64 "Thumbnail.vala"
	if (self->priv->scale == new_scale) {
#line 248 "Thumbnail.c"
		return;
	}
#line 67 "Thumbnail.vala"
	self->priv->scale = new_scale;
#line 253 "Thumbnail.c"
	/* piggy-back on signal handler*/
#line 70 "Thumbnail.vala"
	thumbnail_on_thumbnail_altered (self, self->priv->photo);
#line 257 "Thumbnail.c"
}


#line 73 "Thumbnail.vala"
void thumbnail_paint_high_quality (Thumbnail* self) {
#line 263 "Thumbnail.c"
	GdkPixbuf* pixbuf;
	gboolean _tmp0_;
#line 73 "Thumbnail.vala"
	g_return_if_fail (IS_THUMBNAIL (self));
#line 74 "Thumbnail.vala"
	if (!self->priv->thumb_exposed) {
#line 270 "Thumbnail.c"
		return;
	}
#line 77 "Thumbnail.vala"
	if (self->priv->interp == THUMBNAIL_HIGH_QUALITY_INTERP) {
#line 275 "Thumbnail.c"
		return;
	}
	pixbuf = photo_get_thumbnail (self->priv->photo, self->priv->scale);
	_tmp0_ = FALSE;
#line 86 "Thumbnail.vala"
	if (gdk_pixbuf_get_width (pixbuf) != self->priv->dim.width) {
#line 86 "Thumbnail.vala"
		_tmp0_ = TRUE;
#line 284 "Thumbnail.c"
	} else {
#line 86 "Thumbnail.vala"
		_tmp0_ = gdk_pixbuf_get_height (pixbuf) != self->priv->dim.height;
#line 288 "Thumbnail.c"
	}
	/* only change pixbufs if indeed the image is scaled ... although
	 scale_simple() will probably just return the pixbuf if it sees the stupid case, Gtk.Image
	 does not see the case, and will fire off resized events when the new image (which is not 
	 really new) is added*/
#line 86 "Thumbnail.vala"
	if (_tmp0_) {
#line 296 "Thumbnail.c"
		GdkPixbuf* _tmp1_;
		_tmp1_ = NULL;
#line 87 "Thumbnail.vala"
		pixbuf = (_tmp1_ = gdk_pixbuf_scale_simple (pixbuf, self->priv->dim.width, self->priv->dim.height, THUMBNAIL_HIGH_QUALITY_INTERP), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 88 "Thumbnail.vala"
		layout_item_set_image (LAYOUT_ITEM (self), pixbuf);
#line 303 "Thumbnail.c"
	}
#line 91 "Thumbnail.vala"
	self->priv->interp = THUMBNAIL_HIGH_QUALITY_INTERP;
#line 307 "Thumbnail.c"
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


#line 94 "Thumbnail.vala"
static void thumbnail_real_exposed (LayoutItem* base) {
#line 314 "Thumbnail.c"
	Thumbnail * self;
	GdkPixbuf* pixbuf;
	GdkPixbuf* _tmp0_;
	self = THUMBNAIL (base);
#line 95 "Thumbnail.vala"
	if (self->priv->thumb_exposed) {
#line 321 "Thumbnail.c"
		return;
	}
	pixbuf = photo_get_thumbnail (self->priv->photo, self->priv->scale);
	_tmp0_ = NULL;
#line 99 "Thumbnail.vala"
	pixbuf = (_tmp0_ = scale_pixbuf (pixbuf, self->priv->scale, THUMBNAIL_LOW_QUALITY_INTERP), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp0_);
#line 100 "Thumbnail.vala"
	self->priv->interp = THUMBNAIL_LOW_QUALITY_INTERP;
#line 102 "Thumbnail.vala"
	layout_item_set_image (LAYOUT_ITEM (self), pixbuf);
#line 103 "Thumbnail.vala"
	layout_item_set_image_size (LAYOUT_ITEM (self), self->priv->dim.width, self->priv->dim.height);
#line 105 "Thumbnail.vala"
	self->priv->thumb_exposed = TRUE;
#line 336 "Thumbnail.c"
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
}


#line 108 "Thumbnail.vala"
static void thumbnail_real_unexposed (LayoutItem* base) {
#line 343 "Thumbnail.c"
	Thumbnail * self;
	self = THUMBNAIL (base);
#line 109 "Thumbnail.vala"
	if (!self->priv->thumb_exposed) {
#line 348 "Thumbnail.c"
		return;
	}
#line 112 "Thumbnail.vala"
	layout_item_clear_image (LAYOUT_ITEM (self));
#line 113 "Thumbnail.vala"
	layout_item_set_image_size (LAYOUT_ITEM (self), self->priv->dim.width, self->priv->dim.height);
#line 115 "Thumbnail.vala"
	self->priv->thumb_exposed = FALSE;
#line 357 "Thumbnail.c"
}


#line 118 "Thumbnail.vala"
gboolean thumbnail_is_exposed (Thumbnail* self) {
#line 363 "Thumbnail.c"
	gboolean result;
#line 118 "Thumbnail.vala"
	g_return_val_if_fail (IS_THUMBNAIL (self), FALSE);
#line 367 "Thumbnail.c"
	result = self->priv->thumb_exposed;
	return result;
}


static void thumbnail_class_init (ThumbnailClass * klass) {
	thumbnail_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ThumbnailPrivate));
	LAYOUT_ITEM_CLASS (klass)->exposed = thumbnail_real_exposed;
	LAYOUT_ITEM_CLASS (klass)->unexposed = thumbnail_real_unexposed;
	G_OBJECT_CLASS (klass)->finalize = thumbnail_finalize;
}


static void thumbnail_instance_init (Thumbnail * self) {
	self->priv = THUMBNAIL_GET_PRIVATE (self);
	self->priv->thumb_exposed = FALSE;
	self->priv->interp = THUMBNAIL_LOW_QUALITY_INTERP;
}


static void thumbnail_finalize (GObject* obj) {
	Thumbnail * self;
	self = THUMBNAIL (obj);
	(self->priv->photo == NULL) ? NULL : (self->priv->photo = (g_object_unref (self->priv->photo), NULL));
	G_OBJECT_CLASS (thumbnail_parent_class)->finalize (obj);
}


GType thumbnail_get_type (void) {
	static GType thumbnail_type_id = 0;
	if (thumbnail_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ThumbnailClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) thumbnail_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Thumbnail), 0, (GInstanceInitFunc) thumbnail_instance_init, NULL };
		thumbnail_type_id = g_type_register_static (TYPE_LAYOUT_ITEM, "Thumbnail", &g_define_type_info, 0);
	}
	return thumbnail_type_id;
}




