/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>


#define TYPE_THUMBNAIL_CACHE (thumbnail_cache_get_type ())
#define THUMBNAIL_CACHE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL_CACHE, ThumbnailCache))
#define THUMBNAIL_CACHE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL_CACHE, ThumbnailCacheClass))
#define IS_THUMBNAIL_CACHE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL_CACHE))
#define IS_THUMBNAIL_CACHE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL_CACHE))
#define THUMBNAIL_CACHE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL_CACHE, ThumbnailCacheClass))

typedef struct _ThumbnailCache ThumbnailCache;
typedef struct _ThumbnailCacheClass ThumbnailCacheClass;
typedef struct _ThumbnailCachePrivate ThumbnailCachePrivate;

#define THUMBNAIL_CACHE_TYPE_IMAGE_DATA (thumbnail_cache_image_data_get_type ())
#define THUMBNAIL_CACHE_IMAGE_DATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), THUMBNAIL_CACHE_TYPE_IMAGE_DATA, ThumbnailCacheImageData))
#define THUMBNAIL_CACHE_IMAGE_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), THUMBNAIL_CACHE_TYPE_IMAGE_DATA, ThumbnailCacheImageDataClass))
#define THUMBNAIL_CACHE_IS_IMAGE_DATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), THUMBNAIL_CACHE_TYPE_IMAGE_DATA))
#define THUMBNAIL_CACHE_IS_IMAGE_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), THUMBNAIL_CACHE_TYPE_IMAGE_DATA))
#define THUMBNAIL_CACHE_IMAGE_DATA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), THUMBNAIL_CACHE_TYPE_IMAGE_DATA, ThumbnailCacheImageDataClass))

typedef struct _ThumbnailCacheImageData ThumbnailCacheImageData;
typedef struct _ThumbnailCacheImageDataClass ThumbnailCacheImageDataClass;

#define TYPE_DATABASE_TABLE (database_table_get_type ())
#define DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_TABLE, DatabaseTable))
#define DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_TABLE, DatabaseTableClass))
#define IS_DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_TABLE))
#define IS_DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_TABLE))
#define DATABASE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_TABLE, DatabaseTableClass))

typedef struct _DatabaseTable DatabaseTable;
typedef struct _DatabaseTableClass DatabaseTableClass;

#define TYPE_THUMBNAIL_CACHE_TABLE (thumbnail_cache_table_get_type ())
#define THUMBNAIL_CACHE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL_CACHE_TABLE, ThumbnailCacheTable))
#define THUMBNAIL_CACHE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL_CACHE_TABLE, ThumbnailCacheTableClass))
#define IS_THUMBNAIL_CACHE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL_CACHE_TABLE))
#define IS_THUMBNAIL_CACHE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL_CACHE_TABLE))
#define THUMBNAIL_CACHE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL_CACHE_TABLE, ThumbnailCacheTableClass))

typedef struct _ThumbnailCacheTable ThumbnailCacheTable;
typedef struct _ThumbnailCacheTableClass ThumbnailCacheTableClass;

#define TYPE_PHOTO_TABLE (photo_table_get_type ())
#define PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_TABLE, PhotoTable))
#define PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_TABLE, PhotoTableClass))
#define IS_PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_TABLE))
#define IS_PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_TABLE))
#define PHOTO_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_TABLE, PhotoTableClass))

typedef struct _PhotoTable PhotoTable;
typedef struct _PhotoTableClass PhotoTableClass;

#define TYPE_PHOTO_ID (photo_id_get_type ())
typedef struct _PhotoID PhotoID;
typedef struct _ThumbnailCacheImageDataPrivate ThumbnailCacheImageDataPrivate;

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;
typedef struct _ThumbnailCacheParamSpecImageData ThumbnailCacheParamSpecImageData;

struct _ThumbnailCache {
	GObject parent_instance;
	ThumbnailCachePrivate * priv;
};

struct _ThumbnailCacheClass {
	GObjectClass parent_class;
};

struct _ThumbnailCachePrivate {
	GFile* cache_dir;
	gint scale;
	gulong max_cached_bytes;
	GdkInterpType interp;
	char* jpeg_quality;
	GeeHashMap* cache_map;
	GeeArrayList* cache_lru;
	gulong cached_bytes;
	ThumbnailCacheTable* cache_table;
	PhotoTable* photo_table;
};

struct _PhotoID {
	gint64 id;
};

struct _ThumbnailCacheImageData {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ThumbnailCacheImageDataPrivate * priv;
	GdkPixbuf* pixbuf;
	gulong bytes;
};

struct _ThumbnailCacheImageDataClass {
	GTypeClass parent_class;
	void (*finalize) (ThumbnailCacheImageData *self);
};

struct _Dimensions {
	gint width;
	gint height;
};

struct _ThumbnailCacheParamSpecImageData {
	GParamSpec parent_instance;
};


static ThumbnailCache* thumbnail_cache_big;
static ThumbnailCache* thumbnail_cache_big = NULL;
static ThumbnailCache* thumbnail_cache_medium;
static ThumbnailCache* thumbnail_cache_medium = NULL;
static ThumbnailCache* thumbnail_cache_small;
static ThumbnailCache* thumbnail_cache_small = NULL;
static gpointer thumbnail_cache_image_data_parent_class = NULL;
static gpointer thumbnail_cache_parent_class = NULL;

GType thumbnail_cache_get_type (void);
static gpointer thumbnail_cache_image_data_ref (gpointer instance);
static void thumbnail_cache_image_data_unref (gpointer instance);
static GParamSpec* thumbnail_cache_param_spec_image_data (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
static void thumbnail_cache_value_set_image_data (GValue* value, gpointer v_object);
static gpointer thumbnail_cache_value_get_image_data (const GValue* value);
static GType thumbnail_cache_image_data_get_type (void);
gpointer database_table_ref (gpointer instance);
void database_table_unref (gpointer instance);
GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_database_table (GValue* value, gpointer v_object);
gpointer value_get_database_table (const GValue* value);
GType database_table_get_type (void);
GType thumbnail_cache_table_get_type (void);
GType photo_table_get_type (void);
#define THUMBNAIL_CACHE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_THUMBNAIL_CACHE, ThumbnailCachePrivate))
enum  {
	THUMBNAIL_CACHE_DUMMY_PROPERTY
};
guint int64_hash (void* p);
static guint _int64_hash_ghash_func (void* key);
gboolean int64_equal (void* a, void* b);
static gboolean _int64_equal_gequal_func (void* a, void* b);
static gint64* _int64_dup (gint64* self);
PhotoTable* photo_table_new (void);
PhotoTable* photo_table_construct (GType object_type);
#define THUMBNAIL_CACHE_DEFAULT_INTERP GDK_INTERP_HYPER
#define THUMBNAIL_CACHE_DEFAULT_JPEG_QUALITY 90
#define THUMBNAIL_CACHE_MAX_INMEMORY_DATA_SIZE (256 * 1024)
#define THUMBNAIL_CACHE_BIG_SCALE 360
#define THUMBNAIL_CACHE_MEDIUM_SCALE 128
#define THUMBNAIL_CACHE_SMALL_SCALE 64
#define THUMBNAIL_CACHE_KBYTE ((gulong) 1024)
#define THUMBNAIL_CACHE_MBYTE (1024 * THUMBNAIL_CACHE_KBYTE)
#define THUMBNAIL_CACHE_MAX_BIG_CACHED_BYTES (25 * THUMBNAIL_CACHE_MBYTE)
#define THUMBNAIL_CACHE_MAX_MEDIUM_CACHED_BYTES (15 * THUMBNAIL_CACHE_MBYTE)
#define THUMBNAIL_CACHE_MAX_SMALL_CACHED_BYTES (10 * THUMBNAIL_CACHE_MBYTE)
static ThumbnailCache* thumbnail_cache_new (gint scale, gulong max_cached_bytes, GdkInterpType interp, gint jpeg_quality);
static ThumbnailCache* thumbnail_cache_construct (GType object_type, gint scale, gulong max_cached_bytes, GdkInterpType interp, gint jpeg_quality);
void thumbnail_cache_init (void);
void thumbnail_cache_terminate (void);
GType photo_id_get_type (void);
PhotoID* photo_id_dup (const PhotoID* self);
void photo_id_free (PhotoID* self);
static void _thumbnail_cache_import (ThumbnailCache* self, const PhotoID* photo_id, GdkPixbuf* original, gboolean force);
gboolean spin_event_loop (void);
void thumbnail_cache_import (const PhotoID* photo_id, GdkPixbuf* original, gboolean force);
static void _thumbnail_cache_remove (ThumbnailCache* self, const PhotoID* photo_id);
void thumbnail_cache_remove (const PhotoID* photo_id);
static GdkPixbuf* _thumbnail_cache_fetch (ThumbnailCache* self, const PhotoID* photo_id);
GdkPixbuf* thumbnail_cache_fetch (const PhotoID* photo_id, gint scale);
static void _thumbnail_cache_replace (ThumbnailCache* self, const PhotoID* photo_id, GdkPixbuf* original);
void thumbnail_cache_replace (const PhotoID* photo_id, gint scale, GdkPixbuf* replacement);
static gboolean _thumbnail_cache_exists (ThumbnailCache* self, const PhotoID* photo_id);
gboolean thumbnail_cache_exists (const PhotoID* photo_id);
GFile* app_window_get_data_subdir (const char* name, const char* subname);
ThumbnailCacheTable* thumbnail_cache_table_new (gint scale);
ThumbnailCacheTable* thumbnail_cache_table_construct (GType object_type, gint scale);
static GFile* thumbnail_cache_get_cached_file (ThumbnailCache* self, const PhotoID* photo_id);
char* photo_table_get_name (PhotoTable* self, const PhotoID* photo_id);
gint thumbnail_cache_table_get_filesize (ThumbnailCacheTable* self, const PhotoID* photo_id);
static void thumbnail_cache_store_in_memory (ThumbnailCache* self, const PhotoID* photo_id, GdkPixbuf* thumbnail);
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp);
static gint thumbnail_cache_save_thumbnail (ThumbnailCache* self, GFile* file, GdkPixbuf* pixbuf, GError** error);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
void thumbnail_cache_table_add (ThumbnailCacheTable* self, const PhotoID* photo_id, gint filesize, const Dimensions* dim);
Dimensions dimensions_for_pixbuf (GdkPixbuf* pixbuf);
static gboolean thumbnail_cache_remove_from_memory (ThumbnailCache* self, const PhotoID* photo_id);
void thumbnail_cache_table_replace (ThumbnailCacheTable* self, const PhotoID* photo_id, gint filesize, const Dimensions* dim);
gboolean thumbnail_cache_table_remove (ThumbnailCacheTable* self, const PhotoID* photo_id);
gboolean thumbnail_cache_table_exists (ThumbnailCacheTable* self, const PhotoID* photo_id);
static ThumbnailCacheImageData* thumbnail_cache_image_data_new (GdkPixbuf* pixbuf);
static ThumbnailCacheImageData* thumbnail_cache_image_data_construct (GType object_type, GdkPixbuf* pixbuf);
enum  {
	THUMBNAIL_CACHE_IMAGE_DATA_DUMMY_PROPERTY
};
static void thumbnail_cache_image_data_finalize (ThumbnailCacheImageData* obj);
static void thumbnail_cache_finalize (GObject* obj);

static const gint THUMBNAIL_CACHE_SCALES[] = {THUMBNAIL_CACHE_BIG_SCALE, THUMBNAIL_CACHE_MEDIUM_SCALE, THUMBNAIL_CACHE_SMALL_SCALE};


#line 7 "util.vala"
static guint _int64_hash_ghash_func (void* key) {
#line 221 "ThumbnailCache.c"
	return int64_hash (key);
}


#line 18 "util.vala"
static gboolean _int64_equal_gequal_func (void* a, void* b) {
#line 228 "ThumbnailCache.c"
	return int64_equal (a, b);
}


static gint64* _int64_dup (gint64* self) {
	gint64* dup;
	dup = g_new0 (gint64, 1);
	memcpy (dup, self, sizeof (gint64));
	return dup;
}


/* Doing this because static construct {} not working nor new'ing in the above statement*/
#line 30 "ThumbnailCache.vala"
void thumbnail_cache_init (void) {
#line 244 "ThumbnailCache.c"
	ThumbnailCache* _tmp0_;
	ThumbnailCache* _tmp1_;
	ThumbnailCache* _tmp2_;
	_tmp0_ = NULL;
#line 31 "ThumbnailCache.vala"
	thumbnail_cache_big = (_tmp0_ = thumbnail_cache_new (THUMBNAIL_CACHE_BIG_SCALE, THUMBNAIL_CACHE_MAX_BIG_CACHED_BYTES, THUMBNAIL_CACHE_DEFAULT_INTERP, THUMBNAIL_CACHE_DEFAULT_JPEG_QUALITY), (thumbnail_cache_big == NULL) ? NULL : (thumbnail_cache_big = (g_object_unref (thumbnail_cache_big), NULL)), _tmp0_);
#line 251 "ThumbnailCache.c"
	_tmp1_ = NULL;
#line 32 "ThumbnailCache.vala"
	thumbnail_cache_medium = (_tmp1_ = thumbnail_cache_new (THUMBNAIL_CACHE_MEDIUM_SCALE, THUMBNAIL_CACHE_MAX_MEDIUM_CACHED_BYTES, THUMBNAIL_CACHE_DEFAULT_INTERP, THUMBNAIL_CACHE_DEFAULT_JPEG_QUALITY), (thumbnail_cache_medium == NULL) ? NULL : (thumbnail_cache_medium = (g_object_unref (thumbnail_cache_medium), NULL)), _tmp1_);
#line 255 "ThumbnailCache.c"
	_tmp2_ = NULL;
#line 33 "ThumbnailCache.vala"
	thumbnail_cache_small = (_tmp2_ = thumbnail_cache_new (THUMBNAIL_CACHE_SMALL_SCALE, THUMBNAIL_CACHE_MAX_SMALL_CACHED_BYTES, THUMBNAIL_CACHE_DEFAULT_INTERP, THUMBNAIL_CACHE_DEFAULT_JPEG_QUALITY), (thumbnail_cache_small == NULL) ? NULL : (thumbnail_cache_small = (g_object_unref (thumbnail_cache_small), NULL)), _tmp2_);
#line 259 "ThumbnailCache.c"
}


#line 36 "ThumbnailCache.vala"
void thumbnail_cache_terminate (void) {
#line 265 "ThumbnailCache.c"
}


#line 39 "ThumbnailCache.vala"
void thumbnail_cache_import (const PhotoID* photo_id, GdkPixbuf* original, gboolean force) {
#line 39 "ThumbnailCache.vala"
	g_return_if_fail (GDK_IS_PIXBUF (original));
#line 40 "ThumbnailCache.vala"
	_thumbnail_cache_import (thumbnail_cache_big, &(*photo_id), original, force);
#line 41 "ThumbnailCache.vala"
	spin_event_loop ();
#line 43 "ThumbnailCache.vala"
	_thumbnail_cache_import (thumbnail_cache_medium, &(*photo_id), original, force);
#line 44 "ThumbnailCache.vala"
	spin_event_loop ();
#line 46 "ThumbnailCache.vala"
	_thumbnail_cache_import (thumbnail_cache_small, &(*photo_id), original, force);
#line 47 "ThumbnailCache.vala"
	spin_event_loop ();
#line 285 "ThumbnailCache.c"
}


#line 50 "ThumbnailCache.vala"
void thumbnail_cache_remove (const PhotoID* photo_id) {
#line 51 "ThumbnailCache.vala"
	_thumbnail_cache_remove (thumbnail_cache_big, &(*photo_id));
#line 52 "ThumbnailCache.vala"
	spin_event_loop ();
#line 54 "ThumbnailCache.vala"
	_thumbnail_cache_remove (thumbnail_cache_medium, &(*photo_id));
#line 55 "ThumbnailCache.vala"
	spin_event_loop ();
#line 57 "ThumbnailCache.vala"
	_thumbnail_cache_remove (thumbnail_cache_small, &(*photo_id));
#line 58 "ThumbnailCache.vala"
	spin_event_loop ();
#line 303 "ThumbnailCache.c"
}


#line 61 "ThumbnailCache.vala"
GdkPixbuf* thumbnail_cache_fetch (const PhotoID* photo_id, gint scale) {
#line 309 "ThumbnailCache.c"
	GdkPixbuf* result;
#line 62 "ThumbnailCache.vala"
	if (scale > THUMBNAIL_CACHE_MEDIUM_SCALE) {
#line 313 "ThumbnailCache.c"
		result = _thumbnail_cache_fetch (thumbnail_cache_big, &(*photo_id));
		return result;
	} else {
#line 64 "ThumbnailCache.vala"
		if (scale > THUMBNAIL_CACHE_SMALL_SCALE) {
#line 319 "ThumbnailCache.c"
			result = _thumbnail_cache_fetch (thumbnail_cache_medium, &(*photo_id));
			return result;
		} else {
			result = _thumbnail_cache_fetch (thumbnail_cache_small, &(*photo_id));
			return result;
		}
	}
}


#line 71 "ThumbnailCache.vala"
void thumbnail_cache_replace (const PhotoID* photo_id, gint scale, GdkPixbuf* replacement) {
#line 332 "ThumbnailCache.c"
	ThumbnailCache* cache;
#line 71 "ThumbnailCache.vala"
	g_return_if_fail (GDK_IS_PIXBUF (replacement));
#line 336 "ThumbnailCache.c"
	cache = NULL;
#line 73 "ThumbnailCache.vala"
	switch (scale) {
#line 340 "ThumbnailCache.c"
		case THUMBNAIL_CACHE_SMALL_SCALE:
		{
			ThumbnailCache* _tmp1_;
			ThumbnailCache* _tmp0_;
			_tmp1_ = NULL;
#line 75 "ThumbnailCache.vala"
			_tmp0_ = NULL;
#line 75 "ThumbnailCache.vala"
			cache = (_tmp1_ = (_tmp0_ = thumbnail_cache_small, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL)), _tmp1_);
#line 76 "ThumbnailCache.vala"
			break;
#line 352 "ThumbnailCache.c"
		}
		case THUMBNAIL_CACHE_MEDIUM_SCALE:
		{
			ThumbnailCache* _tmp3_;
			ThumbnailCache* _tmp2_;
			_tmp3_ = NULL;
#line 79 "ThumbnailCache.vala"
			_tmp2_ = NULL;
#line 79 "ThumbnailCache.vala"
			cache = (_tmp3_ = (_tmp2_ = thumbnail_cache_medium, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL)), _tmp3_);
#line 80 "ThumbnailCache.vala"
			break;
#line 365 "ThumbnailCache.c"
		}
		case THUMBNAIL_CACHE_BIG_SCALE:
		{
			ThumbnailCache* _tmp5_;
			ThumbnailCache* _tmp4_;
			_tmp5_ = NULL;
#line 83 "ThumbnailCache.vala"
			_tmp4_ = NULL;
#line 83 "ThumbnailCache.vala"
			cache = (_tmp5_ = (_tmp4_ = thumbnail_cache_big, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL)), _tmp5_);
#line 84 "ThumbnailCache.vala"
			break;
#line 378 "ThumbnailCache.c"
		}
		default:
		{
#line 87 "ThumbnailCache.vala"
			g_error ("ThumbnailCache.vala:87: Unknown scale %d", scale);
#line 88 "ThumbnailCache.vala"
			break;
#line 386 "ThumbnailCache.c"
		}
	}
#line 91 "ThumbnailCache.vala"
	_thumbnail_cache_replace (cache, &(*photo_id), replacement);
#line 391 "ThumbnailCache.c"
	(cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL));
}


#line 94 "ThumbnailCache.vala"
gboolean thumbnail_cache_exists (const PhotoID* photo_id) {
#line 398 "ThumbnailCache.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
#line 95 "ThumbnailCache.vala"
	if (_thumbnail_cache_exists (thumbnail_cache_big, &(*photo_id))) {
#line 95 "ThumbnailCache.vala"
		_tmp1_ = _thumbnail_cache_exists (thumbnail_cache_medium, &(*photo_id));
#line 408 "ThumbnailCache.c"
	} else {
#line 95 "ThumbnailCache.vala"
		_tmp1_ = FALSE;
#line 412 "ThumbnailCache.c"
	}
#line 95 "ThumbnailCache.vala"
	if (_tmp1_) {
#line 95 "ThumbnailCache.vala"
		_tmp0_ = _thumbnail_cache_exists (thumbnail_cache_small, &(*photo_id));
#line 418 "ThumbnailCache.c"
	} else {
#line 95 "ThumbnailCache.vala"
		_tmp0_ = FALSE;
#line 422 "ThumbnailCache.c"
	}
	result = _tmp0_;
	return result;
}


#line 123 "ThumbnailCache.vala"
static ThumbnailCache* thumbnail_cache_construct (GType object_type, gint scale, gulong max_cached_bytes, GdkInterpType interp, gint jpeg_quality) {
#line 431 "ThumbnailCache.c"
	ThumbnailCache * self;
	gboolean _tmp0_;
	GFile* _tmp2_;
	char* _tmp1_;
	char* _tmp3_;
	ThumbnailCacheTable* _tmp4_;
	self = g_object_newv (object_type, 0, NULL);
#line 125 "ThumbnailCache.vala"
	g_assert (scale != 0);
#line 441 "ThumbnailCache.c"
	_tmp0_ = FALSE;
#line 126 "ThumbnailCache.vala"
	if (jpeg_quality >= 0) {
#line 126 "ThumbnailCache.vala"
		_tmp0_ = jpeg_quality <= 100;
#line 447 "ThumbnailCache.c"
	} else {
#line 126 "ThumbnailCache.vala"
		_tmp0_ = FALSE;
#line 451 "ThumbnailCache.c"
	}
#line 126 "ThumbnailCache.vala"
	g_assert (_tmp0_);
#line 455 "ThumbnailCache.c"
	_tmp2_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 128 "ThumbnailCache.vala"
	self->priv->cache_dir = (_tmp2_ = app_window_get_data_subdir ("thumbs", _tmp1_ = g_strdup_printf ("thumbs%d", scale)), (self->priv->cache_dir == NULL) ? NULL : (self->priv->cache_dir = (g_object_unref (self->priv->cache_dir), NULL)), _tmp2_);
#line 461 "ThumbnailCache.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
#line 129 "ThumbnailCache.vala"
	self->priv->scale = scale;
#line 130 "ThumbnailCache.vala"
	self->priv->max_cached_bytes = max_cached_bytes;
#line 131 "ThumbnailCache.vala"
	self->priv->interp = interp;
#line 469 "ThumbnailCache.c"
	_tmp3_ = NULL;
#line 132 "ThumbnailCache.vala"
	self->priv->jpeg_quality = (_tmp3_ = g_strdup_printf ("%d", jpeg_quality), self->priv->jpeg_quality = (g_free (self->priv->jpeg_quality), NULL), _tmp3_);
#line 473 "ThumbnailCache.c"
	_tmp4_ = NULL;
#line 133 "ThumbnailCache.vala"
	self->priv->cache_table = (_tmp4_ = thumbnail_cache_table_new (scale), (self->priv->cache_table == NULL) ? NULL : (self->priv->cache_table = (database_table_unref (self->priv->cache_table), NULL)), _tmp4_);
#line 477 "ThumbnailCache.c"
	return self;
}


#line 123 "ThumbnailCache.vala"
static ThumbnailCache* thumbnail_cache_new (gint scale, gulong max_cached_bytes, GdkInterpType interp, gint jpeg_quality) {
#line 123 "ThumbnailCache.vala"
	return thumbnail_cache_construct (TYPE_THUMBNAIL_CACHE, scale, max_cached_bytes, interp, jpeg_quality);
#line 486 "ThumbnailCache.c"
}


#line 136 "ThumbnailCache.vala"
static GdkPixbuf* _thumbnail_cache_fetch (ThumbnailCache* self, const PhotoID* photo_id) {
#line 492 "ThumbnailCache.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	ThumbnailCacheImageData* data;
	GFile* file;
	char* _tmp2_;
	char* _tmp1_;
	GdkPixbuf* pixbuf;
	gint filesize;
#line 136 "ThumbnailCache.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE (self), NULL);
#line 503 "ThumbnailCache.c"
	_inner_error_ = NULL;
	/* use JPEG in memory cache if available*/
	data = (ThumbnailCacheImageData*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->cache_map), &(*photo_id).id);
#line 139 "ThumbnailCache.vala"
	if (data != NULL) {
#line 509 "ThumbnailCache.c"
		GdkPixbuf* _tmp0_;
#line 140 "ThumbnailCache.vala"
		_tmp0_ = NULL;
#line 513 "ThumbnailCache.c"
		result = (_tmp0_ = data->pixbuf, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
		(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
		return result;
	}
	file = thumbnail_cache_get_cached_file (self, &(*photo_id));
#line 952 "gio-2.0.vapi"
	_tmp2_ = NULL;
#line 361 "DatabaseTables.vala"
	_tmp1_ = NULL;
#line 144 "ThumbnailCache.vala"
	g_debug ("ThumbnailCache.vala:144: Fetching thumbnail for %s from disk [%lld] %s", _tmp1_ = photo_table_get_name (self->priv->photo_table, &(*photo_id)), (*photo_id).id, _tmp2_ = g_file_get_path (file));
#line 525 "ThumbnailCache.c"
	_tmp2_ = (g_free (_tmp2_), NULL);
	_tmp1_ = (g_free (_tmp1_), NULL);
	pixbuf = NULL;
	{
		char* _tmp3_;
		GdkPixbuf* _tmp4_;
		GdkPixbuf* _tmp5_;
		GdkPixbuf* _tmp6_;
#line 952 "gio-2.0.vapi"
		_tmp3_ = NULL;
#line 149 "ThumbnailCache.vala"
		_tmp4_ = NULL;
#line 538 "ThumbnailCache.c"
		_tmp5_ = (_tmp4_ = gdk_pixbuf_new_from_file (_tmp3_ = g_file_get_path (file), &_inner_error_), _tmp3_ = (g_free (_tmp3_), NULL), _tmp4_);
		if (_inner_error_ != NULL) {
			goto __catch16_g_error;
			goto __finally16;
		}
		_tmp6_ = NULL;
#line 149 "ThumbnailCache.vala"
		pixbuf = (_tmp6_ = _tmp5_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp6_);
#line 547 "ThumbnailCache.c"
	}
	goto __finally16;
	__catch16_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 151 "ThumbnailCache.vala"
			g_error ("ThumbnailCache.vala:151: %s", err->message);
#line 558 "ThumbnailCache.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally16:
	if (_inner_error_ != NULL) {
		(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	filesize = thumbnail_cache_table_get_filesize (self->priv->cache_table, &(*photo_id));
#line 155 "ThumbnailCache.vala"
	if (filesize > THUMBNAIL_CACHE_MAX_INMEMORY_DATA_SIZE) {
#line 574 "ThumbnailCache.c"
		char* _tmp7_;
		/* too big to store in memory, so return the pixbuf straight from disk*/
#line 952 "gio-2.0.vapi"
		_tmp7_ = NULL;
#line 157 "ThumbnailCache.vala"
		g_debug ("ThumbnailCache.vala:157: Persistant thumbnail [%lld] %s too large to cache in memory, loading straight from disk", (*photo_id).id, _tmp7_ = g_file_get_path (file));
#line 581 "ThumbnailCache.c"
		_tmp7_ = (g_free (_tmp7_), NULL);
		result = pixbuf;
		(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		return result;
	}
	/* stash in memory for next time*/
#line 164 "ThumbnailCache.vala"
	thumbnail_cache_store_in_memory (self, &(*photo_id), pixbuf);
#line 591 "ThumbnailCache.c"
	result = pixbuf;
	(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	return result;
}


#line 169 "ThumbnailCache.vala"
static void _thumbnail_cache_import (ThumbnailCache* self, const PhotoID* photo_id, GdkPixbuf* original, gboolean force) {
#line 601 "ThumbnailCache.c"
	GError * _inner_error_;
	GFile* file;
	char* _tmp1_;
	char* _tmp0_;
	GdkPixbuf* scaled;
	gint filesize;
	Dimensions _tmp2_ = {0};
#line 169 "ThumbnailCache.vala"
	g_return_if_fail (IS_THUMBNAIL_CACHE (self));
#line 169 "ThumbnailCache.vala"
	g_return_if_fail (GDK_IS_PIXBUF (original));
#line 613 "ThumbnailCache.c"
	_inner_error_ = NULL;
	file = thumbnail_cache_get_cached_file (self, &(*photo_id));
	/* if not forcing the cache operation, check if file exists and is represented in the
	 database before continuing*/
#line 174 "ThumbnailCache.vala"
	if (!force) {
#line 175 "ThumbnailCache.vala"
		if (_thumbnail_cache_exists (self, &(*photo_id))) {
#line 622 "ThumbnailCache.c"
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			return;
		}
	} else {
		/* wipe previous from system and continue*/
#line 179 "ThumbnailCache.vala"
		_thumbnail_cache_remove (self, &(*photo_id));
#line 630 "ThumbnailCache.c"
	}
#line 952 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 361 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 182 "ThumbnailCache.vala"
	g_debug ("ThumbnailCache.vala:182: Importing thumbnail for %s to [%lld] %s", _tmp0_ = photo_table_get_name (self->priv->photo_table, &(*photo_id)), (*photo_id).id, _tmp1_ = g_file_get_path (file));
#line 638 "ThumbnailCache.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
	/* scale according to cache's parameters*/
	scaled = scale_pixbuf (original, self->priv->scale, self->priv->interp);
	/* save scaled image as JPEG*/
	filesize = -1;
	{
#line 191 "ThumbnailCache.vala"
		thumbnail_cache_save_thumbnail (self, file, scaled, &_inner_error_);
#line 648 "ThumbnailCache.c"
		if (_inner_error_ != NULL) {
			goto __catch17_g_error;
			goto __finally17;
		}
	}
	goto __finally17;
	__catch17_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 193 "ThumbnailCache.vala"
			g_error ("ThumbnailCache.vala:193: %s", err->message);
#line 663 "ThumbnailCache.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally17:
	if (_inner_error_ != NULL) {
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(scaled == NULL) ? NULL : (scaled = (g_object_unref (scaled), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* do NOT store in the in-memory cache ... if a lot of photos are being imported at
	 once, this will blow cache locality, especially when the user is viewing one portion
	 of the collection while new photos are added far off the viewport
	 store in database*/
#line 201 "ThumbnailCache.vala"
	thumbnail_cache_table_add (self->priv->cache_table, &(*photo_id), filesize, (_tmp2_ = dimensions_for_pixbuf (scaled), &_tmp2_));
#line 681 "ThumbnailCache.c"
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(scaled == NULL) ? NULL : (scaled = (g_object_unref (scaled), NULL));
}


#line 204 "ThumbnailCache.vala"
static void _thumbnail_cache_replace (ThumbnailCache* self, const PhotoID* photo_id, GdkPixbuf* original) {
#line 689 "ThumbnailCache.c"
	GError * _inner_error_;
	GFile* file;
	char* _tmp1_;
	char* _tmp0_;
	GdkPixbuf* scaled;
	gint filesize;
	Dimensions _tmp3_ = {0};
#line 204 "ThumbnailCache.vala"
	g_return_if_fail (IS_THUMBNAIL_CACHE (self));
#line 204 "ThumbnailCache.vala"
	g_return_if_fail (GDK_IS_PIXBUF (original));
#line 701 "ThumbnailCache.c"
	_inner_error_ = NULL;
	file = thumbnail_cache_get_cached_file (self, &(*photo_id));
#line 952 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 361 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 207 "ThumbnailCache.vala"
	g_debug ("ThumbnailCache.vala:207: Replacing thumbnail for %s with [%lld] %s", _tmp0_ = photo_table_get_name (self->priv->photo_table, &(*photo_id)), (*photo_id).id, _tmp1_ = g_file_get_path (file));
#line 710 "ThumbnailCache.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
	/* Remove from in-memory cache, if present*/
#line 211 "ThumbnailCache.vala"
	thumbnail_cache_remove_from_memory (self, &(*photo_id));
#line 716 "ThumbnailCache.c"
	/* scale to cache's parameters*/
	scaled = scale_pixbuf (original, self->priv->scale, self->priv->interp);
	/* save scaled image as JPEG*/
	filesize = -1;
	{
		gint _tmp2_;
		_tmp2_ = thumbnail_cache_save_thumbnail (self, file, scaled, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch18_g_error;
			goto __finally18;
		}
#line 219 "ThumbnailCache.vala"
		filesize = _tmp2_;
#line 730 "ThumbnailCache.c"
	}
	goto __finally18;
	__catch18_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 221 "ThumbnailCache.vala"
			g_error ("ThumbnailCache.vala:221: %s", err->message);
#line 741 "ThumbnailCache.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally18:
	if (_inner_error_ != NULL) {
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(scaled == NULL) ? NULL : (scaled = (g_object_unref (scaled), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* Store in in-memory cache; a _replace() probably represents a user-initiated
	 action (<cough>rotate</cough>) and the thumbnail will probably be fetched immediately.
	 This means the thumbnail will be cached in scales that aren't immediately needed, but the
	 benefit seems to outweigh the side-effects*/
#line 228 "ThumbnailCache.vala"
	thumbnail_cache_store_in_memory (self, &(*photo_id), scaled);
#line 759 "ThumbnailCache.c"
	/* store changes in database*/
#line 231 "ThumbnailCache.vala"
	thumbnail_cache_table_replace (self->priv->cache_table, &(*photo_id), filesize, (_tmp3_ = dimensions_for_pixbuf (scaled), &_tmp3_));
#line 763 "ThumbnailCache.c"
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(scaled == NULL) ? NULL : (scaled = (g_object_unref (scaled), NULL));
}


#line 234 "ThumbnailCache.vala"
static void _thumbnail_cache_remove (ThumbnailCache* self, const PhotoID* photo_id) {
#line 771 "ThumbnailCache.c"
	GError * _inner_error_;
	GFile* file;
	char* _tmp1_;
	char* _tmp0_;
#line 234 "ThumbnailCache.vala"
	g_return_if_fail (IS_THUMBNAIL_CACHE (self));
#line 778 "ThumbnailCache.c"
	_inner_error_ = NULL;
	file = thumbnail_cache_get_cached_file (self, &(*photo_id));
#line 952 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 361 "DatabaseTables.vala"
	_tmp0_ = NULL;
#line 237 "ThumbnailCache.vala"
	g_debug ("ThumbnailCache.vala:237: Removing thumbnail for %s [%lld] %s", _tmp0_ = photo_table_get_name (self->priv->photo_table, &(*photo_id)), (*photo_id).id, _tmp1_ = g_file_get_path (file));
#line 787 "ThumbnailCache.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
	/* remove from in-memory cache*/
#line 241 "ThumbnailCache.vala"
	thumbnail_cache_remove_from_memory (self, &(*photo_id));
#line 793 "ThumbnailCache.c"
	/* remove from db table*/
#line 244 "ThumbnailCache.vala"
	thumbnail_cache_table_remove (self->priv->cache_table, &(*photo_id));
#line 797 "ThumbnailCache.c"
	/* remove from disk*/
#line 247 "ThumbnailCache.vala"
	if (g_file_query_exists (file, NULL)) {
#line 801 "ThumbnailCache.c"
		{
#line 249 "ThumbnailCache.vala"
			g_file_delete (file, NULL, &_inner_error_);
#line 805 "ThumbnailCache.c"
			if (_inner_error_ != NULL) {
				goto __catch19_g_error;
				goto __finally19;
			}
		}
		goto __finally19;
		__catch19_g_error:
		{
			GError * err;
			err = _inner_error_;
			_inner_error_ = NULL;
			{
#line 251 "ThumbnailCache.vala"
				g_warning ("ThumbnailCache.vala:251: %s", err->message);
#line 820 "ThumbnailCache.c"
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally19:
		if (_inner_error_ != NULL) {
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
}


#line 256 "ThumbnailCache.vala"
static gboolean _thumbnail_cache_exists (ThumbnailCache* self, const PhotoID* photo_id) {
#line 838 "ThumbnailCache.c"
	gboolean result;
	GFile* file;
	gboolean _tmp0_;
#line 256 "ThumbnailCache.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE (self), FALSE);
#line 844 "ThumbnailCache.c"
	file = thumbnail_cache_get_cached_file (self, &(*photo_id));
	_tmp0_ = FALSE;
#line 259 "ThumbnailCache.vala"
	if (g_file_query_exists (file, NULL)) {
#line 259 "ThumbnailCache.vala"
		_tmp0_ = thumbnail_cache_table_exists (self->priv->cache_table, &(*photo_id));
#line 851 "ThumbnailCache.c"
	} else {
#line 259 "ThumbnailCache.vala"
		_tmp0_ = FALSE;
#line 855 "ThumbnailCache.c"
	}
	result = _tmp0_;
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	return result;
}


#line 262 "ThumbnailCache.vala"
static GFile* thumbnail_cache_get_cached_file (ThumbnailCache* self, const PhotoID* photo_id) {
#line 865 "ThumbnailCache.c"
	GFile* result;
	char* _tmp0_;
	GFile* _tmp1_;
#line 262 "ThumbnailCache.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE (self), NULL);
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 263 "ThumbnailCache.vala"
	_tmp1_ = NULL;
#line 875 "ThumbnailCache.c"
	result = (_tmp1_ = g_file_get_child (self->priv->cache_dir, _tmp0_ = g_strdup_printf ("thumb%016llx.jpg", (*photo_id).id)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


#line 266 "ThumbnailCache.vala"
static void thumbnail_cache_store_in_memory (ThumbnailCache* self, const PhotoID* photo_id, GdkPixbuf* thumbnail) {
#line 883 "ThumbnailCache.c"
	ThumbnailCacheImageData* data;
#line 266 "ThumbnailCache.vala"
	g_return_if_fail (IS_THUMBNAIL_CACHE (self));
#line 266 "ThumbnailCache.vala"
	g_return_if_fail (GDK_IS_PIXBUF (thumbnail));
#line 267 "ThumbnailCache.vala"
	thumbnail_cache_remove_from_memory (self, &(*photo_id));
#line 891 "ThumbnailCache.c"
	data = thumbnail_cache_image_data_new (thumbnail);
#line 270 "ThumbnailCache.vala"
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->cache_map), &(*photo_id).id, data);
#line 271 "ThumbnailCache.vala"
	gee_abstract_list_insert (GEE_ABSTRACT_LIST (self->priv->cache_lru), 0, &(*photo_id).id);
#line 273 "ThumbnailCache.vala"
	self->priv->cached_bytes = self->priv->cached_bytes + data->bytes;
#line 276 "ThumbnailCache.vala"
	while (TRUE) {
#line 901 "ThumbnailCache.c"
		gint index;
		gint64* _tmp0_;
		gint64 _tmp1_;
		gint64 id;
		ThumbnailCacheImageData* _tmp2_;
		gboolean removed;
#line 276 "ThumbnailCache.vala"
		if (!(self->priv->cached_bytes > self->priv->max_cached_bytes)) {
#line 276 "ThumbnailCache.vala"
			break;
#line 912 "ThumbnailCache.c"
		}
#line 277 "ThumbnailCache.vala"
		g_assert (gee_collection_get_size (GEE_COLLECTION (self->priv->cache_lru)) > 0);
#line 916 "ThumbnailCache.c"
		index = gee_collection_get_size (GEE_COLLECTION (self->priv->cache_lru)) - 1;
#line 118 "ThumbnailCache.vala"
		_tmp0_ = NULL;
#line 920 "ThumbnailCache.c"
		id = (_tmp1_ = *(_tmp0_ = (gint64*) gee_abstract_list_get (GEE_ABSTRACT_LIST (self->priv->cache_lru), index)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_free (_tmp0_), NULL)), _tmp1_);
#line 281 "ThumbnailCache.vala"
		gee_abstract_list_remove_at (GEE_ABSTRACT_LIST (self->priv->cache_lru), index);
#line 924 "ThumbnailCache.c"
		_tmp2_ = NULL;
#line 283 "ThumbnailCache.vala"
		data = (_tmp2_ = (ThumbnailCacheImageData*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->cache_map), &id), (data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL)), _tmp2_);
#line 284 "ThumbnailCache.vala"
		g_assert (data->bytes <= self->priv->cached_bytes);
#line 285 "ThumbnailCache.vala"
		self->priv->cached_bytes = self->priv->cached_bytes - data->bytes;
#line 287 "ThumbnailCache.vala"
		g_debug ("ThumbnailCache.vala:287: Thumbnail cache %d overflow: removed %lu bytes, now %lu", self->priv->scale, data->bytes, self->priv->cached_bytes);
#line 934 "ThumbnailCache.c"
		removed = gee_abstract_map_remove (GEE_ABSTRACT_MAP (self->priv->cache_map), &id, NULL);
#line 291 "ThumbnailCache.vala"
		g_assert (removed);
#line 938 "ThumbnailCache.c"
	}
	(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
}


#line 295 "ThumbnailCache.vala"
static gboolean thumbnail_cache_remove_from_memory (ThumbnailCache* self, const PhotoID* photo_id) {
#line 946 "ThumbnailCache.c"
	gboolean result;
	ThumbnailCacheImageData* data;
	gboolean removed;
#line 295 "ThumbnailCache.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE (self), FALSE);
#line 952 "ThumbnailCache.c"
	data = (ThumbnailCacheImageData*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->cache_map), &(*photo_id).id);
#line 297 "ThumbnailCache.vala"
	if (data == NULL) {
#line 956 "ThumbnailCache.c"
		result = FALSE;
		(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
		return result;
	}
#line 300 "ThumbnailCache.vala"
	g_assert (self->priv->cached_bytes >= data->bytes);
#line 301 "ThumbnailCache.vala"
	self->priv->cached_bytes = self->priv->cached_bytes - data->bytes;
#line 965 "ThumbnailCache.c"
	/* remove data from in-memory cache*/
	removed = gee_abstract_map_remove (GEE_ABSTRACT_MAP (self->priv->cache_map), &(*photo_id).id, NULL);
#line 305 "ThumbnailCache.vala"
	g_assert (removed);
#line 970 "ThumbnailCache.c"
	/* remove from LRU*/
#line 308 "ThumbnailCache.vala"
	removed = gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->cache_lru), &(*photo_id).id);
#line 309 "ThumbnailCache.vala"
	g_assert (removed);
#line 976 "ThumbnailCache.c"
	result = TRUE;
	(data == NULL) ? NULL : (data = (thumbnail_cache_image_data_unref (data), NULL));
	return result;
}


#line 314 "ThumbnailCache.vala"
static gint thumbnail_cache_save_thumbnail (ThumbnailCache* self, GFile* file, GdkPixbuf* pixbuf, GError** error) {
#line 985 "ThumbnailCache.c"
	gint result;
	GError * _inner_error_;
	char* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	GFileInfo* info;
	gint filesize;
#line 314 "ThumbnailCache.vala"
	g_return_val_if_fail (IS_THUMBNAIL_CACHE (self), 0);
#line 314 "ThumbnailCache.vala"
	g_return_val_if_fail (G_IS_FILE (file), 0);
#line 314 "ThumbnailCache.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), 0);
#line 999 "ThumbnailCache.c"
	_inner_error_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 1003 "ThumbnailCache.c"
	_tmp2_ = (_tmp1_ = gdk_pixbuf_save (pixbuf, _tmp0_ = g_file_get_path (file), "jpeg", &_inner_error_, "quality", self->priv->jpeg_quality, NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return 0;
	}
#line 315 "ThumbnailCache.vala"
	if (!_tmp2_) {
#line 1011 "ThumbnailCache.c"
		char* _tmp3_;
#line 952 "gio-2.0.vapi"
		_tmp3_ = NULL;
#line 316 "ThumbnailCache.vala"
		g_error ("ThumbnailCache.vala:316: Unable to save thumbnail %s", _tmp3_ = g_file_get_path (file));
#line 1017 "ThumbnailCache.c"
		_tmp3_ = (g_free (_tmp3_), NULL);
	}
	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return 0;
	}
	/* this should never be huge*/
#line 322 "ThumbnailCache.vala"
	g_assert (g_file_info_get_size (info) <= G_MAXINT);
#line 1028 "ThumbnailCache.c"
	filesize = (gint) g_file_info_get_size (info);
	result = filesize;
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	return result;
}


#line 102 "ThumbnailCache.vala"
static ThumbnailCacheImageData* thumbnail_cache_image_data_construct (GType object_type, GdkPixbuf* pixbuf) {
#line 1038 "ThumbnailCache.c"
	ThumbnailCacheImageData* self;
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
#line 102 "ThumbnailCache.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 1044 "ThumbnailCache.c"
	self = (ThumbnailCacheImageData*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 103 "ThumbnailCache.vala"
	_tmp0_ = NULL;
#line 103 "ThumbnailCache.vala"
	self->pixbuf = (_tmp1_ = (_tmp0_ = pixbuf, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->pixbuf == NULL) ? NULL : (self->pixbuf = (g_object_unref (self->pixbuf), NULL)), _tmp1_);
#line 1051 "ThumbnailCache.c"
	/* This is not entirely accurate (see Gtk doc note on pixbuf Image Data), but close enough
	 for government work*/
#line 107 "ThumbnailCache.vala"
	self->bytes = (gulong) (gdk_pixbuf_get_rowstride (pixbuf) * gdk_pixbuf_get_height (pixbuf));
#line 1056 "ThumbnailCache.c"
	return self;
}


#line 102 "ThumbnailCache.vala"
static ThumbnailCacheImageData* thumbnail_cache_image_data_new (GdkPixbuf* pixbuf) {
#line 102 "ThumbnailCache.vala"
	return thumbnail_cache_image_data_construct (THUMBNAIL_CACHE_TYPE_IMAGE_DATA, pixbuf);
#line 1065 "ThumbnailCache.c"
}


static void thumbnail_cache_value_image_data_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void thumbnail_cache_value_image_data_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		thumbnail_cache_image_data_unref (value->data[0].v_pointer);
	}
}


static void thumbnail_cache_value_image_data_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = thumbnail_cache_image_data_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer thumbnail_cache_value_image_data_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* thumbnail_cache_value_image_data_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ThumbnailCacheImageData* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = thumbnail_cache_image_data_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* thumbnail_cache_value_image_data_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ThumbnailCacheImageData** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = thumbnail_cache_image_data_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* thumbnail_cache_param_spec_image_data (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ThumbnailCacheParamSpecImageData* spec;
	g_return_val_if_fail (g_type_is_a (object_type, THUMBNAIL_CACHE_TYPE_IMAGE_DATA), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer thumbnail_cache_value_get_image_data (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, THUMBNAIL_CACHE_TYPE_IMAGE_DATA), NULL);
	return value->data[0].v_pointer;
}


static void thumbnail_cache_value_set_image_data (GValue* value, gpointer v_object) {
	ThumbnailCacheImageData* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, THUMBNAIL_CACHE_TYPE_IMAGE_DATA));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, THUMBNAIL_CACHE_TYPE_IMAGE_DATA));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		thumbnail_cache_image_data_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		thumbnail_cache_image_data_unref (old);
	}
}


static void thumbnail_cache_image_data_class_init (ThumbnailCacheImageDataClass * klass) {
	thumbnail_cache_image_data_parent_class = g_type_class_peek_parent (klass);
	THUMBNAIL_CACHE_IMAGE_DATA_CLASS (klass)->finalize = thumbnail_cache_image_data_finalize;
}


static void thumbnail_cache_image_data_instance_init (ThumbnailCacheImageData * self) {
	self->ref_count = 1;
}


static void thumbnail_cache_image_data_finalize (ThumbnailCacheImageData* obj) {
	ThumbnailCacheImageData * self;
	self = THUMBNAIL_CACHE_IMAGE_DATA (obj);
	(self->pixbuf == NULL) ? NULL : (self->pixbuf = (g_object_unref (self->pixbuf), NULL));
}


static GType thumbnail_cache_image_data_get_type (void) {
	static GType thumbnail_cache_image_data_type_id = 0;
	if (thumbnail_cache_image_data_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { thumbnail_cache_value_image_data_init, thumbnail_cache_value_image_data_free_value, thumbnail_cache_value_image_data_copy_value, thumbnail_cache_value_image_data_peek_pointer, "p", thumbnail_cache_value_image_data_collect_value, "p", thumbnail_cache_value_image_data_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ThumbnailCacheImageDataClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) thumbnail_cache_image_data_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ThumbnailCacheImageData), 0, (GInstanceInitFunc) thumbnail_cache_image_data_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		thumbnail_cache_image_data_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ThumbnailCacheImageData", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return thumbnail_cache_image_data_type_id;
}


static gpointer thumbnail_cache_image_data_ref (gpointer instance) {
	ThumbnailCacheImageData* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void thumbnail_cache_image_data_unref (gpointer instance) {
	ThumbnailCacheImageData* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		THUMBNAIL_CACHE_IMAGE_DATA_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void thumbnail_cache_class_init (ThumbnailCacheClass * klass) {
	thumbnail_cache_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ThumbnailCachePrivate));
	G_OBJECT_CLASS (klass)->finalize = thumbnail_cache_finalize;
}


static void thumbnail_cache_instance_init (ThumbnailCache * self) {
	self->priv = THUMBNAIL_CACHE_GET_PRIVATE (self);
	self->priv->cache_map = gee_hash_map_new (G_TYPE_INT64, (GBoxedCopyFunc) _int64_dup, g_free, THUMBNAIL_CACHE_TYPE_IMAGE_DATA, (GBoxedCopyFunc) thumbnail_cache_image_data_ref, thumbnail_cache_image_data_unref, _int64_hash_ghash_func, _int64_equal_gequal_func, g_direct_equal);
	self->priv->cache_lru = gee_array_list_new (G_TYPE_INT64, (GBoxedCopyFunc) _int64_dup, g_free, _int64_equal_gequal_func);
	self->priv->cached_bytes = (gulong) 0;
	self->priv->photo_table = photo_table_new ();
}


static void thumbnail_cache_finalize (GObject* obj) {
	ThumbnailCache * self;
	self = THUMBNAIL_CACHE (obj);
	(self->priv->cache_dir == NULL) ? NULL : (self->priv->cache_dir = (g_object_unref (self->priv->cache_dir), NULL));
	self->priv->jpeg_quality = (g_free (self->priv->jpeg_quality), NULL);
	(self->priv->cache_map == NULL) ? NULL : (self->priv->cache_map = (g_object_unref (self->priv->cache_map), NULL));
	(self->priv->cache_lru == NULL) ? NULL : (self->priv->cache_lru = (g_object_unref (self->priv->cache_lru), NULL));
	(self->priv->cache_table == NULL) ? NULL : (self->priv->cache_table = (database_table_unref (self->priv->cache_table), NULL));
	(self->priv->photo_table == NULL) ? NULL : (self->priv->photo_table = (database_table_unref (self->priv->photo_table), NULL));
	G_OBJECT_CLASS (thumbnail_cache_parent_class)->finalize (obj);
}


GType thumbnail_cache_get_type (void) {
	static GType thumbnail_cache_type_id = 0;
	if (thumbnail_cache_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ThumbnailCacheClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) thumbnail_cache_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ThumbnailCache), 0, (GInstanceInitFunc) thumbnail_cache_instance_init, NULL };
		thumbnail_cache_type_id = g_type_register_static (G_TYPE_OBJECT, "ThumbnailCache", &g_define_type_info, 0);
	}
	return thumbnail_cache_type_id;
}




