/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gio/gio.h>


#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_ROTATION (rotation_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define JPEG_TYPE_MARKER (jpeg_marker_get_type ())

#define JPEG_TYPE_QUALITY (jpeg_quality_get_type ())

typedef enum  {
	ROTATION_CLOCKWISE,
	ROTATION_COUNTERCLOCKWISE,
	ROTATION_MIRROR,
	ROTATION_UPSIDE_DOWN
} Rotation;

struct _Dimensions {
	gint width;
	gint height;
};

typedef enum  {
	JPEG_MARKER_SOI = 0xD8,
	JPEG_MARKER_EOI = 0xD9,
	JPEG_MARKER_APP0 = 0xE0,
	JPEG_MARKER_APP1 = 0xE1
} JpegMarker;

typedef enum  {
	JPEG_QUALITY_LOW = 50,
	JPEG_QUALITY_MEDIUM = 75,
	JPEG_QUALITY_HIGH = 90,
	JPEG_QUALITY_MAXIMUM = 100
} JpegQuality;



GdkColor fetch_color (const char* spec, GdkDrawable* drawable);
GdkColor parse_color (const char* spec);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
guint32 convert_color (guint16 component);
guint32 convert_rgba (const GdkColor* c, guint8 alpha);
GType rotation_get_type (void);
GdkPixbuf* rotation_perform (Rotation self, GdkPixbuf* pixbuf);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
void dimensions_init (Dimensions *self, gint width, gint height);
Dimensions dimensions_get_scaled (Dimensions *self, gint scale);
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp);
guchar shift_color_byte (gint b, gint shift);
void shift_colors (GdkPixbuf* pixbuf, gint red, gint green, gint blue, gint alpha);
gboolean coord_in_rectangle (gint x, gint y, const GdkRectangle* rect);
#define JPEG_MARKER_PREFIX ((guint8) 0xFF)
GType jpeg_marker_get_type (void);
guint8 jpeg_marker_get_byte (JpegMarker self);
GType jpeg_quality_get_type (void);
gint jpeg_quality_get_pct (JpegQuality self);
char* jpeg_quality_get_pct_text (JpegQuality self);
char* jpeg_quality_to_string (JpegQuality self);
gint jpeg_read_marker (GFileInputStream* fins, JpegMarker* marker, GError** error);
gboolean jpeg_is_jpeg (GFile* file, GError** error);
void jpeg_write_marker (GFileOutputStream* fouts, JpegMarker marker, gint length, GError** error);



#line 7 "image_util.vala"
GdkColor parse_color (const char* spec) {
#line 97 "image_util.c"
	GdkColor result;
	result = fetch_color (spec, NULL);
	return result;
}


#line 11 "image_util.vala"
GdkColor fetch_color (const char* spec, GdkDrawable* drawable) {
#line 106 "image_util.c"
	GdkColor result;
	GdkColor color = {0};
#line 13 "image_util.vala"
	if (!gdk_color_parse (spec, &color)) {
#line 14 "image_util.vala"
		g_error ("image_util.vala:14: Can't parse color %s", spec);
#line 113 "image_util.c"
	}
#line 16 "image_util.vala"
	if (drawable == NULL) {
#line 117 "image_util.c"
		GtkWindow* app;
		app = GTK_WINDOW (app_window_get_instance ());
#line 18 "image_util.vala"
		if (app != NULL) {
#line 19 "image_util.vala"
			drawable = GDK_DRAWABLE (GTK_WIDGET (app)->window);
#line 124 "image_util.c"
		}
		(app == NULL) ? NULL : (app = (g_object_unref (app), NULL));
	}
#line 22 "image_util.vala"
	if (drawable != NULL) {
#line 130 "image_util.c"
		GdkColormap* _tmp0_;
		GdkColormap* colormap;
#line 23 "image_util.vala"
		_tmp0_ = NULL;
#line 135 "image_util.c"
		colormap = (_tmp0_ = gdk_drawable_get_colormap (drawable), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 24 "image_util.vala"
		if (colormap == NULL) {
#line 25 "image_util.vala"
			g_error ("image_util.vala:25: Can't get colormap for drawable");
#line 141 "image_util.c"
		}
#line 27 "image_util.vala"
		if (!gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE)) {
#line 28 "image_util.vala"
			g_error ("image_util.vala:28: Can't allocate color %s", spec);
#line 147 "image_util.c"
		}
		(colormap == NULL) ? NULL : (colormap = (g_object_unref (colormap), NULL));
	}
	result = color;
	return result;
}


#line 34 "image_util.vala"
inline guint32 convert_color (guint16 component) {
#line 158 "image_util.c"
	guint32 result;
	result = (guint32) (component / 256);
	return result;
}


#line 38 "image_util.vala"
guint32 convert_rgba (const GdkColor* c, guint8 alpha) {
#line 167 "image_util.c"
	guint32 result;
	result = (((convert_color ((*c).red) << 24) | (convert_color ((*c).green) << 16)) | (convert_color ((*c).blue) << 8)) | alpha;
	return result;
}


#line 49 "image_util.vala"
GdkPixbuf* rotation_perform (Rotation self, GdkPixbuf* pixbuf) {
#line 176 "image_util.c"
	GdkPixbuf* result;
#line 49 "image_util.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 50 "image_util.vala"
	switch (self) {
#line 182 "image_util.c"
		case ROTATION_CLOCKWISE:
		{
			result = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_CLOCKWISE);
			return result;
		}
		case ROTATION_COUNTERCLOCKWISE:
		{
			result = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
			return result;
		}
		case ROTATION_MIRROR:
		{
			result = gdk_pixbuf_flip (pixbuf, TRUE);
			return result;
		}
		case ROTATION_UPSIDE_DOWN:
		{
			result = gdk_pixbuf_flip (pixbuf, FALSE);
			return result;
		}
		default:
		{
			GdkPixbuf* _tmp0_;
#line 64 "image_util.vala"
			g_error ("image_util.vala:64: Unknown rotation: %d", (gint) self);
#line 66 "image_util.vala"
			_tmp0_ = NULL;
#line 210 "image_util.c"
			result = (_tmp0_ = pixbuf, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
			return result;
		}
	}
}



GType rotation_get_type (void) {
	static GType rotation_type_id = 0;
	if (G_UNLIKELY (rotation_type_id == 0)) {
		static const GEnumValue values[] = {{ROTATION_CLOCKWISE, "ROTATION_CLOCKWISE", "clockwise"}, {ROTATION_COUNTERCLOCKWISE, "ROTATION_COUNTERCLOCKWISE", "counterclockwise"}, {ROTATION_MIRROR, "ROTATION_MIRROR", "mirror"}, {ROTATION_UPSIDE_DOWN, "ROTATION_UPSIDE_DOWN", "upside-down"}, {0, NULL, NULL}};
		rotation_type_id = g_enum_register_static ("Rotation", values);
	}
	return rotation_type_id;
}


#line 71 "image_util.vala"
GdkPixbuf* scale_pixbuf (GdkPixbuf* pixbuf, gint scale, GdkInterpType interp) {
#line 231 "image_util.c"
	GdkPixbuf* result;
	Dimensions _tmp0_ = {0};
	Dimensions original;
	Dimensions scaled;
	gboolean _tmp1_;
#line 71 "image_util.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
#line 239 "image_util.c"
	original = (dimensions_init (&_tmp0_, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf)), _tmp0_);
	scaled = dimensions_get_scaled (&original, scale);
	_tmp1_ = FALSE;
#line 74 "image_util.vala"
	if (original.width == scaled.width) {
#line 74 "image_util.vala"
		_tmp1_ = original.height == scaled.height;
#line 247 "image_util.c"
	} else {
#line 74 "image_util.vala"
		_tmp1_ = FALSE;
#line 251 "image_util.c"
	}
#line 74 "image_util.vala"
	if (_tmp1_) {
#line 255 "image_util.c"
		GdkPixbuf* _tmp2_;
#line 75 "image_util.vala"
		_tmp2_ = NULL;
#line 259 "image_util.c"
		result = (_tmp2_ = pixbuf, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
		return result;
	}
	result = gdk_pixbuf_scale_simple (pixbuf, scaled.width, scaled.height, interp);
	return result;
}


#line 80 "image_util.vala"
inline guchar shift_color_byte (gint b, gint shift) {
#line 270 "image_util.c"
	guchar result;
	result = (guchar) CLAMP (b + shift, 0, 255);
	return result;
}


#line 84 "image_util.vala"
void shift_colors (GdkPixbuf* pixbuf, gint red, gint green, gint blue, gint alpha) {
#line 279 "image_util.c"
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gint width;
	gint height;
	gint rowstride;
	gint channels;
	guchar* pixels;
#line 84 "image_util.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 291 "image_util.c"
	_tmp0_ = FALSE;
#line 85 "image_util.vala"
	if (red >= (-255)) {
#line 85 "image_util.vala"
		_tmp0_ = red <= 255;
#line 297 "image_util.c"
	} else {
#line 85 "image_util.vala"
		_tmp0_ = FALSE;
#line 301 "image_util.c"
	}
#line 85 "image_util.vala"
	g_assert (_tmp0_);
#line 305 "image_util.c"
	_tmp1_ = FALSE;
#line 86 "image_util.vala"
	if (green >= (-255)) {
#line 86 "image_util.vala"
		_tmp1_ = green <= 255;
#line 311 "image_util.c"
	} else {
#line 86 "image_util.vala"
		_tmp1_ = FALSE;
#line 315 "image_util.c"
	}
#line 86 "image_util.vala"
	g_assert (_tmp1_);
#line 319 "image_util.c"
	_tmp2_ = FALSE;
#line 87 "image_util.vala"
	if (blue >= (-255)) {
#line 87 "image_util.vala"
		_tmp2_ = blue <= 255;
#line 325 "image_util.c"
	} else {
#line 87 "image_util.vala"
		_tmp2_ = FALSE;
#line 329 "image_util.c"
	}
#line 87 "image_util.vala"
	g_assert (_tmp2_);
#line 333 "image_util.c"
	_tmp3_ = FALSE;
#line 88 "image_util.vala"
	if (alpha >= (-255)) {
#line 88 "image_util.vala"
		_tmp3_ = alpha <= 255;
#line 339 "image_util.c"
	} else {
#line 88 "image_util.vala"
		_tmp3_ = FALSE;
#line 343 "image_util.c"
	}
#line 88 "image_util.vala"
	g_assert (_tmp3_);
#line 347 "image_util.c"
	width = gdk_pixbuf_get_width (pixbuf);
	height = gdk_pixbuf_get_height (pixbuf);
	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
	channels = gdk_pixbuf_get_n_channels (pixbuf);
	pixels = gdk_pixbuf_get_pixels (pixbuf);
#line 96 "image_util.vala"
	g_assert (channels >= 3);
#line 97 "image_util.vala"
	g_assert (gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB);
#line 98 "image_util.vala"
	g_assert (gdk_pixbuf_get_bits_per_sample (pixbuf) == 8);
#line 359 "image_util.c"
	{
		gint y;
		y = 0;
		{
			gboolean _tmp4_;
			_tmp4_ = TRUE;
#line 100 "image_util.vala"
			while (TRUE) {
#line 368 "image_util.c"
				gint y_offset;
#line 100 "image_util.vala"
				if (!_tmp4_) {
#line 100 "image_util.vala"
					y++;
#line 374 "image_util.c"
				}
#line 100 "image_util.vala"
				_tmp4_ = FALSE;
#line 100 "image_util.vala"
				if (!(y < height)) {
#line 100 "image_util.vala"
					break;
#line 382 "image_util.c"
				}
				y_offset = y * rowstride;
				{
					gint x;
					x = 0;
					{
						gboolean _tmp5_;
						_tmp5_ = TRUE;
#line 103 "image_util.vala"
						while (TRUE) {
#line 393 "image_util.c"
							gint offset;
							gboolean _tmp6_;
#line 103 "image_util.vala"
							if (!_tmp5_) {
#line 103 "image_util.vala"
								x++;
#line 400 "image_util.c"
							}
#line 103 "image_util.vala"
							_tmp5_ = FALSE;
#line 103 "image_util.vala"
							if (!(x < width)) {
#line 103 "image_util.vala"
								break;
#line 408 "image_util.c"
							}
							offset = y_offset + (x * channels);
#line 106 "image_util.vala"
							if (red != 0) {
#line 107 "image_util.vala"
								pixels[offset] = shift_color_byte ((gint) pixels[offset], red);
#line 415 "image_util.c"
							}
#line 109 "image_util.vala"
							if (green != 0) {
#line 110 "image_util.vala"
								pixels[offset + 1] = shift_color_byte ((gint) pixels[offset + 1], green);
#line 421 "image_util.c"
							}
#line 112 "image_util.vala"
							if (blue != 0) {
#line 113 "image_util.vala"
								pixels[offset + 2] = shift_color_byte ((gint) pixels[offset + 2], blue);
#line 427 "image_util.c"
							}
							_tmp6_ = FALSE;
#line 115 "image_util.vala"
							if (alpha != 0) {
#line 115 "image_util.vala"
								_tmp6_ = channels >= 4;
#line 434 "image_util.c"
							} else {
#line 115 "image_util.vala"
								_tmp6_ = FALSE;
#line 438 "image_util.c"
							}
#line 115 "image_util.vala"
							if (_tmp6_) {
#line 116 "image_util.vala"
								pixels[offset + 3] = shift_color_byte ((gint) pixels[offset + 3], alpha);
#line 444 "image_util.c"
							}
						}
					}
				}
			}
		}
	}
}


#line 121 "image_util.vala"
gboolean coord_in_rectangle (gint x, gint y, const GdkRectangle* rect) {
#line 457 "image_util.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
#line 122 "image_util.vala"
	if (x >= (*rect).x) {
#line 122 "image_util.vala"
		_tmp2_ = x < ((*rect).x + (*rect).width);
#line 469 "image_util.c"
	} else {
#line 122 "image_util.vala"
		_tmp2_ = FALSE;
#line 473 "image_util.c"
	}
#line 122 "image_util.vala"
	if (_tmp2_) {
#line 122 "image_util.vala"
		_tmp1_ = y >= (*rect).y;
#line 479 "image_util.c"
	} else {
#line 122 "image_util.vala"
		_tmp1_ = FALSE;
#line 483 "image_util.c"
	}
#line 122 "image_util.vala"
	if (_tmp1_) {
#line 122 "image_util.vala"
		_tmp0_ = y <= ((*rect).y + (*rect).height);
#line 489 "image_util.c"
	} else {
#line 122 "image_util.vala"
		_tmp0_ = FALSE;
#line 493 "image_util.c"
	}
	result = _tmp0_;
	return result;
}


#line 135 "image_util.vala"
guint8 jpeg_marker_get_byte (JpegMarker self) {
#line 502 "image_util.c"
	guint8 result;
	result = (guint8) self;
	return result;
}



GType jpeg_marker_get_type (void) {
	static GType jpeg_marker_type_id = 0;
	if (G_UNLIKELY (jpeg_marker_type_id == 0)) {
		static const GEnumValue values[] = {{JPEG_MARKER_SOI, "JPEG_MARKER_SOI", "soi"}, {JPEG_MARKER_EOI, "JPEG_MARKER_EOI", "eoi"}, {JPEG_MARKER_APP0, "JPEG_MARKER_APP0", "app0"}, {JPEG_MARKER_APP1, "JPEG_MARKER_APP1", "app1"}, {0, NULL, NULL}};
		jpeg_marker_type_id = g_enum_register_static ("JpegMarker", values);
	}
	return jpeg_marker_type_id;
}


#line 146 "image_util.vala"
gint jpeg_quality_get_pct (JpegQuality self) {
#line 522 "image_util.c"
	gint result;
	result = (gint) self;
	return result;
}


#line 150 "image_util.vala"
char* jpeg_quality_get_pct_text (JpegQuality self) {
#line 531 "image_util.c"
	char* result;
	result = g_strdup_printf ("%d", (gint) self);
	return result;
}


#line 154 "image_util.vala"
char* jpeg_quality_to_string (JpegQuality self) {
#line 540 "image_util.c"
	char* result;
#line 155 "image_util.vala"
	switch (self) {
#line 544 "image_util.c"
		case JPEG_QUALITY_LOW:
		{
			result = g_strdup_printf ("Low (%d%%)", (gint) self);
			return result;
		}
		case JPEG_QUALITY_MEDIUM:
		{
			result = g_strdup_printf ("Medium (%d%%)", (gint) self);
			return result;
		}
		case JPEG_QUALITY_HIGH:
		{
			result = g_strdup_printf ("High (%d%%)", (gint) self);
			return result;
		}
		case JPEG_QUALITY_MAXIMUM:
		{
			result = g_strdup_printf ("Maximum (%d%%)", (gint) self);
			return result;
		}
	}
#line 169 "image_util.vala"
	g_warn_if_reached ();
#line 568 "image_util.c"
	result = NULL;
	return result;
}



GType jpeg_quality_get_type (void) {
	static GType jpeg_quality_type_id = 0;
	if (G_UNLIKELY (jpeg_quality_type_id == 0)) {
		static const GEnumValue values[] = {{JPEG_QUALITY_LOW, "JPEG_QUALITY_LOW", "low"}, {JPEG_QUALITY_MEDIUM, "JPEG_QUALITY_MEDIUM", "medium"}, {JPEG_QUALITY_HIGH, "JPEG_QUALITY_HIGH", "high"}, {JPEG_QUALITY_MAXIMUM, "JPEG_QUALITY_MAXIMUM", "maximum"}, {0, NULL, NULL}};
		jpeg_quality_type_id = g_enum_register_static ("JpegQuality", values);
	}
	return jpeg_quality_type_id;
}


#line 175 "image_util.vala"
gboolean jpeg_is_jpeg (GFile* file, GError** error) {
#line 587 "image_util.c"
	gboolean result;
	GError * _inner_error_;
	GFileInputStream* fins;
	JpegMarker marker;
	gint segment_length;
	gboolean _tmp0_;
#line 175 "image_util.vala"
	g_return_val_if_fail (G_IS_FILE (file), FALSE);
#line 596 "image_util.c"
	_inner_error_ = NULL;
	fins = g_file_read (file, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return FALSE;
	}
	marker = 0;
	segment_length = jpeg_read_marker (fins, &marker, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
		return FALSE;
	}
	_tmp0_ = FALSE;
#line 182 "image_util.vala"
	if (marker == JPEG_MARKER_SOI) {
#line 182 "image_util.vala"
		_tmp0_ = segment_length == 0;
#line 615 "image_util.c"
	} else {
#line 182 "image_util.vala"
		_tmp0_ = FALSE;
#line 619 "image_util.c"
	}
	/* for now, merely checking for SOI*/
	result = _tmp0_;
	(fins == NULL) ? NULL : (fins = (g_object_unref (fins), NULL));
	return result;
}


#line 185 "image_util.vala"
gint jpeg_read_marker (GFileInputStream* fins, JpegMarker* marker, GError** error) {
#line 630 "image_util.c"
	gint result;
	GError * _inner_error_;
	guint8 byte;
	guint16 length;
	gsize bytes_read;
	gboolean _tmp0_;
#line 185 "image_util.vala"
	g_return_val_if_fail (G_IS_FILE_INPUT_STREAM (fins), 0);
#line 639 "image_util.c"
	_inner_error_ = NULL;
	byte = (guint8) 0;
	length = (guint16) 0;
	bytes_read = 0UL;
#line 190 "image_util.vala"
	g_input_stream_read_all (G_INPUT_STREAM (fins), &byte, (gsize) 1, &bytes_read, NULL, &_inner_error_);
#line 646 "image_util.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return 0;
	}
#line 191 "image_util.vala"
	if (byte != JPEG_MARKER_PREFIX) {
#line 653 "image_util.c"
		result = -1;
		return result;
	}
#line 194 "image_util.vala"
	g_input_stream_read_all (G_INPUT_STREAM (fins), &byte, (gsize) 1, &bytes_read, NULL, &_inner_error_);
#line 659 "image_util.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return 0;
	}
#line 195 "image_util.vala"
	(*marker) = (JpegMarker) byte;
#line 666 "image_util.c"
	_tmp0_ = FALSE;
#line 196 "image_util.vala"
	if ((*marker) == JPEG_MARKER_SOI) {
#line 196 "image_util.vala"
		_tmp0_ = TRUE;
#line 672 "image_util.c"
	} else {
#line 196 "image_util.vala"
		_tmp0_ = (*marker) == JPEG_MARKER_EOI;
#line 676 "image_util.c"
	}
#line 196 "image_util.vala"
	if (_tmp0_) {
#line 680 "image_util.c"
		/* no length*/
		result = 0;
		return result;
	}
#line 201 "image_util.vala"
	g_input_stream_read_all (G_INPUT_STREAM (fins), &length, (gsize) 2, &bytes_read, NULL, &_inner_error_);
#line 687 "image_util.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return 0;
	}
#line 202 "image_util.vala"
	length = GUINT16_FROM_BE (length);
#line 203 "image_util.vala"
	if (length < 2) {
#line 204 "image_util.vala"
		g_debug ("image_util.vala:204: Invalid length %Xh at ofs %llXh", (guint) length, G_FILE_INPUT_STREAM_GET_CLASS (fins)->tell (fins) - 2);
#line 698 "image_util.c"
		result = -1;
		return result;
	}
	/* account for two length bytes already read*/
	result = length - 2;
	return result;
}


/* this writes the marker and a length (if positive)*/
#line 214 "image_util.vala"
void jpeg_write_marker (GFileOutputStream* fouts, JpegMarker marker, gint length, GError** error) {
#line 711 "image_util.c"
	GError * _inner_error_;
	guint8 prefix;
	guint8 byte;
	gsize written;
	guint16 host;
	guint16 motorola;
#line 214 "image_util.vala"
	g_return_if_fail (G_IS_FILE_OUTPUT_STREAM (fouts));
#line 720 "image_util.c"
	_inner_error_ = NULL;
	/* this is required to compile*/
	prefix = JPEG_MARKER_PREFIX;
	byte = jpeg_marker_get_byte (marker);
	written = 0UL;
#line 220 "image_util.vala"
	g_output_stream_write_all (G_OUTPUT_STREAM (fouts), &prefix, (gsize) 1, &written, NULL, &_inner_error_);
#line 728 "image_util.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
#line 221 "image_util.vala"
	g_output_stream_write_all (G_OUTPUT_STREAM (fouts), &byte, (gsize) 1, &written, NULL, &_inner_error_);
#line 735 "image_util.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
#line 223 "image_util.vala"
	if (length <= 0) {
#line 742 "image_util.c"
		return;
	}
	/* +2 to account for length bytes*/
#line 227 "image_util.vala"
	length = length + 2;
#line 748 "image_util.c"
	host = (guint16) length;
	motorola = (guint16) GUINT16_TO_BE (host);
#line 232 "image_util.vala"
	g_output_stream_write_all (G_OUTPUT_STREAM (fouts), &motorola, (gsize) 2, &written, NULL, &_inner_error_);
#line 753 "image_util.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
}




