/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <unique/unique.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_SHOTWELL_COMMAND (shotwell_command_get_type ())

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

typedef enum  {
	SHOTWELL_COMMAND_MOUNTED_CAMERA = 1
} ShotwellCommand;



GType shotwell_command_get_type (void);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
void app_window_mounted_camera_shell_notification (const char* uri);
UniqueResponse on_shotwell_message (UniqueApp* shotwell, gint command, UniqueMessageData* data, guint timestamp);
void debug_init (void);
#define RESOURCES_APP_TITLE "Shotwell"
gboolean app_window_is_mount_uri_supported (const char* uri);
static void _vala_array_add1 (char*** array, int* length, int* size, char* value);
static UniqueResponse _on_shotwell_message_unique_app_message_received (UniqueApp* _sender, gint command, UniqueMessageData* message_data, guint time_, gpointer self);
void debug_terminate (void);
void app_window_init (char** args, int args_length1);
void resources_init (void);
void database_table_init (void);
void thumbnail_cache_init (void);
void photo_init (void);
gboolean verify_databases (char** app_version);
AppWindow* app_window_new (void);
AppWindow* app_window_construct (GType object_type);
void photo_terminate (void);
void thumbnail_cache_terminate (void);
void database_table_terminate (void);
void resources_terminate (void);
void app_window_terminate (void);
void _main (char** args, int args_length1);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);




GType shotwell_command_get_type (void) {
	static GType shotwell_command_type_id = 0;
	if (G_UNLIKELY (shotwell_command_type_id == 0)) {
		static const GEnumValue values[] = {{SHOTWELL_COMMAND_MOUNTED_CAMERA, "SHOTWELL_COMMAND_MOUNTED_CAMERA", "mounted-camera"}, {0, NULL, NULL}};
		shotwell_command_type_id = g_enum_register_static ("ShotwellCommand", values);
	}
	return shotwell_command_type_id;
}


/* user-defined commands must be positive ints*/
#line 12 "main.vala"
UniqueResponse on_shotwell_message (UniqueApp* shotwell, gint command, UniqueMessageData* data, guint timestamp) {
#line 78 "main.c"
	UniqueResponse result;
	UniqueResponse response;
#line 12 "main.vala"
	g_return_val_if_fail (UNIQUE_IS_APP (shotwell), 0);
#line 12 "main.vala"
	g_return_val_if_fail (data != NULL, 0);
#line 85 "main.c"
	response = UNIQUE_RESPONSE_OK;
#line 15 "main.vala"
	switch (command) {
#line 89 "main.c"
		case SHOTWELL_COMMAND_MOUNTED_CAMERA:
		{
			char* _tmp1_;
			AppWindow* _tmp0_;
#line 48 "unique-1.0.vapi"
			_tmp1_ = NULL;
#line 446 "AppWindow.vala"
			_tmp0_ = NULL;
#line 17 "main.vala"
			app_window_mounted_camera_shell_notification (_tmp1_ = unique_message_data_get_text (data));
#line 100 "main.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
			(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 18 "main.vala"
			break;
#line 105 "main.c"
		}
		case UNIQUE_ACTIVATE:
		{
			AppWindow* _tmp2_;
#line 446 "AppWindow.vala"
			_tmp2_ = NULL;
#line 21 "main.vala"
			gtk_window_present_with_time (GTK_WINDOW (_tmp2_ = app_window_get_instance ()), (guint32) timestamp);
#line 114 "main.c"
			(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 22 "main.vala"
			break;
#line 118 "main.c"
		}
		default:
		{
#line 26 "main.vala"
			response = (UniqueResponse) 4;
#line 27 "main.vala"
			break;
#line 126 "main.c"
		}
	}
	result = response;
	return result;
}


static void _vala_array_add1 (char*** array, int* length, int* size, char* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (char*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


#line 12 "main.vala"
static UniqueResponse _on_shotwell_message_unique_app_message_received (UniqueApp* _sender, gint command, UniqueMessageData* message_data, guint time_, gpointer self) {
#line 146 "main.c"
	return on_shotwell_message (_sender, command, message_data, time_);
}


#line 33 "main.vala"
void _main (char** args, int args_length1) {
#line 153 "main.c"
	char** _tmp0_;
	gint mounts_size;
	gint mounts_length1;
	char** mounts;
	UniqueApp* shotwell;
	gboolean _tmp3_;
	char* app_version;
	char* _tmp7_;
	gboolean _tmp6_;
	char* _tmp5_;
	/* init GTK*/
#line 35 "main.vala"
	gtk_init (&args_length1, &args);
#line 167 "main.c"
	/* init debug prior to anything else*/
#line 38 "main.vala"
	debug_init ();
#line 171 "main.c"
	/* set up GLib environment*/
#line 41 "main.vala"
	g_set_application_name (RESOURCES_APP_TITLE);
#line 175 "main.c"
	/* examine command-line arguments for camera mounts
	 (everything else is ignored for now)*/
#line 45 "main.vala"
	_tmp0_ = NULL;
#line 180 "main.c"
	mounts = (_tmp0_ = g_new0 (char*, 0 + 1), mounts_length1 = 0, mounts_size = mounts_length1, _tmp0_);
	{
		gint ctr;
		ctr = 1;
		{
			gboolean _tmp1_;
			_tmp1_ = TRUE;
#line 46 "main.vala"
			while (TRUE) {
#line 46 "main.vala"
				if (!_tmp1_) {
#line 46 "main.vala"
					ctr++;
#line 194 "main.c"
				}
#line 46 "main.vala"
				_tmp1_ = FALSE;
#line 46 "main.vala"
				if (!(ctr < args_length1)) {
#line 46 "main.vala"
					break;
#line 202 "main.c"
				}
#line 47 "main.vala"
				if (app_window_is_mount_uri_supported (args[ctr])) {
#line 206 "main.c"
					const char* _tmp2_;
#line 48 "main.vala"
					_tmp2_ = NULL;
#line 48 "main.vala"
					_vala_array_add1 (&mounts, &mounts_length1, &mounts_size, (_tmp2_ = args[ctr], (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)));
#line 212 "main.c"
				}
			}
		}
	}
	/* single-instance app*/
	shotwell = unique_app_new ("org.yorba.shotwell", NULL);
#line 53 "main.vala"
	unique_app_add_command (shotwell, "MOUNTED_CAMERA", (gint) SHOTWELL_COMMAND_MOUNTED_CAMERA);
#line 54 "main.vala"
	g_signal_connect (shotwell, "message-received", (GCallback) _on_shotwell_message_unique_app_message_received, NULL);
#line 56 "main.vala"
	if ((g_object_get (shotwell, "is-running", &_tmp3_, NULL), _tmp3_)) {
#line 225 "main.c"
		/* send attached cameras & activate the window*/
		{
			char** mount_collection;
			int mount_collection_length1;
			int mount_it;
#line 58 "main.vala"
			mount_collection = mounts;
#line 233 "main.c"
			mount_collection_length1 = mounts_length1;
			for (mount_it = 0; mount_it < mounts_length1; mount_it = mount_it + 1) {
				const char* _tmp4_;
				char* mount;
#line 45 "main.vala"
				_tmp4_ = NULL;
#line 240 "main.c"
				mount = (_tmp4_ = mount_collection[mount_it], (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_));
				{
					UniqueMessageData* data;
					data = unique_message_data_new ();
#line 60 "main.vala"
					unique_message_data_set_text (data, mount, (gssize) (-1));
#line 62 "main.vala"
					unique_app_send_message (shotwell, (gint) SHOTWELL_COMMAND_MOUNTED_CAMERA, data);
#line 249 "main.c"
					mount = (g_free (mount), NULL);
					(data == NULL) ? NULL : (data = (unique_message_data_free (data), NULL));
				}
			}
		}
#line 65 "main.vala"
		unique_app_send_message (shotwell, (gint) UNIQUE_ACTIVATE, NULL);
#line 67 "main.vala"
		debug_terminate ();
#line 259 "main.c"
		mounts = (_vala_array_free (mounts, mounts_length1, (GDestroyNotify) g_free), NULL);
		(shotwell == NULL) ? NULL : (shotwell = (g_object_unref (shotwell), NULL));
		return;
	}
	/* initialize app-wide stuff*/
#line 73 "main.vala"
	app_window_init (args, args_length1);
#line 74 "main.vala"
	resources_init ();
#line 75 "main.vala"
	database_table_init ();
#line 76 "main.vala"
	thumbnail_cache_init ();
#line 77 "main.vala"
	photo_init ();
#line 79 "main.vala"
	g_message ("main.vala:79: Verifying databases ...");
#line 277 "main.c"
	app_version = NULL;
	_tmp7_ = NULL;
	_tmp5_ = NULL;
#line 81 "main.vala"
	if (!(_tmp6_ = verify_databases (&_tmp5_), app_version = (_tmp7_ = _tmp5_, app_version = (g_free (app_version), NULL), _tmp7_), _tmp6_)) {
#line 283 "main.c"
		GtkMessageDialog* dialog;
		dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "The database for your photo library is not compatible with this version of Shotwell.  " "It appears it was created by Shotwell %s.  Please use that version or later.", app_version, NULL));
#line 86 "main.vala"
		gtk_window_set_title (GTK_WINDOW (dialog), RESOURCES_APP_TITLE);
#line 87 "main.vala"
		gtk_dialog_run (GTK_DIALOG (dialog));
#line 88 "main.vala"
		gtk_object_destroy (GTK_OBJECT (dialog));
#line 292 "main.c"
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	} else {
		AppWindow* app_window;
		/* create main application window*/
		app_window = g_object_ref_sink (app_window_new ());
		/* report mount points*/
		{
			char** mount_collection;
			int mount_collection_length1;
			int mount_it;
#line 94 "main.vala"
			mount_collection = mounts;
#line 305 "main.c"
			mount_collection_length1 = mounts_length1;
			for (mount_it = 0; mount_it < mounts_length1; mount_it = mount_it + 1) {
				const char* _tmp8_;
				char* mount;
#line 45 "main.vala"
				_tmp8_ = NULL;
#line 312 "main.c"
				mount = (_tmp8_ = mount_collection[mount_it], (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
				{
#line 95 "main.vala"
					app_window_mounted_camera_shell_notification (mount);
#line 317 "main.c"
					mount = (g_free (mount), NULL);
				}
			}
		}
		/* throw it all on the display*/
#line 98 "main.vala"
		gtk_widget_show_all (GTK_WIDGET (app_window));
#line 325 "main.c"
		/* event loop*/
#line 101 "main.vala"
		gtk_main ();
#line 329 "main.c"
		(app_window == NULL) ? NULL : (app_window = (g_object_unref (app_window), NULL));
	}
#line 104 "main.vala"
	photo_terminate ();
#line 105 "main.vala"
	thumbnail_cache_terminate ();
#line 106 "main.vala"
	database_table_terminate ();
#line 107 "main.vala"
	resources_terminate ();
#line 108 "main.vala"
	app_window_terminate ();
#line 109 "main.vala"
	debug_terminate ();
#line 344 "main.c"
	mounts = (_vala_array_free (mounts, mounts_length1, (GDestroyNotify) g_free), NULL);
	(shotwell == NULL) ? NULL : (shotwell = (g_object_unref (shotwell), NULL));
	app_version = (g_free (app_version), NULL);
}


#line 33 "main.vala"
int main (int argc, char ** argv) {
#line 33 "main.vala"
	g_type_init ();
#line 33 "main.vala"
	_main (argv, argc);
#line 33 "main.vala"
	return 0;
#line 359 "main.c"
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




