/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <float.h>
#include <math.h>
#include <gdk/gdk.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <time.h>
#include <gobject/gvaluecollector.h>


#define TYPE_KEY_VALUE_MAP (key_value_map_get_type ())
#define KEY_VALUE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_KEY_VALUE_MAP, KeyValueMap))
#define KEY_VALUE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_KEY_VALUE_MAP, KeyValueMapClass))
#define IS_KEY_VALUE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_KEY_VALUE_MAP))
#define IS_KEY_VALUE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_KEY_VALUE_MAP))
#define KEY_VALUE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_KEY_VALUE_MAP, KeyValueMapClass))

typedef struct _KeyValueMap KeyValueMap;
typedef struct _KeyValueMapClass KeyValueMapClass;
typedef struct _KeyValueMapPrivate KeyValueMapPrivate;
typedef struct _ParamSpecKeyValueMap ParamSpecKeyValueMap;

#define TYPE_ADJUSTMENT_RELATION (adjustment_relation_get_type ())

#define TYPE_COMPASS_POINT (compass_point_get_type ())

struct _KeyValueMap {
	GTypeInstance parent_instance;
	volatile int ref_count;
	KeyValueMapPrivate * priv;
};

struct _KeyValueMapClass {
	GTypeClass parent_class;
	void (*finalize) (KeyValueMap *self);
};

struct _KeyValueMapPrivate {
	char* group;
	GeeHashMap* map;
};

struct _ParamSpecKeyValueMap {
	GParamSpec parent_instance;
};

typedef enum  {
	ADJUSTMENT_RELATION_BELOW,
	ADJUSTMENT_RELATION_IN_RANGE,
	ADJUSTMENT_RELATION_ABOVE
} AdjustmentRelation;

typedef enum  {
	COMPASS_POINT_NORTH,
	COMPASS_POINT_SOUTH,
	COMPASS_POINT_EAST,
	COMPASS_POINT_WEST
} CompassPoint;


static gpointer key_value_map_parent_class = NULL;

guint int64_hash (void* p);
gboolean int64_equal (void* a, void* b);
guint file_hash (void* key);
gboolean file_equal (void* a, void* b);
gpointer key_value_map_ref (gpointer instance);
void key_value_map_unref (gpointer instance);
GParamSpec* param_spec_key_value_map (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_key_value_map (GValue* value, gpointer v_object);
gpointer value_get_key_value_map (const GValue* value);
GType key_value_map_get_type (void);
#define KEY_VALUE_MAP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_KEY_VALUE_MAP, KeyValueMapPrivate))
enum  {
	KEY_VALUE_MAP_DUMMY_PROPERTY
};
KeyValueMap* key_value_map_new (const char* group);
KeyValueMap* key_value_map_construct (GType object_type, const char* group);
char* key_value_map_get_group (KeyValueMap* self);
GeeSet* key_value_map_get_keys (KeyValueMap* self);
gboolean key_value_map_has_key (KeyValueMap* self, const char* key);
void key_value_map_set_string (KeyValueMap* self, const char* key, const char* value);
void key_value_map_set_int (KeyValueMap* self, const char* key, gint value);
void key_value_map_set_double (KeyValueMap* self, const char* key, double value);
char* key_value_map_get_string (KeyValueMap* self, const char* key, const char* def);
gint key_value_map_get_int (KeyValueMap* self, const char* key, gint def);
double key_value_map_get_double (KeyValueMap* self, const char* key, double def);
GdkPoint key_value_map_get_point (KeyValueMap* self, const char* key, const GdkPoint* def);
void key_value_map_set_point (KeyValueMap* self, const char* key, const GdkPoint* point);
static void key_value_map_finalize (KeyValueMap* obj);
gboolean spin_event_loop (void);
glong find_last_offset (const char* str, gchar c);
void disassemble_filename (const char* basename, char** name, char** ext);
GType adjustment_relation_get_type (void);
AdjustmentRelation get_adjustment_relation (GtkAdjustment* adjustment, gint value);
GType compass_point_get_type (void);
guint64 query_total_file_size (GFile* file_or_dir, GError** error);
time_t query_file_modified (GFile* file, GError** error);
gboolean query_is_directory_empty (GFile* dir, GError** error);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 7 "util.vala"
guint int64_hash (void* p) {
#line 118 "util.c"
	guint result;
	guint8* u8;
	guint hash;
	/* Rotating XOR hash*/
	u8 = (guint8*) p;
	hash = (guint) 0;
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 11 "util.vala"
			while (TRUE) {
#line 11 "util.vala"
				if (!_tmp0_) {
#line 11 "util.vala"
					ctr++;
#line 137 "util.c"
				}
#line 11 "util.vala"
				_tmp0_ = FALSE;
#line 11 "util.vala"
				if (!(ctr < (sizeof (gint64) / sizeof (guint8)))) {
#line 11 "util.vala"
					break;
#line 145 "util.c"
				}
#line 12 "util.vala"
				hash = ((hash << 4) ^ (hash >> 28)) ^ (*(u8++));
#line 149 "util.c"
			}
		}
	}
	result = hash;
	return result;
}


#line 18 "util.vala"
gboolean int64_equal (void* a, void* b) {
#line 160 "util.c"
	gboolean result;
	gint64* bia;
	gint64* bib;
	bia = (gint64*) a;
	bib = (gint64*) b;
	result = (*bia) == (*bib);
	return result;
}


#line 25 "util.vala"
guint file_hash (void* key) {
#line 173 "util.c"
	guint result;
	GFile* file;
	char* _tmp0_;
	guint _tmp1_;
	file = (GFile*) key;
#line 952 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 181 "util.c"
	result = (_tmp1_ = g_str_hash (_tmp0_ = g_file_get_path (G_FILE (file))), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


#line 31 "util.vala"
gboolean file_equal (void* a, void* b) {
#line 189 "util.c"
	gboolean result;
	GFile* afile;
	GFile* bfile;
	char* _tmp1_;
	char* _tmp0_;
	gboolean _tmp2_;
	afile = (GFile*) a;
	bfile = (GFile*) b;
#line 952 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 952 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 202 "util.c"
	result = (_tmp2_ = _vala_strcmp0 (_tmp0_ = g_file_get_path (G_FILE (afile)), _tmp1_ = g_file_get_path (G_FILE (bfile))) == 0, _tmp1_ = (g_free (_tmp1_), NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
	return result;
}


#line 43 "util.vala"
KeyValueMap* key_value_map_construct (GType object_type, const char* group) {
#line 210 "util.c"
	KeyValueMap* self;
	char* _tmp1_;
	const char* _tmp0_;
#line 43 "util.vala"
	g_return_val_if_fail (group != NULL, NULL);
#line 216 "util.c"
	self = (KeyValueMap*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 44 "util.vala"
	_tmp0_ = NULL;
#line 44 "util.vala"
	self->priv->group = (_tmp1_ = (_tmp0_ = group, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->group = (g_free (self->priv->group), NULL), _tmp1_);
#line 223 "util.c"
	return self;
}


#line 43 "util.vala"
KeyValueMap* key_value_map_new (const char* group) {
#line 43 "util.vala"
	return key_value_map_construct (TYPE_KEY_VALUE_MAP, group);
#line 232 "util.c"
}


#line 47 "util.vala"
char* key_value_map_get_group (KeyValueMap* self) {
#line 238 "util.c"
	char* result;
	const char* _tmp0_;
#line 47 "util.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (self), NULL);
#line 48 "util.vala"
	_tmp0_ = NULL;
#line 245 "util.c"
	result = (_tmp0_ = self->priv->group, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 51 "util.vala"
GeeSet* key_value_map_get_keys (KeyValueMap* self) {
#line 253 "util.c"
	GeeSet* result;
#line 51 "util.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (self), NULL);
#line 257 "util.c"
	result = gee_abstract_map_get_keys (GEE_ABSTRACT_MAP (self->priv->map));
	return result;
}


#line 55 "util.vala"
gboolean key_value_map_has_key (KeyValueMap* self, const char* key) {
#line 265 "util.c"
	gboolean result;
#line 55 "util.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (self), FALSE);
#line 55 "util.vala"
	g_return_val_if_fail (key != NULL, FALSE);
#line 271 "util.c"
	result = gee_abstract_map_contains (GEE_ABSTRACT_MAP (self->priv->map), key);
	return result;
}


#line 59 "util.vala"
void key_value_map_set_string (KeyValueMap* self, const char* key, const char* value) {
#line 59 "util.vala"
	g_return_if_fail (IS_KEY_VALUE_MAP (self));
#line 59 "util.vala"
	g_return_if_fail (key != NULL);
#line 59 "util.vala"
	g_return_if_fail (value != NULL);
#line 60 "util.vala"
	g_assert (key != NULL);
#line 62 "util.vala"
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->map), key, value);
#line 289 "util.c"
}


#line 65 "util.vala"
void key_value_map_set_int (KeyValueMap* self, const char* key, gint value) {
#line 295 "util.c"
	char* _tmp0_;
#line 65 "util.vala"
	g_return_if_fail (IS_KEY_VALUE_MAP (self));
#line 65 "util.vala"
	g_return_if_fail (key != NULL);
#line 66 "util.vala"
	g_assert (key != NULL);
#line 99 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 68 "util.vala"
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->map), key, _tmp0_ = g_strdup_printf ("%i", value));
#line 307 "util.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
}


#line 670 "glib-2.0.vapi"
static char* double_to_string (double self) {
#line 314 "util.c"
	char* result;
	const char* _tmp1_;
	gint _tmp0__length1;
	gchar* _tmp0_;
	char* _tmp2_;
#line 671 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 671 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 671 "glib-2.0.vapi"
	_tmp2_ = NULL;
#line 326 "util.c"
	result = (_tmp2_ = (_tmp1_ = g_ascii_dtostr ((_tmp0_ = g_new0 (gchar, G_ASCII_DTOSTR_BUF_SIZE), _tmp0__length1 = G_ASCII_DTOSTR_BUF_SIZE, _tmp0_), G_ASCII_DTOSTR_BUF_SIZE, self), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
	return result;
}


#line 71 "util.vala"
void key_value_map_set_double (KeyValueMap* self, const char* key, double value) {
#line 334 "util.c"
	char* _tmp0_;
#line 71 "util.vala"
	g_return_if_fail (IS_KEY_VALUE_MAP (self));
#line 71 "util.vala"
	g_return_if_fail (key != NULL);
#line 72 "util.vala"
	g_assert (key != NULL);
#line 670 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 74 "util.vala"
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->map), key, _tmp0_ = double_to_string (value));
#line 346 "util.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
}


#line 77 "util.vala"
char* key_value_map_get_string (KeyValueMap* self, const char* key, const char* def) {
#line 353 "util.c"
	char* result;
	char* value;
	const char* _tmp0_;
	const char* _tmp1_;
#line 77 "util.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (self), NULL);
#line 77 "util.vala"
	g_return_val_if_fail (key != NULL, NULL);
#line 362 "util.c"
	value = (char*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->map), key);
	_tmp0_ = NULL;
#line 80 "util.vala"
	if (value != NULL) {
#line 80 "util.vala"
		_tmp0_ = value;
#line 369 "util.c"
	} else {
#line 80 "util.vala"
		_tmp0_ = def;
#line 373 "util.c"
	}
#line 80 "util.vala"
	_tmp1_ = NULL;
#line 377 "util.c"
	result = (_tmp1_ = _tmp0_, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	value = (g_free (value), NULL);
	return result;
	value = (g_free (value), NULL);
}


#line 83 "util.vala"
gint key_value_map_get_int (KeyValueMap* self, const char* key, gint def) {
#line 387 "util.c"
	gint result;
	char* value;
	gint _tmp0_;
#line 83 "util.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (self), 0);
#line 83 "util.vala"
	g_return_val_if_fail (key != NULL, 0);
#line 395 "util.c"
	value = (char*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->map), key);
	_tmp0_ = 0;
#line 86 "util.vala"
	if (value != NULL) {
#line 86 "util.vala"
		_tmp0_ = atoi (value);
#line 402 "util.c"
	} else {
#line 86 "util.vala"
		_tmp0_ = def;
#line 406 "util.c"
	}
	result = _tmp0_;
	value = (g_free (value), NULL);
	return result;
}


#line 89 "util.vala"
double key_value_map_get_double (KeyValueMap* self, const char* key, double def) {
#line 416 "util.c"
	double result;
	char* value;
	double _tmp0_;
#line 89 "util.vala"
	g_return_val_if_fail (IS_KEY_VALUE_MAP (self), 0.0);
#line 89 "util.vala"
	g_return_val_if_fail (key != NULL, 0.0);
#line 424 "util.c"
	value = (char*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->map), key);
	_tmp0_ = 0.0;
#line 92 "util.vala"
	if (value != NULL) {
#line 92 "util.vala"
		_tmp0_ = g_ascii_strtod (value, NULL);
#line 431 "util.c"
	} else {
#line 92 "util.vala"
		_tmp0_ = def;
#line 435 "util.c"
	}
	result = _tmp0_;
	value = (g_free (value), NULL);
	return result;
}


/* REDEYE: redeye reduction operates on circular regions defined by
         (Gdk.Point, int) pairs, where the Gdk.Point specifies the
         bounding circle's center and the the int specifies the circle's
         radius so, get_point( ) and set_point( ) functions have been
         added here to easily encode/decode Gdk.Points as strings.*/
#line 100 "util.vala"
GdkPoint key_value_map_get_point (KeyValueMap* self, const char* key, const GdkPoint* def) {
#line 450 "util.c"
	GdkPoint result;
	char* value;
	value = (char*) gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->map), key);
#line 103 "util.vala"
	if (value == NULL) {
#line 456 "util.c"
		result = (*def);
		value = (g_free (value), NULL);
		return result;
	} else {
		GdkPoint _result_ = {0};
#line 107 "util.vala"
		if (sscanf (value, "(%d, %d)", &_result_.x, &_result_.y) == 2) {
#line 464 "util.c"
			result = _result_;
			value = (g_free (value), NULL);
			return result;
		} else {
			result = (*def);
			value = (g_free (value), NULL);
			return result;
		}
	}
	value = (g_free (value), NULL);
}


#line 114 "util.vala"
void key_value_map_set_point (KeyValueMap* self, const char* key, const GdkPoint* point) {
#line 480 "util.c"
	char* _tmp0_;
#line 114 "util.vala"
	g_return_if_fail (IS_KEY_VALUE_MAP (self));
#line 114 "util.vala"
	g_return_if_fail (key != NULL);
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 115 "util.vala"
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->map), key, _tmp0_ = g_strdup_printf ("(%d, %d)", (*point).x, (*point).y));
#line 490 "util.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
}


static void value_key_value_map_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_key_value_map_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		key_value_map_unref (value->data[0].v_pointer);
	}
}


static void value_key_value_map_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = key_value_map_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_key_value_map_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_key_value_map_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		KeyValueMap* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = key_value_map_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_key_value_map_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	KeyValueMap** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = key_value_map_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_key_value_map (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecKeyValueMap* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_KEY_VALUE_MAP), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_key_value_map (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_KEY_VALUE_MAP), NULL);
	return value->data[0].v_pointer;
}


void value_set_key_value_map (GValue* value, gpointer v_object) {
	KeyValueMap* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_KEY_VALUE_MAP));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_KEY_VALUE_MAP));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		key_value_map_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		key_value_map_unref (old);
	}
}


static void key_value_map_class_init (KeyValueMapClass * klass) {
	key_value_map_parent_class = g_type_class_peek_parent (klass);
	KEY_VALUE_MAP_CLASS (klass)->finalize = key_value_map_finalize;
	g_type_class_add_private (klass, sizeof (KeyValueMapPrivate));
}


static void key_value_map_instance_init (KeyValueMap * self) {
	self->priv = KEY_VALUE_MAP_GET_PRIVATE (self);
	self->priv->map = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal, g_str_equal);
	self->ref_count = 1;
}


static void key_value_map_finalize (KeyValueMap* obj) {
	KeyValueMap * self;
	self = KEY_VALUE_MAP (obj);
	self->priv->group = (g_free (self->priv->group), NULL);
	(self->priv->map == NULL) ? NULL : (self->priv->map = (g_object_unref (self->priv->map), NULL));
}


GType key_value_map_get_type (void) {
	static GType key_value_map_type_id = 0;
	if (key_value_map_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_key_value_map_init, value_key_value_map_free_value, value_key_value_map_copy_value, value_key_value_map_peek_pointer, "p", value_key_value_map_collect_value, "p", value_key_value_map_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (KeyValueMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) key_value_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (KeyValueMap), 0, (GInstanceInitFunc) key_value_map_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		key_value_map_type_id = g_type_register_fundamental (g_type_fundamental_next (), "KeyValueMap", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return key_value_map_type_id;
}


gpointer key_value_map_ref (gpointer instance) {
	KeyValueMap* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void key_value_map_unref (gpointer instance) {
	KeyValueMap* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		KEY_VALUE_MAP_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


/* Returns false if Gtk.quit() was called*/
#line 120 "util.vala"
gboolean spin_event_loop (void) {
#line 643 "util.c"
	gboolean result;
#line 121 "util.vala"
	while (TRUE) {
#line 121 "util.vala"
		if (!gtk_events_pending ()) {
#line 121 "util.vala"
			break;
#line 651 "util.c"
		}
#line 122 "util.vala"
		if (gtk_main_iteration ()) {
#line 655 "util.c"
			result = FALSE;
			return result;
		}
	}
	result = TRUE;
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


#line 129 "util.vala"
glong find_last_offset (const char* str, gchar c) {
#line 675 "util.c"
	glong result;
	glong offset;
#line 129 "util.vala"
	g_return_val_if_fail (str != NULL, 0L);
#line 680 "util.c"
	offset = string_get_length (str);
#line 131 "util.vala"
	while (TRUE) {
#line 131 "util.vala"
		if (!((offset = offset - 1) >= 0)) {
#line 131 "util.vala"
			break;
#line 688 "util.c"
		}
#line 132 "util.vala"
		if (g_utf8_get_char (g_utf8_offset_to_pointer (str, offset)) == c) {
#line 692 "util.c"
			result = offset;
			return result;
		}
	}
	result = (glong) (-1);
	return result;
}


#line 922 "glib-2.0.vapi"
static char* string_substring (const char* self, glong offset, glong len) {
#line 704 "util.c"
	char* result;
	glong string_length;
	const char* start;
#line 922 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 710 "util.c"
	string_length = g_utf8_strlen (self, -1);
#line 924 "glib-2.0.vapi"
	if (offset < 0) {
#line 925 "glib-2.0.vapi"
		offset = string_length + offset;
#line 926 "glib-2.0.vapi"
		g_return_val_if_fail (offset >= 0, NULL);
#line 718 "util.c"
	} else {
#line 928 "glib-2.0.vapi"
		g_return_val_if_fail (offset <= string_length, NULL);
#line 722 "util.c"
	}
#line 930 "glib-2.0.vapi"
	if (len < 0) {
#line 931 "glib-2.0.vapi"
		len = string_length - offset;
#line 728 "util.c"
	}
#line 933 "glib-2.0.vapi"
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
#line 732 "util.c"
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


#line 139 "util.vala"
void disassemble_filename (const char* basename, char** name, char** ext) {
#line 741 "util.c"
	glong offset;
#line 139 "util.vala"
	g_return_if_fail (basename != NULL);
#line 745 "util.c"
	if (name != NULL) {
		*name = NULL;
	}
	if (ext != NULL) {
		*ext = NULL;
	}
	offset = find_last_offset (basename, '.');
#line 141 "util.vala"
	if (offset <= 0) {
#line 755 "util.c"
		char* _tmp1_;
		const char* _tmp0_;
		char* _tmp2_;
		_tmp1_ = NULL;
#line 142 "util.vala"
		_tmp0_ = NULL;
#line 142 "util.vala"
		(*name) = (_tmp1_ = (_tmp0_ = basename, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), (*name) = (g_free ((*name)), NULL), _tmp1_);
#line 764 "util.c"
		_tmp2_ = NULL;
#line 143 "util.vala"
		(*ext) = (_tmp2_ = NULL, (*ext) = (g_free ((*ext)), NULL), _tmp2_);
#line 768 "util.c"
	} else {
		char* _tmp3_;
		char* _tmp4_;
		_tmp3_ = NULL;
#line 145 "util.vala"
		(*name) = (_tmp3_ = string_substring (basename, (glong) 0, offset), (*name) = (g_free ((*name)), NULL), _tmp3_);
#line 775 "util.c"
		_tmp4_ = NULL;
#line 146 "util.vala"
		(*ext) = (_tmp4_ = string_substring (basename, offset + 1, (glong) (-1)), (*ext) = (g_free ((*ext)), NULL), _tmp4_);
#line 779 "util.c"
	}
}



GType adjustment_relation_get_type (void) {
	static GType adjustment_relation_type_id = 0;
	if (G_UNLIKELY (adjustment_relation_type_id == 0)) {
		static const GEnumValue values[] = {{ADJUSTMENT_RELATION_BELOW, "ADJUSTMENT_RELATION_BELOW", "below"}, {ADJUSTMENT_RELATION_IN_RANGE, "ADJUSTMENT_RELATION_IN_RANGE", "in-range"}, {ADJUSTMENT_RELATION_ABOVE, "ADJUSTMENT_RELATION_ABOVE", "above"}, {0, NULL, NULL}};
		adjustment_relation_type_id = g_enum_register_static ("AdjustmentRelation", values);
	}
	return adjustment_relation_type_id;
}


#line 156 "util.vala"
AdjustmentRelation get_adjustment_relation (GtkAdjustment* adjustment, gint value) {
#line 797 "util.c"
	AdjustmentRelation result;
#line 156 "util.vala"
	g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), 0);
#line 157 "util.vala"
	if (value < ((gint) gtk_adjustment_get_value (adjustment))) {
#line 803 "util.c"
		result = ADJUSTMENT_RELATION_BELOW;
		return result;
	} else {
#line 159 "util.vala"
		if (value > ((gint) (gtk_adjustment_get_value (adjustment) + gtk_adjustment_get_page_size (adjustment)))) {
#line 809 "util.c"
			result = ADJUSTMENT_RELATION_ABOVE;
			return result;
		} else {
			result = ADJUSTMENT_RELATION_IN_RANGE;
			return result;
		}
	}
}



GType compass_point_get_type (void) {
	static GType compass_point_type_id = 0;
	if (G_UNLIKELY (compass_point_type_id == 0)) {
		static const GEnumValue values[] = {{COMPASS_POINT_NORTH, "COMPASS_POINT_NORTH", "north"}, {COMPASS_POINT_SOUTH, "COMPASS_POINT_SOUTH", "south"}, {COMPASS_POINT_EAST, "COMPASS_POINT_EAST", "east"}, {COMPASS_POINT_WEST, "COMPASS_POINT_WEST", "west"}, {0, NULL, NULL}};
		compass_point_type_id = g_enum_register_static ("CompassPoint", values);
	}
	return compass_point_type_id;
}


#line 172 "util.vala"
guint64 query_total_file_size (GFile* file_or_dir, GError** error) {
#line 833 "util.c"
	guint64 result;
	GError * _inner_error_;
	GFileType type;
	GFileEnumerator* enumerator;
	guint64 total_bytes;
	GFileInfo* info;
#line 172 "util.vala"
	g_return_val_if_fail (G_IS_FILE (file_or_dir), 0ULL);
#line 842 "util.c"
	_inner_error_ = NULL;
#line 173 "util.vala"
	spin_event_loop ();
#line 846 "util.c"
	type = g_file_query_file_type (file_or_dir, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL);
#line 176 "util.vala"
	if (type == G_FILE_TYPE_REGULAR) {
#line 850 "util.c"
		GFileInfo* info;
		info = NULL;
		{
			GFileInfo* _tmp0_;
			GFileInfo* _tmp1_;
			_tmp0_ = g_file_query_info (file_or_dir, G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch39_g_error;
				goto __finally39;
			}
			_tmp1_ = NULL;
#line 179 "util.vala"
			info = (_tmp1_ = _tmp0_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp1_);
#line 864 "util.c"
		}
		goto __finally39;
		__catch39_g_error:
		{
			GError * err;
			err = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp2_;
#line 952 "gio-2.0.vapi"
				_tmp2_ = NULL;
#line 182 "util.vala"
				g_debug ("util.vala:182: Unable to query filesize for %s: %s", _tmp2_ = g_file_get_path (file_or_dir), err->message);
#line 878 "util.c"
				_tmp2_ = (g_free (_tmp2_), NULL);
				result = (guint64) 0;
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
				(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
				return result;
			}
		}
		__finally39:
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			return 0ULL;
		}
		result = (guint64) g_file_info_get_size (info);
		(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
		return result;
	} else {
#line 188 "util.vala"
		if (type != G_FILE_TYPE_DIRECTORY) {
#line 898 "util.c"
			result = (guint64) 0;
			return result;
		}
	}
	enumerator = g_file_enumerate_children (file_or_dir, "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return 0ULL;
	}
#line 194 "util.vala"
	if (enumerator == NULL) {
#line 910 "util.c"
		result = (guint64) 0;
		(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
		return result;
	}
	total_bytes = (guint64) 0;
	info = NULL;
#line 200 "util.vala"
	while (TRUE) {
#line 919 "util.c"
		GFileInfo* _tmp3_;
		GFileInfo* _tmp4_;
		GFile* _tmp5_;
		guint64 _tmp6_;
		guint64 _tmp7_;
		_tmp3_ = g_file_enumerator_next_file (enumerator, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			return 0ULL;
		}
		_tmp4_ = NULL;
#line 200 "util.vala"
		if (!((info = (_tmp4_ = _tmp3_, (info == NULL) ? NULL : (info = (g_object_unref (info), NULL)), _tmp4_)) != NULL)) {
#line 200 "util.vala"
			break;
#line 937 "util.c"
		}
#line 948 "gio-2.0.vapi"
		_tmp5_ = NULL;
#line 941 "util.c"
		_tmp7_ = (_tmp6_ = query_total_file_size (_tmp5_ = g_file_get_child (file_or_dir, g_file_info_get_name (info)), &_inner_error_), (_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL)), _tmp6_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
			(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
			return 0ULL;
		}
#line 201 "util.vala"
		total_bytes = total_bytes + _tmp7_;
#line 951 "util.c"
	}
	result = total_bytes;
	(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	return result;
}


#line 206 "util.vala"
time_t query_file_modified (GFile* file, GError** error) {
#line 962 "util.c"
	time_t result;
	GError * _inner_error_;
	GFileInfo* info;
	GTimeVal _tmp0_ = {0};
	GTimeVal timestamp;
	_inner_error_ = NULL;
	info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
	timestamp = (g_get_current_time (&_tmp0_), _tmp0_);
#line 211 "util.vala"
	g_file_info_get_modification_time (info, &timestamp);
#line 977 "util.c"
	result = timestamp.tv_sec;
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	return result;
}


#line 216 "util.vala"
gboolean query_is_directory_empty (GFile* dir, GError** error) {
#line 986 "util.c"
	gboolean result;
	GError * _inner_error_;
	GFileEnumerator* enumerator;
	GFileInfo* _tmp0_;
	GFileInfo* _tmp1_;
	gboolean _tmp2_;
#line 216 "util.vala"
	g_return_val_if_fail (G_IS_FILE (dir), FALSE);
#line 995 "util.c"
	_inner_error_ = NULL;
#line 217 "util.vala"
	if (g_file_query_file_type (dir, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) != G_FILE_TYPE_DIRECTORY) {
#line 999 "util.c"
		result = FALSE;
		return result;
	}
	enumerator = g_file_enumerate_children (dir, "*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return FALSE;
	}
#line 221 "util.vala"
	if (enumerator == NULL) {
#line 1010 "util.c"
		result = FALSE;
		(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
		return result;
	}
	_tmp0_ = g_file_enumerator_next_file (enumerator, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
		return FALSE;
	}
#line 165 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 1023 "util.c"
	result = (_tmp2_ = (_tmp1_ = _tmp0_) == NULL, (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
	(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
	return result;
	(enumerator == NULL) ? NULL : (enumerator = (g_object_unref (enumerator), NULL));
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




