/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <libhal.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <gee.h>
#include <gphoto2/gphoto2-context.h>
#include <gphoto.h>
#include <gphoto2/gphoto2-abilities-list.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gphoto2/gphoto2-result.h>
#include <gphoto2/gphoto2-port-result.h>
#include <stdio.h>
#include <gphoto2/gphoto2-port-info-list.h>
#include <gphoto2/gphoto2-list.h>
#include <gphoto2/gphoto2-camera.h>


#define TYPE_FULLSCREEN_WINDOW (fullscreen_window_get_type ())
#define FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindow))
#define FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))
#define IS_FULLSCREEN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FULLSCREEN_WINDOW))
#define IS_FULLSCREEN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FULLSCREEN_WINDOW))
#define FULLSCREEN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FULLSCREEN_WINDOW, FullscreenWindowClass))

typedef struct _FullscreenWindow FullscreenWindow;
typedef struct _FullscreenWindowClass FullscreenWindowClass;
typedef struct _FullscreenWindowPrivate FullscreenWindowPrivate;

#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;
typedef struct _AppWindowPrivate AppWindowPrivate;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;

#define TYPE_COLLECTION_PAGE (collection_page_get_type ())
#define COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_PAGE, CollectionPage))
#define COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_PAGE, CollectionPageClass))
#define IS_COLLECTION_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_PAGE))
#define IS_COLLECTION_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_PAGE))
#define COLLECTION_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_PAGE, CollectionPageClass))

typedef struct _CollectionPage CollectionPage;
typedef struct _CollectionPageClass CollectionPageClass;

#define TYPE_EVENTS_DIRECTORY_PAGE (events_directory_page_get_type ())
#define EVENTS_DIRECTORY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPage))
#define EVENTS_DIRECTORY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPageClass))
#define IS_EVENTS_DIRECTORY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENTS_DIRECTORY_PAGE))
#define IS_EVENTS_DIRECTORY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENTS_DIRECTORY_PAGE))
#define EVENTS_DIRECTORY_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENTS_DIRECTORY_PAGE, EventsDirectoryPageClass))

typedef struct _EventsDirectoryPage EventsDirectoryPage;
typedef struct _EventsDirectoryPageClass EventsDirectoryPageClass;

#define TYPE_SINGLE_PHOTO_PAGE (single_photo_page_get_type ())
#define SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPage))
#define SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))
#define IS_SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SINGLE_PHOTO_PAGE))
#define IS_SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SINGLE_PHOTO_PAGE))
#define SINGLE_PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))

typedef struct _SinglePhotoPage SinglePhotoPage;
typedef struct _SinglePhotoPageClass SinglePhotoPageClass;

#define TYPE_PHOTO_PAGE (photo_page_get_type ())
#define PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_PAGE, PhotoPage))
#define PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_PAGE, PhotoPageClass))
#define IS_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_PAGE))
#define IS_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_PAGE))
#define PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_PAGE, PhotoPageClass))

typedef struct _PhotoPage PhotoPage;
typedef struct _PhotoPageClass PhotoPageClass;

#define TYPE_IMPORT_QUEUE_PAGE (import_queue_page_get_type ())
#define IMPORT_QUEUE_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePage))
#define IMPORT_QUEUE_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePageClass))
#define IS_IMPORT_QUEUE_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMPORT_QUEUE_PAGE))
#define IS_IMPORT_QUEUE_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMPORT_QUEUE_PAGE))
#define IMPORT_QUEUE_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMPORT_QUEUE_PAGE, ImportQueuePageClass))

typedef struct _ImportQueuePage ImportQueuePage;
typedef struct _ImportQueuePageClass ImportQueuePageClass;

#define TYPE_EVENT_PAGE (event_page_get_type ())
#define EVENT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENT_PAGE, EventPage))
#define EVENT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENT_PAGE, EventPageClass))
#define IS_EVENT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENT_PAGE))
#define IS_EVENT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENT_PAGE))
#define EVENT_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENT_PAGE, EventPageClass))

typedef struct _EventPage EventPage;
typedef struct _EventPageClass EventPageClass;

#define TYPE_IMPORT_PAGE (import_page_get_type ())
#define IMPORT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMPORT_PAGE, ImportPage))
#define IMPORT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMPORT_PAGE, ImportPageClass))
#define IS_IMPORT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMPORT_PAGE))
#define IS_IMPORT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMPORT_PAGE))
#define IMPORT_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMPORT_PAGE, ImportPageClass))

typedef struct _ImportPage ImportPage;
typedef struct _ImportPageClass ImportPageClass;

#define TYPE_DATABASE_TABLE (database_table_get_type ())
#define DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DATABASE_TABLE, DatabaseTable))
#define DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DATABASE_TABLE, DatabaseTableClass))
#define IS_DATABASE_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DATABASE_TABLE))
#define IS_DATABASE_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DATABASE_TABLE))
#define DATABASE_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DATABASE_TABLE, DatabaseTableClass))

typedef struct _DatabaseTable DatabaseTable;
typedef struct _DatabaseTableClass DatabaseTableClass;

#define TYPE_PHOTO_TABLE (photo_table_get_type ())
#define PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_TABLE, PhotoTable))
#define PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_TABLE, PhotoTableClass))
#define IS_PHOTO_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_TABLE))
#define IS_PHOTO_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_TABLE))
#define PHOTO_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_TABLE, PhotoTableClass))

typedef struct _PhotoTable PhotoTable;
typedef struct _PhotoTableClass PhotoTableClass;

#define TYPE_EVENT_TABLE (event_table_get_type ())
#define EVENT_TABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EVENT_TABLE, EventTable))
#define EVENT_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EVENT_TABLE, EventTableClass))
#define IS_EVENT_TABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EVENT_TABLE))
#define IS_EVENT_TABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EVENT_TABLE))
#define EVENT_TABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EVENT_TABLE, EventTableClass))

typedef struct _EventTable EventTable;
typedef struct _EventTableClass EventTableClass;

#define TYPE_SIDEBAR (sidebar_get_type ())
#define SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIDEBAR, Sidebar))
#define SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIDEBAR, SidebarClass))
#define IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIDEBAR))
#define IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIDEBAR))
#define SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIDEBAR, SidebarClass))

typedef struct _Sidebar Sidebar;
typedef struct _SidebarClass SidebarClass;

#define TYPE_SIDEBAR_MARKER (sidebar_marker_get_type ())
#define SIDEBAR_MARKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIDEBAR_MARKER, SidebarMarker))
#define SIDEBAR_MARKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIDEBAR_MARKER, SidebarMarkerClass))
#define IS_SIDEBAR_MARKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIDEBAR_MARKER))
#define IS_SIDEBAR_MARKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIDEBAR_MARKER))
#define SIDEBAR_MARKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIDEBAR_MARKER, SidebarMarkerClass))

typedef struct _SidebarMarker SidebarMarker;
typedef struct _SidebarMarkerClass SidebarMarkerClass;

#define TYPE_PHOTO_ID (photo_id_get_type ())
typedef struct _PhotoID PhotoID;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_EVENT_ID (event_id_get_type ())
typedef struct _EventID EventID;

#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;

#define TYPE_THUMBNAIL (thumbnail_get_type ())
#define THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THUMBNAIL, Thumbnail))
#define THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THUMBNAIL, ThumbnailClass))
#define IS_THUMBNAIL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THUMBNAIL))
#define IS_THUMBNAIL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THUMBNAIL))
#define THUMBNAIL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THUMBNAIL, ThumbnailClass))

typedef struct _Thumbnail Thumbnail;
typedef struct _ThumbnailClass ThumbnailClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;

#define APP_WINDOW_TYPE_COMPARE_EVENT_PAGE (app_window_compare_event_page_get_type ())
#define APP_WINDOW_COMPARE_EVENT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APP_WINDOW_TYPE_COMPARE_EVENT_PAGE, AppWindowCompareEventPage))
#define APP_WINDOW_COMPARE_EVENT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APP_WINDOW_TYPE_COMPARE_EVENT_PAGE, AppWindowCompareEventPageClass))
#define APP_WINDOW_IS_COMPARE_EVENT_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APP_WINDOW_TYPE_COMPARE_EVENT_PAGE))
#define APP_WINDOW_IS_COMPARE_EVENT_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APP_WINDOW_TYPE_COMPARE_EVENT_PAGE))
#define APP_WINDOW_COMPARE_EVENT_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APP_WINDOW_TYPE_COMPARE_EVENT_PAGE, AppWindowCompareEventPageClass))

typedef struct _AppWindowCompareEventPage AppWindowCompareEventPage;
typedef struct _AppWindowCompareEventPageClass AppWindowCompareEventPageClass;

#define TYPE_BATCH_IMPORT (batch_import_get_type ())
#define BATCH_IMPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BATCH_IMPORT, BatchImport))
#define BATCH_IMPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BATCH_IMPORT, BatchImportClass))
#define IS_BATCH_IMPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BATCH_IMPORT))
#define IS_BATCH_IMPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BATCH_IMPORT))
#define BATCH_IMPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BATCH_IMPORT, BatchImportClass))

typedef struct _BatchImport BatchImport;
typedef struct _BatchImportClass BatchImportClass;

#define TYPE_BATCH_IMPORT_JOB (batch_import_job_get_type ())
#define BATCH_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BATCH_IMPORT_JOB, BatchImportJob))
#define BATCH_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BATCH_IMPORT_JOB, BatchImportJobClass))
#define IS_BATCH_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BATCH_IMPORT_JOB))
#define IS_BATCH_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BATCH_IMPORT_JOB))
#define BATCH_IMPORT_JOB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BATCH_IMPORT_JOB, BatchImportJobClass))

typedef struct _BatchImportJob BatchImportJob;
typedef struct _BatchImportJobClass BatchImportJobClass;

#define APP_WINDOW_TYPE_FILE_IMPORT_JOB (app_window_file_import_job_get_type ())
#define APP_WINDOW_FILE_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APP_WINDOW_TYPE_FILE_IMPORT_JOB, AppWindowFileImportJob))
#define APP_WINDOW_FILE_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APP_WINDOW_TYPE_FILE_IMPORT_JOB, AppWindowFileImportJobClass))
#define APP_WINDOW_IS_FILE_IMPORT_JOB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APP_WINDOW_TYPE_FILE_IMPORT_JOB))
#define APP_WINDOW_IS_FILE_IMPORT_JOB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APP_WINDOW_TYPE_FILE_IMPORT_JOB))
#define APP_WINDOW_FILE_IMPORT_JOB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APP_WINDOW_TYPE_FILE_IMPORT_JOB, AppWindowFileImportJobClass))

typedef struct _AppWindowFileImportJob AppWindowFileImportJob;
typedef struct _AppWindowFileImportJobClass AppWindowFileImportJobClass;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;
typedef struct _CheckerboardPagePrivate CheckerboardPagePrivate;
typedef struct _CollectionPagePrivate CollectionPagePrivate;
typedef struct _EventPagePrivate EventPagePrivate;

#define TYPE_PAGE_LAYOUT (page_layout_get_type ())
#define PAGE_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE_LAYOUT, PageLayout))
#define PAGE_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE_LAYOUT, PageLayoutClass))
#define IS_PAGE_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE_LAYOUT))
#define IS_PAGE_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE_LAYOUT))
#define PAGE_LAYOUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE_LAYOUT, PageLayoutClass))

typedef struct _PageLayout PageLayout;
typedef struct _PageLayoutClass PageLayoutClass;

#define IMPORT_PAGE_TYPE_REFRESH_RESULT (import_page_refresh_result_get_type ())
typedef struct _BatchImportJobPrivate BatchImportJobPrivate;
typedef struct _AppWindowFileImportJobPrivate AppWindowFileImportJobPrivate;
typedef struct _ComparatorPrivate ComparatorPrivate;
typedef struct _AppWindowCompareEventPagePrivate AppWindowCompareEventPagePrivate;

struct _FullscreenWindow {
	GtkWindow parent_instance;
	FullscreenWindowPrivate * priv;
};

struct _FullscreenWindowClass {
	GtkWindowClass parent_class;
};

struct _FullscreenWindowPrivate {
	GdkModifierType ANY_BUTTON_MASK;
	GtkWindow* toolbar_window;
	GtkUIManager* ui;
	Page* page;
	GtkToolButton* close_button;
	GtkToggleToolButton* pin_button;
	gboolean is_toolbar_shown;
	gboolean waiting_for_invoke;
	time_t left_toolbar_time;
};

struct _AppWindow {
	GtkWindow parent_instance;
	AppWindowPrivate * priv;
};

struct _AppWindowClass {
	GtkWindowClass parent_class;
};

struct _AppWindowPrivate {
	char* import_dir;
	gint events_sort;
	LibHalContext* hal_context;
	DBusGConnection* hal_conn;
	CollectionPage* collection_page;
	EventsDirectoryPage* events_directory_page;
	PhotoPage* photo_page;
	ImportQueuePage* import_queue_page;
	GeeArrayList* event_list;
	GeeHashMap* camera_map;
	GeeArrayList* pages_to_be_removed;
	PhotoTable* photo_table;
	EventTable* event_table;
	Sidebar* sidebar;
	SidebarMarker* cameras_marker;
	GtkNotebook* notebook;
	GtkBox* layout;
	Page* current_page;
	GPContext* null_context;
	CameraAbilitiesList* abilities_list;
	FullscreenWindow* fullscreen_window;
};

struct _PhotoID {
	gint64 id;
};

struct _EventID {
	gint64 id;
};

typedef enum  {
	GPHOTO_ERROR_LIBRARY
} GPhotoError;
#define GPHOTO_ERROR gphoto_error_quark ()
struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkActionGroup* action_group;
	GtkActionGroup* common_action_group;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkToolbar* (*get_toolbar) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*switching_to_fullscreen) (Page* self);
	void (*returning_from_fullscreen) (Page* self);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, const GdkEventKey* event);
	void (*on_move) (Page* self, const GdkRectangle* rect);
	void (*on_resize) (Page* self, const GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
};

struct _CheckerboardPage {
	Page parent_instance;
	CheckerboardPagePrivate * priv;
};

struct _CheckerboardPageClass {
	PageClass parent_class;
	void (*on_selection_changed) (CheckerboardPage* self, gint count);
	GtkMenu* (*get_context_menu) (CheckerboardPage* self);
	void (*on_item_activated) (CheckerboardPage* self, LayoutItem* item);
	gboolean (*on_context_invoked) (CheckerboardPage* self, GtkMenu* context_menu);
	LayoutItem* (*get_fullscreen_photo) (CheckerboardPage* self);
	gboolean (*on_mouse_over) (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
};

struct _CollectionPage {
	CheckerboardPage parent_instance;
	CollectionPagePrivate * priv;
};

struct _CollectionPageClass {
	CheckerboardPageClass parent_class;
	void (*on_photos_menu) (CollectionPage* self);
};

struct _EventPage {
	CollectionPage parent_instance;
	EventPagePrivate * priv;
	EventID event_id;
};

struct _EventPageClass {
	CollectionPageClass parent_class;
};

typedef enum  {
	IMPORT_PAGE_REFRESH_RESULT_OK,
	IMPORT_PAGE_REFRESH_RESULT_BUSY,
	IMPORT_PAGE_REFRESH_RESULT_LOCKED,
	IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR
} ImportPageRefreshResult;

struct _BatchImportJob {
	GTypeInstance parent_instance;
	volatile int ref_count;
	BatchImportJobPrivate * priv;
};

struct _BatchImportJobClass {
	GTypeClass parent_class;
	void (*finalize) (BatchImportJob *self);
	char* (*get_identifier) (BatchImportJob* self);
	gboolean (*prepare) (BatchImportJob* self, GFile** file_to_import, gboolean* copy_to_library);
};

struct _AppWindowFileImportJob {
	BatchImportJob parent_instance;
	AppWindowFileImportJobPrivate * priv;
};

struct _AppWindowFileImportJobClass {
	BatchImportJobClass parent_class;
};

struct _AppWindowFileImportJobPrivate {
	GFile* file_or_dir;
	gboolean copy_to_library;
};

struct _Comparator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ComparatorPrivate * priv;
};

struct _ComparatorClass {
	GTypeClass parent_class;
	void (*finalize) (Comparator *self);
	gint64 (*compare) (Comparator* self, gconstpointer a, gconstpointer b);
};

struct _AppWindowCompareEventPage {
	Comparator parent_instance;
	AppWindowCompareEventPagePrivate * priv;
};

struct _AppWindowCompareEventPageClass {
	ComparatorClass parent_class;
};

struct _AppWindowCompareEventPagePrivate {
	gint event_sort;
	EventTable* event_table;
};


static gpointer fullscreen_window_parent_class = NULL;
extern GdkColor app_window_BG_COLOR;
GdkColor app_window_BG_COLOR = {0};
extern GdkColor app_window_SIDEBAR_BG_COLOR;
GdkColor app_window_SIDEBAR_BG_COLOR = {0};
static AppWindow* app_window_instance;
static AppWindow* app_window_instance = NULL;
static char** app_window_args;
static gint app_window_args_length1;
static char** app_window_args = NULL;
static gint app_window_args_length1 = 0;
static gint app_window_args_size = 0;
static gboolean app_window_user_quit;
static gboolean app_window_user_quit = FALSE;
static gboolean app_window_camera_update_scheduled;
static gboolean app_window_camera_update_scheduled = FALSE;
static gpointer app_window_file_import_job_parent_class = NULL;
static gpointer app_window_compare_event_page_parent_class = NULL;
static gpointer app_window_parent_class = NULL;

GType fullscreen_window_get_type (void);
GType page_get_type (void);
#define FULLSCREEN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_FULLSCREEN_WINDOW, FullscreenWindowPrivate))
enum  {
	FULLSCREEN_WINDOW_DUMMY_PROPERTY
};
#define RESOURCES_PIN_TOOLBAR "shotwell-pin-toolbar"
#define FULLSCREEN_WINDOW_TOOLBAR_INVOCATION_MSEC 250
#define FULLSCREEN_WINDOW_TOOLBAR_DISMISSAL_SEC 2
#define FULLSCREEN_WINDOW_TOOLBAR_CHECK_DISMISSAL_MSEC 500
#define FULLSCREEN_WINDOW_TOOLBAR_OPACITY 0.75
static void fullscreen_window_on_close (FullscreenWindow* self);
static void _fullscreen_window_on_close_gtk_action_callback (GtkAction* action, gpointer self);
GFile* resources_get_ui (const char* filename);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
static void _fullscreen_window_on_close_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
GtkToolbar* page_get_toolbar (Page* self);
static gboolean fullscreen_window_on_motion (FullscreenWindow* self, FullscreenWindow* fsw, const GdkEventMotion* event);
static gboolean _fullscreen_window_on_motion_gtk_widget_motion_notify_event (FullscreenWindow* _sender, const GdkEventMotion* event, gpointer self);
static gboolean fullscreen_window_on_key_pressed (FullscreenWindow* self, const GdkEventKey* event);
static gboolean _fullscreen_window_on_key_pressed_gtk_widget_key_press_event (FullscreenWindow* _sender, const GdkEventKey* event, gpointer self);
static gboolean fullscreen_window_on_key_released (FullscreenWindow* self, const GdkEventKey* event);
static gboolean _fullscreen_window_on_key_released_gtk_widget_key_release_event (FullscreenWindow* _sender, const GdkEventKey* event, gpointer self);
static gboolean fullscreen_window_on_configured (FullscreenWindow* self, const GdkEventConfigure* event);
static gboolean _fullscreen_window_on_configured_gtk_widget_configure_event (FullscreenWindow* _sender, const GdkEventConfigure* event, gpointer self);
static void fullscreen_window_invoke_toolbar (FullscreenWindow* self);
void page_switched_to (Page* self);
FullscreenWindow* fullscreen_window_new (Page* page);
FullscreenWindow* fullscreen_window_construct (GType object_type, Page* page);
void page_switching_from (Page* self);
void app_window_end_fullscreen (AppWindow* self);
static gboolean fullscreen_window_is_pointer_in_toolbar (FullscreenWindow* self);
static gboolean fullscreen_window_on_check_toolbar_invocation (FullscreenWindow* self);
static gboolean _fullscreen_window_on_check_toolbar_invocation_gsource_func (gpointer self);
gboolean page_notify_modifier_pressed (Page* self, const GdkEventKey* event);
gboolean page_notify_modifier_released (Page* self, const GdkEventKey* event);
gboolean page_notify_configure_event (Page* self, const GdkEventConfigure* event);
static gboolean fullscreen_window_on_check_toolbar_dismissal (FullscreenWindow* self);
static gboolean _fullscreen_window_on_check_toolbar_dismissal_gsource_func (gpointer self);
static void fullscreen_window_finalize (GObject* obj);
GType checkerboard_page_get_type (void);
GType collection_page_get_type (void);
GType events_directory_page_get_type (void);
GType single_photo_page_get_type (void);
GType photo_page_get_type (void);
GType import_queue_page_get_type (void);
GType event_page_get_type (void);
GType import_page_get_type (void);
gpointer database_table_ref (gpointer instance);
void database_table_unref (gpointer instance);
GParamSpec* param_spec_database_table (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_database_table (GValue* value, gpointer v_object);
gpointer value_get_database_table (const GValue* value);
GType database_table_get_type (void);
GType photo_table_get_type (void);
GType event_table_get_type (void);
GType sidebar_get_type (void);
gpointer sidebar_marker_ref (gpointer instance);
void sidebar_marker_unref (gpointer instance);
GParamSpec* param_spec_sidebar_marker (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_sidebar_marker (GValue* value, gpointer v_object);
gpointer value_get_sidebar_marker (const GValue* value);
GType sidebar_marker_get_type (void);
#define APP_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_APP_WINDOW, AppWindowPrivate))
enum  {
	APP_WINDOW_DUMMY_PROPERTY
};
GdkColor parse_color (const char* spec);
#define APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING 1
PhotoTable* photo_table_new (void);
PhotoTable* photo_table_construct (GType object_type);
EventTable* event_table_new (void);
EventTable* event_table_construct (GType object_type);
Sidebar* sidebar_new (void);
Sidebar* sidebar_construct (GType object_type);
#define APP_WINDOW_SIDEBAR_MIN_WIDTH 160
#define APP_WINDOW_SIDEBAR_MAX_WIDTH 320
#define THUMBNAIL_MAX_SCALE 360
#define COLLECTION_LAYOUT_LEFT_PADDING 16
#define COLLECTION_LAYOUT_RIGHT_PADDING 16
#define APP_WINDOW_PAGE_MIN_WIDTH ((THUMBNAIL_MAX_SCALE + COLLECTION_LAYOUT_LEFT_PADDING) + COLLECTION_LAYOUT_RIGHT_PADDING)
#define APP_WINDOW_EVENT_LULL_SEC ((glong) ((3 * 60) * 60))
#define APP_WINDOW_EVENT_MAX_DURATION_SEC ((glong) ((12 * 60) * 60))
#define APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING 0
#define APP_WINDOW_DATA_DIR ".shotwell"
static void app_window_on_quit (AppWindow* self);
static void _app_window_on_quit_gtk_action_callback (GtkAction* action, gpointer self);
static void app_window_on_about (AppWindow* self);
static void _app_window_on_about_gtk_action_callback (GtkAction* action, gpointer self);
static void app_window_on_fullscreen (AppWindow* self);
static void _app_window_on_fullscreen_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_IMPORT "shotwell-import"
static void app_window_on_file_import (AppWindow* self);
static void _app_window_on_file_import_gtk_action_callback (GtkAction* action, gpointer self);
static void app_window_on_sort_events (AppWindow* self);
static void _app_window_on_sort_events_gtk_action_callback (GtkAction* action, gpointer self);
static void app_window_on_help_contents (AppWindow* self);
static void _app_window_on_help_contents_gtk_action_callback (GtkAction* action, gpointer self);
#define RESOURCES_APP_TITLE "Shotwell"
GdkPixbuf* resources_get_icon (const char* name, gint scale);
#define RESOURCES_ICON_APP "shotwell.svg"
static gboolean app_window_on_key_pressed (AppWindow* self, const GdkEventKey* event);
static gboolean _app_window_on_key_pressed_gtk_widget_key_press_event (AppWindow* _sender, const GdkEventKey* event, gpointer self);
static gboolean app_window_on_key_released (AppWindow* self, const GdkEventKey* event);
static gboolean _app_window_on_key_released_gtk_widget_key_release_event (AppWindow* _sender, const GdkEventKey* event, gpointer self);
static gboolean app_window_on_configure (AppWindow* self, const GdkEventConfigure* event);
static gboolean _app_window_on_configure_gtk_widget_configure_event (AppWindow* _sender, const GdkEventConfigure* event, gpointer self);
CollectionPage* collection_page_new (const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1);
CollectionPage* collection_page_construct (GType object_type, const char* page_name, const char* ui_filename, GtkActionEntry* child_actions, int child_actions_length1);
EventsDirectoryPage* events_directory_page_new (void);
EventsDirectoryPage* events_directory_page_construct (GType object_type);
PhotoPage* photo_page_new (void);
PhotoPage* photo_page_construct (GType object_type);
void photo_page_set_container (PhotoPage* self, GtkWindow* container);
static void app_window_add_parent_page (AppWindow* self, Page* parent);
static void app_window_add_orphan_page (AppWindow* self, Page* orphan);
GType photo_id_get_type (void);
PhotoID* photo_id_dup (const PhotoID* self);
void photo_id_free (PhotoID* self);
GeeArrayList* photo_table_get_photos (PhotoTable* self);
GType photo_get_type (void);
Photo* photo_fetch (const PhotoID* photo_id);
static void app_window_on_photo_removed (AppWindow* self, Photo* photo);
static void _app_window_on_photo_removed_photo_removed (Photo* _sender, gpointer self);
void collection_page_add_photo (CollectionPage* self, Photo* photo);
GType event_id_get_type (void);
EventID* event_id_dup (const EventID* self);
void event_id_free (EventID* self);
GeeArrayList* event_table_get_events (EventTable* self);
static EventPage* app_window_add_event_page (AppWindow* self, const EventID* event_id);
void sidebar_place_cursor (Sidebar* self, Page* page);
static void app_window_on_sidebar_cursor_changed (AppWindow* self);
static void _app_window_on_sidebar_cursor_changed_gtk_tree_view_cursor_changed (Sidebar* _sender, gpointer self);
static void app_window_create_layout (AppWindow* self, Page* start_page);
static void app_window_on_device_added (LibHalContext* context, const char* udi);
static void _app_window_on_device_added_lib_hal_device_added (LibHalContext* ctx, const char* udi);
static void app_window_on_device_removed (LibHalContext* context, const char* udi);
static void _app_window_on_device_removed_lib_hal_device_removed (LibHalContext* ctx, const char* udi);
GQuark gphoto_error_quark (void);
static void app_window_init_camera_table (AppWindow* self, GError** error);
static void app_window_update_camera_table (AppWindow* self, GError** error);
static void app_window_on_about_link (AppWindow* self, GtkAboutDialog* about_dialog, const char* url);
static void _app_window_on_about_link_gtk_about_dialog_activate_link_func (GtkAboutDialog* about, const char* link_, gpointer self);
AppWindow* app_window_new (void);
AppWindow* app_window_construct (GType object_type);
static char** _vala_array_dup1 (char** self, int length);
GFile* app_window_get_data_dir (void);
void app_window_init (char** args, int args_length1);
void app_window_terminate (void);
static char** _vala_array_dup2 (char** self, int length);
char** app_window_get_commandline_args (int* result_length1);
GFile* app_window_get_exec_file (void);
GFile* app_window_get_exec_dir (void);
GFile* app_window_get_photos_dir (void);
GFile* app_window_get_data_subdir (const char* name, const char* subname);
GFile* app_window_get_temp_dir (void);
#define RESOURCES_PREFIX _PREFIX
GFile* app_window_get_resources_dir (void);
void app_window_error_message (const char* message);
static char* app_window_generate_import_failure_list (GeeList* failed);
void app_window_report_import_failures (const char* name, GeeList* failed, GeeList* skipped);
gboolean app_window_has_user_quit (void);
static void app_window_on_events_sort_changed (AppWindow* self);
static void _app_window_on_events_sort_changed_gtk_radio_action_callback (GtkAction* action, GtkAction* current, gpointer self);
GtkActionGroup* app_window_get_common_action_group (AppWindow* self);
#define RESOURCES_APP_VERSION _VERSION
#define RESOURCES_APP_SUBTITLE "Photo Organizer"
#define RESOURCES_COPYRIGHT "Copyright 2009 Yorba Foundation"
#define RESOURCES_YORBA_URL "http://www.yorba.org"
#define RESOURCES_LICENSE "\nShotwell is free software; you can redistribute it and/or modify it under the \nterms of the GNU Lesser General Public License as published by the Free \nSoftware Foundation; either version 2.1 of the License, or (at your option) \nany later version.\n\nShotwell is distributed in the hope that it will be useful, but WITHOUT \nANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS\nFOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for \nmore details.\n\nYou should have received a copy of the GNU Lesser General Public License \nalong with Shotwell; if not, write to the Free Software Foundation, Inc., \n51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA\n"
#define RESOURCES_ICON_ABOUT_LOGO "shotwell-street.jpg"
static void app_window_real_destroy (GtkObject* base);
static void app_window_open_link (AppWindow* self, const char* url);
#define RESOURCES_HELP_URL "http://trac.yorba.org/wiki/Shotwell"
GType layout_item_get_type (void);
GType thumbnail_get_type (void);
LayoutItem* checkerboard_page_get_fullscreen_photo (CheckerboardPage* self);
CheckerboardPage* photo_page_get_controller (PhotoPage* self);
Thumbnail* photo_page_get_thumbnail (PhotoPage* self);
void photo_page_display (PhotoPage* self, CheckerboardPage* controller, Thumbnail* thumbnail);
void app_window_go_fullscreen (AppWindow* self, FullscreenWindow* fullscreen_window);
void page_switching_to_fullscreen (Page* self);
void page_returning_from_fullscreen (Page* self);
static void app_window_dispatch_import_jobs (AppWindow* self, GSList* uris, const char* job_name, gboolean copy_to_library);
static void _g_slist_free_g_free (GSList* self);
gint app_window_get_events_sort (AppWindow* self);
void sidebar_prune_branch_children (Sidebar* self, SidebarMarker* parent);
SidebarMarker* page_get_marker (Page* self);
static AppWindowCompareEventPage* app_window_compare_event_page_new (gint event_sort);
static AppWindowCompareEventPage* app_window_compare_event_page_construct (GType object_type, gint event_sort);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
static GType app_window_compare_event_page_get_type (void);
SidebarMarker* sidebar_insert_child_sorted (Sidebar* self, SidebarMarker* parent, Page* child, Comparator* comparator);
void sidebar_expand_branch (Sidebar* self, SidebarMarker* marker);
void events_directory_page_notify_sort_changed (EventsDirectoryPage* self, gint sort);
void app_window_set_busy_cursor (AppWindow* self);
void app_window_set_normal_cursor (AppWindow* self);
ImportQueuePage* import_queue_page_new (void);
ImportQueuePage* import_queue_page_construct (GType object_type);
static void app_window_remove_import_queue_row (AppWindow* self);
gpointer batch_import_ref (gpointer instance);
void batch_import_unref (gpointer instance);
GParamSpec* param_spec_batch_import (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_batch_import (GValue* value, gpointer v_object);
gpointer value_get_batch_import (const GValue* value);
GType batch_import_get_type (void);
static void _app_window_remove_import_queue_row_import_queue_page_batch_removed (ImportQueuePage* _sender, BatchImport* batch_import, gpointer self);
static void app_window_insert_page_after (AppWindow* self, SidebarMarker* after_marker, Page* page);
void import_queue_page_enqueue_and_schedule (ImportQueuePage* self, BatchImport* batch_import);
void app_window_enqueue_batch_import (AppWindow* self, BatchImport* batch_import);
gpointer batch_import_job_ref (gpointer instance);
void batch_import_job_unref (gpointer instance);
GParamSpec* param_spec_batch_import_job (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_batch_import_job (GValue* value, gpointer v_object);
gpointer value_get_batch_import_job (const GValue* value);
GType batch_import_job_get_type (void);
static GType app_window_file_import_job_get_type (void);
static AppWindowFileImportJob* app_window_file_import_job_new (const char* uri, gboolean copy_to_library);
static AppWindowFileImportJob* app_window_file_import_job_construct (GType object_type, const char* uri, gboolean copy_to_library);
guint64 query_total_file_size (GFile* file_or_dir, GError** error);
BatchImport* batch_import_new (GeeIterable* jobs, const char* name, guint64 total_bytes);
BatchImport* batch_import_construct (GType object_type, GeeIterable* jobs, const char* name, guint64 total_bytes);
void app_window_switch_to_import_queue_page (AppWindow* self);
gint import_queue_page_get_batch_count (ImportQueuePage* self);
static void app_window_remove_page (AppWindow* self, Page* page);
void checkerboard_page_refresh (CheckerboardPage* self);
void app_window_photo_imported (AppWindow* self, Photo* photo);
#define EVENT_ID_INVALID ((gint64) (-1))
void event_id_init (EventID *self, gint64 id);
time_t photo_get_exposure_time (Photo* self);
char* photo_to_string (Photo* self);
EventID photo_get_event_id (Photo* self);
gboolean event_id_is_valid (EventID *self);
gboolean event_table_set_end_time (EventTable* self, const EventID* event_id, const time_t* end_time);
void events_directory_page_add_event (EventsDirectoryPage* self, const EventID* event_id);
EventID event_table_create (EventTable* self, const PhotoID* primary_photo_id, const time_t* start_time);
PhotoID photo_get_photo_id (Photo* self);
void photo_set_event_id (Photo* self, const EventID* event_id);
GType sorted_list_get_type (void);
void app_window_batch_import_complete (AppWindow* self, SortedList* imported_photos);
EventID photo_table_get_event (PhotoTable* self, const PhotoID* photo_id);
PhotoID event_table_get_primary_photo (EventTable* self, const EventID* event_id);
GeeArrayList* photo_table_get_event_photos (PhotoTable* self, const EventID* event_id);
#define PHOTO_ID_INVALID ((gint64) (-1))
void photo_id_init (PhotoID *self, gint64 id);
gboolean photo_id_is_valid (PhotoID *self);
gboolean event_table_set_primary_photo (EventTable* self, const EventID* event_id, const PhotoID* photo_id);
static void app_window_remove_event_page (AppWindow* self, const EventID* event_id);
gboolean event_table_remove (EventTable* self, const EventID* event_id);
static void app_window_real_drag_data_received (GtkWidget* base, GdkDragContext* context, gint x, gint y, GtkSelectionData* selection_data, guint info, guint time);
void app_window_switch_to_page (AppWindow* self, Page* page);
void app_window_switch_to_collection_page (AppWindow* self);
void app_window_switch_to_events_directory_page (AppWindow* self);
EventPage* app_window_find_event_page (AppWindow* self, const EventID* event_id);
void app_window_switch_to_event (AppWindow* self, const EventID* event_id);
void app_window_switch_to_photo_page (AppWindow* self, CheckerboardPage* controller, Thumbnail* current);
EventPage* event_page_new (const EventID* event_id);
EventPage* event_page_construct (GType object_type, const EventID* event_id);
static void app_window_insert_child_page_sorted (AppWindow* self, SidebarMarker* parent_marker, Page* page, Comparator* comparator);
GType page_layout_get_type (void);
PageLayout* page_get_layout (Page* self);
static void app_window_add_to_notebook (AppWindow* self, Page* page);
static gint app_window_get_notebook_pos (AppWindow* self, Page* page);
SidebarMarker* sidebar_add_parent (Sidebar* self, Page* parent);
SidebarMarker* sidebar_add_child (Sidebar* self, SidebarMarker* parent, Page* child);
static void app_window_add_child_page (AppWindow* self, SidebarMarker* parent_marker, Page* child);
SidebarMarker* sidebar_insert_sibling_after (Sidebar* self, SidebarMarker* after, Page* page);
static gboolean app_window_remove_page_internal (AppWindow* self);
static gboolean _app_window_remove_page_internal_gsource_func (gpointer self);
void sidebar_remove_page (Sidebar* self, Page* page);
GtkMenuBar* page_get_menubar (Page* self);
GtkTreeRowReference* sidebar_marker_get_row (SidebarMarker* self);
static gboolean app_window_is_page_selected (AppWindow* self, Page* page, const GtkTreePath* path);
gboolean import_page_is_refreshed (ImportPage* self);
gboolean import_page_is_busy (ImportPage* self);
GType import_page_refresh_result_get_type (void);
ImportPageRefreshResult import_page_refresh_camera (ImportPage* self);
char* import_page_get_uri (ImportPage* self);
void checkerboard_page_set_page_message (CheckerboardPage* self, const char* message);
gboolean import_page_unmount_camera (ImportPage* self, GMount* mount);
char* import_page_get_refresh_message (ImportPage* self);
static gboolean app_window_is_camera_selected (AppWindow* self, const GtkTreePath* path);
static gboolean app_window_is_event_selected (AppWindow* self, const GtkTreePath* path);
static gboolean app_window_focus_on_current_page (AppWindow* self);
static gboolean _app_window_focus_on_current_page_gsource_func (gpointer self);
static void app_window_do_op (AppWindow* self, int res, const char* op, GError** error);
static void _vala_array_add2 (char*** array, int* length, int* size, char* value);
static char* app_window_esp_usb_to_udi (AppWindow* self, gint camera_count, const char* port, char** full_port);
static char* app_window_get_port_uri (const char* port);
Camera* import_page_get_camera (ImportPage* self);
static void _vala_array_add3 (ImportPage*** array, int* length, int* size, ImportPage* value);
ImportPage* import_page_new (Camera* camera, const char* uri);
ImportPage* import_page_construct (GType object_type, Camera* camera, const char* uri);
SidebarMarker* sidebar_insert_grouping_after (Sidebar* self, SidebarMarker* after, const char* name);
void sidebar_prune_branch (Sidebar* self, SidebarMarker* parent);
static void app_window_schedule_camera_update (void);
static gboolean app_window_background_camera_update (void);
static gboolean _app_window_background_camera_update_gsource_func (gpointer self);
gboolean app_window_is_mount_uri_supported (const char* uri);
void import_page_switch_to_and_refresh (ImportPage* self);
void app_window_mounted_camera_shell_notification (const char* uri);
#define APP_WINDOW_FILE_IMPORT_JOB_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), APP_WINDOW_TYPE_FILE_IMPORT_JOB, AppWindowFileImportJobPrivate))
enum  {
	APP_WINDOW_FILE_IMPORT_JOB_DUMMY_PROPERTY
};
static char* app_window_file_import_job_real_get_identifier (BatchImportJob* base);
static gboolean app_window_file_import_job_real_prepare (BatchImportJob* base, GFile** file_to_import, gboolean* copy);
static void app_window_file_import_job_finalize (BatchImportJob* obj);
#define APP_WINDOW_COMPARE_EVENT_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), APP_WINDOW_TYPE_COMPARE_EVENT_PAGE, AppWindowCompareEventPagePrivate))
enum  {
	APP_WINDOW_COMPARE_EVENT_PAGE_DUMMY_PROPERTY
};
time_t event_table_get_start_time (EventTable* self, const EventID* event_id);
static gint64 app_window_compare_event_page_real_compare (Comparator* base, EventPage* a, EventPage* b);
static void app_window_compare_event_page_finalize (Comparator* obj);
static void app_window_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GtkActionEntry FULLSCREEN_WINDOW_ACTIONS[] = {{"LeaveFullscreen", GTK_STOCK_LEAVE_FULLSCREEN, "Leave _Fullscreen", "Escape", "Leave fullscreen", (GCallback) _fullscreen_window_on_close_gtk_action_callback}};
static const char* APP_WINDOW_SUPPORTED_MOUNT_SCHEMES[] = {"gphoto2:", "disk:", "file:"};
static const GtkTargetEntry APP_WINDOW_DEST_TARGET_ENTRIES[] = {{"text/uri-list", (guint) 0, (guint) 0}};
static const GtkActionEntry APP_WINDOW_COMMON_ACTIONS[] = {{"CommonQuit", GTK_STOCK_QUIT, "_Quit", "<Ctrl>Q", "Quit Shotwell", (GCallback) _app_window_on_quit_gtk_action_callback}, {"CommonAbout", GTK_STOCK_ABOUT, "_About", NULL, "About Shotwell", (GCallback) _app_window_on_about_gtk_action_callback}, {"CommonFullscreen", GTK_STOCK_FULLSCREEN, "_Fullscreen", "F11", "Use Shotwell at fullscreen", (GCallback) _app_window_on_fullscreen_gtk_action_callback}, {"CommonFileImport", RESOURCES_IMPORT, "_Import From Folder...", "<Ctrl>I", "Import photos from disk to library", (GCallback) _app_window_on_file_import_gtk_action_callback}, {"CommonSortEvents", NULL, "Sort _Events", NULL, NULL, (GCallback) _app_window_on_sort_events_gtk_action_callback}, {"CommonHelpContents", GTK_STOCK_HELP, "_Contents", "F1", "More informaton on Shotwell", (GCallback) _app_window_on_help_contents_gtk_action_callback}};
static const GtkRadioActionEntry APP_WINDOW_COMMON_SORT_EVENTS_ORDER_ACTIONS[] = {{"CommonSortEventsAscending", GTK_STOCK_SORT_ASCENDING, "_Ascending", NULL, "Sort photos in an ascending order", APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING}, {"CommonSortEventsDescending", GTK_STOCK_SORT_DESCENDING, "D_escending", NULL, "Sort photos in a descending order", APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING}};
static const char* RESOURCES_AUTHORS[] = {"Jim Nelson <jim@yorba.org>", NULL};


#line 89 "AppWindow.vala"
static void _fullscreen_window_on_close_gtk_action_callback (GtkAction* action, gpointer self) {
#line 864 "AppWindow.c"
	fullscreen_window_on_close (self);
}


#line 89 "AppWindow.vala"
static void _fullscreen_window_on_close_gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 871 "AppWindow.c"
	fullscreen_window_on_close (self);
}


#line 98 "AppWindow.vala"
static gboolean _fullscreen_window_on_motion_gtk_widget_motion_notify_event (FullscreenWindow* _sender, const GdkEventMotion* event, gpointer self) {
#line 878 "AppWindow.c"
	return fullscreen_window_on_motion (self, _sender, event);
}


#line 112 "AppWindow.vala"
static gboolean _fullscreen_window_on_key_pressed_gtk_widget_key_press_event (FullscreenWindow* _sender, const GdkEventKey* event, gpointer self) {
#line 885 "AppWindow.c"
	return fullscreen_window_on_key_pressed (self, event);
}


#line 116 "AppWindow.vala"
static gboolean _fullscreen_window_on_key_released_gtk_widget_key_release_event (FullscreenWindow* _sender, const GdkEventKey* event, gpointer self) {
#line 892 "AppWindow.c"
	return fullscreen_window_on_key_released (self, event);
}


#line 120 "AppWindow.vala"
static gboolean _fullscreen_window_on_configured_gtk_widget_configure_event (FullscreenWindow* _sender, const GdkEventConfigure* event, gpointer self) {
#line 899 "AppWindow.c"
	return fullscreen_window_on_configured (self, event);
}


#line 30 "AppWindow.vala"
FullscreenWindow* fullscreen_window_construct (GType object_type, Page* page) {
#line 906 "AppWindow.c"
	GError * _inner_error_;
	FullscreenWindow * self;
	Page* _tmp1_;
	Page* _tmp0_;
	GFile* ui_file;
	GtkActionGroup* action_group;
	GtkAccelGroup* _tmp4_;
	GtkAccelGroup* accel_group;
	AppWindow* _tmp5_;
	GtkToolbar* toolbar;
#line 30 "AppWindow.vala"
	g_return_val_if_fail (IS_PAGE (page), NULL);
#line 919 "AppWindow.c"
	_inner_error_ = NULL;
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
#line 31 "AppWindow.vala"
	_tmp0_ = NULL;
#line 31 "AppWindow.vala"
	self->priv->page = (_tmp1_ = (_tmp0_ = page, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->page == NULL) ? NULL : (self->priv->page = (g_object_unref (self->priv->page), NULL)), _tmp1_);
#line 927 "AppWindow.c"
	ui_file = resources_get_ui ("fullscreen.ui");
	{
		char* _tmp2_;
#line 952 "gio-2.0.vapi"
		_tmp2_ = NULL;
#line 36 "AppWindow.vala"
		gtk_ui_manager_add_ui_from_file (self->priv->ui, _tmp2_ = g_file_get_path (ui_file), &_inner_error_);
#line 935 "AppWindow.c"
		if (_inner_error_ != NULL) {
			goto __catch0_g_error;
			goto __finally0;
		}
		_tmp2_ = (g_free (_tmp2_), NULL);
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp3_;
#line 952 "gio-2.0.vapi"
			_tmp3_ = NULL;
#line 38 "AppWindow.vala"
			g_error ("AppWindow.vala:38: Error loading UI file %s: %s", _tmp3_ = g_file_get_path (ui_file), err->message);
#line 954 "AppWindow.c"
			_tmp3_ = (g_free (_tmp3_), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		(ui_file == NULL) ? NULL : (ui_file = (g_object_unref (ui_file), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	action_group = gtk_action_group_new ("FullscreenActionGroup");
#line 42 "AppWindow.vala"
	gtk_action_group_add_actions (action_group, FULLSCREEN_WINDOW_ACTIONS, G_N_ELEMENTS (FULLSCREEN_WINDOW_ACTIONS), self);
#line 43 "AppWindow.vala"
	gtk_ui_manager_insert_action_group (self->priv->ui, action_group, 0);
#line 44 "AppWindow.vala"
	gtk_ui_manager_ensure_update (self->priv->ui);
#line 46 "AppWindow.vala"
	_tmp4_ = NULL;
#line 975 "AppWindow.c"
	accel_group = (_tmp4_ = gtk_ui_manager_get_accel_group (self->priv->ui), (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
#line 47 "AppWindow.vala"
	if (accel_group != NULL) {
#line 48 "AppWindow.vala"
		gtk_window_add_accel_group (GTK_WINDOW (self), accel_group);
#line 981 "AppWindow.c"
	}
#line 446 "AppWindow.vala"
	_tmp5_ = NULL;
#line 50 "AppWindow.vala"
	gtk_window_set_screen (GTK_WINDOW (self), gtk_window_get_screen (GTK_WINDOW (_tmp5_ = app_window_get_instance ())));
#line 987 "AppWindow.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
#line 51 "AppWindow.vala"
	gtk_container_set_border_width (GTK_CONTAINER (self), (guint) 0);
#line 53 "AppWindow.vala"
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (self->priv->pin_button), "Pin Toolbar");
#line 54 "AppWindow.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->pin_button), "Pin the toolbar open");
#line 57 "AppWindow.vala"
	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (self->priv->close_button), "Leave fullscreen");
#line 58 "AppWindow.vala"
	g_signal_connect_object (self->priv->close_button, "clicked", (GCallback) _fullscreen_window_on_close_gtk_tool_button_clicked, self, 0);
#line 999 "AppWindow.c"
	toolbar = page_get_toolbar (page);
#line 61 "AppWindow.vala"
	gtk_toolbar_set_show_arrow (toolbar, FALSE);
#line 62 "AppWindow.vala"
	gtk_toolbar_insert (toolbar, GTK_TOOL_ITEM (self->priv->pin_button), -1);
#line 63 "AppWindow.vala"
	gtk_toolbar_insert (toolbar, GTK_TOOL_ITEM (self->priv->close_button), -1);
#line 66 "AppWindow.vala"
	gtk_window_set_screen (self->priv->toolbar_window, gtk_window_get_screen (GTK_WINDOW (self)));
#line 67 "AppWindow.vala"
	gtk_container_set_border_width (GTK_CONTAINER (self->priv->toolbar_window), (guint) 0);
#line 68 "AppWindow.vala"
	gtk_container_add (GTK_CONTAINER (self->priv->toolbar_window), GTK_WIDGET (toolbar));
#line 70 "AppWindow.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (page));
#line 73 "AppWindow.vala"
	gtk_window_fullscreen (GTK_WINDOW (self));
#line 74 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 76 "AppWindow.vala"
	gtk_widget_add_events (GTK_WIDGET (self), (gint) (((GDK_POINTER_MOTION_MASK | GDK_KEY_PRESS_MASK) | GDK_KEY_RELEASE_MASK) | GDK_STRUCTURE_MASK));
#line 78 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "motion-notify-event", (GCallback) _fullscreen_window_on_motion_gtk_widget_motion_notify_event, self, 0);
#line 79 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "key-press-event", (GCallback) _fullscreen_window_on_key_pressed_gtk_widget_key_press_event, self, 0);
#line 80 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "key-release-event", (GCallback) _fullscreen_window_on_key_released_gtk_widget_key_release_event, self, 0);
#line 81 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "configure-event", (GCallback) _fullscreen_window_on_configured_gtk_widget_configure_event, self, 0);
#line 84 "AppWindow.vala"
	fullscreen_window_invoke_toolbar (self);
#line 86 "AppWindow.vala"
	page_switched_to (page);
#line 1033 "AppWindow.c"
	(ui_file == NULL) ? NULL : (ui_file = (g_object_unref (ui_file), NULL));
	(action_group == NULL) ? NULL : (action_group = (g_object_unref (action_group), NULL));
	(accel_group == NULL) ? NULL : (accel_group = (g_object_unref (accel_group), NULL));
	(toolbar == NULL) ? NULL : (toolbar = (g_object_unref (toolbar), NULL));
	return self;
}


#line 30 "AppWindow.vala"
FullscreenWindow* fullscreen_window_new (Page* page) {
#line 30 "AppWindow.vala"
	return fullscreen_window_construct (TYPE_FULLSCREEN_WINDOW, page);
#line 1046 "AppWindow.c"
}


#line 89 "AppWindow.vala"
static void fullscreen_window_on_close (FullscreenWindow* self) {
#line 1052 "AppWindow.c"
	GtkWindow* _tmp0_;
	AppWindow* _tmp1_;
#line 89 "AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 90 "AppWindow.vala"
	gtk_widget_hide (GTK_WIDGET (self->priv->toolbar_window));
#line 1059 "AppWindow.c"
	_tmp0_ = NULL;
#line 91 "AppWindow.vala"
	self->priv->toolbar_window = (_tmp0_ = NULL, (self->priv->toolbar_window == NULL) ? NULL : (self->priv->toolbar_window = (g_object_unref (self->priv->toolbar_window), NULL)), _tmp0_);
#line 93 "AppWindow.vala"
	page_switching_from (self->priv->page);
#line 446 "AppWindow.vala"
	_tmp1_ = NULL;
#line 95 "AppWindow.vala"
	app_window_end_fullscreen (_tmp1_ = app_window_get_instance ());
#line 1069 "AppWindow.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
}


#line 138 "AppWindow.vala"
static gboolean _fullscreen_window_on_check_toolbar_invocation_gsource_func (gpointer self) {
#line 1076 "AppWindow.c"
	return fullscreen_window_on_check_toolbar_invocation (self);
}


#line 98 "AppWindow.vala"
static gboolean fullscreen_window_on_motion (FullscreenWindow* self, FullscreenWindow* fsw, const GdkEventMotion* event) {
#line 1083 "AppWindow.c"
	gboolean result;
	gboolean _tmp0_;
#line 98 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 98 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (fsw), FALSE);
#line 99 "AppWindow.vala"
	if (self->priv->is_toolbar_shown) {
#line 1092 "AppWindow.c"
		result = FALSE;
		return result;
	}
	_tmp0_ = FALSE;
#line 104 "AppWindow.vala"
	if (!self->priv->waiting_for_invoke) {
#line 104 "AppWindow.vala"
		_tmp0_ = fullscreen_window_is_pointer_in_toolbar (self);
#line 1101 "AppWindow.c"
	} else {
#line 104 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 1105 "AppWindow.c"
	}
	/* if pointer is in toolbar height range without the mouse down (i.e. in the middle of an edit
	 operation) and it stays there the necessary amount of time, invoke the toolbar*/
#line 104 "AppWindow.vala"
	if (_tmp0_) {
#line 105 "AppWindow.vala"
		g_timeout_add ((guint) FULLSCREEN_WINDOW_TOOLBAR_INVOCATION_MSEC, _fullscreen_window_on_check_toolbar_invocation_gsource_func, self);
#line 106 "AppWindow.vala"
		self->priv->waiting_for_invoke = TRUE;
#line 1115 "AppWindow.c"
	}
	result = FALSE;
	return result;
}


#line 112 "AppWindow.vala"
static gboolean fullscreen_window_on_key_pressed (FullscreenWindow* self, const GdkEventKey* event) {
#line 1124 "AppWindow.c"
	gboolean result;
	gboolean _tmp0_;
#line 112 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 1129 "AppWindow.c"
	_tmp0_ = FALSE;
#line 113 "AppWindow.vala"
	if ((*event).is_modifier != 0) {
#line 113 "AppWindow.vala"
		_tmp0_ = page_notify_modifier_pressed (self->priv->page, &(*event));
#line 1135 "AppWindow.c"
	} else {
#line 113 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 1139 "AppWindow.c"
	}
	result = _tmp0_;
	return result;
}


#line 116 "AppWindow.vala"
static gboolean fullscreen_window_on_key_released (FullscreenWindow* self, const GdkEventKey* event) {
#line 1148 "AppWindow.c"
	gboolean result;
	gboolean _tmp0_;
#line 116 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 1153 "AppWindow.c"
	_tmp0_ = FALSE;
#line 117 "AppWindow.vala"
	if ((*event).is_modifier != 0) {
#line 117 "AppWindow.vala"
		_tmp0_ = page_notify_modifier_released (self->priv->page, &(*event));
#line 1159 "AppWindow.c"
	} else {
#line 117 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 1163 "AppWindow.c"
	}
	result = _tmp0_;
	return result;
}


#line 120 "AppWindow.vala"
static gboolean fullscreen_window_on_configured (FullscreenWindow* self, const GdkEventConfigure* event) {
#line 1172 "AppWindow.c"
	gboolean result;
#line 120 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 1176 "AppWindow.c"
	result = page_notify_configure_event (self->priv->page, &(*event));
	return result;
}


#line 124 "AppWindow.vala"
static gboolean fullscreen_window_is_pointer_in_toolbar (FullscreenWindow* self) {
#line 1184 "AppWindow.c"
	gboolean result;
	gint y;
	gint height;
	gint py;
	GdkModifierType mask;
	GtkRequisition req = {0};
	gboolean _tmp0_;
#line 124 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 1194 "AppWindow.c"
	y = 0;
	height = 0;
#line 126 "AppWindow.vala"
	gdk_window_get_geometry (GTK_WIDGET (self)->window, NULL, &y, NULL, &height, NULL);
#line 1199 "AppWindow.c"
	py = 0;
	mask = 0;
#line 130 "AppWindow.vala"
	gdk_display_get_pointer (gtk_widget_get_display (GTK_WIDGET (self)), NULL, NULL, &py, &mask);
#line 133 "AppWindow.vala"
	gtk_widget_size_request (GTK_WIDGET (self->priv->toolbar_window), &req);
#line 1206 "AppWindow.c"
	_tmp0_ = FALSE;
#line 135 "AppWindow.vala"
	if ((mask & self->priv->ANY_BUTTON_MASK) == 0) {
#line 135 "AppWindow.vala"
		_tmp0_ = py >= ((y + height) - req.height);
#line 1212 "AppWindow.c"
	} else {
#line 135 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 1216 "AppWindow.c"
	}
	result = _tmp0_;
	return result;
}


#line 138 "AppWindow.vala"
static gboolean fullscreen_window_on_check_toolbar_invocation (FullscreenWindow* self) {
#line 1225 "AppWindow.c"
	gboolean result;
#line 138 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 139 "AppWindow.vala"
	self->priv->waiting_for_invoke = FALSE;
#line 141 "AppWindow.vala"
	if (self->priv->is_toolbar_shown) {
#line 1233 "AppWindow.c"
		result = FALSE;
		return result;
	}
#line 144 "AppWindow.vala"
	if (!fullscreen_window_is_pointer_in_toolbar (self)) {
#line 1239 "AppWindow.c"
		result = FALSE;
		return result;
	}
#line 147 "AppWindow.vala"
	fullscreen_window_invoke_toolbar (self);
#line 1245 "AppWindow.c"
	result = FALSE;
	return result;
}


#line 176 "AppWindow.vala"
static gboolean _fullscreen_window_on_check_toolbar_dismissal_gsource_func (gpointer self) {
#line 1253 "AppWindow.c"
	return fullscreen_window_on_check_toolbar_dismissal (self);
}


#line 152 "AppWindow.vala"
static void fullscreen_window_invoke_toolbar (FullscreenWindow* self) {
#line 1260 "AppWindow.c"
	GtkRequisition req = {0};
	GdkScreen* _tmp0_;
	GdkScreen* screen;
	gint tx;
	gint ty;
#line 152 "AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (self));
#line 153 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->toolbar_window));
#line 156 "AppWindow.vala"
	gtk_widget_size_request (GTK_WIDGET (self->priv->toolbar_window), &req);
#line 1272 "AppWindow.c"
	/* place the toolbar in the center of the screen along the bottom edge*/
#line 159 "AppWindow.vala"
	_tmp0_ = NULL;
#line 1276 "AppWindow.c"
	screen = (_tmp0_ = gtk_window_get_screen (self->priv->toolbar_window), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	tx = (gdk_screen_get_width (screen) - req.width) / 2;
#line 161 "AppWindow.vala"
	if (tx < 0) {
#line 162 "AppWindow.vala"
		tx = 0;
#line 1283 "AppWindow.c"
	}
	ty = gdk_screen_get_height (screen) - req.height;
#line 165 "AppWindow.vala"
	if (ty < 0) {
#line 166 "AppWindow.vala"
		ty = 0;
#line 1290 "AppWindow.c"
	}
#line 168 "AppWindow.vala"
	gtk_window_move (self->priv->toolbar_window, tx, ty);
#line 169 "AppWindow.vala"
	gtk_window_set_opacity (self->priv->toolbar_window, FULLSCREEN_WINDOW_TOOLBAR_OPACITY);
#line 171 "AppWindow.vala"
	self->priv->is_toolbar_shown = TRUE;
#line 173 "AppWindow.vala"
	g_timeout_add ((guint) FULLSCREEN_WINDOW_TOOLBAR_CHECK_DISMISSAL_MSEC, _fullscreen_window_on_check_toolbar_dismissal_gsource_func, self);
#line 1300 "AppWindow.c"
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


#line 176 "AppWindow.vala"
static gboolean fullscreen_window_on_check_toolbar_dismissal (FullscreenWindow* self) {
#line 1307 "AppWindow.c"
	gboolean result;
	time_t _tmp1_ = {0};
	time_t now;
#line 176 "AppWindow.vala"
	g_return_val_if_fail (IS_FULLSCREEN_WINDOW (self), FALSE);
#line 177 "AppWindow.vala"
	if (!self->priv->is_toolbar_shown) {
#line 1315 "AppWindow.c"
		result = FALSE;
		return result;
	}
#line 180 "AppWindow.vala"
	if (self->priv->toolbar_window == NULL) {
#line 1321 "AppWindow.c"
		result = FALSE;
		return result;
	}
	/* if pinned, keep open but keep checking*/
#line 184 "AppWindow.vala"
	if (gtk_toggle_tool_button_get_active (self->priv->pin_button)) {
#line 1328 "AppWindow.c"
		result = TRUE;
		return result;
	}
	/* if the pointer is in toolbar range, keep it alive, but keep checking*/
#line 188 "AppWindow.vala"
	if (fullscreen_window_is_pointer_in_toolbar (self)) {
#line 189 "AppWindow.vala"
		self->priv->left_toolbar_time = 0;
#line 1337 "AppWindow.c"
		result = TRUE;
		return result;
	}
	/* if this is the first time noticed, start the timer and keep checking*/
#line 195 "AppWindow.vala"
	if (self->priv->left_toolbar_time == 0) {
#line 1344 "AppWindow.c"
		time_t _tmp0_ = {0};
#line 196 "AppWindow.vala"
		self->priv->left_toolbar_time = (time (&_tmp0_), _tmp0_);
#line 1348 "AppWindow.c"
		result = TRUE;
		return result;
	}
	/* see if enough time has elapsed*/
	now = (time (&_tmp1_), _tmp1_);
#line 203 "AppWindow.vala"
	g_assert (now >= self->priv->left_toolbar_time);
#line 205 "AppWindow.vala"
	if ((now - self->priv->left_toolbar_time) < FULLSCREEN_WINDOW_TOOLBAR_DISMISSAL_SEC) {
#line 1358 "AppWindow.c"
		result = TRUE;
		return result;
	}
#line 208 "AppWindow.vala"
	gtk_widget_hide (GTK_WIDGET (self->priv->toolbar_window));
#line 209 "AppWindow.vala"
	self->priv->is_toolbar_shown = FALSE;
#line 1366 "AppWindow.c"
	result = FALSE;
	return result;
}


static void fullscreen_window_class_init (FullscreenWindowClass * klass) {
	fullscreen_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (FullscreenWindowPrivate));
	G_OBJECT_CLASS (klass)->finalize = fullscreen_window_finalize;
}


static void fullscreen_window_instance_init (FullscreenWindow * self) {
	self->priv = FULLSCREEN_WINDOW_GET_PRIVATE (self);
	self->priv->ANY_BUTTON_MASK = (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK) | GDK_BUTTON3_MASK;
	self->priv->toolbar_window = g_object_ref_sink ((GtkWindow*) gtk_window_new (GTK_WINDOW_POPUP));
	self->priv->ui = gtk_ui_manager_new ();
	self->priv->close_button = g_object_ref_sink ((GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_LEAVE_FULLSCREEN));
	self->priv->pin_button = g_object_ref_sink ((GtkToggleToolButton*) gtk_toggle_tool_button_new_from_stock (RESOURCES_PIN_TOOLBAR));
	self->priv->is_toolbar_shown = FALSE;
	self->priv->waiting_for_invoke = FALSE;
	self->priv->left_toolbar_time = 0;
}


static void fullscreen_window_finalize (GObject* obj) {
	FullscreenWindow * self;
	self = FULLSCREEN_WINDOW (obj);
	(self->priv->toolbar_window == NULL) ? NULL : (self->priv->toolbar_window = (g_object_unref (self->priv->toolbar_window), NULL));
	(self->priv->ui == NULL) ? NULL : (self->priv->ui = (g_object_unref (self->priv->ui), NULL));
	(self->priv->page == NULL) ? NULL : (self->priv->page = (g_object_unref (self->priv->page), NULL));
	(self->priv->close_button == NULL) ? NULL : (self->priv->close_button = (g_object_unref (self->priv->close_button), NULL));
	(self->priv->pin_button == NULL) ? NULL : (self->priv->pin_button = (g_object_unref (self->priv->pin_button), NULL));
	G_OBJECT_CLASS (fullscreen_window_parent_class)->finalize (obj);
}


GType fullscreen_window_get_type (void) {
	static GType fullscreen_window_type_id = 0;
	if (fullscreen_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (FullscreenWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) fullscreen_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FullscreenWindow), 0, (GInstanceInitFunc) fullscreen_window_instance_init, NULL };
		fullscreen_window_type_id = g_type_register_static (GTK_TYPE_WINDOW, "FullscreenWindow", &g_define_type_info, 0);
	}
	return fullscreen_window_type_id;
}


#line 586 "AppWindow.vala"
static void _app_window_on_quit_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1416 "AppWindow.c"
	app_window_on_quit (self);
}


#line 567 "AppWindow.vala"
static void _app_window_on_about_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1423 "AppWindow.c"
	app_window_on_about (self);
}


#line 607 "AppWindow.vala"
static void _app_window_on_fullscreen_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1430 "AppWindow.c"
	app_window_on_fullscreen (self);
}


#line 670 "AppWindow.vala"
static void _app_window_on_file_import_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1437 "AppWindow.c"
	app_window_on_file_import (self);
}


#line 695 "AppWindow.vala"
static void _app_window_on_sort_events_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1444 "AppWindow.c"
	app_window_on_sort_events (self);
}


#line 595 "AppWindow.vala"
static void _app_window_on_help_contents_gtk_action_callback (GtkAction* action, gpointer self) {
#line 1451 "AppWindow.c"
	app_window_on_help_contents (self);
}


#line 868 "AppWindow.vala"
static gboolean _app_window_on_key_pressed_gtk_widget_key_press_event (AppWindow* _sender, const GdkEventKey* event, gpointer self) {
#line 1458 "AppWindow.c"
	return app_window_on_key_pressed (self, event);
}


#line 872 "AppWindow.vala"
static gboolean _app_window_on_key_released_gtk_widget_key_release_event (AppWindow* _sender, const GdkEventKey* event, gpointer self) {
#line 1465 "AppWindow.c"
	return app_window_on_key_released (self, event);
}


#line 876 "AppWindow.vala"
static gboolean _app_window_on_configure_gtk_widget_configure_event (AppWindow* _sender, const GdkEventConfigure* event, gpointer self) {
#line 1472 "AppWindow.c"
	return app_window_on_configure (self, event);
}


#line 880 "AppWindow.vala"
static void _app_window_on_photo_removed_photo_removed (Photo* _sender, gpointer self) {
#line 1479 "AppWindow.c"
	app_window_on_photo_removed (self, _sender);
}


#line 1295 "AppWindow.vala"
static void _app_window_on_sidebar_cursor_changed_gtk_tree_view_cursor_changed (Sidebar* _sender, gpointer self) {
#line 1486 "AppWindow.c"
	app_window_on_sidebar_cursor_changed (self);
}


#line 1538 "AppWindow.vala"
static void _app_window_on_device_added_lib_hal_device_added (LibHalContext* ctx, const char* udi) {
#line 1493 "AppWindow.c"
	app_window_on_device_added (ctx, udi);
}


#line 1544 "AppWindow.vala"
static void _app_window_on_device_removed_lib_hal_device_removed (LibHalContext* ctx, const char* udi) {
#line 1500 "AppWindow.c"
	app_window_on_device_removed (ctx, udi);
}


#line 583 "AppWindow.vala"
static void _app_window_on_about_link_gtk_about_dialog_activate_link_func (GtkAboutDialog* about, const char* link_, gpointer self) {
#line 1507 "AppWindow.c"
	app_window_on_about_link (self, about, link_);
}


#line 349 "AppWindow.vala"
AppWindow* app_window_construct (GType object_type) {
#line 1514 "AppWindow.c"
	GError * _inner_error_;
	AppWindow * self;
	AppWindow* _tmp1_;
	AppWindow* _tmp0_;
	GdkPixbuf* _tmp2_;
	CollectionPage* _tmp3_;
	EventsDirectoryPage* _tmp4_;
	PhotoPage* _tmp5_;
	GeeArrayList* photo_ids;
	GeeArrayList* event_ids;
	DBusGConnection* _tmp11_;
	DBusGConnection* _tmp12_;
	DBusConnection* _tmp13_;
	gboolean _tmp14_;
	DBusError _tmp15_ = {0};
	DBusError raw;
	_inner_error_ = NULL;
	self = g_object_newv (object_type, 0, NULL);
#line 351 "AppWindow.vala"
	g_assert (app_window_instance == NULL);
#line 1535 "AppWindow.c"
	_tmp1_ = NULL;
#line 352 "AppWindow.vala"
	_tmp0_ = NULL;
#line 352 "AppWindow.vala"
	app_window_instance = (_tmp1_ = (_tmp0_ = self, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (app_window_instance == NULL) ? NULL : (app_window_instance = (g_object_unref (app_window_instance), NULL)), _tmp1_);
#line 354 "AppWindow.vala"
	gtk_window_set_title (GTK_WINDOW (self), RESOURCES_APP_TITLE);
#line 355 "AppWindow.vala"
	gtk_window_set_default_size (GTK_WINDOW (self), 1024, 768);
#line 92 "Resources.vala"
	_tmp2_ = NULL;
#line 356 "AppWindow.vala"
	gtk_window_set_default_icon (_tmp2_ = resources_get_icon (RESOURCES_ICON_APP, 24));
#line 1549 "AppWindow.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 359 "AppWindow.vala"
	gtk_widget_add_events (GTK_WIDGET (self), (gint) ((GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK) | GDK_STRUCTURE_MASK));
#line 360 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "key-press-event", (GCallback) _app_window_on_key_pressed_gtk_widget_key_press_event, self, 0);
#line 361 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "key-release-event", (GCallback) _app_window_on_key_released_gtk_widget_key_release_event, self, 0);
#line 362 "AppWindow.vala"
	g_signal_connect_object (GTK_WIDGET (self), "configure-event", (GCallback) _app_window_on_configure_gtk_widget_configure_event, self, 0);
#line 1559 "AppWindow.c"
	_tmp3_ = NULL;
#line 366 "AppWindow.vala"
	self->priv->collection_page = (_tmp3_ = g_object_ref_sink (collection_page_new (NULL, NULL, NULL, 0)), (self->priv->collection_page == NULL) ? NULL : (self->priv->collection_page = (g_object_unref (self->priv->collection_page), NULL)), _tmp3_);
#line 1563 "AppWindow.c"
	_tmp4_ = NULL;
#line 367 "AppWindow.vala"
	self->priv->events_directory_page = (_tmp4_ = g_object_ref_sink (events_directory_page_new ()), (self->priv->events_directory_page == NULL) ? NULL : (self->priv->events_directory_page = (g_object_unref (self->priv->events_directory_page), NULL)), _tmp4_);
#line 1567 "AppWindow.c"
	_tmp5_ = NULL;
#line 368 "AppWindow.vala"
	self->priv->photo_page = (_tmp5_ = g_object_ref_sink (photo_page_new ()), (self->priv->photo_page == NULL) ? NULL : (self->priv->photo_page = (g_object_unref (self->priv->photo_page), NULL)), _tmp5_);
#line 369 "AppWindow.vala"
	photo_page_set_container (self->priv->photo_page, GTK_WINDOW (self));
#line 372 "AppWindow.vala"
	app_window_add_parent_page (self, PAGE (self->priv->collection_page));
#line 373 "AppWindow.vala"
	app_window_add_parent_page (self, PAGE (self->priv->events_directory_page));
#line 374 "AppWindow.vala"
	app_window_add_orphan_page (self, PAGE (self->priv->photo_page));
#line 1579 "AppWindow.c"
	photo_ids = photo_table_get_photos (self->priv->photo_table);
	{
		GeeIterator* _photo_id_it;
		_photo_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (photo_ids));
#line 378 "AppWindow.vala"
		while (TRUE) {
#line 1586 "AppWindow.c"
			PhotoID* _tmp6_;
			PhotoID _tmp7_ = {0};
			PhotoID photo_id;
			Photo* photo;
#line 378 "AppWindow.vala"
			if (!gee_iterator_next (_photo_id_it)) {
#line 378 "AppWindow.vala"
				break;
#line 1595 "AppWindow.c"
			}
#line 377 "AppWindow.vala"
			_tmp6_ = NULL;
#line 1599 "AppWindow.c"
			photo_id = (_tmp7_ = *(_tmp6_ = (PhotoID*) gee_iterator_get (_photo_id_it)), (_tmp6_ == NULL) ? NULL : (_tmp6_ = (photo_id_free (_tmp6_), NULL)), _tmp7_);
			photo = photo_fetch (&photo_id);
#line 380 "AppWindow.vala"
			g_signal_connect_object (photo, "removed", (GCallback) _app_window_on_photo_removed_photo_removed, self, 0);
#line 382 "AppWindow.vala"
			collection_page_add_photo (self->priv->collection_page, photo);
#line 1606 "AppWindow.c"
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_photo_id_it == NULL) ? NULL : (_photo_id_it = (g_object_unref (_photo_id_it), NULL));
	}
	event_ids = event_table_get_events (self->priv->event_table);
	{
		GeeIterator* _event_id_it;
		_event_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (event_ids));
#line 387 "AppWindow.vala"
		while (TRUE) {
#line 1617 "AppWindow.c"
			EventID* _tmp8_;
			EventID _tmp9_ = {0};
			EventID event_id;
			EventPage* _tmp10_;
#line 387 "AppWindow.vala"
			if (!gee_iterator_next (_event_id_it)) {
#line 387 "AppWindow.vala"
				break;
#line 1626 "AppWindow.c"
			}
#line 386 "AppWindow.vala"
			_tmp8_ = NULL;
#line 1630 "AppWindow.c"
			event_id = (_tmp9_ = *(_tmp8_ = (EventID*) gee_iterator_get (_event_id_it)), (_tmp8_ == NULL) ? NULL : (_tmp8_ = (event_id_free (_tmp8_), NULL)), _tmp9_);
#line 1000 "AppWindow.vala"
			_tmp10_ = NULL;
#line 388 "AppWindow.vala"
			_tmp10_ = app_window_add_event_page (self, &event_id);
#line 1636 "AppWindow.c"
			(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
		}
		(_event_id_it == NULL) ? NULL : (_event_id_it = (g_object_unref (_event_id_it), NULL));
	}
#line 391 "AppWindow.vala"
	sidebar_place_cursor (self->priv->sidebar, PAGE (self->priv->collection_page));
#line 392 "AppWindow.vala"
	gtk_tree_view_expand_all (GTK_TREE_VIEW (self->priv->sidebar));
#line 395 "AppWindow.vala"
	g_signal_connect_object (GTK_TREE_VIEW (self->priv->sidebar), "cursor-changed", (GCallback) _app_window_on_sidebar_cursor_changed_gtk_tree_view_cursor_changed, self, 0);
#line 397 "AppWindow.vala"
	app_window_create_layout (self, PAGE (self->priv->collection_page));
#line 401 "AppWindow.vala"
	gtk_drag_dest_set (GTK_WIDGET (self), GTK_DEST_DEFAULT_ALL, APP_WINDOW_DEST_TARGET_ENTRIES, G_N_ELEMENTS (APP_WINDOW_DEST_TARGET_ENTRIES), (GDK_ACTION_COPY | GDK_ACTION_LINK) | GDK_ACTION_ASK);
#line 1651 "AppWindow.c"
	_tmp11_ = dbus_g_bus_get (DBUS_BUS_SYSTEM, &_inner_error_);
	if (_inner_error_ != NULL) {
		(photo_ids == NULL) ? NULL : (photo_ids = (g_object_unref (photo_ids), NULL));
		(event_ids == NULL) ? NULL : (event_ids = (g_object_unref (event_ids), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	_tmp12_ = NULL;
#line 405 "AppWindow.vala"
	self->priv->hal_conn = (_tmp12_ = _tmp11_, (self->priv->hal_conn == NULL) ? NULL : (self->priv->hal_conn = (dbus_g_connection_unref (self->priv->hal_conn), NULL)), _tmp12_);
#line 111 "dbus-glib-1.vapi"
	_tmp13_ = NULL;
#line 406 "AppWindow.vala"
	if ((_tmp14_ = !libhal_ctx_set_dbus_connection (self->priv->hal_context, _tmp13_ = dbus_g_connection_get_connection (self->priv->hal_conn)), (_tmp13_ == NULL) ? NULL : (_tmp13_ = (dbus_connection_unref (_tmp13_), NULL)), _tmp14_)) {
#line 407 "AppWindow.vala"
		g_error ("AppWindow.vala:407: Unable to set DBus connection for HAL");
#line 1669 "AppWindow.c"
	}
	raw = (dbus_error_init (&_tmp15_), _tmp15_);
#line 410 "AppWindow.vala"
	if (!libhal_ctx_init (self->priv->hal_context, &raw)) {
#line 411 "AppWindow.vala"
		g_error ("AppWindow.vala:411: Unable to initialize context: %s", raw.message);
#line 1676 "AppWindow.c"
	}
#line 413 "AppWindow.vala"
	if (!libhal_ctx_set_device_added (self->priv->hal_context, _app_window_on_device_added_lib_hal_device_added)) {
#line 414 "AppWindow.vala"
		g_error ("AppWindow.vala:414: Unable to register device-added callback");
#line 1682 "AppWindow.c"
	}
#line 415 "AppWindow.vala"
	if (!libhal_ctx_set_device_removed (self->priv->hal_context, _app_window_on_device_removed_lib_hal_device_removed)) {
#line 416 "AppWindow.vala"
		g_error ("AppWindow.vala:416: Unable to register device-removed callback");
#line 1688 "AppWindow.c"
	}
	{
#line 419 "AppWindow.vala"
		app_window_init_camera_table (self, &_inner_error_);
#line 1693 "AppWindow.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == GPHOTO_ERROR) {
				goto __catch1_gphoto_error;
			}
			goto __finally1;
		}
#line 420 "AppWindow.vala"
		app_window_update_camera_table (self, &_inner_error_);
#line 1702 "AppWindow.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == GPHOTO_ERROR) {
				goto __catch1_gphoto_error;
			}
			goto __finally1;
		}
	}
	goto __finally1;
	__catch1_gphoto_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 422 "AppWindow.vala"
			g_error ("AppWindow.vala:422: %s", err->message);
#line 1719 "AppWindow.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		(photo_ids == NULL) ? NULL : (photo_ids = (g_object_unref (photo_ids), NULL));
		(event_ids == NULL) ? NULL : (event_ids = (g_object_unref (event_ids), NULL));
		dbus_error_free (&raw);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
#line 425 "AppWindow.vala"
	gtk_about_dialog_set_url_hook (_app_window_on_about_link_gtk_about_dialog_activate_link_func, g_object_ref (self), g_object_unref);
#line 426 "AppWindow.vala"
	gtk_about_dialog_set_email_hook (_app_window_on_about_link_gtk_about_dialog_activate_link_func, g_object_ref (self), g_object_unref);
#line 1736 "AppWindow.c"
	(photo_ids == NULL) ? NULL : (photo_ids = (g_object_unref (photo_ids), NULL));
	(event_ids == NULL) ? NULL : (event_ids = (g_object_unref (event_ids), NULL));
	dbus_error_free (&raw);
	return self;
}


#line 349 "AppWindow.vala"
AppWindow* app_window_new (void) {
#line 349 "AppWindow.vala"
	return app_window_construct (TYPE_APP_WINDOW);
#line 1748 "AppWindow.c"
}


static char** _vala_array_dup1 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp0_;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp0_ = self[i], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	}
	return result;
}


#line 429 "AppWindow.vala"
void app_window_init (char** args, int args_length1) {
#line 1766 "AppWindow.c"
	GError * _inner_error_;
	char** _tmp2_;
	char** _tmp1_;
	const char* _tmp0_;
	GFile* data_dir;
	_inner_error_ = NULL;
	_tmp2_ = NULL;
#line 430 "AppWindow.vala"
	_tmp1_ = NULL;
#line 264 "AppWindow.vala"
	_tmp0_ = NULL;
#line 430 "AppWindow.vala"
	app_window_args = (_tmp2_ = (_tmp1_ = args, (_tmp1_ == NULL) ? ((gpointer) _tmp1_) : _vala_array_dup1 (_tmp1_, args_length1)), app_window_args = (_vala_array_free (app_window_args, app_window_args_length1, (GDestroyNotify) g_free), NULL), app_window_args_length1 = args_length1, app_window_args_size = app_window_args_length1, _tmp2_);
#line 1780 "AppWindow.c"
	data_dir = app_window_get_data_dir ();
	{
#line 434 "AppWindow.vala"
		if (g_file_query_exists (data_dir, NULL) == FALSE) {
#line 1785 "AppWindow.c"
			gboolean _tmp3_;
			_tmp3_ = g_file_make_directory_with_parents (data_dir, NULL, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch2_g_error;
				goto __finally2;
			}
#line 435 "AppWindow.vala"
			if (!_tmp3_) {
#line 1794 "AppWindow.c"
				char* _tmp4_;
#line 952 "gio-2.0.vapi"
				_tmp4_ = NULL;
#line 436 "AppWindow.vala"
				g_error ("AppWindow.vala:436: Unable to create data directory %s", _tmp4_ = g_file_get_path (data_dir));
#line 1800 "AppWindow.c"
				_tmp4_ = (g_free (_tmp4_), NULL);
			}
		}
	}
	goto __finally2;
	__catch2_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 439 "AppWindow.vala"
			g_error ("AppWindow.vala:439: %s", err->message);
#line 1814 "AppWindow.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally2:
	if (_inner_error_ != NULL) {
		(data_dir == NULL) ? NULL : (data_dir = (g_object_unref (data_dir), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(data_dir == NULL) ? NULL : (data_dir = (g_object_unref (data_dir), NULL));
}


#line 443 "AppWindow.vala"
void app_window_terminate (void) {
#line 1831 "AppWindow.c"
}


#line 446 "AppWindow.vala"
AppWindow* app_window_get_instance (void) {
#line 1837 "AppWindow.c"
	AppWindow* result;
	AppWindow* _tmp0_;
#line 447 "AppWindow.vala"
	_tmp0_ = NULL;
#line 1842 "AppWindow.c"
	result = (_tmp0_ = app_window_instance, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


static char** _vala_array_dup2 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp0_;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp0_ = self[i], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	}
	return result;
}


#line 450 "AppWindow.vala"
char** app_window_get_commandline_args (int* result_length1) {
#line 1862 "AppWindow.c"
	char** result;
	char** _tmp1_;
	const char* _tmp0_;
	char** _tmp2_;
#line 451 "AppWindow.vala"
	_tmp1_ = NULL;
#line 450 "AppWindow.vala"
	_tmp0_ = NULL;
#line 451 "AppWindow.vala"
	_tmp2_ = NULL;
#line 1873 "AppWindow.c"
	result = (_tmp2_ = (_tmp1_ = app_window_args, (_tmp1_ == NULL) ? ((gpointer) _tmp1_) : _vala_array_dup2 (_tmp1_, app_window_args_length1)), *result_length1 = app_window_args_length1, _tmp2_);
	return result;
}


#line 454 "AppWindow.vala"
GFile* app_window_get_exec_file (void) {
#line 1881 "AppWindow.c"
	GFile* result;
	char* _tmp0_;
	GFile* _tmp1_;
#line 2009 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 455 "AppWindow.vala"
	_tmp1_ = NULL;
#line 1889 "AppWindow.c"
	result = (_tmp1_ = g_file_new_for_path (_tmp0_ = g_find_program_in_path (app_window_args[0])), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


#line 458 "AppWindow.vala"
GFile* app_window_get_exec_dir (void) {
#line 1897 "AppWindow.c"
	GFile* result;
	GFile* _tmp0_;
	GFile* _tmp1_;
#line 454 "AppWindow.vala"
	_tmp0_ = NULL;
#line 459 "AppWindow.vala"
	_tmp1_ = NULL;
#line 1905 "AppWindow.c"
	result = (_tmp1_ = g_file_get_parent (_tmp0_ = app_window_get_exec_file ()), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	return result;
}


#line 462 "AppWindow.vala"
GFile* app_window_get_data_dir (void) {
#line 1913 "AppWindow.c"
	GFile* result;
	GFile* _tmp0_;
	GFile* _tmp1_;
#line 978 "gio-2.0.vapi"
	_tmp0_ = NULL;
#line 463 "AppWindow.vala"
	_tmp1_ = NULL;
#line 1921 "AppWindow.c"
	result = (_tmp1_ = g_file_get_child (_tmp0_ = g_file_new_for_path (g_get_home_dir ()), APP_WINDOW_DATA_DIR), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	return result;
}


#line 466 "AppWindow.vala"
GFile* app_window_get_photos_dir (void) {
#line 1929 "AppWindow.c"
	GFile* result;
	const char* _tmp0_;
	char* path;
	GFile* _tmp1_;
	GFile* _tmp2_;
#line 467 "AppWindow.vala"
	_tmp0_ = NULL;
#line 1937 "AppWindow.c"
	path = (_tmp0_ = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
#line 468 "AppWindow.vala"
	if (path != NULL) {
#line 1941 "AppWindow.c"
		result = g_file_new_for_path (path);
		path = (g_free (path), NULL);
		return result;
	}
#line 978 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 471 "AppWindow.vala"
	_tmp2_ = NULL;
#line 1950 "AppWindow.c"
	result = (_tmp2_ = g_file_get_child (_tmp1_ = g_file_new_for_path (g_get_home_dir ()), "Pictures"), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
	path = (g_free (path), NULL);
	return result;
	path = (g_free (path), NULL);
}


#line 474 "AppWindow.vala"
GFile* app_window_get_temp_dir (void) {
#line 1960 "AppWindow.c"
	GFile* result;
	/* TODO: I know, I know.  Better ways to locate a temp file.*/
	result = app_window_get_data_subdir ("tmp", NULL);
	return result;
}


#line 479 "AppWindow.vala"
GFile* app_window_get_data_subdir (const char* name, const char* subname) {
#line 1970 "AppWindow.c"
	GFile* result;
	GError * _inner_error_;
	GFile* _tmp0_;
	GFile* _tmp1_;
	GFile* subdir;
#line 479 "AppWindow.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 1978 "AppWindow.c"
	_inner_error_ = NULL;
#line 462 "AppWindow.vala"
	_tmp0_ = NULL;
#line 480 "AppWindow.vala"
	_tmp1_ = NULL;
#line 1984 "AppWindow.c"
	subdir = (_tmp1_ = g_file_get_child (_tmp0_ = app_window_get_data_dir (), name), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 481 "AppWindow.vala"
	if (subname != NULL) {
#line 1988 "AppWindow.c"
		GFile* _tmp2_;
		_tmp2_ = NULL;
#line 482 "AppWindow.vala"
		subdir = (_tmp2_ = g_file_get_child (subdir, subname), (subdir == NULL) ? NULL : (subdir = (g_object_unref (subdir), NULL)), _tmp2_);
#line 1993 "AppWindow.c"
	}
	{
#line 485 "AppWindow.vala"
		if (g_file_query_exists (subdir, NULL) == FALSE) {
#line 1998 "AppWindow.c"
			gboolean _tmp3_;
			_tmp3_ = g_file_make_directory_with_parents (subdir, NULL, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch3_g_error;
				goto __finally3;
			}
#line 486 "AppWindow.vala"
			if (!_tmp3_) {
#line 2007 "AppWindow.c"
				char* _tmp4_;
#line 952 "gio-2.0.vapi"
				_tmp4_ = NULL;
#line 487 "AppWindow.vala"
				g_error ("AppWindow.vala:487: Unable to create data subdirectory %s", _tmp4_ = g_file_get_path (subdir));
#line 2013 "AppWindow.c"
				_tmp4_ = (g_free (_tmp4_), NULL);
			}
		}
	}
	goto __finally3;
	__catch3_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 490 "AppWindow.vala"
			g_error ("AppWindow.vala:490: %s", err->message);
#line 2027 "AppWindow.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally3:
	if (_inner_error_ != NULL) {
		(subdir == NULL) ? NULL : (subdir = (g_object_unref (subdir), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	result = subdir;
	return result;
}


#line 496 "AppWindow.vala"
GFile* app_window_get_resources_dir (void) {
#line 2045 "AppWindow.c"
	GFile* result;
	GFile* exec_dir;
	GFile* prefix_dir;
	exec_dir = app_window_get_exec_dir ();
	prefix_dir = g_file_new_for_path (RESOURCES_PREFIX);
	/* if running in the prefix'd path, the app has been installed and is running from there;
	 use its installed resources; otherwise running locally, so use local resources*/
#line 502 "AppWindow.vala"
	if (g_file_has_prefix (exec_dir, prefix_dir)) {
#line 2055 "AppWindow.c"
		GFile* _tmp0_;
		GFile* _tmp1_;
#line 948 "gio-2.0.vapi"
		_tmp0_ = NULL;
#line 503 "AppWindow.vala"
		_tmp1_ = NULL;
#line 2062 "AppWindow.c"
		result = (_tmp1_ = g_file_get_child (_tmp0_ = g_file_get_child (prefix_dir, "share"), "shotwell"), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
		(exec_dir == NULL) ? NULL : (exec_dir = (g_object_unref (exec_dir), NULL));
		(prefix_dir == NULL) ? NULL : (prefix_dir = (g_object_unref (prefix_dir), NULL));
		return result;
	} else {
		result = app_window_get_exec_dir ();
		(exec_dir == NULL) ? NULL : (exec_dir = (g_object_unref (exec_dir), NULL));
		(prefix_dir == NULL) ? NULL : (prefix_dir = (g_object_unref (prefix_dir), NULL));
		return result;
	}
	(exec_dir == NULL) ? NULL : (exec_dir = (g_object_unref (exec_dir), NULL));
	(prefix_dir == NULL) ? NULL : (prefix_dir = (g_object_unref (prefix_dir), NULL));
}


#line 508 "AppWindow.vala"
void app_window_error_message (const char* message) {
#line 2080 "AppWindow.c"
	AppWindow* _tmp0_;
	GtkMessageDialog* _tmp1_;
	GtkMessageDialog* dialog;
#line 508 "AppWindow.vala"
	g_return_if_fail (message != NULL);
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 509 "AppWindow.vala"
	_tmp1_ = NULL;
#line 2090 "AppWindow.c"
	dialog = (_tmp1_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (GTK_WINDOW (_tmp0_ = app_window_get_instance ()), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", message, NULL)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 511 "AppWindow.vala"
	gtk_window_set_title (GTK_WINDOW (dialog), RESOURCES_APP_TITLE);
#line 512 "AppWindow.vala"
	gtk_dialog_run (GTK_DIALOG (dialog));
#line 513 "AppWindow.vala"
	gtk_object_destroy (GTK_OBJECT (dialog));
#line 2098 "AppWindow.c"
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


#line 516 "AppWindow.vala"
static char* app_window_generate_import_failure_list (GeeList* failed) {
#line 2105 "AppWindow.c"
	char* result;
	char* list;
#line 516 "AppWindow.vala"
	g_return_val_if_fail (GEE_IS_LIST (failed), NULL);
#line 517 "AppWindow.vala"
	if (gee_collection_get_size (GEE_COLLECTION (failed)) == 0) {
#line 2112 "AppWindow.c"
		result = NULL;
		return result;
	}
	list = g_strdup ("");
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 521 "AppWindow.vala"
			while (TRUE) {
#line 2125 "AppWindow.c"
				gboolean _tmp1_;
				char* _tmp4_;
				char* _tmp3_;
				char* _tmp2_;
#line 521 "AppWindow.vala"
				if (!_tmp0_) {
#line 521 "AppWindow.vala"
					ctr++;
#line 2134 "AppWindow.c"
				}
#line 521 "AppWindow.vala"
				_tmp0_ = FALSE;
#line 2138 "AppWindow.c"
				_tmp1_ = FALSE;
#line 521 "AppWindow.vala"
				if (ctr < 4) {
#line 521 "AppWindow.vala"
					_tmp1_ = ctr < gee_collection_get_size (GEE_COLLECTION (failed));
#line 2144 "AppWindow.c"
				} else {
#line 521 "AppWindow.vala"
					_tmp1_ = FALSE;
#line 2148 "AppWindow.c"
				}
#line 521 "AppWindow.vala"
				if (!_tmp1_) {
#line 521 "AppWindow.vala"
					break;
#line 2154 "AppWindow.c"
				}
				_tmp4_ = NULL;
#line 820 "glib-2.0.vapi"
				_tmp3_ = NULL;
#line 516 "AppWindow.vala"
				_tmp2_ = NULL;
#line 522 "AppWindow.vala"
				list = (_tmp4_ = g_strconcat (list, _tmp3_ = g_strdup_printf ("%s\n", _tmp2_ = (char*) gee_list_get (failed, ctr)), NULL), list = (g_free (list), NULL), _tmp4_);
#line 2163 "AppWindow.c"
				_tmp3_ = (g_free (_tmp3_), NULL);
				_tmp2_ = (g_free (_tmp2_), NULL);
			}
		}
	}
#line 524 "AppWindow.vala"
	if (gee_collection_get_size (GEE_COLLECTION (failed)) > 4) {
#line 2171 "AppWindow.c"
		char* _tmp6_;
		char* _tmp5_;
		_tmp6_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp5_ = NULL;
#line 525 "AppWindow.vala"
		list = (_tmp6_ = g_strconcat (list, _tmp5_ = g_strdup_printf ("%d more photo(s) not imported.\n", gee_collection_get_size (GEE_COLLECTION (failed)) - 4), NULL), list = (g_free (list), NULL), _tmp6_);
#line 2179 "AppWindow.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
	}
	result = list;
	return result;
}


#line 530 "AppWindow.vala"
void app_window_report_import_failures (const char* name, GeeList* failed, GeeList* skipped) {
#line 2189 "AppWindow.c"
	char* failed_list;
	char* skipped_list;
	gboolean _tmp0_;
	char* message;
#line 530 "AppWindow.vala"
	g_return_if_fail (name != NULL);
#line 530 "AppWindow.vala"
	g_return_if_fail (GEE_IS_LIST (failed));
#line 530 "AppWindow.vala"
	g_return_if_fail (GEE_IS_LIST (skipped));
#line 2200 "AppWindow.c"
	failed_list = app_window_generate_import_failure_list (failed);
	skipped_list = app_window_generate_import_failure_list (skipped);
	_tmp0_ = FALSE;
#line 535 "AppWindow.vala"
	if (failed_list == NULL) {
#line 535 "AppWindow.vala"
		_tmp0_ = skipped_list == NULL;
#line 2208 "AppWindow.c"
	} else {
#line 535 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 2212 "AppWindow.c"
	}
#line 535 "AppWindow.vala"
	if (_tmp0_) {
#line 2216 "AppWindow.c"
		failed_list = (g_free (failed_list), NULL);
		skipped_list = (g_free (skipped_list), NULL);
		return;
	}
	message = g_strdup_printf ("Import from %s did not complete.\n", name);
#line 540 "AppWindow.vala"
	if (failed_list != NULL) {
#line 2224 "AppWindow.c"
		char* _tmp2_;
		char* _tmp1_;
		char* _tmp3_;
		_tmp2_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 541 "AppWindow.vala"
		message = (_tmp2_ = g_strconcat (message, _tmp1_ = g_strdup_printf ("\n%d photos failed due to error:\n", gee_collection_get_size (GEE_COLLECTION (failed))), NULL), message = (g_free (message), NULL), _tmp2_);
#line 2233 "AppWindow.c"
		_tmp1_ = (g_free (_tmp1_), NULL);
		_tmp3_ = NULL;
#line 542 "AppWindow.vala"
		message = (_tmp3_ = g_strconcat (message, failed_list, NULL), message = (g_free (message), NULL), _tmp3_);
#line 2238 "AppWindow.c"
	}
#line 545 "AppWindow.vala"
	if (skipped_list != NULL) {
#line 2242 "AppWindow.c"
		char* _tmp5_;
		char* _tmp4_;
		char* _tmp6_;
		_tmp5_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp4_ = NULL;
#line 546 "AppWindow.vala"
		message = (_tmp5_ = g_strconcat (message, _tmp4_ = g_strdup_printf ("\n%d photos were skipped:\n", gee_collection_get_size (GEE_COLLECTION (skipped))), NULL), message = (g_free (message), NULL), _tmp5_);
#line 2251 "AppWindow.c"
		_tmp4_ = (g_free (_tmp4_), NULL);
		_tmp6_ = NULL;
#line 547 "AppWindow.vala"
		message = (_tmp6_ = g_strconcat (message, skipped_list, NULL), message = (g_free (message), NULL), _tmp6_);
#line 2256 "AppWindow.c"
	}
#line 550 "AppWindow.vala"
	app_window_error_message (message);
#line 2260 "AppWindow.c"
	failed_list = (g_free (failed_list), NULL);
	skipped_list = (g_free (skipped_list), NULL);
	message = (g_free (message), NULL);
}


#line 553 "AppWindow.vala"
gboolean app_window_has_user_quit (void) {
#line 2269 "AppWindow.c"
	gboolean result;
	result = app_window_user_quit;
	return result;
}


#line 704 "AppWindow.vala"
static void _app_window_on_events_sort_changed_gtk_radio_action_callback (GtkAction* action, GtkAction* current, gpointer self) {
#line 2278 "AppWindow.c"
	app_window_on_events_sort_changed (self);
}


#line 557 "AppWindow.vala"
GtkActionGroup* app_window_get_common_action_group (AppWindow* self) {
#line 2285 "AppWindow.c"
	GtkActionGroup* result;
	GtkActionGroup* action_group;
#line 557 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 2290 "AppWindow.c"
	/* each page gets its own one*/
	action_group = gtk_action_group_new ("CommonActionGroup");
#line 560 "AppWindow.vala"
	gtk_action_group_add_actions (action_group, APP_WINDOW_COMMON_ACTIONS, G_N_ELEMENTS (APP_WINDOW_COMMON_ACTIONS), self);
#line 561 "AppWindow.vala"
	gtk_action_group_add_radio_actions (action_group, APP_WINDOW_COMMON_SORT_EVENTS_ORDER_ACTIONS, G_N_ELEMENTS (APP_WINDOW_COMMON_SORT_EVENTS_ORDER_ACTIONS), APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING, (GCallback) _app_window_on_events_sort_changed_gtk_radio_action_callback, self);
#line 2297 "AppWindow.c"
	result = action_group;
	return result;
}


#line 567 "AppWindow.vala"
static void app_window_on_about (AppWindow* self) {
#line 2305 "AppWindow.c"
	GdkPixbuf* _tmp0_;
#line 567 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 2309 "AppWindow.c"
	/* TODO: More thorough About box*/
#line 92 "Resources.vala"
	_tmp0_ = NULL;
#line 569 "AppWindow.vala"
	gtk_show_about_dialog (GTK_WINDOW (self), "version", RESOURCES_APP_VERSION, "comments", RESOURCES_APP_SUBTITLE, "copyright", RESOURCES_COPYRIGHT, "website", RESOURCES_YORBA_URL, "license", RESOURCES_LICENSE, "website-label", "Visit the Yorba web site", "authors", RESOURCES_AUTHORS, "logo", _tmp0_ = resources_get_icon (RESOURCES_ICON_ABOUT_LOGO, -1), NULL);
#line 2315 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


/* This callback needs to be installed for the links to be active in the About dialog.  However,
 this callback doesn't actually have to do anything in order to activate the URL.*/
#line 583 "AppWindow.vala"
static void app_window_on_about_link (AppWindow* self, GtkAboutDialog* about_dialog, const char* url) {
#line 583 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 583 "AppWindow.vala"
	g_return_if_fail (GTK_IS_ABOUT_DIALOG (about_dialog));
#line 583 "AppWindow.vala"
	g_return_if_fail (url != NULL);
#line 2330 "AppWindow.c"
}


#line 586 "AppWindow.vala"
static void app_window_on_quit (AppWindow* self) {
#line 586 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 587 "AppWindow.vala"
	app_window_user_quit = TRUE;
#line 588 "AppWindow.vala"
	gtk_main_quit ();
#line 2342 "AppWindow.c"
}


#line 591 "AppWindow.vala"
static void app_window_real_destroy (GtkObject* base) {
#line 2348 "AppWindow.c"
	AppWindow * self;
	self = APP_WINDOW (base);
#line 592 "AppWindow.vala"
	app_window_on_quit (self);
#line 2353 "AppWindow.c"
}


#line 595 "AppWindow.vala"
static void app_window_on_help_contents (AppWindow* self) {
#line 595 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 596 "AppWindow.vala"
	app_window_open_link (self, RESOURCES_HELP_URL);
#line 2363 "AppWindow.c"
}


#line 599 "AppWindow.vala"
static void app_window_open_link (AppWindow* self, const char* url) {
#line 2369 "AppWindow.c"
	GError * _inner_error_;
#line 599 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 599 "AppWindow.vala"
	g_return_if_fail (url != NULL);
#line 2375 "AppWindow.c"
	_inner_error_ = NULL;
	{
#line 601 "AppWindow.vala"
		gtk_show_uri (gdk_drawable_get_screen (GDK_DRAWABLE (GTK_WIDGET (self)->window)), url, (guint32) GDK_CURRENT_TIME, &_inner_error_);
#line 2380 "AppWindow.c"
		if (_inner_error_ != NULL) {
			goto __catch4_g_error;
			goto __finally4;
		}
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 603 "AppWindow.vala"
			g_critical ("AppWindow.vala:603: Unable to load URL: %s", err->message);
#line 2395 "AppWindow.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally4:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 607 "AppWindow.vala"
static void app_window_on_fullscreen (AppWindow* self) {
#line 2410 "AppWindow.c"
	CheckerboardPage* controller;
	Thumbnail* start;
	gboolean _tmp6_;
	PhotoPage* fs_photo;
	FullscreenWindow* fs_window;
#line 607 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 2418 "AppWindow.c"
	controller = NULL;
	start = NULL;
#line 611 "AppWindow.vala"
	if (IS_CHECKERBOARD_PAGE (self->priv->current_page)) {
#line 2423 "AppWindow.c"
		LayoutItem* item;
		CheckerboardPage* _tmp1_;
		CheckerboardPage* _tmp0_;
		Thumbnail* _tmp3_;
		Thumbnail* _tmp2_;
		item = checkerboard_page_get_fullscreen_photo (CHECKERBOARD_PAGE (self->priv->current_page));
#line 613 "AppWindow.vala"
		if (item == NULL) {
#line 614 "AppWindow.vala"
			g_message ("AppWindow.vala:614: No fullscreen photo for this view");
#line 2434 "AppWindow.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			(controller == NULL) ? NULL : (controller = (g_object_unref (controller), NULL));
			(start == NULL) ? NULL : (start = (g_object_unref (start), NULL));
			return;
		}
		_tmp1_ = NULL;
#line 619 "AppWindow.vala"
		_tmp0_ = NULL;
#line 619 "AppWindow.vala"
		controller = (_tmp1_ = (_tmp0_ = CHECKERBOARD_PAGE (self->priv->current_page), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (controller == NULL) ? NULL : (controller = (g_object_unref (controller), NULL)), _tmp1_);
#line 2445 "AppWindow.c"
		_tmp3_ = NULL;
#line 620 "AppWindow.vala"
		_tmp2_ = NULL;
#line 620 "AppWindow.vala"
		start = (_tmp3_ = (_tmp2_ = THUMBNAIL (item), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (start == NULL) ? NULL : (start = (g_object_unref (start), NULL)), _tmp3_);
#line 2451 "AppWindow.c"
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	} else {
#line 621 "AppWindow.vala"
		if (IS_PHOTO_PAGE (self->priv->current_page)) {
#line 2456 "AppWindow.c"
			CheckerboardPage* _tmp4_;
			Thumbnail* _tmp5_;
			_tmp4_ = NULL;
#line 622 "AppWindow.vala"
			controller = (_tmp4_ = photo_page_get_controller (PHOTO_PAGE (self->priv->current_page)), (controller == NULL) ? NULL : (controller = (g_object_unref (controller), NULL)), _tmp4_);
#line 2462 "AppWindow.c"
			_tmp5_ = NULL;
#line 623 "AppWindow.vala"
			start = (_tmp5_ = photo_page_get_thumbnail (PHOTO_PAGE (self->priv->current_page)), (start == NULL) ? NULL : (start = (g_object_unref (start), NULL)), _tmp5_);
#line 2466 "AppWindow.c"
		} else {
#line 625 "AppWindow.vala"
			g_message ("AppWindow.vala:625: Unable to present fullscreen view for this page");
#line 2470 "AppWindow.c"
			(controller == NULL) ? NULL : (controller = (g_object_unref (controller), NULL));
			(start == NULL) ? NULL : (start = (g_object_unref (start), NULL));
			return;
		}
	}
	_tmp6_ = FALSE;
#line 630 "AppWindow.vala"
	if (controller == NULL) {
#line 630 "AppWindow.vala"
		_tmp6_ = TRUE;
#line 2481 "AppWindow.c"
	} else {
#line 630 "AppWindow.vala"
		_tmp6_ = start == NULL;
#line 2485 "AppWindow.c"
	}
#line 630 "AppWindow.vala"
	if (_tmp6_) {
#line 2489 "AppWindow.c"
		(controller == NULL) ? NULL : (controller = (g_object_unref (controller), NULL));
		(start == NULL) ? NULL : (start = (g_object_unref (start), NULL));
		return;
	}
	fs_photo = g_object_ref_sink (photo_page_new ());
	fs_window = g_object_ref_sink (fullscreen_window_new (PAGE (fs_photo)));
#line 635 "AppWindow.vala"
	photo_page_set_container (fs_photo, GTK_WINDOW (fs_window));
#line 636 "AppWindow.vala"
	photo_page_display (fs_photo, controller, start);
#line 638 "AppWindow.vala"
	app_window_go_fullscreen (self, fs_window);
#line 2502 "AppWindow.c"
	(controller == NULL) ? NULL : (controller = (g_object_unref (controller), NULL));
	(start == NULL) ? NULL : (start = (g_object_unref (start), NULL));
	(fs_photo == NULL) ? NULL : (fs_photo = (g_object_unref (fs_photo), NULL));
	(fs_window == NULL) ? NULL : (fs_window = (g_object_unref (fs_window), NULL));
}


#line 641 "AppWindow.vala"
void app_window_go_fullscreen (AppWindow* self, FullscreenWindow* fullscreen_window) {
#line 2512 "AppWindow.c"
	FullscreenWindow* _tmp1_;
	FullscreenWindow* _tmp0_;
#line 641 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 641 "AppWindow.vala"
	g_return_if_fail (IS_FULLSCREEN_WINDOW (fullscreen_window));
#line 2519 "AppWindow.c"
	/* if already fullscreen, use that*/
#line 643 "AppWindow.vala"
	if (self->priv->fullscreen_window != NULL) {
#line 644 "AppWindow.vala"
		gtk_window_present (GTK_WINDOW (self->priv->fullscreen_window));
#line 2525 "AppWindow.c"
		return;
	}
#line 649 "AppWindow.vala"
	page_switching_to_fullscreen (self->priv->current_page);
#line 2530 "AppWindow.c"
	_tmp1_ = NULL;
#line 651 "AppWindow.vala"
	_tmp0_ = NULL;
#line 651 "AppWindow.vala"
	self->priv->fullscreen_window = (_tmp1_ = (_tmp0_ = fullscreen_window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->fullscreen_window == NULL) ? NULL : (self->priv->fullscreen_window = (g_object_unref (self->priv->fullscreen_window), NULL)), _tmp1_);
#line 652 "AppWindow.vala"
	gtk_window_present (GTK_WINDOW (self->priv->fullscreen_window));
#line 653 "AppWindow.vala"
	gtk_widget_hide (GTK_WIDGET (self));
#line 2540 "AppWindow.c"
}


#line 656 "AppWindow.vala"
void app_window_end_fullscreen (AppWindow* self) {
#line 2546 "AppWindow.c"
	FullscreenWindow* _tmp0_;
#line 656 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 657 "AppWindow.vala"
	if (self->priv->fullscreen_window == NULL) {
#line 2552 "AppWindow.c"
		return;
	}
#line 660 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 662 "AppWindow.vala"
	gtk_widget_hide (GTK_WIDGET (self->priv->fullscreen_window));
#line 2559 "AppWindow.c"
	_tmp0_ = NULL;
#line 663 "AppWindow.vala"
	self->priv->fullscreen_window = (_tmp0_ = NULL, (self->priv->fullscreen_window == NULL) ? NULL : (self->priv->fullscreen_window = (g_object_unref (self->priv->fullscreen_window), NULL)), _tmp0_);
#line 665 "AppWindow.vala"
	page_returning_from_fullscreen (self->priv->current_page);
#line 667 "AppWindow.vala"
	gtk_window_present (GTK_WINDOW (self));
#line 2567 "AppWindow.c"
}


static void _g_slist_free_g_free (GSList* self) {
	g_slist_foreach (self, (GFunc) g_free, NULL);
	g_slist_free (self);
}


#line 670 "AppWindow.vala"
static void app_window_on_file_import (AppWindow* self) {
#line 2579 "AppWindow.c"
	char* _tmp2_;
	char* _tmp1_;
	GFile* _tmp0_;
	GtkCheckButton* _tmp3_;
	GtkCheckButton* copy_toggle;
	GtkFileChooserDialog* import_dialog;
	gint response;
	char* _tmp5_;
#line 670 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 820 "glib-2.0.vapi"
	_tmp2_ = NULL;
#line 947 "gio-2.0.vapi"
	_tmp1_ = NULL;
#line 466 "AppWindow.vala"
	_tmp0_ = NULL;
#line 671 "AppWindow.vala"
	_tmp3_ = NULL;
#line 2598 "AppWindow.c"
	copy_toggle = (_tmp3_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_mnemonic (_tmp2_ = g_strdup_printf ("_Copy files to %s photo library", _tmp1_ = g_file_get_basename (_tmp0_ = app_window_get_photos_dir ())))), _tmp2_ = (g_free (_tmp2_), NULL), _tmp1_ = (g_free (_tmp1_), NULL), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp3_);
#line 673 "AppWindow.vala"
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (copy_toggle), TRUE);
#line 2602 "AppWindow.c"
	import_dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new ("Import From Folder", NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL));
#line 678 "AppWindow.vala"
	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (import_dialog), TRUE);
#line 679 "AppWindow.vala"
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (import_dialog), self->priv->import_dir);
#line 680 "AppWindow.vala"
	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (import_dialog), GTK_WIDGET (copy_toggle));
#line 2610 "AppWindow.c"
	response = gtk_dialog_run (GTK_DIALOG (import_dialog));
#line 684 "AppWindow.vala"
	if (response == GTK_RESPONSE_OK) {
#line 2614 "AppWindow.c"
		GSList* _tmp4_;
#line 5558 "gtk+-2.0.vapi"
		_tmp4_ = NULL;
#line 685 "AppWindow.vala"
		app_window_dispatch_import_jobs (self, _tmp4_ = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (import_dialog)), "folders", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (copy_toggle)));
#line 2620 "AppWindow.c"
		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (_g_slist_free_g_free (_tmp4_), NULL));
	}
	_tmp5_ = NULL;
#line 687 "AppWindow.vala"
	self->priv->import_dir = (_tmp5_ = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (import_dialog)), self->priv->import_dir = (g_free (self->priv->import_dir), NULL), _tmp5_);
#line 688 "AppWindow.vala"
	gtk_object_destroy (GTK_OBJECT (import_dialog));
#line 2628 "AppWindow.c"
	(copy_toggle == NULL) ? NULL : (copy_toggle = (g_object_unref (copy_toggle), NULL));
	(import_dialog == NULL) ? NULL : (import_dialog = (g_object_unref (import_dialog), NULL));
}


#line 691 "AppWindow.vala"
gint app_window_get_events_sort (AppWindow* self) {
#line 2636 "AppWindow.c"
	gint result;
#line 691 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), 0);
#line 2640 "AppWindow.c"
	result = self->priv->events_sort;
	return result;
}


#line 695 "AppWindow.vala"
static void app_window_on_sort_events (AppWindow* self) {
#line 2648 "AppWindow.c"
	GtkRadioAction* _tmp0_;
	GtkRadioAction* action;
#line 695 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 2653 "AppWindow.c"
	/* any member of the group can be told the current value*/
#line 697 "AppWindow.vala"
	_tmp0_ = NULL;
#line 2657 "AppWindow.c"
	action = (_tmp0_ = GTK_RADIO_ACTION (gtk_action_group_get_action (self->priv->current_page->common_action_group, "CommonSortEventsAscending")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 699 "AppWindow.vala"
	g_assert (action != NULL);
#line 701 "AppWindow.vala"
	gtk_radio_action_set_current_value (action, self->priv->events_sort);
#line 2663 "AppWindow.c"
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
}


#line 704 "AppWindow.vala"
static void app_window_on_events_sort_changed (AppWindow* self) {
#line 2670 "AppWindow.c"
	GtkRadioAction* _tmp0_;
	GtkRadioAction* action;
	gboolean _tmp1_;
	SidebarMarker* _tmp2_;
	AppWindowCompareEventPage* comparator;
	SidebarMarker* _tmp5_;
#line 704 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 2679 "AppWindow.c"
	/* any member of the group knows the value*/
#line 706 "AppWindow.vala"
	_tmp0_ = NULL;
#line 2683 "AppWindow.c"
	action = (_tmp0_ = GTK_RADIO_ACTION (gtk_action_group_get_action (self->priv->current_page->common_action_group, "CommonSortEventsAscending")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 708 "AppWindow.vala"
	g_assert (action != NULL);
#line 710 "AppWindow.vala"
	self->priv->events_sort = gtk_radio_action_get_current_value (action);
#line 2689 "AppWindow.c"
	_tmp1_ = FALSE;
#line 711 "AppWindow.vala"
	if (self->priv->events_sort == APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING) {
#line 711 "AppWindow.vala"
		_tmp1_ = TRUE;
#line 2695 "AppWindow.c"
	} else {
#line 711 "AppWindow.vala"
		_tmp1_ = self->priv->events_sort == APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING;
#line 2699 "AppWindow.c"
	}
#line 711 "AppWindow.vala"
	g_assert (_tmp1_);
#line 2703 "AppWindow.c"
	/* rebuild sidebar with new sorting rules ... start by pruning branch from sidebar
	 (note that this doesn't remove the pages from the notebook object)*/
#line 102 "Page.vala"
	_tmp2_ = NULL;
#line 715 "AppWindow.vala"
	sidebar_prune_branch_children (self->priv->sidebar, _tmp2_ = page_get_marker (PAGE (self->priv->events_directory_page)));
#line 2710 "AppWindow.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (sidebar_marker_unref (_tmp2_), NULL));
	comparator = app_window_compare_event_page_new (self->priv->events_sort);
	/* re-insert each page in the sidebar in the new order ... does not add page
	 to notebook again or create a new layout*/
	{
		GeeIterator* _event_page_it;
		/* re-insert each page in the sidebar in the new order ... does not add page
		 to notebook again or create a new layout*/
		_event_page_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->event_list));
		/* re-insert each page in the sidebar in the new order ... does not add page
		 to notebook again or create a new layout*/
#line 721 "AppWindow.vala"
		while (TRUE) {
#line 2724 "AppWindow.c"
			EventPage* event_page;
			SidebarMarker* _tmp4_;
			SidebarMarker* _tmp3_;
			/* re-insert each page in the sidebar in the new order ... does not add page
			 to notebook again or create a new layout*/
#line 721 "AppWindow.vala"
			if (!gee_iterator_next (_event_page_it)) {
#line 2732 "AppWindow.c"
				/* re-insert each page in the sidebar in the new order ... does not add page
				 to notebook again or create a new layout*/
#line 721 "AppWindow.vala"
				break;
#line 2737 "AppWindow.c"
			}
			/* re-insert each page in the sidebar in the new order ... does not add page
			 to notebook again or create a new layout*/
			event_page = (EventPage*) gee_iterator_get (_event_page_it);
#line 168 "Sidebar.vala"
			_tmp4_ = NULL;
#line 102 "Page.vala"
			_tmp3_ = NULL;
#line 722 "AppWindow.vala"
			_tmp4_ = sidebar_insert_child_sorted (self->priv->sidebar, _tmp3_ = page_get_marker (PAGE (self->priv->events_directory_page)), PAGE (event_page), COMPARATOR (comparator));
#line 2748 "AppWindow.c"
			(_tmp4_ == NULL) ? NULL : (_tmp4_ = (sidebar_marker_unref (_tmp4_), NULL));
			(_tmp3_ == NULL) ? NULL : (_tmp3_ = (sidebar_marker_unref (_tmp3_), NULL));
			(event_page == NULL) ? NULL : (event_page = (g_object_unref (event_page), NULL));
		}
		(_event_page_it == NULL) ? NULL : (_event_page_it = (g_object_unref (_event_page_it), NULL));
	}
	/* pruning will collapse the branch, expand automatically
	 TODO: Only expand if already expanded?*/
#line 102 "Page.vala"
	_tmp5_ = NULL;
#line 726 "AppWindow.vala"
	sidebar_expand_branch (self->priv->sidebar, _tmp5_ = page_get_marker (PAGE (self->priv->events_directory_page)));
#line 2761 "AppWindow.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (sidebar_marker_unref (_tmp5_), NULL));
	/* set the tree cursor to the current page, which might have been lost in the
	 delete/insert*/
#line 730 "AppWindow.vala"
	sidebar_place_cursor (self->priv->sidebar, self->priv->current_page);
#line 2767 "AppWindow.c"
	/* the events directory page needs to know about this*/
#line 733 "AppWindow.vala"
	events_directory_page_notify_sort_changed (self->priv->events_directory_page, self->priv->events_sort);
#line 2771 "AppWindow.c"
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
	(comparator == NULL) ? NULL : (comparator = (comparator_unref (comparator), NULL));
}


#line 736 "AppWindow.vala"
void app_window_set_busy_cursor (AppWindow* self) {
#line 2779 "AppWindow.c"
	GdkCursor* _tmp0_;
#line 736 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 2783 "AppWindow.c"
	_tmp0_ = NULL;
#line 737 "AppWindow.vala"
	gdk_window_set_cursor (GTK_WIDGET (self)->window, _tmp0_ = gdk_cursor_new (GDK_WATCH));
#line 2787 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gdk_cursor_unref (_tmp0_), NULL));
}


#line 740 "AppWindow.vala"
void app_window_set_normal_cursor (AppWindow* self) {
#line 2794 "AppWindow.c"
	GdkCursor* _tmp0_;
#line 740 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 2798 "AppWindow.c"
	_tmp0_ = NULL;
#line 741 "AppWindow.vala"
	gdk_window_set_cursor (GTK_WIDGET (self)->window, _tmp0_ = gdk_cursor_new (GDK_ARROW));
#line 2802 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gdk_cursor_unref (_tmp0_), NULL));
}


#line 777 "AppWindow.vala"
static void _app_window_remove_import_queue_row_import_queue_page_batch_removed (ImportQueuePage* _sender, BatchImport* batch_import, gpointer self) {
#line 2809 "AppWindow.c"
	app_window_remove_import_queue_row (self);
}


#line 744 "AppWindow.vala"
void app_window_enqueue_batch_import (AppWindow* self, BatchImport* batch_import) {
#line 744 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 744 "AppWindow.vala"
	g_return_if_fail (IS_BATCH_IMPORT (batch_import));
#line 745 "AppWindow.vala"
	if (self->priv->import_queue_page == NULL) {
#line 2822 "AppWindow.c"
		ImportQueuePage* _tmp0_;
		SidebarMarker* _tmp1_;
		_tmp0_ = NULL;
#line 746 "AppWindow.vala"
		self->priv->import_queue_page = (_tmp0_ = g_object_ref_sink (import_queue_page_new ()), (self->priv->import_queue_page == NULL) ? NULL : (self->priv->import_queue_page = (g_object_unref (self->priv->import_queue_page), NULL)), _tmp0_);
#line 747 "AppWindow.vala"
		g_signal_connect_object (self->priv->import_queue_page, "batch-removed", (GCallback) _app_window_remove_import_queue_row_import_queue_page_batch_removed, self, 0);
#line 102 "Page.vala"
		_tmp1_ = NULL;
#line 749 "AppWindow.vala"
		app_window_insert_page_after (self, _tmp1_ = page_get_marker (PAGE (self->priv->events_directory_page)), PAGE (self->priv->import_queue_page));
#line 2834 "AppWindow.c"
		(_tmp1_ == NULL) ? NULL : (_tmp1_ = (sidebar_marker_unref (_tmp1_), NULL));
	}
#line 752 "AppWindow.vala"
	import_queue_page_enqueue_and_schedule (self->priv->import_queue_page, batch_import);
#line 2839 "AppWindow.c"
}


#line 755 "AppWindow.vala"
static void app_window_dispatch_import_jobs (AppWindow* self, GSList* uris, const char* job_name, gboolean copy_to_library) {
#line 2845 "AppWindow.c"
	GError * _inner_error_;
	GeeArrayList* jobs;
	guint64 total_bytes;
#line 755 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 755 "AppWindow.vala"
	g_return_if_fail (job_name != NULL);
#line 2853 "AppWindow.c"
	_inner_error_ = NULL;
	jobs = gee_array_list_new (APP_WINDOW_TYPE_FILE_IMPORT_JOB, (GBoxedCopyFunc) batch_import_job_ref, batch_import_job_unref, g_direct_equal);
	total_bytes = (guint64) 0;
	{
		GSList* uri_collection;
		GSList* uri_it;
#line 759 "AppWindow.vala"
		uri_collection = uris;
#line 2862 "AppWindow.c"
		for (uri_it = uri_collection; uri_it != NULL; uri_it = uri_it->next) {
			const char* _tmp4_;
			char* uri;
#line 755 "AppWindow.vala"
			_tmp4_ = NULL;
#line 759 "AppWindow.vala"
			uri = (_tmp4_ = (const char*) uri_it->data, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_));
#line 2870 "AppWindow.c"
			{
				AppWindowFileImportJob* _tmp0_;
				_tmp0_ = NULL;
#line 760 "AppWindow.vala"
				gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (jobs), _tmp0_ = app_window_file_import_job_new (uri, copy_to_library));
#line 2876 "AppWindow.c"
				(_tmp0_ == NULL) ? NULL : (_tmp0_ = (batch_import_job_unref (_tmp0_), NULL));
				{
					GFile* _tmp1_;
					guint64 _tmp2_;
					guint64 _tmp3_;
#line 979 "gio-2.0.vapi"
					_tmp1_ = NULL;
#line 2884 "AppWindow.c"
					_tmp3_ = (_tmp2_ = query_total_file_size (_tmp1_ = g_file_new_for_uri (uri), &_inner_error_), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
					if (_inner_error_ != NULL) {
						goto __catch5_g_error;
						goto __finally5;
					}
#line 763 "AppWindow.vala"
					total_bytes = total_bytes + _tmp3_;
#line 2892 "AppWindow.c"
				}
				goto __finally5;
				__catch5_g_error:
				{
					GError * err;
					err = _inner_error_;
					_inner_error_ = NULL;
					{
#line 765 "AppWindow.vala"
						g_debug ("AppWindow.vala:765: Unable to query filesize of %s: %s", uri, err->message);
#line 2903 "AppWindow.c"
						(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
					}
				}
				__finally5:
				if (_inner_error_ != NULL) {
					uri = (g_free (uri), NULL);
					(jobs == NULL) ? NULL : (jobs = (g_object_unref (jobs), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				uri = (g_free (uri), NULL);
			}
		}
	}
#line 769 "AppWindow.vala"
	if (gee_collection_get_size (GEE_COLLECTION (jobs)) > 0) {
#line 2921 "AppWindow.c"
		BatchImport* batch_import;
		batch_import = batch_import_new (GEE_ITERABLE (jobs), job_name, total_bytes);
#line 771 "AppWindow.vala"
		app_window_enqueue_batch_import (self, batch_import);
#line 772 "AppWindow.vala"
		app_window_switch_to_import_queue_page (self);
#line 2928 "AppWindow.c"
		(batch_import == NULL) ? NULL : (batch_import = (batch_import_unref (batch_import), NULL));
	}
	(jobs == NULL) ? NULL : (jobs = (g_object_unref (jobs), NULL));
}


#line 777 "AppWindow.vala"
static void app_window_remove_import_queue_row (AppWindow* self) {
#line 777 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 778 "AppWindow.vala"
	if (import_queue_page_get_batch_count (self->priv->import_queue_page) == 0) {
#line 2941 "AppWindow.c"
		ImportQueuePage* _tmp0_;
#line 779 "AppWindow.vala"
		app_window_remove_page (self, PAGE (self->priv->import_queue_page));
#line 2945 "AppWindow.c"
		_tmp0_ = NULL;
#line 780 "AppWindow.vala"
		self->priv->import_queue_page = (_tmp0_ = NULL, (self->priv->import_queue_page == NULL) ? NULL : (self->priv->import_queue_page = (g_object_unref (self->priv->import_queue_page), NULL)), _tmp0_);
#line 2949 "AppWindow.c"
	}
}


#line 784 "AppWindow.vala"
void app_window_photo_imported (AppWindow* self, Photo* photo) {
#line 784 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 784 "AppWindow.vala"
	g_return_if_fail (IS_PHOTO (photo));
#line 2960 "AppWindow.c"
	/* want to know when it's removed from the system for cleanup*/
#line 786 "AppWindow.vala"
	g_signal_connect_object (photo, "removed", (GCallback) _app_window_on_photo_removed_photo_removed, self, 0);
#line 2964 "AppWindow.c"
	/* automatically add to the Photos page*/
#line 789 "AppWindow.vala"
	collection_page_add_photo (self->priv->collection_page, photo);
#line 790 "AppWindow.vala"
	checkerboard_page_refresh (CHECKERBOARD_PAGE (self->priv->collection_page));
#line 2970 "AppWindow.c"
}


#line 793 "AppWindow.vala"
void app_window_batch_import_complete (AppWindow* self, SortedList* imported_photos) {
#line 2976 "AppWindow.c"
	time_t last_exposure;
	time_t current_event_start;
	EventID _tmp0_ = {0};
	EventID current_event_id;
	EventPage* current_event_page;
#line 793 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 793 "AppWindow.vala"
	g_return_if_fail (IS_SORTED_LIST (imported_photos));
#line 794 "AppWindow.vala"
	g_debug ("AppWindow.vala:794: Processing imported photos to create events ...");
#line 2988 "AppWindow.c"
	/* walk through photos, splitting into events based on criteria*/
	last_exposure = 0;
	current_event_start = 0;
	current_event_id = (event_id_init (&_tmp0_, EVENT_ID_INVALID), _tmp0_);
	current_event_page = NULL;
	{
		GeeIterator* _photo_it;
		_photo_it = gee_iterable_iterator (GEE_ITERABLE (imported_photos));
#line 801 "AppWindow.vala"
		while (TRUE) {
#line 2999 "AppWindow.c"
			Photo* photo;
			time_t exposure_time;
			EventID _tmp2_ = {0};
			gboolean create_event;
			char* _tmp6_;
#line 801 "AppWindow.vala"
			if (!gee_iterator_next (_photo_it)) {
#line 801 "AppWindow.vala"
				break;
#line 3009 "AppWindow.c"
			}
			photo = (Photo*) gee_iterator_get (_photo_it);
			exposure_time = photo_get_exposure_time (photo);
#line 804 "AppWindow.vala"
			if (exposure_time == 0) {
#line 3015 "AppWindow.c"
				char* _tmp1_;
				/* no time recorded; skip*/
#line 208 "Photo.vala"
				_tmp1_ = NULL;
#line 806 "AppWindow.vala"
				g_debug ("AppWindow.vala:806: Skipping event assignment to %s: No exposure time", _tmp1_ = photo_to_string (photo));
#line 3022 "AppWindow.c"
				_tmp1_ = (g_free (_tmp1_), NULL);
				(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
#line 808 "AppWindow.vala"
				continue;
#line 3027 "AppWindow.c"
			}
#line 811 "AppWindow.vala"
			if (event_id_is_valid ((_tmp2_ = photo_get_event_id (photo), &_tmp2_))) {
#line 3031 "AppWindow.c"
				char* _tmp3_;
				/* already part of an event; skip*/
#line 208 "Photo.vala"
				_tmp3_ = NULL;
#line 813 "AppWindow.vala"
				g_debug ("AppWindow.vala:813: Skipping event assignment to %s: Already part of event %lld", _tmp3_ = photo_to_string (photo), photo_get_event_id (photo).id);
#line 3038 "AppWindow.c"
				_tmp3_ = (g_free (_tmp3_), NULL);
				(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
#line 816 "AppWindow.vala"
				continue;
#line 3043 "AppWindow.c"
			}
			/* see if enough time has elapsed to create a new event, or to store this photo in
			 the current one*/
			create_event = FALSE;
#line 822 "AppWindow.vala"
			if (last_exposure == 0) {
#line 3050 "AppWindow.c"
				/* first photo, start a new event*/
#line 824 "AppWindow.vala"
				create_event = TRUE;
#line 3054 "AppWindow.c"
			} else {
#line 826 "AppWindow.vala"
				g_assert (last_exposure <= exposure_time);
#line 827 "AppWindow.vala"
				g_assert (current_event_start <= exposure_time);
#line 829 "AppWindow.vala"
				if ((exposure_time - last_exposure) >= APP_WINDOW_EVENT_LULL_SEC) {
#line 3062 "AppWindow.c"
					/* enough time has passed between photos to signify a new event*/
#line 831 "AppWindow.vala"
					create_event = TRUE;
#line 3066 "AppWindow.c"
				} else {
#line 832 "AppWindow.vala"
					if ((exposure_time - current_event_start) >= APP_WINDOW_EVENT_MAX_DURATION_SEC) {
#line 3070 "AppWindow.c"
						/* the current event has gone on for too long, stop here and start a new one*/
#line 834 "AppWindow.vala"
						create_event = TRUE;
#line 3074 "AppWindow.c"
					}
				}
			}
#line 838 "AppWindow.vala"
			if (create_event) {
#line 3080 "AppWindow.c"
				PhotoID _tmp4_ = {0};
				EventPage* _tmp5_;
#line 839 "AppWindow.vala"
				if (event_id_is_valid (&current_event_id)) {
#line 840 "AppWindow.vala"
					g_assert (last_exposure != 0);
#line 841 "AppWindow.vala"
					event_table_set_end_time (self->priv->event_table, &current_event_id, &last_exposure);
#line 843 "AppWindow.vala"
					events_directory_page_add_event (self->priv->events_directory_page, &current_event_id);
#line 844 "AppWindow.vala"
					checkerboard_page_refresh (CHECKERBOARD_PAGE (self->priv->events_directory_page));
#line 3093 "AppWindow.c"
				}
#line 847 "AppWindow.vala"
				current_event_start = exposure_time;
#line 848 "AppWindow.vala"
				current_event_id = event_table_create (self->priv->event_table, (_tmp4_ = photo_get_photo_id (photo), &_tmp4_), &current_event_start);
#line 3099 "AppWindow.c"
				_tmp5_ = NULL;
#line 850 "AppWindow.vala"
				current_event_page = (_tmp5_ = app_window_add_event_page (self, &current_event_id), (current_event_page == NULL) ? NULL : (current_event_page = (g_object_unref (current_event_page), NULL)), _tmp5_);
#line 852 "AppWindow.vala"
				g_debug ("AppWindow.vala:852: Created event [%lld]", current_event_id.id);
#line 3105 "AppWindow.c"
			}
#line 855 "AppWindow.vala"
			g_assert (event_id_is_valid (&current_event_id));
#line 208 "Photo.vala"
			_tmp6_ = NULL;
#line 857 "AppWindow.vala"
			g_debug ("AppWindow.vala:857: Adding %s to event %lld (exposure=%ld last_exposure=%ld)", _tmp6_ = photo_to_string (photo), current_event_id.id, exposure_time, last_exposure);
#line 3113 "AppWindow.c"
			_tmp6_ = (g_free (_tmp6_), NULL);
#line 860 "AppWindow.vala"
			photo_set_event_id (photo, &current_event_id);
#line 862 "AppWindow.vala"
			collection_page_add_photo (COLLECTION_PAGE (current_event_page), photo);
#line 864 "AppWindow.vala"
			last_exposure = exposure_time;
#line 3121 "AppWindow.c"
			(photo == NULL) ? NULL : (photo = (g_object_unref (photo), NULL));
		}
		(_photo_it == NULL) ? NULL : (_photo_it = (g_object_unref (_photo_it), NULL));
	}
	(current_event_page == NULL) ? NULL : (current_event_page = (g_object_unref (current_event_page), NULL));
}


#line 868 "AppWindow.vala"
static gboolean app_window_on_key_pressed (AppWindow* self, const GdkEventKey* event) {
#line 3132 "AppWindow.c"
	gboolean result;
	gboolean _tmp0_;
#line 868 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 3137 "AppWindow.c"
	_tmp0_ = FALSE;
#line 869 "AppWindow.vala"
	if ((*event).is_modifier != 0) {
#line 869 "AppWindow.vala"
		_tmp0_ = page_notify_modifier_pressed (self->priv->current_page, &(*event));
#line 3143 "AppWindow.c"
	} else {
#line 869 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 3147 "AppWindow.c"
	}
	result = _tmp0_;
	return result;
}


#line 872 "AppWindow.vala"
static gboolean app_window_on_key_released (AppWindow* self, const GdkEventKey* event) {
#line 3156 "AppWindow.c"
	gboolean result;
	gboolean _tmp0_;
#line 872 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 3161 "AppWindow.c"
	_tmp0_ = FALSE;
#line 873 "AppWindow.vala"
	if ((*event).is_modifier != 0) {
#line 873 "AppWindow.vala"
		_tmp0_ = page_notify_modifier_released (self->priv->current_page, &(*event));
#line 3167 "AppWindow.c"
	} else {
#line 873 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 3171 "AppWindow.c"
	}
	result = _tmp0_;
	return result;
}


#line 876 "AppWindow.vala"
static gboolean app_window_on_configure (AppWindow* self, const GdkEventConfigure* event) {
#line 3180 "AppWindow.c"
	gboolean result;
#line 876 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 3184 "AppWindow.c"
	result = page_notify_configure_event (self->priv->current_page, &(*event));
	return result;
}


#line 880 "AppWindow.vala"
static void app_window_on_photo_removed (AppWindow* self, Photo* photo) {
#line 3192 "AppWindow.c"
	PhotoID photo_id;
	EventID event_id;
	gboolean _tmp0_;
#line 880 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 880 "AppWindow.vala"
	g_return_if_fail (IS_PHOTO (photo));
#line 3200 "AppWindow.c"
	photo_id = photo_get_photo_id (photo);
	/* update event's primary photo if this is the one; remove event if no more photos in it*/
	event_id = photo_table_get_event (self->priv->photo_table, &photo_id);
	_tmp0_ = FALSE;
#line 885 "AppWindow.vala"
	if (event_id_is_valid (&event_id)) {
#line 885 "AppWindow.vala"
		_tmp0_ = event_table_get_primary_photo (self->priv->event_table, &event_id).id == photo_id.id;
#line 3209 "AppWindow.c"
	} else {
#line 885 "AppWindow.vala"
		_tmp0_ = FALSE;
#line 3213 "AppWindow.c"
	}
#line 885 "AppWindow.vala"
	if (_tmp0_) {
#line 3217 "AppWindow.c"
		GeeArrayList* photo_ids;
		PhotoID _tmp1_ = {0};
		PhotoID found;
		photo_ids = photo_table_get_event_photos (self->priv->photo_table, &event_id);
		found = (photo_id_init (&_tmp1_, PHOTO_ID_INVALID), _tmp1_);
		/* TODO: For now, simply selecting the first photo possible*/
		{
			GeeIterator* _id_it;
			/* TODO: For now, simply selecting the first photo possible*/
			_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (photo_ids));
			/* TODO: For now, simply selecting the first photo possible*/
#line 890 "AppWindow.vala"
			while (TRUE) {
#line 3231 "AppWindow.c"
				PhotoID* _tmp2_;
				PhotoID _tmp3_ = {0};
				PhotoID id;
				/* TODO: For now, simply selecting the first photo possible*/
#line 890 "AppWindow.vala"
				if (!gee_iterator_next (_id_it)) {
#line 3238 "AppWindow.c"
					/* TODO: For now, simply selecting the first photo possible*/
#line 890 "AppWindow.vala"
					break;
#line 3242 "AppWindow.c"
				}
				/* TODO: For now, simply selecting the first photo possible*/
#line 886 "AppWindow.vala"
				_tmp2_ = NULL;
#line 3247 "AppWindow.c"
				id = (_tmp3_ = *(_tmp2_ = (PhotoID*) gee_iterator_get (_id_it)), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (photo_id_free (_tmp2_), NULL)), _tmp3_);
#line 891 "AppWindow.vala"
				if (id.id != photo_id.id) {
#line 892 "AppWindow.vala"
					found = id;
#line 894 "AppWindow.vala"
					break;
#line 3255 "AppWindow.c"
				}
			}
			(_id_it == NULL) ? NULL : (_id_it = (g_object_unref (_id_it), NULL));
		}
#line 898 "AppWindow.vala"
		if (photo_id_is_valid (&found)) {
#line 899 "AppWindow.vala"
			event_table_set_primary_photo (self->priv->event_table, &event_id, &found);
#line 3264 "AppWindow.c"
		} else {
			/* this indicates this is the last photo of the event, so no more event*/
#line 902 "AppWindow.vala"
			g_assert (gee_collection_get_size (GEE_COLLECTION (photo_ids)) <= 1);
#line 903 "AppWindow.vala"
			app_window_remove_event_page (self, &event_id);
#line 904 "AppWindow.vala"
			event_table_remove (self->priv->event_table, &event_id);
#line 3273 "AppWindow.c"
		}
		(photo_ids == NULL) ? NULL : (photo_ids = (g_object_unref (photo_ids), NULL));
	}
}


#line 909 "AppWindow.vala"
static void app_window_real_drag_data_received (GtkWidget* base, GdkDragContext* context, gint x, gint y, GtkSelectionData* selection_data, guint info, guint time) {
#line 3282 "AppWindow.c"
	AppWindow * self;
	char** _tmp1_;
	gint uris_array_size;
	gint uris_array_length1;
	char** _tmp0_;
	char** uris_array;
	GSList* uris;
	self = APP_WINDOW (base);
#line 909 "AppWindow.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 909 "AppWindow.vala"
	g_return_if_fail (selection_data != NULL);
#line 3295 "AppWindow.c"
	/* don't accept drops from our own application*/
#line 912 "AppWindow.vala"
	if (gtk_drag_get_source_widget (context) != NULL) {
#line 913 "AppWindow.vala"
		gtk_drag_finish (context, FALSE, FALSE, (guint32) time);
#line 3301 "AppWindow.c"
		return;
	}
#line 918 "AppWindow.vala"
	_tmp1_ = NULL;
#line 3306 "AppWindow.c"
	_tmp0_ = NULL;
	uris_array = (_tmp1_ = _tmp0_ = gtk_selection_data_get_uris (selection_data), uris_array_length1 = _vala_array_length (_tmp0_), uris_array_size = uris_array_length1, _tmp1_);
	uris = NULL;
	{
		char** uri_collection;
		int uri_collection_length1;
		int uri_it;
#line 921 "AppWindow.vala"
		uri_collection = uris_array;
#line 3316 "AppWindow.c"
		uri_collection_length1 = uris_array_length1;
		for (uri_it = 0; uri_it < uris_array_length1; uri_it = uri_it + 1) {
			const char* _tmp3_;
			char* uri;
#line 918 "AppWindow.vala"
			_tmp3_ = NULL;
#line 3323 "AppWindow.c"
			uri = (_tmp3_ = uri_collection[uri_it], (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
			{
				const char* _tmp2_;
#line 922 "AppWindow.vala"
				_tmp2_ = NULL;
#line 922 "AppWindow.vala"
				uris = g_slist_append (uris, (_tmp2_ = uri, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)));
#line 3331 "AppWindow.c"
				uri = (g_free (uri), NULL);
			}
		}
	}
#line 925 "AppWindow.vala"
	if (context->suggested_action == GDK_ACTION_ASK) {
#line 3338 "AppWindow.c"
		char* msg;
		char* _tmp6_;
		char* _tmp5_;
		GFile* _tmp4_;
		AppWindow* _tmp7_;
		GtkMessageDialog* _tmp8_;
		GtkMessageDialog* dialog;
		GtkResponseType _result_;
		msg = g_strdup ("Shotwell can copy or move the photos into your %s directory, or it can " "link to the photos without duplicating them.");
		_tmp6_ = NULL;
#line 947 "gio-2.0.vapi"
		_tmp5_ = NULL;
#line 466 "AppWindow.vala"
		_tmp4_ = NULL;
#line 928 "AppWindow.vala"
		msg = (_tmp6_ = g_strdup_printf (msg, _tmp5_ = g_file_get_basename (_tmp4_ = app_window_get_photos_dir ())), msg = (g_free (msg), NULL), _tmp6_);
#line 3355 "AppWindow.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 446 "AppWindow.vala"
		_tmp7_ = NULL;
#line 930 "AppWindow.vala"
		_tmp8_ = NULL;
#line 3362 "AppWindow.c"
		dialog = (_tmp8_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (GTK_WINDOW (_tmp7_ = app_window_get_instance ()), GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_CANCEL, msg, NULL)), (_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL)), _tmp8_);
#line 933 "AppWindow.vala"
		gtk_dialog_add_button (GTK_DIALOG (dialog), "Copy into Library", (gint) GDK_ACTION_COPY);
#line 934 "AppWindow.vala"
		gtk_dialog_add_button (GTK_DIALOG (dialog), "Create Links", (gint) GDK_ACTION_LINK);
#line 935 "AppWindow.vala"
		gtk_window_set_title (GTK_WINDOW (dialog), "Import to Library");
#line 3370 "AppWindow.c"
		_result_ = (GtkResponseType) gtk_dialog_run (GTK_DIALOG (dialog));
#line 939 "AppWindow.vala"
		gtk_object_destroy (GTK_OBJECT (dialog));
#line 941 "AppWindow.vala"
		switch (_result_) {
#line 3376 "AppWindow.c"
			case GDK_ACTION_COPY:
			case GDK_ACTION_LINK:
			{
#line 944 "AppWindow.vala"
				context->action = (GdkDragAction) _result_;
#line 945 "AppWindow.vala"
				break;
#line 3384 "AppWindow.c"
			}
			default:
			{
#line 949 "AppWindow.vala"
				gtk_drag_finish (context, FALSE, FALSE, (guint32) time);
#line 3390 "AppWindow.c"
				msg = (g_free (msg), NULL);
				(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
				uris_array = (_vala_array_free (uris_array, uris_array_length1, (GDestroyNotify) g_free), NULL);
				(uris == NULL) ? NULL : (uris = (_g_slist_free_g_free (uris), NULL));
				return;
			}
		}
		msg = (g_free (msg), NULL);
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	} else {
		/* use the suggested action*/
#line 955 "AppWindow.vala"
		context->action = context->suggested_action;
#line 3404 "AppWindow.c"
	}
#line 958 "AppWindow.vala"
	app_window_dispatch_import_jobs (self, uris, "drag-and-drop", context->action == GDK_ACTION_COPY);
#line 960 "AppWindow.vala"
	gtk_drag_finish (context, TRUE, FALSE, (guint32) time);
#line 3410 "AppWindow.c"
	uris_array = (_vala_array_free (uris_array, uris_array_length1, (GDestroyNotify) g_free), NULL);
	(uris == NULL) ? NULL : (uris = (_g_slist_free_g_free (uris), NULL));
}


#line 963 "AppWindow.vala"
void app_window_switch_to_collection_page (AppWindow* self) {
#line 963 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 964 "AppWindow.vala"
	app_window_switch_to_page (self, PAGE (self->priv->collection_page));
#line 3422 "AppWindow.c"
}


#line 967 "AppWindow.vala"
void app_window_switch_to_events_directory_page (AppWindow* self) {
#line 967 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 968 "AppWindow.vala"
	app_window_switch_to_page (self, PAGE (self->priv->events_directory_page));
#line 3432 "AppWindow.c"
}


#line 971 "AppWindow.vala"
void app_window_switch_to_event (AppWindow* self, const EventID* event_id) {
#line 3438 "AppWindow.c"
	EventPage* page;
#line 971 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 3442 "AppWindow.c"
	page = app_window_find_event_page (self, &(*event_id));
#line 973 "AppWindow.vala"
	if (page == NULL) {
#line 974 "AppWindow.vala"
		g_debug ("AppWindow.vala:974: Cannot find page for event %lld", (*event_id).id);
#line 3448 "AppWindow.c"
		(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		return;
	}
#line 979 "AppWindow.vala"
	app_window_switch_to_page (self, PAGE (page));
#line 3454 "AppWindow.c"
	(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
}


#line 982 "AppWindow.vala"
void app_window_switch_to_photo_page (AppWindow* self, CheckerboardPage* controller, Thumbnail* current) {
#line 982 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 982 "AppWindow.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (controller));
#line 982 "AppWindow.vala"
	g_return_if_fail (IS_THUMBNAIL (current));
#line 983 "AppWindow.vala"
	photo_page_display (self->priv->photo_page, controller, current);
#line 984 "AppWindow.vala"
	app_window_switch_to_page (self, PAGE (self->priv->photo_page));
#line 3471 "AppWindow.c"
}


#line 987 "AppWindow.vala"
void app_window_switch_to_import_queue_page (AppWindow* self) {
#line 987 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 988 "AppWindow.vala"
	app_window_switch_to_page (self, PAGE (self->priv->import_queue_page));
#line 3481 "AppWindow.c"
}


#line 991 "AppWindow.vala"
EventPage* app_window_find_event_page (AppWindow* self, const EventID* event_id) {
#line 3487 "AppWindow.c"
	EventPage* result;
#line 991 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 3491 "AppWindow.c"
	{
		GeeIterator* _page_it;
		_page_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->event_list));
#line 992 "AppWindow.vala"
		while (TRUE) {
#line 3497 "AppWindow.c"
			EventPage* page;
#line 992 "AppWindow.vala"
			if (!gee_iterator_next (_page_it)) {
#line 992 "AppWindow.vala"
				break;
#line 3503 "AppWindow.c"
			}
			page = (EventPage*) gee_iterator_get (_page_it);
#line 993 "AppWindow.vala"
			if (page->event_id.id == (*event_id).id) {
#line 3508 "AppWindow.c"
				result = page;
				(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
				return result;
			}
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		}
		(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
	}
	result = NULL;
	return result;
}


#line 1000 "AppWindow.vala"
static EventPage* app_window_add_event_page (AppWindow* self, const EventID* event_id) {
#line 3524 "AppWindow.c"
	EventPage* result;
	EventPage* event_page;
	GeeArrayList* photo_ids;
	AppWindowCompareEventPage* _tmp4_;
	SidebarMarker* _tmp3_;
#line 1000 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 3532 "AppWindow.c"
	event_page = g_object_ref_sink (event_page_new (&(*event_id)));
	photo_ids = photo_table_get_event_photos (self->priv->photo_table, &(*event_id));
	{
		GeeIterator* _photo_id_it;
		_photo_id_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (photo_ids));
#line 1004 "AppWindow.vala"
		while (TRUE) {
#line 3540 "AppWindow.c"
			PhotoID* _tmp0_;
			PhotoID _tmp1_ = {0};
			PhotoID photo_id;
			Photo* _tmp2_;
#line 1004 "AppWindow.vala"
			if (!gee_iterator_next (_photo_id_it)) {
#line 1004 "AppWindow.vala"
				break;
#line 3549 "AppWindow.c"
			}
#line 1003 "AppWindow.vala"
			_tmp0_ = NULL;
#line 3553 "AppWindow.c"
			photo_id = (_tmp1_ = *(_tmp0_ = (PhotoID*) gee_iterator_get (_photo_id_it)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (photo_id_free (_tmp0_), NULL)), _tmp1_);
#line 137 "Photo.vala"
			_tmp2_ = NULL;
#line 1005 "AppWindow.vala"
			collection_page_add_photo (COLLECTION_PAGE (event_page), _tmp2_ = photo_fetch (&photo_id));
#line 3559 "AppWindow.c"
			(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
		}
		(_photo_id_it == NULL) ? NULL : (_photo_id_it = (g_object_unref (_photo_id_it), NULL));
	}
	_tmp4_ = NULL;
#line 102 "Page.vala"
	_tmp3_ = NULL;
#line 1007 "AppWindow.vala"
	app_window_insert_child_page_sorted (self, _tmp3_ = page_get_marker (PAGE (self->priv->events_directory_page)), PAGE (event_page), COMPARATOR (_tmp4_ = app_window_compare_event_page_new (app_window_get_events_sort (self))));
#line 3569 "AppWindow.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (comparator_unref (_tmp4_), NULL));
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (sidebar_marker_unref (_tmp3_), NULL));
#line 1009 "AppWindow.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->event_list), event_page);
#line 3574 "AppWindow.c"
	result = event_page;
	(photo_ids == NULL) ? NULL : (photo_ids = (g_object_unref (photo_ids), NULL));
	return result;
}


#line 1014 "AppWindow.vala"
static void app_window_remove_event_page (AppWindow* self, const EventID* event_id) {
#line 3583 "AppWindow.c"
	EventPage* page;
#line 1014 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 3587 "AppWindow.c"
	page = app_window_find_event_page (self, &(*event_id));
#line 1016 "AppWindow.vala"
	g_assert (page != NULL);
#line 1018 "AppWindow.vala"
	app_window_remove_page (self, PAGE (page));
#line 1019 "AppWindow.vala"
	gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->event_list), page);
#line 3595 "AppWindow.c"
	(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
}


#line 1022 "AppWindow.vala"
static void app_window_add_to_notebook (AppWindow* self, Page* page) {
#line 3602 "AppWindow.c"
	PageLayout* _tmp0_;
	gint _tmp1_;
	gint pos;
#line 1022 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1022 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 3610 "AppWindow.c"
	/* need to show all before handing over to notebook*/
#line 1024 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (page));
#line 62 "Page.vala"
	_tmp0_ = NULL;
#line 3616 "AppWindow.c"
	pos = (_tmp1_ = gtk_notebook_append_page (self->priv->notebook, GTK_WIDGET (_tmp0_ = page_get_layout (page)), NULL), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 1027 "AppWindow.vala"
	g_assert (pos >= 0);
#line 3620 "AppWindow.c"
}


#line 1030 "AppWindow.vala"
static gint app_window_get_notebook_pos (AppWindow* self, Page* page) {
#line 3626 "AppWindow.c"
	gint result;
	PageLayout* _tmp0_;
	gint _tmp1_;
	gint pos;
#line 1030 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), 0);
#line 1030 "AppWindow.vala"
	g_return_val_if_fail (IS_PAGE (page), 0);
#line 62 "Page.vala"
	_tmp0_ = NULL;
#line 3637 "AppWindow.c"
	pos = (_tmp1_ = gtk_notebook_page_num (self->priv->notebook, GTK_WIDGET (_tmp0_ = page_get_layout (page))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 1032 "AppWindow.vala"
	g_assert (pos != (-1));
#line 3641 "AppWindow.c"
	result = pos;
	return result;
}


#line 1037 "AppWindow.vala"
static void app_window_add_parent_page (AppWindow* self, Page* parent) {
#line 3649 "AppWindow.c"
	SidebarMarker* _tmp0_;
#line 1037 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1037 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (parent));
#line 1038 "AppWindow.vala"
	app_window_add_to_notebook (self, parent);
#line 96 "Sidebar.vala"
	_tmp0_ = NULL;
#line 1040 "AppWindow.vala"
	_tmp0_ = sidebar_add_parent (self->priv->sidebar, parent);
#line 3661 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL));
#line 1042 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->notebook));
#line 3665 "AppWindow.c"
}


#line 1045 "AppWindow.vala"
static void app_window_add_child_page (AppWindow* self, SidebarMarker* parent_marker, Page* child) {
#line 3671 "AppWindow.c"
	SidebarMarker* _tmp0_;
#line 1045 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1045 "AppWindow.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (parent_marker));
#line 1045 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (child));
#line 1046 "AppWindow.vala"
	app_window_add_to_notebook (self, child);
#line 104 "Sidebar.vala"
	_tmp0_ = NULL;
#line 1048 "AppWindow.vala"
	_tmp0_ = sidebar_add_child (self->priv->sidebar, parent_marker, child);
#line 3685 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL));
#line 1050 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->notebook));
#line 3689 "AppWindow.c"
}


#line 1053 "AppWindow.vala"
static void app_window_insert_page_after (AppWindow* self, SidebarMarker* after_marker, Page* page) {
#line 3695 "AppWindow.c"
	SidebarMarker* _tmp0_;
#line 1053 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1053 "AppWindow.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (after_marker));
#line 1053 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 1054 "AppWindow.vala"
	app_window_add_to_notebook (self, page);
#line 156 "Sidebar.vala"
	_tmp0_ = NULL;
#line 1056 "AppWindow.vala"
	_tmp0_ = sidebar_insert_sibling_after (self->priv->sidebar, after_marker, page);
#line 3709 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL));
#line 1058 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->notebook));
#line 3713 "AppWindow.c"
}


#line 1061 "AppWindow.vala"
static void app_window_insert_child_page_sorted (AppWindow* self, SidebarMarker* parent_marker, Page* page, Comparator* comparator) {
#line 3719 "AppWindow.c"
	SidebarMarker* _tmp0_;
#line 1061 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1061 "AppWindow.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (parent_marker));
#line 1061 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 1061 "AppWindow.vala"
	g_return_if_fail (IS_COMPARATOR (comparator));
#line 1063 "AppWindow.vala"
	app_window_add_to_notebook (self, page);
#line 168 "Sidebar.vala"
	_tmp0_ = NULL;
#line 1065 "AppWindow.vala"
	_tmp0_ = sidebar_insert_child_sorted (self->priv->sidebar, parent_marker, page, comparator);
#line 3735 "AppWindow.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (sidebar_marker_unref (_tmp0_), NULL));
#line 1067 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->notebook));
#line 3739 "AppWindow.c"
}


/* an orphan page is a Page that exists in the notebook (and can therefore be switched to) but
 is not listed in the sidebar*/
#line 1072 "AppWindow.vala"
static void app_window_add_orphan_page (AppWindow* self, Page* orphan) {
#line 1072 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1072 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (orphan));
#line 1073 "AppWindow.vala"
	app_window_add_to_notebook (self, orphan);
#line 1075 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (self->priv->notebook));
#line 3755 "AppWindow.c"
}


#line 1097 "AppWindow.vala"
static gboolean _app_window_remove_page_internal_gsource_func (gpointer self) {
#line 3761 "AppWindow.c"
	return app_window_remove_page_internal (self);
}


#line 1078 "AppWindow.vala"
static void app_window_remove_page (AppWindow* self, Page* page) {
#line 1078 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1078 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 3772 "AppWindow.c"
	/* a handful of pages just don't go away*/
#line 1080 "AppWindow.vala"
	g_assert (page != PAGE (self->priv->collection_page));
#line 1081 "AppWindow.vala"
	g_assert (page != PAGE (self->priv->events_directory_page));
#line 1082 "AppWindow.vala"
	g_assert (page != PAGE (self->priv->photo_page));
#line 3780 "AppWindow.c"
	/* because removing a page while executing inside a signal or from a call from the page
	 itself causes problems (i.e. the page being unref'd beneath its feet), schedule all
	 removals outside of UI event and in Idle handler*/
#line 1087 "AppWindow.vala"
	if (!gee_abstract_collection_contains (GEE_ABSTRACT_COLLECTION (self->priv->pages_to_be_removed), page)) {
#line 1088 "AppWindow.vala"
		gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->pages_to_be_removed), page);
#line 1089 "AppWindow.vala"
		g_idle_add (_app_window_remove_page_internal_gsource_func, self);
#line 3790 "AppWindow.c"
	}
	/* switch away if necessary to collection page (which is always present)*/
#line 1093 "AppWindow.vala"
	if (self->priv->current_page == page) {
#line 1094 "AppWindow.vala"
		app_window_switch_to_collection_page (self);
#line 3797 "AppWindow.c"
	}
}


#line 1097 "AppWindow.vala"
static gboolean app_window_remove_page_internal (AppWindow* self) {
#line 3804 "AppWindow.c"
	gboolean result;
#line 1097 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 1099 "AppWindow.vala"
	while (TRUE) {
#line 3810 "AppWindow.c"
		Page* page;
		gint pos;
#line 1099 "AppWindow.vala"
		if (!(gee_collection_get_size (GEE_COLLECTION (self->priv->pages_to_be_removed)) > 0)) {
#line 1099 "AppWindow.vala"
			break;
#line 3817 "AppWindow.c"
		}
		page = (Page*) gee_abstract_list_get (GEE_ABSTRACT_LIST (self->priv->pages_to_be_removed), 0);
		/* remove from notebook*/
		pos = app_window_get_notebook_pos (self, page);
#line 1104 "AppWindow.vala"
		g_assert (pos >= 0);
#line 1105 "AppWindow.vala"
		gtk_notebook_remove_page (self->priv->notebook, pos);
#line 3826 "AppWindow.c"
		/* remove from sidebar, if present*/
#line 1108 "AppWindow.vala"
		sidebar_remove_page (self->priv->sidebar, page);
#line 1110 "AppWindow.vala"
		gee_abstract_list_remove_at (GEE_ABSTRACT_LIST (self->priv->pages_to_be_removed), 0);
#line 3832 "AppWindow.c"
		(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
	}
	result = FALSE;
	return result;
}


#line 1116 "AppWindow.vala"
static void app_window_create_layout (AppWindow* self, Page* start_page) {
#line 3842 "AppWindow.c"
	GtkScrolledWindow* scrolled_sidebar;
	GtkFrame* left_frame;
	GtkFrame* right_frame;
	GtkHPaned* client_paned;
#line 1116 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1116 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (start_page));
#line 3851 "AppWindow.c"
	/* use a Notebook to hold all the pages, which are switched when a sidebar child is selected*/
#line 1118 "AppWindow.vala"
	gtk_notebook_set_show_tabs (self->priv->notebook, FALSE);
#line 1119 "AppWindow.vala"
	gtk_notebook_set_show_border (self->priv->notebook, FALSE);
#line 1121 "AppWindow.vala"
	gtk_widget_modify_base (GTK_WIDGET (self->priv->sidebar), GTK_STATE_NORMAL, &app_window_SIDEBAR_BG_COLOR);
#line 3859 "AppWindow.c"
	/* put the sidebar in a scrolling window*/
	scrolled_sidebar = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL));
#line 1125 "AppWindow.vala"
	gtk_scrolled_window_set_policy (scrolled_sidebar, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
#line 1126 "AppWindow.vala"
	gtk_container_add (GTK_CONTAINER (scrolled_sidebar), GTK_WIDGET (self->priv->sidebar));
#line 3866 "AppWindow.c"
	/* layout the selection tree to the left of the collection/toolbar box with an adjustable
	 gutter between them, framed for presentation*/
	left_frame = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL));
#line 1131 "AppWindow.vala"
	gtk_container_add (GTK_CONTAINER (left_frame), GTK_WIDGET (scrolled_sidebar));
#line 1132 "AppWindow.vala"
	gtk_frame_set_shadow_type (left_frame, GTK_SHADOW_IN);
#line 3874 "AppWindow.c"
	right_frame = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL));
#line 1135 "AppWindow.vala"
	gtk_container_add (GTK_CONTAINER (right_frame), GTK_WIDGET (self->priv->notebook));
#line 1136 "AppWindow.vala"
	gtk_frame_set_shadow_type (right_frame, GTK_SHADOW_IN);
#line 3880 "AppWindow.c"
	client_paned = g_object_ref_sink ((GtkHPaned*) gtk_hpaned_new ());
#line 1139 "AppWindow.vala"
	gtk_paned_pack1 (GTK_PANED (client_paned), GTK_WIDGET (left_frame), FALSE, FALSE);
#line 1140 "AppWindow.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->priv->sidebar), APP_WINDOW_SIDEBAR_MIN_WIDTH, -1);
#line 1141 "AppWindow.vala"
	gtk_paned_pack2 (GTK_PANED (client_paned), GTK_WIDGET (right_frame), TRUE, FALSE);
#line 3888 "AppWindow.c"
	/* TODO: Calc according to layout's size, to give sidebar a maximum width*/
#line 1143 "AppWindow.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->priv->notebook), APP_WINDOW_PAGE_MIN_WIDTH, -1);
#line 1145 "AppWindow.vala"
	gtk_box_pack_end (self->priv->layout, GTK_WIDGET (client_paned), TRUE, TRUE, (guint) 0);
#line 1147 "AppWindow.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->priv->layout));
#line 1149 "AppWindow.vala"
	app_window_switch_to_page (self, start_page);
#line 3898 "AppWindow.c"
	(scrolled_sidebar == NULL) ? NULL : (scrolled_sidebar = (g_object_unref (scrolled_sidebar), NULL));
	(left_frame == NULL) ? NULL : (left_frame = (g_object_unref (left_frame), NULL));
	(right_frame == NULL) ? NULL : (right_frame = (g_object_unref (right_frame), NULL));
	(client_paned == NULL) ? NULL : (client_paned = (g_object_unref (client_paned), NULL));
}


#line 1152 "AppWindow.vala"
void app_window_switch_to_page (AppWindow* self, Page* page) {
#line 3908 "AppWindow.c"
	gint pos;
	GtkMenuBar* _tmp1_;
	GtkAccelGroup* _tmp2_;
	GtkAccelGroup* accel_group;
	Page* _tmp4_;
	Page* _tmp3_;
#line 1152 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1152 "AppWindow.vala"
	g_return_if_fail (IS_PAGE (page));
#line 1153 "AppWindow.vala"
	if (page == self->priv->current_page) {
#line 3921 "AppWindow.c"
		return;
	}
#line 1156 "AppWindow.vala"
	if (self->priv->current_page != NULL) {
#line 1157 "AppWindow.vala"
		page_switching_from (self->priv->current_page);
#line 1159 "AppWindow.vala"
		gtk_window_remove_accel_group (GTK_WINDOW (self), gtk_ui_manager_get_accel_group (self->priv->current_page->ui));
#line 3930 "AppWindow.c"
	}
	pos = app_window_get_notebook_pos (self, page);
#line 1163 "AppWindow.vala"
	if (pos >= 0) {
#line 1164 "AppWindow.vala"
		gtk_notebook_set_current_page (self->priv->notebook, pos);
#line 3937 "AppWindow.c"
	}
	/* switch menus*/
#line 1167 "AppWindow.vala"
	if (self->priv->current_page != NULL) {
#line 3942 "AppWindow.c"
		GtkMenuBar* _tmp0_;
#line 110 "Page.vala"
		_tmp0_ = NULL;
#line 1168 "AppWindow.vala"
		gtk_container_remove (GTK_CONTAINER (self->priv->layout), GTK_WIDGET (_tmp0_ = page_get_menubar (self->priv->current_page)));
#line 3948 "AppWindow.c"
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	}
#line 110 "Page.vala"
	_tmp1_ = NULL;
#line 1169 "AppWindow.vala"
	gtk_box_pack_start (self->priv->layout, GTK_WIDGET (_tmp1_ = page_get_menubar (page)), FALSE, FALSE, (guint) 0);
#line 3955 "AppWindow.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 1171 "AppWindow.vala"
	_tmp2_ = NULL;
#line 3959 "AppWindow.c"
	accel_group = (_tmp2_ = gtk_ui_manager_get_accel_group (page->ui), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
#line 1172 "AppWindow.vala"
	if (accel_group != NULL) {
#line 1173 "AppWindow.vala"
		gtk_window_add_accel_group (GTK_WINDOW (self), accel_group);
#line 3965 "AppWindow.c"
	}
	/* do this prior to changing selection, as the change will fire a cursor-changed event,
	 which will then call this function again*/
	_tmp4_ = NULL;
#line 1177 "AppWindow.vala"
	_tmp3_ = NULL;
#line 1177 "AppWindow.vala"
	self->priv->current_page = (_tmp4_ = (_tmp3_ = page, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (self->priv->current_page == NULL) ? NULL : (self->priv->current_page = (g_object_unref (self->priv->current_page), NULL)), _tmp4_);
#line 1179 "AppWindow.vala"
	sidebar_place_cursor (self->priv->sidebar, page);
#line 1181 "AppWindow.vala"
	gtk_widget_show_all (GTK_WIDGET (page));
#line 1183 "AppWindow.vala"
	page_switched_to (page);
#line 3980 "AppWindow.c"
	(accel_group == NULL) ? NULL : (accel_group = (g_object_unref (accel_group), NULL));
}


#line 1186 "AppWindow.vala"
static gboolean app_window_is_page_selected (AppWindow* self, Page* page, const GtkTreePath* path) {
#line 3987 "AppWindow.c"
	gboolean result;
	SidebarMarker* marker;
	GtkTreePath* _tmp1_;
	GtkTreeRowReference* _tmp0_;
	gboolean _tmp2_;
#line 1186 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 1186 "AppWindow.vala"
	g_return_val_if_fail (IS_PAGE (page), FALSE);
#line 1186 "AppWindow.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 3999 "AppWindow.c"
	marker = page_get_marker (page);
#line 1188 "AppWindow.vala"
	if (marker == NULL) {
#line 4003 "AppWindow.c"
		result = FALSE;
		(marker == NULL) ? NULL : (marker = (sidebar_marker_unref (marker), NULL));
		return result;
	}
#line 4594 "gtk+-2.0.vapi"
	_tmp1_ = NULL;
#line 14 "Sidebar.vala"
	_tmp0_ = NULL;
#line 4012 "AppWindow.c"
	result = (_tmp2_ = gtk_tree_path_compare (path, _tmp1_ = gtk_tree_row_reference_get_path (_tmp0_ = sidebar_marker_get_row (marker))) == 0, (_tmp1_ == NULL) ? NULL : (_tmp1_ = (gtk_tree_path_free (_tmp1_), NULL)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (gtk_tree_row_reference_free (_tmp0_), NULL)), _tmp2_);
	(marker == NULL) ? NULL : (marker = (sidebar_marker_unref (marker), NULL));
	return result;
	(marker == NULL) ? NULL : (marker = (sidebar_marker_unref (marker), NULL));
}


#line 1194 "AppWindow.vala"
static gboolean app_window_is_camera_selected (AppWindow* self, const GtkTreePath* path) {
#line 4022 "AppWindow.c"
	gboolean result;
	GError * _inner_error_;
#line 1194 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 1194 "AppWindow.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 4029 "AppWindow.c"
	_inner_error_ = NULL;
	{
		GeeCollection* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _page_it;
#line 39 "gee-1.0.vapi"
		_tmp0_ = NULL;
#line 4037 "AppWindow.c"
		_tmp1_ = NULL;
		_page_it = (_tmp1_ = gee_iterable_iterator (GEE_ITERABLE (_tmp0_ = gee_abstract_map_get_values (GEE_ABSTRACT_MAP (self->priv->camera_map)))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 1195 "AppWindow.vala"
		while (TRUE) {
#line 4042 "AppWindow.c"
			ImportPage* page;
			gboolean _tmp2_;
			ImportPageRefreshResult res;
#line 1195 "AppWindow.vala"
			if (!gee_iterator_next (_page_it)) {
#line 1195 "AppWindow.vala"
				break;
#line 4050 "AppWindow.c"
			}
			page = (ImportPage*) gee_iterator_get (_page_it);
#line 1196 "AppWindow.vala"
			if (!app_window_is_page_selected (self, PAGE (page), path)) {
#line 4055 "AppWindow.c"
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
#line 1197 "AppWindow.vala"
				continue;
#line 4059 "AppWindow.c"
			}
#line 1199 "AppWindow.vala"
			app_window_switch_to_page (self, PAGE (page));
#line 4063 "AppWindow.c"
			_tmp2_ = FALSE;
#line 1201 "AppWindow.vala"
			if (import_page_is_refreshed (page)) {
#line 1201 "AppWindow.vala"
				_tmp2_ = TRUE;
#line 4069 "AppWindow.c"
			} else {
#line 1201 "AppWindow.vala"
				_tmp2_ = import_page_is_busy (page);
#line 4073 "AppWindow.c"
			}
#line 1201 "AppWindow.vala"
			if (_tmp2_) {
#line 4077 "AppWindow.c"
				result = TRUE;
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
				return result;
			}
			res = import_page_refresh_camera (page);
#line 1206 "AppWindow.vala"
			switch (res) {
#line 4086 "AppWindow.c"
				case IMPORT_PAGE_REFRESH_RESULT_OK:
				case IMPORT_PAGE_REFRESH_RESULT_BUSY:
				{
					{
					}
#line 1211 "AppWindow.vala"
					break;
#line 4094 "AppWindow.c"
				}
				case IMPORT_PAGE_REFRESH_RESULT_LOCKED:
				{
					{
						char* _tmp3_;
						char* _tmp4_;
						GFile* _tmp5_;
						GFile* uri;
						GMount* mount;
						/* if locked because it's mounted, offer to unmount*/
#line 214 "ImportPage.vala"
						_tmp3_ = NULL;
#line 1215 "AppWindow.vala"
						g_debug ("AppWindow.vala:1215: Checking if %s is mounted ...", _tmp3_ = import_page_get_uri (page));
#line 4109 "AppWindow.c"
						_tmp3_ = (g_free (_tmp3_), NULL);
#line 214 "ImportPage.vala"
						_tmp4_ = NULL;
#line 1217 "AppWindow.vala"
						_tmp5_ = NULL;
#line 4115 "AppWindow.c"
						uri = (_tmp5_ = g_file_new_for_uri (_tmp4_ = import_page_get_uri (page)), _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_);
						mount = NULL;
						{
							GMount* _tmp6_;
							GMount* _tmp7_;
							_tmp6_ = g_file_find_enclosing_mount (uri, NULL, &_inner_error_);
							if (_inner_error_ != NULL) {
								goto __catch6_g_error;
								goto __finally6;
							}
							_tmp7_ = NULL;
#line 1221 "AppWindow.vala"
							mount = (_tmp7_ = _tmp6_, (mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL)), _tmp7_);
#line 4129 "AppWindow.c"
						}
						goto __finally6;
						__catch6_g_error:
						{
							GError * err;
							err = _inner_error_;
							_inner_error_ = NULL;
							{
								(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
							}
						}
						__finally6:
						if (_inner_error_ != NULL) {
							(uri == NULL) ? NULL : (uri = (g_object_unref (uri), NULL));
							(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
							(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
							(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
							g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
							g_clear_error (&_inner_error_);
							return FALSE;
						}
						/* error means not mounted*/
#line 1226 "AppWindow.vala"
						if (mount != NULL) {
#line 4154 "AppWindow.c"
							GtkMessageDialog* dialog;
							gint dialog_res;
							/* it's mounted, offer to unmount for the user*/
							dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (GTK_WINDOW (self), GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "The camera is locked for use as a mounted drive.  " "Shotwell can only access the camera when it's unlocked.  " "Do you want Shotwell to unmount it for you?", NULL));
#line 1234 "AppWindow.vala"
							gtk_window_set_title (GTK_WINDOW (dialog), RESOURCES_APP_TITLE);
#line 4161 "AppWindow.c"
							dialog_res = gtk_dialog_run (GTK_DIALOG (dialog));
#line 1236 "AppWindow.vala"
							gtk_object_destroy (GTK_OBJECT (dialog));
#line 1238 "AppWindow.vala"
							if (dialog_res != GTK_RESPONSE_YES) {
#line 1239 "AppWindow.vala"
								checkerboard_page_set_page_message (CHECKERBOARD_PAGE (page), "Please unmount the camera.");
#line 1240 "AppWindow.vala"
								checkerboard_page_refresh (CHECKERBOARD_PAGE (page));
#line 4171 "AppWindow.c"
							} else {
#line 1242 "AppWindow.vala"
								import_page_unmount_camera (page, mount);
#line 4175 "AppWindow.c"
							}
							(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
						} else {
							GtkMessageDialog* dialog;
							/* it's not mounted, so another application must have it locked*/
							dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (GTK_WINDOW (self), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "The camera is locked by another application.  " "Shotwell can only access the camera when it's unlocked.  " "Please close any other application using the camera and try again.", NULL));
#line 1252 "AppWindow.vala"
							gtk_window_set_title (GTK_WINDOW (dialog), RESOURCES_APP_TITLE);
#line 1253 "AppWindow.vala"
							gtk_dialog_run (GTK_DIALOG (dialog));
#line 1254 "AppWindow.vala"
							gtk_object_destroy (GTK_OBJECT (dialog));
#line 1256 "AppWindow.vala"
							checkerboard_page_set_page_message (CHECKERBOARD_PAGE (page), "Please close any other application using the camera.");
#line 1257 "AppWindow.vala"
							checkerboard_page_refresh (CHECKERBOARD_PAGE (page));
#line 4192 "AppWindow.c"
							(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
						}
						(uri == NULL) ? NULL : (uri = (g_object_unref (uri), NULL));
						(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
					}
#line 1259 "AppWindow.vala"
					break;
#line 4200 "AppWindow.c"
				}
				case IMPORT_PAGE_REFRESH_RESULT_LIBRARY_ERROR:
				{
					{
						char* _tmp9_;
						char* _tmp8_;
#line 820 "glib-2.0.vapi"
						_tmp9_ = NULL;
#line 226 "ImportPage.vala"
						_tmp8_ = NULL;
#line 1262 "AppWindow.vala"
						app_window_error_message (_tmp9_ = g_strdup_printf ("Unable to fetch previews from the camera:\n%s", _tmp8_ = import_page_get_refresh_message (page)));
#line 4213 "AppWindow.c"
						_tmp9_ = (g_free (_tmp9_), NULL);
						_tmp8_ = (g_free (_tmp8_), NULL);
					}
#line 1263 "AppWindow.vala"
					break;
#line 4219 "AppWindow.c"
				}
				default:
				{
					{
#line 1266 "AppWindow.vala"
						g_error ("AppWindow.vala:1266: Unknown result type %d", (gint) res);
#line 4226 "AppWindow.c"
					}
#line 1267 "AppWindow.vala"
					break;
#line 4230 "AppWindow.c"
				}
			}
			result = TRUE;
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
			(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
			return result;
		}
		(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
	}
	/* not found*/
	result = FALSE;
	return result;
}


#line 1277 "AppWindow.vala"
static gboolean app_window_is_event_selected (AppWindow* self, const GtkTreePath* path) {
#line 4248 "AppWindow.c"
	gboolean result;
#line 1277 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 1277 "AppWindow.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 4254 "AppWindow.c"
	{
		GeeIterator* _page_it;
		_page_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->event_list));
#line 1278 "AppWindow.vala"
		while (TRUE) {
#line 4260 "AppWindow.c"
			EventPage* page;
#line 1278 "AppWindow.vala"
			if (!gee_iterator_next (_page_it)) {
#line 1278 "AppWindow.vala"
				break;
#line 4266 "AppWindow.c"
			}
			page = (EventPage*) gee_iterator_get (_page_it);
#line 1279 "AppWindow.vala"
			if (app_window_is_page_selected (self, PAGE (page), path)) {
#line 1280 "AppWindow.vala"
				app_window_switch_to_page (self, PAGE (page));
#line 4273 "AppWindow.c"
				result = TRUE;
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
				return result;
			}
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		}
		(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
	}
	result = FALSE;
	return result;
}


#line 1289 "AppWindow.vala"
static gboolean app_window_focus_on_current_page (AppWindow* self) {
#line 4290 "AppWindow.c"
	gboolean result;
#line 1289 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), FALSE);
#line 1290 "AppWindow.vala"
	gtk_widget_grab_focus (GTK_WIDGET (self->priv->current_page));
#line 4296 "AppWindow.c"
	result = FALSE;
	return result;
}


#line 1289 "AppWindow.vala"
static gboolean _app_window_focus_on_current_page_gsource_func (gpointer self) {
#line 4304 "AppWindow.c"
	return app_window_focus_on_current_page (self);
}


#line 1295 "AppWindow.vala"
static void app_window_on_sidebar_cursor_changed (AppWindow* self) {
#line 4311 "AppWindow.c"
	GtkTreePath* path;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp0_;
#line 1295 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 4317 "AppWindow.c"
	path = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
#line 1297 "AppWindow.vala"
	gtk_tree_view_get_cursor (GTK_TREE_VIEW (self->priv->sidebar), &_tmp0_, NULL);
#line 1297 "AppWindow.vala"
	path = (_tmp1_ = _tmp0_, (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp1_);
#line 1299 "AppWindow.vala"
	if (app_window_is_page_selected (self, PAGE (self->priv->collection_page), path)) {
#line 1300 "AppWindow.vala"
		app_window_switch_to_collection_page (self);
#line 4329 "AppWindow.c"
	} else {
#line 1301 "AppWindow.vala"
		if (app_window_is_page_selected (self, PAGE (self->priv->events_directory_page), path)) {
#line 1302 "AppWindow.vala"
			app_window_switch_to_events_directory_page (self);
#line 4335 "AppWindow.c"
		} else {
			gboolean _tmp2_;
			_tmp2_ = FALSE;
#line 1303 "AppWindow.vala"
			if (self->priv->import_queue_page != NULL) {
#line 1303 "AppWindow.vala"
				_tmp2_ = app_window_is_page_selected (self, PAGE (self->priv->import_queue_page), path);
#line 4343 "AppWindow.c"
			} else {
#line 1303 "AppWindow.vala"
				_tmp2_ = FALSE;
#line 4347 "AppWindow.c"
			}
#line 1303 "AppWindow.vala"
			if (_tmp2_) {
#line 1304 "AppWindow.vala"
				app_window_switch_to_import_queue_page (self);
#line 4353 "AppWindow.c"
			} else {
#line 1305 "AppWindow.vala"
				if (app_window_is_camera_selected (self, path)) {
#line 4357 "AppWindow.c"
				} else {
#line 1307 "AppWindow.vala"
					if (app_window_is_event_selected (self, path)) {
#line 4361 "AppWindow.c"
					} else {
					}
				}
			}
		}
	}
	/* event page selected and updated
	 nothing recognized selected
	 this has to be done in Idle handler because the focus/ won't change properly inside 
	 this signal*/
#line 1315 "AppWindow.vala"
	g_idle_add (_app_window_focus_on_current_page_gsource_func, self);
#line 4374 "AppWindow.c"
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
}


#line 1318 "AppWindow.vala"
static void app_window_do_op (AppWindow* self, int res, const char* op, GError** error) {
#line 4381 "AppWindow.c"
	GError * _inner_error_;
#line 1318 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 1318 "AppWindow.vala"
	g_return_if_fail (op != NULL);
#line 4387 "AppWindow.c"
	_inner_error_ = NULL;
#line 1319 "AppWindow.vala"
	if (res != GP_OK) {
#line 4391 "AppWindow.c"
		_inner_error_ = g_error_new (GPHOTO_ERROR, GPHOTO_ERROR_LIBRARY, "[%d] Unable to %s: %s", (gint) res, op, gp_port_result_as_string (res));
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == GPHOTO_ERROR) {
				g_propagate_error (error, _inner_error_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
	}
}


#line 1323 "AppWindow.vala"
static void app_window_init_camera_table (AppWindow* self, GError** error) {
#line 4409 "AppWindow.c"
	GError * _inner_error_;
	CameraAbilitiesList* _tmp2_;
	int _tmp1_;
	CameraAbilitiesList* _tmp0_;
#line 1323 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 4416 "AppWindow.c"
	_inner_error_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 1324 "AppWindow.vala"
	app_window_do_op (self, (_tmp1_ = gp_abilities_list_new (&_tmp0_), self->priv->abilities_list = (_tmp2_ = _tmp0_, (self->priv->abilities_list == NULL) ? NULL : (self->priv->abilities_list = (gp_abilities_list_free (self->priv->abilities_list), NULL)), _tmp2_), _tmp1_), "create camera abilities list", &_inner_error_);
#line 4422 "AppWindow.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == GPHOTO_ERROR) {
			g_propagate_error (error, _inner_error_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
#line 1325 "AppWindow.vala"
	app_window_do_op (self, gp_abilities_list_load (self->priv->abilities_list, self->priv->null_context), "load camera abilities list", &_inner_error_);
#line 4435 "AppWindow.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == GPHOTO_ERROR) {
			g_propagate_error (error, _inner_error_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
}


static void _vala_array_add2 (char*** array, int* length, int* size, char* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (char*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


/* USB (or libusb) is a funny beast; if only one USB device is present (i.e. the camera),
 then a single camera is detected at port usb:.  However, if multiple USB devices are
 present (including non-cameras), then the first attached camera will be listed twice,
 first at usb:, then at usb:xxx,yyy.  If the usb: device is removed, another usb:xxx,yyy
 device will lose its full-path name and be referred to as usb: only.

 This function gleans the full port name of a particular port, even if it's the unadorned
 "usb:", by using HAL.*/
#line 1336 "AppWindow.vala"
static char* app_window_esp_usb_to_udi (AppWindow* self, gint camera_count, const char* port, char** full_port) {
#line 4469 "AppWindow.c"
	char* result;
	DBusError _tmp0_ = {0};
	DBusError raw;
	char** _tmp2_;
	gint udis_size;
	gint udis_length1;
	gint _tmp1_;
	char** udis;
	char** _tmp3_;
	gint usbs_size;
	gint usbs_length1;
	char** usbs;
	gint bus;
	gint device;
#line 1336 "AppWindow.vala"
	g_return_val_if_fail (IS_APP_WINDOW (self), NULL);
#line 1336 "AppWindow.vala"
	g_return_val_if_fail (port != NULL, NULL);
#line 4488 "AppWindow.c"
	if (full_port != NULL) {
		*full_port = NULL;
	}
	/* sanity*/
#line 1338 "AppWindow.vala"
	g_assert (camera_count > 0);
#line 1340 "AppWindow.vala"
	g_debug ("AppWindow.vala:1340: ESP: camera_count=%d port=%s", camera_count, port);
#line 4497 "AppWindow.c"
	raw = (dbus_error_init (&_tmp0_), _tmp0_);
#line 1343 "AppWindow.vala"
	_tmp2_ = NULL;
#line 4501 "AppWindow.c"
	udis = (_tmp2_ = libhal_find_device_by_capability (self->priv->hal_context, "camera", &_tmp1_, &raw), udis_length1 = _tmp1_, udis_size = udis_length1, _tmp2_);
#line 1345 "AppWindow.vala"
	_tmp3_ = NULL;
#line 4505 "AppWindow.c"
	usbs = (_tmp3_ = g_new0 (char*, 0 + 1), usbs_length1 = 0, usbs_size = usbs_length1, _tmp3_);
	{
		char** udi_collection;
		int udi_collection_length1;
		int udi_it;
#line 1346 "AppWindow.vala"
		udi_collection = udis;
#line 4513 "AppWindow.c"
		udi_collection_length1 = udis_length1;
		for (udi_it = 0; udi_it < udis_length1; udi_it = udi_it + 1) {
			const char* _tmp7_;
			char* udi;
#line 1343 "AppWindow.vala"
			_tmp7_ = NULL;
#line 4520 "AppWindow.c"
			udi = (_tmp7_ = udi_collection[udi_it], (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
			{
				char* _tmp4_;
				gboolean _tmp5_;
#line 43 "hal.vapi"
				_tmp4_ = NULL;
#line 1347 "AppWindow.vala"
				if ((_tmp5_ = _vala_strcmp0 (_tmp4_ = libhal_device_get_property_string (self->priv->hal_context, udi, "info.subsystem", &raw), "usb") == 0, _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_)) {
#line 4529 "AppWindow.c"
					const char* _tmp6_;
#line 1348 "AppWindow.vala"
					_tmp6_ = NULL;
#line 1348 "AppWindow.vala"
					_vala_array_add2 (&usbs, &usbs_length1, &usbs_size, (_tmp6_ = udi, (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)));
#line 4535 "AppWindow.c"
				}
				udi = (g_free (udi), NULL);
			}
		}
	}
	/* if GPhoto detects one camera, and HAL reports one USB camera, all is swell*/
#line 1352 "AppWindow.vala"
	if (camera_count == 1) {
#line 1353 "AppWindow.vala"
		if (usbs_length1 == 1) {
#line 4546 "AppWindow.c"
			const char* _tmp8_;
			char* usb;
			gint hal_bus;
			gint hal_device;
#line 1354 "AppWindow.vala"
			_tmp8_ = NULL;
#line 4553 "AppWindow.c"
			usb = (_tmp8_ = usbs[0], (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
			hal_bus = libhal_device_get_property_int (self->priv->hal_context, usb, "usb.bus_number", &raw);
			hal_device = libhal_device_get_property_int (self->priv->hal_context, usb, "usb.linux.device_number", &raw);
#line 1359 "AppWindow.vala"
			if (_vala_strcmp0 (port, "usb:") == 0) {
#line 4559 "AppWindow.c"
				char* _tmp9_;
				/* the most likely case, so make a full path*/
				_tmp9_ = NULL;
#line 1361 "AppWindow.vala"
				(*full_port) = (_tmp9_ = g_strdup_printf ("usb:%03d,%03d", hal_bus, hal_device), (*full_port) = (g_free ((*full_port)), NULL), _tmp9_);
#line 4565 "AppWindow.c"
			} else {
				char* _tmp11_;
				const char* _tmp10_;
				_tmp11_ = NULL;
#line 1363 "AppWindow.vala"
				_tmp10_ = NULL;
#line 1363 "AppWindow.vala"
				(*full_port) = (_tmp11_ = (_tmp10_ = port, (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), (*full_port) = (g_free ((*full_port)), NULL), _tmp11_);
#line 4574 "AppWindow.c"
			}
#line 1366 "AppWindow.vala"
			g_debug ("AppWindow.vala:1366: ESP: port=%s full_port=%s udi=%s", port, (*full_port), usb);
#line 4578 "AppWindow.c"
			result = usb;
			dbus_error_free (&raw);
			udis = (_vala_array_free (udis, udis_length1, (GDestroyNotify) g_free), NULL);
			usbs = (_vala_array_free (usbs, usbs_length1, (GDestroyNotify) g_free), NULL);
			return result;
		}
	}
	/* with more than one camera, skip the mirrored "usb:" port*/
#line 1373 "AppWindow.vala"
	if (_vala_strcmp0 (port, "usb:") == 0) {
#line 1374 "AppWindow.vala"
		g_debug ("AppWindow.vala:1374: ESP: Skipping %s", port);
#line 4591 "AppWindow.c"
		result = NULL;
		dbus_error_free (&raw);
		udis = (_vala_array_free (udis, udis_length1, (GDestroyNotify) g_free), NULL);
		usbs = (_vala_array_free (usbs, usbs_length1, (GDestroyNotify) g_free), NULL);
		return result;
	}
	/* parse out the bus and device ID*/
	bus = 0;
	device = 0;
#line 1381 "AppWindow.vala"
	if (sscanf (port, "usb:%d,%d", &bus, &device) < 2) {
#line 1382 "AppWindow.vala"
		g_error ("AppWindow.vala:1382: ESP: Failed to scanf %s", port);
#line 4605 "AppWindow.c"
	}
	{
		char** usb_collection;
		int usb_collection_length1;
		int usb_it;
#line 1384 "AppWindow.vala"
		usb_collection = usbs;
#line 4613 "AppWindow.c"
		usb_collection_length1 = usbs_length1;
		for (usb_it = 0; usb_it < usbs_length1; usb_it = usb_it + 1) {
			const char* _tmp15_;
			char* usb;
#line 1345 "AppWindow.vala"
			_tmp15_ = NULL;
#line 4620 "AppWindow.c"
			usb = (_tmp15_ = usb_collection[usb_it], (_tmp15_ == NULL) ? NULL : g_strdup (_tmp15_));
			{
				gint hal_bus;
				gint hal_device;
				gboolean _tmp12_;
				hal_bus = libhal_device_get_property_int (self->priv->hal_context, usb, "usb.bus_number", &raw);
				hal_device = libhal_device_get_property_int (self->priv->hal_context, usb, "usb.linux.device_number", &raw);
				_tmp12_ = FALSE;
#line 1388 "AppWindow.vala"
				if (bus == hal_bus) {
#line 1388 "AppWindow.vala"
					_tmp12_ = device == hal_device;
#line 4633 "AppWindow.c"
				} else {
#line 1388 "AppWindow.vala"
					_tmp12_ = FALSE;
#line 4637 "AppWindow.c"
				}
#line 1388 "AppWindow.vala"
				if (_tmp12_) {
#line 4641 "AppWindow.c"
					char* _tmp14_;
					const char* _tmp13_;
					_tmp14_ = NULL;
#line 1389 "AppWindow.vala"
					_tmp13_ = NULL;
#line 1389 "AppWindow.vala"
					(*full_port) = (_tmp14_ = (_tmp13_ = port, (_tmp13_ == NULL) ? NULL : g_strdup (_tmp13_)), (*full_port) = (g_free ((*full_port)), NULL), _tmp14_);
#line 1391 "AppWindow.vala"
					g_debug ("AppWindow.vala:1391: ESP: port=%s full_port=%s udi=%s", port, (*full_port), usb);
#line 4651 "AppWindow.c"
					result = usb;
					dbus_error_free (&raw);
					udis = (_vala_array_free (udis, udis_length1, (GDestroyNotify) g_free), NULL);
					usbs = (_vala_array_free (usbs, usbs_length1, (GDestroyNotify) g_free), NULL);
					return result;
				}
				usb = (g_free (usb), NULL);
			}
		}
	}
#line 1397 "AppWindow.vala"
	g_debug ("AppWindow.vala:1397: ESP: No UDI found for port=%s", port);
#line 4664 "AppWindow.c"
	result = NULL;
	dbus_error_free (&raw);
	udis = (_vala_array_free (udis, udis_length1, (GDestroyNotify) g_free), NULL);
	usbs = (_vala_array_free (usbs, usbs_length1, (GDestroyNotify) g_free), NULL);
	return result;
}


#line 1402 "AppWindow.vala"
static char* app_window_get_port_uri (const char* port) {
#line 4675 "AppWindow.c"
	char* result;
#line 1402 "AppWindow.vala"
	g_return_val_if_fail (port != NULL, NULL);
#line 4679 "AppWindow.c"
	result = g_strdup_printf ("gphoto2://[%s]/", port);
	return result;
}


static void _vala_array_add3 (ImportPage*** array, int* length, int* size, ImportPage* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (ImportPage*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


#line 1406 "AppWindow.vala"
static void app_window_update_camera_table (AppWindow* self, GError** error) {
#line 4697 "AppWindow.c"
	GError * _inner_error_;
	GPPortInfoList* port_info_list;
	GPPortInfoList* _tmp2_;
	int _tmp1_;
	GPPortInfoList* _tmp0_;
	CameraList* camera_list;
	CameraList* _tmp5_;
	int _tmp4_;
	CameraList* _tmp3_;
	GeeHashMap* detected_map;
	ImportPage** _tmp20_;
	gint missing_size;
	gint missing_length1;
	ImportPage** missing;
	gboolean _tmp58_;
#line 1406 "AppWindow.vala"
	g_return_if_fail (IS_APP_WINDOW (self));
#line 4715 "AppWindow.c"
	_inner_error_ = NULL;
	/* need to do this because virtual ports come and go in the USB world (and probably others)*/
	port_info_list = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 1409 "AppWindow.vala"
	app_window_do_op (self, (_tmp1_ = gp_port_info_list_new (&_tmp0_), port_info_list = (_tmp2_ = _tmp0_, (port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL)), _tmp2_), _tmp1_), "create port list", &_inner_error_);
#line 4723 "AppWindow.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == GPHOTO_ERROR) {
			g_propagate_error (error, _inner_error_);
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			return;
		} else {
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
#line 1410 "AppWindow.vala"
	app_window_do_op (self, gp_port_info_list_load (port_info_list), "load port list", &_inner_error_);
#line 4738 "AppWindow.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == GPHOTO_ERROR) {
			g_propagate_error (error, _inner_error_);
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			return;
		} else {
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	camera_list = NULL;
	_tmp5_ = NULL;
	_tmp3_ = NULL;
#line 1413 "AppWindow.vala"
	app_window_do_op (self, (_tmp4_ = gp_list_new (&_tmp3_), camera_list = (_tmp5_ = _tmp3_, (camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL)), _tmp5_), _tmp4_), "create camera list", &_inner_error_);
#line 4756 "AppWindow.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == GPHOTO_ERROR) {
			g_propagate_error (error, _inner_error_);
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
			return;
		} else {
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
#line 1414 "AppWindow.vala"
	app_window_do_op (self, gp_abilities_list_detect (self->priv->abilities_list, port_info_list, camera_list, self->priv->null_context), "detect cameras", &_inner_error_);
#line 4773 "AppWindow.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == GPHOTO_ERROR) {
			g_propagate_error (error, _inner_error_);
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
			return;
		} else {
			(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
			(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	detected_map = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal, g_str_equal);
	{
		gint ctr;
		ctr = 0;
		{
			gboolean _tmp6_;
			_tmp6_ = TRUE;
#line 1419 "AppWindow.vala"
			while (TRUE) {
#line 4797 "AppWindow.c"
				char* name;
				const char* _tmp10_;
				char* _tmp9_;
				int _tmp8_;
				const char* _tmp7_;
				char* port;
				const char* _tmp14_;
				char* _tmp13_;
				int _tmp12_;
				const char* _tmp11_;
#line 1419 "AppWindow.vala"
				if (!_tmp6_) {
#line 1419 "AppWindow.vala"
					ctr++;
#line 4812 "AppWindow.c"
				}
#line 1419 "AppWindow.vala"
				_tmp6_ = FALSE;
#line 1419 "AppWindow.vala"
				if (!(ctr < gp_list_count (camera_list))) {
#line 1419 "AppWindow.vala"
					break;
#line 4820 "AppWindow.c"
				}
				name = NULL;
#line 1421 "AppWindow.vala"
				_tmp10_ = NULL;
#line 4825 "AppWindow.c"
				_tmp9_ = NULL;
				_tmp7_ = NULL;
#line 1421 "AppWindow.vala"
				app_window_do_op (self, (_tmp8_ = gp_list_get_name (camera_list, ctr, &_tmp7_), name = (_tmp9_ = (_tmp10_ = _tmp7_, (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), name = (g_free (name), NULL), _tmp9_), _tmp8_), "get detected camera name", &_inner_error_);
#line 4830 "AppWindow.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == GPHOTO_ERROR) {
						g_propagate_error (error, _inner_error_);
						name = (g_free (name), NULL);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						return;
					} else {
						name = (g_free (name), NULL);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
				port = NULL;
#line 1424 "AppWindow.vala"
				_tmp14_ = NULL;
#line 4852 "AppWindow.c"
				_tmp13_ = NULL;
				_tmp11_ = NULL;
#line 1424 "AppWindow.vala"
				app_window_do_op (self, (_tmp12_ = gp_list_get_value (camera_list, ctr, &_tmp11_), port = (_tmp13_ = (_tmp14_ = _tmp11_, (_tmp14_ == NULL) ? NULL : g_strdup (_tmp14_)), port = (g_free (port), NULL), _tmp13_), _tmp12_), "get detected camera port", &_inner_error_);
#line 4857 "AppWindow.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == GPHOTO_ERROR) {
						g_propagate_error (error, _inner_error_);
						name = (g_free (name), NULL);
						port = (g_free (port), NULL);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						return;
					} else {
						name = (g_free (name), NULL);
						port = (g_free (port), NULL);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
#line 1426 "AppWindow.vala"
				g_debug ("AppWindow.vala:1426: Detected %s @ %s", name, port);
#line 4880 "AppWindow.c"
				/* do some USB ESP*/
#line 1429 "AppWindow.vala"
				if (g_str_has_prefix (port, "usb:")) {
#line 4884 "AppWindow.c"
					char* full_port;
					char* _tmp17_;
					char* _tmp16_;
					char* _tmp15_;
					char* udi;
					char* _tmp19_;
					const char* _tmp18_;
					full_port = NULL;
					_tmp17_ = NULL;
					_tmp16_ = NULL;
					_tmp15_ = NULL;
					udi = (_tmp16_ = app_window_esp_usb_to_udi (self, gp_list_count (camera_list), port, &_tmp15_), full_port = (_tmp17_ = _tmp15_, full_port = (g_free (full_port), NULL), _tmp17_), _tmp16_);
#line 1432 "AppWindow.vala"
					if (udi == NULL) {
#line 4899 "AppWindow.c"
						full_port = (g_free (full_port), NULL);
						udi = (g_free (udi), NULL);
						name = (g_free (name), NULL);
						port = (g_free (port), NULL);
#line 1433 "AppWindow.vala"
						continue;
#line 4906 "AppWindow.c"
					}
					_tmp19_ = NULL;
#line 1435 "AppWindow.vala"
					_tmp18_ = NULL;
#line 1435 "AppWindow.vala"
					port = (_tmp19_ = (_tmp18_ = full_port, (_tmp18_ == NULL) ? NULL : g_strdup (_tmp18_)), port = (g_free (port), NULL), _tmp19_);
#line 4913 "AppWindow.c"
					full_port = (g_free (full_port), NULL);
					udi = (g_free (udi), NULL);
				}
#line 1438 "AppWindow.vala"
				gee_abstract_map_set (GEE_ABSTRACT_MAP (detected_map), port, name);
#line 4919 "AppWindow.c"
				name = (g_free (name), NULL);
				port = (g_free (port), NULL);
			}
		}
	}
	/* first, find cameras that have disappeared*/
#line 1442 "AppWindow.vala"
	_tmp20_ = NULL;
#line 4928 "AppWindow.c"
	missing = (_tmp20_ = g_new0 (ImportPage*, 0 + 1), missing_length1 = 0, missing_size = missing_length1, _tmp20_);
	{
		GeeCollection* _tmp21_;
		GeeIterator* _tmp22_;
		GeeIterator* _page_it;
#line 39 "gee-1.0.vapi"
		_tmp21_ = NULL;
#line 4936 "AppWindow.c"
		_tmp22_ = NULL;
		_page_it = (_tmp22_ = gee_iterable_iterator (GEE_ITERABLE (_tmp21_ = gee_abstract_map_get_values (GEE_ABSTRACT_MAP (self->priv->camera_map)))), (_tmp21_ == NULL) ? NULL : (_tmp21_ = (g_object_unref (_tmp21_), NULL)), _tmp22_);
#line 1443 "AppWindow.vala"
		while (TRUE) {
#line 4941 "AppWindow.c"
			ImportPage* page;
			Camera* camera;
			GPPortInfo port_info;
			GPPortInfo _tmp25_;
			int _tmp24_;
			GPPortInfo _tmp23_;
			CameraAbilities abilities;
			CameraAbilities _tmp28_;
			int _tmp27_;
			CameraAbilities _tmp26_;
			ImportPage* _tmp29_;
#line 1443 "AppWindow.vala"
			if (!gee_iterator_next (_page_it)) {
#line 1443 "AppWindow.vala"
				break;
#line 4957 "AppWindow.c"
			}
			page = (ImportPage*) gee_iterator_get (_page_it);
			camera = import_page_get_camera (page);
#line 1447 "AppWindow.vala"
			app_window_do_op (self, (_tmp24_ = gp_camera_get_port_info (camera, &_tmp23_), port_info = (_tmp25_ = _tmp23_,  (&port_info), _tmp25_), _tmp24_), "retrieve missing camera port information", &_inner_error_);
#line 4963 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					 (&port_info);
					(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					 (&port_info);
					(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
#line 1450 "AppWindow.vala"
			app_window_do_op (self, (_tmp27_ = gp_camera_get_abilities (camera, &_tmp26_), abilities = (_tmp28_ = _tmp26_,  (&abilities), _tmp28_), _tmp27_), "retrieve camera abilities", &_inner_error_);
#line 4992 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					 (&port_info);
					 (&abilities);
					(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					 (&port_info);
					 (&abilities);
					(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
#line 1452 "AppWindow.vala"
			if (gee_abstract_map_contains (GEE_ABSTRACT_MAP (detected_map), port_info.path)) {
#line 1453 "AppWindow.vala"
				g_debug ("AppWindow.vala:1453: Found page for %s @ %s in detected cameras", abilities.model, port_info.path);
#line 5025 "AppWindow.c"
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
				 (&port_info);
				 (&abilities);
#line 1455 "AppWindow.vala"
				continue;
#line 5032 "AppWindow.c"
			}
#line 1458 "AppWindow.vala"
			g_debug ("AppWindow.vala:1458: %s @ %s missing", abilities.model, port_info.path);
#line 1460 "AppWindow.vala"
			_tmp29_ = NULL;
#line 1460 "AppWindow.vala"
			_vala_array_add3 (&missing, &missing_length1, &missing_size, (_tmp29_ = page, (_tmp29_ == NULL) ? NULL : g_object_ref (_tmp29_)));
#line 5040 "AppWindow.c"
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
			(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
			 (&port_info);
			 (&abilities);
		}
		(_page_it == NULL) ? NULL : (_page_it = (g_object_unref (_page_it), NULL));
	}
	/* have to remove from hash map outside of iterator*/
	{
		ImportPage** page_collection;
		int page_collection_length1;
		int page_it;
#line 1464 "AppWindow.vala"
		page_collection = missing;
#line 5055 "AppWindow.c"
		page_collection_length1 = missing_length1;
		for (page_it = 0; page_it < missing_length1; page_it = page_it + 1) {
			ImportPage* _tmp37_;
			ImportPage* page;
#line 1442 "AppWindow.vala"
			_tmp37_ = NULL;
#line 5062 "AppWindow.c"
			page = (_tmp37_ = page_collection[page_it], (_tmp37_ == NULL) ? NULL : g_object_ref (_tmp37_));
			{
				Camera* camera;
				GPPortInfo port_info;
				GPPortInfo _tmp32_;
				int _tmp31_;
				GPPortInfo _tmp30_;
				CameraAbilities abilities;
				CameraAbilities _tmp35_;
				int _tmp34_;
				CameraAbilities _tmp33_;
				char* _tmp36_;
				camera = import_page_get_camera (page);
#line 1468 "AppWindow.vala"
				app_window_do_op (self, (_tmp31_ = gp_camera_get_port_info (camera, &_tmp30_), port_info = (_tmp32_ = _tmp30_,  (&port_info), _tmp32_), _tmp31_), "retrieve missing camera port information", &_inner_error_);
#line 5078 "AppWindow.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == GPHOTO_ERROR) {
						g_propagate_error (error, _inner_error_);
						(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
						(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
						 (&port_info);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						return;
					} else {
						(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
						(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
						 (&port_info);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
#line 1471 "AppWindow.vala"
				app_window_do_op (self, (_tmp34_ = gp_camera_get_abilities (camera, &_tmp33_), abilities = (_tmp35_ = _tmp33_,  (&abilities), _tmp35_), _tmp34_), "retrieve missing camera abilities", &_inner_error_);
#line 5105 "AppWindow.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == GPHOTO_ERROR) {
						g_propagate_error (error, _inner_error_);
						(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
						(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
						 (&port_info);
						 (&abilities);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						return;
					} else {
						(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
						(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
						 (&port_info);
						 (&abilities);
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
#line 1473 "AppWindow.vala"
				g_debug ("AppWindow.vala:1473: Removing from camera table: %s @ %s", abilities.model, port_info.path);
#line 1402 "AppWindow.vala"
				_tmp36_ = NULL;
#line 1475 "AppWindow.vala"
				gee_abstract_map_remove (GEE_ABSTRACT_MAP (self->priv->camera_map), _tmp36_ = app_window_get_port_uri (port_info.path), NULL);
#line 5138 "AppWindow.c"
				_tmp36_ = (g_free (_tmp36_), NULL);
#line 1476 "AppWindow.vala"
				app_window_remove_page (self, PAGE (page));
#line 5142 "AppWindow.c"
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
				 (&port_info);
				 (&abilities);
			}
		}
	}
	/* add cameras which were not present before*/
	{
		GeeSet* _tmp38_;
		GeeIterator* _tmp39_;
		GeeIterator* _port_it;
		/* add cameras which were not present before*/
#line 38 "gee-1.0.vapi"
		_tmp38_ = NULL;
#line 5158 "AppWindow.c"
		_tmp39_ = NULL;
		_port_it = (_tmp39_ = gee_iterable_iterator (GEE_ITERABLE (_tmp38_ = gee_abstract_map_get_keys (GEE_ABSTRACT_MAP (detected_map)))), (_tmp38_ == NULL) ? NULL : (_tmp38_ = (g_object_unref (_tmp38_), NULL)), _tmp39_);
		/* add cameras which were not present before*/
#line 1480 "AppWindow.vala"
		while (TRUE) {
#line 5164 "AppWindow.c"
			char* port;
			char* name;
			char* uri;
			gint index;
			GPPortInfo port_info;
			char* _tmp44_;
			GPPortInfo _tmp43_;
			int _tmp42_;
			GPPortInfo _tmp41_;
			CameraAbilities camera_abilities;
			char* _tmp49_;
			CameraAbilities _tmp48_;
			int _tmp47_;
			CameraAbilities _tmp46_;
			Camera* camera;
			char* _tmp53_;
			Camera* _tmp52_;
			int _tmp51_;
			Camera* _tmp50_;
			char* _tmp54_;
			char* _tmp55_;
			ImportPage* page;
			/* add cameras which were not present before*/
#line 1480 "AppWindow.vala"
			if (!gee_iterator_next (_port_it)) {
#line 5190 "AppWindow.c"
				/* add cameras which were not present before*/
#line 1480 "AppWindow.vala"
				break;
#line 5194 "AppWindow.c"
			}
			/* add cameras which were not present before*/
			port = (char*) gee_iterator_get (_port_it);
			name = (char*) gee_abstract_map_get (GEE_ABSTRACT_MAP (detected_map), port);
			uri = app_window_get_port_uri (port);
#line 1484 "AppWindow.vala"
			if (gee_abstract_map_contains (GEE_ABSTRACT_MAP (self->priv->camera_map), uri)) {
#line 5202 "AppWindow.c"
				/* already known about*/
#line 1486 "AppWindow.vala"
				g_debug ("AppWindow.vala:1486: %s @ %s already registered, skipping", name, port);
#line 5206 "AppWindow.c"
				port = (g_free (port), NULL);
				name = (g_free (name), NULL);
				uri = (g_free (uri), NULL);
#line 1488 "AppWindow.vala"
				continue;
#line 5212 "AppWindow.c"
			}
			index = gp_port_info_list_lookup_path (port_info_list, port);
#line 1492 "AppWindow.vala"
			if (index < 0) {
#line 5217 "AppWindow.c"
				char* _tmp40_;
#line 820 "glib-2.0.vapi"
				_tmp40_ = NULL;
#line 1493 "AppWindow.vala"
				app_window_do_op (self, (int) index, _tmp40_ = g_strdup_printf ("lookup port %s", port), &_inner_error_);
#line 5223 "AppWindow.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == GPHOTO_ERROR) {
						g_propagate_error (error, _inner_error_);
						port = (g_free (port), NULL);
						name = (g_free (name), NULL);
						uri = (g_free (uri), NULL);
						(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						return;
					} else {
						port = (g_free (port), NULL);
						name = (g_free (name), NULL);
						uri = (g_free (uri), NULL);
						(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
				_tmp40_ = (g_free (_tmp40_), NULL);
			}
#line 820 "glib-2.0.vapi"
			_tmp44_ = NULL;
#line 1496 "AppWindow.vala"
			app_window_do_op (self, (_tmp42_ = gp_port_info_list_get_info (port_info_list, index, &_tmp41_), port_info = (_tmp43_ = _tmp41_,  (&port_info), _tmp43_), _tmp42_), _tmp44_ = g_strdup_printf ("get port info for %s", port), &_inner_error_);
#line 5256 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
			_tmp44_ = (g_free (_tmp44_), NULL);
			/* this should match, every time*/
#line 1499 "AppWindow.vala"
			g_assert (_vala_strcmp0 (port, port_info.path) == 0);
#line 1501 "AppWindow.vala"
			index = gp_abilities_list_lookup_model (self->priv->abilities_list, name);
#line 1502 "AppWindow.vala"
			if (index < 0) {
#line 5293 "AppWindow.c"
				char* _tmp45_;
#line 820 "glib-2.0.vapi"
				_tmp45_ = NULL;
#line 1503 "AppWindow.vala"
				app_window_do_op (self, (int) index, _tmp45_ = g_strdup_printf ("lookup camera model %s", name), &_inner_error_);
#line 5299 "AppWindow.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == GPHOTO_ERROR) {
						g_propagate_error (error, _inner_error_);
						port = (g_free (port), NULL);
						name = (g_free (name), NULL);
						uri = (g_free (uri), NULL);
						 (&port_info);
						(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						return;
					} else {
						port = (g_free (port), NULL);
						name = (g_free (name), NULL);
						uri = (g_free (uri), NULL);
						 (&port_info);
						(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
						(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
						(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
						(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
						missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
				_tmp45_ = (g_free (_tmp45_), NULL);
			}
#line 820 "glib-2.0.vapi"
			_tmp49_ = NULL;
#line 1506 "AppWindow.vala"
			app_window_do_op (self, (_tmp47_ = gp_abilities_list_get_abilities (self->priv->abilities_list, index, &_tmp46_), camera_abilities = (_tmp48_ = _tmp46_,  (&camera_abilities), _tmp48_), _tmp47_), _tmp49_ = g_strdup_printf ("lookup camera abilities for %s", name), &_inner_error_);
#line 5334 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
			_tmp49_ = (g_free (_tmp49_), NULL);
			camera = NULL;
#line 820 "glib-2.0.vapi"
			_tmp53_ = NULL;
#line 5369 "AppWindow.c"
			_tmp52_ = NULL;
			_tmp50_ = NULL;
#line 1510 "AppWindow.vala"
			app_window_do_op (self, (_tmp51_ = gp_camera_new (&_tmp50_), camera = (_tmp52_ = _tmp50_, (camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL)), _tmp52_), _tmp51_), _tmp53_ = g_strdup_printf ("create camera object for %s", name), &_inner_error_);
#line 5374 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
			_tmp53_ = (g_free (_tmp53_), NULL);
#line 820 "glib-2.0.vapi"
			_tmp54_ = NULL;
#line 1511 "AppWindow.vala"
			app_window_do_op (self, gp_camera_set_abilities (camera, camera_abilities), _tmp54_ = g_strdup_printf ("set camera abilities for %s", name), &_inner_error_);
#line 5412 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
			_tmp54_ = (g_free (_tmp54_), NULL);
#line 820 "glib-2.0.vapi"
			_tmp55_ = NULL;
#line 1512 "AppWindow.vala"
			app_window_do_op (self, gp_camera_set_port_info (camera, port_info), _tmp55_ = g_strdup_printf ("set port info for %s on %s", name, port), &_inner_error_);
#line 5450 "AppWindow.c"
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == GPHOTO_ERROR) {
					g_propagate_error (error, _inner_error_);
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					return;
				} else {
					port = (g_free (port), NULL);
					name = (g_free (name), NULL);
					uri = (g_free (uri), NULL);
					 (&port_info);
					 (&camera_abilities);
					(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
					(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
					(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
					(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
					(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
					missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
			_tmp55_ = (g_free (_tmp55_), NULL);
#line 1514 "AppWindow.vala"
			g_debug ("AppWindow.vala:1514: Adding to camera table: %s @ %s", name, port);
#line 5486 "AppWindow.c"
			page = g_object_ref_sink (import_page_new (camera, uri));
			/* create the Cameras row if this is the first one*/
#line 1519 "AppWindow.vala"
			if (self->priv->cameras_marker == NULL) {
#line 5491 "AppWindow.c"
				SidebarMarker* _tmp57_;
				SidebarMarker* _tmp56_;
				_tmp57_ = NULL;
#line 102 "Page.vala"
				_tmp56_ = NULL;
#line 1520 "AppWindow.vala"
				self->priv->cameras_marker = (_tmp57_ = sidebar_insert_grouping_after (self->priv->sidebar, _tmp56_ = page_get_marker (PAGE (self->priv->events_directory_page)), "Cameras"), (self->priv->cameras_marker == NULL) ? NULL : (self->priv->cameras_marker = (sidebar_marker_unref (self->priv->cameras_marker), NULL)), _tmp57_);
#line 5499 "AppWindow.c"
				(_tmp56_ == NULL) ? NULL : (_tmp56_ = (sidebar_marker_unref (_tmp56_), NULL));
			}
#line 1523 "AppWindow.vala"
			app_window_add_child_page (self, self->priv->cameras_marker, PAGE (page));
#line 1525 "AppWindow.vala"
			gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->camera_map), uri, page);
#line 5506 "AppWindow.c"
			/* automagically expand the Cameras branch so the user sees the attached camera(s)*/
#line 1528 "AppWindow.vala"
			sidebar_expand_branch (self->priv->sidebar, self->priv->cameras_marker);
#line 5510 "AppWindow.c"
			port = (g_free (port), NULL);
			name = (g_free (name), NULL);
			uri = (g_free (uri), NULL);
			 (&port_info);
			 (&camera_abilities);
			(camera == NULL) ? NULL : (camera = (gp_camera_unref (camera), NULL));
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		}
		(_port_it == NULL) ? NULL : (_port_it = (g_object_unref (_port_it), NULL));
	}
	_tmp58_ = FALSE;
#line 1532 "AppWindow.vala"
	if (gee_map_get_size (GEE_MAP (self->priv->camera_map)) == 0) {
#line 1532 "AppWindow.vala"
		_tmp58_ = self->priv->cameras_marker != NULL;
#line 5526 "AppWindow.c"
	} else {
#line 1532 "AppWindow.vala"
		_tmp58_ = FALSE;
#line 5530 "AppWindow.c"
	}
	/* if no cameras present, remove row*/
#line 1532 "AppWindow.vala"
	if (_tmp58_) {
#line 5535 "AppWindow.c"
		SidebarMarker* _tmp59_;
#line 1533 "AppWindow.vala"
		sidebar_prune_branch (self->priv->sidebar, self->priv->cameras_marker);
#line 5539 "AppWindow.c"
		_tmp59_ = NULL;
#line 1534 "AppWindow.vala"
		self->priv->cameras_marker = (_tmp59_ = NULL, (self->priv->cameras_marker == NULL) ? NULL : (self->priv->cameras_marker = (sidebar_marker_unref (self->priv->cameras_marker), NULL)), _tmp59_);
#line 5543 "AppWindow.c"
	}
	(port_info_list == NULL) ? NULL : (port_info_list = (gp_port_info_list_free (port_info_list), NULL));
	(camera_list == NULL) ? NULL : (camera_list = (gp_list_unref (camera_list), NULL));
	(detected_map == NULL) ? NULL : (detected_map = (g_object_unref (detected_map), NULL));
	missing = (_vala_array_free (missing, missing_length1, (GDestroyNotify) g_object_unref), NULL);
}


#line 1538 "AppWindow.vala"
static void app_window_on_device_added (LibHalContext* context, const char* udi) {
#line 1538 "AppWindow.vala"
	g_return_if_fail (context != NULL);
#line 1538 "AppWindow.vala"
	g_return_if_fail (udi != NULL);
#line 1539 "AppWindow.vala"
	g_debug ("AppWindow.vala:1539: on_device_added: %s", udi);
#line 1541 "AppWindow.vala"
	app_window_schedule_camera_update ();
#line 5562 "AppWindow.c"
}


#line 1544 "AppWindow.vala"
static void app_window_on_device_removed (LibHalContext* context, const char* udi) {
#line 1544 "AppWindow.vala"
	g_return_if_fail (context != NULL);
#line 1544 "AppWindow.vala"
	g_return_if_fail (udi != NULL);
#line 1545 "AppWindow.vala"
	g_debug ("AppWindow.vala:1545: on_device_removed: %s", udi);
#line 1547 "AppWindow.vala"
	app_window_schedule_camera_update ();
#line 5576 "AppWindow.c"
}


#line 1560 "AppWindow.vala"
static gboolean _app_window_background_camera_update_gsource_func (gpointer self) {
#line 5582 "AppWindow.c"
	return app_window_background_camera_update ();
}


/* Device add/removes often arrive in pairs; this allows for a single
 update to occur when they come in all at once*/
#line 1552 "AppWindow.vala"
static void app_window_schedule_camera_update (void) {
#line 1553 "AppWindow.vala"
	if (app_window_camera_update_scheduled) {
#line 5593 "AppWindow.c"
		return;
	}
#line 1556 "AppWindow.vala"
	g_timeout_add ((guint) 500, _app_window_background_camera_update_gsource_func, NULL);
#line 1557 "AppWindow.vala"
	app_window_camera_update_scheduled = TRUE;
#line 5600 "AppWindow.c"
}


#line 1560 "AppWindow.vala"
static gboolean app_window_background_camera_update (void) {
#line 5606 "AppWindow.c"
	gboolean result;
	GError * _inner_error_;
	_inner_error_ = NULL;
#line 1561 "AppWindow.vala"
	g_debug ("AppWindow.vala:1561: background_camera_update");
#line 5612 "AppWindow.c"
	{
		AppWindow* _tmp0_;
#line 446 "AppWindow.vala"
		_tmp0_ = NULL;
#line 1564 "AppWindow.vala"
		app_window_update_camera_table (_tmp0_ = app_window_get_instance (), &_inner_error_);
#line 5619 "AppWindow.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == GPHOTO_ERROR) {
				goto __catch7_gphoto_error;
			}
			goto __finally7;
		}
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	}
	goto __finally7;
	__catch7_gphoto_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 1566 "AppWindow.vala"
			g_debug ("AppWindow.vala:1566: Error updating camera table: %s", err->message);
#line 5637 "AppWindow.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally7:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
#line 1569 "AppWindow.vala"
	app_window_camera_update_scheduled = FALSE;
#line 5649 "AppWindow.c"
	result = FALSE;
	return result;
}


#line 1574 "AppWindow.vala"
gboolean app_window_is_mount_uri_supported (const char* uri) {
#line 5657 "AppWindow.c"
	gboolean result;
#line 1574 "AppWindow.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 5661 "AppWindow.c"
	{
		const char** scheme_collection;
		int scheme_collection_length1;
		int scheme_it;
#line 1575 "AppWindow.vala"
		scheme_collection = APP_WINDOW_SUPPORTED_MOUNT_SCHEMES;
#line 5668 "AppWindow.c"
		scheme_collection_length1 = G_N_ELEMENTS (APP_WINDOW_SUPPORTED_MOUNT_SCHEMES);
		for (scheme_it = 0; scheme_it < G_N_ELEMENTS (APP_WINDOW_SUPPORTED_MOUNT_SCHEMES); scheme_it = scheme_it + 1) {
			const char* _tmp0_;
			char* scheme;
#line 227 "AppWindow.vala"
			_tmp0_ = NULL;
#line 5675 "AppWindow.c"
			scheme = (_tmp0_ = scheme_collection[scheme_it], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
			{
#line 1576 "AppWindow.vala"
				if (g_str_has_prefix (uri, scheme)) {
#line 5680 "AppWindow.c"
					result = TRUE;
					scheme = (g_free (scheme), NULL);
					return result;
				}
				scheme = (g_free (scheme), NULL);
			}
		}
	}
	result = FALSE;
	return result;
}


#line 942 "glib-2.0.vapi"
static char* string_replace (const char* self, const char* old, const char* replacement) {
#line 5696 "AppWindow.c"
	char* result;
	GError * _inner_error_;
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 5705 "AppWindow.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
#line 2667 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 944 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 5716 "AppWindow.c"
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch9_g_regex_error;
			}
			goto __finally9;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch9_g_regex_error;
			}
			goto __finally9;
		}
		result = _tmp2_;
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		return result;
	}
	goto __finally9;
	__catch9_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 947 "glib-2.0.vapi"
			g_assert_not_reached ();
#line 5745 "AppWindow.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally9:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


#line 1583 "AppWindow.vala"
void app_window_mounted_camera_shell_notification (const char* uri) {
#line 5760 "AppWindow.c"
	GError * _inner_error_;
	GFile* uri_file;
	GMount* mount;
	char* alt_uri;
	char* _tmp5_;
	AppWindow* _tmp4_;
	ImportPage* _tmp6_;
	ImportPage* page;
	gboolean _tmp7_;
#line 1583 "AppWindow.vala"
	g_return_if_fail (uri != NULL);
#line 5772 "AppWindow.c"
	_inner_error_ = NULL;
#line 1584 "AppWindow.vala"
	g_debug ("AppWindow.vala:1584: mount point reported: %s", uri);
#line 1586 "AppWindow.vala"
	if (!app_window_is_mount_uri_supported (uri)) {
#line 1587 "AppWindow.vala"
		g_debug ("AppWindow.vala:1587: Unsupported mount scheme: %s", uri);
#line 5780 "AppWindow.c"
		return;
	}
	uri_file = g_file_new_for_uri (uri);
	mount = NULL;
	{
		GMount* _tmp0_;
		GMount* _tmp1_;
		_tmp0_ = g_file_find_enclosing_mount (uri_file, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch8_g_error;
			goto __finally8;
		}
		_tmp1_ = NULL;
#line 1596 "AppWindow.vala"
		mount = (_tmp1_ = _tmp0_, (mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL)), _tmp1_);
#line 5796 "AppWindow.c"
	}
	goto __finally8;
	__catch8_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
#line 1598 "AppWindow.vala"
			g_debug ("AppWindow.vala:1598: %s", err->message);
#line 5807 "AppWindow.c"
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			(uri_file == NULL) ? NULL : (uri_file = (g_object_unref (uri_file), NULL));
			(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
			return;
		}
	}
	__finally8:
	if (_inner_error_ != NULL) {
		(uri_file == NULL) ? NULL : (uri_file = (g_object_unref (uri_file), NULL));
		(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* convert file: URIs into gphoto disk: URIs*/
	alt_uri = NULL;
#line 1605 "AppWindow.vala"
	if (g_str_has_prefix (uri, "file://")) {
#line 5826 "AppWindow.c"
		char* _tmp3_;
		char* _tmp2_;
		_tmp3_ = NULL;
#line 942 "glib-2.0.vapi"
		_tmp2_ = NULL;
#line 1606 "AppWindow.vala"
		alt_uri = (_tmp3_ = app_window_get_port_uri (_tmp2_ = string_replace (uri, "file://", "disk:")), alt_uri = (g_free (alt_uri), NULL), _tmp3_);
#line 5834 "AppWindow.c"
		_tmp2_ = (g_free (_tmp2_), NULL);
	}
#line 954 "gio-2.0.vapi"
	_tmp5_ = NULL;
#line 446 "AppWindow.vala"
	_tmp4_ = NULL;
#line 1608 "AppWindow.vala"
	_tmp6_ = NULL;
#line 5843 "AppWindow.c"
	page = (_tmp6_ = (ImportPage*) gee_abstract_map_get (GEE_ABSTRACT_MAP ((_tmp4_ = app_window_get_instance ())->priv->camera_map), _tmp5_ = g_file_get_uri (uri_file)), _tmp5_ = (g_free (_tmp5_), NULL), (_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL)), _tmp6_);
	_tmp7_ = FALSE;
#line 1609 "AppWindow.vala"
	if (page == NULL) {
#line 1609 "AppWindow.vala"
		_tmp7_ = alt_uri != NULL;
#line 5850 "AppWindow.c"
	} else {
#line 1609 "AppWindow.vala"
		_tmp7_ = FALSE;
#line 5854 "AppWindow.c"
	}
#line 1609 "AppWindow.vala"
	if (_tmp7_) {
#line 5858 "AppWindow.c"
		ImportPage* _tmp9_;
		AppWindow* _tmp8_;
		_tmp9_ = NULL;
#line 446 "AppWindow.vala"
		_tmp8_ = NULL;
#line 1610 "AppWindow.vala"
		page = (_tmp9_ = (ImportPage*) gee_abstract_map_get (GEE_ABSTRACT_MAP ((_tmp8_ = app_window_get_instance ())->priv->camera_map), alt_uri), (page == NULL) ? NULL : (page = (g_object_unref (page), NULL)), _tmp9_);
#line 5866 "AppWindow.c"
		(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
	}
#line 1612 "AppWindow.vala"
	if (page == NULL) {
#line 5871 "AppWindow.c"
		char* _tmp10_;
#line 954 "gio-2.0.vapi"
		_tmp10_ = NULL;
#line 1613 "AppWindow.vala"
		g_debug ("AppWindow.vala:1613: Unable to find import page for %s", _tmp10_ = g_file_get_uri (uri_file));
#line 5877 "AppWindow.c"
		_tmp10_ = (g_free (_tmp10_), NULL);
		(uri_file == NULL) ? NULL : (uri_file = (g_object_unref (uri_file), NULL));
		(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
		alt_uri = (g_free (alt_uri), NULL);
		(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		return;
	}
	/* don't unmount mass storage cameras, as they are then unavailble to gPhoto*/
#line 1619 "AppWindow.vala"
	if (!g_str_has_prefix (uri, "file://")) {
#line 1620 "AppWindow.vala"
		if (!import_page_unmount_camera (page, mount)) {
#line 1621 "AppWindow.vala"
			app_window_error_message ("Unable to unmount the camera at this time.");
#line 5892 "AppWindow.c"
		}
	} else {
#line 1623 "AppWindow.vala"
		import_page_switch_to_and_refresh (page);
#line 5897 "AppWindow.c"
	}
	(uri_file == NULL) ? NULL : (uri_file = (g_object_unref (uri_file), NULL));
	(mount == NULL) ? NULL : (mount = (g_object_unref (mount), NULL));
	alt_uri = (g_free (alt_uri), NULL);
	(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
}


#line 273 "AppWindow.vala"
static AppWindowFileImportJob* app_window_file_import_job_construct (GType object_type, const char* uri, gboolean copy_to_library) {
#line 5908 "AppWindow.c"
	AppWindowFileImportJob* self;
	GFile* _tmp0_;
#line 273 "AppWindow.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 5913 "AppWindow.c"
	self = (AppWindowFileImportJob*) g_type_create_instance (object_type);
	_tmp0_ = NULL;
#line 274 "AppWindow.vala"
	self->priv->file_or_dir = (_tmp0_ = g_file_new_for_uri (uri), (self->priv->file_or_dir == NULL) ? NULL : (self->priv->file_or_dir = (g_object_unref (self->priv->file_or_dir), NULL)), _tmp0_);
#line 275 "AppWindow.vala"
	self->priv->copy_to_library = copy_to_library;
#line 5920 "AppWindow.c"
	return self;
}


#line 273 "AppWindow.vala"
static AppWindowFileImportJob* app_window_file_import_job_new (const char* uri, gboolean copy_to_library) {
#line 273 "AppWindow.vala"
	return app_window_file_import_job_construct (APP_WINDOW_TYPE_FILE_IMPORT_JOB, uri, copy_to_library);
#line 5929 "AppWindow.c"
}


#line 278 "AppWindow.vala"
static char* app_window_file_import_job_real_get_identifier (BatchImportJob* base) {
#line 5935 "AppWindow.c"
	AppWindowFileImportJob * self;
	char* result;
	self = APP_WINDOW_FILE_IMPORT_JOB (base);
	result = g_file_get_uri (self->priv->file_or_dir);
	return result;
}


#line 282 "AppWindow.vala"
static gboolean app_window_file_import_job_real_prepare (BatchImportJob* base, GFile** file_to_import, gboolean* copy) {
#line 5946 "AppWindow.c"
	AppWindowFileImportJob * self;
	gboolean result;
	GFile* _tmp1_;
	GFile* _tmp0_;
	self = APP_WINDOW_FILE_IMPORT_JOB (base);
	if (file_to_import != NULL) {
		*file_to_import = NULL;
	}
	_tmp1_ = NULL;
#line 283 "AppWindow.vala"
	_tmp0_ = NULL;
#line 283 "AppWindow.vala"
	(*file_to_import) = (_tmp1_ = (_tmp0_ = self->priv->file_or_dir, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), ((*file_to_import) == NULL) ? NULL : ((*file_to_import) = (g_object_unref ((*file_to_import)), NULL)), _tmp1_);
#line 284 "AppWindow.vala"
	(*copy) = self->priv->copy_to_library;
#line 5962 "AppWindow.c"
	result = TRUE;
	return result;
}


static void app_window_file_import_job_class_init (AppWindowFileImportJobClass * klass) {
	app_window_file_import_job_parent_class = g_type_class_peek_parent (klass);
	BATCH_IMPORT_JOB_CLASS (klass)->finalize = app_window_file_import_job_finalize;
	g_type_class_add_private (klass, sizeof (AppWindowFileImportJobPrivate));
	BATCH_IMPORT_JOB_CLASS (klass)->get_identifier = app_window_file_import_job_real_get_identifier;
	BATCH_IMPORT_JOB_CLASS (klass)->prepare = app_window_file_import_job_real_prepare;
}


static void app_window_file_import_job_instance_init (AppWindowFileImportJob * self) {
	self->priv = APP_WINDOW_FILE_IMPORT_JOB_GET_PRIVATE (self);
}


static void app_window_file_import_job_finalize (BatchImportJob* obj) {
	AppWindowFileImportJob * self;
	self = APP_WINDOW_FILE_IMPORT_JOB (obj);
	(self->priv->file_or_dir == NULL) ? NULL : (self->priv->file_or_dir = (g_object_unref (self->priv->file_or_dir), NULL));
	BATCH_IMPORT_JOB_CLASS (app_window_file_import_job_parent_class)->finalize (obj);
}


static GType app_window_file_import_job_get_type (void) {
	static GType app_window_file_import_job_type_id = 0;
	if (app_window_file_import_job_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AppWindowFileImportJobClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) app_window_file_import_job_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppWindowFileImportJob), 0, (GInstanceInitFunc) app_window_file_import_job_instance_init, NULL };
		app_window_file_import_job_type_id = g_type_register_static (TYPE_BATCH_IMPORT_JOB, "AppWindowFileImportJob", &g_define_type_info, 0);
	}
	return app_window_file_import_job_type_id;
}


#line 294 "AppWindow.vala"
static AppWindowCompareEventPage* app_window_compare_event_page_construct (GType object_type, gint event_sort) {
#line 6002 "AppWindow.c"
	AppWindowCompareEventPage* self;
	gboolean _tmp0_;
	self = (AppWindowCompareEventPage*) g_type_create_instance (object_type);
	_tmp0_ = FALSE;
#line 295 "AppWindow.vala"
	if (event_sort == APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING) {
#line 295 "AppWindow.vala"
		_tmp0_ = TRUE;
#line 6011 "AppWindow.c"
	} else {
#line 295 "AppWindow.vala"
		_tmp0_ = event_sort == APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING;
#line 6015 "AppWindow.c"
	}
#line 295 "AppWindow.vala"
	g_assert (_tmp0_);
#line 297 "AppWindow.vala"
	self->priv->event_sort = event_sort;
#line 6021 "AppWindow.c"
	return self;
}


#line 294 "AppWindow.vala"
static AppWindowCompareEventPage* app_window_compare_event_page_new (gint event_sort) {
#line 294 "AppWindow.vala"
	return app_window_compare_event_page_construct (APP_WINDOW_TYPE_COMPARE_EVENT_PAGE, event_sort);
#line 6030 "AppWindow.c"
}


#line 300 "AppWindow.vala"
static gint64 app_window_compare_event_page_real_compare (Comparator* base, EventPage* a, EventPage* b) {
#line 6036 "AppWindow.c"
	AppWindowCompareEventPage * self;
	gint64 result;
	gint64 start_a;
	gint64 start_b;
	self = APP_WINDOW_COMPARE_EVENT_PAGE (base);
#line 300 "AppWindow.vala"
	g_return_val_if_fail (IS_EVENT_PAGE (a), 0LL);
#line 300 "AppWindow.vala"
	g_return_val_if_fail (IS_EVENT_PAGE (b), 0LL);
#line 6046 "AppWindow.c"
	start_a = (gint64) event_table_get_start_time (self->priv->event_table, &a->event_id);
	start_b = (gint64) event_table_get_start_time (self->priv->event_table, &b->event_id);
#line 304 "AppWindow.vala"
	switch (self->priv->event_sort) {
#line 6051 "AppWindow.c"
		case APP_WINDOW_SORT_EVENTS_ORDER_ASCENDING:
		{
			result = start_a - start_b;
			return result;
		}
		default:
		{
			result = start_b - start_a;
			return result;
		}
	}
}


static void app_window_compare_event_page_class_init (AppWindowCompareEventPageClass * klass) {
	app_window_compare_event_page_parent_class = g_type_class_peek_parent (klass);
	COMPARATOR_CLASS (klass)->finalize = app_window_compare_event_page_finalize;
	g_type_class_add_private (klass, sizeof (AppWindowCompareEventPagePrivate));
	COMPARATOR_CLASS (klass)->compare = app_window_compare_event_page_real_compare;
}


static void app_window_compare_event_page_instance_init (AppWindowCompareEventPage * self) {
	self->priv = APP_WINDOW_COMPARE_EVENT_PAGE_GET_PRIVATE (self);
	self->priv->event_table = event_table_new ();
}


static void app_window_compare_event_page_finalize (Comparator* obj) {
	AppWindowCompareEventPage * self;
	self = APP_WINDOW_COMPARE_EVENT_PAGE (obj);
	(self->priv->event_table == NULL) ? NULL : (self->priv->event_table = (database_table_unref (self->priv->event_table), NULL));
	COMPARATOR_CLASS (app_window_compare_event_page_parent_class)->finalize (obj);
}


static GType app_window_compare_event_page_get_type (void) {
	static GType app_window_compare_event_page_type_id = 0;
	if (app_window_compare_event_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AppWindowCompareEventPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) app_window_compare_event_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppWindowCompareEventPage), 0, (GInstanceInitFunc) app_window_compare_event_page_instance_init, NULL };
		app_window_compare_event_page_type_id = g_type_register_static (TYPE_COMPARATOR, "AppWindowCompareEventPage", &g_define_type_info, 0);
	}
	return app_window_compare_event_page_type_id;
}


static void app_window_class_init (AppWindowClass * klass) {
	app_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AppWindowPrivate));
	GTK_OBJECT_CLASS (klass)->destroy = app_window_real_destroy;
	GTK_WIDGET_CLASS (klass)->drag_data_received = app_window_real_drag_data_received;
	G_OBJECT_CLASS (klass)->finalize = app_window_finalize;
	app_window_BG_COLOR = parse_color ("#444");
	app_window_SIDEBAR_BG_COLOR = parse_color ("#EEE");
}


static void app_window_instance_init (AppWindow * self) {
	const char* _tmp0_;
	self->priv = APP_WINDOW_GET_PRIVATE (self);
	self->priv->import_dir = (_tmp0_ = g_get_home_dir (), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
#line 267 "AppWindow.vala"
	_tmp0_ = NULL;
#line 6115 "AppWindow.c"
	self->priv->events_sort = APP_WINDOW_SORT_EVENTS_ORDER_DESCENDING;
	self->priv->hal_context = libhal_ctx_new ();
	self->priv->hal_conn = NULL;
	self->priv->collection_page = NULL;
	self->priv->events_directory_page = NULL;
	self->priv->photo_page = NULL;
	self->priv->import_queue_page = NULL;
	self->priv->event_list = gee_array_list_new (TYPE_EVENT_PAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->camera_map = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, TYPE_IMPORT_PAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_str_hash, g_str_equal, g_direct_equal);
	self->priv->pages_to_be_removed = gee_array_list_new (TYPE_PAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->photo_table = photo_table_new ();
	self->priv->event_table = event_table_new ();
	self->priv->sidebar = g_object_ref_sink (sidebar_new ());
	self->priv->cameras_marker = NULL;
	self->priv->notebook = g_object_ref_sink ((GtkNotebook*) gtk_notebook_new ());
	self->priv->layout = GTK_BOX (g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)));
	self->priv->current_page = NULL;
	self->priv->null_context = gp_context_new ();
	self->priv->fullscreen_window = NULL;
}


static void app_window_finalize (GObject* obj) {
	AppWindow * self;
	self = APP_WINDOW (obj);
	self->priv->import_dir = (g_free (self->priv->import_dir), NULL);
	(self->priv->hal_context == NULL) ? NULL : (self->priv->hal_context = (libhal_ctx_free (self->priv->hal_context), NULL));
	(self->priv->hal_conn == NULL) ? NULL : (self->priv->hal_conn = (dbus_g_connection_unref (self->priv->hal_conn), NULL));
	(self->priv->collection_page == NULL) ? NULL : (self->priv->collection_page = (g_object_unref (self->priv->collection_page), NULL));
	(self->priv->events_directory_page == NULL) ? NULL : (self->priv->events_directory_page = (g_object_unref (self->priv->events_directory_page), NULL));
	(self->priv->photo_page == NULL) ? NULL : (self->priv->photo_page = (g_object_unref (self->priv->photo_page), NULL));
	(self->priv->import_queue_page == NULL) ? NULL : (self->priv->import_queue_page = (g_object_unref (self->priv->import_queue_page), NULL));
	(self->priv->event_list == NULL) ? NULL : (self->priv->event_list = (g_object_unref (self->priv->event_list), NULL));
	(self->priv->camera_map == NULL) ? NULL : (self->priv->camera_map = (g_object_unref (self->priv->camera_map), NULL));
	(self->priv->pages_to_be_removed == NULL) ? NULL : (self->priv->pages_to_be_removed = (g_object_unref (self->priv->pages_to_be_removed), NULL));
	(self->priv->photo_table == NULL) ? NULL : (self->priv->photo_table = (database_table_unref (self->priv->photo_table), NULL));
	(self->priv->event_table == NULL) ? NULL : (self->priv->event_table = (database_table_unref (self->priv->event_table), NULL));
	(self->priv->sidebar == NULL) ? NULL : (self->priv->sidebar = (g_object_unref (self->priv->sidebar), NULL));
	(self->priv->cameras_marker == NULL) ? NULL : (self->priv->cameras_marker = (sidebar_marker_unref (self->priv->cameras_marker), NULL));
	(self->priv->notebook == NULL) ? NULL : (self->priv->notebook = (g_object_unref (self->priv->notebook), NULL));
	(self->priv->layout == NULL) ? NULL : (self->priv->layout = (g_object_unref (self->priv->layout), NULL));
	(self->priv->current_page == NULL) ? NULL : (self->priv->current_page = (g_object_unref (self->priv->current_page), NULL));
	(self->priv->null_context == NULL) ? NULL : (self->priv->null_context = (gp_context_unref (self->priv->null_context), NULL));
	(self->priv->abilities_list == NULL) ? NULL : (self->priv->abilities_list = (gp_abilities_list_free (self->priv->abilities_list), NULL));
	(self->priv->fullscreen_window == NULL) ? NULL : (self->priv->fullscreen_window = (g_object_unref (self->priv->fullscreen_window), NULL));
	G_OBJECT_CLASS (app_window_parent_class)->finalize (obj);
}


GType app_window_get_type (void) {
	static GType app_window_type_id = 0;
	if (app_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AppWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) app_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppWindow), 0, (GInstanceInitFunc) app_window_instance_init, NULL };
		app_window_type_id = g_type_register_static (GTK_TYPE_WINDOW, "AppWindow", &g_define_type_info, 0);
	}
	return app_window_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




