/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>


#define TYPE_EDITING_TOOL_WINDOW (editing_tool_window_get_type ())
#define EDITING_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EDITING_TOOL_WINDOW, EditingToolWindow))
#define EDITING_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EDITING_TOOL_WINDOW, EditingToolWindowClass))
#define IS_EDITING_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EDITING_TOOL_WINDOW))
#define IS_EDITING_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EDITING_TOOL_WINDOW))
#define EDITING_TOOL_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EDITING_TOOL_WINDOW, EditingToolWindowClass))

typedef struct _EditingToolWindow EditingToolWindow;
typedef struct _EditingToolWindowClass EditingToolWindowClass;
typedef struct _EditingToolWindowPrivate EditingToolWindowPrivate;

#define TYPE_PHOTO_CANVAS (photo_canvas_get_type ())
#define PHOTO_CANVAS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO_CANVAS, PhotoCanvas))
#define PHOTO_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO_CANVAS, PhotoCanvasClass))
#define IS_PHOTO_CANVAS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO_CANVAS))
#define IS_PHOTO_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO_CANVAS))
#define PHOTO_CANVAS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO_CANVAS, PhotoCanvasClass))

typedef struct _PhotoCanvas PhotoCanvas;
typedef struct _PhotoCanvasClass PhotoCanvasClass;
typedef struct _PhotoCanvasPrivate PhotoCanvasPrivate;

#define TYPE_PHOTO (photo_get_type ())
#define PHOTO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PHOTO, Photo))
#define PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PHOTO, PhotoClass))
#define IS_PHOTO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PHOTO))
#define IS_PHOTO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PHOTO))
#define PHOTO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PHOTO, PhotoClass))

typedef struct _Photo Photo;
typedef struct _PhotoClass PhotoClass;

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

#define TYPE_BOX (box_get_type ())
typedef struct _Box Box;
typedef struct _ParamSpecPhotoCanvas ParamSpecPhotoCanvas;

#define TYPE_EDITING_TOOL (editing_tool_get_type ())
#define EDITING_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EDITING_TOOL, EditingTool))
#define EDITING_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EDITING_TOOL, EditingToolClass))
#define IS_EDITING_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EDITING_TOOL))
#define IS_EDITING_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EDITING_TOOL))
#define EDITING_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EDITING_TOOL, EditingToolClass))

typedef struct _EditingTool EditingTool;
typedef struct _EditingToolClass EditingToolClass;
typedef struct _EditingToolPrivate EditingToolPrivate;
typedef struct _ParamSpecEditingTool ParamSpecEditingTool;

#define TYPE_CROP_TOOL (crop_tool_get_type ())
#define CROP_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CROP_TOOL, CropTool))
#define CROP_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CROP_TOOL, CropToolClass))
#define IS_CROP_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CROP_TOOL))
#define IS_CROP_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CROP_TOOL))
#define CROP_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CROP_TOOL, CropToolClass))

typedef struct _CropTool CropTool;
typedef struct _CropToolClass CropToolClass;
typedef struct _CropToolPrivate CropToolPrivate;

#define CROP_TOOL_TYPE_CROP_TOOL_WINDOW (crop_tool_crop_tool_window_get_type ())
#define CROP_TOOL_CROP_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CROP_TOOL_TYPE_CROP_TOOL_WINDOW, CropToolCropToolWindow))
#define CROP_TOOL_CROP_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CROP_TOOL_TYPE_CROP_TOOL_WINDOW, CropToolCropToolWindowClass))
#define CROP_TOOL_IS_CROP_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CROP_TOOL_TYPE_CROP_TOOL_WINDOW))
#define CROP_TOOL_IS_CROP_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CROP_TOOL_TYPE_CROP_TOOL_WINDOW))
#define CROP_TOOL_CROP_TOOL_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CROP_TOOL_TYPE_CROP_TOOL_WINDOW, CropToolCropToolWindowClass))

typedef struct _CropToolCropToolWindow CropToolCropToolWindow;
typedef struct _CropToolCropToolWindowClass CropToolCropToolWindowClass;

#define TYPE_BOX_LOCATION (box_location_get_type ())
typedef struct _CropToolCropToolWindowPrivate CropToolCropToolWindowPrivate;

#define TYPE_BOX_COMPLEMENTS (box_complements_get_type ())

#define TYPE_REDEYE_INSTANCE (redeye_instance_get_type ())
typedef struct _RedeyeInstance RedeyeInstance;

#define TYPE_REDEYE_TOOL (redeye_tool_get_type ())
#define REDEYE_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_REDEYE_TOOL, RedeyeTool))
#define REDEYE_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_REDEYE_TOOL, RedeyeToolClass))
#define IS_REDEYE_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_REDEYE_TOOL))
#define IS_REDEYE_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_REDEYE_TOOL))
#define REDEYE_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_REDEYE_TOOL, RedeyeToolClass))

typedef struct _RedeyeTool RedeyeTool;
typedef struct _RedeyeToolClass RedeyeToolClass;
typedef struct _RedeyeToolPrivate RedeyeToolPrivate;

#define REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW (redeye_tool_redeye_tool_window_get_type ())
#define REDEYE_TOOL_REDEYE_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW, RedeyeToolRedeyeToolWindow))
#define REDEYE_TOOL_REDEYE_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW, RedeyeToolRedeyeToolWindowClass))
#define REDEYE_TOOL_IS_REDEYE_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW))
#define REDEYE_TOOL_IS_REDEYE_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW))
#define REDEYE_TOOL_REDEYE_TOOL_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW, RedeyeToolRedeyeToolWindowClass))

typedef struct _RedeyeToolRedeyeToolWindow RedeyeToolRedeyeToolWindow;
typedef struct _RedeyeToolRedeyeToolWindowClass RedeyeToolRedeyeToolWindowClass;
typedef struct _RedeyeToolRedeyeToolWindowPrivate RedeyeToolRedeyeToolWindowPrivate;

#define TYPE_ADJUST_TOOL (adjust_tool_get_type ())
#define ADJUST_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ADJUST_TOOL, AdjustTool))
#define ADJUST_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ADJUST_TOOL, AdjustToolClass))
#define IS_ADJUST_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ADJUST_TOOL))
#define IS_ADJUST_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ADJUST_TOOL))
#define ADJUST_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ADJUST_TOOL, AdjustToolClass))

typedef struct _AdjustTool AdjustTool;
typedef struct _AdjustToolClass AdjustToolClass;
typedef struct _AdjustToolPrivate AdjustToolPrivate;

#define ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW (adjust_tool_adjust_tool_window_get_type ())
#define ADJUST_TOOL_ADJUST_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW, AdjustToolAdjustToolWindow))
#define ADJUST_TOOL_ADJUST_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW, AdjustToolAdjustToolWindowClass))
#define ADJUST_TOOL_IS_ADJUST_TOOL_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW))
#define ADJUST_TOOL_IS_ADJUST_TOOL_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW))
#define ADJUST_TOOL_ADJUST_TOOL_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW, AdjustToolAdjustToolWindowClass))

typedef struct _AdjustToolAdjustToolWindow AdjustToolAdjustToolWindow;
typedef struct _AdjustToolAdjustToolWindowClass AdjustToolAdjustToolWindowClass;

#define TYPE_COLOR_TRANSFORMATION_KIND (color_transformation_kind_get_type ())
typedef struct _AdjustToolAdjustToolWindowPrivate AdjustToolAdjustToolWindowPrivate;

#define TYPE_IMAGE_HISTOGRAM (image_histogram_get_type ())
#define IMAGE_HISTOGRAM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_IMAGE_HISTOGRAM, ImageHistogram))
#define IMAGE_HISTOGRAM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_IMAGE_HISTOGRAM, ImageHistogramClass))
#define IS_IMAGE_HISTOGRAM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_IMAGE_HISTOGRAM))
#define IS_IMAGE_HISTOGRAM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_IMAGE_HISTOGRAM))
#define IMAGE_HISTOGRAM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_IMAGE_HISTOGRAM, ImageHistogramClass))

typedef struct _ImageHistogram ImageHistogram;
typedef struct _ImageHistogramClass ImageHistogramClass;

#define TYPE_COLOR_TRANSFORMATION_FACTORY (color_transformation_factory_get_type ())
#define COLOR_TRANSFORMATION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactory))
#define COLOR_TRANSFORMATION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactoryClass))
#define IS_COLOR_TRANSFORMATION_FACTORY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY))
#define IS_COLOR_TRANSFORMATION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_TRANSFORMATION_FACTORY))
#define COLOR_TRANSFORMATION_FACTORY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLOR_TRANSFORMATION_FACTORY, ColorTransformationFactoryClass))

typedef struct _ColorTransformationFactory ColorTransformationFactory;
typedef struct _ColorTransformationFactoryClass ColorTransformationFactoryClass;

#define TYPE_COLOR_TRANSFORMATION (color_transformation_get_type ())
#define COLOR_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLOR_TRANSFORMATION, ColorTransformation))
#define COLOR_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLOR_TRANSFORMATION, ColorTransformationClass))
#define IS_COLOR_TRANSFORMATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLOR_TRANSFORMATION))
#define IS_COLOR_TRANSFORMATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_TRANSFORMATION))
#define COLOR_TRANSFORMATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLOR_TRANSFORMATION, ColorTransformationClass))

typedef struct _ColorTransformation ColorTransformation;
typedef struct _ColorTransformationClass ColorTransformationClass;

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_COLOR_TRANSFORMATION_INSTANCE (color_transformation_instance_get_type ())
typedef struct _ColorTransformationInstance ColorTransformationInstance;

struct _EditingToolWindow {
	GtkWindow parent_instance;
	EditingToolWindowPrivate * priv;
};

struct _EditingToolWindowClass {
	GtkWindowClass parent_class;
};

struct _EditingToolWindowPrivate {
	GtkWindow* container;
	GtkFrame* layout_frame;
	gboolean user_moved;
};

/* The PhotoCanvas is an interface object between an EditingTool and its host.  It provides objects
 and primitives for an EditingTool to obtain information about the image, to draw on the host's
 canvas, and to be signalled when the canvas and its pixbuf changes (is resized).*/
struct _PhotoCanvas {
	GTypeInstance parent_instance;
	volatile int ref_count;
	PhotoCanvasPrivate * priv;
};

struct _PhotoCanvasClass {
	GTypeClass parent_class;
	void (*finalize) (PhotoCanvas *self);
	void (*repaint) (PhotoCanvas* self);
};

struct _PhotoCanvasPrivate {
	GtkWindow* container;
	GdkWindow* drawing_window;
	Photo* photo;
	GdkGC* default_gc;
	GdkDrawable* drawable;
	GdkPixbuf* scaled;
	GdkRectangle scaled_position;
};

struct _Dimensions {
	gint width;
	gint height;
};

struct _Box {
	gint left;
	gint top;
	gint right;
	gint bottom;
};

struct _ParamSpecPhotoCanvas {
	GParamSpec parent_instance;
};

struct _EditingTool {
	GTypeInstance parent_instance;
	volatile int ref_count;
	EditingToolPrivate * priv;
	PhotoCanvas* canvas;
};

struct _EditingToolClass {
	GTypeClass parent_class;
	void (*finalize) (EditingTool *self);
	void (*activate) (EditingTool* self, PhotoCanvas* canvas);
	void (*deactivate) (EditingTool* self);
	EditingToolWindow* (*get_tool_window) (EditingTool* self);
	GdkPixbuf* (*get_unscaled_pixbuf) (EditingTool* self, Photo* photo);
	void (*on_left_click) (EditingTool* self, gint x, gint y);
	void (*on_left_released) (EditingTool* self, gint x, gint y);
	void (*on_motion) (EditingTool* self, gint x, gint y, GdkModifierType mask);
	gboolean (*on_keypress) (EditingTool* self, const GdkEventKey* event);
	void (*paint) (EditingTool* self, GdkGC* gc, GdkDrawable* drawable);
};

struct _ParamSpecEditingTool {
	GParamSpec parent_instance;
};

struct _CropTool {
	EditingTool parent_instance;
	CropToolPrivate * priv;
};

struct _CropToolClass {
	EditingToolClass parent_class;
};

typedef enum  {
	BOX_LOCATION_OUTSIDE,
	BOX_LOCATION_INSIDE,
	BOX_LOCATION_TOP_SIDE,
	BOX_LOCATION_LEFT_SIDE,
	BOX_LOCATION_RIGHT_SIDE,
	BOX_LOCATION_BOTTOM_SIDE,
	BOX_LOCATION_TOP_LEFT,
	BOX_LOCATION_BOTTOM_LEFT,
	BOX_LOCATION_TOP_RIGHT,
	BOX_LOCATION_BOTTOM_RIGHT
} BoxLocation;

struct _CropToolPrivate {
	CropToolCropToolWindow* crop_tool_window;
	GdkPixbuf* color_shifted;
	GdkCursorType current_cursor_type;
	BoxLocation in_manipulation;
	GdkGC* wide_black_gc;
	GdkGC* wide_white_gc;
	GdkGC* thin_white_gc;
	Box scaled_crop;
	gint last_grab_x;
	gint last_grab_y;
};

struct _CropToolCropToolWindow {
	EditingToolWindow parent_instance;
	CropToolCropToolWindowPrivate * priv;
	GtkButton* apply_button;
	GtkButton* cancel_button;
};

struct _CropToolCropToolWindowClass {
	EditingToolWindowClass parent_class;
};

typedef enum  {
	BOX_COMPLEMENTS_NONE,
	BOX_COMPLEMENTS_VERTICAL,
	BOX_COMPLEMENTS_HORIZONTAL,
	BOX_COMPLEMENTS_BOTH
} BoxComplements;

struct _RedeyeInstance {
	GdkPoint center;
	gint radius;
};

struct _RedeyeTool {
	EditingTool parent_instance;
	RedeyeToolPrivate * priv;
};

struct _RedeyeToolClass {
	EditingToolClass parent_class;
};

struct _RedeyeToolPrivate {
	GdkGC* thin_white_gc;
	GdkGC* wider_gray_gc;
	RedeyeToolRedeyeToolWindow* redeye_tool_window;
	RedeyeInstance user_interaction_instance;
	gboolean is_reticle_move_in_progress;
	GdkPoint reticle_move_mouse_start_point;
	GdkPoint reticle_move_anchor;
	GdkCursor* cached_arrow_cursor;
	GdkCursor* cached_grab_cursor;
	GdkRectangle old_scaled_pixbuf_position;
};

struct _RedeyeToolRedeyeToolWindow {
	EditingToolWindow parent_instance;
	RedeyeToolRedeyeToolWindowPrivate * priv;
	GtkButton* apply_button;
	GtkButton* close_button;
	GtkHScale* slider;
};

struct _RedeyeToolRedeyeToolWindowClass {
	EditingToolWindowClass parent_class;
};

struct _RedeyeToolRedeyeToolWindowPrivate {
	GtkLabel* slider_label;
};

struct _AdjustTool {
	EditingTool parent_instance;
	AdjustToolPrivate * priv;
};

struct _AdjustToolClass {
	EditingToolClass parent_class;
};

struct _AdjustToolPrivate {
	AdjustToolAdjustToolWindow* adjust_tool_window;
	gboolean suppress_effect_redraw;
	GdkPixbuf* draw_to_pixbuf;
};

typedef enum  {
	COLOR_TRANSFORMATION_KIND_EXPOSURE,
	COLOR_TRANSFORMATION_KIND_SATURATION,
	COLOR_TRANSFORMATION_KIND_TINT,
	COLOR_TRANSFORMATION_KIND_TEMPERATURE
} ColorTransformationKind;

struct _AdjustToolAdjustToolWindow {
	EditingToolWindow parent_instance;
	AdjustToolAdjustToolWindowPrivate * priv;
	GtkHScale* exposure_slider;
	GtkHScale* saturation_slider;
	GtkHScale* tint_slider;
	GtkHScale* temperature_slider;
	GtkButton* apply_button;
	GtkButton* reset_button;
	GtkButton* cancel_button;
	GtkImage* histogram_image_tray;
};

struct _AdjustToolAdjustToolWindowClass {
	EditingToolWindowClass parent_class;
};

struct _ColorTransformationInstance {
	ColorTransformationKind kind;
	float parameter;
};


static gpointer editing_tool_window_parent_class = NULL;
static gpointer photo_canvas_parent_class = NULL;
static gpointer editing_tool_parent_class = NULL;
static gpointer crop_tool_crop_tool_window_parent_class = NULL;
static gpointer crop_tool_parent_class = NULL;
static gpointer redeye_tool_redeye_tool_window_parent_class = NULL;
static gpointer redeye_tool_parent_class = NULL;
static gpointer adjust_tool_adjust_tool_window_parent_class = NULL;
static gpointer adjust_tool_parent_class = NULL;

GType editing_tool_window_get_type (void);
#define EDITING_TOOL_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_EDITING_TOOL_WINDOW, EditingToolWindowPrivate))
enum  {
	EDITING_TOOL_WINDOW_DUMMY_PROPERTY
};
#define EDITING_TOOL_WINDOW_FRAME_BORDER 6
EditingToolWindow* editing_tool_window_new (GtkWindow* container);
EditingToolWindow* editing_tool_window_construct (GType object_type, GtkWindow* container);
static void editing_tool_window_real_add (GtkContainer* base, GtkWidget* widget);
gboolean editing_tool_window_has_user_moved (EditingToolWindow* self);
static gboolean editing_tool_window_real_button_press_event (GtkWidget* base, const GdkEventButton* event);
#define FULLSCREEN_WINDOW_TOOLBAR_OPACITY 0.75
static void editing_tool_window_real_realize (GtkWidget* base);
static gboolean editing_tool_window_unsteal_focus (EditingToolWindow* self);
static gboolean _editing_tool_window_unsteal_focus_gsource_func (gpointer self);
static gboolean editing_tool_window_real_focus (GtkWidget* base, GtkDirectionType direction);
static void editing_tool_window_finalize (GObject* obj);
gpointer photo_canvas_ref (gpointer instance);
void photo_canvas_unref (gpointer instance);
GParamSpec* param_spec_photo_canvas (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_photo_canvas (GValue* value, gpointer v_object);
gpointer value_get_photo_canvas (const GValue* value);
GType photo_canvas_get_type (void);
GType photo_get_type (void);
#define PHOTO_CANVAS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PHOTO_CANVAS, PhotoCanvasPrivate))
enum  {
	PHOTO_CANVAS_DUMMY_PROPERTY
};
PhotoCanvas* photo_canvas_new (GtkWindow* container, GdkWindow* drawing_window, Photo* photo, GdkGC* default_gc, GdkDrawable* drawable, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
PhotoCanvas* photo_canvas_construct (GType object_type, GtkWindow* container, GdkWindow* drawing_window, Photo* photo, GdkGC* default_gc, GdkDrawable* drawable, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
GdkRectangle photo_canvas_get_scaled_pixbuf_position (PhotoCanvas* self);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
Dimensions photo_get_dimensions (Photo* self);
GdkPoint photo_canvas_active_to_unscaled_point (PhotoCanvas* self, const GdkPoint* active_point);
GdkRectangle photo_canvas_active_to_unscaled_rect (PhotoCanvas* self, const GdkRectangle* active_rect);
GdkPoint photo_canvas_user_to_active_point (PhotoCanvas* self, const GdkPoint* user_point);
GdkRectangle photo_canvas_user_to_active_rect (PhotoCanvas* self, const GdkRectangle* user_rect);
Photo* photo_canvas_get_photo (PhotoCanvas* self);
GtkWindow* photo_canvas_get_container (PhotoCanvas* self);
GdkWindow* photo_canvas_get_drawing_window (PhotoCanvas* self);
GdkGC* photo_canvas_get_default_gc (PhotoCanvas* self);
GdkDrawable* photo_canvas_get_drawable (PhotoCanvas* self);
void photo_canvas_set_drawable (PhotoCanvas* self, GdkGC* default_gc, GdkDrawable* drawable);
GdkPixbuf* photo_canvas_get_scaled_pixbuf (PhotoCanvas* self);
void photo_canvas_resized_pixbuf (PhotoCanvas* self, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
void photo_canvas_repaint (PhotoCanvas* self);
static void photo_canvas_real_repaint (PhotoCanvas* self);
void photo_canvas_paint_pixbuf (PhotoCanvas* self, GdkPixbuf* pixbuf);
GType box_get_type (void);
Box* box_dup (const Box* self);
void box_free (Box* self);
gint box_get_width (Box *self);
gint box_get_height (Box *self);
void photo_canvas_paint_pixbuf_area (PhotoCanvas* self, GdkPixbuf* pixbuf, const Box* source_area);
GdkRectangle box_get_rectangle (Box *self);
void photo_canvas_draw_box (PhotoCanvas* self, GdkGC* gc, const Box* box);
void photo_canvas_draw_horizontal_line (PhotoCanvas* self, GdkGC* gc, gint x, gint y, gint width);
void photo_canvas_draw_vertical_line (PhotoCanvas* self, GdkGC* gc, gint x, gint y, gint height);
void photo_canvas_erase_horizontal_line (PhotoCanvas* self, gint x, gint y, gint width);
void photo_canvas_draw_circle (PhotoCanvas* self, GdkGC* gc, gint active_center_x, gint active_center_y, gint radius);
void photo_canvas_erase_vertical_line (PhotoCanvas* self, gint x, gint y, gint height);
void photo_canvas_erase_box (PhotoCanvas* self, const Box* box);
void photo_canvas_invalidate_area (PhotoCanvas* self, const Box* area);
static void photo_canvas_finalize (PhotoCanvas* obj);
gpointer editing_tool_ref (gpointer instance);
void editing_tool_unref (gpointer instance);
GParamSpec* param_spec_editing_tool (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_editing_tool (GValue* value, gpointer v_object);
gpointer value_get_editing_tool (const GValue* value);
GType editing_tool_get_type (void);
enum  {
	EDITING_TOOL_DUMMY_PROPERTY
};
void editing_tool_activate (EditingTool* self, PhotoCanvas* canvas);
static void editing_tool_real_activate (EditingTool* self, PhotoCanvas* canvas);
void editing_tool_deactivate (EditingTool* self);
static void editing_tool_real_deactivate (EditingTool* self);
gboolean editing_tool_is_activated (EditingTool* self);
EditingToolWindow* editing_tool_get_tool_window (EditingTool* self);
static EditingToolWindow* editing_tool_real_get_tool_window (EditingTool* self);
GdkPixbuf* editing_tool_get_unscaled_pixbuf (EditingTool* self, Photo* photo);
static GdkPixbuf* editing_tool_real_get_unscaled_pixbuf (EditingTool* self, Photo* photo);
void editing_tool_on_left_click (EditingTool* self, gint x, gint y);
static void editing_tool_real_on_left_click (EditingTool* self, gint x, gint y);
void editing_tool_on_left_released (EditingTool* self, gint x, gint y);
static void editing_tool_real_on_left_released (EditingTool* self, gint x, gint y);
void editing_tool_on_motion (EditingTool* self, gint x, gint y, GdkModifierType mask);
static void editing_tool_real_on_motion (EditingTool* self, gint x, gint y, GdkModifierType mask);
gboolean editing_tool_on_keypress (EditingTool* self, const GdkEventKey* event);
static gboolean editing_tool_real_on_keypress (EditingTool* self, const GdkEventKey* event);
void editing_tool_paint (EditingTool* self, GdkGC* gc, GdkDrawable* drawable);
static void editing_tool_real_paint (EditingTool* self, GdkGC* gc, GdkDrawable* drawable);
void editing_tool_notify_apply (EditingTool* self);
void editing_tool_notify_cancel (EditingTool* self);
static void editing_tool_finalize (EditingTool* obj);
GType crop_tool_get_type (void);
static GType crop_tool_crop_tool_window_get_type (void);
GType box_location_get_type (void);
#define CROP_TOOL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CROP_TOOL, CropToolPrivate))
enum  {
	CROP_TOOL_DUMMY_PROPERTY
};
#define CROP_TOOL_CROP_INIT_X_PCT 0.15
#define CROP_TOOL_CROP_INIT_Y_PCT 0.15
#define CROP_TOOL_CROP_MIN_WIDTH 100
#define CROP_TOOL_CROP_MIN_HEIGHT 100
#define CROP_TOOL_CROP_EXTERIOR_SATURATION 0.00f
#define CROP_TOOL_CROP_EXTERIOR_RED_SHIFT (-32)
#define CROP_TOOL_CROP_EXTERIOR_GREEN_SHIFT (-32)
#define CROP_TOOL_CROP_EXTERIOR_BLUE_SHIFT (-32)
#define CROP_TOOL_CROP_EXTERIOR_ALPHA_SHIFT 0
CropTool* crop_tool_new (void);
CropTool* crop_tool_construct (GType object_type);
static void crop_tool_prepare_gc (CropTool* self, GdkGC* default_gc, GdkDrawable* drawable);
static void _crop_tool_prepare_gc_photo_canvas_new_drawable (PhotoCanvas* _sender, GdkGC* default_gc, GdkDrawable* drawable, gpointer self);
static void crop_tool_on_resized_pixbuf (CropTool* self, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position);
static void _crop_tool_on_resized_pixbuf_photo_canvas_resized_scaled_pixbuf (PhotoCanvas* _sender, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position, gpointer self);
static void crop_tool_prepare_visuals (CropTool* self, GdkPixbuf* pixbuf);
static CropToolCropToolWindow* crop_tool_crop_tool_window_new (GtkWindow* container);
static CropToolCropToolWindow* crop_tool_crop_tool_window_construct (GType object_type, GtkWindow* container);
static void crop_tool_on_crop_apply (CropTool* self);
static void _crop_tool_on_crop_apply_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _editing_tool_notify_cancel_gtk_button_clicked (GtkButton* _sender, gpointer self);
Dimensions photo_get_uncropped_dimensions (Photo* self);
gboolean photo_get_crop (Photo* self, Box* crop);
void box_init (Box *self, gint left, gint top, gint right, gint bottom);
Box box_get_scaled_proportional (Box *self, const Dimensions* orig, const Dimensions* scaled);
Dimensions dimensions_for_rectangle (const GdkRectangle* rect);
static void crop_tool_real_activate (EditingTool* base, PhotoCanvas* canvas);
static void crop_tool_real_deactivate (EditingTool* base);
static EditingToolWindow* crop_tool_real_get_tool_window (EditingTool* base);
gboolean photo_has_crop (Photo* self);
GdkPixbuf* photo_get_pixbuf (Photo* self, gint exceptions, gint scale, GdkInterpType interp, GError** error);
#define PHOTO_EXCEPTION_CROP (1 << 1)
static GdkPixbuf* crop_tool_real_get_unscaled_pixbuf (EditingTool* base, Photo* photo);
GdkColor fetch_color (const char* spec, GdkDrawable* drawable);
Dimensions dimensions_for_pixbuf (GdkPixbuf* pixbuf);
void shift_colors (GdkPixbuf* pixbuf, gint red, gint green, gint blue, gint alpha);
Box box_get_offset (Box *self, gint xofs, gint yofs);
BoxLocation box_approx_location (Box *self, gint x, gint y);
static void crop_tool_real_on_left_click (EditingTool* base, gint x, gint y);
static void crop_tool_update_cursor (CropTool* self, gint x, gint y);
static void crop_tool_real_on_left_released (EditingTool* base, gint x, gint y);
static gboolean crop_tool_on_canvas_manipulation (CropTool* self, gint x, gint y);
static void crop_tool_real_on_motion (EditingTool* base, gint x, gint y, GdkModifierType mask);
static void crop_tool_paint_crop_tool (CropTool* self, const Box* crop);
static void crop_tool_real_paint (EditingTool* base, GdkGC* gc, GdkDrawable* drawable);
gboolean photo_set_crop (Photo* self, const Box* crop);
static void crop_tool_crop_resized (CropTool* self, const Box* new_crop);
static void crop_tool_crop_moved (CropTool* self, const Box* new_crop);
gboolean box_equals (Box *self, const Box* box);
static void crop_tool_erase_crop_tool (CropTool* self, const Box* crop);
GType box_complements_get_type (void);
BoxComplements box_resized_complements (Box *self, const Box* resized, Box* horizontal, gboolean* horizontal_enlarged, Box* vertical, gboolean* vertical_enlarged);
BoxComplements box_shifted_complements (Box *self, const Box* shifted, Box* horizontal_this, Box* vertical_this, Box* horizontal_shifted, Box* vertical_shifted);
Box box_get_reduced (Box *self, gint amount);
enum  {
	CROP_TOOL_CROP_TOOL_WINDOW_DUMMY_PROPERTY
};
#define CROP_TOOL_CROP_TOOL_WINDOW_CONTROL_SPACING 8
static void crop_tool_crop_tool_window_finalize (GObject* obj);
static void crop_tool_finalize (EditingTool* obj);
GType redeye_instance_get_type (void);
RedeyeInstance* redeye_instance_dup (const RedeyeInstance* self);
void redeye_instance_free (RedeyeInstance* self);
#define REDEYE_INSTANCE_MIN_RADIUS 4
#define REDEYE_INSTANCE_MAX_RADIUS 32
#define REDEYE_INSTANCE_DEFAULT_RADIUS 10
static void redeye_instance_init (RedeyeInstance *self);
GdkRectangle redeye_instance_to_bounds_rect (const RedeyeInstance* inst);
RedeyeInstance redeye_instance_from_bounds_rect (const GdkRectangle* rect);
GType redeye_tool_get_type (void);
static GType redeye_tool_redeye_tool_window_get_type (void);
#define REDEYE_TOOL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_REDEYE_TOOL, RedeyeToolPrivate))
enum  {
	REDEYE_TOOL_DUMMY_PROPERTY
};
static RedeyeInstance redeye_tool_new_interaction_instance (RedeyeTool* self, PhotoCanvas* canvas);
static void redeye_tool_prepare_gc (RedeyeTool* self, GdkGC* default_gc, GdkDrawable* drawable);
static void redeye_tool_draw_redeye_instance (RedeyeTool* self, const RedeyeInstance* inst);
static gboolean redeye_tool_on_size_slider_adjust (RedeyeTool* self, GtkScrollType type);
gboolean photo_add_redeye_instance (Photo* self, const RedeyeInstance* inst_unscaled);
static void redeye_tool_on_apply (RedeyeTool* self);
static void redeye_tool_on_canvas_resize (RedeyeTool* self);
static void _redeye_tool_prepare_gc_photo_canvas_new_drawable (PhotoCanvas* _sender, GdkGC* default_gc, GdkDrawable* drawable, gpointer self);
static void _redeye_tool_on_canvas_resize_photo_canvas_resized_scaled_pixbuf (PhotoCanvas* _sender, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position, gpointer self);
static RedeyeToolRedeyeToolWindow* redeye_tool_redeye_tool_window_new (GtkWindow* container);
static RedeyeToolRedeyeToolWindow* redeye_tool_redeye_tool_window_construct (GType object_type, GtkWindow* container);
static gboolean _redeye_tool_on_size_slider_adjust_gtk_range_change_value (GtkHScale* _sender, GtkScrollType scroll, double new_value, gpointer self);
static void _redeye_tool_on_apply_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void redeye_tool_real_activate (EditingTool* base, PhotoCanvas* canvas);
static void redeye_tool_real_deactivate (EditingTool* base);
static EditingToolWindow* redeye_tool_real_get_tool_window (EditingTool* base);
static void redeye_tool_real_paint (EditingTool* base, GdkGC* gc, GdkDrawable* drawable);
gboolean coord_in_rectangle (gint x, gint y, const GdkRectangle* rect);
static void redeye_tool_real_on_left_click (EditingTool* base, gint x, gint y);
static void redeye_tool_real_on_left_released (EditingTool* base, gint x, gint y);
static void redeye_tool_real_on_motion (EditingTool* base, gint x, gint y, GdkModifierType mask);
RedeyeTool* redeye_tool_new (void);
RedeyeTool* redeye_tool_construct (GType object_type);
#define REDEYE_TOOL_REDEYE_TOOL_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW, RedeyeToolRedeyeToolWindowPrivate))
enum  {
	REDEYE_TOOL_REDEYE_TOOL_WINDOW_DUMMY_PROPERTY
};
#define REDEYE_TOOL_REDEYE_TOOL_WINDOW_CONTROL_SPACING 8
static void redeye_tool_redeye_tool_window_finalize (GObject* obj);
static void redeye_tool_finalize (EditingTool* obj);
GType adjust_tool_get_type (void);
static GType adjust_tool_adjust_tool_window_get_type (void);
#define ADJUST_TOOL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_ADJUST_TOOL, AdjustToolPrivate))
enum  {
	ADJUST_TOOL_DUMMY_PROPERTY
};
#define ADJUST_TOOL_SLIDER_MIN_VALUE (-16)
#define ADJUST_TOOL_SLIDER_MAX_VALUE 16
#define ADJUST_TOOL_SLIDER_WIDTH 160
static AdjustToolAdjustToolWindow* adjust_tool_adjust_tool_window_new (GtkWindow* container);
static AdjustToolAdjustToolWindow* adjust_tool_adjust_tool_window_construct (GType object_type, GtkWindow* container);
GType color_transformation_kind_get_type (void);
float photo_get_adjustment_parameter (Photo* self, ColorTransformationKind adjust_kind);
static void adjust_tool_on_apply (AdjustTool* self);
static void _adjust_tool_on_apply_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void adjust_tool_on_reset (AdjustTool* self);
static void _adjust_tool_on_reset_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void adjust_tool_on_cancel (AdjustTool* self);
static void _adjust_tool_on_cancel_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void adjust_tool_on_adjustment (AdjustTool* self);
static void _adjust_tool_on_adjustment_gtk_range_value_changed (GtkHScale* _sender, gpointer self);
static void adjust_tool_on_canvas_resize (AdjustTool* self);
static void _adjust_tool_on_canvas_resize_photo_canvas_resized_scaled_pixbuf (PhotoCanvas* _sender, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position, gpointer self);
ImageHistogram* image_histogram_new (GdkPixbuf* pixbuf);
ImageHistogram* image_histogram_construct (GType object_type, GdkPixbuf* pixbuf);
gpointer image_histogram_ref (gpointer instance);
void image_histogram_unref (gpointer instance);
GParamSpec* param_spec_image_histogram (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_image_histogram (GValue* value, gpointer v_object);
gpointer value_get_image_histogram (const GValue* value);
GType image_histogram_get_type (void);
GdkPixbuf* image_histogram_get_graphic (ImageHistogram* self);
static void adjust_tool_real_activate (EditingTool* base, PhotoCanvas* canvas);
static EditingToolWindow* adjust_tool_real_get_tool_window (EditingTool* base);
static void adjust_tool_real_deactivate (EditingTool* base);
gpointer color_transformation_factory_ref (gpointer instance);
void color_transformation_factory_unref (gpointer instance);
GParamSpec* param_spec_color_transformation_factory (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_color_transformation_factory (GValue* value, gpointer v_object);
gpointer value_get_color_transformation_factory (const GValue* value);
GType color_transformation_factory_get_type (void);
ColorTransformationFactory* color_transformation_factory_get_instance (void);
gpointer color_transformation_ref (gpointer instance);
void color_transformation_unref (gpointer instance);
GParamSpec* param_spec_color_transformation (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_color_transformation (GValue* value, gpointer v_object);
gpointer value_get_color_transformation (const GValue* value);
GType color_transformation_get_type (void);
ColorTransformation* color_transformation_factory_from_parameter (ColorTransformationFactory* self, ColorTransformationKind kind, float parameter);
ColorTransformation* color_transformation_compose_against (ColorTransformation* self, ColorTransformation* transform);
void color_transformation_transform_existing_pixbuf (ColorTransformation* transform, GdkPixbuf* in_pixbuf, GdkPixbuf* out_pixbuf);
static void adjust_tool_real_paint (EditingTool* base, GdkGC* gc, GdkDrawable* drawable);
#define PHOTO_EXCEPTION_ADJUST (1 << 3)
static GdkPixbuf* adjust_tool_real_get_unscaled_pixbuf (EditingTool* base, Photo* photo);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
void app_window_set_busy_cursor (AppWindow* self);
GType color_transformation_instance_get_type (void);
ColorTransformationInstance* color_transformation_instance_dup (const ColorTransformationInstance* self);
void color_transformation_instance_free (ColorTransformationInstance* self);
gboolean photo_set_adjustments (Photo* self, GeeArrayList* adjustments);
void app_window_set_normal_cursor (AppWindow* self);
AdjustTool* adjust_tool_new (void);
AdjustTool* adjust_tool_construct (GType object_type);
enum  {
	ADJUST_TOOL_ADJUST_TOOL_WINDOW_DUMMY_PROPERTY
};
#define IMAGE_HISTOGRAM_GRAPHIC_WIDTH 256
#define IMAGE_HISTOGRAM_GRAPHIC_HEIGHT 100
static void adjust_tool_adjust_tool_window_finalize (GObject* obj);
static void adjust_tool_finalize (EditingTool* obj);


static void g_cclosure_user_marshal_VOID__OBJECT_OBJECT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void g_cclosure_user_marshal_VOID__BOXED_OBJECT_BOXED (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

#line 14 "EditingTools.vala"
EditingToolWindow* editing_tool_window_construct (GType object_type, GtkWindow* container) {
#line 709 "EditingTools.c"
	EditingToolWindow * self;
	GtkWindow* _tmp1_;
	GtkWindow* _tmp0_;
	GtkFrame* outer_frame;
#line 14 "EditingTools.vala"
	g_return_val_if_fail (GTK_IS_WINDOW (container), NULL);
#line 716 "EditingTools.c"
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
#line 15 "EditingTools.vala"
	_tmp0_ = NULL;
#line 15 "EditingTools.vala"
	self->priv->container = (_tmp1_ = (_tmp0_ = container, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->container == NULL) ? NULL : (self->priv->container = (g_object_unref (self->priv->container), NULL)), _tmp1_);
#line 17 "EditingTools.vala"
	gtk_window_set_type_hint (GTK_WINDOW (self), GDK_WINDOW_TYPE_HINT_TOOLBAR);
#line 18 "EditingTools.vala"
	gtk_window_set_transient_for (GTK_WINDOW (self), container);
#line 19 "EditingTools.vala"
	GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (self), GTK_CAN_FOCUS);
#line 20 "EditingTools.vala"
	gtk_window_set_accept_focus (GTK_WINDOW (self), FALSE);
#line 21 "EditingTools.vala"
	gtk_window_set_focus_on_map (GTK_WINDOW (self), FALSE);
#line 733 "EditingTools.c"
	outer_frame = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL));
#line 24 "EditingTools.vala"
	gtk_container_set_border_width (GTK_CONTAINER (outer_frame), (guint) 0);
#line 25 "EditingTools.vala"
	gtk_frame_set_shadow_type (outer_frame, GTK_SHADOW_OUT);
#line 27 "EditingTools.vala"
	gtk_container_set_border_width (GTK_CONTAINER (self->priv->layout_frame), (guint) EDITING_TOOL_WINDOW_FRAME_BORDER);
#line 28 "EditingTools.vala"
	gtk_frame_set_shadow_type (self->priv->layout_frame, GTK_SHADOW_NONE);
#line 30 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (outer_frame), GTK_WIDGET (self->priv->layout_frame));
#line 31 "EditingTools.vala"
	GTK_CONTAINER_CLASS (editing_tool_window_parent_class)->add (GTK_CONTAINER (GTK_WINDOW (self)), GTK_WIDGET (outer_frame));
#line 33 "EditingTools.vala"
	gtk_widget_add_events (GTK_WIDGET (self), (gint) (GDK_BUTTON_PRESS_MASK | GDK_FOCUS_CHANGE_MASK));
#line 749 "EditingTools.c"
	(outer_frame == NULL) ? NULL : (outer_frame = (g_object_unref (outer_frame), NULL));
	return self;
}


#line 14 "EditingTools.vala"
EditingToolWindow* editing_tool_window_new (GtkWindow* container) {
#line 14 "EditingTools.vala"
	return editing_tool_window_construct (TYPE_EDITING_TOOL_WINDOW, container);
#line 759 "EditingTools.c"
}


#line 36 "EditingTools.vala"
static void editing_tool_window_real_add (GtkContainer* base, GtkWidget* widget) {
#line 765 "EditingTools.c"
	EditingToolWindow * self;
	self = EDITING_TOOL_WINDOW (base);
#line 36 "EditingTools.vala"
	g_return_if_fail (GTK_IS_WIDGET (widget));
#line 37 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (self->priv->layout_frame), widget);
#line 772 "EditingTools.c"
}


#line 40 "EditingTools.vala"
gboolean editing_tool_window_has_user_moved (EditingToolWindow* self) {
#line 778 "EditingTools.c"
	gboolean result;
#line 40 "EditingTools.vala"
	g_return_val_if_fail (IS_EDITING_TOOL_WINDOW (self), FALSE);
#line 782 "EditingTools.c"
	result = self->priv->user_moved;
	return result;
}


#line 44 "EditingTools.vala"
static gboolean editing_tool_window_real_button_press_event (GtkWidget* base, const GdkEventButton* event) {
#line 790 "EditingTools.c"
	EditingToolWindow * self;
	gboolean result;
	self = EDITING_TOOL_WINDOW (base);
	/* LMB only*/
#line 46 "EditingTools.vala"
	if ((*event).button != 1) {
#line 797 "EditingTools.c"
		gboolean _tmp0_;
		_tmp0_ = FALSE;
#line 47 "EditingTools.vala"
		if (GTK_WIDGET_CLASS (editing_tool_window_parent_class)->button_press_event != NULL) {
#line 47 "EditingTools.vala"
			_tmp0_ = GTK_WIDGET_CLASS (editing_tool_window_parent_class)->button_press_event (GTK_WIDGET (GTK_WINDOW (self)), &(*event));
#line 804 "EditingTools.c"
		} else {
#line 47 "EditingTools.vala"
			_tmp0_ = TRUE;
#line 808 "EditingTools.c"
		}
		result = _tmp0_;
		return result;
	}
#line 49 "EditingTools.vala"
	gtk_window_begin_move_drag (GTK_WINDOW (self), (gint) (*event).button, (gint) (*event).x_root, (gint) (*event).y_root, (*event).time);
#line 50 "EditingTools.vala"
	self->priv->user_moved = TRUE;
#line 817 "EditingTools.c"
	result = TRUE;
	return result;
}


#line 55 "EditingTools.vala"
static void editing_tool_window_real_realize (GtkWidget* base) {
#line 825 "EditingTools.c"
	EditingToolWindow * self;
	self = EDITING_TOOL_WINDOW (base);
#line 56 "EditingTools.vala"
	gtk_window_set_opacity (GTK_WINDOW (self), FULLSCREEN_WINDOW_TOOLBAR_OPACITY);
#line 58 "EditingTools.vala"
	GTK_WIDGET_CLASS (editing_tool_window_parent_class)->realize (GTK_WIDGET (GTK_WINDOW (self)));
#line 832 "EditingTools.c"
}


#line 72 "EditingTools.vala"
static gboolean _editing_tool_window_unsteal_focus_gsource_func (gpointer self) {
#line 838 "EditingTools.c"
	return editing_tool_window_unsteal_focus (self);
}


/* This is necessary because some window managers (Metacity appears to be guilty of it) seem to
 ignore the set_focus_on_map flag, and give the toolbar focus when it appears on the screen.
 Thereafter, thanks to set_accept_focus, the toolbar will never accept it.  Because changing
 focus inside of a focus signal is problematic, if the toolbar ever does receive
 focus, it schedules a task to give it back to its owner.*/
#line 66 "EditingTools.vala"
static gboolean editing_tool_window_real_focus (GtkWidget* base, GtkDirectionType direction) {
#line 850 "EditingTools.c"
	EditingToolWindow * self;
	gboolean result;
	self = EDITING_TOOL_WINDOW (base);
#line 67 "EditingTools.vala"
	g_idle_add_full (G_PRIORITY_HIGH, _editing_tool_window_unsteal_focus_gsource_func, g_object_ref (self), g_object_unref);
#line 856 "EditingTools.c"
	result = TRUE;
	return result;
}


#line 72 "EditingTools.vala"
static gboolean editing_tool_window_unsteal_focus (EditingToolWindow* self) {
#line 864 "EditingTools.c"
	gboolean result;
#line 72 "EditingTools.vala"
	g_return_val_if_fail (IS_EDITING_TOOL_WINDOW (self), FALSE);
#line 73 "EditingTools.vala"
	gtk_window_present_with_time (self->priv->container, (guint32) GDK_CURRENT_TIME);
#line 870 "EditingTools.c"
	result = FALSE;
	return result;
}


static void editing_tool_window_class_init (EditingToolWindowClass * klass) {
	editing_tool_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (EditingToolWindowPrivate));
	GTK_CONTAINER_CLASS (klass)->add = editing_tool_window_real_add;
	GTK_WIDGET_CLASS (klass)->button_press_event = editing_tool_window_real_button_press_event;
	GTK_WIDGET_CLASS (klass)->realize = editing_tool_window_real_realize;
	GTK_WIDGET_CLASS (klass)->focus = editing_tool_window_real_focus;
	G_OBJECT_CLASS (klass)->finalize = editing_tool_window_finalize;
}


static void editing_tool_window_instance_init (EditingToolWindow * self) {
	self->priv = EDITING_TOOL_WINDOW_GET_PRIVATE (self);
	self->priv->layout_frame = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL));
	self->priv->user_moved = FALSE;
}


static void editing_tool_window_finalize (GObject* obj) {
	EditingToolWindow * self;
	self = EDITING_TOOL_WINDOW (obj);
	(self->priv->container == NULL) ? NULL : (self->priv->container = (g_object_unref (self->priv->container), NULL));
	(self->priv->layout_frame == NULL) ? NULL : (self->priv->layout_frame = (g_object_unref (self->priv->layout_frame), NULL));
	G_OBJECT_CLASS (editing_tool_window_parent_class)->finalize (obj);
}


GType editing_tool_window_get_type (void) {
	static GType editing_tool_window_type_id = 0;
	if (editing_tool_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (EditingToolWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) editing_tool_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EditingToolWindow), 0, (GInstanceInitFunc) editing_tool_window_instance_init, NULL };
		editing_tool_window_type_id = g_type_register_static (GTK_TYPE_WINDOW, "EditingToolWindow", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return editing_tool_window_type_id;
}


#line 91 "EditingTools.vala"
PhotoCanvas* photo_canvas_construct (GType object_type, GtkWindow* container, GdkWindow* drawing_window, Photo* photo, GdkGC* default_gc, GdkDrawable* drawable, GdkPixbuf* scaled, const GdkRectangle* scaled_position) {
#line 915 "EditingTools.c"
	PhotoCanvas* self;
	GtkWindow* _tmp1_;
	GtkWindow* _tmp0_;
	GdkWindow* _tmp3_;
	GdkWindow* _tmp2_;
	Photo* _tmp5_;
	Photo* _tmp4_;
	GdkGC* _tmp7_;
	GdkGC* _tmp6_;
	GdkDrawable* _tmp9_;
	GdkDrawable* _tmp8_;
	GdkPixbuf* _tmp11_;
	GdkPixbuf* _tmp10_;
#line 91 "EditingTools.vala"
	g_return_val_if_fail (GTK_IS_WINDOW (container), NULL);
#line 91 "EditingTools.vala"
	g_return_val_if_fail (GDK_IS_WINDOW (drawing_window), NULL);
#line 91 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO (photo), NULL);
#line 91 "EditingTools.vala"
	g_return_val_if_fail (GDK_IS_GC (default_gc), NULL);
#line 91 "EditingTools.vala"
	g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
#line 91 "EditingTools.vala"
	g_return_val_if_fail (GDK_IS_PIXBUF (scaled), NULL);
#line 941 "EditingTools.c"
	self = (PhotoCanvas*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 93 "EditingTools.vala"
	_tmp0_ = NULL;
#line 93 "EditingTools.vala"
	self->priv->container = (_tmp1_ = (_tmp0_ = container, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->container == NULL) ? NULL : (self->priv->container = (g_object_unref (self->priv->container), NULL)), _tmp1_);
#line 948 "EditingTools.c"
	_tmp3_ = NULL;
#line 94 "EditingTools.vala"
	_tmp2_ = NULL;
#line 94 "EditingTools.vala"
	self->priv->drawing_window = (_tmp3_ = (_tmp2_ = drawing_window, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->drawing_window == NULL) ? NULL : (self->priv->drawing_window = (g_object_unref (self->priv->drawing_window), NULL)), _tmp3_);
#line 954 "EditingTools.c"
	_tmp5_ = NULL;
#line 95 "EditingTools.vala"
	_tmp4_ = NULL;
#line 95 "EditingTools.vala"
	self->priv->photo = (_tmp5_ = (_tmp4_ = photo, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (self->priv->photo == NULL) ? NULL : (self->priv->photo = (g_object_unref (self->priv->photo), NULL)), _tmp5_);
#line 960 "EditingTools.c"
	_tmp7_ = NULL;
#line 96 "EditingTools.vala"
	_tmp6_ = NULL;
#line 96 "EditingTools.vala"
	self->priv->default_gc = (_tmp7_ = (_tmp6_ = default_gc, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (self->priv->default_gc == NULL) ? NULL : (self->priv->default_gc = (g_object_unref (self->priv->default_gc), NULL)), _tmp7_);
#line 966 "EditingTools.c"
	_tmp9_ = NULL;
#line 97 "EditingTools.vala"
	_tmp8_ = NULL;
#line 97 "EditingTools.vala"
	self->priv->drawable = (_tmp9_ = (_tmp8_ = drawable, (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_)), (self->priv->drawable == NULL) ? NULL : (self->priv->drawable = (g_object_unref (self->priv->drawable), NULL)), _tmp9_);
#line 972 "EditingTools.c"
	_tmp11_ = NULL;
#line 98 "EditingTools.vala"
	_tmp10_ = NULL;
#line 98 "EditingTools.vala"
	self->priv->scaled = (_tmp11_ = (_tmp10_ = scaled, (_tmp10_ == NULL) ? NULL : g_object_ref (_tmp10_)), (self->priv->scaled == NULL) ? NULL : (self->priv->scaled = (g_object_unref (self->priv->scaled), NULL)), _tmp11_);
#line 99 "EditingTools.vala"
	self->priv->scaled_position = (*scaled_position);
#line 980 "EditingTools.c"
	return self;
}


#line 91 "EditingTools.vala"
PhotoCanvas* photo_canvas_new (GtkWindow* container, GdkWindow* drawing_window, Photo* photo, GdkGC* default_gc, GdkDrawable* drawable, GdkPixbuf* scaled, const GdkRectangle* scaled_position) {
#line 91 "EditingTools.vala"
	return photo_canvas_construct (TYPE_PHOTO_CANVAS, container, drawing_window, photo, default_gc, drawable, scaled, scaled_position);
#line 989 "EditingTools.c"
}


#line 106 "EditingTools.vala"
GdkPoint photo_canvas_active_to_unscaled_point (PhotoCanvas* self, const GdkPoint* active_point) {
#line 995 "EditingTools.c"
	GdkPoint result;
	GdkRectangle scaled_position;
	Dimensions unscaled_dims;
	double scale_factor_x;
	double scale_factor_y;
	GdkPoint _result_ = {0};
	scaled_position = photo_canvas_get_scaled_pixbuf_position (self);
	unscaled_dims = photo_get_dimensions (self->priv->photo);
	scale_factor_x = ((double) unscaled_dims.width) / ((double) scaled_position.width);
	scale_factor_y = ((double) unscaled_dims.height) / ((double) scaled_position.height);
#line 116 "EditingTools.vala"
	_result_.x = (gint) ((((double) (*active_point).x) * scale_factor_x) + 0.5);
#line 117 "EditingTools.vala"
	_result_.y = (gint) ((((double) (*active_point).y) * scale_factor_y) + 0.5);
#line 1010 "EditingTools.c"
	result = _result_;
	return result;
}


#line 122 "EditingTools.vala"
GdkRectangle photo_canvas_active_to_unscaled_rect (PhotoCanvas* self, const GdkRectangle* active_rect) {
#line 1018 "EditingTools.c"
	GdkRectangle result;
	GdkPoint upper_left = {0};
	GdkPoint lower_right = {0};
	GdkRectangle unscaled_rect = {0};
#line 125 "EditingTools.vala"
	upper_left.x = (*active_rect).x;
#line 126 "EditingTools.vala"
	upper_left.y = (*active_rect).y;
#line 127 "EditingTools.vala"
	lower_right.x = upper_left.x + (*active_rect).width;
#line 128 "EditingTools.vala"
	lower_right.y = upper_left.y + (*active_rect).height;
#line 130 "EditingTools.vala"
	upper_left = photo_canvas_active_to_unscaled_point (self, &upper_left);
#line 131 "EditingTools.vala"
	lower_right = photo_canvas_active_to_unscaled_point (self, &lower_right);
#line 134 "EditingTools.vala"
	unscaled_rect.x = upper_left.x;
#line 135 "EditingTools.vala"
	unscaled_rect.y = upper_left.y;
#line 136 "EditingTools.vala"
	unscaled_rect.width = lower_right.x - upper_left.x;
#line 137 "EditingTools.vala"
	unscaled_rect.height = lower_right.y - upper_left.y;
#line 1043 "EditingTools.c"
	result = unscaled_rect;
	return result;
}


#line 142 "EditingTools.vala"
GdkPoint photo_canvas_user_to_active_point (PhotoCanvas* self, const GdkPoint* user_point) {
#line 1051 "EditingTools.c"
	GdkPoint result;
	GdkRectangle active_offsets;
	GdkPoint _result_ = {0};
	active_offsets = photo_canvas_get_scaled_pixbuf_position (self);
#line 146 "EditingTools.vala"
	_result_.x = (*user_point).x - active_offsets.x;
#line 147 "EditingTools.vala"
	_result_.y = (*user_point).y - active_offsets.y;
#line 1060 "EditingTools.c"
	result = _result_;
	return result;
}


#line 152 "EditingTools.vala"
GdkRectangle photo_canvas_user_to_active_rect (PhotoCanvas* self, const GdkRectangle* user_rect) {
#line 1068 "EditingTools.c"
	GdkRectangle result;
	GdkPoint upper_left = {0};
	GdkPoint lower_right = {0};
	GdkRectangle active_rect = {0};
#line 155 "EditingTools.vala"
	upper_left.x = (*user_rect).x;
#line 156 "EditingTools.vala"
	upper_left.y = (*user_rect).y;
#line 157 "EditingTools.vala"
	lower_right.x = upper_left.x + (*user_rect).width;
#line 158 "EditingTools.vala"
	lower_right.y = upper_left.y + (*user_rect).height;
#line 160 "EditingTools.vala"
	upper_left = photo_canvas_user_to_active_point (self, &upper_left);
#line 161 "EditingTools.vala"
	lower_right = photo_canvas_user_to_active_point (self, &lower_right);
#line 164 "EditingTools.vala"
	active_rect.x = upper_left.x;
#line 165 "EditingTools.vala"
	active_rect.y = upper_left.y;
#line 166 "EditingTools.vala"
	active_rect.width = lower_right.x - upper_left.x;
#line 167 "EditingTools.vala"
	active_rect.height = lower_right.y - upper_left.y;
#line 1093 "EditingTools.c"
	result = active_rect;
	return result;
}


#line 172 "EditingTools.vala"
Photo* photo_canvas_get_photo (PhotoCanvas* self) {
#line 1101 "EditingTools.c"
	Photo* result;
	Photo* _tmp0_;
#line 172 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO_CANVAS (self), NULL);
#line 173 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1108 "EditingTools.c"
	result = (_tmp0_ = self->priv->photo, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 176 "EditingTools.vala"
GtkWindow* photo_canvas_get_container (PhotoCanvas* self) {
#line 1116 "EditingTools.c"
	GtkWindow* result;
	GtkWindow* _tmp0_;
#line 176 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO_CANVAS (self), NULL);
#line 177 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1123 "EditingTools.c"
	result = (_tmp0_ = self->priv->container, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 180 "EditingTools.vala"
GdkWindow* photo_canvas_get_drawing_window (PhotoCanvas* self) {
#line 1131 "EditingTools.c"
	GdkWindow* result;
	GdkWindow* _tmp0_;
#line 180 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO_CANVAS (self), NULL);
#line 181 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1138 "EditingTools.c"
	result = (_tmp0_ = self->priv->drawing_window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 184 "EditingTools.vala"
GdkGC* photo_canvas_get_default_gc (PhotoCanvas* self) {
#line 1146 "EditingTools.c"
	GdkGC* result;
	GdkGC* _tmp0_;
#line 184 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO_CANVAS (self), NULL);
#line 185 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1153 "EditingTools.c"
	result = (_tmp0_ = self->priv->default_gc, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 188 "EditingTools.vala"
GdkDrawable* photo_canvas_get_drawable (PhotoCanvas* self) {
#line 1161 "EditingTools.c"
	GdkDrawable* result;
	GdkDrawable* _tmp0_;
#line 188 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO_CANVAS (self), NULL);
#line 189 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1168 "EditingTools.c"
	result = (_tmp0_ = self->priv->drawable, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 192 "EditingTools.vala"
void photo_canvas_set_drawable (PhotoCanvas* self, GdkGC* default_gc, GdkDrawable* drawable) {
#line 1176 "EditingTools.c"
	GdkGC* _tmp1_;
	GdkGC* _tmp0_;
	GdkDrawable* _tmp3_;
	GdkDrawable* _tmp2_;
#line 192 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 192 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (default_gc));
#line 192 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 1187 "EditingTools.c"
	_tmp1_ = NULL;
#line 193 "EditingTools.vala"
	_tmp0_ = NULL;
#line 193 "EditingTools.vala"
	self->priv->default_gc = (_tmp1_ = (_tmp0_ = default_gc, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->default_gc == NULL) ? NULL : (self->priv->default_gc = (g_object_unref (self->priv->default_gc), NULL)), _tmp1_);
#line 1193 "EditingTools.c"
	_tmp3_ = NULL;
#line 194 "EditingTools.vala"
	_tmp2_ = NULL;
#line 194 "EditingTools.vala"
	self->priv->drawable = (_tmp3_ = (_tmp2_ = drawable, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->drawable == NULL) ? NULL : (self->priv->drawable = (g_object_unref (self->priv->drawable), NULL)), _tmp3_);
#line 196 "EditingTools.vala"
	g_signal_emit_by_name (self, "new-drawable", default_gc, drawable);
#line 1201 "EditingTools.c"
}


#line 199 "EditingTools.vala"
GdkPixbuf* photo_canvas_get_scaled_pixbuf (PhotoCanvas* self) {
#line 1207 "EditingTools.c"
	GdkPixbuf* result;
	GdkPixbuf* _tmp0_;
#line 199 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO_CANVAS (self), NULL);
#line 200 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1214 "EditingTools.c"
	result = (_tmp0_ = self->priv->scaled, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 203 "EditingTools.vala"
GdkRectangle photo_canvas_get_scaled_pixbuf_position (PhotoCanvas* self) {
#line 1222 "EditingTools.c"
	GdkRectangle result;
	result = self->priv->scaled_position;
	return result;
}


#line 207 "EditingTools.vala"
void photo_canvas_resized_pixbuf (PhotoCanvas* self, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position) {
#line 1231 "EditingTools.c"
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
#line 207 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 207 "EditingTools.vala"
	g_return_if_fail (GDK_IS_PIXBUF (scaled));
#line 1238 "EditingTools.c"
	_tmp1_ = NULL;
#line 208 "EditingTools.vala"
	_tmp0_ = NULL;
#line 208 "EditingTools.vala"
	self->priv->scaled = (_tmp1_ = (_tmp0_ = scaled, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->scaled == NULL) ? NULL : (self->priv->scaled = (g_object_unref (self->priv->scaled), NULL)), _tmp1_);
#line 209 "EditingTools.vala"
	self->priv->scaled_position = (*scaled_position);
#line 211 "EditingTools.vala"
	g_signal_emit_by_name (self, "resized-scaled-pixbuf", &(*old_dim), scaled, &(*scaled_position));
#line 1248 "EditingTools.c"
}


#line 214 "EditingTools.vala"
static void photo_canvas_real_repaint (PhotoCanvas* self) {
#line 1254 "EditingTools.c"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
	g_critical ("Type `%s' does not implement abstract method `photo_canvas_repaint'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


#line 214 "EditingTools.vala"
void photo_canvas_repaint (PhotoCanvas* self) {
#line 214 "EditingTools.vala"
	PHOTO_CANVAS_GET_CLASS (self)->repaint (self);
#line 1265 "EditingTools.c"
}


/* Because the editing tool should not have any need to draw on the gutters outside the photo,
 and it's a pain to constantly calculate where it's laid out on the drawable, these convenience
 methods automatically adjust for its position.

 If these methods are not used, all painting to the drawable should be offet by
 get_scaled_pixbuf_position().x and get_scaled_pixbuf_position().y*/
#line 223 "EditingTools.vala"
void photo_canvas_paint_pixbuf (PhotoCanvas* self, GdkPixbuf* pixbuf) {
#line 223 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 223 "EditingTools.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 224 "EditingTools.vala"
	GDK_DRAWABLE_GET_CLASS (self->priv->drawable)->draw_pixbuf (self->priv->drawable, self->priv->default_gc, pixbuf, 0, 0, self->priv->scaled_position.x, self->priv->scaled_position.y, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf), GDK_RGB_DITHER_NORMAL, 0, 0);
#line 1283 "EditingTools.c"
}


#line 231 "EditingTools.vala"
void photo_canvas_paint_pixbuf_area (PhotoCanvas* self, GdkPixbuf* pixbuf, const Box* source_area) {
#line 231 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 231 "EditingTools.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 232 "EditingTools.vala"
	GDK_DRAWABLE_GET_CLASS (self->priv->drawable)->draw_pixbuf (self->priv->drawable, self->priv->default_gc, pixbuf, (*source_area).left, (*source_area).top, self->priv->scaled_position.x + (*source_area).left, self->priv->scaled_position.y + (*source_area).top, box_get_width (&(*source_area)), box_get_height (&(*source_area)), GDK_RGB_DITHER_NORMAL, 0, 0);
#line 1295 "EditingTools.c"
}


#line 239 "EditingTools.vala"
void photo_canvas_draw_box (PhotoCanvas* self, GdkGC* gc, const Box* box) {
#line 1301 "EditingTools.c"
	GdkRectangle rect;
#line 239 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 239 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 1307 "EditingTools.c"
	rect = box_get_rectangle (&(*box));
#line 241 "EditingTools.vala"
	rect.x = rect.x + self->priv->scaled_position.x;
#line 242 "EditingTools.vala"
	rect.y = rect.y + self->priv->scaled_position.y;
#line 1313 "EditingTools.c"
	/* See note at gtk_drawable_draw_rectangle for info on off-by-one with unfilled rectangles*/
#line 245 "EditingTools.vala"
	GDK_DRAWABLE_GET_CLASS (self->priv->drawable)->draw_rectangle (self->priv->drawable, gc, FALSE, rect.x, rect.y, rect.width - 1, rect.height - 1);
#line 1317 "EditingTools.c"
}


#line 248 "EditingTools.vala"
void photo_canvas_draw_horizontal_line (PhotoCanvas* self, GdkGC* gc, gint x, gint y, gint width) {
#line 248 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 248 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 249 "EditingTools.vala"
	x = x + self->priv->scaled_position.x;
#line 250 "EditingTools.vala"
	y = y + self->priv->scaled_position.y;
#line 252 "EditingTools.vala"
	gdk_draw_line (self->priv->drawable, gc, x, y, (x + width) - 1, y);
#line 1333 "EditingTools.c"
}


#line 255 "EditingTools.vala"
void photo_canvas_draw_vertical_line (PhotoCanvas* self, GdkGC* gc, gint x, gint y, gint height) {
#line 255 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 255 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 256 "EditingTools.vala"
	x = x + self->priv->scaled_position.x;
#line 257 "EditingTools.vala"
	y = y + self->priv->scaled_position.y;
#line 259 "EditingTools.vala"
	gdk_draw_line (self->priv->drawable, gc, x, y, x, (y + height) - 1);
#line 1349 "EditingTools.c"
}


#line 262 "EditingTools.vala"
void photo_canvas_erase_horizontal_line (PhotoCanvas* self, gint x, gint y, gint width) {
#line 262 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 263 "EditingTools.vala"
	GDK_DRAWABLE_GET_CLASS (self->priv->drawable)->draw_pixbuf (self->priv->drawable, self->priv->default_gc, self->priv->scaled, x, y, self->priv->scaled_position.x + x, self->priv->scaled_position.y + y, width, 1, GDK_RGB_DITHER_NORMAL, 0, 0);
#line 1359 "EditingTools.c"
}


#line 270 "EditingTools.vala"
void photo_canvas_draw_circle (PhotoCanvas* self, GdkGC* gc, gint active_center_x, gint active_center_y, gint radius) {
#line 1365 "EditingTools.c"
	gint center_x;
	gint center_y;
	GdkRectangle bounds = {0};
	GdkDrawable* _tmp0_;
#line 270 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 270 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 1374 "EditingTools.c"
	center_x = active_center_x + photo_canvas_get_scaled_pixbuf_position (self).x;
	center_y = active_center_y + photo_canvas_get_scaled_pixbuf_position (self).y;
#line 276 "EditingTools.vala"
	bounds.x = center_x - radius;
#line 277 "EditingTools.vala"
	bounds.y = center_y - radius;
#line 278 "EditingTools.vala"
	bounds.width = 2 * radius;
#line 279 "EditingTools.vala"
	bounds.height = bounds.width;
#line 188 "EditingTools.vala"
	_tmp0_ = NULL;
#line 281 "EditingTools.vala"
	gdk_draw_arc (_tmp0_ = photo_canvas_get_drawable (self), gc, FALSE, bounds.x, bounds.y, bounds.width, bounds.height, 0, 360 * 64);
#line 1389 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 285 "EditingTools.vala"
void photo_canvas_erase_vertical_line (PhotoCanvas* self, gint x, gint y, gint height) {
#line 285 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 286 "EditingTools.vala"
	GDK_DRAWABLE_GET_CLASS (self->priv->drawable)->draw_pixbuf (self->priv->drawable, self->priv->default_gc, self->priv->scaled, x, y, self->priv->scaled_position.x + x, self->priv->scaled_position.y + y, 1, height, GDK_RGB_DITHER_NORMAL, 0, 0);
#line 1400 "EditingTools.c"
}


#line 293 "EditingTools.vala"
void photo_canvas_erase_box (PhotoCanvas* self, const Box* box) {
#line 293 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 294 "EditingTools.vala"
	photo_canvas_erase_horizontal_line (self, (*box).left, (*box).top, box_get_width (&(*box)));
#line 295 "EditingTools.vala"
	photo_canvas_erase_horizontal_line (self, (*box).left, (*box).bottom, box_get_width (&(*box)));
#line 297 "EditingTools.vala"
	photo_canvas_erase_vertical_line (self, (*box).left, (*box).top, box_get_height (&(*box)));
#line 298 "EditingTools.vala"
	photo_canvas_erase_vertical_line (self, (*box).right, (*box).top, box_get_height (&(*box)));
#line 1416 "EditingTools.c"
}


#line 301 "EditingTools.vala"
void photo_canvas_invalidate_area (PhotoCanvas* self, const Box* area) {
#line 1422 "EditingTools.c"
	GdkRectangle rect;
#line 301 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (self));
#line 1426 "EditingTools.c"
	rect = box_get_rectangle (&(*area));
#line 303 "EditingTools.vala"
	rect.x = rect.x + self->priv->scaled_position.x;
#line 304 "EditingTools.vala"
	rect.y = rect.y + self->priv->scaled_position.y;
#line 306 "EditingTools.vala"
	gdk_window_invalidate_rect (self->priv->drawing_window, &rect, FALSE);
#line 1434 "EditingTools.c"
}


static void value_photo_canvas_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_photo_canvas_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		photo_canvas_unref (value->data[0].v_pointer);
	}
}


static void value_photo_canvas_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = photo_canvas_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_photo_canvas_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_photo_canvas_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		PhotoCanvas* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = photo_canvas_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_photo_canvas_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	PhotoCanvas** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = photo_canvas_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_photo_canvas (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecPhotoCanvas* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_PHOTO_CANVAS), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_photo_canvas (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_PHOTO_CANVAS), NULL);
	return value->data[0].v_pointer;
}


void value_set_photo_canvas (GValue* value, gpointer v_object) {
	PhotoCanvas* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_PHOTO_CANVAS));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_PHOTO_CANVAS));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		photo_canvas_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		photo_canvas_unref (old);
	}
}


static void photo_canvas_class_init (PhotoCanvasClass * klass) {
	photo_canvas_parent_class = g_type_class_peek_parent (klass);
	PHOTO_CANVAS_CLASS (klass)->finalize = photo_canvas_finalize;
	g_type_class_add_private (klass, sizeof (PhotoCanvasPrivate));
	PHOTO_CANVAS_CLASS (klass)->repaint = photo_canvas_real_repaint;
	g_signal_new ("new_drawable", TYPE_PHOTO_CANVAS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_OBJECT, G_TYPE_NONE, 2, GDK_TYPE_GC, GDK_TYPE_DRAWABLE);
	g_signal_new ("resized_scaled_pixbuf", TYPE_PHOTO_CANVAS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__BOXED_OBJECT_BOXED, G_TYPE_NONE, 3, TYPE_DIMENSIONS, GDK_TYPE_PIXBUF, GDK_TYPE_RECTANGLE);
}


static void photo_canvas_instance_init (PhotoCanvas * self) {
	self->priv = PHOTO_CANVAS_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void photo_canvas_finalize (PhotoCanvas* obj) {
	PhotoCanvas * self;
	self = PHOTO_CANVAS (obj);
	(self->priv->container == NULL) ? NULL : (self->priv->container = (g_object_unref (self->priv->container), NULL));
	(self->priv->drawing_window == NULL) ? NULL : (self->priv->drawing_window = (g_object_unref (self->priv->drawing_window), NULL));
	(self->priv->photo == NULL) ? NULL : (self->priv->photo = (g_object_unref (self->priv->photo), NULL));
	(self->priv->default_gc == NULL) ? NULL : (self->priv->default_gc = (g_object_unref (self->priv->default_gc), NULL));
	(self->priv->drawable == NULL) ? NULL : (self->priv->drawable = (g_object_unref (self->priv->drawable), NULL));
	(self->priv->scaled == NULL) ? NULL : (self->priv->scaled = (g_object_unref (self->priv->scaled), NULL));
}


GType photo_canvas_get_type (void) {
	static GType photo_canvas_type_id = 0;
	if (photo_canvas_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_photo_canvas_init, value_photo_canvas_free_value, value_photo_canvas_copy_value, value_photo_canvas_peek_pointer, "p", value_photo_canvas_collect_value, "p", value_photo_canvas_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (PhotoCanvasClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_canvas_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PhotoCanvas), 0, (GInstanceInitFunc) photo_canvas_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		photo_canvas_type_id = g_type_register_fundamental (g_type_fundamental_next (), "PhotoCanvas", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return photo_canvas_type_id;
}


gpointer photo_canvas_ref (gpointer instance) {
	PhotoCanvas* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void photo_canvas_unref (gpointer instance) {
	PhotoCanvas* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		PHOTO_CANVAS_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


/* base.activate() should always be called by an overriding member to ensure the base class
 gets to set up and store the PhotoCanvas in the canvas member field.  More importantly,
 the activated signal is called here, and should only be called once the tool is completely
 initialized.*/
#line 325 "EditingTools.vala"
static void editing_tool_real_activate (EditingTool* self, PhotoCanvas* canvas) {
#line 1595 "EditingTools.c"
	PhotoCanvas* _tmp1_;
	PhotoCanvas* _tmp0_;
#line 325 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 325 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (canvas));
#line 1602 "EditingTools.c"
	/* multiple activates are not tolerated*/
#line 327 "EditingTools.vala"
	g_assert (self->canvas == NULL);
#line 1606 "EditingTools.c"
	_tmp1_ = NULL;
#line 329 "EditingTools.vala"
	_tmp0_ = NULL;
#line 329 "EditingTools.vala"
	self->canvas = (_tmp1_ = (_tmp0_ = canvas, (_tmp0_ == NULL) ? NULL : photo_canvas_ref (_tmp0_)), (self->canvas == NULL) ? NULL : (self->canvas = (photo_canvas_unref (self->canvas), NULL)), _tmp1_);
#line 331 "EditingTools.vala"
	g_signal_emit_by_name (self, "activated");
#line 1614 "EditingTools.c"
}


#line 325 "EditingTools.vala"
void editing_tool_activate (EditingTool* self, PhotoCanvas* canvas) {
#line 325 "EditingTools.vala"
	EDITING_TOOL_GET_CLASS (self)->activate (self, canvas);
#line 1622 "EditingTools.c"
}


/* Like activate(), this should always be called from an overriding subclass.*/
#line 335 "EditingTools.vala"
static void editing_tool_real_deactivate (EditingTool* self) {
#line 1629 "EditingTools.c"
	PhotoCanvas* _tmp0_;
#line 335 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 1633 "EditingTools.c"
	/* multiple deactivates are tolerated*/
#line 337 "EditingTools.vala"
	if (self->canvas == NULL) {
#line 1637 "EditingTools.c"
		return;
	}
	_tmp0_ = NULL;
#line 340 "EditingTools.vala"
	self->canvas = (_tmp0_ = NULL, (self->canvas == NULL) ? NULL : (self->canvas = (photo_canvas_unref (self->canvas), NULL)), _tmp0_);
#line 342 "EditingTools.vala"
	g_signal_emit_by_name (self, "deactivated");
#line 1645 "EditingTools.c"
}


#line 335 "EditingTools.vala"
void editing_tool_deactivate (EditingTool* self) {
#line 335 "EditingTools.vala"
	EDITING_TOOL_GET_CLASS (self)->deactivate (self);
#line 1653 "EditingTools.c"
}


#line 345 "EditingTools.vala"
gboolean editing_tool_is_activated (EditingTool* self) {
#line 1659 "EditingTools.c"
	gboolean result;
#line 345 "EditingTools.vala"
	g_return_val_if_fail (IS_EDITING_TOOL (self), FALSE);
#line 1663 "EditingTools.c"
	result = self->canvas != NULL;
	return result;
}


#line 349 "EditingTools.vala"
static EditingToolWindow* editing_tool_real_get_tool_window (EditingTool* self) {
#line 1671 "EditingTools.c"
	EditingToolWindow* result;
#line 349 "EditingTools.vala"
	g_return_val_if_fail (IS_EDITING_TOOL (self), NULL);
#line 1675 "EditingTools.c"
	result = NULL;
	return result;
}


#line 349 "EditingTools.vala"
EditingToolWindow* editing_tool_get_tool_window (EditingTool* self) {
#line 349 "EditingTools.vala"
	return EDITING_TOOL_GET_CLASS (self)->get_tool_window (self);
#line 1685 "EditingTools.c"
}


/* This allows the EditingTool to specify which pixbuf to display during the tool's
 operation.  Returning null means the host should use the pixbuf associated with the current
 Photo.  Note: This will be called before activate(), primarily to display the pixbuf before
 the tool is on the screen, and before paint_full() is hooked in.  It also means the PhotoCanvas
 will have this pixbuf rather than one from the Photo class.

 Note this this method doesn't need to be returning the "proper" pixbuf on-the-fly (i.e.
 a pixbuf with unsaved tool edits in it).  That can be handled in the paint() virtual method.*/
#line 361 "EditingTools.vala"
static GdkPixbuf* editing_tool_real_get_unscaled_pixbuf (EditingTool* self, Photo* photo) {
#line 1699 "EditingTools.c"
	GdkPixbuf* result;
#line 361 "EditingTools.vala"
	g_return_val_if_fail (IS_EDITING_TOOL (self), NULL);
#line 361 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO (photo), NULL);
#line 1705 "EditingTools.c"
	result = NULL;
	return result;
}


#line 361 "EditingTools.vala"
GdkPixbuf* editing_tool_get_unscaled_pixbuf (EditingTool* self, Photo* photo) {
#line 361 "EditingTools.vala"
	return EDITING_TOOL_GET_CLASS (self)->get_unscaled_pixbuf (self, photo);
#line 1715 "EditingTools.c"
}


#line 365 "EditingTools.vala"
static void editing_tool_real_on_left_click (EditingTool* self, gint x, gint y) {
#line 365 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 1723 "EditingTools.c"
}


#line 365 "EditingTools.vala"
void editing_tool_on_left_click (EditingTool* self, gint x, gint y) {
#line 365 "EditingTools.vala"
	EDITING_TOOL_GET_CLASS (self)->on_left_click (self, x, y);
#line 1731 "EditingTools.c"
}


#line 368 "EditingTools.vala"
static void editing_tool_real_on_left_released (EditingTool* self, gint x, gint y) {
#line 368 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 1739 "EditingTools.c"
}


#line 368 "EditingTools.vala"
void editing_tool_on_left_released (EditingTool* self, gint x, gint y) {
#line 368 "EditingTools.vala"
	EDITING_TOOL_GET_CLASS (self)->on_left_released (self, x, y);
#line 1747 "EditingTools.c"
}


#line 371 "EditingTools.vala"
static void editing_tool_real_on_motion (EditingTool* self, gint x, gint y, GdkModifierType mask) {
#line 371 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 1755 "EditingTools.c"
}


#line 371 "EditingTools.vala"
void editing_tool_on_motion (EditingTool* self, gint x, gint y, GdkModifierType mask) {
#line 371 "EditingTools.vala"
	EDITING_TOOL_GET_CLASS (self)->on_motion (self, x, y, mask);
#line 1763 "EditingTools.c"
}


#line 374 "EditingTools.vala"
static gboolean editing_tool_real_on_keypress (EditingTool* self, const GdkEventKey* event) {
#line 1769 "EditingTools.c"
	gboolean result;
#line 374 "EditingTools.vala"
	g_return_val_if_fail (IS_EDITING_TOOL (self), FALSE);
#line 1773 "EditingTools.c"
	result = FALSE;
	return result;
}


#line 374 "EditingTools.vala"
gboolean editing_tool_on_keypress (EditingTool* self, const GdkEventKey* event) {
#line 374 "EditingTools.vala"
	return EDITING_TOOL_GET_CLASS (self)->on_keypress (self, event);
#line 1783 "EditingTools.c"
}


#line 378 "EditingTools.vala"
static void editing_tool_real_paint (EditingTool* self, GdkGC* gc, GdkDrawable* drawable) {
#line 378 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 378 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 378 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 1795 "EditingTools.c"
}


#line 378 "EditingTools.vala"
void editing_tool_paint (EditingTool* self, GdkGC* gc, GdkDrawable* drawable) {
#line 378 "EditingTools.vala"
	EDITING_TOOL_GET_CLASS (self)->paint (self, gc, drawable);
#line 1803 "EditingTools.c"
}


#line 381 "EditingTools.vala"
void editing_tool_notify_apply (EditingTool* self) {
#line 381 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 382 "EditingTools.vala"
	g_signal_emit_by_name (self, "applied");
#line 1813 "EditingTools.c"
}


#line 385 "EditingTools.vala"
void editing_tool_notify_cancel (EditingTool* self) {
#line 385 "EditingTools.vala"
	g_return_if_fail (IS_EDITING_TOOL (self));
#line 386 "EditingTools.vala"
	g_signal_emit_by_name (self, "cancelled");
#line 1823 "EditingTools.c"
}


static void value_editing_tool_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_editing_tool_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		editing_tool_unref (value->data[0].v_pointer);
	}
}


static void value_editing_tool_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = editing_tool_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_editing_tool_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_editing_tool_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		EditingTool* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = editing_tool_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_editing_tool_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	EditingTool** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = editing_tool_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_editing_tool (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecEditingTool* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_EDITING_TOOL), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_editing_tool (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_EDITING_TOOL), NULL);
	return value->data[0].v_pointer;
}


void value_set_editing_tool (GValue* value, gpointer v_object) {
	EditingTool* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_EDITING_TOOL));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_EDITING_TOOL));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		editing_tool_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		editing_tool_unref (old);
	}
}


static void editing_tool_class_init (EditingToolClass * klass) {
	editing_tool_parent_class = g_type_class_peek_parent (klass);
	EDITING_TOOL_CLASS (klass)->finalize = editing_tool_finalize;
	EDITING_TOOL_CLASS (klass)->activate = editing_tool_real_activate;
	EDITING_TOOL_CLASS (klass)->deactivate = editing_tool_real_deactivate;
	EDITING_TOOL_CLASS (klass)->get_tool_window = editing_tool_real_get_tool_window;
	EDITING_TOOL_CLASS (klass)->get_unscaled_pixbuf = editing_tool_real_get_unscaled_pixbuf;
	EDITING_TOOL_CLASS (klass)->on_left_click = editing_tool_real_on_left_click;
	EDITING_TOOL_CLASS (klass)->on_left_released = editing_tool_real_on_left_released;
	EDITING_TOOL_CLASS (klass)->on_motion = editing_tool_real_on_motion;
	EDITING_TOOL_CLASS (klass)->on_keypress = editing_tool_real_on_keypress;
	EDITING_TOOL_CLASS (klass)->paint = editing_tool_real_paint;
	g_signal_new ("activated", TYPE_EDITING_TOOL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("deactivated", TYPE_EDITING_TOOL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("applied", TYPE_EDITING_TOOL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("cancelled", TYPE_EDITING_TOOL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void editing_tool_instance_init (EditingTool * self) {
	self->canvas = NULL;
	self->ref_count = 1;
}


static void editing_tool_finalize (EditingTool* obj) {
	EditingTool * self;
	self = EDITING_TOOL (obj);
	(self->canvas == NULL) ? NULL : (self->canvas = (photo_canvas_unref (self->canvas), NULL));
}


GType editing_tool_get_type (void) {
	static GType editing_tool_type_id = 0;
	if (editing_tool_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_editing_tool_init, value_editing_tool_free_value, value_editing_tool_copy_value, value_editing_tool_peek_pointer, "p", value_editing_tool_collect_value, "p", value_editing_tool_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (EditingToolClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) editing_tool_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EditingTool), 0, (GInstanceInitFunc) editing_tool_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		editing_tool_type_id = g_type_register_fundamental (g_type_fundamental_next (), "EditingTool", &g_define_type_info, &g_define_type_fundamental_info, G_TYPE_FLAG_ABSTRACT);
	}
	return editing_tool_type_id;
}


gpointer editing_tool_ref (gpointer instance) {
	EditingTool* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void editing_tool_unref (gpointer instance) {
	EditingTool* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		EDITING_TOOL_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 439 "EditingTools.vala"
CropTool* crop_tool_construct (GType object_type) {
#line 1984 "EditingTools.c"
	CropTool* self;
	self = (CropTool*) g_type_create_instance (object_type);
	return self;
}


#line 439 "EditingTools.vala"
CropTool* crop_tool_new (void) {
#line 439 "EditingTools.vala"
	return crop_tool_construct (TYPE_CROP_TOOL);
#line 1995 "EditingTools.c"
}


#line 497 "EditingTools.vala"
static void _crop_tool_prepare_gc_photo_canvas_new_drawable (PhotoCanvas* _sender, GdkGC* default_gc, GdkDrawable* drawable, gpointer self) {
#line 2001 "EditingTools.c"
	crop_tool_prepare_gc (self, default_gc, drawable);
}


#line 527 "EditingTools.vala"
static void _crop_tool_on_resized_pixbuf_photo_canvas_resized_scaled_pixbuf (PhotoCanvas* _sender, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position, gpointer self) {
#line 2008 "EditingTools.c"
	crop_tool_on_resized_pixbuf (self, old_dim, scaled, scaled_position);
}


#line 599 "EditingTools.vala"
static void _crop_tool_on_crop_apply_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 2015 "EditingTools.c"
	crop_tool_on_crop_apply (self);
}


#line 385 "EditingTools.vala"
static void _editing_tool_notify_cancel_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 2022 "EditingTools.c"
	editing_tool_notify_cancel (self);
}


#line 442 "EditingTools.vala"
static void crop_tool_real_activate (EditingTool* base, PhotoCanvas* canvas) {
#line 2029 "EditingTools.c"
	CropTool * self;
	GdkDrawable* _tmp1_;
	GdkGC* _tmp0_;
	GdkPixbuf* _tmp2_;
	CropToolCropToolWindow* _tmp4_;
	GtkWindow* _tmp3_;
	Photo* _tmp5_;
	Dimensions _tmp6_ = {0};
	Dimensions uncropped_dim;
	Box crop = {0};
	Photo* _tmp7_;
	gboolean _tmp8_;
	Dimensions _tmp11_ = {0};
	GdkRectangle _tmp10_ = {0};
	self = CROP_TOOL (base);
#line 442 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (canvas));
#line 443 "EditingTools.vala"
	g_signal_connect (canvas, "new-drawable", (GCallback) _crop_tool_prepare_gc_photo_canvas_new_drawable, self);
#line 444 "EditingTools.vala"
	g_signal_connect (canvas, "resized-scaled-pixbuf", (GCallback) _crop_tool_on_resized_pixbuf_photo_canvas_resized_scaled_pixbuf, self);
#line 188 "EditingTools.vala"
	_tmp1_ = NULL;
#line 184 "EditingTools.vala"
	_tmp0_ = NULL;
#line 446 "EditingTools.vala"
	crop_tool_prepare_gc (self, _tmp0_ = photo_canvas_get_default_gc (canvas), _tmp1_ = photo_canvas_get_drawable (canvas));
#line 2057 "EditingTools.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 199 "EditingTools.vala"
	_tmp2_ = NULL;
#line 447 "EditingTools.vala"
	crop_tool_prepare_visuals (self, _tmp2_ = photo_canvas_get_scaled_pixbuf (canvas));
#line 2064 "EditingTools.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
	/* create the crop tool window, where the user can apply or cancel the crop*/
	_tmp4_ = NULL;
#line 176 "EditingTools.vala"
	_tmp3_ = NULL;
#line 450 "EditingTools.vala"
	self->priv->crop_tool_window = (_tmp4_ = g_object_ref_sink (crop_tool_crop_tool_window_new (_tmp3_ = photo_canvas_get_container (canvas))), (self->priv->crop_tool_window == NULL) ? NULL : (self->priv->crop_tool_window = (g_object_unref (self->priv->crop_tool_window), NULL)), _tmp4_);
#line 2072 "EditingTools.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 451 "EditingTools.vala"
	g_signal_connect (self->priv->crop_tool_window->apply_button, "clicked", (GCallback) _crop_tool_on_crop_apply_gtk_button_clicked, self);
#line 452 "EditingTools.vala"
	g_signal_connect (self->priv->crop_tool_window->cancel_button, "clicked", (GCallback) _editing_tool_notify_cancel_gtk_button_clicked, EDITING_TOOL (self));
#line 2078 "EditingTools.c"
	/* obtain crop dimensions and paint against the uncropped photo*/
#line 172 "EditingTools.vala"
	_tmp5_ = NULL;
#line 2082 "EditingTools.c"
	uncropped_dim = (_tmp6_ = photo_get_uncropped_dimensions (_tmp5_ = photo_canvas_get_photo (canvas)), (_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL)), _tmp6_);
#line 172 "EditingTools.vala"
	_tmp7_ = NULL;
#line 458 "EditingTools.vala"
	if ((_tmp8_ = !photo_get_crop (_tmp7_ = photo_canvas_get_photo (canvas), &crop), (_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL)), _tmp8_)) {
#line 2088 "EditingTools.c"
		gint xofs;
		gint yofs;
		Box _tmp9_ = {0};
		xofs = (gint) (uncropped_dim.width * CROP_TOOL_CROP_INIT_X_PCT);
		yofs = (gint) (uncropped_dim.height * CROP_TOOL_CROP_INIT_Y_PCT);
		/* initialize the actual crop in absolute coordinates, not relative
		 to the photo's position on the canvas*/
#line 464 "EditingTools.vala"
		crop = (box_init (&_tmp9_, xofs, yofs, uncropped_dim.width - xofs, uncropped_dim.height - yofs), _tmp9_);
#line 2098 "EditingTools.c"
	}
	/* scale the crop to the scaled photo's size ... the scaled crop is maintained in
	 coordinates not relative to photo's position on canvas*/
#line 469 "EditingTools.vala"
	self->priv->scaled_crop = box_get_scaled_proportional (&crop, &uncropped_dim, (_tmp11_ = dimensions_for_rectangle ((_tmp10_ = photo_canvas_get_scaled_pixbuf_position (canvas), &_tmp10_)), &_tmp11_));
#line 472 "EditingTools.vala"
	EDITING_TOOL_CLASS (crop_tool_parent_class)->activate (EDITING_TOOL (self), canvas);
#line 2106 "EditingTools.c"
}


#line 475 "EditingTools.vala"
static void crop_tool_real_deactivate (EditingTool* base) {
#line 2112 "EditingTools.c"
	CropTool * self;
	GdkCursor* _tmp2_;
	GdkWindow* _tmp1_;
	self = CROP_TOOL (base);
#line 476 "EditingTools.vala"
	if (self->priv->crop_tool_window != NULL) {
#line 2119 "EditingTools.c"
		CropToolCropToolWindow* _tmp0_;
#line 477 "EditingTools.vala"
		gtk_widget_hide (GTK_WIDGET (self->priv->crop_tool_window));
#line 2123 "EditingTools.c"
		_tmp0_ = NULL;
#line 478 "EditingTools.vala"
		self->priv->crop_tool_window = (_tmp0_ = NULL, (self->priv->crop_tool_window == NULL) ? NULL : (self->priv->crop_tool_window = (g_object_unref (self->priv->crop_tool_window), NULL)), _tmp0_);
#line 2127 "EditingTools.c"
	}
	/* make sure the cursor isn't set to a modify indicator*/
	_tmp2_ = NULL;
#line 180 "EditingTools.vala"
	_tmp1_ = NULL;
#line 482 "EditingTools.vala"
	gdk_window_set_cursor (_tmp1_ = photo_canvas_get_drawing_window (EDITING_TOOL (self)->canvas), _tmp2_ = gdk_cursor_new (GDK_ARROW));
#line 2135 "EditingTools.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (gdk_cursor_unref (_tmp2_), NULL));
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 484 "EditingTools.vala"
	EDITING_TOOL_CLASS (crop_tool_parent_class)->deactivate (EDITING_TOOL (self));
#line 2140 "EditingTools.c"
}


#line 487 "EditingTools.vala"
static EditingToolWindow* crop_tool_real_get_tool_window (EditingTool* base) {
#line 2146 "EditingTools.c"
	CropTool * self;
	EditingToolWindow* result;
	EditingToolWindow* _tmp0_;
	self = CROP_TOOL (base);
#line 488 "EditingTools.vala"
	_tmp0_ = NULL;
#line 2153 "EditingTools.c"
	result = (_tmp0_ = EDITING_TOOL_WINDOW (self->priv->crop_tool_window), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 491 "EditingTools.vala"
static GdkPixbuf* crop_tool_real_get_unscaled_pixbuf (EditingTool* base, Photo* photo) {
#line 2161 "EditingTools.c"
	CropTool * self;
	GdkPixbuf* result;
	GError * _inner_error_;
	GdkPixbuf* _tmp0_;
	self = CROP_TOOL (base);
#line 491 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO (photo), NULL);
#line 2169 "EditingTools.c"
	_inner_error_ = NULL;
	_tmp0_ = NULL;
#line 494 "EditingTools.vala"
	if (photo_has_crop (photo)) {
#line 2174 "EditingTools.c"
		GdkPixbuf* _tmp1_;
		_tmp1_ = NULL;
#line 494 "EditingTools.vala"
		_tmp0_ = (_tmp1_ = photo_get_pixbuf (photo, PHOTO_EXCEPTION_CROP, 0, GDK_INTERP_HYPER, &_inner_error_), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 2179 "EditingTools.c"
		if (_inner_error_ != NULL) {
			(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	} else {
		GdkPixbuf* _tmp2_;
		_tmp2_ = NULL;
#line 494 "EditingTools.vala"
		_tmp0_ = (_tmp2_ = NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
#line 2191 "EditingTools.c"
	}
	/* show the uncropped photo for editing, but return null if no crop so the current pixbuf
	 is used*/
	result = _tmp0_;
	return result;
}


#line 497 "EditingTools.vala"
static void crop_tool_prepare_gc (CropTool* self, GdkGC* default_gc, GdkDrawable* drawable) {
#line 2202 "EditingTools.c"
	GdkGCValues _tmp0_ = {0};
	GdkGCValues gc_values;
	GdkGCValuesMask mask;
	GdkGC* _tmp1_;
	GdkGC* _tmp2_;
	GdkGC* _tmp3_;
#line 497 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 497 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (default_gc));
#line 497 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 2215 "EditingTools.c"
	gc_values = (memset (&_tmp0_, 0, sizeof (GdkGCValues)), _tmp0_);
#line 499 "EditingTools.vala"
	gc_values.foreground = fetch_color ("#000", drawable);
#line 500 "EditingTools.vala"
	gc_values.function = GDK_COPY;
#line 501 "EditingTools.vala"
	gc_values.fill = GDK_SOLID;
#line 502 "EditingTools.vala"
	gc_values.line_width = 1;
#line 503 "EditingTools.vala"
	gc_values.line_style = GDK_LINE_SOLID;
#line 504 "EditingTools.vala"
	gc_values.cap_style = GDK_CAP_BUTT;
#line 505 "EditingTools.vala"
	gc_values.join_style = GDK_JOIN_MITER;
#line 2231 "EditingTools.c"
	mask = (((((GDK_GC_FOREGROUND | GDK_GC_FUNCTION) | GDK_GC_FILL) | GDK_GC_LINE_WIDTH) | GDK_GC_LINE_STYLE) | GDK_GC_CAP_STYLE) | GDK_GC_JOIN_STYLE;
	_tmp1_ = NULL;
#line 516 "EditingTools.vala"
	self->priv->wide_black_gc = (_tmp1_ = gdk_gc_new_with_values (drawable, &gc_values, mask), (self->priv->wide_black_gc == NULL) ? NULL : (self->priv->wide_black_gc = (g_object_unref (self->priv->wide_black_gc), NULL)), _tmp1_);
#line 518 "EditingTools.vala"
	gc_values.foreground = fetch_color ("#FFF", drawable);
#line 2238 "EditingTools.c"
	_tmp2_ = NULL;
#line 520 "EditingTools.vala"
	self->priv->wide_white_gc = (_tmp2_ = gdk_gc_new_with_values (drawable, &gc_values, mask), (self->priv->wide_white_gc == NULL) ? NULL : (self->priv->wide_white_gc = (g_object_unref (self->priv->wide_white_gc), NULL)), _tmp2_);
#line 522 "EditingTools.vala"
	gc_values.line_width = 0;
#line 2244 "EditingTools.c"
	_tmp3_ = NULL;
#line 524 "EditingTools.vala"
	self->priv->thin_white_gc = (_tmp3_ = gdk_gc_new_with_values (drawable, &gc_values, mask), (self->priv->thin_white_gc == NULL) ? NULL : (self->priv->thin_white_gc = (g_object_unref (self->priv->thin_white_gc), NULL)), _tmp3_);
#line 2248 "EditingTools.c"
}


#line 527 "EditingTools.vala"
static void crop_tool_on_resized_pixbuf (CropTool* self, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position) {
#line 2254 "EditingTools.c"
	Dimensions new_dim;
	Photo* _tmp0_;
	Dimensions _tmp1_ = {0};
	Dimensions uncropped_dim;
	Box crop;
#line 527 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 527 "EditingTools.vala"
	g_return_if_fail (GDK_IS_PIXBUF (scaled));
#line 2264 "EditingTools.c"
	new_dim = dimensions_for_pixbuf (scaled);
#line 172 "EditingTools.vala"
	_tmp0_ = NULL;
#line 2268 "EditingTools.c"
	uncropped_dim = (_tmp1_ = photo_get_uncropped_dimensions (_tmp0_ = photo_canvas_get_photo (EDITING_TOOL (self)->canvas)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	/* rescale to full crop*/
	crop = box_get_scaled_proportional (&self->priv->scaled_crop, &(*old_dim), &uncropped_dim);
	/* rescale back to new size*/
#line 535 "EditingTools.vala"
	self->priv->scaled_crop = box_get_scaled_proportional (&crop, &uncropped_dim, &new_dim);
#line 537 "EditingTools.vala"
	crop_tool_prepare_visuals (self, scaled);
#line 2277 "EditingTools.c"
}


#line 540 "EditingTools.vala"
static void crop_tool_prepare_visuals (CropTool* self, GdkPixbuf* pixbuf) {
#line 2283 "EditingTools.c"
	GdkPixbuf* _tmp0_;
#line 540 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 540 "EditingTools.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 2289 "EditingTools.c"
	/* create color shifted pixbuf for crop exterior*/
	_tmp0_ = NULL;
#line 542 "EditingTools.vala"
	self->priv->color_shifted = (_tmp0_ = gdk_pixbuf_copy (pixbuf), (self->priv->color_shifted == NULL) ? NULL : (self->priv->color_shifted = (g_object_unref (self->priv->color_shifted), NULL)), _tmp0_);
#line 543 "EditingTools.vala"
	shift_colors (self->priv->color_shifted, CROP_TOOL_CROP_EXTERIOR_RED_SHIFT, CROP_TOOL_CROP_EXTERIOR_GREEN_SHIFT, CROP_TOOL_CROP_EXTERIOR_BLUE_SHIFT, CROP_TOOL_CROP_EXTERIOR_ALPHA_SHIFT);
#line 2296 "EditingTools.c"
}


#line 547 "EditingTools.vala"
static void crop_tool_real_on_left_click (EditingTool* base, gint x, gint y) {
#line 2302 "EditingTools.c"
	CropTool * self;
	GdkRectangle scaled_pixbuf_pos;
	Box offset_scaled_crop;
	self = CROP_TOOL (base);
	scaled_pixbuf_pos = photo_canvas_get_scaled_pixbuf_position (EDITING_TOOL (self)->canvas);
	/* scaled_crop is not maintained relative to photo's position on canvas*/
	offset_scaled_crop = box_get_offset (&self->priv->scaled_crop, scaled_pixbuf_pos.x, scaled_pixbuf_pos.y);
	/* determine where the mouse down landed and store for future events*/
#line 554 "EditingTools.vala"
	self->priv->in_manipulation = box_approx_location (&offset_scaled_crop, x, y);
#line 555 "EditingTools.vala"
	self->priv->last_grab_x = x = x - scaled_pixbuf_pos.x;
#line 556 "EditingTools.vala"
	self->priv->last_grab_y = y = y - scaled_pixbuf_pos.y;
#line 2317 "EditingTools.c"
	/* repaint because the crop changes on a mouse down*/
#line 559 "EditingTools.vala"
	photo_canvas_repaint (EDITING_TOOL (self)->canvas);
#line 2321 "EditingTools.c"
}


#line 562 "EditingTools.vala"
static void crop_tool_real_on_left_released (EditingTool* base, gint x, gint y) {
#line 2327 "EditingTools.c"
	CropTool * self;
	self = CROP_TOOL (base);
	/* nothing to do if released outside of the crop box*/
#line 564 "EditingTools.vala"
	if (self->priv->in_manipulation == BOX_LOCATION_OUTSIDE) {
#line 2333 "EditingTools.c"
		return;
	}
	/* end manipulation*/
#line 568 "EditingTools.vala"
	self->priv->in_manipulation = BOX_LOCATION_OUTSIDE;
#line 569 "EditingTools.vala"
	self->priv->last_grab_x = -1;
#line 570 "EditingTools.vala"
	self->priv->last_grab_y = -1;
#line 572 "EditingTools.vala"
	crop_tool_update_cursor (self, x, y);
#line 2345 "EditingTools.c"
	/* repaint because crop changes when released*/
#line 575 "EditingTools.vala"
	photo_canvas_repaint (EDITING_TOOL (self)->canvas);
#line 2349 "EditingTools.c"
}


#line 578 "EditingTools.vala"
static void crop_tool_real_on_motion (EditingTool* base, gint x, gint y, GdkModifierType mask) {
#line 2355 "EditingTools.c"
	CropTool * self;
	self = CROP_TOOL (base);
	/* only deal with manipulating the crop tool when click-and-dragging one of the edges
	 or the interior*/
#line 581 "EditingTools.vala"
	if (self->priv->in_manipulation != BOX_LOCATION_OUTSIDE) {
#line 582 "EditingTools.vala"
		crop_tool_on_canvas_manipulation (self, x, y);
#line 2364 "EditingTools.c"
	}
#line 584 "EditingTools.vala"
	crop_tool_update_cursor (self, x, y);
#line 2368 "EditingTools.c"
}


#line 587 "EditingTools.vala"
static void crop_tool_real_paint (EditingTool* base, GdkGC* gc, GdkDrawable* drawable) {
#line 2374 "EditingTools.c"
	CropTool * self;
	GdkPixbuf* _tmp0_;
	self = CROP_TOOL (base);
#line 587 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 587 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 2382 "EditingTools.c"
	/* painter's algorithm: from the bottom up, starting with the color shifted portion of the
	 photo outside the crop*/
#line 590 "EditingTools.vala"
	photo_canvas_paint_pixbuf (EDITING_TOOL (self)->canvas, self->priv->color_shifted);
#line 2387 "EditingTools.c"
	/* paint exposed (cropped) part of pixbuf minus crop border*/
#line 199 "EditingTools.vala"
	_tmp0_ = NULL;
#line 593 "EditingTools.vala"
	photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, _tmp0_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), &self->priv->scaled_crop);
#line 2393 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	/* paint crop tool last*/
#line 596 "EditingTools.vala"
	crop_tool_paint_crop_tool (self, &self->priv->scaled_crop);
#line 2398 "EditingTools.c"
}


#line 599 "EditingTools.vala"
static void crop_tool_on_crop_apply (CropTool* self) {
#line 2404 "EditingTools.c"
	Dimensions _tmp3_ = {0};
	Dimensions _tmp2_ = {0};
	Photo* _tmp1_;
	GdkRectangle _tmp0_ = {0};
	Box _tmp4_ = {0};
	Box crop;
	Photo* _tmp5_;
#line 599 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 2414 "EditingTools.c"
	/* up-scale scaled crop to photo's dimensions*/
#line 172 "EditingTools.vala"
	_tmp1_ = NULL;
#line 2418 "EditingTools.c"
	crop = (_tmp4_ = box_get_scaled_proportional (&self->priv->scaled_crop, (_tmp2_ = dimensions_for_rectangle ((_tmp0_ = photo_canvas_get_scaled_pixbuf_position (EDITING_TOOL (self)->canvas), &_tmp0_)), &_tmp2_), (_tmp3_ = photo_get_uncropped_dimensions (_tmp1_ = photo_canvas_get_photo (EDITING_TOOL (self)->canvas)), &_tmp3_)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp4_);
	/* store the new crop*/
#line 172 "EditingTools.vala"
	_tmp5_ = NULL;
#line 606 "EditingTools.vala"
	photo_set_crop (_tmp5_ = photo_canvas_get_photo (EDITING_TOOL (self)->canvas), &crop);
#line 2425 "EditingTools.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
	/* signal application*/
#line 609 "EditingTools.vala"
	editing_tool_notify_apply (EDITING_TOOL (self));
#line 2430 "EditingTools.c"
}


#line 612 "EditingTools.vala"
static void crop_tool_update_cursor (CropTool* self, gint x, gint y) {
#line 2436 "EditingTools.c"
	GdkRectangle scaled_pos;
	Box offset_scaled_crop;
	GdkCursorType cursor_type;
#line 612 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 2442 "EditingTools.c"
	/* scaled_crop is not maintained relative to photo's position on canvas*/
	scaled_pos = photo_canvas_get_scaled_pixbuf_position (EDITING_TOOL (self)->canvas);
	offset_scaled_crop = box_get_offset (&self->priv->scaled_crop, scaled_pos.x, scaled_pos.y);
	cursor_type = GDK_ARROW;
#line 618 "EditingTools.vala"
	switch (box_approx_location (&offset_scaled_crop, x, y)) {
#line 2449 "EditingTools.c"
		case BOX_LOCATION_LEFT_SIDE:
		{
#line 620 "EditingTools.vala"
			cursor_type = GDK_LEFT_SIDE;
#line 621 "EditingTools.vala"
			break;
#line 2456 "EditingTools.c"
		}
		case BOX_LOCATION_TOP_SIDE:
		{
#line 624 "EditingTools.vala"
			cursor_type = GDK_TOP_SIDE;
#line 625 "EditingTools.vala"
			break;
#line 2464 "EditingTools.c"
		}
		case BOX_LOCATION_RIGHT_SIDE:
		{
#line 628 "EditingTools.vala"
			cursor_type = GDK_RIGHT_SIDE;
#line 629 "EditingTools.vala"
			break;
#line 2472 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_SIDE:
		{
#line 632 "EditingTools.vala"
			cursor_type = GDK_BOTTOM_SIDE;
#line 633 "EditingTools.vala"
			break;
#line 2480 "EditingTools.c"
		}
		case BOX_LOCATION_TOP_LEFT:
		{
#line 636 "EditingTools.vala"
			cursor_type = GDK_TOP_LEFT_CORNER;
#line 637 "EditingTools.vala"
			break;
#line 2488 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_LEFT:
		{
#line 640 "EditingTools.vala"
			cursor_type = GDK_BOTTOM_LEFT_CORNER;
#line 641 "EditingTools.vala"
			break;
#line 2496 "EditingTools.c"
		}
		case BOX_LOCATION_TOP_RIGHT:
		{
#line 644 "EditingTools.vala"
			cursor_type = GDK_TOP_RIGHT_CORNER;
#line 645 "EditingTools.vala"
			break;
#line 2504 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_RIGHT:
		{
#line 648 "EditingTools.vala"
			cursor_type = GDK_BOTTOM_RIGHT_CORNER;
#line 649 "EditingTools.vala"
			break;
#line 2512 "EditingTools.c"
		}
		case BOX_LOCATION_INSIDE:
		{
#line 652 "EditingTools.vala"
			cursor_type = GDK_FLEUR;
#line 653 "EditingTools.vala"
			break;
#line 2520 "EditingTools.c"
		}
		default:
		{
#line 657 "EditingTools.vala"
			break;
#line 2526 "EditingTools.c"
		}
	}
#line 660 "EditingTools.vala"
	if (cursor_type != self->priv->current_cursor_type) {
#line 2531 "EditingTools.c"
		GdkCursor* cursor;
		GdkWindow* _tmp0_;
		cursor = gdk_cursor_new (cursor_type);
#line 180 "EditingTools.vala"
		_tmp0_ = NULL;
#line 662 "EditingTools.vala"
		gdk_window_set_cursor (_tmp0_ = photo_canvas_get_drawing_window (EDITING_TOOL (self)->canvas), cursor);
#line 2539 "EditingTools.c"
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 663 "EditingTools.vala"
		self->priv->current_cursor_type = cursor_type;
#line 2543 "EditingTools.c"
		(cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL));
	}
}


#line 667 "EditingTools.vala"
static gboolean crop_tool_on_canvas_manipulation (CropTool* self, gint x, gint y) {
#line 2551 "EditingTools.c"
	gboolean result;
	GdkRectangle scaled_pos;
	gint left;
	gint top;
	gint right;
	gint bottom;
	gint width;
	gint height;
	Box _tmp0_ = {0};
	Box new_crop;
#line 667 "EditingTools.vala"
	g_return_val_if_fail (IS_CROP_TOOL (self), FALSE);
#line 2564 "EditingTools.c"
	scaled_pos = photo_canvas_get_scaled_pixbuf_position (EDITING_TOOL (self)->canvas);
	/* scaled_crop is maintained in coordinates non-relative to photo's position on canvas ...
	 but bound tool to photo itself*/
#line 672 "EditingTools.vala"
	x = x - scaled_pos.x;
#line 673 "EditingTools.vala"
	if (x < 0) {
#line 674 "EditingTools.vala"
		x = 0;
#line 2574 "EditingTools.c"
	} else {
#line 675 "EditingTools.vala"
		if (x >= scaled_pos.width) {
#line 676 "EditingTools.vala"
			x = scaled_pos.width - 1;
#line 2580 "EditingTools.c"
		}
	}
#line 678 "EditingTools.vala"
	y = y - scaled_pos.y;
#line 679 "EditingTools.vala"
	if (y < 0) {
#line 680 "EditingTools.vala"
		y = 0;
#line 2589 "EditingTools.c"
	} else {
#line 681 "EditingTools.vala"
		if (y >= scaled_pos.height) {
#line 682 "EditingTools.vala"
			y = scaled_pos.height - 1;
#line 2595 "EditingTools.c"
		}
	}
	/* need to make manipulations outside of box structure, because its methods do sanity
	 checking*/
	left = self->priv->scaled_crop.left;
	top = self->priv->scaled_crop.top;
	right = self->priv->scaled_crop.right;
	bottom = self->priv->scaled_crop.bottom;
#line 691 "EditingTools.vala"
	switch (self->priv->in_manipulation) {
#line 2606 "EditingTools.c"
		case BOX_LOCATION_LEFT_SIDE:
		{
#line 693 "EditingTools.vala"
			left = x;
#line 694 "EditingTools.vala"
			break;
#line 2613 "EditingTools.c"
		}
		case BOX_LOCATION_TOP_SIDE:
		{
#line 697 "EditingTools.vala"
			top = y;
#line 698 "EditingTools.vala"
			break;
#line 2621 "EditingTools.c"
		}
		case BOX_LOCATION_RIGHT_SIDE:
		{
#line 701 "EditingTools.vala"
			right = x;
#line 702 "EditingTools.vala"
			break;
#line 2629 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_SIDE:
		{
#line 705 "EditingTools.vala"
			bottom = y;
#line 706 "EditingTools.vala"
			break;
#line 2637 "EditingTools.c"
		}
		case BOX_LOCATION_TOP_LEFT:
		{
#line 709 "EditingTools.vala"
			top = y;
#line 710 "EditingTools.vala"
			left = x;
#line 711 "EditingTools.vala"
			break;
#line 2647 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_LEFT:
		{
#line 714 "EditingTools.vala"
			bottom = y;
#line 715 "EditingTools.vala"
			left = x;
#line 716 "EditingTools.vala"
			break;
#line 2657 "EditingTools.c"
		}
		case BOX_LOCATION_TOP_RIGHT:
		{
#line 719 "EditingTools.vala"
			top = y;
#line 720 "EditingTools.vala"
			right = x;
#line 721 "EditingTools.vala"
			break;
#line 2667 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_RIGHT:
		{
#line 724 "EditingTools.vala"
			bottom = y;
#line 725 "EditingTools.vala"
			right = x;
#line 726 "EditingTools.vala"
			break;
#line 2677 "EditingTools.c"
		}
		case BOX_LOCATION_INSIDE:
		{
			gint delta_x;
			gint delta_y;
			gint width;
			gint height;
			gint adj_width;
			gint adj_height;
#line 729 "EditingTools.vala"
			g_assert (self->priv->last_grab_x >= 0);
#line 730 "EditingTools.vala"
			g_assert (self->priv->last_grab_y >= 0);
#line 2691 "EditingTools.c"
			delta_x = x - self->priv->last_grab_x;
			delta_y = y - self->priv->last_grab_y;
#line 735 "EditingTools.vala"
			self->priv->last_grab_x = x;
#line 736 "EditingTools.vala"
			self->priv->last_grab_y = y;
#line 2698 "EditingTools.c"
			width = (right - left) + 1;
			height = (bottom - top) + 1;
#line 741 "EditingTools.vala"
			left = left + delta_x;
#line 742 "EditingTools.vala"
			top = top + delta_y;
#line 743 "EditingTools.vala"
			right = right + delta_x;
#line 744 "EditingTools.vala"
			bottom = bottom + delta_y;
#line 747 "EditingTools.vala"
			if (left < 0) {
#line 748 "EditingTools.vala"
				left = 0;
#line 2713 "EditingTools.c"
			}
#line 750 "EditingTools.vala"
			if (top < 0) {
#line 751 "EditingTools.vala"
				top = 0;
#line 2719 "EditingTools.c"
			}
#line 753 "EditingTools.vala"
			if (right >= scaled_pos.width) {
#line 754 "EditingTools.vala"
				right = scaled_pos.width - 1;
#line 2725 "EditingTools.c"
			}
#line 756 "EditingTools.vala"
			if (bottom >= scaled_pos.height) {
#line 757 "EditingTools.vala"
				bottom = scaled_pos.height - 1;
#line 2731 "EditingTools.c"
			}
			adj_width = (right - left) + 1;
			adj_height = (bottom - top) + 1;
#line 763 "EditingTools.vala"
			if (adj_width != width) {
#line 764 "EditingTools.vala"
				if (delta_x < 0) {
#line 765 "EditingTools.vala"
					right = (left + width) - 1;
#line 2741 "EditingTools.c"
				} else {
#line 767 "EditingTools.vala"
					left = (right - width) + 1;
#line 2745 "EditingTools.c"
				}
			}
#line 770 "EditingTools.vala"
			if (adj_height != height) {
#line 771 "EditingTools.vala"
				if (delta_y < 0) {
#line 772 "EditingTools.vala"
					bottom = (top + height) - 1;
#line 2754 "EditingTools.c"
				} else {
#line 774 "EditingTools.vala"
					top = (bottom - height) + 1;
#line 2758 "EditingTools.c"
				}
			}
#line 776 "EditingTools.vala"
			break;
#line 2763 "EditingTools.c"
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
	width = (right - left) + 1;
	height = (bottom - top) + 1;
	/* max sure minimums are respected ... have to adjust the right value depending on what's
	 being manipulated*/
#line 788 "EditingTools.vala"
	switch (self->priv->in_manipulation) {
#line 2777 "EditingTools.c"
		case BOX_LOCATION_LEFT_SIDE:
		case BOX_LOCATION_TOP_LEFT:
		case BOX_LOCATION_BOTTOM_LEFT:
		{
#line 792 "EditingTools.vala"
			if (width < CROP_TOOL_CROP_MIN_WIDTH) {
#line 793 "EditingTools.vala"
				left = right - CROP_TOOL_CROP_MIN_WIDTH;
#line 2786 "EditingTools.c"
			}
#line 794 "EditingTools.vala"
			break;
#line 2790 "EditingTools.c"
		}
		case BOX_LOCATION_RIGHT_SIDE:
		case BOX_LOCATION_TOP_RIGHT:
		case BOX_LOCATION_BOTTOM_RIGHT:
		{
#line 799 "EditingTools.vala"
			if (width < CROP_TOOL_CROP_MIN_WIDTH) {
#line 800 "EditingTools.vala"
				right = left + CROP_TOOL_CROP_MIN_WIDTH;
#line 2800 "EditingTools.c"
			}
#line 801 "EditingTools.vala"
			break;
#line 2804 "EditingTools.c"
		}
		default:
		{
#line 804 "EditingTools.vala"
			break;
#line 2810 "EditingTools.c"
		}
	}
#line 807 "EditingTools.vala"
	switch (self->priv->in_manipulation) {
#line 2815 "EditingTools.c"
		case BOX_LOCATION_TOP_SIDE:
		case BOX_LOCATION_TOP_LEFT:
		case BOX_LOCATION_TOP_RIGHT:
		{
#line 811 "EditingTools.vala"
			if (height < CROP_TOOL_CROP_MIN_HEIGHT) {
#line 812 "EditingTools.vala"
				top = bottom - CROP_TOOL_CROP_MIN_HEIGHT;
#line 2824 "EditingTools.c"
			}
#line 813 "EditingTools.vala"
			break;
#line 2828 "EditingTools.c"
		}
		case BOX_LOCATION_BOTTOM_SIDE:
		case BOX_LOCATION_BOTTOM_LEFT:
		case BOX_LOCATION_BOTTOM_RIGHT:
		{
#line 818 "EditingTools.vala"
			if (height < CROP_TOOL_CROP_MIN_HEIGHT) {
#line 819 "EditingTools.vala"
				bottom = top + CROP_TOOL_CROP_MIN_HEIGHT;
#line 2838 "EditingTools.c"
			}
#line 820 "EditingTools.vala"
			break;
#line 2842 "EditingTools.c"
		}
		default:
		{
#line 823 "EditingTools.vala"
			break;
#line 2848 "EditingTools.c"
		}
	}
	new_crop = (box_init (&_tmp0_, left, top, right, bottom), _tmp0_);
#line 828 "EditingTools.vala"
	if (self->priv->in_manipulation != BOX_LOCATION_INSIDE) {
#line 829 "EditingTools.vala"
		crop_tool_crop_resized (self, &new_crop);
#line 2856 "EditingTools.c"
	} else {
#line 831 "EditingTools.vala"
		crop_tool_crop_moved (self, &new_crop);
#line 2860 "EditingTools.c"
	}
	/* load new values*/
#line 834 "EditingTools.vala"
	self->priv->scaled_crop = new_crop;
#line 2865 "EditingTools.c"
	result = FALSE;
	return result;
}


#line 839 "EditingTools.vala"
static void crop_tool_crop_resized (CropTool* self, const Box* new_crop) {
#line 2873 "EditingTools.c"
	Box horizontal = {0};
	gboolean horizontal_enlarged;
	Box vertical = {0};
	gboolean vertical_enlarged;
	BoxComplements complements;
	gboolean _tmp0_;
	gboolean _tmp6_;
#line 839 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 840 "EditingTools.vala"
	if (box_equals (&self->priv->scaled_crop, &(*new_crop))) {
#line 2885 "EditingTools.c"
		/* no change*/
		return;
	}
	/* erase crop and rule-of-thirds lines*/
#line 846 "EditingTools.vala"
	crop_tool_erase_crop_tool (self, &self->priv->scaled_crop);
#line 847 "EditingTools.vala"
	photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &self->priv->scaled_crop);
#line 2894 "EditingTools.c"
	horizontal_enlarged = FALSE;
	vertical_enlarged = FALSE;
	complements = box_resized_complements (&self->priv->scaled_crop, &(*new_crop), &horizontal, &horizontal_enlarged, &vertical, &vertical_enlarged);
	/* this should never happen ... this means that the operation wasn't a resize*/
#line 857 "EditingTools.vala"
	g_assert (complements != BOX_COMPLEMENTS_NONE);
#line 2901 "EditingTools.c"
	_tmp0_ = FALSE;
#line 859 "EditingTools.vala"
	if (complements == BOX_COMPLEMENTS_HORIZONTAL) {
#line 859 "EditingTools.vala"
		_tmp0_ = TRUE;
#line 2907 "EditingTools.c"
	} else {
#line 859 "EditingTools.vala"
		_tmp0_ = complements == BOX_COMPLEMENTS_BOTH;
#line 2911 "EditingTools.c"
	}
#line 859 "EditingTools.vala"
	if (_tmp0_) {
#line 2915 "EditingTools.c"
		GdkPixbuf* _tmp1_;
		GdkPixbuf* _tmp5_;
		GdkPixbuf* pb;
		_tmp1_ = NULL;
#line 860 "EditingTools.vala"
		if (horizontal_enlarged) {
#line 2922 "EditingTools.c"
			GdkPixbuf* _tmp2_;
			_tmp2_ = NULL;
#line 860 "EditingTools.vala"
			_tmp1_ = (_tmp2_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_);
#line 2927 "EditingTools.c"
		} else {
			GdkPixbuf* _tmp4_;
			GdkPixbuf* _tmp3_;
			_tmp4_ = NULL;
#line 860 "EditingTools.vala"
			_tmp3_ = NULL;
#line 860 "EditingTools.vala"
			_tmp1_ = (_tmp4_ = (_tmp3_ = self->priv->color_shifted, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp4_);
#line 2936 "EditingTools.c"
		}
#line 860 "EditingTools.vala"
		_tmp5_ = NULL;
#line 2940 "EditingTools.c"
		pb = (_tmp5_ = _tmp1_, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
#line 861 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, pb, &horizontal);
#line 863 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &horizontal);
#line 2946 "EditingTools.c"
		(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
		(pb == NULL) ? NULL : (pb = (g_object_unref (pb), NULL));
	}
	_tmp6_ = FALSE;
#line 866 "EditingTools.vala"
	if (complements == BOX_COMPLEMENTS_VERTICAL) {
#line 866 "EditingTools.vala"
		_tmp6_ = TRUE;
#line 2955 "EditingTools.c"
	} else {
#line 866 "EditingTools.vala"
		_tmp6_ = complements == BOX_COMPLEMENTS_BOTH;
#line 2959 "EditingTools.c"
	}
#line 866 "EditingTools.vala"
	if (_tmp6_) {
#line 2963 "EditingTools.c"
		GdkPixbuf* _tmp7_;
		GdkPixbuf* _tmp11_;
		GdkPixbuf* pb;
		_tmp7_ = NULL;
#line 867 "EditingTools.vala"
		if (vertical_enlarged) {
#line 2970 "EditingTools.c"
			GdkPixbuf* _tmp8_;
			_tmp8_ = NULL;
#line 867 "EditingTools.vala"
			_tmp7_ = (_tmp8_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), (_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL)), _tmp8_);
#line 2975 "EditingTools.c"
		} else {
			GdkPixbuf* _tmp10_;
			GdkPixbuf* _tmp9_;
			_tmp10_ = NULL;
#line 867 "EditingTools.vala"
			_tmp9_ = NULL;
#line 867 "EditingTools.vala"
			_tmp7_ = (_tmp10_ = (_tmp9_ = self->priv->color_shifted, (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL)), _tmp10_);
#line 2984 "EditingTools.c"
		}
#line 867 "EditingTools.vala"
		_tmp11_ = NULL;
#line 2988 "EditingTools.c"
		pb = (_tmp11_ = _tmp7_, (_tmp11_ == NULL) ? NULL : g_object_ref (_tmp11_));
#line 868 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, pb, &vertical);
#line 870 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &vertical);
#line 2994 "EditingTools.c"
		(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
		(pb == NULL) ? NULL : (pb = (g_object_unref (pb), NULL));
	}
#line 873 "EditingTools.vala"
	crop_tool_paint_crop_tool (self, &(*new_crop));
#line 874 "EditingTools.vala"
	photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &(*new_crop));
#line 3002 "EditingTools.c"
}


#line 877 "EditingTools.vala"
static void crop_tool_crop_moved (CropTool* self, const Box* new_crop) {
#line 3008 "EditingTools.c"
	Box scaled_horizontal = {0};
	Box scaled_vertical = {0};
	Box new_horizontal = {0};
	Box new_vertical = {0};
	BoxComplements complements;
	gboolean _tmp0_;
	gboolean _tmp2_;
#line 877 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 878 "EditingTools.vala"
	if (box_equals (&self->priv->scaled_crop, &(*new_crop))) {
#line 3020 "EditingTools.c"
		/* no change*/
		return;
	}
	/* erase crop and rule-of-thirds lines*/
#line 884 "EditingTools.vala"
	crop_tool_erase_crop_tool (self, &self->priv->scaled_crop);
#line 885 "EditingTools.vala"
	photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &self->priv->scaled_crop);
#line 3029 "EditingTools.c"
	complements = box_shifted_complements (&self->priv->scaled_crop, &(*new_crop), &scaled_horizontal, &scaled_vertical, &new_horizontal, &new_vertical);
	_tmp0_ = FALSE;
#line 894 "EditingTools.vala"
	if (complements == BOX_COMPLEMENTS_HORIZONTAL) {
#line 894 "EditingTools.vala"
		_tmp0_ = TRUE;
#line 3036 "EditingTools.c"
	} else {
#line 894 "EditingTools.vala"
		_tmp0_ = complements == BOX_COMPLEMENTS_BOTH;
#line 3040 "EditingTools.c"
	}
#line 894 "EditingTools.vala"
	if (_tmp0_) {
#line 3044 "EditingTools.c"
		GdkPixbuf* _tmp1_;
		/* paint in the horizontal complements appropriately*/
#line 896 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, self->priv->color_shifted, &scaled_horizontal);
#line 199 "EditingTools.vala"
		_tmp1_ = NULL;
#line 897 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, _tmp1_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), &new_horizontal);
#line 3053 "EditingTools.c"
		(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 899 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &scaled_horizontal);
#line 900 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &new_horizontal);
#line 3059 "EditingTools.c"
	}
	_tmp2_ = FALSE;
#line 903 "EditingTools.vala"
	if (complements == BOX_COMPLEMENTS_VERTICAL) {
#line 903 "EditingTools.vala"
		_tmp2_ = TRUE;
#line 3066 "EditingTools.c"
	} else {
#line 903 "EditingTools.vala"
		_tmp2_ = complements == BOX_COMPLEMENTS_BOTH;
#line 3070 "EditingTools.c"
	}
#line 903 "EditingTools.vala"
	if (_tmp2_) {
#line 3074 "EditingTools.c"
		GdkPixbuf* _tmp3_;
		/* paint in vertical complements appropriately*/
#line 905 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, self->priv->color_shifted, &scaled_vertical);
#line 199 "EditingTools.vala"
		_tmp3_ = NULL;
#line 906 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, _tmp3_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), &new_vertical);
#line 3083 "EditingTools.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 908 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &scaled_vertical);
#line 909 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &new_vertical);
#line 3089 "EditingTools.c"
	}
#line 912 "EditingTools.vala"
	if (complements == BOX_COMPLEMENTS_NONE) {
#line 3093 "EditingTools.c"
		GdkPixbuf* _tmp4_;
		/* this means the two boxes have no intersection, not that they're equal ... since
		 there's no intersection, fill in both new and old with apropriate pixbufs*/
#line 915 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, self->priv->color_shifted, &self->priv->scaled_crop);
#line 199 "EditingTools.vala"
		_tmp4_ = NULL;
#line 916 "EditingTools.vala"
		photo_canvas_paint_pixbuf_area (EDITING_TOOL (self)->canvas, _tmp4_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), &(*new_crop));
#line 3103 "EditingTools.c"
		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 918 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &self->priv->scaled_crop);
#line 919 "EditingTools.vala"
		photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &(*new_crop));
#line 3109 "EditingTools.c"
	}
	/* paint crop in new location*/
#line 923 "EditingTools.vala"
	crop_tool_paint_crop_tool (self, &(*new_crop));
#line 924 "EditingTools.vala"
	photo_canvas_invalidate_area (EDITING_TOOL (self)->canvas, &(*new_crop));
#line 3116 "EditingTools.c"
}


#line 927 "EditingTools.vala"
static void crop_tool_paint_crop_tool (CropTool* self, const Box* crop) {
#line 3122 "EditingTools.c"
	Box _tmp0_ = {0};
	Box _tmp1_ = {0};
#line 927 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 3127 "EditingTools.c"
	/* paint rule-of-thirds lines if user is manipulating the crop*/
#line 929 "EditingTools.vala"
	if (self->priv->in_manipulation != BOX_LOCATION_OUTSIDE) {
#line 3131 "EditingTools.c"
		gint one_third_x;
		gint one_third_y;
		one_third_x = box_get_width (&(*crop)) / 3;
		one_third_y = box_get_height (&(*crop)) / 3;
#line 933 "EditingTools.vala"
		photo_canvas_draw_horizontal_line (EDITING_TOOL (self)->canvas, self->priv->thin_white_gc, (*crop).left, (*crop).top + one_third_y, box_get_width (&(*crop)));
#line 934 "EditingTools.vala"
		photo_canvas_draw_horizontal_line (EDITING_TOOL (self)->canvas, self->priv->thin_white_gc, (*crop).left, (*crop).top + (one_third_y * 2), box_get_width (&(*crop)));
#line 936 "EditingTools.vala"
		photo_canvas_draw_vertical_line (EDITING_TOOL (self)->canvas, self->priv->thin_white_gc, (*crop).left + one_third_x, (*crop).top, box_get_height (&(*crop)));
#line 937 "EditingTools.vala"
		photo_canvas_draw_vertical_line (EDITING_TOOL (self)->canvas, self->priv->thin_white_gc, (*crop).left + (one_third_x * 2), (*crop).top, box_get_height (&(*crop)));
#line 3144 "EditingTools.c"
	}
	/* outer rectangle ... outer line in black, inner in white, corners fully black*/
#line 941 "EditingTools.vala"
	photo_canvas_draw_box (EDITING_TOOL (self)->canvas, self->priv->wide_black_gc, &(*crop));
#line 942 "EditingTools.vala"
	photo_canvas_draw_box (EDITING_TOOL (self)->canvas, self->priv->wide_white_gc, (_tmp0_ = box_get_reduced (&(*crop), 1), &_tmp0_));
#line 943 "EditingTools.vala"
	photo_canvas_draw_box (EDITING_TOOL (self)->canvas, self->priv->wide_white_gc, (_tmp1_ = box_get_reduced (&(*crop), 2), &_tmp1_));
#line 3153 "EditingTools.c"
}


#line 946 "EditingTools.vala"
static void crop_tool_erase_crop_tool (CropTool* self, const Box* crop) {
#line 3159 "EditingTools.c"
	Box _tmp0_ = {0};
	Box _tmp1_ = {0};
#line 946 "EditingTools.vala"
	g_return_if_fail (IS_CROP_TOOL (self));
#line 3164 "EditingTools.c"
	/* erase rule-of-thirds lines if user is manipulating the crop*/
#line 948 "EditingTools.vala"
	if (self->priv->in_manipulation != BOX_LOCATION_OUTSIDE) {
#line 3168 "EditingTools.c"
		gint one_third_x;
		gint one_third_y;
		one_third_x = box_get_width (&(*crop)) / 3;
		one_third_y = box_get_height (&(*crop)) / 3;
#line 952 "EditingTools.vala"
		photo_canvas_erase_horizontal_line (EDITING_TOOL (self)->canvas, (*crop).left, (*crop).top + one_third_y, box_get_width (&(*crop)));
#line 953 "EditingTools.vala"
		photo_canvas_erase_horizontal_line (EDITING_TOOL (self)->canvas, (*crop).left, (*crop).top + (one_third_y * 2), box_get_width (&(*crop)));
#line 955 "EditingTools.vala"
		photo_canvas_erase_vertical_line (EDITING_TOOL (self)->canvas, (*crop).left + one_third_x, (*crop).top, box_get_height (&(*crop)));
#line 956 "EditingTools.vala"
		photo_canvas_erase_vertical_line (EDITING_TOOL (self)->canvas, (*crop).left + (one_third_x * 2), (*crop).top, box_get_height (&(*crop)));
#line 3181 "EditingTools.c"
	}
	/* erase border*/
#line 960 "EditingTools.vala"
	photo_canvas_erase_box (EDITING_TOOL (self)->canvas, &(*crop));
#line 961 "EditingTools.vala"
	photo_canvas_erase_box (EDITING_TOOL (self)->canvas, (_tmp0_ = box_get_reduced (&(*crop), 1), &_tmp0_));
#line 962 "EditingTools.vala"
	photo_canvas_erase_box (EDITING_TOOL (self)->canvas, (_tmp1_ = box_get_reduced (&(*crop), 2), &_tmp1_));
#line 3190 "EditingTools.c"
}


#line 409 "EditingTools.vala"
static CropToolCropToolWindow* crop_tool_crop_tool_window_construct (GType object_type, GtkWindow* container) {
#line 3196 "EditingTools.c"
	CropToolCropToolWindow * self;
	GtkHBox* layout;
#line 409 "EditingTools.vala"
	g_return_val_if_fail (GTK_IS_WINDOW (container), NULL);
#line 410 "EditingTools.vala"
	self = (CropToolCropToolWindow*) editing_tool_window_construct (object_type, container);
#line 412 "EditingTools.vala"
	gtk_widget_set_tooltip_text (GTK_WIDGET (self->cancel_button), "Return to current photo dimensions");
#line 413 "EditingTools.vala"
	gtk_button_set_image_position (self->cancel_button, GTK_POS_LEFT);
#line 415 "EditingTools.vala"
	gtk_widget_set_tooltip_text (GTK_WIDGET (self->apply_button), "Set the crop for this photo");
#line 416 "EditingTools.vala"
	gtk_button_set_image_position (self->apply_button, GTK_POS_LEFT);
#line 3211 "EditingTools.c"
	layout = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, CROP_TOOL_CROP_TOOL_WINDOW_CONTROL_SPACING));
#line 419 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (layout), GTK_WIDGET (self->apply_button));
#line 420 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (layout), GTK_WIDGET (self->cancel_button));
#line 422 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (layout));
#line 3219 "EditingTools.c"
	(layout == NULL) ? NULL : (layout = (g_object_unref (layout), NULL));
	return self;
}


#line 409 "EditingTools.vala"
static CropToolCropToolWindow* crop_tool_crop_tool_window_new (GtkWindow* container) {
#line 409 "EditingTools.vala"
	return crop_tool_crop_tool_window_construct (CROP_TOOL_TYPE_CROP_TOOL_WINDOW, container);
#line 3229 "EditingTools.c"
}


static void crop_tool_crop_tool_window_class_init (CropToolCropToolWindowClass * klass) {
	crop_tool_crop_tool_window_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = crop_tool_crop_tool_window_finalize;
}


static void crop_tool_crop_tool_window_instance_init (CropToolCropToolWindow * self) {
	self->apply_button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_APPLY));
	self->cancel_button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_CANCEL));
}


static void crop_tool_crop_tool_window_finalize (GObject* obj) {
	CropToolCropToolWindow * self;
	self = CROP_TOOL_CROP_TOOL_WINDOW (obj);
	(self->apply_button == NULL) ? NULL : (self->apply_button = (g_object_unref (self->apply_button), NULL));
	(self->cancel_button == NULL) ? NULL : (self->cancel_button = (g_object_unref (self->cancel_button), NULL));
	G_OBJECT_CLASS (crop_tool_crop_tool_window_parent_class)->finalize (obj);
}


static GType crop_tool_crop_tool_window_get_type (void) {
	static GType crop_tool_crop_tool_window_type_id = 0;
	if (crop_tool_crop_tool_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CropToolCropToolWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) crop_tool_crop_tool_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CropToolCropToolWindow), 0, (GInstanceInitFunc) crop_tool_crop_tool_window_instance_init, NULL };
		crop_tool_crop_tool_window_type_id = g_type_register_static (TYPE_EDITING_TOOL_WINDOW, "CropToolCropToolWindow", &g_define_type_info, 0);
	}
	return crop_tool_crop_tool_window_type_id;
}


static void crop_tool_class_init (CropToolClass * klass) {
	crop_tool_parent_class = g_type_class_peek_parent (klass);
	EDITING_TOOL_CLASS (klass)->finalize = crop_tool_finalize;
	g_type_class_add_private (klass, sizeof (CropToolPrivate));
	EDITING_TOOL_CLASS (klass)->activate = crop_tool_real_activate;
	EDITING_TOOL_CLASS (klass)->deactivate = crop_tool_real_deactivate;
	EDITING_TOOL_CLASS (klass)->get_tool_window = crop_tool_real_get_tool_window;
	EDITING_TOOL_CLASS (klass)->get_unscaled_pixbuf = crop_tool_real_get_unscaled_pixbuf;
	EDITING_TOOL_CLASS (klass)->on_left_click = crop_tool_real_on_left_click;
	EDITING_TOOL_CLASS (klass)->on_left_released = crop_tool_real_on_left_released;
	EDITING_TOOL_CLASS (klass)->on_motion = crop_tool_real_on_motion;
	EDITING_TOOL_CLASS (klass)->paint = crop_tool_real_paint;
}


static void crop_tool_instance_init (CropTool * self) {
	self->priv = CROP_TOOL_GET_PRIVATE (self);
	self->priv->crop_tool_window = NULL;
	self->priv->color_shifted = NULL;
	self->priv->current_cursor_type = GDK_ARROW;
	self->priv->in_manipulation = BOX_LOCATION_OUTSIDE;
	self->priv->wide_black_gc = NULL;
	self->priv->wide_white_gc = NULL;
	self->priv->thin_white_gc = NULL;
	self->priv->last_grab_x = -1;
	self->priv->last_grab_y = -1;
}


static void crop_tool_finalize (EditingTool* obj) {
	CropTool * self;
	self = CROP_TOOL (obj);
	(self->priv->crop_tool_window == NULL) ? NULL : (self->priv->crop_tool_window = (g_object_unref (self->priv->crop_tool_window), NULL));
	(self->priv->color_shifted == NULL) ? NULL : (self->priv->color_shifted = (g_object_unref (self->priv->color_shifted), NULL));
	(self->priv->wide_black_gc == NULL) ? NULL : (self->priv->wide_black_gc = (g_object_unref (self->priv->wide_black_gc), NULL));
	(self->priv->wide_white_gc == NULL) ? NULL : (self->priv->wide_white_gc = (g_object_unref (self->priv->wide_white_gc), NULL));
	(self->priv->thin_white_gc == NULL) ? NULL : (self->priv->thin_white_gc = (g_object_unref (self->priv->thin_white_gc), NULL));
	EDITING_TOOL_CLASS (crop_tool_parent_class)->finalize (obj);
}


GType crop_tool_get_type (void) {
	static GType crop_tool_type_id = 0;
	if (crop_tool_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CropToolClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) crop_tool_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CropTool), 0, (GInstanceInitFunc) crop_tool_instance_init, NULL };
		crop_tool_type_id = g_type_register_static (TYPE_EDITING_TOOL, "CropTool", &g_define_type_info, 0);
	}
	return crop_tool_type_id;
}


#line 974 "EditingTools.vala"
static void redeye_instance_init (RedeyeInstance *self) {
#line 3317 "EditingTools.c"
	GdkPoint _tmp0_ = {0};
	GdkPoint default_center;
	memset (self, 0, sizeof (RedeyeInstance));
	default_center = (memset (&_tmp0_, 0, sizeof (GdkPoint)), _tmp0_);
#line 976 "EditingTools.vala"
	(*self).center = default_center;
#line 977 "EditingTools.vala"
	(*self).radius = REDEYE_INSTANCE_DEFAULT_RADIUS;
#line 3326 "EditingTools.c"
}


#line 980 "EditingTools.vala"
GdkRectangle redeye_instance_to_bounds_rect (const RedeyeInstance* inst) {
#line 3332 "EditingTools.c"
	GdkRectangle result;
	GdkRectangle _result_ = {0};
#line 982 "EditingTools.vala"
	_result_.x = (*inst).center.x - (*inst).radius;
#line 983 "EditingTools.vala"
	_result_.y = (*inst).center.y - (*inst).radius;
#line 984 "EditingTools.vala"
	_result_.width = 2 * (*inst).radius;
#line 985 "EditingTools.vala"
	_result_.height = _result_.width;
#line 3343 "EditingTools.c"
	result = _result_;
	return result;
}


#line 990 "EditingTools.vala"
RedeyeInstance redeye_instance_from_bounds_rect (const GdkRectangle* rect) {
#line 3351 "EditingTools.c"
	RedeyeInstance result;
	GdkRectangle in_rect;
	RedeyeInstance _tmp0_ = {0};
	RedeyeInstance _result_;
	in_rect = (*rect);
	_result_ = (redeye_instance_init (&_tmp0_), _tmp0_);
#line 994 "EditingTools.vala"
	_result_.radius = (in_rect.width + in_rect.height) / 4;
#line 995 "EditingTools.vala"
	_result_.center.x = in_rect.x + _result_.radius;
#line 996 "EditingTools.vala"
	_result_.center.y = in_rect.y + _result_.radius;
#line 3364 "EditingTools.c"
	result = _result_;
	return result;
}


RedeyeInstance* redeye_instance_dup (const RedeyeInstance* self) {
	RedeyeInstance* dup;
	dup = g_new0 (RedeyeInstance, 1);
	memcpy (dup, self, sizeof (RedeyeInstance));
	return dup;
}


void redeye_instance_free (RedeyeInstance* self) {
	g_free (self);
}


GType redeye_instance_get_type (void) {
	static GType redeye_instance_type_id = 0;
	if (redeye_instance_type_id == 0) {
		redeye_instance_type_id = g_boxed_type_register_static ("RedeyeInstance", (GBoxedCopyFunc) redeye_instance_dup, (GBoxedFreeFunc) redeye_instance_free);
	}
	return redeye_instance_type_id;
}


#line 1048 "EditingTools.vala"
static RedeyeInstance redeye_tool_new_interaction_instance (RedeyeTool* self, PhotoCanvas* canvas) {
#line 3394 "EditingTools.c"
	RedeyeInstance result;
	GdkRectangle photo_bounds;
	GdkPoint photo_center = {0};
	RedeyeInstance _tmp0_ = {0};
	RedeyeInstance _result_;
	photo_bounds = photo_canvas_get_scaled_pixbuf_position (canvas);
#line 1051 "EditingTools.vala"
	photo_center.x = photo_bounds.x + (photo_bounds.width / 2);
#line 1052 "EditingTools.vala"
	photo_center.y = photo_bounds.y + (photo_bounds.height / 2);
#line 3405 "EditingTools.c"
	_result_ = (redeye_instance_init (&_tmp0_), _tmp0_);
#line 1055 "EditingTools.vala"
	_result_.center.x = photo_center.x;
#line 1056 "EditingTools.vala"
	_result_.center.y = photo_center.y;
#line 1057 "EditingTools.vala"
	_result_.radius = REDEYE_INSTANCE_DEFAULT_RADIUS;
#line 3413 "EditingTools.c"
	result = _result_;
	return result;
}


#line 1062 "EditingTools.vala"
static void redeye_tool_prepare_gc (RedeyeTool* self, GdkGC* default_gc, GdkDrawable* drawable) {
#line 3421 "EditingTools.c"
	GdkGCValues _tmp0_ = {0};
	GdkGCValues gc_values;
	GdkGCValuesMask mask;
	GdkGC* _tmp1_;
	GdkGC* _tmp2_;
#line 1062 "EditingTools.vala"
	g_return_if_fail (IS_REDEYE_TOOL (self));
#line 1062 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (default_gc));
#line 1062 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 3433 "EditingTools.c"
	gc_values = (memset (&_tmp0_, 0, sizeof (GdkGCValues)), _tmp0_);
#line 1064 "EditingTools.vala"
	gc_values.function = GDK_COPY;
#line 1065 "EditingTools.vala"
	gc_values.fill = GDK_SOLID;
#line 1066 "EditingTools.vala"
	gc_values.line_style = GDK_LINE_SOLID;
#line 1067 "EditingTools.vala"
	gc_values.cap_style = GDK_CAP_BUTT;
#line 1068 "EditingTools.vala"
	gc_values.join_style = GDK_JOIN_MITER;
#line 3445 "EditingTools.c"
	mask = (((((GDK_GC_FOREGROUND | GDK_GC_FUNCTION) | GDK_GC_FILL) | GDK_GC_LINE_WIDTH) | GDK_GC_LINE_STYLE) | GDK_GC_CAP_STYLE) | GDK_GC_JOIN_STYLE;
#line 1079 "EditingTools.vala"
	gc_values.foreground = fetch_color ("#222", drawable);
#line 1080 "EditingTools.vala"
	gc_values.line_width = 1;
#line 3451 "EditingTools.c"
	_tmp1_ = NULL;
#line 1081 "EditingTools.vala"
	self->priv->wider_gray_gc = (_tmp1_ = gdk_gc_new_with_values (drawable, &gc_values, mask), (self->priv->wider_gray_gc == NULL) ? NULL : (self->priv->wider_gray_gc = (g_object_unref (self->priv->wider_gray_gc), NULL)), _tmp1_);
#line 1083 "EditingTools.vala"
	gc_values.foreground = fetch_color ("#FFF", drawable);
#line 1084 "EditingTools.vala"
	gc_values.line_width = 1;
#line 3459 "EditingTools.c"
	_tmp2_ = NULL;
#line 1085 "EditingTools.vala"
	self->priv->thin_white_gc = (_tmp2_ = gdk_gc_new_with_values (drawable, &gc_values, mask), (self->priv->thin_white_gc == NULL) ? NULL : (self->priv->thin_white_gc = (g_object_unref (self->priv->thin_white_gc), NULL)), _tmp2_);
#line 3463 "EditingTools.c"
}


#line 1088 "EditingTools.vala"
static void redeye_tool_draw_redeye_instance (RedeyeTool* self, const RedeyeInstance* inst) {
#line 1088 "EditingTools.vala"
	g_return_if_fail (IS_REDEYE_TOOL (self));
#line 1089 "EditingTools.vala"
	photo_canvas_draw_circle (EDITING_TOOL (self)->canvas, self->priv->wider_gray_gc, (*inst).center.x, (*inst).center.y, (*inst).radius - 1);
#line 1091 "EditingTools.vala"
	photo_canvas_draw_circle (EDITING_TOOL (self)->canvas, self->priv->thin_white_gc, (*inst).center.x, (*inst).center.y, (*inst).radius - 2);
#line 3475 "EditingTools.c"
}


#line 1095 "EditingTools.vala"
static gboolean redeye_tool_on_size_slider_adjust (RedeyeTool* self, GtkScrollType type) {
#line 3481 "EditingTools.c"
	gboolean result;
#line 1095 "EditingTools.vala"
	g_return_val_if_fail (IS_REDEYE_TOOL (self), FALSE);
#line 1096 "EditingTools.vala"
	self->priv->user_interaction_instance.radius = (gint) gtk_range_get_value (GTK_RANGE (self->priv->redeye_tool_window->slider));
#line 1099 "EditingTools.vala"
	photo_canvas_repaint (EDITING_TOOL (self)->canvas);
#line 3489 "EditingTools.c"
	result = FALSE;
	return result;
}


#line 1104 "EditingTools.vala"
static void redeye_tool_on_apply (RedeyeTool* self) {
#line 3497 "EditingTools.c"
	GdkRectangle bounds_rect_user;
	GdkRectangle bounds_rect_active;
	GdkRectangle bounds_rect_unscaled;
	RedeyeInstance instance_unscaled;
	Photo* _tmp0_;
#line 1104 "EditingTools.vala"
	g_return_if_fail (IS_REDEYE_TOOL (self));
#line 3505 "EditingTools.c"
	bounds_rect_user = redeye_instance_to_bounds_rect (&self->priv->user_interaction_instance);
	bounds_rect_active = photo_canvas_user_to_active_rect (EDITING_TOOL (self)->canvas, &bounds_rect_user);
	bounds_rect_unscaled = photo_canvas_active_to_unscaled_rect (EDITING_TOOL (self)->canvas, &bounds_rect_active);
	instance_unscaled = redeye_instance_from_bounds_rect (&bounds_rect_unscaled);
#line 172 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1116 "EditingTools.vala"
	photo_add_redeye_instance (_tmp0_ = photo_canvas_get_photo (EDITING_TOOL (self)->canvas), &instance_unscaled);
#line 3514 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 1118 "EditingTools.vala"
	editing_tool_notify_apply (EDITING_TOOL (self));
#line 3518 "EditingTools.c"
}


#line 1121 "EditingTools.vala"
static void redeye_tool_on_canvas_resize (RedeyeTool* self) {
#line 3524 "EditingTools.c"
	GdkRectangle scaled_pixbuf_position;
	double scale_factor;
#line 1121 "EditingTools.vala"
	g_return_if_fail (IS_REDEYE_TOOL (self));
#line 3529 "EditingTools.c"
	scaled_pixbuf_position = photo_canvas_get_scaled_pixbuf_position (EDITING_TOOL (self)->canvas);
#line 1125 "EditingTools.vala"
	self->priv->user_interaction_instance.center.x = self->priv->user_interaction_instance.center.x - self->priv->old_scaled_pixbuf_position.x;
#line 1126 "EditingTools.vala"
	self->priv->user_interaction_instance.center.y = self->priv->user_interaction_instance.center.y - self->priv->old_scaled_pixbuf_position.y;
#line 3535 "EditingTools.c"
	scale_factor = ((double) scaled_pixbuf_position.width) / ((double) self->priv->old_scaled_pixbuf_position.width);
#line 1131 "EditingTools.vala"
	self->priv->user_interaction_instance.center.x = (gint) ((((double) self->priv->user_interaction_instance.center.x) * scale_factor) + 0.5);
#line 1134 "EditingTools.vala"
	self->priv->user_interaction_instance.center.y = (gint) ((((double) self->priv->user_interaction_instance.center.y) * scale_factor) + 0.5);
#line 1138 "EditingTools.vala"
	self->priv->user_interaction_instance.center.x = self->priv->user_interaction_instance.center.x + scaled_pixbuf_position.x;
#line 1139 "EditingTools.vala"
	self->priv->user_interaction_instance.center.y = self->priv->user_interaction_instance.center.y + scaled_pixbuf_position.y;
#line 1141 "EditingTools.vala"
	self->priv->old_scaled_pixbuf_position = scaled_pixbuf_position;
#line 3547 "EditingTools.c"
}


#line 1062 "EditingTools.vala"
static void _redeye_tool_prepare_gc_photo_canvas_new_drawable (PhotoCanvas* _sender, GdkGC* default_gc, GdkDrawable* drawable, gpointer self) {
#line 3553 "EditingTools.c"
	redeye_tool_prepare_gc (self, default_gc, drawable);
}


#line 1121 "EditingTools.vala"
static void _redeye_tool_on_canvas_resize_photo_canvas_resized_scaled_pixbuf (PhotoCanvas* _sender, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position, gpointer self) {
#line 3560 "EditingTools.c"
	redeye_tool_on_canvas_resize (self);
}


#line 1095 "EditingTools.vala"
static gboolean _redeye_tool_on_size_slider_adjust_gtk_range_change_value (GtkHScale* _sender, GtkScrollType scroll, double new_value, gpointer self) {
#line 3567 "EditingTools.c"
	return redeye_tool_on_size_slider_adjust (self, scroll);
}


#line 1104 "EditingTools.vala"
static void _redeye_tool_on_apply_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 3574 "EditingTools.c"
	redeye_tool_on_apply (self);
}


#line 1144 "EditingTools.vala"
static void redeye_tool_real_activate (EditingTool* base, PhotoCanvas* canvas) {
#line 3581 "EditingTools.c"
	RedeyeTool * self;
	GdkDrawable* _tmp1_;
	GdkGC* _tmp0_;
	RedeyeToolRedeyeToolWindow* _tmp3_;
	GtkWindow* _tmp2_;
	GdkCursor* _tmp4_;
	GdkCursor* _tmp5_;
	self = REDEYE_TOOL (base);
#line 1144 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (canvas));
#line 1145 "EditingTools.vala"
	self->priv->user_interaction_instance = redeye_tool_new_interaction_instance (self, canvas);
#line 1147 "EditingTools.vala"
	g_signal_connect (canvas, "new-drawable", (GCallback) _redeye_tool_prepare_gc_photo_canvas_new_drawable, self);
#line 188 "EditingTools.vala"
	_tmp1_ = NULL;
#line 184 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1149 "EditingTools.vala"
	redeye_tool_prepare_gc (self, _tmp0_ = photo_canvas_get_default_gc (canvas), _tmp1_ = photo_canvas_get_drawable (canvas));
#line 3602 "EditingTools.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 1151 "EditingTools.vala"
	g_signal_connect (canvas, "resized-scaled-pixbuf", (GCallback) _redeye_tool_on_canvas_resize_photo_canvas_resized_scaled_pixbuf, self);
#line 1153 "EditingTools.vala"
	self->priv->old_scaled_pixbuf_position = photo_canvas_get_scaled_pixbuf_position (canvas);
#line 3609 "EditingTools.c"
	_tmp3_ = NULL;
#line 176 "EditingTools.vala"
	_tmp2_ = NULL;
#line 1155 "EditingTools.vala"
	self->priv->redeye_tool_window = (_tmp3_ = g_object_ref_sink (redeye_tool_redeye_tool_window_new (_tmp2_ = photo_canvas_get_container (canvas))), (self->priv->redeye_tool_window == NULL) ? NULL : (self->priv->redeye_tool_window = (g_object_unref (self->priv->redeye_tool_window), NULL)), _tmp3_);
#line 3615 "EditingTools.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 1156 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->redeye_tool_window->slider), (double) self->priv->user_interaction_instance.radius);
#line 1157 "EditingTools.vala"
	g_signal_connect (GTK_RANGE (self->priv->redeye_tool_window->slider), "change-value", (GCallback) _redeye_tool_on_size_slider_adjust_gtk_range_change_value, self);
#line 1158 "EditingTools.vala"
	g_signal_connect (self->priv->redeye_tool_window->apply_button, "clicked", (GCallback) _redeye_tool_on_apply_gtk_button_clicked, self);
#line 1159 "EditingTools.vala"
	g_signal_connect (self->priv->redeye_tool_window->close_button, "clicked", (GCallback) _editing_tool_notify_cancel_gtk_button_clicked, EDITING_TOOL (self));
#line 3625 "EditingTools.c"
	_tmp4_ = NULL;
#line 1161 "EditingTools.vala"
	self->priv->cached_arrow_cursor = (_tmp4_ = gdk_cursor_new (GDK_ARROW), (self->priv->cached_arrow_cursor == NULL) ? NULL : (self->priv->cached_arrow_cursor = (gdk_cursor_unref (self->priv->cached_arrow_cursor), NULL)), _tmp4_);
#line 3629 "EditingTools.c"
	_tmp5_ = NULL;
#line 1162 "EditingTools.vala"
	self->priv->cached_grab_cursor = (_tmp5_ = gdk_cursor_new (GDK_FLEUR), (self->priv->cached_grab_cursor == NULL) ? NULL : (self->priv->cached_grab_cursor = (gdk_cursor_unref (self->priv->cached_grab_cursor), NULL)), _tmp5_);
#line 1164 "EditingTools.vala"
	EDITING_TOOL_CLASS (redeye_tool_parent_class)->activate (EDITING_TOOL (self), canvas);
#line 3635 "EditingTools.c"
}


#line 1167 "EditingTools.vala"
static void redeye_tool_real_deactivate (EditingTool* base) {
#line 3641 "EditingTools.c"
	RedeyeTool * self;
	self = REDEYE_TOOL (base);
#line 1168 "EditingTools.vala"
	if (self->priv->redeye_tool_window != NULL) {
#line 3646 "EditingTools.c"
		RedeyeToolRedeyeToolWindow* _tmp0_;
#line 1169 "EditingTools.vala"
		gtk_widget_hide (GTK_WIDGET (self->priv->redeye_tool_window));
#line 3650 "EditingTools.c"
		_tmp0_ = NULL;
#line 1170 "EditingTools.vala"
		self->priv->redeye_tool_window = (_tmp0_ = NULL, (self->priv->redeye_tool_window == NULL) ? NULL : (self->priv->redeye_tool_window = (g_object_unref (self->priv->redeye_tool_window), NULL)), _tmp0_);
#line 3654 "EditingTools.c"
	}
#line 1173 "EditingTools.vala"
	EDITING_TOOL_CLASS (redeye_tool_parent_class)->deactivate (EDITING_TOOL (self));
#line 3658 "EditingTools.c"
}


#line 1176 "EditingTools.vala"
static EditingToolWindow* redeye_tool_real_get_tool_window (EditingTool* base) {
#line 3664 "EditingTools.c"
	RedeyeTool * self;
	EditingToolWindow* result;
	EditingToolWindow* _tmp0_;
	self = REDEYE_TOOL (base);
#line 1177 "EditingTools.vala"
	_tmp0_ = NULL;
#line 3671 "EditingTools.c"
	result = (_tmp0_ = EDITING_TOOL_WINDOW (self->priv->redeye_tool_window), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 1180 "EditingTools.vala"
static void redeye_tool_real_paint (EditingTool* base, GdkGC* gc, GdkDrawable* drawable) {
#line 3679 "EditingTools.c"
	RedeyeTool * self;
	GdkPixbuf* _tmp0_;
	RedeyeInstance active_inst;
	self = REDEYE_TOOL (base);
#line 1180 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 1180 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 199 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1182 "EditingTools.vala"
	photo_canvas_paint_pixbuf (EDITING_TOOL (self)->canvas, _tmp0_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas));
#line 3692 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	/* user_interaction_instance has its radius in user coords, and
	           draw_redeye_instance expects active region coords */
	active_inst = self->priv->user_interaction_instance;
#line 1187 "EditingTools.vala"
	active_inst.center = photo_canvas_user_to_active_point (EDITING_TOOL (self)->canvas, &self->priv->user_interaction_instance.center);
#line 1189 "EditingTools.vala"
	redeye_tool_draw_redeye_instance (self, &active_inst);
#line 3701 "EditingTools.c"
}


#line 1192 "EditingTools.vala"
static void redeye_tool_real_on_left_click (EditingTool* base, gint x, gint y) {
#line 3707 "EditingTools.c"
	RedeyeTool * self;
	GdkRectangle bounds_rect;
	self = REDEYE_TOOL (base);
	bounds_rect = redeye_instance_to_bounds_rect (&self->priv->user_interaction_instance);
#line 1196 "EditingTools.vala"
	if (coord_in_rectangle (x, y, &bounds_rect)) {
#line 1197 "EditingTools.vala"
		self->priv->is_reticle_move_in_progress = TRUE;
#line 1198 "EditingTools.vala"
		self->priv->reticle_move_mouse_start_point.x = x;
#line 1199 "EditingTools.vala"
		self->priv->reticle_move_mouse_start_point.y = y;
#line 1200 "EditingTools.vala"
		self->priv->reticle_move_anchor = self->priv->user_interaction_instance.center;
#line 3722 "EditingTools.c"
	}
}


#line 1204 "EditingTools.vala"
static void redeye_tool_real_on_left_released (EditingTool* base, gint x, gint y) {
#line 3729 "EditingTools.c"
	RedeyeTool * self;
	self = REDEYE_TOOL (base);
#line 1205 "EditingTools.vala"
	self->priv->is_reticle_move_in_progress = FALSE;
#line 3734 "EditingTools.c"
}


#line 1208 "EditingTools.vala"
static void redeye_tool_real_on_motion (EditingTool* base, gint x, gint y, GdkModifierType mask) {
#line 3740 "EditingTools.c"
	RedeyeTool * self;
	self = REDEYE_TOOL (base);
#line 1209 "EditingTools.vala"
	if (self->priv->is_reticle_move_in_progress) {
#line 3745 "EditingTools.c"
		GdkRectangle active_region_rect;
		gint x_clamp_low;
		gint y_clamp_low;
		gint x_clamp_high;
		gint y_clamp_high;
		gint delta_x;
		gint delta_y;
		active_region_rect = photo_canvas_get_scaled_pixbuf_position (EDITING_TOOL (self)->canvas);
		x_clamp_low = (active_region_rect.x + self->priv->user_interaction_instance.radius) + 1;
		y_clamp_low = (active_region_rect.y + self->priv->user_interaction_instance.radius) + 1;
		x_clamp_high = ((active_region_rect.x + active_region_rect.width) - self->priv->user_interaction_instance.radius) - 1;
		y_clamp_high = ((active_region_rect.y + active_region_rect.height) - self->priv->user_interaction_instance.radius) - 1;
		delta_x = x - self->priv->reticle_move_mouse_start_point.x;
		delta_y = y - self->priv->reticle_move_mouse_start_point.y;
#line 1228 "EditingTools.vala"
		self->priv->user_interaction_instance.center.x = self->priv->reticle_move_anchor.x + delta_x;
#line 1230 "EditingTools.vala"
		self->priv->user_interaction_instance.center.y = self->priv->reticle_move_anchor.y + delta_y;
#line 1233 "EditingTools.vala"
		self->priv->user_interaction_instance.center.x = CLAMP (self->priv->reticle_move_anchor.x + delta_x, x_clamp_low, x_clamp_high);
#line 1236 "EditingTools.vala"
		self->priv->user_interaction_instance.center.y = CLAMP (self->priv->reticle_move_anchor.y + delta_y, y_clamp_low, y_clamp_high);
#line 1240 "EditingTools.vala"
		photo_canvas_repaint (EDITING_TOOL (self)->canvas);
#line 3770 "EditingTools.c"
	} else {
		GdkRectangle bounds;
		bounds = redeye_instance_to_bounds_rect (&self->priv->user_interaction_instance);
#line 1245 "EditingTools.vala"
		if (coord_in_rectangle (x, y, &bounds)) {
#line 3776 "EditingTools.c"
			GdkWindow* _tmp0_;
#line 180 "EditingTools.vala"
			_tmp0_ = NULL;
#line 1246 "EditingTools.vala"
			gdk_window_set_cursor (_tmp0_ = photo_canvas_get_drawing_window (EDITING_TOOL (self)->canvas), self->priv->cached_grab_cursor);
#line 3782 "EditingTools.c"
			(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
		} else {
			GdkWindow* _tmp1_;
#line 180 "EditingTools.vala"
			_tmp1_ = NULL;
#line 1248 "EditingTools.vala"
			gdk_window_set_cursor (_tmp1_ = photo_canvas_get_drawing_window (EDITING_TOOL (self)->canvas), self->priv->cached_arrow_cursor);
#line 3790 "EditingTools.c"
			(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
		}
	}
}


#line 1002 "EditingTools.vala"
RedeyeTool* redeye_tool_construct (GType object_type) {
#line 3799 "EditingTools.c"
	RedeyeTool* self;
	self = (RedeyeTool*) g_type_create_instance (object_type);
	return self;
}


#line 1002 "EditingTools.vala"
RedeyeTool* redeye_tool_new (void) {
#line 1002 "EditingTools.vala"
	return redeye_tool_construct (TYPE_REDEYE_TOOL);
#line 3810 "EditingTools.c"
}


#line 1015 "EditingTools.vala"
static RedeyeToolRedeyeToolWindow* redeye_tool_redeye_tool_window_construct (GType object_type, GtkWindow* container) {
#line 3816 "EditingTools.c"
	RedeyeToolRedeyeToolWindow * self;
	GtkHBox* layout;
#line 1015 "EditingTools.vala"
	g_return_val_if_fail (GTK_IS_WINDOW (container), NULL);
#line 1016 "EditingTools.vala"
	self = (RedeyeToolRedeyeToolWindow*) editing_tool_window_construct (object_type, container);
#line 1018 "EditingTools.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->slider), 80, -1);
#line 1019 "EditingTools.vala"
	gtk_scale_set_draw_value (GTK_SCALE (self->slider), FALSE);
#line 1021 "EditingTools.vala"
	gtk_widget_set_tooltip_text (GTK_WIDGET (self->close_button), "Close the red-eye tool");
#line 1022 "EditingTools.vala"
	gtk_button_set_image_position (self->close_button, GTK_POS_LEFT);
#line 1024 "EditingTools.vala"
	gtk_widget_set_tooltip_text (GTK_WIDGET (self->apply_button), "Remove any red-eye effects in the selected region");
#line 1025 "EditingTools.vala"
	gtk_button_set_image_position (self->apply_button, GTK_POS_LEFT);
#line 3835 "EditingTools.c"
	layout = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, REDEYE_TOOL_REDEYE_TOOL_WINDOW_CONTROL_SPACING));
#line 1028 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (layout), GTK_WIDGET (self->priv->slider_label));
#line 1029 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (layout), GTK_WIDGET (self->slider));
#line 1030 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (layout), GTK_WIDGET (self->close_button));
#line 1031 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (layout), GTK_WIDGET (self->apply_button));
#line 1033 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (layout));
#line 3847 "EditingTools.c"
	(layout == NULL) ? NULL : (layout = (g_object_unref (layout), NULL));
	return self;
}


#line 1015 "EditingTools.vala"
static RedeyeToolRedeyeToolWindow* redeye_tool_redeye_tool_window_new (GtkWindow* container) {
#line 1015 "EditingTools.vala"
	return redeye_tool_redeye_tool_window_construct (REDEYE_TOOL_TYPE_REDEYE_TOOL_WINDOW, container);
#line 3857 "EditingTools.c"
}


static void redeye_tool_redeye_tool_window_class_init (RedeyeToolRedeyeToolWindowClass * klass) {
	redeye_tool_redeye_tool_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (RedeyeToolRedeyeToolWindowPrivate));
	G_OBJECT_CLASS (klass)->finalize = redeye_tool_redeye_tool_window_finalize;
}


static void redeye_tool_redeye_tool_window_instance_init (RedeyeToolRedeyeToolWindow * self) {
	self->priv = REDEYE_TOOL_REDEYE_TOOL_WINDOW_GET_PRIVATE (self);
	self->priv->slider_label = g_object_ref_sink ((GtkLabel*) gtk_label_new_with_mnemonic ("Size:"));
	self->apply_button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_APPLY));
	self->close_button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_CLOSE));
	self->slider = g_object_ref_sink ((GtkHScale*) gtk_hscale_new_with_range ((double) REDEYE_INSTANCE_MIN_RADIUS, (double) REDEYE_INSTANCE_MAX_RADIUS, 1.0));
}


static void redeye_tool_redeye_tool_window_finalize (GObject* obj) {
	RedeyeToolRedeyeToolWindow * self;
	self = REDEYE_TOOL_REDEYE_TOOL_WINDOW (obj);
	(self->priv->slider_label == NULL) ? NULL : (self->priv->slider_label = (g_object_unref (self->priv->slider_label), NULL));
	(self->apply_button == NULL) ? NULL : (self->apply_button = (g_object_unref (self->apply_button), NULL));
	(self->close_button == NULL) ? NULL : (self->close_button = (g_object_unref (self->close_button), NULL));
	(self->slider == NULL) ? NULL : (self->slider = (g_object_unref (self->slider), NULL));
	G_OBJECT_CLASS (redeye_tool_redeye_tool_window_parent_class)->finalize (obj);
}


static GType redeye_tool_redeye_tool_window_get_type (void) {
	static GType redeye_tool_redeye_tool_window_type_id = 0;
	if (redeye_tool_redeye_tool_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (RedeyeToolRedeyeToolWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) redeye_tool_redeye_tool_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RedeyeToolRedeyeToolWindow), 0, (GInstanceInitFunc) redeye_tool_redeye_tool_window_instance_init, NULL };
		redeye_tool_redeye_tool_window_type_id = g_type_register_static (TYPE_EDITING_TOOL_WINDOW, "RedeyeToolRedeyeToolWindow", &g_define_type_info, 0);
	}
	return redeye_tool_redeye_tool_window_type_id;
}


static void redeye_tool_class_init (RedeyeToolClass * klass) {
	redeye_tool_parent_class = g_type_class_peek_parent (klass);
	EDITING_TOOL_CLASS (klass)->finalize = redeye_tool_finalize;
	g_type_class_add_private (klass, sizeof (RedeyeToolPrivate));
	EDITING_TOOL_CLASS (klass)->activate = redeye_tool_real_activate;
	EDITING_TOOL_CLASS (klass)->deactivate = redeye_tool_real_deactivate;
	EDITING_TOOL_CLASS (klass)->get_tool_window = redeye_tool_real_get_tool_window;
	EDITING_TOOL_CLASS (klass)->paint = redeye_tool_real_paint;
	EDITING_TOOL_CLASS (klass)->on_left_click = redeye_tool_real_on_left_click;
	EDITING_TOOL_CLASS (klass)->on_left_released = redeye_tool_real_on_left_released;
	EDITING_TOOL_CLASS (klass)->on_motion = redeye_tool_real_on_motion;
}


static void redeye_tool_instance_init (RedeyeTool * self) {
	self->priv = REDEYE_TOOL_GET_PRIVATE (self);
	self->priv->thin_white_gc = NULL;
	self->priv->wider_gray_gc = NULL;
	self->priv->redeye_tool_window = NULL;
	self->priv->is_reticle_move_in_progress = FALSE;
}


static void redeye_tool_finalize (EditingTool* obj) {
	RedeyeTool * self;
	self = REDEYE_TOOL (obj);
	(self->priv->thin_white_gc == NULL) ? NULL : (self->priv->thin_white_gc = (g_object_unref (self->priv->thin_white_gc), NULL));
	(self->priv->wider_gray_gc == NULL) ? NULL : (self->priv->wider_gray_gc = (g_object_unref (self->priv->wider_gray_gc), NULL));
	(self->priv->redeye_tool_window == NULL) ? NULL : (self->priv->redeye_tool_window = (g_object_unref (self->priv->redeye_tool_window), NULL));
	(self->priv->cached_arrow_cursor == NULL) ? NULL : (self->priv->cached_arrow_cursor = (gdk_cursor_unref (self->priv->cached_arrow_cursor), NULL));
	(self->priv->cached_grab_cursor == NULL) ? NULL : (self->priv->cached_grab_cursor = (gdk_cursor_unref (self->priv->cached_grab_cursor), NULL));
	EDITING_TOOL_CLASS (redeye_tool_parent_class)->finalize (obj);
}


GType redeye_tool_get_type (void) {
	static GType redeye_tool_type_id = 0;
	if (redeye_tool_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (RedeyeToolClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) redeye_tool_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (RedeyeTool), 0, (GInstanceInitFunc) redeye_tool_instance_init, NULL };
		redeye_tool_type_id = g_type_register_static (TYPE_EDITING_TOOL, "RedeyeTool", &g_define_type_info, 0);
	}
	return redeye_tool_type_id;
}


#line 1459 "EditingTools.vala"
static void _adjust_tool_on_apply_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 3945 "EditingTools.c"
	adjust_tool_on_apply (self);
}


#line 1445 "EditingTools.vala"
static void _adjust_tool_on_reset_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 3952 "EditingTools.c"
	adjust_tool_on_reset (self);
}


#line 1441 "EditingTools.vala"
static void _adjust_tool_on_cancel_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 3959 "EditingTools.c"
	adjust_tool_on_cancel (self);
}


#line 1501 "EditingTools.vala"
static void _adjust_tool_on_adjustment_gtk_range_value_changed (GtkHScale* _sender, gpointer self) {
#line 3966 "EditingTools.c"
	adjust_tool_on_adjustment (self);
}


#line 1507 "EditingTools.vala"
static void _adjust_tool_on_canvas_resize_photo_canvas_resized_scaled_pixbuf (PhotoCanvas* _sender, const Dimensions* old_dim, GdkPixbuf* scaled, const GdkRectangle* scaled_position, gpointer self) {
#line 3973 "EditingTools.c"
	adjust_tool_on_canvas_resize (self);
}


#line 1347 "EditingTools.vala"
static void adjust_tool_real_activate (EditingTool* base, PhotoCanvas* canvas) {
#line 3980 "EditingTools.c"
	AdjustTool * self;
	AdjustToolAdjustToolWindow* _tmp1_;
	GtkWindow* _tmp0_;
	Photo* _tmp2_;
	float _tmp3_;
	float exposure_param;
	Photo* _tmp4_;
	float _tmp5_;
	float saturation_param;
	Photo* _tmp6_;
	float _tmp7_;
	float tint_param;
	Photo* _tmp8_;
	float _tmp9_;
	float temperature_param;
	GdkPixbuf* _tmp11_;
	GdkPixbuf* _tmp10_;
	GdkPixbuf* _tmp13_;
	ImageHistogram* _tmp12_;
	self = ADJUST_TOOL (base);
#line 1347 "EditingTools.vala"
	g_return_if_fail (IS_PHOTO_CANVAS (canvas));
#line 4003 "EditingTools.c"
	_tmp1_ = NULL;
#line 176 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1348 "EditingTools.vala"
	self->priv->adjust_tool_window = (_tmp1_ = g_object_ref_sink (adjust_tool_adjust_tool_window_new (_tmp0_ = photo_canvas_get_container (canvas))), (self->priv->adjust_tool_window == NULL) ? NULL : (self->priv->adjust_tool_window = (g_object_unref (self->priv->adjust_tool_window), NULL)), _tmp1_);
#line 4009 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 172 "EditingTools.vala"
	_tmp2_ = NULL;
#line 4013 "EditingTools.c"
	exposure_param = (_tmp3_ = photo_get_adjustment_parameter (_tmp2_ = photo_canvas_get_photo (canvas), COLOR_TRANSFORMATION_KIND_EXPOSURE), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL)), _tmp3_);
#line 1353 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->exposure_slider), (double) exposure_param);
#line 172 "EditingTools.vala"
	_tmp4_ = NULL;
#line 4019 "EditingTools.c"
	saturation_param = (_tmp5_ = photo_get_adjustment_parameter (_tmp4_ = photo_canvas_get_photo (canvas), COLOR_TRANSFORMATION_KIND_SATURATION), (_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL)), _tmp5_);
#line 1357 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->saturation_slider), (double) saturation_param);
#line 172 "EditingTools.vala"
	_tmp6_ = NULL;
#line 4025 "EditingTools.c"
	tint_param = (_tmp7_ = photo_get_adjustment_parameter (_tmp6_ = photo_canvas_get_photo (canvas), COLOR_TRANSFORMATION_KIND_TINT), (_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL)), _tmp7_);
#line 1361 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->tint_slider), (double) tint_param);
#line 172 "EditingTools.vala"
	_tmp8_ = NULL;
#line 4031 "EditingTools.c"
	temperature_param = (_tmp9_ = photo_get_adjustment_parameter (_tmp8_ = photo_canvas_get_photo (canvas), COLOR_TRANSFORMATION_KIND_TEMPERATURE), (_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL)), _tmp9_);
#line 1365 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->temperature_slider), (double) temperature_param);
#line 1367 "EditingTools.vala"
	g_signal_connect (self->priv->adjust_tool_window->apply_button, "clicked", (GCallback) _adjust_tool_on_apply_gtk_button_clicked, self);
#line 1368 "EditingTools.vala"
	g_signal_connect (self->priv->adjust_tool_window->reset_button, "clicked", (GCallback) _adjust_tool_on_reset_gtk_button_clicked, self);
#line 1369 "EditingTools.vala"
	g_signal_connect (self->priv->adjust_tool_window->cancel_button, "clicked", (GCallback) _adjust_tool_on_cancel_gtk_button_clicked, self);
#line 1370 "EditingTools.vala"
	g_signal_connect (GTK_RANGE (self->priv->adjust_tool_window->exposure_slider), "value-changed", (GCallback) _adjust_tool_on_adjustment_gtk_range_value_changed, self);
#line 1371 "EditingTools.vala"
	g_signal_connect (GTK_RANGE (self->priv->adjust_tool_window->saturation_slider), "value-changed", (GCallback) _adjust_tool_on_adjustment_gtk_range_value_changed, self);
#line 1373 "EditingTools.vala"
	g_signal_connect (GTK_RANGE (self->priv->adjust_tool_window->tint_slider), "value-changed", (GCallback) _adjust_tool_on_adjustment_gtk_range_value_changed, self);
#line 1374 "EditingTools.vala"
	g_signal_connect (GTK_RANGE (self->priv->adjust_tool_window->temperature_slider), "value-changed", (GCallback) _adjust_tool_on_adjustment_gtk_range_value_changed, self);
#line 1377 "EditingTools.vala"
	g_signal_connect (canvas, "resized-scaled-pixbuf", (GCallback) _adjust_tool_on_canvas_resize_photo_canvas_resized_scaled_pixbuf, self);
#line 4051 "EditingTools.c"
	_tmp11_ = NULL;
#line 199 "EditingTools.vala"
	_tmp10_ = NULL;
#line 1379 "EditingTools.vala"
	self->priv->draw_to_pixbuf = (_tmp11_ = gdk_pixbuf_copy (_tmp10_ = photo_canvas_get_scaled_pixbuf (canvas)), (self->priv->draw_to_pixbuf == NULL) ? NULL : (self->priv->draw_to_pixbuf = (g_object_unref (self->priv->draw_to_pixbuf), NULL)), _tmp11_);
#line 4057 "EditingTools.c"
	(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
#line 642 "ColorTransformation.vala"
	_tmp13_ = NULL;
#line 4061 "EditingTools.c"
	_tmp12_ = NULL;
#line 1380 "EditingTools.vala"
	gtk_image_set_from_pixbuf (self->priv->adjust_tool_window->histogram_image_tray, _tmp13_ = image_histogram_get_graphic (_tmp12_ = image_histogram_new (self->priv->draw_to_pixbuf)));
#line 4065 "EditingTools.c"
	(_tmp13_ == NULL) ? NULL : (_tmp13_ = (g_object_unref (_tmp13_), NULL));
	(_tmp12_ == NULL) ? NULL : (_tmp12_ = (image_histogram_unref (_tmp12_), NULL));
#line 1383 "EditingTools.vala"
	EDITING_TOOL_CLASS (adjust_tool_parent_class)->activate (EDITING_TOOL (self), canvas);
#line 4070 "EditingTools.c"
}


#line 1386 "EditingTools.vala"
static EditingToolWindow* adjust_tool_real_get_tool_window (EditingTool* base) {
#line 4076 "EditingTools.c"
	AdjustTool * self;
	EditingToolWindow* result;
	EditingToolWindow* _tmp0_;
	self = ADJUST_TOOL (base);
#line 1387 "EditingTools.vala"
	_tmp0_ = NULL;
#line 4083 "EditingTools.c"
	result = (_tmp0_ = EDITING_TOOL_WINDOW (self->priv->adjust_tool_window), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 1390 "EditingTools.vala"
static void adjust_tool_real_deactivate (EditingTool* base) {
#line 4091 "EditingTools.c"
	AdjustTool * self;
	GdkPixbuf* _tmp1_;
	self = ADJUST_TOOL (base);
#line 1391 "EditingTools.vala"
	if (self->priv->adjust_tool_window != NULL) {
#line 4097 "EditingTools.c"
		AdjustToolAdjustToolWindow* _tmp0_;
#line 1392 "EditingTools.vala"
		gtk_widget_hide (GTK_WIDGET (self->priv->adjust_tool_window));
#line 4101 "EditingTools.c"
		_tmp0_ = NULL;
#line 1393 "EditingTools.vala"
		self->priv->adjust_tool_window = (_tmp0_ = NULL, (self->priv->adjust_tool_window == NULL) ? NULL : (self->priv->adjust_tool_window = (g_object_unref (self->priv->adjust_tool_window), NULL)), _tmp0_);
#line 4105 "EditingTools.c"
	}
	_tmp1_ = NULL;
#line 1396 "EditingTools.vala"
	self->priv->draw_to_pixbuf = (_tmp1_ = NULL, (self->priv->draw_to_pixbuf == NULL) ? NULL : (self->priv->draw_to_pixbuf = (g_object_unref (self->priv->draw_to_pixbuf), NULL)), _tmp1_);
#line 1398 "EditingTools.vala"
	EDITING_TOOL_CLASS (adjust_tool_parent_class)->deactivate (EDITING_TOOL (self));
#line 4112 "EditingTools.c"
}


#line 1401 "EditingTools.vala"
static void adjust_tool_real_paint (EditingTool* base, GdkGC* gc, GdkDrawable* drawable) {
#line 4118 "EditingTools.c"
	AdjustTool * self;
	self = ADJUST_TOOL (base);
#line 1401 "EditingTools.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 1401 "EditingTools.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 1402 "EditingTools.vala"
	if (!self->priv->suppress_effect_redraw) {
#line 4127 "EditingTools.c"
		ColorTransformationFactory* _tmp0_;
		ColorTransformation* _tmp1_;
		ColorTransformation* tint_transform;
		ColorTransformationFactory* _tmp2_;
		ColorTransformation* _tmp3_;
		ColorTransformation* temperature_transform;
		ColorTransformationFactory* _tmp4_;
		ColorTransformation* _tmp5_;
		ColorTransformation* saturation_transform;
		ColorTransformationFactory* _tmp6_;
		ColorTransformation* _tmp7_;
		ColorTransformation* exposure_transform;
		ColorTransformation* _tmp9_;
		ColorTransformation* _tmp8_;
		ColorTransformation* _tmp10_;
		ColorTransformation* composite_transform;
		GdkPixbuf* _tmp11_;
#line 431 "ColorTransformation.vala"
		_tmp0_ = NULL;
#line 1404 "EditingTools.vala"
		_tmp1_ = NULL;
#line 4149 "EditingTools.c"
		tint_transform = (_tmp1_ = color_transformation_factory_from_parameter (_tmp0_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_TINT, (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->tint_slider))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (color_transformation_factory_unref (_tmp0_), NULL)), _tmp1_);
#line 431 "ColorTransformation.vala"
		_tmp2_ = NULL;
#line 1409 "EditingTools.vala"
		_tmp3_ = NULL;
#line 4155 "EditingTools.c"
		temperature_transform = (_tmp3_ = color_transformation_factory_from_parameter (_tmp2_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_TEMPERATURE, (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->temperature_slider))), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (color_transformation_factory_unref (_tmp2_), NULL)), _tmp3_);
#line 431 "ColorTransformation.vala"
		_tmp4_ = NULL;
#line 1414 "EditingTools.vala"
		_tmp5_ = NULL;
#line 4161 "EditingTools.c"
		saturation_transform = (_tmp5_ = color_transformation_factory_from_parameter (_tmp4_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_SATURATION, (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->saturation_slider))), (_tmp4_ == NULL) ? NULL : (_tmp4_ = (color_transformation_factory_unref (_tmp4_), NULL)), _tmp5_);
#line 431 "ColorTransformation.vala"
		_tmp6_ = NULL;
#line 1419 "EditingTools.vala"
		_tmp7_ = NULL;
#line 4167 "EditingTools.c"
		exposure_transform = (_tmp7_ = color_transformation_factory_from_parameter (_tmp6_ = color_transformation_factory_get_instance (), COLOR_TRANSFORMATION_KIND_EXPOSURE, (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->exposure_slider))), (_tmp6_ == NULL) ? NULL : (_tmp6_ = (color_transformation_factory_unref (_tmp6_), NULL)), _tmp7_);
#line 129 "ColorTransformation.vala"
		_tmp9_ = NULL;
#line 129 "ColorTransformation.vala"
		_tmp8_ = NULL;
#line 1423 "EditingTools.vala"
		_tmp10_ = NULL;
#line 4175 "EditingTools.c"
		composite_transform = (_tmp10_ = color_transformation_compose_against (_tmp9_ = color_transformation_compose_against (_tmp8_ = color_transformation_compose_against (exposure_transform, saturation_transform), temperature_transform), tint_transform), (_tmp9_ == NULL) ? NULL : (_tmp9_ = (color_transformation_unref (_tmp9_), NULL)), (_tmp8_ == NULL) ? NULL : (_tmp8_ = (color_transformation_unref (_tmp8_), NULL)), _tmp10_);
#line 1430 "EditingTools.vala"
		_tmp11_ = NULL;
#line 1429 "EditingTools.vala"
		color_transformation_transform_existing_pixbuf (composite_transform, photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas), (_tmp11_ = self->priv->draw_to_pixbuf, (_tmp11_ == NULL) ? NULL : g_object_ref (_tmp11_)));
#line 4181 "EditingTools.c"
		(tint_transform == NULL) ? NULL : (tint_transform = (color_transformation_unref (tint_transform), NULL));
		(temperature_transform == NULL) ? NULL : (temperature_transform = (color_transformation_unref (temperature_transform), NULL));
		(saturation_transform == NULL) ? NULL : (saturation_transform = (color_transformation_unref (saturation_transform), NULL));
		(exposure_transform == NULL) ? NULL : (exposure_transform = (color_transformation_unref (exposure_transform), NULL));
		(composite_transform == NULL) ? NULL : (composite_transform = (color_transformation_unref (composite_transform), NULL));
	}
#line 1433 "EditingTools.vala"
	photo_canvas_paint_pixbuf (EDITING_TOOL (self)->canvas, self->priv->draw_to_pixbuf);
#line 4190 "EditingTools.c"
}


#line 1436 "EditingTools.vala"
static GdkPixbuf* adjust_tool_real_get_unscaled_pixbuf (EditingTool* base, Photo* photo) {
#line 4196 "EditingTools.c"
	AdjustTool * self;
	GdkPixbuf* result;
	GError * _inner_error_;
	GdkPixbuf* _tmp0_;
	self = ADJUST_TOOL (base);
#line 1436 "EditingTools.vala"
	g_return_val_if_fail (IS_PHOTO (photo), NULL);
#line 4204 "EditingTools.c"
	_inner_error_ = NULL;
	_tmp0_ = photo_get_pixbuf (photo, PHOTO_EXCEPTION_ADJUST, 0, GDK_INTERP_HYPER, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	result = _tmp0_;
	return result;
}


#line 1441 "EditingTools.vala"
static void adjust_tool_on_cancel (AdjustTool* self) {
#line 1441 "EditingTools.vala"
	g_return_if_fail (IS_ADJUST_TOOL (self));
#line 1442 "EditingTools.vala"
	editing_tool_notify_cancel (EDITING_TOOL (self));
#line 4223 "EditingTools.c"
}


#line 1445 "EditingTools.vala"
static void adjust_tool_on_reset (AdjustTool* self) {
#line 4229 "EditingTools.c"
	GdkPixbuf* _tmp1_;
	ImageHistogram* _tmp0_;
#line 1445 "EditingTools.vala"
	g_return_if_fail (IS_ADJUST_TOOL (self));
#line 1446 "EditingTools.vala"
	self->priv->suppress_effect_redraw = TRUE;
#line 1448 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->exposure_slider), 0.0);
#line 1449 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->saturation_slider), 0.0);
#line 1450 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->temperature_slider), 0.0);
#line 1451 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->priv->adjust_tool_window->tint_slider), 0.0);
#line 1453 "EditingTools.vala"
	self->priv->suppress_effect_redraw = FALSE;
#line 1454 "EditingTools.vala"
	photo_canvas_repaint (EDITING_TOOL (self)->canvas);
#line 642 "ColorTransformation.vala"
	_tmp1_ = NULL;
#line 4250 "EditingTools.c"
	_tmp0_ = NULL;
#line 1455 "EditingTools.vala"
	gtk_image_set_from_pixbuf (self->priv->adjust_tool_window->histogram_image_tray, _tmp1_ = image_histogram_get_graphic (_tmp0_ = image_histogram_new (self->priv->draw_to_pixbuf)));
#line 4254 "EditingTools.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (image_histogram_unref (_tmp0_), NULL));
}


#line 1459 "EditingTools.vala"
static void adjust_tool_on_apply (AdjustTool* self) {
#line 4262 "EditingTools.c"
	EditingToolWindow* _tmp0_;
	AppWindow* _tmp1_;
	GeeArrayList* adjustments;
	ColorTransformationInstance current_instance = {0};
	float exposure_param;
	float saturation_param;
	float tint_param;
	float temperature_param;
	Photo* _tmp2_;
	AppWindow* _tmp3_;
#line 1459 "EditingTools.vala"
	g_return_if_fail (IS_ADJUST_TOOL (self));
#line 1460 "EditingTools.vala"
	self->priv->suppress_effect_redraw = TRUE;
#line 349 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1462 "EditingTools.vala"
	gtk_widget_hide (GTK_WIDGET (_tmp0_ = editing_tool_get_tool_window (EDITING_TOOL (self))));
#line 4281 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 446 "AppWindow.vala"
	_tmp1_ = NULL;
#line 1464 "EditingTools.vala"
	app_window_set_busy_cursor (_tmp1_ = app_window_get_instance ());
#line 4287 "EditingTools.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	adjustments = gee_array_list_new (TYPE_COLOR_TRANSFORMATION_INSTANCE, (GBoxedCopyFunc) color_transformation_instance_dup, color_transformation_instance_free, g_direct_equal);
	exposure_param = (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->exposure_slider));
#line 1472 "EditingTools.vala"
	current_instance.kind = COLOR_TRANSFORMATION_KIND_EXPOSURE;
#line 1473 "EditingTools.vala"
	current_instance.parameter = exposure_param;
#line 1474 "EditingTools.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (adjustments), &current_instance);
#line 4297 "EditingTools.c"
	saturation_param = (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->saturation_slider));
#line 1478 "EditingTools.vala"
	current_instance.kind = COLOR_TRANSFORMATION_KIND_SATURATION;
#line 1479 "EditingTools.vala"
	current_instance.parameter = saturation_param;
#line 1480 "EditingTools.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (adjustments), &current_instance);
#line 4305 "EditingTools.c"
	tint_param = (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->tint_slider));
#line 1484 "EditingTools.vala"
	current_instance.kind = COLOR_TRANSFORMATION_KIND_TINT;
#line 1485 "EditingTools.vala"
	current_instance.parameter = tint_param;
#line 1486 "EditingTools.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (adjustments), &current_instance);
#line 4313 "EditingTools.c"
	temperature_param = (float) gtk_range_get_value (GTK_RANGE (self->priv->adjust_tool_window->temperature_slider));
#line 1490 "EditingTools.vala"
	current_instance.kind = COLOR_TRANSFORMATION_KIND_TEMPERATURE;
#line 1491 "EditingTools.vala"
	current_instance.parameter = temperature_param;
#line 1492 "EditingTools.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (adjustments), &current_instance);
#line 172 "EditingTools.vala"
	_tmp2_ = NULL;
#line 1494 "EditingTools.vala"
	photo_set_adjustments (_tmp2_ = photo_canvas_get_photo (EDITING_TOOL (self)->canvas), adjustments);
#line 4325 "EditingTools.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 1496 "EditingTools.vala"
	editing_tool_notify_apply (EDITING_TOOL (self));
#line 446 "AppWindow.vala"
	_tmp3_ = NULL;
#line 1498 "EditingTools.vala"
	app_window_set_normal_cursor (_tmp3_ = app_window_get_instance ());
#line 4333 "EditingTools.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
	(adjustments == NULL) ? NULL : (adjustments = (g_object_unref (adjustments), NULL));
}


#line 1501 "EditingTools.vala"
static void adjust_tool_on_adjustment (AdjustTool* self) {
#line 4341 "EditingTools.c"
	GdkPixbuf* _tmp1_;
	ImageHistogram* _tmp0_;
#line 1501 "EditingTools.vala"
	g_return_if_fail (IS_ADJUST_TOOL (self));
#line 1502 "EditingTools.vala"
	photo_canvas_repaint (EDITING_TOOL (self)->canvas);
#line 642 "ColorTransformation.vala"
	_tmp1_ = NULL;
#line 4350 "EditingTools.c"
	_tmp0_ = NULL;
#line 1503 "EditingTools.vala"
	gtk_image_set_from_pixbuf (self->priv->adjust_tool_window->histogram_image_tray, _tmp1_ = image_histogram_get_graphic (_tmp0_ = image_histogram_new (self->priv->draw_to_pixbuf)));
#line 4354 "EditingTools.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (image_histogram_unref (_tmp0_), NULL));
}


#line 1507 "EditingTools.vala"
static void adjust_tool_on_canvas_resize (AdjustTool* self) {
#line 4362 "EditingTools.c"
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
#line 1507 "EditingTools.vala"
	g_return_if_fail (IS_ADJUST_TOOL (self));
#line 4367 "EditingTools.c"
	_tmp1_ = NULL;
#line 199 "EditingTools.vala"
	_tmp0_ = NULL;
#line 1508 "EditingTools.vala"
	self->priv->draw_to_pixbuf = (_tmp1_ = gdk_pixbuf_copy (_tmp0_ = photo_canvas_get_scaled_pixbuf (EDITING_TOOL (self)->canvas)), (self->priv->draw_to_pixbuf == NULL) ? NULL : (self->priv->draw_to_pixbuf = (g_object_unref (self->priv->draw_to_pixbuf), NULL)), _tmp1_);
#line 4373 "EditingTools.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 1254 "EditingTools.vala"
AdjustTool* adjust_tool_construct (GType object_type) {
#line 4380 "EditingTools.c"
	AdjustTool* self;
	self = (AdjustTool*) g_type_create_instance (object_type);
	return self;
}


#line 1254 "EditingTools.vala"
AdjustTool* adjust_tool_new (void) {
#line 1254 "EditingTools.vala"
	return adjust_tool_construct (TYPE_ADJUST_TOOL);
#line 4391 "EditingTools.c"
}


#line 1276 "EditingTools.vala"
static AdjustToolAdjustToolWindow* adjust_tool_adjust_tool_window_construct (GType object_type, GtkWindow* container) {
#line 4397 "EditingTools.c"
	AdjustToolAdjustToolWindow * self;
	GtkTable* slider_organizer;
	GtkLabel* exposure_label;
	GtkLabel* saturation_label;
	GtkLabel* tint_label;
	GtkLabel* temperature_label;
	GtkHBox* button_layouter;
	GtkFrame* histogram_tray_wrapper;
	GtkVBox* pane_layouter;
#line 1276 "EditingTools.vala"
	g_return_val_if_fail (GTK_IS_WINDOW (container), NULL);
#line 1277 "EditingTools.vala"
	self = (AdjustToolAdjustToolWindow*) editing_tool_window_construct (object_type, container);
#line 1279 "EditingTools.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->histogram_image_tray), IMAGE_HISTOGRAM_GRAPHIC_WIDTH, IMAGE_HISTOGRAM_GRAPHIC_HEIGHT);
#line 4413 "EditingTools.c"
	slider_organizer = g_object_ref_sink ((GtkTable*) gtk_table_new ((guint) 4, (guint) 2, FALSE));
#line 1283 "EditingTools.vala"
	gtk_table_set_row_spacings (slider_organizer, (guint) 12);
#line 1284 "EditingTools.vala"
	gtk_table_set_col_spacings (slider_organizer, (guint) 12);
#line 4419 "EditingTools.c"
	exposure_label = g_object_ref_sink ((GtkLabel*) gtk_label_new_with_mnemonic ("Exposure:"));
#line 1287 "EditingTools.vala"
	gtk_misc_set_alignment (GTK_MISC (exposure_label), 0.0f, 0.5f);
#line 1288 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (exposure_label), (guint) 0, (guint) 1, (guint) 0, (guint) 1);
#line 1289 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (self->exposure_slider), (guint) 1, (guint) 2, (guint) 0, (guint) 1);
#line 1290 "EditingTools.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->exposure_slider), ADJUST_TOOL_SLIDER_WIDTH, -1);
#line 1291 "EditingTools.vala"
	gtk_scale_set_draw_value (GTK_SCALE (self->exposure_slider), FALSE);
#line 1292 "EditingTools.vala"
	gtk_range_set_update_policy (GTK_RANGE (self->exposure_slider), GTK_UPDATE_DISCONTINUOUS);
#line 4433 "EditingTools.c"
	saturation_label = g_object_ref_sink ((GtkLabel*) gtk_label_new_with_mnemonic ("Saturation:"));
#line 1295 "EditingTools.vala"
	gtk_misc_set_alignment (GTK_MISC (saturation_label), 0.0f, 0.5f);
#line 1296 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (saturation_label), (guint) 0, (guint) 1, (guint) 1, (guint) 2);
#line 1297 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (self->saturation_slider), (guint) 1, (guint) 2, (guint) 1, (guint) 2);
#line 1298 "EditingTools.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->saturation_slider), ADJUST_TOOL_SLIDER_WIDTH, -1);
#line 1299 "EditingTools.vala"
	gtk_scale_set_draw_value (GTK_SCALE (self->saturation_slider), FALSE);
#line 1300 "EditingTools.vala"
	gtk_range_set_update_policy (GTK_RANGE (self->saturation_slider), GTK_UPDATE_DISCONTINUOUS);
#line 4447 "EditingTools.c"
	tint_label = g_object_ref_sink ((GtkLabel*) gtk_label_new_with_mnemonic ("Tint:"));
#line 1303 "EditingTools.vala"
	gtk_misc_set_alignment (GTK_MISC (tint_label), 0.0f, 0.5f);
#line 1304 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (tint_label), (guint) 0, (guint) 1, (guint) 2, (guint) 3);
#line 1305 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (self->tint_slider), (guint) 1, (guint) 2, (guint) 2, (guint) 3);
#line 1306 "EditingTools.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->tint_slider), ADJUST_TOOL_SLIDER_WIDTH, -1);
#line 1307 "EditingTools.vala"
	gtk_scale_set_draw_value (GTK_SCALE (self->tint_slider), FALSE);
#line 1308 "EditingTools.vala"
	gtk_range_set_update_policy (GTK_RANGE (self->tint_slider), GTK_UPDATE_DISCONTINUOUS);
#line 4461 "EditingTools.c"
	temperature_label = g_object_ref_sink ((GtkLabel*) gtk_label_new_with_mnemonic ("Temperature:"));
#line 1312 "EditingTools.vala"
	gtk_misc_set_alignment (GTK_MISC (temperature_label), 0.0f, 0.5f);
#line 1313 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (temperature_label), (guint) 0, (guint) 1, (guint) 3, (guint) 4);
#line 1314 "EditingTools.vala"
	gtk_table_attach_defaults (slider_organizer, GTK_WIDGET (self->temperature_slider), (guint) 1, (guint) 2, (guint) 3, (guint) 4);
#line 1315 "EditingTools.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->temperature_slider), ADJUST_TOOL_SLIDER_WIDTH, -1);
#line 1316 "EditingTools.vala"
	gtk_scale_set_draw_value (GTK_SCALE (self->temperature_slider), FALSE);
#line 1317 "EditingTools.vala"
	gtk_range_set_update_policy (GTK_RANGE (self->temperature_slider), GTK_UPDATE_DISCONTINUOUS);
#line 4475 "EditingTools.c"
	button_layouter = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 8));
#line 1320 "EditingTools.vala"
	gtk_box_set_homogeneous (GTK_BOX (button_layouter), TRUE);
#line 1321 "EditingTools.vala"
	gtk_box_pack_start (GTK_BOX (button_layouter), GTK_WIDGET (self->cancel_button), TRUE, TRUE, (guint) 1);
#line 1322 "EditingTools.vala"
	gtk_box_pack_start (GTK_BOX (button_layouter), GTK_WIDGET (self->reset_button), TRUE, TRUE, (guint) 1);
#line 1323 "EditingTools.vala"
	gtk_box_pack_start (GTK_BOX (button_layouter), GTK_WIDGET (self->apply_button), TRUE, TRUE, (guint) 1);
#line 4485 "EditingTools.c"
	histogram_tray_wrapper = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL));
#line 1326 "EditingTools.vala"
	gtk_frame_set_shadow_type (histogram_tray_wrapper, GTK_SHADOW_ETCHED_IN);
#line 1327 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (histogram_tray_wrapper), GTK_WIDGET (self->histogram_image_tray));
#line 4491 "EditingTools.c"
	pane_layouter = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 8));
#line 1330 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (pane_layouter), GTK_WIDGET (histogram_tray_wrapper));
#line 1331 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (pane_layouter), GTK_WIDGET (slider_organizer));
#line 1332 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (pane_layouter), GTK_WIDGET (button_layouter));
#line 1334 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->exposure_slider), 0.0);
#line 1335 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->saturation_slider), 0.0);
#line 1336 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->tint_slider), 0.0);
#line 1337 "EditingTools.vala"
	gtk_range_set_value (GTK_RANGE (self->temperature_slider), 0.0);
#line 1339 "EditingTools.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (pane_layouter));
#line 4509 "EditingTools.c"
	(slider_organizer == NULL) ? NULL : (slider_organizer = (g_object_unref (slider_organizer), NULL));
	(exposure_label == NULL) ? NULL : (exposure_label = (g_object_unref (exposure_label), NULL));
	(saturation_label == NULL) ? NULL : (saturation_label = (g_object_unref (saturation_label), NULL));
	(tint_label == NULL) ? NULL : (tint_label = (g_object_unref (tint_label), NULL));
	(temperature_label == NULL) ? NULL : (temperature_label = (g_object_unref (temperature_label), NULL));
	(button_layouter == NULL) ? NULL : (button_layouter = (g_object_unref (button_layouter), NULL));
	(histogram_tray_wrapper == NULL) ? NULL : (histogram_tray_wrapper = (g_object_unref (histogram_tray_wrapper), NULL));
	(pane_layouter == NULL) ? NULL : (pane_layouter = (g_object_unref (pane_layouter), NULL));
	return self;
}


#line 1276 "EditingTools.vala"
static AdjustToolAdjustToolWindow* adjust_tool_adjust_tool_window_new (GtkWindow* container) {
#line 1276 "EditingTools.vala"
	return adjust_tool_adjust_tool_window_construct (ADJUST_TOOL_TYPE_ADJUST_TOOL_WINDOW, container);
#line 4526 "EditingTools.c"
}


static void adjust_tool_adjust_tool_window_class_init (AdjustToolAdjustToolWindowClass * klass) {
	adjust_tool_adjust_tool_window_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = adjust_tool_adjust_tool_window_finalize;
}


static void adjust_tool_adjust_tool_window_instance_init (AdjustToolAdjustToolWindow * self) {
	self->exposure_slider = g_object_ref_sink ((GtkHScale*) gtk_hscale_new_with_range ((double) ADJUST_TOOL_SLIDER_MIN_VALUE, (double) ADJUST_TOOL_SLIDER_MAX_VALUE, 1.0));
	self->saturation_slider = g_object_ref_sink ((GtkHScale*) gtk_hscale_new_with_range ((double) ADJUST_TOOL_SLIDER_MIN_VALUE, (double) ADJUST_TOOL_SLIDER_MAX_VALUE, 1.0));
	self->tint_slider = g_object_ref_sink ((GtkHScale*) gtk_hscale_new_with_range ((double) ADJUST_TOOL_SLIDER_MIN_VALUE, (double) ADJUST_TOOL_SLIDER_MAX_VALUE, 1.0));
	self->temperature_slider = g_object_ref_sink ((GtkHScale*) gtk_hscale_new_with_range ((double) ADJUST_TOOL_SLIDER_MIN_VALUE, (double) ADJUST_TOOL_SLIDER_MAX_VALUE, 1.0));
	self->apply_button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_APPLY));
	self->reset_button = g_object_ref_sink ((GtkButton*) gtk_button_new_with_label ("Reset"));
	self->cancel_button = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_CANCEL));
	self->histogram_image_tray = g_object_ref_sink ((GtkImage*) gtk_image_new ());
}


static void adjust_tool_adjust_tool_window_finalize (GObject* obj) {
	AdjustToolAdjustToolWindow * self;
	self = ADJUST_TOOL_ADJUST_TOOL_WINDOW (obj);
	(self->exposure_slider == NULL) ? NULL : (self->exposure_slider = (g_object_unref (self->exposure_slider), NULL));
	(self->saturation_slider == NULL) ? NULL : (self->saturation_slider = (g_object_unref (self->saturation_slider), NULL));
	(self->tint_slider == NULL) ? NULL : (self->tint_slider = (g_object_unref (self->tint_slider), NULL));
	(self->temperature_slider == NULL) ? NULL : (self->temperature_slider = (g_object_unref (self->temperature_slider), NULL));
	(self->apply_button == NULL) ? NULL : (self->apply_button = (g_object_unref (self->apply_button), NULL));
	(self->reset_button == NULL) ? NULL : (self->reset_button = (g_object_unref (self->reset_button), NULL));
	(self->cancel_button == NULL) ? NULL : (self->cancel_button = (g_object_unref (self->cancel_button), NULL));
	(self->histogram_image_tray == NULL) ? NULL : (self->histogram_image_tray = (g_object_unref (self->histogram_image_tray), NULL));
	G_OBJECT_CLASS (adjust_tool_adjust_tool_window_parent_class)->finalize (obj);
}


static GType adjust_tool_adjust_tool_window_get_type (void) {
	static GType adjust_tool_adjust_tool_window_type_id = 0;
	if (adjust_tool_adjust_tool_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AdjustToolAdjustToolWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) adjust_tool_adjust_tool_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AdjustToolAdjustToolWindow), 0, (GInstanceInitFunc) adjust_tool_adjust_tool_window_instance_init, NULL };
		adjust_tool_adjust_tool_window_type_id = g_type_register_static (TYPE_EDITING_TOOL_WINDOW, "AdjustToolAdjustToolWindow", &g_define_type_info, 0);
	}
	return adjust_tool_adjust_tool_window_type_id;
}


static void adjust_tool_class_init (AdjustToolClass * klass) {
	adjust_tool_parent_class = g_type_class_peek_parent (klass);
	EDITING_TOOL_CLASS (klass)->finalize = adjust_tool_finalize;
	g_type_class_add_private (klass, sizeof (AdjustToolPrivate));
	EDITING_TOOL_CLASS (klass)->activate = adjust_tool_real_activate;
	EDITING_TOOL_CLASS (klass)->get_tool_window = adjust_tool_real_get_tool_window;
	EDITING_TOOL_CLASS (klass)->deactivate = adjust_tool_real_deactivate;
	EDITING_TOOL_CLASS (klass)->paint = adjust_tool_real_paint;
	EDITING_TOOL_CLASS (klass)->get_unscaled_pixbuf = adjust_tool_real_get_unscaled_pixbuf;
}


static void adjust_tool_instance_init (AdjustTool * self) {
	self->priv = ADJUST_TOOL_GET_PRIVATE (self);
	self->priv->adjust_tool_window = NULL;
	self->priv->suppress_effect_redraw = FALSE;
	self->priv->draw_to_pixbuf = NULL;
}


static void adjust_tool_finalize (EditingTool* obj) {
	AdjustTool * self;
	self = ADJUST_TOOL (obj);
	(self->priv->adjust_tool_window == NULL) ? NULL : (self->priv->adjust_tool_window = (g_object_unref (self->priv->adjust_tool_window), NULL));
	(self->priv->draw_to_pixbuf == NULL) ? NULL : (self->priv->draw_to_pixbuf = (g_object_unref (self->priv->draw_to_pixbuf), NULL));
	EDITING_TOOL_CLASS (adjust_tool_parent_class)->finalize (obj);
}


GType adjust_tool_get_type (void) {
	static GType adjust_tool_type_id = 0;
	if (adjust_tool_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AdjustToolClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) adjust_tool_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AdjustTool), 0, (GInstanceInitFunc) adjust_tool_instance_init, NULL };
		adjust_tool_type_id = g_type_register_static (TYPE_EDITING_TOOL, "AdjustTool", &g_define_type_info, 0);
	}
	return adjust_tool_type_id;
}



static void g_cclosure_user_marshal_VOID__OBJECT_OBJECT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer data1, gpointer arg_1, gpointer arg_2, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_OBJECT callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_object (param_values + 1), g_value_get_object (param_values + 2), data2);
}


static void g_cclosure_user_marshal_VOID__BOXED_OBJECT_BOXED (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__BOXED_OBJECT_BOXED) (gpointer data1, gpointer arg_1, gpointer arg_2, gpointer arg_3, gpointer data2);
	register GMarshalFunc_VOID__BOXED_OBJECT_BOXED callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 4);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__BOXED_OBJECT_BOXED) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_boxed (param_values + 1), g_value_get_object (param_values + 2), g_value_get_boxed (param_values + 3), data2);
}



