/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU LGPL (version 2.1 or later).
 * See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <gee.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>


#define TYPE_PAGE_LAYOUT (page_layout_get_type ())
#define PAGE_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE_LAYOUT, PageLayout))
#define PAGE_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE_LAYOUT, PageLayoutClass))
#define IS_PAGE_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE_LAYOUT))
#define IS_PAGE_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE_LAYOUT))
#define PAGE_LAYOUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE_LAYOUT, PageLayoutClass))

typedef struct _PageLayout PageLayout;
typedef struct _PageLayoutClass PageLayoutClass;
typedef struct _PageLayoutPrivate PageLayoutPrivate;

#define TYPE_PAGE (page_get_type ())
#define PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAGE, Page))
#define PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAGE, PageClass))
#define IS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAGE))
#define IS_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAGE))
#define PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAGE, PageClass))

typedef struct _Page Page;
typedef struct _PageClass PageClass;
typedef struct _PagePrivate PagePrivate;

#define TYPE_SIDEBAR_MARKER (sidebar_marker_get_type ())
#define SIDEBAR_MARKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SIDEBAR_MARKER, SidebarMarker))
#define SIDEBAR_MARKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SIDEBAR_MARKER, SidebarMarkerClass))
#define IS_SIDEBAR_MARKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SIDEBAR_MARKER))
#define IS_SIDEBAR_MARKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SIDEBAR_MARKER))
#define SIDEBAR_MARKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SIDEBAR_MARKER, SidebarMarkerClass))

typedef struct _SidebarMarker SidebarMarker;
typedef struct _SidebarMarkerClass SidebarMarkerClass;

#define PAGE_TYPE_TARGET_TYPE (page_target_type_get_type ())

#define TYPE_APP_WINDOW (app_window_get_type ())
#define APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_APP_WINDOW, AppWindow))
#define APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_APP_WINDOW, AppWindowClass))
#define IS_APP_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_APP_WINDOW))
#define IS_APP_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_APP_WINDOW))
#define APP_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_APP_WINDOW, AppWindowClass))

typedef struct _AppWindow AppWindow;
typedef struct _AppWindowClass AppWindowClass;

#define TYPE_CHECKERBOARD_PAGE (checkerboard_page_get_type ())
#define CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPage))
#define CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))
#define IS_CHECKERBOARD_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CHECKERBOARD_PAGE))
#define IS_CHECKERBOARD_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CHECKERBOARD_PAGE))
#define CHECKERBOARD_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CHECKERBOARD_PAGE, CheckerboardPageClass))

typedef struct _CheckerboardPage CheckerboardPage;
typedef struct _CheckerboardPageClass CheckerboardPageClass;
typedef struct _CheckerboardPagePrivate CheckerboardPagePrivate;

#define TYPE_LAYOUT_ITEM (layout_item_get_type ())
#define LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LAYOUT_ITEM, LayoutItem))
#define LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LAYOUT_ITEM, LayoutItemClass))
#define IS_LAYOUT_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LAYOUT_ITEM))
#define IS_LAYOUT_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LAYOUT_ITEM))
#define LAYOUT_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LAYOUT_ITEM, LayoutItemClass))

typedef struct _LayoutItem LayoutItem;
typedef struct _LayoutItemClass LayoutItemClass;

#define TYPE_COLLECTION_LAYOUT (collection_layout_get_type ())
#define COLLECTION_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_LAYOUT, CollectionLayout))
#define COLLECTION_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_LAYOUT, CollectionLayoutClass))
#define IS_COLLECTION_LAYOUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_LAYOUT))
#define IS_COLLECTION_LAYOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_LAYOUT))
#define COLLECTION_LAYOUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_LAYOUT, CollectionLayoutClass))

typedef struct _CollectionLayout CollectionLayout;
typedef struct _CollectionLayoutClass CollectionLayoutClass;

#define TYPE_COMPARATOR (comparator_get_type ())
#define COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COMPARATOR, Comparator))
#define COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COMPARATOR, ComparatorClass))
#define IS_COMPARATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COMPARATOR))
#define IS_COMPARATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COMPARATOR))
#define COMPARATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COMPARATOR, ComparatorClass))

typedef struct _Comparator Comparator;
typedef struct _ComparatorClass ComparatorClass;
typedef struct _CollectionLayoutPrivate CollectionLayoutPrivate;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;

#define TYPE_COMPASS_POINT (compass_point_get_type ())

#define TYPE_BOX (box_get_type ())
typedef struct _Box Box;

#define TYPE_ADJUSTMENT_RELATION (adjustment_relation_get_type ())

#define TYPE_SINGLE_PHOTO_PAGE (single_photo_page_get_type ())
#define SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPage))
#define SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))
#define IS_SINGLE_PHOTO_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SINGLE_PHOTO_PAGE))
#define IS_SINGLE_PHOTO_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SINGLE_PHOTO_PAGE))
#define SINGLE_PHOTO_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPageClass))

typedef struct _SinglePhotoPage SinglePhotoPage;
typedef struct _SinglePhotoPageClass SinglePhotoPageClass;
typedef struct _SinglePhotoPagePrivate SinglePhotoPagePrivate;

#define SINGLE_PHOTO_PAGE_TYPE_UPDATE_REASON (single_photo_page_update_reason_get_type ())

#define TYPE_DIMENSIONS (dimensions_get_type ())
typedef struct _Dimensions Dimensions;

struct _PageLayout {
	GtkVBox parent_instance;
	PageLayoutPrivate * priv;
	Page* page;
};

struct _PageLayoutClass {
	GtkVBoxClass parent_class;
};

struct _Page {
	GtkScrolledWindow parent_instance;
	PagePrivate * priv;
	GtkUIManager* ui;
	GtkActionGroup* action_group;
	GtkActionGroup* common_action_group;
};

struct _PageClass {
	GtkScrolledWindowClass parent_class;
	GtkMenuBar* (*get_menubar) (Page* self);
	GtkToolbar* (*get_toolbar) (Page* self);
	void (*switching_from) (Page* self);
	void (*switched_to) (Page* self);
	void (*switching_to_fullscreen) (Page* self);
	void (*returning_from_fullscreen) (Page* self);
	gboolean (*source_drag_failed) (Page* self, GdkDragContext* context, GtkDragResult drag_result);
	gboolean (*on_left_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_click) (Page* self, const GdkEventButton* event);
	gboolean (*on_left_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_middle_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_right_released) (Page* self, const GdkEventButton* event);
	gboolean (*on_ctrl_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_ctrl_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_alt_released) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_pressed) (Page* self, const GdkEventKey* event);
	gboolean (*on_shift_released) (Page* self, const GdkEventKey* event);
	void (*on_move) (Page* self, const GdkRectangle* rect);
	void (*on_resize) (Page* self, const GdkRectangle* rect);
	gboolean (*on_configure) (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
	gboolean (*on_motion) (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
};

struct _PagePrivate {
	char* page_name;
	PageLayout* layout;
	GtkMenuBar* menu_bar;
	SidebarMarker* marker;
	GdkRectangle last_position;
	GtkWidget* event_source;
	gboolean dnd_enabled;
	gboolean in_view;
};

typedef enum  {
	PAGE_TARGET_TYPE_URI_LIST
} PageTargetType;

struct _CheckerboardPage {
	Page parent_instance;
	CheckerboardPagePrivate * priv;
};

struct _CheckerboardPageClass {
	PageClass parent_class;
	void (*on_selection_changed) (CheckerboardPage* self, gint count);
	GtkMenu* (*get_context_menu) (CheckerboardPage* self);
	void (*on_item_activated) (CheckerboardPage* self, LayoutItem* item);
	gboolean (*on_context_invoked) (CheckerboardPage* self, GtkMenu* context_menu);
	LayoutItem* (*get_fullscreen_photo) (CheckerboardPage* self);
	gboolean (*on_mouse_over) (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
};

struct _CheckerboardPagePrivate {
	GtkMenu* context_menu;
	CollectionLayout* layout;
	GeeHashSet* selected_items;
	LayoutItem* last_clicked_item;
	LayoutItem* highlighted;
	gboolean drag_select;
	GdkPoint drag_start;
	GdkRectangle selection_band;
	GdkGC* selection_gc;
	guint32 selection_transparency_color;
	gboolean autoscroll_scheduled;
};

struct _CollectionLayout {
	GtkLayout parent_instance;
	CollectionLayoutPrivate * priv;
	SortedList* items;
};

struct _CollectionLayoutClass {
	GtkLayoutClass parent_class;
};

typedef enum  {
	COMPASS_POINT_NORTH,
	COMPASS_POINT_SOUTH,
	COMPASS_POINT_EAST,
	COMPASS_POINT_WEST
} CompassPoint;

struct _Box {
	gint left;
	gint top;
	gint right;
	gint bottom;
};

typedef enum  {
	ADJUSTMENT_RELATION_BELOW,
	ADJUSTMENT_RELATION_IN_RANGE,
	ADJUSTMENT_RELATION_ABOVE
} AdjustmentRelation;

typedef enum  {
	SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT,
	SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS
} SinglePhotoPageUpdateReason;

struct _Dimensions {
	gint width;
	gint height;
};

struct _SinglePhotoPage {
	Page parent_instance;
	SinglePhotoPagePrivate * priv;
	GdkGC* canvas_gc;
	GtkDrawingArea* canvas;
	GtkViewport* viewport;
};

struct _SinglePhotoPageClass {
	PageClass parent_class;
	void (*new_drawable) (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable);
	void (*updated_pixbuf) (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
	void (*paint) (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
};

struct _SinglePhotoPagePrivate {
	GdkPixmap* pixmap;
	Dimensions pixmap_dim;
	GdkPixbuf* unscaled;
	GdkPixbuf* scaled;
	GdkRectangle scaled_pos;
	GdkInterpType default_interp;
	GdkInterpType interp;
	SinglePhotoPage* improval_scheduled;
	gboolean reschedule_improval;
};


static gpointer page_layout_parent_class = NULL;
static gpointer page_parent_class = NULL;
static GdkPixbuf* checkerboard_page_selection_interior;
static GdkPixbuf* checkerboard_page_selection_interior = NULL;
static gpointer checkerboard_page_parent_class = NULL;
static gpointer single_photo_page_parent_class = NULL;

GType page_layout_get_type (void);
GType page_get_type (void);
enum  {
	PAGE_LAYOUT_DUMMY_PROPERTY
};
GtkToolbar* page_get_toolbar (Page* self);
PageLayout* page_layout_new (Page* page);
PageLayout* page_layout_construct (GType object_type, Page* page);
static void page_layout_finalize (GObject* obj);
gpointer sidebar_marker_ref (gpointer instance);
void sidebar_marker_unref (gpointer instance);
GParamSpec* param_spec_sidebar_marker (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_sidebar_marker (GValue* value, gpointer v_object);
gpointer value_get_sidebar_marker (const GValue* value);
GType sidebar_marker_get_type (void);
#define PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PAGE, PagePrivate))
enum  {
	PAGE_DUMMY_PROPERTY
};
GType page_target_type_get_type (void);
#define PAGE_KEY_CTRL_L gdk_keyval_from_name ("Control_L")
#define PAGE_KEY_CTRL_R gdk_keyval_from_name ("Control_R")
#define PAGE_KEY_ALT_L gdk_keyval_from_name ("Alt_L")
#define PAGE_KEY_ALT_R gdk_keyval_from_name ("Alt_R")
#define PAGE_KEY_SHIFT_L gdk_keyval_from_name ("Shift_L")
#define PAGE_KEY_SHIFT_R gdk_keyval_from_name ("Shift_R")
Page* page_new (const char* page_name);
Page* page_construct (GType object_type, const char* page_name);
char* page_get_page_name (Page* self);
PageLayout* page_get_layout (Page* self);
void page_set_page_name (Page* self, const char* page_name);
static gboolean page_on_button_pressed_internal (Page* self, const GdkEventButton* event);
static gboolean _page_on_button_pressed_internal_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static gboolean page_on_button_released_internal (Page* self, const GdkEventButton* event);
static gboolean _page_on_button_released_internal_gtk_widget_button_release_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static gboolean page_on_motion_internal (Page* self, const GdkEventMotion* event);
static gboolean _page_on_motion_internal_gtk_widget_motion_notify_event (GtkWidget* _sender, const GdkEventMotion* event, gpointer self);
void page_set_event_source (Page* self, GtkWidget* event_source);
GtkWidget* page_get_event_source (Page* self);
void page_set_marker (Page* self, SidebarMarker* marker);
SidebarMarker* page_get_marker (Page* self);
void page_clear_marker (Page* self);
GtkMenuBar* page_get_menubar (Page* self);
static GtkMenuBar* page_real_get_menubar (Page* self);
static GtkToolbar* page_real_get_toolbar (Page* self);
void page_switching_from (Page* self);
static void page_real_switching_from (Page* self);
void page_switched_to (Page* self);
static void page_real_switched_to (Page* self);
gboolean page_is_in_view (Page* self);
void page_switching_to_fullscreen (Page* self);
static void page_real_switching_to_fullscreen (Page* self);
void page_returning_from_fullscreen (Page* self);
static void page_real_returning_from_fullscreen (Page* self);
void page_set_item_sensitive (Page* self, const char* path, gboolean sensitive);
void page_init_ui_start (Page* self, const char* ui_filename, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1);
void page_init_ui_bind (Page* self, const char* menubar_path);
void page_init_ui (Page* self, const char* ui_filename, const char* menubar_path, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1);
GFile* resources_get_ui (const char* filename);
void page_init_load_ui (Page* self, const char* ui_filename);
GType app_window_get_type (void);
AppWindow* app_window_get_instance (void);
GtkActionGroup* app_window_get_common_action_group (AppWindow* self);
static void page_on_drag_begin (Page* self, GdkDragContext* context);
static void _page_on_drag_begin_gtk_widget_drag_begin (GtkWidget* _sender, GdkDragContext* context, gpointer self);
static void page_on_drag_data_get (Page* self, GdkDragContext* context, GtkSelectionData* selection_data, guint info, guint time);
static void _page_on_drag_data_get_gtk_widget_drag_data_get (GtkWidget* _sender, GdkDragContext* context, GtkSelectionData* selection_data, guint info, guint time_, gpointer self);
static void page_on_drag_data_delete (Page* self, GdkDragContext* context);
static void _page_on_drag_data_delete_gtk_widget_drag_data_delete (GtkWidget* _sender, GdkDragContext* context, gpointer self);
static void page_on_drag_end (Page* self, GdkDragContext* context);
static void _page_on_drag_end_gtk_widget_drag_end (GtkWidget* _sender, GdkDragContext* context, gpointer self);
static gboolean page_on_drag_failed (Page* self, GdkDragContext* context, GtkDragResult drag_result);
static gboolean _page_on_drag_failed_gtk_widget_drag_failed (GtkWidget* _sender, GdkDragContext* p0, GtkDragResult p1, gpointer self);
void page_enable_drag_source (Page* self, GdkDragAction actions);
void page_disable_drag_source (Page* self);
gboolean page_is_dnd_enabled (Page* self);
gboolean page_source_drag_failed (Page* self, GdkDragContext* context, GtkDragResult drag_result);
static gboolean page_real_source_drag_failed (Page* self, GdkDragContext* context, GtkDragResult drag_result);
gboolean page_on_left_click (Page* self, const GdkEventButton* event);
static gboolean page_real_on_left_click (Page* self, const GdkEventButton* event);
gboolean page_on_middle_click (Page* self, const GdkEventButton* event);
static gboolean page_real_on_middle_click (Page* self, const GdkEventButton* event);
gboolean page_on_right_click (Page* self, const GdkEventButton* event);
static gboolean page_real_on_right_click (Page* self, const GdkEventButton* event);
gboolean page_on_left_released (Page* self, const GdkEventButton* event);
static gboolean page_real_on_left_released (Page* self, const GdkEventButton* event);
gboolean page_on_middle_released (Page* self, const GdkEventButton* event);
static gboolean page_real_on_middle_released (Page* self, const GdkEventButton* event);
gboolean page_on_right_released (Page* self, const GdkEventButton* event);
static gboolean page_real_on_right_released (Page* self, const GdkEventButton* event);
gboolean page_on_ctrl_pressed (Page* self, const GdkEventKey* event);
static gboolean page_real_on_ctrl_pressed (Page* self, const GdkEventKey* event);
gboolean page_on_ctrl_released (Page* self, const GdkEventKey* event);
static gboolean page_real_on_ctrl_released (Page* self, const GdkEventKey* event);
gboolean page_on_alt_pressed (Page* self, const GdkEventKey* event);
static gboolean page_real_on_alt_pressed (Page* self, const GdkEventKey* event);
gboolean page_on_alt_released (Page* self, const GdkEventKey* event);
static gboolean page_real_on_alt_released (Page* self, const GdkEventKey* event);
gboolean page_on_shift_pressed (Page* self, const GdkEventKey* event);
static gboolean page_real_on_shift_pressed (Page* self, const GdkEventKey* event);
gboolean page_on_shift_released (Page* self, const GdkEventKey* event);
static gboolean page_real_on_shift_released (Page* self, const GdkEventKey* event);
gboolean page_notify_modifier_pressed (Page* self, const GdkEventKey* event);
gboolean page_notify_modifier_released (Page* self, const GdkEventKey* event);
void page_on_move (Page* self, const GdkRectangle* rect);
static void page_real_on_move (Page* self, const GdkRectangle* rect);
void page_on_resize (Page* self, const GdkRectangle* rect);
static void page_real_on_resize (Page* self, const GdkRectangle* rect);
gboolean page_on_configure (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
static gboolean page_real_on_configure (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect);
gboolean page_notify_configure_event (Page* self, const GdkEventConfigure* event);
gboolean page_on_motion (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
static gboolean page_real_on_motion (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
static void page_finalize (GObject* obj);
GType checkerboard_page_get_type (void);
GType layout_item_get_type (void);
GType collection_layout_get_type (void);
#define CHECKERBOARD_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CHECKERBOARD_PAGE, CheckerboardPagePrivate))
enum  {
	CHECKERBOARD_PAGE_DUMMY_PROPERTY
};
CollectionLayout* collection_layout_new (void);
CollectionLayout* collection_layout_construct (GType object_type);
#define CHECKERBOARD_PAGE_AUTOSCROLL_PIXELS 50
#define CHECKERBOARD_PAGE_AUTOSCROLL_TICKS_MSEC 50
static void checkerboard_page_on_layout_exposed (CheckerboardPage* self);
static void _checkerboard_page_on_layout_exposed_collection_layout_expose_after (CollectionLayout* _sender, gpointer self);
static void checkerboard_page_on_layout_mapped (CheckerboardPage* self);
static void _checkerboard_page_on_layout_mapped_gtk_widget_map (CollectionLayout* _sender, gpointer self);
CheckerboardPage* checkerboard_page_new (const char* page_name);
CheckerboardPage* checkerboard_page_construct (GType object_type, const char* page_name);
void checkerboard_page_init_context_menu (CheckerboardPage* self, const char* path);
void checkerboard_page_on_selection_changed (CheckerboardPage* self, gint count);
static void checkerboard_page_real_on_selection_changed (CheckerboardPage* self, gint count);
GtkMenu* checkerboard_page_get_context_menu (CheckerboardPage* self);
static GtkMenu* checkerboard_page_real_get_context_menu (CheckerboardPage* self);
void checkerboard_page_on_item_activated (CheckerboardPage* self, LayoutItem* item);
static void checkerboard_page_real_on_item_activated (CheckerboardPage* self, LayoutItem* item);
gboolean checkerboard_page_on_context_invoked (CheckerboardPage* self, GtkMenu* context_menu);
static gboolean checkerboard_page_real_on_context_invoked (CheckerboardPage* self, GtkMenu* context_menu);
void collection_layout_set_in_view (CollectionLayout* self, gboolean in_view);
static void checkerboard_page_real_switching_from (Page* base);
static void checkerboard_page_real_switched_to (Page* base);
LayoutItem* checkerboard_page_get_fullscreen_photo (CheckerboardPage* self);
static LayoutItem* checkerboard_page_real_get_fullscreen_photo (CheckerboardPage* self);
void collection_layout_refresh (CollectionLayout* self);
void checkerboard_page_refresh (CheckerboardPage* self);
void collection_layout_set_message (CollectionLayout* self, const char* text);
void checkerboard_page_set_page_message (CheckerboardPage* self, const char* message);
void collection_layout_set_refresh_on_resize (CollectionLayout* self, gboolean refresh_on_resize);
void checkerboard_page_set_refresh_on_resize (CheckerboardPage* self, gboolean refresh_on_resize);
gpointer comparator_ref (gpointer instance);
void comparator_unref (gpointer instance);
GParamSpec* param_spec_comparator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_comparator (GValue* value, gpointer v_object);
gpointer value_get_comparator (const GValue* value);
GType comparator_get_type (void);
void collection_layout_set_comparator (CollectionLayout* self, Comparator* cmp);
void checkerboard_page_set_layout_comparator (CheckerboardPage* self, Comparator* cmp);
LayoutItem* collection_layout_get_item_at_pixel (CollectionLayout* self, double xd, double yd);
LayoutItem* checkerboard_page_get_item_at_pixel (CheckerboardPage* self, double x, double y);
GType sorted_list_get_type (void);
GeeIterable* checkerboard_page_get_items (CheckerboardPage* self);
GeeIterable* checkerboard_page_get_selected (CheckerboardPage* self);
void collection_layout_add_item (CollectionLayout* self, LayoutItem* item);
void checkerboard_page_add_item (CheckerboardPage* self, LayoutItem* item);
void collection_layout_remove_item (CollectionLayout* self, LayoutItem* item);
void checkerboard_page_remove_item (CheckerboardPage* self, LayoutItem* item);
gint checkerboard_page_remove_selected (CheckerboardPage* self);
gint sorted_list_get_size (SortedList* self);
void collection_layout_clear (CollectionLayout* self);
gint checkerboard_page_remove_all (CheckerboardPage* self);
gint checkerboard_page_get_count (CheckerboardPage* self);
gboolean layout_item_is_selected (LayoutItem* self);
void layout_item_select (LayoutItem* self);
void checkerboard_page_select_all (CheckerboardPage* self);
void layout_item_unselect (LayoutItem* self);
void checkerboard_page_unselect_all (CheckerboardPage* self);
void checkerboard_page_unselect_all_but (CheckerboardPage* self, LayoutItem* exception);
gboolean sorted_list_contains (SortedList* self, gconstpointer item);
void checkerboard_page_select (CheckerboardPage* self, LayoutItem* item);
void checkerboard_page_unselect (CheckerboardPage* self, LayoutItem* item);
gboolean layout_item_toggle_select (LayoutItem* self);
void checkerboard_page_toggle_select (CheckerboardPage* self, LayoutItem* item);
gint checkerboard_page_get_selected_count (CheckerboardPage* self);
GType compass_point_get_type (void);
void checkerboard_page_move_cursor (CheckerboardPage* self, CompassPoint point);
LayoutItem* checkerboard_page_get_first_item (CheckerboardPage* self);
void checkerboard_page_cursor_to_item (CheckerboardPage* self, LayoutItem* item);
LayoutItem* checkerboard_page_get_last_item (CheckerboardPage* self);
static gboolean checkerboard_page_real_key_press_event (GtkWidget* base, const GdkEventKey* event);
GType box_get_type (void);
Box* box_dup (const Box* self);
void box_free (Box* self);
gboolean checkerboard_page_get_selected_box (CheckerboardPage* self, Box* selected_box);
gint layout_item_get_column (LayoutItem* self);
gint layout_item_get_row (LayoutItem* self);
Box box_rubber_band (Box *self, const GdkPoint* point);
void checkerboard_page_select_all_in_box (CheckerboardPage* self, const Box* box);
static gboolean checkerboard_page_real_on_left_click (Page* base, const GdkEventButton* event);
static gboolean checkerboard_page_real_on_left_released (Page* base, const GdkEventButton* event);
static gboolean checkerboard_page_real_on_right_click (Page* base, const GdkEventButton* event);
void layout_item_unbrighten (LayoutItem* self);
void layout_item_brighten (LayoutItem* self);
gboolean checkerboard_page_on_mouse_over (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
static gboolean checkerboard_page_real_on_mouse_over (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask);
Box box_from_points (const GdkPoint* corner1, const GdkPoint* corner2);
GdkRectangle box_get_rectangle (Box *self);
static void checkerboard_page_updated_selection_band (CheckerboardPage* self);
GType adjustment_relation_get_type (void);
AdjustmentRelation get_adjustment_relation (GtkAdjustment* adjustment, gint value);
static gboolean checkerboard_page_selection_autoscroll (CheckerboardPage* self);
static gboolean _checkerboard_page_selection_autoscroll_gsource_func (gpointer self);
static gboolean checkerboard_page_real_on_motion (Page* base, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask);
GeeList* collection_layout_intersection (CollectionLayout* self, const GdkRectangle* rect);
GdkColor fetch_color (const char* spec, GdkDrawable* drawable);
#define LAYOUT_ITEM_SELECTED_COLOR "#0FF"
guint32 convert_rgba (const GdkColor* c, guint8 alpha);
gpointer sorted_list_get (SortedList* self, gint index);
gint sorted_list_index_of (SortedList* self, gconstpointer item);
LayoutItem* checkerboard_page_get_next_item (CheckerboardPage* self, LayoutItem* current);
LayoutItem* checkerboard_page_get_previous_item (CheckerboardPage* self, LayoutItem* current);
#define COLLECTION_LAYOUT_ROW_GUTTER_PADDING 24
LayoutItem* collection_layout_get_item_at_coordinate (CollectionLayout* self, gint col, gint row);
LayoutItem* collection_layout_get_item_relative_to (CollectionLayout* self, LayoutItem* item, CompassPoint point);
void box_init (Box *self, gint left, gint top, gint right, gint bottom);
gboolean box_contains (Box *self, const GdkPoint* point);
static void checkerboard_page_finalize (GObject* obj);
GType single_photo_page_get_type (void);
GType single_photo_page_update_reason_get_type (void);
GType dimensions_get_type (void);
Dimensions* dimensions_dup (const Dimensions* self);
void dimensions_free (Dimensions* self);
#define SINGLE_PHOTO_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SINGLE_PHOTO_PAGE, SinglePhotoPagePrivate))
enum  {
	SINGLE_PHOTO_PAGE_DUMMY_PROPERTY
};
void dimensions_init (Dimensions *self, gint width, gint height);
#define SINGLE_PHOTO_PAGE_FAST_INTERP GDK_INTERP_NEAREST
#define SINGLE_PHOTO_PAGE_QUALITY_INTERP GDK_INTERP_HYPER
static void single_photo_page_on_viewport_resize (SinglePhotoPage* self);
static void _single_photo_page_on_viewport_resize_gtk_widget_size_allocate (GtkViewport* _sender, const GdkRectangle* allocation, gpointer self);
static gboolean single_photo_page_on_canvas_exposed (SinglePhotoPage* self, const GdkEventExpose* event);
static gboolean _single_photo_page_on_canvas_exposed_gtk_widget_expose_event (GtkDrawingArea* _sender, const GdkEventExpose* event, gpointer self);
SinglePhotoPage* single_photo_page_new (const char* page_name);
SinglePhotoPage* single_photo_page_construct (GType object_type, const char* page_name);
void single_photo_page_set_default_interp (SinglePhotoPage* self, GdkInterpType default_interp);
void single_photo_page_repaint (SinglePhotoPage* self, GdkInterpType repaint_interp);
void single_photo_page_set_pixbuf (SinglePhotoPage* self, GdkPixbuf* unscaled);
GdkDrawable* single_photo_page_get_drawable (SinglePhotoPage* self);
Dimensions single_photo_page_get_drawable_dim (SinglePhotoPage* self);
GdkPixbuf* single_photo_page_get_unscaled_pixbuf (SinglePhotoPage* self);
GdkPixbuf* single_photo_page_get_scaled_pixbuf (SinglePhotoPage* self);
GdkRectangle single_photo_page_get_scaled_pixbuf_position (SinglePhotoPage* self);
gboolean coord_in_rectangle (gint x, gint y, const GdkRectangle* rect);
gboolean single_photo_page_is_inside_pixbuf (SinglePhotoPage* self, gint x, gint y);
void single_photo_page_invalidate (SinglePhotoPage* self, const GdkRectangle* rect);
void single_photo_page_invalidate_all (SinglePhotoPage* self);
void single_photo_page_new_drawable (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable);
static void single_photo_page_real_new_drawable (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable);
void single_photo_page_updated_pixbuf (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
static void single_photo_page_real_updated_pixbuf (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim);
void single_photo_page_paint (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
static void single_photo_page_real_paint (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable);
void single_photo_page_default_repaint (SinglePhotoPage* self);
Dimensions dimensions_for_rectangle (const GdkRectangle* rect);
static void single_photo_page_init_pixmap (SinglePhotoPage* self, gint width, gint height);
static void single_photo_page_schedule_improval (SinglePhotoPage* self);
Dimensions dimensions_for_pixbuf (GdkPixbuf* pixbuf);
Dimensions dimensions_get_scaled_proportional (Dimensions *self, const Dimensions* viewport);
static gboolean single_photo_page_image_improval (SinglePhotoPage* self);
static gboolean _single_photo_page_image_improval_gsource_func (gpointer self);
static void single_photo_page_finalize (GObject* obj);

static const GtkTargetEntry PAGE_SOURCE_TARGET_ENTRIES[] = {{"text/uri-list", (guint) GTK_TARGET_OTHER_APP, (guint) PAGE_TARGET_TYPE_URI_LIST}};


#line 10 "Page.vala"
PageLayout* page_layout_construct (GType object_type, Page* page) {
#line 584 "Page.c"
	PageLayout * self;
	Page* _tmp1_;
	Page* _tmp0_;
	GtkToolbar* _tmp2_;
	gboolean _tmp3_;
#line 10 "Page.vala"
	g_return_val_if_fail (IS_PAGE (page), NULL);
#line 592 "Page.c"
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
#line 11 "Page.vala"
	_tmp0_ = NULL;
#line 11 "Page.vala"
	self->page = (_tmp1_ = (_tmp0_ = page, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->page == NULL) ? NULL : (self->page = (g_object_unref (self->page), NULL)), _tmp1_);
#line 13 "Page.vala"
	gtk_box_set_homogeneous (GTK_BOX (self), FALSE);
#line 14 "Page.vala"
	gtk_box_set_spacing (GTK_BOX (self), 0);
#line 16 "Page.vala"
	gtk_box_pack_start (GTK_BOX (self), GTK_WIDGET (page), TRUE, TRUE, (guint) 0);
#line 114 "Page.vala"
	_tmp2_ = NULL;
#line 17 "Page.vala"
	if ((_tmp3_ = (_tmp2_ = page_get_toolbar (page)) != NULL, (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL)), _tmp3_)) {
#line 609 "Page.c"
		GtkToolbar* _tmp4_;
#line 114 "Page.vala"
		_tmp4_ = NULL;
#line 18 "Page.vala"
		gtk_box_pack_end (GTK_BOX (self), GTK_WIDGET (_tmp4_ = page_get_toolbar (page)), FALSE, FALSE, (guint) 0);
#line 615 "Page.c"
		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
	}
	return self;
}


#line 10 "Page.vala"
PageLayout* page_layout_new (Page* page) {
#line 10 "Page.vala"
	return page_layout_construct (TYPE_PAGE_LAYOUT, page);
#line 626 "Page.c"
}


static void page_layout_class_init (PageLayoutClass * klass) {
	page_layout_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = page_layout_finalize;
}


static void page_layout_instance_init (PageLayout * self) {
}


static void page_layout_finalize (GObject* obj) {
	PageLayout * self;
	self = PAGE_LAYOUT (obj);
	(self->page == NULL) ? NULL : (self->page = (g_object_unref (self->page), NULL));
	G_OBJECT_CLASS (page_layout_parent_class)->finalize (obj);
}


GType page_layout_get_type (void) {
	static GType page_layout_type_id = 0;
	if (page_layout_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PageLayoutClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) page_layout_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PageLayout), 0, (GInstanceInitFunc) page_layout_instance_init, NULL };
		page_layout_type_id = g_type_register_static (GTK_TYPE_VBOX, "PageLayout", &g_define_type_info, 0);
	}
	return page_layout_type_id;
}



GType page_target_type_get_type (void) {
	static GType page_target_type_type_id = 0;
	if (G_UNLIKELY (page_target_type_type_id == 0)) {
		static const GEnumValue values[] = {{PAGE_TARGET_TYPE_URI_LIST, "PAGE_TARGET_TYPE_URI_LIST", "uri-list"}, {0, NULL, NULL}};
		page_target_type_type_id = g_enum_register_static ("PageTargetType", values);
	}
	return page_target_type_type_id;
}


#line 52 "Page.vala"
Page* page_construct (GType object_type, const char* page_name) {
#line 671 "Page.c"
	Page * self;
	char* _tmp1_;
	const char* _tmp0_;
#line 52 "Page.vala"
	g_return_val_if_fail (page_name != NULL, NULL);
#line 677 "Page.c"
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
#line 53 "Page.vala"
	_tmp0_ = NULL;
#line 53 "Page.vala"
	self->priv->page_name = (_tmp1_ = (_tmp0_ = page_name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->page_name = (g_free (self->priv->page_name), NULL), _tmp1_);
#line 55 "Page.vala"
	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (self), GTK_CAN_FOCUS);
#line 686 "Page.c"
	return self;
}


#line 52 "Page.vala"
Page* page_new (const char* page_name) {
#line 52 "Page.vala"
	return page_construct (TYPE_PAGE, page_name);
#line 695 "Page.c"
}


#line 58 "Page.vala"
char* page_get_page_name (Page* self) {
#line 701 "Page.c"
	char* result;
	const char* _tmp0_;
#line 58 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), NULL);
#line 59 "Page.vala"
	_tmp0_ = NULL;
#line 708 "Page.c"
	result = (_tmp0_ = self->priv->page_name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 62 "Page.vala"
PageLayout* page_get_layout (Page* self) {
#line 716 "Page.c"
	PageLayout* result;
	PageLayout* _tmp1_;
	PageLayout* _tmp2_;
#line 62 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), NULL);
#line 722 "Page.c"
	/* This only places the Page into a PageLayout if requested;
	 this is how a Page can live inside AppWindow's notebook or
	 on its own in another window with a separate layout.*/
#line 66 "Page.vala"
	if (self->priv->layout != NULL) {
#line 728 "Page.c"
		PageLayout* _tmp0_;
#line 67 "Page.vala"
		_tmp0_ = NULL;
#line 732 "Page.c"
		result = (_tmp0_ = self->priv->layout, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
		return result;
	}
#line 69 "Page.vala"
	g_assert (gtk_widget_get_parent (GTK_WIDGET (self)) == NULL);
#line 738 "Page.c"
	_tmp1_ = NULL;
#line 70 "Page.vala"
	self->priv->layout = (_tmp1_ = g_object_ref_sink (page_layout_new (self)), (self->priv->layout == NULL) ? NULL : (self->priv->layout = (g_object_unref (self->priv->layout), NULL)), _tmp1_);
#line 72 "Page.vala"
	_tmp2_ = NULL;
#line 744 "Page.c"
	result = (_tmp2_ = self->priv->layout, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
	return result;
}


#line 75 "Page.vala"
void page_set_page_name (Page* self, const char* page_name) {
#line 752 "Page.c"
	char* _tmp1_;
	const char* _tmp0_;
#line 75 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 75 "Page.vala"
	g_return_if_fail (page_name != NULL);
#line 759 "Page.c"
	_tmp1_ = NULL;
#line 76 "Page.vala"
	_tmp0_ = NULL;
#line 76 "Page.vala"
	self->priv->page_name = (_tmp1_ = (_tmp0_ = page_name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->page_name = (g_free (self->priv->page_name), NULL), _tmp1_);
#line 765 "Page.c"
}


#line 266 "Page.vala"
static gboolean _page_on_button_pressed_internal_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
#line 771 "Page.c"
	return page_on_button_pressed_internal (self, event);
}


#line 285 "Page.vala"
static gboolean _page_on_button_released_internal_gtk_widget_button_release_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
#line 778 "Page.c"
	return page_on_button_released_internal (self, event);
}


#line 387 "Page.vala"
static gboolean _page_on_motion_internal_gtk_widget_motion_notify_event (GtkWidget* _sender, const GdkEventMotion* event, gpointer self) {
#line 785 "Page.c"
	return page_on_motion_internal (self, event);
}


#line 79 "Page.vala"
void page_set_event_source (Page* self, GtkWidget* event_source) {
#line 792 "Page.c"
	GtkWidget* _tmp1_;
	GtkWidget* _tmp0_;
#line 79 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 79 "Page.vala"
	g_return_if_fail (GTK_IS_WIDGET (event_source));
#line 80 "Page.vala"
	g_assert (self->priv->event_source == NULL);
#line 801 "Page.c"
	_tmp1_ = NULL;
#line 82 "Page.vala"
	_tmp0_ = NULL;
#line 82 "Page.vala"
	self->priv->event_source = (_tmp1_ = (_tmp0_ = event_source, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->event_source == NULL) ? NULL : (self->priv->event_source = (g_object_unref (self->priv->event_source), NULL)), _tmp1_);
#line 83 "Page.vala"
	GTK_WIDGET_SET_FLAGS (event_source, GTK_CAN_FOCUS);
#line 809 "Page.c"
	/* interested in mouse button and motion events on the event source*/
#line 86 "Page.vala"
	gtk_widget_add_events (event_source, (gint) ((((GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK) | GDK_POINTER_MOTION_MASK) | GDK_POINTER_MOTION_HINT_MASK) | GDK_BUTTON_MOTION_MASK));
#line 89 "Page.vala"
	g_signal_connect_object (event_source, "button-press-event", (GCallback) _page_on_button_pressed_internal_gtk_widget_button_press_event, self, 0);
#line 90 "Page.vala"
	g_signal_connect_object (event_source, "button-release-event", (GCallback) _page_on_button_released_internal_gtk_widget_button_release_event, self, 0);
#line 91 "Page.vala"
	g_signal_connect_object (event_source, "motion-notify-event", (GCallback) _page_on_motion_internal_gtk_widget_motion_notify_event, self, 0);
#line 819 "Page.c"
}


#line 94 "Page.vala"
GtkWidget* page_get_event_source (Page* self) {
#line 825 "Page.c"
	GtkWidget* result;
	GtkWidget* _tmp0_;
#line 94 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), NULL);
#line 95 "Page.vala"
	_tmp0_ = NULL;
#line 832 "Page.c"
	result = (_tmp0_ = self->priv->event_source, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 98 "Page.vala"
void page_set_marker (Page* self, SidebarMarker* marker) {
#line 840 "Page.c"
	SidebarMarker* _tmp1_;
	SidebarMarker* _tmp0_;
#line 98 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 98 "Page.vala"
	g_return_if_fail (IS_SIDEBAR_MARKER (marker));
#line 847 "Page.c"
	_tmp1_ = NULL;
#line 99 "Page.vala"
	_tmp0_ = NULL;
#line 99 "Page.vala"
	self->priv->marker = (_tmp1_ = (_tmp0_ = marker, (_tmp0_ == NULL) ? NULL : sidebar_marker_ref (_tmp0_)), (self->priv->marker == NULL) ? NULL : (self->priv->marker = (sidebar_marker_unref (self->priv->marker), NULL)), _tmp1_);
#line 853 "Page.c"
}


#line 102 "Page.vala"
SidebarMarker* page_get_marker (Page* self) {
#line 859 "Page.c"
	SidebarMarker* result;
	SidebarMarker* _tmp0_;
#line 102 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), NULL);
#line 103 "Page.vala"
	_tmp0_ = NULL;
#line 866 "Page.c"
	result = (_tmp0_ = self->priv->marker, (_tmp0_ == NULL) ? NULL : sidebar_marker_ref (_tmp0_));
	return result;
}


#line 106 "Page.vala"
void page_clear_marker (Page* self) {
#line 874 "Page.c"
	SidebarMarker* _tmp0_;
#line 106 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 878 "Page.c"
	_tmp0_ = NULL;
#line 107 "Page.vala"
	self->priv->marker = (_tmp0_ = NULL, (self->priv->marker == NULL) ? NULL : (self->priv->marker = (sidebar_marker_unref (self->priv->marker), NULL)), _tmp0_);
#line 882 "Page.c"
}


#line 110 "Page.vala"
static GtkMenuBar* page_real_get_menubar (Page* self) {
#line 888 "Page.c"
	GtkMenuBar* result;
	GtkMenuBar* _tmp0_;
#line 110 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), NULL);
#line 111 "Page.vala"
	_tmp0_ = NULL;
#line 895 "Page.c"
	result = (_tmp0_ = self->priv->menu_bar, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 110 "Page.vala"
GtkMenuBar* page_get_menubar (Page* self) {
#line 110 "Page.vala"
	return PAGE_GET_CLASS (self)->get_menubar (self);
#line 905 "Page.c"
}


#line 114 "Page.vala"
static GtkToolbar* page_real_get_toolbar (Page* self) {
#line 911 "Page.c"
	g_return_val_if_fail (IS_PAGE (self), NULL);
	g_critical ("Type `%s' does not implement abstract method `page_get_toolbar'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


#line 114 "Page.vala"
GtkToolbar* page_get_toolbar (Page* self) {
#line 114 "Page.vala"
	return PAGE_GET_CLASS (self)->get_toolbar (self);
#line 922 "Page.c"
}


#line 116 "Page.vala"
static void page_real_switching_from (Page* self) {
#line 116 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 117 "Page.vala"
	self->priv->in_view = FALSE;
#line 932 "Page.c"
}


#line 116 "Page.vala"
void page_switching_from (Page* self) {
#line 116 "Page.vala"
	PAGE_GET_CLASS (self)->switching_from (self);
#line 940 "Page.c"
}


#line 120 "Page.vala"
static void page_real_switched_to (Page* self) {
#line 120 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 121 "Page.vala"
	self->priv->in_view = TRUE;
#line 950 "Page.c"
}


#line 120 "Page.vala"
void page_switched_to (Page* self) {
#line 120 "Page.vala"
	PAGE_GET_CLASS (self)->switched_to (self);
#line 958 "Page.c"
}


#line 124 "Page.vala"
gboolean page_is_in_view (Page* self) {
#line 964 "Page.c"
	gboolean result;
#line 124 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 968 "Page.c"
	result = self->priv->in_view;
	return result;
}


#line 128 "Page.vala"
static void page_real_switching_to_fullscreen (Page* self) {
#line 128 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 978 "Page.c"
}


#line 128 "Page.vala"
void page_switching_to_fullscreen (Page* self) {
#line 128 "Page.vala"
	PAGE_GET_CLASS (self)->switching_to_fullscreen (self);
#line 986 "Page.c"
}


#line 131 "Page.vala"
static void page_real_returning_from_fullscreen (Page* self) {
#line 131 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 994 "Page.c"
}


#line 131 "Page.vala"
void page_returning_from_fullscreen (Page* self) {
#line 131 "Page.vala"
	PAGE_GET_CLASS (self)->returning_from_fullscreen (self);
#line 1002 "Page.c"
}


#line 134 "Page.vala"
void page_set_item_sensitive (Page* self, const char* path, gboolean sensitive) {
#line 134 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 134 "Page.vala"
	g_return_if_fail (path != NULL);
#line 135 "Page.vala"
	g_object_set (gtk_ui_manager_get_widget (self->ui, path), "sensitive", sensitive, NULL);
#line 1014 "Page.c"
}


#line 138 "Page.vala"
void page_init_ui (Page* self, const char* ui_filename, const char* menubar_path, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1) {
#line 138 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 138 "Page.vala"
	g_return_if_fail (ui_filename != NULL);
#line 138 "Page.vala"
	g_return_if_fail (action_group_name != NULL);
#line 140 "Page.vala"
	page_init_ui_start (self, ui_filename, action_group_name, entries, entries_length1, toggle_entries, toggle_entries_length1);
#line 141 "Page.vala"
	page_init_ui_bind (self, menubar_path);
#line 1030 "Page.c"
}


#line 144 "Page.vala"
void page_init_load_ui (Page* self, const char* ui_filename) {
#line 1036 "Page.c"
	GError * _inner_error_;
	GFile* ui_file;
#line 144 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 144 "Page.vala"
	g_return_if_fail (ui_filename != NULL);
#line 1043 "Page.c"
	_inner_error_ = NULL;
	ui_file = resources_get_ui (ui_filename);
	{
		char* _tmp0_;
#line 952 "gio-2.0.vapi"
		_tmp0_ = NULL;
#line 148 "Page.vala"
		gtk_ui_manager_add_ui_from_file (self->ui, _tmp0_ = g_file_get_path (ui_file), &_inner_error_);
#line 1052 "Page.c"
		if (_inner_error_ != NULL) {
			goto __catch24_g_error;
			goto __finally24;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
	}
	goto __finally24;
	__catch24_g_error:
	{
		GError * gle;
		gle = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp1_;
#line 952 "gio-2.0.vapi"
			_tmp1_ = NULL;
#line 150 "Page.vala"
			g_error ("Page.vala:150: Error loading UI file %s: %s", _tmp1_ = g_file_get_path (ui_file), gle->message);
#line 1071 "Page.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
			(gle == NULL) ? NULL : (gle = (g_error_free (gle), NULL));
		}
	}
	__finally24:
	if (_inner_error_ != NULL) {
		(ui_file == NULL) ? NULL : (ui_file = (g_object_unref (ui_file), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(ui_file == NULL) ? NULL : (ui_file = (g_object_unref (ui_file), NULL));
}


#line 154 "Page.vala"
void page_init_ui_start (Page* self, const char* ui_filename, const char* action_group_name, GtkActionEntry* entries, int entries_length1, GtkToggleActionEntry* toggle_entries, int toggle_entries_length1) {
#line 1089 "Page.c"
	GtkActionGroup* _tmp0_;
#line 154 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 154 "Page.vala"
	g_return_if_fail (ui_filename != NULL);
#line 154 "Page.vala"
	g_return_if_fail (action_group_name != NULL);
#line 156 "Page.vala"
	page_init_load_ui (self, ui_filename);
#line 1099 "Page.c"
	_tmp0_ = NULL;
#line 158 "Page.vala"
	self->action_group = (_tmp0_ = gtk_action_group_new (action_group_name), (self->action_group == NULL) ? NULL : (self->action_group = (g_object_unref (self->action_group), NULL)), _tmp0_);
#line 159 "Page.vala"
	if (entries != NULL) {
#line 160 "Page.vala"
		gtk_action_group_add_actions (self->action_group, entries, entries_length1, self);
#line 1107 "Page.c"
	}
#line 161 "Page.vala"
	if (toggle_entries != NULL) {
#line 162 "Page.vala"
		gtk_action_group_add_toggle_actions (self->action_group, toggle_entries, toggle_entries_length1, self);
#line 1113 "Page.c"
	}
}


#line 165 "Page.vala"
void page_init_ui_bind (Page* self, const char* menubar_path) {
#line 1120 "Page.c"
	GtkActionGroup* _tmp1_;
	AppWindow* _tmp0_;
#line 165 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 166 "Page.vala"
	gtk_ui_manager_insert_action_group (self->ui, self->action_group, 0);
#line 1127 "Page.c"
	_tmp1_ = NULL;
#line 446 "AppWindow.vala"
	_tmp0_ = NULL;
#line 167 "Page.vala"
	self->common_action_group = (_tmp1_ = app_window_get_common_action_group (_tmp0_ = app_window_get_instance ()), (self->common_action_group == NULL) ? NULL : (self->common_action_group = (g_object_unref (self->common_action_group), NULL)), _tmp1_);
#line 1133 "Page.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 168 "Page.vala"
	gtk_ui_manager_insert_action_group (self->ui, self->common_action_group, 0);
#line 170 "Page.vala"
	if (menubar_path != NULL) {
#line 1139 "Page.c"
		GtkMenuBar* _tmp3_;
		GtkMenuBar* _tmp2_;
		_tmp3_ = NULL;
#line 171 "Page.vala"
		_tmp2_ = NULL;
#line 171 "Page.vala"
		self->priv->menu_bar = (_tmp3_ = (_tmp2_ = GTK_MENU_BAR (gtk_ui_manager_get_widget (self->ui, menubar_path)), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->menu_bar == NULL) ? NULL : (self->priv->menu_bar = (g_object_unref (self->priv->menu_bar), NULL)), _tmp3_);
#line 1147 "Page.c"
	}
#line 173 "Page.vala"
	gtk_ui_manager_ensure_update (self->ui);
#line 1151 "Page.c"
}


#line 214 "Page.vala"
static void _page_on_drag_begin_gtk_widget_drag_begin (GtkWidget* _sender, GdkDragContext* context, gpointer self) {
#line 1157 "Page.c"
	page_on_drag_begin (self, context);
}


#line 218 "Page.vala"
static void _page_on_drag_data_get_gtk_widget_drag_data_get (GtkWidget* _sender, GdkDragContext* context, GtkSelectionData* selection_data, guint info, guint time_, gpointer self) {
#line 1164 "Page.c"
	page_on_drag_data_get (self, context, selection_data, info, time_);
}


#line 223 "Page.vala"
static void _page_on_drag_data_delete_gtk_widget_drag_data_delete (GtkWidget* _sender, GdkDragContext* context, gpointer self) {
#line 1171 "Page.c"
	page_on_drag_data_delete (self, context);
}


#line 227 "Page.vala"
static void _page_on_drag_end_gtk_widget_drag_end (GtkWidget* _sender, GdkDragContext* context, gpointer self) {
#line 1178 "Page.c"
	page_on_drag_end (self, context);
}


#line 238 "Page.vala"
static gboolean _page_on_drag_failed_gtk_widget_drag_failed (GtkWidget* _sender, GdkDragContext* p0, GtkDragResult p1, gpointer self) {
#line 1185 "Page.c"
	return page_on_drag_failed (self, p0, p1);
}


/* This method enables drag-and-drop on the event source and routes its events through this
 object*/
#line 178 "Page.vala"
void page_enable_drag_source (Page* self, GdkDragAction actions) {
#line 178 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 179 "Page.vala"
	if (self->priv->dnd_enabled) {
#line 1198 "Page.c"
		return;
	}
#line 182 "Page.vala"
	g_assert (self->priv->event_source != NULL);
#line 184 "Page.vala"
	gtk_drag_source_set (self->priv->event_source, GDK_BUTTON1_MASK, PAGE_SOURCE_TARGET_ENTRIES, G_N_ELEMENTS (PAGE_SOURCE_TARGET_ENTRIES), actions);
#line 185 "Page.vala"
	g_signal_connect_object (self->priv->event_source, "drag-begin", (GCallback) _page_on_drag_begin_gtk_widget_drag_begin, self, 0);
#line 186 "Page.vala"
	g_signal_connect_object (self->priv->event_source, "drag-data-get", (GCallback) _page_on_drag_data_get_gtk_widget_drag_data_get, self, 0);
#line 187 "Page.vala"
	g_signal_connect_object (self->priv->event_source, "drag-data-delete", (GCallback) _page_on_drag_data_delete_gtk_widget_drag_data_delete, self, 0);
#line 188 "Page.vala"
	g_signal_connect_object (self->priv->event_source, "drag-end", (GCallback) _page_on_drag_end_gtk_widget_drag_end, self, 0);
#line 189 "Page.vala"
	g_signal_connect_object (self->priv->event_source, "drag-failed", (GCallback) _page_on_drag_failed_gtk_widget_drag_failed, self, 0);
#line 191 "Page.vala"
	self->priv->dnd_enabled = TRUE;
#line 1217 "Page.c"
}


#line 194 "Page.vala"
void page_disable_drag_source (Page* self) {
#line 1223 "Page.c"
	guint _tmp0_;
	guint _tmp1_;
	guint _tmp2_;
	guint _tmp3_;
	guint _tmp4_;
#line 194 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 195 "Page.vala"
	if (!self->priv->dnd_enabled) {
#line 1233 "Page.c"
		return;
	}
#line 198 "Page.vala"
	g_assert (self->priv->event_source != NULL);
#line 200 "Page.vala"
	g_signal_handlers_disconnect_matched (self->priv->event_source, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("drag-begin", GTK_TYPE_WIDGET, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _page_on_drag_begin_gtk_widget_drag_begin, self);
#line 201 "Page.vala"
	g_signal_handlers_disconnect_matched (self->priv->event_source, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("drag-data-get", GTK_TYPE_WIDGET, &_tmp1_, NULL, FALSE), _tmp1_), 0, NULL, (GCallback) _page_on_drag_data_get_gtk_widget_drag_data_get, self);
#line 202 "Page.vala"
	g_signal_handlers_disconnect_matched (self->priv->event_source, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("drag-data-delete", GTK_TYPE_WIDGET, &_tmp2_, NULL, FALSE), _tmp2_), 0, NULL, (GCallback) _page_on_drag_data_delete_gtk_widget_drag_data_delete, self);
#line 203 "Page.vala"
	g_signal_handlers_disconnect_matched (self->priv->event_source, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("drag-end", GTK_TYPE_WIDGET, &_tmp3_, NULL, FALSE), _tmp3_), 0, NULL, (GCallback) _page_on_drag_end_gtk_widget_drag_end, self);
#line 204 "Page.vala"
	g_signal_handlers_disconnect_matched (self->priv->event_source, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("drag-failed", GTK_TYPE_WIDGET, &_tmp4_, NULL, FALSE), _tmp4_), 0, NULL, (GCallback) _page_on_drag_failed_gtk_widget_drag_failed, self);
#line 205 "Page.vala"
	gtk_drag_source_unset (self->priv->event_source);
#line 207 "Page.vala"
	self->priv->dnd_enabled = FALSE;
#line 1252 "Page.c"
}


#line 210 "Page.vala"
gboolean page_is_dnd_enabled (Page* self) {
#line 1258 "Page.c"
	gboolean result;
#line 210 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1262 "Page.c"
	result = self->priv->dnd_enabled;
	return result;
}


#line 214 "Page.vala"
static void page_on_drag_begin (Page* self, GdkDragContext* context) {
#line 214 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 214 "Page.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 215 "Page.vala"
	g_signal_emit_by_name (GTK_WIDGET (self), "drag-begin", context);
#line 1276 "Page.c"
}


#line 218 "Page.vala"
static void page_on_drag_data_get (Page* self, GdkDragContext* context, GtkSelectionData* selection_data, guint info, guint time) {
#line 218 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 218 "Page.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 218 "Page.vala"
	g_return_if_fail (selection_data != NULL);
#line 220 "Page.vala"
	g_signal_emit_by_name (GTK_WIDGET (self), "drag-data-get", context, selection_data, info, time);
#line 1290 "Page.c"
}


#line 223 "Page.vala"
static void page_on_drag_data_delete (Page* self, GdkDragContext* context) {
#line 223 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 223 "Page.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 224 "Page.vala"
	g_signal_emit_by_name (GTK_WIDGET (self), "drag-data-delete", context);
#line 1302 "Page.c"
}


#line 227 "Page.vala"
static void page_on_drag_end (Page* self, GdkDragContext* context) {
#line 227 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 227 "Page.vala"
	g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
#line 228 "Page.vala"
	g_signal_emit_by_name (GTK_WIDGET (self), "drag-end", context);
#line 1314 "Page.c"
}


/* wierdly, Gtk 2.16.1 doesn't supply a drag_failed virtual method in the GtkWidget impl ...
 Vala binds to it, but it's not available in gtkwidget.h, and so gcc complains.  Have to
 makeshift one for now.*/
#line 234 "Page.vala"
static gboolean page_real_source_drag_failed (Page* self, GdkDragContext* context, GtkDragResult drag_result) {
#line 1323 "Page.c"
	gboolean result;
#line 234 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 234 "Page.vala"
	g_return_val_if_fail (GDK_IS_DRAG_CONTEXT (context), FALSE);
#line 1329 "Page.c"
	result = FALSE;
	return result;
}


#line 234 "Page.vala"
gboolean page_source_drag_failed (Page* self, GdkDragContext* context, GtkDragResult drag_result) {
#line 234 "Page.vala"
	return PAGE_GET_CLASS (self)->source_drag_failed (self, context, drag_result);
#line 1339 "Page.c"
}


#line 238 "Page.vala"
static gboolean page_on_drag_failed (Page* self, GdkDragContext* context, GtkDragResult drag_result) {
#line 1345 "Page.c"
	gboolean result;
#line 238 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 238 "Page.vala"
	g_return_val_if_fail (GDK_IS_DRAG_CONTEXT (context), FALSE);
#line 1351 "Page.c"
	result = page_source_drag_failed (self, context, drag_result);
	return result;
}


#line 242 "Page.vala"
static gboolean page_real_on_left_click (Page* self, const GdkEventButton* event) {
#line 1359 "Page.c"
	gboolean result;
#line 242 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1363 "Page.c"
	result = FALSE;
	return result;
}


#line 242 "Page.vala"
gboolean page_on_left_click (Page* self, const GdkEventButton* event) {
#line 242 "Page.vala"
	return PAGE_GET_CLASS (self)->on_left_click (self, event);
#line 1373 "Page.c"
}


#line 246 "Page.vala"
static gboolean page_real_on_middle_click (Page* self, const GdkEventButton* event) {
#line 1379 "Page.c"
	gboolean result;
#line 246 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1383 "Page.c"
	result = FALSE;
	return result;
}


#line 246 "Page.vala"
gboolean page_on_middle_click (Page* self, const GdkEventButton* event) {
#line 246 "Page.vala"
	return PAGE_GET_CLASS (self)->on_middle_click (self, event);
#line 1393 "Page.c"
}


#line 250 "Page.vala"
static gboolean page_real_on_right_click (Page* self, const GdkEventButton* event) {
#line 1399 "Page.c"
	gboolean result;
#line 250 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1403 "Page.c"
	result = FALSE;
	return result;
}


#line 250 "Page.vala"
gboolean page_on_right_click (Page* self, const GdkEventButton* event) {
#line 250 "Page.vala"
	return PAGE_GET_CLASS (self)->on_right_click (self, event);
#line 1413 "Page.c"
}


#line 254 "Page.vala"
static gboolean page_real_on_left_released (Page* self, const GdkEventButton* event) {
#line 1419 "Page.c"
	gboolean result;
#line 254 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1423 "Page.c"
	result = FALSE;
	return result;
}


#line 254 "Page.vala"
gboolean page_on_left_released (Page* self, const GdkEventButton* event) {
#line 254 "Page.vala"
	return PAGE_GET_CLASS (self)->on_left_released (self, event);
#line 1433 "Page.c"
}


#line 258 "Page.vala"
static gboolean page_real_on_middle_released (Page* self, const GdkEventButton* event) {
#line 1439 "Page.c"
	gboolean result;
#line 258 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1443 "Page.c"
	result = FALSE;
	return result;
}


#line 258 "Page.vala"
gboolean page_on_middle_released (Page* self, const GdkEventButton* event) {
#line 258 "Page.vala"
	return PAGE_GET_CLASS (self)->on_middle_released (self, event);
#line 1453 "Page.c"
}


#line 262 "Page.vala"
static gboolean page_real_on_right_released (Page* self, const GdkEventButton* event) {
#line 1459 "Page.c"
	gboolean result;
#line 262 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1463 "Page.c"
	result = FALSE;
	return result;
}


#line 262 "Page.vala"
gboolean page_on_right_released (Page* self, const GdkEventButton* event) {
#line 262 "Page.vala"
	return PAGE_GET_CLASS (self)->on_right_released (self, event);
#line 1473 "Page.c"
}


#line 266 "Page.vala"
static gboolean page_on_button_pressed_internal (Page* self, const GdkEventButton* event) {
#line 1479 "Page.c"
	gboolean result;
#line 266 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 267 "Page.vala"
	switch ((*event).button) {
#line 1485 "Page.c"
		case 1:
		{
#line 269 "Page.vala"
			if (self->priv->event_source != NULL) {
#line 270 "Page.vala"
				gtk_widget_grab_focus (self->priv->event_source);
#line 1492 "Page.c"
			}
			result = page_on_left_click (self, &(*event));
			return result;
		}
		case 2:
		{
			result = page_on_middle_click (self, &(*event));
			return result;
		}
		case 3:
		{
			result = page_on_right_click (self, &(*event));
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}


#line 285 "Page.vala"
static gboolean page_on_button_released_internal (Page* self, const GdkEventButton* event) {
#line 1518 "Page.c"
	gboolean result;
#line 285 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 286 "Page.vala"
	switch ((*event).button) {
#line 1524 "Page.c"
		case 1:
		{
			result = page_on_left_released (self, &(*event));
			return result;
		}
		case 2:
		{
			result = page_on_middle_released (self, &(*event));
			return result;
		}
		case 3:
		{
			result = page_on_right_released (self, &(*event));
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}


#line 301 "Page.vala"
static gboolean page_real_on_ctrl_pressed (Page* self, const GdkEventKey* event) {
#line 1551 "Page.c"
	gboolean result;
#line 301 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1555 "Page.c"
	result = FALSE;
	return result;
}


#line 301 "Page.vala"
gboolean page_on_ctrl_pressed (Page* self, const GdkEventKey* event) {
#line 301 "Page.vala"
	return PAGE_GET_CLASS (self)->on_ctrl_pressed (self, event);
#line 1565 "Page.c"
}


#line 305 "Page.vala"
static gboolean page_real_on_ctrl_released (Page* self, const GdkEventKey* event) {
#line 1571 "Page.c"
	gboolean result;
#line 305 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1575 "Page.c"
	result = FALSE;
	return result;
}


#line 305 "Page.vala"
gboolean page_on_ctrl_released (Page* self, const GdkEventKey* event) {
#line 305 "Page.vala"
	return PAGE_GET_CLASS (self)->on_ctrl_released (self, event);
#line 1585 "Page.c"
}


#line 309 "Page.vala"
static gboolean page_real_on_alt_pressed (Page* self, const GdkEventKey* event) {
#line 1591 "Page.c"
	gboolean result;
#line 309 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1595 "Page.c"
	result = FALSE;
	return result;
}


#line 309 "Page.vala"
gboolean page_on_alt_pressed (Page* self, const GdkEventKey* event) {
#line 309 "Page.vala"
	return PAGE_GET_CLASS (self)->on_alt_pressed (self, event);
#line 1605 "Page.c"
}


#line 313 "Page.vala"
static gboolean page_real_on_alt_released (Page* self, const GdkEventKey* event) {
#line 1611 "Page.c"
	gboolean result;
#line 313 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1615 "Page.c"
	result = FALSE;
	return result;
}


#line 313 "Page.vala"
gboolean page_on_alt_released (Page* self, const GdkEventKey* event) {
#line 313 "Page.vala"
	return PAGE_GET_CLASS (self)->on_alt_released (self, event);
#line 1625 "Page.c"
}


#line 317 "Page.vala"
static gboolean page_real_on_shift_pressed (Page* self, const GdkEventKey* event) {
#line 1631 "Page.c"
	gboolean result;
#line 317 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1635 "Page.c"
	result = FALSE;
	return result;
}


#line 317 "Page.vala"
gboolean page_on_shift_pressed (Page* self, const GdkEventKey* event) {
#line 317 "Page.vala"
	return PAGE_GET_CLASS (self)->on_shift_pressed (self, event);
#line 1645 "Page.c"
}


#line 321 "Page.vala"
static gboolean page_real_on_shift_released (Page* self, const GdkEventKey* event) {
#line 1651 "Page.c"
	gboolean result;
#line 321 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1655 "Page.c"
	result = FALSE;
	return result;
}


#line 321 "Page.vala"
gboolean page_on_shift_released (Page* self, const GdkEventKey* event) {
#line 321 "Page.vala"
	return PAGE_GET_CLASS (self)->on_shift_released (self, event);
#line 1665 "Page.c"
}


#line 325 "Page.vala"
gboolean page_notify_modifier_pressed (Page* self, const GdkEventKey* event) {
#line 1671 "Page.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 325 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1678 "Page.c"
	_tmp0_ = FALSE;
#line 328 "Page.vala"
	if ((*event).keyval == PAGE_KEY_CTRL_L) {
#line 328 "Page.vala"
		_tmp0_ = TRUE;
#line 1684 "Page.c"
	} else {
#line 328 "Page.vala"
		_tmp0_ = (*event).keyval == PAGE_KEY_CTRL_R;
#line 1688 "Page.c"
	}
	/* can't use a switch statement here due to this bug:
	 http://bugzilla.gnome.org/show_bug.cgi?id=585292*/
#line 328 "Page.vala"
	if (_tmp0_) {
#line 1694 "Page.c"
		result = page_on_ctrl_pressed (self, &(*event));
		return result;
	}
	_tmp1_ = FALSE;
#line 331 "Page.vala"
	if ((*event).keyval == PAGE_KEY_ALT_L) {
#line 331 "Page.vala"
		_tmp1_ = TRUE;
#line 1703 "Page.c"
	} else {
#line 331 "Page.vala"
		_tmp1_ = (*event).keyval == PAGE_KEY_ALT_R;
#line 1707 "Page.c"
	}
#line 331 "Page.vala"
	if (_tmp1_) {
#line 1711 "Page.c"
		result = page_on_alt_pressed (self, &(*event));
		return result;
	}
	_tmp2_ = FALSE;
#line 334 "Page.vala"
	if ((*event).keyval == PAGE_KEY_SHIFT_L) {
#line 334 "Page.vala"
		_tmp2_ = TRUE;
#line 1720 "Page.c"
	} else {
#line 334 "Page.vala"
		_tmp2_ = (*event).keyval == PAGE_KEY_SHIFT_R;
#line 1724 "Page.c"
	}
#line 334 "Page.vala"
	if (_tmp2_) {
#line 1728 "Page.c"
		result = page_on_shift_pressed (self, &(*event));
		return result;
	}
	result = FALSE;
	return result;
}


#line 340 "Page.vala"
gboolean page_notify_modifier_released (Page* self, const GdkEventKey* event) {
#line 1739 "Page.c"
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 340 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1746 "Page.c"
	_tmp0_ = FALSE;
#line 343 "Page.vala"
	if ((*event).keyval == PAGE_KEY_CTRL_L) {
#line 343 "Page.vala"
		_tmp0_ = TRUE;
#line 1752 "Page.c"
	} else {
#line 343 "Page.vala"
		_tmp0_ = (*event).keyval == PAGE_KEY_CTRL_R;
#line 1756 "Page.c"
	}
	/* can't use a switch statement here due to this bug:
	 http://bugzilla.gnome.org/show_bug.cgi?id=585292*/
#line 343 "Page.vala"
	if (_tmp0_) {
#line 1762 "Page.c"
		result = page_on_ctrl_released (self, &(*event));
		return result;
	}
	_tmp1_ = FALSE;
#line 346 "Page.vala"
	if ((*event).keyval == PAGE_KEY_ALT_L) {
#line 346 "Page.vala"
		_tmp1_ = TRUE;
#line 1771 "Page.c"
	} else {
#line 346 "Page.vala"
		_tmp1_ = (*event).keyval == PAGE_KEY_ALT_R;
#line 1775 "Page.c"
	}
#line 346 "Page.vala"
	if (_tmp1_) {
#line 1779 "Page.c"
		result = page_on_alt_released (self, &(*event));
		return result;
	}
	_tmp2_ = FALSE;
#line 349 "Page.vala"
	if ((*event).keyval == PAGE_KEY_SHIFT_L) {
#line 349 "Page.vala"
		_tmp2_ = TRUE;
#line 1788 "Page.c"
	} else {
#line 349 "Page.vala"
		_tmp2_ = (*event).keyval == PAGE_KEY_SHIFT_R;
#line 1792 "Page.c"
	}
#line 349 "Page.vala"
	if (_tmp2_) {
#line 1796 "Page.c"
		result = page_on_shift_released (self, &(*event));
		return result;
	}
	result = FALSE;
	return result;
}


#line 355 "Page.vala"
static void page_real_on_move (Page* self, const GdkRectangle* rect) {
#line 355 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 1809 "Page.c"
}


#line 355 "Page.vala"
void page_on_move (Page* self, const GdkRectangle* rect) {
#line 355 "Page.vala"
	PAGE_GET_CLASS (self)->on_move (self, rect);
#line 1817 "Page.c"
}


#line 358 "Page.vala"
static void page_real_on_resize (Page* self, const GdkRectangle* rect) {
#line 358 "Page.vala"
	g_return_if_fail (IS_PAGE (self));
#line 1825 "Page.c"
}


#line 358 "Page.vala"
void page_on_resize (Page* self, const GdkRectangle* rect) {
#line 358 "Page.vala"
	PAGE_GET_CLASS (self)->on_resize (self, rect);
#line 1833 "Page.c"
}


#line 361 "Page.vala"
static gboolean page_real_on_configure (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect) {
#line 1839 "Page.c"
	gboolean result;
#line 361 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1843 "Page.c"
	result = FALSE;
	return result;
}


#line 361 "Page.vala"
gboolean page_on_configure (Page* self, const GdkEventConfigure* event, const GdkRectangle* rect) {
#line 361 "Page.vala"
	return PAGE_GET_CLASS (self)->on_configure (self, event, rect);
#line 1853 "Page.c"
}


#line 365 "Page.vala"
gboolean page_notify_configure_event (Page* self, const GdkEventConfigure* event) {
#line 1859 "Page.c"
	gboolean result;
	GdkRectangle _tmp0_ = {0};
	GdkRectangle rect;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 365 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1867 "Page.c"
	rect = (memset (&_tmp0_, 0, sizeof (GdkRectangle)), _tmp0_);
#line 367 "Page.vala"
	rect.x = (*event).x;
#line 368 "Page.vala"
	rect.y = (*event).y;
#line 369 "Page.vala"
	rect.width = (*event).width;
#line 370 "Page.vala"
	rect.height = (*event).height;
#line 1877 "Page.c"
	_tmp1_ = FALSE;
#line 372 "Page.vala"
	if (self->priv->last_position.x != rect.x) {
#line 372 "Page.vala"
		_tmp1_ = TRUE;
#line 1883 "Page.c"
	} else {
#line 372 "Page.vala"
		_tmp1_ = self->priv->last_position.y != rect.y;
#line 1887 "Page.c"
	}
#line 372 "Page.vala"
	if (_tmp1_) {
#line 373 "Page.vala"
		page_on_move (self, &rect);
#line 1893 "Page.c"
	}
	_tmp2_ = FALSE;
#line 375 "Page.vala"
	if (self->priv->last_position.width != rect.width) {
#line 375 "Page.vala"
		_tmp2_ = TRUE;
#line 1900 "Page.c"
	} else {
#line 375 "Page.vala"
		_tmp2_ = self->priv->last_position.height != rect.height;
#line 1904 "Page.c"
	}
#line 375 "Page.vala"
	if (_tmp2_) {
#line 376 "Page.vala"
		page_on_resize (self, &rect);
#line 1910 "Page.c"
	}
#line 378 "Page.vala"
	self->priv->last_position = rect;
#line 1914 "Page.c"
	result = page_on_configure (self, &(*event), &rect);
	return result;
}


#line 383 "Page.vala"
static gboolean page_real_on_motion (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask) {
#line 1922 "Page.c"
	gboolean result;
#line 383 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1926 "Page.c"
	result = FALSE;
	return result;
}


#line 383 "Page.vala"
gboolean page_on_motion (Page* self, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask) {
#line 383 "Page.vala"
	return PAGE_GET_CLASS (self)->on_motion (self, event, x, y, mask);
#line 1936 "Page.c"
}


#line 387 "Page.vala"
static gboolean page_on_motion_internal (Page* self, const GdkEventMotion* event) {
#line 1942 "Page.c"
	gboolean result;
	gint x;
	gint y;
	GdkModifierType mask;
#line 387 "Page.vala"
	g_return_val_if_fail (IS_PAGE (self), FALSE);
#line 1949 "Page.c"
	x = 0;
	y = 0;
	mask = 0;
#line 390 "Page.vala"
	if ((*event).is_hint) {
#line 1955 "Page.c"
		GtkAdjustment* _tmp0_;
		GtkAdjustment* hadj;
		GtkAdjustment* _tmp1_;
		GtkAdjustment* vadj;
#line 391 "Page.vala"
		gdk_window_get_pointer (self->priv->event_source->window, &x, &y, &mask);
#line 393 "Page.vala"
		_tmp0_ = NULL;
#line 1964 "Page.c"
		hadj = (_tmp0_ = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 394 "Page.vala"
		_tmp1_ = NULL;
#line 1968 "Page.c"
		vadj = (_tmp1_ = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
		/* adjust x and y to viewport values*/
#line 397 "Page.vala"
		x = CLAMP (x + ((gint) gtk_adjustment_get_value (hadj)), (gint) gtk_adjustment_get_lower (hadj), (gint) gtk_adjustment_get_upper (hadj));
#line 398 "Page.vala"
		y = CLAMP (y + ((gint) gtk_adjustment_get_value (vadj)), (gint) gtk_adjustment_get_lower (vadj), (gint) gtk_adjustment_get_upper (vadj));
#line 1975 "Page.c"
		(hadj == NULL) ? NULL : (hadj = (g_object_unref (hadj), NULL));
		(vadj == NULL) ? NULL : (vadj = (g_object_unref (vadj), NULL));
	} else {
#line 400 "Page.vala"
		x = (gint) (*event).x;
#line 401 "Page.vala"
		y = (gint) (*event).y;
#line 402 "Page.vala"
		mask = (*event).state;
#line 1985 "Page.c"
	}
	result = page_on_motion (self, &(*event), x, y, mask);
	return result;
}


static void page_class_init (PageClass * klass) {
	page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (PagePrivate));
	PAGE_CLASS (klass)->get_menubar = page_real_get_menubar;
	PAGE_CLASS (klass)->get_toolbar = page_real_get_toolbar;
	PAGE_CLASS (klass)->switching_from = page_real_switching_from;
	PAGE_CLASS (klass)->switched_to = page_real_switched_to;
	PAGE_CLASS (klass)->switching_to_fullscreen = page_real_switching_to_fullscreen;
	PAGE_CLASS (klass)->returning_from_fullscreen = page_real_returning_from_fullscreen;
	PAGE_CLASS (klass)->source_drag_failed = page_real_source_drag_failed;
	PAGE_CLASS (klass)->on_left_click = page_real_on_left_click;
	PAGE_CLASS (klass)->on_middle_click = page_real_on_middle_click;
	PAGE_CLASS (klass)->on_right_click = page_real_on_right_click;
	PAGE_CLASS (klass)->on_left_released = page_real_on_left_released;
	PAGE_CLASS (klass)->on_middle_released = page_real_on_middle_released;
	PAGE_CLASS (klass)->on_right_released = page_real_on_right_released;
	PAGE_CLASS (klass)->on_ctrl_pressed = page_real_on_ctrl_pressed;
	PAGE_CLASS (klass)->on_ctrl_released = page_real_on_ctrl_released;
	PAGE_CLASS (klass)->on_alt_pressed = page_real_on_alt_pressed;
	PAGE_CLASS (klass)->on_alt_released = page_real_on_alt_released;
	PAGE_CLASS (klass)->on_shift_pressed = page_real_on_shift_pressed;
	PAGE_CLASS (klass)->on_shift_released = page_real_on_shift_released;
	PAGE_CLASS (klass)->on_move = page_real_on_move;
	PAGE_CLASS (klass)->on_resize = page_real_on_resize;
	PAGE_CLASS (klass)->on_configure = page_real_on_configure;
	PAGE_CLASS (klass)->on_motion = page_real_on_motion;
	G_OBJECT_CLASS (klass)->finalize = page_finalize;
}


static void page_instance_init (Page * self) {
	GdkRectangle _tmp0_ = {0};
	self->priv = PAGE_GET_PRIVATE (self);
	self->ui = gtk_ui_manager_new ();
	self->action_group = NULL;
	self->common_action_group = NULL;
	self->priv->layout = NULL;
	self->priv->menu_bar = NULL;
	self->priv->marker = NULL;
	self->priv->last_position = (memset (&_tmp0_, 0, sizeof (GdkRectangle)), _tmp0_);
	self->priv->event_source = NULL;
	self->priv->dnd_enabled = FALSE;
	self->priv->in_view = FALSE;
}


static void page_finalize (GObject* obj) {
	Page * self;
	self = PAGE (obj);
	(self->ui == NULL) ? NULL : (self->ui = (g_object_unref (self->ui), NULL));
	(self->action_group == NULL) ? NULL : (self->action_group = (g_object_unref (self->action_group), NULL));
	(self->common_action_group == NULL) ? NULL : (self->common_action_group = (g_object_unref (self->common_action_group), NULL));
	self->priv->page_name = (g_free (self->priv->page_name), NULL);
	(self->priv->layout == NULL) ? NULL : (self->priv->layout = (g_object_unref (self->priv->layout), NULL));
	(self->priv->menu_bar == NULL) ? NULL : (self->priv->menu_bar = (g_object_unref (self->priv->menu_bar), NULL));
	(self->priv->marker == NULL) ? NULL : (self->priv->marker = (sidebar_marker_unref (self->priv->marker), NULL));
	(self->priv->event_source == NULL) ? NULL : (self->priv->event_source = (g_object_unref (self->priv->event_source), NULL));
	G_OBJECT_CLASS (page_parent_class)->finalize (obj);
}


GType page_get_type (void) {
	static GType page_type_id = 0;
	if (page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Page), 0, (GInstanceInitFunc) page_instance_init, NULL };
		page_type_id = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, "Page", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return page_type_id;
}


#line 988 "Page.vala"
static void _checkerboard_page_on_layout_exposed_collection_layout_expose_after (CollectionLayout* _sender, gpointer self) {
#line 2065 "Page.c"
	checkerboard_page_on_layout_exposed (self);
}


#line 967 "Page.vala"
static void _checkerboard_page_on_layout_mapped_gtk_widget_map (CollectionLayout* _sender, gpointer self) {
#line 2072 "Page.c"
	checkerboard_page_on_layout_mapped (self);
}


#line 429 "Page.vala"
CheckerboardPage* checkerboard_page_construct (GType object_type, const char* page_name) {
#line 2079 "Page.c"
	CheckerboardPage * self;
#line 429 "Page.vala"
	g_return_val_if_fail (page_name != NULL, NULL);
#line 430 "Page.vala"
	self = (CheckerboardPage*) page_construct (object_type, page_name);
#line 432 "Page.vala"
	page_set_event_source (PAGE (self), GTK_WIDGET (self->priv->layout));
#line 433 "Page.vala"
	g_signal_connect_object (self->priv->layout, "expose-after", (GCallback) _checkerboard_page_on_layout_exposed_collection_layout_expose_after, self, 0);
#line 434 "Page.vala"
	g_signal_connect_object (GTK_WIDGET (self->priv->layout), "map", (GCallback) _checkerboard_page_on_layout_mapped_gtk_widget_map, self, 0);
#line 436 "Page.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->priv->layout));
#line 2093 "Page.c"
	return self;
}


#line 429 "Page.vala"
CheckerboardPage* checkerboard_page_new (const char* page_name) {
#line 429 "Page.vala"
	return checkerboard_page_construct (TYPE_CHECKERBOARD_PAGE, page_name);
#line 2102 "Page.c"
}


#line 439 "Page.vala"
void checkerboard_page_init_context_menu (CheckerboardPage* self, const char* path) {
#line 2108 "Page.c"
	GtkMenu* _tmp1_;
	GtkMenu* _tmp0_;
#line 439 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 439 "Page.vala"
	g_return_if_fail (path != NULL);
#line 2115 "Page.c"
	_tmp1_ = NULL;
#line 440 "Page.vala"
	_tmp0_ = NULL;
#line 440 "Page.vala"
	self->priv->context_menu = (_tmp1_ = (_tmp0_ = GTK_MENU (gtk_ui_manager_get_widget (PAGE (self)->ui, path)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->context_menu == NULL) ? NULL : (self->priv->context_menu = (g_object_unref (self->priv->context_menu), NULL)), _tmp1_);
#line 2121 "Page.c"
}


#line 443 "Page.vala"
static void checkerboard_page_real_on_selection_changed (CheckerboardPage* self, gint count) {
#line 443 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 2129 "Page.c"
}


#line 443 "Page.vala"
void checkerboard_page_on_selection_changed (CheckerboardPage* self, gint count) {
#line 443 "Page.vala"
	CHECKERBOARD_PAGE_GET_CLASS (self)->on_selection_changed (self, count);
#line 2137 "Page.c"
}


#line 446 "Page.vala"
static GtkMenu* checkerboard_page_real_get_context_menu (CheckerboardPage* self) {
#line 2143 "Page.c"
	GtkMenu* result;
	GtkMenu* _tmp0_;
#line 446 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 447 "Page.vala"
	_tmp0_ = NULL;
#line 2150 "Page.c"
	result = (_tmp0_ = self->priv->context_menu, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 446 "Page.vala"
GtkMenu* checkerboard_page_get_context_menu (CheckerboardPage* self) {
#line 446 "Page.vala"
	return CHECKERBOARD_PAGE_GET_CLASS (self)->get_context_menu (self);
#line 2160 "Page.c"
}


#line 450 "Page.vala"
static void checkerboard_page_real_on_item_activated (CheckerboardPage* self, LayoutItem* item) {
#line 450 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 450 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 2170 "Page.c"
}


#line 450 "Page.vala"
void checkerboard_page_on_item_activated (CheckerboardPage* self, LayoutItem* item) {
#line 450 "Page.vala"
	CHECKERBOARD_PAGE_GET_CLASS (self)->on_item_activated (self, item);
#line 2178 "Page.c"
}


#line 453 "Page.vala"
static gboolean checkerboard_page_real_on_context_invoked (CheckerboardPage* self, GtkMenu* context_menu) {
#line 2184 "Page.c"
	gboolean result;
#line 453 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), FALSE);
#line 453 "Page.vala"
	g_return_val_if_fail (GTK_IS_MENU (context_menu), FALSE);
#line 2190 "Page.c"
	result = TRUE;
	return result;
}


#line 453 "Page.vala"
gboolean checkerboard_page_on_context_invoked (CheckerboardPage* self, GtkMenu* context_menu) {
#line 453 "Page.vala"
	return CHECKERBOARD_PAGE_GET_CLASS (self)->on_context_invoked (self, context_menu);
#line 2200 "Page.c"
}


#line 457 "Page.vala"
static void checkerboard_page_real_switching_from (Page* base) {
#line 2206 "Page.c"
	CheckerboardPage * self;
	self = CHECKERBOARD_PAGE (base);
#line 458 "Page.vala"
	collection_layout_set_in_view (self->priv->layout, FALSE);
#line 460 "Page.vala"
	PAGE_CLASS (checkerboard_page_parent_class)->switching_from (PAGE (self));
#line 2213 "Page.c"
}


#line 463 "Page.vala"
static void checkerboard_page_real_switched_to (Page* base) {
#line 2219 "Page.c"
	CheckerboardPage * self;
	self = CHECKERBOARD_PAGE (base);
#line 464 "Page.vala"
	collection_layout_set_in_view (self->priv->layout, TRUE);
#line 466 "Page.vala"
	PAGE_CLASS (checkerboard_page_parent_class)->switched_to (PAGE (self));
#line 2226 "Page.c"
}


#line 469 "Page.vala"
static LayoutItem* checkerboard_page_real_get_fullscreen_photo (CheckerboardPage* self) {
#line 2232 "Page.c"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
	g_critical ("Type `%s' does not implement abstract method `checkerboard_page_get_fullscreen_photo'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


#line 469 "Page.vala"
LayoutItem* checkerboard_page_get_fullscreen_photo (CheckerboardPage* self) {
#line 469 "Page.vala"
	return CHECKERBOARD_PAGE_GET_CLASS (self)->get_fullscreen_photo (self);
#line 2243 "Page.c"
}


#line 471 "Page.vala"
void checkerboard_page_refresh (CheckerboardPage* self) {
#line 471 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 472 "Page.vala"
	gtk_widget_show_all (GTK_WIDGET (self));
#line 473 "Page.vala"
	if (GTK_WIDGET (self->priv->layout)->window != NULL) {
#line 474 "Page.vala"
		gdk_window_invalidate_rect (GTK_WIDGET (self->priv->layout)->window, NULL, TRUE);
#line 2257 "Page.c"
	}
#line 475 "Page.vala"
	collection_layout_refresh (self->priv->layout);
#line 2261 "Page.c"
}


#line 478 "Page.vala"
void checkerboard_page_set_page_message (CheckerboardPage* self, const char* message) {
#line 478 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 478 "Page.vala"
	g_return_if_fail (message != NULL);
#line 479 "Page.vala"
	collection_layout_set_message (self->priv->layout, message);
#line 2273 "Page.c"
}


#line 482 "Page.vala"
void checkerboard_page_set_refresh_on_resize (CheckerboardPage* self, gboolean refresh_on_resize) {
#line 482 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 483 "Page.vala"
	collection_layout_set_refresh_on_resize (self->priv->layout, refresh_on_resize);
#line 2283 "Page.c"
}


#line 486 "Page.vala"
void checkerboard_page_set_layout_comparator (CheckerboardPage* self, Comparator* cmp) {
#line 486 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 486 "Page.vala"
	g_return_if_fail (IS_COMPARATOR (cmp));
#line 487 "Page.vala"
	collection_layout_set_comparator (self->priv->layout, cmp);
#line 2295 "Page.c"
}


#line 490 "Page.vala"
LayoutItem* checkerboard_page_get_item_at_pixel (CheckerboardPage* self, double x, double y) {
#line 2301 "Page.c"
	LayoutItem* result;
#line 490 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 2305 "Page.c"
	result = collection_layout_get_item_at_pixel (self->priv->layout, x, y);
	return result;
}


#line 494 "Page.vala"
GeeIterable* checkerboard_page_get_items (CheckerboardPage* self) {
#line 2313 "Page.c"
	GeeIterable* result;
	GeeIterable* _tmp0_;
#line 494 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 495 "Page.vala"
	_tmp0_ = NULL;
#line 2320 "Page.c"
	result = (_tmp0_ = GEE_ITERABLE (self->priv->layout->items), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 498 "Page.vala"
GeeIterable* checkerboard_page_get_selected (CheckerboardPage* self) {
#line 2328 "Page.c"
	GeeIterable* result;
	GeeIterable* _tmp0_;
#line 498 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 499 "Page.vala"
	_tmp0_ = NULL;
#line 2335 "Page.c"
	result = (_tmp0_ = GEE_ITERABLE (self->priv->selected_items), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 502 "Page.vala"
void checkerboard_page_add_item (CheckerboardPage* self, LayoutItem* item) {
#line 502 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 502 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 503 "Page.vala"
	collection_layout_add_item (self->priv->layout, item);
#line 2349 "Page.c"
}


#line 506 "Page.vala"
void checkerboard_page_remove_item (CheckerboardPage* self, LayoutItem* item) {
#line 506 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 506 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 507 "Page.vala"
	gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), item);
#line 508 "Page.vala"
	collection_layout_remove_item (self->priv->layout, item);
#line 2363 "Page.c"
}


#line 511 "Page.vala"
gint checkerboard_page_remove_selected (CheckerboardPage* self) {
#line 2369 "Page.c"
	gint result;
	gint count;
#line 511 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), 0);
#line 2374 "Page.c"
	count = gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items));
	{
		GeeIterator* _item_it;
		_item_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 514 "Page.vala"
		while (TRUE) {
#line 2381 "Page.c"
			LayoutItem* item;
#line 514 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 514 "Page.vala"
				break;
#line 2387 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 515 "Page.vala"
			collection_layout_remove_item (self->priv->layout, item);
#line 2392 "Page.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 517 "Page.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 2399 "Page.c"
	result = count;
	return result;
}


#line 522 "Page.vala"
gint checkerboard_page_remove_all (CheckerboardPage* self) {
#line 2407 "Page.c"
	gint result;
	gint count;
#line 522 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), 0);
#line 2412 "Page.c"
	count = sorted_list_get_size (self->priv->layout->items);
#line 525 "Page.vala"
	collection_layout_clear (self->priv->layout);
#line 526 "Page.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 2418 "Page.c"
	result = count;
	return result;
}


#line 531 "Page.vala"
gint checkerboard_page_get_count (CheckerboardPage* self) {
#line 2426 "Page.c"
	gint result;
#line 531 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), 0);
#line 2430 "Page.c"
	result = sorted_list_get_size (self->priv->layout->items);
	return result;
}


#line 535 "Page.vala"
void checkerboard_page_select_all (CheckerboardPage* self) {
#line 2438 "Page.c"
	gboolean changed;
#line 535 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 2442 "Page.c"
	changed = FALSE;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->priv->layout->items));
#line 537 "Page.vala"
		while (TRUE) {
#line 2449 "Page.c"
			LayoutItem* item;
#line 537 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 537 "Page.vala"
				break;
#line 2455 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 538 "Page.vala"
			if (!layout_item_is_selected (item)) {
#line 539 "Page.vala"
				gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), item);
#line 540 "Page.vala"
				layout_item_select (item);
#line 541 "Page.vala"
				changed = TRUE;
#line 2466 "Page.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 545 "Page.vala"
	if (changed) {
#line 546 "Page.vala"
		checkerboard_page_on_selection_changed (self, gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)));
#line 2476 "Page.c"
	}
}


#line 549 "Page.vala"
void checkerboard_page_unselect_all (CheckerboardPage* self) {
#line 549 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 550 "Page.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)) == 0) {
#line 2487 "Page.c"
		return;
	}
	{
		GeeIterator* _item_it;
		_item_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 553 "Page.vala"
		while (TRUE) {
#line 2495 "Page.c"
			LayoutItem* item;
#line 553 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 553 "Page.vala"
				break;
#line 2501 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 554 "Page.vala"
			g_assert (layout_item_is_selected (item));
#line 555 "Page.vala"
			layout_item_unselect (item);
#line 2508 "Page.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 558 "Page.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 560 "Page.vala"
	checkerboard_page_on_selection_changed (self, 0);
#line 2517 "Page.c"
}


#line 563 "Page.vala"
void checkerboard_page_unselect_all_but (CheckerboardPage* self, LayoutItem* exception) {
#line 2523 "Page.c"
	gboolean changed;
#line 563 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 563 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (exception));
#line 564 "Page.vala"
	g_assert (layout_item_is_selected (exception));
#line 566 "Page.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)) == 0) {
#line 2533 "Page.c"
		return;
	}
	changed = FALSE;
	{
		GeeIterator* _item_it;
		_item_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 570 "Page.vala"
		while (TRUE) {
#line 2542 "Page.c"
			LayoutItem* item;
#line 570 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 570 "Page.vala"
				break;
#line 2548 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 571 "Page.vala"
			g_assert (layout_item_is_selected (item));
#line 572 "Page.vala"
			if (item != exception) {
#line 573 "Page.vala"
				layout_item_unselect (item);
#line 574 "Page.vala"
				changed = TRUE;
#line 2559 "Page.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
#line 578 "Page.vala"
	gee_abstract_collection_clear (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 579 "Page.vala"
	gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), exception);
#line 581 "Page.vala"
	if (changed) {
#line 582 "Page.vala"
		checkerboard_page_on_selection_changed (self, 1);
#line 2573 "Page.c"
	}
}


#line 585 "Page.vala"
void checkerboard_page_select (CheckerboardPage* self, LayoutItem* item) {
#line 585 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 585 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 586 "Page.vala"
	g_assert (sorted_list_contains (self->priv->layout->items, item));
#line 588 "Page.vala"
	if (!layout_item_is_selected (item)) {
#line 589 "Page.vala"
		layout_item_select (item);
#line 590 "Page.vala"
		gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), item);
#line 592 "Page.vala"
		checkerboard_page_on_selection_changed (self, gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)));
#line 2594 "Page.c"
	}
}


#line 596 "Page.vala"
void checkerboard_page_unselect (CheckerboardPage* self, LayoutItem* item) {
#line 596 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 596 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 597 "Page.vala"
	g_assert (sorted_list_contains (self->priv->layout->items, item));
#line 599 "Page.vala"
	if (layout_item_is_selected (item)) {
#line 600 "Page.vala"
		layout_item_unselect (item);
#line 601 "Page.vala"
		gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), item);
#line 603 "Page.vala"
		checkerboard_page_on_selection_changed (self, gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)));
#line 2615 "Page.c"
	}
}


#line 607 "Page.vala"
void checkerboard_page_toggle_select (CheckerboardPage* self, LayoutItem* item) {
#line 607 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 607 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 608 "Page.vala"
	if (layout_item_toggle_select (item)) {
#line 2628 "Page.c"
		/* now selected*/
#line 610 "Page.vala"
		gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), item);
#line 2632 "Page.c"
	} else {
		/* now unselected*/
#line 613 "Page.vala"
		gee_abstract_collection_remove (GEE_ABSTRACT_COLLECTION (self->priv->selected_items), item);
#line 2637 "Page.c"
	}
#line 616 "Page.vala"
	checkerboard_page_on_selection_changed (self, gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)));
#line 2641 "Page.c"
}


#line 619 "Page.vala"
gint checkerboard_page_get_selected_count (CheckerboardPage* self) {
#line 2647 "Page.c"
	gint result;
#line 619 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), 0);
#line 2651 "Page.c"
	result = gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items));
	return result;
}


#line 623 "Page.vala"
static gboolean checkerboard_page_real_key_press_event (GtkWidget* base, const GdkEventKey* event) {
#line 2659 "Page.c"
	CheckerboardPage * self;
	gboolean result;
	gboolean handled;
	GQuark _tmp3_;
	const char* _tmp2_;
	static GQuark _tmp3__label0 = 0;
	static GQuark _tmp3__label1 = 0;
	static GQuark _tmp3__label2 = 0;
	static GQuark _tmp3__label3 = 0;
	static GQuark _tmp3__label4 = 0;
	static GQuark _tmp3__label5 = 0;
	static GQuark _tmp3__label6 = 0;
	static GQuark _tmp3__label7 = 0;
	static GQuark _tmp3__label8 = 0;
	static GQuark _tmp3__label9 = 0;
	static GQuark _tmp3__label10 = 0;
	static GQuark _tmp3__label11 = 0;
	static GQuark _tmp3__label12 = 0;
	static GQuark _tmp3__label13 = 0;
	gboolean _tmp4_;
	self = CHECKERBOARD_PAGE (base);
	handled = TRUE;
#line 625 "Page.vala"
	_tmp2_ = NULL;
#line 2684 "Page.c"
	_tmp2_ = gdk_keyval_name ((*event).keyval);
	_tmp3_ = (NULL == _tmp2_) ? 0 : g_quark_from_string (_tmp2_);
	if ((_tmp3_ == ((0 != _tmp3__label0) ? _tmp3__label0 : (_tmp3__label0 = g_quark_from_static_string ("Up")))) || (_tmp3_ == ((0 != _tmp3__label1) ? _tmp3__label1 : (_tmp3__label1 = g_quark_from_static_string ("KP_Up")))))
	do {
#line 628 "Page.vala"
		checkerboard_page_move_cursor (self, COMPASS_POINT_NORTH);
#line 629 "Page.vala"
		break;
#line 2693 "Page.c"
	} while (0); else if ((_tmp3_ == ((0 != _tmp3__label2) ? _tmp3__label2 : (_tmp3__label2 = g_quark_from_static_string ("Down")))) || (_tmp3_ == ((0 != _tmp3__label3) ? _tmp3__label3 : (_tmp3__label3 = g_quark_from_static_string ("KP_Down")))))
	do {
#line 633 "Page.vala"
		checkerboard_page_move_cursor (self, COMPASS_POINT_SOUTH);
#line 634 "Page.vala"
		break;
#line 2700 "Page.c"
	} while (0); else if ((_tmp3_ == ((0 != _tmp3__label4) ? _tmp3__label4 : (_tmp3__label4 = g_quark_from_static_string ("Left")))) || (_tmp3_ == ((0 != _tmp3__label5) ? _tmp3__label5 : (_tmp3__label5 = g_quark_from_static_string ("KP_Left")))))
	do {
#line 638 "Page.vala"
		checkerboard_page_move_cursor (self, COMPASS_POINT_WEST);
#line 639 "Page.vala"
		break;
#line 2707 "Page.c"
	} while (0); else if ((_tmp3_ == ((0 != _tmp3__label6) ? _tmp3__label6 : (_tmp3__label6 = g_quark_from_static_string ("Right")))) || (_tmp3_ == ((0 != _tmp3__label7) ? _tmp3__label7 : (_tmp3__label7 = g_quark_from_static_string ("KP_Right")))))
	do {
#line 643 "Page.vala"
		checkerboard_page_move_cursor (self, COMPASS_POINT_EAST);
#line 644 "Page.vala"
		break;
#line 2714 "Page.c"
	} while (0); else if ((_tmp3_ == ((0 != _tmp3__label8) ? _tmp3__label8 : (_tmp3__label8 = g_quark_from_static_string ("Home")))) || (_tmp3_ == ((0 != _tmp3__label9) ? _tmp3__label9 : (_tmp3__label9 = g_quark_from_static_string ("KP_Home")))))
	do {
		LayoutItem* first;
		first = checkerboard_page_get_first_item (self);
#line 649 "Page.vala"
		if (first != NULL) {
#line 650 "Page.vala"
			checkerboard_page_cursor_to_item (self, first);
#line 2723 "Page.c"
		}
		(first == NULL) ? NULL : (first = (g_object_unref (first), NULL));
#line 651 "Page.vala"
		break;
#line 2728 "Page.c"
	} while (0); else if ((_tmp3_ == ((0 != _tmp3__label10) ? _tmp3__label10 : (_tmp3__label10 = g_quark_from_static_string ("End")))) || (_tmp3_ == ((0 != _tmp3__label11) ? _tmp3__label11 : (_tmp3__label11 = g_quark_from_static_string ("KP_End")))))
	do {
		LayoutItem* last;
		last = checkerboard_page_get_last_item (self);
#line 656 "Page.vala"
		if (last != NULL) {
#line 657 "Page.vala"
			checkerboard_page_cursor_to_item (self, last);
#line 2737 "Page.c"
		}
		(last == NULL) ? NULL : (last = (g_object_unref (last), NULL));
#line 658 "Page.vala"
		break;
#line 2742 "Page.c"
	} while (0); else if ((_tmp3_ == ((0 != _tmp3__label12) ? _tmp3__label12 : (_tmp3__label12 = g_quark_from_static_string ("Return")))) || (_tmp3_ == ((0 != _tmp3__label13) ? _tmp3__label13 : (_tmp3__label13 = g_quark_from_static_string ("KP_Enter")))))
	do {
#line 662 "Page.vala"
		if (checkerboard_page_get_selected_count (self) == 1) {
#line 2747 "Page.c"
			{
				GeeIterable* _tmp0_;
				GeeIterator* _tmp1_;
				GeeIterator* _item_it;
#line 498 "Page.vala"
				_tmp0_ = NULL;
#line 2754 "Page.c"
				_tmp1_ = NULL;
				_item_it = (_tmp1_ = gee_iterable_iterator (_tmp0_ = checkerboard_page_get_selected (self)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 663 "Page.vala"
				while (TRUE) {
#line 2759 "Page.c"
					LayoutItem* item;
#line 663 "Page.vala"
					if (!gee_iterator_next (_item_it)) {
#line 663 "Page.vala"
						break;
#line 2765 "Page.c"
					}
					item = (LayoutItem*) gee_iterator_get (_item_it);
#line 664 "Page.vala"
					checkerboard_page_on_item_activated (self, item);
#line 2770 "Page.c"
					(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
#line 666 "Page.vala"
					break;
#line 2774 "Page.c"
				}
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
			}
		} else {
#line 669 "Page.vala"
			handled = FALSE;
#line 2781 "Page.c"
		}
#line 671 "Page.vala"
		break;
#line 2785 "Page.c"
	} while (0); else
	do {
#line 674 "Page.vala"
		handled = FALSE;
#line 675 "Page.vala"
		break;
#line 2792 "Page.c"
	} while (0);
#line 678 "Page.vala"
	if (handled) {
#line 2796 "Page.c"
		result = TRUE;
		return result;
	}
	_tmp4_ = FALSE;
#line 681 "Page.vala"
	if (GTK_WIDGET_CLASS (checkerboard_page_parent_class)->key_press_event != NULL) {
#line 681 "Page.vala"
		_tmp4_ = GTK_WIDGET_CLASS (checkerboard_page_parent_class)->key_press_event (GTK_WIDGET (PAGE (self)), &(*event));
#line 2805 "Page.c"
	} else {
#line 681 "Page.vala"
		_tmp4_ = TRUE;
#line 2809 "Page.c"
	}
	result = _tmp4_;
	return result;
}


#line 684 "Page.vala"
static gboolean checkerboard_page_real_on_left_click (Page* base, const GdkEventButton* event) {
#line 2818 "Page.c"
	CheckerboardPage * self;
	gboolean result;
	gboolean _tmp0_;
	guint state;
	LayoutItem* item;
	LayoutItem* _tmp3_;
	LayoutItem* _tmp2_;
	self = CHECKERBOARD_PAGE (base);
	_tmp0_ = FALSE;
#line 686 "Page.vala"
	if ((*event).type != GDK_BUTTON_PRESS) {
#line 686 "Page.vala"
		_tmp0_ = (*event).type != GDK_2BUTTON_PRESS;
#line 2832 "Page.c"
	} else {
#line 686 "Page.vala"
		_tmp0_ = FALSE;
#line 2836 "Page.c"
	}
	/* only interested in single-click and double-clicks for now*/
#line 686 "Page.vala"
	if (_tmp0_) {
#line 2841 "Page.c"
		result = FALSE;
		return result;
	}
	/* mask out the modifiers we're interested in*/
	state = (*event).state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK);
	/* use clicks for multiple selection and activation only; single selects are handled by
	 button release, to allow for multiple items to be selected then dragged*/
	item = checkerboard_page_get_item_at_pixel (self, (*event).x, (*event).y);
#line 695 "Page.vala"
	if (item != NULL) {
#line 696 "Page.vala"
		switch (state) {
#line 2854 "Page.c"
			case GDK_CONTROL_MASK:
			{
#line 700 "Page.vala"
				checkerboard_page_toggle_select (self, item);
#line 701 "Page.vala"
				break;
#line 2861 "Page.c"
			}
			case GDK_SHIFT_MASK:
			{
				Box selected_box = {0};
				GdkPoint _tmp1_ = {0};
				GdkPoint point;
				Box new_selected_box;
#line 705 "Page.vala"
				if (!checkerboard_page_get_selected_box (self, &selected_box)) {
#line 706 "Page.vala"
					break;
#line 2873 "Page.c"
				}
				point = (memset (&_tmp1_, 0, sizeof (GdkPoint)), _tmp1_);
#line 709 "Page.vala"
				point.x = layout_item_get_column (item);
#line 710 "Page.vala"
				point.y = layout_item_get_row (item);
#line 2880 "Page.c"
				new_selected_box = box_rubber_band (&selected_box, &point);
#line 714 "Page.vala"
				checkerboard_page_select_all_in_box (self, &new_selected_box);
#line 715 "Page.vala"
				break;
#line 2886 "Page.c"
			}
			case GDK_CONTROL_MASK | GDK_SHIFT_MASK:
			{
#line 719 "Page.vala"
				break;
#line 2892 "Page.c"
			}
			default:
			{
#line 722 "Page.vala"
				if ((*event).type == GDK_2BUTTON_PRESS) {
#line 723 "Page.vala"
					checkerboard_page_on_item_activated (self, item);
#line 2900 "Page.c"
				} else {
					/* if user has selected multiple items and is preparing for a drag, don't
					 want to unselect immediately, otherwise, let the released handler deal
					 with it*/
#line 728 "Page.vala"
					if (checkerboard_page_get_selected_count (self) == 1) {
#line 729 "Page.vala"
						checkerboard_page_unselect_all (self);
#line 2909 "Page.c"
					}
#line 730 "Page.vala"
					checkerboard_page_select (self, item);
#line 2913 "Page.c"
				}
#line 732 "Page.vala"
				break;
#line 2917 "Page.c"
			}
		}
	} else {
		/* user clicked on "dead" area*/
#line 736 "Page.vala"
		checkerboard_page_unselect_all (self);
#line 2924 "Page.c"
	}
	_tmp3_ = NULL;
#line 739 "Page.vala"
	_tmp2_ = NULL;
#line 739 "Page.vala"
	self->priv->last_clicked_item = (_tmp3_ = (_tmp2_ = item, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->last_clicked_item == NULL) ? NULL : (self->priv->last_clicked_item = (g_object_unref (self->priv->last_clicked_item), NULL)), _tmp3_);
#line 2931 "Page.c"
	/* need to determine if the signal should be passed to the DnD handlers
	 Return true to block the DnD handler, false otherwise*/
#line 744 "Page.vala"
	if (item == NULL) {
#line 745 "Page.vala"
		self->priv->drag_select = TRUE;
#line 746 "Page.vala"
		self->priv->drag_start.x = (gint) (*event).x;
#line 747 "Page.vala"
		self->priv->drag_start.y = (gint) (*event).y;
#line 748 "Page.vala"
		self->priv->selection_band.width = 0;
#line 749 "Page.vala"
		self->priv->selection_band.height = 0;
#line 2946 "Page.c"
		result = TRUE;
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		return result;
	}
	result = gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)) == 0;
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	return result;
}


#line 757 "Page.vala"
static gboolean checkerboard_page_real_on_left_released (Page* base, const GdkEventButton* event) {
#line 2959 "Page.c"
	CheckerboardPage * self;
	gboolean result;
	LayoutItem* item;
	self = CHECKERBOARD_PAGE (base);
	/* if drag-selecting, stop here and do nothing else*/
#line 759 "Page.vala"
	if (self->priv->drag_select) {
#line 760 "Page.vala"
		self->priv->drag_select = FALSE;
#line 761 "Page.vala"
		self->priv->selection_band.width = 0;
#line 762 "Page.vala"
		self->priv->selection_band.height = 0;
#line 2973 "Page.c"
		/* force a repaint to remove the selection band*/
#line 765 "Page.vala"
		gdk_window_invalidate_rect (GTK_LAYOUT (self->priv->layout)->bin_window, NULL, FALSE);
#line 2977 "Page.c"
		result = TRUE;
		return result;
	}
	/* only interested in non-modified button releases*/
#line 771 "Page.vala"
	if (((*event).state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) != 0) {
#line 2984 "Page.c"
		result = FALSE;
		return result;
	}
	item = checkerboard_page_get_item_at_pixel (self, (*event).x, (*event).y);
#line 775 "Page.vala"
	if (item == NULL) {
#line 2991 "Page.c"
		/* released button on "dead" area*/
		result = TRUE;
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		return result;
	}
#line 780 "Page.vala"
	if (self->priv->last_clicked_item != item) {
#line 2999 "Page.c"
		/* user released mouse button after moving it off the initial item, or moved from dead
		 space onto one.  either way, unselect everything*/
#line 783 "Page.vala"
		checkerboard_page_unselect_all (self);
#line 3004 "Page.c"
	} else {
		/* the idea is, if a user single-clicks on an item with no modifiers, then all other items
		 should be deselected, however, if they single-click in order to drag one or more items,
		 they should remain selected, hence performing this here rather than on_left_click
		 (item may not be selected if an unimplemented modifier key was used)*/
#line 789 "Page.vala"
		if (layout_item_is_selected (item)) {
#line 790 "Page.vala"
			checkerboard_page_unselect_all_but (self, item);
#line 3014 "Page.c"
		}
	}
	result = TRUE;
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	return result;
}


#line 796 "Page.vala"
static gboolean checkerboard_page_real_on_right_click (Page* base, const GdkEventButton* event) {
#line 3025 "Page.c"
	CheckerboardPage * self;
	gboolean result;
	LayoutItem* item;
	GtkMenu* context_menu;
	self = CHECKERBOARD_PAGE (base);
	/* only interested in single-clicks for now*/
#line 798 "Page.vala"
	if ((*event).type != GDK_BUTTON_PRESS) {
#line 3034 "Page.c"
		result = FALSE;
		return result;
	}
	/* get what's right-clicked upon*/
	item = checkerboard_page_get_item_at_pixel (self, (*event).x, (*event).y);
#line 803 "Page.vala"
	if (item != NULL) {
#line 3042 "Page.c"
		/* mask out the modifiers we're interested in*/
#line 805 "Page.vala"
		switch ((*event).state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) {
#line 3046 "Page.c"
			case GDK_CONTROL_MASK:
			{
#line 808 "Page.vala"
				checkerboard_page_toggle_select (self, item);
#line 809 "Page.vala"
				break;
#line 3053 "Page.c"
			}
			case GDK_SHIFT_MASK:
			{
#line 813 "Page.vala"
				break;
#line 3059 "Page.c"
			}
			case GDK_CONTROL_MASK | GDK_SHIFT_MASK:
			{
#line 817 "Page.vala"
				break;
#line 3065 "Page.c"
			}
			default:
			{
#line 822 "Page.vala"
				if (!layout_item_is_selected (item)) {
#line 823 "Page.vala"
					checkerboard_page_unselect_all (self);
#line 824 "Page.vala"
					checkerboard_page_select (self, item);
#line 3075 "Page.c"
				}
#line 826 "Page.vala"
				break;
#line 3079 "Page.c"
			}
		}
	} else {
		/* clicked in "dead" space, unselect everything*/
#line 830 "Page.vala"
		checkerboard_page_unselect_all (self);
#line 3086 "Page.c"
	}
	context_menu = checkerboard_page_get_context_menu (self);
#line 834 "Page.vala"
	if (context_menu == NULL) {
#line 3091 "Page.c"
		result = FALSE;
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		(context_menu == NULL) ? NULL : (context_menu = (g_object_unref (context_menu), NULL));
		return result;
	}
#line 837 "Page.vala"
	if (!checkerboard_page_on_context_invoked (self, context_menu)) {
#line 3099 "Page.c"
		result = FALSE;
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		(context_menu == NULL) ? NULL : (context_menu = (g_object_unref (context_menu), NULL));
		return result;
	}
#line 840 "Page.vala"
	gtk_menu_popup (context_menu, NULL, NULL, NULL, NULL, (*event).button, (*event).time);
#line 3107 "Page.c"
	result = TRUE;
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	(context_menu == NULL) ? NULL : (context_menu = (g_object_unref (context_menu), NULL));
	return result;
}


#line 845 "Page.vala"
static gboolean checkerboard_page_real_on_mouse_over (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask) {
#line 3117 "Page.c"
	gboolean result;
	LayoutItem* _tmp2_;
	LayoutItem* _tmp1_;
#line 845 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), FALSE);
#line 845 "Page.vala"
	g_return_val_if_fail ((item == NULL) || IS_LAYOUT_ITEM (item), FALSE);
#line 3125 "Page.c"
	/* if hovering over the last hovered item, or both are null (nothing highlighted and
	 hovering over empty space), do nothing*/
#line 848 "Page.vala"
	if (item == self->priv->highlighted) {
#line 3130 "Page.c"
		result = TRUE;
		return result;
	}
	/* either something new is highlighted or now hovering over empty space, so dim old item*/
#line 852 "Page.vala"
	if (self->priv->highlighted != NULL) {
#line 3137 "Page.c"
		LayoutItem* _tmp0_;
#line 853 "Page.vala"
		layout_item_unbrighten (self->priv->highlighted);
#line 3141 "Page.c"
		_tmp0_ = NULL;
#line 854 "Page.vala"
		self->priv->highlighted = (_tmp0_ = NULL, (self->priv->highlighted == NULL) ? NULL : (self->priv->highlighted = (g_object_unref (self->priv->highlighted), NULL)), _tmp0_);
#line 3145 "Page.c"
	}
	/* if over empty space, done*/
#line 858 "Page.vala"
	if (item == NULL) {
#line 3150 "Page.c"
		result = TRUE;
		return result;
	}
	/* brighten the new item*/
#line 862 "Page.vala"
	layout_item_brighten (item);
#line 3157 "Page.c"
	_tmp2_ = NULL;
#line 863 "Page.vala"
	_tmp1_ = NULL;
#line 863 "Page.vala"
	self->priv->highlighted = (_tmp2_ = (_tmp1_ = item, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->highlighted == NULL) ? NULL : (self->priv->highlighted = (g_object_unref (self->priv->highlighted), NULL)), _tmp2_);
#line 3163 "Page.c"
	result = TRUE;
	return result;
}


#line 845 "Page.vala"
gboolean checkerboard_page_on_mouse_over (CheckerboardPage* self, LayoutItem* item, gint x, gint y, GdkModifierType mask) {
#line 845 "Page.vala"
	return CHECKERBOARD_PAGE_GET_CLASS (self)->on_mouse_over (self, item, x, y, mask);
#line 3173 "Page.c"
}


#line 920 "Page.vala"
static gboolean _checkerboard_page_selection_autoscroll_gsource_func (gpointer self) {
#line 3179 "Page.c"
	return checkerboard_page_selection_autoscroll (self);
}


#line 868 "Page.vala"
static gboolean checkerboard_page_real_on_motion (Page* base, const GdkEventMotion* event, gint x, gint y, GdkModifierType mask) {
#line 3186 "Page.c"
	CheckerboardPage * self;
	gboolean result;
	LayoutItem* _tmp0_;
	gboolean _tmp1_;
	GdkPoint _tmp2_ = {0};
	GdkPoint drag_end;
	Box _tmp3_ = {0};
	gboolean _tmp4_;
	self = CHECKERBOARD_PAGE (base);
	/* report what item the mouse is hovering over*/
#line 490 "Page.vala"
	_tmp0_ = NULL;
#line 870 "Page.vala"
	if ((_tmp1_ = !checkerboard_page_on_mouse_over (self, _tmp0_ = checkerboard_page_get_item_at_pixel (self, (double) x, (double) y), x, y, mask), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_)) {
#line 3201 "Page.c"
		result = FALSE;
		return result;
	}
	/* go no further if not drag-selecting*/
#line 874 "Page.vala"
	if (!self->priv->drag_select) {
#line 3208 "Page.c"
		result = FALSE;
		return result;
	}
	drag_end = (memset (&_tmp2_, 0, sizeof (GdkPoint)), _tmp2_);
#line 878 "Page.vala"
	drag_end.x = x;
#line 879 "Page.vala"
	drag_end.y = y;
#line 3217 "Page.c"
	/* save new drag rectangle*/
#line 882 "Page.vala"
	self->priv->selection_band = box_get_rectangle ((_tmp3_ = box_from_points (&self->priv->drag_start, &drag_end), &_tmp3_));
#line 884 "Page.vala"
	checkerboard_page_updated_selection_band (self);
#line 3223 "Page.c"
	_tmp4_ = FALSE;
#line 887 "Page.vala"
	if (!self->priv->autoscroll_scheduled) {
#line 888 "Page.vala"
		_tmp4_ = get_adjustment_relation (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)), y) != ADJUSTMENT_RELATION_IN_RANGE;
#line 3229 "Page.c"
	} else {
#line 887 "Page.vala"
		_tmp4_ = FALSE;
#line 3233 "Page.c"
	}
	/* if out of bounds, schedule a check to auto-scroll the viewport*/
#line 887 "Page.vala"
	if (_tmp4_) {
#line 889 "Page.vala"
		g_timeout_add ((guint) CHECKERBOARD_PAGE_AUTOSCROLL_TICKS_MSEC, _checkerboard_page_selection_autoscroll_gsource_func, self);
#line 890 "Page.vala"
		self->priv->autoscroll_scheduled = TRUE;
#line 3242 "Page.c"
	}
	result = TRUE;
	return result;
}


#line 896 "Page.vala"
static void checkerboard_page_updated_selection_band (CheckerboardPage* self) {
#line 3251 "Page.c"
	GeeList* intersection;
	GeeArrayList* outside;
#line 896 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 897 "Page.vala"
	g_assert (self->priv->drag_select);
#line 3258 "Page.c"
	/* get all items inside the selection*/
	intersection = collection_layout_intersection (self->priv->layout, &self->priv->selection_band);
	/* deselect everything not in the intersection ... needs to be done outside the iterator*/
	outside = gee_array_list_new (TYPE_LAYOUT_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	{
		GeeIterator* _item_it;
		_item_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 904 "Page.vala"
		while (TRUE) {
#line 3268 "Page.c"
			LayoutItem* item;
#line 904 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 904 "Page.vala"
				break;
#line 3274 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 905 "Page.vala"
			if (!gee_collection_contains (GEE_COLLECTION (intersection), item)) {
#line 906 "Page.vala"
				gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (outside), item);
#line 3281 "Page.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	{
		GeeIterator* _item_it;
		_item_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (outside));
#line 909 "Page.vala"
		while (TRUE) {
#line 3292 "Page.c"
			LayoutItem* item;
#line 909 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 909 "Page.vala"
				break;
#line 3298 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 910 "Page.vala"
			checkerboard_page_unselect (self, item);
#line 3303 "Page.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* select everything in the intersection*/
	{
		GeeIterator* _item_it;
		/* select everything in the intersection*/
		_item_it = gee_iterable_iterator (GEE_ITERABLE (intersection));
		/* select everything in the intersection*/
#line 913 "Page.vala"
		while (TRUE) {
#line 3316 "Page.c"
			LayoutItem* item;
			/* select everything in the intersection*/
#line 913 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 3321 "Page.c"
				/* select everything in the intersection*/
#line 913 "Page.vala"
				break;
#line 3325 "Page.c"
			}
			/* select everything in the intersection*/
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 914 "Page.vala"
			checkerboard_page_select (self, item);
#line 3331 "Page.c"
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	/* for a refresh to paint the selection band*/
#line 917 "Page.vala"
	gdk_window_invalidate_rect (GTK_LAYOUT (self->priv->layout)->bin_window, NULL, FALSE);
#line 3339 "Page.c"
	(intersection == NULL) ? NULL : (intersection = (g_object_unref (intersection), NULL));
	(outside == NULL) ? NULL : (outside = (g_object_unref (outside), NULL));
}


#line 920 "Page.vala"
static gboolean checkerboard_page_selection_autoscroll (CheckerboardPage* self) {
#line 3347 "Page.c"
	gboolean result;
	GtkAdjustment* _tmp0_;
	GtkAdjustment* vadj;
	gint x;
	gint y;
	GdkModifierType mask;
	gint new_value;
#line 920 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), FALSE);
#line 921 "Page.vala"
	if (!self->priv->drag_select) {
#line 922 "Page.vala"
		self->priv->autoscroll_scheduled = FALSE;
#line 3361 "Page.c"
		result = FALSE;
		return result;
	}
	/* as the viewport never scrolls horizontally, only interested in vertical*/
#line 928 "Page.vala"
	_tmp0_ = NULL;
#line 3368 "Page.c"
	vadj = (_tmp0_ = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	x = 0;
	y = 0;
	mask = 0;
#line 932 "Page.vala"
	gdk_window_get_pointer (GTK_LAYOUT (self->priv->layout)->bin_window, &x, &y, &mask);
#line 3375 "Page.c"
	new_value = (gint) gtk_adjustment_get_value (vadj);
#line 935 "Page.vala"
	switch (get_adjustment_relation (vadj, y)) {
#line 3379 "Page.c"
		case ADJUSTMENT_RELATION_BELOW:
		{
#line 937 "Page.vala"
			new_value = new_value - CHECKERBOARD_PAGE_AUTOSCROLL_PIXELS;
#line 938 "Page.vala"
			self->priv->selection_band.y = self->priv->selection_band.y - CHECKERBOARD_PAGE_AUTOSCROLL_PIXELS;
#line 939 "Page.vala"
			if (self->priv->selection_band.y < ((gint) gtk_adjustment_get_lower (vadj))) {
#line 940 "Page.vala"
				self->priv->selection_band.y = (gint) gtk_adjustment_get_lower (vadj);
#line 3390 "Page.c"
			}
#line 941 "Page.vala"
			break;
#line 3394 "Page.c"
		}
		case ADJUSTMENT_RELATION_ABOVE:
		{
#line 944 "Page.vala"
			new_value = new_value + CHECKERBOARD_PAGE_AUTOSCROLL_PIXELS;
#line 945 "Page.vala"
			self->priv->selection_band.height = self->priv->selection_band.height + CHECKERBOARD_PAGE_AUTOSCROLL_PIXELS;
#line 946 "Page.vala"
			if ((self->priv->selection_band.y + self->priv->selection_band.height) > ((gint) gtk_adjustment_get_upper (vadj))) {
#line 947 "Page.vala"
				self->priv->selection_band.height = ((gint) gtk_adjustment_get_upper (vadj)) - self->priv->selection_band.y;
#line 3406 "Page.c"
			}
#line 948 "Page.vala"
			break;
#line 3410 "Page.c"
		}
		case ADJUSTMENT_RELATION_IN_RANGE:
		{
#line 951 "Page.vala"
			self->priv->autoscroll_scheduled = FALSE;
#line 3416 "Page.c"
			result = FALSE;
			(vadj == NULL) ? NULL : (vadj = (g_object_unref (vadj), NULL));
			return result;
		}
		default:
		{
#line 956 "Page.vala"
			g_warn_if_reached ();
#line 957 "Page.vala"
			break;
#line 3427 "Page.c"
		}
	}
#line 960 "Page.vala"
	gtk_adjustment_set_value (vadj, (double) new_value);
#line 962 "Page.vala"
	checkerboard_page_updated_selection_band (self);
#line 3434 "Page.c"
	result = TRUE;
	(vadj == NULL) ? NULL : (vadj = (g_object_unref (vadj), NULL));
	return result;
}


#line 967 "Page.vala"
static void checkerboard_page_on_layout_mapped (CheckerboardPage* self) {
#line 3443 "Page.c"
	GdkColor selection_color;
	GdkGCValues _tmp0_ = {0};
	GdkGCValues gc_values;
	GdkGCValuesMask mask;
	GdkGC* _tmp1_;
#line 967 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 3451 "Page.c"
	/* set up selection colors*/
	selection_color = fetch_color (LAYOUT_ITEM_SELECTED_COLOR, GDK_DRAWABLE (GTK_LAYOUT (self->priv->layout)->bin_window));
#line 970 "Page.vala"
	self->priv->selection_transparency_color = convert_rgba (&selection_color, (guint8) 0x40);
#line 3456 "Page.c"
	/* set up GC's for painting selection*/
	gc_values = (memset (&_tmp0_, 0, sizeof (GdkGCValues)), _tmp0_);
#line 974 "Page.vala"
	gc_values.foreground = selection_color;
#line 975 "Page.vala"
	gc_values.function = GDK_COPY;
#line 976 "Page.vala"
	gc_values.fill = GDK_SOLID;
#line 977 "Page.vala"
	gc_values.line_width = 0;
#line 3467 "Page.c"
	mask = ((GDK_GC_FOREGROUND | GDK_GC_FUNCTION) | GDK_GC_FILL) | GDK_GC_LINE_WIDTH;
	_tmp1_ = NULL;
#line 985 "Page.vala"
	self->priv->selection_gc = (_tmp1_ = gdk_gc_new_with_values (GDK_DRAWABLE (GTK_LAYOUT (self->priv->layout)->bin_window), &gc_values, mask), (self->priv->selection_gc == NULL) ? NULL : (self->priv->selection_gc = (g_object_unref (self->priv->selection_gc), NULL)), _tmp1_);
#line 3472 "Page.c"
}


#line 988 "Page.vala"
static void checkerboard_page_on_layout_exposed (CheckerboardPage* self) {
#line 3478 "Page.c"
	gboolean _tmp0_;
	gint view_top;
	gint view_left;
	gint view_height;
	gint view_width;
	gint visible_x;
	gint visible_y;
	gint _tmp1_;
	gint visible_width;
	gint _tmp2_;
	gint visible_height;
	gboolean _tmp3_;
#line 988 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 3493 "Page.c"
	_tmp0_ = FALSE;
#line 990 "Page.vala"
	if (self->priv->selection_band.width <= 1) {
#line 990 "Page.vala"
		_tmp0_ = TRUE;
#line 3499 "Page.c"
	} else {
#line 990 "Page.vala"
		_tmp0_ = self->priv->selection_band.height <= 1;
#line 3503 "Page.c"
	}
	/* this method only used to draw selection rectangle*/
#line 990 "Page.vala"
	if (_tmp0_) {
#line 3508 "Page.c"
		return;
	}
#line 993 "Page.vala"
	g_assert (self->priv->selection_gc != NULL);
#line 3513 "Page.c"
	view_top = (gint) gtk_adjustment_get_value (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)));
	view_left = (gint) gtk_adjustment_get_value (gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self)));
	view_height = (gint) gtk_adjustment_get_page_size (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)));
	view_width = (gint) gtk_adjustment_get_page_size (gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self)));
	/* only interested in painting the visible selection interior*/
	visible_x = MAX (self->priv->selection_band.x, view_left);
	visible_y = MAX (self->priv->selection_band.y, view_top);
	_tmp1_ = 0;
#line 1004 "Page.vala"
	if (self->priv->selection_band.x >= view_left) {
#line 1004 "Page.vala"
		_tmp1_ = self->priv->selection_band.width;
#line 3526 "Page.c"
	} else {
#line 1005 "Page.vala"
		_tmp1_ = (self->priv->selection_band.x + self->priv->selection_band.width) - view_left;
#line 3530 "Page.c"
	}
	visible_width = _tmp1_;
#line 1006 "Page.vala"
	visible_width = CLAMP (visible_width, (gint) gtk_adjustment_get_lower (gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self))), view_width);
#line 3535 "Page.c"
	_tmp2_ = 0;
#line 1008 "Page.vala"
	if (self->priv->selection_band.y >= view_top) {
#line 1008 "Page.vala"
		_tmp2_ = self->priv->selection_band.height;
#line 3541 "Page.c"
	} else {
#line 1009 "Page.vala"
		_tmp2_ = (self->priv->selection_band.y + self->priv->selection_band.height) - view_top;
#line 3545 "Page.c"
	}
	visible_height = _tmp2_;
#line 1010 "Page.vala"
	visible_height = CLAMP (visible_height, (gint) gtk_adjustment_get_lower (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self))), view_height);
#line 3550 "Page.c"
	_tmp3_ = FALSE;
#line 1013 "Page.vala"
	if (visible_width > 1) {
#line 1013 "Page.vala"
		_tmp3_ = visible_height > 1;
#line 3556 "Page.c"
	} else {
#line 1013 "Page.vala"
		_tmp3_ = FALSE;
#line 3560 "Page.c"
	}
	/* pixelate selection rectangle interior*/
#line 1013 "Page.vala"
	if (_tmp3_) {
#line 3565 "Page.c"
		gboolean _tmp4_;
		gboolean _tmp5_;
		_tmp4_ = FALSE;
		_tmp5_ = FALSE;
#line 1017 "Page.vala"
		if (checkerboard_page_selection_interior == NULL) {
#line 1017 "Page.vala"
			_tmp5_ = TRUE;
#line 3574 "Page.c"
		} else {
#line 1017 "Page.vala"
			_tmp5_ = gdk_pixbuf_get_width (checkerboard_page_selection_interior) < visible_width;
#line 3578 "Page.c"
		}
#line 1017 "Page.vala"
		if (_tmp5_) {
#line 1017 "Page.vala"
			_tmp4_ = TRUE;
#line 3584 "Page.c"
		} else {
#line 1018 "Page.vala"
			_tmp4_ = gdk_pixbuf_get_height (checkerboard_page_selection_interior) < visible_height;
#line 3588 "Page.c"
		}
		/* generate a pixbuf of the selection color with a transparency to paint over the
		 visible selection area ... reuse old pixbuf (which is shared among all instances)
		 if possible*/
#line 1017 "Page.vala"
		if (_tmp4_) {
#line 3595 "Page.c"
			GdkPixbuf* _tmp6_;
			_tmp6_ = NULL;
#line 1019 "Page.vala"
			checkerboard_page_selection_interior = (_tmp6_ = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, visible_width, visible_height), (checkerboard_page_selection_interior == NULL) ? NULL : (checkerboard_page_selection_interior = (g_object_unref (checkerboard_page_selection_interior), NULL)), _tmp6_);
#line 1021 "Page.vala"
			gdk_pixbuf_fill (checkerboard_page_selection_interior, self->priv->selection_transparency_color);
#line 3602 "Page.c"
		}
#line 1024 "Page.vala"
		GDK_DRAWABLE_GET_CLASS (GDK_DRAWABLE (GTK_LAYOUT (self->priv->layout)->bin_window))->draw_pixbuf (GDK_DRAWABLE (GTK_LAYOUT (self->priv->layout)->bin_window), self->priv->selection_gc, checkerboard_page_selection_interior, 0, 0, visible_x, visible_y, visible_width, visible_height, GDK_RGB_DITHER_NORMAL, 0, 0);
#line 3606 "Page.c"
	}
	/* border*/
#line 1029 "Page.vala"
	gdk_draw_rectangle (GDK_DRAWABLE (GTK_LAYOUT (self->priv->layout)->bin_window), self->priv->selection_gc, FALSE, self->priv->selection_band.x, self->priv->selection_band.y, self->priv->selection_band.width - 1, self->priv->selection_band.height - 1);
#line 3611 "Page.c"
}


#line 1033 "Page.vala"
LayoutItem* checkerboard_page_get_first_item (CheckerboardPage* self) {
#line 3617 "Page.c"
	LayoutItem* result;
	LayoutItem* _tmp0_;
#line 1033 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 3622 "Page.c"
	_tmp0_ = NULL;
#line 1034 "Page.vala"
	if (sorted_list_get_size (self->priv->layout->items) != 0) {
#line 3626 "Page.c"
		LayoutItem* _tmp1_;
		_tmp1_ = NULL;
#line 1034 "Page.vala"
		_tmp0_ = (_tmp1_ = (LayoutItem*) sorted_list_get (self->priv->layout->items, 0), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 3631 "Page.c"
	} else {
		LayoutItem* _tmp2_;
		_tmp2_ = NULL;
#line 1034 "Page.vala"
		_tmp0_ = (_tmp2_ = NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
#line 3637 "Page.c"
	}
	result = _tmp0_;
	return result;
}


#line 1037 "Page.vala"
LayoutItem* checkerboard_page_get_last_item (CheckerboardPage* self) {
#line 3646 "Page.c"
	LayoutItem* result;
	LayoutItem* _tmp0_;
#line 1037 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 3651 "Page.c"
	_tmp0_ = NULL;
#line 1038 "Page.vala"
	if (sorted_list_get_size (self->priv->layout->items) != 0) {
#line 3655 "Page.c"
		LayoutItem* _tmp1_;
		_tmp1_ = NULL;
#line 1038 "Page.vala"
		_tmp0_ = (_tmp1_ = (LayoutItem*) sorted_list_get (self->priv->layout->items, sorted_list_get_size (self->priv->layout->items) - 1), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
#line 3660 "Page.c"
	} else {
		LayoutItem* _tmp2_;
		_tmp2_ = NULL;
#line 1038 "Page.vala"
		_tmp0_ = (_tmp2_ = NULL, (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp2_);
#line 3666 "Page.c"
	}
	result = _tmp0_;
	return result;
}


#line 1041 "Page.vala"
LayoutItem* checkerboard_page_get_next_item (CheckerboardPage* self, LayoutItem* current) {
#line 3675 "Page.c"
	LayoutItem* result;
	gint index;
#line 1041 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 1041 "Page.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (current), NULL);
#line 1042 "Page.vala"
	if (sorted_list_get_size (self->priv->layout->items) == 0) {
#line 3684 "Page.c"
		result = NULL;
		return result;
	}
	index = sorted_list_index_of (self->priv->layout->items, current);
	/* although items may be added while the page is away, not handling situations where an active
	 item is removed*/
#line 1049 "Page.vala"
	g_assert (index >= 0);
#line 1051 "Page.vala"
	index++;
#line 1052 "Page.vala"
	if (index >= sorted_list_get_size (self->priv->layout->items)) {
#line 1053 "Page.vala"
		index = 0;
#line 3699 "Page.c"
	}
	result = (LayoutItem*) sorted_list_get (self->priv->layout->items, index);
	return result;
}


#line 1058 "Page.vala"
LayoutItem* checkerboard_page_get_previous_item (CheckerboardPage* self, LayoutItem* current) {
#line 3708 "Page.c"
	LayoutItem* result;
	gint index;
#line 1058 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), NULL);
#line 1058 "Page.vala"
	g_return_val_if_fail (IS_LAYOUT_ITEM (current), NULL);
#line 1059 "Page.vala"
	if (sorted_list_get_size (self->priv->layout->items) == 0) {
#line 3717 "Page.c"
		result = NULL;
		return result;
	}
	index = sorted_list_index_of (self->priv->layout->items, current);
	/* although items may be added while the page is away, not handling situations where an active
	 item is removed*/
#line 1066 "Page.vala"
	g_assert (index >= 0);
#line 1068 "Page.vala"
	index--;
#line 1069 "Page.vala"
	if (index < 0) {
#line 1070 "Page.vala"
		index = sorted_list_get_size (self->priv->layout->items) - 1;
#line 3732 "Page.c"
	}
	result = (LayoutItem*) sorted_list_get (self->priv->layout->items, index);
	return result;
}


#line 1075 "Page.vala"
void checkerboard_page_cursor_to_item (CheckerboardPage* self, LayoutItem* item) {
#line 3741 "Page.c"
	GtkAdjustment* _tmp0_;
	GtkAdjustment* vadj;
	gboolean _tmp1_;
	gint top;
#line 1075 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 1075 "Page.vala"
	g_return_if_fail (IS_LAYOUT_ITEM (item));
#line 1076 "Page.vala"
	g_assert (sorted_list_contains (self->priv->layout->items, item));
#line 1078 "Page.vala"
	checkerboard_page_unselect_all (self);
#line 1079 "Page.vala"
	checkerboard_page_select (self, item);
#line 3756 "Page.c"
	/* if item is in any way out of view, scroll to it*/
#line 1082 "Page.vala"
	_tmp0_ = NULL;
#line 3760 "Page.c"
	vadj = (_tmp0_ = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp1_ = FALSE;
#line 1083 "Page.vala"
	if (get_adjustment_relation (vadj, GTK_WIDGET (item)->allocation.y) == ADJUSTMENT_RELATION_IN_RANGE) {
#line 1084 "Page.vala"
		_tmp1_ = get_adjustment_relation (vadj, GTK_WIDGET (item)->allocation.y + GTK_WIDGET (item)->allocation.height) == ADJUSTMENT_RELATION_IN_RANGE;
#line 3767 "Page.c"
	} else {
#line 1083 "Page.vala"
		_tmp1_ = FALSE;
#line 3771 "Page.c"
	}
#line 1083 "Page.vala"
	if (_tmp1_) {
#line 3775 "Page.c"
		(vadj == NULL) ? NULL : (vadj = (g_object_unref (vadj), NULL));
		return;
	}
	/* scroll to see the new item*/
	top = 0;
#line 1089 "Page.vala"
	if (GTK_WIDGET (item)->allocation.y < gtk_adjustment_get_value (vadj)) {
#line 1090 "Page.vala"
		top = GTK_WIDGET (item)->allocation.y;
#line 1091 "Page.vala"
		top = top - (COLLECTION_LAYOUT_ROW_GUTTER_PADDING / 2);
#line 3787 "Page.c"
	} else {
#line 1093 "Page.vala"
		top = (GTK_WIDGET (item)->allocation.y + GTK_WIDGET (item)->allocation.height) - ((gint) gtk_adjustment_get_page_size (vadj));
#line 1094 "Page.vala"
		top = top + (COLLECTION_LAYOUT_ROW_GUTTER_PADDING / 2);
#line 3793 "Page.c"
	}
#line 1097 "Page.vala"
	gtk_adjustment_set_value (vadj, (double) top);
#line 3797 "Page.c"
	(vadj == NULL) ? NULL : (vadj = (g_object_unref (vadj), NULL));
}


#line 1100 "Page.vala"
void checkerboard_page_move_cursor (CheckerboardPage* self, CompassPoint point) {
#line 3804 "Page.c"
	gint first_col;
	gint first_row;
	LayoutItem* item;
	LayoutItem* _tmp1_;
#line 1100 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 3811 "Page.c"
	/* if nothing is selected, simply select the first and exit*/
#line 1102 "Page.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)) == 0) {
#line 3815 "Page.c"
		LayoutItem* _tmp0_;
#line 184 "CollectionLayout.vala"
		_tmp0_ = NULL;
#line 1103 "Page.vala"
		checkerboard_page_cursor_to_item (self, _tmp0_ = (LayoutItem*) sorted_list_get (self->priv->layout->items, 0));
#line 3821 "Page.c"
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
		return;
	}
	/* find the "first" selected item, which for now is the topmost, leftmost item in the layout
	 TODO: Revisit if this is the right choice.*/
	first_col = G_MAXINT;
	first_row = G_MAXINT;
	{
		GeeIterator* _selected_it;
		_selected_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 1112 "Page.vala"
		while (TRUE) {
#line 3834 "Page.c"
			LayoutItem* selected;
#line 1112 "Page.vala"
			if (!gee_iterator_next (_selected_it)) {
#line 1112 "Page.vala"
				break;
#line 3840 "Page.c"
			}
			selected = (LayoutItem*) gee_iterator_get (_selected_it);
#line 1113 "Page.vala"
			first_col = MIN (layout_item_get_column (selected), first_col);
#line 1114 "Page.vala"
			first_row = MIN (layout_item_get_row (selected), first_row);
#line 3847 "Page.c"
			(selected == NULL) ? NULL : (selected = (g_object_unref (selected), NULL));
		}
		(_selected_it == NULL) ? NULL : (_selected_it = (g_object_unref (_selected_it), NULL));
	}
	item = collection_layout_get_item_at_coordinate (self->priv->layout, first_col, first_row);
#line 1118 "Page.vala"
	g_assert (item != NULL);
#line 3855 "Page.c"
	/* if more than one selected, select the first without moving, to not surprise the user*/
#line 1121 "Page.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)) > 1) {
#line 1122 "Page.vala"
		checkerboard_page_cursor_to_item (self, item);
#line 3861 "Page.c"
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		return;
	}
	_tmp1_ = NULL;
#line 1127 "Page.vala"
	item = (_tmp1_ = collection_layout_get_item_relative_to (self->priv->layout, item, point), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), _tmp1_);
#line 1128 "Page.vala"
	if (item != NULL) {
#line 1129 "Page.vala"
		checkerboard_page_cursor_to_item (self, item);
#line 3872 "Page.c"
	}
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


#line 1132 "Page.vala"
gboolean checkerboard_page_get_selected_box (CheckerboardPage* self, Box* selected_box) {
#line 3880 "Page.c"
	gboolean result;
	gint left;
	gint top;
	gint right;
	gint bottom;
	Box _tmp0_ = {0};
#line 1132 "Page.vala"
	g_return_val_if_fail (IS_CHECKERBOARD_PAGE (self), FALSE);
#line 1133 "Page.vala"
	if (gee_collection_get_size (GEE_COLLECTION (self->priv->selected_items)) == 0) {
#line 3891 "Page.c"
		result = FALSE;
		return result;
	}
	left = G_MAXINT;
	top = G_MAXINT;
	right = G_MININT;
	bottom = G_MININT;
	{
		GeeIterator* _selected_it;
		_selected_it = gee_abstract_collection_iterator (GEE_ABSTRACT_COLLECTION (self->priv->selected_items));
#line 1140 "Page.vala"
		while (TRUE) {
#line 3904 "Page.c"
			LayoutItem* selected;
#line 1140 "Page.vala"
			if (!gee_iterator_next (_selected_it)) {
#line 1140 "Page.vala"
				break;
#line 3910 "Page.c"
			}
			selected = (LayoutItem*) gee_iterator_get (_selected_it);
#line 1141 "Page.vala"
			left = MIN (layout_item_get_column (selected), left);
#line 1142 "Page.vala"
			top = MIN (layout_item_get_row (selected), top);
#line 1143 "Page.vala"
			right = MAX (layout_item_get_column (selected), right);
#line 1144 "Page.vala"
			bottom = MAX (layout_item_get_row (selected), bottom);
#line 3921 "Page.c"
			(selected == NULL) ? NULL : (selected = (g_object_unref (selected), NULL));
		}
		(_selected_it == NULL) ? NULL : (_selected_it = (g_object_unref (_selected_it), NULL));
	}
#line 1147 "Page.vala"
	(*selected_box) = (box_init (&_tmp0_, left, top, right, bottom), _tmp0_);
#line 3928 "Page.c"
	result = TRUE;
	return result;
}


#line 1152 "Page.vala"
void checkerboard_page_select_all_in_box (CheckerboardPage* self, const Box* box) {
#line 3936 "Page.c"
	GdkPoint _tmp0_ = {0};
	GdkPoint point;
#line 1152 "Page.vala"
	g_return_if_fail (IS_CHECKERBOARD_PAGE (self));
#line 3941 "Page.c"
	point = (memset (&_tmp0_, 0, sizeof (GdkPoint)), _tmp0_);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator (GEE_ITERABLE (self->priv->layout->items));
#line 1154 "Page.vala"
		while (TRUE) {
#line 3948 "Page.c"
			LayoutItem* item;
#line 1154 "Page.vala"
			if (!gee_iterator_next (_item_it)) {
#line 1154 "Page.vala"
				break;
#line 3954 "Page.c"
			}
			item = (LayoutItem*) gee_iterator_get (_item_it);
#line 1155 "Page.vala"
			point.x = layout_item_get_column (item);
#line 1156 "Page.vala"
			point.y = layout_item_get_row (item);
#line 1157 "Page.vala"
			if (box_contains (&(*box), &point)) {
#line 1158 "Page.vala"
				checkerboard_page_select (self, item);
#line 3965 "Page.c"
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
}


static void checkerboard_page_class_init (CheckerboardPageClass * klass) {
	checkerboard_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (CheckerboardPagePrivate));
	CHECKERBOARD_PAGE_CLASS (klass)->on_selection_changed = checkerboard_page_real_on_selection_changed;
	CHECKERBOARD_PAGE_CLASS (klass)->get_context_menu = checkerboard_page_real_get_context_menu;
	CHECKERBOARD_PAGE_CLASS (klass)->on_item_activated = checkerboard_page_real_on_item_activated;
	CHECKERBOARD_PAGE_CLASS (klass)->on_context_invoked = checkerboard_page_real_on_context_invoked;
	PAGE_CLASS (klass)->switching_from = checkerboard_page_real_switching_from;
	PAGE_CLASS (klass)->switched_to = checkerboard_page_real_switched_to;
	CHECKERBOARD_PAGE_CLASS (klass)->get_fullscreen_photo = checkerboard_page_real_get_fullscreen_photo;
	GTK_WIDGET_CLASS (klass)->key_press_event = checkerboard_page_real_key_press_event;
	PAGE_CLASS (klass)->on_left_click = checkerboard_page_real_on_left_click;
	PAGE_CLASS (klass)->on_left_released = checkerboard_page_real_on_left_released;
	PAGE_CLASS (klass)->on_right_click = checkerboard_page_real_on_right_click;
	CHECKERBOARD_PAGE_CLASS (klass)->on_mouse_over = checkerboard_page_real_on_mouse_over;
	PAGE_CLASS (klass)->on_motion = checkerboard_page_real_on_motion;
	G_OBJECT_CLASS (klass)->finalize = checkerboard_page_finalize;
}


static void checkerboard_page_instance_init (CheckerboardPage * self) {
	GdkPoint _tmp1_ = {0};
	self->priv = CHECKERBOARD_PAGE_GET_PRIVATE (self);
	self->priv->context_menu = NULL;
	self->priv->layout = g_object_ref_sink (collection_layout_new ());
	self->priv->selected_items = gee_hash_set_new (TYPE_LAYOUT_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_hash, g_direct_equal);
	self->priv->last_clicked_item = NULL;
	self->priv->highlighted = NULL;
	self->priv->drag_select = FALSE;
	self->priv->drag_start = (memset (&_tmp1_, 0, sizeof (GdkPoint)), _tmp1_);
	self->priv->selection_gc = NULL;
	self->priv->selection_transparency_color = (guint32) 0;
	self->priv->autoscroll_scheduled = FALSE;
}


static void checkerboard_page_finalize (GObject* obj) {
	CheckerboardPage * self;
	self = CHECKERBOARD_PAGE (obj);
	(self->priv->context_menu == NULL) ? NULL : (self->priv->context_menu = (g_object_unref (self->priv->context_menu), NULL));
	(self->priv->layout == NULL) ? NULL : (self->priv->layout = (g_object_unref (self->priv->layout), NULL));
	(self->priv->selected_items == NULL) ? NULL : (self->priv->selected_items = (g_object_unref (self->priv->selected_items), NULL));
	(self->priv->last_clicked_item == NULL) ? NULL : (self->priv->last_clicked_item = (g_object_unref (self->priv->last_clicked_item), NULL));
	(self->priv->highlighted == NULL) ? NULL : (self->priv->highlighted = (g_object_unref (self->priv->highlighted), NULL));
	(self->priv->selection_gc == NULL) ? NULL : (self->priv->selection_gc = (g_object_unref (self->priv->selection_gc), NULL));
	G_OBJECT_CLASS (checkerboard_page_parent_class)->finalize (obj);
}


GType checkerboard_page_get_type (void) {
	static GType checkerboard_page_type_id = 0;
	if (checkerboard_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CheckerboardPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) checkerboard_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CheckerboardPage), 0, (GInstanceInitFunc) checkerboard_page_instance_init, NULL };
		checkerboard_page_type_id = g_type_register_static (TYPE_PAGE, "CheckerboardPage", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return checkerboard_page_type_id;
}



GType single_photo_page_update_reason_get_type (void) {
	static GType single_photo_page_update_reason_type_id = 0;
	if (G_UNLIKELY (single_photo_page_update_reason_type_id == 0)) {
		static const GEnumValue values[] = {{SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO, "SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO", "new-photo"}, {SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT, "SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT", "quality-improvement"}, {SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS, "SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS", "resized-canvas"}, {0, NULL, NULL}};
		single_photo_page_update_reason_type_id = g_enum_register_static ("SinglePhotoPageUpdateReason", values);
	}
	return single_photo_page_update_reason_type_id;
}


#line 1261 "Page.vala"
static void _single_photo_page_on_viewport_resize_gtk_widget_size_allocate (GtkViewport* _sender, const GdkRectangle* allocation, gpointer self) {
#line 4046 "Page.c"
	single_photo_page_on_viewport_resize (self);
}


#line 1265 "Page.vala"
static gboolean _single_photo_page_on_canvas_exposed_gtk_widget_expose_event (GtkDrawingArea* _sender, const GdkEventExpose* event, gpointer self) {
#line 4053 "Page.c"
	return single_photo_page_on_canvas_exposed (self, event);
}


#line 1187 "Page.vala"
SinglePhotoPage* single_photo_page_construct (GType object_type, const char* page_name) {
#line 4060 "Page.c"
	SinglePhotoPage * self;
#line 1187 "Page.vala"
	g_return_val_if_fail (page_name != NULL, NULL);
#line 1188 "Page.vala"
	self = (SinglePhotoPage*) page_construct (object_type, page_name);
#line 1190 "Page.vala"
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 1192 "Page.vala"
	gtk_viewport_set_shadow_type (self->viewport, GTK_SHADOW_NONE);
#line 1193 "Page.vala"
	gtk_container_set_border_width (GTK_CONTAINER (self->viewport), (guint) 0);
#line 1194 "Page.vala"
	gtk_container_add (GTK_CONTAINER (self->viewport), GTK_WIDGET (self->canvas));
#line 1196 "Page.vala"
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->viewport));
#line 1200 "Page.vala"
	gtk_widget_set_double_buffered (GTK_WIDGET (self->canvas), FALSE);
#line 1201 "Page.vala"
	gtk_widget_add_events (GTK_WIDGET (self->canvas), (gint) ((GDK_EXPOSURE_MASK | GDK_STRUCTURE_MASK) | GDK_SUBSTRUCTURE_MASK));
#line 1204 "Page.vala"
	g_signal_connect_object (GTK_WIDGET (self->viewport), "size-allocate", (GCallback) _single_photo_page_on_viewport_resize_gtk_widget_size_allocate, self, 0);
#line 1205 "Page.vala"
	g_signal_connect_object (GTK_WIDGET (self->canvas), "expose-event", (GCallback) _single_photo_page_on_canvas_exposed_gtk_widget_expose_event, self, 0);
#line 1207 "Page.vala"
	page_set_event_source (PAGE (self), GTK_WIDGET (self->canvas));
#line 4086 "Page.c"
	return self;
}


#line 1187 "Page.vala"
SinglePhotoPage* single_photo_page_new (const char* page_name) {
#line 1187 "Page.vala"
	return single_photo_page_construct (TYPE_SINGLE_PHOTO_PAGE, page_name);
#line 4095 "Page.c"
}


#line 1210 "Page.vala"
void single_photo_page_set_default_interp (SinglePhotoPage* self, GdkInterpType default_interp) {
#line 1210 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1211 "Page.vala"
	self->priv->default_interp = default_interp;
#line 4105 "Page.c"
}


#line 1214 "Page.vala"
void single_photo_page_set_pixbuf (SinglePhotoPage* self, GdkPixbuf* unscaled) {
#line 4111 "Page.c"
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
	GdkPixmap* _tmp2_;
#line 1214 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1214 "Page.vala"
	g_return_if_fail (GDK_IS_PIXBUF (unscaled));
#line 4119 "Page.c"
	_tmp1_ = NULL;
#line 1215 "Page.vala"
	_tmp0_ = NULL;
#line 1215 "Page.vala"
	self->priv->unscaled = (_tmp1_ = (_tmp0_ = unscaled, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->unscaled == NULL) ? NULL : (self->priv->unscaled = (g_object_unref (self->priv->unscaled), NULL)), _tmp1_);
#line 4125 "Page.c"
	/* flush pixmap to force repaint*/
	_tmp2_ = NULL;
#line 1218 "Page.vala"
	self->priv->pixmap = (_tmp2_ = NULL, (self->priv->pixmap == NULL) ? NULL : (self->priv->pixmap = (g_object_unref (self->priv->pixmap), NULL)), _tmp2_);
#line 4130 "Page.c"
	/* need to make sure this happens*/
#line 1221 "Page.vala"
	gtk_widget_realize (GTK_WIDGET (self->canvas));
#line 1223 "Page.vala"
	single_photo_page_repaint (self, self->priv->default_interp);
#line 4136 "Page.c"
}


#line 1226 "Page.vala"
GdkDrawable* single_photo_page_get_drawable (SinglePhotoPage* self) {
#line 4142 "Page.c"
	GdkDrawable* result;
	GdkDrawable* _tmp0_;
#line 1226 "Page.vala"
	g_return_val_if_fail (IS_SINGLE_PHOTO_PAGE (self), NULL);
#line 1227 "Page.vala"
	_tmp0_ = NULL;
#line 4149 "Page.c"
	result = (_tmp0_ = GDK_DRAWABLE (self->priv->pixmap), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 1230 "Page.vala"
Dimensions single_photo_page_get_drawable_dim (SinglePhotoPage* self) {
#line 4157 "Page.c"
	Dimensions result;
	result = self->priv->pixmap_dim;
	return result;
}


#line 1234 "Page.vala"
GdkPixbuf* single_photo_page_get_unscaled_pixbuf (SinglePhotoPage* self) {
#line 4166 "Page.c"
	GdkPixbuf* result;
	GdkPixbuf* _tmp0_;
#line 1234 "Page.vala"
	g_return_val_if_fail (IS_SINGLE_PHOTO_PAGE (self), NULL);
#line 1235 "Page.vala"
	_tmp0_ = NULL;
#line 4173 "Page.c"
	result = (_tmp0_ = self->priv->unscaled, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


#line 1238 "Page.vala"
GdkPixbuf* single_photo_page_get_scaled_pixbuf (SinglePhotoPage* self) {
#line 4181 "Page.c"
	GdkPixbuf* result;
	GdkPixbuf* _tmp0_;
#line 1238 "Page.vala"
	g_return_val_if_fail (IS_SINGLE_PHOTO_PAGE (self), NULL);
#line 1239 "Page.vala"
	_tmp0_ = NULL;
#line 4188 "Page.c"
	result = (_tmp0_ = self->priv->scaled, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


/* Returns a rectangle describing the pixbuf in relation to the canvas*/
#line 1243 "Page.vala"
GdkRectangle single_photo_page_get_scaled_pixbuf_position (SinglePhotoPage* self) {
#line 4197 "Page.c"
	GdkRectangle result;
	result = self->priv->scaled_pos;
	return result;
}


#line 1247 "Page.vala"
gboolean single_photo_page_is_inside_pixbuf (SinglePhotoPage* self, gint x, gint y) {
#line 4206 "Page.c"
	gboolean result;
#line 1247 "Page.vala"
	g_return_val_if_fail (IS_SINGLE_PHOTO_PAGE (self), FALSE);
#line 4210 "Page.c"
	result = coord_in_rectangle (x, y, &self->priv->scaled_pos);
	return result;
}


#line 1251 "Page.vala"
void single_photo_page_invalidate (SinglePhotoPage* self, const GdkRectangle* rect) {
#line 1251 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1252 "Page.vala"
	if (GTK_WIDGET (self->canvas)->window != NULL) {
#line 1253 "Page.vala"
		gdk_window_invalidate_rect (GTK_WIDGET (self->canvas)->window, &(*rect), FALSE);
#line 4224 "Page.c"
	}
}


#line 1256 "Page.vala"
void single_photo_page_invalidate_all (SinglePhotoPage* self) {
#line 1256 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1257 "Page.vala"
	if (GTK_WIDGET (self->canvas)->window != NULL) {
#line 1258 "Page.vala"
		gdk_window_invalidate_rect (GTK_WIDGET (self->canvas)->window, NULL, FALSE);
#line 4237 "Page.c"
	}
}


#line 1261 "Page.vala"
static void single_photo_page_on_viewport_resize (SinglePhotoPage* self) {
#line 1261 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1262 "Page.vala"
	single_photo_page_repaint (self, self->priv->default_interp);
#line 4248 "Page.c"
}


#line 1265 "Page.vala"
static gboolean single_photo_page_on_canvas_exposed (SinglePhotoPage* self, const GdkEventExpose* event) {
#line 4254 "Page.c"
	gboolean result;
#line 1265 "Page.vala"
	g_return_val_if_fail (IS_SINGLE_PHOTO_PAGE (self), FALSE);
#line 4258 "Page.c"
	/* to avoid multiple exposes*/
#line 1267 "Page.vala"
	if ((*event).count > 0) {
#line 4262 "Page.c"
		result = FALSE;
		return result;
	}
	/* draw pixmap onto canvas unless it's not been instantiated, in which case draw black
	 (so either old image or contents of another page is not left on screen)*/
#line 1272 "Page.vala"
	if (self->priv->pixmap != NULL) {
#line 1273 "Page.vala"
		GDK_DRAWABLE_GET_CLASS (GDK_DRAWABLE (GTK_WIDGET (self->canvas)->window))->draw_drawable (GDK_DRAWABLE (GTK_WIDGET (self->canvas)->window), self->canvas_gc, GDK_DRAWABLE (self->priv->pixmap), (*event).area.x, (*event).area.y, (*event).area.x, (*event).area.y, (*event).area.width, (*event).area.height);
#line 4272 "Page.c"
	} else {
#line 1276 "Page.vala"
		GDK_DRAWABLE_GET_CLASS (GDK_DRAWABLE (GTK_WIDGET (self->canvas)->window))->draw_rectangle (GDK_DRAWABLE (GTK_WIDGET (self->canvas)->window), gtk_widget_get_style (GTK_WIDGET (self->canvas))->black_gc, TRUE, (*event).area.x, (*event).area.y, (*event).area.width, (*event).area.height);
#line 4276 "Page.c"
	}
	result = TRUE;
	return result;
}


#line 1283 "Page.vala"
static void single_photo_page_real_new_drawable (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable) {
#line 1283 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1283 "Page.vala"
	g_return_if_fail (GDK_IS_GC (default_gc));
#line 1283 "Page.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 4291 "Page.c"
}


#line 1283 "Page.vala"
void single_photo_page_new_drawable (SinglePhotoPage* self, GdkGC* default_gc, GdkDrawable* drawable) {
#line 1283 "Page.vala"
	SINGLE_PHOTO_PAGE_GET_CLASS (self)->new_drawable (self, default_gc, drawable);
#line 4299 "Page.c"
}


#line 1286 "Page.vala"
static void single_photo_page_real_updated_pixbuf (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim) {
#line 1286 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1286 "Page.vala"
	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
#line 4309 "Page.c"
}


#line 1286 "Page.vala"
void single_photo_page_updated_pixbuf (SinglePhotoPage* self, GdkPixbuf* pixbuf, SinglePhotoPageUpdateReason reason, const Dimensions* old_dim) {
#line 1286 "Page.vala"
	SINGLE_PHOTO_PAGE_GET_CLASS (self)->updated_pixbuf (self, pixbuf, reason, old_dim);
#line 4317 "Page.c"
}


#line 1289 "Page.vala"
static void single_photo_page_real_paint (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable) {
#line 1289 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1289 "Page.vala"
	g_return_if_fail (GDK_IS_GC (gc));
#line 1289 "Page.vala"
	g_return_if_fail (GDK_IS_DRAWABLE (drawable));
#line 1290 "Page.vala"
	GDK_DRAWABLE_GET_CLASS (drawable)->draw_pixbuf (drawable, gc, self->priv->scaled, 0, 0, self->priv->scaled_pos.x, self->priv->scaled_pos.y, -1, -1, GDK_RGB_DITHER_NORMAL, 0, 0);
#line 4331 "Page.c"
}


#line 1289 "Page.vala"
void single_photo_page_paint (SinglePhotoPage* self, GdkGC* gc, GdkDrawable* drawable) {
#line 1289 "Page.vala"
	SINGLE_PHOTO_PAGE_GET_CLASS (self)->paint (self, gc, drawable);
#line 4339 "Page.c"
}


#line 1294 "Page.vala"
void single_photo_page_default_repaint (SinglePhotoPage* self) {
#line 1294 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1295 "Page.vala"
	single_photo_page_repaint (self, self->priv->default_interp);
#line 4349 "Page.c"
}


#line 1298 "Page.vala"
void single_photo_page_repaint (SinglePhotoPage* self, GdkInterpType repaint_interp) {
#line 4355 "Page.c"
	gboolean _tmp0_;
	gint width;
	gint height;
	gboolean _tmp1_;
	gboolean new_photo;
	Dimensions old_scaled_dim;
	gboolean _tmp2_;
	gboolean _tmp5_;
#line 1298 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 4366 "Page.c"
	_tmp0_ = FALSE;
#line 1300 "Page.vala"
	if (self->priv->unscaled == NULL) {
#line 1300 "Page.vala"
		_tmp0_ = TRUE;
#line 4372 "Page.c"
	} else {
#line 1300 "Page.vala"
		_tmp0_ = GTK_WIDGET (self->canvas)->window == NULL;
#line 4376 "Page.c"
	}
	/* no image or window, no painting*/
#line 1300 "Page.vala"
	if (_tmp0_) {
#line 4381 "Page.c"
		return;
	}
	width = GTK_WIDGET (self->viewport)->allocation.width;
	height = GTK_WIDGET (self->viewport)->allocation.height;
	_tmp1_ = FALSE;
#line 1306 "Page.vala"
	if (width <= 0) {
#line 1306 "Page.vala"
		_tmp1_ = TRUE;
#line 4391 "Page.c"
	} else {
#line 1306 "Page.vala"
		_tmp1_ = height <= 0;
#line 4395 "Page.c"
	}
#line 1306 "Page.vala"
	if (_tmp1_) {
#line 4399 "Page.c"
		return;
	}
	new_photo = self->priv->pixmap == NULL;
	/* save if reporting an image being rescaled*/
	old_scaled_dim = dimensions_for_rectangle (&self->priv->scaled_pos);
	_tmp2_ = FALSE;
#line 1315 "Page.vala"
	if (self->priv->pixmap_dim.width != width) {
#line 1315 "Page.vala"
		_tmp2_ = TRUE;
#line 4410 "Page.c"
	} else {
#line 1315 "Page.vala"
		_tmp2_ = self->priv->pixmap_dim.height != height;
#line 4414 "Page.c"
	}
	/* attempt to reuse pixmap*/
#line 1315 "Page.vala"
	if (_tmp2_) {
#line 4419 "Page.c"
		GdkPixmap* _tmp3_;
		_tmp3_ = NULL;
#line 1316 "Page.vala"
		self->priv->pixmap = (_tmp3_ = NULL, (self->priv->pixmap == NULL) ? NULL : (self->priv->pixmap = (g_object_unref (self->priv->pixmap), NULL)), _tmp3_);
#line 4424 "Page.c"
	}
	/* if necessary, create a pixmap as large as the entire viewport*/
#line 1319 "Page.vala"
	if (self->priv->pixmap == NULL) {
#line 1320 "Page.vala"
		single_photo_page_init_pixmap (self, width, height);
#line 4431 "Page.c"
	} else {
		gboolean _tmp4_;
		_tmp4_ = FALSE;
#line 1321 "Page.vala"
		if (repaint_interp == SINGLE_PHOTO_PAGE_FAST_INTERP) {
#line 1321 "Page.vala"
			_tmp4_ = self->priv->interp == SINGLE_PHOTO_PAGE_QUALITY_INTERP;
#line 4439 "Page.c"
		} else {
#line 1321 "Page.vala"
			_tmp4_ = FALSE;
#line 4443 "Page.c"
		}
#line 1321 "Page.vala"
		if (_tmp4_) {
#line 4447 "Page.c"
			/* block calls where the pixmap is not being regenerated and the caller is asking for
			 a lower interp*/
#line 1324 "Page.vala"
			repaint_interp = SINGLE_PHOTO_PAGE_QUALITY_INTERP;
#line 4452 "Page.c"
		}
	}
	_tmp5_ = FALSE;
#line 1328 "Page.vala"
	if (self->priv->scaled == NULL) {
#line 1328 "Page.vala"
		_tmp5_ = TRUE;
#line 4460 "Page.c"
	} else {
#line 1328 "Page.vala"
		_tmp5_ = self->priv->interp != repaint_interp;
#line 4464 "Page.c"
	}
	/* rescale if canvas rescaled or better quality is requested*/
#line 1328 "Page.vala"
	if (_tmp5_) {
#line 4469 "Page.c"
		GdkPixbuf* _tmp6_;
		SinglePhotoPageUpdateReason reason;
		_tmp6_ = NULL;
#line 1329 "Page.vala"
		self->priv->scaled = (_tmp6_ = gdk_pixbuf_scale_simple (self->priv->unscaled, self->priv->scaled_pos.width, self->priv->scaled_pos.height, repaint_interp), (self->priv->scaled == NULL) ? NULL : (self->priv->scaled = (g_object_unref (self->priv->scaled), NULL)), _tmp6_);
#line 4475 "Page.c"
		reason = SINGLE_PHOTO_PAGE_UPDATE_REASON_RESIZED_CANVAS;
#line 1332 "Page.vala"
		if (new_photo) {
#line 1333 "Page.vala"
			reason = SINGLE_PHOTO_PAGE_UPDATE_REASON_NEW_PHOTO;
#line 4481 "Page.c"
		} else {
			gboolean _tmp7_;
			_tmp7_ = FALSE;
#line 1334 "Page.vala"
			if (self->priv->interp == SINGLE_PHOTO_PAGE_FAST_INTERP) {
#line 1334 "Page.vala"
				_tmp7_ = repaint_interp == SINGLE_PHOTO_PAGE_QUALITY_INTERP;
#line 4489 "Page.c"
			} else {
#line 1334 "Page.vala"
				_tmp7_ = FALSE;
#line 4493 "Page.c"
			}
#line 1334 "Page.vala"
			if (_tmp7_) {
#line 1335 "Page.vala"
				reason = SINGLE_PHOTO_PAGE_UPDATE_REASON_QUALITY_IMPROVEMENT;
#line 4499 "Page.c"
			}
		}
#line 1337 "Page.vala"
		self->priv->interp = repaint_interp;
#line 1339 "Page.vala"
		single_photo_page_updated_pixbuf (self, self->priv->scaled, reason, &old_scaled_dim);
#line 4506 "Page.c"
	}
#line 1342 "Page.vala"
	single_photo_page_paint (self, self->canvas_gc, GDK_DRAWABLE (self->priv->pixmap));
#line 4510 "Page.c"
	/* invalidate everything*/
#line 1345 "Page.vala"
	single_photo_page_invalidate_all (self);
#line 4514 "Page.c"
	/* schedule improvement if low-quality pixbuf was used*/
#line 1348 "Page.vala"
	if (self->priv->interp != SINGLE_PHOTO_PAGE_QUALITY_INTERP) {
#line 1349 "Page.vala"
		single_photo_page_schedule_improval (self);
#line 4520 "Page.c"
	}
}


#line 1352 "Page.vala"
static void single_photo_page_init_pixmap (SinglePhotoPage* self, gint width, gint height) {
#line 4527 "Page.c"
	GdkPixmap* _tmp0_;
	Dimensions _tmp1_ = {0};
	GdkPixbuf* _tmp2_;
	Dimensions _tmp3_ = {0};
	Dimensions scaled_dim;
	GdkGC* _tmp5_;
	GdkGC* _tmp4_;
#line 1352 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1353 "Page.vala"
	g_assert (self->priv->unscaled != NULL);
#line 1354 "Page.vala"
	g_assert (GTK_WIDGET (self->canvas)->window != NULL);
#line 4541 "Page.c"
	_tmp0_ = NULL;
#line 1356 "Page.vala"
	self->priv->pixmap = (_tmp0_ = gdk_pixmap_new (GDK_DRAWABLE (GTK_WIDGET (self->canvas)->window), width, height, -1), (self->priv->pixmap == NULL) ? NULL : (self->priv->pixmap = (g_object_unref (self->priv->pixmap), NULL)), _tmp0_);
#line 1357 "Page.vala"
	self->priv->pixmap_dim = (dimensions_init (&_tmp1_, width, height), _tmp1_);
#line 4547 "Page.c"
	/* need a new pixbuf to fit this scale*/
	_tmp2_ = NULL;
#line 1360 "Page.vala"
	self->priv->scaled = (_tmp2_ = NULL, (self->priv->scaled == NULL) ? NULL : (self->priv->scaled = (g_object_unref (self->priv->scaled), NULL)), _tmp2_);
#line 4552 "Page.c"
	/* determine size of pixbuf that will fit on the canvas*/
	scaled_dim = dimensions_get_scaled_proportional ((_tmp3_ = dimensions_for_pixbuf (self->priv->unscaled), &_tmp3_), &self->priv->pixmap_dim);
#line 1365 "Page.vala"
	g_assert (width >= scaled_dim.width);
#line 1366 "Page.vala"
	g_assert (height >= scaled_dim.height);
#line 4559 "Page.c"
	/* center pixbuf on the canvas*/
#line 1369 "Page.vala"
	self->priv->scaled_pos.x = (width - scaled_dim.width) / 2;
#line 1370 "Page.vala"
	self->priv->scaled_pos.y = (height - scaled_dim.height) / 2;
#line 1371 "Page.vala"
	self->priv->scaled_pos.width = scaled_dim.width;
#line 1372 "Page.vala"
	self->priv->scaled_pos.height = scaled_dim.height;
#line 4569 "Page.c"
	_tmp5_ = NULL;
#line 1374 "Page.vala"
	_tmp4_ = NULL;
#line 1374 "Page.vala"
	self->canvas_gc = (_tmp5_ = (_tmp4_ = gtk_widget_get_style (GTK_WIDGET (self->canvas))->fg_gc[(gint) GTK_STATE_NORMAL], (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (self->canvas_gc == NULL) ? NULL : (self->canvas_gc = (g_object_unref (self->canvas_gc), NULL)), _tmp5_);
#line 4575 "Page.c"
	/* resize canvas for the pixmap (that is, the entire viewport)*/
#line 1377 "Page.vala"
	gtk_widget_set_size_request (GTK_WIDGET (self->canvas), width, height);
#line 4579 "Page.c"
	/* draw background*/
#line 1380 "Page.vala"
	GDK_DRAWABLE_GET_CLASS (GDK_DRAWABLE (self->priv->pixmap))->draw_rectangle (GDK_DRAWABLE (self->priv->pixmap), gtk_widget_get_style (GTK_WIDGET (self->canvas))->black_gc, TRUE, 0, 0, width, height);
#line 1382 "Page.vala"
	single_photo_page_new_drawable (self, self->canvas_gc, GDK_DRAWABLE (self->priv->pixmap));
#line 4585 "Page.c"
}


#line 1399 "Page.vala"
static gboolean _single_photo_page_image_improval_gsource_func (gpointer self) {
#line 4591 "Page.c"
	return single_photo_page_image_improval (self);
}


#line 1385 "Page.vala"
static void single_photo_page_schedule_improval (SinglePhotoPage* self) {
#line 4598 "Page.c"
	SinglePhotoPage* _tmp1_;
	SinglePhotoPage* _tmp0_;
#line 1385 "Page.vala"
	g_return_if_fail (IS_SINGLE_PHOTO_PAGE (self));
#line 1386 "Page.vala"
	if (self->priv->improval_scheduled != NULL) {
#line 1387 "Page.vala"
		self->priv->reschedule_improval = TRUE;
#line 4607 "Page.c"
		return;
	}
#line 1392 "Page.vala"
	g_idle_add (_single_photo_page_image_improval_gsource_func, self);
#line 4612 "Page.c"
	/* because Idle doesn't maintain a ref to this, need to maintain one ourself
	 (in case the page is destroyed between schedules)*/
	_tmp1_ = NULL;
#line 1396 "Page.vala"
	_tmp0_ = NULL;
#line 1396 "Page.vala"
	self->priv->improval_scheduled = (_tmp1_ = (_tmp0_ = self, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->improval_scheduled == NULL) ? NULL : (self->priv->improval_scheduled = (g_object_unref (self->priv->improval_scheduled), NULL)), _tmp1_);
#line 4620 "Page.c"
}


#line 1399 "Page.vala"
static gboolean single_photo_page_image_improval (SinglePhotoPage* self) {
#line 4626 "Page.c"
	gboolean result;
	SinglePhotoPage* _tmp0_;
#line 1399 "Page.vala"
	g_return_val_if_fail (IS_SINGLE_PHOTO_PAGE (self), FALSE);
#line 1400 "Page.vala"
	if (self->priv->reschedule_improval) {
#line 1401 "Page.vala"
		self->priv->reschedule_improval = FALSE;
#line 4635 "Page.c"
		result = TRUE;
		return result;
	}
#line 1406 "Page.vala"
	single_photo_page_repaint (self, SINGLE_PHOTO_PAGE_QUALITY_INTERP);
#line 4641 "Page.c"
	/* do not touch self after clearing this*/
	_tmp0_ = NULL;
#line 1409 "Page.vala"
	self->priv->improval_scheduled = (_tmp0_ = NULL, (self->priv->improval_scheduled == NULL) ? NULL : (self->priv->improval_scheduled = (g_object_unref (self->priv->improval_scheduled), NULL)), _tmp0_);
#line 4646 "Page.c"
	result = FALSE;
	return result;
}


static void single_photo_page_class_init (SinglePhotoPageClass * klass) {
	single_photo_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SinglePhotoPagePrivate));
	SINGLE_PHOTO_PAGE_CLASS (klass)->new_drawable = single_photo_page_real_new_drawable;
	SINGLE_PHOTO_PAGE_CLASS (klass)->updated_pixbuf = single_photo_page_real_updated_pixbuf;
	SINGLE_PHOTO_PAGE_CLASS (klass)->paint = single_photo_page_real_paint;
	G_OBJECT_CLASS (klass)->finalize = single_photo_page_finalize;
}


static void single_photo_page_instance_init (SinglePhotoPage * self) {
	Dimensions _tmp2_ = {0};
	GdkRectangle _tmp3_ = {0};
	self->priv = SINGLE_PHOTO_PAGE_GET_PRIVATE (self);
	self->canvas_gc = NULL;
	self->canvas = g_object_ref_sink ((GtkDrawingArea*) gtk_drawing_area_new ());
	self->viewport = g_object_ref_sink ((GtkViewport*) gtk_viewport_new (NULL, NULL));
	self->priv->pixmap = NULL;
	self->priv->pixmap_dim = (dimensions_init (&_tmp2_, 0, 0), _tmp2_);
	self->priv->unscaled = NULL;
	self->priv->scaled = NULL;
	self->priv->scaled_pos = (memset (&_tmp3_, 0, sizeof (GdkRectangle)), _tmp3_);
	self->priv->default_interp = SINGLE_PHOTO_PAGE_FAST_INTERP;
	self->priv->interp = SINGLE_PHOTO_PAGE_FAST_INTERP;
	self->priv->improval_scheduled = NULL;
	self->priv->reschedule_improval = FALSE;
}


static void single_photo_page_finalize (GObject* obj) {
	SinglePhotoPage * self;
	self = SINGLE_PHOTO_PAGE (obj);
	(self->canvas_gc == NULL) ? NULL : (self->canvas_gc = (g_object_unref (self->canvas_gc), NULL));
	(self->canvas == NULL) ? NULL : (self->canvas = (g_object_unref (self->canvas), NULL));
	(self->viewport == NULL) ? NULL : (self->viewport = (g_object_unref (self->viewport), NULL));
	(self->priv->pixmap == NULL) ? NULL : (self->priv->pixmap = (g_object_unref (self->priv->pixmap), NULL));
	(self->priv->unscaled == NULL) ? NULL : (self->priv->unscaled = (g_object_unref (self->priv->unscaled), NULL));
	(self->priv->scaled == NULL) ? NULL : (self->priv->scaled = (g_object_unref (self->priv->scaled), NULL));
	(self->priv->improval_scheduled == NULL) ? NULL : (self->priv->improval_scheduled = (g_object_unref (self->priv->improval_scheduled), NULL));
	G_OBJECT_CLASS (single_photo_page_parent_class)->finalize (obj);
}


GType single_photo_page_get_type (void) {
	static GType single_photo_page_type_id = 0;
	if (single_photo_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SinglePhotoPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) single_photo_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SinglePhotoPage), 0, (GInstanceInitFunc) single_photo_page_instance_init, NULL };
		single_photo_page_type_id = g_type_register_static (TYPE_PAGE, "SinglePhotoPage", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return single_photo_page_type_id;
}




